/*-------------------------------------------------------------------------------*/
  /*
    AdfCardBase.h
    Class AdfCardBase
    16-Nov-2004
  */
#ifndef __AdfCardBase__
#define __AdfCardBase__
/*-------------------------------------------------------------------------------*/

#include "L1Cal_IIb_TCC/AdfChipPal.h"
#include "L1Cal_IIb_TCC/AdfDacCtrl.h"
#include "L1Cal_IIb_TCC/AdfChipFpga.h"
#include "L1Cal_IIb_TCC/AdfChipFpga_T7.h"

class CResultFile  ; // forward declaration 

/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/

class CAdfCardBase 
{
/////////////////////////////
// Constructor/Destructor
public:

	         CAdfCardBase ( );

             CAdfCardBase ( unsigned char   _ubVertMaster,
                            unsigned char   _ubVertSlave,
                            unsigned char   _ubCardSlot,
                            CAdfCardBase**  _ppoPrevCard        = NULL, // this pointer is modified to point to this Card
                            CString*        _posCardDescription = NULL, // a string to describe the card
                            unsigned long   _ulSign_Eta         = 0,    // the eta sign of the TT serviced by this card
                            unsigned long   _ulMagn_Eta_By4     = 0,    // the eta magn segment of the TT serviced by this card
                            unsigned long   _ulPhi_By4          = 0 ) ; // the phi segment of the TT serviced by this card

            ~CAdfCardBase ( );

protected:

    void     BasicConstruct( CAdfCardBase**     _ppoPrevCard ) ; 

/////////////////////////////
// static

protected:

    static const unsigned short     m_gkawPRNSeed [ KiTotEtaPerAdf ]
                                                  [ KiTotPhiPerAdf ] 
                                                  [ eTot_TT_Chan_Type ] ;

/////////////////////////////
// Attributes
public:

    //String describing the card functionality
    CString             m_osCardDescription ;

    //The Trigger Towers serviced by this card
    unsigned long       m_ulSign_Eta ;     // the sign of eta 0=positive, 1=negative
    unsigned long       m_ulMagn_Eta_By4 ; // the magnitude eta segment, 0=1:4, 1=5:8, etc
    unsigned long       m_ulPhi_By4 ;      // the phi segment 0=1:4, 1=5:8, etc

    // The board control PAL
    CAdfChipPal*        m_poBcPal ;

    // The Data Path FPGAs
//  CAdfChipFpga*       m_apoDpFpga[KiTotFpgaPerAdf] ;

    // The T3 FPGAs
//  CAdfChipFpga_T3*    m_apoT3Fpga[KiTotFpgaPerAdf] ;

    // The T4 FPGAs
//  CAdfChipFpga_T4*    m_apoT4Fpga[KiTotFpgaPerAdf] ;

    // The T7 FPGAs
    CAdfChipFpga_T7*    m_apoT7Fpga[KiTotFpgaPerAdf] ;

    // High level pedestal DAC control
    CAdfDacCtrl*        m_poDacCtrl ;

protected:

  //CBaseCrate*     m_poMotherCrate ;
    CAdfCardBase*      m_poNextCard ;
    CAdfCardBase*      m_poPrevCard ;

    CAdfChipBase*      m_poFirstFpga ; // starting point down chain of links through all Main Array Fpgas
    CAdfChipBase*      m_poLastFpga ; // starting point down chain of links through all Main Array Fpgas

    unsigned short  m_uwTotChip ;

    unsigned char   m_ubVertMaster ;
    unsigned char   m_ubVertSlave ;
    unsigned char   m_ubCardSlot ;

    bool            m_lDeSkewed ;

protected:

    TulVmeAddress   m_ulVmeAddress ;

/////////////////////////////
// Operations
public:

    ESuccess        Initialize        ( EYesNo              _eSkipInitLookup = No ) ;

    ESuccess        f_okStopPRN           ( ) ;
    ESuccess        f_okStartPRN          ( ) ;
    ESuccess        f_okLoadPRNSeeds      ( ) ;
    ESuccess        f_okSendPseudoRandom  ( ) ;
    ESuccess        f_okFreezePseudoRandom( ) ;
    ESuccess        f_okSendBlsData       ( ) ;

    ESuccess        f_okInitiateChLinkDeSkew   ( ) ;

    ESuccess        f_okRestartRawAdcAddrGen   ( ) ;
    ESuccess        f_okRestartOutputEtAddrGen ( ) ;


    TCString        f_osBaseAddress   ( ) ; // e.g. return "@Master#0/Slave#1/Slot# 2"
    TCString        f_osVmeAddress    ( ) ; // e.g. return "@0x01020000"

    CAdfCardBase*   f_poNextCard      ( ) { return m_poNextCard  ; } ;
    CAdfCardBase*   f_poPrevCard      ( ) { return m_poPrevCard  ; } ;

    unsigned char   f_ubVertMaster    ( ) { return m_ubVertMaster; } ;
    unsigned char   f_ubVertSlave     ( ) { return m_ubVertSlave ; } ;
    unsigned char   f_ubCardSlot      ( ) { return m_ubCardSlot  ; } ;
    TulVmeAddress   f_ulVmeAddress    ( ) { return m_ulVmeAddress; } ;


    bool            f_lGet_DeSkewed    ( ) { return m_lDeSkewed ;  } ;
    void            f_vClear_DeSkewed  ( ) { m_lDeSkewed = false ; } ;


static  CAdfCardBase*  f_poFindCard   ( unsigned char   _ubVertMaster,
                                        unsigned char   _ubVertSlave,
                                        unsigned char   _ubCardSlot,
                                        CAdfCardBase*      _poLastCard = NULL ) ; // pointer to the last of any chain of cards

    CAdfChipBase*      f_poFindFpga   ( unsigned char   _ubChipAddress ) ; 

    void            Dump              ( bool               _lIncludeReadOnly = true,
                                        CResultFile*       _poResultFile = NULL ) ;

protected:

///////////////////////////////////////////
// Access Privileges outside of this class

// special access privilege so that the base Fpga constructor 
// can read the card address, etc and write the LastFpga variable, etc
friend class CAdfChipBase ;

// special access privilege so that the base register constructor 
// can read the fpga address, etc 
friend class CAdfRegBase ;

};


/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
#endif // __AdfCardBase__
