/*-------------------------------------------------------------------------------*/
  /*
    AdfChipFpga_T7.h
    Class CAdfChipFpga_T7
    09-Mar-2005
  */
#ifndef __AdfChipFpga_T7__
#define __AdfChipFpga_T7__
/*-------------------------------------------------------------------------------*/

#include "L1Cal_IIb_TCC/AdfChipFpga.h"
#include "L1Cal_IIb_TCC/AdfRegRawAdc.h"
#include "L1Cal_IIb_TCC/AdfRegEtLookup.h"
#include "L1Cal_IIb_TCC/AdfRegOutputEt.h"
#include "trigmon/L1CalTcc_MonitData_BeamX.hpp" // for kL1CalTcc_Tot_LiveBX 

class CAdfCardBase ;           // forward declaration (don't need the details)

/*-------------------------------------------------------------------------------*/

//define an ADC Array Register Data Type
#define KiAdfAdcSampleLength_T7  KiTotTicksInTurn*KiTotSamplesPerTick
typedef TuwRegWord          TAdcArrayT7 [KiAdfAdcSampleLength_T7] ;

//define an array of histograms for all subticks (note Tick number goes 1:159 with 0 unused)
typedef unsigned short       Ta3uwSTicksHisto [KiTotTicksInTurn+1][KiTotSamplesPerTick][KiSizeOfTAdcHisto] ;

//define an array of histograms for all Live Crossings 
typedef unsigned short       Ta2uwLiveXHisto [kL1CalTcc_Tot_LiveBX][KiSizeOfTAdfEtHisto] ;

//define maximum Latency Adjust Value
#define     KiAdfMaxAlignmentDelay                 15    //max Latency Adjust (4-bits)

/*-------------------------------------------------------------------------------*/

class CAdfChipFpga_T7 : public CAdfChipFpga
{
/////////////////////////////
// Constructor/Destructor
public:

             CAdfChipFpga_T7();

             CAdfChipFpga_T7   ( CAdfCardBase*   _poMotherCard,
                                 unsigned char   _ubChipAddress );

            ~CAdfChipFpga_T7();

/////////////////////////////
// Attributes
public:

    // Global registers

    //  Global Reg #0   Raw ADC Mem Blk Address Generator   Control
    CAdfRegBase*   m_poGlb_RawAdcAddrGen_Ctrl ;
    //  Global Reg #1   Raw ADC Mem Blk Address Generator Control   Highest Address
    CAdfRegBase*   m_poGlb_RawAdcAddrGen_Size ;
    //  Global Reg #2
    CAdfRegBase*   m_poGlb_RawAdcAddrGen_Curr ;

    //  Global Reg #3   Final Output Data Mem Blk Address Generator   Control
    CAdfRegBase*   m_poGlb_FinalOutAddrGen_Ctrl ;
    //  Global Reg #4   Final Output Data Mem Blk Address Generator Control   Highest Address
    CAdfRegBase*   m_poGlb_FinalOutAddrGen_Size ;
    //  Global Reg #5
    CAdfRegBase*   m_poGlb_FinalOutAddrGen_Curr ;

    //  Global Reg #6
    CAdfRegBase*   m_poGlb_Status ;

    //  Global Reg #7
    CAdfRegBase*   m_poGlb_RawAdc_State ;

    //  Global Reg #16
    CAdfRegBase*   m_poGlb_FinalOut_Sample ;

    // Per TT registers
    CAdfRegBase*        m_a3poTt_RawAdc[KiTotEtaPerAdfFpga][KiTotPhiPerAdfFpga][eTot_TT_Chan_Type] ;

    CAdfRegBase*        m_a3poTt_InputCtrl[KiTotEtaPerAdfFpga][KiTotPhiPerAdfFpga][eTot_TT_Chan_Type] ;

    CAdfRegBase*        m_a3poTt_OutputCtrl[KiTotEtaPerAdfFpga][KiTotPhiPerAdfFpga][eTot_TT_Chan_Type] ;

    CAdfRegBase*        m_a3poTt_PrnSeed[KiTotEtaPerAdfFpga][KiTotPhiPerAdfFpga][eTot_TT_Chan_Type] ;

    CAdfRegOutputEt*    m_a2poTt_MemBlkEmHdOut[KiTotEtaPerAdfFpga][KiTotPhiPerAdfFpga] ;

    CAdfRegRawAdc*      m_a3poTt_MemBlkRawAdc[KiTotEtaPerAdfFpga][KiTotPhiPerAdfFpga][eTot_TT_Chan_Type] ;

    CAdfRegEtLookup*    m_a3poTt_MemBlkEtLookup[KiTotEtaPerAdfFpga][KiTotPhiPerAdfFpga][eTot_TT_Chan_Type] ;


protected:

    ERawAdc_MemMode_Type        m_eRawAdc_MemMode_Current ;
    EAdfOut_MemMode_Type        m_eAdfOut_MemMode_Current ;

/////////////////////////////
// Operations
public:

    ESuccess Initialize             ( EYesNo              _eSkipInitLookup = No ) ;

    ESuccess RequestNewRawAdcData   ( ETT_Chan_Type       _eTtType,
                                      unsigned long       _ulRelEta,
                                      unsigned long       _ulRelPhi,
                                      unsigned int        _uiThreshold = 0 ) ;

    ESuccess CheckRawAdcDataCaptured( ETT_Chan_Type       _eTtType,
                                      unsigned long       _ulRelEta,
                                      unsigned long       _ulRelPhi ) ;

    ESuccess ADC_Capture            ( ETT_Chan_Type       _eTtType,
                                      unsigned long       _ulRelEta,
                                      unsigned long       _ulRelPhi,
                                      unsigned int        _uiThreshold,
                                      unsigned int        _uiMaxWaitSec,
                                      TAdcArrayT7         _auwAdcArray,
                                      CResultFile*        _poCaptureFile = NULL ) ;

    ESuccess ADC_HistoLiveX         ( ETT_Chan_Type       _eTtType,
                                      unsigned long       _ulRelEta,
                                      unsigned long       _ulRelPhi,
                                      TAdcHisto* const    _kpauwHistogram,
                                      const unsigned long _kulSampleSize,
                                      unsigned short*     _puwDacValue = NULL ) ;

    ESuccess ADC_HistoEachSubTick   ( ETT_Chan_Type       _eTtType,
                                      unsigned long       _ulRelEta,
                                      unsigned long       _ulRelPhi,
                                      Ta3uwSTicksHisto*   _pa3ulHistogram,
                                      const unsigned long _kulSampleSize ) ;

    ESuccess OutputEt_HistoEachLiveX( ETT_Chan_Type       _eTtType,
                                      unsigned long       _ulRelEta,
                                      unsigned long       _ulRelPhi,
                                      Ta2uwLiveXHisto*    _pa2uwHistogram,
                                      const unsigned long _kulSampleSize ) ;

    ESuccess ExcludeTower           ( ETT_Chan_Type           _eTtType,
                                      const unsigned long     _ulRelEta,
                                      const unsigned long     _ulRelPhi ) ;

    ESuccess SetOutputConstant      ( ETT_Chan_Type           _eTtType,
                                      const unsigned long     _ulRelEta,
                                      const unsigned long     _ulRelPhi,
                                      unsigned char           _ubConstantData ) ;

    ESuccess SetRawAdcThresh        ( ETT_Chan_Type           _eTtType,
                                      const unsigned long     _ulRelEta,
                                      const unsigned long     _ulRelPhi,
                                      unsigned char           _ubThreshold ) ;

    ESuccess SetAlignmentDelay      ( ETT_Chan_Type           _eTtType,
                                      const unsigned long     _ulRelEta,
                                      const unsigned long     _ulRelPhi,
                                      unsigned char           _ubAlignmentDelay ) ;

    ESuccess SetSimuRawAdc          ( ETT_Chan_Type           _eTtType,
                                      const unsigned long     _ulRelEta,
                                      const unsigned long     _ulRelPhi,
                                      const unsigned short    _uwSimuRawAdc,
                                      const unsigned long     _ulTickNum,
                                      const unsigned long     _ulSubTickNum ) ;

    ESuccess SetSimuOutputEt        ( ETT_Chan_Type           _eTtType,
                                      const unsigned long     _ulRelEta,
                                      const unsigned long     _ulRelPhi,
                                      const unsigned char     _ubSimuOutputEt,
                                      const unsigned long     _ulTickNum ) ;

    ESuccess ReadMonitOutputEt      ( const unsigned long     _ulRelEta,
                                      const unsigned long     _ulRelPhi,
                                      unsigned char*          _pubMonitOutputEtEm,
                                      unsigned char*          _pubMonitOutputEtHd,
                                      const unsigned long     _ulTickNum ) ;

    ERawAdc_MemMode_Type     f_eRawAdc_MemMode_Current         ( ) { return m_eRawAdc_MemMode_Current ; } ;
    EAdfOut_MemMode_Type     f_eAdfOut_MemMode_Current         ( ) { return m_eAdfOut_MemMode_Current ; } ;
    
    ESuccess f_okLoadPRNSeed        ( ETT_Chan_Type           _eTtType,
                                      const unsigned long     _ulRelEta,
                                      const unsigned long     _ulRelPhi,
                                      const unsigned short    _uwSeed ) ;

    ESuccess f_okRestartRawAdcAddrGen   ( ) ;
    ESuccess f_okRestartOutputEtAddrGen ( ) ;

    ESuccess f_okRawAdc_MuxMode     ( ETT_Chan_Type           _eTtType,
                                      const unsigned long     _ulRelEta,
                                      const unsigned long     _ulRelPhi,
                                      ERawAdc_MuxMode_Type    _eRawAdc_MuxMode ) ;

    ESuccess f_okRawAdc_MemMode     ( ETT_Chan_Type           _eTtType,
                                      const unsigned long     _ulRelEta,
                                      const unsigned long     _ulRelPhi,
                                      ERawAdc_MemMode_Type    _eRawAdc_MemMode ) ;

    ESuccess f_okAdfOut_MuxMode     ( ETT_Chan_Type           _eTtType,
                                      const unsigned long     _ulRelEta,
                                      const unsigned long     _ulRelPhi,
                                      EAdfOut_MuxMode_Type    _eAdfOut_MuxMode ) ;

    ESuccess f_okAdfOut_MemMode     ( ETT_Chan_Type            _eTtType,
                                      const unsigned long      _ulRelEta,
                                      const unsigned long      _ulRelPhi,
                                      EAdfOut_MemMode_Type     _eAdfOut_MemMode ) ;

protected:

};

// These are offsets of specific registers in the Fpga
// these offsets are from the Fpga base address

#define     KiRegAddrGlb_RawAdcAddrGen_Ctrl     0x200
#define     KiRegAddrGlb_RawAdcAddrGen_Size     0x201
#define     KiRegAddrGlb_RawAdcAddrGen_Curr     0x202

#define     KiRegAddrGlb_FinalOutAddrGen_Ctrl   0x203
#define     KiRegAddrGlb_FinalOutAddrGen_Size   0x204
#define     KiRegAddrGlb_FinalOutAddrGen_Curr   0x205

#define     KiRegAddrGlb_Status                 0x206
#define     KiRegAddrGlb_RawAdc_State           0x207
#define     KiRegAddrGlb_FinalOut_Sample        0x210

#define     KiRegAddrTt_RawAdc_BaseOffset       0x300
#define     KiRegAddrTt_InputCtrl_BaseOffset    0x302
#define     KiRegAddrTt_OutputCtrl_BaseOffset   0x304
#define     KiRegAddrTt_PrnSeed_BaseOffset      0x306

#define     KiRegAddrTt_MemBlkEmHdOut_BaseOffset    0x0400
#define     KiRegAddrTt_MemBlkRawAdc_BaseOffset     0x0800
#define     KiRegAddrTt_MemBlkEtLookup_BaseOffset   0x1000

#define     KiRegAddrTt_MemBlkEmHdOut_Size          0x0400
#define     KiRegAddrTt_MemBlkRawAdc_Size           0x0400
#define     KiRegAddrTt_MemBlkEtLookup_Size         0x0800

#define     KiRegAddrTt_BaseShiftPerPhi             0x2000
#define     KiRegAddrTt_BaseShiftPerEta             0x8000


// Bit fields and Bit Masks to control registers

// Bit fields for m_poGlb_RawAdcAddrGen_Ctrl or m_poGlb_FinalOutAddrGen_Ctrl
#define     kuiAddrGen_Start_BitMask            0x0001
#define     kuiAddrGen_FreeRun_BitMask          0x0002
#define     kuiAddrGen_FollowScld_BitMask       0x0004
#define     kuiAddrGen_WriteEnable_BitMask      0x0008

// Bit fields for m_poGlb_Status
#define     kuiGlbStatus_RawAdcAGenRun_BitMask  0x0001
#define     kuiGlbStatus_RawAdcAGenWEn_BitMask  0x0002
#define     kuiGlbStatus_FinOutAGenRun_BitMask  0x0010
#define     kuiGlbStatus_FinOutAGenWEn_BitMask  0x0020
#define     kuiGlbStatus_BeginTurn_BitMask      0x0100
#define     kuiGlbStatus_LiveBX_BitMask         0x0200
#define     kuiGlbStatus_SaveMonitData_BitMask  0x0400
#define     kuiGlbStatus_SclInit_BitMask        0x0800
#define     kuiGlbStatus_ScldSpare_BitMask      0x1000
#define     kuiGlbStatus_DpFpgaID_BitMask       0x4000
#define     kuiGlbStatus_PrngShifting_BitMask   0x8000

// Bit fields for m_a3poTt_InputCtrl
#define     kuiTtInputCtrl_Thresh_BitMask               0x00ff
#define     kuiTtInputCtrl_Delay_BitMask                0x0f00
#define     kuiTtInputCtrl_SendSimuData_BitMask         0x1000
#define     kuiTtInputCtrl_StopOnThresh_BitMask         0x4000
#define     kuiTtInputCtrl_Delay_Shift                  8       // how many bits to shift Delay value

// Bit fields for m_a3poTt_OutputCtrl
#define     kuiTtOutputCtrl_FixedData_BitMask           0x00ff
#define     kuiTtOutputCtrl_MuxSel_BitMask              0x0700
#define     kuiTtOutputCtrl_StopOnThresh_BitMask        0x1000

#define     kuiTtOutputCtrl_MuxSel_Filter_Always        0x0000
#define     kuiTtOutputCtrl_MuxSel_ConstEt_Always       0x0100
#define     kuiTtOutputCtrl_MuxSel_PRN_Always           0x0200
#define     kuiTtOutputCtrl_MuxSel_SimulBlock_Always    0x0300
#define     kuiTtOutputCtrl_MuxSel_Filter_LiveX         0x0400
#define     kuiTtOutputCtrl_MuxSel_Fixed_LiveX          0x0500
#define     kuiTtOutputCtrl_MuxSel_PRN_LiveX            0x0600
#define     kuiTtOutputCtrl_MuxSel_SimulBlock_LiveX     0x0700


/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
#endif // __AdfChipFpga_T7__
