/*-------------------------------------------------------------------------------*/
  /*
    AdfChipPal.h
    Class CAdfChipPal
    11-Nov-04
  */
#ifndef __AdfChipPal__
#define __AdfChipPal__
/*-------------------------------------------------------------------------------*/

#include "L1Cal_IIb_TCC/AdfRegBase.h"
#include "L1Cal_IIb_TCC/AdfChipBase.h"

class CAdfCardBase ;           // forward declaration 

/*-------------------------------------------------------------------------------*/

class CAdfChipPal : public CAdfChipBase
{
/////////////////////////////
// Constructor/Destructor
public:

	         CAdfChipPal();

             CAdfChipPal    ( CAdfCardBase*   _poMotherCard,
                              unsigned char   _ubChipAddress );

            ~CAdfChipPal();

/////////////////////////////
// Attributes
public:

    CAdfRegBase*   m_poCtrlEnables ;    // FA 0
    CAdfRegBase*   m_poCtrlCrateStat ;  // FA 1
    CAdfRegBase*   m_poReadFpgaStat ;   // FA 2
    CAdfRegBase*   m_poReadCrateStat ;  // FA 3
    CAdfRegBase*   m_poSerDacData ;     // FA 4
    CAdfRegBase*   m_poConfigCtrl ;     // FA 5
    CAdfRegBase*   m_poConfigData ;     // FA 6
    CAdfRegBase*   m_poConfigStatus ;   // FA 7

protected:
 
/////////////////////////////
// Operations
public:

    ESuccess Initialize             ( ) ;

    ESuccess InitiateFpgaConfig     ( unsigned short      _uwConfigEnab,
                                      unsigned char       _ubVersionMajor = 0 ,
                                      unsigned char       _ubVersionMinor = 0 ) ;

    ESuccess WriteFpgaConfig        ( unsigned short      _uwConfigEnab,
                                      unsigned char*      _aubByteStream,
                                      int                 _iNumBytes ) ;

    ESuccess TerminateFpgaConfig    ( unsigned short      _uwConfigEnab ) ;
 
    ESuccess GetFpgaConfiguredMask  ( unsigned short*  _puwFpgaConfiguredMask ) ;

    ESuccess SetStatusToScld        ( unsigned long ulStatusNum, 
                                      unsigned long ulStatusState ) ;

    ESuccess InitiateChLinkDeSkew   ( ) ;

    ESuccess f_okStopPRN            ( ) ;
    ESuccess f_okStartPRN           ( ) ;

protected:

};

// These are address offsets of specific registers 
// these offsets are from the Chip base address 

#define KiRegAddrCtrlEnables        0
#define KiRegAddrCtrlCrateStat      1
#define KiRegAddrReadFpgaStat       2
#define KiRegAddrReadCrateStat      3
#define KiRegAddrSerDacData         4
#define KiRegAddrConfigCtrl         5
#define KiRegAddrConfigData         6
#define KiRegAddrConfigStatus       7

// These are bitfields of specific registers 

#define KiCtrlEnables_ChanLinkDeskewBar 0x01
#define KiCtrlEnables_ChanLinkDCBal     0x02
#define KiCtrlEnables_CtrlPalLed        0x08
#define KiCtrlEnables_EnableFpgaConfig  0x10
#define KiCtrlEnables_EnableLoadDACs    0x20
#define KiCtrlEnables_EnablePalAccess   0x40
#define KiCtrlEnables_EnableADCs        0x80

#define KiCtrlCrateStat_CrateStat_0     0x01
#define KiCtrlCrateStat_CrateStat_1     0x02
#define KiCtrlCrateStat_CrateStat_2     0x04
#define KiCtrlCrateStat_CrateStat_3     0x08
#define KiCtrlCrateStat_CrateToScld_0   0x10
#define KiCtrlCrateStat_CrateToScld_1   0x20
#define KiCtrlCrateStat_ReqDacCSBar     0x40

#define KiReadFpgaStat_Fpga_0_Status_0  0x01
#define KiReadFpgaStat_Fpga_0_Status_1  0x02
#define KiReadFpgaStat_Fpga_0_Status_2  0x04
#define KiReadFpgaStat_Fpga_0_Status_3  0x08
#define KiReadFpgaStat_Fpga_1_Status_0  0x10
#define KiReadFpgaStat_Fpga_1_Status_1  0x20
#define KiReadFpgaStat_Fpga_1_Status_2  0x40
#define KiReadFpgaStat_Fpga_1_Status_3  0x80

#define KiReadCrateStat_CrateStat_0     0x01
#define KiReadCrateStat_CrateStat_1     0x02
#define KiReadCrateStat_CrateStat_2     0x04
#define KiReadCrateStat_CrateStat_3     0x08
#define KiReadCrateStat_LastDacSerOut   0x10

#define KiConfigCtrl_ReqCnfgPrgBar_0    0x01
#define KiConfigCtrl_ReqCnfgPrgBar_1    0x02
#define KiConfigCtrl_CnfgChipSelBar_0   0x04
#define KiConfigCtrl_CnfgChipSelBar_1   0x08
#define KiConfigCtrl_CnfgRdWrBar        0x10
#define KiConfigCtrl_PrnEnabled         0x10 // after configuration, this bits controls the PRNs
#define KiConfigCtrl_Shift_ChipSelToLsb    2  // to shift the CnfgChipSelBar bits to become to LSBits


#define KiConfigStatus_CnfgInitBar_0    0x01
#define KiConfigStatus_CnfgInitBar_1    0x02
#define KiConfigStatus_CnfgBusy_0       0x04
#define KiConfigStatus_CnfgBusy_1       0x08
#define KiConfigStatus_CnfgDone_0       0x10
#define KiConfigStatus_CnfgDone_1       0x20

#define KiConfigCtrl_Shift_CnfgBusyToLsb   2  // to shift the CnfgBusy bits to become to LSBits
#define KiConfigCtrl_Shift_CnfgDoneToLsb   4  // to shift the CnfgDone bits to become to LSBits

/*-------------------------------------------------------------------------------*/

// maximum number of times the InitiateFpgaConfig routine reads the status words
// waiting for an ok to start sending the configuration stream.
#define KMaxConfigWaitInit               1000

// maximum number of times the TerminateFpgaConfigroutine reads the status words
// waiting for device to advertized it has been successfully configured
#define KMaxConfigWaitDone               1000

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
#endif // __AdfChipPal__
