/*-------------------------------------------------------------------------------*/
  /*
    AdfDacCtrl.h
    Class CAdfDacCtrl
    17-Jan-2001
  */
#ifndef __AdfDacCtrl__
#define __AdfDacCtrl__
/*-------------------------------------------------------------------------------*/

#include "L1Cal_IIb_TCC/AdfChipPal.h"


// This is a special purpose structure for alternate access
// of Serial Dac Command (32-bit word) by bit fields, or whole longword
union OneDacCommand
{
    unsigned long   ulLong ;

    struct LongByFieldPhi
    {
        unsigned int TrailDontCare :  4 ;
        unsigned int DacValue      : 12 ;
        unsigned int ChanType      :  1 ;
        unsigned int ChanRelPhi    :  2 ;
        unsigned int MustBeZero    :  1 ; // Must be zero to select single channel
        unsigned int Command       :  4 ;
        unsigned int LeadDontCare  :  8 ;
    } ByFieldRel ;

    struct LongByFieldChan
    {
        unsigned int TrailDontCare :  4 ;
        unsigned int DacValue      : 12 ;
        unsigned int Channel       :  4 ;
        unsigned int Command       :  4 ;
        unsigned int LeadDontCare  :  8 ;
    } ByFieldChan ;

} ;

#define     KiDacCmdCommandNoOp      0xf
#define     KiDacCmdCommandUpdate    0x3

#define     KiDacCmdChannelTypeEm    0x0
#define     KiDacCmdChannelTypeHd    0x1

#define     KiDacCmdChannelAll       0xf

#define     KiDacCmdMaxDacValue      0xfff


class CAdfDacCtrl
{

/////////////////////////////
// Constructor/Destructor
public:

    CAdfDacCtrl       ( unsigned char   _ubVertMaster,
                        unsigned char   _ubVertSlave,
                        unsigned char   _ubCardSlot ) ;

    CAdfDacCtrl       ( CAdfChipPal*    _poBrdCtrlPal ) ;

   ~CAdfDacCtrl();

/////////////////////////////
// Attributes
public:

private:

    CAdfChipPal*    m_poBrdCtrlPal ;

    EYesNo          m_eynDeleteDummyAdfInDestructor;

    OneDacCommand   m_xSerialDacCmd[KiTotEtaPerAdf] ; // serial command can set 1x channel per DAC
                                                    // i.e. one EM or HD channel for one Phi for each Eta
                                                    // Send the MSB First.
    /*
      ZER DAC         ZER DAC         ZER DAC         ZER DAC
      Eta E      -->  Eta E+1    -->  Eta E+2    -->  Eta N+3
      Phi P:P+3       Phi P:P+3       Phi P:P+3       Phi P:P+3
      EM & HD         EM & HD         EM & HD         EM & HD
    */


/////////////////////////////
// Operations
public:

    ESuccess    f_okForceDefault      ( ) ;

    ESuccess    f_okWriteOneDac       ( ETT_Chan_Type        _eTTChanType,
                                        unsigned long const  _kulRelEta,
                                        unsigned long const  _kulRelPhi,
                                        unsigned long const  _kulValue ) ;

    TCString    f_osHostCardAddress   ( ) ; // e.g. return "Master#0/Slave#1/Slot# 2"

private:

    ESuccess    f_okSendSerialCommand ( ) ;

    ESuccess    f_okCommandSetup      ( ) ;
    ESuccess    f_okSendOneDacCtrlWord( unsigned long const _kulCmdData ) ;
    ESuccess    f_okCommandComplete   ( ) ;
};

/////////////////////////////////////////////////////////////////////////////
#define SET_CMD_NO_OP( )                                                    \
                                                                            \
    for ( int iRelEta = 0 ; iRelEta < KiTotEtaPerAdf ; iRelEta++ )          \
    {                                                                       \
        m_xSerialDacCmd[iRelEta].ulLong = 0 ;                               \
        m_xSerialDacCmd[iRelEta].ByFieldRel.Command = KiDacCmdCommandNoOp ; \
    }                                                                       \
/////////////////////////////////////////////////////////////////////////////
#define SET_CMD_UPDATE_ETA( REL_ETA )                                       \
                                                                            \
    m_xSerialDacCmd[REL_ETA].ByFieldRel.Command = KiDacCmdCommandUpdate ;   \
                                                                            \
/////////////////////////////////////////////////////////////////////////////
#define SET_CMD_CHANNEL_PHI( REL_ETA, REL_PHI )                             \
                                                                            \
    m_xSerialDacCmd[REL_ETA].ByFieldRel.ChanRelPhi = REL_PHI ;              \
                                                                            \
/////////////////////////////////////////////////////////////////////////////
#define SET_CMD_CHANNEL_EM( REL_ETA )                                       \
                                                                            \
    m_xSerialDacCmd[REL_ETA].ByFieldRel.ChanType = KiDacCmdChannelTypeEm ;  \
                                                                            \
/////////////////////////////////////////////////////////////////////////////
#define SET_CMD_CHANNEL_HD( REL_ETA )                                       \
                                                                            \
    m_xSerialDacCmd[REL_ETA].ByFieldRel.ChanType = KiDacCmdChannelTypeHd ;  \
                                                                            \
/////////////////////////////////////////////////////////////////////////////
#define SET_CMD_DAC_VALUE( REL_ETA, DAC_VALUE )                             \
                                                                            \
    m_xSerialDacCmd[REL_ETA].ByFieldRel.DacValue = DAC_VALUE ;              \
                                                                            \
/////////////////////////////////////////////////////////////////////////////

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
#endif // __AdfDacCtrl__
