/*-------------------------------------------------------------------------------*/
  /*
    AdfRegBase.h
    Class AdfRegBase
    16-Nov-2004
  */
#ifndef __AdfRegBase__
#define __AdfRegBase__
/*-------------------------------------------------------------------------------*/

class CAdfCardBase ;   // forward declaration 
class CAdfChipBase ;   // forward declaration 

class CResultFile  ;   // forward declaration 

/*-------------------------------------------------------------------------------*/

//define an ADC Register Histogram Data Type (from 0 to 1023)
#define KiAdfDacMaxOutput       0x3ff
#define KiSizeOfTAdcHisto       0x400
typedef unsigned short          TAdcHisto [KiSizeOfTAdcHisto] ;

//define an Output Et Histogram Data Type (from 0 to 255)
#define KiAdfEtMaxOutput        0xff
#define KiSizeOfTAdfEtHisto     0x100
typedef unsigned short          TAdfEtHisto [KiSizeOfTAdfEtHisto] ;

class CAdfRegBase
{

/////////////////////////////
// Constructor/Destructor
public:

             CAdfRegBase   ( );
             
             CAdfRegBase   ( CAdfChipBase*      _poMotherFpga,
                             unsigned short     _uwRegAddress,
                             TuwRegWord         _uwWriteMask = 0xffff,
                             TuwRegWord         _uwReadMask = 0xffff,
                             TuwRegWord         _uwInitValue = 0,
                             TuwRegWord         _uwDefaultProgr = 0,
                             unsigned char      _ubPipelineDepth = 0 ) ;

             CAdfRegBase      ( unsigned char      _ubVertMaster,
                             unsigned char      _ubVertSlave,
                             unsigned char      _ubCardSlot,
                             unsigned char      _ubChipAddress,
                             unsigned short     _uwRegAddress,
                             CAdfRegBase**         _ppoPrevReg = NULL, // this pointer is modified to point to this register
                             TuwRegWord         _uwWriteMask = 0xffff,
                             TuwRegWord         _uwReadMask = 0xffff ) ;

             CAdfRegBase      ( TulVmeAddress      _ulVmeAddress,
                             CAdfRegBase**         _ppoPrevReg = NULL, // this pointer is modified to point to this register
                             TuwRegWord         _uwWriteMask = 0xffff,
                             TuwRegWord         _uwReadMask = 0xffff ) ;

            ~CAdfRegBase();

            void RemoveFromChain() ; // Fix up linked chain to skip this register (to do before deleting it)

protected:

    void     BasicConstruct( TulVmeAddress   _ulVmeAddress,
                             CAdfRegBase**      _ppoPrevReg = NULL,
                             TuwRegWord      _uwWriteMask = 0xffff,
                             TuwRegWord      _uwReadMask = 0xffff ) ;

/////////////////////////////
// Attributes
public:

protected:
    CAdfChipBase*      m_poMotherFpga ;
    CAdfRegBase*       m_poPrevReg ;
    CAdfRegBase*       m_poNextReg ;

    unsigned short  m_uwRegAddress ;    // The Register Address on the mother FPGA
    TulVmeAddress   m_ulVmeAddress ;    // The VME Address as seen from the communication crate
    TuwRegWord*     m_puwVirtAddress ;  // The Memory Mapped Virtual Address for this register

//    EYesNo          m_ynInitialized ;
    EYesNo          m_ynModified ;
    EYesNo          m_ynVolatile ;

//    EYesNo          m_ynSkipIO ;
//    EYesNo          m_ynTestScan ;

    TuwRegWord      m_uwInitValue ;
    TuwRegWord      m_uwDefaultProgr ;
    TuwRegWord      m_uwReadMask ;
    TuwRegWord      m_uwWriteMask ;
//    unsigned char   m_ubPipelineDepth ;

    TuwRegWord      m_uwLastWrite ;
    TuwRegWord      m_uwLastRead ;

    TuwRegWord      m_uwSavedContent ;

/////////////////////////////
// Operations
public:
         ESuccess       f_okReadReg        ( TuwRegWord* const          _kpuwReadData = NULL ) ;

         ESuccess       f_okWriteReg       ( const TuwRegWord* const    _kpuwWriteData ) ;
    
         ESuccess       f_okWriteOnly      ( const TuwRegWord* const    _kpuwWriteData) ; //(no Verify)
    
         ESuccess       f_okVerifyReg      (  ) ;
    
         ESuccess       f_okSaveContent    ( TuwRegWord* const     _kpuwSavedData = NULL ) ;
    
         ESuccess       f_okMultiRead      ( TuwRegWord* const     _kpuwReadData,
                                             const unsigned long   _kulRequestCount,
                                             const TuwRegWord      _kuiCondMask = 0x0000,
                                             const TuwRegWord      _kuiCondValue = 0x0000,
                                             const unsigned int    _kuiMaxTry = 1, 
                                             unsigned int*         _kpuiTotMatch = NULL, 
                                             unsigned int*         _kpuiTotReadTry = NULL, 
                                             CResultFile*          _poAdcDumpFile = NULL ) ;
    
         ESuccess       f_okMultiWrite     ( TuwRegWord* const     _kpuwWriteData,
                                             const unsigned long   _kulRepeatCount ) ;

         ESuccess       f_okForceRegBits   ( TuwRegWord const      _kuwBitMask ) ;
         ESuccess       f_okClearRegBits   ( TuwRegWord const      _kuwBitMask ) ;
         ESuccess       f_okPulseBitsHigh  ( TuwRegWord const      _kuwBitMask ) ;
         ESuccess       f_okPulseBitsLow   ( TuwRegWord const      _kuwBitMask ) ;

         ESuccess       f_okReadBlock      ( TuwRegWord* const     _pauwArray,
                                             const unsigned long   _kulArraySize,
                                             const unsigned long   _kulRegAddrIncr = 0,
                                             TuwRegWord* const     _puwMaxValue = NULL,
                                             CResultFile*          _poDumpFile = NULL ) ;

         ESuccess       f_okReadAverage    ( double*               _pfAverage,
                                             const unsigned long   _kulSampleSize ) ;

         TCString       f_osVmeAddress     ( ) ; // e.g. return "@VmeAddr 0x01020000"
         TCString       f_osRegAddress     ( ) ; // e.g. return "@Master#0/Slave#1/Slot# 2/Chip# 0/Reg#  0"

         TuwRegWord     f_uwReadMask       ( )      { return m_uwReadMask; } ;
         TuwRegWord     f_uwWriteMask      ( )      { return m_uwWriteMask; } ;
         CAdfRegBase*      f_poNextReg        ( )      { return m_poNextReg; } ;
         CAdfRegBase*      f_poPrevReg        ( )      { return m_poPrevReg; } ;
         TulVmeAddress  f_ulVmeAddress     ( )      { return m_ulVmeAddress; } ;
         TuwRegWord     f_uwLastWrite      ( )      { return m_uwLastWrite; } ;
         TuwRegWord     f_uwLastRead       ( )      { return m_uwLastRead; } ;
         TuwRegWord     f_uwSavedContent   ( )      { return m_uwSavedContent; } ;

         void           f_vClearModified   ( )       { m_ynModified = No ; return ; } ;
         EYesNo         f_ynModified       ( )       { return m_ynModified; } ;

         ESuccess       f_okAddToReadMask ( TuwRegWord _uwMoreReadMaskBits )      
            { m_uwReadMask |= _uwMoreReadMaskBits ; return Ok ; } ;

         ESuccess       f_okAddToWriteMask ( TuwRegWord _uwMoreWriteMaskBits )      
            { m_uwWriteMask |= _uwMoreWriteMaskBits ; return Ok ; } ;

         ESuccess       f_okSetFlagVolatile ( )     { m_ynVolatile = Yes ; return Ok ; } ;

  static TulVmeAddress  ConvertToVmeAddress( unsigned char      _ubVertMaster,
                                             unsigned char      _ubVertSlave,
                                             unsigned char      _ubCardSlot,
                                             unsigned char      _ubChipAddress = 0,
                                             unsigned short     _uwRegAddress  = 0 ) ;   

  static TulVmeAddress  ConvertToVmeOffset ( unsigned char      _ubChipAddress,
                                             unsigned short     _uwRegAddress  = 0 ) ;   

         void           UnpackVmeAddress   ( unsigned char      *_ubVertMaster,
                                             unsigned char      *_ubVertSlave,
                                             unsigned char      *_ubCardSlot,
                                             unsigned char      *_ubChipAddress,
                                             unsigned short     *_uwRegAddress ) ;

        // Read the same Address a number of times
        ESuccess        f_okHisto10bits      ( TAdcHisto* const        _kpauwHistogram,
                                               const unsigned long     _kulSampleSize,
                                               const unsigned long     _kulRegAddrIncr = 0,
                                               CResultFile*            _poAdcDumpFile = NULL ) ;

        // Histogramming support functions
static  double          HistoAverage10bit    ( TAdcHisto* const        _kpauwHistogram,
                                               const unsigned long     _kulSampleSize ) ;
static  double          HistoDeviation10bit  ( TAdcHisto* const        _kpauwHistogram,
                                               const unsigned long     _kulSampleSize,
                                               const double            _kfAverage ) ;
static  unsigned long   HistoTailPopul10bit  ( TAdcHisto* const        _kpauwHistogram,
                                               const unsigned long     _kulTailStartAt ) ;

static  double          HistoAverage8bit     ( TAdfEtHisto* const      _kpauwHistogram,
                                               const unsigned long     _kulSampleSize ) ;
static  double          HistoDeviation8bit   ( TAdfEtHisto* const      _kpauwHistogram,
                                               const unsigned long     _kulSampleSize,
                                               const double            _kfAverage ) ;
static  unsigned long   HistoTailPopul8bit   ( TAdfEtHisto* const      _kpauwHistogram,
                                               const unsigned long     _kulTailStartAt ) ;


         void           Dump               ( bool               _lIncludeReadOnly = true,
                                             CResultFile*       _poResultFile = NULL ) ;
/* 
    ESuccess f_okGetHistogram   ( TuwRegWord*   _auwReadHist ) ;

    ESuccess f_okInitialize     (  ) ;
    ESuccess f_okDefault        (  ) ;
    ESuccess f_okSaveProgr      (  ) ;
    ESuccess f_okRestoreProgr   (  ) ;
    ESuccess f_okSetHsroDepth   (  ) ;
    ESuccess f_okSetMonitDepth  (  ) ;
    ESuccess f_okModifEntry     (  ) ;
*/ 

protected:

///////////////////////////////////////////
// Access Privileges outside of this class

//special access privilege so that the derived register classes can access private data members
friend class CAdfRegRawAdc ;
friend class CAdfRegEtLookup ;
friend class CAdfRegOutputEt ;
};

/////////////////////////////////////////////
// Macro to cleanly remove a register
/////////////////////////////////////////////
#define DELETE_REGISTER( REGISTER_POINTER ) \
    REGISTER_POINTER->RemoveFromChain () ;  \
    delete REGISTER_POINTER ;               \
    REGISTER_POINTER = NULL ;               \
/////////////////////////////////////////////

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
#endif // __AdfRegBase__
