/*-------------------------------------------------------------------------------*/
  /*
    CustomDataTypes.h
    23-April-98
    19-Nov-98  Remove pre-processor compiled constants, now in GlobalConstants.h
  */
/*-------------------------------------------------------------------------------*/
#if defined (__CUSTOM_DATA_TYPES__) // prevent double definition
#else
    #define __CUSTOM_DATA_TYPES__
/*-------------------------------------------------------------------------------*/

    // define a 64 bit integer variable
    #ifdef WIN32

        typedef     __int64                 int64;
        typedef     unsigned __int64        uint64;

    #elif defined Linux

        typedef     long long int           int64;
        typedef     unsigned long long int  uint64;

    #endif

/*-------------------------------------------------------------------------------*/

    // This is a personal definition of Boolean as an enumerated type
    // (instead of the VC++ BOOLEAN which is an integer, and thus not
    // letting the compiler detect type mismatches).
    enum EBoolean
    {
        False,      /* false = 0, true = 1 , similar to BOOLEAN*/
        True
    };


    // This is a personal enumerated Type
    // used to avoid the confusion of the multiple variants of Boolean
    enum EYesNo
    {
        No,         /* No = 0, Yes = 1 , values picked similar to Boolean*/
        Yes
    };


    // This is a personal enumerated type
    // used for error return which avoid using a Boolean
    enum ESuccess
    {
        Ok,         /* Ok = 0 */
        Error,      /* Error = 1 */
        Warning     /* Warning = 2 */
    };


    // Register Data Type: 16 bit word
    typedef     unsigned short      TuwRegWord ;

    // CBus Register Data Type: 8 bit word
    typedef     unsigned char       TubCBusRegByte ;

    // VME Address Data Type: 32 bit unsigned longword
    typedef     unsigned long       TulVmeAddress ;

    //define a type for Specific Trigger Numbers (0..128)
//    typedef     unsigned long       TSpTrgNum ;

    //define a type for And-Or Input Term Numbers (0..256)
//    typedef     unsigned long       TAoTermNum ;

    // General purpose recording of signal state,
    // including the case where the signal is not driven to a known state
    enum ESignalState
    {
        Negated             = 0,
        Asserted            = 1,
        State_Unknown       = 2
    } ;

    // Choices of And-Or Input Term Programming for each Term, and each Specific Trigger
    // Important Note: The values for these enumerated values must follow
    // the Signal State enumerated type values (for the single chance test)
    enum EAoTermProg
    {
        Require_False       = Negated,
        Require_True        = Asserted,
        Dont_Care           = State_Unknown
    } ;

    //Generic Obey/Ignore Status, e.g. for a SpTrg's source of disable
    //also used to specify programming of FOM terms
    enum EIgnoreObey
    {
        Ignore              = 0,
        Obey                = 1,
        Unspecified         = 2
    } ;

    //Generic Enabled/Disabled Status, e.g. for a SpTrg's Coor enable status
    enum EDisableEnable
    {
        Disable             = 0,
        Enable              = 1
    } ;


/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/

    // Trigger Tower Eta Sign:
    // Positive or Negative
    enum ETT_Eta_Sign
    {
        eTT_Eta_Pos         = 0,
        eTT_Eta_Neg         = 1,
        eTot_TT_Eta_Sign    = 2
    } ;

    // Trigger Tower Eta Magnitude:
    enum ETT_Eta_Magn
    {
        eTT_Eta_1           = 1,
        eTT_Eta_2           = 2,
        eTT_Eta_3           = 3,
        eTT_Eta_4           = 4,
        eTT_Eta_5           = 5,
        eTT_Eta_6           = 6,
        eTT_Eta_7           = 7,
        eTT_Eta_8           = 8,
        eTT_Eta_9           = 9,
        eTT_Eta_10          = 10,
        eTT_Eta_11          = 11,
        eTT_Eta_12          = 12,
        eTT_Eta_13          = 13,
        eTT_Eta_14          = 14,
        eTT_Eta_15          = 15,
        eTT_Eta_16          = 16,
        eTT_Eta_17          = 17,
        eTT_Eta_18          = 18,
        eTT_Eta_19          = 19,
        eTT_Eta_20          = 20,
        eTot_TT_Eta         = 20
    } ;

    enum ETT_Eta_Magn_By4
    {
        eTT_Eta_1_4         = 0,
        eTT_Eta_5_8         = 1,
        eTT_Eta_9_12        = 2,
        eTT_Eta_13_16       = 3,
        eTT_Eta_17_20       = 4,
        eTot_TT_Eta_By4     = 5
    } ;

    enum ETT_Eta_Magn_By8
    {
        eTT_Eta_1__8         = 0,
        eTT_Eta_9__16        = 1,
        eTT_Eta_17__20       = 2, // double underscore to differentiate from ETT_Eta_Magn_By8 value
        eTot_TT_Eta_By8      = 3
    } ;

    // Trigger Tower Phi
    enum ETT_Eta_Phi
    {
        eTT_Phi_1           = 1,
        eTT_Phi_2           = 2,
        eTT_Phi_3           = 3,
        eTT_Phi_4           = 4,
        eTT_Phi_5           = 5,
        eTT_Phi_6           = 6,
        eTT_Phi_7           = 7,
        eTT_Phi_8           = 8,
        eTT_Phi_9           = 9,
        eTT_Phi_10          = 10,
        eTT_Phi_11          = 11,
        eTT_Phi_12          = 12,
        eTT_Phi_13          = 13,
        eTT_Phi_14          = 14,
        eTT_Phi_15          = 15,
        eTT_Phi_16          = 16,
        eTT_Phi_17          = 17,
        eTT_Phi_18          = 18,
        eTT_Phi_19          = 19,
        eTT_Phi_20          = 20,
        eTT_Phi_21          = 21,
        eTT_Phi_22          = 22,
        eTT_Phi_23          = 23,
        eTT_Phi_24          = 24,
        eTT_Phi_25          = 25,
        eTT_Phi_26          = 26,
        eTT_Phi_27          = 27,
        eTT_Phi_28          = 28,
        eTT_Phi_29          = 29,
        eTT_Phi_30          = 30,
        eTT_Phi_31          = 31,
        eTT_Phi_32          = 32,
        eTot_TT_Phi         = 32
    } ;

    enum ETT_Eta_Phi_By8
    {
        eTT_Phi_1_8         = 0,
        eTT_Phi_9_16        = 1,
        eTT_Phi_17_24       = 2,
        eTT_Phi_25_32       = 3,
        eTot_TT_Phi_By8     = 4
    } ;

    enum ETT_Eta_Phi_By4
    {
        eTT_Phi_1_4         = 0,
        eTT_Phi_5_8         = 1,
        eTT_Phi_9_12        = 2,
        eTT_Phi_13_16       = 3,
        eTT_Phi_17_20       = 4,
        eTT_Phi_21_24       = 5,
        eTT_Phi_25_28       = 6,
        eTT_Phi_29_32       = 7,
        eTot_TT_Phi_By4     = 8
    } ;

    // 2x Types of Trigger Tower Channels
    enum ETT_Chan_Type
    {
        eTT_EM              = 0,
        eTT_HD              = 1,
        eTot_TT_Chan_Type   = 2
    } ;

    // Types of Data the ADF Raw ADC Stage can send to the Filter
    enum ERawAdc_MuxMode_Type 
    {
        eRawAdc_MuxMode_BlsAligned        = 0,
        eRawAdc_MuxMode_SimuTurn          = 1,
        eTot_RawAdc_MuxMode_Type          = 2
    } ;

    // Types of Usage for the Memory of the ADF Raw ADC Stage
    enum ERawAdc_MemMode_Type
    {
        eRawAdc_MemMode_CaptureMonitor    = 0,
        eRawAdc_MemMode_CaptureAtThresh   = 1,
        eRawAdc_MemMode_CaptureImmediate  = 2,
        eRawAdc_MemMode_PlaySimuTurn      = 3,
        eTot_RawAdc_MemMode_Type          = 4
    } ;

    // Types of Data the ADF Output Et Stage can send to the TABs
    enum EAdfOut_MuxMode_Type
    {
        eAdfOut_MuxMode_Filtered_Always   = 0,
        eAdfOut_MuxMode_Filtered_LiveX    = 1,
        eAdfOut_MuxMode_Constant_Always   = 2,
        eAdfOut_MuxMode_Prn_Always        = 3,
        eAdfOut_MuxMode_Prn_LiveX         = 4,
        eAdfOut_MuxMode_SimuTurn_Always   = 5,
        eAdfOut_MuxMode_SimuTurn_LiveX    = 6,
        eTot_AdfOut_MuxMode_Type          = 7
    } ;

    // Types of Usage for the Memory of the ADF Raw ADC Stage
    enum EAdfOut_MemMode_Type
    {
        eAdfOut_MemMode_CaptureMonitor    = 0,
        eAdfOut_MemMode_CaptureAtThresh   = 1,
        eAdfOut_MemMode_CaptureImmediate  = 2,
        eAdfOut_MemMode_PlaySimuTurn      = 3,
        eTot_AdfOut_MemMode_Type          = 4
    } ;


    // 3x Types of Reference Comparators 
    enum ETabRefSet_Type
    {
        eTab_RefSet_EMEt      = 0,
        eTab_RefSet_JetEt     = 1,
        eTab_RefSet_TTEt      = 2,
        eTot_Tab_RefSet_Type  = 3
    } ;

    // 3x Types of Algorithms
    enum ETabMask_Type
    {
        eTab_Mask_EM          = 0,
        eTab_Mask_Jet         = 1,
        eTab_Mask_Tau         = 2,
        eTot_Tab_Mask_Type    = 3
    } ;

    // 7x Ref Set of Each Type
    enum ETT_RefSet_Num
    {
        eTT_RefSet_0        = 0,
        eTT_RefSet_1        = 1,
        eTT_RefSet_2        = 2,
        eTT_RefSet_3        = 3,
        eTT_RefSet_4        = 4,
        eTT_RefSet_5        = 5,
        eTT_RefSet_6        = 6,
        eTot_TT_RefSet      = 7,
        eTot_TT_RefSet_Obj  = 7,
        eTot_TT_RefSet_TT   = 3
    } ;


    // This is an enumerated type
    // to report the results of a comparison (e.g. CAT2, or CAT3)
    enum EComparator_State
    {
        State_EQ_,         /* Equal To              = 0 */
        State_LT_,         /* Strictly Lesser Than  = 1 */
        State_GT_,         /* Strictly Greater Than = 2 */
        State_Illegal_     /* Illegal State         = 3 */
    };

    // define Return values for Coor Message Handling
    enum EGuiCommandStatus
    {
        GuiReply_Ok,       /* GuiReply_Ok   = 0 */
        GuiReply_Bad,      /* GuiReply_Bad  = 1 */
        GuiReply_None      /* GuiReply_None = 2 */
    };

    // Types of Find_DAC bahaviors
    enum EFindDacMode_Type
    {
        eFindDacMode_All636    = 0,
        eFindDacMode_LiveX     = 1,
        eTot_FindDacMode_Type  = 2
    } ;

    // Types of Run State
    enum ERun_State
    {
        eRun_Undefined = 0,
        eRun_Defined   = 1,
        eRun_Running   = 2,
        eRun_Paused    = 3,
        eRun_Done      = 4,
        eRun_Aborted   = 5
    } ;

    // Types of Event State
    enum EEvt_State
    {
        eEvt_Undefined = 0,
        eEvt_Arming    = 1,
        eEvt_Waiting   = 2,
        eEvt_Reading   = 3,
        eEvt_Writing   = 4,
        eEvt_Done      = 5,
        eEvt_CoolDown  = 6
    } ;

/*-------------------------------------------------------------------------------*/
#endif //__CUSTOM_DATA_TYPES__

