#if !defined ( BIT3_ADAPTOR_61X_ALREADY_DEFINED )
    #define    BIT3_ADAPTOR_61X_ALREADY_DEFINED
/*-------------------------------------------------------------------------------*/
  /*
    HandleBit3Adaptor61x.h
    Class CBit3Adaptor61x
    30-Apr-1999
  */
/*-------------------------------------------------------------------------------*/

#define     KiDevNameStringLength       32 

typedef enum { VmeA16,  VmeA24,  VmeA32,  Dpm }             EVmeAmodAddrSpace ;
typedef enum { VmeAmodSuper, VmeAmodNonPriv, VmeAmodNevis } EVmeAmodPrivilege ;
typedef enum { VmeAmodProgr, VmeAmodData, VmeAmodBlock }    EVmeAmodAccessType ;
typedef enum { VmeRead,  VmeWrite,  VmeReadWrite }          EVmeAccessRestrict ;

#define     Ki_BT_AMOD_NEVIS    27      // User defined Address Modifier for Nevis Tab Crate

class CBit3Adaptor61x ;//forward declaration

// Interrupt Service Routine Parameters
struct XInterruptParameters
{
	CBit3Adaptor61x*	pBit3Adaptor ;
	ESuccess			eVmeStatus ;	
} ;

class CBit3Adaptor61x
{

/////////////////////////////
// Constructor/Destructor
public:

    CBit3Adaptor61x();

    CBit3Adaptor61x   ( unsigned int            _uiDevUnitNum,
                        EVmeAmodAddrSpace       _eAddrSpace    = VmeA32,
                        EVmeAmodPrivilege       _eAmodPriv     = VmeAmodSuper,
                        EVmeAmodAccessType      _eAmodType     = VmeAmodData,
                        EVmeAccessRestrict      _eRestrict     = VmeReadWrite ) ;

   ~CBit3Adaptor61x();

/////////////////////////////
// Attributes
public:

    // this is the Bit3 Adaptor Number that is connected to various sub-systems
    static int              mg_iAdaptorNumForL1Cal ;

    // Bit3 Adaptor Card Name as required by Bit3 API
    char                    m_sDevName[KiDevNameStringLength+1] ;

    // Adaptor Status
    ESuccess                m_eStatus ;

    // Interrupt Service Routine Parameters
    XInterruptParameters	m_xIcbrParameters ;

    // Index of the Adaptor on the PCI bus.
    // This is not directly related to the PCI slot number.
    unsigned int            m_uiAdaptorNum ;

    // Resource Type A16, A24, A32, or DPM
    EVmeAmodAddrSpace       m_eAddrSpace ;

    // Address Modifier Information besides address space
    EVmeAmodPrivilege       m_eAmodPriv ;
    EVmeAmodAccessType      m_eAmodType ;

    // resulting Addres Modifier
    int                     m_iAmod ;

    // Access Mode: Read, Write or RW
    EVmeAccessRestrict      m_eAccessRestrict ;

private:

    // Handle to Bit3 Adaptor
    void*                   m_pBit3Descriptor ;

/////////////////////////////
// Operations
public:

    // Look for the existence of a particular adapter (without generating error messages)
static ESuccess ProbeForAdaptor           ( unsigned int    _uiAdaptorNum ) ;

    // Map an arbitrary segment of VME memory 
    // Return pointer to mapped resource on Success, and NULL on failure
    void*       MemoryMapThisVmeSegment   ( TulVmeAddress   _ulBaseAddress,
                                            TulVmeAddress   _ulByteLength,
                                            EYesNo          _ynByteSwapping = No ) ;

    // Un-Map a previously mapped memory segment
    ESuccess    UnMapThisVmeSegment       ( void*               _pMappedAddress,
                                            TulVmeAddress       _ulByteLength ) ;

    // Check if povided buffer is aligned, and return offset if desired
    ESuccess    CheckBufferAligned        ( void*           _pBuffer,
                                            unsigned long*  _pAlignOff = NULL ) ;


    // Call to Bit3 API to read/write a word of a given size at the specified address
    // with a NumBytes argument of 1 it will be 1x D8 transfer
    //                             2 it will be 1x D16 transfer
    //                             4 it will be 1x D32 transfer
    // The BT_INFO_DATAWIDTH variable accessible from bt_set_info()
    // can change the above default behavior
    // cf. documentation for behavior with other transfer sizes,
    //     which will result in more than one VME transfer
    ESuccess    ReadVmeData    ( TulVmeAddress          _ulVmeAddress, 
                                 void* const            _kpvReadData,
                                 unsigned int const     _kuiNumBytes ) ;

    ESuccess    WriteVmeData   ( TulVmeAddress          _ulVmeAddress, 
                                 void* const            _kpvWriteData,
                                 unsigned int const     _kuiNumBytes ) ;

    // Read and look for errors in Bit3 Status 
    ESuccess    CheckBit3Status ( ) ;

    // Clear and Check Bit3 Status 
    ESuccess    ClearBit3Status ( ) ;

private:

    // Build a Descriptive ASCII string to identify one interface
    ESuccess    BuildBit3AdaptorName ( ) ;

    // Open Handle to Bit3 Adaptor
    ESuccess    OpenBit3Adaptor ( ) ;

    // Close Handle to Bit3 Adaptor
    ESuccess    CloseBit3Adaptor ( ) ;

    // Initialize the adaptor
    ESuccess    InitBit3Adaptor ( ) ;

    // Just look if remote crate is powered and cable connected,
    // with no screen message about power being on/off.
    ESuccess    Bit3Powered ( ) ;

    // registering of Interrupt Service Routine with Bit3 Device Driver
    ESuccess    RegisterBit3Interrupts ( ) ;

    // Remove the Interrupt Service Routine 
    ESuccess    RemoveBit3Interrupts ( ) ;

    // translate a Bit3 error code into an error message
    CString     GetBit3ErrorMessage ( long _zStatus ) ;

};


// Bit3 Adaptor Number that is connected to each sub-system
#define     KiAdaptorNumForL1FW     0
#define     KiAdaptorNumForL1Cal     KiAdaptorNumForL1FW

#define     KiMaxBit3AdaptorNumber  9   //Maximum Bit3 Adaptor Number (i.e. 10 Model 617/618 Max)
/*-------------------------------------------------------------------------------*/
#endif // defined BIT3_ADAPTOR_61X_ALREADY_DEFINED
