/*----------------------------------------------------------------------------------*/
// Modified 19-Mar-98 : Add Keywords Register_Address and Write_Value
// Modified           : Add Keywords Read_Register_Address and Write_Verify
// Modified           : Add Keywords Source_Mask and Receiver_Mask
// Modified           : Add Keywords To_Chip_Address and To_Register_Address
// Modified 12-Jun-98 : Add Keywords Source_Description
// Modified 28-Oct-98 : Add variables for variables in command files
/*----------------------------------------------------------------------------------*/

#include "L1Cal_IIb_TCC/HandleSymbols.h"

#define KiMaxCommandLineLength      132
#define diNotYetAssigned            0xffffffff

// pointer to function returning Boolean
typedef ESuccess fKeyword_Action ( void* _pxParse, 
                                   void* _pxCustom,
                                   void* _pObject ) ;

struct Parse_Structure
{
    const char*      pacFile_Name ;

    int              iLineNum ;

    struct Parse_Structure_Values
    {
	    unsigned long  ulCrate ;
	    unsigned long  ulT962_Wire ;
	    unsigned long  ulT962_Maestro_Slot ;
        unsigned long  ulVertical_Master ;
	    unsigned long  ulVertical_Slave ;
	    unsigned long  ulCard_Slot ;
	    unsigned long  ulTo_Card_Slot ;
	    unsigned long  ulChip_Address ;
	    unsigned long  ulTo_Chip_Address ;
	    unsigned long  ulChip_Mask ;
	    unsigned long  ulRegister_Address ;
	    unsigned long  ulTo_Register_Address ;
	    unsigned long  ulRegister_Start ; // for legacy compatibility
	    unsigned long  ulRegister_Count ; // for legacy compatibility
	    unsigned long  ulRead_Register_Address ;
	    unsigned long  ulWrite_Value ;
	    unsigned long  ulWrite_Verify ;
	    unsigned long  ulSource_Mask ;
	    unsigned long  ulReceiver_Mask ;
	    unsigned long  ulConfig_TabGab ;
	    unsigned long  ulTabGab_Module ;
	    unsigned long  ulGroup_Address ;
	    unsigned long  ulSerial_Address ;
	    unsigned long  ulSerial_Data_Write ;
	    unsigned long  ulSerial_Address_Read ;
        unsigned long  ulBit3_Adaptor ;
        unsigned long  ulAddress_Space ;
        unsigned long  ulData_Size ;
	    unsigned long  ulVME_Address ;
	    unsigned long  ulRead_VME ;
	    unsigned long  ulTT_Channel ;
        unsigned long  ulTT_Eta_Sign ;
        unsigned long  ulTT_Eta_Magn ;
        unsigned long  ulTT_Phi ;
        unsigned long  ulTo_TT_Eta_Sign ;
        unsigned long  ulTo_TT_Eta_Magn ;
        unsigned long  ulTo_TT_Phi ;
        unsigned long  ulTick ; 
        unsigned long  ulTo_Tick ; 
        unsigned long  ulSubTick ; 
        unsigned long  ulTo_SubTick ; 
        unsigned long  ulAlignment_Delay ;
        unsigned long  ulPedestal_DAC ;
        double         fdPed_Average ;
        double         fdPed_Deviation ;
        unsigned long  ulZeroEngAdcE ;
        unsigned long  ulZeroEngOutEt ;
        double         fdLookup_Slope ;
        unsigned long  ulRawAdc_Simu ;
        unsigned long  ulAdfOut_Simu ;
        unsigned long  ulAdfOut_Const ;
        unsigned long  ulRawAdc_Thresh ;
        unsigned long  ulRawAdc_MuxMode ;
        unsigned long  ulRawAdc_MemMode ;
        unsigned long  ulAdfOut_MuxMode ;
        unsigned long  ulAdfOut_MemMode ;

        unsigned long  ulLBN_Value ;
        unsigned long  ulL1Cal_Operational ;
        unsigned long  ulMonitControlTimeMs ;
        unsigned long  ulBaseAlignmentDelay ;
        unsigned long  ulOffsetTick1RawAdcMonit ;
        unsigned long  ulOffsetTick1RawAdcSimu ;
        unsigned long  ulOffsetTick1OutputEtMonit ;
        unsigned long  ulOffsetTick1OutputEtSimu ;
        unsigned long  ulOffsetTriggeredToTickAtCapture ;
        unsigned long  ulTtiImmediateIo ;
        unsigned long  ulFindDacMode ;
        unsigned long  ulTabEta38Value ;
        unsigned long  ulMaxDeSkewTryAtInit ;
        unsigned long  ulAbortInitOnError ;
        unsigned long  ulSkipTabGabOnInit ;
        unsigned long  ulMonitIgnoreTabGab ;

        CString        osSource_Description ;
	    CString        osGeneric_String_Parameter ;
    } Values ; // update Init_Parse_Structure when changing this

    struct Parse_Structure_Actions
    {
        fKeyword_Action*  pfFile_Opened ; // this additional hook is not a keywords

        fKeyword_Action*  pfCrate ;
        fKeyword_Action*  pfT962_Wire ;
        fKeyword_Action*  pfT962_Maestro_Slot ;
        fKeyword_Action*  pfVertical_Master ;
	    fKeyword_Action*  pfVertical_Slave ;
	    fKeyword_Action*  pfCard_Slot ;
	    fKeyword_Action*  pfTo_Card_Slot ;
	    fKeyword_Action*  pfChip_Address ;
	    fKeyword_Action*  pfTo_Chip_Address ;
	    fKeyword_Action*  pfChip_Mask ;
	    fKeyword_Action*  pfRegister_Address ;
	    fKeyword_Action*  pfTo_Register_Address ;
	    fKeyword_Action*  pfRegister_Start ; // for legacy compatibility
	    fKeyword_Action*  pfRegister_Count ; // for legacy compatibility
	    fKeyword_Action*  pfRead_Register_Address ;
	    fKeyword_Action*  pfWrite_Value ;
	    fKeyword_Action*  pfWrite_Verify ;
	    fKeyword_Action*  pfSource_Mask ;
	    fKeyword_Action*  pfReceiver_Mask ;
	    fKeyword_Action*  pfConfig_TabGab ;
	    fKeyword_Action*  pfTabGab_Module ;
	    fKeyword_Action*  pfGroup_Address ;
	    fKeyword_Action*  pfSerial_Address ;
	    fKeyword_Action*  pfSerial_Data_Write ;
	    fKeyword_Action*  pfSerial_Address_Read ;
        fKeyword_Action*  pfBit3_Adaptor ;
        fKeyword_Action*  pfAddress_Space ;
        fKeyword_Action*  pfData_Size ;
	    fKeyword_Action*  pfVME_Address ;
	    fKeyword_Action*  pfRead_VME ;
	    fKeyword_Action*  pfSource_Description ;

	    fKeyword_Action*  pfConfig_File ;
	    fKeyword_Action*  pfTabGab_PackageVersion ;
	    fKeyword_Action*  pfTabGab_FirmwareDir ;
	    fKeyword_Action*  pfTabGab_ConfigDir ;
	    fKeyword_Action*  pfTabGab_WorkDir ;

	    fKeyword_Action*  pfTT_Channel ;
	    fKeyword_Action*  pfTT_Eta_Sign ;
	    fKeyword_Action*  pfTT_Eta_Magn ;
	    fKeyword_Action*  pfTT_Phi ;
	    fKeyword_Action*  pfTo_TT_Eta_Sign ;
	    fKeyword_Action*  pfTo_TT_Eta_Magn ;
	    fKeyword_Action*  pfTo_TT_Phi ;
        fKeyword_Action*  pfTick ; 
        fKeyword_Action*  pfTo_Tick ; 
        fKeyword_Action*  pfSubTick ; 
        fKeyword_Action*  pfTo_SubTick ; 
        fKeyword_Action*  pfAlignment_Delay ;
        fKeyword_Action*  pfPedestal_DAC ;
        fKeyword_Action*  pfPed_Average  ;
        fKeyword_Action*  pfPed_Deviation ;
        fKeyword_Action*  pfZeroEngAdcE ;
        fKeyword_Action*  pfZeroEngOutEt ;
        fKeyword_Action*  pfLookup_Slope ;
        fKeyword_Action*  pfRawAdc_Simu ;
        fKeyword_Action*  pfAdfOut_Simu ;
        fKeyword_Action*  pfAdfOut_Const ;
        fKeyword_Action*  pfRawAdc_Thresh ;
        fKeyword_Action*  pfRawAdc_MuxMode ;
        fKeyword_Action*  pfRawAdc_MemMode ;
        fKeyword_Action*  pfAdfOut_MuxMode ;
        fKeyword_Action*  pfAdfOut_MemMode ;

	    fKeyword_Action*  pfLBN_Value ;
	    fKeyword_Action*  pfL1Cal_Operational ;
	    fKeyword_Action*  pfMonitControlTimeMs ;
	    fKeyword_Action*  pfBaseAlignmentDelay ;
	    fKeyword_Action*  pfOffsetTick1RawAdcMonit ;
	    fKeyword_Action*  pfOffsetTick1RawAdcSimu ;
	    fKeyword_Action*  pfOffsetTick1OutputEtMonit ;
	    fKeyword_Action*  pfOffsetTick1OutputEtSimu ;
	    fKeyword_Action*  pfOffsetTriggeredToTickAtCapture ;
	    fKeyword_Action*  pfTtiImmediateIo ;
	    fKeyword_Action*  pfFindDacMode ;
	    fKeyword_Action*  pfTabEta38Value ;
	    fKeyword_Action*  pfMaxDeSkewTryAtInit ;
	    fKeyword_Action*  pfAbortInitOnError ;
	    fKeyword_Action*  pfSkipTabGabOnInit ;
	    fKeyword_Action*  pfMonitIgnoreTabGab ;

        fKeyword_Action*  pfCall_Reg_IO ;
        fKeyword_Action*  pfCall_Serial_IO ;
	    fKeyword_Action*  pfCall_Vme_IO ;
	    fKeyword_Action*  pfCall_Cbus_IO ;
	    fKeyword_Action*  pfCall_Config_Fpga ;
	    fKeyword_Action*  pfCall_Self_Msg ;
	    fKeyword_Action*  pfCall_TT_Info ;
	    fKeyword_Action*  pfCall_Config_T962 ;

	    fKeyword_Action*  pfSend_Msg_To_Self ;

	    fKeyword_Action*  pfDone_With_File ; // this additional hook is not a keywords

    } Actions ; // update Init_Parse_Structure when changing this

    CSymbolSet  oSymbolSet ;

} ;

/*----------------------------------------------------------------------------------*/
// Open a command file on disk and execute actions passed as arguments 
/*----------------------------------------------------------------------------------*/
ESuccess Read_Command_File( Parse_Structure*    _pxParse, 
                            void*               _pxCustom,
                            void*               _pObject ) ;


/*----------------------------------------------------------------------------------*/
// Initialize a Parse_Structure 
/*----------------------------------------------------------------------------------*/
ESuccess Init_Parse_Structure ( Parse_Structure*    _pxParse ) ;

