/*-------------------------------------------------------------------------------*/
  /*
    HandleLogFile.h
    Class CLogFile
     26-oCT-98
  */
#ifndef __HandleLogFile__
#define __HandleLogFile__
/*-------------------------------------------------------------------------------*/

#include "L1Cal_IIb_TCC/UtilsStrings.h"

#if !defined ( NO_MS_CONSOLE )
    #include <time.h>
#endif

// default Maximum LogFile Message Length, 
// we chose 2x80 lines, same as for console
#define KiLogFileMsgLengthDefault           160

class CLogFile
{
/////////////////////////////
// Constructor/Destructor
public:

	 CLogFile();


     ~CLogFile();

/////////////////////////////
// Attributes
public:

private:

    static CString          m_osOriginalFile_Name ; // Original log file name
    static CString          m_osCurrentFile_Name ;  // Current log file name

    static FILE*            mg_hLogFile ;

#if !defined ( NO_MS_CONSOLE )
    static unsigned long    mg_ulFloodCountThreshold ;
    static clock_t          mg_zFloodTimeThresholdSet   ; // in units of (1 / CLOCKS_PER_SEC) seconds
    static clock_t          mg_zFloodTimeThresholdClear ; // in units of (1 / CLOCKS_PER_SEC) seconds
    static clock_t          mg_zFloodTimeLastMsg ;
    static unsigned long    mg_ulFloodLevel ;
#endif

    static EYesNo           mg_ynFloodDeclared ;

/////////////////////////////
// Operations
public:

    static    CString   NextFileName  ( ) ;

    static    ESuccess  NewLogFile    ( ) ;

    static    ESuccess  CloseLogFile  ( ) ;

    static    ESuccess  FlushLogFile  ( ) ;

    static    ESuccess  WriteLine     ( const char*       const _sString,
                                        const int               _iMsgLengthMax = KiLogFileMsgLengthDefault ) ; 

    static    CString   OriginalFileName  ( ) { return m_osOriginalFile_Name ; } ;
    static    CString   CurrentFileName   ( ) { return m_osCurrentFile_Name ; } ;

};

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
// define some console message constants

// Fixed length of all Log File Lines
#define KiLogFileLineLength                 132   

//the end of the line will show the date and time in the following format
#ifdef WIN32
    //  %%dd-mmm-yyyy hh:mm:ss.mmm
    //  12345678901234567890123456 = 26 chars 
    #define KiLogFileTimeStampLength            26
#else
    //  %%dd-mmm-yyyy hh:mm:ss
    //  1234567890123456789012 = 22 without the milliseconds
    #define KiLogFileTimeStampLength            22
#endif

//Maximum number of characters displayable on a single line
//(Longer messages will be split up)
#define KiLogFileMaxCharPerLine             KiLogFileLineLength - KiLogFileTimeStampLength

// Define All Line Prefix
#define     KiLogFilePrefixContinuation     "_$" 

/*-------------------------------------------------------------------------------*/
#endif // __HandleLogFile__
