/*-------------------------------------------------------------------------------*/
  /*
    HsroChannel.h
    Class CHsroChannel
    1-Sep-00  HSRO record structures
  */
#ifndef __HsroChannel__
#define __HsroChannel__
/*-------------------------------------------------------------------------------*/

class CHsroVrbCard ; //forward declaration
class CNevisCard   ; //forward declaration

#define     KiHsroLongInHeader           3
#define     KiHsroLongInTabEvent        93
#define     KiHsroLongInGabEvent         8
#define     KiHsroLongInTrailer          1
#define     KiHsroLongInTabPadding      11

#define     KiHsroWordInHeader           2 * KiHsroLongInHeader
#define     KiHsroWordInTabEvent         2 * KiHsroLongInTabEvent
#define     KiHsroWordInGabEvent         2 * KiHsroLongInGabEvent
#define     KiHsroWordInTrailer          2 * KiHsroLongInTrailer
#define     KiHsroWordInTabPadding       2 * KiHsroLongInTabPadding

#define     KiHsroTotLongPerTab         KiHsroLongInHeader     \
                                       +KiHsroLongInTabEvent   \
                                       +KiHsroLongInTrailer    \
                                       +KiHsroLongInTabPadding
#define     KiHsroTotLongPerGab         KiHsroLongInHeader     \
                                       +KiHsroLongInGabEvent   \
                                       +KiHsroLongInTrailer

#define     KiHsroWordInTabForParity    KiHsroWordInHeader     \
                                       +KiHsroWordInTabEvent   \
                                       +KiHsroWordInTrailer    \
                                       -1

#define     KiHsroWordInGabForParity    KiHsroWordInHeader     \
                                       +KiHsroWordInGabEvent   \
                                       +KiHsroWordInTrailer    \
                                       -1

#define     KiHsroTotWordPerTab         KiHsroTotLongPerTab+KiHsroTotLongPerTab
#define     KiHsroTotWordPerGab         KiHsroTotLongPerGab+KiHsroTotLongPerGab

//This is for a generic Card Channel, not knowing what system is readout.
//http://www.nevis.columbia.edu/~evans/l1cal/hardware/tab/l2_data.html#l2tab says 184 words
//with margin, use 100 longwords 
//first test showed 216 bytes per VRB channel, i.e. 216 words per TAB.
#define     KiHsroMaxLongPerCard       128

struct  HsroHeaderDataByField
{
    unsigned char           ubNumberObjects ;
    unsigned char           ubHeaderLength ;
    unsigned char           ubObjectLength ;
    unsigned char           ubHObjFormat ;
    unsigned char           ubL2DataType ;
    unsigned char           ubTickNumber ;
    unsigned short          uwTurnNumber ;
    unsigned char           ubVersionMajor ;
    unsigned char           ubVersionMinor ;
    unsigned char           ubProcBits ;
    unsigned char           ubL2Status ;
} ;


union HsroHeaderData
{
    HsroHeaderDataByField   xByField ;
    TuwRegWord              auwWord[KiHsroWordInHeader] ;
    unsigned long           ulLong ;
} ;

struct  HsroTrailerDataByField
{
    unsigned char           ubTickNumber ;
    unsigned char           ubL2DataType ;
    unsigned short          uwParity ;
} ;

union  HsroTrailerData
{
    HsroTrailerDataByField  xByField ;
    TuwRegWord              auwWord[KiHsroWordInTrailer] ;
    unsigned long           ulLong ;
} ;

union   HsroTabEventData
{
    TuwRegWord              auwByWord[KiHsroWordInTabEvent] ;
    unsigned long           aulByLong[KiHsroLongInTabEvent] ;
} ;

struct  OneTabHsroDataByField
{
    HsroHeaderData          xHeader ;
    HsroTabEventData        xEventData ;
    HsroTrailerData         xTrailer ;
    TuwRegWord              auwPaddingByWord[KiHsroWordInTabPadding] ;
} ;

union   OneTabHsroData
{
    OneTabHsroDataByField   xByField ;
    TuwRegWord              auwByWord[KiHsroTotWordPerTab] ;
    unsigned long           aulByLong[KiHsroTotLongPerTab] ;
} ;

union   HsroGabEventData
{
    TuwRegWord              auwByWord[KiHsroWordInGabEvent] ;
    unsigned long           aulByLong[KiHsroLongInGabEvent] ;
} ;

struct  OneGabHsroDataByField
{
    HsroHeaderData          xHeader ;
    HsroGabEventData        xEventData ;
    HsroTrailerData         xTrailer ;
} ;

union   OneGabHsroData
{
    OneGabHsroDataByField   xByField ;
    TuwRegWord              auwByWord[KiHsroTotWordPerGab] ;
    unsigned long           aulByLong[KiHsroTotLongPerGab] ;
} ;

union   OneCardHsroDataMax
{
    OneTabHsroDataByField   xByField ;
    unsigned char           aubByByte[4*KiHsroMaxLongPerCard] ;
    unsigned short          auwByWord[2*KiHsroMaxLongPerCard] ;
    unsigned long           aulByLong[KiHsroMaxLongPerCard] ; 
} ;

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/

class CHsroChannel 
{
/////////////////////////////
// Constructor/Destructor
public:

             CHsroChannel        ( ) ;

            ~CHsroChannel        ( );

        void FurtherConstruct    ( CNevisCard*         _poCardBase ) ; 

/////////////////////////////
// Attributes

public:

    //A place to record Hsro from one card
    OneCardHsroDataMax     m_xHsroData_Expected ;
    OneCardHsroDataMax     m_xHsroData_Actual ;

protected:

    //Access to Base Card 
    CNevisCard*         m_poNevisCard ;

    //Vrb Channel Coordinates
    unsigned char       m_ubVrb_Slot ;
    unsigned char       m_ubVtm_Channel ;

    unsigned short      m_uwHsroTotWord ;

    //Pointer to access corresponding VRB Card
    CHsroVrbCard*       m_poVrbCard ;

/////////////////////////////
// Operations

public:

    ESuccess            HsroReadBsfDataFromMonitReg ( ) ; 
    ESuccess            HsroReadFromVbd ( ) ; 
    ESuccess            HsroCompareData ( ) ; 

    unsigned char       f_ubVrb_Slot    () { return m_ubVrb_Slot    ; } ;
    unsigned char       f_ubVtm_Channel () { return m_ubVtm_Channel ; } ;

    unsigned short      f_uwHsroTotWord () { return m_uwHsroTotWord ; } ;

protected:

};

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
#endif // __HsroChannel__

