/*-------------------------------------------------------------------------------*/
  /*
    HsroVbdCard.h
    Class CHsroVbdCard
    8-Sep-00  VBD Card
  */
#ifndef __HsroVbdCard__
#define __HsroVbdCard__
/*-------------------------------------------------------------------------------*/

#include "L1Cal_IIb_TCC/HsroVrbCard.h"
#include "L1Cal_IIb_TCC/HsroL1CalVrbAddresses.h" // for Ki_Tot_Vrb_L1Cal_Readout 

#define     KiVbdReadoutTotLongVbdHeader       8  // 16 words = 8 longwords

//A Memory structure that we can lay on top of a pointer
//to the base address of a VRB Controller card and use to address its registers
struct Vbd_A16_Access_Csr
{
/*0000        */    unsigned short uwCsr_0 ;
/*0002        */    unsigned short uwCsr_1 ;
/*0004        */    unsigned short uwCsr_2 ;
/*0006        */    unsigned short uwCsr_3 ;
/*0008        */    unsigned short uwCrateType ;
/*000A        */    unsigned short uwAdressEventNumber ;
/*000C        */    unsigned short uwAddressCrateID ;
                    unsigned short uwReserved_0x0E ;
/*0010        */    unsigned short uwDataReadFormatControlReg ;
/*0012        */    unsigned short uwParameterReadFormatControlReg ;
/*0014        */    unsigned short uwParameterReadAdressMsWord ;
} ;

struct VbdHeaderByField
{
    unsigned long   ulTotLongWordInEvent ;
    unsigned short  uwCrateID ;
    unsigned short  uwL3TransferNum ;
} ;

union VbdHeader
{
    VbdHeaderByField xByField ;
    unsigned long   aulByLong[KiVbdReadoutTotLongVbdHeader] ;
} ;

struct WordCountInVbdByField
{
    unsigned short  uwLongWordCount ;
    unsigned short  uwReserved ;
} ;

union WordCountInVbd
{
    WordCountInVbdByField xByField ;
    unsigned long         ulByLong ;
} ;

struct VrbSectionInVbd
{
    WordCountInVbd  xLongWordCount ;
    VrbHeader       xVrbHeader ;
    VrbChannelMax   axVrbChan[2*KiHsroVrbTotVtmLink] ; // some VRBs won't use all 4x VTM links= 8*Chan
} ;

// These two structures are only used as upper limit on buffer size for bit3 mapping purpose
// because some VRBs won't get data on all 4x Chan
struct VbdReadoutByField
{
    VbdHeader       xVbdHeader ;
    VrbSectionInVbd axVrbSection[Ki_Tot_Vrb_L1Cal_Readout] ;
} ;

union Vbd_A24_Access_Buf
{
    VbdReadoutByField   xByField ;
    unsigned long       aulBulk[sizeof(VbdReadoutByField)/4] ; // maximum size
} ;

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
class CHsroVbdCard
{
/////////////////////////////
// Constructor/Destructor
public:

             CHsroVbdCard ( unsigned long       _ulVrbCardCode ) ;

             CHsroVbdCard ( TulVmeAddress       _ulA24VmeAddr,
                            CString             _osCardDescription ) ;

            ~CHsroVbdCard ( );

/////////////////////////////
// Attributes

public:

protected:

    //VBD Card ID Code
    unsigned long       m_ulSpecificCardCode ;

    //VME Access to Card A16 Base Address
    TulVmeAddress       m_ulA16VmeAddr ;    // The A16 VME Address as seen as A32 from the communication crate

    //Virtual Address for Control Registers
    Vbd_A16_Access_Csr* m_pxA16Access_Csr ; // The Memory Mapped Virtual Address for the Card Base

    //Virtual Address for List of Pointers to Word Counts
    unsigned short*     m_pauwA16Access_WordCount ;

    //Virtual Address for List of Base Adrsses of Data Blocks
    unsigned short*     m_pauwA16Access_BaseData ;

    //VME Access to Card A24 Base Address
    TulVmeAddress       m_ulA24VmeAddr ;    // The A24 VME Address as seen as A32 from the communication crate

    //Virtual Address for Data Bufer
    Vbd_A24_Access_Buf* m_pxA24Access_Buffer ;


    //Card Slot in VRB Crate
    unsigned char       m_ubCard_Slot ;

    //String describing the card functionality
    CString             m_osCardDescription ;

/////////////////////////////
// Operations

public:

    ESuccess Initialize           ( ) ;

    ESuccess Dump                 ( unsigned long   _ulTotLongDump ) ;

    ESuccess Analyze              ( ) ;

    TulVmeAddress       f_ulA24VmeAddr       () { return m_ulA24VmeAddr ; } ;
    Vbd_A24_Access_Buf* f_pxA24Access_Buffer () { return m_pxA24Access_Buffer ; } ;

protected:

};

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
// size of one VBD card (maximum) A16 address space for Lower Control Registers
#define KiHsroVbdCardA16AddrSize_Csr            0x0100

// Define some of the VRB Controller relative addresses (for cross-checking above data structure)
#define KiHsroVbdAddrReadMSWord                 0x0014  // Parameter Read Adrs Bits 31-16



// Offset to VBD card A16 address for List of Pointers to Word Counts
#define KiHsroVbdCardA16AddrOffset_WordCount    0x1000

// size of one VBD card (maximum) A16 address space for List of Pointers to Word Counts
#define KiHsroVbdCardA16AddrSize_WordCount      0x0100


// Offset to VBD card A16 address for List of Base Adrsses of Data Blocks
#define KiHsroVbdCardA16AddrOffset_BaseData     0x1800

// size of one VBD card (maximum) A16 address space for List of Base Adrsses of Data Blocks
#define KiHsroVbdCardA16AddrSize_BaseData       0x0100


// size of one VBD card (maximum) A24 address space for Data Buffer
#define KiHsroVbdCardA24AddrSize_Buffer       0x002000
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
//Control Values for Vbd_A16_Access_Csr->uwCsr_0 Register
#define KiHsroVbbMaskLoadTimeOutAndUnlockVbd    0x000C
#define KiHsroVbbMaskResetAndClearErrFifo       0x0080
#define KiHsroVbbMaskLoadTimeOutAndLockVbd      0x001C

//Control Values for Vbd_A16_Access_Csr->uwCrateType Register
#define KiHsroVbbMaskCrateType                  0x0002

//Control Values for Vbd_A16_Access_Csr->uwAdressEventNumber Register
#define KiHsroVbbMaskAddrEventNum               0xC0D2

//Control Values for Vbd_A16_Access_Csr->uwAddressCrateID Register
#define KiHsroVbbMaskAddrCrateID                0xC0D0

//Control Values for Vbd_A16_Access_Csr->uwDataReadFormatControlReg Register
#define KiHsroVbbMaskDataReadFormat             0xBBF9

//Control Values for Vbd_A16_Access_Csr->uwParameterReadFormatControlReg Register
#define KiHsroVbbMaskParamReadFormat            0xEDED

//Control Values for Vbd_A16_Access_Csr->uwParameterReadAdressMsWord Register
#define KiHsroVbbMaskParamReadMsWord            0xFF0B


//Simple Mask to strip a word down to its lower 24 bits
#define KiHsroVrbMask24bit                      0x00FFFFFF
//and shift the upper word down to the lower word
#define KiHsroVrbShiftDown16bits                16
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
#endif // __HsroVbdCard__

