/*-------------------------------------------------------------------------------*/
  /*
    HsroVrbCard.h
    Class CHsroVrbCard
    5-Sep-00  VRB Card
  */
#ifndef __HsroVrbCard__
#define __HsroVrbCard__
/*-------------------------------------------------------------------------------*/

#include "L1Cal_IIb_TCC/HsroChannel.h"

#define     KiHsroVrbTotVtmLink        4
#define     KiHsroVrbTotVrbChan        2*KiHsroVrbTotVtmLink
#define     KiHsroVrbTotVrbBuf        64

//Maximum number of VRB cards in a readout crate (this value just a guess!)
#define Ki_Max_Vrb_in_Readout_Crate                 13

// Each Word from one The Card is split on 2 channels 
// with the Lower Byte going to channel N (N even)
// and the upper Byte going to channel N+1
// This is (by choice) already a multiple of 128 bit so that the VRB doesn't add padding.
#define     KiHsroVrbTotBytePerChan    KiHsroTotWordPerCard  
#define     KiHsroVrbTotLongPerChan    10  // somehow "KiHsroTotWordPerCard / 4" comes up with 39 !!

//This is for a generic Vrb Channel, not knowing what system is readout.
// The data from THE card is split over two channels
#define     KiHsroVrbMaxLongPerChan    KiHsroMaxLongPerCard / 2

//A Memory structure that we can lay on top of a pointer 
//to the base address of a VRB card and use to address its registers
struct Vrb_A24_Access
{
/*00000000      R */    unsigned short uwVrbId ;
/*00000002      R */    unsigned short uwConfigurationSetting ;
/*00000004      R */    unsigned short uwDateCode ; // YMDD->Y=LastDigitOfYear;M=Month(Jan=1,Dec=C);DD=Day
/*00000006      R */    unsigned short uwModuleSerialNumber ;
/*00000008      R */    unsigned short uwModuleType ;
                        unsigned char  ubReserved_0x08_0x0C[0x0C-0x08-2] ; //dumb but unique name (unsused)
                                                                           //and brute calculation to
                                                                           //fill the address space gap 
/*0000000C      R */    unsigned short uwResetCode ;
/*0000000E      RW*/    unsigned short uwUserInfo ;
/*00000010      R */    unsigned short uwOutputDataFifoD16Upper ;
/*00000012      R */    unsigned short uwOutputDataFifoD16Lower ; 
                        unsigned char  ubReserved_0x12_0x18[0x18 - 0x12 - 2] ;
/*00000018      R */    unsigned short uwOutputDataFifoD32 ;
                        unsigned char  ubReserved_0x18_0x22[0x22 - 0x18 - 2] ;
/*00000022      RW*/    unsigned short uwReadoutBufferNumber ;
                        unsigned char  ubReserved_0x22_0x26[0x26 - 0x22 - 2] ;
/*00000026      RW*/    unsigned short uwReadoutBunchCrossingNumber ;
/*00000028      RW*/    unsigned short uwScanBufferNumber ;
/*0000002A      RW*/    unsigned short uwScanEeventNumber ;
                        unsigned char  ubReserved_0x2A_0x30[0x30 - 0x2A - 2] ;
/*00000030      R */    unsigned short uwScanByteCount ;
/*00000032      R */    unsigned short uwScanWordCount ;
/*00000034      R */    unsigned short uwScanLongCount ;
                        unsigned char  ubReserved_0x34_0x38[0x38 - 0x34 - 2] ;
/*00000038      R */    unsigned short uwCurrentStatus ;
/*0000003A      RW*/    unsigned short uwLatchedStatus ;
/*0000003C      RW*/    unsigned short uwVrbReset ;
                        unsigned char  ubReserved_0x3C_0x40[0x40 - 0x3C - 2] ;
/*00000040      RW*/    unsigned short uwVrbControl ;
/*00000042      RW*/    unsigned short uwVtmControl ;
/*00000044      R */    unsigned short uwMonitorStatus ;
/*00000046      R */    unsigned short uwMonitorCount ;
/*00000048      RW*/    unsigned short uwMonitorChannel ;
/*0000004A      RW*/    unsigned short uwMonitorControl ;
                        unsigned char  ubReserved_0x4A_0x70[0x70 - 0x4A - 2] ;
/*00000070      RW*/    unsigned short uwChannelEnable ;
/*00000072      RW*/    unsigned short uwEmulationModeEnable ;
                        unsigned char  ubReserved_0x72_0x76[0x76 - 0x72 - 2] ;
/*00000076      RW*/    unsigned short uwGrayCodeEnable ;
/*00000078-7E   R */    unsigned short uwVtmLinkPower[KiHsroVrbTotVtmLink] ;
/*00000080-8E   R */    unsigned short uwChannelControlWords[KiHsroVrbTotVrbChan] ;
                        unsigned char  ubReserved_0x8E_0xA0[0xA0 - 0x8E - 2] ;
/*000000A0-AE   RW*/    unsigned short uwChannelStatusEnable[KiHsroVrbTotVrbChan] ;
                        unsigned char  ubReserved_0xAE_0x300[0x300 - 0xAE - 2] ;
/*00000300-7E   RW*/    unsigned short uwBufferStartAddress[KiHsroVrbTotVrbBuf] ;
/*00000380-FE   RW*/    unsigned short uwBufferSize[KiHsroVrbTotVrbBuf] ;
} ;

struct VrbHeaderByField
{
    unsigned long   ulTotByteCount ;
    unsigned char   ubEventNumber ;
    unsigned char   ubSlotNum ;
    unsigned short  uwUserInfo ;
    unsigned short  uwConfigInfo ;
    unsigned short  uwFirmwareVersion ;
    unsigned long   ulStatusWord ;
    unsigned short  auwChanByteCount[KiHsroVrbTotVrbChan] ; // Byte Count for each of the VRB's 8 channels
} ;

union VrbHeader
{
    VrbHeaderByField xByField ;
    unsigned long   aulByLong[sizeof(VrbHeaderByField)/4] ;
} ;

union VrbChannelMax
{
    unsigned char   aubByByte[4*KiHsroVrbMaxLongPerChan] ;
    unsigned char   auwByWord[2*KiHsroVrbMaxLongPerChan] ;
    unsigned long   aulByLong[KiHsroVrbMaxLongPerChan] ; 
} ;



/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
class CAdfCardBase ; // forward declaration

class CHsroVrbCard 
{
/////////////////////////////
// Constructor/Destructor
public:

             CHsroVrbCard         ( unsigned long       _ulSpecificCardCode,
                                    CHsroVrbCard**      _ppoPrevCard ) ; 

            ~CHsroVrbCard         ( );

/////////////////////////////
// Attributes

public:

protected:

    //VRB Card ID Code
    unsigned long       m_ulSpecificCardCode ;

    //Pointer to Previous VRB Card In Crate
    CHsroVrbCard*       m_poPrevCard ; 

    //VME Access
    TulVmeAddress       m_ulA24VmeAddr ; // The A24 VME Address as seen as A32 from the communication crate
    Vrb_A24_Access*     m_pxA24Access ;  // The Memory Mapped Virtual Address for the Card Base

    //Card Slot in VRB Crate
    unsigned char       m_ubCard_Slot ;

    //Pointer to the L1Cal Cards connected to each Channel
    CNevisCard*         m_apoNevisCard[KiHsroVrbTotVtmLink] ;

    //String describing the card functionality
    CString             m_osCardDescription ;

    //A copy of the data written to the user info field 
    //that will appear in the VRB Header data
    unsigned short      m_uwUserInfo ;

/////////////////////////////
// Operations

public:

    ESuccess Initialize           ( ) ;

    unsigned char   f_ubCard_Slot ( ) { return m_ubCard_Slot ; } ;

    TulVmeAddress   f_ulA24VmeAddr( ) { return m_ulA24VmeAddr ; } ;

    CNevisCard*     f_poNevisCard ( int i ) { return m_apoNevisCard[i % KiHsroVrbTotVtmLink] ; } ;

    unsigned short  f_uwUserInfo  ( ) { return m_uwUserInfo ; } ;

static    ESuccess MergeOneChannelPair    ( VrbChannelMax*      _pxVrbChanLow,
                                            VrbChannelMax*      _pxVrbChanHig,
                                            OneCardHsroDataMax* _pxActualData,
                                            unsigned long       _ulTotLong = KiHsroVrbTotLongPerChan ) ;
protected:

//special access privilege so that the Hsro Channel Object can find and register itself
friend class CHsroChannel ;

};

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
// size of one VRB card (maximum) address space
#define KiHsroVrbCardAddressSize    0x000400

// Define Last VRB relative addresses (for cross-checking above data structure)
#define KiHsroVrbBaseAddrBuffSize   0x000380  // -000003FE Buffer 0-63 Size (bytes)


// Also define addresses used during programming of the VBD
#define KiHsroVrbOffsetWordCount    0x000032  // Address of Scan Word Count

// Also define addresses used during programming of the VBD
#define KiHsroVrbOffsetOutputFifo   0x000018  // Address of Output Data Fifo

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
//Mask to Enable the lower VRB Channel Pair, further channels can be computed by shifting
#define KiHsroVrbMaskEnableChanPair 0x0003
#define KiHsroVrbMaskEnableShiftSize     2

//Control Value for Vrb_A24_Access->uwEmulationModeEnable Register
#define KiHsroVrbNoEmulation        0x0000

//Control Value for Vrb_A24_Access->uwGrayCodeEnable Register
#define KiHsroVrbNoGrayCoding       0x0000

//Control Value for Vrb_A24_Access->uwVrbControl Register
#define KiHsroVrbEnableVrbcSignals  0x0003

//Control Values for Vrb_A24_Access->uwVrbReset Register
#define KiHsroVrbResetVrb           0x0000
#define KiHsroVrbResetVtm           0x0001

//Control Value for Vrb_A24_Access->uwLatchedStatus Register
#define KiHsroVrbResetStatus        0x0000
#define KiHsroVrbStatusClear        0x0000 // also to check uwCurrentStatus

//Control Value for Vrb_A24_Access->uwBufferStartAddress[] Registers
#define KiHsroVrbTotBuffer          16
#define KiHsroVrbBufferSize         0x800

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
#endif // __HsroVrbCard__

