/*-------------------------------------------------------------------------------*/
  /*
    HsroVrbContrCard.h
    Class CHsroVrbContrCard
    7-Sep-00  VRB Controller Card
  */
#ifndef __HsroVrbContrCard__
#define __HsroVrbContrCard__
/*-------------------------------------------------------------------------------*/

//A Memory structure that we can lay on top of a pointer 
//to the base address of a VRB Controller card and use to address its registers
struct VrbC_A24_Access
{
/*00000000      R */    unsigned short uwBase ;
                        unsigned char  ubReserved_0x00_0x0A[0x0A-0x00-2] ; //dumb but unique name (unsused)
                                                                           //and brute calculation to
                                                                           //fill the address space gap 
/*0000000A D16  W */    unsigned short uwVrbcReset ;
                        unsigned char  ubReserved_0x0A_0xB2[0xB2-0x0A-2] ; 
/*000000B2 D8   W */    unsigned char  ubPdaqSelect ;
                        unsigned char  ubReserved_0xB3 ; 
/*000000B4 D8   W */    unsigned char  ubSerialNumber ;
                        unsigned char  ubReserved_0xB4_0xD0[0xD0-0xB4-1] ; // -1 here because previous variable was byte only
/*000000D0 D16  W */    unsigned short uwSlvRdyEmul ;
/*000000D2 D8   R */    unsigned char  ubSclStatus ;
} ;

struct VrbC_A16_Access
{
/*0000      R */        unsigned short uwBase ;
                        unsigned char  ubReserved_0x00_0xD0[0xD0-0x00-2] ; //dumb but unique name (unsused)
                                                                           //and brute calculation to
                                                                           //fill the address space gap 
/*00D0      W */        unsigned short uwCrateID ;
/*00D2      W */        unsigned short uwL3TransfNum ;
} ;



/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
class CHsroVrbContrCard 
{
/////////////////////////////
// Constructor/Destructor
public:

             CHsroVrbContrCard    ( unsigned long           _ulVrbCardCode ) ; 

            ~CHsroVrbContrCard    ( );

/////////////////////////////
// Attributes

public:

protected:

    //VRB Controller Card ID Code
    unsigned long       m_ulSpecificCardCode ;

    //VME Access
    TulVmeAddress       m_ulA24VmeAddr ; // The A24 VME Address as seen as A32 from the communication crate
    VrbC_A24_Access*    m_pxA24Access ;  // The Memory Mapped Virtual Address for the Card Base

    TulVmeAddress       m_ulA16VmeAddr ; // The A16 VME Address as seen as A32 from the communication crate
    VrbC_A16_Access*    m_pxA16Access ;  // The Memory Mapped Virtual Address for the Card Base

    //Card Slot in VRB Crate
    unsigned char       m_ubCard_Slot ;

    //String describing the card functionality
    CString             m_osCardDescription ;

/////////////////////////////
// Operations

public:

    ESuccess Initialize           ( ) ;

protected:

};

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
// size of one VRB Controller card (maximum) A24 address space 
#define KiHsroVrbContrCardA24AddrSize   0x00000100

// Define Last VRB Controller relative addresses (for cross-checking above data structure)
#define KiHsroVrbContrAddrSclStatus     0x000000D2  // SCL Status 


// size of one VRB Controller card (maximum) A16 address space
#define KiHsroVrbContrCardA16AddrSize   0x0100

// Define some of the VRB Controller relative addresses (for cross-checking above data structure)
#define KiHsroVrbContrShortIOL3Transf   0x00D2  // L3 Transfer Number

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
//Control Values for Vrb_A24_Access->uwVrbcReset Register
#define KiHsroVrbMaskResetVrbc          0x0000

//Control Values for Vrb_A24_Access->ubPdaqSelect Register
#define KiHsroVrbMaskSelPdaq              0x01

//Control Values for Vrb_A24_Access->ubSlvRdyEmul Register
#define KiHsroVrbMaskSlvRdy               0x03

//Status Values for Vrb_A24_Access->ubSclStatus Register
#define KiHsroVrbMaskSclStatus            0x0F

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
#endif // __HsroVrbContrCard__

