/*-------------------------------------------------------------------------------*/
  /*
    L1Cal.h
    Class CL1Cal
  */
/*-------------------------------------------------------------------------------*/

#include "L1Cal_IIb_TCC/L1Cal_TrgTwr.h"

#include "L1Cal_IIb_TCC/AdfCardBase.h"

#include "L1Cal_IIb_TCC/NevisTabCard.h"
#include "L1Cal_IIb_TCC/NevisGabCard.h"

#include "L1Cal_IIb_TCC/MessageCommand.h" // for f_okRangeInCoverage

class CL1Cal
{

/////////////////////////////
// Constructor/Destructor
public:

    public:
         CL1Cal();
        ~CL1Cal();

/////////////////////////////
// Attributes
public:

    // We (typically) should have only one L1 Cal Trig,
    // this is a way to let any module get a hold of it
    static CL1Cal*       mg_poL1Cal ;

    // This Flag shows when the L1Cal has been initialized and is operational
    static volatile EYesNo       mg_eynL1CalOperational ;

    // This Flag allows to control behavior upon Initialization Errors
    static volatile EYesNo       mg_eynAbortInitOnError ;

    // This Flag allows to skip Initialization of VMESCL, TABs and GAB
    static volatile EYesNo       mg_eynSkipTabGabOnInit ; 

    // This Flag allows The TTI command files to immediately program the requested values
    static volatile EYesNo       mg_eynTtiImmediateIo ;

    // This Flag controls the behavior of FindDAC
    static volatile EFindDacMode_Type     mg_eFindDacMode ;

    // This Flag controls the value passed to Tab_Set_Eta38 at initialization
    static volatile unsigned long         mg_ulTabEta38Value ;

    // This value is used during Initialization to control the iterative de-skewing and verification step
    static volatile unsigned char         mg_ubMaxDeSkewTryAtInit ;

    // This value is the Base Alignment Delay common to all Trigger Towers
    // while deviations from this are added via TT info command files
    static volatile unsigned char         mg_ubBaseAlignDelay ;

    // This is the Register Address Offset within the Raw ADC Memory Block
    // where TCC will read Monitoring Data for Tick#1/SubTick#0
    static volatile unsigned short        mg_uwOffsetTick1RawAdcMonit ;

    // This is the Register Address Offset within the Raw ADC Memory Block
    // where TCC will write Simulated Data for Tick#1/SubTick#0
    static volatile unsigned short        mg_uwOffsetTick1RawAdcSimu ;

    // This is the Register Address Offset within the Output Et Memory Block
    // where TCC will read Monitoring Data for Tick#1
    static volatile unsigned short        mg_uwOffsetTick1OutputEtMonit ;

    // This is the Register Address Offset within the Output Et Memory Block
    // where TCC will write Simulated Data for Tick#1
    static volatile unsigned short        mg_uwOffsetTick1OutputEtSimu ;

    // This value is the Tick Offset between the Triggered Crossing Tick Number
    // and the Tick Number currently recorded at the time the Captured Monitoring Data
    // Signal stops the Address Generator(s)
    static volatile unsigned short        mg_uwOffsetTriggeredToTickAtCapture ;

    // Hold the initialized value (i.e. full scale) of each reference set type
    static  unsigned short                mg_auwRefSetInit [eTot_Tab_RefSet_Type] ;

public:

    // pointers to all L1Cal cards
    /////////////////////////////

    // ADF-2 Cards
    CAdfCardBase*       m_apoAdf [eTot_TT_Eta_Sign] [eTot_TT_Eta_By4] [eTot_TT_Phi_By4] ;

    // ADF-2 Maestro Cards
    CAdfCardBase*       m_poAdfMaestroImmediateMonit ;
    CAdfCardBase*       m_poAdfMaestroRequestMonit ;

    // link to crawl thru the list of all L1Cal card
    CAdfCardBase*       m_pLastAdfCard ;


    //TAB Cards
    CNevisTabCard*      m_apoTab [eTot_TT_Phi_By4] ;


    // GAB Carde
    CNevisGabCard*      m_poGab ;


    // pointers to all L1Cal Towers
    /////////////////////////////

    CL1Cal_TrgTwr*       m_a3poTrgTwr[eTot_TT_Eta_Sign] [eTot_TT_Eta+1] [eTot_TT_Phi+1] ;
    // Note the +1 above so that TT_Eta=1 or TT_Phi=1 goes to index [1]
    // The pointers of indices zero are left untouched

    CL1Cal_TrgTwr*       m_poLast_TrgTwr ; //The last Trigger Tower in the chain (maintained but not actively used)

    // Reference Set Programming
    EYesNo               m_a2eRefSetalloc   [eTot_Tab_RefSet_Type] [eTot_TT_RefSet] ;
    unsigned short       m_a5uwRefSetThresh [eTot_Tab_RefSet_Type] [eTot_TT_RefSet] [eTot_TT_Eta_Sign] [eTot_TT_Eta_By4] [eTot_TT_Phi_By4] ;

protected:

    //Remember current Trigger Tower Coverage

    unsigned long   m_ulSignEtaMin, m_ulSignEtaMax ;

    unsigned long   m_ulMagnEtaMin, m_ulMagnEtaMax ;

    unsigned long   m_ulPhiMin,     m_ulPhiMax ;

/////////////////////////////
// Operations

public:

    ESuccess Initialize       ( ) ;

    ESuccess Iterative_DeSkew ( ) ;


    ESuccess SetCoverageSignEta   ( unsigned long   _ulTTEtaSignMin,
                                    unsigned long   _ulTTEtaSignMax ) ;

    ESuccess SetCoverageMagnEta   ( unsigned long   _ulTTEtaMagnMin,
                                    unsigned long   _ulTTEtaMagnMax ) ;

    ESuccess SetCoveragePhi       ( unsigned long   _ulTTPhiMin,
                                    unsigned long   _ulTTPhiMax     ) ;

    unsigned long   f_ulTTEtaSignMin() { return m_ulSignEtaMin ; } ;
    unsigned long   f_ulTTEtaSignMax() { return m_ulSignEtaMax ; } ;

    unsigned long   f_ulTTEtaMagnMax() { return m_ulMagnEtaMax ; } ;
    unsigned long   f_ulTTEtaMagnMin() { return m_ulMagnEtaMin ; } ;

    unsigned long   f_ulTTPhiMin    () { return m_ulPhiMin ; } ;
    unsigned long   f_ulTTPhiMax    () { return m_ulPhiMax ; } ;

    CString         f_osCoverage  () ; // Show coverage e.g. "eta[-20,+20];phi[1,32]"


    ESuccess    f_okRequestMonitData        ( ) ;
    ESuccess    f_okDontRequestMonitData    ( ) ;

    ESuccess    f_okCheckMonitDataCaptured  ( EYesNo*           _peCaptured,
                                              unsigned short*   _pulTickNum = NULL ) ;

    ESuccess    f_okCaptureMonitData        ( ) ;


    ESuccess    f_okRangeInCoverage   ( TAllTriggerTowers*      _pxTT ) ;


    
    ESuccess    f_okFind_DAC          ( ETT_Chan_Type           _eTTType,
                                        TAllTriggerTowers*      _pxTT ) ;

    ESuccess    f_okAdjust_Ped        ( ETT_Chan_Type           _eTTType,
                                        TAllTriggerTowers*      _pxTT ) ;

    ESuccess    f_okRevert_Ped        ( ETT_Chan_Type           _eTTType,
                                        TAllTriggerTowers*      _pxTT ) ;

    ESuccess    f_okProfile_BLS       ( ETT_Chan_Type           _eTTType,
                                        TAllTriggerTowers*      _pxTT ) ;
    
    ESuccess    f_okProfile_AdfEt     ( ETT_Chan_Type           _eTTType,
                                        TAllTriggerTowers*      _pxTT ) ;
    
    
    ESuccess    f_okExclude_Tower     ( ETT_Chan_Type           _eTTType,
                                        TAllTriggerTowers*      _pxTT,
                                        bool                    _lVerbose = true ) ;


    ESuccess    f_okAdfOut_Constant   ( ETT_Chan_Type           _eTTType,
                                        TAllTriggerTowers*      _pxTT,
                                        const unsigned long     _ulConstantData ) ;



    ESuccess    f_okRawAdc_Thresh     ( ETT_Chan_Type           _eTTType,
                                        TAllTriggerTowers*      _pxTT,
                                        const unsigned long     _ulThreshold ) ;


static CL1Cal_TrgTwr* GetTrigTwrPointer   ( unsigned long   _ulTTEtaSign,
                                            unsigned long   _ulTTEtaMagn,
                                            unsigned long   _ulTTPhi ) ;
    ESuccess    f_okRefSetEMEtProg    ( const unsigned long     _ulRefSetNum,
                                        TAllTriggerTowers*      _pxTT,
                                        const double            _fdThreshold )
                                      { return f_okRefSetProg  ( eTab_RefSet_EMEt,
                                                                _ulRefSetNum,
                                                                _pxTT,
                                                                _fdThreshold ) ; } ;

    ESuccess    f_okRefSetJetEtProg   ( const unsigned long     _ulRefSetNum,
                                        TAllTriggerTowers*      _pxTT,
                                        const double            _fdThreshold )
                                      { return f_okRefSetProg  ( eTab_RefSet_JetEt,
                                                                _ulRefSetNum,
                                                                _pxTT,
                                                                _fdThreshold ) ; } ;

    ESuccess    f_okRefSetTTEtProg    ( const unsigned long     _ulRefSetNum,
                                        TAllTriggerTowers*      _pxTT,
                                        const double            _fdThreshold )
                                      { return f_okRefSetProg  ( eTab_RefSet_TTEt,
                                                                _ulRefSetNum,
                                                                _pxTT,
                                                                _fdThreshold ) ; } ;


    ESuccess    f_okRefSetEMEtDump    ( const unsigned long     _ulRefSetNum,
                                        TAllTriggerTowers*      _pxTT,
                                        const double            _fdThreshold )
                                      { return f_okRefSetDump  ( eTab_RefSet_EMEt,
                                                                _ulRefSetNum,
                                                                _pxTT,
                                                                _fdThreshold ) ; } ;

    ESuccess    f_okRefSetJetEtDump   ( const unsigned long     _ulRefSetNum,
                                        TAllTriggerTowers*      _pxTT,
                                        const double            _fdThreshold )
                                      { return f_okRefSetDump  ( eTab_RefSet_JetEt,
                                                                _ulRefSetNum,
                                                                _pxTT,
                                                                _fdThreshold ) ; } ;

    ESuccess    f_okRefSetTTEtDump    ( const unsigned long     _ulRefSetNum,
                                        TAllTriggerTowers*      _pxTT,
                                        const double            _fdThreshold )
                                      { return f_okRefSetDump  ( eTab_RefSet_TTEt,
                                                                _ulRefSetNum,
                                                                _pxTT,
                                                                _fdThreshold ) ; } ;


    ESuccess    f_okRefSetEMEtDeall   ( const unsigned long     _ulRefSetNum )
                                      { return f_okRefSetDeall ( eTab_RefSet_EMEt,
                                                                _ulRefSetNum ) ; } ;

    ESuccess    f_okRefSetJetEtDeall ( const unsigned long     _ulRefSetNum )
                                      { return f_okRefSetDeall ( eTab_RefSet_JetEt,
                                                                _ulRefSetNum ) ; } ;

    ESuccess    f_okRefSetTTEtDeall   ( const unsigned long     _ulRefSetNum )
                                      { return f_okRefSetDeall ( eTab_RefSet_TTEt,
                                                                _ulRefSetNum ) ; } ;


    ESuccess    f_okL2MaskEM          ( TAllTriggerTowers*      _pxTT,
                                        const unsigned long     _ulRefSetNum )
                                      { return f_okL2Mask      ( eTab_Mask_EM,
                                                                _pxTT,
                                                                _ulRefSetNum ) ; } ;

    ESuccess    f_okL2MaskJet         ( TAllTriggerTowers*      _pxTT,
                                        const unsigned long     _ulRefSetNum )
                                      { return f_okL2Mask      ( eTab_Mask_Jet,
                                                                _pxTT,
                                                                _ulRefSetNum ) ; } ;

    ESuccess    f_okL2MaskTau         ( TAllTriggerTowers*      _pxTT,
                                        const unsigned long     _ulRefSetNum )
                                      { return f_okL2Mask      ( eTab_Mask_Tau,
                                                                _pxTT,
                                                                _ulRefSetNum ) ; } ;



    ESuccess    f_okAlgoUseEmFraction     ( TAllTriggerTowers*      _pxTT,
                                            bool                    _lUseEmFraction ) ;

    ESuccess    f_okAlgoUseEmIsolation    ( TAllTriggerTowers*      _pxTT,
                                            bool                    _lUseEmIsolation ) ;

    ESuccess    f_okAlgoEmIsolationEtCut  ( TAllTriggerTowers*      _pxTT,
                                            const double            _fdThreshold ) ;

    ESuccess    f_okAlgoTauEtCoreCut      ( TAllTriggerTowers*      _pxTT,
                                            const double            _fdThreshold ) ;

    ESuccess    f_okAlgoGlobalSumEtCut    ( TAllTriggerTowers*      _pxTT,
                                            const double            _fdThreshold ) ;

    ESuccess    f_okAlgoUseIcrJetObjects  ( TAllTriggerTowers*      _pxTT,
                                            bool                    _lUseIcr ) ;

    ESuccess    f_okAlgoUseIcrGlobalSums  ( TAllTriggerTowers*      _pxTT,
                                            bool                    _lUseIcr ) ;



    ESuccess    f_okTermEmAll         ( unsigned int            _uiComparatorNum,
                                        unsigned int            _uiRefSetNum,
                                        unsigned int            _uiThreshold ) ;

    ESuccess    f_okTermEmCentral     ( unsigned int            _uiComparatorNum,
                                        unsigned int            _uiRefSetNum,
                                        unsigned int            _uiThreshold ) ;

    ESuccess    f_okTermJetAll        ( unsigned int            _uiComparatorNum,
                                        unsigned int            _uiRefSetNum,
                                        unsigned int            _uiThreshold ) ;

    ESuccess    f_okTermJetCentral    ( unsigned int            _uiComparatorNum,
                                        unsigned int            _uiRefSetNum,
                                        unsigned int            _uiThreshold ) ;

    ESuccess    f_okTermTauAll        ( unsigned int            _uiComparatorNum,
                                        unsigned int            _uiRefSetNum,
                                        unsigned int            _uiTauRatioNum, 
                                        unsigned int            _uiThreshold ) ;

    ESuccess    f_okTermTauCentral    ( unsigned int            _uiComparatorNum,
                                        unsigned int            _uiRefSetNum,
                                        unsigned int            _uiTauRatioNum, 
                                        unsigned int            _uiThreshold ) ;

    ESuccess    f_okTermIsolEmAll     ( unsigned int            _uiComparatorNum,
                                        unsigned int            _uiRefSetNum,
                                        unsigned int            _uiThreshold ) ;

    ESuccess    f_okTermIsolEmCentral ( unsigned int            _uiComparatorNum,
                                        unsigned int            _uiRefSetNum,
                                        unsigned int            _uiThreshold ) ;

    ESuccess    f_okTermMissEt        ( unsigned int            _uiComparatorNum,
                                        const double            _fdThreshold ) ;

    ESuccess    f_okTermTotEt         ( unsigned int            _uiComparatorNum,
                                        const double            _fdThreshold ) ;

    ESuccess    f_okTermAcoplanarJet          ( unsigned int            _uiComparatorNum,
                                                unsigned int            _uiRefSetNum1,
                                                unsigned int            _uiRefSetNum2,
                                                unsigned int            _uiDelta_Phi ) ;

    ESuccess    f_okTermNonCollinearLowEtJet  ( unsigned int            _uiComparatorNum,
                                                unsigned int            _uiRefSetNum1,
                                                unsigned int            _uiRefSetNum2,
                                                unsigned int            _uiDelta_Phi ) ;

    ESuccess    f_okTermNonCollinearMonoJet   ( unsigned int            _uiComparatorNum,
                                                unsigned int            _uiRefSetNum1,
                                                unsigned int            _uiRefSetNum2,
                                                unsigned int            _uiDelta_Phi ) ;

    ESuccess    f_okTermJetFree               ( unsigned int            _uiComparatorNum,
                                                unsigned int            _uiRefSetNum1,
                                                unsigned int            _uiRefSetNum2,
                                                unsigned int            _uiDelta_Phi ) ;

    ESuccess    f_okTermBackToBackEm          ( unsigned int            _uiComparatorNum,
                                                unsigned int            _uiRefSetNum,
                                                unsigned int            _uiDelta_Phi ) ;



    ESuccess    f_okRawAdc_MuxMode    ( ETT_Chan_Type           _eTTType,
                                        TAllTriggerTowers*      _pxTT,
                                        ERawAdc_MuxMode_Type    _eRawAdc_MuxMode ) ;

    ESuccess    f_okRawAdc_MemMode    ( ETT_Chan_Type           _eTTType,
                                        TAllTriggerTowers*      _pxTT,
                                        ERawAdc_MemMode_Type    _eRawAdc_MemMode ) ;

    ESuccess    f_okAdfOut_MuxMode    ( ETT_Chan_Type           _eTTType,
                                        TAllTriggerTowers*      _pxTT,
                                        EAdfOut_MuxMode_Type    _eAdfOut_MuxMode ) ;

    ESuccess    f_okAdfOut_MemMode    ( ETT_Chan_Type           _eTTType,
                                        TAllTriggerTowers*      _pxTT,
                                        EAdfOut_MemMode_Type    _eAdfOut_MemMode ) ;


protected:

    ESuccess    f_okRefSetProg        ( ETabRefSet_Type         _eRefSetType,
                                        const unsigned long     _ulRefSetNum,
                                        TAllTriggerTowers*      _pxTT,
                                        const double            _fdThreshold ) ;

    ESuccess    f_okRefSetDump        ( ETabRefSet_Type         _eRefSetType,
                                        const unsigned long     _ulRefSetNum,
                                        TAllTriggerTowers*      _pxTT,
                                        const double            _fdThreshold ) ;

    ESuccess    f_okRefSetDeall       ( ETabRefSet_Type         _eRefSetType,
                                        const unsigned long     _ulRefSetNum ) ;

    ESuccess    f_okL2Mask            ( ETabMask_Type           _eMaskType,
                                        TAllTriggerTowers*      _pxTT,
                                        const unsigned long     _ulRefSetNum ) ;

    ESuccess    f_okRefSetInit        ( ETabRefSet_Type         _eRefSetType,
                                        const unsigned long     _ulRefSetNum ) ;

    CString     f_okRefSetDump        ( ETabRefSet_Type         _eRefSetType,
                                        unsigned long           _ulTTEtaSign,
                                        unsigned long           _ulTTEtaMagnBy4,
                                        unsigned long           _ulTTPhiBy4 ) ;


} ;
