/*-------------------------------------------------------------------------------*/
  /*
    L1Cal_TrgTwr.h
    Class CL1Cal_TrgTwr
  */
/*-------------------------------------------------------------------------------*/
#if defined (__L1Cal_TrgTwr__) // prevent double definition
#else
    #define  __L1Cal_TrgTwr__
/*-------------------------------------------------------------------------------*/

#include "L1Cal_IIb_TCC/AdfCardBase.h"
#include "L1Cal_IIb_TCC/AdfChipFpga_T7.h"

// Here is a value we use to initialize Lookup Slope
#define kfSlopeUndefined    -1.0

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
class Et_Lookup_Coeff // just a struct that knows how to initialize itself
{
public :

    Et_Lookup_Coeff ( )
      : uwZeroEngAdcE  (0),
        uwZeroEngOutEt (0),
        fdLookupSlope (kfSlopeUndefined)
    {} ;

public :

    unsigned short  uwZeroEngAdcE ;
    unsigned short  uwZeroEngOutEt ;    // use short even though this is a byte
    double          fdLookupSlope ;

};

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
class CL1Cal         ; //forward declaration
class CResultFile    ; //forward declaration

class CL1Cal_TrgTwr
{

/////////////////////////////
// Constructor/Destructor
public:

             CL1Cal_TrgTwr( unsigned long       _ulTTEtaSign,
                            unsigned long       _ulTTEtaMagn,
                            unsigned long       _ulTTPhi,
                            CL1Cal_TrgTwr**     _ppoPrev_L1Cal_TrgTwr = NULL,
                            TuwRegWord          _uwRawAdcZer   = KiRawAdcZer,
                            TuwRegWord          _uwOutputEtZer = KiOutputEtZer,
                            CAdfCardBase*       _poMotherAdf = NULL ) ;

             CL1Cal_TrgTwr( unsigned long       _ulTTEtaSign,
                            unsigned long       _ulTTEtaMagn,
                            unsigned long       _ulTTPhi,
                            CL1Cal*             _poL1Cal ) ;

            ~CL1Cal_TrgTwr ();


protected:

    void     BasicConstruct(unsigned long       _ulTTEtaSign,
                            unsigned long       _ulTTEtaMagn,
                            unsigned long       _ulTTPhi,
                            CL1Cal_TrgTwr**     _ppoPrev_L1Cal_TrgTwr,
                            TuwRegWord          _uwRawAdcZer,
                            TuwRegWord          _uwOutputEtZer ) ;

/////////////////////////////
// Attributes
public:

    // Remember Trigger Tower calibration
    /////////////////////////////////////

    unsigned char       m_ubRelAlignDelay[eTot_TT_Chan_Type] ;
    unsigned long       m_ulPedestalControlDAC[eTot_TT_Chan_Type] ;

    // Remember additional info from Find_DAC's result files
    double              m_fPedestalAverage[eTot_TT_Chan_Type] ;  // Pedestal Histogram average
    double              m_fPedestalStdDev[eTot_TT_Chan_Type] ;   // Pedestal Histogram Standard Deviation

    double              m_fPrevPedStdDev[eTot_TT_Chan_Type] ;    // Previous Pedestal Standard Deviation
    unsigned long       m_ulPrevPedCtrlDAC[eTot_TT_Chan_Type] ;  // Previous Pedestal Control value

    //Remember Trigger Tower Et Lookup coefficients
    Et_Lookup_Coeff     m_axEtLookupCoeff[eTot_TT_Chan_Type] ;

    //Trigger Tower Zero Energy Response
    TuwRegWord          m_uwRawAdcZer ;    // Raw ADC E Zero Energy Response
    TuwRegWord          m_uwOutputEtZer ;  // Final Output Et Zero Energy Response

    // Remember Trigger Tower simulated values
    unsigned short      m_uwSimuRawAdc[eTot_TT_Chan_Type][KiTotTicksInTurn+1][KiTotSamplesPerTick] ;
    unsigned char       m_ubSimuOutputEt[eTot_TT_Chan_Type][KiTotTicksInTurn+1] ;

protected:

    // link to crawl thru the list of all L1Cal_TrgTwr
    CL1Cal_TrgTwr*      m_poPrev_L1Cal_TrgTwr ;

    //Trigger Tower Coordinates
    unsigned long       m_ulTTEtaSign ;
    unsigned long       m_ulTTEtaMagn ;
    unsigned long       m_ulTTPhi ;

    // ADF Card handling this TrgTwr...
    CAdfCardBase*       m_poMotherAdf ;
    //...and relative channel
    unsigned long       m_ulRelEta ; // relative on card
    unsigned long       m_ulRelPhi ;
    unsigned long       m_ulFpgaThisRelEta ; // which Fpga handles this rel_eta
    unsigned long       m_ulRelEtaOnFpga ;   // which rel_eta on that Fpga

public:


/////////////////////////////
// Operations

public:

    ESuccess    Initialize            ( ) ;
    ESuccess    InitEtLookup          ( ETT_Chan_Type       _eTTChanType ) ;
    ESuccess    InitAlignmentDelay    ( ETT_Chan_Type       _eTTChanType ) ;
    ESuccess    InitPedestalDac       ( ETT_Chan_Type       _eTTChanType ) ;

    ESuccess    VerifyProgramming     ( ) ;
    ESuccess    VerifyZeresp          ( ETT_Chan_Type       _eTTChanType ) ;
    ESuccess    VerifyEtLookup        ( ETT_Chan_Type       _eTTChanType ) ;

    unsigned long       f_ulTTEtaSign () { return m_ulTTEtaSign ; } ;
    unsigned long       f_ulTTEtaMagn () { return m_ulTTEtaMagn ; } ;
    unsigned long       f_ulTTPhi ()     { return m_ulTTPhi ; } ;

    CAdfCardBase*       f_poMotherAdf () { return m_poMotherAdf ; } ;

    // String of coordinates e.g. "TT(-20, 1)"
    CString         f_osTrgTwr_Coord  ( ) ;

static   CString    f_osTrgTwr_Coord  ( unsigned long   _ulTTEtaSign,
                                        unsigned long   _ulTTEtaMagn,
                                        unsigned long   _ulTTPhi ) ;

    // String of coordinates e.g. "EM_TT(-20, 1)"
    CString         f_osTrgTwr_Coord  ( ETT_Chan_Type   _eTTChanType ) ;

static   CString    f_osTrgTwr_Coord  ( ETT_Chan_Type   _eTTChanType,
                                        unsigned long   _ulTTEtaSign,
                                        unsigned long   _ulTTEtaMagn,
                                        unsigned long   _ulTTPhi ) ;

    // String of coordinates and TT Comparator Refence e.g. "TT(-20, 1) EM Et Thresh #0"
    CString         f_osTrgTwr_RefThresh  ( ETabRefSet_Type _eRefSetType,
                                            unsigned long   _ulRefSetNum ) ;

static   CString    f_osTrgTwr_RefThresh  ( ETabRefSet_Type _eRefSetType,
                                            unsigned long   _ulRefSetNum,
                                            unsigned long   _ulTTEtaSign,
                                            unsigned long   _ulTTEtaMagn,
                                            unsigned long   _ulTTPhi ) ;

    // String of coordinate Range e.g. "eta(-20:+20),phi(1:32)"
static   CString    f_osTrgTwr_Range  ( unsigned long   _ulTTEtaSignMin,
                                        unsigned long   _ulTTEtaSignMax,
                                        unsigned long   _ulTTEtaMagnMin,
                                        unsigned long   _ulTTEtaMagnMax,
                                        unsigned long   _ulTTPhiMin,
                                        unsigned long   _ulTTPhiMax ) ;

    // String of coordinate Range e.g. "EM&HD eta(-20:+20),phi(1:32)"
static   CString    f_osTrgTwr_Range  ( ETT_Chan_Type   _eTTChanTypeMin,
                                        ETT_Chan_Type   _eTTChanTypeMax,
                                        unsigned long   _ulTTEtaSignMin,
                                        unsigned long   _ulTTEtaSignMax,
                                        unsigned long   _ulTTEtaMagnMin,
                                        unsigned long   _ulTTEtaMagnMax,
                                        unsigned long   _ulTTPhiMin,
                                        unsigned long   _ulTTPhiMax ) ;

    ESuccess    f_okExclude_Tower     ( ETT_Chan_Type           _eTTChanType,
                                        bool                    _lVerbose = false ) ;

    ESuccess    Find_DAC              ( ETT_Chan_Type       _eTTChanType,
                                        TuwRegWord*         _puwDacValue = NULL,
                                        double*             _pfHistoAve = NULL,
                                        double*             _pfHistoDev = NULL,
                                        double*             _pfStdDevAve = NULL,
                                        bool                _lShowDetails = false,
                                        CResultFile*        _poHistoFile = NULL ) ;

    ESuccess    Adjust_Ped            ( ETT_Chan_Type       _eTTChanType,
                                        TuwRegWord*         _puwDacValue = NULL,
                                        double*             _pfHistoAve = NULL,
                                        double*             _pfHistoDev = NULL,
                                        short*              _piwDacAdjust = NULL,
                                        bool                _lShowDetails = false,
                                        CResultFile*        _poHistoFile = NULL ) ;

    ESuccess    Revert_Ped            ( ETT_Chan_Type       _eTTChanType,
                                        TuwRegWord*         _puwDacValue = NULL,
                                        bool                _lShowDetails = false ) ;

    ESuccess    Profile_BLS           ( ETT_Chan_Type       _eTTChanType,
                                        double*             _pAveOfAveAll,
                                        double*             _pDevOfAveAll,
                                        double*             _pAveOfDevAll,
                                        double*             _pDevOfDevAll,
                                        double*             _pAveOfAveLiveX,
                                        double*             _pDevOfAveLiveX,
                                        double*             _pAveOfDevLiveX,
                                        double*             _pDevOfDevLiveX,
                                        bool                _lShowDetails,
                                        CResultFile*        _poHistoFile ) ;

    ESuccess    Profile_AdfEt         ( ETT_Chan_Type       _eTTChanType,
                                        double*             _pAveOfAveLiveX,
                                        double*             _pDevOfAveLiveX,
                                        double*             _pAveOfDevLiveX,
                                        double*             _pDevOfDevLiveX,
                                        bool                _lShowDetails,
                                        CResultFile*        _poHistoFile ) ;

    ESuccess    ADC_Capture           ( ETT_Chan_Type       _eTTChanType,
                                        unsigned int        _uiThreshold,
                                        unsigned int        _uiMaxTry,
                                        TAdcArrayT7         _auwAdcArray,
                                        CResultFile*        _poCaptureFile = NULL ) ;

    ESuccess    f_okSetSimuRawAdc     ( ETT_Chan_Type           _eTTChanType,
                                        const unsigned short*   _puwSimuRawAdc  = NULL,    // omitted means use member variable
                                        const unsigned long*    _pulTickNum     = NULL,    // omitted means setup all ticks
                                        const unsigned long*    _pulSubTickNum  = NULL ) ; // omitted means setup all subticks

    ESuccess    f_okSetSimuOutputEt   ( ETT_Chan_Type           _eTTChanType,
                                        const unsigned char*    _pubSimuOutputEt = NULL,    // omitted means use member variable
                                        const unsigned long*    _pulTickNum      = NULL ) ; // omitted means setup all ticks

    ESuccess    f_okReadMonitOutputEt ( unsigned char*          _pubMonitOutputEtEm,
                                        unsigned char*          _pubMonitOutputEtHd,
                                        const unsigned long*    _pulTickNum      = NULL ) ; // omitted means setup all ticks


    ESuccess    f_okOutputConstant    ( ETT_Chan_Type           _eTTChanType,
                                        const unsigned char     _ubConstantData ) ;

    ESuccess    f_okRawAdcThresh      ( ETT_Chan_Type           _eTTChanType,
                                        const unsigned char     _ubThreshold ) ;


    ESuccess    f_okRawAdc_MuxMode    ( ETT_Chan_Type           _eTTChanType,
                                        ERawAdc_MuxMode_Type    _eRawAdc_MuxMode ) ;

    ESuccess    f_okRawAdc_MemMode    ( ETT_Chan_Type           _eTTChanType,
                                        ERawAdc_MemMode_Type    _eRawAdc_MemMode ) ;


    ESuccess    f_okAdfOut_MuxMode    ( ETT_Chan_Type           _eTTChanType,
                                        EAdfOut_MuxMode_Type    _eAdfOut_MuxMode ) ;

    ESuccess    f_okAdfOut_MemMode    ( ETT_Chan_Type           _eTTChanType,
                                        EAdfOut_MemMode_Type    _eAdfOut_MemMode ) ;

protected:

};

/*-------------------------------------------------------------------------------*/
#endif //__L1Cal_TrgTwr__

