/*-------------------------------------------------------------------------------*/
  /*
     MessageCommand.h
     7-Oct-99
     28-Oct-99 Coor Command Parsing split onto separate files
  */
#ifndef __MessageCommand__
#define __MessageCommand__
/*-------------------------------------------------------------------------------*/

//Global Constants used various places
#include "L1Cal_IIb_TCC/GlobalConstants.h"

//Custom Data Type Definitions
#include "L1Cal_IIb_TCC/CustomDataTypes.h"

//Simple List Template
#include "L1Cal_IIb_TCC/TemplateList.h"

//maximum length of Msg payloads back to Coor
#define     KiMaxLengthReplyTextToCoor          400 // does not include Id & status; just optional reply text
                                                    // still not big enough to hold 256 andor terms * 4-5 char and rest of text
                                                    // must be small enough so that KiMaxLengthFullReply is less than KiMaxLengthStringOut used in Itc server 

//maximum length of a Keyword in COOR messages
#define     KiMaxLengthKeywordInMessageCommand   64


//maximum item index for various entities in COOR message 
#define     KiMaxIndexInCoorCmd_SpecTrig         127  // still needed for start run

#define     KiMaxValueInOutputConstant           0xff
#define     KiMaxValueInRawAdcThresh             0xff

#define     KiMinIndexInCoorCmd_RefSet_Tab       0
#define     KiMaxIndexInCoorCmd_RefSet_Tab       6
#define     KiMaxIndexInCoorCmd_RefSet_TT        2 // There are only 3x TT RefSet 

#define     KiMinIndexInCoorCmd_RefSet_Gab       1
#define     KiMaxIndexInCoorCmd_RefSet_Gab       6

#define     KiMaxIndexInCoorCmd_EM_All_Term                14
#define     KiMaxIndexInCoorCmd_EM_Central_Term            10
#define     KiMaxIndexInCoorCmd_Jet_All_Term               14
#define     KiMaxIndexInCoorCmd_Jet_Central_Term           10
#define     KiMaxIndexInCoorCmd_Tau_All_Term               12
#define     KiMaxIndexInCoorCmd_Tau_Central_Term           12
#define     KiMaxIndexInCoorCmd_Isolated_EM_All_Term       10
#define     KiMaxIndexInCoorCmd_Isolated_EM_Central_Term   10
#define     KiMaxIndexInCoorCmd_Missing_Et_Term             4
#define     KiMaxIndexInCoorCmd_Total_Et_Term               4
#define     KiMaxIndexInCoorCmd_Acoplanar_Jet_Term          4
#define     KiMaxIndexInCoorCmd_NonCollinear_LowEt_Jet_Term 8
#define     KiMaxIndexInCoorCmd_NonCollinear_Mono_Jet_Term  4
#define     KiMaxIndexInCoorCmd_Back_To_Back_EM_Term        4
#define     KiMaxIndexInCoorCmd_Jet_Free_Term               4

#define     KiMinThreshInCoorCmd_Term                       1
#define     KiMaxThreshInCoorCmd_EM_All_Term                4
#define     KiMaxThreshInCoorCmd_EM_Central_Term            4
#define     KiMaxThreshInCoorCmd_Jet_All_Term               4
#define     KiMaxThreshInCoorCmd_Jet_Central_Term           4
#define     KiMaxThreshInCoorCmd_Tau_All_Term               2
#define     KiMaxThreshInCoorCmd_Tau_Central_Term           2
#define     KiMaxThreshInCoorCmd_Isolated_EM_All_Term       2
#define     KiMaxThreshInCoorCmd_Isolated_EM_Central_Term   2
#define     KiMaxThreshInCoorCmd_Missing_Et_Term            4
#define     KiMaxThreshInCoorCmd_Total_Et_Term              4

#define     KiMax_TT_Eta_Sign                    2  // pos, neg

#define     KiMin_TT_Eta_Magn                    1  // 1..20
#define     KiMax_TT_Eta_Magn                   20  // 1..20

//#define     KiMax_TT_Eta_Normal                 18  // 1..18 are normal towers, 19&20 are ICD and massless gap 
#define     KiMax_TT_Eta_Normal                 20  

#define     KiMin_TT_Phi                         1  // 1..32
#define     KiMax_TT_Phi                        32  // 1..32

#define     KiMax_TabChip_Eta_Width              4  // +/- 1..4, 5..8, 9..12, 13..16, 17..20
#define     KiMax_TabChip_Phi_Width              4  // 1..4, 5..8, 9..12, 13..16, 17..20, 21..24, 25..28, 29..32

//maximum item index in COOR message 
#define     KiMaxItemIndexInMessageCommand       1000


// define Return values for Coor Message Handling
    enum ECoorMessageStatus 
    {
        Reply_Ok,       /* Reply_Ok   = 0 */
        Reply_Bad,      /* Reply_Bad  = 1 */
        Reply_None      /* Reply_None = 2 */
    }; 

    // define allowed "polarity" options for Parsed Command Items
    // Not specified, Asserted (e.g. 34) and Negated (e.g. -45)
    enum  EItemPolarity 
    { 
        ItemNegated         = Require_False,    /* ItemNegated    = 0 */
        ItemAsserted        = Require_True,     /* ItemAsserted   = 1 */
        ItemNotPresent      = Dont_Care         /* ItemNotPresent = 2 */
    }  ;

// define Token Type Code during low level parsing = 1st pass = Tokenize
    enum  ETokenType      
    { 
        TokenIsNotDefined,      /* TokenIsNotDefined        =  0 */
        TokenIsKeyword,         /* TokenIsKeyword           =  1 */
        TokenIsValueAsserted,   /* TokenIsValueAsserted     =  2 */
        TokenIsValueNegated,    /* TokenIsValueNegated      =  3 */
        TokenIsValueLower,      /* TokenIsValueLower        =  4 */
        TokenIsValueUpper,      /* TokenIsValueUpper        =  5 */
        TokenIsValueLowerNeg,   /* TokenIsValueLowerNeg     =  6 */
        TokenIsValueUpperNeg,   /* TokenIsValueUpperNeg     =  7 */
        TokenIsOpenParenthesis, /* TokenIsOpenParenthesis   =  8 */
        TokenIsCloseParenthesis,/* TokenIsCloseParenthesis  =  9 */
        TokenIsValueDecimal     /* TokenIsValueDecimal      = 10 */
    } ;

// simple struct to hold both the Token Type Code and the token location in the string.
    struct TokenLocAndType 
    {
        unsigned int    uiCharNum ;
        ETokenType      eTokenType ;
    } ;

// structure definition for arrays of booleans of Trigger Tower Eta and Phi 
// this is used to convey which Trigger Towers a particulat command applies to
    struct TAllTriggerTowers
        {
        bool    aalTT_Eta [KiMax_TT_Eta_Sign] [KiMax_TT_Eta_Magn+1] ;
        bool    alTT_Phi [KiMax_TT_Phi+1] ;
    } ;
    
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
class CMessageCommand
{
/////////////////////////////
// Constructor/Destructor

public:

    CMessageCommand () ;
   ~CMessageCommand () ;

/////////////////////////////
// Member Variables

public:

    unsigned int        m_uiCoorCmdCode ;
    unsigned int        m_uiCoorSubCmdCode ;

    unsigned int        m_uiObjectIndex ;
    EItemPolarity       m_eObjectPolarity ;

    unsigned int        m_uiValue ;
    EItemPolarity       m_eValuePolarity ;

    EItemPolarity       m_aubItemPolarity[KiMaxItemIndexInMessageCommand+1] ;

    TAllTriggerTowers   m_xTT ;

    unsigned int        m_uiRefSetNum ;
    unsigned int        m_uiAuxiliary ;

    double              m_fdThreshold ;

private:
    
/////////////////////////////
// Member Functions

public: 

private:
    
    void    ResetCommand ( ) ;

} ;

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
class CMessageCommandLine
{
/////////////////////////////
// Constructor/Destructor

public:

    CMessageCommandLine () ;
   ~CMessageCommandLine () ;

/////////////////////////////
// Member Variables

public:

// Tokenized Command List 

    TList < TokenLocAndType, unsigned int > m_lTokenList ;

// Interpreted Command List 

    TList < CMessageCommand, unsigned int > m_lCommandList ;

private:

    const char*         m_psRawCommand ;
    const char*         m_pcCurrentCommandCharNum ;
    const char*         m_pcCurrentTokenCharNum   ;

    char*               m_psReplyText ;
    unsigned int        m_uiCurrentReplyLength ;
    unsigned int        m_uiMaxReplyLength ;

/////////////////////////////
// Member Functions

public: 

    void    NewRawCommand   ( const char*     _psMessageCommand,
                              char*           _psReplyText,
                              unsigned int    _uiMaxReplyLength ) ;

    char*   GetNextToken    ( char*           _psToken ) ;    

    void    CopyReplyText   ( const char*     _psPreplyText ) ;

    void    AppendReplyText ( const char*     _psPreplyText ) ;

    unsigned int CurTokenCharNum ( ) { // return current parsed Token ending position
                                    return (unsigned int) m_pcCurrentTokenCharNum 
                                         - (unsigned int) m_psRawCommand ; } ;

    const char*  GetAtChar  ( unsigned int     _uiCharNum ) { 
                                    return &m_psRawCommand[_uiCharNum] ; };    

private:
    
    void    ResetCommand ( ) ;

    void    ClearReplyText ( ) ;

} ;

/*-------------------------------------------------------------------------------*/
#endif // __MessageCommand__
