/*-------------------------------------------------------------------------------*/
  /*
    MonitControl.h
    11-Oct-2005
  */
#ifndef __MonitControl__
#define __MonitControl__
/*-------------------------------------------------------------------------------*/

#include "DataManip.h"

// How long to wait for Collect_Status before Forcing Monit Data Capture (6 seconds = 6000 ms)
#define     KiMonitControlTimeMsMinimum    1000  // minimum allowed
#define     KiMonitControlTimeMsDefault    6000  // default initial value

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
// Initialize the Monitoring Server
// (called only once)
ESuccess  InitMonitServices ( ) ;

//Shutdown the Monitoring Pool Manager
ESuccess  ShutdownMonitServices ( ) ;

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
class CMonitControl
{
/////////////////////////////
// Constructor/Destructor
public:

	 CMonitControl ();

    ~CMonitControl ();

/////////////////////////////
// Attributes
public:

private:

    // How long to wait for Collect_Status before Forcing Monit Data Capture
    static  unsigned long    mg_ulmsMonitWaitForCollectStatus ;

    // Global Flag to Prevent the Monit Server from generating Trigger IO.
    // We declare it as a volatile variable so that repeated tests 
    // do not get optimized out
    static  volatile  EYesNo  mg_eynMonit_Server_Silenced ;

    // This Flag allows to skip collecting monitoring information from TABs and GAB
    static  volatile  EYesNo  mg_eynMonitIgnoreTabGab ; 

    // Global Flag to Control Logging of Monitoring Server Transactions
    static  volatile  EYesNo  mg_eynMonit_Server_Tracing ;

    //This is used to record consecutive failures in CaptureMonitData
    //and stop trying after the threshold is reached 
    static  unsigned int      mg_uiConsecutiveFailures ;


/////////////////////////////
// Operations
public:

    static  unsigned long   fg_ulmsMonitWaitForCollectStatus () 
                                                    { return mg_ulmsMonitWaitForCollectStatus ; } ;

    inline static   EYesNo  fg_eynMonit_Server_Silenced () 
                                                    { return mg_eynMonit_Server_Silenced ; } ;

    static  void            Set_Monit_Server_Silenced ( EYesNo  _eSilenced = Yes ) 
                                                    { mg_eynMonit_Server_Silenced = _eSilenced ; 
                                                      return ; } ;

    inline static   EYesNo  fg_eynMonitIgnoreTabGab   () 
                                                    { return mg_eynMonitIgnoreTabGab ; } ;

    static  void            Set_MonitIgnoreTabGab   ( EYesNo  _eIgnoreTabGab = Yes ) 
                                                    { mg_eynMonitIgnoreTabGab = _eIgnoreTabGab ; 
                                                      return ; } ;

    static  void            Set_Monit_Server_Time   ( const unsigned long     _ulTime_ms ) 
                                                    { mg_ulmsMonitWaitForCollectStatus = _ulTime_ms ; 
                                                      return ; } ;

    inline static   EYesNo  fg_eynMonit_Server_Tracing () 
                                                    { return mg_eynMonit_Server_Tracing ; } ;

    static  void            Set_Monit_Server_Tracing  ( EYesNo  _eSilenced = Yes ) 
                                                    { mg_eynMonit_Server_Tracing = _eSilenced ; 
                                                      return ; } ;

    static ESuccess  RequestMonitData       ( ) ;
    static ESuccess  DontRequestMonitData   ( ) ;
    static ESuccess  CheckMonitDataCaptured ( EYesNo* _peCaptured ) ;
    static ESuccess  ForceCaptureMonitData  ( EYesNo* _peCaptured ) ;
    static void      ResetFailureDetection  ( ) { mg_uiConsecutiveFailures = 0 ; } ;


private:

    static void HandleAccessError ( ) ;

};

/*-------------------------------------------------------------------------------*/
//This is used to watch consecutive failures in CaptureMonitData
//and stop trying after the threshold is reached 
#define                 KiConsecutiveFailureThreshold   24

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
#endif // __MonitControl__
