/*-------------------------------------------------------------------------------*/
  /*
    NevisTabChip.h
    Class NevisTabChip
    23-Sep-2005
  */
#ifndef __NevisTabChip__
#define __NevisTabChip__
/*-------------------------------------------------------------------------------*/

class CNevisTabCard ;   // forward declaration 

/*-------------------------------------------------------------------------------*/

class CNevisTabChip
{
/////////////////////////////
// Constructor/Destructor
public:

             CNevisTabChip( CNevisTabCard*      _poMotherCard,
                            unsigned long       _ulChipAddress,
                            unsigned long       _ulTTEtaSign    = 0,
                            unsigned long       _ulTTEtaMagnBy4 = 0 ) ;

            ~CNevisTabChip();

/////////////////////////////
// Attributes
public:

protected:

    CNevisTabCard*      m_poMotherCard ;

    unsigned long       m_ulChipAddress ;

    unsigned long       m_ulTTEtaSign ;
    unsigned long       m_ulTTEtaMagnBy4 ;

    unsigned short      m_uwLastMonitStatus ;

    unsigned long       m_ulRefSetThresh [eTot_Tab_RefSet_Type][eTot_TT_RefSet] ;

    bool                m_lJetUseIcr ;
    bool                m_lGlobalSumsUseIcr ;

    bool                m_lUseEmIsolation ;
    bool                m_lUseEmFraction ;
    unsigned long       m_ulIsolRefSetNum ;
    unsigned long       m_ulIsolCutValue ;

    unsigned long       m_ulL2MaskRefSet [eTot_Tab_Mask_Type] ;

/////////////////////////////
// Operations
public:

    ESuccess Initialize  ( ) ;

    unsigned char       f_ulChipAddress     ( ) { return m_ulChipAddress ; } ;

    unsigned short      f_uwLastMonitStatus ( ) { return m_uwLastMonitStatus ; } ;

    TCString            f_osBaseAddress     ( ) ; // e.g. return "@Module#0/Chip# 0"
    TCString            f_osTrgTwr_Range    ( ) ; // e.g. return "eta(-8:-5),phi(1:4)"

    ESuccess Read_Monit_Status    ( unsigned short*  _puwChipStatus = NULL ) ;

    ESuccess Check_ChLink_Status  ( EYesNo*          _paeChLinkOk = NULL ) ;

    ESuccess RefSethreshProg      ( ETabRefSet_Type  _eRefSetType,
                                    unsigned long    _ulTab_Thresh_Num,
                                    unsigned long    _ulSet_AdcCount_Thresh,
                                    bool             _lVerbose = false ) ;

    ESuccess L2Mask               ( ETabMask_Type    _eMaskType,
                                    unsigned long    _ulRefSetNum,
                                    bool             _lVerbose = false ) ;

    ESuccess EmIsolProg           ( unsigned long    _ulSet_Exponent_Thresh,
                                    unsigned long    _ulSet_Enabled ) ;

    ESuccess EmHdFracProg         ( unsigned long    _ulSet_Exponent_Thresh,
                                    unsigned long    _ulSet_Enabled ) ;


    ESuccess AlgoUseEmFraction    ( bool             _lUseEmFraction,
                                    bool             _lVerbose = false ) ;

    ESuccess AlgoUseEmIsolation   ( bool             _lUseEmIsolation,
                                    bool             _lVerbose = false ) ;

    ESuccess AlgoEmIsolationEtCut ( unsigned long    _ulSet_AdcCount_Thresh,
                                    bool             _lVerbose = false ) ;

    ESuccess AlgoTauEtCoreCut     ( unsigned long    _ulSet_AdcCount_Thresh,
                                    bool             _lVerbose = false ) ;

    ESuccess AlgoGlobalSumEtCut   ( unsigned long    _ulSet_AdcCount_Thresh,
                                    bool             _lVerbose = false ) ;

    ESuccess AlgoUseIcrJetObjects ( bool             _lUseIcr,
                                    bool             _lVerbose = false ) ;

    ESuccess AlgoUseIcrGlobalSums ( bool             _lUseIcr,
                                    bool             _lVerbose = false ) ;

// Show Range e.g. "eta(-20:-17),phi(1:4)"
    CString  f_osTabChip_Range    ( ) ;

static  CString  f_osTabChip_Range        ( unsigned long   _ulTTEtaSign,
                                            unsigned long   _ulTTEtaMagnBy4,
                                            unsigned long   _ulTTPhiBy4 ) ;

// String representation of Refence Set Type e.g. "EM E"
static  CString  f_osTabChip_RefSet       ( ETabRefSet_Type _eRefSetType ) ;

// String representation of L2 Mask Type e.g. "Tau"
static  CString  f_osTabChip_Mask         ( ETabMask_Type   _eMaskType ) ;

// String representation of Comparator Refence e.g. "EM Et Thresh #0"
static  CString  f_osTabChip_RefThresh    ( ETabRefSet_Type _eRefSetType,
                                            unsigned long   _ulRefSetNum ) ;


protected:


///////////////////////////////////////////
// Access Privileges outside of this class

// special access privilege so that the base register constructor 
// can read the fpga address, etc 
// and write the LastReg variable, etc
//friend class CAdfRegBase ;

};


// These are constant to help decode the TAB FPGA Status words,
//
// Cable A:  (nominal eta, nominal phi + 4) 
// Cable B:  (nominal eta, nominal phi    )
// Cable C:  (nominal eta, nominal phi - 4)
//
//   Bit Number     Bit Meaning
//
    //       0          parity error on input cables C
    //       1          parity error on input cables B
    //       2          parity error on input cables A
    //       3          data sync error on input cables C
    //       4          data sync error on input cables B
    //       5          data sync error on input cables A
    //       6          BC mismatch on input cables C
    //       7          BC mismatch on input cables B
    //       8          BC mismatch on input cables A
    //       9          PLL lost lock
    //       10         mode_ff
    //       11         PRN test data error on cables C
    //       12         PRN test data error on cables B
    //       13         PRN test data error on cables A
    //       14         Expert Bit 1 (changes for spec tests)
    //       15         Expert Bit 2 (changes for spec tests)

#define     KiTabStatusParityMask       0x0007
#define     KiTabStatusSyncErMask       0x0038
#define     KiTabStatusBcMisMMask       0x01c0
//#define     KiTabStatusPllLostLockMask  0x0200
#define     KiTabStatusModeFFMask       0x0400
#define     KiTabStatusPrnErrMask       0x3800
//#define     KiTabStatusSerAlignMask     0x4000
//#define     KiTabStatusPllLockedMask    0x8000

#define     KiTabStatusParityShiftDown  0
#define     KiTabStatusSyncErShiftDown  3
#define     KiTabStatusBcMisMShiftDown  6
#define     KiTabStatusPrnErrShiftDown  11


/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
#endif // __NevisTabChip__
