/*-------------------------------------------------------------------------------*/
  /*
    T962.h
    Class CT962
  */
#ifndef __T962__
#define __T962__
/*-------------------------------------------------------------------------------*/

#include "L1Cal_IIb_TCC/T962_Wire.h"
#include "L1Cal_IIb_TCC/T962_ReadCrate.h"

// define some dimensions

#define kTot_T962_Crate  2 
#define kTot_T962_Adf    16 
#define kTot_T962_Wire   480

#define kData_Enable_NIM_Trigger     0xff50 
#define kData_Disable_NIM_Trigger    0xff40

//cosmetic names for ResultFile->OpenForWrite
#define kResultAscii        false
#define kResultBinary        true
#define kSuppressInfoMsg    false

//cosmetic names for binary file format
#define kWriteLong                           4
#define kWriteShort                          2
#define kFixedHeaderWord            0x0000D480
#define kFileFormatVersion              0x0001


#include "thread_util/Thread.hpp"
#include "thread_util/Thread_Util.hpp"
using     namespace thread_util ;

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
using namespace thread_util ; //needed when switched to thread_util V00-10-01

class CT962 : public Thread
{

/////////////////////////////
// Constructor/Destructor
public:

    public:
         CT962();
        ~CT962();

/////////////////////////////
// Attributes
public:

    // We (typically) should have only one L1 Cal Trig,
    // this is a way to let any module get a hold of it
    static CT962*       mg_poT962 ;

public:

    // pointers to all L1Cal cards
    /////////////////////////////

    // ADF-2 Cards
    CAdfCardBase*       m_apoAdf [kTot_T962_Adf] ;
    unsigned long       m_ulAdfCount ;

    // ADF-2 Cards
    CAdfCardBase*       m_poMaestro ;

    // T962 Wire
    CT962_Wire*         m_apoWire[kTot_T962_Wire] ;
    unsigned long       m_ulWireCount ;


    // ADF-2 Maestro Cards
    CAdfCardBase*       m_poAdfMaestroImmediateMonit ;
    CAdfCardBase*       m_poAdfMaestroRequestMonit ;

    // link to crawl thru the list of all L1Cal card
    CAdfCardBase*       m_pLastAdfCard ;


protected:

    // Run Number
    volatile unsigned long  m_ulRunNumber ;

    // Run State
    volatile ERun_State     m_eRunState ;

    // Current Event Number
    volatile unsigned long  m_ulEvtNumber ;

    // Current time waiting for event 
    volatile unsigned long  m_ulEvtWaitHundredthSec ;

    // Max Number of Event to collect
    volatile unsigned long  m_ulEvtMax ;

    // Event State
    volatile EEvt_State     m_eEvtState ;
    
    // Minimum CoolDown between events
    volatile unsigned long  m_ulCoolDown_Seconds ;

    // Current CoolDown
    volatile unsigned long  m_ulCoolDown ;

    // Mode Ascii or Binary
    volatile bool           m_lModeBinary ;

    // Directory for output files
    TCString                m_sDirectory ;

    // Threads to read each crate
    CT962_ReadCrate*         m_oThreadReadCrate[kTot_T962_Crate] ;

/////////////////////////////
// Operations

public:

    virtual void* run () ;

    ESuccess Add_Wire            ( unsigned long    _ulWireNum,
                                   unsigned char    _ubCrateNum,
                                   unsigned char    _ubCardSlot,
                                   unsigned char    _ubChipAddress,
                                   unsigned short   _uwRegAddress,
                                   unsigned long    _ulRegCount ) ;

    ESuccess Set_Maestro         ( unsigned char    _ubCrateNum,
                                   unsigned char    _ubCardSlot ) ;


    ESuccess Start_Run           ( unsigned long    _ulRunNumber,
                                   unsigned long    _ulEvents,
                                   unsigned long    _ulCoolDown,
                                   unsigned long    _ulModeBinary,
                                   char*            _psDirectory ) ;

    ESuccess Pause_Run           ( ) ;

    ESuccess Resume_Run          ( ) ;
    
    ESuccess Stop_Run            ( ) ;

    ESuccess Status              ( unsigned long*   _pulRunNumber,
                                   char*            _psRunState,
                                   unsigned long*   _pulEvtNumber,
                                   char*            _psEvtState ) ;

    TCString f_osRunState        ( ERun_State*     _peRunState = NULL ) ;
    TCString f_osEvtState        ( EEvt_State*     _peEvtState = NULL ) ;

protected:


} ;
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
#endif // __T962__
