/*-------------------------------------------------------------------------------*/
  /*
     TemplateList.h
     Template Class TList
     19-Oct-99
  */
#ifndef __TemplateList__
#define __TemplateList__
/*-------------------------------------------------------------------------------*/

//Simple Pair handling Template
#include "L1Cal_IIb_TCC/TemplateBinome.h"

#include "L1Cal_IIb_TCC/HandleConsole.h"

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/

template < class T_Key, class T_Value > class TList
{
/////////////////////////////
// Constructor/Destructor

public:

    TList    ( )       { m_poFirstEntry         = NULL ;
                         m_poLastEntry          = NULL ;
                         m_ulTotEntries         = 0 ; } ;

   ~TList    ( )       { DeleteAllEntries () ; } ;

/////////////////////////////
// Attributes

public:


private:

    TBinomialEntry < T_Key, T_Value > *     m_poFirstEntry ;

    TBinomialEntry < T_Key, T_Value > *     m_poLastEntry ;

    unsigned long                           m_ulTotEntries ;

/////////////////////////////
// Operations

public:

    ESuccess            AddEntry            ( const T_Key*    _poEntryKey   = NULL, 
                                              const T_Value*  _poEntryValue = NULL ) ;

    ESuccess            AddEntry            ( const T_Key     _oEntryKey, 
                                              const T_Value   _oEntryValue ) 
                                             { return AddEntry ( &_oEntryKey, &_oEntryValue ) ; };

    ESuccess            ModifyLastEntryKey  ( const T_Key*   _poEntryKey   ) { m_poLastEntry->ModifyKey ( _poEntryKey ) ; return Ok ; } ;
    ESuccess            ModifyLastEntryKey  ( const T_Key    _oEntryKey   )  { return ModifyLastEntryKey( &_oEntryKey ) ; } ;

    ESuccess            ModifyLastEntryValue( const T_Value* _poEntryValue ) { m_poLastEntry->ModifyValue ( _poEntryValue ) ; return Ok ; } ;
    ESuccess            ModifyLastEntryValue( const T_Value  _oEntryValue )  { return ModifyLastEntryValue( &_oEntryValue ) ; } ;

    ESuccess            DeleteAllEntries ( ) ;

    TBinomialEntry < T_Key, T_Value > *   
                        FirstEntry ( ) const { return m_poFirstEntry ; } ;

    TBinomialEntry < T_Key, T_Value > *   
                        NextEntry  ( const TBinomialEntry < T_Key, T_Value > * _poEntry ) 
                                       const { return _poEntry->GetNext() ; } ;

    TBinomialEntry < T_Key, T_Value > *   
                        LastEntry  ( ) const { return m_poLastEntry ; } ;

    TBinomialEntry < T_Key, T_Value > *   
                        PrevEntry  ( const TBinomialEntry < T_Key, T_Value > * _poEntry ) 
                                       const { return _poEntry->GetPrev() ; } ;


/////////////////////////////
// Overloaded assignement operator
public:
    const TList < T_Key, T_Value > &   operator=  ( const TList < T_Key, T_Value > &   _oSourceList ) ;
} ;


/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/


/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
template < class T_Key, class T_Value > 
ESuccess            TList < T_Key, T_Value > 
                         ::AddEntry     ( const T_Key*          _poEntryKey, 
                                          const T_Value*        _poEntryValue ) 
{
	ESuccess            eReturnStatus = Ok ;
    TBinomialEntry < T_Key, T_Value > *     poEntry ;

    //\\//

    m_ulTotEntries += 1 ;

    poEntry = new TBinomialEntry < T_Key, T_Value > 
                                 ( _poEntryKey, 
                                   _poEntryValue,
                                   m_poLastEntry ) ;

    if ( m_poLastEntry == NULL ) 
        // if this is the first entry, remember this pointer
        m_poFirstEntry = poEntry ;
    else
        // add the forward link to the former last entry
        // to point to this new last entry
        m_poLastEntry->SetLinkNext ( poEntry ) ;

    // This is now the last entry
    m_poLastEntry = poEntry ;

    return eReturnStatus ;

} ;

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
template < class T_Key, class T_Value > 
ESuccess            TList < T_Key, T_Value > 
                         ::DeleteAllEntries ( ) 
{
	ESuccess            eReturnStatus = Ok ;
    TBinomialEntry < T_Key, T_Value > *     poEntry ;
    unsigned int        uiEntryNum ;

    //\\//

    if ( m_poLastEntry != NULL )
    for ( uiEntryNum = m_ulTotEntries ; uiEntryNum > 0 ; uiEntryNum -- )
    {
        if ( m_poLastEntry == NULL )
        {
            CConsoleAccess::InternalErrorLine ( " Internal Error/TList::DeleteAllEntries: not enough Entries in linked list"  ) ;
            goto Done ;
        }

        poEntry = m_poLastEntry ;
        m_poLastEntry = poEntry->GetPrev () ;
        delete poEntry ;
    }

    if ( m_poLastEntry != NULL ) 
    {
        CConsoleAccess::InternalErrorLine ( " Internal Error/TList::DeleteAllEntries: too many Entries in linked list"  ) ;
    }

Done:

    m_ulTotEntries = 0 ;
    m_poFirstEntry = NULL ;
    m_poLastEntry = NULL ; 

    return eReturnStatus ;
} ;

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
template < class T_Key, class T_Value > 
const TList < T_Key, T_Value > &   TList < T_Key, T_Value > 
                                        ::operator=  ( const TList< T_Key, T_Value > &   _oSourceList ) 
{
    TBinomialEntry < T_Key, T_Value > *     poEntry ;

    //\\//

    if ( this->m_poFirstEntry != NULL )
    {//this Entry set already had symbols defined, 
        //delete these old ones first
        this->DeleteAllEntries() ;
    }

    //start from the end of the chain of the Source Symbol Set
    poEntry = _oSourceList.m_poFirstEntry ;

    //one at a time, copy those symbols to the new list
    while ( poEntry != NULL )
    {
        this->AddEntry ( poEntry->GetKey(),
                         poEntry->GetValue() ) ;
        poEntry = poEntry->GetLinkNext() ;
    }

    return *this ;
} ;


/*-------------------------------------------------------------------------------*/
#endif // __TemplateList__
