/*-------------------------------------------------------------------------------*/
  /*
     UtilsStrings.h
  */
#ifndef __UtilsStrings__
#define __UtilsStrings__
/*-------------------------------------------------------------------------------*/


/*------------------------------------------------------------------*/
// converts integer iNumber to a string representing its decimal value
// (or another optional Base iRadix)
// and pad the left of the resutling string with leading spaces 
// (or other optional padding character cPadChar) 
// to produce a CString with a minimum length of MinDigits+2 characters
// the two leading characters are '0x' for Hexadecimal, 
// '0d' for decimal and '0b' for binary
// The optional parameter iSpace_Every asks for the addition of a space
// character between groups of output digits (e.g. group binary output 
// by set of 4 bits: 0101 0001 1100). 
// The default (-1) gives no additional spacing

CString I_to_A( const int iNumber, 
                const int iMinDigits = 1, 
			    const int iRadix = 10, 
                const char cPadChar = ' ',
				const int iSpace_Every = -1 ) ;
CString I_to_A( const unsigned int iNumber, 
                const int iMinDigits = 1, 
			    const int iRadix = 10, 
                const char cPadChar = ' ',
				const int iSpace_Every = -1 ) ;
CString I_to_A( const long iNumber, 
                const int iMinDigits = 1, 
			    const int iRadix = 10, 
                const char cPadChar = ' ',
				const int iSpace_Every = -1 ) ;
CString I_to_A( const unsigned long iNumber, 
                const int iMinDigits = 1, 
			    const int iRadix = 10, 
                const char cPadChar = ' ',
				const int iSpace_Every = -1 ) ;
/*------------------------------------------------------------------*/


/*------------------------------------------------------------------*/
// converts CString to unsigned integer repesented by the string 
// while accepting decimal, hexadecimal or binary input
// decimal inputs MAY be preficed with '0d'
// hexadecimal inputs MUST be prefaced with '0x'
// binary inputs MUST be prefaced with '0b'

unsigned int  A_to_uI( CString osString ) ;
/*------------------------------------------------------------------*/

/*------------------------------------------------------------------*/
// converts CString to the double float repesented by the string 
// while accepting integral, decimal, exponential notation
// 
// 1     !these are all equivalent representations
// 1.
// 1.0
// 1e0
// 1E0 
// 1d0
// 1D0 
// 1.e0
// 1.0e0
// .1e1
// 0.1e1
// 0.1e+1
// 10e-1
// 10.e-1
// 10.0e-1

double        A_to_fD( CString osString ) ;
/*------------------------------------------------------------------*/

/*------------------------------------------------------------------*/
// converts CString to boolean repesented by the string 
// returns false if the string is interpreted as zero by A_to_uI
// returns true otherwise

bool          A_to_L ( CString osString ) ;
/*------------------------------------------------------------------*/

/*------------------------------------------------------------------*/
// converts CString to eTTChanType repesented by the string 
// returns -1 if the string cannot be matched to the legal values
//      "TT_EM"
//      "TT_HD"

ETT_Chan_Type        A_to_TTChanType ( CString osString ) ;
/*------------------------------------------------------------------*/

/*------------------------------------------------------------------*/
// converts CString to ETT_Eta_Sign repesented by the string 
// returns -1 if the string cannot be matched to the legal values
//      "+"
//      "-"

ETT_Eta_Sign         A_to_TTEtaSign ( CString osString ) ;
/*------------------------------------------------------------------*/


/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
// converts Floating Point Number _flNumber to a string representing its decimal value
// with _iDecDigits digits after decimal point
// and pad the left of the resutling string with leading spaces 
// to produce a CString with a minimum length of _iMinChar characters.

CString F_to_A( double      _flNumber, 
                const int   _iDecDigits = 0,
                const int   _iMinChar = 2 ) ;

/*------------------------------------------------------------------*/
// Create an ASCII Time String of the format:
// Sun May 01 20:27:01 1994

CString AsciiTime () ;
/*------------------------------------------------------------------*/


/*------------------------------------------------------------------*/
// Create an ASCII Time String of the format:
// dd-mmm-yyyy hh:mm:ss.mmm

CString TimeStamp ( time_t*     _piTime = NULL ) ;

#ifdef WIN32
    //  dd-mmm-yyyy hh:mm:ss.mmm
    //  123456789012345678901234 = 24 chars 
    #define KiTimeStampLength            24
    //  hhmmss
    //  123456 = 6 chars 
    #define KiTimeTagLength               6
#else
    //  dd-mmm-yyyy hh:mm:ss
    //  12345678901234567890 = 20 chars 
    #define KiTimeStampLength            20
#endif
/*------------------------------------------------------------------*/
// Create an ASCII Time String of the format:
// hhmmss

CString DateTag ( time_t*     _piTime = NULL ) ;
/*------------------------------------------------------------------*/

/*------------------------------------------------------------------*/
// Create an ASCII Time String of the format:
// yyyymmdd

CString TimeTag ( time_t*     _piTime = NULL ) ;
/*------------------------------------------------------------------*/


/*------------------------------------------------------------------*/
// Create an ASCII String with a poorman dump of an histogram:
// e.g. "( 3@2 1@3 1@4 1@5 3@6 1@8 )  Avr = 4.40 Dev = 2.01"

CString     DumpHistogram     ( unsigned short* const   _kpauwHistogram,
                                const unsigned long     _kulSampleSize,
                                double* const           _kpfAverage = NULL,
                                double* const           _kpfDeviation = NULL ) ;
/*------------------------------------------------------------------*/

/*------------------------------------------------------------------*/
// Create an ASCII String with a poorman dump of peaks :
// e.g. "Peak @ 456 = ( 8 8 23 56 45 13 9 9 )"


CString     DumpPeaks         ( unsigned short* const   _kpauwArray,
                                const unsigned short    _kuwArraySize, 
                                const unsigned short    _kuwThreshold, 
                                unsigned short* const   _kpuwMaxValue ) ;
/*------------------------------------------------------------------*/


/*------------------------------------------------------------------*/
/*------------------------------------------------------------------*/
// implement some of the MSVC string utilities, not in ANSI C
/*------------------------------------------------------------------*/
/*------------------------------------------------------------------*/


    // Force characters to uppercase, same as _strupr in windows
    char* strupr( char* _string ) ;

    // Force characters to lowercase, same as _strlwr in windows
    char* strlwr( char* _string ) ;

#if !defined Win32 

    // Force characters to uppercase, same as _strupr in windows
    std::string strupr( std::string _oString ) ;

    // case-independent strings compare 
    int stricmp( const char *string1, const char *string2 ) ;

#endif


/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
#endif // __UtilsStrings__
