//////////////////////////////////////////////////////////////////////////////
// File: L1CalTcc_MonitData_Adf_OutputEt.hpp
//
//  Monitoring block type: L1Cal ADF Trigger Tower Output Et to TAB
//
// Created: 11-Oct-2005 Philippe Laurens 
//////////////////////////////////////////////////////////////////////////////

#ifndef __L1CalTcc_MonitData_Adf_OutputEt__
#define __L1CalTcc_MonitData_Adf_OutputEt__

//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////
// Variable Size Definitions
//////////////////////////////////////////////////////////////////////////////
#include "trigmon/L1CalTcc_MonitData_BasicTypes.hpp"

//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////
// All Data Blocks returned by TCC start with a Simple Header
//////////////////////////////////////////////////////////////////////////////
#include "trigmon/L1CalTcc_MonitData_Header.hpp"

//////////////////////////////////////////////////////////////////////////////
// Beam Crossing Number Monitoring (Sub-)Structure 
//////////////////////////////////////////////////////////////////////////////
#include "trigmon/L1CalTcc_MonitData_BeamX.hpp"

///////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////
// This is the Data Block Content Definition for Type "L1Cal ADF Trigger Tower Output Et to TAB"
///////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////
// This structure describes the content of the Opaque ITC message received from TCC 
// A block of this type (eL1CalTcc_BlockType_Adf_OutputEt) can be requested on the 
// Monitoring Server Port #52348.

struct L1CalTcc_Block_Adf_OutputEt
{
    L1CalTcc_Block_Header           xHeader ;

    L1CalTcc_BeamX_Number           xBeamX ;

    // These are 4-dimensional arrays of bytes 
    // holding the filtered Output Et value sent by the ADF cards to the TAB cards.
    // Only the data for the 36 live crossings is monitored because 
    // the ADF card overrides the BLS signals for non-live crossing 
    // to always report a zero energy value (i.e. 8 counts) to the TABs.
    // cf. kL1CalTcc_Tick_For_LiveBX_Index_XX for matching the LiveBX Index to a Tick Number.

    uint8                           a4ubEM_Et   [kL1CalTcc_Tot_TT_Eta_Sign]
                                                [kL1CalTcc_Tot_TT_Eta_Magn]
                                                [kL1CalTcc_Tot_TT_Phi]
                                                [kL1CalTcc_Tot_LiveBX] ;

    uint8                           a4ubHD_Et   [kL1CalTcc_Tot_TT_Eta_Sign]
                                                [kL1CalTcc_Tot_TT_Eta_Magn]
                                                [kL1CalTcc_Tot_TT_Phi]
                                                [kL1CalTcc_Tot_LiveBX] ;

} ;

//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////

#endif // __L1CalTcc_MonitData_Adf_OutputEt__
