////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////

#include "L1Cal_IIb_TCC/StdAfx.h"
#include "L1Cal_IIb_TCC/ListOfIPPortNumbers.h"

#include "ace/Thread_Manager.h"
#include "thread_util/Thread_Util.hpp"
using     namespace thread_util ;

#define  _Debug_Project_String "L1Cal_IIb_TCC Coor Access"

#include "itc/itc.hpp"
#include "itc/String_Message.hpp"

#include "ace/Get_Opt.h"
typedef ACE_Get_Opt Get_Options;

//#include <strstrea.h> // already included via above header files

//-------------------------------------------------------------------------------
char host[80] = "localhost";
int portnum = KiIP_Port_CoorToTcc ;

const size_t buf_size = 1024 ;
char sStrBuf[ buf_size ] = "[Coor_Access] " ;
int iAfterMsgID = strlen ( sStrBuf ) ;
char AnswerBad[17+1]     = "[Coor_Access] Bad" ;
char tab1[81]            = " above error  12345678901234567890123456789012345678901234567890123456789012345" ;
char tab2[81]            = " from request below    1         2         3         4         5         6     " ;

#ifdef WIN32
HANDLE ghConsole ;    //Handle to Standard Output from Console
#endif

_ITC_NAMESPACE_USING

//-------------------------------------------------------------------------------
//
// Function receiving all messages
//
//-------------------------------------------------------------------------------

void got_message(const IPC_Message &m)
{

    String_Message*         pMsg ;
    Connection_Handle       Con_Handle ;

    //\\//

    switch ( m.ID() )
    {

    case String_Message::MSG_ID :

        // create a pointer to this message to access it as a String_Message
        pMsg = new String_Message ( m ) ;

#ifdef WIN32
        // override default screen color before bad acknowlegement
        if ( strncmp ( pMsg->str(), AnswerBad , strlen(AnswerBad) ) == 0 )
            SetConsoleTextAttribute   ( ghConsole,
                                        FOREGROUND_RED | FOREGROUND_INTENSITY ) ;
#endif

        // write message to the screen
        std::cout << pMsg->str() << std::endl;

#ifdef WIN32
        // return to default screen color after bad acknowlegement
        if ( strncmp ( pMsg->str(), AnswerBad , strlen(AnswerBad) ) == 0 )
            SetConsoleTextAttribute   ( ghConsole,
                                        FOREGROUND_RED | FOREGROUND_GREEN | FOREGROUND_BLUE ) ;
#endif

        if ( strncmp ( pMsg->str(), AnswerBad , strlen(AnswerBad) ) == 0 )
            std::cout << tab1 << std::endl << tab2 << std::endl << sStrBuf << std::endl ;

        // delete temporary additional message reference
        delete pMsg ;
    break;

    case Conn_Start_Msg::MSG_ID :
        std::cout << "-----------------------------------------------------------------------------"
                  << std::endl;
        INFO(("Client Got Message - connection START message\n"));
//        m.get_connection().info().dump();
        Con_Handle = m.get_connection() ;
        std::cout << "Connected to <"
                  << Con_Handle.info().program_name
                  << "> Version <" << Con_Handle.info().program_version
                  << "> ITC port <"
                  << Con_Handle.info().remote_host << ":" << Con_Handle.info().remote_port << ">"
                  << std::endl;
        std::cout << "-----------------------------------------------------------------------------"
                  << std::endl;
    break;

    case Conn_Stop_Msg::MSG_ID :
        std::cout << std::endl
                  << "-----------------------------------------------------------------------------"
                  << std::endl;
        INFO(("Client Got Message - connection STOP message\n"));
//        m.get_connection().info().dump();
    break;

    case Conn_Brkn_Msg::MSG_ID :
        std::cout << std::endl
                  << "-----------------------------------------------------------------------------"
                  << std::endl;
        INFO(("Client Got Message - connection BROKEN message\n"));
//        m.get_connection().info().dump();
        m.get_connection().stop() ;
    break;

    case Conn_Open_Msg::MSG_ID :
        std::cout << std::endl
                  << "-----------------------------------------------------------------------------"
                  << std::endl;
        INFO(("Client Got Message - connection RECONNECTED message\n"));
//        m.get_connection().info().dump();
    break;

    default:

        INFO(("Client Got Message = ID=%2d  - No=%5u length %3u  remote=%5u  reply no=%5u\n",
        m.ID(),m.no(),m.length(),m.remote_no(),m.reply_no()));

    }
}

//-------------------------------------------------------------------------------
//
// Function which sends messages
//
//-------------------------------------------------------------------------------

void send_message ( SOCK_Connector &con, char*   msg_string )
{
    ENTERING ( "send_message(SOCK_Connector &con, char* msg_string)" ) ;

    IPC_STATUS ret = con.send_now ( String_Message(msg_string), 5.0 ) ;

    if ( ret == IPC_SUCCESS )
    {
    }
    else if (ret == IPC_TIMEOUT )
    {
      DEB0((" There is a timeout for send "));
    }
    else if (ret == IPC_STOPPED )
    {
      DEB0((" Connection is STOPPED "));
    }
    else if (ret == IPC_BROKEN )
    {
      DEB0((" Connection is BROKEN "));
    }
    else if (ret == IPC_QUEUED )
    {
      DEB0((" Message is QUEUED"));
    }
    else
    {
      DEB0((" Connection has error %d - %p",ret,"error"));
    }
}

//-------------------------------------------------------------------------------
//                       MAIN
//-------------------------------------------------------------------------------

int main (int argc, char *argv[])
{
    char            sMsgFile[132+1] = "";
    ifstream        oMsgFile ;
    unsigned long   ulWait = 1 ;

    //\\//

    ENTERING("main(int argc, char *argv[])");

    // Set ITC debugging info
    //
    info("CoorAccess", KsVersion, argv[0]);
    deb(0,std::cout,true,true);

#ifdef WIN32
    //Allocate Console
    AllocConsole();

    //Get Handle to Standard Output created with Console
    ghConsole = GetStdHandle(STD_OUTPUT_HANDLE);
#endif

    //
    // Handle options
    //
    Get_Options getopt( argc, argv, "h:H:i:I:p:P:w:W:?", 1 );

    for ( int c; (c = getopt ()) != -1; )
    {

        switch ( c )
        {

            case 'i':
            case 'I':
              strcpy ( sMsgFile, getopt.optarg ) ;
              cout << "Opening "
                   << sMsgFile
                   << std::endl ;
              oMsgFile.open ( sMsgFile ) ;
              break;

            case 'h':
            case 'H':
              strcpy ( host, getopt.optarg ) ;
              break;

            case 'p':
            case 'P':
              portnum = atoi ( getopt.optarg ) ;
              break;

            case 'w':
            case 'W':
              char *pcEndChar, **ppcEndChar ;
              ppcEndChar = &pcEndChar ;
              ulWait = strtoul ( getopt.optarg, ppcEndChar, 10 ) ;
              break;

            case '?':
            default:
              ACE_ERROR_RETURN ((LM_ERROR,
      		             "usage: coor_access [-i <input_filename>] [-h <host>] [-?] \n"), 0);
        }
    }

    // advertize target server
    std::cout << "CoorAccess Connecting to <" << host << ":" << portnum << ">"
              << std::endl;

    // Create connector (client ) with the automatic reconnect
    SOCK_Auto_Connector con(portnum,host,0.5);
    con.set_send_buffer(130000);

    // Register callback
    register_callback ( con, IPC_Message::MSG_ID, got_message ) ;

    // start the receiver, sender
    con.start_recver ( 30.0 ) ;
    con.start_sender ( 10, 30.0 ) ;

    Thread_Util::delay ( (float) 1.0 ) ;

    Connection_Handle Con_Handle = con.get_handle() ;

    while ( Con_Handle.info().status != Connection_Info::STOPPED )
    {
        Thread_Util::delay ( (float) ulWait ); // give server a chance to service previous message

        if ( sMsgFile[0] == 0x00 )
        {// Interactive Input from Command Line
            cout << endl << "Msg or 'Q'=Quit : " ;

            // get a message from stdin
            cin.getline( &sStrBuf[iAfterMsgID], buf_size );

            if ( ( strcmp(&sStrBuf[iAfterMsgID],"bye")== 0 )
              || ( strcmp(&sStrBuf[iAfterMsgID],"Q")  == 0 )
              || ( strcmp(&sStrBuf[iAfterMsgID],"q")  == 0 ) ) break; // out of outer while loop
        }
        else
        {//Automiatic Input from specified Command File.
            if ( oMsgFile.eof () != 0 ) // for last line send the file name to appear in TCC logfile
            {
                strcpy ( &sStrBuf[iAfterMsgID], "# COOR_Access Done with Command File <" ) ;
                strcat ( &sStrBuf[iAfterMsgID], sMsgFile ) ;
                strcat ( &sStrBuf[iAfterMsgID], ">" ) ;
                sMsgFile[0] = 0x00 ; // from now on, switch to command line input
            }
            else
                // get next conmmand from input file
                oMsgFile.getline ( &sStrBuf[iAfterMsgID], buf_size-iAfterMsgID ) ;
        }

        // send the message
        send_message ( con, sStrBuf ) ;
    }


    std::cout << "CoorAccess closed to <"
              << Con_Handle.info().program_name
              << "> Version <" << Con_Handle.info().program_version
              << "> ITC port <"
              << Con_Handle.info().remote_host << ":" << Con_Handle.info().remote_port << ">"
              << std::endl;

    std::cout << "<CR> to exit" << std::endl;
    std::cin.getline( sStrBuf, 1 );

    return 0 ;

}
