/*-------------------------------------------------------------------------------*/
  /*
  L1Cal_IIb_TCC.cpp
  Main Program
  Start and stop L1Cal_IIb_TCC
  */
/*-------------------------------------------------------------------------------*/

#include "L1Cal_IIb_TCC/StdAfx.h"

#include                            "L1Cal_IIb_TCC/L1Cal_IIb_TCC.h"

#include "L1Cal_IIb_TCC/CustomDataTypes.h"
#include "L1Cal_IIb_TCC/HandleConsole.h"
#include "L1Cal_IIb_TCC/HandleLogFile.h"
#include "L1Cal_IIb_TCC/HandleHardwareAccess.h"
#include "L1Cal_IIb_TCC/ListOfEnvironmentVariables.h"

#include <xercesc/util/PlatformUtils.hpp>

#include "L1Cal_IIb_TCC/HandleRemoteConsole.h"
#include "L1Cal_IIb_TCC/HandleCoorConnection.h"
#include "L1Cal_IIb_TCC/HandleGuiConnection.h"
#include "L1Cal_IIb_TCC/MonitControl.h"

#include "L1Cal_IIb_TCC/GuiCommandParser.h"

#include "L1Cal_IIb_TCC/HandleCommandFiles.h"
#include "L1Cal_IIb_TCC/CommandFileMaster.h"
#include "L1Cal_IIb_TCC/ListOfMasterCommandFiles.h"

//#include "L1Cal_IIb_TCC/HsroL1Cal.h"

//#include "L1Cal_IIb_TCC/L1Cal.h"
#include "L1Cal_IIb_TCC/T962.h"

#ifdef WIN32
    // Global variable to remember the default drive letter
    extern int giDefaultDriveID ;
#endif

//#include <stdlib.h>
#ifdef WIN32
    #include <direct.h> // for _getdrive, _chdrive
#endif


/////////////////////////////////////////////////////////////////////////////
// L1Cal_IIb_TCC application
/////////////////////////////////////////////////////////////////////////////

int main () 
{

    L1Cal_IIb_Start() ;

    while ( TRUE ) 
    {
        Sleep ( 60* KiSleepOneSecond ) ; 

        // Use this oportunity to flush buffered data to logfile every minute
        CLogFile::FlushLogFile  ( ) ;

    }

    return 0 ;

}

/////////////////////////////////////////////////////////////////////////////
// L1Cal_IIb_TCC initialization
/////////////////////////////////////////////////////////////////////////////

int L1Cal_IIb_Start()
{

    CConsoleAccess  oConsole ;
    CLogFile        oLogFile ;

    ESuccess        eStatus = Ok ;

    //\\//

#ifdef WIN32

        //////////////////////////////////////////////////////////////////////
        //define environment variables for use by this executable only 
	    //(and spawned processes)
        _putenv ( KsEnvL1Cal_IIb_TCC ) ;
        _putenv ( KsEnvExo ) ;
        _putenv ( KsEnvDoc ) ;
        _putenv ( KsEnvRio ) ;
        _putenv ( KsEnvCnx ) ;
        _putenv ( KsEnvRnd ) ;

    #if defined ( SITE_MSU ) 

        _putenv ( KsEnvConfig_MSU ) ;
        _putenv ( KsEnvLog_MSU ) ;
        _putenv ( KsEnvMail_MSU ) ;

    #elif defined ( SITE_DZERO ) 

        _putenv ( KsEnvConfig_D0 ) ;
        _putenv ( KsEnvLog_D0 ) ;
        _putenv ( KsEnvMail_D0 ) ;

    #endif

#endif

    //////////////////////////////////////////////////////////////////////

    oConsole.NewConsole () ;

    //////////////////////////////////////////////////////////////////////

    oLogFile.NewLogFile () ;

    //////////////////////////////////////////////////////////////////////

    oConsole.WriteLine ( "------------------ L1Cal IIb TCC -----------------" ) ;
    oConsole.WriteLine ( "" ) ;
    
    //////////////////////////////////////////////////////////////////////
    // check for existence of externally defined precompiler constant 
    // for IO method mode and advertize configuration 

#if defined ( VME_IO_BIT3_MMAP ) 
    #pragma message ( " >>>>>>> VME R/W Access Mode: Direct via Memory Mapping" )
    oConsole.WriteLine ( "VME R/W Access Mode: Direct via Memory Mapping" ) ;

#elif defined ( VME_IO_BIT3_API ) 
    #pragma message ( " >>>>>>> VME R/W Access Mode: Individual Calls to Bit3 API" )
    oConsole.WriteLine ( "VME R/W Access Mode: Individual Calls to Bit3 API" ) ;

#elif defined ( VME_IO_FAKE )  
    #pragma message ( " >>>>>>> VME R/W Access Mode: Faked to allocated Memory" )
    oConsole.WriteLine ( "VME R/W Access Mode: Faked to allocated Memory" ) ;

#elif defined ( VME_IO_NONE )  
    #pragma message ( " >>>>>>> VME R/W Access Mode: None for development version" )
    oConsole.WriteLine ( "VME R/W Access Mode: None for development version" ) ;

#else
    #error    NO VME ACCESS OPTION HAS BEEN DEFINED 
     cause a compiler error when none of the above compile flags has been defined!
#endif


    //////////////////////////////////////////////////////////////////////
    // check for existence of externally defined precompiler constant 
    // for Trigger Site and advertize configuration 

#if defined ( SITE_MSU ) 
    #pragma message ( " >>>>>>> Hardware Configuration: MSU TestRack" )
    oConsole.WriteLine ( "Hardware Configuration: MSU TestRack" ) ;

#elif defined ( SITE_DZERO ) 
    #pragma message ( " >>>>>>> Hardware Configuration: DZero Trigger" )
    oConsole.WriteLine ( "Hardware Configuration: DZero Trigger" ) ;

#else
    #error    NO SITE LOCATION OPTION HAS BEEN DEFINED 
     cause a compiler error when none of the above compile flags has been defined!
#endif

    //////////////////////////////////////////////////////////////////////
    //write this much to the logfile for an early proof of logfile service
    oLogFile.FlushLogFile () ;

    //////////////////////////////////////////////////////////////////////
    // initialize the VME interface to the Framework
    CHardwareAccess*  poFrameworkInterface = new ( CHardwareAccess ) ; // use new instead of direct object for persistence

    if ( poFrameworkInterface->mg_poBit3BusAdaptor->m_eStatus != Ok )
    {
        oConsole.WriteLine ( "Error Initializing Access to the L1 Framework" ) ;
    }

    //////////////////////////////////////////////////////////////////////
    // create the L1Cal Readout Object
    //(before the L1Cal Object so that TabGab Cards can fine pointers to VRBs)
#if defined VME_IO_NO_VERTICAL  // test stand run without the vertical interconnect, i.e. no readout
#else
    CHsroL1Cal::mg_poHsroL1Cal        = new CHsroL1Cal () ;
#endif

    //////////////////////////////////////////////////////////////////////
    // create the L1Cal Object
//    CL1Cal::mg_poL1Cal                = new CL1Cal () ;
    CT962::mg_poT962                = new CT962 () ;

    // start DAQ thread
    CT962::mg_poT962-> start () ;


    //skip a line
    oConsole.WriteLine ( "" ) ;

    //////////////////////////////////////////////////////////////////////
    //start the remote console server
    InitRemoteConsole () ;

    //////////////////////////////////////////////////////////////////////
    //start the Monitoring Pool Manager, Monit Data Server, and Luminosity Server
//    InitMonitServices () ;

    //////////////////////////////////////////////////////////////////////
    //start the COOR Connection Server
    InitCoorConnectionServer () ;

    //////////////////////////////////////////////////////////////////////
    // initialize the Xerces library
    try 
    {
        XMLPlatformUtils::Initialize();
    }
    catch (const XMLException& toCatch) 
    {
        char* message = XMLString::transcode(toCatch.getMessage());
        oConsole.ErrorLine ( (CString) "Error during XML initialization "
                            + message ) ;
        CGuiCommandParser::XMLrelease( message ) ;
    }

    //////////////////////////////////////////////////////////////////////
    //start the GUI Connection Server
    InitGuiCommandServer () ;


    //////////////////////////////////////////////////////////////////////
    // Restrict Initial L1Cal Trigger Tower Coverage 
    // (may be overriden by boot_auxi)

//    CL1Cal::mg_poL1Cal->SetCoverageMagnEta   ( 1,  16 ) ;

    //////////////////////////////////////////////////////////////////////
    //Execute the Boot_Auxi File

    Custom_Structure_Master     xCustom ;
    CommandFileMaster ( osBoot_Auxi, &xCustom, NULL ) ;

    //////////////////////////////////////////////////////////////////////
    //write this much to the logfile before starting dialog box
    oLogFile.FlushLogFile () ;

    //skip a line
    oConsole.WriteLine ( "" ) ;


    return 0 ;
}

    
    
    
/////////////////////////////////////////////////////////////////////////////
// L1Cal_IIb_TCC shutdown
/////////////////////////////////////////////////////////////////////////////

int L1Cal_IIb_Stop()
{
    CConsoleAccess  oConsole ;
    CLogFile        oLogFile ;

    ESuccess        eStatus = Ok ;

    //\\//

    //////////////////////////////////////////////////////////////////////
    
    // last message and close logfile 
    oConsole.WriteLine ( "" ) ;
    oConsole.WriteLine ( "-------------------- Exit -------------------" ) ;

    // Flush Logfile (because the CloseLogFile never gets executed)
    oLogFile.FlushLogFile () ;

    //////////////////////////////////////////////////////////////////////
    //stop the COOR Connection Client
//    ShutdownCoorConnectionClient () ;

    //stop the COOR Connection Server
    ShutdownCoorConnectionServer () ;   // Weird: This somehow stops the application (delete gpConsumer)

    //stop the GUI Connection Server
    ShutdownGuiCommandServer () ;   

    //stop the Monitoring Pool Manager, Monit Data Server, and Luminosity Server
//    ShutdownMonitServices () ;

    //stop the remote console server
    ShutdownRemoteConsole () ;

    // delete the L1Cal and Readout Objects
    delete CT962::mg_poT962 ;
//    delete CL1Cal::mg_poL1Cal ;
//    delete CHsroL1Cal::mg_poHsroL1Cal ;

    // Close logfile 
    oLogFile.CloseLogFile () ;

	return 0 ;
}
