////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////

#include "L1Cal_IIb_TCC/StdAfx.h"
#include "L1Cal_IIb_TCC/ListOfIPPortNumbers.h"

#include <math.h>

#include "ace/Thread_Manager.h"
#include "thread_util/Thread_Util.hpp"
using     namespace thread_util ;

#include "ace/Get_Opt.h"
typedef ACE_Get_Opt Get_Options;

#define  _Debug_Project_String "TrgMon Access"

#include "itc/itc.hpp"
#include "itc/Opaque_Message.hpp"

// Monitoring Data Record definition
#include "trigmon/L1CalTcc_MonitData.hpp"
#include "trigmon/L1CalTcc_MonitData_Adf_OutputEt.hpp"

/*-------------------------------------------------------------------------------*/

char host[80] = "localhost";
int portnum = KiIP_Port_MonitServer ;

L1CalTcc_Block_Adf_OutputEt xBlock_Adf_Et_Previous ;
bool                    glAverage    = false ;
bool                    glDeviation  = false ;
unsigned int            guiLiveXidx  = 0 ; // default use first live X

#ifdef WIN32
HANDLE ghConsole ;    //Handle to Standard Output from Console
#endif

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
// simple routine to prepare highlighting when ADC count out of normal range
#define     KiFlagAdcRedLow         5
#define     KiFlagAdcBluLow         6
#define     KiFlagAdcBrtLow         7
#define     KiFlagAdcBrtHig         9
#define     KiFlagAdcBluHig        12
#define     KiFlagAdcRedHig        24

#define     KiFlagStdDevBrt         1
#define     KiFlagStdDevBlu         2
#define     KiFlagStdDevRed         3

bool CheckForHighlight ( unsigned char _ubTT_DAC ) 
{
    bool lDACFlagged = false ;
    //\\//

    if ( ( ( glDeviation == true ) 
        && ( _ubTT_DAC >= KiFlagStdDevBrt ) ) 
      || ( ( glDeviation == false ) 
        && ( ( _ubTT_DAC <= KiFlagAdcBrtLow ) 
          || ( _ubTT_DAC >= KiFlagAdcBrtHig ) ) ) )
    {
        lDACFlagged = true ;
        #ifdef WIN32 
        SetConsoleTextAttribute   ( ghConsole, 
                                    FOREGROUND_RED | FOREGROUND_GREEN | FOREGROUND_BLUE | FOREGROUND_INTENSITY ) ;
        #endif
    }

    if ( ( ( glDeviation == true ) 
        && ( _ubTT_DAC >= KiFlagStdDevBlu ) ) 
      || ( ( glDeviation == false ) 
        && ( ( _ubTT_DAC <= KiFlagAdcBluLow ) 
          || ( _ubTT_DAC >= KiFlagAdcBluHig ) ) ) )
    {
        lDACFlagged = true ;
        #ifdef WIN32 
        SetConsoleTextAttribute   ( ghConsole, 
                                    FOREGROUND_BLUE | FOREGROUND_INTENSITY ) ;
        #endif
    }

    if ( ( ( glDeviation == true ) 
        && ( _ubTT_DAC >= KiFlagStdDevRed ) ) 
      || ( ( glDeviation == false ) 
        && ( ( _ubTT_DAC <= KiFlagAdcRedLow ) 
          || ( _ubTT_DAC >= KiFlagAdcRedHig ) ) ) )
    {
        lDACFlagged = true ;
        #ifdef WIN32 
        SetConsoleTextAttribute   ( ghConsole, 
                                    FOREGROUND_RED | FOREGROUND_INTENSITY ) ;
        #endif
    }

    return lDACFlagged ;
}

void ClearHighlight ( ) 
{
    #ifdef WIN32 
    SetConsoleTextAttribute   ( ghConsole, 
                                FOREGROUND_RED | FOREGROUND_GREEN | FOREGROUND_BLUE ) ;
    #endif
}
/*-------------------------------------------------------------------------------*/
#ifdef WIN32 
    //#include "../src/UtilsBits.h" only used for the function below
    union MultiAccessQuad
    {
	    uint64 uqQuad ;
	    
        struct DoubleByLong
        {
		    unsigned long ulLong_0 ;
		    unsigned long ulLong_1 ;
        } ByLong ;
    } ;

    unsigned    long    fulTruncToLong    ( uint64  _uqBitMask ) 
    {
        MultiAccessQuad     xBitMask ;

        //\\//

        xBitMask.uqQuad = _uqBitMask ;

        return      xBitMask.ByLong.ulLong_0 ;
    }

    union MultiAccessLong
    {
	    unsigned long uqLong ;
	    
        struct LongByByte
        {
		    unsigned char ulByte_0 ;
		    unsigned char ulByte_1 ;
		    unsigned char ulByte_2 ;
		    unsigned char ulByte_3 ;
        } ByByte ;
    } ;

    unsigned    char    fubTruncToByte    ( unsigned long  _ulBitMask ) 
    {
        MultiAccessLong     xBitMask ;

        //\\//

        xBitMask.uqLong = _ulBitMask ;

        return      xBitMask.ByByte.ulByte_0 ;
    }

    char*  I_to_Hex ( const unsigned long ulNumber )
    {
	    static char Buf[20],  *pBuf = Buf ;
        Buf[0] = '0' ; Buf[1] = 'x' ; 

        _ultoa ( ulNumber, pBuf+2, 16 ) ;

	    return pBuf ;
    }

    char*  I_to_A ( const unsigned long ulNumber,
                    const unsigned long ulDigits = 0 )
    {
	    static char Buf[20],  *pBuf = Buf ;
	    int iLength ;
	    int iBlanks ;

        _itoa ( ulNumber, pBuf, 10 ) ;
        iLength = strlen( pBuf ) ;
        iBlanks = ulDigits - iLength ;
        if ( iBlanks > 0 )
        {
            for ( int i = iLength ; i >= 0 ; i-- ) // this includes the terminating null char
                Buf[i+iBlanks] = Buf[i] ;
            for ( int i = iBlanks-1 ; i >= 0 ; i-- )
                Buf[i] = ' ' ;
        }

	    return pBuf ;
    }

#else // not WIN32, simply use the L1Cal_IIb_TCC library

    #include "L1Cal_IIb_TCC/UtilsStrings.h"
    #include "L1Cal_IIb_TCC/UtilsBits.h"

#endif // WIN32
/*-------------------------------------------------------------------------------*/
// simple routine to describe the current trigger tower range
CString         f_osTrgTwr_Range  ( unsigned long   _ulTTEtaSignMin,
                                    unsigned long   _ulTTEtaSignMax,
                                    unsigned long   _ulTTEtaMagnMin,
                                    unsigned long   _ulTTEtaMagnMax,
                                    unsigned long   _ulTTPhiMin,
                                    unsigned long   _ulTTPhiMax ) 
{ // Show Range e.g. "eta(-20:+20),phi(1:32)"
  //              or "eta(+5),phi(6)"
  //              or "eta(-5),phi(6)"
  //              or "eta(+5:+8),phi(6)"
  //              or "eta(-8:-5),phi(6)"
  //              or "eta(-8:-5 & +5:+8),phi(1:32)"
  //              or "eta(-8 & +8),phi(1:32)"
  //              or "eta(-8:+8),phi(1:32)"

    TCString  osMsg ;

    //\\//

    if ( _ulTTEtaSignMax == eTT_Eta_Neg ) 
        osMsg = (CString) "eta(-" + I_to_A(_ulTTEtaMagnMax) ;
    else 
        osMsg = (CString) "eta(+" + I_to_A(_ulTTEtaMagnMin) ;

    if ( ( _ulTTEtaMagnMax != _ulTTEtaMagnMin ) 
      || ( _ulTTEtaSignMax != _ulTTEtaSignMin ) ) // otherwise eta already all done
    {
        if ( _ulTTEtaMagnMax != _ulTTEtaMagnMin ) 
        {
            if ( ( _ulTTEtaSignMax != _ulTTEtaSignMin )
              && ( _ulTTEtaMagnMin != eTT_Phi_1 ) ) // coverage does not cross zero
                osMsg += (CString) ":-" + I_to_A(_ulTTEtaMagnMin) 
                      + " & +" + I_to_A(_ulTTEtaMagnMin) ;

            if ( _ulTTEtaSignMin == eTT_Eta_Neg ) 
                osMsg += (CString) ":-" + I_to_A(_ulTTEtaMagnMin) ;
            else 
                osMsg += (CString) ":+" + I_to_A(_ulTTEtaMagnMax) ;
        }
        else // must be ( _ulTTEtaSignMax != _ulTTEtaSignMin ) && ( _ulTTEtaMagnMax == _ulTTEtaMagnMin ) 
        {
                osMsg += (CString) " & +" + I_to_A(_ulTTEtaMagnMin) ;
        }
    }
    
    osMsg += (CString) "),phi(" 
           + I_to_A(_ulTTPhiMin) ;

    if ( _ulTTPhiMax != _ulTTPhiMin )
        osMsg += (CString) ":" + I_to_A(_ulTTPhiMax) ;

    osMsg += ")" ;

    return osMsg ;
}



_ITC_NAMESPACE_USING

//-------------------------------------------------------------------------------
//
// Functions Displaying the Data
//
//-------------------------------------------------------------------------------

/*-------------------------------------------------------------------------------*/
//simple function to calculate the average value of the ADC count sample
unsigned char FindAverage ( unsigned char* _aubADC )
{
    unsigned long ulTemp = 0 ;
    unsigned long ulAverage = 0 ;
    unsigned long ulLiveXidx ;
    //\\//
    for ( ulLiveXidx = 0 ;
          ulLiveXidx < kL1CalTcc_Tot_LiveBX ;
          ulLiveXidx++ )
        ulTemp += _aubADC[ulLiveXidx] ;

    ulAverage = ( ulTemp + kL1CalTcc_Tot_LiveBX/2 )
              / kL1CalTcc_Tot_LiveBX ;

    return fubTruncToByte ( ulAverage ) ;
}
/*-------------------------------------------------------------------------------*/
//simple function to calculate the Standard Deviation of the ADC count sample
unsigned char FindDeviation ( unsigned char* _aubADC )
{
    unsigned char   ubAverage = FindAverage ( _aubADC ) ;
    long            ilScratch ;
    unsigned long   ulTemp = 0 ;
    double          ufDeviation ;
    unsigned long   ulLiveXidx ;
    //\\//
    for ( ulLiveXidx = 0 ;
          ulLiveXidx < kL1CalTcc_Tot_LiveBX ;
          ulLiveXidx++ )
    {
        ilScratch = ( _aubADC[ulLiveXidx] - ubAverage ) ;
        ulTemp += ilScratch * ilScratch ;
    }

    ufDeviation = sqrt ( (double) ulTemp 
                       / kL1CalTcc_Tot_LiveBX ) ;

    return fubTruncToByte ( (long) (ufDeviation + 0.5) ) ; //typecast conversion will truncate
}
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
//simple function to display one ADC count (one slice or average)
void DisplayOneAdc ( unsigned char* _aubADC )
{
    unsigned char   ubTT_DAC ;
    bool            lDACFlagged ;
    //\\//

    if ( guiLiveXidx >= kL1CalTcc_Tot_LiveBX ) 
    {
        std::cout   << " Error: Slice Index must be 0<=N<=35" 
                    << std::endl ;
        guiLiveXidx = 0 ;
    }

    if ( glAverage == true )
        ubTT_DAC = FindAverage ( _aubADC ) ;
    else if ( glDeviation == true )
        ubTT_DAC = FindDeviation ( _aubADC ) ;
    else
        ubTT_DAC = _aubADC [guiLiveXidx] ;

    lDACFlagged = CheckForHighlight ( ubTT_DAC ) ;

    std::cout << I_to_A( ubTT_DAC, 4 ) ;

    if ( lDACFlagged == true ) ClearHighlight () ;
    
    //strcat ( sLineHD, I_to_A( ubTT_DAC, 4 ) ) ; // for a logfile?
}
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
bool CheckSanityRangeBoundaries ( L1CalTcc_Block_Adf_OutputEt*  _pxBlock_Adf_Et ) 
{
    bool    lReturnValue ;
    //\\//
    // Check sanity of advertized coverage boundaries
    if ( _pxBlock_Adf_Et->xHeader.ubTTEtaSignMin > _pxBlock_Adf_Et->xHeader.ubTTEtaSignMax ) 
    {
        std::cout   << " Error: TT Eta Sign Boundary Ordering :" 
                    << " Min=" << _pxBlock_Adf_Et->xHeader.ubTTEtaSignMin 
                    << ";Max=" << _pxBlock_Adf_Et->xHeader.ubTTEtaSignMax 
                    << std::endl ;
        lReturnValue = false ;
    }
    if ( _pxBlock_Adf_Et->xHeader.ubTTEtaSignMax >= kL1CalTcc_Tot_TT_Eta_Sign ) 
    {
        std::cout   << " Error: Max TT Eta Sign out of Range:" 
                    << " Min=" << _pxBlock_Adf_Et->xHeader.ubTTEtaSignMin 
                    << ";Max=" << _pxBlock_Adf_Et->xHeader.ubTTEtaSignMax 
                    << std::endl ;
        lReturnValue = false ;
    }

    if ( _pxBlock_Adf_Et->xHeader.ubTTEtaMagnMin > _pxBlock_Adf_Et->xHeader.ubTTEtaMagnMax ) 
    {
        std::cout   << " Error: TT Eta Magn Boundary Ordering :" 
                    << " Min=" << _pxBlock_Adf_Et->xHeader.ubTTEtaMagnMin 
                    << ";Max=" << _pxBlock_Adf_Et->xHeader.ubTTEtaMagnMax 
                    << std::endl ;
        lReturnValue = false ;
    }
    if ( ( _pxBlock_Adf_Et->xHeader.ubTTEtaMagnMin == 0 ) 
      || ( _pxBlock_Adf_Et->xHeader.ubTTEtaMagnMax >  kL1CalTcc_Tot_TT_Eta_Magn ) )
    {
        std::cout   << " Error: Max TT Eta Magn out of Range:" 
                    << " Min=" << _pxBlock_Adf_Et->xHeader.ubTTEtaMagnMin 
                    << ";Max=" << _pxBlock_Adf_Et->xHeader.ubTTEtaMagnMax 
                    << std::endl ;
        lReturnValue = false ;
    }

    if ( _pxBlock_Adf_Et->xHeader.ubTTPhiMin > _pxBlock_Adf_Et->xHeader.ubTTPhiMax ) 
    {
        std::cout   << " Error: TT Phi Boundary Ordering :" 
                    << " Min=" << _pxBlock_Adf_Et->xHeader.ubTTPhiMin 
                    << ";Max=" << _pxBlock_Adf_Et->xHeader.ubTTPhiMax 
                    << std::endl ;
        lReturnValue = false ;
    }
    if ( ( _pxBlock_Adf_Et->xHeader.ubTTPhiMin == 0 ) 
      || ( _pxBlock_Adf_Et->xHeader.ubTTPhiMax >  kL1CalTcc_Tot_TT_Phi ) )
    {
        std::cout   << " Error: Max TT Phi out of Range:" 
                    << " Min=" << _pxBlock_Adf_Et->xHeader.ubTTPhiMin 
                    << ";Max=" << _pxBlock_Adf_Et->xHeader.ubTTPhiMax 
                    << std::endl ;
        lReturnValue = false ;
    }

    return lReturnValue ;
}
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
void Display_ADC ( L1CalTcc_Block_Adf_OutputEt*  _pxBlock_Adf_Et )
{
    unsigned int    uiTTEtaSign ;
    char            cEtaSign ;
    unsigned int    uiTTEtaMagn ;
    long            ilTTEtaMagnIncr ;
    unsigned int    uiTTPhi ;

    char            sLineEM[80+1] ;
    char            sLineHD[80+1] ;

    //\\//

    // Check sanity of advertized TT coverage boundaries
    if ( CheckSanityRangeBoundaries ( _pxBlock_Adf_Et ) == false ) goto Abort ;

    // Display the array of TT ADC values
    // We are going to display in the following order -20,..,-1,+1,.,+20
    // (We use this "weird" loop end test because decrementing the unsigned integer 
    // past 0 gives large positive number)
    for ( uiTTEtaSign = _pxBlock_Adf_Et->xHeader.ubTTEtaSignMax ; 
          ( uiTTEtaSign == _pxBlock_Adf_Et->xHeader.ubTTEtaSignMax ) 
       || ( uiTTEtaSign == _pxBlock_Adf_Et->xHeader.ubTTEtaSignMin ) ;
          uiTTEtaSign -- )
    {
        std::cout   << std::endl ;

        if ( uiTTEtaSign == kL1CalTcc_Tot_TT_Eta_Pos )
        {
            cEtaSign = '+' ;
            uiTTEtaMagn = _pxBlock_Adf_Et->xHeader.ubTTEtaMagnMin ;
            ilTTEtaMagnIncr = +1 ;
        }
        else 
        {
            cEtaSign = '-' ;
            uiTTEtaMagn = _pxBlock_Adf_Et->xHeader.ubTTEtaMagnMax ;
            ilTTEtaMagnIncr = -1 ;
        }

        // use "while" because we increment for positive eta and decrement for negative eta
        while ( ( uiTTEtaMagn >= _pxBlock_Adf_Et->xHeader.ubTTEtaMagnMin )
             && ( uiTTEtaMagn <= _pxBlock_Adf_Et->xHeader.ubTTEtaMagnMax ) ) 
        {
            //Start with Lower Half of Phi values (1:16)
            /////////////////////////////////////////////////

            if ( _pxBlock_Adf_Et->xHeader.ubTTPhiMin <= kL1CalTcc_Tot_TT_Phi/2 )
            {
                //Start with HD ADC Counts
                sprintf ( sLineHD, 
                          " Eta=%c%2.2u   HD", 
                          cEtaSign, uiTTEtaMagn ) ;
                std::cout   << sLineHD ;
                //Add each Lower Phi Values to the prepared Lines
                for ( uiTTPhi = 1 ; 
                      uiTTPhi <= kL1CalTcc_Tot_TT_Phi/2 ; 
                      uiTTPhi++ )
                    DisplayOneAdc ( _pxBlock_Adf_Et->a4ubHD_Et  [uiTTEtaSign]
                                                                [uiTTEtaMagn-1]
                                                                [uiTTPhi-1] ) ;
                std::cout << std::endl ;// Terminate the line

                //Now for EM ADC Counts
                strcpy  ( sLineEM, " Phi= 1:16 EM" ) ;
                std::cout   << sLineEM ;
                //Add each Lower Phi Values to the prepared Lines
                for ( uiTTPhi = 1 ; 
                      uiTTPhi <= kL1CalTcc_Tot_TT_Phi/2 ; 
                      uiTTPhi++ )
                    DisplayOneAdc ( _pxBlock_Adf_Et->a4ubEM_Et  [uiTTEtaSign]
                                                                [uiTTEtaMagn-1]
                                                                [uiTTPhi-1] ) ;
                std::cout << std::endl ;// Terminate the line
            }

            //Now repeat for Upper Half of Phi values (17:32)
            /////////////////////////////////////////////////
            if ( _pxBlock_Adf_Et->xHeader.ubTTPhiMax > kL1CalTcc_Tot_TT_Phi/2 )
            {
                //Start with HD ADC Counts
                sprintf ( sLineHD, 
                          " Eta=%c%2.2u   HD", 
                          cEtaSign, uiTTEtaMagn ) ;
                std::cout   << sLineHD ;
                //Add each Lower Phi Values to the prepared Lines
                for ( uiTTPhi = kL1CalTcc_Tot_TT_Phi/2 +1 ; 
                      uiTTPhi <= kL1CalTcc_Tot_TT_Phi ; 
                      uiTTPhi++ )
                    DisplayOneAdc ( _pxBlock_Adf_Et->a4ubHD_Et  [uiTTEtaSign]
                                                                [uiTTEtaMagn-1]
                                                                [uiTTPhi-1] ) ;
                std::cout << std::endl ;// Terminate the line

                //Now for EM ADC Counts
                strcpy  ( sLineEM, " Phi=17:32 EM" ) ;
                std::cout   << sLineEM ;
                //Add each Lower Phi Values to the prepared Lines
                for ( uiTTPhi = kL1CalTcc_Tot_TT_Phi/2 +1 ; 
                      uiTTPhi <= kL1CalTcc_Tot_TT_Phi ; 
                      uiTTPhi++ )
                    DisplayOneAdc ( _pxBlock_Adf_Et->a4ubEM_Et  [uiTTEtaSign]
                                                                [uiTTEtaMagn-1]
                                                                [uiTTPhi-1] ) ;
                std::cout << std::endl ;// Terminate the line
            }

            //Skip a line between Eta values
            std::cout   << std::endl ;

            //Increment or Decrement depending on Sign Eta 
            uiTTEtaMagn += ilTTEtaMagnIncr ;
        }
        std::cout   << "- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -" 
                    << std::endl ;
    }

	if ( glAverage == true )
        std::cout   << "Display Mode Used: Average (For Live Crossing Index 0:"
                    << I_to_A(kL1CalTcc_Tot_LiveBX-1)
                    << ")"
                    << std::endl;
	else if ( glDeviation == true )
        std::cout   << "Display Mode Used: Standard Deviation (For Live Crossing Index 0:"
                    << I_to_A(kL1CalTcc_Tot_LiveBX-1)
                    << ")"
                    << std::endl;
    else
        std::cout   << "Display Mode Used: Live Crossing Index #"
                    << guiLiveXidx 
                    << std::endl;
Abort:

    std::cout   << std::endl ;
}
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
void Display_Data (L1CalTcc_Block_Adf_OutputEt*  _pxBlock_Adf_Et_Current )
{

    static bool     first = TRUE ;
    static float    fBeamX_Rate  = (float)kL1CalTcc_BeamX_Rate ;
    static uint64   uPrevious_BeamX ;

    uint64          uCurrent_BeamX ;
    long            iDelta_BeamX ;
    float           fDelta_BeamX ;
//    float           fDelta_Count ;

    //\\//

    // Note: In a real application we should keep all 64 bits in arithmetic
    uCurrent_BeamX = kL1CalTcc_L1fw_Max_TickNum * fulTruncToLong(_pxBlock_Adf_Et_Current->xBeamX.uqTurnNum)
                   + _pxBlock_Adf_Et_Current->xBeamX.ulTickNum ;

    if ( first ) 
    {
        first = FALSE ;
        iDelta_BeamX = 0 ;
    }
    else
    {
        if ( uCurrent_BeamX > uPrevious_BeamX ) 
            iDelta_BeamX = fulTruncToLong( uCurrent_BeamX - uPrevious_BeamX ) ;
        else 
            iDelta_BeamX = 0 ;

        fDelta_BeamX = (float) iDelta_BeamX ;
    }


    //Display the array of Trigger Tower ADC
    Display_ADC ( _pxBlock_Adf_Et_Current ) ;


    // display current coverage
    std::cout   << "Currrent L1Cal Coverage : "
                <<  f_osTrgTwr_Range  ( _pxBlock_Adf_Et_Current->xHeader.ubTTEtaSignMin,
                                        _pxBlock_Adf_Et_Current->xHeader.ubTTEtaSignMax,
                                        _pxBlock_Adf_Et_Current->xHeader.ubTTEtaMagnMin,
                                        _pxBlock_Adf_Et_Current->xHeader.ubTTEtaMagnMax,
                                        _pxBlock_Adf_Et_Current->xHeader.ubTTPhiMin,
                                        _pxBlock_Adf_Et_Current->xHeader.ubTTPhiMax ) 
                << std::endl;
                                    
    // special flags
    if ( _pxBlock_Adf_Et_Current->xHeader.ubSystemNonOperational != 0 ) 
        std::cout   << "L1Cal Trigger is Flagged Non-Operational"
                    << std::endl;

    if ( _pxBlock_Adf_Et_Current->xHeader.ubMonitDataNotCurrent != 0 ) 
        std::cout   << "Monit Data is Flagged Not Current"
                    << std::endl;

    if ( _pxBlock_Adf_Et_Current->xHeader.ubTriggeredBeamX != 0 ) 
        std::cout   << "Monit Data is from a Triggered Crossing"
                    << std::endl;
    else
        std::cout   << "Monit Data is from a Random Crossing"
                    << std::endl;

    // Display Tick and Turn Scaler Counts
    std::cout   << "Tick / Turn = "
				<< (int) _pxBlock_Adf_Et_Current->xBeamX.ulTickNum
                << " / "
				<< fulTruncToLong(_pxBlock_Adf_Et_Current->xBeamX.uqTurnNum) 
                << std::endl;

    if ( iDelta_BeamX != 0 )
        std::cout   << "Integration Period Between Last Two Snapshots = " 
                    << fDelta_BeamX / fBeamX_Rate 
                    << " s" 
                    << std::endl;

    //Remember this snapshot which will be used as Previous snapshot in the next call
    //note: we actually copy the data, not just the pointer, because the
    //      ITC message holding this data will be deleted soon.
    xBlock_Adf_Et_Previous = *_pxBlock_Adf_Et_Current ;
    uPrevious_BeamX = uCurrent_BeamX ;

    return ;
}

//-------------------------------------------------------------------------------
//
// Function receiving all messages
//
//-------------------------------------------------------------------------------

void got_message(const IPC_Message &m)
{
    Opaque_Message*             pMsg ;
    Connection_Handle           Con_Handle ;
    L1CalTcc_Block_Adf_OutputEt*    pMonitBlock ;

    //\\//

    switch ( m.ID() ) 
    {

    case Opaque_Message::MSG_ID :
        // This is the Monitoring data payload
        pMsg = new Opaque_Message ( m ) ;
        pMonitBlock = (L1CalTcc_Block_Adf_OutputEt*) pMsg->data() ;
        if ( pMonitBlock->xHeader.ulBlockType == eL1CalTcc_BlockType_Adf_OutputEt )
            Display_Data ( pMonitBlock ) ;
        else 
            std::cout << "Error: Message Received Has Wrong Type = " 
                      << pMonitBlock->xHeader.ulBlockType
                      << std::endl;
        delete pMsg ;
    break;

    case Conn_Start_Msg::MSG_ID :
        std::cout << std::endl;
        std::cout << "-----------------------------------------------------------------------------" 
                  << std::endl;
        INFO(("Client Got Message - connection START message\n"));
//        m.get_connection().info().dump();
        Con_Handle = m.get_connection() ;
        std::cout << "Connected to <"
                  << Con_Handle.info().program_name
                  << "> Version <" << Con_Handle.info().program_version
                  << "> ITC port <" 
                  << Con_Handle.info().remote_host << ":" << Con_Handle.info().remote_port << ">"
                  << std::endl;
        std::cout << "-----------------------------------------------------------------------------" 
                  << std::endl;
    break;

    case Conn_Stop_Msg::MSG_ID :
        std::cout << "-----------------------------------------------------------------------------" 
                  << std::endl;
        INFO(("Client Got Message - connection STOP message\n"));
//        m.get_connection().info().dump();
    break;

    case Conn_Brkn_Msg::MSG_ID :
        std::cout << "-----------------------------------------------------------------------------" 
                  << std::endl;
        INFO(("Client Got Message - connection BROKEN message\n"));
//        m.get_connection().info().dump();
//        m.get_connection().stop() ;
    break;

    case Conn_Open_Msg::MSG_ID :
        std::cout << "-----------------------------------------------------------------------------" 
                  << std::endl;
        INFO(("Client Got Message - connection RECONNECTED message\n"));
//        m.get_connection().info().dump();
    break;

    default:

        INFO(("Client Got Message = ID=%2d  - No=%5u length %3u  remote=%5u  reply no=%5u\n",
        m.ID(),m.no(),m.length(),m.remote_no(),m.reply_no()));

    }
}

//-------------------------------------------------------------------------------
//
// Function which sends messages
//
//-------------------------------------------------------------------------------

void send_request ( SOCK_Connector &con, L1CalTcc_BlockType   _eRequest_Type ) 
{

    L1CalTcc_Monit_Request   xRequest ;

    //\\//

    ENTERING ( "send_request(SOCK_Connector &con, char* msg_string)" ) ;

    xRequest.ulBlockType = _eRequest_Type ;

    IPC_STATUS ret = con.send_now ( Opaque_Message( &xRequest, sizeof(L1CalTcc_Monit_Request) ), 1.0 ) ;

    if ( ret == IPC_SUCCESS ) 
    {
    }
    else if (ret == IPC_TIMEOUT ) 
    {
      DEB0((" There is a timeout for send "));
    }
    else if (ret == IPC_STOPPED ) 
    {
      DEB0((" Connection is STOPPED "));
    }      
    else if (ret == IPC_BROKEN ) 
    {      
      DEB0((" Connection is BROKEN "));
    }      
    else if (ret == IPC_QUEUED ) 
    {
      DEB0((" Message is QUEUED"));
    }      
    else 
    {
      DEB0((" Connection has error %d - %p",ret,"error"));
    }
}

//-------------------------------------------------------------------------------
//                       MAIN
//-------------------------------------------------------------------------------

int main (int argc, char *argv[]) 
{
    bool            lAuto_Update = false ;

    //\\//
  
    ENTERING("main(int argc, char *argv[])");

    // Set ITC debugging info 
    //
    info("Toy Cal ADC", KsVersion, argv[0]);
    deb(0,std::cout,true,true);

#ifdef WIN32
    //Allocate Console 
    AllocConsole();

    //Get Handle to Standard Output created with Console
    ghConsole = GetStdHandle(STD_OUTPUT_HANDLE);
#endif

    //
    // Handle options
    //

    Get_Options getopt( argc, argv, "01234567aAh:H:p:P:sSvV?", 1 );

    for ( int c; (c = getopt ()) != -1; ) 
    switch ( c ) 
    {
        case '0':
        case '1':
        case '2':
        case '3':
        case '4':
        case '5':
        case '6':
        case '7':
          guiLiveXidx = c - '0' ;
          break;      

        case 'a':
        case 'A':
          lAuto_Update = true ;
          break;      

        case 'h':
        case 'H':
          strcpy ( host, getopt.optarg ) ;
          break;      

        case 'p':
        case 'P':
          portnum = atoi ( getopt.optarg ) ;
          break;      

        case 'v':
        case 'V':
          glAverage = true ;
          break;      

        case 's':
        case 'S':
          glDeviation = true ;
          break;      

        case '?':
        default:
          ACE_ERROR_RETURN ((LM_ERROR,
      		         "usage: trgmon_hsro [-a] [-h <host>] [-?] \n"), 0);
    }

    // advertize target server
    std::cout << "Toy Cal ADC Connecting to <" << host << ":" << portnum << ">"
              << std::endl;

    // Create connector (client ) with the automatic reconnect
    SOCK_Auto_Connector con(portnum,host,0.5);  
    con.set_send_buffer(130000);

    // Register callback
    register_callback ( con, IPC_Message::MSG_ID, got_message ) ;

    // start the receiver, sender
    con.start_recver ( 30.0 ) ;
    con.start_sender ( 10, 30.0 ) ;

    //Wait a bit for the connection to be made before displaying messages
    Thread_Util::delay ( (float) 3.0 ); 

    const size_t buf_size = 1024;
    char buf[ buf_size ];
    Connection_Handle Con_Handle = con.get_handle() ;
    while ( Con_Handle.info().status != Connection_Info::STOPPED )
    {
        buf[0] = 0x00 ;
        if ( !lAuto_Update ) 
        {
            // get a message from stdin
            cout << endl << "<CR>=Refresh,'0:35'=LiveXindex#,'V'=Average, 'S'=Deviation, 'Q'=Quit : " ;
            cin.getline( buf, buf_size ) ;

            if ( ( strcmp(buf,"bye")== 0 )
              || ( strcmp(buf,"Q")  == 0 )
              || ( strcmp(buf,"q")  == 0 ) ) break; // out of outer while loop
        
            if ( ( strcmp(buf,"S")  == 0 )  
              || ( strcmp(buf,"s")  == 0 ) ) { glDeviation = true ; 
                                               glAverage   = false ; } 

            if ( ( strcmp(buf,"V")  == 0 )  
              || ( strcmp(buf,"v")  == 0 ) ) { glAverage   = true ; 
                                               glDeviation = false ; } 

            if ( ( strcmp(buf,"0")  == 0 )  
              || ( ( atoi(buf) > 0 ) 
                && ( atoi(buf) < kL1CalTcc_Tot_LiveBX ) ) ) { guiLiveXidx  = atoi(buf) ; 
                                                              glAverage    = false ; 
                                                              glDeviation  = false ; }

            if ( ( strcmp(buf,"AUTO")  == 0 )  
              || ( strcmp(buf,"auto")  == 0 ) ) lAuto_Update = true ;
        }

        // send the request
        if ( buf[0] == 0x00 )
          send_request ( con, eL1CalTcc_BlockType_Adf_OutputEt ) ; // get a new sample
        else
          Display_ADC ( &xBlock_Adf_Et_Previous ) ; // just re-display the same data, maybe for another slice

        //hold off a bit for the asynchronous display
        Thread_Util::delay ( (float) 2.0 ); 
    }


    std::cout << "Toy Cal ADC closed to <"
              << Con_Handle.info().program_name
              << "> Version <" << Con_Handle.info().program_version
              << "> ITC port <" 
              << Con_Handle.info().remote_host << ":" << Con_Handle.info().remote_port << ">"
              << std::endl;
 
    std::cout << "<CR> to exit" << std::endl;
    std::cin.getline( buf, 1 );

    return 0 ;

}
////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////
