/*---------------------------------------------------------
  InterfaceItc.h
    Simple Interface to ITC functionality
    Callable from C for Python interface
    18-oct-2004
/*---------------------------------------------------------*/

#include "L1Cal_IIb_TCC/StdAfx.h"

#include "InterfaceItc.h"

#include "L1Cal_IIb_TCC/HandleItcClient.h"
#include "itc/general.hpp"

/*---------------------------------------------------------*/
/*---------------------------------------------------------*/
// global variables
char cached_itc_version[] = "0123456789" ; //and provide storage 

/*---------------------------------------------------------*/
/*---------------------------------------------------------*/

char*   itc_version         ( ) 
{

    strncpy ( cached_itc_version, itc::version.c_str(), 10 ) ;

    return cached_itc_version ;

} ;


int itc_new_client   ( int _portnum, char* _host ) 
{
    ItcClient*      new_client ;

    //\\//

    new_client = new ItcClient ( (unsigned short) _portnum, _host ) ;

    return (int) new_client ;

} ;



int itc_send_message ( int _client, char* _message ) 
{
    ESuccess        eStatus ;
    ItcClient*      existing_client = (ItcClient*) _client;

    //\\//

    //clear the last reply
    eStatus = existing_client->ClearReply  ( ) ;
    //send the message 
    eStatus = existing_client->SendMessage ( _message ) ;

    return (int) eStatus ;

} ;


int itc_get_reply    ( int _client, char* _reply, int _maxlen, int _timeout_sec ) 
{
    ESuccess        eStatus ;
    ItcClient*      existing_client = (ItcClient*) _client;
    CString         reply ;
    int             iWait ;
    const unsigned long OneTenthSecond = 100 ; // 100 milliseconds

    //\\//

    for ( iWait = 0 ; iWait <= 10*_timeout_sec ; iWait ++ ) 
    {
        eStatus = existing_client->LastReply( &reply ) ;

        if ( eStatus == Ok )
            break ;
        else
            Sleep ( OneTenthSecond ) ;
    }

    if ( ( eStatus == Ok )
      && ( iWait <= 10*_timeout_sec ) ) 
    {
        strncpy ( _reply, reply.c_str(), _maxlen ) ;
        _reply[_maxlen-1] = 0 ; // force string termination
    }
    else
        strcpy ( _reply, "reply timeout") ;

    return (int) eStatus ;

} ;


void itc_del_client   ( int _client ) 
{
    ItcClient*      existing_client = (ItcClient*) _client;

    //\\//

    delete ( existing_client ) ;

    return ;

} ;
