/*-------------------------------------------------------------------------------*/
  /*
    AdfCardBase.cpp
    16-Nov-2004
  */
/*-------------------------------------------------------------------------------*/

#include "L1Cal_IIb_TCC/StdAfx.h"

#include "L1Cal_IIb_TCC/GlobalConstants.h"
#include "L1Cal_IIb_TCC/CustomDataTypes.h"

#include                            "L1Cal_IIb_TCC/AdfCardBase.h"

#include "L1Cal_IIb_TCC/HandleConsole.h"
#include "L1Cal_IIb_TCC/HandleResultFile.h"
#include "L1Cal_IIb_TCC/UtilsStrings.h"


/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
// static  variable initialization

const unsigned short    CAdfCardBase ::m_gkawPRNSeed[ KiTotEtaPerAdf ]
                                                    [ KiTotPhiPerAdf ]
                                                    [ eTot_TT_Chan_Type ]
                              = { 0x0000,       // Channel Link signal  TX_0
                                  0xfffe,       // Channel Link signal  TX_1
                                  0x012e,       // Channel Link signal  TX_2
                                  0x0c26,       // Channel Link signal  TX_3
                                  0xa128,       // Channel Link signal  TX_4
                                  0x16ea,       // Channel Link signal  TX_5
                                  0x2ab4,       // Channel Link signal  TX_6
                                  0x36ba,       // Channel Link signal  TX_7
                                  0xaa00,       // Channel Link signal  TX_8
                                  0x0332,       // Channel Link signal  TX_9
                                  0x1ffe,       // Channel Link signal  TX_10
                                  0xa002,       // Channel Link signal  TX_11
                                  0x1f0e,       // Channel Link signal  TX_12
                                  0xc002,       // Channel Link signal  TX_13
                                  0x7202,       // Channel Link signal  TX_14
                                  0xe412,       // Channel Link signal  TX_15
                                  0x7208,       // Channel Link signal  TX_16
                                  0x3208,       // Channel Link signal  TX_17
                                  0x0018,       // Channel Link signal  TX_18
                                  0x0019,       // Channel Link signal  TX_19
                                  0x0020,       // Channel Link signal  TX_20
                                  0x0021,       // Channel Link signal  TX_21
                                  0x0022,       // Channel Link signal  TX_22
                                  0x0023,       // Channel Link signal  TX_23
                                  0x0024,       // Channel Link signal  TX_24
                                  0x0025,       // Channel Link signal  TX_25
                                  0x0026,       // Channel Link signal  TX_26
                                  0x0027,       // Channel Link signal  TX_27
                                  0x0028,       // Channel Link signal  TX_28
                                  0x0029,       // Channel Link signal  TX_29
                                  0x0030,       // Channel Link signal  TX_30
                                  0x0031 } ;    // Channel Link signal  TX_31

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
CAdfCardBase::CAdfCardBase()
{

}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
CAdfCardBase::CAdfCardBase    ( unsigned char   _ubVertMaster,
                                unsigned char   _ubVertSlave,
                                unsigned char   _ubCardSlot,
                                CAdfCardBase**  _ppoPrevCard,
                                CString*        _posCardDescription,
                                unsigned long   _ulSign_Eta,
                                unsigned long   _ulMagn_Eta_By4,
                                unsigned long   _ulPhi_By4 )
{

    //\\//

    m_ubVertMaster  = _ubVertMaster ;
    m_ubVertSlave   = _ubVertSlave ;
    m_ubCardSlot    = _ubCardSlot ;

    m_ulSign_Eta    = _ulSign_Eta     ;
    m_ulMagn_Eta_By4= _ulMagn_Eta_By4 ;
    m_ulPhi_By4     = _ulPhi_By4      ;

    if ( _posCardDescription == NULL )
        m_osCardDescription   = "Generic Adf Card" ;
    else
        m_osCardDescription   = *_posCardDescription ;

    BasicConstruct ( _ppoPrevCard ) ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
void     CAdfCardBase::BasicConstruct  ( CAdfCardBase**     _ppoPrevCard )
{

    m_ulVmeAddress = CAdfRegBase::ConvertToVmeAddress ( m_ubVertMaster,
                                                        m_ubVertSlave,
                                                        m_ubCardSlot,
                                                        0,
                                                        0 ) ;
    m_poFirstFpga = NULL ;
    m_poLastFpga  = NULL ;

    //start out with no Chip
    //(which is immediately automatically incremented for the Vme Int and Suport Fpga below)
    m_uwTotChip = 0 ;

    m_lDeSkewed = false ;

    //All cards have one Board Control Pal
    m_poBcPal          = new CAdfChipPal      ( this,        // link back to mother card
                                                0 ) ;        // Chip Address

    for ( unsigned long   ulFpgaNum = 0 ; ulFpgaNum < KiTotFpgaPerAdf ; ulFpgaNum ++ )
    {
        //Access to Data Path Fpgas (maybe not)
//        m_apoDpFpga[ulFpgaNum] = new CAdfChipFpga     ( this,        // link back to mother card
//                                                        ulFpgaNum ) ;        // Chip Address

        //T3 Adf Fpgas (maybe not there)
//        m_apoT3Fpga[ulFpgaNum] = new CAdfChipFpga_T3  ( this,        // link back to mother card
//                                                        ulFpgaNum ) ;        // Chip Address
        //T4 Adf Fpgas (maybe not there)
//        m_apoT4Fpga[ulFpgaNum] = new CAdfChipFpga_T4  ( this,        // link back to mother card
//                                                        ulFpgaNum ) ;        // Chip Address
        //T7 Adf Fpgas (maybe not there)
        m_apoT7Fpga[ulFpgaNum] = new CAdfChipFpga_T7  ( this,        // link back to mother card
                                                        ulFpgaNum ) ;        // Chip Address
    }

    // High level Access to Control Pedestal DACs
    m_poDacCtrl        = new CAdfDacCtrl  ( m_poBcPal ) ;

    // grab the pointer to the previous Card in the chain,
    // and return a pointer to *this* Card to be used as the next "previous card"
    // or as the last card recorded for this Crate.
    if ( _ppoPrevCard != NULL ) // but only if this optional parameter was provided
    {
        m_poPrevCard    = *_ppoPrevCard ;
        *_ppoPrevCard = this ;
        if ( m_poPrevCard != NULL )
            m_poPrevCard->m_poNextCard = this ; // update the forward chain
    }
    else
    {
        m_poPrevCard = NULL ;
    }

    //There is no next card yet,
    //the next card to be defined will update this pointer,
    //as done just above
    m_poNextCard = NULL ;

}
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
CAdfCardBase::~CAdfCardBase()
{
    delete ( m_poBcPal ) ;
    for ( unsigned long   ulFpgaNum = 0 ; ulFpgaNum < KiTotFpgaPerAdf ; ulFpgaNum ++ )
    {
//        delete ( m_apoDpFpga[ulFpgaNum] ) ;
//        delete ( m_apoT3Fpga[ulFpgaNum] ) ;
//        delete ( m_apoT4Fpga[ulFpgaNum] ) ;
        delete ( m_apoT7Fpga[ulFpgaNum] ) ;
    }
}
/*-------------------------------------------------------------------------------*/

ESuccess CAdfCardBase::Initialize ( EYesNo              _eSkipInitLookup )
{
    ESuccess        eReturnStatus = Ok ;

    //\\//

    CConsoleAccess::WriteLine ( (CString) " "
                                 + f_osBaseAddress ( )
                                 + " Initializing "
                                 + m_osCardDescription ) ;


    // First "Wake Up" the Card via the Board Control PAL
    if ( m_poBcPal->Initialize ( ) != Ok )
    {
        eReturnStatus = Error ;
//        goto Abort ;
    }


    // Initialize Fpgas

    for ( unsigned long   ulFpgaNum = 0 ; ulFpgaNum < KiTotFpgaPerAdf ; ulFpgaNum ++ )
    {
        //Access to Data Path Fpgas (maybe not)
//        if ( m_apoDpFpga[ulFpgaNum]->Initialize ()  != Ok )
//            eReturnStatus = Error ;

        //T3 Adf Fpgas (maybe not there)
//        if ( m_apoT3Fpga[ulFpgaNum]->Initialize ()  != Ok )
//            eReturnStatus = Error ;

        //T4 Adf Fpgas (maybe not there)
//        if ( m_apoT4Fpga[ulFpgaNum]->Initialize ()  != Ok )
//            eReturnStatus = Error ;

        //T7 Adf Fpgas (maybe not there)
        if ( m_apoT7Fpga[ulFpgaNum]->Initialize ( _eSkipInitLookup )  != Ok )
            eReturnStatus = Error ;
    }

//Abort:
    return eReturnStatus ;
}


/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess CAdfCardBase::f_okStopPRN ( )
{
    return m_poBcPal->f_okStopPRN ( )  ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess CAdfCardBase::f_okStartPRN ( )
{
    return m_poBcPal->f_okStartPRN ( )  ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess CAdfCardBase::f_okLoadPRNSeeds ( )
{
    ESuccess        eReturnStatus = Ok ;

    unsigned int eTtType ;
    unsigned int uiRelEta ;
    unsigned int uiRelPhi ;

    //\\//

    for ( unsigned long   ulFpgaNum = 0 ; ulFpgaNum < KiTotFpgaPerAdf ; ulFpgaNum ++ )
    for ( uiRelEta  = 0 ; uiRelEta  < KiTotEtaPerAdfFpga ; uiRelEta  ++ )
    for ( uiRelPhi  = 0 ; uiRelPhi  < KiTotPhiPerAdfFpga ; uiRelPhi  ++ )
    for ( eTtType   = 0 ; eTtType   < eTot_TT_Chan_Type  ; eTtType ++ )
    if ( m_apoT7Fpga[ulFpgaNum]->f_okLoadPRNSeed ( (ETT_Chan_Type) eTtType,
                                                                   uiRelEta,
                                                                   uiRelPhi,
                                                                   m_gkawPRNSeed[KiTotFpgaPerAdf*ulFpgaNum+uiRelEta]
                                                                                [uiRelPhi]
                                                                                [eTtType] )  != Ok )
        eReturnStatus = Error ;

    return eReturnStatus ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess CAdfCardBase::f_okSendPseudoRandom  ( )
{
    ESuccess        eReturnStatus = Ok ;

    unsigned int eTtType ;
    unsigned int uiRelEta ;
    unsigned int uiRelPhi ;

    //\\//

    for ( unsigned long   ulFpgaNum = 0 ; ulFpgaNum < KiTotFpgaPerAdf ; ulFpgaNum ++ )
    for ( uiRelEta  = 0 ; uiRelEta  < KiTotEtaPerAdfFpga ; uiRelEta  ++ )
    for ( uiRelPhi  = 0 ; uiRelPhi  < KiTotPhiPerAdfFpga ; uiRelPhi  ++ )
    for ( eTtType   = 0 ; eTtType   < eTot_TT_Chan_Type  ; eTtType ++ )
    if ( m_apoT7Fpga[ulFpgaNum]->f_okAdfOut_MuxMode      ( (ETT_Chan_Type) eTtType,
                                                                           uiRelEta,
                                                                           uiRelPhi,
                                                                           eAdfOut_MuxMode_Prn_Always )  != Ok )
        eReturnStatus = Error ;

    return eReturnStatus ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess CAdfCardBase::f_okFreezePseudoRandom  ( )
{
    ESuccess        eReturnStatus = Ok ;

    unsigned int eTtType ;
    unsigned int uiRelEta ;
    unsigned int uiRelPhi ;

    //\\//

    for ( unsigned long   ulFpgaNum = 0 ; ulFpgaNum < KiTotFpgaPerAdf ; ulFpgaNum ++ )
    for ( uiRelEta  = 0 ; uiRelEta  < KiTotEtaPerAdfFpga ; uiRelEta  ++ )
    for ( uiRelPhi  = 0 ; uiRelPhi  < KiTotPhiPerAdfFpga ; uiRelPhi  ++ )
    for ( eTtType   = 0 ; eTtType   < eTot_TT_Chan_Type  ; eTtType ++ )
    if ( m_apoT7Fpga[ulFpgaNum]->f_okLoadPRNSeed ( (ETT_Chan_Type) eTtType,
                                                                   uiRelEta,
                                                                   uiRelPhi,
                                                                   0xffff )  != Ok )
        eReturnStatus = Error ;

    return eReturnStatus ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess CAdfCardBase::f_okSendBlsData  ( )
{
    ESuccess        eReturnStatus = Ok ;

    unsigned int eTtType ;
    unsigned int uiRelEta ;
    unsigned int uiRelPhi ;

    //\\//

    for ( unsigned long   ulFpgaNum = 0 ; ulFpgaNum < KiTotFpgaPerAdf ; ulFpgaNum ++ )
    for ( uiRelEta  = 0 ; uiRelEta  < KiTotEtaPerAdfFpga ; uiRelEta  ++ )
    for ( uiRelPhi  = 0 ; uiRelPhi  < KiTotPhiPerAdfFpga ; uiRelPhi  ++ )
    for ( eTtType   = 0 ; eTtType   < eTot_TT_Chan_Type  ; eTtType ++ )
    if ( m_apoT7Fpga[ulFpgaNum]->f_okAdfOut_MuxMode      ( (ETT_Chan_Type) eTtType,
                                                                           uiRelEta,
                                                                           uiRelPhi,
                                                                           eAdfOut_MuxMode_Filtered_LiveX )  != Ok )
        eReturnStatus = Error ;

    return eReturnStatus ;
}
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess CAdfCardBase::f_okInitiateChLinkDeSkew   ( )
{
    CConsoleAccess::WriteLine ( (CString) " De-Skew outputs from "
                                         + m_osCardDescription
                                         + " "
                                         + f_osBaseAddress () ) ;

    m_lDeSkewed = true ;

    return m_poBcPal->InitiateChLinkDeSkew ( ) ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess CAdfCardBase::f_okRestartRawAdcAddrGen   ( )
{
    ESuccess        eReturnStatus = Ok ;

    //\\//

    for ( unsigned long   ulFpgaNum = 0 ; ulFpgaNum < KiTotFpgaPerAdf ; ulFpgaNum ++ )
        if ( m_apoT7Fpga[ulFpgaNum]->f_okRestartRawAdcAddrGen( )  != Ok )
            eReturnStatus = Error ;

    return eReturnStatus ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess CAdfCardBase::f_okRestartOutputEtAddrGen ( )
{
    ESuccess        eReturnStatus = Ok ;

    //\\//

    for ( unsigned long   ulFpgaNum = 0 ; ulFpgaNum < KiTotFpgaPerAdf ; ulFpgaNum ++ )
        if ( m_apoT7Fpga[ulFpgaNum]->f_okRestartOutputEtAddrGen ( )  != Ok )
            eReturnStatus = Error ;

    return eReturnStatus ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
TCString CAdfCardBase::f_osBaseAddress     ( ) // e.g. return "@Master#0/Slave#1/Slot# 2"
{

    TCString  oMsg ;

    //\\//

    oMsg = "@Master#" + I_to_A( m_ubVertMaster, 1 )
         + "/Slave#"  + I_to_A( m_ubVertSlave, 1 )
         + "/Slot#"   + I_to_A( m_ubCardSlot, 2 ) ;

    return oMsg ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
TCString CAdfCardBase::f_osVmeAddress     ( )  // e.g. return "@0x01020000"
{
    TCString  oMsg ;

    //\\//

    oMsg = (TCString) "@" + I_to_A ( m_ulVmeAddress, 8, 16, '0' ) ;

    return oMsg ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
CAdfCardBase*      CAdfCardBase::f_poFindCard     ( unsigned char   _ubVertMaster,
                                                    unsigned char   _ubVertSlave,
                                                    unsigned char   _ubCardSlot,
                                                    CAdfCardBase*   _poLastCard )
{
    CAdfCardBase*      poCard ;

    unsigned long   ulVmeAddress ;

    //\\//

    poCard = _poLastCard  ;

    ulVmeAddress = CAdfRegBase::ConvertToVmeAddress   ( _ubVertMaster,
                                                        _ubVertSlave,
                                                        _ubCardSlot ) ;
    while ( poCard != NULL )
    {
        // Did we find it?
        if ( poCard->f_ulVmeAddress() == ulVmeAddress ) break ;

        // It wasn't this one, so travel to the next one in the chain until we reach the end
        poCard  = poCard->f_poPrevCard() ;
    }

    // If we didn't find it, poCard is now NULL
    // which is the value we need to return

    return poCard ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
CAdfChipBase*      CAdfCardBase::f_poFindFpga  ( unsigned char   _ubChipAddress )
{
    CAdfChipBase*      poFpga ;
    unsigned long   ulVmeAddress ;
    unsigned short  uwChipCount ;

    //\\//

    ulVmeAddress = CAdfRegBase::ConvertToVmeAddress  ( m_ubVertMaster,
                                                    m_ubVertSlave,
                                                    m_ubCardSlot,
                                                   _ubChipAddress ) ;
    poFpga = m_poFirstFpga ;

    uwChipCount = 0 ;
    while ( poFpga != NULL )
    {
        uwChipCount ++ ;

        if ( uwChipCount > m_uwTotChip )
        {
            CConsoleAccess::InternalErrorLine ( (CString) "CAdfCardBase::f_poFindFpga: "
                                                        + "Too Many Fpgas found while travelling the chain" ) ;
            break ;
        }

        // Did we find it?
        if ( poFpga->f_ulVmeAddress() == ulVmeAddress ) break ;

        // It wasn't this one, so travel to the next one in the chain until we reach the end
        poFpga  = poFpga->f_poNextFpga() ;
    }

    // If we didn't find it, poFpga is now NULL
    // which is the value we need to return

    return poFpga ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
void  CAdfCardBase::Dump  ( bool               _lIncludeReadOnly,
                            CResultFile*       _poResultFile )
{
    CAdfChipBase*      poFpga ;
    unsigned short  uwChipCount ;

    CString     oMsg ;

    //\\//

    //Prepare Message showing which Fpga is being dumped
    oMsg = (CString) "Card: "
          + f_osBaseAddress()
          + "  "
          + f_osVmeAddress() ;

    //send message to Result File or Screen
    if ( _poResultFile != NULL )
    {
        _poResultFile->WriteLine ( "" ) ;
        _poResultFile->WriteLine ( m_osCardDescription ) ;
        _poResultFile->WriteLine ( oMsg ) ;
    }
    else
    {
        CConsoleAccess::WriteLine ( "" ) ;
        CConsoleAccess::WriteLine ( m_osCardDescription ) ;
        CConsoleAccess::WriteLine ( oMsg ) ;
    }

    //Loop over all registers on that Fpga
    poFpga = m_poFirstFpga ;
    uwChipCount = 0 ;
    while ( poFpga != NULL )
    {
        uwChipCount ++ ;

        if ( uwChipCount > m_uwTotChip )
        {
            CConsoleAccess::InternalErrorLine ( (CString) "CAdfCardBase::Dump: "
                                                        + "Too Many Fpgas found while travelling the chain" ) ;
            CConsoleAccess::InternalErrorLine ( (CString) "_for: " + m_osCardDescription ) ;
            break ;
        }

        // Dump this Fpga
        poFpga->Dump  ( _lIncludeReadOnly,
                        _poResultFile ) ;

        // travel to the next one in the chain until we reach the end
        poFpga = poFpga->f_poNextFpga() ;
    }

    if ( uwChipCount != m_uwTotChip )
    {
        CConsoleAccess::InternalErrorLine ( (CString) "CAdfCardBase::Dump: "
                                                    + "Too Few Fpgas found while travelling the chain" ) ;
        CConsoleAccess::InternalErrorLine ( (CString) "_for: " + m_osCardDescription ) ;
    }
    return ;
}


