/*-------------------------------------------------------------------------------*/
  /*
    AdfChipBase.cpp
    16-Nov-2004
  */
/*-------------------------------------------------------------------------------*/

#include "L1Cal_IIb_TCC/StdAfx.h"

#include "L1Cal_IIb_TCC/GlobalConstants.h"
#include "L1Cal_IIb_TCC/CustomDataTypes.h"

#include                            "L1Cal_IIb_TCC/AdfChipBase.h"

#include "L1Cal_IIb_TCC/AdfCardBase.h"

#include "L1Cal_IIb_TCC/HandleConsole.h"
#include "L1Cal_IIb_TCC/UtilsStrings.h"

#include "L1Cal_IIb_TCC/HandleResultFile.h"

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
CAdfChipBase::CAdfChipBase()
{
	// TODO: add one-time construction code here

}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
CAdfChipBase::CAdfChipBase  ( CAdfCardBase*      _poMotherCard,
                        unsigned char   _ubChipAddress )
{
    m_poMotherCard  = _poMotherCard ;
    m_ubChipAddress = _ubChipAddress ;

    m_ulVmeAddress  = CAdfRegBase::ConvertToVmeAddress   ( m_poMotherCard->m_ubVertMaster,
														m_poMotherCard->m_ubVertSlave,
														m_poMotherCard->m_ubCardSlot,
														m_ubChipAddress,
														0 ) ; 
    //start out with no registers
    m_uwTotRegister = 0 ;

    // Update the number of Chips on the mother Card
    if ( m_poMotherCard != NULL )
        m_poMotherCard->m_uwTotChip ++ ;

    // starting point down chain of links through all Registers
    m_poFirstReg    = NULL ; 
    m_poLastReg     = NULL ; 

    // grab the pointer to the previous Fpga in the chain
    if ( m_poMotherCard != NULL )
        m_poPrevFpga    = m_poMotherCard->m_poLastFpga ;
    else
        m_poPrevFpga    = NULL ;

    // Update the Mother Card pointer to this last child Fpga in the Chain
    if ( m_poMotherCard != NULL )
        m_poMotherCard->m_poLastFpga = this ;

    //update the previous Fpga's pointer to next fpga
    if ( m_poPrevFpga != NULL )
        m_poPrevFpga->m_poNextFpga = this ;

    //There is no next Fpga yet, 
    //the next Fpga to be defined will update this pointer, 
    //as done above 
    m_poNextFpga = NULL ;

    // Initialize the Mother Card's pointer to the first child Fpga in the Chain
    if ( m_poMotherCard != NULL )
    if ( m_poMotherCard->m_poFirstFpga == NULL ) 
        m_poMotherCard->m_poFirstFpga = this ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
CAdfChipBase::~CAdfChipBase()
{
}
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
// Fix up linked chain to skip this Fpga (to do before deleting it)
void CAdfChipBase::RemoveFromChain ( ) 
{
    // Fix the Mother Card's pointer to the last child Fpga in the Chain
    if ( m_poMotherCard != NULL ) 
    if ( m_poMotherCard->m_poLastFpga == this ) 
        m_poMotherCard->m_poLastFpga = m_poPrevFpga ;

    // Fix the Mother Card's pointer to the first child Fpga in the Chain
    if ( m_poMotherCard != NULL ) 
    if ( m_poMotherCard->m_poFirstFpga == this ) 
        m_poMotherCard->m_poFirstFpga = m_poNextFpga ;

    // Make the previous Fpga skip to the next
    if ( m_poPrevFpga != NULL )
        m_poPrevFpga->m_poNextFpga = m_poNextFpga ;

    // Make the next Fpga skip to the previous 
    if ( m_poNextFpga != NULL )
        m_poNextFpga->m_poPrevFpga = m_poPrevFpga ;

    // Update the number of Chips on the mother Card
    if ( m_poMotherCard != NULL ) 
        m_poMotherCard->m_uwTotChip -- ;

    return ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
TCString CAdfChipBase::f_osBaseAddress     ( ) // e.g. return "Master#0/Slave#1/Slot# 2/Chip# 0"
{

    TCString  oMsg ;

    //\\//

    oMsg = "@Master#" + I_to_A( m_poMotherCard->m_ubVertMaster, 1 )
         + "/Slave#"  + I_to_A( m_poMotherCard->m_ubVertSlave, 1 )
         + "/Slot#"   + I_to_A( m_poMotherCard->m_ubCardSlot, 2 ) 
         + "/Chip#"   + I_to_A( m_ubChipAddress, 2 ) ;

    return oMsg ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
TCString CAdfChipBase::f_osVmeAddress     ( )  // e.g. return "@0x01020000"
{
    TCString  oMsg ;

    //\\//

    oMsg = (TCString) "@" + I_to_A ( m_ulVmeAddress, 8, 16, '0' ) ; 

    return oMsg ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
CAdfRegBase* CAdfChipBase::f_poFindRegister ( unsigned short     _uwRegAddress ) 
{
    CAdfRegBase*  poRegister ;

    unsigned long   ulVmeAddress ;
    
    //\\//

    ulVmeAddress = CAdfRegBase::ConvertToVmeAddress  ( m_poMotherCard->m_ubVertMaster,
                                                    m_poMotherCard->m_ubVertSlave,
                                                    m_poMotherCard->m_ubCardSlot,
                                                    m_ubChipAddress,
                                                   _uwRegAddress ) ; 
    poRegister = m_poFirstReg ;

    while ( poRegister != NULL ) 
    {
        // Did we find it?
        if ( poRegister->f_ulVmeAddress() == ulVmeAddress ) break ;

        // It wasn't this one, so travel to the next one in the chain until we reach the end
        poRegister  = poRegister->f_poNextReg() ;
    }

    // If we didn't find it, poRegister is now NULL 
    // which is the value we need to return
   
    return poRegister ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
void  CAdfChipBase::Dump  ( bool               _lIncludeReadOnly,
                            CResultFile*       _poResultFile ) 
{
    CAdfRegBase*       poRegister ;
    unsigned short  uwRegisterCount ;

    CString     oMsg ;

    //\\//


    //Prepare Message showing which Fpga is being dumped
    oMsg = (CString) " Fpga#"
          + I_to_A ( m_ubChipAddress, 2 )
          + ": " 
          + f_osBaseAddress() 
          + "  "
          + f_osVmeAddress() ;

    //send message to Result File or Screen
    if ( _poResultFile != NULL ) 
    {
        _poResultFile->WriteLine ( "" ) ;
        _poResultFile->WriteLine ( oMsg ) ;
    }
    else
    {
        CConsoleAccess::WriteLine ( "" ) ;
        CConsoleAccess::WriteLine ( oMsg ) ;
    }

    //Loop over all registers on that Fpga
    poRegister = m_poFirstReg ;
    uwRegisterCount = 0 ;
    while ( poRegister != NULL ) 
    {
        uwRegisterCount ++ ;

        if ( uwRegisterCount > m_uwTotRegister )
        {
            CConsoleAccess::InternalErrorLine ( (CString) "CAdfChipBase::Dump: "
                                                    + "Too Many Registers found while travelling the chain" ) ;
            if ( m_poMotherCard != NULL )
                CConsoleAccess::InternalErrorLine ( (CString) "_for: " + m_poMotherCard->m_osCardDescription ) ;
            break ;
        }

        // Dump this Register
        poRegister->Dump  ( _lIncludeReadOnly,
                            _poResultFile ) ;

        // travel to the next one in the chain until we reach the end
        poRegister = poRegister->f_poNextReg() ;
    }

    if ( uwRegisterCount != m_uwTotRegister )
    {
        CConsoleAccess::InternalErrorLine ( (CString) "CAdfChipBase::Dump: "
                                                    + "Too Few Registers found while travelling the chain" ) ;
        if ( m_poMotherCard != NULL )
            CConsoleAccess::InternalErrorLine ( (CString) "_for: " + m_poMotherCard->m_osCardDescription ) ;
    }

    return ;
}
