/*-------------------------------------------------------------------------------*/
  /*
    AdfChipPal.cpp
    Class CAdfChipPal
    11-Nov-04
  */
/*-------------------------------------------------------------------------------*/

#include "L1Cal_IIb_TCC/StdAfx.h"

#include "L1Cal_IIb_TCC/GlobalConstants.h"
#include "L1Cal_IIb_TCC/CustomDataTypes.h"

#include                            "L1Cal_IIb_TCC/AdfChipPal.h"

#include "L1Cal_IIb_TCC/DataManip.h"
#include "L1Cal_IIb_TCC/UtilsStrings.h"

#include "L1Cal_IIb_TCC/HandleConsole.h"
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
CAdfChipPal::CAdfChipPal()
{
	// TODO: add one-time construction code here

}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
CAdfChipPal::CAdfChipPal  ( CAdfCardBase*      _poMotherCard,
                            unsigned char   _ubChipAddress )

           : CAdfChipBase    (                 _poMotherCard,  // this calls the base class contructor
                                            _ubChipAddress ) 

{
    m_poCtrlEnables     = new CAdfRegBase ( this,                // link back to the Mother Fpga
                                            KiRegAddrCtrlEnables, // Register Address
                                            0x00ff,              // Write Mask 
                                            0x00ff,              // Read Mask
		                                    0,                   // InitValue
	                                        0 ) ;                // Default Progr

    m_poCtrlCrateStat   = new CAdfRegBase ( this,                // link back to the Mother Fpga
                                            KiRegAddrCtrlCrateStat, // Register Address
                                            0x00ff,              // Write Mask 
                                            0x00ff,              // Read Mask
		                                    0,                   // InitValue
	                                        0 ) ;                // Default Progr

    m_poReadFpgaStat    = new CAdfRegBase ( this,                // link back to the Mother Fpga
                                            KiRegAddrReadFpgaStat, // Register Address
                                            0x0000,              // Write Mask 
                                            0x00ff,              // Read Mask
		                                    0,                   // InitValue
	                                        0 ) ;                // Default Progr

    m_poReadCrateStat   = new CAdfRegBase ( this,                // link back to the Mother Fpga
                                            KiRegAddrReadCrateStat, // Register Address
                                            0x0000,              // Write Mask 
                                            0x00ff,              // Read Mask
		                                    0,                   // InitValue
	                                        0 ) ;                // Default Progr

    m_poSerDacData      = new CAdfRegBase ( this,                // link back to the Mother Fpga
                                            KiRegAddrSerDacData, // Register Address
                                            0x0001,              // Write Mask 
                                            0x0000,              // Read Mask
		                                    0,                   // InitValue
	                                        0 ) ;                // Default Progr

    m_poConfigCtrl      = new CAdfRegBase ( this,                // link back to the Mother Fpga
                                            KiRegAddrConfigCtrl, // Register Address
                                            0x00ff,              // Write Mask 
                                            0x00ff,              // Read Mask
		                                    0,                   // InitValue
	                                        0 ) ;                // Default Progr

    m_poConfigData      = new CAdfRegBase ( this,                // link back to the Mother Fpga
                                            KiRegAddrConfigData, // Register Address
                                            0x00ff,              // Write Mask 
                                            0x0000,              // Read Mask
		                                    0,                   // InitValue
	                                        0 ) ;                // Default Progr

    m_poConfigStatus    = new CAdfRegBase ( this,                // link back to the Mother Fpga
                                            KiRegAddrConfigStatus, // Register Address
                                            0x0000,              // Write Mask 
                                            0x00ff,              // Read Mask
		                                    0,                   // InitValue
	                                        0 ) ;                // Default Progr
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
CAdfChipPal::~CAdfChipPal()
{
    delete m_poCtrlEnables ;  
    delete m_poCtrlCrateStat ;  
    delete m_poReadFpgaStat;  
    delete m_poReadCrateStat ;
    delete m_poSerDacData ;  
    delete m_poConfigCtrl ;  
    delete m_poConfigData ;  
    delete m_poConfigStatus ;  
}
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess CAdfChipPal::Initialize     ( ) 

{

    ESuccess        eReturnStatus = Ok ;

    TuwRegWord      uwTemp ;

    //\\//

    // reset the Modified flag of all registers 
    // so that they may be initialized even if their content had changed
    m_poCtrlEnables->f_vClearModified () ;
    m_poCtrlCrateStat->f_vClearModified () ;
    m_poReadFpgaStat->f_vClearModified () ;
    m_poReadCrateStat->f_vClearModified () ;
    m_poSerDacData->f_vClearModified () ;
    m_poConfigCtrl->f_vClearModified () ;
    m_poConfigData->f_vClearModified () ;
    m_poConfigStatus->f_vClearModified () ;  

    ////////////////////////////////////////////////////////////////////

    //    I.1)  Start Up the ADCs
    //
    //       While at the same time
    //           Lock out the possibility of FPGA Configuration
    //           Lock out the possibility of Pedestal DAC download
    //           Disable BC PAL Access signals
    //
    //        At Register Address 0   Write 0x81
    //
    uwTemp = KiCtrlEnables_EnableADCs
           | KiCtrlEnables_ChanLinkDeskewBar ;
    if ( m_poCtrlEnables->f_okWriteReg ( &uwTemp ) != Ok ) 
        eReturnStatus = Error ;


    //    I.2)  Channel Link in DC_Balance Mode  (DeSkew Operation NOT running)
    //
    //        At Register Address 0   Write 0x83
    //
    uwTemp = KiCtrlEnables_EnableADCs
           | KiCtrlEnables_ChanLinkDeskewBar
           | KiCtrlEnables_ChanLinkDCBal ; 
    if ( m_poCtrlEnables->f_okWriteReg ( &uwTemp ) != Ok ) 
        eReturnStatus = Error ;


    //    I.3)  Channel Link in DC_Balance Mode  and  DeSkew Operation Running.
    //
    //        At Register Address 0   Write 0x82
    //
    uwTemp = KiCtrlEnables_EnableADCs
           | KiCtrlEnables_ChanLinkDCBal ; 
    if ( m_poCtrlEnables->f_okWriteReg ( &uwTemp ) != Ok ) 
        eReturnStatus = Error ;


    //    I.4)  Channel Link in DC_Balance Mode  (DeSkew Operation NOT running)
    //
    //        At Register Address 0   Write 0x83
    //
    uwTemp = KiCtrlEnables_EnableADCs
           | KiCtrlEnables_ChanLinkDeskewBar
           | KiCtrlEnables_ChanLinkDCBal ; 
    if ( m_poCtrlEnables->f_okWriteReg ( &uwTemp ) != Ok ) 
        eReturnStatus = Error ;


    //    I.5)  BC PAL LED #1 ON.
    //
    //       This register will be left with
    //
    //           DeSkew Operation NOT running
    //           Channel Link in DC_Balance Mode
    //           BC PAL LED #1 ON
    //           Lock out the possibility of FPGA Configuration
    //           Lock out the possibility of Pedestal DAC download
    //           Disable BC PAL Access signals
    //           ADCs running
    //
    //       At Register Address 0   Write 0x8b
    //
    uwTemp = KiCtrlEnables_EnableADCs
           | KiCtrlEnables_ChanLinkDeskewBar
           | KiCtrlEnables_CtrlPalLed
           | KiCtrlEnables_ChanLinkDCBal ; 
    if ( m_poCtrlEnables->f_okWriteReg ( &uwTemp ) != Ok ) 
        eReturnStatus = Error ;


    //    I.6)  Setup to the Configuration Control Register
    //
    //       Disable the Configuration of both FPGAs
    //       Set CNFG_RDWR_B to a "0" --> Voltage Low.
    //           This is now a User I/O line and setting CNFG_RDWR_B low
    //           now controls the 32 Pseudo Random Number generators
    //           on this ADF-2 card which will now all be stopped from running.
    //
    //       At Register Address 5   Write 0x0f
    //
    uwTemp = KiConfigCtrl_ReqCnfgPrgBar_0
           | KiConfigCtrl_ReqCnfgPrgBar_1
           | KiConfigCtrl_CnfgChipSelBar_0
           | KiConfigCtrl_CnfgChipSelBar_1 ;
    if ( m_poConfigCtrl->f_okWriteReg ( &uwTemp ) != Ok ) 
        eReturnStatus = Error ;
    

    //    I.7)  Write to the Board Level Control Register #2
    //
    //       Set the  Crate Status Lines. 3:0  to  No Contribution.
    //       Set the  Crate to SCLD  signals  1:0  to Zero.
    //       Do not set the Pedestal DAC's Chip Select to the active state.
    //
    //       At Register Address 1   Write 0x40
    //
    uwTemp = KiCtrlCrateStat_ReqDacCSBar ; 
    if ( m_poCtrlCrateStat->f_okWriteReg ( &uwTemp ) != Ok ) 
        eReturnStatus = Error ;
    
    
    return eReturnStatus ;
}
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess CAdfChipPal::InitiateFpgaConfig      ( unsigned short      _uwConfigEnab,
                                                unsigned char       _ubVersionMajor,
                                                unsigned char       _ubVersionMinor ) 
// cf. http://www.pa.msu.edu/hep/d0/ftp/run2b/l1cal/hardware/adf_2/general/fpga_configuration.txt
{
	ESuccess        eReturnStatus = Ok ;
	ESuccess        eSuccess ;

//    TuwRegWord      uwVersion ;

	TuwRegWord      uwTemp ;

    TuwRegWord      uwConfigNotReady ;
    unsigned int    ulWaitCount ;

    //\\//

    //Record the Main Array FPGA Version into the Scratch RAM
/*
    uwVersion = ( _ubVersionMajor << KiShiftOneByte ) + _ubVersionMinor ;
    for ( int iMsaFpgaNum = 0 ; iMsaFpgaNum < KiTheCardTotMaFpga ; iMsaFpgaNum++ )
    {
        if ( ( _ulConfigEnab & ( 0x00000001 << (iMsaFpgaNum+1) ) ) != 0x00000000 )
    	if ( m_poScratchRam[iMsaFpgaNum]->f_okWriteReg ( &uwVersion ) != Ok ) 
            eReturnStatus = Error ;
    }
*/

    // 0. force an appropriate initial programming of the configuration control register

    uwTemp = KiConfigCtrl_ReqCnfgPrgBar_0 
           | KiConfigCtrl_ReqCnfgPrgBar_1
           | KiConfigCtrl_CnfgChipSelBar_0
           | KiConfigCtrl_CnfgChipSelBar_1
           | KiConfigCtrl_CnfgRdWrBar ;
	eSuccess = m_poConfigCtrl->f_okWriteReg ( &uwTemp ) ;
    if ( eSuccess != Ok ) eReturnStatus = Error ;

    // 1. Enable Fpga configuration 

    eSuccess = m_poCtrlEnables->f_okForceRegBits ( KiCtrlEnables_EnableFpgaConfig ) ;
    if ( eSuccess != Ok ) eReturnStatus = Error ;

    // 2. Initiate request to clear the current configuration on the FPGA(s)
    
	eSuccess = m_poConfigCtrl->f_okClearRegBits ( _uwConfigEnab ) ; 
    if ( eSuccess != Ok ) eReturnStatus = Error ;

    // 3.  Set IO direction on FPGAs to write.

	eSuccess = m_poConfigCtrl->f_okClearRegBits ( KiConfigCtrl_CnfgRdWrBar ) ;
    if ( eSuccess != Ok ) eReturnStatus = Error ;

    // 4. Complete request to clear the current configuration on the FPGA(s) 
    
	eSuccess = m_poConfigCtrl->f_okForceRegBits ( _uwConfigEnab ) ; 
    if ( eSuccess != Ok ) eReturnStatus = Error ;

    //5.  Wait until device memory is cleared and is ready for configuration data.

	// finding a ZERO on a CNFG_INIT_B bit for an FPGA that is being configured, 
    // tells us the FPGA is NOT done clearing its configuration and
    // is not ready to receive its first byte of configuration data
    // and we should check again.
    for ( ulWaitCount = 0 ; ulWaitCount < KMaxConfigWaitInit ; ulWaitCount++ )
    {
        // read the status word 
        m_poConfigStatus->f_okReadReg ( &uwConfigNotReady ) ;
        
        // first invert each bit so that chips NOT ready show up as ones 
		uwConfigNotReady ^= 0xffff ;

        // trim the mask to the set of FPGAs currently being configured
        // (the CNFG_INIT_B bits happen to be the LSB of the status word)
        uwConfigNotReady &= _uwConfigEnab ;

        // Now if any bit is left high, 
        // one of the FPGAs we care about wasn't ready 
        // and we should wait some more
        if ( uwConfigNotReady == 0x0000 ) break ;
    }
    if ( uwConfigNotReady != 0x0000 ) 
    {
        eReturnStatus = Error ;
        CConsoleAccess::ErrorLine ( (CString) "_Timed out waiting for CNFG_INIT_B to go high for Chip Mask =" 
                                             + I_to_A ( uwConfigNotReady, 2, 2, '0') ) ; 
    }
    else 
        CConsoleAccess::WriteLine ( (CString) "_CNFG_INIT_B went high after reading it " 
                                             + I_to_A ( ulWaitCount ) 
                                             + " time(s)" ) ;

    // 6.  Select device(s) to be configured.
    // (the CNFG_CS_B bits happen to be two bits above the two LSBs of the status word)

    eSuccess = m_poConfigCtrl->f_okClearRegBits ( _uwConfigEnab << KiConfigCtrl_Shift_ChipSelToLsb ) ;
    if ( eSuccess != Ok ) eReturnStatus = Error ;

    return eReturnStatus ;
}


/*----------------------------------------------------------------------------------*/
/*----------------------------------------------------------------------------------*/
// Purpose: Send the content of an FPGA config file to the target register for FPGA
// programming.  The mask of which FPGAs are being configured, etc, must have been 
// programmed before calling this file.
// Here we just pour the content of an array of bytes onto the proper VME Address
// These bytes were read from an S-record file outside and before calling this function
// The array of byte is sent one byte at a time to the D16 target register, 
// as 16 bit words where only the least significant byte of each longword is non-zero.
/*----------------------------------------------------------------------------------*/
ESuccess CAdfChipPal::WriteFpgaConfig     ( unsigned short      _uwConfigEnab,
                                            unsigned char*      _aubByteStream,
                                            int                 _iNumBytes ) 

// cf. http://www.pa.msu.edu/hep/d0/ftp/run2b/l1cal/hardware/adf_2/general/fpga_configuration.txt

    // This function never returns an error status, always retrun Ok.
    // No error is reported even if the routine gave up waiting 
    // for some fpga(s) to report they are ready for the next byte.  
    // The Byte stream is force fed to the FPGA's, and the Configuration status
    // should be read afterwards from the board's fpga configured register.
{
	ESuccess            eReturnStatus = Ok ;

    MultiAccessShort    xBitStreamWord ;

    //\\//

    xBitStreamWord.uwWord    = 0 ;


    // 7.  Write up to 510,324 bytes (4,082,592 bits) of configuration data 
    //     to targeted Virtex-II XC2V1000 ADF2 Data Path FPGA(s) one byte at a time.

    for ( int iRunningAddress = 0; iRunningAddress < _iNumBytes ; iRunningAddress++ )
	{
        // write next configuration byte as low byte in 16 bit word
		xBitStreamWord.ByByte.ubByte_0 = _aubByteStream[iRunningAddress] ;
		m_poConfigData->f_okWriteOnly ( &xBitStreamWord.uwWord ) ;
    }

    return eReturnStatus ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess CAdfChipPal::TerminateFpgaConfig  ( unsigned short      _uwConfigEnab ) 

// cf. http://www.pa.msu.edu/hep/d0/ftp/run2b/l1cal/hardware/adf_2/general/fpga_configuration.txt

{
	ESuccess        eReturnStatus = Ok ;
	ESuccess        eSuccess ;

    TuwRegWord      uwConfigNotDone ;
    unsigned int    ulWaitCount ;

    //\\//

    //8.  De-select FPGA device(s) 

    eSuccess = m_poConfigCtrl->f_okForceRegBits ( KiConfigCtrl_CnfgChipSelBar_0
                                                | KiConfigCtrl_CnfgChipSelBar_1 ) ;
    if ( eSuccess != Ok ) eReturnStatus = Error ;


    //9.  Deselect Write Configuratiton mode 

    eSuccess = m_poConfigCtrl->f_okForceRegBits ( KiConfigCtrl_CnfgRdWrBar ) ;
    if ( eSuccess != Ok ) eReturnStatus = Error ;


    //10.  Check if CNFG_DONE(X) is asserted.

	// Finding a ZERO on a CNFG_DONE bit for an FPGA that is being configured, 
    // tells us the FPGA was not yet successfully configured 
    // and we should check again.
    for ( ulWaitCount = 0 ; ulWaitCount < KMaxConfigWaitDone ; ulWaitCount++ )
    {
        // read the status word 
        m_poConfigStatus->f_okReadReg ( &uwConfigNotDone ) ;
        
        // first invert each bit so that chips NOT ready show up as ones 
		uwConfigNotDone ^= 0xffff ;
        // shift the relevant pair of bits down to LSBits
		uwConfigNotDone >>= KiConfigCtrl_Shift_CnfgDoneToLsb ;
        // trim the mask to the set of FPGAs currently being Configured
        // (the CNFG_INIT_B bits happen to be the LSB of the status word)
        uwConfigNotDone &= _uwConfigEnab ;

        // Now if any bit is left high, 
        // one of the FPGAs wasn't ready 
        // and we should wait some more
        if ( uwConfigNotDone == 0x0000 ) break ;
    }
    if ( uwConfigNotDone != 0x0000 ) 
    {
        CConsoleAccess::ErrorLine ( (CString) "_Timed out waiting on CNFG_DONE for Chip Mask =" 
                                             + I_to_A ( uwConfigNotDone, 2, 2, '0') ) ; 
        eReturnStatus = Error ;
    }
    else if ( ulWaitCount != 0 ) 
        CConsoleAccess::WriteLine ( (CString) "_Waited reading CNFG_DONE " 
                                             + I_to_A ( ulWaitCount ) 
                                             + " time(s)" ) ;

    //11. Disable FPGA configuration.

    eSuccess = m_poCtrlEnables->f_okClearRegBits ( KiCtrlEnables_EnableFpgaConfig ) ;
    if ( eSuccess != Ok ) eReturnStatus = Error ;

    // done 
    return eReturnStatus ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
    
ESuccess CAdfChipPal::GetFpgaConfiguredMask  ( unsigned short*  _puwFpgaConfiguredMask ) 
{
	ESuccess        eReturnStatus = Ok ;

    //\\//

    // read the status word 
    m_poConfigStatus->f_okReadReg ( _puwFpgaConfiguredMask ) ;
    
    // shift the relevant pair of bits down to LSBits
	*_puwFpgaConfiguredMask >>= KiConfigCtrl_Shift_CnfgDoneToLsb ;

    return eReturnStatus ;
}


/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess CAdfChipPal::SetStatusToScld         ( unsigned long ulStatusNum, 
                                                unsigned long ulStatusState ) 
{
	ESuccess        eReturnStatus = Ok ;

    TuwRegWord      uwTemp ;
    TuwRegWord      uwBitControlMask ;

    //\\//

    // determine the bitmask that will control this status line
    if      ( ulStatusNum == 0 ) // control bit for status #0
        uwBitControlMask = KiCtrlCrateStat_CrateToScld_0 ; 
    else if ( ulStatusNum == 1 ) // control bit for status #1
        uwBitControlMask = KiCtrlCrateStat_CrateToScld_1 ; 
    else
    {
        CConsoleAccess::InternalErrorLine ( (CString) "CAdfChipPal::SetStatusToScld: Illegal Status Number " ) ; 
        eReturnStatus = Error ;
        goto Abort ;
    }
    

    // read the Crate Status word 
    if ( m_poCtrlCrateStat->f_okReadReg ( &uwTemp ) != Ok )
        eReturnStatus = Error ;

    // Modify the value 
    if ( ulStatusState == 0 ) // clear the control bit for the desired status bit
        uwTemp &= 0xffff ^ uwBitControlMask ; 
    else                      // set the control bit for the desired status bit
        uwTemp |= uwBitControlMask ; 


    // Write it back 
    if ( m_poCtrlCrateStat->f_okWriteReg ( &uwTemp ) != Ok )
        eReturnStatus = Error ;

Abort :

    return eReturnStatus ;
}


/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess CAdfChipPal::InitiateChLinkDeSkew       ( ) 
{
    // Pulse DeSkew Control Bit
    //  i.e. Write 0x82, then 0x83 (unless register was no longer as initialized)
    return m_poCtrlEnables->f_okPulseBitsLow  ( KiCtrlEnables_ChanLinkDeskewBar ) ;
}


/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess CAdfChipPal::f_okStartPRN( ) 
{
    //       Set CNFG_RDWR_B to a "1 
    //           This is now a User I/O line and setting CNFG_RDWR_B low
    //           now controls the 32 Pseudo Random Number generators
    //           on this ADF-2 card which will now all be stopped from running.

    return m_poConfigCtrl->f_okForceRegBits ( KiConfigCtrl_PrnEnabled ) ;
}


/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess CAdfChipPal::f_okStopPRN( ) 
{
    //       Set CNFG_RDWR_B to a "1 
    //           This is now a User I/O line and setting CNFG_RDWR_B low
    //           now controls the 32 Pseudo Random Number generators
    //           on this ADF-2 card which will now all be stopped from running.

    return m_poConfigCtrl->f_okClearRegBits ( KiConfigCtrl_PrnEnabled ) ;
}





