/*-------------------------------------------------------------------------------*/
  /*
    AdfDacCtrl.cpp
    Class CAdfDacCtrl
    7-Jan-2005
  */
/*-------------------------------------------------------------------------------*/
#include "L1Cal_IIb_TCC/StdAfx.h"

#include "L1Cal_IIb_TCC/GlobalConstants.h"
#include "L1Cal_IIb_TCC/CustomDataTypes.h"

#include                            "L1Cal_IIb_TCC/AdfDacCtrl.h"

#include "L1Cal_IIb_TCC/AdfCardBase.h"

#include "L1Cal_IIb_TCC/UtilsStrings.h"
#include "L1Cal_IIb_TCC/UtilsBits.h"

#include "L1Cal_IIb_TCC/HandleConsole.h"

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
CAdfDacCtrl::CAdfDacCtrl      ( unsigned char   _ubVertMaster,
                                unsigned char   _ubVertSlave,
                                unsigned char   _ubCardSlot )
{

    CAdfCardBase* poDummyAdfCard = new CAdfCardBase ( _ubVertMaster,
                                                      _ubVertSlave,
                                                      _ubCardSlot ) ;
    m_poBrdCtrlPal = poDummyAdfCard->m_poBcPal ;

    m_eynDeleteDummyAdfInDestructor = Yes ;

    SET_CMD_NO_OP( ) ;

}


/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
CAdfDacCtrl::CAdfDacCtrl      ( CAdfChipPal*    _poBrdCtrlPal )
{

    m_poBrdCtrlPal = _poBrdCtrlPal ;

    m_eynDeleteDummyAdfInDestructor = No ;

    SET_CMD_NO_OP( ) ;

}


/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
CAdfDacCtrl::~CAdfDacCtrl ( )
{
    if ( m_eynDeleteDummyAdfInDestructor == Yes )
        delete m_poBrdCtrlPal->m_poMotherCard ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
TCString CAdfDacCtrl::f_osHostCardAddress   ( ) // e.g. return "Master#0/Slave#1/Slot# 2"
{
    return m_poBrdCtrlPal->m_poMotherCard->f_osBaseAddress () ;
}
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess    CAdfDacCtrl::f_okWriteOneDac  ( ETT_Chan_Type        _eTTChanType,
                                            unsigned long const  _kulRelEta,
                                            unsigned long const  _kulRelPhi,
                                            unsigned long const  _kulValue )
{
    ESuccess        eReturnStatus = Ok ;

    //\\//

    //////////////////////////////////
    // first build the desired command 
    //////////////////////////////////

    // Clear out previous command
    SET_CMD_NO_OP   ( ) ;


    //Select the proper Dac in the chain
    if ( _kulRelEta >= KiTotEtaPerAdf )
    {
        eReturnStatus = Error ;
        CConsoleAccess::InternalErrorLine ( "CAdfDacCtrl::f_okWriteZer: Illegal RelEta Argument" ) ;
    }

    SET_CMD_UPDATE_ETA ( _kulRelEta ) ;

    // select the proper channel pair (8 channels = 4 pairs of EM/HD)
    if ( _kulRelPhi >= KiTotPhiPerAdf )
    {
        eReturnStatus = Error ;
        CConsoleAccess::InternalErrorLine ( "CAdfDacCtrl::f_okWriteZer: Illegal RelPhi Argument" ) ;
    }

    SET_CMD_CHANNEL_PHI( _kulRelEta, _kulRelPhi ) ;

    // select the proper channel in the pair (1 channel in the pair)
    switch ( _eTTChanType )
    {
        case eTT_EM : SET_CMD_CHANNEL_EM ( _kulRelEta ) ; break ;

        case eTT_HD : SET_CMD_CHANNEL_HD ( _kulRelEta ) ; break ;

        default :
            CConsoleAccess::InternalErrorLine ( "CAdfDacCtrl::f_okWriteZer: Illegal ChanType Argument" ) ;
            eReturnStatus = Error ;
    }

    // Specify the new Dac COntrol Value
    if ( _kulValue > KiDacCmdMaxDacValue )
    {
        eReturnStatus = Error ;
        CConsoleAccess::InternalErrorLine ( "CAdfDacCtrl::f_okWriteZer: Illegal DacValuwe Argument" ) ;
    }

    SET_CMD_DAC_VALUE ( _kulRelEta, _kulValue ) ;



    //////////////////////////////////
    // now we can send the serial command string
    //////////////////////////////////

    // no point in writing command if we had argument problems
    if ( eReturnStatus != Error ) 

        eReturnStatus = f_okSendSerialCommand ( ) ;

    return eReturnStatus ;
}
/*-------------------------------------------------------------------------------*/


/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess    CAdfDacCtrl::f_okSendSerialCommand      ( )

{
    ESuccess        eReturnStatus = Ok ;

    //\\//

    //Start the transaction by asserting the Chip Select Bit
    f_okCommandSetup ( ) ;


    //Send the 4 Dac command words, 
    //The last one in the chain has to be sent first
    for ( int iRelEta = KiTotEtaPerAdf-1 ; iRelEta >= 0 ; iRelEta-- )         
        f_okSendOneDacCtrlWord ( m_xSerialDacCmd[iRelEta].ulLong ) ;                              
    
    //Finish the transaction and execute the command by de-asserting the Chip Select Bit
    f_okCommandComplete ( ) ;

    return eReturnStatus ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess    CAdfDacCtrl::f_okCommandSetup ( )

{
    ESuccess        eSuccess ;
    ESuccess        eReturnStatus = Ok ;

    //\\//

    //   1. Enable Ped DAC Programming

    eSuccess = m_poBrdCtrlPal->m_poCtrlEnables->f_okForceRegBits ( KiCtrlEnables_EnableLoadDACs ) ;
    if ( eSuccess != Ok ) eReturnStatus = Error ;


    //   2. Set  DAC_CHIP_SELECT_B  active  (Voltage Low).

    eSuccess = m_poBrdCtrlPal->m_poCtrlCrateStat->f_okClearRegBits ( KiCtrlCrateStat_ReqDacCSBar ) ;
    if ( eSuccess != Ok ) eReturnStatus = Error ;

    return eReturnStatus ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess    CAdfDacCtrl::f_okSendOneDacCtrlWord ( unsigned long const _kulCmdData ) 

{
    ESuccess        eSuccess ;
    ESuccess        eReturnStatus = Ok ;
	TuwRegWord      uwTemp ;

    //\\//

    //   3. Shift out 32 bits.
    //         The serial DAC data is
    //         placed in the LSBit of the 16 bit VME Word.

    for ( int iBitNum = 31 ; iBitNum >= 0 ; iBitNum -- ) 
    {
        if ( ( _kulCmdData & ( 0x00000001 << iBitNum ) ) != 0 ) 
            uwTemp = 0x0001 ;
        else
            uwTemp = 0x0000 ;

        eSuccess = m_poBrdCtrlPal->m_poSerDacData->f_okWriteOnly ( &uwTemp ) ;
        if ( eSuccess != Ok ) eReturnStatus = Error ;
    }

    return eReturnStatus ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess    CAdfDacCtrl::f_okCommandComplete   ( )

{
    ESuccess        eSuccess ;
    ESuccess        eReturnStatus = Ok ;

    //\\//

    //   4. Set  DAC_CHIP_SELECT_B  inactive  (Voltage High).

    eSuccess = m_poBrdCtrlPal->m_poCtrlCrateStat->f_okForceRegBits ( KiCtrlCrateStat_ReqDacCSBar ) ;
    if ( eSuccess != Ok ) eReturnStatus = Error ;


    //   5. Repeat steps 2:4 as necessary

    // that would be for programming more than one Dac.  Nothing here


    //   6. Disable Ped DAC Programming.

    eSuccess = m_poBrdCtrlPal->m_poCtrlEnables->f_okClearRegBits ( KiCtrlEnables_EnableLoadDACs ) ;
    if ( eSuccess != Ok ) eReturnStatus = Error ;


    return eReturnStatus ;
}
/*-------------------------------------------------------------------------------*/

