/*-------------------------------------------------------------------------------*/
  /*
    AdfRegBase.cpp
    16-Nov-2004
  */
/*-------------------------------------------------------------------------------*/

#include "L1Cal_IIb_TCC/StdAfx.h"

#include "L1Cal_IIb_TCC/GlobalConstants.h"
#include "L1Cal_IIb_TCC/CustomDataTypes.h"

#include                            "L1Cal_IIb_TCC/AdfRegBase.h"

#include "L1Cal_IIb_TCC/AdfChipBase.h"
#include "L1Cal_IIb_TCC/AdfCardBase.h"

#include "L1Cal_IIb_TCC/HandleHardwareAccess.h"
#include "L1Cal_IIb_TCC/AdfVmeAddress.h"

#include "L1Cal_IIb_TCC/HandleResultFile.h"

#include "L1Cal_IIb_TCC/DataManip.h"
#include "L1Cal_IIb_TCC/UtilsStrings.h"

#include <math.h> // for f_okHisto10bits 

// access to the console
// static means created only once throughout life of program (to avoid wasting time)
// *AND* internal linkage (to avoid clash between modules)
#include "L1Cal_IIb_TCC/HandleConsole.h"
static CConsoleAccess oConsole ;

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
CAdfRegBase::CAdfRegBase()
{

}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
CAdfRegBase::CAdfRegBase         ( CAdfChipBase*    _poMotherFpga,
                                   unsigned short   _uwRegAddress,
                                   TuwRegWord       _uwWriteMask,
                                   TuwRegWord       _uwReadMask,
                                   TuwRegWord       _uwInitValue,
                                   TuwRegWord       _uwDefaultProgr,
                                   unsigned char    _ubPipelineDepth )
    {

    CAdfRegBase*  poLastReg ;

    //\\//

    m_ulVmeAddress      = ConvertToVmeAddress ( _poMotherFpga->m_poMotherCard->m_ubVertMaster,
                                                _poMotherFpga->m_poMotherCard->m_ubVertSlave,
                                                _poMotherFpga->m_poMotherCard->m_ubCardSlot,
                                                _poMotherFpga->m_ubChipAddress,
                                                _uwRegAddress ) ; 

    if ( _poMotherFpga != NULL ) 
        poLastReg = _poMotherFpga ->m_poLastReg ;
    else 
        poLastReg = NULL ;

    BasicConstruct ( m_ulVmeAddress,
                    &poLastReg,
                     _uwWriteMask,
                     _uwReadMask ) ;

    m_poMotherFpga      = _poMotherFpga ; // to do after BasicConstruct which zeroes it



    m_uwInitValue       = _uwInitValue ;
    m_uwDefaultProgr    = _uwDefaultProgr ;
    
//    m_ubPipelineDepth   = _ubPipelineDepth ;
    
    // Update the Mother Fpga's pointer to the last child Register in the Chain
    if ( m_poMotherFpga != NULL ) 
        m_poMotherFpga->m_poLastReg = this ;

    // Update the number of registers on the mother Fpga
    if ( m_poMotherFpga != NULL ) 
        m_poMotherFpga->m_uwTotRegister ++ ;

    // Initialize the Mother Fpga's pointer to the first child Register in the Chain
    if ( m_poMotherFpga != NULL ) 
    if ( m_poMotherFpga->m_poFirstReg == NULL ) 
        m_poMotherFpga->m_poFirstReg = this ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/

CAdfRegBase::CAdfRegBase         ( unsigned char    _ubVertMaster,
                                   unsigned char    _ubVertSlave,
                                   unsigned char    _ubCardSlot,
                                   unsigned char    _ubChipAddress,
                                   unsigned short   _uwRegAddress,
                                   CAdfRegBase**    _ppoPrevReg, /*** a better name would be poLastReg, since it is updated here ***/
                                   TuwRegWord       _uwWriteMask,
                                   TuwRegWord       _uwReadMask ) 
{

    //\\//

    m_ulVmeAddress      = ConvertToVmeAddress ( _ubVertMaster,
                                                _ubVertSlave,
                                                _ubCardSlot,
                                                _ubChipAddress,
                                                _uwRegAddress ) ;

    BasicConstruct ( m_ulVmeAddress,
                     _ppoPrevReg,
                     _uwWriteMask,
                     _uwReadMask ) ;

}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
CAdfRegBase::CAdfRegBase             ( TulVmeAddress    _ulVmeAddress,
                                       CAdfRegBase**       _ppoPrevReg,
                                       TuwRegWord       _uwWriteMask,
                                       TuwRegWord       _uwReadMask ) 
{

    BasicConstruct ( _ulVmeAddress,
                     _ppoPrevReg,
                     _uwWriteMask,
                     _uwReadMask ) ;

}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
void    CAdfRegBase::BasicConstruct   ( TulVmeAddress   _ulVmeAddress,
                                        CAdfRegBase**      _ppoPrevReg,
                                        TuwRegWord      _uwWriteMask,
                                        TuwRegWord      _uwReadMask ) 
{
    unsigned char    ubVertMaster ;
    unsigned char    ubVertSlave ;
    unsigned char    ubCardSlot ;
    unsigned char    ubChipAddress ;
    unsigned short   uwRegAddress ;
                              
    TuwRegWord*     puwAdfCardBaseVirtAddress ;

    //\\//

    // get the register coordinates from the VME address
    m_ulVmeAddress      = _ulVmeAddress ;

    UnpackVmeAddress ( &ubVertMaster, 
                       &ubVertSlave, 
                       &ubCardSlot, 
                       &ubChipAddress, 
                       &uwRegAddress ) ;

    puwAdfCardBaseVirtAddress = CHardwareAccess::MemoryMapThisCard( ubVertMaster,
                                                                    ubVertSlave,
                                                                    ubCardSlot ) ;

#if ( defined ( VME_IO_BIT3_MMAP ) || defined ( VME_IO_FAKE ) ) 

    if ( puwAdfCardBaseVirtAddress == NULL ) 
    oConsole.ErrorLine ( (CString) "_Error Mapping" + f_osRegAddress() ) ;

    // when computing the offset, remember that puwAdfCardBaseVirtAddress 
    // is a pointer to 2-byte words.
    // In the C language adding a number N to a pointer means incrementing 
    // the pointer by N times the declared data size
    m_puwVirtAddress    = puwAdfCardBaseVirtAddress
                        + ( ConvertToVmeOffset ( ubChipAddress, uwRegAddress ) / 2 ) ; 

#endif // VME_IO_BIT3_MMAP || VME_IO_FAKE

    m_poMotherFpga      = NULL ;

    m_uwRegAddress      = uwRegAddress ;

    m_uwInitValue       = 0 ;
    m_uwDefaultProgr    = 0 ;
    
//    m_ubPipelineDepth   = 0 ;

    m_uwWriteMask       = _uwWriteMask ;
    m_uwReadMask        = _uwReadMask ;

//    m_ynInitialized     = No ;
    m_ynModified        = No ;
    m_ynVolatile        = No ;

//    m_ynSkipIO          = No ;
//    m_ynTestScan        = No ;

    m_uwLastWrite       = 0 ;
    m_uwLastRead        = 0 ;
    m_uwSavedContent    = 0 ;

    // grab the pointer to the previous Register in the chain,
    // and return a pointer to *this* Register to be used as the next "previous Reg"
    // or as the last Register recorded For this Fpga.
    if ( _ppoPrevReg != NULL ) // but only if this optional parameter was provided
    {
        m_poPrevReg = *_ppoPrevReg ;
        *_ppoPrevReg = this ;
        if ( m_poPrevReg != NULL )
            m_poPrevReg->m_poNextReg = this ; // update the forward chain
    }
    else
    {
        m_poPrevReg = NULL ;
    }

    //There is no next register yet, 
    //the next register to be defined will update this pointer, 
    //as done just above
    m_poNextReg = NULL ;

}


/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
CAdfRegBase::~CAdfRegBase()
{
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
// Fix up linked chain to skip this register (to do before deleting it)
void CAdfRegBase::RemoveFromChain ( ) 
{
    // Fix the Mother Fpga's pointer to the last child Register in the Chain
    if ( m_poMotherFpga != NULL ) 
    if ( m_poMotherFpga->m_poLastReg == this ) 
        m_poMotherFpga->m_poLastReg = m_poPrevReg ;

    // Fix the Mother Fpga's pointer to the first child Register in the Chain
    if ( m_poMotherFpga != NULL ) 
    if ( m_poMotherFpga->m_poFirstReg == this ) 
        m_poMotherFpga->m_poFirstReg = m_poNextReg ;

    // Make the previous register skip to the next
    if ( m_poPrevReg != NULL )
        m_poPrevReg->m_poNextReg = m_poNextReg ;

    // Make the next register skip to the previous 
    if ( m_poNextReg != NULL )
        m_poNextReg->m_poPrevReg = m_poPrevReg ;

    // Update the number of registers on the mother Fpga
    if ( m_poMotherFpga != NULL ) 
        m_poMotherFpga->m_uwTotRegister -- ;

    return ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess CAdfRegBase::f_okReadReg        ( TuwRegWord* const     _kpuwReadData ) 
{
    ESuccess        eReturnStatus = Ok ;

    TuwRegWord      uwRegData ;

    //\\//

#if defined ( VME_IO_BIT3_MMAP ) // Memory Mapped Data Access

    // We have the virtual address of the register mapped throug the Bit3 Adaptor
    // just access it as you would a regular variable
    uwRegData = *m_puwVirtAddress ;

    // see if the VME access operation caused a bit3 device error interrupt
    if ( CHardwareAccess::mg_poBit3BusAdaptor->m_xIcbrParameters.eVmeStatus != Ok ) 
    {
        if ( CHardwareAccess::mg_poBit3BusAdaptor->m_xIcbrParameters.eVmeStatus != Error ) 
            oConsole.ErrorLine ( (CString) "Illegal Value found in Bit3 Adaptor Interrupt Parameters" ) ;
        CHardwareAccess::mg_poBit3BusAdaptor->m_xIcbrParameters.eVmeStatus = Ok ;
        eReturnStatus = Error ;
    }

#elif defined ( VME_IO_BIT3_API ) // Explicit call to the Bit3 API routines

    ESuccess        eStatus ;
    //\\//

    eStatus = CHardwareAccess::ReadVmeReg ( m_ulVmeAddress, &uwRegData ) ;

    if ( eStatus != Ok ) 
    {
        oConsole.ErrorLine ( (CString) "_Reading" + I_to_A ( uwRegData, 4, 16, '0' )
                                      + f_osRegAddress() ) ;
        eReturnStatus = Error ;
    }

#elif defined ( VME_IO_FAKE )  // no attached framework: e.g. for Alpha

    // We have a virtual address mapped to momory somewhere
    uwRegData = *m_puwVirtAddress ;

    #if defined ( VME_IO_FAKE_LATENCY )  // simulate the access time

        for ( unsigned long x = 0 ; x < 1000 ; x ++ )  // waste time ~5us
            uwRegData = *m_puwVirtAddress ;

    #endif
        
#elif defined ( VME_IO_NONE )  // no IO operation: e.g. for Alpha

    uwRegData = m_uwLastWrite ;

#endif
    
    // mask off all bits not in the ReadMask
    uwRegData &= m_uwReadMask ;
        
    // remember the last value read 
    m_uwLastRead = uwRegData ;

    //write value to return in argument, if present
    if ( _kpuwReadData != NULL ) *_kpuwReadData = uwRegData ;

    //\\//
    return eReturnStatus ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess CAdfRegBase::f_okWriteReg       ( const TuwRegWord* const     _kpuwWriteData ) 
{
    ESuccess        eReturnStatus = Ok ;

    //\\//

    // we want to check that the previously written value is still there
	// but we skip checking on the first write ever to this register
    // and some registers may not be checkable this way
	//***  this may not be the final implementation of this feature
    if ( 
         ( ( m_ynModified == Yes ) /*|| ( m_ynInitialized == Yes )*/ ) // i.e. we already wrote once to this register
      && ( m_ynVolatile == No ) // i.e. its content could not be modified by accessing *other* registers
      ) 
	{
		// compare the fraction of what can be read back to what was last written
        if ( f_okReadReg  ( ) != Ok ) eReturnStatus = Error ;       
	
        if ( ( m_uwLastRead & m_uwWriteMask) !=  (m_uwLastWrite & m_uwReadMask) ) 
		{
			oConsole.ErrorLine ( (CString) "Pre-Write Check Mismatch: Read = " 
										  + I_to_A ( m_uwLastRead, 4, 16, '0' ) 
										  + " / Last Written = "
										  + I_to_A ( m_uwLastWrite, 4, 16, '0' ) ) ;
			oConsole.ErrorLine ( (CString) "_Read Mask = " 
										  + I_to_A ( m_uwReadMask, 4, 16, '0' ) 
										  + " / Write Mask = "
										  + I_to_A ( m_uwWriteMask, 4, 16, '0' ) ) ;
			oConsole.ErrorLine ( (CString) "_" + f_osRegAddress() ) ;
//	        eReturnStatus = Error ; /*** or warning?***/
		}
	}


    // Now update the Register Content (i.e Write with no Verify)
    if ( f_okWriteOnly ( _kpuwWriteData ) != Ok ) eReturnStatus = Error ;       
        
    // compare the fraction of what can be read back to what was ju written
    if ( f_okReadReg  ( ) != Ok ) eReturnStatus = Error ;       

    if ( ( m_uwLastRead & m_uwWriteMask) !=  (m_uwLastWrite & m_uwReadMask) ) 
    {
        oConsole.ErrorLine ( (CString) "Post-Write Check Mismatch: Read = " 
                                      + I_to_A ( m_uwLastRead, 4, 16, '0' ) 
                                      + " / Last Written = "
                                      + I_to_A ( m_uwLastWrite, 4, 16, '0' ) ) ;
        oConsole.ErrorLine ( (CString) "_Read Mask = " 
                                      + I_to_A ( m_uwReadMask, 4, 16, '0' ) 
                                      + " / Write Mask = "
                                      + I_to_A ( m_uwWriteMask, 4, 16, '0' ) ) ;
        oConsole.ErrorLine ( (CString) "_" + f_osRegAddress() ) ;

//        eReturnStatus = Error ;  Don't call this an error quite yet, not until the second try fails too

		// Try Writing one more time
		oConsole.WriteLine ( (CString) "Recover Attempt: Try Writing Register Again" ) ;
		f_okWriteOnly ( _kpuwWriteData ) ;  //(i.e Write with no Verify)
		f_okReadReg  ( ) ; 
		if ( ( m_uwLastRead & m_uwWriteMask) !=  (m_uwLastWrite & m_uwReadMask) ) 
		{
			oConsole.ErrorLine ( (CString) "2nd Post-Write Check Mismatch: Read = " 
										  + I_to_A ( m_uwLastRead, 4, 16, '0' ) 
										  + " / Last Written = "
										  + I_to_A ( m_uwLastWrite, 4, 16, '0' ) ) ;
			oConsole.ErrorLine ( (CString) "_Read Mask = " 
										  + I_to_A ( m_uwReadMask, 4, 16, '0' ) 
										  + " / Write Mask = "
										  + I_to_A ( m_uwWriteMask, 4, 16, '0' ) ) ;
			oConsole.ErrorLine ( (CString) "_" + f_osRegAddress() ) ;
            eReturnStatus = Error ;  // Now we can call this a non-recovered IO error
		}
		else
			oConsole.WriteLine ( (CString) "Second Register Write Successful" ) ;
    }

    return eReturnStatus ;
}
    
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess CAdfRegBase::f_okVerifyReg      (  ) 
{
    oConsole.InternalErrorLine ( (CString) "Not Implemented: CAdfRegBase::VerifyReg" ) ;
    return Ok ;
}
    
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess CAdfRegBase::f_okSaveContent    ( TuwRegWord* const     _kpuwSavedData ) 
{
    ESuccess        eReturnStatus = Ok ;

    //\\//

    if ( _kpuwSavedData == NULL )
	{
		eReturnStatus = f_okReadReg  ( ) ; 
		if ( eReturnStatus == Ok ) m_uwSavedContent = m_uwLastRead ;
	}
    else
        m_uwSavedContent = *_kpuwSavedData ;

    return eReturnStatus ;
}
    
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess CAdfRegBase::f_okWriteOnly      ( const TuwRegWord* const     _kpuwWriteData)  //(no Verify)
{ 
    ESuccess        eReturnStatus = Ok ;
    TuwRegWord      uwWriteData ;

    //\\//

    // make a copy of the "const" data before modifying it
    uwWriteData = *_kpuwWriteData ;

    // mask off all bits not in the WriteMask
    uwWriteData &= m_uwWriteMask ;

#if defined ( VME_IO_BIT3_MMAP ) 

    // We have the virtual address of the register mapped throug the Bit3 Adaptor
    // just access it as you would a regular variable
    *m_puwVirtAddress = uwWriteData ;

    // see if the VME access operation caused a bit3 device error interrupt
    if ( CHardwareAccess::mg_poBit3BusAdaptor->m_xIcbrParameters.eVmeStatus != Ok ) 
    {
        if ( CHardwareAccess::mg_poBit3BusAdaptor->m_xIcbrParameters.eVmeStatus != Error ) 
            oConsole.ErrorLine ( (CString) "Illegal Value found in Bit3 Adaptor Interrupt Parameters" ) ;
        CHardwareAccess::mg_poBit3BusAdaptor->m_xIcbrParameters.eVmeStatus = Ok ;
        eReturnStatus = Error ;
    }
        
#elif defined ( VME_IO_BIT3_API )

    ESuccess        eStatus ;
    //\\//

    eStatus = CHardwareAccess::WriteVmeReg ( m_ulVmeAddress, &uwWriteData ) ;

    if ( eStatus != Ok ) 
    {
        oConsole.ErrorLine ( (CString) "_Writing" + I_to_A ( uwWriteData, 4, 16, '0' )
                                      + f_osRegAddress() ) ;
        eReturnStatus = Error ;
    }

#elif defined ( VME_IO_FAKE )  // no attached framework: e.g. for Alpha

    // We have a virtual address mapped to memory somewhere
    *m_puwVirtAddress = uwWriteData ;

    #if defined ( VME_IO_FAKE_LATENCY )  // simulate the access time

        for ( unsigned long x = 0 ; x < 1000 ; x ++ )  // waste time ~5us
            *m_puwVirtAddress = uwWriteData ;

    #endif

#elif defined ( VME_IO_NONE )  // no IO operation at all: e.g. for Alpha
                                 // m_uwLastWrite will remember what was last written

#endif
    
	// ok now we have modified this register
	m_ynModified = Yes ;
    
    // remember the last value written, masking off all bits not in the WriteMask
    m_uwLastWrite = uwWriteData  ;

    return eReturnStatus ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess CAdfRegBase::f_okMultiRead        ( TuwRegWord* const     _kpuwReadData,
                                             const unsigned long   _kulRequestCount,
                                             const TuwRegWord      _kuiCondMask,
                                             const TuwRegWord      _kuiCondValue,
                                             const unsigned int    _kuiMaxSecTry, 
                                             unsigned int*         _kpuiTotMatch, 
                                             unsigned int*         _kpuiTotReadTry, 
                                             CResultFile*          _poAdcDumpFile ) 
{
    ESuccess        eReturnStatus = Ok ;

    unsigned long   ulSampleMatch = 0 ;
    unsigned long   ulTryNum, ulTryMax ;
    TuwRegWord      uwReadData ;

    //\\//

    // translate the time constraint into a number of IOs
    ulTryMax = 1000000 * _kuiMaxSecTry ; // start with total number of microseconds
#if defined VME_IO_NO_VERTICAL  
    ulTryMax = ulTryMax / 2 ; // 2us per read 
#else
    ulTryMax = ulTryMax / 5 ; // 5us per read 
#endif

    // start hunting
    for ( ulTryNum = 1 ;
          ulTryNum < ulTryMax ;
          ulTryNum ++ )
    {
        // try reading the register (again)
        f_okReadReg ( &uwReadData ) ;

        if ( ( uwReadData & _kuiCondMask ) == _kuiCondValue ) 
        {
            // that's a new successful sample
            ulSampleMatch  ++ ;

            // the first one is sent back via call argument
            if ( ulSampleMatch == 1 ) *_kpuwReadData = uwReadData ;

            //dump sample to file if requested
            if ( _poAdcDumpFile != NULL )
                _poAdcDumpFile->WriteLine ( I_to_A ( uwReadData ) ) ;

            // See if we collected as many as requested
            if ( ulSampleMatch == _kulRequestCount ) break ;
        }
    }
    
    // See if we found the requested number of samples
    if ( ulSampleMatch != _kulRequestCount )  eReturnStatus = Error ;

    // return number of 
    if ( _kpuiTotMatch   != NULL )  *_kpuiTotMatch   = ulSampleMatch ;
    if ( _kpuiTotReadTry != NULL )  *_kpuiTotReadTry = ulTryNum ;


    return eReturnStatus ;
}
    
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
/*
ESuccess CAdfRegBase::f_okMultiWrite     ( TuwRegWord* const     _kpuwWriteData,
                                             const unsigned long   _kulRepeatCount ) 
{
    return Ok ;
}
 
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess CAdfRegBase::f_okForceRegBits   ( TuwRegWord const      _kuwBitMask ) 
{
    ESuccess        eReturnStatus = Ok ;
    TuwRegWord      uwTemp ;

    //\\//

    //read current content
	if ( f_okReadReg ( &uwTemp ) != Ok ) eReturnStatus = Error ;

    //force requested bits
    uwTemp |= _kuwBitMask ;

    //write new content
	if ( f_okWriteReg ( &uwTemp ) != Ok ) eReturnStatus = Error ;

    return eReturnStatus ;
}
    
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess CAdfRegBase::f_okClearRegBits   ( TuwRegWord const      _kuwBitMask ) 
{
    ESuccess        eReturnStatus = Ok ;
    TuwRegWord      uwTemp ;

    //\\//

    //read current content
	if ( f_okReadReg ( &uwTemp ) != Ok ) eReturnStatus = Error ;

    //clear requested bits
    uwTemp &= ( _kuwBitMask ^ 0xffff ) ;

    //write new content
	if ( f_okWriteReg ( &uwTemp ) != Ok ) eReturnStatus = Error ;

    return eReturnStatus ;
}
    
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess CAdfRegBase::f_okPulseBitsHigh  ( TuwRegWord const      _kuwBitMask ) 
{
    ESuccess        eReturnStatus = Ok ;
    TuwRegWord      uwTemp ;

    //\\//

    //read current content
	if ( f_okReadReg ( &uwTemp ) != Ok ) eReturnStatus = Error ;

    //force requested bit(s)
    uwTemp |= _kuwBitMask ;

    //write new content
	if ( f_okWriteReg ( &uwTemp ) != Ok ) eReturnStatus = Error ;

    //clear requested bit(s)
    uwTemp &= ( _kuwBitMask ^ 0xffff ) ;

    //write new content
	if ( f_okWriteReg ( &uwTemp ) != Ok ) eReturnStatus = Error ;

    return eReturnStatus ;
}
    
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess CAdfRegBase::f_okPulseBitsLow    ( TuwRegWord const      _kuwBitMask ) 
{
    ESuccess        eReturnStatus = Ok ;
    TuwRegWord      uwTemp ;

    //\\//

    //read current content
	if ( f_okReadReg ( &uwTemp ) != Ok ) eReturnStatus = Error ;

    //clear requested bit(s)
    uwTemp &= ( _kuwBitMask ^ 0xffff ) ;

    //write new content
	if ( f_okWriteReg ( &uwTemp ) != Ok ) eReturnStatus = Error ;

    //force requested bit(s)
    uwTemp |= _kuwBitMask ;

    //write new content
	if ( f_okWriteReg ( &uwTemp ) != Ok ) eReturnStatus = Error ;

    return eReturnStatus ;
}
    
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess CAdfRegBase::f_okReadBlock       ( TuwRegWord* const       _pauwArray,
                                            const unsigned long     _kulArraySize,
                                            const unsigned long     _kulRegAddrIncr,
                                            TuwRegWord* const       _puwMaxValue,
                                            CResultFile*            _poDumpFile )
{
    ESuccess        eReturnStatus = Ok ;

    unsigned long   ulSampleNum ;
    TuwRegWord*     puwVirtAddress ;
    TuwRegWord      uwReadData ;
    TuwRegWord      uwMaxData = 0 ;

    //\\//

    // we may have to increment the address of the register accessed
    // so we make a copy of the base address
    puwVirtAddress = m_puwVirtAddress ;

    for ( ulSampleNum = 0 ;
          ulSampleNum < _kulArraySize ;
          ulSampleNum ++ )
    {
#if  ( defined ( VME_IO_FAKE )  || defined ( VME_IO_NONE ) )  

        // simulate some noise
        static double fCenterBin = 8.0 ;
        double fSample ;
        fSample = fCenterBin + 4.75 * pow ( ( (double)rand()/(double)RAND_MAX - 0.5 ) , 3.0 );
        if ( fSample <    0.0 ) fSample =   0.1 ;
        if ( fSample >= 255.0 ) fSample = 254.9 ;
            uwReadData = (unsigned char) (fSample  + 0.5) ;

        // add a coarse pulse once in a while
        static unsigned long ulPulseDelay = 0 ;
        static unsigned long ulPulseAmplitude = 0 ;
        ulPulseDelay ++ ;
        if ( ulPulseDelay > 5000 ) 
        {
            ulPulseAmplitude += 40 ;
            if ( ulPulseAmplitude > 200 )
            {
                uwReadData += 75 ;
                ulPulseDelay     = 0 ;
                ulPulseAmplitude = 0 ;
            }
            else
                uwReadData += ulPulseAmplitude ;
        }

        // add an occasional weird spike
        if ( ( (float)rand() / (float)RAND_MAX ) <= 1.0E-5 )
            uwReadData += 500 ;

#else

        //read the register via its mapped address
        uwReadData  = *puwVirtAddress ;

        //increment virtual address as needed
        //this is address algebra on word data
        //so adding 1 (word) actually increments the address by 2 (bytes)
        puwVirtAddress += _kulRegAddrIncr ;

#endif

        // mask off all bits not in the ReadMask
        uwReadData &= KiAdfDacMaxOutput ;
        
        //update array 
        _pauwArray[ulSampleNum] = uwReadData ;

        //keep track of maximum value
        if ( uwReadData > uwMaxData ) uwMaxData = uwReadData ;

        //dump sample to file if requested
        if ( _poDumpFile != NULL )
        _poDumpFile->WriteLine ( I_to_A ( uwReadData ) ) ;

    }

    // remember the last value read 
    m_uwLastRead = uwReadData ;

    //pass maximum value back, if present
    if ( _puwMaxValue != NULL ) *_puwMaxValue = uwMaxData ;

    return eReturnStatus ;
}
    
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess CAdfRegBase::f_okReadAverage    ( double*               _pfAverage,
                                           const unsigned long   _kulSampleSize ) 
{
    ESuccess        eReturnStatus = Ok ;

    unsigned long   ulSampleNum ;
    TuwRegWord      uwReadData ;

    //\\//

    // use the return value to first build a running sum 
    *_pfAverage = 0 ;


    for ( ulSampleNum = 0 ;
          ulSampleNum < _kulSampleSize ;
          ulSampleNum ++ )
    {
        //read the register via its mapped address
        uwReadData  = *m_puwVirtAddress ;

        // mask off all bits not in the ReadMask
        uwReadData &= KiAdfDacMaxOutput ;
        
        //update the running sum
        *_pfAverage += uwReadData ;
    }

    // remember the last value read 
    m_uwLastRead = uwReadData ;

    //turn the running sum into an average
    *_pfAverage /= _kulSampleSize ;

    return eReturnStatus ;
}
    
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
TCString CAdfRegBase::f_osRegAddress     ( ) // e.g. return "@Master#0/Slave#1/Slot# 2/Chip# 0/Reg#  0"
{

    TCString  oMsg ;
    unsigned char    ubVertMaster ;
    unsigned char    ubVertSlave ;
    unsigned char    ubCardSlot ;
    unsigned char    ubChipAddress ;
    unsigned short   uwRegAddress ;

    //\\//

/* cannot access private members of BaseFpga */
/*    if ( this->m_poMotherFpga != NULL ) 
    {// This register hangs off an Fpga, Card,...
        ubVertMaster  = this->m_poMotherFpga->m_poMotherCard->m_ubVertMaster ;
        ubVertSlave   = this->m_poMotherFpga->m_poMotherCard->m_ubVertSlave ;
        ubCardSlot    = this->m_poMotherFpga->m_poMotherCard->m_ubCardSlot ;
        ubChipAddress = this->m_poMotherFpga->m_ubChipAddress ;
        uwRegAddress  = this->m_uwRegAddress ;
    }
    else
    {// this is a standalone register, just need to extract the address
*/        UnpackVmeAddress ( &ubVertMaster, 
                           &ubVertSlave, 
                           &ubCardSlot, 
                           &ubChipAddress, 
                           &uwRegAddress ) ;
//    }
    
#if defined VME_IO_DUAL_BIT3

    // use the upper byte for crate number
    // anything with slave or master non zero is crate 1
    oMsg = "@Crate#"  + I_to_A( ubVertSlave, 1 )
         + "/Slot#"   + I_to_A( ubCardSlot, 2 )
         + "/Chip#"   + I_to_A( ubChipAddress, 1 )
         + "/Reg#"    + I_to_A( uwRegAddress, 5 ) ;

#else

    oMsg = "@Master#" + I_to_A( ubVertMaster, 1 )
         + "/Slave#"  + I_to_A( ubVertSlave, 1 )
         + "/Slot#"   + I_to_A( ubCardSlot, 2 )
         + "/Chip#"   + I_to_A( ubChipAddress, 2 )
         + "/Reg#"    + I_to_A( uwRegAddress, 3 ) ;

#endif 

    return oMsg ;
}
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
TCString CAdfRegBase::f_osVmeAddress     ( )  // e.g. return "@0x01020000"
{
    TCString  oMsg ;

    //\\//

    oMsg = (TCString) "@" + I_to_A ( m_ulVmeAddress, 8, 16, '0' ) ; 

    return oMsg ;
}


/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
TulVmeAddress  CAdfRegBase::ConvertToVmeAddress       ( unsigned char      _ubVertMaster,
                                                     unsigned char      _ubVertSlave,
                                                     unsigned char      _ubCardSlot,
                                                     unsigned char      _ubChipAddress,
                                                     unsigned short     _uwRegAddress ) 
{
    VME_Reg_Address     xVmeAddress ;

    //\\//
    
#if defined VME_IO_DUAL_BIT3
    // use the upper byte for crate number
    // anything with slave or master non zero is crate 1
    if ( ( _ubVertSlave  != 0 ) 
      || ( _ubVertMaster != 0 ) )
    {
	    _ubVertSlave  = 1 ;
	    _ubVertMaster = 0 ;
    }
#endif 

    xVmeAddress.ulAddress       = 0 ;
	xVmeAddress.Field.uiReg     = _uwRegAddress ;
	xVmeAddress.Field.uiChip    = _ubChipAddress ;
	xVmeAddress.Field.uiCard    = _ubCardSlot ;
	xVmeAddress.Field.uiSlave   = _ubVertSlave ;
	xVmeAddress.Field.uiMaster  = _ubVertMaster ;
    xVmeAddress.Field.uiMustBe1 = 1 ;

#if defined VME_IO_NO_VERTICAL
	// test stand without the vertical interconnect
	xVmeAddress.Field.uiMustBe1 = 0 ;
    // use the upper byte for crate number
    // anything with slave or master non zero is crate 1
    if ( ( _ubVertSlave  != 0 ) 
      || ( _ubVertMaster != 0 ) )
    {
	    xVmeAddress.Field.uiSlave   = 1 ;
	    xVmeAddress.Field.uiMaster  = 0 ;
    }
#endif 

    return xVmeAddress.ulAddress ;
}


/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
TulVmeAddress  CAdfRegBase::ConvertToVmeOffset        ( unsigned char      _ubChipAddress,
                                                     unsigned short     _uwRegAddress ) 
{
    VME_Reg_Address     xVmeAddress ;

    //\\//
    
    xVmeAddress.ulAddress       = 0 ;
	xVmeAddress.Field.uiReg     = _uwRegAddress ;
	xVmeAddress.Field.uiChip    = _ubChipAddress ;

    return xVmeAddress.ulAddress ;
}


/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
void           CAdfRegBase::UnpackVmeAddress          ( unsigned char      *_ubVertMaster,
                                                     unsigned char      *_ubVertSlave,
                                                     unsigned char      *_ubCardSlot,
                                                     unsigned char      *_ubChipAddress,
                                                     unsigned short     *_uwRegAddress ) 
{
    VME_Reg_Address     xVmeAddress ;

    //\\//
    xVmeAddress.ulAddress       = m_ulVmeAddress ;

	*_uwRegAddress   = xVmeAddress.Field.uiReg ;
	*_ubChipAddress  = xVmeAddress.Field.uiChip ;
	*_ubCardSlot     = xVmeAddress.Field.uiCard ;
	*_ubVertSlave    = xVmeAddress.Field.uiSlave ;
	*_ubVertMaster   = xVmeAddress.Field.uiMaster ;
	
    return  ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
void  CAdfRegBase::Dump    ( bool               _lIncludeReadOnly,
                             CResultFile*       _poResultFile ) 
{
    CString     oMsg ;

    //\\//

    //skip Read Only Registers as requested
    if ( _lIncludeReadOnly == FALSE ) 
    if ( m_uwWriteMask == 0x0000 ) 
        return ; 

    //Prepare Message showing which Fpga is being dumped
    oMsg    = "  Reg#"
            + I_to_A ( m_uwRegAddress, 3 )
            + " "
            + f_osVmeAddress() 
            + " IOMask:R=" 
            + I_to_A ( m_uwReadMask, 4, 16, '0' ) 
            + "/W="
            + I_to_A ( m_uwWriteMask, 4, 16, '0' ) 
            + " Prev:R=" 
            + I_to_A ( m_uwLastRead, 4, 16, '0' ) 
            + "/W="
            + I_to_A ( m_uwLastWrite, 4, 16, '0' ) ;

    //Read current value
    f_okReadReg () ;

    oMsg   += " Now:" 
            + I_to_A ( m_uwLastRead, 5 ) ;

  
    //send message to Result File or Screen
    if ( _poResultFile != NULL ) 
        _poResultFile->WriteLine ( oMsg ) ;
    else
        CConsoleAccess::WriteLine ( oMsg ) ;

    return  ;
}
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess    CAdfRegBase::f_okHisto10bits  ( TAdcHisto* const        _kpauwHistogram,
                                            const unsigned long     _kulSampleSize,
                                            const unsigned long     _kulRegAddrIncr,
                                            CResultFile*            _poAdcDumpFile )
{
    ESuccess        eReturnStatus = Ok ;
    unsigned long   ulSampleNum ;
    unsigned long   ulBinNum ;
    TuwRegWord*     puwVirtAddress ;
    TuwRegWord      uwReadData ;

	//\\//

    //clear the histogram
    for ( ulBinNum  = 0 ; 
          ulBinNum <= KiAdfDacMaxOutput ;
          ulBinNum  ++ )
         (*_kpauwHistogram)[ulBinNum] = 0 ;

    // we may have to increment the address of the register accessed
    // so we make a copy of the base address
    puwVirtAddress = m_puwVirtAddress ;

    for ( ulSampleNum = 0 ;
          ulSampleNum < _kulSampleSize ;
          ulSampleNum ++ )
    {
#if  ( defined ( VME_IO_FAKE )  || defined ( VME_IO_NONE ) )  

        static double fCenterBin = 8.0 ;
        double fSample ;
        fSample = fCenterBin + 4.75 * pow ( ( (double)rand()/(double)RAND_MAX - 0.5 ) , 3.0 );
        if ( fSample <    0.0 ) fSample =   0.1 ;
        if ( fSample >= 255.0 ) fSample = 254.9 ;
        uwReadData = (unsigned char) (fSample  + 0.5) ;

#else

        //read the register via its mapped address
        uwReadData  = *puwVirtAddress ;

        //increment virtual address as needed
        //this is address algebra on word data
        //so adding 1 (word) actually increments the address by 2 (bytes)
        puwVirtAddress += _kulRegAddrIncr ;

#endif

        // mask off all bits not in the ReadMask
        uwReadData &= KiAdfDacMaxOutput ;
        
        //increment histogram
        (*_kpauwHistogram)[uwReadData] += 1 ;

        //dump sample to file if requested
        if ( _poAdcDumpFile != NULL )
        _poAdcDumpFile->WriteLine ( I_to_A ( uwReadData ) ) ;

    }

    // remember the last value read 
    m_uwLastRead = uwReadData ;

    return eReturnStatus ;
}
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
inline  double fSquare ( double _fValue ) { return _fValue * _fValue ; } ;
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
double  CAdfRegBase::HistoAverage10bit        ( TAdcHisto* const        _kpauwHistogram,
                                                const unsigned long     _kulSampleSize )
{
    unsigned int    i ; 
    unsigned int    sum_to_i = 0 ; 
    //\\//
    for ( i  = 0 ; i <= KiAdfDacMaxOutput ; i ++ )
        sum_to_i += (*_kpauwHistogram)[i]  * i ; 

    if ( _kulSampleSize >= 0 ) 
        return (double) sum_to_i / (double) _kulSampleSize ; 
    else 
        return 0 ; 
} ;
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
double  CAdfRegBase::HistoDeviation10bit      ( TAdcHisto* const        _kpauwHistogram,
                                                const unsigned long     _kulSampleSize,
                                                const double            _kfAverage )
{
    unsigned int    i ; 
    double          sum_to_i = 0.0 ; 
    //\\//
    for ( i  = 0 ; i <= KiAdfDacMaxOutput ; i ++ )
        sum_to_i += fSquare ( _kfAverage - (double) i ) * (double) (*_kpauwHistogram)[i] ; 

    if ( _kulSampleSize >= 0 ) 
        return sqrt ( sum_to_i / (double) _kulSampleSize ) ; 
    else 
        return 0 ; 
} ;
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
unsigned long   CAdfRegBase::HistoTailPopul10bit   ( TAdcHisto* const        _kpauwHistogram,
                                                     const unsigned long     _kulTailStartAt )
{
    unsigned int    i ; 
    unsigned long   tail_sum = 0 ; 
    //\\//
    for ( i  = _kulTailStartAt ; i <= KiAdfDacMaxOutput ; i ++ )
        tail_sum += (*_kpauwHistogram)[i] ; 

    return tail_sum ; 
} ;
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
double  CAdfRegBase::HistoAverage8bit         ( TAdfEtHisto* const      _kpauwHistogram,
                                                const unsigned long     _kulSampleSize )
{
    unsigned int    i ; 
    unsigned int    sum_to_i = 0 ; 
    //\\//
    for ( i  = 0 ; i <= KiAdfEtMaxOutput ; i ++ )
        sum_to_i += (*_kpauwHistogram)[i]  * i ; 

    if ( _kulSampleSize >= 0 ) 
        return (double) sum_to_i / (double) _kulSampleSize ; 
    else 
        return 0 ; 
} ;
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
double  CAdfRegBase::HistoDeviation8bit       ( TAdfEtHisto* const      _kpauwHistogram,
                                                const unsigned long     _kulSampleSize,
                                                const double            _kfAverage )
{
    unsigned int    i ; 
    double          sum_to_i = 0.0 ; 
    //\\//
    for ( i  = 0 ; i <= KiAdfEtMaxOutput ; i ++ )
        sum_to_i += fSquare ( _kfAverage - (double) i ) * (double) (*_kpauwHistogram)[i] ; 

    if ( _kulSampleSize >= 0 ) 
        return sqrt ( sum_to_i / (double) _kulSampleSize ) ; 
    else 
        return 0 ; 
} ;
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
unsigned long   CAdfRegBase::HistoTailPopul8bit    ( TAdfEtHisto* const      _kpauwHistogram,
                                                     const unsigned long     _kulTailStartAt )
{
    unsigned int    i ; 
    unsigned long   tail_sum = 0 ; 
    //\\//
    for ( i  = _kulTailStartAt ; i <= KiAdfEtMaxOutput ; i ++ )
        tail_sum += (*_kpauwHistogram)[i] ; 

    return tail_sum ; 
} ;
/*-------------------------------------------------------------------------------*/

