/*-------------------------------------------------------------------------------*/
  /*
    AdfRegEtLookup.cpp
    14-Sep-2005
  */
/*-------------------------------------------------------------------------------*/

#include "L1Cal_IIb_TCC/StdAfx.h"

#include "L1Cal_IIb_TCC/GlobalConstants.h"
#include "L1Cal_IIb_TCC/CustomDataTypes.h"

#include                            "L1Cal_IIb_TCC/AdfRegEtLookup.h"

//#include "L1Cal_IIb_TCC/DataManip.h"
#include "L1Cal_IIb_TCC/UtilsStrings.h"

//#include <math.h> // for f_okHisto10bits 

// access to the console
// static means created only once throughout life of program (to avoid wasting time)
// *AND* internal linkage (to avoid clash between modules)
#include "L1Cal_IIb_TCC/HandleConsole.h"
static CConsoleAccess oConsole ;

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
CAdfRegEtLookup::CAdfRegEtLookup      ( CAdfChipBase*       _poMotherFpga,
                                        unsigned short      _uwBaseRegAddress,
                                        unsigned short      _uwInputZeresp,
                                        unsigned short      _uwOutputZeresp,
                                        double              _fdSlope ) 
             : CAdfRegBase            (                     _poMotherFpga, 
                                                            _uwBaseRegAddress,
                                                             0xffff,            // write mask
                                                             0xffff )           // read mask
{
    m_poScratchReg = new CAdfRegBase  ( _poMotherFpga, 
                                        _uwBaseRegAddress,
                                         0xffff,            // write mask
                                         0xffff ) ;         // read mask

    m_uwInputZeresp  = _uwInputZeresp  ;
    m_uwOutputZeresp = _uwOutputZeresp ;
    m_fdSlope        = _fdSlope        ;

}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
CAdfRegEtLookup::~CAdfRegEtLookup         ( ) 
{

    delete m_poScratchReg ;

}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess CAdfRegEtLookup::f_okLoadLookup  ( unsigned short*     _puwInputZeresp,
                                            unsigned short*     _puwOutputZeresp,
                                            double*             _pfdSlope,
                                            bool                _lSkipVerify ) 
// Initialize Et Lookup Memory

{
    ESuccess        eReturnStatus = Ok ;

    int             iEtValue ;
    unsigned int    uiFilteredEValue ;
    TuwRegWord      uwLookupEtValue ;

    //\\//

    // see if we were given new coefficients
    if ( _puwInputZeresp  != NULL ) m_uwInputZeresp  = *_puwInputZeresp  ;
    if ( _puwOutputZeresp != NULL ) m_uwOutputZeresp = *_puwOutputZeresp ;
    if ( _pfdSlope        != NULL ) m_fdSlope        = *_pfdSlope        ;


    // write the lookup memory (straight write with no verification yet)
    for ( uiFilteredEValue = 0 ; uiFilteredEValue < Ki10BitRange ; uiFilteredEValue ++ )
    {
        // we can use the crude conversion (i.e. truncation after adding 1/2 ) 
        // without worrying about the truncation of negative values
        // because we will zero out any negative result anyway
        iEtValue = (int) ( m_fdSlope * ( (float) uiFilteredEValue 
                                       - (float) m_uwInputZeresp ) 
                         + (float) m_uwOutputZeresp 
                         + 0.5 ) ;

        // handle low saturation 
        if ( iEtValue < 0 ) 
            uwLookupEtValue = 0 ;
        else 
            uwLookupEtValue = (unsigned short) iEtValue ;

        // handle high saturation (e.g. even the devide by 4 saturates on the last 2 counts)
        if ( uwLookupEtValue > 0xff ) uwLookupEtValue = 0xff ;

        // program the value via the temporary register by adjusting the targetted address
        m_poScratchReg->m_puwVirtAddress = m_puwVirtAddress +   uiFilteredEValue ; // pointer arithmetic will add 2 (=size of short) per address
        m_poScratchReg->m_ulVmeAddress   = m_ulVmeAddress   + 2*uiFilteredEValue ; // do both VME+Virtual addr to skip worrying about bit3 access method.
    
        if ( m_poScratchReg->f_okWriteOnly ( &uwLookupEtValue ) != Ok )
        {
            eReturnStatus = Error ;
            goto Abort ;
        }
    }

    // Now verify the whole thing
    // if ( eReturnStatus == Ok )  f_okVerifyLookup ( ) ;  No, this is done later, and during Full Init only

Abort :

    return eReturnStatus ;
}
    
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess CAdfRegEtLookup::f_okVerifyLookup   ( )

// Verify the content of Et Lookup Memory

{
    ESuccess        eReturnStatus = Ok ;

    int             iEtValue ;
    unsigned int    uiFilteredEValue ;
    TuwRegWord      uwLookupEtValue ;

    TuwRegWord      uwActualValue ;
    unsigned int    iTotError = 0 ;

    //\\//

    for ( uiFilteredEValue = 0 ; uiFilteredEValue < Ki10BitRange ; uiFilteredEValue ++ )
    {
        // we can use the crude conversion (i.e. truncation after adding 1/2 ) 
        // without worrying about the truncation of negative values
        // because we will zero out any negative result anyway
        iEtValue = (int) ( m_fdSlope * ( (float) uiFilteredEValue 
                                       - (float) m_uwInputZeresp ) 
                         + (float) m_uwOutputZeresp 
                         + 0.5 ) ;

        // handle low saturation 
        if ( iEtValue < 0 ) 
            uwLookupEtValue = 0 ;
        else 
            uwLookupEtValue = (unsigned short) iEtValue ;

        // handle high saturation (e.g. even the devide by 4 saturates on the last 2 counts)
        if ( uwLookupEtValue > 0xff ) uwLookupEtValue = 0xff ;

        // program the value via the temporary register by adjusting the targetted address
        m_poScratchReg->m_puwVirtAddress = m_puwVirtAddress +   uiFilteredEValue ; // pointer arithmetic will add 2 (=size of short) per address
        m_poScratchReg->m_ulVmeAddress   = m_ulVmeAddress   + 2*uiFilteredEValue ; // do both VME+Virtual addr to skip worrying about bit3 access method.
    
        if ( m_poScratchReg->f_okReadReg ( &uwActualValue ) != Ok )
        {
            eReturnStatus = Error ;
            oConsole.ErrorLine ( (CString) " Verification Aborted after IO Error" ) ;
            oConsole.ErrorLine ( (CString) "_" + m_poScratchReg->f_osRegAddress ( ) ) ; // will match because rebuilt from vme address
            goto Abort ;
        }

        if ( uwActualValue != uwLookupEtValue )
        {
            iTotError ++ ;
            if ( eReturnStatus != Error )
            {
                oConsole.ErrorLine ( (CString) " Verification Error at Input E = "
                                              + I_to_A ( uiFilteredEValue ) 
                                              + " Found Output Et " 
                                              + I_to_A ( uwActualValue ) 
                                              + " Instead of "
                                              + I_to_A ( uwLookupEtValue ) ) ;
                oConsole.ErrorLine ( (CString) "_" + m_poScratchReg->f_osRegAddress ( ) ) ; // will match because rebuilt from vme address
            }
            eReturnStatus = Error ;
        }
    }

    if ( eReturnStatus == Error )
        oConsole.ErrorLine ( (CString) "_Found a total of "
                                      + I_to_A ( iTotError ) 
                                      + " Errors for this Et Lookup Table" ) ;

Abort :

    return eReturnStatus ;
}
    
