/*-------------------------------------------------------------------------------*/
  /*
    AdfRegOutputEt.cpp
    13-Nov-2005
  */
/*-------------------------------------------------------------------------------*/

#include "L1Cal_IIb_TCC/StdAfx.h"

#include "L1Cal_IIb_TCC/GlobalConstants.h"
#include "L1Cal_IIb_TCC/CustomDataTypes.h"

#include                            "L1Cal_IIb_TCC/AdfRegOutputEt.h"

#include "L1Cal_IIb_TCC/AdfChipFpga_T7.h"

#include "L1Cal_IIb_TCC/L1Cal.h"

//#include "L1Cal_IIb_TCC/DataManip.h"
#include "L1Cal_IIb_TCC/UtilsBits.h"
#include "L1Cal_IIb_TCC/UtilsStrings.h"

// access to the console
// static means created only once throughout life of program (to avoid wasting time)
// *AND* internal linkage (to avoid clash between modules)
#include "L1Cal_IIb_TCC/HandleConsole.h"
static CConsoleAccess oConsole ;

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
CAdfRegOutputEt::CAdfRegOutputEt      ( CAdfChipFpga_T7*    _poMotherFpga,
                                        unsigned short      _uwBaseRegAddress )
               : CAdfRegBase          (                     _poMotherFpga,
                                                            _uwBaseRegAddress,
                                                             0xffff,            // write mask
                                                             0xffff )           // read mask
{
    m_poScratchReg = new CAdfRegBase  ( _poMotherFpga,
                                        _uwBaseRegAddress,
                                         0xffff,            // write mask
                                         0xffff ) ;         // read mask

    m_poMotherFpga      = _poMotherFpga ; 

    m_poScratchReg->f_okSetFlagVolatile ( ) ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
CAdfRegOutputEt::~CAdfRegOutputEt         ( )
{

    delete m_poScratchReg ;

}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess CAdfRegOutputEt::SetSimuOutputEt   ( ETT_Chan_Type           _eTTType,
                                              const unsigned char     _ubSimuOutputEt,
                                              const unsigned long     _ulTickNum )
{
    ESuccess        eReturnStatus = Ok ;

    unsigned long   ulBaseVmeAddress ;
    TuwRegWord*     puwBaseVirtualAddress ;

    unsigned long   ulMemoryOffset ;

    TuwRegWord      uwTemp ;

    //\\//

    //check input parameters
    if ( ( _eTTType          >= eTot_TT_Chan_Type )
      || ( _ubSimuOutputEt   >= KiMaxLookupZeroEngOutEt )
      || ( _ulTickNum        == 0 )
      || ( _ulTickNum        >  KiTotTicksInTurn ) )
    {
        eReturnStatus = Error ;
        CConsoleAccess::InternalErrorLine ( (CString) "CAdfRegOutputEt::SetSimuOutputEt: Illegal Parameter" ) ;
        goto Abort ;
    }

    // make copies of the base addresses so that we can increment them
    ulBaseVmeAddress        = m_ulVmeAddress ;
    puwBaseVirtualAddress   = m_puwVirtAddress ;

    // build a one dimensional index
    ulMemoryOffset = _ulTickNum - 1 ;

    // apply fixed address shift
    ulMemoryOffset += CL1Cal::mg_uwOffsetTick1OutputEtSimu ;

    // wrap around as needed
    ulMemoryOffset %= KiTotTicksInTurn ;

    // program the value via the temporary register by adjusting the targetted address
    m_poScratchReg->m_puwVirtAddress = puwBaseVirtualAddress +   ulMemoryOffset ; // pointer arithmetic will add 2 (=size of short) per address
    m_poScratchReg->m_ulVmeAddress   = ulBaseVmeAddress      + 2*ulMemoryOffset ; // do both VME+Virtual addr to skip worrying about bit3 access method.

    if ( m_poScratchReg->f_okReadReg ( &uwTemp ) != Ok )
        eReturnStatus = Error ;

    if ( _eTTType == eTT_EM ) // EM Et is stored in the upper Byte
        uwTemp = ( uwTemp & 0x00ff ) & ( _ubSimuOutputEt << KiShiftOneByte ) ;
    else // HD Et is stored in the lower Byte
        uwTemp = ( uwTemp & 0xff00 ) & ( _ubSimuOutputEt ) ;

    if ( m_poScratchReg->f_okWriteReg ( &uwTemp ) != Ok )
        eReturnStatus = Error ;

Abort :
    return eReturnStatus ;
}
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess CAdfRegOutputEt::ReadMonitOutputEt ( unsigned char*          _pubMonitOutputEtEm,
                                              unsigned char*          _pubMonitOutputEtHd,
                                              const unsigned long     _ulTickNum )
{
    ESuccess        eReturnStatus = Ok ;

    unsigned long   ulBaseVmeAddress ;
    TuwRegWord*     puwBaseVirtualAddress ;

    unsigned long   ulMemoryOffset ;

    TuwRegWord      uwTemp ;

    //\\//

    //check input parameters
    if ( ( _ulTickNum  == 0 )
      || ( _ulTickNum  >  KiTotTicksInTurn ) )
    {
        eReturnStatus = Error ;
        CConsoleAccess::InternalErrorLine ( (CString) "CAdfRegOutputEt::ReadMonitOutputEt: Illegal Parameter" ) ;
        goto Abort ;
    }

    // make copies of the base addresses so that we can increment them
    ulBaseVmeAddress        = m_ulVmeAddress ;
    puwBaseVirtualAddress   = m_puwVirtAddress ;

    // build a one dimensional index
    ulMemoryOffset = _ulTickNum - 1 ;

    // apply fixed address shift
    ulMemoryOffset += CL1Cal::mg_uwOffsetTick1OutputEtMonit ;
    // We need to use a different value if we are using the memory for simulated data
    if ( m_poMotherFpga != NULL ) 
    if ( m_poMotherFpga->f_eAdfOut_MemMode_Current() == eAdfOut_MemMode_PlaySimuTurn )
        ulMemoryOffset += CL1Cal::mg_uwOffsetTick1OutputEtSimu ;
    

    // wrap around as needed
    ulMemoryOffset %= KiTotTicksInTurn ;

    // Read the value via the temporary register by adjusting the targetted address
    m_poScratchReg->m_puwVirtAddress = puwBaseVirtualAddress +   ulMemoryOffset ; // pointer arithmetic will add 2 (=size of short) per address
    m_poScratchReg->m_ulVmeAddress   = ulBaseVmeAddress      + 2*ulMemoryOffset ; // do both VME+Virtual addr to skip worrying about bit3 access method.

    if ( m_poScratchReg->f_okReadReg ( &uwTemp ) != Ok )
        eReturnStatus = Error ;

    // extract each byte, as needed
    if ( _pubMonitOutputEtEm != NULL ) // EM Et is stored in the upper Byte
        *_pubMonitOutputEtEm  = fubTruncToByte ( uwTemp >> KiShiftOneByte ) ;

    if ( _pubMonitOutputEtHd != NULL ) // HD Et is stored in the lower Byte
        *_pubMonitOutputEtHd  = fubTruncToByte ( uwTemp );

Abort :
    return eReturnStatus ;
}
/*-------------------------------------------------------------------------------*/

