/*-------------------------------------------------------------------------------*/
  /*
    AdfRegRawAdc.cpp
    13-Nov-2005
  */
/*-------------------------------------------------------------------------------*/

#include "L1Cal_IIb_TCC/StdAfx.h"

#include "L1Cal_IIb_TCC/GlobalConstants.h"
#include "L1Cal_IIb_TCC/CustomDataTypes.h"

#include                            "L1Cal_IIb_TCC/AdfRegRawAdc.h"

#include "L1Cal_IIb_TCC/L1Cal.h"

//#include "L1Cal_IIb_TCC/DataManip.h"
#include "L1Cal_IIb_TCC/UtilsStrings.h"

// access to the console
// static means created only once throughout life of program (to avoid wasting time)
// *AND* internal linkage (to avoid clash between modules)
#include "L1Cal_IIb_TCC/HandleConsole.h"
static CConsoleAccess oConsole ;

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
CAdfRegRawAdc::CAdfRegRawAdc          ( CAdfChipFpga_T7*    _poMotherFpga,
                                        unsigned short      _uwBaseRegAddress ) 
             : CAdfRegBase            (                     _poMotherFpga, 
                                                            _uwBaseRegAddress,
                                                             0xffff,            // write mask
                                                             0xffff )           // read mask
{
    m_poScratchReg = new CAdfRegBase  ( _poMotherFpga, 
                                        _uwBaseRegAddress,
                                         0xffff,            // write mask
                                         0xffff ) ;         // read mask

    m_poMotherFpga      = _poMotherFpga ; 

    m_poScratchReg->f_okSetFlagVolatile ( ) ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
CAdfRegRawAdc::~CAdfRegRawAdc         ( ) 
{

    delete m_poScratchReg ;

}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess CAdfRegRawAdc::SetSimuRawAdc   ( const unsigned short    _uwSimuRawAdc,
                                          const unsigned long     _ulTickNum,
                                          const unsigned long     _ulSubTickNum ) 
{
    ESuccess        eReturnStatus = Ok ;

    unsigned long   ulBaseVmeAddress ;
    TuwRegWord*     puwBaseVirtualAddress ;

    unsigned long   ulMemoryOffset ;

    //\\//

    //check input parameters
    if ( ( _uwSimuRawAdc     >= KiMaxLookupZeroEngAdcE ) 
      || ( _ulTickNum        == 0 )
      || ( _ulTickNum        >  KiTotTicksInTurn ) 
      || ( _ulSubTickNum     >= KiTotSamplesPerTick ) )
    {
        eReturnStatus = Error ;
        CConsoleAccess::InternalErrorLine ( (CString) "CAdfRegRawAdc::SetSimuRawAdc: Illegal Parameter" ) ;
        goto Abort ;
    }

    // make copies of the base addresses so that we can increment them
    ulBaseVmeAddress        = m_ulVmeAddress ;
    puwBaseVirtualAddress   = m_puwVirtAddress ;

    // build a one dimensional index
    ulMemoryOffset = KiTotSamplesPerTick * ( _ulTickNum - 1 ) + _ulSubTickNum ;

    // apply fixed address shift 
    ulMemoryOffset += CL1Cal::mg_uwOffsetTick1RawAdcSimu ;

    // wrap around as needed 
    ulMemoryOffset %= KiTotSamplesPerTick * KiTotTicksInTurn ;

    // program the value via the temporary register by adjusting the targetted address
    m_poScratchReg->m_puwVirtAddress = puwBaseVirtualAddress +   ulMemoryOffset ; // pointer arithmetic will add 2 (=size of short) per address
    m_poScratchReg->m_ulVmeAddress   = ulBaseVmeAddress      + 2*ulMemoryOffset ; // do both VME+Virtual addr to skip worrying about bit3 access method.

    if ( m_poScratchReg->f_okWriteReg ( &_uwSimuRawAdc ) != Ok )
        eReturnStatus = Error ;

Abort :
    return eReturnStatus ;
}
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess CAdfRegRawAdc::ReadMonitRawAdc     ( unsigned short*         _puwMonitRawAdc,
                                              const unsigned long     _ulTickNum,
                                              const unsigned long     _ulSubTickNum )
{
    ESuccess        eReturnStatus = Ok ;

    unsigned long   ulBaseVmeAddress ;
    TuwRegWord*     puwBaseVirtualAddress ;

    unsigned long   ulMemoryOffset ;

    //\\//

    //check input parameters
    if ( ( _ulTickNum        == 0 )
      || ( _ulTickNum        >  KiTotTicksInTurn ) 
      || ( _ulSubTickNum     >= KiTotSamplesPerTick ) )
    {
        eReturnStatus = Error ;
        CConsoleAccess::InternalErrorLine ( (CString) "CAdfRegOutputEt::ReadMonitRawAdc: Illegal Parameter" ) ;
        goto Abort ;
    }

    // make copies of the base addresses so that we can increment them
    ulBaseVmeAddress        = m_ulVmeAddress ;
    puwBaseVirtualAddress   = m_puwVirtAddress ;

    // build a one dimensional index
    ulMemoryOffset = KiTotSamplesPerTick * ( _ulTickNum - 1 ) + _ulSubTickNum ;

    // apply fixed address shift
    ulMemoryOffset += CL1Cal::mg_uwOffsetTick1RawAdcMonit ;
    // We need to use a different value if we are using the memory for simulated data
    if ( m_poMotherFpga != NULL ) 
    if ( m_poMotherFpga->f_eRawAdc_MemMode_Current() == eRawAdc_MemMode_PlaySimuTurn )
        ulMemoryOffset += CL1Cal::mg_uwOffsetTick1RawAdcSimu ;
    

    // wrap around as needed
    ulMemoryOffset %= KiTotSamplesPerTick * KiTotTicksInTurn ;

    // Read the value via the temporary register by adjusting the targetted address
    m_poScratchReg->m_puwVirtAddress = puwBaseVirtualAddress +   ulMemoryOffset ; // pointer arithmetic will add 2 (=size of short) per address
    m_poScratchReg->m_ulVmeAddress   = ulBaseVmeAddress      + 2*ulMemoryOffset ; // do both VME+Virtual addr to skip worrying about bit3 access method.

    if ( m_poScratchReg->f_okReadReg ( _puwMonitRawAdc ) != Ok )
        eReturnStatus = Error ;

Abort :
    return eReturnStatus ;
}
/*-------------------------------------------------------------------------------*/

