/*-------------------------------------------------------------------------------*/
  /*
    CommandFileConfigFpga.cpp
    2-Sep-99 split off from dialog box
    Note: this command file does not abort on FPGA load error
  */
/*-------------------------------------------------------------------------------*/
#include "L1Cal_IIb_TCC/StdAfx.h"

#include "L1Cal_IIb_TCC/GlobalConstants.h"
#include "L1Cal_IIb_TCC/CustomDataTypes.h"

//#include "L1Cal_IIb_TCC/AdfRegBase.h"
//#include "L1Cal_IIb_TCC/AdfChipBase.h"
#include "L1Cal_IIb_TCC/AdfCardBase.h"

#include "L1Cal_IIb_TCC/NevisCard.h"

#include "L1Cal_IIb_TCC/HandleCommandFiles.h"

#include                            "L1Cal_IIb_TCC/CommandFileConfigFpga.h"

#include "L1Cal_IIb_TCC/HandleConfigAdfFpga.h"

#include "tabgab/tcc_to_tabgab_config.h"
#include "tabgab/util.h"

#include "L1Cal_IIb_TCC/UtilsStrings.h"
#include "L1Cal_IIb_TCC/UtilsBits.h"

// static means created only once throughout life of program (to avoid wasting time)
// *AND* internal linkage (to avoid clash between modules)
#include "L1Cal_IIb_TCC/HandleConsole.h"
static CConsoleAccess oConsole ;

// array to go from FPGA number to FPGA Mask
const unsigned int    auiNumToMask[17+1] = { /* 0*/ 0x00000001, // VME Interface is NOT programmable 
                                       /* 1*/ 0x00000002, // Main Array FPGA #1
                                       /* 2*/ 0x00000004,
                                       /* 3*/ 0x00000008,
                                       /* 4*/ 0x00000010,
                                       /* 5*/ 0x00000020,
                                       /* 6*/ 0x00000040,
                                       /* 7*/ 0x00000080,
                                       /* 8*/ 0x00000100,
                                       /* 9*/ 0x00000200,
                                       /*10*/ 0x00000400,
                                       /*11*/ 0x00000800,
                                       /*12*/ 0x00001000,
                                       /*13*/ 0x00002000,
                                       /*14*/ 0x00004000,
                                       /*15*/ 0x00008000,
                                       /*16*/ 0x00010000, // Main Array FPGA #16
                                       /*17*/ 0x00020000 } ; // Support FPGA

//Global variable to hold 
CString     goLastConfigStatus = "" ;

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess CommandFileConfigFpga    ( CString	                        _osCommand_File_Name,
                                    Custom_Structure_ConfigFpga*    _pxCustom, 
                                    void*                           _pDlgThis )
{
    ESuccess            eReturnStatus = Ok ;

    ESuccess            eSuccess ;
    Parse_Structure     xParse ;

    //\\//

    // write to console 
	oConsole.WriteLine ( "" ) ;
	oConsole.WriteLine ( (CString) "%% Configure FPGA Command File : "
                        		 +  _osCommand_File_Name ) ;
    
    // Initialize the Command File Parser control Structure
    eSuccess = Init_Parse_Structure ( &xParse ) ;

    xParse.pacFile_Name = LPCTSTR(_osCommand_File_Name) ;

    //register the legal keywords and 
    //the call back points for the apropriate keywords
    xParse.Actions.pfFile_Opened            = (fKeyword_Action*) ConfigFPGA_Action_File_Opened ;
    xParse.Actions.pfVertical_Master        = NULL ; // i.e. Keyword ok but no action
    xParse.Actions.pfVertical_Slave         = NULL ;
    xParse.Actions.pfCard_Slot              = NULL ;
    xParse.Actions.pfChip_Address           = (fKeyword_Action*) ConfigFPGA_Action_Chip_Address ;
    xParse.Actions.pfChip_Mask              = NULL ;
    xParse.Actions.pfConfig_File            = (fKeyword_Action*) ConfigFPGA_Action_Parse_Config_File ;
    xParse.Actions.pfConfig_TabGab          = (fKeyword_Action*) ConfigFPGA_Action_Parse_Config_TabGab ;
    xParse.Actions.pfDone_With_File         = (fKeyword_Action*) ConfigFPGA_Action_Done_With_File ;
    
    _pxCustom->uiTotFPGAThisFile        = 0 ;
    _pxCustom->uiTotTabGabThisFile      = 0 ;
    for ( int i = 0 ; i < KiMaxRecursivity ; i++ )
    {
        _pxCustom->auiTotFPGAPrevFile[i] = 0 ;
        _pxCustom->auiTotTabGabPrevFile[i] = 0 ;
    }
    _pxCustom->uiFPGA_Config_Error    = 0 ;

    // call the general purpose command file parser 
    eSuccess = Read_Command_File ( &xParse, _pxCustom, _pDlgThis ) ;

    goLastConfigStatus      = "%%Total Configured" ;
    if ( _pxCustom->uiTotFPGAThisFile != 0 ) 
        goLastConfigStatus      += " / ADF FPGA = " 
                                + I_to_A ( _pxCustom->uiTotFPGAThisFile ) ;
    if ( _pxCustom->uiTotTabGabThisFile != 0 ) 
        goLastConfigStatus      += " / TabGab Module = " 
                                + I_to_A ( _pxCustom->uiTotTabGabThisFile ) ;
    goLastConfigStatus          += " / Error = "
                                +  I_to_A ( _pxCustom->uiFPGA_Config_Error ) ;

    // This command file does not abort on error, 
    // the error status has thus to be retrieved from the custom parameters     
    if ( _pxCustom->uiFPGA_Config_Error == 0 )
    {
        oConsole.SystemLine ( goLastConfigStatus ) ;
    }
    else
    {
        oConsole.ErrorLine ( goLastConfigStatus ) ;
    	eReturnStatus = Error ;
    }

    return eReturnStatus ;
}


/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/


/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess ConfigFPGA_Action_Chip_Address          ( void* _pxParse, 
                                                   void* _pxCustom,
                                                   void* _pDlgThis )
{
    Parse_Structure*                pxParse     = (Parse_Structure*)                _pxParse ;
    Custom_Structure_ConfigFpga*    pxCustom    = (Custom_Structure_ConfigFpga*)    _pxCustom ;

	ESuccess         lReturnStatus = Ok ;

    //\\//

    // verify that the Chip Number is within the range
    if ( ( pxParse->Values.ulChip_Address < 1 ) | ( pxParse->Values.ulChip_Address > 17 ) )
    {
        oConsole.ErrorLine ( (CString) "_ Chip Number out of Range at Line #"
                                      + I_to_A ( pxParse->iLineNum ) ) ;
	    lReturnStatus = Error ;
	    goto Done ;
    }

    // This keyword specifies ONE fpga, so add it to the fpga mask, in a cumulative way.
    // the mask will be cleared after each corresponding Configuration.
    if ( pxParse->Values.ulChip_Mask == diNotYetAssigned ) 
        pxParse->Values.ulChip_Mask = auiNumToMask[pxParse->Values.ulChip_Address] ;
	else
        pxParse->Values.ulChip_Mask = pxParse->Values.ulChip_Mask | auiNumToMask[pxParse->Values.ulChip_Address] ;

Done:
    return lReturnStatus ;
}


/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess ConfigFPGA_Action_Parse_Config_File     ( void* _pxParse, 
                                                   void* _pxCustom,
                                                   void* _pDlgThis )
{
    Parse_Structure*                pxParse     = (Parse_Structure*)                _pxParse ;
    Custom_Structure_ConfigFpga*    pxCustom    = (Custom_Structure_ConfigFpga*)    _pxCustom ;

	ESuccess        lReturnStatus = Ok ;

    ESuccess        eSuccess ;

    CAdfCardBase*   poTargetCard ;

    EYesNo          eVerbose = No ;

    //\\//

    // check all other keywords have been assigned
    if ( ( pxParse->Values.ulVertical_Master  == diNotYetAssigned ) 
      || ( pxParse->Values.ulVertical_Slave   == diNotYetAssigned ) 
      || ( pxParse->Values.ulCard_Slot        == diNotYetAssigned ) 
      || ( pxParse->Values.ulChip_Mask        == diNotYetAssigned ) ) 
    {
        oConsole.ErrorLine ( (CString) "_ Missing Keyword(s) before Config File at Line #"
                                      + I_to_A ( pxParse->iLineNum ) ) ;
	    lReturnStatus = Error ;
	    goto Done ;
    }

    // create a temporary card object for the target card
    poTargetCard = new CAdfCardBase  (  fubTruncToByte( pxParse->Values.ulVertical_Master ) ,
                                        fubTruncToByte( pxParse->Values.ulVertical_Slave ) ,
                                        fubTruncToByte( pxParse->Values.ulCard_Slot ) ) ;

    // call the Configuration utility
    eSuccess = ConfigureAdfFpga     ( poTargetCard,
                                      pxParse->Values.ulChip_Mask,
            	                      pxParse->Values.osGeneric_String_Parameter ) ;

    //keep track of the number of errors
    if ( eSuccess != Ok )
    {
//        lReturnStatus = Error ;  Do report error to avoid aborting
        pxCustom->uiFPGA_Config_Error++ ;
    }

    // keep track of how many FPGAs have been Configured
    pxCustom->uiTotFPGAThisFile += fulNumBitSet ( pxParse->Values.ulChip_Mask ) ;

    // clear the FPGA Mask so that the set of FPGAs to be Configured needs to be re-specified
    pxParse->Values.ulChip_Mask = diNotYetAssigned ;
    
    // now delete temporary card object 
    delete poTargetCard ;

Done:
//    return lReturnStatus ;
    return Ok ; // always ok so that we do not abort on error
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess ConfigFPGA_Action_Parse_Config_TabGab           ( void* _pxParse, 
                                                           void* _pxCustom,
                                                           void* _pDlgThis )
{
    Parse_Structure*                pxParse     = (Parse_Structure*)                _pxParse ;
    Custom_Structure_ConfigFpga*    pxCustom    = (Custom_Structure_ConfigFpga*)    _pxCustom ;

	ESuccess        lReturnStatus = Ok ;

    EYesNo          eVerbose = No ;

    //\\//

    // display parameters
    oConsole.WriteLine ( (CString) " Configuring TAB/GAB module #"
                                    + I_to_A ( pxParse->Values.ulConfig_TabGab ) ) ;

    // call the Configuration utility
    // and keep track of the number of errors
    if ( ReportTabGabStatus (  Tab_or_Gab_Configure ( pxParse->Values.ulConfig_TabGab ),
                              "Tab_or_Gab_Configure",
                              "ConfigFPGA_Action_Parse_Config_TabGab" ) != Ok )
    {
//        lReturnStatus = Error ;  Do report error to avoid aborting
        pxCustom->uiFPGA_Config_Error++ ;
    }

    // keep track of how many TabGab Modules have been Configured
    pxCustom->uiTotTabGabThisFile += 1 ;

    return lReturnStatus ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess ConfigFPGA_Action_File_Opened           ( void* _pxParse, 
                                                   void* _pxCustom,
                                                   void* _pDlgThis )
{
    Parse_Structure*                pxParse     = (Parse_Structure*)                _pxParse ;
    Custom_Structure_ConfigFpga*    pxCustom    = (Custom_Structure_ConfigFpga*)    _pxCustom ;

	ESuccess         lReturnStatus = Ok ;

    //\\//

    oConsole.WriteLine ( "" ) ;

    //shift up (push) the content of the PrevFile array...
    for ( int i = KiMaxRecursivity-1 ; i > 0 ; i-- )
    {
        pxCustom->auiTotFPGAPrevFile[i]= pxCustom->auiTotFPGAPrevFile[i-1] ;
        pxCustom->auiTotTabGabPrevFile[i]= pxCustom->auiTotTabGabPrevFile[i-1] ;
    }

    //to make room to store the current running total...
    pxCustom->auiTotFPGAPrevFile[0]     = pxCustom->uiTotFPGAThisFile ;
    pxCustom->auiTotTabGabPrevFile[0]   = pxCustom->uiTotTabGabThisFile ;

    //and now we have a local place for this (maybe recursive) Configuration command file
    pxCustom->uiTotFPGAThisFile      = 0 ;
    pxCustom->uiTotTabGabThisFile    = 0 ;

    return lReturnStatus ;
}


/*-------------------------------------------------------------------------------*/
ESuccess ConfigFPGA_Action_Done_With_File        ( void* _pxParse, 
                                                   void* _pxCustom,
                                                   void* _pDlgThis )
{
    Parse_Structure*                pxParse     = (Parse_Structure*)                _pxParse ;
    Custom_Structure_ConfigFpga*    pxCustom    = (Custom_Structure_ConfigFpga*)    _pxCustom ;

	ESuccess         lReturnStatus = Ok ;

    CString          osConfigStatus ;

    //\\//

    osConfigStatus      = "_ after Configuring: " ;
    if ( pxCustom->uiTotFPGAThisFile != 0 ) 
        osConfigStatus      += "   " 
                            +  I_to_A ( pxCustom->uiTotFPGAThisFile ) 
                            +  " ADF FPGA(s)" ;
    if ( pxCustom->uiTotTabGabThisFile != 0 ) 
        osConfigStatus      += "   " 
                            +  I_to_A ( pxCustom->uiTotTabGabThisFile ) 
                            + " TabGab Module(s)" ;

    oConsole.WriteLine ( osConfigStatus ) ;
    oConsole.WriteLine ( "" ) ;

    //add the contribution of this file to the previous total 
    //(which had been pushed in the first element of the PrevFile array)
    //and we now have the current grand total
    pxCustom->uiTotFPGAThisFile     += pxCustom->auiTotFPGAPrevFile[0] ;
    pxCustom->uiTotTabGabThisFile   += pxCustom->auiTotTabGabPrevFile[0] ;

    //shift down (pop) the content of the array ...
    for ( int i = 0 ; i < KiMaxRecursivity-1 ; i++ )
    {
        pxCustom->auiTotFPGAPrevFile[i]  = pxCustom->auiTotFPGAPrevFile[i+1] ;
        pxCustom->auiTotTabGabPrevFile[i]= pxCustom->auiTotTabGabPrevFile[i+1] ;
    }

    //and force a zero in the last element
    pxCustom->auiTotFPGAPrevFile[KiMaxRecursivity-1]   = 0 ;
    pxCustom->auiTotTabGabPrevFile[KiMaxRecursivity-1] = 0 ;

    return lReturnStatus ;

}

