/*-------------------------------------------------------------------------------*/
  /*
    CommandFileConfigT962.cpp
  */
/*-------------------------------------------------------------------------------*/
#include "L1Cal_IIb_TCC/StdAfx.h"

#include "L1Cal_IIb_TCC/HandleCommandFiles.h"

#include                        "L1Cal_IIb_TCC/CommandFileConfigT962.h"

#include "L1Cal_IIb_TCC/T962.h"

#include "L1Cal_IIb_TCC/UtilsStrings.h"
#include "L1Cal_IIb_TCC/UtilsBits.h"

// static means created only once throughout life of program (to avoid wasting time)
// *AND* internal linkage (to avoid clash between modules)
#include "L1Cal_IIb_TCC/HandleConsole.h"
static CConsoleAccess oConsole ;

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess CommandFileConfigT962    ( CString	                        _osCommand_File_Name,
                                    Custom_Structure_T962*          _pxCustom, 
                                    void*                           _pDlgThis )
{
	ESuccess            eReturnStatus = Ok ;

    ESuccess            eSuccess ;
    Parse_Structure     xParse ;

    CString             oMsg ;

    //\\//

    // write to console 
	oConsole.WriteLine ( "" ) ;
	oConsole.WriteLine ( (CString) "%% Register IO Command File : "
                        		 +  _osCommand_File_Name ) ;
    
    // Initialize the Command File Parser control Structure
    eSuccess = Init_Parse_Structure ( &xParse ) ;

    xParse.pacFile_Name = LPCTSTR(_osCommand_File_Name) ;

    //register the legal keywords and 
    //the call back points for the apropriate keywords
    xParse.Actions.pfT962_Wire        = (fKeyword_Action*) T962_Action_Parse_Wire ;
    xParse.Actions.pfT962_Maestro_Slot= (fKeyword_Action*) T962_Action_Parse_Maestro_Slot  ;
    xParse.Actions.pfCrate            = NULL ;
    xParse.Actions.pfCard_Slot        = NULL ;
    xParse.Actions.pfChip_Address     = NULL ;
    xParse.Actions.pfRegister_Start   = NULL ;
    xParse.Actions.pfRegister_Count   = (fKeyword_Action*) T962_Action_Parse_Register_Count ;
    
    // Initialize Total Number of register IO performed
    _pxCustom->uiTotWire = 0 ;

    // call the general purpose command file parser 
    eSuccess = Read_Command_File ( &xParse, _pxCustom, _pDlgThis ) ;

    if ( eSuccess == Ok  )
    {
        oMsg = "%%Done with T962 Config after Registering "
             + I_to_A ( _pxCustom->uiTotWire )
             + " Wires" ;
        oConsole.SystemLine ( oMsg ) ;
        oConsole.WriteLine ( "" ) ;
//        delete CT962::mg_poT962 ;  // temporary debug to watch all registered objects
    }
    else
    {
        oMsg = "%%Abort T962 Config after Registering "
             + I_to_A ( _pxCustom->uiTotWire )
             + " Wires" ;
        oConsole.ErrorLine ( oMsg ) ;
    	eReturnStatus = Error ;
    }

    return eReturnStatus ;
}


/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/


/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess T962_Action_Parse_Maestro_Slot           ( void* _pxParse, 
                                                    void* _pxCustom,
                                                    void* _pDlgThis )
{
    Parse_Structure*                pxParse     = (Parse_Structure*)                _pxParse ;
    Custom_Structure_T962*          pxCustom    = (Custom_Structure_T962*)         _pxCustom ;

    ESuccess        lReturnStatus = Ok ;

    //\\//

    // check all other keywords have been assigned
    if ( ( pxParse->Values.ulCrate            == diNotYetAssigned ) ) 
    {
        oConsole.ErrorLine ( (CString) "_ Missing Keyword before assignment at Line #"
                                      + I_to_A ( pxParse->iLineNum ) ) ;
	    lReturnStatus = Error ;
	    goto Done ;
    }

    // consistency check for crate number
    if ( pxParse->Values.ulCrate            > 1 ) 
    {
        oConsole.ErrorLine ( (CString) "_ Crate Number out of range at Line #"
                                      + I_to_A ( pxParse->iLineNum ) ) ;
	    lReturnStatus = Error ;
	    goto Done ;
    }

    // Register the Maestro Card
    if ( CT962::mg_poT962->Set_Maestro ( fubTruncToByte( pxParse->Values.ulCrate ),
                                         fubTruncToByte( pxParse->Values.ulT962_Maestro_Slot ) )  != Ok )
    {
        oConsole.ErrorLine ( (CString) "_ T962 Wire Regsitration Error at Line #"
                                      + I_to_A ( pxParse->iLineNum ) ) ;
	    lReturnStatus = Error ;
	    goto Done ;
    }

Done:
    return lReturnStatus ;
}


/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess T962_Action_Parse_Wire                   ( void* _pxParse, 
                                                    void* _pxCustom,
                                                    void* _pDlgThis )
{
    Parse_Structure*                pxParse     = (Parse_Structure*)               _pxParse ;
    Custom_Structure_T962*          pxCustom    = (Custom_Structure_T962*)         _pxCustom ;

    ESuccess        lReturnStatus = Ok ;

    //\\//

    // clear all other keywords 
    pxParse->Values.ulCrate            = diNotYetAssigned ;
    pxParse->Values.ulCard_Slot        = diNotYetAssigned ; 
    pxParse->Values.ulChip_Address     = diNotYetAssigned ;
    pxParse->Values.ulRegister_Start   = diNotYetAssigned ;
    pxParse->Values.ulRegister_Count   = diNotYetAssigned ;

    return lReturnStatus ;
}



/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess T962_Action_Parse_Register_Count         ( void* _pxParse, 
                                                    void* _pxCustom,
                                                    void* _pDlgThis )
{
    Parse_Structure*                pxParse      = (Parse_Structure*)               _pxParse ;
    Custom_Structure_T962*          pxCustom     = (Custom_Structure_T962*)         _pxCustom ;

    ESuccess        lReturnStatus = Ok ;

    //\\//

    // check all other keywords have been assigned
    if ( ( pxParse->Values.ulT962_Wire        == diNotYetAssigned ) 
      || ( pxParse->Values.ulCrate            == diNotYetAssigned ) 
      || ( pxParse->Values.ulCard_Slot        == diNotYetAssigned ) 
      || ( pxParse->Values.ulChip_Address     == diNotYetAssigned ) 
      || ( pxParse->Values.ulRegister_Start   == diNotYetAssigned ) ) 
    {
        oConsole.ErrorLine ( (CString) "_ Missing Keywords before assignment at Line #"
                                      + I_to_A ( pxParse->iLineNum ) ) ;
	    lReturnStatus = Error ;
	    goto Done ;
    }

    // consistency check for crate number
    if ( pxParse->Values.ulCrate            > 1 ) 
    {
        oConsole.ErrorLine ( (CString) "_ Crate Number out of range at Line #"
                                      + I_to_A ( pxParse->iLineNum ) ) ;
	    lReturnStatus = Error ;
	    goto Done ;
    }


    // Register the wire
    if ( CT962::mg_poT962->Add_Wire   ( pxParse->Values.ulT962_Wire,
                                        fubTruncToByte( pxParse->Values.ulCrate ),
                                        fubTruncToByte( pxParse->Values.ulCard_Slot ),
                                        fubTruncToByte( pxParse->Values.ulChip_Address ),
                                        fuwTruncToWord( pxParse->Values.ulRegister_Start ),
                                        pxParse->Values.ulRegister_Count ) != Ok )
    {
        oConsole.ErrorLine ( (CString) "_ T962 Wire Regsitration Error at Line #"
                                      + I_to_A ( pxParse->iLineNum ) ) ;
	    lReturnStatus = Error ;
	    goto Done ;
    }
    else 
    {
        // Increment Total Number wires registered
        pxCustom->uiTotWire++ ;
    }
     

Done:
    return lReturnStatus ;
}



/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
