/*-------------------------------------------------------------------------------*/
  /*
    CommandFileMaster.cpp
    10-Sep-99
  */
/*-------------------------------------------------------------------------------*/
#include "L1Cal_IIb_TCC/StdAfx.h"

#include "L1Cal_IIb_TCC/GlobalConstants.h"
#include "L1Cal_IIb_TCC/CustomDataTypes.h"

#include "L1Cal_IIb_TCC/HandleCommandFiles.h"
#include "L1Cal_IIb_TCC/CommandFileRegisterIO.h"
#include "L1Cal_IIb_TCC/CommandFileSerialIO.h"
#include "L1Cal_IIb_TCC/CommandFileVmeIO.h"
#include "L1Cal_IIb_TCC/CommandFileConfigFpga.h"
#include "L1Cal_IIb_TCC/CommandFileSelfMsg.h"
#include "L1Cal_IIb_TCC/CommandFileTrigTwrInfo.h"
#include "L1Cal_IIb_TCC/CommandFileConfigT962.h"

#include                            "L1Cal_IIb_TCC/CommandFileMaster.h"

#include "tabgab/tcc_to_tabgab_env.h"

#include "L1Cal_IIb_TCC/L1Cal.h"

#include "L1Cal_IIb_TCC/MonitControl.h"

#include "L1Cal_IIb_TCC/UtilsStrings.h"
#include "L1Cal_IIb_TCC/UtilsBits.h"

// static means created only once throughout life of program (to avoid wasting time)
// *AND* internal linkage (to avoid clash between modules)
#include "L1Cal_IIb_TCC/HandleConsole.h"
static CConsoleAccess oConsole ;

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess CommandFileMaster        ( CString	                        _osCommand_File_Name,
                                    Custom_Structure_Master*        _pxCustom, 
                                    void*                           _pDlgThis )
{
	ESuccess            eReturnStatus = Ok ;

    ESuccess            eSuccess ;
    Parse_Structure     xParse ;

    CString             oMsg ;

    //\\//

    // write to console 
	oConsole.WriteLine ( "" ) ;

	oConsole.WriteLine ( (CString) "%%% Master Command File : "
                        		 +  _osCommand_File_Name ) ;
    
    // Initialize the Command File Parser control Structure
    eSuccess = Init_Parse_Structure ( &xParse ) ;

    xParse.pacFile_Name = LPCTSTR(_osCommand_File_Name) ;

    //register the legal keywords and 
    //the call back points for the apropriate keywords
    xParse.Actions.pfCall_Reg_IO        = (fKeyword_Action*) Master_Action_Parse_Call_Reg_IO  ;
    xParse.Actions.pfCall_Serial_IO     = (fKeyword_Action*) Master_Action_Parse_Call_Serial_IO  ;
    xParse.Actions.pfCall_Config_Fpga   = (fKeyword_Action*) Master_Action_Parse_Call_Config_Fpga  ;
    xParse.Actions.pfCall_TT_Info       = (fKeyword_Action*) Master_Action_Parse_Call_TT_Info  ;
    xParse.Actions.pfCall_Config_T962   = (fKeyword_Action*) Master_Action_Parse_Call_Config_T962 ;
    xParse.Actions.pfCall_Vme_IO        = (fKeyword_Action*) Master_Action_Parse_Call_Vme_IO  ;
    xParse.Actions.pfCall_Self_Msg      = (fKeyword_Action*) Master_Action_Parse_Call_Self_Msg  ;

    // we also allow direct Self Message Commands
    xParse.Actions.pfSend_Msg_To_Self   = (fKeyword_Action*) SelfMsg_Action_Parse_Send_Msg_To_Self ;

    // and high level directives 
    xParse.Actions.pfL1Cal_Operational              = (fKeyword_Action*) Master_Action_Parse_L1Cal_Operational ;
    xParse.Actions.pfMonitControlTimeMs             = (fKeyword_Action*) Master_Action_Parse_MonitControlTimeMs ;
    xParse.Actions.pfBaseAlignmentDelay             = (fKeyword_Action*) Master_Action_Parse_BaseAlignmentDelay ;
    xParse.Actions.pfOffsetTick1RawAdcMonit         = (fKeyword_Action*) Master_Action_Parse_OffsetTick1RawAdcMonit ;
    xParse.Actions.pfOffsetTick1RawAdcSimu          = (fKeyword_Action*) Master_Action_Parse_OffsetTick1RawAdcSimu ;
    xParse.Actions.pfOffsetTick1OutputEtMonit       = (fKeyword_Action*) Master_Action_Parse_OffsetTick1OutputEtMonit ;
    xParse.Actions.pfOffsetTick1OutputEtSimu        = (fKeyword_Action*) Master_Action_Parse_OffsetTick1OutputEtSimu ;
    xParse.Actions.pfOffsetTriggeredToTickAtCapture = (fKeyword_Action*) Master_Action_Parse_OffsetTriggeredToTickAtCapture ;
    xParse.Actions.pfMaxDeSkewTryAtInit             = (fKeyword_Action*) Master_Action_Parse_MaxDeSkewTryAtInit ;
    xParse.Actions.pfAbortInitOnError               = (fKeyword_Action*) Master_Action_Parse_AbortInitOnError ;
    xParse.Actions.pfAbortInitOnError               = (fKeyword_Action*) Master_Action_Parse_AbortInitOnError ;
    xParse.Actions.pfSkipTabGabOnInit               = (fKeyword_Action*) Master_Action_Parse_SkipTabGabOnInit ;
    xParse.Actions.pfMonitIgnoreTabGab              = (fKeyword_Action*) Master_Action_Parse_MonitIgnoreTabGab ;
    xParse.Actions.pfTtiImmediateIo                 = (fKeyword_Action*) Master_Action_Parse_TtiImmediateIo ;
    xParse.Actions.pfFindDacMode                    = (fKeyword_Action*) Master_Action_Parse_FindDacMode ;
    xParse.Actions.pfTabEta38Value                  = (fKeyword_Action*) Master_Action_Parse_TabEta38Value ;

    xParse.Actions.pfTabGab_PackageVersion          = (fKeyword_Action*) Master_Action_Parse_TabGab_PackageVersion ;
    xParse.Actions.pfTabGab_FirmwareDir             = (fKeyword_Action*) Master_Action_Parse_TabGab_FirmwareDir ;
    xParse.Actions.pfTabGab_ConfigDir               = (fKeyword_Action*) Master_Action_Parse_TabGab_ConfigDir ;
    xParse.Actions.pfTabGab_WorkDir                 = (fKeyword_Action*) Master_Action_Parse_TabGab_WorkDir ;

    // Initialize Total Number of register IO performed
    _pxCustom->uiTotCommand = 0 ;

    // call the general purpose command file parser 
    eSuccess = Read_Command_File ( &xParse, _pxCustom, NULL ) ;

	oConsole.WriteLine ( "" ) ;

    if ( eSuccess == Ok  )
    {
        oMsg = "%%%Done after Executing   "
             + I_to_A ( _pxCustom->uiTotCommand  )
             + " Master File Commands" ;
        oConsole.WriteLine ( oMsg ) ;
    }
    else
    {
        oMsg = "%%%Abort after Executing   "
             + I_to_A ( _pxCustom->uiTotCommand  )
             + " Master File Commands" ;
        oConsole.ErrorLine ( oMsg ) ;
    	eReturnStatus = Error ;
    }

    return eReturnStatus ;
}


/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess Master_Action_Parse_Call_Reg_IO          ( void* _pxParse, 
                                                    void* _pxCustom,
                                                    void* _pDlgThis )
{
    Parse_Structure*                pxParse         = (Parse_Structure*)                _pxParse ;
    Custom_Structure_Master*        pxCustom_Master = (Custom_Structure_Master*)        _pxCustom ;

    Custom_Structure_RegIO          xCustom_RegIO ;

    ESuccess        eSuccess ;
    ESuccess        eReturnStatus = Ok ;

    //\\//

    // check all other keywords have been assigned
    // Does not apply here

    // Increment Total Number of register IO performed
    pxCustom_Master->uiTotCommand++ ;

    //execute the Register IO command file
    eSuccess = CommandFileRegisterIO ( pxParse->Values.osGeneric_String_Parameter, &xCustom_RegIO, NULL ) ;
    if ( eSuccess != Ok ) eReturnStatus = eSuccess ;

//Done:

    return eReturnStatus ;
}


/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess Master_Action_Parse_Call_Serial_IO       ( void* _pxParse, 
                                                    void* _pxCustom,
                                                    void* _pDlgThis )
{
    Parse_Structure*                pxParse         = (Parse_Structure*)                _pxParse ;
    Custom_Structure_Master*        pxCustom_Master = (Custom_Structure_Master*)        _pxCustom ;

    Custom_Structure_SerIO          xCustom_SerIO ;

    ESuccess        eSuccess ;
    ESuccess        eReturnStatus = Ok ;

    //\\//

    // check all other keywords have been assigned
    // Does not apply here

    // Increment Total Number of register IO performed
    pxCustom_Master->uiTotCommand++ ;

    //execute the Register IO command file
    eSuccess = CommandFileSerialIO ( pxParse->Values.osGeneric_String_Parameter, &xCustom_SerIO, NULL ) ;
    if ( eSuccess != Ok ) eReturnStatus = eSuccess ;

//Done:

    return eReturnStatus ;
}


/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess Master_Action_Parse_Call_Vme_IO          ( void* _pxParse, 
                                                    void* _pxCustom,
                                                    void* _pDlgThis )
{
    Parse_Structure*                pxParse         = (Parse_Structure*)                _pxParse ;
    Custom_Structure_Master*        pxCustom_Master = (Custom_Structure_Master*)        _pxCustom ;

    Custom_Structure_VmeIO          xCustom_VmeIO ;

    ESuccess        eSuccess ;
    ESuccess        eReturnStatus = Ok ;

    //\\//

    // check all other keywords have been assigned
    // Does not apply here

    // Increment Total Number of register IO performed
    pxCustom_Master->uiTotCommand++ ;

    //execute the Register IO command file
    eSuccess = CommandFileVmeIO ( pxParse->Values.osGeneric_String_Parameter, &xCustom_VmeIO, NULL ) ;
    if ( eSuccess != Ok ) eReturnStatus = eSuccess ;

//Done:

    return eReturnStatus ;
}


/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess Master_Action_Parse_Call_Config_Fpga     ( void* _pxParse, 
                                                    void* _pxCustom,
                                                    void* _pDlgThis )
{
    Parse_Structure*                pxParse         = (Parse_Structure*)                _pxParse ;
    Custom_Structure_Master*        pxCustom_Master = (Custom_Structure_Master*)        _pxCustom ;

    Custom_Structure_ConfigFpga     xCustom_ConfigFpga ;

    ESuccess        eSuccess ;
    ESuccess        eReturnStatus = Ok ;

    //\\//

    // check all other keywords have been assigned
    // Does not apply here

    // Increment Total Number of register IO performed
    pxCustom_Master->uiTotCommand++ ;

    //execute the Register IO command file
    eSuccess = CommandFileConfigFpga ( pxParse->Values.osGeneric_String_Parameter, &xCustom_ConfigFpga, NULL ) ;
    if ( eSuccess != Ok ) eReturnStatus = eSuccess ;

//Done:

    return eReturnStatus ;
}


/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess Master_Action_Parse_Call_Self_Msg        ( void* _pxParse, 
                                                    void* _pxCustom,
                                                    void* _pDlgThis )
{
    Parse_Structure*                pxParse         = (Parse_Structure*)                _pxParse ;
    Custom_Structure_Master*        pxCustom_Master = (Custom_Structure_Master*)        _pxCustom ;

    Custom_Structure_SelfMsg        xCustom_SelfMsg ;

    ESuccess        eSuccess ;
    ESuccess        eReturnStatus = Ok ;

    //\\//

    // check all other keywords have been assigned
    // Does not apply here

    // Increment Total Number of register IO performed
    pxCustom_Master->uiTotCommand++ ;

    //execute the Self Message command file
    eSuccess = CommandFileSelfMsg ( pxParse->Values.osGeneric_String_Parameter, &xCustom_SelfMsg, NULL ) ;
    if ( eSuccess != Ok ) eReturnStatus = eSuccess ;

//Done:

    return eReturnStatus ;
}


/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess Master_Action_Parse_Call_TT_Info         ( void* _pxParse, 
                                                    void* _pxCustom,
                                                    void* _pDlgThis )
{
    Parse_Structure*                pxParse         = (Parse_Structure*)                _pxParse ;
    Custom_Structure_Master*        pxCustom_Master = (Custom_Structure_Master*)        _pxCustom ;

    Custom_Structure_TT_Info        xCustom_TT_Info ;

    ESuccess        eSuccess ;
    ESuccess        eReturnStatus = Ok ;

    //\\//

    // check all other keywords have been assigned
    // Does not apply here

    // Increment Total Number of register IO performed
    pxCustom_Master->uiTotCommand++ ;

    //execute the Trigger Tower Info command file
    eSuccess = CommandFileTrigTwrInfo ( pxParse->Values.osGeneric_String_Parameter, &xCustom_TT_Info, NULL ) ;
    if ( eSuccess != Ok ) eReturnStatus = eSuccess ;

//Done:

    return eReturnStatus ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess Master_Action_Parse_Call_Config_T962     ( void* _pxParse, 
                                                    void* _pxCustom,
                                                    void* _pDlgThis )
{
    Parse_Structure*                pxParse         = (Parse_Structure*)                _pxParse ;
    Custom_Structure_Master*        pxCustom_Master = (Custom_Structure_Master*)        _pxCustom ;

    Custom_Structure_T962        xCustom_T962 ;

    ESuccess        eSuccess ;
    ESuccess        eReturnStatus = Ok ;

    //\\//

    // check all other keywords have been assigned
    // Does not apply here

    // Increment Total Number of register IO performed
    pxCustom_Master->uiTotCommand++ ;

    //execute the Trigger Tower Info command file
    eSuccess = CommandFileConfigT962 ( pxParse->Values.osGeneric_String_Parameter, &xCustom_T962, NULL ) ;
    if ( eSuccess != Ok ) eReturnStatus = eSuccess ;

//Done:

    return eReturnStatus ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess Master_Action_Parse_L1Cal_Operational    ( void* _pxParse, 
                                                    void* _pxCustom,
                                                    void* _pDlgThis )
{
    Parse_Structure*                pxParse         = (Parse_Structure*)                _pxParse ;
    Custom_Structure_Master*        pxCustom_Master = (Custom_Structure_Master*)        _pxCustom ;

    ESuccess        eReturnStatus = Ok ;

    //\\//

    //Check the Value is within maximum
    if ( pxParse->Values.ulL1Cal_Operational > 1 ) 
    {
        oConsole.ErrorLine ( (CString) "_ <L1Cal_Operational:> keyword only allows values 0 or 1 at Line #"
                                      + I_to_A ( pxParse->iLineNum ) ) ;
	    eReturnStatus = Error ;
	    goto Done ;
    }

    // Increment Total Number of commands executed
    pxCustom_Master->uiTotCommand++ ;


	if ( pxParse->Values.ulL1Cal_Operational == 1 )
    {
        oConsole.SystemLine ( (CString) " L1Cal Declared Operational" ) ;
        CL1Cal::mg_eynL1CalOperational = Yes ;
        oConsole.SystemLine ( (CString) " Trigger Tower Coverage is "
                                      + CL1Cal::mg_poL1Cal->f_osCoverage() ) ;
    }
    else 
    {
        oConsole.SystemLine ( (CString) " L1Cal Declared NON-Operational" ) ;
        CL1Cal::mg_eynL1CalOperational = No ;
    }


Done:

    return eReturnStatus ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess Master_Action_Parse_MonitControlTimeMs   ( void* _pxParse, 
                                                    void* _pxCustom,
                                                    void* _pDlgThis )
{
    Parse_Structure*                pxParse         = (Parse_Structure*)                _pxParse ;
    Custom_Structure_Master*        pxCustom_Master = (Custom_Structure_Master*)        _pxCustom ;

    ESuccess        eReturnStatus = Ok ;

    unsigned long   ulTime_ms ;

    //\\//

    // Increment Total Number of commands executed
    pxCustom_Master->uiTotCommand++ ;

    // retrieve the value specified
    ulTime_ms = pxParse->Values.ulMonitControlTimeMs  ;

    // Set the new value 
    if ( ulTime_ms == 0 ) 
    {
        oConsole.SystemLine ( (CString) " Disabling Monitoring Data Collection" ) ;
        CMonitControl::Set_Monit_Server_Silenced ( Yes ) ;
    }
    else if ( ulTime_ms >= KiMonitControlTimeMsMinimum ) 
    {
        oConsole.SystemLine ( (CString) " Setting Monitoring Data Collection Control Time to "
                                        + I_to_A(ulTime_ms) 
                                        + " ms" ) ;
        CMonitControl::Set_Monit_Server_Time     ( ulTime_ms ) ;
        CMonitControl::Set_Monit_Server_Silenced ( No ) ;
    }
    else // user probably didn't mean less than one second
    {
        oConsole.ErrorLine ( (CString) " Monit Control Time too small at Line #"
                                      + I_to_A ( pxParse->iLineNum ) ) ;
	    eReturnStatus = Error ;
    }

    return eReturnStatus ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess Master_Action_Parse_BaseAlignmentDelay   ( void* _pxParse, 
                                                    void* _pxCustom,
                                                    void* _pDlgThis )
{
    Parse_Structure*                pxParse         = (Parse_Structure*)                _pxParse ;
    Custom_Structure_Master*        pxCustom_Master = (Custom_Structure_Master*)        _pxCustom ;

    ESuccess        eReturnStatus = Ok ;

    //\\//

    //Check the Value is within maximum
    if ( pxParse->Values.ulBaseAlignmentDelay > KiAdfMaxAlignmentDelay ) 
    {
        oConsole.ErrorLine ( (CString) "_ BaseAlignmentDelay value out range at Line #"
                                      + I_to_A ( pxParse->iLineNum ) ) ;
	    eReturnStatus = Error ;
	    goto Done ;
    }

    // Increment Total Number of commands executed
    pxCustom_Master->uiTotCommand++ ;

    // Set the new value 
    CL1Cal::mg_ubBaseAlignDelay = pxParse->Values.ulBaseAlignmentDelay ;
    oConsole.SystemLine ( (CString) " Set the L1Cal Trigger Tower Base Alignment Delay = " 
                                   + I_to_A ( CL1Cal::mg_ubBaseAlignDelay ) ) ;
Done:
    return eReturnStatus ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess Master_Action_Parse_OffsetTick1RawAdcMonit   ( void* _pxParse, 
                                                        void* _pxCustom,
                                                        void* _pDlgThis )
    {
    Parse_Structure*                pxParse         = (Parse_Structure*)                _pxParse ;
    Custom_Structure_Master*        pxCustom_Master = (Custom_Structure_Master*)        _pxCustom ;

    ESuccess        eReturnStatus = Ok ;

    //\\//

    //Check the Value is within maximum
    if ( pxParse->Values.ulOffsetTick1RawAdcMonit >= KiTotSamplesPerTick*KiTotTicksInTurn ) 
    {
        oConsole.ErrorLine ( (CString) "_ OffsetTick1RawAdcMonit value out range at Line #"
                                      + I_to_A ( pxParse->iLineNum ) ) ;
	    eReturnStatus = Error ;
	    goto Done ;
    }

    // Increment Total Number of commands executed
    pxCustom_Master->uiTotCommand++ ;

    // Set the new value 
    CL1Cal::mg_uwOffsetTick1RawAdcMonit = pxParse->Values.ulOffsetTick1RawAdcMonit ;
    oConsole.SystemLine ( (CString) " Set the Offset of Tick#1 SubTick#0 "
                                   + "for reading Raw ADC Monitoring Data = " 
                                   + I_to_A ( CL1Cal::mg_uwOffsetTick1RawAdcMonit ) ) ;
Done:
    return eReturnStatus ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess Master_Action_Parse_OffsetTick1RawAdcSimu    ( void* _pxParse, 
                                                        void* _pxCustom,
                                                        void* _pDlgThis )
    {
    Parse_Structure*                pxParse         = (Parse_Structure*)                _pxParse ;
    Custom_Structure_Master*        pxCustom_Master = (Custom_Structure_Master*)        _pxCustom ;

    ESuccess        eReturnStatus = Ok ;

    //\\//

    //Check the Value is within maximum
    if ( pxParse->Values.ulOffsetTick1RawAdcSimu >= KiTotSamplesPerTick*KiTotTicksInTurn ) 
    {
        oConsole.ErrorLine ( (CString) "_ OffsetTick1RawAdcSimu value out range at Line #"
                                      + I_to_A ( pxParse->iLineNum ) ) ;
	    eReturnStatus = Error ;
	    goto Done ;
    }

    // Increment Total Number of commands executed
    pxCustom_Master->uiTotCommand++ ;

    // Set the new value 
    CL1Cal::mg_uwOffsetTick1RawAdcSimu = pxParse->Values.ulOffsetTick1RawAdcSimu ;
    oConsole.SystemLine ( (CString) " Set the Offset of Tick#1 SubTick#0 "
                                   + "for writing Raw ADC Simulation Data = " 
                                   + I_to_A ( CL1Cal::mg_uwOffsetTick1RawAdcSimu ) ) ;
Done:
    return eReturnStatus ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess Master_Action_Parse_OffsetTick1OutputEtMonit ( void* _pxParse, 
                                                        void* _pxCustom,
                                                        void* _pDlgThis )
{
    Parse_Structure*                pxParse         = (Parse_Structure*)                _pxParse ;
    Custom_Structure_Master*        pxCustom_Master = (Custom_Structure_Master*)        _pxCustom ;

    ESuccess        eReturnStatus = Ok ;

    //\\//

    //Check the Value is within maximum
    if ( pxParse->Values.ulOffsetTick1OutputEtMonit >= KiTotTicksInTurn ) 
    {
        oConsole.ErrorLine ( (CString) "_ OffsetTick1OutputEtMonit value out range at Line #"
                                      + I_to_A ( pxParse->iLineNum ) ) ;
	    eReturnStatus = Error ;
	    goto Done ;
    }

    // Increment Total Number of commands executed
    pxCustom_Master->uiTotCommand++ ;

    // Set the new value 
    CL1Cal::mg_uwOffsetTick1OutputEtMonit = pxParse->Values.ulOffsetTick1OutputEtMonit ;
    oConsole.SystemLine ( (CString) " Set the Offset of Tick#1 "
                                   + "for reading Output Et Monitoring Data = " 
                                   + I_to_A ( CL1Cal::mg_uwOffsetTick1OutputEtMonit ) ) ;
Done:
    return eReturnStatus ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess Master_Action_Parse_OffsetTick1OutputEtSimu  ( void* _pxParse, 
                                                        void* _pxCustom,
                                                        void* _pDlgThis )
{
    Parse_Structure*                pxParse         = (Parse_Structure*)                _pxParse ;
    Custom_Structure_Master*        pxCustom_Master = (Custom_Structure_Master*)        _pxCustom ;

    ESuccess        eReturnStatus = Ok ;

    //\\//

    //Check the Value is within maximum
    if ( pxParse->Values.ulOffsetTick1OutputEtSimu >= KiTotTicksInTurn ) 
    {
        oConsole.ErrorLine ( (CString) "_ OffsetTick1OutputEtSimu value out range at Line #"
                                      + I_to_A ( pxParse->iLineNum ) ) ;
	    eReturnStatus = Error ;
	    goto Done ;
    }

    // Increment Total Number of commands executed
    pxCustom_Master->uiTotCommand++ ;

    // Set the new value 
    CL1Cal::mg_uwOffsetTick1OutputEtSimu = pxParse->Values.ulOffsetTick1OutputEtSimu ;
    oConsole.SystemLine ( (CString) " Set the Offset of Tick#1 "
                                   + "for writing Output Et Simulation Data = " 
                                   + I_to_A ( CL1Cal::mg_uwOffsetTick1OutputEtSimu ) ) ;
Done:
    return eReturnStatus ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess Master_Action_Parse_OffsetTriggeredToTickAtCapture   ( void* _pxParse, 
                                                                void* _pxCustom,
                                                                void* _pDlgThis )
{
    Parse_Structure*                pxParse         = (Parse_Structure*)                _pxParse ;
    Custom_Structure_Master*        pxCustom_Master = (Custom_Structure_Master*)        _pxCustom ;

    ESuccess        eReturnStatus = Ok ;

    //\\//

    //Check the Value is within maximum
    if ( pxParse->Values.ulOffsetTriggeredToTickAtCapture >= KiTotTicksInTurn ) 
    {
        oConsole.ErrorLine ( (CString) "_ OffsetTriggeredToTickAtCapture value out range at Line #"
                                      + I_to_A ( pxParse->iLineNum ) ) ;
	    eReturnStatus = Error ;
	    goto Done ;
    }

    // Increment Total Number of commands executed
    pxCustom_Master->uiTotCommand++ ;

    // Set the new value 
    CL1Cal::mg_uwOffsetTriggeredToTickAtCapture = pxParse->Values.ulOffsetTriggeredToTickAtCapture ;
    oConsole.SystemLine ( (CString) " Set the Triggered to Tick at Capture Offset = " 
                                   + I_to_A ( CL1Cal::mg_uwOffsetTriggeredToTickAtCapture ) ) ;
Done:
    return eReturnStatus ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess Master_Action_Parse_TtiImmediateIo       ( void* _pxParse, 
                                                    void* _pxCustom,
                                                    void* _pDlgThis )
{
    Parse_Structure*                pxParse         = (Parse_Structure*)                _pxParse ;
    Custom_Structure_Master*        pxCustom_Master = (Custom_Structure_Master*)        _pxCustom ;

    ESuccess        eReturnStatus = Ok ;

    //\\//

    // check all other keywords have been assigned
    // Does not apply here

    //Check the Value is within maximum
    if ( pxParse->Values.ulTtiImmediateIo > 1 ) 
    {
        oConsole.ErrorLine ( (CString) "_ <TtiImmediateIo:> keyword only allows values 0 or 1 at Line #"
                                      + I_to_A ( pxParse->iLineNum ) ) ;
	    eReturnStatus = Error ;
	    goto Done ;
    }

    // Increment Total Number of register IO performed
    pxCustom_Master->uiTotCommand++ ;


	if ( pxParse->Values.ulTtiImmediateIo == 1 )
    {
        oConsole.SystemLine ( (CString) " TTI Command Files will do immediate IO" ) ;
        CL1Cal::mg_eynTtiImmediateIo = Yes ;
    }
    else 
    {
        oConsole.SystemLine ( (CString) " TTI Command Files will only remember values for next Initialize" ) ;
        CL1Cal::mg_eynTtiImmediateIo = No ;
    }


Done:

    return eReturnStatus ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess Master_Action_Parse_MaxDeSkewTryAtInit   ( void* _pxParse, 
                                                    void* _pxCustom,
                                                    void* _pDlgThis )
{
    Parse_Structure*                pxParse         = (Parse_Structure*)                _pxParse ;
    Custom_Structure_Master*        pxCustom_Master = (Custom_Structure_Master*)        _pxCustom ;

    ESuccess        eReturnStatus = Ok ;

    //\\//

    // check all other keywords have been assigned
    // Does not apply here

    //Check the Value is within maximum
    if ( pxParse->Values.ulMaxDeSkewTryAtInit > 50 ) 
    {
        oConsole.ErrorLine ( (CString) "_ <MaxDeSkewTryAtInit:> keyword only allows values 0:50 Line #"
                                      + I_to_A ( pxParse->iLineNum ) ) ;
	    eReturnStatus = Error ;
	    goto Done ;
    }

    // Increment Total Number of register IO performed
    pxCustom_Master->uiTotCommand++ ;

    CL1Cal::mg_ubMaxDeSkewTryAtInit = fubTruncToByte( pxParse->Values.ulMaxDeSkewTryAtInit ) ;

	if ( CL1Cal::mg_ubMaxDeSkewTryAtInit == 0 )
        oConsole.SystemLine ( (CString) " Initialization will skip the iterative de-skewing and verification step" ) ;
    else 
        oConsole.SystemLine ( (CString) " Initialization will perform up to "
                                        + I_to_A ( CL1Cal::mg_ubMaxDeSkewTryAtInit ) 
                                        + " iterative de-skews with final verification" ) ;


Done:

    return eReturnStatus ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess Master_Action_Parse_AbortInitOnError     ( void* _pxParse, 
                                                    void* _pxCustom,
                                                    void* _pDlgThis )
{
    Parse_Structure*                pxParse         = (Parse_Structure*)                _pxParse ;
    Custom_Structure_Master*        pxCustom_Master = (Custom_Structure_Master*)        _pxCustom ;

    ESuccess        eReturnStatus = Ok ;

    //\\//

    // check all other keywords have been assigned
    // Does not apply here

    //Check the Value is within maximum
    if ( pxParse->Values.ulAbortInitOnError > 1 ) 
    {
        oConsole.ErrorLine ( (CString) "_ <AbortInitOnError:> keyword only allows values 0 or 1 at Line #"
                                      + I_to_A ( pxParse->iLineNum ) ) ;
	    eReturnStatus = Error ;
	    goto Done ;
    }

    // Increment Total Number of register IO performed
    pxCustom_Master->uiTotCommand++ ;


	if ( pxParse->Values.ulAbortInitOnError == 1 )
    {
        oConsole.SystemLine ( (CString) " Initialization will Abort when it encounters an error" ) ;
        CL1Cal::mg_eynAbortInitOnError = Yes ;
    }
    else 
    {
        oConsole.SystemLine ( (CString) " Initialization will NOT Abort when it encounters an error" ) ;
        CL1Cal::mg_eynAbortInitOnError = No ;
    }


Done:

    return eReturnStatus ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess Master_Action_Parse_FindDacMode          ( void* _pxParse, 
                                                    void* _pxCustom,
                                                    void* _pDlgThis )
{
    Parse_Structure*                pxParse         = (Parse_Structure*)                _pxParse ;
    Custom_Structure_Master*        pxCustom_Master = (Custom_Structure_Master*)        _pxCustom ;

    ESuccess        eReturnStatus = Ok ;

    //\\//

    // check all other keywords have been assigned
    // Does not apply here

    //Check the Value is within maximum
    if ( pxParse->Values.ulFindDacMode >= eTot_FindDacMode_Type ) 
    {
        oConsole.ErrorLine ( (CString) "_ <FindDacMode:> keyword value out of range at Line #"
                                      + I_to_A ( pxParse->iLineNum ) ) ;
	    eReturnStatus = Error ;
	    goto Done ;
    }

    // Increment Total Number of register IO performed
    pxCustom_Master->uiTotCommand++ ;

    // update value 
    CL1Cal::mg_eFindDacMode = (EFindDacMode_Type) pxParse->Values.ulFindDacMode ;

    // display action
	if (  CL1Cal::mg_eFindDacMode == eFindDacMode_All636 )
        oConsole.SystemLine ( (CString) " Find_DAC will histogram random Raw ADC samples" ) ;

    else if ( CL1Cal::mg_eFindDacMode == eFindDacMode_LiveX )
        oConsole.SystemLine ( (CString) " Find_DAC will histogram only Raw ADC samples used for Live Crossings" ) ;
    
    else 
        oConsole.SystemLine ( (CString) " Find_DAC will use Mode #" 
                                        + I_to_A ( pxParse->Values.ulFindDacMode ) ) ;

Done:

    return eReturnStatus ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess Master_Action_Parse_TabEta38Value        ( void* _pxParse, 
                                                    void* _pxCustom,
                                                    void* _pDlgThis )
{
    Parse_Structure*                pxParse         = (Parse_Structure*)                _pxParse ;
    Custom_Structure_Master*        pxCustom_Master = (Custom_Structure_Master*)        _pxCustom ;

    ESuccess        eReturnStatus = Ok ;

    //\\//

    // check all other keywords have been assigned
    // Does not apply here

    //Check the Value is within maximum
    if ( pxParse->Values.ulTabEta38Value > 255 ) 
    {
        oConsole.ErrorLine ( (CString) "_ <TabEta38Value:> keyword value out of range at Line #"
                                      + I_to_A ( pxParse->iLineNum ) ) ;
	    eReturnStatus = Error ;
	    goto Done ;
    }

    // Increment Total Number of register IO performed
    pxCustom_Master->uiTotCommand++ ;

    // update value 
    CL1Cal::mg_ulTabEta38Value = pxParse->Values.ulTabEta38Value ;

    // display action
    oConsole.SystemLine ( (CString) " Tab_Eta38 Value will be initialized using the value " 
                                    + I_to_A ( pxParse->Values.ulTabEta38Value ) ) ;

Done:

    return eReturnStatus ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess Master_Action_Parse_SkipTabGabOnInit     ( void* _pxParse, 
                                                    void* _pxCustom,
                                                    void* _pDlgThis )
{
    Parse_Structure*                pxParse         = (Parse_Structure*)                _pxParse ;
    Custom_Structure_Master*        pxCustom_Master = (Custom_Structure_Master*)        _pxCustom ;

    ESuccess        eReturnStatus = Ok ;

    //\\//

    // check all other keywords have been assigned
    // Does not apply here

    //Check the Value is within maximum
    if ( pxParse->Values.ulSkipTabGabOnInit > 1 ) 
    {
        oConsole.ErrorLine ( (CString) "_ <SkipTabGabOnInit:> keyword only allows values 0 or 1 at Line #"
                                      + I_to_A ( pxParse->iLineNum ) ) ;
	    eReturnStatus = Error ;
	    goto Done ;
    }

    // Increment Total Number of register IO performed
    pxCustom_Master->uiTotCommand++ ;


	if ( pxParse->Values.ulSkipTabGabOnInit == 1 )
    {
        oConsole.SystemLine ( (CString) " Initialization will Skip VMESCL, TABs, and GAB" ) ;
        CL1Cal::mg_eynSkipTabGabOnInit = Yes ;
    }
    else 
    {
        oConsole.SystemLine ( (CString) " Initialization will Include VMESCL, TABs, and GAB" ) ;
        CL1Cal::mg_eynSkipTabGabOnInit = No ;
    }


Done:

    return eReturnStatus ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess Master_Action_Parse_MonitIgnoreTabGab    ( void* _pxParse, 
                                                    void* _pxCustom,
                                                    void* _pDlgThis )
{
    Parse_Structure*                pxParse         = (Parse_Structure*)                _pxParse ;
    Custom_Structure_Master*        pxCustom_Master = (Custom_Structure_Master*)        _pxCustom ;

    ESuccess        eReturnStatus = Ok ;

    //\\//

    // check all other keywords have been assigned
    // Does not apply here

    //Check the Value is within maximum
    if ( pxParse->Values.ulMonitIgnoreTabGab > 1 ) 
    {
        oConsole.ErrorLine ( (CString) "_ <MonitIgnoreTabGab:> keyword only allows values 0 or 1 at Line #"
                                      + I_to_A ( pxParse->iLineNum ) ) ;
	    eReturnStatus = Error ;
	    goto Done ;
    }

    // Increment Total Number of register IO performed
    pxCustom_Master->uiTotCommand++ ;


	if ( pxParse->Values.ulMonitIgnoreTabGab == 1 )
    {
        oConsole.SystemLine ( (CString) " Monitoring will Skip Collecting data from TABs and GAB" ) ;
        CMonitControl::Set_MonitIgnoreTabGab ( Yes ) ;
    }
    else 
    {
        oConsole.SystemLine ( (CString) " Monitoring will Include Collecting data from TABs and GAB" ) ;
        CMonitControl::Set_MonitIgnoreTabGab ( No ) ;
    }


Done:

    return eReturnStatus ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess Master_Action_Parse_TabGab_PackageVersion( void* _pxParse, 
                                                    void* _pxCustom,
                                                    void* _pDlgThis )
{
    Parse_Structure*                pxParse         = (Parse_Structure*)                _pxParse ;
    Custom_Structure_Master*        pxCustom_Master = (Custom_Structure_Master*)        _pxCustom ;

    ESuccess        eReturnStatus = Ok ;

    //\\//

    // check all other keywords have been assigned
    // Does not apply here

    // Increment Total Number of command performed
    pxCustom_Master->uiTotCommand++ ;

    oConsole.SystemLine ( (CString) " Setting TabGab Package Version to <"
                                    + pxParse->Values.osGeneric_String_Parameter
                                    + ">" ) ;

    //call the TabGab function
    eReturnStatus = ReportTabGabStatus (  Tab_Gab_Set_Package_Version ( pxParse->Values.osGeneric_String_Parameter ),
                                         "Tab_Gab_Set_Package_Version",
                                         "Master_Action_Parse_TabGab_PackageVersion" ) ;

    return eReturnStatus ;
}


/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess Master_Action_Parse_TabGab_FirmwareDir   ( void* _pxParse, 
                                                    void* _pxCustom,
                                                    void* _pDlgThis )
{
    Parse_Structure*                pxParse         = (Parse_Structure*)                _pxParse ;
    Custom_Structure_Master*        pxCustom_Master = (Custom_Structure_Master*)        _pxCustom ;

    ESuccess        eReturnStatus = Ok ;

    //\\//

    // check all other keywords have been assigned
    // Does not apply here

    // Increment Total Number of command performed
    pxCustom_Master->uiTotCommand++ ;

    oConsole.SystemLine ( (CString) " Setting TabGab Firmware Directory to <"
                                    + pxParse->Values.osGeneric_String_Parameter
                                    + ">" ) ;

    //call the TabGab function
    eReturnStatus = ReportTabGabStatus (  Tab_Gab_Set_Firmware_Dir ( pxParse->Values.osGeneric_String_Parameter ),
                                         "Tab_Gab_Set_Firmware_Dir",
                                         "Master_Action_Parse_TabGab_FirmwareDir" ) ;

    return eReturnStatus ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess Master_Action_Parse_TabGab_ConfigDir     ( void* _pxParse, 
                                                    void* _pxCustom,
                                                    void* _pDlgThis )
{
    Parse_Structure*                pxParse         = (Parse_Structure*)                _pxParse ;
    Custom_Structure_Master*        pxCustom_Master = (Custom_Structure_Master*)        _pxCustom ;

    ESuccess        eReturnStatus = Ok ;

    //\\//

    // check all other keywords have been assigned
    // Does not apply here

    // Increment Total Number of command performed
    pxCustom_Master->uiTotCommand++ ;

    oConsole.SystemLine ( (CString) " Setting TabGab Configuration Directory to <"
                                    + pxParse->Values.osGeneric_String_Parameter
                                    + ">" ) ;

    //call the TabGab function
    eReturnStatus = ReportTabGabStatus (  Tab_Gab_Set_Config_Dir ( pxParse->Values.osGeneric_String_Parameter ),
                                         "Tab_Gab_Set_Config_Dir",
                                         "Master_Action_Parse_TabGab_ConfigDir" ) ;

    return eReturnStatus ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess Master_Action_Parse_TabGab_WorkDir       ( void* _pxParse, 
                                                    void* _pxCustom,
                                                    void* _pDlgThis )
{
    Parse_Structure*                pxParse         = (Parse_Structure*)                _pxParse ;
    Custom_Structure_Master*        pxCustom_Master = (Custom_Structure_Master*)        _pxCustom ;

    ESuccess        eReturnStatus = Ok ;

    //\\//

    // check all other keywords have been assigned
    // Does not apply here

    // Increment Total Number of command performed
    pxCustom_Master->uiTotCommand++ ;

    oConsole.SystemLine ( (CString) " Setting TabGab Work Directory to <"
                                    + pxParse->Values.osGeneric_String_Parameter
                                    + ">" ) ;

    //call the TabGab function
    eReturnStatus = ReportTabGabStatus (  Tab_Gab_Set_Work_Dir ( pxParse->Values.osGeneric_String_Parameter ),
                                         "Tab_Gab_Set_Work_Dir",
                                         "Master_Action_Parse_TabGab_WorkDir" ) ;

    return eReturnStatus ;
}

