/*-------------------------------------------------------------------------------*/
  /*
    CommandFileRegisterIO.cpp
    3-Sep-99 split off from dialog box
  */
/*-------------------------------------------------------------------------------*/
#include "L1Cal_IIb_TCC/StdAfx.h"

#include "L1Cal_IIb_TCC/HandleCommandFiles.h"

#include                        "L1Cal_IIb_TCC/CommandFileRegisterIO.h"

#include "L1Cal_IIb_TCC/AdfRegBase.h"

#include "L1Cal_IIb_TCC/UtilsStrings.h"
#include "L1Cal_IIb_TCC/UtilsBits.h"

// static means created only once throughout life of program (to avoid wasting time)
// *AND* internal linkage (to avoid clash between modules)
#include "L1Cal_IIb_TCC/HandleConsole.h"
static CConsoleAccess oConsole ;

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess CommandFileRegisterIO    ( CString	                        _osCommand_File_Name,
                                    Custom_Structure_RegIO*         _pxCustom, 
                                    void*                           _pDlgThis )
{
	ESuccess            eReturnStatus = Ok ;

    ESuccess            eSuccess ;
    Parse_Structure     xParse ;

    CString             oMsg ;

    //\\//

    // write to console 
	oConsole.WriteLine ( "" ) ;
	oConsole.WriteLine ( (CString) "%% Register IO Command File : "
                        		 +  _osCommand_File_Name ) ;
    
    // Initialize the Command File Parser control Structure
    eSuccess = Init_Parse_Structure ( &xParse ) ;

    xParse.pacFile_Name = LPCTSTR(_osCommand_File_Name) ;

    //register the legal keywords and 
    //the call back points for the apropriate keywords
    xParse.Actions.pfVertical_Master  = NULL ;
    xParse.Actions.pfVertical_Slave   = NULL ;
    xParse.Actions.pfCard_Slot        = NULL ;
    xParse.Actions.pfChip_Address     = NULL ;
    xParse.Actions.pfRegister_Address = NULL ;
    xParse.Actions.pfRead_Register_Address = (fKeyword_Action*) RegIO_Action_Parse_Read_Register_Address ;
    xParse.Actions.pfWrite_Value      = (fKeyword_Action*) RegIO_Action_Parse_Write_Value ;
    xParse.Actions.pfWrite_Verify     = (fKeyword_Action*) RegIO_Action_Parse_Write_Verify ;
    
    // Initialize Total Number of register IO performed
    _pxCustom->uiTotReg = 0 ;

    // call the general purpose command file parser 
    eSuccess = Read_Command_File ( &xParse, _pxCustom, _pDlgThis ) ;

    if ( eSuccess == Ok  )
    {
        oMsg = "%%Done after Executing   "
             + I_to_A ( _pxCustom->uiTotReg )
             + " Reg IO Commands" ;
        oConsole.WriteLine ( oMsg ) ;
    }
    else
    {
        oMsg = "%%Abort after Executing   "
             + I_to_A ( _pxCustom->uiTotReg )
             + " Reg IO Commands" ;
        oConsole.ErrorLine ( oMsg ) ;
    	eReturnStatus = Error ;
    }

    return eReturnStatus ;
}


/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/


/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess RegIO_Action_Parse_Read_Register_Address ( void* _pxParse, 
                                                    void* _pxCustom,
                                                    void* _pDlgThis )
{
    Parse_Structure*                pxParse     = (Parse_Structure*)                _pxParse ;
    Custom_Structure_RegIO*         pxCustom    = (Custom_Structure_RegIO*)         _pxCustom ;

    CAdfRegBase*    oRegister ; 
    TuwRegWord      uwDataIn ;
    ESuccess        eStatus ;

    ESuccess        lReturnStatus = Ok ;

    //\\//

    // check all other keywords have been assigned
    if ( ( pxParse->Values.ulVertical_Master  == diNotYetAssigned ) 
      || ( pxParse->Values.ulVertical_Slave   == diNotYetAssigned ) 
      || ( pxParse->Values.ulCard_Slot        == diNotYetAssigned ) 
      || ( pxParse->Values.ulChip_Address     == diNotYetAssigned ) ) 
    {
        oConsole.ErrorLine ( (CString) "_ Missing Keywords before assignment at Line #"
                                      + I_to_A ( pxParse->iLineNum ) ) ;
	    lReturnStatus = Error ;
	    goto Done ;
    }

    // Increment Total Number of register IO performed
    pxCustom->uiTotReg++ ;

    // transfer Read_Register_Address value to the Register_Address value to keep context to this register
	pxParse->Values.ulRegister_Address = pxParse->Values.ulRead_Register_Address ;
     
    // construct the register object
    oRegister = new CAdfRegBase   ( fubTruncToByte( pxParse->Values.ulVertical_Master ),
                                    fubTruncToByte( pxParse->Values.ulVertical_Slave ),
                                    fubTruncToByte( pxParse->Values.ulCard_Slot ),
                                    fubTruncToByte( pxParse->Values.ulChip_Address ),
                                    fuwTruncToWord( pxParse->Values.ulRead_Register_Address ) ) ;
	
    // Read the Register
    eStatus = oRegister->f_okReadReg ( &uwDataIn ) ;

    if ( eStatus != Ok ) 
    {
        lReturnStatus = Error ;
        oConsole.ErrorLine ( (CString) " Register Read Failed " ) ;
    }

    delete oRegister ;

Done:

    return lReturnStatus ;
}



/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess RegIO_Action_Parse_Write_Value           ( void* _pxParse, 
                                                    void* _pxCustom,
                                                    void* _pDlgThis )
{
    Parse_Structure*                pxParse     = (Parse_Structure*)                _pxParse ;
    Custom_Structure_RegIO*         pxCustom    = (Custom_Structure_RegIO*)         _pxCustom ;

    CAdfRegBase*    oRegister ; 
    TuwRegWord      uwDataOut ;
    ESuccess        eStatus ;

    ESuccess        lReturnStatus = Ok ;

    //\\//

    // check all other keywords have been assigned
    if ( ( pxParse->Values.ulVertical_Master  == diNotYetAssigned ) 
      || ( pxParse->Values.ulVertical_Slave   == diNotYetAssigned ) 
      || ( pxParse->Values.ulCard_Slot        == diNotYetAssigned ) 
      || ( pxParse->Values.ulChip_Address     == diNotYetAssigned ) 
      || ( pxParse->Values.ulRegister_Address == diNotYetAssigned ) ) 
    {
        oConsole.ErrorLine ( (CString) "_ Missing Keywords before assignment at Line #"
                                      + I_to_A ( pxParse->iLineNum ) ) ;
	    lReturnStatus = Error ;
	    goto Done ;
    }

    // Increment Total Number of register IO performed
    pxCustom->uiTotReg++ ;
     
    // construct the register object
    oRegister = new CAdfRegBase   ( fubTruncToByte( pxParse->Values.ulVertical_Master ),
                                    fubTruncToByte( pxParse->Values.ulVertical_Slave ),
                                    fubTruncToByte( pxParse->Values.ulCard_Slot ),
                                    fubTruncToByte( pxParse->Values.ulChip_Address ),
                                    fuwTruncToWord( pxParse->Values.ulRead_Register_Address ) ) ;
	
	uwDataOut = fuwTruncToWord ( pxParse->Values.ulWrite_Value ) ;

    // Write the Register
    eStatus = oRegister->f_okWriteOnly ( &uwDataOut ) ;
   
    if ( eStatus != Ok ) 
    {
        lReturnStatus = Error ;
        oConsole.ErrorLine ( (CString) " Register Write Failed " ) ;
    }

    delete oRegister ;

Done:
    return lReturnStatus ;
}



/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess RegIO_Action_Parse_Write_Verify          ( void* _pxParse, 
                                                    void* _pxCustom,
                                                    void* _pDlgThis )
{
    Parse_Structure*                pxParse     = (Parse_Structure*)                _pxParse ;
    Custom_Structure_RegIO*         pxCustom    = (Custom_Structure_RegIO*)         _pxCustom ;

    CAdfRegBase*    oRegister ; 
    TuwRegWord      uwDataOut ;
    TuwRegWord      uwDataIn ;
    ESuccess        eStatus ;

    ESuccess        lReturnStatus = Ok ;

    //\\//

    // check all other keywords have been assigned
    if ( ( pxParse->Values.ulVertical_Master  == diNotYetAssigned ) 
      || ( pxParse->Values.ulVertical_Slave   == diNotYetAssigned ) 
      || ( pxParse->Values.ulCard_Slot        == diNotYetAssigned ) 
      || ( pxParse->Values.ulChip_Address     == diNotYetAssigned ) 
      || ( pxParse->Values.ulRegister_Address == diNotYetAssigned ) ) 
    {
        oConsole.ErrorLine ( (CString) "_ Missing Keywords before assignment at Line #"
                                      + I_to_A ( pxParse->iLineNum ) ) ;
	    lReturnStatus = Error ;
	    goto Done ;
    }

    // Increment Total Number of register IO performed
    pxCustom->uiTotReg++ ;
     
    // construct the register object
    oRegister = new CAdfRegBase   ( fubTruncToByte( pxParse->Values.ulVertical_Master ),
                                    fubTruncToByte( pxParse->Values.ulVertical_Slave ),
                                    fubTruncToByte( pxParse->Values.ulCard_Slot ),
                                    fubTruncToByte( pxParse->Values.ulChip_Address ),
                                    fuwTruncToWord( pxParse->Values.ulRead_Register_Address ) ) ;
	
	uwDataOut = fuwTruncToWord ( pxParse->Values.ulWrite_Verify ) ;


    // Write the Register
    eStatus = oRegister->f_okWriteReg ( &uwDataOut ) ;

    if ( eStatus != Ok ) 
    {
        lReturnStatus = Error ;
        oConsole.ErrorLine ( (CString) " Register Write Failed " ) ;
    }

    // Read the Register
    eStatus = oRegister->f_okReadReg ( &uwDataIn ) ;

    if ( eStatus != Ok ) 
    {
        lReturnStatus = Error ;
        oConsole.ErrorLine ( (CString) " Register Read Failed " ) ;
    }
	
    if ( uwDataOut != uwDataIn )
    {
        oConsole.ErrorLine ( (CString) " Verif : Read " 
	  	                                + I_to_A( uwDataIn, 5, 10, ' ' )
		                                + " (=" 
		                                + I_to_A( uwDataIn, 16, 2,'0' )
		                                + ")"
                                        + " Instead of " 
	  	                                + I_to_A( uwDataOut, 5, 10, ' ' )
		                                + " (=" 
		                                + I_to_A( uwDataOut, 16, 2,'0' )
		                                + ")" ) ;
        oConsole.ErrorLine ( (CString) " _ " 
	  	                                + oRegister->f_osRegAddress() ) ;
	    lReturnStatus = Error ;
    }

    delete oRegister ;

Done:
    return lReturnStatus ;
}

