/*-------------------------------------------------------------------------------*/
  /*
    CommandFileSerialIO.cpp
    3-Sep-99 split off from dialog box
  */
/*-------------------------------------------------------------------------------*/
#include "L1Cal_IIb_TCC/StdAfx.h"

#include "L1Cal_IIb_TCC/HandleCommandFiles.h"

#include                        "L1Cal_IIb_TCC/CommandFileSerialIO.h"

#include "L1Cal_IIb_TCC/NevisCard.h"
#include "tabgab/vmescl.h"

#include "L1Cal_IIb_TCC/UtilsStrings.h"
//#include "L1Cal_IIb_TCC/UtilsBits.h"

// static means created only once throughout life of program (to avoid wasting time)
// *AND* internal linkage (to avoid clash between modules)
#include "L1Cal_IIb_TCC/HandleConsole.h"
static CConsoleAccess oConsole ;

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess CommandFileSerialIO      ( CString	                        _osCommand_File_Name,
                                    Custom_Structure_SerIO*         _pxCustom, 
                                    void*                           _pDlgThis )
{
	ESuccess            eReturnStatus = Ok ;

    ESuccess            eSuccess ;
    Parse_Structure     xParse ;

    CString             oMsg ;

    //\\//

    // write to console 
	oConsole.WriteLine ( "" ) ;
	oConsole.WriteLine ( (CString) "%% Register IO Command File : "
                        		 +  _osCommand_File_Name ) ;
    
    // Initialize the Command File Parser control Structure
    eSuccess = Init_Parse_Structure ( &xParse ) ;

    xParse.pacFile_Name = LPCTSTR(_osCommand_File_Name) ;

    //register the legal keywords and 
    //the call back points for the apropriate keywords
    xParse.Actions.pfTabGab_Module         = NULL ;
    xParse.Actions.pfChip_Address          = NULL ;
    xParse.Actions.pfGroup_Address         = NULL ;
    xParse.Actions.pfSerial_Address        = NULL ;
    xParse.Actions.pfSerial_Data_Write     = (fKeyword_Action*) SerIO_Action_Parse_Serial_Data_Write ;
    xParse.Actions.pfSerial_Address_Read   = (fKeyword_Action*) SerIO_Action_Parse_Serial_Address_Read ;
    
    // Initialize Total Number of register IO performed
    _pxCustom->uiTotReg = 0 ;

    // call the general purpose command file parser 
    eSuccess = Read_Command_File ( &xParse, _pxCustom, _pDlgThis ) ;

    if ( eSuccess == Ok  )
    {
        oMsg = "%%Done after Executing   "
             + I_to_A ( _pxCustom->uiTotReg )
             + " Serial IO Commands" ;
        oConsole.WriteLine ( oMsg ) ;
    }
    else
    {
        oMsg = "%%Abort after Executing   "
             + I_to_A ( _pxCustom->uiTotReg )
             + " Serial IO Commands" ;
        oConsole.ErrorLine ( oMsg ) ;
    	eReturnStatus = Error ;
    }

    return eReturnStatus ;
}


/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/


/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess SerIO_Action_Parse_Serial_Address_Read   ( void* _pxParse, 
                                                    void* _pxCustom,
                                                    void* _pDlgThis )
{
    Parse_Structure*                pxParse     = (Parse_Structure*)                _pxParse ;
    Custom_Structure_SerIO*         pxCustom    = (Custom_Structure_SerIO*)         _pxCustom ;

    unsigned long   ulDataIn ;
    int             iStatus ;

    ESuccess        lReturnStatus = Ok ;

    //\\//

    // check all other keywords have been assigned
    if ( ( pxParse->Values.ulTabGab_Module    == diNotYetAssigned ) 
      || ( pxParse->Values.ulChip_Address     == diNotYetAssigned ) 
      || ( pxParse->Values.ulGroup_Address    == diNotYetAssigned ) ) 
    {
        oConsole.ErrorLine ( (CString) "_ Missing Keywords before assignment at Line #"
                                      + I_to_A ( pxParse->iLineNum ) ) ;
	    lReturnStatus = Error ;
	    goto Done ;
    }

    // Increment Total Number of register IO performed
    pxCustom->uiTotReg++ ;

    // transfer Read_Register_Address value to the Register_Address value to keep context to this register
	pxParse->Values.ulSerial_Address = pxParse->Values.ulSerial_Address_Read ;
     
    // Read the Register
/*
    eStatus = ReportTabGabStatus ( vmescl_read_serial (pxParse->Values.ulTabGab_Module, 
		                                               pxParse->Values.ulChip_Address, 
		                                               pxParse->Values.ulGroup_Address,
		                                               pxParse->Values.ulSerial_Address,
		                                               pxParse->Values.ulSerial_Data_Write,
		                                              &ulDataIn ),
                                  "vmescl_read_serial",
                                  "SerIO_Action_Parse_Serial_Address_Read" ) ;
*/
    iStatus = vmescl_read_serial ( pxParse->Values.ulTabGab_Module, 
		                           pxParse->Values.ulChip_Address, 
		                           pxParse->Values.ulGroup_Address,
		                           pxParse->Values.ulSerial_Address,
		                           pxParse->Values.ulSerial_Data_Write,
                                  &ulDataIn )  ;

    if ( iStatus != EXIT_SUCCESS ) 
    {
        lReturnStatus = Error ;
        oConsole.ErrorLine ( (CString) " Serial Read Failed " ) ;
    }

Done:

    return lReturnStatus ;
}



/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess SerIO_Action_Parse_Serial_Data_Write     ( void* _pxParse, 
                                                    void* _pxCustom,
                                                    void* _pDlgThis )
{
    Parse_Structure*                pxParse     = (Parse_Structure*)                _pxParse ;
    Custom_Structure_SerIO*         pxCustom    = (Custom_Structure_SerIO*)         _pxCustom ;

    int             iStatus ;

    ESuccess        lReturnStatus = Ok ;

    //\\//

    // check all other keywords have been assigned
    if ( ( pxParse->Values.ulTabGab_Module    == diNotYetAssigned ) 
      || ( pxParse->Values.ulChip_Address     == diNotYetAssigned ) 
      || ( pxParse->Values.ulGroup_Address    == diNotYetAssigned ) 
      || ( pxParse->Values.ulSerial_Address   == diNotYetAssigned ) ) 
    {
        oConsole.ErrorLine ( (CString) "_ Missing Keywords before assignment at Line #"
                                      + I_to_A ( pxParse->iLineNum ) ) ;
	    lReturnStatus = Error ;
	    goto Done ;
    }

    // Increment Total Number of register IO performed
    pxCustom->uiTotReg++ ;
     
    // Write the Register
/*
    eStatus = ReportTabGabStatus ( vmescl_write_serial(pxParse->Values.ulTabGab_Module, 
		                                               pxParse->Values.ulChip_Address, 
		                                               pxParse->Values.ulGroup_Address,
		                                               pxParse->Values.ulSerial_Address,
		                                               pxParse->Values.ulSerial_Data_Write),
                                  "vmescl_write_serial",
                                  "SerIO_Action_Parse_Serial_Data_Write" ) ;
*/   
    iStatus = vmescl_write_serial( pxParse->Values.ulTabGab_Module, 
		                           pxParse->Values.ulChip_Address, 
		                           pxParse->Values.ulGroup_Address,
		                           pxParse->Values.ulSerial_Address,
		                           pxParse->Values.ulSerial_Data_Write )  ;

    if ( iStatus != EXIT_SUCCESS ) 
    {
        lReturnStatus = Error ;
        oConsole.ErrorLine ( (CString) " Serial Write Failed " ) ;
    }

Done:
    return lReturnStatus ;
}
