/*-------------------------------------------------------------------------------*/
  /*
    CommandFileTrigTwrInfo.cpp
    10-Oct-2005
  */
/*-------------------------------------------------------------------------------*/
#include "L1Cal_IIb_TCC/StdAfx.h"

#include "L1Cal_IIb_TCC/HandleCommandFiles.h"

#include                        "L1Cal_IIb_TCC/CommandFileTrigTwrInfo.h"

#include "L1Cal_IIb_TCC/L1Cal.h"

#include "L1Cal_IIb_TCC/UtilsStrings.h"
#include "L1Cal_IIb_TCC/UtilsBits.h"

// static means created only once throughout life of program (to avoid wasting time)
// *AND* internal linkage (to avoid clash between modules)
#include "L1Cal_IIb_TCC/HandleConsole.h"
static CConsoleAccess oConsole ;

/*-------------------------------------------------------------------------------*/
void swap_ul  ( unsigned long* pulA, 
                unsigned long* pulB )
{ // simple function to swap two longword quantities
  unsigned long ulTemp ;
  ulTemp = *pulA ;   *pulA = *pulB ;  *pulB = ulTemp ;
  return ;
}
/////////////////////////////////
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess CommandFileTrigTwrInfo   ( CString	                        _osCommand_File_Name,
                                    Custom_Structure_TT_Info*       _pxCustom, 
                                    void*                           _pDlgThis )
{
	ESuccess            eReturnStatus = Ok ;

    ESuccess            eSuccess ;
    Parse_Structure     xParse ;

    CString             oMsg ;

    //\\//

    // write to console 
	oConsole.WriteLine ( "" ) ;
	oConsole.WriteLine ( (CString) "%% Trigger Tower Info Command File : "
                        		 +  _osCommand_File_Name ) ;
    
    // Initialize the Command File Parser control Structure
    eSuccess = Init_Parse_Structure ( &xParse ) ;

    xParse.pacFile_Name = LPCTSTR(_osCommand_File_Name) ;

    //register the legal keywords and 
    //the call back points for the apropriate keywords
    xParse.Actions.pfTT_Channel     = (fKeyword_Action*) TT_Info_Action_Parse_TT_Channel ;
    xParse.Actions.pfTT_Eta_Sign    = (fKeyword_Action*) TT_Info_Action_Parse_TT_Eta_Sign ;
    xParse.Actions.pfTT_Eta_Magn    = (fKeyword_Action*) TT_Info_Action_Parse_TT_Eta_Magn ;
    xParse.Actions.pfTT_Phi         = (fKeyword_Action*) TT_Info_Action_Parse_TT_Phi ;
    xParse.Actions.pfTo_TT_Eta_Sign = (fKeyword_Action*) TT_Info_Action_Parse_To_TT_Eta_Sign ;
    xParse.Actions.pfTo_TT_Eta_Magn = (fKeyword_Action*) TT_Info_Action_Parse_To_TT_Eta_Magn ;
    xParse.Actions.pfTo_TT_Phi      = (fKeyword_Action*) TT_Info_Action_Parse_To_TT_Phi ;
    xParse.Actions.pfTick           = (fKeyword_Action*) TT_Info_Action_Parse_Tick ; 
    xParse.Actions.pfTo_Tick        = (fKeyword_Action*) TT_Info_Action_Parse_To_Tick ; 
    xParse.Actions.pfSubTick        = (fKeyword_Action*) TT_Info_Action_Parse_SubTick ; 
    xParse.Actions.pfTo_SubTick     = (fKeyword_Action*) TT_Info_Action_Parse_To_SubTick ; 

    xParse.Actions.pfAlignment_Delay= (fKeyword_Action*) TT_Info_Action_Parse_Alignment_Delay ;
    xParse.Actions.pfPedestal_DAC   = (fKeyword_Action*) TT_Info_Action_Parse_Pedestal_DAC ;
    xParse.Actions.pfPed_Average    = (fKeyword_Action*) TT_Info_Action_Parse_Ped_Average ;
    xParse.Actions.pfPed_Deviation  = (fKeyword_Action*) TT_Info_Action_Parse_Ped_Deviation ;

    xParse.Actions.pfZeroEngAdcE    = (fKeyword_Action*) TT_Info_Action_Parse_Zero_Energy_Adc_E ;
    xParse.Actions.pfZeroEngOutEt   = (fKeyword_Action*) TT_Info_Action_Parse_Zero_Energy_Output_Et ;
    xParse.Actions.pfLookup_Slope   = (fKeyword_Action*) TT_Info_Action_Parse_Lookup_Slope ;
    
    xParse.Actions.pfRawAdc_Simu    = (fKeyword_Action*) TT_Info_Action_Parse_RawAdc_Simu ;
    xParse.Actions.pfAdfOut_Simu    = (fKeyword_Action*) TT_Info_Action_Parse_AdfOut_Simu ;
    xParse.Actions.pfAdfOut_Const   = (fKeyword_Action*) TT_Info_Action_Parse_AdfOut_Const ;
    xParse.Actions.pfRawAdc_Thresh  = (fKeyword_Action*) TT_Info_Action_Parse_RawAdc_Thresh ;

    xParse.Actions.pfRawAdc_MuxMode = (fKeyword_Action*) TT_Info_Action_Parse_RawAdc_MuxMode ;
    xParse.Actions.pfRawAdc_MemMode = (fKeyword_Action*) TT_Info_Action_Parse_RawAdc_MemMode ;
    xParse.Actions.pfAdfOut_MuxMode = (fKeyword_Action*) TT_Info_Action_Parse_AdfOut_MuxMode ;
    xParse.Actions.pfAdfOut_MemMode = (fKeyword_Action*) TT_Info_Action_Parse_AdfOut_MemMode ;

    // Initialize Total Number of Dac Values written
    _pxCustom->uiTotAlignmentDelay  = 0 ;
    _pxCustom->uiTotPedDacValue     = 0 ;
    _pxCustom->uiTotLookupSlope     = 0 ;
    _pxCustom->uiTotRawAdc_Simu     = 0 ;
    _pxCustom->uiTotAdfOut_Simu     = 0 ;
    _pxCustom->uiTotAdfOut_Const    = 0 ;
    _pxCustom->uiTotRawAdc_Thresh   = 0 ;
    _pxCustom->uiTotRawAdc_MuxMode  = 0 ;
    _pxCustom->uiTotRawAdc_MemMode  = 0 ;
    _pxCustom->uiTotAdfOut_MuxMode  = 0 ;
    _pxCustom->uiTotAdfOut_MemMode  = 0 ;

    // call the general purpose command file parser 
    eSuccess = Read_Command_File ( &xParse, _pxCustom, _pDlgThis ) ;

    oMsg = "defining:\n" ;
    if ( _pxCustom->uiTotAlignmentDelay != 0 )
        oMsg += "_ " + I_to_A ( _pxCustom->uiTotAlignmentDelay )  + " Alignment Delays\n" ;

    if ( _pxCustom->uiTotPedDacValue != 0 )
        oMsg += "_ " + I_to_A ( _pxCustom->uiTotPedDacValue )     + " Pedestal Dacs\n" ;
    
    if ( _pxCustom->uiTotLookupSlope != 0 )
        oMsg += "_ " + I_to_A ( _pxCustom->uiTotLookupSlope )     + " Et Lookup Slopes\n" ;
    
    if ( _pxCustom->uiTotRawAdc_Simu != 0 )
        oMsg += "_ " + I_to_A ( _pxCustom->uiTotRawAdc_Simu )     + " Raw ADC Simu Values\n" ;
    
    if ( _pxCustom->uiTotAdfOut_Simu != 0 )
        oMsg += "_ " + I_to_A ( _pxCustom->uiTotAdfOut_Simu )     + " ADF Output Simu Values\n" ;
    
    if ( _pxCustom->uiTotAdfOut_Const != 0 )
        oMsg += "_ " + I_to_A ( _pxCustom->uiTotAdfOut_Const )    + " ADF Output Const Values\n" ;
    
    if ( _pxCustom->uiTotRawAdc_Thresh != 0 )
        oMsg += "_ " + I_to_A ( _pxCustom->uiTotRawAdc_Thresh )   + " Raw ADC Thresh Values\n" ;
    
    if ( _pxCustom->uiTotRawAdc_MuxMode != 0 )
        oMsg += "_ " + I_to_A ( _pxCustom->uiTotRawAdc_MuxMode )  + " Raw ADC Mux Modes\n" ;
    
    if ( _pxCustom->uiTotRawAdc_MemMode != 0 )
        oMsg += "_ " + I_to_A ( _pxCustom->uiTotRawAdc_MemMode )  + " Raw ADC Mem Modes\n" ;
    
    if ( _pxCustom->uiTotAdfOut_MuxMode != 0 )
        oMsg += "_ " + I_to_A ( _pxCustom->uiTotAdfOut_MuxMode )  + " Output Et Mux Modes\n" ;
    
    if ( _pxCustom->uiTotAdfOut_MemMode != 0 )
        oMsg += "_ " + I_to_A ( _pxCustom->uiTotAdfOut_MemMode )  + " Output Et Mem Modes\n" ;


    if ( eSuccess == Ok  )
    {
        oConsole.WriteLine ( (CString) "%%Done after " + oMsg ) ;
    }
    else
    {
        oConsole.ErrorLine ( (CString) "%%Abort after " + oMsg ) ;
    	eReturnStatus = Error ;
    }

    return eReturnStatus ;
}


/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/


/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess TT_Info_Action_Parse_TT_Channel          ( void* _pxParse, 
                                                    void* _pxCustom,
                                                    void* _pDlgThis )
{
    Parse_Structure*                pxParse     = (Parse_Structure*)                _pxParse ;
    Custom_Structure_TT_Info*       pxCustom    = (Custom_Structure_TT_Info*)       _pxCustom ;

    ESuccess        lReturnStatus = Ok ;

    //\\//

    // verify the Coordinate is within existing range
    if ( pxParse->Values.ulTT_Channel >= eTot_TT_Chan_Type )
    {
        oConsole.ErrorLine ( (CString) "_ Illegal Trigger Tower <TT_Channel> Value at Line #"
                                      + I_to_A ( pxParse->iLineNum ) ) ;
	    lReturnStatus = Error ;
	    goto Done ;
    }

Done:
    return lReturnStatus ;
}


/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess TT_Info_Action_Parse_TT_Eta_Sign         ( void* _pxParse, 
                                                    void* _pxCustom,
                                                    void* _pDlgThis )
{
    Parse_Structure*                pxParse     = (Parse_Structure*)                _pxParse ;
    Custom_Structure_TT_Info*       pxCustom    = (Custom_Structure_TT_Info*)       _pxCustom ;

    ESuccess        lReturnStatus = Ok ;

    //\\//

    // Invalidate upper bound for a TT Eta Sign Range that may have been previously defined
    pxParse->Values.ulTo_TT_Eta_Sign = diNotYetAssigned ;

    // verify the Coordinate is within existing range
    // note: allow definition out of current coverage 
    if ( pxParse->Values.ulTT_Eta_Sign > KiL1CalTotSignEta ) 
    {
        oConsole.ErrorLine ( (CString) "_ Illegal Trigger Tower <TT_Eta_Sign> Value at Line #"
                                      + I_to_A ( pxParse->iLineNum ) ) ;
	    lReturnStatus = Error ;
	    goto Done ;
    }

Done:
    return lReturnStatus ;
}


/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess TT_Info_Action_Parse_TT_Eta_Magn         ( void* _pxParse, 
                                                    void* _pxCustom,
                                                    void* _pDlgThis )
{
    Parse_Structure*                pxParse     = (Parse_Structure*)                _pxParse ;
    Custom_Structure_TT_Info*       pxCustom    = (Custom_Structure_TT_Info*)       _pxCustom ;

    ESuccess        lReturnStatus = Ok ;

    //\\//

    // Invalidate upper bound for a TT Eta Magn Range that may have been previously defined
    pxParse->Values.ulTo_TT_Eta_Magn = diNotYetAssigned ;

    // verify the Coordinate is within existing range
    // note: allow definition out of current coverage 
    if ( pxParse->Values.ulTT_Eta_Magn > KiL1CalTotMagnEta ) 
    {
        oConsole.ErrorLine ( (CString) "_ Illegal Trigger Tower <TT_Eta_Magn> Value at Line #"
                                      + I_to_A ( pxParse->iLineNum ) ) ;
	    lReturnStatus = Error ;
	    goto Done ;
    }

Done:
    return lReturnStatus ;
}


/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess TT_Info_Action_Parse_TT_Phi              ( void* _pxParse, 
                                                    void* _pxCustom,
                                                    void* _pDlgThis )
{
    Parse_Structure*                pxParse     = (Parse_Structure*)                _pxParse ;
    Custom_Structure_TT_Info*       pxCustom    = (Custom_Structure_TT_Info*)       _pxCustom ;

    ESuccess        lReturnStatus = Ok ;

    //\\//

    // Invalidate upper bound for a TT Phi Range that may have been previously defined
    pxParse->Values.ulTo_TT_Phi = diNotYetAssigned ;

    // verify the Coordinate is within existing range
    // note: allow definition out of current coverage 
    if ( pxParse->Values.ulTT_Phi > KiL1CalTotPhi ) 
    {
        oConsole.ErrorLine ( (CString) "_ Illegal Trigger Tower <TT_Phi> Value at Line #"
                                      + I_to_A ( pxParse->iLineNum ) ) ;
	    lReturnStatus = Error ;
	    goto Done ;
    }

Done:
    return lReturnStatus ;
}


/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess TT_Info_Action_Parse_To_TT_Eta_Sign      ( void* _pxParse, 
                                                    void* _pxCustom,
                                                    void* _pDlgThis )
{
    Parse_Structure*                pxParse     = (Parse_Structure*)                _pxParse ;
    Custom_Structure_TT_Info*       pxCustom    = (Custom_Structure_TT_Info*)       _pxCustom ;

    ESuccess        lReturnStatus = Ok ;

    //\\//

    // verify a lower bound for the TT Eta Sign Range has been previously defined
    if ( pxParse->Values.ulTT_Eta_Sign == diNotYetAssigned ) 
    {
        oConsole.ErrorLine ( (CString) "_ Missing TT_Eta_Sign before To_TT_Eta_Sign at Line #"
                                      + I_to_A ( pxParse->iLineNum ) ) ;
	    lReturnStatus = Error ;
	    goto Done ;
    }

    // verify the Coordinate is within existing range
    // note: allow definition out of current coverage 
    if ( pxParse->Values.ulTo_TT_Eta_Sign > KiL1CalTotSignEta ) 
    {
        oConsole.ErrorLine ( (CString) "_ Illegal Trigger Tower <To_TT_Eta_Sign> Value at Line #"
                                      + I_to_A ( pxParse->iLineNum ) ) ;
	    lReturnStatus = Error ;
	    goto Done ;
    }

Done:
    return lReturnStatus ;
}



/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess TT_Info_Action_Parse_To_TT_Eta_Magn      ( void* _pxParse, 
                                                    void* _pxCustom,
                                                    void* _pDlgThis )
{
    Parse_Structure*                pxParse     = (Parse_Structure*)                _pxParse ;
    Custom_Structure_TT_Info*       pxCustom    = (Custom_Structure_TT_Info*)       _pxCustom ;

    ESuccess        lReturnStatus = Ok ;

    //\\//

    // verify a lower bound for the TT Eta Magn Range has been previously defined
    if ( pxParse->Values.ulTT_Eta_Magn == diNotYetAssigned ) 
    {
        oConsole.ErrorLine ( (CString) "_ Missing TT_Eta_Magn before To_TT_Eta_Magn at Line #"
                                      + I_to_A ( pxParse->iLineNum ) ) ;
	    lReturnStatus = Error ;
	    goto Done ;
    }

    // verify the Coordinate is within existing range
    // note: allow definition out of current coverage 
    if ( pxParse->Values.ulTo_TT_Eta_Magn > KiL1CalTotMagnEta ) 
    {
        oConsole.ErrorLine ( (CString) "_ Illegal Trigger Tower <To_TT_Eta_Magn> Value at Line #"
                                      + I_to_A ( pxParse->iLineNum ) ) ;
	    lReturnStatus = Error ;
	    goto Done ;
    }

Done:
    return lReturnStatus ;
}



/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess TT_Info_Action_Parse_To_TT_Phi           ( void* _pxParse, 
                                                    void* _pxCustom,
                                                    void* _pDlgThis )
{
    Parse_Structure*                pxParse     = (Parse_Structure*)                _pxParse ;
    Custom_Structure_TT_Info*       pxCustom    = (Custom_Structure_TT_Info*)       _pxCustom ;

    ESuccess        lReturnStatus = Ok ;

    //\\//

    // verify a lower bound for the TT Phi Range has been previously defined
    if ( pxParse->Values.ulTT_Phi == diNotYetAssigned ) 
    {
        oConsole.ErrorLine ( (CString) "_ Missing TT_Phi before To_TT_Phi at Line #"
                                      + I_to_A ( pxParse->iLineNum ) ) ;
	    lReturnStatus = Error ;
	    goto Done ;
    }


    // verify the Coordinate is within existing range
    // note: allow definition out of current coverage 
    if ( pxParse->Values.ulTo_TT_Phi > KiL1CalTotPhi ) 
    {
        oConsole.ErrorLine ( (CString) "_ Illegal Trigger Tower <To_TT_Phi> Value at Line #"
                                      + I_to_A ( pxParse->iLineNum ) ) ;
	    lReturnStatus = Error ;
	    goto Done ;
    }

Done:
    return lReturnStatus ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/

/////////////////////////////////////////////////////////////////////////////
#define DEFINE_TT_COORD_RANGE_VAR()                                         \
    unsigned long   ulTTEtaSign, ulTTEtaSignLower, ulTTEtaSignUpper ;       \
    unsigned long   ulTTEtaMagn, ulTTEtaMagnLower, ulTTEtaMagnUpper ;       \
    unsigned long   ulTTPhi,     ulTTPhiLower,     ulTTPhiUpper ;           \
/////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////
#define DEFINE_TICK_RANGE_VAR()                                             \
    unsigned long   ulTickNum, ulTickNumLower, ulTickNumUpper ;             \
/////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////
#define DEFINE_SUBTICK_RANGE_VAR()                                          \
    unsigned long   ulSubTickNum, ulSubTickNumLower, ulSubTickNumUpper ;    \
/////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////
#define CHECK_TT_COORD_ASSIGNED( THIS_KEYWORD_NAME )                        \
    /* check needed keywords have been assigned */                          \
    if ( ( pxParse->Values.ulTT_Eta_Sign      == diNotYetAssigned )         \
      || ( pxParse->Values.ulTT_Eta_Magn      == diNotYetAssigned )         \
      || ( pxParse->Values.ulTT_Phi           == diNotYetAssigned ) )       \
    {                                                                       \
        oConsole.ErrorLine ( (CString) "_ Missing TT Coordinates before <"  \
                                      + #THIS_KEYWORD_NAME                  \
                                      + "> assignment at Line #"            \
                                      + I_to_A ( pxParse->iLineNum ) ) ;    \
	    lReturnStatus = Error ;                                             \
	    goto Done ;                                                         \
    }                                                                       \
/////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////
#define CHECK_TICK_ASSIGNED( THIS_KEYWORD_NAME )                            \
    /* check needed keywords have been assigned */                          \
    if ( pxParse->Values.ulTick      == diNotYetAssigned )                  \
    {                                                                       \
        oConsole.ErrorLine ( (CString) "_ Missing Tick Number before <"     \
                                      + #THIS_KEYWORD_NAME                  \
                                      + "> assignment at Line #"            \
                                      + I_to_A ( pxParse->iLineNum ) ) ;    \
	    lReturnStatus = Error ;                                             \
	    goto Done ;                                                         \
    }                                                                       \
/////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////
#define CHECK_SUBTICK_ASSIGNED( THIS_KEYWORD_NAME )                         \
    /* check needed keywords have been assigned */                          \
    if ( pxParse->Values.ulSubTick      == diNotYetAssigned )               \
    {                                                                       \
        oConsole.ErrorLine ( (CString) "_ Missing SubTick Number before <"  \
                                      + #THIS_KEYWORD_NAME                  \
                                      + "> assignment at Line #"            \
                                      + I_to_A ( pxParse->iLineNum ) ) ;    \
	    lReturnStatus = Error ;                                             \
	    goto Done ;                                                         \
    }                                                                       \
/////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////
#define CHECK_TT_CHANNEL_ASSIGNED( THIS_KEYWORD_NAME )                      \
    /* check needed keywords have been assigned */                          \
    if ( pxParse->Values.ulTT_Channel       == diNotYetAssigned )           \
    {                                                                       \
        oConsole.ErrorLine ( (CString) "_ Missing TT Channel before <"      \
                                      + #THIS_KEYWORD_NAME                  \
                                      + "> assignment at Line #"            \
                                      + I_to_A ( pxParse->iLineNum ) ) ;    \
	    lReturnStatus = Error ;                                             \
	    goto Done ;                                                         \
    }                                                                       \
/////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////
#define RETRIEVE_ONE_TT_COORD_RANGE( COORD_VAR, COORD_KEYWORD )             \
    ul##COORD_VAR##Lower = pxParse->Values.ul##COORD_KEYWORD ;              \
    if ( pxParse->Values.ulTo_##COORD_KEYWORD == diNotYetAssigned )         \
        ul##COORD_VAR##Upper = ul##COORD_VAR##Lower ;                       \
    else                                                                    \
        ul##COORD_VAR##Upper = pxParse->Values.ulTo_##COORD_KEYWORD ;       \
    if ( ul##COORD_VAR##Lower > ul##COORD_VAR##Upper )                      \
        swap_ul( &ul##COORD_VAR##Lower, &ul##COORD_VAR##Upper ) ;           \
/////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////
#define RETRIEVE_TICK_RANGE( )                                              \
        RETRIEVE_ONE_TT_COORD_RANGE( TickNum, Tick )                        \
/////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////
#define RETRIEVE_SUBTICK_RANGE( )                                           \
        RETRIEVE_ONE_TT_COORD_RANGE( SubTickNum, SubTick )                  \
/////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////
#define RETRIEVE_ALL_TT_COORD_RANGE( )                                      \
    RETRIEVE_ONE_TT_COORD_RANGE ( TTEtaSign,   TT_Eta_Sign ) ;              \
    RETRIEVE_ONE_TT_COORD_RANGE ( TTEtaMagn,   TT_Eta_Magn ) ;              \
    RETRIEVE_ONE_TT_COORD_RANGE ( TTPhi,       TT_Phi ) ;                   \
/////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////
#define LOOP_OVER_TT_COORD_RANGE( )                                         \
    for ( ulTTEtaSign  = ulTTEtaSignLower ;                                 \
          ulTTEtaSign <= ulTTEtaSignUpper ;                                 \
          ulTTEtaSign++ )                                                   \
    for ( ulTTEtaMagn  = ulTTEtaMagnLower ;                                 \
          ulTTEtaMagn <= ulTTEtaMagnUpper ;                                 \
          ulTTEtaMagn++ )                                                   \
    for ( ulTTPhi      = ulTTPhiLower     ;                                 \
          ulTTPhi     <= ulTTPhiUpper     ;                                 \
          ulTTPhi++ )                                                       \
/////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////
#define LOOP_OVER_TICK_RANGE( )                                             \
    for ( ulTickNum  = ulTickNumLower ;                                     \
          ulTickNum <= ulTickNumUpper ;                                     \
          ulTickNum ++ )                                                    \
/////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////
#define LOOP_OVER_SUBTICK_RANGE( )                                          \
    for ( ulSubTickNum  = ulSubTickNumLower ;                               \
          ulSubTickNum <= ulSubTickNumUpper ;                               \
          ulSubTickNum ++ )                                                 \
/////////////////////////////////////////////////////////////////////////////

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess TT_Info_Action_Parse_Tick                ( void* _pxParse, 
                                                    void* _pxCustom,
                                                    void* _pDlgThis )
{
    Parse_Structure*                pxParse     = (Parse_Structure*)                _pxParse ;
    Custom_Structure_TT_Info*       pxCustom    = (Custom_Structure_TT_Info*)       _pxCustom ;

    ESuccess        lReturnStatus = Ok ;

    //\\//

    // Invalidate upper bound for a Tick Range that may have been previously defined
    pxParse->Values.ulTo_Tick = diNotYetAssigned ;

    // verify the Coordinate is within existing range
    if ( pxParse->Values.ulTick > KiTotTicksInTurn ) 
    {
        oConsole.ErrorLine ( (CString) "_ Illegal <Tick> Value at Line #"
                                      + I_to_A ( pxParse->iLineNum ) ) ;
	    lReturnStatus = Error ;
	    goto Done ;
    }

Done:
    return lReturnStatus ;
}


/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess TT_Info_Action_Parse_To_Tick             ( void* _pxParse, 
                                                    void* _pxCustom,
                                                    void* _pDlgThis )
{
    Parse_Structure*                pxParse     = (Parse_Structure*)                _pxParse ;
    Custom_Structure_TT_Info*       pxCustom    = (Custom_Structure_TT_Info*)       _pxCustom ;

    ESuccess        lReturnStatus = Ok ;

    //\\//

    // verify a lower bound for the Tick Range has been previously defined
    if ( pxParse->Values.ulTick == diNotYetAssigned ) 
    {
        oConsole.ErrorLine ( (CString) "_ Missing <Tick> before <To_Tick> at Line #"
                                      + I_to_A ( pxParse->iLineNum ) ) ;
	    lReturnStatus = Error ;
	    goto Done ;
    }


    // verify the Coordinate is within existing range
    if ( pxParse->Values.ulTo_Tick > KiTotTicksInTurn ) 
    {
        oConsole.ErrorLine ( (CString) "_ Illegal <To_Tick> Value at Line #"
                                      + I_to_A ( pxParse->iLineNum ) ) ;
	    lReturnStatus = Error ;
	    goto Done ;
    }

Done:
    return lReturnStatus ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess TT_Info_Action_Parse_SubTick             ( void* _pxParse, 
                                                    void* _pxCustom,
                                                    void* _pDlgThis )
{
    Parse_Structure*                pxParse     = (Parse_Structure*)                _pxParse ;
    Custom_Structure_TT_Info*       pxCustom    = (Custom_Structure_TT_Info*)       _pxCustom ;

    ESuccess        lReturnStatus = Ok ;

    //\\//

    // Invalidate upper bound for a SubTick Range that may have been previously defined
    pxParse->Values.ulTo_SubTick = diNotYetAssigned ;

    // verify the Coordinate is within existing range
    if ( pxParse->Values.ulSubTick >= KiTotSamplesPerTick ) 
    {
        oConsole.ErrorLine ( (CString) "_ Illegal <SubTick> Value at Line #"
                                      + I_to_A ( pxParse->iLineNum ) ) ;
	    lReturnStatus = Error ;
	    goto Done ;
    }

Done:
    return lReturnStatus ;
}


/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess TT_Info_Action_Parse_To_SubTick          ( void* _pxParse, 
                                                    void* _pxCustom,
                                                    void* _pDlgThis )
{
    Parse_Structure*                pxParse     = (Parse_Structure*)                _pxParse ;
    Custom_Structure_TT_Info*       pxCustom    = (Custom_Structure_TT_Info*)       _pxCustom ;

    ESuccess        lReturnStatus = Ok ;

    //\\//

    // verify a lower bound for the SubTick Range has been previously defined
    if ( pxParse->Values.ulSubTick == diNotYetAssigned ) 
    {
        oConsole.ErrorLine ( (CString) "_ Missing <SubTick> before <To_SubTick> at Line #"
                                      + I_to_A ( pxParse->iLineNum ) ) ;
	    lReturnStatus = Error ;
	    goto Done ;
    }


    // verify the Coordinate is within existing range
    if ( pxParse->Values.ulTo_SubTick >= KiTotSamplesPerTick ) 
    {
        oConsole.ErrorLine ( (CString) "_ Illegal <To_SubTick> Value at Line #"
                                      + I_to_A ( pxParse->iLineNum ) ) ;
	    lReturnStatus = Error ;
	    goto Done ;
    }

Done:
    return lReturnStatus ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess TT_Info_Action_Parse_Alignment_Delay     ( void* _pxParse, 
                                                    void* _pxCustom,
                                                    void* _pDlgThis )
{
    Parse_Structure*                pxParse     = (Parse_Structure*)                _pxParse ;
    Custom_Structure_TT_Info*       pxCustom    = (Custom_Structure_TT_Info*)       _pxCustom ;

    ESuccess        lReturnStatus = Ok ;

    unsigned char   ubAlignmentDelay ;
    ETT_Chan_Type   eTTChanType ;

    DEFINE_TT_COORD_RANGE_VAR () ;

    //\\//

    // check all other keywords have been assigned
    CHECK_TT_COORD_ASSIGNED   ( Alignment_Delay ) ;
    CHECK_TT_CHANNEL_ASSIGNED ( Alignment_Delay ) ;

    //Check the Alignment_Delay is not exceeding 4-bit maximum
    if ( pxParse->Values.ulAlignment_Delay > KiAdfMaxAlignmentDelay ) 
    {
        oConsole.ErrorLine ( (CString) "_ Alignment_Delay too large at Line #"
                                      + I_to_A ( pxParse->iLineNum ) ) ;
	    lReturnStatus = Error ;
	    goto Done ;
    }

    // retrieve the new value
    // including the global base value
    ubAlignmentDelay = fubTruncToByte ( pxParse->Values.ulAlignment_Delay ) ;

    //Check the final Alignment_Delay will not be exceeding 4-bit maximum
    if ( CL1Cal::mg_ubBaseAlignDelay + ubAlignmentDelay > KiAdfMaxAlignmentDelay ) 
    {
        oConsole.ErrorLine ( (CString) "_ Base Value will cause Alignement Delay overflow at Line #"
                                      + I_to_A ( pxParse->iLineNum ) ) ;
    }

    //retrieve the tower type
    eTTChanType = (ETT_Chan_Type) pxParse->Values.ulTT_Channel ;

    //Now loop over all values
    RETRIEVE_ALL_TT_COORD_RANGE ( ) ;
    LOOP_OVER_TT_COORD_RANGE ( ) 
    {
        // Increment Total Number of Alignement Delay Values written
        pxCustom->uiTotAlignmentDelay ++ ;
     
        // Remember the new value
        CL1Cal::mg_poL1Cal->
            m_a3poTrgTwr[ulTTEtaSign][ulTTEtaMagn][ulTTPhi]->
            m_ubRelAlignDelay[eTTChanType] 
                = ubAlignmentDelay ;

        // Are we just reading in the file, or also doing the IO
        if ( CL1Cal::mg_poL1Cal->mg_eynTtiImmediateIo == Yes )
        if ( CL1Cal::mg_poL1Cal->m_a3poTrgTwr[ulTTEtaSign]
                                             [ulTTEtaMagn]
                                             [ulTTPhi]
                               ->InitAlignmentDelay ( eTTChanType ) 
                               != Ok ) 
        {
            lReturnStatus = Error ;
            oConsole.ErrorLine ( (CString) " Alignment Delay Write failed for " 
                                         + CL1Cal_TrgTwr::f_osTrgTwr_Coord ( eTTChanType,
                                                                            ulTTEtaSign,
                                                                            ulTTEtaMagn,
                                                                            ulTTPhi ) ) ;
        }
   }

Done:
    return lReturnStatus ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess TT_Info_Action_Parse_Pedestal_DAC        ( void* _pxParse, 
                                                    void* _pxCustom,
                                                    void* _pDlgThis )
{
    Parse_Structure*                pxParse     = (Parse_Structure*)                _pxParse ;
    Custom_Structure_TT_Info*       pxCustom    = (Custom_Structure_TT_Info*)       _pxCustom ;

    ESuccess        lReturnStatus = Ok ;

    unsigned short  uwDacValue ;
    ETT_Chan_Type   eTTChanType ;

    DEFINE_TT_COORD_RANGE_VAR () ;

    //\\//

    // check all other keywords have been assigned
    CHECK_TT_COORD_ASSIGNED   ( Pedestal_DAC ) ;
    CHECK_TT_CHANNEL_ASSIGNED ( Pedestal_DAC ) ;

    //Check the Dac Value is not exceeding 12-bit maximum
    if ( pxParse->Values.ulPedestal_DAC > KiDacCmdMaxDacValue ) 
    {
        oConsole.ErrorLine ( (CString) "_ Pedestal_DAC too large at Line #"
                                      + I_to_A ( pxParse->iLineNum ) ) ;
	    lReturnStatus = Error ;
	    goto Done ;
    }

    //retrieve the new value 
    uwDacValue = fuwTruncToWord ( pxParse->Values.ulPedestal_DAC ) ;

    //retrieve the tower coordinates, and Dac Value
    eTTChanType = (ETT_Chan_Type) pxParse->Values.ulTT_Channel ;
    
    //Now loop over all values
    RETRIEVE_ALL_TT_COORD_RANGE ( ) ;
    LOOP_OVER_TT_COORD_RANGE ( ) 
    {
        // Increment Total Number of Dac Values written
        pxCustom->uiTotPedDacValue ++ ;
     
        // remember the previous Pedestal Control Value for later comparison
        CL1Cal::mg_poL1Cal->
            m_a3poTrgTwr[ulTTEtaSign][ulTTEtaMagn][ulTTPhi]->
            m_ulPrevPedCtrlDAC[eTTChanType] 
                = CL1Cal::mg_poL1Cal->
                    m_a3poTrgTwr[ulTTEtaSign][ulTTEtaMagn][ulTTPhi]->
                    m_ulPedestalControlDAC[eTTChanType] ;

        // remember the Current Pedestal Control DAC Value 
        CL1Cal::mg_poL1Cal->
            m_a3poTrgTwr[ulTTEtaSign][ulTTEtaMagn][ulTTPhi]->
            m_ulPedestalControlDAC[eTTChanType] 
                = uwDacValue ;

        // Are we just reading in the file, or also doing the IO?
        if ( CL1Cal::mg_poL1Cal->mg_eynTtiImmediateIo == Yes )
        if ( CL1Cal::mg_poL1Cal->m_a3poTrgTwr[ulTTEtaSign]
                                             [ulTTEtaMagn]
                                             [ulTTPhi]
                               ->InitPedestalDac ( eTTChanType ) 
                               != Ok ) 
        {
            lReturnStatus = Error ;
            oConsole.ErrorLine ( (CString) " Pedestal DAC Write failed for " 
                                         + CL1Cal_TrgTwr::f_osTrgTwr_Coord ( eTTChanType,
                                                                            ulTTEtaSign,
                                                                            ulTTEtaMagn,
                                                                            ulTTPhi ) ) ;
        }

    }

Done:
    return lReturnStatus ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess TT_Info_Action_Parse_Ped_Average         ( void* _pxParse, 
                                                    void* _pxCustom,
                                                    void* _pDlgThis )
{
    Parse_Structure*                pxParse     = (Parse_Structure*)                _pxParse ;
    Custom_Structure_TT_Info*       pxCustom    = (Custom_Structure_TT_Info*)       _pxCustom ;

    ESuccess        lReturnStatus = Ok ;

    ETT_Chan_Type   eTTChanType ;

    double          fdPed_Average ;

    DEFINE_TT_COORD_RANGE_VAR () ;

    //\\//

    // check all other keywords have been assigned
    CHECK_TT_COORD_ASSIGNED   ( Ped_Average ) ;
    CHECK_TT_CHANNEL_ASSIGNED ( Ped_Average ) ;

    //retrieve the new value 
    fdPed_Average = pxParse->Values.fdPed_Average ;

    //retrieve the tower type 
    eTTChanType = (ETT_Chan_Type) pxParse->Values.ulTT_Channel ;

    //Now loop over all values
    RETRIEVE_ALL_TT_COORD_RANGE ( ) ;
    LOOP_OVER_TT_COORD_RANGE ( ) 
    {
        // remember the new value
        CL1Cal::mg_poL1Cal->
            m_a3poTrgTwr[ulTTEtaSign][ulTTEtaMagn][ulTTPhi]->
            m_fPedestalAverage[eTTChanType] 
                = fdPed_Average ;
    }
Done:
    return lReturnStatus ;
}


/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess TT_Info_Action_Parse_Ped_Deviation       ( void* _pxParse, 
                                                    void* _pxCustom,
                                                    void* _pDlgThis )
{
    Parse_Structure*                pxParse     = (Parse_Structure*)                _pxParse ;
    Custom_Structure_TT_Info*       pxCustom    = (Custom_Structure_TT_Info*)       _pxCustom ;

    ESuccess        lReturnStatus = Ok ;

    ETT_Chan_Type   eTTChanType ;

    double          fdPed_Deviation ;

    DEFINE_TT_COORD_RANGE_VAR () ;

    //\\//

    // check all other keywords have been assigned
    CHECK_TT_COORD_ASSIGNED   ( Ped_Deviation ) ;
    CHECK_TT_CHANNEL_ASSIGNED ( Ped_Deviation ) ;

    //retrieve the new value 
    fdPed_Deviation = pxParse->Values.fdPed_Deviation ;

    //retrieve the tower type 
    eTTChanType = (ETT_Chan_Type) pxParse->Values.ulTT_Channel ;

    //Now loop over all values
    RETRIEVE_ALL_TT_COORD_RANGE ( ) ;
    LOOP_OVER_TT_COORD_RANGE ( ) 
    {
        // remember the previous Pedestal Standard Deviation for later comparison
        CL1Cal::mg_poL1Cal->
            m_a3poTrgTwr[ulTTEtaSign][ulTTEtaMagn][ulTTPhi]->
            m_fPrevPedStdDev[eTTChanType] 
                = CL1Cal::mg_poL1Cal->
                    m_a3poTrgTwr[ulTTEtaSign][ulTTEtaMagn][ulTTPhi]->
                    m_fPedestalStdDev[eTTChanType] ;

        // remember the Current Histogram Standard Deviation
        CL1Cal::mg_poL1Cal->
            m_a3poTrgTwr[ulTTEtaSign][ulTTEtaMagn][ulTTPhi]->
            m_fPedestalStdDev[eTTChanType] 
                = fdPed_Deviation ;
    }
Done:
    return lReturnStatus ;
}




/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess TT_Info_Action_Parse_Zero_Energy_Adc_E       ( void* _pxParse, 
                                                        void* _pxCustom,
                                                        void* _pDlgThis )
{
    Parse_Structure*                pxParse     = (Parse_Structure*)                _pxParse ;
    Custom_Structure_TT_Info*       pxCustom    = (Custom_Structure_TT_Info*)       _pxCustom ;

    ESuccess        lReturnStatus = Ok ;

    ETT_Chan_Type   eTTChanType ;

    unsigned short  uwZeroEngAdcE ;

    DEFINE_TT_COORD_RANGE_VAR () ;

    //\\//

    // verify the Lookup Input Zero Energy Response is within rational range
    if ( pxParse->Values.ulZeroEngAdcE > KiMaxLookupZeroEngAdcE )
    {
        oConsole.ErrorLine ( (CString) "_ Illegal Et Lookup <Zero_Energy_Adc_E> Value at Line #"
                                      + I_to_A ( pxParse->iLineNum ) ) ;
	    lReturnStatus = Error ;
	    goto Done ;
    }

    // check all other keywords have been assigned
    CHECK_TT_COORD_ASSIGNED   ( Zero_Energy_Adc_E ) ;
    CHECK_TT_CHANNEL_ASSIGNED ( Zero_Energy_Adc_E ) ;

    //retrieve the new value 
    uwZeroEngAdcE = fuwTruncToWord ( pxParse->Values.ulZeroEngAdcE ) ;

    //retrieve the tower type 
    eTTChanType = (ETT_Chan_Type) pxParse->Values.ulTT_Channel ;

    //Now loop over all values
    RETRIEVE_ALL_TT_COORD_RANGE ( ) ;
    LOOP_OVER_TT_COORD_RANGE ( ) 
    {
        // remember the new value for lookup
        CL1Cal::mg_poL1Cal->
            m_a3poTrgTwr[ulTTEtaSign][ulTTEtaMagn][ulTTPhi]->
                m_axEtLookupCoeff[eTTChanType].uwZeroEngAdcE  
                    = uwZeroEngAdcE ;

        // remember the new value for Find_DAC
        CL1Cal::mg_poL1Cal->
            m_a3poTrgTwr[ulTTEtaSign][ulTTEtaMagn][ulTTPhi]->m_uwRawAdcZer  = uwZeroEngAdcE ;
    }

Done:
    return lReturnStatus ;
}
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess TT_Info_Action_Parse_Zero_Energy_Output_Et   ( void* _pxParse, 
                                                        void* _pxCustom,
                                                        void* _pDlgThis )
{
    Parse_Structure*                pxParse     = (Parse_Structure*)                _pxParse ;
    Custom_Structure_TT_Info*       pxCustom    = (Custom_Structure_TT_Info*)       _pxCustom ;

    ESuccess        lReturnStatus = Ok ;

    ETT_Chan_Type   eTTChanType ;

    unsigned short  uwZeroEngOutEt ;

    DEFINE_TT_COORD_RANGE_VAR () ;

    //\\//

    // verify the Lookup Output Zero Energy Response is within rational range
    if ( pxParse->Values.ulZeroEngOutEt > KiMaxLookupZeroEngOutEt )
    {
        oConsole.ErrorLine ( (CString) "_ Illegal Et Lookup <Zero_Energy_Output_Et> Value at Line #"
                                      + I_to_A ( pxParse->iLineNum ) ) ;
	    lReturnStatus = Error ;
	    goto Done ;
    }

    // check all other keywords have been assigned
    CHECK_TT_COORD_ASSIGNED   ( Zero_Energy_Output_Et ) ;
    CHECK_TT_CHANNEL_ASSIGNED ( Zero_Energy_Output_Et ) ;

    //retrieve the new value 
    uwZeroEngOutEt = fuwTruncToWord ( pxParse->Values.ulZeroEngOutEt ) ;

    //retrieve the tower type 
    eTTChanType = (ETT_Chan_Type) pxParse->Values.ulTT_Channel ;

    //Now loop over all values
    RETRIEVE_ALL_TT_COORD_RANGE ( ) ;
    LOOP_OVER_TT_COORD_RANGE ( ) 
    {
        // remember the new value
        CL1Cal::mg_poL1Cal->
            m_a3poTrgTwr[ulTTEtaSign][ulTTEtaMagn][ulTTPhi]->
            m_axEtLookupCoeff[eTTChanType].uwZeroEngOutEt  
                = uwZeroEngOutEt ;
    }

Done:
    return lReturnStatus ;
}
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess TT_Info_Action_Parse_Lookup_Slope        ( void* _pxParse, 
                                                    void* _pxCustom,
                                                    void* _pDlgThis )
{
    Parse_Structure*                pxParse     = (Parse_Structure*)                _pxParse ;
    Custom_Structure_TT_Info*       pxCustom    = (Custom_Structure_TT_Info*)       _pxCustom ;

    ESuccess        lReturnStatus = Ok ;

    ETT_Chan_Type   eTTChanType ;

    double          fdLookupSlope ;

    DEFINE_TT_COORD_RANGE_VAR () ;

    //\\//

    // verify the Et Lookup Slope is within rational range
    if ( ( pxParse->Values.fdLookup_Slope < 0.0 )
      || ( pxParse->Values.fdLookup_Slope > KfMaxLookupSlope ) )
    {
        oConsole.ErrorLine ( (CString) "_ Illegal Et Lookup <Lookup_Slope> Value at Line #"
                                      + I_to_A ( pxParse->iLineNum ) ) ;
	    lReturnStatus = Error ;
	    goto Done ;
    }

    // check all other keywords have been assigned
    CHECK_TT_COORD_ASSIGNED   ( Lookup_Slope ) ;
    CHECK_TT_CHANNEL_ASSIGNED ( Lookup_Slope ) ;

    //retrieve the new value 
    fdLookupSlope = pxParse->Values.fdLookup_Slope ;

    //retrieve the tower type 
    eTTChanType = (ETT_Chan_Type) pxParse->Values.ulTT_Channel ;

    //Now loop over all values
    RETRIEVE_ALL_TT_COORD_RANGE ( ) ;
    LOOP_OVER_TT_COORD_RANGE ( ) 
    {
        // Increment Total Number of Lookup Slope Values written
        pxCustom->uiTotLookupSlope ++ ;

        // remember the new value
        CL1Cal::mg_poL1Cal->
            m_a3poTrgTwr[ulTTEtaSign][ulTTEtaMagn][ulTTPhi]->
            m_axEtLookupCoeff[eTTChanType].fdLookupSlope 
                = fdLookupSlope ;

        // Are we just reading in the file, or also doing the IO?
        if ( CL1Cal::mg_poL1Cal->mg_eynTtiImmediateIo == Yes )
        if ( CL1Cal::mg_poL1Cal->m_a3poTrgTwr[ulTTEtaSign]
                                             [ulTTEtaMagn]
                                             [ulTTPhi]
                               ->InitEtLookup  ( eTTChanType ) 
                               != Ok ) 
        {
            lReturnStatus = Error ;
            oConsole.ErrorLine ( (CString) " Writing Et Lookup Memory failed for " 
                                         + CL1Cal_TrgTwr::f_osTrgTwr_Coord ( eTTChanType,
                                                                            ulTTEtaSign,
                                                                            ulTTEtaMagn,
                                                                            ulTTPhi ) ) ;
        }
    }

Done:
    return lReturnStatus ;
}
/*-------------------------------------------------------------------------------*/


/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess TT_Info_Action_Parse_RawAdc_Simu         ( void* _pxParse, 
                                                    void* _pxCustom,
                                                    void* _pDlgThis )
{
    Parse_Structure*                pxParse     = (Parse_Structure*)                _pxParse ;
    Custom_Structure_TT_Info*       pxCustom    = (Custom_Structure_TT_Info*)       _pxCustom ;

    ESuccess        lReturnStatus = Ok ;

    ETT_Chan_Type   eTTChanType ;

    unsigned short  uwRawAdc_Simu ;

    DEFINE_TT_COORD_RANGE_VAR () ;
    DEFINE_TICK_RANGE_VAR () ;
    DEFINE_SUBTICK_RANGE_VAR () ;

    //\\//

    // verify the Raw ADC is within rational range
    if ( ( pxParse->Values.ulRawAdc_Simu > KiMaxLookupZeroEngAdcE ) )
    {
        oConsole.ErrorLine ( (CString) "_ Illegal <RawAdc_Simu> Value at Line #"
                                      + I_to_A ( pxParse->iLineNum ) ) ;
	    lReturnStatus = Error ;
	    goto Done ;
    }

    // check all other keywords have been assigned
    CHECK_TT_COORD_ASSIGNED   ( RawAdc_Simu ) ;
    CHECK_TT_CHANNEL_ASSIGNED ( RawAdc_Simu ) ;

    //retrieve the new value 
    uwRawAdc_Simu = fuwTruncToWord ( pxParse->Values.ulRawAdc_Simu );

    //retrieve the tower type 
    eTTChanType = (ETT_Chan_Type) pxParse->Values.ulTT_Channel ;

    //Now loop over all values
    RETRIEVE_ALL_TT_COORD_RANGE ( ) ;
    RETRIEVE_TICK_RANGE ( ) ;
    RETRIEVE_SUBTICK_RANGE ( ) ;

    LOOP_OVER_TT_COORD_RANGE ( ) 
    LOOP_OVER_TICK_RANGE ( ) 
    LOOP_OVER_SUBTICK_RANGE ( ) 
    {
        // Increment Total Number of Raw ADC Values written
        pxCustom->uiTotRawAdc_Simu ++ ;

        // remember the new value
        CL1Cal::mg_poL1Cal->
            m_a3poTrgTwr[ulTTEtaSign][ulTTEtaMagn][ulTTPhi]->
            m_uwSimuRawAdc[eTTChanType][ulTickNum][ulSubTickNum]
                = uwRawAdc_Simu ;

        // Are we just reading in the file, or also doing the IO?
        if ( CL1Cal::mg_poL1Cal->mg_eynTtiImmediateIo == Yes )
        if ( CL1Cal::mg_poL1Cal->m_a3poTrgTwr[ulTTEtaSign]
                                             [ulTTEtaMagn]
                                             [ulTTPhi]
                               ->f_okSetSimuRawAdc ( eTTChanType, NULL, &ulTickNum, &ulSubTickNum ) 
                               != Ok ) 
        {
            lReturnStatus = Error ;
            oConsole.ErrorLine ( (CString) " Writing Raw ADC failed for " 
                                         + CL1Cal_TrgTwr::f_osTrgTwr_Coord ( eTTChanType,
                                                                            ulTTEtaSign,
                                                                            ulTTEtaMagn,
                                                                            ulTTPhi ) 
                                         + " Tick #"
                                         + I_to_A ( ulTickNum, 3 ) 
                                         + " SubTick #"
                                         + I_to_A ( ulSubTickNum ) ) ;
        }
    }

Done:
    return lReturnStatus ;
}
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess TT_Info_Action_Parse_AdfOut_Simu         ( void* _pxParse, 
                                                    void* _pxCustom,
                                                    void* _pDlgThis )
{
    Parse_Structure*                pxParse     = (Parse_Structure*)                _pxParse ;
    Custom_Structure_TT_Info*       pxCustom    = (Custom_Structure_TT_Info*)       _pxCustom ;

    ESuccess        lReturnStatus = Ok ;

    ETT_Chan_Type   eTTChanType ;

    unsigned char   ubAdfOut_Simu ;

    DEFINE_TT_COORD_RANGE_VAR () ;
    DEFINE_TICK_RANGE_VAR () ;

    //\\//

    // verify the Output Et is within rational range
    if ( ( pxParse->Values.ulAdfOut_Simu > KiMaxLookupZeroEngOutEt ) )
    {
        oConsole.ErrorLine ( (CString) "_ Illegal <AdfOut_Simu> Value at Line #"
                                      + I_to_A ( pxParse->iLineNum ) ) ;
	    lReturnStatus = Error ;
	    goto Done ;
    }

    // check all other keywords have been assigned
    CHECK_TT_COORD_ASSIGNED   ( AdfOut_Simu ) ;
    CHECK_TT_CHANNEL_ASSIGNED ( AdfOut_Simu ) ;

    //retrieve the new value 
    ubAdfOut_Simu = fubTruncToByte( pxParse->Values.ulAdfOut_Simu ) ;

    //retrieve the tower type 
    eTTChanType = (ETT_Chan_Type) pxParse->Values.ulTT_Channel ;

    //Now loop over all values
    RETRIEVE_ALL_TT_COORD_RANGE ( ) ;
    RETRIEVE_TICK_RANGE ( ) ;
    LOOP_OVER_TT_COORD_RANGE ( ) 
    LOOP_OVER_TICK_RANGE ( ) 
    {
        // Increment Total Number of Ouptut Et Values written
        pxCustom->uiTotAdfOut_Simu ++ ;

        // remember the new value
        CL1Cal::mg_poL1Cal->
            m_a3poTrgTwr[ulTTEtaSign][ulTTEtaMagn][ulTTPhi]->
            m_ubSimuOutputEt[eTTChanType][ulTickNum]
                = ubAdfOut_Simu ;

        // Are we just reading in the file, or also doing the IO?
        if ( CL1Cal::mg_poL1Cal->mg_eynTtiImmediateIo == Yes )
        if ( CL1Cal::mg_poL1Cal->m_a3poTrgTwr[ulTTEtaSign]
                                             [ulTTEtaMagn]
                                             [ulTTPhi]
                               ->f_okSetSimuOutputEt( eTTChanType, NULL, &ulTickNum ) 
                               != Ok ) 
        {
            lReturnStatus = Error ;
            oConsole.ErrorLine ( (CString) " Writing Output Et failed for " 
                                         + CL1Cal_TrgTwr::f_osTrgTwr_Coord ( eTTChanType,
                                                                            ulTTEtaSign,
                                                                            ulTTEtaMagn,
                                                                            ulTTPhi ) 
                                         + " Tick #"
                                         + I_to_A ( ulTickNum, 3 ) ) ;
        }
    }

Done:
    return lReturnStatus ;
}
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess TT_Info_Action_Parse_AdfOut_Const        ( void* _pxParse, 
                                                    void* _pxCustom,
                                                    void* _pDlgThis )
{
    Parse_Structure*                pxParse     = (Parse_Structure*)                _pxParse ;
    Custom_Structure_TT_Info*       pxCustom    = (Custom_Structure_TT_Info*)       _pxCustom ;

    ESuccess        lReturnStatus = Ok ;

    ETT_Chan_Type   eTTChanType ;

    unsigned char   ubAdfOut_Const ;

    DEFINE_TT_COORD_RANGE_VAR () ;

    //\\//

    // verify the Output Et is within rational range
    if ( ( pxParse->Values.ulAdfOut_Const > 0xff ) )
    {
        oConsole.ErrorLine ( (CString) "_ Illegal <AdfOut_Const> Value at Line #"
                                      + I_to_A ( pxParse->iLineNum ) ) ;
	    lReturnStatus = Error ;
	    goto Done ;
    }

    // check all other keywords have been assigned
    CHECK_TT_COORD_ASSIGNED   ( AdfOut_Const ) ;
    CHECK_TT_CHANNEL_ASSIGNED ( AdfOut_Const ) ;

    //retrieve the new value 
    ubAdfOut_Const = fubTruncToByte( pxParse->Values.ulAdfOut_Const ) ;

    //retrieve the tower type 
    eTTChanType = (ETT_Chan_Type) pxParse->Values.ulTT_Channel ;

    //Now loop over all values
    RETRIEVE_ALL_TT_COORD_RANGE ( ) ;
    LOOP_OVER_TT_COORD_RANGE ( ) 
    {
        // Increment Total Number of Ouptut Et Values written
        pxCustom->uiTotAdfOut_Const ++ ;

        // Now do it
        if ( CL1Cal::mg_poL1Cal->m_a3poTrgTwr[ulTTEtaSign]
                                             [ulTTEtaMagn]
                                             [ulTTPhi]
                               ->f_okOutputConstant   ( eTTChanType, ubAdfOut_Const ) 
                               != Ok ) 
        {
            lReturnStatus = Error ;
            oConsole.ErrorLine ( (CString) " Writing Output Et Constant failed for " 
                                         + CL1Cal_TrgTwr::f_osTrgTwr_Coord ( eTTChanType,
                                                                            ulTTEtaSign,
                                                                            ulTTEtaMagn,
                                                                            ulTTPhi ) ) ;
        }
    }

Done:
    return lReturnStatus ;
}
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess TT_Info_Action_Parse_RawAdc_Thresh       ( void* _pxParse, 
                                                    void* _pxCustom,
                                                    void* _pDlgThis )
{
    Parse_Structure*                pxParse     = (Parse_Structure*)                _pxParse ;
    Custom_Structure_TT_Info*       pxCustom    = (Custom_Structure_TT_Info*)       _pxCustom ;

    ESuccess        lReturnStatus = Ok ;

    ETT_Chan_Type   eTTChanType ;

    unsigned char   ubRawAdc_Thresh ;

    DEFINE_TT_COORD_RANGE_VAR () ;

    //\\//

    // verify the Output Et is within rational range
    if ( ( pxParse->Values.ulRawAdc_Thresh > 0xff ) )
    {
        oConsole.ErrorLine ( (CString) "_ Illegal <RawAdc_Thresh> Value at Line #"
                                      + I_to_A ( pxParse->iLineNum ) ) ;
	    lReturnStatus = Error ;
	    goto Done ;
    }

    // check all other keywords have been assigned
    CHECK_TT_COORD_ASSIGNED   ( RawAdc_Thresh ) ;
    CHECK_TT_CHANNEL_ASSIGNED ( RawAdc_Thresh ) ;

    //retrieve the new value 
    ubRawAdc_Thresh = fubTruncToByte( pxParse->Values.ulRawAdc_Thresh ) ;

    //retrieve the tower type 
    eTTChanType = (ETT_Chan_Type) pxParse->Values.ulTT_Channel ;

    //Now loop over all values
    RETRIEVE_ALL_TT_COORD_RANGE ( ) ;
    LOOP_OVER_TT_COORD_RANGE ( ) 
    {
        // Increment Total Number of Ouptut Et Values written
        pxCustom->uiTotRawAdc_Thresh ++ ;

        // Now do it
        if ( CL1Cal::mg_poL1Cal->m_a3poTrgTwr[ulTTEtaSign]
                                             [ulTTEtaMagn]
                                             [ulTTPhi]
                               ->f_okRawAdcThresh ( eTTChanType, ubRawAdc_Thresh ) 
                               != Ok ) 
        {
            lReturnStatus = Error ;
            oConsole.ErrorLine ( (CString) " Writing Raw ADC Threshold failed for " 
                                         + CL1Cal_TrgTwr::f_osTrgTwr_Coord ( eTTChanType,
                                                                            ulTTEtaSign,
                                                                            ulTTEtaMagn,
                                                                            ulTTPhi ) ) ;
        }
    }

Done:
    return lReturnStatus ;
}
/*-------------------------------------------------------------------------------*/


/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess TT_Info_Action_Parse_RawAdc_MuxMode      ( void* _pxParse, 
                                                    void* _pxCustom,
                                                    void* _pDlgThis )
{
    Parse_Structure*                pxParse     = (Parse_Structure*)                _pxParse ;
    Custom_Structure_TT_Info*       pxCustom    = (Custom_Structure_TT_Info*)       _pxCustom ;

    ESuccess        lReturnStatus = Ok ;

    ETT_Chan_Type           eTTChanType ;
    ERawAdc_MuxMode_Type    eRawAdc_MuxMode ;

    DEFINE_TT_COORD_RANGE_VAR () ;

    //\\//

    // verify the Raw ADC is within rational range
    if ( pxParse->Values.ulRawAdc_MuxMode > eTot_RawAdc_MuxMode_Type )
    {
        oConsole.ErrorLine ( (CString) "_ Illegal <RawAdc_MuxMode> Value at Line #"
                                      + I_to_A ( pxParse->iLineNum ) ) ;
	    lReturnStatus = Error ;
	    goto Done ;
    }

    // check all other keywords have been assigned
    CHECK_TT_COORD_ASSIGNED   ( RawAdc_MuxMode ) ;
    CHECK_TT_CHANNEL_ASSIGNED ( RawAdc_MuxMode ) ;

    //retrieve the new value 
    eRawAdc_MuxMode = (ERawAdc_MuxMode_Type) pxParse->Values.ulRawAdc_MuxMode ;

    //retrieve the tower type 
    eTTChanType = (ETT_Chan_Type) pxParse->Values.ulTT_Channel ;

    //Now loop over all values
    RETRIEVE_ALL_TT_COORD_RANGE ( ) ;

    LOOP_OVER_TT_COORD_RANGE ( ) 
    {
        // Increment Total Number of Raw ADC Values written
        pxCustom->uiTotRawAdc_MuxMode ++ ;

        // remember the new value
        if ( CL1Cal::mg_poL1Cal->m_a3poTrgTwr[ulTTEtaSign]
                                             [ulTTEtaMagn]
                                             [ulTTPhi]
                               ->f_okRawAdc_MuxMode( eTTChanType, eRawAdc_MuxMode ) != Ok ) 
        {
            lReturnStatus = Error ;
            oConsole.ErrorLine ( (CString) " SetRaw ADC Mux Type failed for " 
                                         + CL1Cal_TrgTwr::f_osTrgTwr_Coord ( eTTChanType,
                                                                            ulTTEtaSign,
                                                                            ulTTEtaMagn,
                                                                            ulTTPhi ) ) ;
        }
    }

Done:
    return lReturnStatus ;
}
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess TT_Info_Action_Parse_RawAdc_MemMode      ( void* _pxParse, 
                                                    void* _pxCustom,
                                                    void* _pDlgThis )
{
    Parse_Structure*                pxParse     = (Parse_Structure*)                _pxParse ;
    Custom_Structure_TT_Info*       pxCustom    = (Custom_Structure_TT_Info*)       _pxCustom ;

    ESuccess        lReturnStatus = Ok ;

    ETT_Chan_Type           eTTChanType ;
    ERawAdc_MemMode_Type    eRawAdc_MemMode ;

    DEFINE_TT_COORD_RANGE_VAR () ;

    //\\//

    // verify the Raw ADC is within rational range
    if ( pxParse->Values.ulRawAdc_MemMode > eTot_RawAdc_MemMode_Type )
    {
        oConsole.ErrorLine ( (CString) "_ Illegal <RawAdc_MemMode> Value at Line #"
                                      + I_to_A ( pxParse->iLineNum ) ) ;
	    lReturnStatus = Error ;
	    goto Done ;
    }

    // check all other keywords have been assigned
    CHECK_TT_COORD_ASSIGNED   ( RawAdc_MemMode ) ;
    CHECK_TT_CHANNEL_ASSIGNED ( RawAdc_MemMode ) ;

    //retrieve the new value 
    eRawAdc_MemMode = (ERawAdc_MemMode_Type) pxParse->Values.ulRawAdc_MemMode ;

    //retrieve the tower type 
    eTTChanType = (ETT_Chan_Type) pxParse->Values.ulTT_Channel ;

    //Now loop over all values
    RETRIEVE_ALL_TT_COORD_RANGE ( ) ;

    LOOP_OVER_TT_COORD_RANGE ( ) 
    {
        // Increment Total Number of Raw ADC Values written
        pxCustom->uiTotRawAdc_MemMode ++ ;

        // remember the new value
        if ( CL1Cal::mg_poL1Cal->m_a3poTrgTwr[ulTTEtaSign]
                                             [ulTTEtaMagn]
                                             [ulTTPhi]
                               ->f_okRawAdc_MemMode( eTTChanType, eRawAdc_MemMode ) != Ok ) 
        {
            lReturnStatus = Error ;
            oConsole.ErrorLine ( (CString) " SetRaw ADC Mux Type failed for " 
                                         + CL1Cal_TrgTwr::f_osTrgTwr_Coord ( eTTChanType,
                                                                            ulTTEtaSign,
                                                                            ulTTEtaMagn,
                                                                            ulTTPhi ) ) ;
        }
    }

Done:
    return lReturnStatus ;
}
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess TT_Info_Action_Parse_AdfOut_MuxMode      ( void* _pxParse, 
                                                    void* _pxCustom,
                                                    void* _pDlgThis )
{
    Parse_Structure*                pxParse     = (Parse_Structure*)                _pxParse ;
    Custom_Structure_TT_Info*       pxCustom    = (Custom_Structure_TT_Info*)       _pxCustom ;

    ESuccess        lReturnStatus = Ok ;

    ETT_Chan_Type           eTTChanType ;
    EAdfOut_MuxMode_Type    eAdfOut_MuxMode ;

    DEFINE_TT_COORD_RANGE_VAR () ;

    //\\//

    // verify the Raw ADC is within rational range
    if ( pxParse->Values.ulAdfOut_MuxMode > eTot_AdfOut_MuxMode_Type )
    {
        oConsole.ErrorLine ( (CString) "_ Illegal <AdfOut_MuxMode> Value at Line #"
                                      + I_to_A ( pxParse->iLineNum ) ) ;
	    lReturnStatus = Error ;
	    goto Done ;
    }

    // check all other keywords have been assigned
    CHECK_TT_COORD_ASSIGNED   ( AdfOut_MuxMode ) ;
    CHECK_TT_CHANNEL_ASSIGNED ( AdfOut_MuxMode ) ;

    //retrieve the new value 
    eAdfOut_MuxMode = (EAdfOut_MuxMode_Type) pxParse->Values.ulAdfOut_MuxMode ;

    //retrieve the tower type 
    eTTChanType = (ETT_Chan_Type) pxParse->Values.ulTT_Channel ;

    //Now loop over all values
    RETRIEVE_ALL_TT_COORD_RANGE ( ) ;

    LOOP_OVER_TT_COORD_RANGE ( ) 
    {
        // Increment Total Number of Raw ADC Values written
        pxCustom->uiTotAdfOut_MuxMode ++ ;

        // remember the new value
        if ( CL1Cal::mg_poL1Cal->m_a3poTrgTwr[ulTTEtaSign]
                                             [ulTTEtaMagn]
                                             [ulTTPhi]
                               ->f_okAdfOut_MuxMode( eTTChanType, eAdfOut_MuxMode ) != Ok ) 
        {
            lReturnStatus = Error ;
            oConsole.ErrorLine ( (CString) " SetRaw ADC Mux Type failed for " 
                                         + CL1Cal_TrgTwr::f_osTrgTwr_Coord ( eTTChanType,
                                                                            ulTTEtaSign,
                                                                            ulTTEtaMagn,
                                                                            ulTTPhi ) ) ;
        }
    }

Done:
    return lReturnStatus ;
}
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess TT_Info_Action_Parse_AdfOut_MemMode      ( void* _pxParse, 
                                                    void* _pxCustom,
                                                    void* _pDlgThis )
{
    Parse_Structure*                pxParse     = (Parse_Structure*)                _pxParse ;
    Custom_Structure_TT_Info*       pxCustom    = (Custom_Structure_TT_Info*)       _pxCustom ;

    ESuccess        lReturnStatus = Ok ;

    ETT_Chan_Type               eTTChanType ;
    EAdfOut_MemMode_Type        eAdfOut_MemMode ;

    DEFINE_TT_COORD_RANGE_VAR () ;

    //\\//

    // verify the Raw ADC is within rational range
    if ( pxParse->Values.ulAdfOut_MemMode > eTot_AdfOut_MemMode_Type )
    {
        oConsole.ErrorLine ( (CString) "_ Illegal <AdfOut_MemMode> Value at Line #"
                                      + I_to_A ( pxParse->iLineNum ) ) ;
	    lReturnStatus = Error ;
	    goto Done ;
    }

    // check all other keywords have been assigned
    CHECK_TT_COORD_ASSIGNED   ( AdfOut_MemMode ) ;
    CHECK_TT_CHANNEL_ASSIGNED ( AdfOut_MemMode ) ;

    //retrieve the new value 
    eAdfOut_MemMode = (EAdfOut_MemMode_Type) pxParse->Values.ulAdfOut_MemMode ;

    //retrieve the tower type 
    eTTChanType = (ETT_Chan_Type) pxParse->Values.ulTT_Channel ;

    //Now loop over all values
    RETRIEVE_ALL_TT_COORD_RANGE ( ) ;

    LOOP_OVER_TT_COORD_RANGE ( ) 
    {
        // Increment Total Number of Raw ADC Values written
        pxCustom->uiTotAdfOut_MemMode ++ ;

        // remember the new value
        if ( CL1Cal::mg_poL1Cal->m_a3poTrgTwr[ulTTEtaSign]
                                             [ulTTEtaMagn]
                                             [ulTTPhi]
                               ->f_okAdfOut_MemMode( eTTChanType, eAdfOut_MemMode ) != Ok ) 
        {
            lReturnStatus = Error ;
            oConsole.ErrorLine ( (CString) " SetRaw ADC Mux Type failed for " 
                                         + CL1Cal_TrgTwr::f_osTrgTwr_Coord ( eTTChanType,
                                                                            ulTTEtaSign,
                                                                            ulTTEtaMagn,
                                                                            ulTTPhi ) ) ;
        }
    }

Done:
    return lReturnStatus ;
}
/*-------------------------------------------------------------------------------*/

