/*-------------------------------------------------------------------------------*/
  /*
    CommandFileVmeIO.cpp
    8-Oct-2005
  */
/*-------------------------------------------------------------------------------*/
#include "L1Cal_IIb_TCC/StdAfx.h"

#include "L1Cal_IIb_TCC/HandleBit3Adaptor61x.h"

#include "L1Cal_IIb_TCC/HandleCommandFiles.h"

#include                        "L1Cal_IIb_TCC/CommandFileVmeIO.h"

//#include "L1Cal_IIb_TCC/AdfRegBase.h"

#include "L1Cal_IIb_TCC/UtilsStrings.h"
#include "L1Cal_IIb_TCC/UtilsBits.h"

// static means created only once throughout life of program (to avoid wasting time)
// *AND* internal linkage (to avoid clash between modules)
#include "L1Cal_IIb_TCC/HandleConsole.h"
static CConsoleAccess oConsole ;

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess CommandFileVmeIO         ( CString	                        _osCommand_File_Name,
                                    Custom_Structure_VmeIO*         _pxCustom, 
                                    void*                           _pDlgThis )
{
    ESuccess            eReturnStatus = Ok ;

    ESuccess            eSuccess ;
    Parse_Structure     xParse ;

    CString             oMsg ;

    //\\//

    // write to console 
	oConsole.WriteLine ( "" ) ;
	oConsole.WriteLine ( (CString) "%% VME IO Command  File : "
                        		 +  _osCommand_File_Name ) ;

    // Initialize the Command File Parser control Structure
    eSuccess = Init_Parse_Structure ( &xParse ) ;

    xParse.pacFile_Name = LPCTSTR(_osCommand_File_Name) ;

    //register the legal keywords and 
    //the call back points for the apropriate keywords
    xParse.Actions.pfFile_Opened      = (fKeyword_Action*) VmeIO_Action_File_Opened ;
    xParse.Actions.pfBit3_Adaptor     = (fKeyword_Action*) VmeIO_Action_Bit3_Adaptor ;
    xParse.Actions.pfAddress_Space    = (fKeyword_Action*) VmeIO_Action_Address_Space ;
    xParse.Actions.pfData_Size        = (fKeyword_Action*) VmeIO_Action_Data_Size ;
    xParse.Actions.pfVME_Address      = NULL ;
    xParse.Actions.pfRead_VME         = (fKeyword_Action*) VmeIO_Action_Parse_Read_VME ;
    xParse.Actions.pfWrite_Value      = (fKeyword_Action*) VmeIO_Action_Parse_Write_Value ;
    xParse.Actions.pfWrite_Verify     = (fKeyword_Action*) VmeIO_Action_Parse_Write_Verify ;
    xParse.Actions.pfDone_With_File   = (fKeyword_Action*) VmeIO_Action_Done_With_File ;

    // set default settings
    xParse.Values.ulAddress_Space = KiCodeA32 ;
    xParse.Values.ulData_Size = KiCodeD16 ; 
    xParse.Values.ulBit3_Adaptor = CBit3Adaptor61x::mg_iAdaptorNumForL1Cal ;

    _pxCustom->eSwapping = No ; //fixed
    _pxCustom->osAccessMethod = KsAPI ; //fixed
    
    // Initialize Bit3 Apdaptor Pointer
    _pxCustom->poBit3Adaptor = NULL  ;

    // Initialize Total Number of register IO performed
    _pxCustom->uiTotReg = 0 ;


    // call the general purpose command file parser 
    ///////////////////////////////////////////////
    if ( Read_Command_File ( &xParse, _pxCustom, _pDlgThis ) != Ok )
	    eReturnStatus = Error ;


    // release the Bit3 Adaptor 
    if ( _pxCustom->poBit3Adaptor != NULL )
        delete _pxCustom->poBit3Adaptor ;


    if ( eReturnStatus == Ok  )
    {
        oMsg = "%%Done after Executing   "
             + I_to_A ( _pxCustom->uiTotReg )
             + " IO Commands" ;
        oConsole.WriteLine ( oMsg ) ;
    }
    else
    {
        oMsg = "%%Abort after Executing   "
             + I_to_A ( _pxCustom->uiTotReg )
             + " IO Commands" ;
        oConsole.ErrorLine ( oMsg ) ;
    }

    return eReturnStatus ;
}


/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/


/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess VmeIO_Action_File_Opened                  ( void* _pxParse, 
                                                     void* _pxCustom,
                                                     void* _pDlgThis )
{
    Parse_Structure*                pxParse     = (Parse_Structure*)                _pxParse ;
    Custom_Structure_VmeIO*         pxCustom    = (Custom_Structure_VmeIO*)         _pxCustom ;

    ESuccess        lReturnStatus = Ok ;

    //\\//
 
//Done:
    return lReturnStatus ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess VmeIO_Action_Bit3_Adaptor                 ( void* _pxParse, 
                                                     void* _pxCustom,
                                                     void* _pDlgThis )
{
    Parse_Structure*                pxParse     = (Parse_Structure*)                _pxParse ;
    Custom_Structure_VmeIO*         pxCustom    = (Custom_Structure_VmeIO*)         _pxCustom ;

    ESuccess        lReturnStatus = Ok ;

    //\\//
 
    // check Adaptor Number is in the right general range
    if ( pxParse->Values.ulBit3_Adaptor > KiMaxBit3AdaptorNumber ) 
    {
        oConsole.ErrorLine ( (CString) "_ Illegal Bit3_Adaptor Value at Line #"
                                      + I_to_A ( pxParse->iLineNum ) ) ;
	    lReturnStatus = Error ;
	    goto Done ;
    }

    //Switching to requested adaptor number (if it has changed)
    //will happen in VmeIO_Check_Ready_For_Access

Done:
    return lReturnStatus ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess VmeIO_Action_Address_Space                ( void* _pxParse, 
                                                     void* _pxCustom,
                                                     void* _pDlgThis )
{
    Parse_Structure*                pxParse     = (Parse_Structure*)                _pxParse ;
    Custom_Structure_VmeIO*         pxCustom    = (Custom_Structure_VmeIO*)         _pxCustom ;

    ESuccess        lReturnStatus = Ok ;

    //\\//

    //retrieve the requested Address Space Type
    if ( ( pxParse->Values.ulAddress_Space != KiCodeA32 )
      && ( pxParse->Values.ulAddress_Space != KiCodeA24 )
      && ( pxParse->Values.ulAddress_Space != KiCodeA16 )
      && ( pxParse->Values.ulAddress_Space != KiCodeDpm ) )
    {
        oConsole.ErrorLine ( (CString) "_ Illegal Address_Space Value at Line #"
                                      + I_to_A ( pxParse->iLineNum ) ) ;
	    lReturnStatus = Error ;
	    goto Done ;
    }

    //switching to requested Address Space (if it has changed)
    //will happen in VmeIO_Check_Ready_For_Access

Done:
    return lReturnStatus ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess VmeIO_Action_Data_Size                    ( void* _pxParse, 
                                                     void* _pxCustom,
                                                     void* _pDlgThis )
{
    Parse_Structure*                pxParse     = (Parse_Structure*)                _pxParse ;
    Custom_Structure_VmeIO*         pxCustom    = (Custom_Structure_VmeIO*)         _pxCustom ;

    ESuccess        lReturnStatus = Ok ;

    //\\//
 
    //Check the requested Data Size Type
    if ( ( pxParse->Values.ulData_Size != KiCodeD32 )
      && ( pxParse->Values.ulData_Size != KiCodeD16 )
      && ( pxParse->Values.ulData_Size != KiCodeD8  ) )
    {
        oConsole.ErrorLine ( (CString) "_ Illegal Data_Size Value at Line #"
                                      + I_to_A ( pxParse->iLineNum ) ) ;
	    lReturnStatus = Error ;
	    goto Done ;
    }

    //switching to requested Address Space (if it has changed)
    //will happen in VmeIO_Check_Ready_For_Access

Done:
    return lReturnStatus ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess VmeIO_Action_Parse_Read_VME   ( void* _pxParse, 
                                         void* _pxCustom,
                                         void* _pDlgThis )
{
    Parse_Structure*                pxParse     = (Parse_Structure*)                _pxParse ;
    Custom_Structure_VmeIO*         pxCustom    = (Custom_Structure_VmeIO*)         _pxCustom ;

    unsigned long   ulDataIn ;
    void*           pVirtAddress ;

    ESuccess        eStatus = Ok ;

    ESuccess        lReturnStatus = Ok ;

    //\\//

    // check all other keywords have been assigned
    // ( nothing to do in this case )

    // That's where we see if we need to switch Bit3 Adaptor
    // and also where we restore current context after a Recursive Call
    if ( VmeIO_Check_Ready_For_Access ( _pxParse, _pxCustom, _pDlgThis ) != Ok )
    {
	    lReturnStatus = Error ;
	    goto Done ;
    }

    // Increment Total Number of register IO performed
    pxCustom->uiTotReg++ ;

    // transfer Read_VME value to the Function_Address value to keep context to this register
	pxParse->Values.ulVME_Address = pxParse->Values.ulRead_VME ;
     	
    // Read the Register
    ulDataIn = 0 ;
    if ( pxCustom->osAccessMethod == KsMMAP ) // *** Note *** always API in command files
    {
        pVirtAddress = pxCustom->poBit3Adaptor->MemoryMapThisVmeSegment   ( pxParse->Values.ulRead_VME, 
                                                                            pxCustom->uiNumBytes, 
                                                                            pxCustom->eSwapping ) ;

#ifndef VME_IO_NONE
        if ( pVirtAddress != 0 ) 
        {
            switch ( pxCustom->uiNumBytes )
            {
                case 4 : ulDataIn = *(unsigned long*) pVirtAddress ; break ;
                case 2 : ulDataIn = *(unsigned short*)pVirtAddress ; break ;
                case 1 : ulDataIn = *(unsigned char*) pVirtAddress ; break ;
            }
        }
        else 
        {
            lReturnStatus = Error ;
        }
#endif

    }
    else // *** Note *** always API in command files
    {
	    // read the VME Register by calling the Bit3 API
        eStatus = pxCustom->poBit3Adaptor->ReadVmeData ( pxParse->Values.ulRead_VME, 
                                                        &ulDataIn,
                                                         pxCustom->uiNumBytes ) ;
        if ( eStatus != Ok ) 
        {
            lReturnStatus = Error ;
            oConsole.ErrorLine ( (CString) " Register Read Failed " ) ;
        }
    }

    // Now release the Bit3 Mapping resources.
    if ( pxCustom->osAccessMethod == KsMMAP ) 
         pxCustom->poBit3Adaptor->UnMapThisVmeSegment ( pVirtAddress, pxCustom->uiNumBytes ) ;

Done:

    return lReturnStatus ;
}



/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess VmeIO_Action_Parse_Write_Value            ( void* _pxParse, 
                                                     void* _pxCustom,
                                                     void* _pDlgThis )
{
    Parse_Structure*                pxParse     = (Parse_Structure*)                _pxParse ;
    Custom_Structure_VmeIO*         pxCustom    = (Custom_Structure_VmeIO*)         _pxCustom ;

    unsigned long   ulDataOut ;
    void*           pVirtAddress ;

    ESuccess        eStatus = Ok ;

    ESuccess        lReturnStatus = Ok ;

    //\\//

    // check all other keywords have been assigned
    if ( pxParse->Values.ulVME_Address  == diNotYetAssigned ) 
    {
        oConsole.ErrorLine ( (CString) "_ Missing Keywords before assignment at Line #"
                                      + I_to_A ( pxParse->iLineNum ) ) ;
	    lReturnStatus = Error ;
	    goto Done ;
    }

    // That's where we see if we need to switch Bit3 Adaptor
    // and also where we restore current context after a Recursive Call
    if ( VmeIO_Check_Ready_For_Access ( _pxParse, _pxCustom, _pDlgThis ) != Ok )
    {
	    lReturnStatus = Error ;
	    goto Done ;
    }

    // Increment Total Number of register IO performed
    pxCustom->uiTotReg++ ;
     
	
    // retrieve the data value to write
    switch ( pxCustom->uiNumBytes )
    {
        case 4 : ulDataOut = pxParse->Values.ulWrite_Value ; break ;
        case 2 : ulDataOut = fuwTruncToWord ( pxParse->Values.ulWrite_Value ) ; break ;
        case 1 : ulDataOut = fubTruncToByte ( pxParse->Values.ulWrite_Value ) ; break ;
    }

        
    // Now make the IO operation happen
    if ( pxCustom->osAccessMethod == KsMMAP ) // *** Note *** always API in command files
    {
        pVirtAddress = pxCustom->poBit3Adaptor->MemoryMapThisVmeSegment   ( pxParse->Values.ulVME_Address, 
                                                                            pxCustom->uiNumBytes, 
                                                                            pxCustom->eSwapping ) ;
        
#ifndef VME_IO_NONE
        if ( pVirtAddress != 0 ) 
        {
            switch ( pxCustom->uiNumBytes )
            {
                case 4 : *(unsigned long*) pVirtAddress = ulDataOut ; break ;
                case 2 : *(unsigned short*)pVirtAddress = fuwTruncToWord ( ulDataOut ) ; break ;
                case 1 : *(unsigned char*) pVirtAddress = fubTruncToByte ( ulDataOut ) ; break ;
            }
        }
        else 
        {
            lReturnStatus = Error ;
        }
#endif

    }
    else // *** Note *** always API in command files
    {
	    // Write the VME Register by calling the Bit3 API
        eStatus = pxCustom->poBit3Adaptor->WriteVmeData   ( pxParse->Values.ulVME_Address, 
                                                           &ulDataOut,
                                                            pxCustom->uiNumBytes ) ;
        if ( eStatus != Ok ) 
        {
            lReturnStatus = Error ;
            oConsole.ErrorLine ( (CString) " Register Write Failed " ) ;
        }
    }


    // Now release the Bit3 Mapping resources.
    if ( pxCustom->osAccessMethod == KsMMAP ) 
         pxCustom->poBit3Adaptor->UnMapThisVmeSegment ( pVirtAddress, pxCustom->uiNumBytes ) ;

Done:
    return lReturnStatus ;
}



/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess VmeIO_Action_Parse_Write_Verify           ( void* _pxParse, 
                                                     void* _pxCustom,
                                                     void* _pDlgThis )
{
    Parse_Structure*                pxParse     = (Parse_Structure*)                _pxParse ;
    Custom_Structure_VmeIO*         pxCustom    = (Custom_Structure_VmeIO*)         _pxCustom ;

    unsigned long   ulDataOut ;
    unsigned long   ulDataIn ;
    void*           pVirtAddress ;

    ESuccess        eStatus ;

    ESuccess        lReturnStatus = Ok ;

    //\\//

    // check all other keywords have been assigned
    if ( pxParse->Values.ulVME_Address  == diNotYetAssigned ) 
    {
        oConsole.ErrorLine ( (CString) "_ Missing Keywords before assignment at Line #"
                                      + I_to_A ( pxParse->iLineNum ) ) ;
	    lReturnStatus = Error ;
	    goto Done ;
    }

    // That's where we see if we need to switch Bit3 Adaptor
    // and also where we restore current context after a Recursive Call
    if ( VmeIO_Check_Ready_For_Access ( _pxParse, _pxCustom, _pDlgThis ) != Ok )
    {
	    lReturnStatus = Error ;
	    goto Done ;
    }

    // Increment Total Number of register IO performed
    pxCustom->uiTotReg++ ;
     
	
    // retrieve the data value to write
    switch ( pxCustom->uiNumBytes )
    {
        case 4 : ulDataOut = pxParse->Values.ulWrite_Value ; break ;
        case 2 : ulDataOut = fuwTruncToWord ( pxParse->Values.ulWrite_Value ) ; break ;
        case 1 : ulDataOut = fubTruncToByte ( pxParse->Values.ulWrite_Value ) ; break ;
    }

        
    // Now make the IO operation happen
    if ( pxCustom->osAccessMethod == KsMMAP ) // *** Note *** always API in command files
    {
        pVirtAddress = pxCustom->poBit3Adaptor->MemoryMapThisVmeSegment   ( pxParse->Values.ulVME_Address, 
                                                                            pxCustom->uiNumBytes, 
                                                                            pxCustom->eSwapping ) ;
        
#ifndef VME_IO_NONE
        if ( pVirtAddress != 0 ) 
        {
            switch ( pxCustom->uiNumBytes )
            {
                case 4 : *(unsigned long*) pVirtAddress = ulDataOut ; break ;
                case 2 : *(unsigned short*)pVirtAddress = fuwTruncToWord ( ulDataOut ) ; break ;
                case 1 : *(unsigned char*) pVirtAddress = fubTruncToByte ( ulDataOut ) ; break ;
            }
        }
        else 
        {
            lReturnStatus = Error ;
        }
#endif


    }
    else // *** Note *** always API in command files
    {
	    // Write the VME Register by calling the Bit3 API
        eStatus = pxCustom->poBit3Adaptor->WriteVmeData   ( pxParse->Values.ulVME_Address, 
                                                           &ulDataOut,
                                                            pxCustom->uiNumBytes ) ;
        if ( eStatus != Ok ) 
        {
            lReturnStatus = Error ;
            oConsole.ErrorLine ( (CString) " Register Write Failed " ) ;
        }
    }


    // now read it back
    ulDataIn = 0 ;
    if ( pxCustom->osAccessMethod == KsMMAP ) // *** Note *** always API in command files
    {
#ifndef VME_IO_NONE
        if ( pVirtAddress != 0 ) 
        {
            switch ( pxCustom->uiNumBytes )
            {
                case 4 : ulDataIn = *(unsigned long*) pVirtAddress ; break ;
                case 2 : ulDataIn = *(unsigned short*)pVirtAddress ; break ;
                case 1 : ulDataIn = *(unsigned char*) pVirtAddress ; break ;
            }
        }
#endif
    }
    else // *** Note *** always API in command files
    {
	    // read the VME Register by calling the Bit3 API
        eStatus = pxCustom->poBit3Adaptor->ReadVmeData    ( pxParse->Values.ulVME_Address, 
                                                           &ulDataIn,
                                                            pxCustom->uiNumBytes ) ;
        if ( eStatus != Ok ) 
        {
            lReturnStatus = Error ;
            oConsole.ErrorLine ( (CString) " Register Read Failed " ) ;
        }
    }


    // display result on separate console
    if ( ulDataOut != ulDataIn )
    {
        oConsole.ErrorLine ( (CString) pxCustom->osAccessMethod + "/" 
                                       + pxCustom->osAddressSpace + "/" + pxCustom->osTransferSize 
                                       + " Verif : " 
	  	                               + I_to_A( ulDataIn, 1, 10 )
		                               + " (=" 
		                               + I_to_A( ulDataIn, 1, 16 )
                                       + ") Instead of " 
	  	                               + I_to_A( ulDataOut, 1, 10 )
		                               + " (=" 
		                               + I_to_A( ulDataOut, 1, 16 )
		                               + ") @ VME Address " 
		                               + I_to_A( pxParse->Values.ulVME_Address, 8, 16, '0' ) ) ;
	    lReturnStatus = Error ;
    }

    // Now release the Bit3 Mapping resources.
    if ( pxCustom->osAccessMethod == KsMMAP ) 
         pxCustom->poBit3Adaptor->UnMapThisVmeSegment ( pVirtAddress, pxCustom->uiNumBytes ) ;


Done:

    return lReturnStatus ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess VmeIO_Action_Done_With_File               ( void* _pxParse, 
                                                     void* _pxCustom,
                                                     void* _pDlgThis )
{
    Parse_Structure*                pxParse     = (Parse_Structure*)                _pxParse ;
    Custom_Structure_VmeIO*         pxCustom    = (Custom_Structure_VmeIO*)         _pxCustom ;

    ESuccess        lReturnStatus = Ok ;

    //\\//

//Done:
    return lReturnStatus ;
}

/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
//This function is not automatically called after the occurence of a Keyword, 
// but is explicitely invoked in all the VME Access actions above.

//A copy of the *pxParse Structure (which generic to all command file types) 
//is passed during recursive calls, so that changes in called files do not affect
//the calling file.
//The actuall *pxCuston Structure (which is of diferent type for each Command File Type)
//is passed during recursive calls, and its changes are propagated back to the calling file
//In this routine we return to the content of the pxParse variables to know what access
//type to do and if a change of Bit3 Adaptor is necessary.
ESuccess VmeIO_Check_Ready_For_Access              ( void* _pxParse, 
                                                     void* _pxCustom,
                                                     void* _pDlgThis )
{
    Parse_Structure*                pxParse     = (Parse_Structure*)                _pxParse ;
    Custom_Structure_VmeIO*         pxCustom    = (Custom_Structure_VmeIO*)         _pxCustom ;

    ESuccess        lReturnStatus = Ok ;
    EYesNo          lAddressSpaceHasChanged = No ;

    //\\//

    //First check if this it the first time we get called
    if ( pxCustom->poBit3Adaptor == NULL )
        lAddressSpaceHasChanged = Yes ;

    //retrieve the requested adaptor number if it has changed
    pxCustom->uiAdaptor_Num = pxParse->Values.ulBit3_Adaptor ;

    if ( pxCustom->poBit3Adaptor != NULL )
    if ( pxCustom->poBit3Adaptor->m_uiAdaptorNum != pxParse->Values.ulBit3_Adaptor )
        lAddressSpaceHasChanged = Yes ;

    //retrieve the requested Address Space Type
    if      ( pxParse->Values.ulAddress_Space == KiCodeA32 )
    {
        pxCustom->eAddrSpace = VmeA32 ; 
        pxCustom->osAddressSpace = "A32" ;
        if ( pxCustom->poBit3Adaptor != NULL )
        if ( pxCustom->poBit3Adaptor->m_eAddrSpace != VmeA32 ) 
            lAddressSpaceHasChanged = Yes ;
    }
    else if ( pxParse->Values.ulAddress_Space == KiCodeA24 )
    {
        pxCustom->eAddrSpace = VmeA24 ;
        pxCustom->osAddressSpace = "A24" ;
        if ( pxCustom->poBit3Adaptor != NULL )
        if ( pxCustom->poBit3Adaptor->m_eAddrSpace != VmeA24 ) 
            lAddressSpaceHasChanged = Yes ;
    }
    else if ( pxParse->Values.ulAddress_Space == KiCodeA16 )
    {
        pxCustom->eAddrSpace = VmeA16 ; 
        pxCustom->osAddressSpace = "A16" ;
        if ( pxCustom->poBit3Adaptor != NULL )
        if ( pxCustom->poBit3Adaptor->m_eAddrSpace != VmeA16 ) 
            lAddressSpaceHasChanged = Yes ;
    }
    else if ( pxParse->Values.ulAddress_Space ==  KiCodeDpm )
    {
        pxCustom->eAddrSpace = Dpm ; 
        pxCustom->osAddressSpace = "DPM" ;
        if ( pxCustom->poBit3Adaptor != NULL )
        if ( pxCustom->poBit3Adaptor->m_eAddrSpace != Dpm ) 
            lAddressSpaceHasChanged = Yes ;
    }
    else 
    {
        oConsole.InternalErrorLine ( (CString) "_ Illegal Address_Space Value at Line #"
                                      + I_to_A ( pxParse->iLineNum ) ) ;
	    lReturnStatus = Error ;
	    goto Done ;
    }

    //retrieve the requested Data Size Type
    if      ( pxParse->Values.ulData_Size == KiCodeD32 )
    {
        pxCustom->osTransferSize = "D32" ; 
        pxCustom->uiNumBytes = 4 ; 
    }
    else if ( pxParse->Values.ulData_Size == KiCodeD16 )
    {
        pxCustom->osTransferSize = "D16" ; 
        pxCustom->uiNumBytes = 2 ; 
    }
    else if ( pxParse->Values.ulData_Size == KiCodeD8 )
    {
        pxCustom->osTransferSize = "D8 " ; 
        pxCustom->uiNumBytes = 1 ; 
    }
    else 
    {
        oConsole.InternalErrorLine ( (CString) "_ Illegal Data_Size Value at Line #"
                                      + I_to_A ( pxParse->iLineNum ) ) ;
	    lReturnStatus = Error ;
	    goto Done ;
    }

    //switch to requested Address Space if it has changed
    if ( lAddressSpaceHasChanged == Yes )
    {
        // release the previous Bit3 Adaptor if needed
        if ( pxCustom->poBit3Adaptor != NULL )
        delete pxCustom->poBit3Adaptor ;

        // connect to new Bit3 Apdaptor 
        pxCustom->poBit3Adaptor = new CBit3Adaptor61x ( pxCustom->uiAdaptor_Num, pxCustom->eAddrSpace ) ;

        if ( pxCustom->poBit3Adaptor->m_eStatus == Error )
        {
            oConsole.ErrorLine ( (CString) "_ Error Initializing Bit3 Adaptor Object at Line #"
                                          + I_to_A ( pxParse->iLineNum ) ) ;
	        lReturnStatus = Error ;
	        goto Done ;
        }
    }

Done:
    return lReturnStatus ;
}
