/*-------------------------------------------------------------------------------*/
  /*
     GuiCommandDispatcher.cpp
     2-Nov-04
  */
/*-------------------------------------------------------------------------------*/
#include "L1Cal_IIb_TCC/StdAfx.h"

#include "L1Cal_IIb_TCC/GlobalConstants.h"
#include "L1Cal_IIb_TCC/CustomDataTypes.h"

#include "L1Cal_IIb_TCC/GuiCommand.h"
#include                            "L1Cal_IIb_TCC/GuiCommandDispatcher.h"

#include "L1Cal_IIb_TCC/GuiCommandParser.h"

#include <xercesc/dom/DOM.hpp>
#include <xercesc/util/XMLString.hpp>
#include <xercesc/framework/MemBufInputSource.hpp>

#include "L1Cal_IIb_TCC/HandleConsole.h"
#include "L1Cal_IIb_TCC/UtilsStrings.h"

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/

/////////////////////////////////////////////////////////////
#define IF_CMD_MATCH( COMMAND )                             \
                                                            \
    if ( CGuiCommandParser::                                \
          GuiCommandMatch ( _poGuiCommandLine,              \
                             COMMAND ) )                    \
                                                            \
/////////////////////////////////////////////////////////////
#define GET_CMD_ATTR( PSTR, ELEMENT, ATTR )                 \
                                                            \
    char* PSTR = CGuiCommandParser::                        \
                GuiGetEltAttrName  ( _poGuiCommandLine,     \
                                      ELEMENT,              \
                                      ATTR )                \
                                                            \
/////////////////////////////////////////////////////////////
#define GET_CMD_ATTR_WD( PSTR, ELEMENT, ATTR, DEFAULT )     \
                                                            \
    char* PSTR = CGuiCommandParser::                        \
                GuiGetEltAttrName  ( _poGuiCommandLine,     \
                                      ELEMENT,              \
                                      ATTR,                 \
                                      DEFAULT )             \
                                                            \
/////////////////////////////////////////////////////////////
#define IF_ATTR_OK1( PSTR1 )                                \
                                                            \
    if ( ( PSTR1 == NULL ) )                                \
      eReturnStatus = GuiReply_Bad ;                        \
    else                                                    \
                                                            \
/////////////////////////////////////////////////////////////
#define IF_ATTR_OK2( PSTR1, PSTR2 )                         \
                                                            \
    if ( ( PSTR1 == NULL )                                  \
      || ( PSTR2 == NULL ) )                                \
      eReturnStatus = GuiReply_Bad ;                        \
    else                                                    \
                                                            \
/////////////////////////////////////////////////////////////
#define IF_ATTR_OK3( PSTR1, PSTR2, PSTR3 )                  \
                                                            \
    if ( ( PSTR1 == NULL )                                  \
      || ( PSTR2 == NULL )                                  \
      || ( PSTR3 == NULL ) )                                \
      eReturnStatus = GuiReply_Bad ;                        \
    else                                                    \
                                                            \
/////////////////////////////////////////////////////////////
#define IF_ATTR_OK4( PSTR1, PSTR2, PSTR3, PSTR4 )           \
                                                            \
    if ( ( PSTR1 == NULL )                                  \
      || ( PSTR2 == NULL )                                  \
      || ( PSTR3 == NULL )                                  \
      || ( PSTR4 == NULL ) )                                \
      eReturnStatus = GuiReply_Bad ;                        \
    else                                                    \
                                                            \
/////////////////////////////////////////////////////////////
#define IF_ATTR_OK5( PSTR1, PSTR2, PSTR3, PSTR4, PSTR5 )    \
                                                            \
    if ( ( PSTR1 == NULL )                                  \
      || ( PSTR2 == NULL )                                  \
      || ( PSTR3 == NULL )                                  \
      || ( PSTR4 == NULL )                                  \
      || ( PSTR5 == NULL ) )                                \
      eReturnStatus = GuiReply_Bad ;                        \
    else                                                    \
                                                            \
/////////////////////////////////////////////////////////////
#define IF_ATTR_OK6( PSTR1, PSTR2, PSTR3, PSTR4, PSTR5,     \
                     PSTR6 )                                \
                                                            \
    if ( ( PSTR1 == NULL )                                  \
      || ( PSTR2 == NULL )                                  \
      || ( PSTR3 == NULL )                                  \
      || ( PSTR4 == NULL )                                  \
      || ( PSTR5 == NULL )                                  \
      || ( PSTR6 == NULL ) )                                \
      eReturnStatus = GuiReply_Bad ;                        \
    else                                                    \
                                                            \
/////////////////////////////////////////////////////////////
#define IF_ATTR_OK7( PSTR1, PSTR2, PSTR3, PSTR4, PSTR5,     \
                     PSTR6, PSTR7 )                         \
                                                            \
    if ( ( PSTR1 == NULL )                                  \
      || ( PSTR2 == NULL )                                  \
      || ( PSTR3 == NULL )                                  \
      || ( PSTR4 == NULL )                                  \
      || ( PSTR5 == NULL )                                  \
      || ( PSTR6 == NULL )                                  \
      || ( PSTR7 == NULL ) )                                \
      eReturnStatus = GuiReply_Bad ;                        \
    else                                                    \
                                                            \
/////////////////////////////////////////////////////////////
#define IF_ATTR_OK8( PSTR1, PSTR2, PSTR3, PSTR4, PSTR5,     \
                     PSTR6, PSTR7, PSTR8 )                  \
                                                            \
    if ( ( PSTR1 == NULL )                                  \
      || ( PSTR2 == NULL )                                  \
      || ( PSTR3 == NULL )                                  \
      || ( PSTR4 == NULL )                                  \
      || ( PSTR5 == NULL )                                  \
      || ( PSTR6 == NULL )                                  \
      || ( PSTR7 == NULL )                                  \
      || ( PSTR8 == NULL ) )                                \
      eReturnStatus = GuiReply_Bad ;                        \
    else                                                    \
                                                            \
/////////////////////////////////////////////////////////////
#define IF_ATTR_OK9( PSTR1, PSTR2, PSTR3, PSTR4, PSTR5,     \
                     PSTR6, PSTR7, PSTR8, PSTR9 )           \
                                                            \
    if ( ( PSTR1 == NULL )                                  \
      || ( PSTR2 == NULL )                                  \
      || ( PSTR3 == NULL )                                  \
      || ( PSTR4 == NULL )                                  \
      || ( PSTR5 == NULL )                                  \
      || ( PSTR6 == NULL )                                  \
      || ( PSTR7 == NULL )                                  \
      || ( PSTR8 == NULL )                                  \
      || ( PSTR9 == NULL ) )                                \
      eReturnStatus = GuiReply_Bad ;                        \
    else                                                    \
                                                            \
/////////////////////////////////////////////////////////////
#define IF_ATTR_OK10(PSTR1, PSTR2, PSTR3, PSTR4, PSTR5,     \
                     PSTR6, PSTR7, PSTR8, PSTR9, PSTR10 )   \
                                                            \
    if ( ( PSTR1 == NULL )                                  \
      || ( PSTR2 == NULL )                                  \
      || ( PSTR3 == NULL )                                  \
      || ( PSTR4 == NULL )                                  \
      || ( PSTR5 == NULL )                                  \
      || ( PSTR6 == NULL )                                  \
      || ( PSTR7 == NULL )                                  \
      || ( PSTR8 == NULL )                                  \
      || ( PSTR9 == NULL )                                  \
      || ( PSTR10== NULL ) )                                \
      eReturnStatus = GuiReply_Bad ;                        \
    else                                                    \
                                                            \
/////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////
#define IF_OPTIONAL_SPECIFIED( PSTR )                       \
    if ( ( PSTR != NULL ) && ( *PSTR != 0x00 ) )            \
/////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////
#define RELEASE_STR1( PSTR1 )                               \
                                                            \
    CGuiCommandParser::XMLrelease ( PSTR1 ) ;               \
                                                            \
/////////////////////////////////////////////////////////////
#define RELEASE_STR2( PSTR1, PSTR2 )                        \
                                                            \
    CGuiCommandParser::XMLrelease ( PSTR1 ) ;               \
    CGuiCommandParser::XMLrelease ( PSTR2 ) ;               \
                                                            \
/////////////////////////////////////////////////////////////
#define RELEASE_STR3( PSTR1, PSTR2, PSTR3 )                 \
                                                            \
    CGuiCommandParser::XMLrelease ( PSTR1 ) ;               \
    CGuiCommandParser::XMLrelease ( PSTR2 ) ;               \
    CGuiCommandParser::XMLrelease ( PSTR3 ) ;               \
                                                            \
/////////////////////////////////////////////////////////////
#define RELEASE_STR4( PSTR1, PSTR2, PSTR3, PSTR4 )          \
                                                            \
    CGuiCommandParser::XMLrelease ( PSTR1 ) ;               \
    CGuiCommandParser::XMLrelease ( PSTR2 ) ;               \
    CGuiCommandParser::XMLrelease ( PSTR3 ) ;               \
    CGuiCommandParser::XMLrelease ( PSTR4 ) ;               \
                                                            \
/////////////////////////////////////////////////////////////
#define RELEASE_STR5( PSTR1, PSTR2, PSTR3, PSTR4, PSTR5 )   \
                                                            \
    CGuiCommandParser::XMLrelease ( PSTR1 ) ;               \
    CGuiCommandParser::XMLrelease ( PSTR2 ) ;               \
    CGuiCommandParser::XMLrelease ( PSTR3 ) ;               \
    CGuiCommandParser::XMLrelease ( PSTR4 ) ;               \
    CGuiCommandParser::XMLrelease ( PSTR5 ) ;               \
                                                            \
/////////////////////////////////////////////////////////////
#define RELEASE_STR6( PSTR1, PSTR2, PSTR3, PSTR4, PSTR5,    \
                      PSTR6 )                               \
                                                            \
    CGuiCommandParser::XMLrelease ( PSTR1 ) ;               \
    CGuiCommandParser::XMLrelease ( PSTR2 ) ;               \
    CGuiCommandParser::XMLrelease ( PSTR3 ) ;               \
    CGuiCommandParser::XMLrelease ( PSTR4 ) ;               \
    CGuiCommandParser::XMLrelease ( PSTR5 ) ;               \
    CGuiCommandParser::XMLrelease ( PSTR6 ) ;               \
                                                            \
/////////////////////////////////////////////////////////////
#define RELEASE_STR7( PSTR1, PSTR2, PSTR3, PSTR4, PSTR5,    \
                      PSTR6, PSTR7 )                        \
                                                            \
    CGuiCommandParser::XMLrelease ( PSTR1 ) ;               \
    CGuiCommandParser::XMLrelease ( PSTR2 ) ;               \
    CGuiCommandParser::XMLrelease ( PSTR3 ) ;               \
    CGuiCommandParser::XMLrelease ( PSTR4 ) ;               \
    CGuiCommandParser::XMLrelease ( PSTR5 ) ;               \
    CGuiCommandParser::XMLrelease ( PSTR6 ) ;               \
    CGuiCommandParser::XMLrelease ( PSTR7 ) ;               \
                                                            \
/////////////////////////////////////////////////////////////
#define RELEASE_STR8( PSTR1, PSTR2, PSTR3, PSTR4, PSTR5,    \
                      PSTR6, PSTR7, PSTR8 )                 \
                                                            \
    CGuiCommandParser::XMLrelease ( PSTR1 ) ;               \
    CGuiCommandParser::XMLrelease ( PSTR2 ) ;               \
    CGuiCommandParser::XMLrelease ( PSTR3 ) ;               \
    CGuiCommandParser::XMLrelease ( PSTR4 ) ;               \
    CGuiCommandParser::XMLrelease ( PSTR5 ) ;               \
    CGuiCommandParser::XMLrelease ( PSTR6 ) ;               \
    CGuiCommandParser::XMLrelease ( PSTR7 ) ;               \
    CGuiCommandParser::XMLrelease ( PSTR8 ) ;               \
                                                            \
/////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////
#define RELEASE_STR9( PSTR1, PSTR2, PSTR3, PSTR4, PSTR5,    \
                      PSTR6, PSTR7, PSTR8, PSTR9 )          \
                                                            \
    CGuiCommandParser::XMLrelease ( PSTR1 ) ;               \
    CGuiCommandParser::XMLrelease ( PSTR2 ) ;               \
    CGuiCommandParser::XMLrelease ( PSTR3 ) ;               \
    CGuiCommandParser::XMLrelease ( PSTR4 ) ;               \
    CGuiCommandParser::XMLrelease ( PSTR5 ) ;               \
    CGuiCommandParser::XMLrelease ( PSTR6 ) ;               \
    CGuiCommandParser::XMLrelease ( PSTR7 ) ;               \
    CGuiCommandParser::XMLrelease ( PSTR8 ) ;               \
    CGuiCommandParser::XMLrelease ( PSTR9 ) ;               \
                                                            \
/////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////
#define RELEASE_STR10(PSTR1, PSTR2, PSTR3, PSTR4, PSTR5,    \
                      PSTR6, PSTR7, PSTR8, PSTR9, PSTR10 )  \
                                                            \
    CGuiCommandParser::XMLrelease ( PSTR1 ) ;               \
    CGuiCommandParser::XMLrelease ( PSTR2 ) ;               \
    CGuiCommandParser::XMLrelease ( PSTR3 ) ;               \
    CGuiCommandParser::XMLrelease ( PSTR4 ) ;               \
    CGuiCommandParser::XMLrelease ( PSTR5 ) ;               \
    CGuiCommandParser::XMLrelease ( PSTR6 ) ;               \
    CGuiCommandParser::XMLrelease ( PSTR7 ) ;               \
    CGuiCommandParser::XMLrelease ( PSTR8 ) ;               \
    CGuiCommandParser::XMLrelease ( PSTR9 ) ;               \
    CGuiCommandParser::XMLrelease ( PSTR10) ;               \
                                                            \
/////////////////////////////////////////////////////////////

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
EGuiCommandStatus CGuiCommandDispatcher::GuiCommandDispatch       ( CGuiCommandLine*                 _poGuiCommandLine,
                                                                    CGuiCommandExecuter*             _pxGuiCommandExecute ) 
{ 
    EGuiCommandStatus   eReturnStatus = GuiReply_Ok ;
    char                sReplyText[KiMaxLengthReplyTextToGui] ;

    //\\//

    /////////////////////////////////////////////////////////////////////
    // Show ccomment found within command xml text, if any
    /////////////////////////////////////////////////////////////////////
    GET_CMD_ATTR_WD( psComment, KsGuiXml_Comment,  KsGuiXml_Text, "" ) ;
    IF_ATTR_OK1( psComment ) 

        if ( ( psComment != NULL ) && ( *psComment != 0x00 ) )
            CConsoleAccess::WarningLine ( (CString) " Comment : \"" 
                                                    + psComment 
                                                    + "\"" ) ;

    RELEASE_STR1( psComment ) 

    /////////////////////////////////////////////////////////////////////
    /// 
    /////////////////////////////////////////////////////////////////////
         IF_CMD_MATCH( KsGuiCmd_Vio_ReadReg ) 
    {
        GET_CMD_ATTR   ( psVmeAddr,   KsGuiXml_Reg,  KsGuiXml_Addr         ) ;
        GET_CMD_ATTR_WD( psRepeatCnt, KsGuiXml_Data, KsGuiXml_RepeatCnt,    "1"     ) ;
        GET_CMD_ATTR_WD( psAddrSpace, KsGuiXml_Reg,  KsGuiXml_AddrSpace,    "A32"   ) ;
        GET_CMD_ATTR_WD( psDataSize,  KsGuiXml_Reg,  KsGuiXml_DataSize,     "D16"   ) ;
        GET_CMD_ATTR_WD( psAccPriv,   KsGuiXml_Reg,  KsGuiXml_AcccessPriv,  "Super" ) ;
        GET_CMD_ATTR_WD( psAccType,   KsGuiXml_Reg,  KsGuiXml_AccessType,   "Data"  ) ;
        GET_CMD_ATTR_WD( psIOMethod,  KsGuiXml_Reg,  KsGuiXml_IOMethod,     "API"   ) ;

        IF_ATTR_OK7( psVmeAddr, psRepeatCnt, psAddrSpace, psDataSize, psAccPriv, psAccType, psIOMethod )

            eReturnStatus = _pxGuiCommandExecute->Vio_ReadReg   ( A_to_uI ( psVmeAddr ),
                                                                  A_to_uI ( psRepeatCnt ),
                                                                  psAddrSpace, 
                                                                  psDataSize, 
                                                                  psAccPriv, 
                                                                  psAccType, 
                                                                  psIOMethod,
                                                                 _poGuiCommandLine ) ;

        RELEASE_STR7( psVmeAddr, psRepeatCnt, psAddrSpace, psDataSize, psAccPriv, psAccType, psIOMethod )
    }

    /////////////////////////////////////////////////////////////////////
    /// 
    /////////////////////////////////////////////////////////////////////
    else IF_CMD_MATCH( KsGuiCmd_Vio_WriteReg ) 
    {
        GET_CMD_ATTR   ( psVmeAddr,   KsGuiXml_Reg,  KsGuiXml_Addr   ) ;
        GET_CMD_ATTR   ( psDataOut,   KsGuiXml_Data, KsGuiXml_Out    ) ;
        GET_CMD_ATTR_WD( psRepeatCnt, KsGuiXml_Data, KsGuiXml_RepeatCnt,    "1"     ) ;
        GET_CMD_ATTR_WD( psAddrSpace, KsGuiXml_Reg,  KsGuiXml_AddrSpace,    "A32"   ) ;
        GET_CMD_ATTR_WD( psDataSize,  KsGuiXml_Reg,  KsGuiXml_DataSize,     "D16"   ) ;
        GET_CMD_ATTR_WD( psAccPriv,   KsGuiXml_Reg,  KsGuiXml_AcccessPriv,  "Super" ) ;
        GET_CMD_ATTR_WD( psAccType,   KsGuiXml_Reg,  KsGuiXml_AccessType,   "Data"  ) ;
        GET_CMD_ATTR_WD( psIOMethod,  KsGuiXml_Reg,  KsGuiXml_IOMethod,     "API"   ) ;

        IF_ATTR_OK8( psVmeAddr, psDataOut, psRepeatCnt, psAddrSpace, psDataSize, psAccPriv, psAccType, psIOMethod )

            eReturnStatus = _pxGuiCommandExecute->Vio_WriteReg  ( A_to_uI ( psVmeAddr ),
                                                                  A_to_uI ( psDataOut ),
                                                                  A_to_uI ( psRepeatCnt ),
                                                                  psAddrSpace, 
                                                                  psDataSize, 
                                                                  psAccPriv, 
                                                                  psAccType, 
                                                                  psIOMethod,
                                                                 _poGuiCommandLine ) ;

        RELEASE_STR8( psVmeAddr, psDataOut, psRepeatCnt, psAddrSpace, psDataSize, psAccPriv, psAccType, psIOMethod )
    }

    /////////////////////////////////////////////////////////////////////
    /// 
    /////////////////////////////////////////////////////////////////////
    else IF_CMD_MATCH( KsGuiCmd_Rio_ReadReg ) 
    {
        GET_CMD_ATTR( psAdfMaster, KsGuiXml_Adf,  KsGuiXml_Master ) ;
        GET_CMD_ATTR( psAdfSlave,  KsGuiXml_Adf,  KsGuiXml_Slave  ) ;
        GET_CMD_ATTR( psAdfSlot,   KsGuiXml_Adf,  KsGuiXml_Slot   ) ;
        GET_CMD_ATTR( psChipNum,   KsGuiXml_Chip, KsGuiXml_Num    ) ;
        GET_CMD_ATTR( psRegAddr,   KsGuiXml_Reg,  KsGuiXml_Addr   ) ;

        IF_ATTR_OK5( psAdfMaster, psAdfSlave, psAdfSlot, psChipNum, psRegAddr )

            eReturnStatus = _pxGuiCommandExecute->Rio_ReadReg   ( A_to_uI ( psAdfMaster ),
                                                                  A_to_uI ( psAdfSlave  ),
                                                                  A_to_uI ( psAdfSlot   ),
                                                                  A_to_uI ( psChipNum   ),
                                                                  A_to_uI ( psRegAddr   ),
                                                                 _poGuiCommandLine ) ;

        RELEASE_STR5( psAdfMaster, psAdfSlave, psAdfSlot, psChipNum, psRegAddr ) 
    }

    /////////////////////////////////////////////////////////////////////
    /// 
    /////////////////////////////////////////////////////////////////////
    else IF_CMD_MATCH( KsGuiCmd_Rio_WriteReg ) 
    {
        GET_CMD_ATTR( psAdfMaster, KsGuiXml_Adf,  KsGuiXml_Master ) ;
        GET_CMD_ATTR( psAdfSlave,  KsGuiXml_Adf,  KsGuiXml_Slave  ) ;
        GET_CMD_ATTR( psAdfSlot,   KsGuiXml_Adf,  KsGuiXml_Slot   ) ;
        GET_CMD_ATTR( psChipNum,   KsGuiXml_Chip, KsGuiXml_Num    ) ;
        GET_CMD_ATTR( psRegAddr,   KsGuiXml_Reg,  KsGuiXml_Addr   ) ;
        GET_CMD_ATTR( psDataOut,   KsGuiXml_Data, KsGuiXml_Out    ) ;

        IF_ATTR_OK6( psAdfMaster, psAdfSlave, psAdfSlot, psChipNum, psRegAddr, psDataOut )

            eReturnStatus = _pxGuiCommandExecute->Rio_WriteReg  ( A_to_uI ( psAdfMaster ),
                                                                  A_to_uI ( psAdfSlave  ),
                                                                  A_to_uI ( psAdfSlot   ),
                                                                  A_to_uI ( psChipNum   ),
                                                                  A_to_uI ( psRegAddr   ),
                                                                  A_to_uI ( psDataOut   ),
                                                                 _poGuiCommandLine ) ;

        RELEASE_STR6( psAdfMaster, psAdfSlave, psAdfSlot, psChipNum, psRegAddr, psDataOut )
    }

    /////////////////////////////////////////////////////////////////////
    /// 
    /////////////////////////////////////////////////////////////////////
	else IF_CMD_MATCH( KsGuiCmd_Mio_MultiRead ) 
    {
        GET_CMD_ATTR( psAdfMaster,      KsGuiXml_Adf,       KsGuiXml_Master     ) ;
        GET_CMD_ATTR( psAdfSlave,       KsGuiXml_Adf,       KsGuiXml_Slave      ) ;
        GET_CMD_ATTR( psAdfSlot,        KsGuiXml_Adf,       KsGuiXml_Slot       ) ;
        GET_CMD_ATTR( psChipNum,        KsGuiXml_Chip,      KsGuiXml_Num        ) ;
        GET_CMD_ATTR( psRegAddr,        KsGuiXml_Reg,       KsGuiXml_Addr       ) ;
        GET_CMD_ATTR( psRegSamples,     KsGuiXml_Reg,       KsGuiXml_Samples    ) ;
        GET_CMD_ATTR( psCondMask,       KsGuiXml_Trigger,   KsGuiXml_Mask     ) ;
        GET_CMD_ATTR( psCondValue,      KsGuiXml_Trigger,   KsGuiXml_Value     ) ;
        GET_CMD_ATTR( psMax_Try,        KsGuiXml_Trigger,   KsGuiXml_Max_Try    ) ;
        GET_CMD_ATTR( psRegDumpToFile,  KsGuiXml_Trigger,   KsGuiXml_DumpToFile ) ;

        IF_ATTR_OK10( psAdfMaster,  psAdfSlave, psAdfSlot,   psChipNum, psRegAddr, 
                      psRegSamples, psCondMask, psCondValue, psMax_Try, psRegDumpToFile  )

            eReturnStatus = _pxGuiCommandExecute->Mio_MultiRead  ( A_to_uI ( psAdfMaster ),
                                                                   A_to_uI ( psAdfSlave  ),
                                                                   A_to_uI ( psAdfSlot   ),
                                                                   A_to_uI ( psChipNum   ),
                                                                   A_to_uI ( psRegAddr   ),
                                                                   A_to_uI ( psRegSamples),
                                                                   A_to_uI ( psCondMask  ),
                                                                   A_to_uI ( psCondValue ),
                                                                   A_to_uI ( psMax_Try   ),
                                                                   A_to_uI ( psRegDumpToFile),
                                                                  _poGuiCommandLine ) ;

        RELEASE_STR10( psAdfMaster,  psAdfSlave, psAdfSlot,   psChipNum, psRegAddr, 
                       psRegSamples, psCondMask, psCondValue, psMax_Try, psRegDumpToFile  )
    }

    /////////////////////////////////////////////////////////////////////
    /// 
    /////////////////////////////////////////////////////////////////////
	else IF_CMD_MATCH( KsGuiCmd_Hst_10BitHisto ) 
    {
        GET_CMD_ATTR( psAdfMaster,      KsGuiXml_Adf,  KsGuiXml_Master      ) ;
        GET_CMD_ATTR( psAdfSlave,       KsGuiXml_Adf,  KsGuiXml_Slave       ) ;
        GET_CMD_ATTR( psAdfSlot,        KsGuiXml_Adf,  KsGuiXml_Slot        ) ;
        GET_CMD_ATTR( psChipNum,        KsGuiXml_Chip, KsGuiXml_Num         ) ;
        GET_CMD_ATTR( psRegAddr,        KsGuiXml_Reg,  KsGuiXml_Addr        ) ;
        GET_CMD_ATTR( psRegSamples,     KsGuiXml_Reg,  KsGuiXml_Samples     ) ;
        GET_CMD_ATTR( psRegAddrIncr,    KsGuiXml_Reg,  KsGuiXml_AddrIncr    ) ;
        GET_CMD_ATTR( psRegDumpToFile,  KsGuiXml_Reg,  KsGuiXml_DumpToFile  ) ;

        IF_ATTR_OK8( psAdfMaster, psAdfSlave, psAdfSlot, psChipNum, psRegAddr, 
                     psRegSamples, psRegAddrIncr, psRegDumpToFile  )

            eReturnStatus = _pxGuiCommandExecute->Hst_10BitHisto ( A_to_uI ( psAdfMaster ),
                                                                   A_to_uI ( psAdfSlave  ),
                                                                   A_to_uI ( psAdfSlot   ),
                                                                   A_to_uI ( psChipNum   ),
                                                                   A_to_uI ( psRegAddr   ),
                                                                   A_to_uI ( psRegSamples),
                                                                   A_to_uI ( psRegAddrIncr),
                                                                   A_to_uI ( psRegDumpToFile),
                                                                  _poGuiCommandLine ) ;

        RELEASE_STR8( psAdfMaster, psAdfSlave, psAdfSlot, psChipNum, psRegAddr, 
                      psRegSamples, psRegAddrIncr, psRegDumpToFile  )
    }

    /////////////////////////////////////////////////////////////////////
    /// 
    /////////////////////////////////////////////////////////////////////
    else IF_CMD_MATCH( KsGuiCmd_Sio_ReadReg ) 
    {
        GET_CMD_ATTR( psModule,    KsGuiXml_Tab,    KsGuiXml_Module ) ;
        GET_CMD_ATTR( psChip,      KsGuiXml_Chip,   KsGuiXml_Num ) ;
        GET_CMD_ATTR( psGroup,     KsGuiXml_Reg,    KsGuiXml_Group ) ;
        GET_CMD_ATTR( psSer_Addr,  KsGuiXml_Reg,    KsGuiXml_Ser_Addr ) ;
        GET_CMD_ATTR( psSer_Data,  KsGuiXml_Reg,    KsGuiXml_Ser_Data ) ;

        IF_ATTR_OK5( psModule, psChip, psGroup, psSer_Addr, psSer_Data )

            eReturnStatus = _pxGuiCommandExecute->Sio_ReadReg   ( A_to_uI ( psModule    ),
                                                                  A_to_uI ( psChip      ),
                                                                  A_to_uI ( psGroup     ),
                                                                  A_to_uI ( psSer_Addr  ),
                                                                  A_to_uI ( psSer_Data  ),
                                                                 _poGuiCommandLine ) ;

        RELEASE_STR5( psModule, psChip, psGroup, psSer_Addr, psSer_Data )
    }

    /////////////////////////////////////////////////////////////////////
    /// 
    /////////////////////////////////////////////////////////////////////
    else IF_CMD_MATCH( KsGuiCmd_Sio_WriteReg ) 
    {
        GET_CMD_ATTR( psModule,    KsGuiXml_Tab,    KsGuiXml_Module ) ;
        GET_CMD_ATTR( psChip,      KsGuiXml_Chip,   KsGuiXml_Num ) ;
        GET_CMD_ATTR( psGroup,     KsGuiXml_Reg,    KsGuiXml_Group ) ;
        GET_CMD_ATTR( psSer_Addr,  KsGuiXml_Reg,    KsGuiXml_Ser_Addr ) ;
        GET_CMD_ATTR( psSer_Data,  KsGuiXml_Data,   KsGuiXml_Ser_Data ) ;

        IF_ATTR_OK5( psModule, psChip, psGroup, psSer_Addr, psSer_Data )

            eReturnStatus = _pxGuiCommandExecute->Sio_WriteReg  ( A_to_uI ( psModule    ),
                                                                  A_to_uI ( psChip      ),
                                                                  A_to_uI ( psGroup     ),
                                                                  A_to_uI ( psSer_Addr  ),
                                                                  A_to_uI ( psSer_Data  ),
                                                                 _poGuiCommandLine ) ;

        RELEASE_STR5( psModule, psChip, psGroup, psSer_Addr, psSer_Data )
    }

    /////////////////////////////////////////////////////////////////////
    /// 
    /////////////////////////////////////////////////////////////////////
    else IF_CMD_MATCH( KsGuiCmd_Sio_VmeSclStatus ) 
    {
            eReturnStatus = _pxGuiCommandExecute->Sio_VmeSclStatus ( _poGuiCommandLine ) ;
    }

    /////////////////////////////////////////////////////////////////////
    /// 
    /////////////////////////////////////////////////////////////////////
    else IF_CMD_MATCH( KsGuiCmd_Cfg_ConfigFpga ) 
    {
        GET_CMD_ATTR( psAdfMaster, KsGuiXml_Adf,  KsGuiXml_Master ) ;
        GET_CMD_ATTR( psAdfSlave,  KsGuiXml_Adf,  KsGuiXml_Slave  ) ;
        GET_CMD_ATTR( psAdfSlot,   KsGuiXml_Adf,  KsGuiXml_Slot   ) ;
        GET_CMD_ATTR( psFpgaMask,  KsGuiXml_Fpga, KsGuiXml_Mask   ) ;
        GET_CMD_ATTR( psFileName,  KsGuiXml_File, KsGuiXml_Name   ) ;

        IF_ATTR_OK5( psAdfMaster, psAdfSlave, psAdfSlot, psFpgaMask, psFileName )

            eReturnStatus = _pxGuiCommandExecute->Cfg_ConfigFPGA( A_to_uI ( psAdfMaster ),
                                                                  A_to_uI ( psAdfSlave  ),
                                                                  A_to_uI ( psAdfSlot   ),
                                                                  A_to_uI ( psFpgaMask  ),
                                                                  psFileName,
                                                                 _poGuiCommandLine ) ;

        RELEASE_STR5( psAdfMaster, psAdfSlave, psAdfSlot, psFpgaMask, psFileName ) 
    }

    /////////////////////////////////////////////////////////////////////
    /// 
    /////////////////////////////////////////////////////////////////////
    else IF_CMD_MATCH( KsGuiCmd_Tcf_ConfigTabGab ) 
    {
        GET_CMD_ATTR( psAdfModule,   KsGuiXml_Tab,     KsGuiXml_Module    ) ;

        IF_ATTR_OK1( psAdfModule )

            eReturnStatus = _pxGuiCommandExecute->Tcf_ConfigTabGab  ( A_to_uI ( psAdfModule ),
                                                                     _poGuiCommandLine ) ;

        RELEASE_STR1( psAdfModule )
    }

    /////////////////////////////////////////////////////////////////////
    /// 
    /////////////////////////////////////////////////////////////////////
    else IF_CMD_MATCH( KsGuiCmd_Rrt_ClearAll ) 
    {

            eReturnStatus = _pxGuiCommandExecute->Rrt_ClearAll  ( _poGuiCommandLine ) ;

    }

    /////////////////////////////////////////////////////////////////////
    /// 
    /////////////////////////////////////////////////////////////////////
    else IF_CMD_MATCH( KsGuiCmd_Rrt_AddRange ) 
    {
        GET_CMD_ATTR( psAdfMaster,  KsGuiXml_Adf,  KsGuiXml_Master  ) ;
        GET_CMD_ATTR( psAdfSlave,   KsGuiXml_Adf,  KsGuiXml_Slave   ) ;
        GET_CMD_ATTR( psAdfSlot,    KsGuiXml_Adf,  KsGuiXml_Slot    ) ;
        GET_CMD_ATTR( psChipNum,    KsGuiXml_Chip, KsGuiXml_Num     ) ;
        GET_CMD_ATTR( psRegAddr,    KsGuiXml_Reg,  KsGuiXml_Addr    ) ;
        GET_CMD_ATTR( psRegAddrCnt, KsGuiXml_Reg,  KsGuiXml_AddrCnt ) ;

        IF_ATTR_OK6( psAdfMaster, psAdfSlave, psAdfSlot, psChipNum, psRegAddr, psRegAddrCnt )

            eReturnStatus = _pxGuiCommandExecute->Rrt_AddRange  ( A_to_uI ( psAdfMaster  ),
                                                                  A_to_uI ( psAdfSlave   ),
                                                                  A_to_uI ( psAdfSlot    ),
                                                                  A_to_uI ( psChipNum    ),
                                                                  A_to_uI ( psRegAddr    ),
                                                                  A_to_uI ( psRegAddrCnt ),
                                                                 _poGuiCommandLine ) ;

        RELEASE_STR6( psAdfMaster, psAdfSlave, psAdfSlot, psChipNum, psRegAddr, psRegAddrCnt )
    }

    /////////////////////////////////////////////////////////////////////
    /// 
    /////////////////////////////////////////////////////////////////////
    else IF_CMD_MATCH( KsGuiCmd_Rrt_StartTest ) 
    {
        GET_CMD_ATTR   ( psTestLoop,         KsGuiXml_Test, KsGuiXml_Loop ) ;
        GET_CMD_ATTR_WD( psExponent,         KsGuiXml_Test, KsGuiXml_Exponent,         "0" ) ;
        GET_CMD_ATTR_WD( psOnErrorStop,      KsGuiXml_Test, KsGuiXml_OnErrorStop,      "1" ) ;
        GET_CMD_ATTR_WD( psOnErrorVerifyAll, KsGuiXml_Test, KsGuiXml_OnErrorVerifyAll, "0" ) ;

        IF_ATTR_OK4( psTestLoop, psExponent, psOnErrorStop, psOnErrorVerifyAll )

            eReturnStatus = _pxGuiCommandExecute->Rrt_StartTest ( A_to_uI ( psTestLoop         ),
                                                                  A_to_uI ( psExponent         ),
                                                                  A_to_L  ( psOnErrorStop      ),
                                                                  A_to_L  ( psOnErrorVerifyAll ),
                                                                 _poGuiCommandLine ) ;

        RELEASE_STR4( psTestLoop, psExponent, psOnErrorStop, psOnErrorVerifyAll )
    }

    /////////////////////////////////////////////////////////////////////
    /// 
    /////////////////////////////////////////////////////////////////////
	else IF_CMD_MATCH( KsGuiCmd_Adc_Capture ) 
    {
        GET_CMD_ATTR_WD( psAdfMaster,  KsGuiXml_Adf,     KsGuiXml_Master,   "0" ) ;
        GET_CMD_ATTR_WD( psAdfSlave,   KsGuiXml_Adf,     KsGuiXml_Slave,    "0" ) ;
        GET_CMD_ATTR_WD( psAdfSlot,    KsGuiXml_Adf,     KsGuiXml_Slot,     "0" ) ;
        GET_CMD_ATTR_WD( psTtRelEta,   KsGuiXml_TrgTwr,  KsGuiXml_Rel_Eta,  "0" ) ;
        GET_CMD_ATTR_WD( psTtRelPhi,   KsGuiXml_TrgTwr,  KsGuiXml_Rel_Phi,  "0" ) ;
        GET_CMD_ATTR_WD( psTtEtaSign,  KsGuiXml_TrgTwr,  KsGuiXml_Eta_Sign, "0" ) ;
        GET_CMD_ATTR_WD( psTtEtaMagn,  KsGuiXml_TrgTwr,  KsGuiXml_Eta_Magn, "1" ) ;
        GET_CMD_ATTR_WD( psTtPhi,      KsGuiXml_TrgTwr,  KsGuiXml_Phi,      "1" ) ;
        GET_CMD_ATTR( psTtType,         KsGuiXml_TrgTwr,    KsGuiXml_Type       ) ;
        GET_CMD_ATTR( psThreshold,      KsGuiXml_Trigger,   KsGuiXml_Thresh     ) ;
        GET_CMD_ATTR( psMax_Try,        KsGuiXml_Trigger,   KsGuiXml_Max_Try    ) ;
        GET_CMD_ATTR( psRegDumpToFile,  KsGuiXml_Trigger,   KsGuiXml_DumpToFile ) ;

        IF_ATTR_OK5(     psAdfMaster,    psAdfSlave,     psAdfSlot, 
                         psTtRelEta,     psTtRelPhi )
        IF_ATTR_OK3(     psTtEtaSign,    psTtEtaMagn,    psTtPhi )
        IF_ATTR_OK4(     psTtType,       psThreshold,    psMax_Try,      
                         psRegDumpToFile )

            eReturnStatus = _pxGuiCommandExecute->Adc_Capture   ( A_to_uI ( psAdfMaster ),
                                                                  A_to_uI ( psAdfSlave  ),
                                                                  A_to_uI ( psAdfSlot   ),
                                                                  A_to_uI ( psTtRelEta   ),
                                                                  A_to_uI ( psTtRelPhi   ),
                                                           A_to_TTEtaSign ( psTtEtaSign  ),
                                                                  A_to_uI ( psTtEtaMagn  ),
                                                                  A_to_uI ( psTtPhi      ),
                                                          A_to_TTChanType ( psTtType     ),
                                                                  A_to_uI ( psThreshold ),
                                                                  A_to_uI ( psMax_Try   ),
                                                                  A_to_uI ( psRegDumpToFile),
                                                                 _poGuiCommandLine ) ;

        RELEASE_STR5(    psAdfMaster,    psAdfSlave,     psAdfSlot, 
                         psTtRelEta,     psTtRelPhi )
        RELEASE_STR3(    psTtEtaSign,    psTtEtaMagn,    psTtPhi )
        RELEASE_STR4(    psTtType,       psThreshold,    psMax_Try,      
                         psRegDumpToFile )
    }

    /////////////////////////////////////////////////////////////////////
    /// 
    /////////////////////////////////////////////////////////////////////
    else IF_CMD_MATCH( KsGuiCmd_Dac_ProgOneDac ) 
    {
        GET_CMD_ATTR_WD( psAdfMaster,  KsGuiXml_Adf,     KsGuiXml_Master,   "0" ) ;
        GET_CMD_ATTR_WD( psAdfSlave,   KsGuiXml_Adf,     KsGuiXml_Slave,    "0" ) ;
        GET_CMD_ATTR_WD( psAdfSlot,    KsGuiXml_Adf,     KsGuiXml_Slot,     "0" ) ;
        GET_CMD_ATTR_WD( psTtRelEta,   KsGuiXml_TrgTwr,  KsGuiXml_Rel_Eta,  "0" ) ;
        GET_CMD_ATTR_WD( psTtRelPhi,   KsGuiXml_TrgTwr,  KsGuiXml_Rel_Phi,  "0" ) ;
        GET_CMD_ATTR_WD( psTtEtaSign,  KsGuiXml_TrgTwr,  KsGuiXml_Eta_Sign, "0" ) ;
        GET_CMD_ATTR_WD( psTtEtaMagn,  KsGuiXml_TrgTwr,  KsGuiXml_Eta_Magn, "1" ) ;
        GET_CMD_ATTR_WD( psTtPhi,      KsGuiXml_TrgTwr,  KsGuiXml_Phi,      "1" ) ;
        GET_CMD_ATTR( psTtType,     KsGuiXml_TrgTwr,  KsGuiXml_Type    ) ;
        GET_CMD_ATTR( psDacValue,   KsGuiXml_Dac,     KsGuiXml_Value   ) ;

        IF_ATTR_OK5(     psAdfMaster,    psAdfSlave,     psAdfSlot, 
                         psTtRelEta,     psTtRelPhi )
        IF_ATTR_OK3(     psTtEtaSign,    psTtEtaMagn,    psTtPhi )
        IF_ATTR_OK2(     psTtType,       psDacValue )

            eReturnStatus = _pxGuiCommandExecute->Dac_ProgOneDac( A_to_uI ( psAdfMaster  ),
                                                                  A_to_uI ( psAdfSlave   ),
                                                                  A_to_uI ( psAdfSlot    ),
                                                                  A_to_uI ( psTtRelEta   ),
                                                                  A_to_uI ( psTtRelPhi   ),
                                                           A_to_TTEtaSign ( psTtEtaSign  ),
                                                                  A_to_uI ( psTtEtaMagn  ),
                                                                  A_to_uI ( psTtPhi      ),
                                                          A_to_TTChanType ( psTtType     ),
                                                                  A_to_uI ( psDacValue   ),
                                                                 _poGuiCommandLine ) ;

        RELEASE_STR5(    psAdfMaster,    psAdfSlave,     psAdfSlot, 
                         psTtRelEta,     psTtRelPhi )
        RELEASE_STR3(    psTtEtaSign,    psTtEtaMagn,    psTtPhi )
        RELEASE_STR2(    psTtType,       psDacValue )
    }

    /////////////////////////////////////////////////////////////////////
    /// 
    /////////////////////////////////////////////////////////////////////
    else IF_CMD_MATCH( KsGuiCmd_Dac_Find_DAC ) 
    {
        GET_CMD_ATTR_WD( psAdfMaster,  KsGuiXml_Adf,     KsGuiXml_Master,   "0" ) ;
        GET_CMD_ATTR_WD( psAdfSlave,   KsGuiXml_Adf,     KsGuiXml_Slave,    "0" ) ;
        GET_CMD_ATTR_WD( psAdfSlot,    KsGuiXml_Adf,     KsGuiXml_Slot,     "0" ) ;
        GET_CMD_ATTR_WD( psTtRelEta,   KsGuiXml_TrgTwr,  KsGuiXml_Rel_Eta,  "0" ) ;
        GET_CMD_ATTR_WD( psTtRelPhi,   KsGuiXml_TrgTwr,  KsGuiXml_Rel_Phi,  "0" ) ;
        GET_CMD_ATTR_WD( psTtEtaSign,  KsGuiXml_TrgTwr,  KsGuiXml_Eta_Sign, "0" ) ;
        GET_CMD_ATTR_WD( psTtEtaMagn,  KsGuiXml_TrgTwr,  KsGuiXml_Eta_Magn, "1" ) ;
        GET_CMD_ATTR_WD( psTtPhi,      KsGuiXml_TrgTwr,  KsGuiXml_Phi,      "1" ) ;
        GET_CMD_ATTR( psTtType,     KsGuiXml_TrgTwr,  KsGuiXml_Type    ) ;
        GET_CMD_ATTR( psAdcValue,   KsGuiXml_Adc,     KsGuiXml_Value   ) ;
        GET_CMD_ATTR( psAdcDumpToFile, KsGuiXml_Adc,  KsGuiXml_DumpToFile  ) ;

        IF_ATTR_OK5(     psAdfMaster,    psAdfSlave,     psAdfSlot, 
                         psTtRelEta,     psTtRelPhi )
        IF_ATTR_OK3(     psTtEtaSign,    psTtEtaMagn,    psTtPhi )
        IF_ATTR_OK3(     psTtType,       psAdcValue,     psAdcDumpToFile )

            eReturnStatus = _pxGuiCommandExecute->Dac_Find_DAC  ( A_to_uI ( psAdfMaster  ),
                                                                  A_to_uI ( psAdfSlave   ),
                                                                  A_to_uI ( psAdfSlot    ),
                                                                  A_to_uI ( psTtRelEta   ),
                                                                  A_to_uI ( psTtRelPhi   ),
                                                           A_to_TTEtaSign ( psTtEtaSign  ),
                                                                  A_to_uI ( psTtEtaMagn  ),
                                                                  A_to_uI ( psTtPhi      ),
                                                          A_to_TTChanType ( psTtType     ),
                                                                  A_to_uI ( psAdcValue   ),
                                                                  A_to_uI ( psAdcDumpToFile),
                                                                 _poGuiCommandLine ) ;

        RELEASE_STR5(    psAdfMaster,    psAdfSlave,     psAdfSlot, 
                         psTtRelEta,     psTtRelPhi )
        RELEASE_STR3(    psTtEtaSign,    psTtEtaMagn,    psTtPhi )
        RELEASE_STR3(    psTtType,       psAdcValue,     psAdcDumpToFile )
    }

    /////////////////////////////////////////////////////////////////////
    /// 
    /////////////////////////////////////////////////////////////////////
    else IF_CMD_MATCH( KsGuiCmd_Adf_InitCard ) 
    {
        GET_CMD_ATTR( psAdfMaster, KsGuiXml_Adf,  KsGuiXml_Master ) ;
        GET_CMD_ATTR( psAdfSlave,  KsGuiXml_Adf,  KsGuiXml_Slave  ) ;
        GET_CMD_ATTR( psAdfSlot,   KsGuiXml_Adf,  KsGuiXml_Slot   ) ;

        IF_ATTR_OK3( psAdfMaster, psAdfSlave, psAdfSlot )

            eReturnStatus = _pxGuiCommandExecute->Adf_InitCard  ( A_to_uI ( psAdfMaster ),
                                                                  A_to_uI ( psAdfSlave  ),
                                                                  A_to_uI ( psAdfSlot   ),
                                                                 _poGuiCommandLine ) ;

        RELEASE_STR3( psAdfMaster, psAdfSlave, psAdfSlot ) 
    }

    /////////////////////////////////////////////////////////////////////
    /// 
    /////////////////////////////////////////////////////////////////////
    else IF_CMD_MATCH( KsGuiCmd_Adf_SendPrn ) 
    {
        GET_CMD_ATTR( psAdfMaster, KsGuiXml_Adf,  KsGuiXml_Master ) ;
        GET_CMD_ATTR( psAdfSlave,  KsGuiXml_Adf,  KsGuiXml_Slave  ) ;
        GET_CMD_ATTR( psAdfSlot,   KsGuiXml_Adf,  KsGuiXml_Slot   ) ;

        IF_ATTR_OK3( psAdfMaster, psAdfSlave, psAdfSlot )

            eReturnStatus = _pxGuiCommandExecute->Adf_SendPrn   ( A_to_uI ( psAdfMaster ),
                                                                  A_to_uI ( psAdfSlave  ),
                                                                  A_to_uI ( psAdfSlot   ),
                                                                 _poGuiCommandLine ) ;

        RELEASE_STR3( psAdfMaster, psAdfSlave, psAdfSlot ) 
    }

    /////////////////////////////////////////////////////////////////////
    /// 
    /////////////////////////////////////////////////////////////////////
    else IF_CMD_MATCH( KsGuiCmd_Adf_SendBls ) 
    {
        GET_CMD_ATTR( psAdfMaster, KsGuiXml_Adf,  KsGuiXml_Master ) ;
        GET_CMD_ATTR( psAdfSlave,  KsGuiXml_Adf,  KsGuiXml_Slave  ) ;
        GET_CMD_ATTR( psAdfSlot,   KsGuiXml_Adf,  KsGuiXml_Slot   ) ;

        IF_ATTR_OK3( psAdfMaster, psAdfSlave, psAdfSlot )

            eReturnStatus = _pxGuiCommandExecute->Adf_SendBls   ( A_to_uI ( psAdfMaster ),
                                                                  A_to_uI ( psAdfSlave  ),
                                                                  A_to_uI ( psAdfSlot   ),
                                                                 _poGuiCommandLine ) ;

        RELEASE_STR3( psAdfMaster, psAdfSlave, psAdfSlot ) 
    }

    /////////////////////////////////////////////////////////////////////
    /// 
    /////////////////////////////////////////////////////////////////////
    else IF_CMD_MATCH( KsGuiCmd_Roc_AnalyzeEvt ) 
    {
            eReturnStatus = _pxGuiCommandExecute->Roc_AnalyzeEvt  ( _poGuiCommandLine ) ;
    }

    /////////////////////////////////////////////////////////////////////
    /// 
    /////////////////////////////////////////////////////////////////////
    else IF_CMD_MATCH( KsGuiCmd_Roc_DumpEvt ) 
    {
        GET_CMD_ATTR( psTotWord, KsGuiXml_Data,  KsGuiXml_LongCnt ) ;

        IF_ATTR_OK1( psTotWord )

            eReturnStatus = _pxGuiCommandExecute->Roc_DumpEvt  ( A_to_uI ( psTotWord ),
                                                                 _poGuiCommandLine ) ;

        RELEASE_STR1( psTotWord ) 
    }

    /////////////////////////////////////////////////////////////////////
    /// 
    /////////////////////////////////////////////////////////////////////
    else IF_CMD_MATCH( KsGuiCmd_Com_ParseFile ) 
    {
        GET_CMD_ATTR_WD( psComFile_Vio, KsGuiXml_Vio_File,  KsGuiXml_Name, "" ) ;
        GET_CMD_ATTR_WD( psComFile_Rio, KsGuiXml_Rio_File,  KsGuiXml_Name, "" ) ;
        GET_CMD_ATTR_WD( psComFile_Sio, KsGuiXml_Sio_File,  KsGuiXml_Name, "" ) ;
        GET_CMD_ATTR_WD( psComFile_Cfg, KsGuiXml_Cfg_File,  KsGuiXml_Name, "" ) ;
        GET_CMD_ATTR_WD( psComFile_Tti, KsGuiXml_Tti_File,  KsGuiXml_Name, "" ) ;
        GET_CMD_ATTR_WD( psComFile_Msg, KsGuiXml_Msg_File,  KsGuiXml_Name, "" ) ;
        GET_CMD_ATTR_WD( psComFile_Mcf, KsGuiXml_Mcf_File,  KsGuiXml_Name, "" ) ;

        IF_OPTIONAL_SPECIFIED      ( psComFile_Vio )
            eReturnStatus = _pxGuiCommandExecute->Cmd_Vio_File ( psComFile_Vio, _poGuiCommandLine ) ;
        else IF_OPTIONAL_SPECIFIED ( psComFile_Rio )
            eReturnStatus = _pxGuiCommandExecute->Cmd_Rio_File ( psComFile_Rio, _poGuiCommandLine ) ;
        else IF_OPTIONAL_SPECIFIED ( psComFile_Sio )
            eReturnStatus = _pxGuiCommandExecute->Cmd_Sio_File ( psComFile_Sio, _poGuiCommandLine ) ;
        else IF_OPTIONAL_SPECIFIED ( psComFile_Cfg )
            eReturnStatus = _pxGuiCommandExecute->Cmd_Cfg_File ( psComFile_Cfg, _poGuiCommandLine ) ;
        else IF_OPTIONAL_SPECIFIED ( psComFile_Tti )
            eReturnStatus = _pxGuiCommandExecute->Cmd_Tti_File ( psComFile_Tti, _poGuiCommandLine ) ;
        else IF_OPTIONAL_SPECIFIED ( psComFile_Msg )
            eReturnStatus = _pxGuiCommandExecute->Cmd_Msg_File ( psComFile_Msg, _poGuiCommandLine ) ;
        else IF_OPTIONAL_SPECIFIED ( psComFile_Mcf )
            eReturnStatus = _pxGuiCommandExecute->Cmd_Mcf_File ( psComFile_Mcf, _poGuiCommandLine ) ;
        else 
        {
            eReturnStatus = GuiReply_Bad ;
            CConsoleAccess::ErrorLine ( " No Command File Type Found in Gui Command" ) ;
        }
        RELEASE_STR7( psComFile_Vio, psComFile_Rio, psComFile_Sio, psComFile_Cfg, psComFile_Tti, psComFile_Msg, psComFile_Mcf ) 
    }

    /////////////////////////////////////////////////////////////////////
    /// 
    /////////////////////////////////////////////////////////////////////
    else IF_CMD_MATCH( KsGuiCmd_Com_File_Legacy ) 
    {
        GET_CMD_ATTR_WD( psComFile_Vio, KsGuiXml_Vio_File,  KsGuiXml_Name, "" ) ;
        GET_CMD_ATTR_WD( psComFile_Rio, KsGuiXml_Rio_File,  KsGuiXml_Name, "" ) ;
        GET_CMD_ATTR_WD( psComFile_Sio, KsGuiXml_Sio_File,  KsGuiXml_Name, "" ) ;
        GET_CMD_ATTR_WD( psComFile_Cfg, KsGuiXml_Cfg_File,  KsGuiXml_Name, "" ) ;
        GET_CMD_ATTR_WD( psComFile_Tti, KsGuiXml_Tti_File,  KsGuiXml_Name, "" ) ;
        GET_CMD_ATTR_WD( psComFile_Msg, KsGuiXml_Msg_File,  KsGuiXml_Name, "" ) ;
        GET_CMD_ATTR_WD( psComFile_Mcf, KsGuiXml_Mcf_File,  KsGuiXml_Name, "" ) ;

        IF_OPTIONAL_SPECIFIED      ( psComFile_Vio )
            eReturnStatus = _pxGuiCommandExecute->Cmd_Vio_File ( psComFile_Vio, _poGuiCommandLine ) ;
        else IF_OPTIONAL_SPECIFIED ( psComFile_Rio )
            eReturnStatus = _pxGuiCommandExecute->Cmd_Rio_File ( psComFile_Rio, _poGuiCommandLine ) ;
        else IF_OPTIONAL_SPECIFIED ( psComFile_Sio )
            eReturnStatus = _pxGuiCommandExecute->Cmd_Sio_File ( psComFile_Sio, _poGuiCommandLine ) ;
        else IF_OPTIONAL_SPECIFIED ( psComFile_Cfg )
            eReturnStatus = _pxGuiCommandExecute->Cmd_Cfg_File ( psComFile_Cfg, _poGuiCommandLine ) ;
        else IF_OPTIONAL_SPECIFIED ( psComFile_Tti )
            eReturnStatus = _pxGuiCommandExecute->Cmd_Tti_File ( psComFile_Tti, _poGuiCommandLine ) ;
        else IF_OPTIONAL_SPECIFIED ( psComFile_Msg )
            eReturnStatus = _pxGuiCommandExecute->Cmd_Msg_File ( psComFile_Msg, _poGuiCommandLine ) ;
        else IF_OPTIONAL_SPECIFIED ( psComFile_Mcf )
            eReturnStatus = _pxGuiCommandExecute->Cmd_Mcf_File ( psComFile_Mcf, _poGuiCommandLine ) ;
        else 
        {
            eReturnStatus = GuiReply_Bad ;
            CConsoleAccess::ErrorLine ( " No Command File Type Found in Gui Command" ) ;
        }
        RELEASE_STR7( psComFile_Vio, psComFile_Rio, psComFile_Sio, psComFile_Cfg, psComFile_Tti, psComFile_Msg, psComFile_Mcf ) 
    }

    /////////////////////////////////////////////////////////////////////
    /// 
    /////////////////////////////////////////////////////////////////////
    else IF_CMD_MATCH( KsGuiCmd_Daq_Run_Control ) 
    {
        GET_CMD_ATTR_WD( psRun_Number, KsGuiXml_Run_Params,  KsGuiXml_Run_Number,  "" ) ;
        GET_CMD_ATTR_WD( psEventMax,   KsGuiXml_Run_Params,  KsGuiXml_Evt_Max,    "0" ) ;
        GET_CMD_ATTR_WD( psCoolDown,   KsGuiXml_Run_Params,  KsGuiXml_CoolDown,   "0" ) ;
        GET_CMD_ATTR_WD( psModeBinary, KsGuiXml_Run_Params,  KsGuiXml_Mode_Bin,   "0" ) ;
        GET_CMD_ATTR_WD( psDirectory,  KsGuiXml_Run_Params,  KsGuiXml_Evt_Dir,     "" ) ;

        GET_CMD_ATTR_WD( psPause,      KsGuiXml_Run_Control, KsGuiXml_Pause,       "" ) ;
        GET_CMD_ATTR_WD( psResume,     KsGuiXml_Run_Control, KsGuiXml_Resume,      "" ) ;
        GET_CMD_ATTR_WD( psStop,       KsGuiXml_Run_Control, KsGuiXml_Stop,        "" ) ;

        IF_OPTIONAL_SPECIFIED ( psRun_Number )
            eReturnStatus = _pxGuiCommandExecute->Daq_Start_Run ( A_to_uI ( psRun_Number ), 
                                                                  A_to_uI ( psEventMax ),
                                                                  A_to_uI ( psCoolDown ), 
                                                                  A_to_uI ( psModeBinary ), 
                                                                  psDirectory, 
                                                                  _poGuiCommandLine ) ;

        else IF_OPTIONAL_SPECIFIED ( psPause )
            eReturnStatus = _pxGuiCommandExecute->Daq_Pause_Run ( _poGuiCommandLine ) ;
        
        else IF_OPTIONAL_SPECIFIED ( psResume )
            eReturnStatus = _pxGuiCommandExecute->Daq_Resume_Run( _poGuiCommandLine ) ;
        
        else IF_OPTIONAL_SPECIFIED ( psStop )
            eReturnStatus = _pxGuiCommandExecute->Daq_Stop_Run  ( _poGuiCommandLine ) ;
        
        else //just return the run status
            eReturnStatus = _pxGuiCommandExecute->Daq_Status    ( _poGuiCommandLine ) ;

        RELEASE_STR8( psRun_Number, psEventMax, psCoolDown, psModeBinary, psDirectory, psPause, psResume, psStop ) 
    }

    /////////////////////////////////////////////////////////////////////
    /// 
    /////////////////////////////////////////////////////////////////////
    else 
    {
        strcpy ( sReplyText, "<Gui Command Not implemented = " ) ;
        char*  psCommandName = XMLString::transcode( _poGuiCommandLine->m_pXMLcommand_name ) ; 
        strcat ( sReplyText, psCommandName ) ;
        CGuiCommandParser::XMLrelease ( psCommandName ) ;
        strcat ( sReplyText, ">" ) ;
        CConsoleAccess::ErrorLine     ( sReplyText ) ;
        _poGuiCommandLine->CopyReplyText ( sReplyText ) ;
        eReturnStatus = GuiReply_Bad ;
        goto Done ;
    }

Done:

    return eReturnStatus ;
}

