/*-------------------------------------------------------------------------------*/
  /*
     GuiCommandParser.cpp
     02-Nov-2004
  */
/*-------------------------------------------------------------------------------*/
#include "L1Cal_IIb_TCC/StdAfx.h"

#include "L1Cal_IIb_TCC/GlobalConstants.h"
#include "L1Cal_IIb_TCC/CustomDataTypes.h"

#include "L1Cal_IIb_TCC/GuiCommand.h"
#include                            "L1Cal_IIb_TCC/GuiCommandParser.h"

#include "L1Cal_IIb_TCC/HandleConsole.h"
#include "L1Cal_IIb_TCC/UtilsStrings.h"
#include "L1Cal_IIb_TCC/UtilsParsing.h"

// XML parser
#include <xercesc/dom/DOM.hpp>
#include <xercesc/framework/MemBufInputSource.hpp>

XERCES_CPP_NAMESPACE_USE

#include <iostream>

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/


/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
CGuiCommandParser::CGuiCommandParser () 
{

}


/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
CGuiCommandParser::~CGuiCommandParser () 
{
}


/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
EGuiCommandStatus      CGuiCommandParser::ParseXmlMessage     ( CGuiCommandLine*  _poGuiCommandLine ) 
{ 
    EGuiCommandStatus  eReturnStatus = GuiReply_Ok ;
    char               sReplyText[KiMaxLengthReplyTextToGui] ;
    const char*        psCommandPayload ;

    //\\//

    //find beginning of the command payload 
    psCommandPayload = FirstTokenBegin ( _poGuiCommandLine->m_pcCurrentCommandCharNum ) ;

    if ( psCommandPayload == NULL ) 
    {
        eReturnStatus = GuiReply_Bad ;
        strcpy ( sReplyText, "<No Command found after Command ID>" ) ;
        CConsoleAccess::ErrorLine     ( sReplyText ) ;
        _poGuiCommandLine->CopyReplyText ( sReplyText ) ;
        goto Done ;
    }
    else
    {
        // turn memory buffer into an InputSource
        MemBufInputSource  xmlBuf ( (XMLByte*)psCommandPayload, strlen(psCommandPayload), "GuiCmd" ) ;

        // now we can try parsing the XML in the message payload
        try 
        {
            _poGuiCommandLine->m_pDOMparser->parse(xmlBuf);
        }
        catch (const SAXException& toCatch) 
        {
            char* message = XMLString::transcode(toCatch.getMessage());
            CConsoleAccess::ErrorLine     ( (CString) "SAXException message is: " +  message ) ;
            CGuiCommandParser::XMLrelease ( message );
            eReturnStatus = GuiReply_Bad ;
            goto Done ;
        }
        catch (const XMLException& toCatch) 
        {
            char* message = XMLString::transcode(toCatch.getMessage());
            CConsoleAccess::ErrorLine     ( (CString) "XMLException message is: " +  message ) ;
            CGuiCommandParser::XMLrelease ( message );
            eReturnStatus = GuiReply_Bad ;
            goto Done ;
        }
        catch (const DOMException& toCatch) 
        {
            char* message = XMLString::transcode(toCatch.msg);
            CConsoleAccess::ErrorLine     ( (CString) "DOMException message is: " +  message ) ;
            CGuiCommandParser::XMLrelease ( message );
            eReturnStatus = GuiReply_Bad ;
            goto Done ;
        }
        catch (...) 
        {
            CConsoleAccess::ErrorLine     ( "Unexpected XML parsing Exception " ) ;
            eReturnStatus = GuiReply_Bad ;
            goto Done ;
        }

    }

Done:

    return eReturnStatus ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
EGuiCommandStatus      CGuiCommandParser::FindGuiCommand     ( CGuiCommandLine*  _poGuiCommandLine ) 
{ 
    EGuiCommandStatus   eReturnStatus = GuiReply_Ok ;
    char                sReplyText[KiMaxLengthReplyTextToGui] ;

    DOMNodeList*        pXmlCommandNodeList ;
    DOMAttr*            pXmlCommandNameAttr ;

    XMLCh*              pXmlCommandTag = NULL ;
    XMLCh*              pXmlNameTag    = NULL ;

    //\\//

    // now we start interpreting the XML in the message payload
        /** Adopt the DOM document
          * The caller will adopt the DOMDocument and thus is responsible to
          * call DOMDocument::release() to release the associated memory.
          * The parser will not delete it.   The ownership is transferred
          * from the parser to the caller.
          */
    // The comsumer will be responsible to release the document by calling CGuiCommandParser::ReleaseXmlMessage below
    _poGuiCommandLine->m_pXMLdocument = _poGuiCommandLine->m_pDOMparser->adoptDocument() ;

    // Find the node <COMMAND>
    pXmlCommandTag      = XMLString::transcode( KsGuiXml_Command ) ;

    pXmlCommandNodeList = _poGuiCommandLine->m_pXMLdocument->getElementsByTagName( pXmlCommandTag ) ;

    if ( pXmlCommandNodeList->getLength() != 0 ) 
    {
        _poGuiCommandLine->m_pXMLcommand_node_element = (DOMElement*) pXmlCommandNodeList->item(0) ;

        pXmlNameTag         = XMLString::transcode( KsGuiXml_Name ) ;
        pXmlCommandNameAttr = _poGuiCommandLine->m_pXMLcommand_node_element->getAttributeNode( pXmlNameTag ) ;
        if ( pXmlCommandNameAttr != NULL ) 
            _poGuiCommandLine->m_pXMLcommand_name = (XMLCh*) pXmlCommandNameAttr->getValue () ;
        else 
        {
            eReturnStatus = GuiReply_Bad ;
            strcpy ( sReplyText, "<No attribute NAME found in COMMAND>" ) ;
            CConsoleAccess::ErrorLine     ( sReplyText ) ;
            _poGuiCommandLine->CopyReplyText ( sReplyText ) ;
            goto Done ;
        }

    }


    else
    {
        eReturnStatus = GuiReply_Bad ;
        strcpy ( sReplyText, "<No node COMMAND found>" ) ;
        CConsoleAccess::ErrorLine     ( sReplyText ) ;
        _poGuiCommandLine->CopyReplyText ( sReplyText ) ;
        goto Done ;
    }

    CGuiCommandParser::XMLrelease ( pXmlCommandTag ) ;
    CGuiCommandParser::XMLrelease ( pXmlNameTag ) ;

Done:

    return eReturnStatus ;
}


/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
EGuiCommandStatus      CGuiCommandParser::ReleaseXmlMessage  ( CGuiCommandLine*  _poGuiCommandLine ) 
{ 
    EGuiCommandStatus  eReturnStatus = GuiReply_Ok ;

    //release the document memory
    _poGuiCommandLine->m_pXMLdocument->release() ;

    return eReturnStatus ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
bool                    CGuiCommandParser::GuiCommandMatch ( CGuiCommandLine*  _poGuiCommandLine,
                                                             char*             _psCommandName ) 
{
    bool bMatch = false ;

    //\\//

    XMLCh* pXMLCommandName = NULL ;
    pXMLCommandName = XMLString::transcode( _psCommandName ) ;
    if ( XMLString::compareIString ( _poGuiCommandLine->m_pXMLcommand_name, pXMLCommandName )  == 0 ) 

        bMatch = true ;

    CGuiCommandParser::XMLrelease ( pXMLCommandName ) ;

    return bMatch ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
char*                  CGuiCommandParser::GuiGetEltAttrName  ( CGuiCommandLine*  _poGuiCommandLine,
                                                               char*             _psElementTag,
                                                               char*             _psAttributeTag,
                                                               char*             _psDefaultValue ) 
{
    char*           psAttrValue = NULL ;
    char            sReplyText[KiMaxLengthReplyTextToGui] ;

    DOMNodeList*    pXmlNodeList ;
    DOMElement*     pXmlElement ;
    DOMAttr*        pXmlAttribute ;

    XMLCh*          pXmlElementTag   = NULL ;
    XMLCh*          pXmlAttributeTag = NULL ;

    //\\//

    pXmlElementTag = XMLString::transcode( _psElementTag ) ;
    pXmlNodeList   = _poGuiCommandLine->m_pXMLcommand_node_element->getElementsByTagName( pXmlElementTag ) ;
    
    if ( pXmlNodeList->getLength() != 0 ) 
    {
        pXmlElement = (DOMElement*) pXmlNodeList->item(0) ;
        pXmlAttributeTag = XMLString::transcode(_psAttributeTag) ;
        pXmlAttribute = pXmlElement->getAttributeNode( pXmlAttributeTag ) ;
        if ( pXmlAttribute != NULL ) 
        {
            psAttrValue = XMLString::transcode( pXmlAttribute->getValue () ) ;
        }
        else if ( _psDefaultValue != NULL ) 
        {
            XMLCh*    pxmlsDefaultValue = XMLString::transcode( _psDefaultValue ) ;
            psAttrValue = XMLString::transcode( pxmlsDefaultValue ) ;
            XMLString::release ( &pxmlsDefaultValue ) ;
        }
        else
        {                        
            strcpy ( sReplyText, "<NO attribute " ) ;
            strcat ( sReplyText, _psAttributeTag ) ;
            strcat ( sReplyText, " found in element " ) ;
            strcat ( sReplyText, _psElementTag ) ;
            strcat ( sReplyText, ">" ) ;
            CConsoleAccess::ErrorLine     ( sReplyText ) ;
            _poGuiCommandLine->CopyReplyText ( sReplyText ) ;
        }
    }
    else if ( _psDefaultValue != NULL ) 
    {
        XMLCh*    pxmlsDefaultValue = XMLString::transcode( _psDefaultValue ) ;
        psAttrValue = XMLString::transcode( pxmlsDefaultValue ) ;
        XMLString::release ( &pxmlsDefaultValue ) ;
    }
    else 
    {
        strcpy ( sReplyText, "<NO element " ) ;
        strcat ( sReplyText, _psElementTag ) ;
        strcat ( sReplyText, " found>" ) ;
        CConsoleAccess::ErrorLine     ( sReplyText ) ;
        _poGuiCommandLine->CopyReplyText ( sReplyText ) ;
    }

    CGuiCommandParser::XMLrelease ( pXmlElementTag ) ;
    CGuiCommandParser::XMLrelease ( pXmlAttributeTag ) ;

    return psAttrValue ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
void CGuiCommandParser::XMLrelease ( char* _pstring ) 
    {
    if ( _pstring  != NULL ) XMLString::release( &_pstring ) ;
    _pstring  = NULL ;
    return ;
}
void CGuiCommandParser::XMLrelease ( XMLCh* _pstring ) 
{
    if ( _pstring  != NULL ) XMLString::release( &_pstring ) ;
    _pstring  = NULL ;
    return ;
}
