/*-------------------------------------------------------------------------------*/
  /*
     GuiDlgIoVme.cpp
     06-Oct-2005
  */
/*-------------------------------------------------------------------------------*/

#include "L1Cal_IIb_TCC/StdAfx.h"

#include "L1Cal_IIb_TCC/HandleHardwareAccess.h"

#include                            "L1Cal_IIb_TCC/GuiDlgIoVme.h"

#include "L1Cal_IIb_TCC/UtilsStrings.h"
#include "L1Cal_IIb_TCC/UtilsBits.h"
#include "L1Cal_IIb_TCC/UtilsFiles.h"
#include "L1Cal_IIb_TCC/HandleCommandFiles.h"

#include "L1Cal_IIb_TCC/CommandFileVmeIO.h"

// static means created only once throughout life of program (to avoid wasting time)
// *AND* internal linkage (to avoid clash between modules)
#include "L1Cal_IIb_TCC/HandleConsole.h"
static CConsoleAccess oConsole ;

#include "L1Cal_IIb_TCC/HandleLogFile.h"


/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess DlgIoVme_ReadReg               ( unsigned long       _ulVmeAddress,
                                          unsigned long*      _pulDataIn,
                                          unsigned long       _ulRepeatCnt,
                                          char*               _psAddressSpace,
                                          char*               _psDataSize,
                                          char*               _psAccessPriv,
                                          char*               _psAccessType,
                                          char*               _psBit3Access ) 
{
    ESuccess        eReturnStatus = Ok ;

	int		            iAdaptor_Num ;
	unsigned int        uiAccess_Method ;
    unsigned int        uiNumBytes ;
    EVmeAmodAddrSpace   eAddrSpace ;
    EVmeAmodPrivilege   eAmodPriv ;
    EVmeAmodAccessType  eAmodType ;
    EYesNo              eSwapping = No ;
    void*               pVirtAddress ;
    
	unsigned int        j ;

    //\\//

    // only one Bit3 Adaptor is supported
	iAdaptor_Num   = 0 ;

    // retrieve the desired VME Address Space
    if ( _psAddressSpace == NULL ) 
        eAddrSpace = VmeA32 ;
    else if ( stricmp( _psAddressSpace, "A16" ) == 0 ) 
        eAddrSpace = VmeA16 ;
    else if ( stricmp( _psAddressSpace, "A24" ) == 0 ) 
        eAddrSpace = VmeA24 ;
    else if ( stricmp( _psAddressSpace, "A32" ) == 0 ) 
        eAddrSpace = VmeA32 ;
    else if ( stricmp( _psAddressSpace, "DPM" ) == 0 ) 
        eAddrSpace = Dpm ;
    else 
        CConsoleAccess::InternalErrorLine (  (CString) " Internal Error/DlgIoVme_ReadReg: Illegal Address Space <" 
                                                     + _psAddressSpace + ">" ) ;

    // retrieve the desired VME Address Modifier Information
    if ( _psAccessPriv == NULL ) 
        eAmodPriv = VmeAmodSuper ;
    else if ( stricmp( _psAccessPriv, "Super" ) == 0 ) 
        eAmodPriv = VmeAmodSuper ;
    else if ( stricmp( _psAccessPriv, "NonPriv" ) == 0 ) 
        eAmodPriv = VmeAmodNonPriv ;
    else 
        CConsoleAccess::InternalErrorLine (  (CString) " Internal Error/DlgIoVme_ReadReg: Illegal Access Privilege <" 
                                                     + _psAccessPriv + ">" ) ;

    if ( _psAccessType == NULL ) 
        eAmodType = VmeAmodData ;
    else if ( stricmp( _psAccessType, "Program" ) == 0 ) 
        eAmodType = VmeAmodProgr ;
    else if ( stricmp( _psAccessType, "Data" ) == 0 ) 
        eAmodType = VmeAmodData ;
    else if ( stricmp( _psAccessType, "Block" ) == 0 ) 
        eAmodType = VmeAmodBlock ;
    else 
        CConsoleAccess::InternalErrorLine (  (CString) " Internal Error/DlgIoVme_ReadReg: Illegal Access Type <" 
                                                     + _psAccessType + ">" ) ;

    // retrieve the desired VME transfer size
    if ( _psDataSize == NULL ) 
        uiNumBytes = 2 ;
    else if ( stricmp( _psDataSize, "D8" ) == 0 ) 
        uiNumBytes = 1 ;
    else if ( stricmp( _psDataSize, "D16" ) == 0 ) 
        uiNumBytes = 2 ;
    else if ( stricmp( _psDataSize, "D32" ) == 0 ) 
        uiNumBytes = 4 ;
    else 
        CConsoleAccess::InternalErrorLine (  (CString) " Internal Error/DlgIoVme_ReadReg: Illegal Data Size <" 
                                                     + _psDataSize + ">" ) ;

    // retrieve the desired VME Access Method
    if ( _psBit3Access == NULL ) 
        uiAccess_Method = KFlag_API ;
    else if ( stricmp( _psBit3Access, "API" ) == 0 ) 
        uiAccess_Method = KFlag_API ;
    else if ( stricmp( _psBit3Access, "MMAP" ) == 0 ) 
        uiAccess_Method = KFlag_MAP ;
    else 
        CConsoleAccess::InternalErrorLine (  (CString) " Internal Error/DlgIoVme_ReadReg: Illegal Bit3 Access Method <" 
                                                     + _psBit3Access + ">" ) ;

    // connect to Bit3 Apdaptor
    CBit3Adaptor61x   oBit3Adaptor ( iAdaptor_Num, eAddrSpace, eAmodPriv, eAmodType, VmeReadWrite ) ;
	// Make sure we start with an initialized adaptor
    oBit3Adaptor.ClearBit3Status ( ) ;

    // Now ready to go
    *_pulDataIn = 0 ;
    if ( uiAccess_Method  == KFlag_MAP ) 
    {
        pVirtAddress = oBit3Adaptor.MemoryMapThisVmeSegment ( _ulVmeAddress, uiNumBytes, eSwapping ) ;

#ifndef VME_IO_NONE
        switch ( uiNumBytes )
        {
            case 1 : *_pulDataIn = *(unsigned char*) pVirtAddress ; break ;
            case 2 : *_pulDataIn = *(unsigned short*)pVirtAddress ; break ;
            case 4 : *_pulDataIn = *(unsigned long*) pVirtAddress ; break ;
        }
#endif

    }
    else
    {
	    // read the VME Register by calling the Bit3 API
        if ( oBit3Adaptor.ReadVmeData ( _ulVmeAddress, 
                                        _pulDataIn,
                                         uiNumBytes ) != Ok ) 
            eReturnStatus = Error ;
    }

    	
	// display action on console
    CConsoleAccess::WriteLine ( (CString) _psBit3Access 
                                       + "/" + _psAddressSpace + "/" + _psDataSize 
                                       + " Read  : " 
	  	                               + I_to_A( *_pulDataIn, 10, 10, ' ' )
		                               + " (=" 
		                               + I_to_A( *_pulDataIn, 8, 16,'0' )
		                               + ") @ VME Address " 
		                               + I_to_A( _ulVmeAddress, 8, 16, '0' ) ) ;
	
	for ( j=1; j < _ulRepeatCnt ; j++) // do it a bunch more times, if requested
	{
        if ( uiAccess_Method  == KFlag_MAP ) 
        {
#ifndef VME_IO_NONE
            switch ( uiNumBytes )
            {
                case 1 : *_pulDataIn = *(unsigned char*) pVirtAddress ; break ;
                case 2 : *_pulDataIn = *(unsigned short*)pVirtAddress ; break ;
                case 4 : *_pulDataIn = *(unsigned long*) pVirtAddress ; break ;
            }
#endif

#if defined ( VME_IO_FAKE )  // simulate the ~5 us per IO
            for ( unsigned long z = 0 ; z < 1000 ; z ++ ) 
                *_pulDataIn = *(unsigned short*)pVirtAddress ; 
#endif
        }
        else
        {
	        // read the VME Register by calling the Bit3 API
            *_pulDataIn = 0 ;
            oBit3Adaptor.ReadVmeData   ( _ulVmeAddress, 
                                         _pulDataIn,
                                          uiNumBytes ) ;
        }
	}
	
    // Now release the Bit3 Mapping resources.
    // The handle to the Bit3 Adaptor will be closed automatically
    // when the Bit3 Adaptor C++ object is destroyed upon exiting this routine
    if ( uiAccess_Method  == KFlag_MAP ) 
        oBit3Adaptor.UnMapThisVmeSegment ( pVirtAddress, uiNumBytes ) ;


    return eReturnStatus ;
}
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess DlgIoVme_WriteReg              ( unsigned long       _ulVmeAddress,
                                          unsigned long       _ulDataOut,
                                          unsigned long*      _pulDataIn,
                                          unsigned long       _ulRepeatCnt,
                                          char*               _psAddressSpace,
                                          char*               _psDataSize,
                                          char*               _psAccessPriv,
                                          char*               _psAccessType,
                                          char*               _psBit3Access ) 
{
    ESuccess        eReturnStatus = Ok ;

	int		            iAdaptor_Num ;
	unsigned int        uiAccess_Method ;
    unsigned int        uiNumBytes ;
	bool                lRead_Back ;
	bool                lInvertDataOut ;
    EVmeAmodAddrSpace   eAddrSpace ;
    EVmeAmodPrivilege   eAmodPriv ;
    EVmeAmodAccessType  eAmodType ;
    EYesNo              eSwapping = No ;
    void*               pVirtAddress ;
    
	unsigned int        j ;

    //\\//


    // only one Bit3 Adaptor is supported
	iAdaptor_Num   = 0 ;
    // read back data just written
	lRead_Back = true ;
    // Do not invert the data being written (left over from Ironics IO)
	lInvertDataOut = false ;

    // retrieve the desired VME Address Space
    if ( _psAddressSpace == NULL ) 
        eAddrSpace = VmeA32 ;
    else if ( stricmp( _psAddressSpace, "A16" ) == 0 ) 
        eAddrSpace = VmeA16 ;
    else if ( stricmp( _psAddressSpace, "A24" ) == 0 ) 
        eAddrSpace = VmeA24 ;
    else if ( stricmp( _psAddressSpace, "A32" ) == 0 ) 
        eAddrSpace = VmeA32 ;
    else if ( stricmp( _psAddressSpace, "DPM" ) == 0 ) 
        eAddrSpace = Dpm ;
    else 
        CConsoleAccess::InternalErrorLine (  (CString) " Internal Error/DlgIoVme_ReadReg: Illegal Address Space <" 
                                                     + _psAddressSpace + ">" ) ;

    // retrieve the desired VME Address Modifier Information
    if ( _psAccessPriv == NULL ) 
        eAmodPriv = VmeAmodSuper ;
    else if ( stricmp( _psAccessPriv, "Super" ) == 0 ) 
        eAmodPriv = VmeAmodSuper ;
    else if ( stricmp( _psAccessPriv, "NonPriv" ) == 0 ) 
        eAmodPriv = VmeAmodNonPriv ;
    else 
        CConsoleAccess::InternalErrorLine (  (CString) " Internal Error/DlgIoVme_ReadReg: Illegal Access Privilege <" 
                                                     + _psAccessPriv + ">" ) ;

    if ( _psAccessType == NULL ) 
        eAmodType = VmeAmodData ;
    else if ( stricmp( _psAccessType, "Program" ) == 0 ) 
        eAmodType = VmeAmodProgr ;
    else if ( stricmp( _psAccessType, "Data" ) == 0 ) 
        eAmodType = VmeAmodData ;
    else if ( stricmp( _psAccessType, "Block" ) == 0 ) 
        eAmodType = VmeAmodBlock ;
    else 
        CConsoleAccess::InternalErrorLine (  (CString) " Internal Error/DlgIoVme_ReadReg: Illegal Access Type <" 
                                                     + _psAccessType + ">" ) ;

    // retrieve the desired VME transfer size
    if ( _psDataSize == NULL ) 
        uiNumBytes = 2 ;
    else if ( stricmp( _psDataSize, "D8" ) == 0 ) 
        uiNumBytes = 1 ;
    else if ( stricmp( _psDataSize, "D16" ) == 0 ) 
        uiNumBytes = 2 ;
    else if ( stricmp( _psDataSize, "D32" ) == 0 ) 
        uiNumBytes = 4 ;
    else 
        CConsoleAccess::InternalErrorLine (  (CString) " Internal Error/DlgIoVme_ReadReg: Illegal Data Size <" 
                                                     + _psDataSize + ">" ) ;

    // retrieve the desired VME Access Method
    if ( _psBit3Access == NULL ) 
        uiAccess_Method = KFlag_API ;
    else if ( stricmp( _psBit3Access, "API" ) == 0 ) 
        uiAccess_Method = KFlag_API ;
    else if ( stricmp( _psBit3Access, "MMAP" ) == 0 ) 
        uiAccess_Method = KFlag_MAP ;
    else 
        CConsoleAccess::InternalErrorLine (  (CString) " Internal Error/DlgIoVme_ReadReg: Illegal Bit3 Access Method <" 
                                                     + _psBit3Access + ">" ) ;

    // connect to Bit3 Apdaptor
    CBit3Adaptor61x   oBit3Adaptor ( iAdaptor_Num, eAddrSpace, eAmodPriv, eAmodType, VmeReadWrite ) ;
	// Make sure we start with an initialized adaptor
    oBit3Adaptor.ClearBit3Status ( ) ;


    // invert the data written out if requested
    if ( lInvertDataOut == true ) 
    switch ( uiNumBytes )
    {
        case 1 : _ulDataOut ^= 0xff ; break ;
        case 2 : _ulDataOut ^= 0xffff ; break ;
        case 4 : _ulDataOut ^= 0xffffffff ; break ;
    }

	// display action on console
    CConsoleAccess::WriteLine ( (CString) _psBit3Access 
                                       + "/" + _psAddressSpace + "/" + _psDataSize 
                                       + " Write : " 
	  	                               + I_to_A( _ulDataOut, 10, 10, ' ' )
		                               + " (=" 
		                               + I_to_A( _ulDataOut, 8, 16,'0' )
		                               + ") @ VME Address " 
		                               + I_to_A( _ulVmeAddress, 8, 16, '0' ) ) ;

        
    // Now make the IO operation happen
    if ( uiAccess_Method == KFlag_MAP ) 
    {
        pVirtAddress = oBit3Adaptor.MemoryMapThisVmeSegment ( _ulVmeAddress, uiNumBytes, eSwapping ) ;
        
#ifndef VME_IO_NONE
        switch ( uiNumBytes )
        {
            case 1 : *(unsigned char*) pVirtAddress = fubTruncToByte ( _ulDataOut ) ; break ;
            case 2 : *(unsigned short*)pVirtAddress = fuwTruncToWord ( _ulDataOut ) ; break ;
            case 4 : *(unsigned long*) pVirtAddress = _ulDataOut ; break ;
        }
#endif

    }
    else
    {
	    // Write the VME Register by calling the Bit3 API
        if ( oBit3Adaptor.WriteVmeData( _ulVmeAddress, 
                                        &_ulDataOut,
                                         uiNumBytes ) != Ok ) 
            eReturnStatus = Error ;
    }


    if ( lRead_Back == true ) 
    {
        *_pulDataIn = 0 ;
        if ( uiAccess_Method == KFlag_MAP ) 
        {
#ifndef VME_IO_NONE
            switch ( uiNumBytes )
            {
                case 1 : *_pulDataIn = *(unsigned char*) pVirtAddress ; break ;
                case 2 : *_pulDataIn = *(unsigned short*)pVirtAddress ; break ;
                case 4 : *_pulDataIn = *(unsigned long*) pVirtAddress ; break ;
            }
#endif
        }
        else
        {
	        // read the VME Register by calling the Bit3 API
            if ( oBit3Adaptor.ReadVmeData ( _ulVmeAddress, 
                                            _pulDataIn,
                                             uiNumBytes ) != Ok ) 
                eReturnStatus = Error ;
        }

        // display result on separate console
        CConsoleAccess::WriteLine ( (CString) _psBit3Access 
                                           + "/" + _psAddressSpace + "/" + _psDataSize 
                                           + " Read  : " 
	  	                                   + I_to_A( *_pulDataIn, 10, 10, ' ' )
		                                   + " (=" 
		                                   + I_to_A( *_pulDataIn, 8, 16,'0' )
		                                   + ") @ VME Address " 
		                                   + I_to_A( _ulVmeAddress, 8, 16, '0' ) ) ;
    }

    	

    // restore the data written out if it had been inverted
    if ( lInvertDataOut == true ) 
    switch ( uiNumBytes )
    {
        case 1 : _ulDataOut ^= 0xff ; break ;
        case 2 : _ulDataOut ^= 0xffff ; break ;
        case 4 : _ulDataOut ^= 0xffffffff ; break ;
    }

	for ( j=1; j < _ulRepeatCnt ; j++) // do it a bunch more times, if requested
	{
        if ( uiAccess_Method == KFlag_MAP ) 
        {        
#ifndef VME_IO_NONE
            switch ( uiNumBytes )
            {
                case 1 : *(unsigned char*) pVirtAddress = fubTruncToByte ( _ulDataOut ) ; break ;
                case 2 : *(unsigned short*)pVirtAddress = fuwTruncToWord ( _ulDataOut ) ; break ;
                case 4 : *(unsigned long*) pVirtAddress = _ulDataOut ; break ;
            }
#endif
        }
        else
        {
	        // Write the VME Register by calling the Bit3 API
            oBit3Adaptor.WriteVmeData  ( _ulVmeAddress, 
                                        &_ulDataOut,
                                         uiNumBytes ) ;
        }

        if ( lRead_Back == true ) 
        {
            if ( uiAccess_Method == KFlag_MAP ) 
            {
#ifndef VME_IO_NONE
                switch ( uiNumBytes )
                {
                    case 1 : *_pulDataIn = *(unsigned char*) pVirtAddress ; break ;
                    case 2 : *_pulDataIn = *(unsigned short*)pVirtAddress ; break ;
                    case 4 : *_pulDataIn = *(unsigned long*) pVirtAddress ; break ;
                }
#endif
            }
            else
            {
	            // read the VME Register by calling the Bit3 API
                *_pulDataIn = 0 ;
                oBit3Adaptor.ReadVmeData   ( _ulVmeAddress, 
                                             _pulDataIn,
                                              uiNumBytes ) ;
            }
        }
	}	

    // Now release the Bit3 Mapping resources.
    // The handle to the Bit3 Adaptor will be closed automatically
    // when the Bit3 Adaptor C++ object is destroyed upon exiting this routine.
    if ( uiAccess_Method == KFlag_MAP ) 
        oBit3Adaptor.UnMapThisVmeSegment ( pVirtAddress, uiNumBytes ) ;


    return eReturnStatus ;
}
/*-------------------------------------------------------------------------------*/
