/*-------------------------------------------------------------------------------*/
  /*
     GuiDlgTestRandReg.cpp
     15-Dec-2004
  */
/*-------------------------------------------------------------------------------*/

#include "L1Cal_IIb_TCC/StdAfx.h"

#include "L1Cal_IIb_TCC/AdfRegBase.h"
#include                            "L1Cal_IIb_TCC/GuiDlgTestRandReg.h"

#include "L1Cal_IIb_TCC/UtilsStrings.h"
#include "L1Cal_IIb_TCC/UtilsFiles.h"
#include "L1Cal_IIb_TCC/UtilsBits.h"
#include "L1Cal_IIb_TCC/UtilsRandom.h"
#include "L1Cal_IIb_TCC/HandleCommandFiles.h"

// static means created only once throughout life of program (to avoid wasting time)
// *AND* internal linkage (to avoid clash between modules)
#include "L1Cal_IIb_TCC/HandleConsole.h"
static CConsoleAccess oConsole ;

#include "L1Cal_IIb_TCC/HandleLogFile.h"

/////////////////////////////////////////////////////////////////////////////
///  Global Variables
/////////////////////////////////////////////////////////////////////////////

	CString	        g_osStatusMessage ;

   	unsigned int    g_uiTotReg ;
    CAdfRegBase**   g_apoReg ;    //this will hold a dynamic list of pointers to register
    CAdfRegBase*    g_poLastReg ; //last in linked chain of registers
    

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess DlgTestRandReg_AddRange ( unsigned int     _uiMasterNum,
                                   unsigned int     _uiSlaveNum,
                                   unsigned int     _uiSlotNum,
                                   unsigned int     _uiChipNum,
                                   unsigned int     _uiRegAddrStart,
                                   unsigned int     _uiRegAddrCount ) 
{
    ESuccess        eReturnStatus = Ok ;

    unsigned int    uiRegAddress ;
    TuwRegWord      uwData_In ;
    TuwRegWord      uw0x0000 = 0x0000 ;

    CAdfRegBase*    poRegister ;
    TulVmeAddress   ulVmeAddress ;
    ESuccess        eStatus ;
    
    //\\//

//	UpdateData(KReadFromScreen); 

    // display the request action
    // create a temporary register, just to display its address
    ulVmeAddress = CAdfRegBase::ConvertToVmeAddress   ( fubTruncToByte(_uiMasterNum),
                                                        fubTruncToByte(_uiSlaveNum),
                                                        fubTruncToByte(_uiSlotNum),
                                                        fubTruncToByte(_uiChipNum),
                                                        fuwTruncToWord(_uiRegAddrStart) ) ;    
    poRegister = new CAdfRegBase   ( ulVmeAddress ) ; 

    oConsole.WriteLine ( (CString) " Adding "
                                   + I_to_A ( _uiRegAddrCount ) 
                                   + " Register(s) Starting "
                                   + poRegister->f_osRegAddress() ) ;
    // delete temporary register
    delete poRegister ;


    for ( uiRegAddress = _uiRegAddrStart ; 
	      uiRegAddress < _uiRegAddrStart + _uiRegAddrCount ; 
		  uiRegAddress++ ) 
	{ 
        // build a vme address for this register
        ulVmeAddress = CAdfRegBase::ConvertToVmeAddress   ( fubTruncToByte(_uiMasterNum),
                                                            fubTruncToByte(_uiSlaveNum),
                                                            fubTruncToByte(_uiSlotNum),
                                                            fubTruncToByte(_uiChipNum),
                                                            fuwTruncToWord(uiRegAddress) ) ;    

        // look to see if this register has already been defined before
        poRegister = DlgTestRandReg_FindRegister ( ulVmeAddress ) ;
        
        if ( poRegister != NULL )
        {
            eReturnStatus = Error ;
            oConsole.ErrorLine ( (CString) " Register Rejected: Already Included in Test Coverage" ) ;
            oConsole.ErrorLine ( (CString) "_" + poRegister->f_osRegAddress() ) ;
            break ; 
        }

        // create the register
        poRegister = new CAdfRegBase   ( ulVmeAddress,
                                        &g_poLastReg,
                                         0xffff,    //   _uwWriteMask 
                                         0xffff ) ; //   _uwReadMask 
        
        poRegister->f_okReadReg   ( &uwData_In ) ;

        // display result on separate 
/*
        oConsole.WriteLine ( (CString) " Adding Reg #"
	  	                                 + I_to_A( g_uiTotReg, 3 )
                                         + ", current content = "
	  	                                 + I_to_A( uwData_In, 5, 10, ' ' )
                                         + " = " 
							             + I_to_A( uwData_In, 4, 16, '0' ) ) ;
        oConsole.WriteLine ( (CString) "_" + poRegister->f_osRegAddress() ) ;
*/
        // verify this register exists
        eStatus = poRegister->f_okWriteReg ( &uw0x0000 ) ;
        if ( eStatus != Ok )
        {
            eReturnStatus = Error ;
            oConsole.ErrorLine ( (CString) "_Register Rejected: Cannot Initialize to 0x0000" ) ;

            //now unwind from having defined this register
            g_poLastReg = poRegister->f_poPrevReg() ;
            delete poRegister ;
        }
        else
        {
            g_uiTotReg++ ;
        }

    }

	g_osStatusMessage = "Total Reg in Test " + I_to_A ( g_uiTotReg, 3, 10, ' ', 3 ) ;
    oConsole.WriteLine ( g_osStatusMessage ) ;

    return eReturnStatus ; 	
}



/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
void DlgTestRandReg_ClearAll ()
{
//    UpdateData(KReadFromScreen); 

    // start with new set of registers every time we enter the test
    DlgTestRandReg_ClearAllEntries ( ) ;	

	g_osStatusMessage = "Total Reg in Test " + I_to_A ( g_uiTotReg, 3, 10, ' ', 3 ) ;
    oConsole.WriteLine ( g_osStatusMessage ) ;

//    UpdateData(KWriteToScreen); 
}


/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
void DlgTestRandReg_StartTest ( unsigned int     _uiTestLoop,
                                unsigned int     _uiExponent,
                                bool             _lOnErrorStop,
                                bool             _lOnErrorVerifyAll ) 
{
	unsigned long   ulLoopNum ;
    ESuccess        eLoopStatus = Ok ;
    unsigned long   ulErrorCount ;

    unsigned long   ulTestRegNum ;

    TuwRegWord      uwData_Out ;

    CAdfRegBase*    poRegister ;
    ESuccess        eStatus ;

    //\\
    
//    UpdateData(KReadFromScreen); 

    oConsole.WriteLine ( "" ) ; 

    //reset the progress bar
//    m_oProgress_Control.SetPos( 0 ) ;
        
    // Check that soem register(s) have been defined
    if ( g_uiTotReg == 0 )
    {
        g_osStatusMessage = " No Register Defined, Test Aborted" ;
        oConsole.ErrorLine ( g_osStatusMessage ) ; 
        goto Done ;
    }

    // define the array of pointers used for direct random access
    eStatus = DlgTestRandReg_BuildPointerArray ( ) ; 
    if ( eStatus != Ok ) 
    {
        g_osStatusMessage = " Pointer Array Initialization Error, Test Aborted" ;
        oConsole.ErrorLine ( g_osStatusMessage ) ; 
        goto Done ;
    }

    // zero all regsiters at the beginning of the test
    eStatus = DlgTestRandReg_ZeroAllRegisters ( ) ; 
    if ( eStatus != Ok ) 
    {
        g_osStatusMessage = " Register Initialization Error, Test Aborted" ;
        oConsole.ErrorLine ( g_osStatusMessage ) ; 
        goto Done ;
    }
    
    //put together the pieces of the requested number of loops
	if      ( _uiExponent == KFlag_x1   )  _uiTestLoop *= KValue_x1 ;
	else if ( _uiExponent == KFlag_kilo )  _uiTestLoop *= KValue_kilo ;
	else if ( _uiExponent == KFlag_Mega )  _uiTestLoop *= KValue_Mega ;
	else if ( _uiExponent == KFlag_Giga )  _uiTestLoop *= KValue_Giga ;
	_uiExponent = KFlag_x1 ;

    // advertize beginning of test
    g_osStatusMessage = "Starting "
                      + I_to_A ( _uiTestLoop, 3, 10, ' ', 3 )
                      + " Loop(s) of Random Register Test " ;

    oConsole.WriteLine ( (CString) "\n" 
                                 + " _____ " + g_osStatusMessage + " ____________________" 
                                 + "\n" ) ;
//    UpdateData(KWriteToScreen); 
//	UpdateWindow() ; // ask for window repaint

    //reset error counter
	ulErrorCount = 0 ;

    for ( ulLoopNum = 0 ; 
          ulLoopNum < _uiTestLoop ; 
        )
    {
		for ( int i = 1 ; 
		      ( i <= 100000 ) && ( ulLoopNum < _uiTestLoop ) ; 
			  i++ ) 
		{ // 100 k loops at a time, then show progress

            ulLoopNum++ ;
            eLoopStatus = Ok ;
		
			// pick a random register in the list
			ulTestRegNum = ulRandom ( 0, g_uiTotReg-1 ) ; 
            poRegister = g_apoReg[ulTestRegNum] ;

			// pick some new random value to write 
			uwData_Out = fuwTruncToWord ( ulRandom ( 0, 0xffff) ) ;

            eStatus = poRegister->f_okWriteReg ( &uwData_Out ) ;

            if ( eStatus != Ok ) 
            {
                ulErrorCount++ ;
                eLoopStatus = Error ;
			}

            // display loop number if we had error(s) during this loop
            if ( eLoopStatus == Error ) 
            {   

                //Verify All Registers
                if ( _lOnErrorVerifyAll == TRUE ) 
                {
                    DlgTestRandReg_VerifyAllAgain() ;
                }

                oConsole.WriteLine ( (CString) " _____ Above Error(s) for Loop #"
                                              + I_to_A ( ulLoopNum, 3, 10, ' ', 3 )
                                              + " _________________________________" 
                                              + "\n" ) ;
                //Beep
//                if ( m_lOnErrorBeep == TRUE ) 
//                {
//                    MessageBeep(MB_ICONEXCLAMATION);
//                }

                //Abort the test
                if ( _lOnErrorStop == TRUE )
                {
                    oConsole.WriteLine ( (CString) " _____ Test Stopped on Error _____" + "\n" ) ;
                    goto Done ;
                }

            }
		}

		// paint some feedback to the dialog box every 100k loops
		// "%" is the C "Remainder operator"
		g_osStatusMessage = "Total Reg " + I_to_A ( g_uiTotReg,   3, 10, ' ', 3 ) 
						+ " /Loop "    + I_to_A ( ulLoopNum,    3, 10, ' ', 3 )
						+ " /Error "   + I_to_A ( ulErrorCount, 3, 10, ' ', 3 ) 
						+ " -- Running..." ;
//		m_oProgress_Control.SetPos( (int) (100 * ( (float) ulLoopNum / (float) _uiTestLoop ) ) ) ;
//        UpdateData(KWriteToScreen); 
//		UpdateWindow() ; // ask for window repaint
	}

Done :

    // if the test was aborted, set the number of loops to what was actually acomplished
    if ( ulLoopNum < _uiTestLoop ) _uiTestLoop = ulLoopNum ;

    // prepare summary for read-only message on Dialog Box
    g_osStatusMessage = "  Loop "    + I_to_A ( ulLoopNum,      3, 10, ' ', 3 )
					  + " /Error "   + I_to_A ( ulErrorCount,   3, 10, ' ', 3 )
					  + " -- Done..." ;

    // write summary to console
    oConsole.WriteLine ( (CString) " _____ Done: "
                                  + I_to_A ( ulErrorCount,                        3, 10, ' ', 3 )
                                  + " Error(s) for "
                                  + I_to_A ( ulLoopNum, 3, 10, ' ', 3 )
                                  + " Loop(s) of Random Register Test ______" 
                                  + "\n" ) ;

    // reset the Loop Count request to a begign value (=1)
	_uiTestLoop = 1 ; 
	
    // display result back in popup box
//    UpdateData(KWriteToScreen); 

    //Flush latest messages to logfile
    CLogFile::FlushLogFile () ;
		
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
CAdfRegBase*   DlgTestRandReg_FindRegister  ( TulVmeAddress  _ulVmeAddress ) 
{
    ESuccess        lReturnStatus = Ok ;

    unsigned int    uiRegAddress ;

    CAdfRegBase*       poRegister = NULL ;

    //\\//

    //make sure there are some registers to scan through
    if ( g_poLastReg == NULL ) goto Done ;

    //start from the last member in the chain
    poRegister = g_poLastReg ;

    // crawl through all registers defined so far
    for ( uiRegAddress = g_uiTotReg ;
	      uiRegAddress > 0  ; 
          uiRegAddress-- ) 
    {
        if ( poRegister == NULL ) 
        {
            oConsole.InternalErrorLine ( (CString) " Internal Error/FindRegister:"
                                                 + " End of Register Chain Found Before Expected" ) ;
            goto Done ;
        }

        if ( poRegister->f_ulVmeAddress() == _ulVmeAddress ) goto Done ;

        // jump to the previous register
        poRegister = poRegister->f_poPrevReg() ;

    }

    if ( poRegister != NULL ) 
    {
        oConsole.InternalErrorLine ( (CString) " Internal Error/FindRegister:"
                                             + " Too Many Registers in Chain " ) ;
    }

    poRegister = NULL ;

Done:

    return poRegister ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess    DlgTestRandReg_BuildPointerArray     ( ) 
{
    ESuccess        lReturnStatus = Ok ;

    unsigned int    uiRegIndex ;

    CAdfRegBase*    poRegister ;

    //\\//

    oConsole.WriteLine ( (CString) " Building an Array of Pointers For Random Access to Register Objects" ) ;

    if ( ( g_poLastReg == NULL ) || ( g_uiTotReg == 0 ) ) 
    {
        oConsole.InternalErrorLine ( (CString) " Internal Error/BuildPointerArray:"
                                             + " No Registers Defined " ) ;
	    lReturnStatus = Error ;
	    goto Done ;
    }

    if ( g_apoReg != NULL ) 
    {   // delete any previous array (there are brackets because it is an array) 
        // and clear the pointer for future reference
        delete [] g_apoReg ;
        g_apoReg = NULL ;
    }

    // now that we know how many registers are defined, 
    // we can create an array of pointers 
    g_apoReg = new ( CAdfRegBase* [g_uiTotReg] ) ;

    //start from the last member in the chain
    poRegister = g_poLastReg ;
    // crawl through all registers defined
    for ( uiRegIndex = g_uiTotReg ;
	      uiRegIndex > 0  ; 
          uiRegIndex-- ) 
    {
        if ( poRegister == NULL ) 
        {
            oConsole.InternalErrorLine ( (CString) " Internal Error/BuildPointerArray:"
                                                 + " End of Register Chain Found Before Expected" ) ;
	        lReturnStatus = Error ;
	        goto Done ;
        }

        //remember the address of this register for direct access through array
        g_apoReg[uiRegIndex-1] = poRegister ;
        // jump to the previous register
        poRegister = poRegister->f_poPrevReg() ;
    }

    if ( poRegister != NULL ) 
    {
        oConsole.InternalErrorLine ( (CString) " Internal Error/BuildPointerArray:"
                                             + " Too Many Registers in Chain " ) ;
	    lReturnStatus = Error ;
	    goto Done ;
    }

Done:

    return lReturnStatus ;
}


/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess    DlgTestRandReg_ZeroAllRegisters    ( ) 
{
    ESuccess        lReturnStatus = Ok ;
    ESuccess        eStatus ;

    unsigned int    uiRegIndex ;

    CAdfRegBase*    poRegister ;

    TuwRegWord      uwZero = 0x0000 ;


    //\\//

    oConsole.WriteLine ( (CString) " Zeroeing All Registers in the Test" ) ;

    if ( ( g_poLastReg != NULL ) && ( g_uiTotReg != 0 ) ) 
    {
        //start from the last member in the chain
        poRegister = g_poLastReg ;

        // crawl through all registers defined
        for ( uiRegIndex = g_uiTotReg ;
	          uiRegIndex > 0  ; 
              uiRegIndex-- ) 
        {
            // cross-check there was no data loss in the array of pointers 
            // (if it was already built)
            if ( ( g_apoReg != NULL ) 
              && ( g_apoReg[uiRegIndex-1] != poRegister ) )
            {
                //cleanup error: register object address lost
                oConsole.InternalErrorLine ( (CString) " Internal Error/ZeroAllRegisters:"
                                                     + " Register Object Address Lost " ) ;
	            lReturnStatus = Error ;
	            goto Done ;
            }

            // Clear the Register,
            // do NOT assume you know what the previous content was
            eStatus = poRegister->f_okWriteOnly ( &uwZero ) ;
            if ( eStatus != Ok ) lReturnStatus = Error ;
            eStatus = poRegister->f_okWriteReg ( &uwZero ) ;
            if ( eStatus != Ok ) lReturnStatus = Error ;

			// jump to the previous register
			poRegister = poRegister->f_poPrevReg() ;
        }
    }

Done:

    return lReturnStatus ;
}


/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess    DlgTestRandReg_ClearAllEntries ( ) 
{
    ESuccess        lReturnStatus = Ok ;

    unsigned int    uiRegIndex ;

    CAdfRegBase*    poRegister ;
    CAdfRegBase*    poRegTemp ;

    //\\//

    oConsole.WriteLine ( (CString) " Clearing All Previously Defined Register Entries " ) ;

    if ( ( g_poLastReg != NULL ) && ( g_uiTotReg != 0 ) ) 
    {
        //start from the last member in the chain
        poRegister = g_poLastReg ;

        // crawl through all registers defined
        for ( uiRegIndex = g_uiTotReg ;
	          uiRegIndex > 0  ; 
              uiRegIndex-- ) 
        {
            if ( poRegister == NULL ) 
            {
                oConsole.InternalErrorLine ( (CString) " Internal Error/ClearAllEntries:"
                                                     + " End of Register Chain Found Before Expected" ) ;
	            lReturnStatus = Error ;
	            goto Done ;
            }

            // use current register to find previous register in chain,
            // but first remember current register to delete it.
            poRegTemp   = poRegister ;
            poRegister  = poRegister->f_poPrevReg() ;
            delete poRegTemp ;
        }

              if ( poRegister != NULL ) 
        {
            oConsole.InternalErrorLine ( (CString) " Internal Error/ClearAllEntries:"
                                                 + " Too Many Registers in Chain " ) ;
	        lReturnStatus = Error ;
	        goto Done ;
        }

    }

Done:

    g_poLastReg = NULL ;
    g_uiTotReg  = 0 ;
    if ( g_apoReg != NULL ) 
    {   // delete the array (there are brackets because it is an array) 
        // and clear the pointer for future reference
        delete [] g_apoReg ;
        g_apoReg = NULL ;
    }

	g_osStatusMessage = "Select Registers to Test" ;	

    return lReturnStatus ;

}



/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess    DlgTestRandReg_VerifyAllAgain() 
{
    ESuccess        lReturnStatus = Ok ;
    ESuccess        eStatus ;

    unsigned int    uiRegIndex ;

    CAdfRegBase*    poRegister ;

    //\\//

    oConsole.WriteLine ( (CString) " Verifying All Registers"
                                 + " (starting from the last defined)\n" ) ;

    if ( ( g_poLastReg != NULL ) && ( g_uiTotReg != 0 ) ) 
    {
        //start from the last member in the chain
        poRegister = g_poLastReg ;

        // crawl through all registers defined
        for ( uiRegIndex = g_uiTotReg ;
	          uiRegIndex > 0  ; 
              uiRegIndex-- ) 
        {
            // cross-check there was no data loss in the array of pointers 
            // (if it was already built)
            if ( ( g_apoReg != NULL ) 
              && ( g_apoReg[uiRegIndex-1] != poRegister ) )
            {
                //cleanup error: register object address lost
                oConsole.InternalErrorLine ( (CString) " Internal Error/VerifyAllAgain:"
                                                     + " Register Object Address Lost " ) ;
	            lReturnStatus = Error ;
	            goto Done ;
            }

            // Read the Register,
            eStatus = poRegister->f_okReadReg ( ) ;
            if ( eStatus != Ok ) lReturnStatus = Error ;

            //verify content
            if ( ( poRegister->f_uwLastRead() != poRegister->f_uwLastWrite() ) //mismatch
              && ( poRegister->f_ynModified() == Yes ) )//and the register has already been written once
            {
                oConsole.ErrorLine ( (CString) " Error during Rand Reg Test Verify All: Read = "
                                              + I_to_A ( poRegister->f_uwLastRead(), 4, 16, '0' ) 
                                              + " / Last Written = "
                                              + I_to_A ( poRegister->f_uwLastWrite(), 4, 16, '0' ) ) ;
                oConsole.ErrorLine ( (CString) "_" + poRegister->f_osRegAddress() + "\n" ) ;

	            lReturnStatus = Error ;
            }

            // jump to the previous register
			poRegister = poRegister->f_poPrevReg() ;
        }
    }

Done:

    return lReturnStatus ;
	
}
