/*-------------------------------------------------------------------------------*/
  /*
    Bit3Adaptor61x.cpp
    26-Aug-98
  */
/*-------------------------------------------------------------------------------*/
#include "L1Cal_IIb_TCC/StdAfx.h"

#include "L1Cal_IIb_TCC/GlobalConstants.h"
#include "L1Cal_IIb_TCC/CustomDataTypes.h"

#include "L1Cal_IIb_TCC/UtilsStrings.h"

#include                            "L1Cal_IIb_TCC/HandleBit3Adaptor61x.h"

//bit3 VME Adaptor Programming Interface
#ifdef  WIN32

//    #define BT983       // BT983 for bit3 V2.4
    #define BT984     // BT984 for newer driver series e.g. bit3 V1.3.0

    #ifdef  BT984
        #define __winxp__
        #define BT_NTDRIVER
    #endif

#else
    #define BT1003
#endif

#include "bit3/btapi.h"


// static means created only once throughout life of program (to avoid wasting time)
// *AND* internal linkage (to avoid clash between modules)
#include "L1Cal_IIb_TCC/HandleConsole.h"
static CConsoleAccess oConsole ;


/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
// Static data members must be initialized at file scope

int     CBit3Adaptor61x::mg_iAdaptorNumForL1Cal = KiAdaptorNumForL1Cal ;

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
CBit3Adaptor61x::CBit3Adaptor61x  ( ) 
{
}


/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
CBit3Adaptor61x::CBit3Adaptor61x  ( unsigned int            _uiDevUnitNum,
                                    EVmeAmodAddrSpace       _eAddrSpace,
                                    EVmeAmodPrivilege       _eAmodPriv,
                                    EVmeAmodAccessType      _eAmodType,
                                    EVmeAccessRestrict      _eAccessRestrict ) 
{
	//\\//

    // Remember Adaptor ID
    m_uiAdaptorNum      = _uiDevUnitNum ;

    // Resource Type A16, A24, A32, or DPM
    m_eAddrSpace        = _eAddrSpace ;

    // Address Modifier Information besides address space
    m_eAmodPriv         = _eAmodPriv ;
    m_eAmodType         = _eAmodType ;

    // Access Mode: Read, Write or RW
    m_eAccessRestrict   = _eAccessRestrict ;

    // Initial Adaptor Status
	m_eStatus           = Ok ;

    // Initial Interrupt Service Routine Parameters
    m_xIcbrParameters.pBit3Adaptor  = this ;
    m_xIcbrParameters.eVmeStatus    = Ok ;
    
    // build bit3 device name for 32 bit address access
    /////////////////////////////////////
    if ( BuildBit3AdaptorName ( ) != Ok ) 
    {
        oConsole.InternalErrorLine ( (CString) "Failure Building Device Name for "
                                     + "Bit3 Adaptor #" 
                                     + I_to_A (m_uiAdaptorNum) ) ;
		m_eStatus = Error ;
        goto done ;
	}


    // open handle to bit 3 device driver
    /////////////////////////////////////
    if ( OpenBit3Adaptor ( ) != Ok ) 
    {
        oConsole.ErrorLine ( (CString) "Failure Opening Access to "
                                     + "Bit3 Adaptor #" 
                                     + I_to_A (m_uiAdaptorNum) ) ;

		m_eStatus = Error ;
        goto done ;
	}


    // Initialize the Adaptor
    ///////////////////////////////////
    if ( InitBit3Adaptor ( ) != Ok ) 
    {
        oConsole.ErrorLine ( (CString) "Failure Initializing "
                                     + "Bit3 Adaptor #" 
                                     + I_to_A (m_uiAdaptorNum) ) ;
		m_eStatus = Warning ;
	}



    // Register Interrupt Service Routine(s)
    ///////////////////////////////////
/*
    if ( RegisterBit3Interrupts ( ) != Ok ) 
    {
        oConsole.ErrorLine ( (CString) "Failure Registering Interrupt(s) for"
                                     + "Bit3 Adaptor #" 
                                     + I_to_A (m_uiAdaptorNum) ) ;
		m_eStatus = Error ;
	}
*/
done:

//#if ( defined ( VME_IO_BIT3_MMAP ) || defined ( VME_IO_BIT3_API ) )

    CString         osAddrSpace         = "A??" ;
    CString         osAccessRestrict    = "RW??" ;

	switch ( m_eAddrSpace )
	{
        case VmeA32 : osAddrSpace = "A32" ; break ;
        case VmeA24 : osAddrSpace = "A24" ; break ;
        case VmeA16 : osAddrSpace = "A16" ; break ;
        case Dpm    : osAddrSpace = "DPM" ; break ;
	}

	switch ( m_eAccessRestrict )
	{
	    case VmeRead		: osAccessRestrict = "Read" ; break ;
	    case VmeWrite		: osAccessRestrict = "Write" ; break ;
	    case VmeReadWrite	: osAccessRestrict = "Read/Write"  ; break ;
	}

    if ( m_eStatus == Ok ) 
        oConsole.WriteLine ( (CString) "New Connection to "
                                     + "Bit3 Adapt #" 
                                     + I_to_A (m_uiAdaptorNum) 
                                     + " for " + osAccessRestrict 
                                     + " to " + osAddrSpace
                                     + " Space; Addr Modif " 
                                     + I_to_A (m_iAmod, 2, 16, '0' ) ) ;
    else 
        oConsole.ErrorLine ( (CString) "Connection Failed to "
                                     + "Bit3 Adapt #" 
                                     + I_to_A (m_uiAdaptorNum) 
                                     + " for " + osAccessRestrict 
                                     + " to " + osAddrSpace
                                     + " Space; Addr Modif " 
                                     + I_to_A (m_iAmod, 2, 16, '0' ) ) ;

//#endif // VME_IO_BIT3_MMAP || VME_IO_BIT3_API

    return ;
}


/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
CBit3Adaptor61x::~CBit3Adaptor61x ( ) 
{

    // de-register the Interrupt Service Routines
    ///////////////////////////////////
 /*
    if ( RemoveBit3Interrupts ( ) != Ok ) 
    {
        oConsole.ErrorLine ( (CString) "Failure Removing Interrupt Service Routines for"
                                     + "Bit3 Adaptor #" 
                                     + I_to_A (m_uiAdaptorNum) ) ;
		m_eStatus = Error ;
	}
*/
    // close handle to bit3 device driver
    ///////////////////////////////////
    if ( CloseBit3Adaptor ( ) != Ok ) 
    {
        oConsole.ErrorLine ( (CString) "Failure Closing Access to "
                                     + "Bit3 Adaptor #" 
                                     + I_to_A (m_uiAdaptorNum) ) ;
		m_eStatus = Error ;
	}

}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess CBit3Adaptor61x::BuildBit3AdaptorName  ( ) 
{
    char*				psDevName = m_sDevName ;

    bt_dev_t			zLogDev ;

    //\\//

	switch ( m_eAddrSpace )
	{
	    case VmeA16 : zLogDev = BT_DEV_A16 ; break ;
	    case VmeA24 : zLogDev = BT_DEV_A24 ; break ;
	    case VmeA32 : zLogDev = BT_DEV_A32 ; break ;
	    case Dpm    : zLogDev = BT_DEV_DP ; break ;
	    default :
            oConsole.InternalErrorLine ( (CString) "BuildBit3AdaptorName/AccessType: Undefined Resource Type" ) ;
		    m_eStatus = Error ;
		    goto done ;
	}

#if ( defined ( VME_IO_BIT3_MMAP ) || defined ( VME_IO_BIT3_API ) ) 

    
    // build bit3 device name for 32 bit address access
    ///////////////////////////////////////////////////
    psDevName = bt_gen_name( m_uiAdaptorNum, zLogDev, psDevName, KiDevNameStringLength ) ;

    oConsole.WriteLine ( (CString) " AdaptorNum="
                                  + I_to_A ( m_uiAdaptorNum ) 
                                  + " AddressSpace="
                                  + I_to_A ( zLogDev ) 
                                  + " DeviceName=" 
                                  + psDevName  ) ;

    if ( psDevName == NULL )

        // Note: error will be explained after examining status returned by call to bt_open
        oConsole.ErrorLine ( (CString) "BuildBit3AdaptorName/bt_gen_name: Error, returned NULL pointer " ) ;

#else

    strcpy ( m_sDevName, "[NO IO]" ) ;

#endif // ( VME_IO_BIT3_MMAP ) || ( VME_IO_BIT3_API ) )

done:
	return m_eStatus ;
}
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess CBit3Adaptor61x::OpenBit3Adaptor  ( ) 
{
    char*				psDevName = m_sDevName ;

    bt_accessflag_t		zAccessFlags ;

    //\\//

	switch ( m_eAccessRestrict )
	{
	    case VmeRead		: zAccessFlags = BT_RD ; break ;
	    case VmeWrite		: zAccessFlags = BT_WR ; break ;
	    case VmeReadWrite	: zAccessFlags = BT_RDWR ; break ;
	    default :
            oConsole.InternalErrorLine ( (CString) "BuildBit3AdaptorName/AccessMode: Undefined Access Mode" ) ;
		    m_eStatus = Error ;
		    goto done ;
	}

#if ( defined ( VME_IO_BIT3_MMAP ) || defined ( VME_IO_BIT3_API ) ) 

    bt_error_t			zStatus ;

    /////////////////////////////////////
    // open handle to bit 3 device driver
    /////////////////////////////////////

    zStatus = bt_open   ( (bt_desc_t*) &m_pBit3Descriptor, 
                                        psDevName,
                                        zAccessFlags ) ;

	if ( zStatus != BT_SUCCESS )
	{
        oConsole.ErrorLine ( (CString) "OpenBit3Adaptor/bt_open: " 
                                      + GetBit3ErrorMessage ( zStatus ) ) ;
        oConsole.ErrorLine ( (CString) "_DeviceName=" + psDevName  
                                      + " Access="
                                      + I_to_A ( zAccessFlags ) ) ;
		m_eStatus = Error ;
        goto done ;
	}

    /////////////////////////////////////
    // Set proper swapping mode to allow byte access 
    /////////////////////////////////////

    int Xint ;

    // following 1x statement temporary for debugger access
    zStatus = bt_get_info ( (bt_desc_t) m_pBit3Descriptor, 
                                        BT_INFO_SWAP,
                       (bt_devdata_t*) &Xint ) ;


    Xint = BT_SWAP_BSBD ;
    zStatus = bt_set_info ( (bt_desc_t) m_pBit3Descriptor, 
                                        BT_INFO_SWAP,
                                        Xint ) ;

	if ( zStatus != BT_SUCCESS )
    {
        oConsole.ErrorLine ( (CString) "OpenBit3Adaptor/bt_set_info(SWAP): " 
                                      + GetBit3ErrorMessage ( zStatus ) ) ;
		m_eStatus = Error ;
    }

    /////////////////////////////////////
    // Set Address Modifier 
    /////////////////////////////////////

    // following 2x statement temporary for debugger access
    zStatus = bt_get_info ( (bt_desc_t) m_pBit3Descriptor, 
                                        BT_INFO_MMAP_AMOD,
                       (bt_devdata_t*) &m_iAmod ) ;

    zStatus = bt_get_info ( (bt_desc_t) m_pBit3Descriptor, 
                                        BT_INFO_PIO_AMOD,
                       (bt_devdata_t*) &m_iAmod ) ;

    zStatus = bt_get_info ( (bt_desc_t) m_pBit3Descriptor, 
                                        BT_INFO_DMA_AMOD,
                       (bt_devdata_t*) &m_iAmod ) ;

#endif // ( VME_IO_BIT3_MMAP ) || ( VME_IO_BIT3_API ) )

    m_iAmod = 0 ;
	switch ( m_eAddrSpace )
	{
	    case VmeA16 : 
            switch ( m_eAmodPriv )
            {
                case VmeAmodSuper :
                    switch ( m_eAmodType )
                    {   case VmeAmodProgr : /* illegal here */         break ;
                        case VmeAmodData  : m_iAmod = BT_AMOD_A16_SD ; break ;
                        case VmeAmodBlock : /* not supported here */   break ;
                    } break ;

                case VmeAmodNonPriv :
                    switch ( m_eAmodType )
                    {   case VmeAmodProgr : /* illegal here */         break ;
                        case VmeAmodData  : m_iAmod = BT_AMOD_A16_ND ; break ;
                        case VmeAmodBlock : /* not supported here */   break ;
                    } break ;
                case VmeAmodNevis :
                        /* not supported here */   break ;
            } break ;

	    case VmeA24 : 
            switch ( m_eAmodPriv )
            {
                case VmeAmodSuper :
                    switch ( m_eAmodType )
                    {   case VmeAmodProgr : m_iAmod = BT_AMOD_A24_SP ; break ;
                        case VmeAmodData  : m_iAmod = BT_AMOD_A24_SD ; break ;
                        case VmeAmodBlock : /* not supported here */   break ;
                    } break ;

                case VmeAmodNonPriv :
                    switch ( m_eAmodType )
                    {   case VmeAmodProgr : m_iAmod = BT_AMOD_A24_NP ; break ;
                        case VmeAmodData  : m_iAmod = BT_AMOD_A24_ND ; break ;
                        case VmeAmodBlock : /* not supported here */   break ;
                    } break ;

                case VmeAmodNevis :
                    m_iAmod = Ki_BT_AMOD_NEVIS ; // User defined Address Modifier for Nevis Tab Crate
                    break ;
            } break ;

        case VmeA32 : 
            switch ( m_eAmodPriv )
            {
                case VmeAmodSuper :
                    switch ( m_eAmodType )
                    {   case VmeAmodProgr : m_iAmod = BT_AMOD_A32_SP ; break ;
                        case VmeAmodData  : m_iAmod = BT_AMOD_A32_SD ; break ;
                        case VmeAmodBlock : /* not supported here */   break ;
                    } break ;

                case VmeAmodNonPriv :
                    switch ( m_eAmodType )
                    {   case VmeAmodProgr : m_iAmod = BT_AMOD_A32_NP ; break ;
                        case VmeAmodData  : m_iAmod = BT_AMOD_A32_ND ; break ;
                        case VmeAmodBlock : /* not supported here */   break ;
                    } break ;
                case VmeAmodNevis :
                        /* not supported here */   break ;
            } break ;
        case Dpm : // no address modifier here
            break ;
	}

    //Check there is a valid Address Modifier
    if ( m_iAmod == 0 ) 
    {
        oConsole.InternalErrorLine ( (CString) "Unsupported Address Modifier Requested: Using Bit3 Default." ) ;
		m_eStatus = Error ;
    }
#if ( defined ( VME_IO_BIT3_MMAP ) || defined ( VME_IO_BIT3_API ) ) 
    else 
    {    
        zStatus = bt_set_info ( (bt_desc_t) m_pBit3Descriptor, 
                                            BT_INFO_MMAP_AMOD,
                                            m_iAmod ) ;

	    if ( zStatus != BT_SUCCESS )
        {
            oConsole.ErrorLine ( (CString) "OpenBit3Adaptor/bt_set_info(MMAP_AMOD): " 
                                          + GetBit3ErrorMessage ( zStatus ) ) ;
		    m_eStatus = Error ;
        }

        zStatus = bt_set_info ( (bt_desc_t) m_pBit3Descriptor, 
                                            BT_INFO_PIO_AMOD,
                                            m_iAmod ) ;

	    if ( zStatus != BT_SUCCESS )
        {
            oConsole.ErrorLine ( (CString) "OpenBit3Adaptor/bt_set_info(PIO_AMOD): " 
                                          + GetBit3ErrorMessage ( zStatus ) ) ;
	        m_eStatus = Error ;
        }

        zStatus = bt_set_info ( (bt_desc_t) m_pBit3Descriptor, 
                                            BT_INFO_DMA_AMOD,
                                            m_iAmod ) ;

	    if ( zStatus != BT_SUCCESS )
        {
            oConsole.ErrorLine ( (CString) "OpenBit3Adaptor/bt_set_info(DMA_AMOD): " 
                                          + GetBit3ErrorMessage ( zStatus ) ) ;
		    m_eStatus = Error ;
        }

	}

    /////////////////////////////////////
    // Set Non Block Transfer Mode
    /////////////////////////////////////

    // following 1x statement temporary for debugger access
    zStatus = bt_get_info ( (bt_desc_t) m_pBit3Descriptor, 
                                        BT_INFO_BLOCK,
                       (bt_devdata_t*) &Xint ) ;

    Xint = FALSE ;
    zStatus = bt_set_info ( (bt_desc_t) m_pBit3Descriptor, 
                                        BT_INFO_BLOCK,
                                        Xint ) ;

	if ( zStatus != BT_SUCCESS )
    {
        oConsole.ErrorLine ( (CString) "OpenBit3Adaptor/bt_set_info(BLOCK): " 
                                      + GetBit3ErrorMessage ( zStatus ) ) ;
		m_eStatus = Error ;
    }

    /////////////////////////////////////
    // Set DMA to use 16-bit transfer
    /////////////////////////////////////

    // following 1x statement temporary for debugger access
    zStatus = bt_get_info ( (bt_desc_t) m_pBit3Descriptor, 
                                        BT_INFO_DATAWIDTH,
                       (bt_devdata_t*) &Xint ) ;

    Xint = BT_WIDTH_D16 ;
    zStatus = bt_set_info ( (bt_desc_t) m_pBit3Descriptor, 
                                        BT_INFO_DATAWIDTH,
                                        Xint ) ;

	if ( zStatus != BT_SUCCESS )
    {
        oConsole.ErrorLine ( (CString) "OpenBit3Adaptor/bt_set_info(BT_INFO_DATAWIDTH): " 
                                      + GetBit3ErrorMessage ( zStatus ) ) ;
		m_eStatus = Error ;
    }

    /////////////////////////////////////
    // Set DMA to NOT use Block Transfer
    /////////////////////////////////////

    // following 1x statement temporary for debugger access
    zStatus = bt_get_info ( (bt_desc_t) m_pBit3Descriptor, 
                                        BT_INFO_BLOCK,
                       (bt_devdata_t*) &Xint ) ;

    Xint = FALSE ;
    zStatus = bt_set_info ( (bt_desc_t) m_pBit3Descriptor, 
                                        BT_INFO_BLOCK,
                                        Xint ) ;

	if ( zStatus != BT_SUCCESS )
    {
        oConsole.ErrorLine ( (CString) "OpenBit3Adaptor/bt_set_info(BT_INFO_BLOCK): " 
                                      + GetBit3ErrorMessage ( zStatus ) ) ;
		m_eStatus = Error ;
    }

    /////////////////////////////////////
    // Set DMA to start when transfer larger than 20 bytes
    /////////////////////////////////////

    // following 1x statement temporary for debugger access
    zStatus = bt_get_info ( (bt_desc_t) m_pBit3Descriptor, 
                                        BT_INFO_DMA_THRESHOLD,
                       (bt_devdata_t*) &Xint ) ;

    Xint = 20 ;
    zStatus = bt_set_info ( (bt_desc_t) m_pBit3Descriptor, 
                                        BT_INFO_DMA_THRESHOLD,
                                        Xint ) ;

	if ( zStatus != BT_SUCCESS )
    {
        oConsole.ErrorLine ( (CString) "OpenBit3Adaptor/bt_set_info(BT_INFO_DMA_THRESHOLD): " 
                                      + GetBit3ErrorMessage ( zStatus ) ) ;
		m_eStatus = Error ;
    }

    /////////////////////////////////////
    // Set DMA to not wait and poll, but use interrupt 
    /////////////////////////////////////

    // following 1x statement temporary for debugger access
    zStatus = bt_get_info ( (bt_desc_t) m_pBit3Descriptor, 
                                        BT_INFO_DMA_POLL_CEILING,
                       (bt_devdata_t*) &Xint ) ;

    Xint = 20 ;
    zStatus = bt_set_info ( (bt_desc_t) m_pBit3Descriptor, 
                                        BT_INFO_DMA_POLL_CEILING,
                                        Xint ) ;

	if ( zStatus != BT_SUCCESS )
    {
        oConsole.ErrorLine ( (CString) "OpenBit3Adaptor/bt_set_info(BT_INFO_DATAWIDTH): " 
                                      + GetBit3ErrorMessage ( zStatus ) ) ;
		m_eStatus = Error ;
    }

    /////////////////////////////////////
    // Query alignment quirement
    /////////////////////////////////////

    // following 1x statement temporary for debugger access
    zStatus = bt_get_info ( (bt_desc_t) m_pBit3Descriptor, 
                                        BT_INFO_BIND_ALIGN,
                       (bt_devdata_t*) &Xint ) ;

	if ( zStatus == BT_SUCCESS )
    {
        oConsole.WriteLine ( (CString) "OpenBit3Adaptor["
                                      + I_to_A ( m_uiAdaptorNum ) 
                                      + "]/bt_get_info(BT_INFO_BIND_ALIGN): " 
                                      + I_to_A ( Xint ) ) ;
    }
    else
    {
        oConsole.ErrorLine ( (CString) "OpenBit3Adaptor/bt_get_info(BT_INFO_BIND_ALIGN): " 
                                      + GetBit3ErrorMessage ( zStatus ) ) ;
		m_eStatus = Error ;
    }

    /////////////////////////////////////
    // Query Bind Size
    /////////////////////////////////////

    // following 1x statement temporary for debugger access
    zStatus = bt_get_info ( (bt_desc_t) m_pBit3Descriptor, 
                                        BT_INFO_BIND_SIZE,
                       (bt_devdata_t*) &Xint ) ;

	if ( zStatus == BT_SUCCESS )
    {
        oConsole.WriteLine ( (CString) "OpenBit3Adaptor["
                                      + I_to_A ( m_uiAdaptorNum ) 
                                      + "]/bt_get_info(BT_INFO_BIND_SIZE): " 
                                      + I_to_A ( Xint ) ) ;
    }
    else 
    {
        oConsole.ErrorLine ( (CString) "OpenBit3Adaptor/bt_get_info(BT_INFO_BIND_SIZE): " 
                                      + GetBit3ErrorMessage ( zStatus ) ) ;
		m_eStatus = Error ;
    }

#endif // ( VME_IO_BIT3_MMAP ) || ( VME_IO_BIT3_API ) )

done:

	return m_eStatus ;
}


/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess CBit3Adaptor61x::CloseBit3Adaptor ( ) 
{

    //\\//

#if ( defined ( VME_IO_BIT3_MMAP ) || defined ( VME_IO_BIT3_API ) ) 
  
    bt_error_t  zStatus ;

      // close handle to bit3 device driver
    zStatus = bt_close ( (bt_desc_t) m_pBit3Descriptor ) ;
	if ( zStatus != BT_SUCCESS )
	{
        oConsole.ErrorLine ( (CString) "CloseBit3Adaptor/bt_close: " 
                                      + GetBit3ErrorMessage ( zStatus ) ) ;
		m_eStatus = Error ;
	}

#endif // ( VME_IO_BIT3_MMAP ) || ( VME_IO_BIT3_API ) )

	return m_eStatus ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess    CBit3Adaptor61x::ProbeForAdaptor          ( unsigned int    _uiAdaptorNum ) 
{            
	ESuccess            eReturnStatus = Ok ;

#if ( defined ( VME_IO_BIT3_MMAP ) || defined ( VME_IO_BIT3_API ) ) 

    void*               pBit3Descriptor = 0 ; // Handle to scratch Bit3 Adaptor
    char                sDevName[KiDevNameStringLength+1] ;
    char*				psDevName = sDevName ;
    bt_error_t			zStatus ;
    bt_data32_t         zAdaptorStatus ;

    //\\//

    ///////////////////////////////////////
    // If we bail out in the next few lines 
    // it will be because we couldn't talk to the driver or locate an adaptor
	eReturnStatus = Error ;  

    // build bit3 device name for 32 bit address access
    psDevName = bt_gen_name( _uiAdaptorNum, BT_DEV_A32, psDevName, KiDevNameStringLength ) ;
    if ( psDevName == NULL ) goto done ;

    // open handle to bit 3 device driver
    zStatus = bt_open   ( (bt_desc_t*) &pBit3Descriptor, psDevName, BT_RD ) ;
	if ( zStatus != BT_SUCCESS ) goto done ; // there is no adaptor

    // Clear errors and get status
    zStatus = bt_clrerr ( (bt_desc_t) pBit3Descriptor ) ;
	// if ( zStatus != BT_SUCCESS ) goto done ; // It is ok if we have not been able to clear errors
    zStatus = bt_status ( (bt_desc_t) pBit3Descriptor, &zAdaptorStatus ) ;
    if ( zStatus != BT_SUCCESS )
	{// this an error getting the status
        oConsole.InternalErrorLine ( (CString) "ProbeForAdaptor/bt_status: " 
                                      + I_to_A ( zStatus, 4, 16, '0' ) ) ;
        goto done ; 
	}
//	if ( zStatus != BT_SUCCESS ) goto done ; // this is the status of the command, not of the adaptor

    ///////////////////////////////////////
    // If we bail out in the next few lines 
    // it will be because the remote crate is disconnected or powered off
	eReturnStatus = Warning ;  

    //look at the proper bit from the status retieved above
    if ( (zAdaptorStatus & BT_INTR_POWER) != 0  ) goto done ;

    /////////////////////////////////////
    // If we got this far,
    // we have now found an adaptor, and it was connected to a remote crate with power on
	eReturnStatus = Ok ;  

/////////////////////////////////////
done:
    if ( pBit3Descriptor != 0 ) bt_close ( (bt_desc_t) pBit3Descriptor ) ;

#endif // ( VME_IO_BIT3_MMAP ) || ( VME_IO_BIT3_API ) )

    return eReturnStatus ;
}


/*-------------------------------------------------------------------------------*/
ESuccess    CBit3Adaptor61x::InitBit3Adaptor ( )
{
	EYesNo		ynAdaptorOk = Yes ;

#if ( defined ( VME_IO_BIT3_MMAP ) || defined ( VME_IO_BIT3_API ) ) 

    bt_error_t  zStatus ;

    //\\//

    //Initialize Status to assume things are ok
    m_eStatus = Ok ;

    ///////////////////////////////////
    // Initialize the Adaptor... if needed
    ///////////////////////////////////

    // only initialize the Bit3 interface if the status reported shows errors
	if ( CheckBit3Status ( ) != Ok )
    { 
        oConsole.WriteLine ( "Initializing the Bit3 Bus Adaptor" ) ;

	    zStatus = bt_init ( (bt_desc_t) m_pBit3Descriptor ) ;

        //assume that adaptor was properly initialized (then verify)
	    ynAdaptorOk = Yes ;

	    if ( zStatus != BT_SUCCESS )
	    {
            oConsole.ErrorLine ( (CString) "InitBit3Adaptor/bt_init: " 
                                          + GetBit3ErrorMessage ( zStatus ) ) ;
		    ynAdaptorOk = No ;
	    }
        else
        {
            oConsole.WriteLine ( "InitBit3Adaptor/bt_init: Success" ) ;

		    // even if bt_init claimed success, still check status
		    // as we found that bt_status may still return a lingering
            // "remote bus was reset" while bt_init didn't flag anything
		    if ( CheckBit3Status ( ) != Ok )
			    ynAdaptorOk = No ;
	    }
    }

    if ( ( ynAdaptorOk == No ) && ( Bit3Powered () == Ok ) )
	{
		// see if bt_reset can fix the problem
        oConsole.WriteLine ( (CString) "Power On in remote crate, "
                                     + "but Bit3 Status still not cleared" + "\n"
                                     + "Try Resetting the Remote Bus" ) ;

	    zStatus = bt_reset ( (bt_desc_t) m_pBit3Descriptor ) ;

		//assume that adaptor was properly reset (then verify)
		ynAdaptorOk = Yes ;

		if ( zStatus != BT_SUCCESS )
	    {
            oConsole.ErrorLine ( (CString) "InitBit3Adaptor/bt_reset: " 
                                          + GetBit3ErrorMessage ( zStatus ) ) ;
			ynAdaptorOk = No ;
	    }
        else
        {
            oConsole.WriteLine ( "InitBit3Adaptor/bt_reset: Success" ) ;

			// even if bt_reset claimed success, still check status
			// as bt_status may still return "remote bus was reset"
			if ( CheckBit3Status ( ) != Ok )
				ynAdaptorOk = No ;
        }

    }

    if ( ynAdaptorOk == No )
		m_eStatus = Error ;

#endif // ( VME_IO_BIT3_MMAP ) || ( VME_IO_BIT3_API ) )

	return m_eStatus ;
}

/*-------------------------------------------------------------------------------*/
ESuccess  CBit3Adaptor61x::CheckBit3Status ( ) 
{

    m_eStatus = Ok ;

#if ( defined ( VME_IO_BIT3_MMAP ) || defined ( VME_IO_BIT3_API ) ) 

    bt_error_t      zStatus ;
    bt_data32_t     zAdaptorStatus ;
    bt_data32_t*    pzAdaptorStatus = &zAdaptorStatus ;

    //\\//

    // First check for Adaptor Error
    ////////////////////////////////
    zStatus = bt_chkerr ( (bt_desc_t) m_pBit3Descriptor ) ;
    
    if ( zStatus != BT_SUCCESS )
	{// get the full report of the error as a multi-line string
        oConsole.ErrorLine ( (CString) "CheckBit3Status/bt_chkerr: " 
                                      + GetBit3ErrorMessage ( zStatus ) ) ;
        m_eStatus = Error ;
	}

    // Also explicitely check and unpack the Adaptor Status
    // This might be un-necessary if checkerr didn't spot a problem
    ///////////////////////////////////////////////////////
    zStatus = bt_status ( (bt_desc_t) m_pBit3Descriptor,
             		                  &zAdaptorStatus ) ;
    
    if ( zStatus != BT_SUCCESS )
	{// this an error getting the status
        oConsole.InternalErrorLine ( (CString) "CheckBit3Status/bt_status: " 
                                      + GetBit3ErrorMessage ( zStatus ) ) ;
        m_eStatus = Error ;
	}
    else
    {//examine the status word returned
/*/ newer Model 984 bit3 driver says "not used any more"
        if ( (zAdaptorStatus & BT_INTR_XMITR)     == 0 ) 
        {
            oConsole.ErrorLine ( "CheckBit3Status: Local node is NOT transmitter " ) ;
            m_eStatus = Error ;
        }
/*/

        if ( (zAdaptorStatus & BT_INTR_RRESET)    != 0 ) 
        {
            oConsole.ErrorLine ( "CheckBit3Status: Remote bus was reset " ) ;
            m_eStatus = Error ;
        }

        if ( (zAdaptorStatus & BT_INTR_POWER)     != 0  ) 
        {
            oConsole.ErrorLine ( "CheckBit3Status: Remote bus power off or cable " ) ;
            m_eStatus = Error ;
        }

/*/
        // this bit3 error code is not defined on linux nor in the newer model 984 windows driver?
        if ( (zAdaptorStatus & BT_INTR_LRC)       != 0  ) 
        {
            oConsole.ErrorLine ( "CheckBit3Status: Logitudinal Redundancy Check error " ) ;
            m_eStatus = Error ;
        }
/*/

        if ( (zAdaptorStatus & BT_INTR_TIMEOUT)   != 0  ) 
        {
            oConsole.ErrorLine ( "CheckBit3Status: Interface timeout " ) ;
            m_eStatus = Error ;
        }

        if ( (zAdaptorStatus & BT_INTR_REMBUS)    != 0  ) 
        {
            oConsole.ErrorLine ( "CheckBit3Status: Remote bus error " ) ;
            m_eStatus = Error ;
        }

        if ( (zAdaptorStatus & BT_INTR_PARITY)    != 0  ) 
        {
            oConsole.ErrorLine ( "CheckBit3Status: Interface parity error " ) ;
            m_eStatus = Error ;
        }
    }

#endif // VME_IO_BIT3_MMAP || VME_IO_BIT3_API

    return m_eStatus ;
}

/*-------------------------------------------------------------------------------*/
ESuccess  CBit3Adaptor61x::ClearBit3Status ( ) 
{
	EYesNo		ynAdaptorOk = Yes ;

#if ( defined ( VME_IO_BIT3_MMAP ) || defined ( VME_IO_BIT3_API ) ) 

    bt_error_t  zStatus ;

    //\\//

    m_eStatus = Ok ;

    //    oConsole.WriteLine ( "Clearing Bit3 Model 617 Adaptor Errors" ) ;

    zStatus = bt_clrerr ( (bt_desc_t) m_pBit3Descriptor ) ;

    //assume that adaptor error were properly cleared (then verify)
	ynAdaptorOk = Yes ;

    if ( zStatus != BT_SUCCESS )
	{// this an error getting to the adaptor
        oConsole.ErrorLine ( (CString) "ClearBit3Status/bt_clrerr: " 
                                      + GetBit3ErrorMessage ( zStatus ) ) ;
		ynAdaptorOk = No ;
	}
	else 
	{
		// even if bt_clrerr claimed success, still check status
		// as bt_status may still return "remote bus was reset"
		if ( CheckBit3Status ( ) != Ok )
			ynAdaptorOk = No ;
	}

    if ( ynAdaptorOk == No )
	{
		//try re-initializing Bit 3 Adaptor
		if ( InitBit3Adaptor () != Ok ) 
			m_eStatus = Error ;
	}

#endif // VME_IO_BIT3_MMAP || VME_IO_BIT3_API

    return m_eStatus ;
}


/*-------------------------------------------------------------------------------*/
ESuccess  CBit3Adaptor61x::Bit3Powered ( ) 
{

#if ( defined ( VME_IO_BIT3_MMAP ) || defined ( VME_IO_BIT3_API ) ) 

    bt_error_t      zStatus ;
    bt_data32_t     zAdaptorStatus ;

    //\\//

    // check and unpack the apropriate flag from the Adaptor Status 
    ///////////////////////////////////////////////////////
    zStatus = bt_status ( (bt_desc_t) m_pBit3Descriptor,
                                     &zAdaptorStatus ) ;
    
    if ( zStatus != BT_SUCCESS )
	{// this an error getting the status
        oConsole.InternalErrorLine ( (CString) "Bit3Powered/bt_status: " 
                                      + GetBit3ErrorMessage ( zStatus ) ) ;
        m_eStatus = Error ;
	}
    else 
    {
        if ( (zAdaptorStatus & BT_INTR_POWER) != 0  ) 
            m_eStatus = Error ;
    }   

#endif // VME_IO_BIT3_MMAP || VME_IO_BIT3_API

    return m_eStatus ;
}


#if ( defined ( VME_IO_BIT3_MMAP ) || defined ( VME_IO_BIT3_API ) ) 
/*-------------------------------------------------------------------------------*/
CString CBit3Adaptor61x::GetBit3ErrorMessage ( long _zStatus ) 
{
    const int   kiMaxMessageSize = 2048 ;
    char        sBit3Message[kiMaxMessageSize+1] ;
    char        sMessagePrefix[] = "" ;

    if ( bt_strerror  ( (bt_desc_t) m_pBit3Descriptor, 
                       (bt_error_t) _zStatus, 
                                    sMessagePrefix, 
                                    sBit3Message, 
                                    kiMaxMessageSize ) == NULL )
    {
        strcpy ( sBit3Message, "<bt_strerror failed to retrieve bit3 error string>" ) ;
    }

    return sBit3Message ;
}
#endif //VME_IO_BIT3_MMAP || VME_IO_BIT3_API

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
void*       CBit3Adaptor61x::MemoryMapThisVmeSegment  ( TulVmeAddress   _ulBaseAddress,
                                                        TulVmeAddress   _ulByteLength,
                                                        EYesNo          _ynByteSwapping ) 
{            

    void*  pMappedAddress = NULL ;

    //\\//

#if defined ( VME_IO_BIT3_MMAP ) 

    bt_devaddr_t    zBaseAddress ; 
    size_t          zSize        ; 
    bt_swap_t       zSwapping    ; 
    bt_accessflag_t zAccessFlags ;
    bt_error_t      zStatus      ;

    //\\//

    zBaseAddress = _ulBaseAddress ;

    zSize        = _ulByteLength ; 

    zAccessFlags = BT_RDWR ; // read-write access

    if ( _ynByteSwapping == Yes ) 
        // always byte swap on Byte Data, and additionally Byte swap on non-byte data if desired
    {        
        zSwapping = BT_SWAP_BSBD_BSNBD ;  // not used at them moment
    }
    else
    {
        zSwapping = BT_SWAP_BSBD ; // preserve data value for word and longword transfers 
                                   // and allows proper behavior on byte access
    }


    zStatus = bt_mmap   ( (bt_desc_t) m_pBit3Descriptor, 
                                     &pMappedAddress , 
                                      zBaseAddress, 
                                      zSize, 
                                      zAccessFlags, 
                                      zSwapping ) ;
	
    
    if ( zStatus != BT_SUCCESS )
	{
        oConsole.ErrorLine ( (CString) "MemoryMapThisVmeSegment/bt_mmap: " 
                                      + GetBit3ErrorMessage ( zStatus ) ) ;
        m_eStatus = Error ;
        pMappedAddress = NULL ;
	}

#elif defined ( VME_IO_BIT3_API ) 

#elif defined ( VME_IO_FAKE ) 

    //there is no bit3 interface, and no remote bus
    //we can still allocate bulk virtual space of the requested size
    pMappedAddress = new char[_ulByteLength] ;

    //Initialize this block of memory
    for ( unsigned int i = 0 ; i < _ulByteLength ; i++ )
        ((char*) pMappedAddress) [i] = 0 ;

#elif defined ( VME_IO_NONE ) 

#endif 

    return pMappedAddress ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess    CBit3Adaptor61x::UnMapThisVmeSegment  ( void*           _pMappedAddress,
                                                    TulVmeAddress   _ulByteLength ) 
{

#if defined ( VME_IO_BIT3_MMAP ) 

    bt_error_t  zStatus ;

    //\\//

    zStatus = bt_unmmap   ( (bt_desc_t) m_pBit3Descriptor, 
                                        _pMappedAddress,
                                        _ulByteLength ) ;
	
    if ( zStatus != BT_SUCCESS )
	{
        oConsole.ErrorLine ( (CString) "UnMapThisVmeSegment/bt_unmmap: " 
                                      + GetBit3ErrorMessage ( zStatus ) ) ;
        m_eStatus = Error ;
	}

#endif // VME_IO_BIT3_MMAP

#if defined ( VME_IO_FAKE ) 

    // this is not just a pointer to void 
    // and we need to delete an underlying array
    delete[] (char*)_pMappedAddress ;

#endif // VME_IO_FAKE

    return m_eStatus ;

}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess    CBit3Adaptor61x::CheckBufferAligned   ( void*           _pBuffer,
                                                    unsigned long*  _pAlignOff ) 
{

    bt_error_t      zStatus ;
    unsigned long   ulAlignSize ;
    unsigned long   ulAlignOff ;

    //\\//

    m_eStatus = Ok ;

    zStatus = bt_get_info ( (bt_desc_t) m_pBit3Descriptor, 
                                        BT_INFO_BIND_ALIGN,
                       (bt_devdata_t*) &ulAlignSize ) ;

    ulAlignOff = BT_ALIGN_PTR(_pBuffer, ulAlignSize) ;

    if ( _pAlignOff != NULL ) *_pAlignOff = ulAlignOff ;

    if ( ulAlignOff != 0 ) 
	{
        m_eStatus = Error ;
	}

    return m_eStatus ;

}

/*-------------------------------------------------------------------------------*/
ESuccess    CBit3Adaptor61x::ReadVmeData  ( TulVmeAddress          _ulVmeAddress, 
                                            void* const            _kpvReadData,
                                            unsigned int const     _kuiNumBytes ) 
{
    m_eStatus = Ok ;

#if ( defined ( VME_IO_BIT3_MMAP ) || defined ( VME_IO_BIT3_API ) ) 

    bt_error_t  zStatus ;

    size_t      zNumBytesRead ;

    //\\//
    
    zStatus = bt_read ( (bt_desc_t) m_pBit3Descriptor,
        			                _kpvReadData,
		                            _ulVmeAddress,
                		            _kuiNumBytes,
                                   &zNumBytesRead) ;

    if ( zStatus != BT_SUCCESS )
    {
        oConsole.ErrorLine ( (CString) "Failure on Bit3 Adaptor #" 
                                      + I_to_A (m_uiAdaptorNum) 
                                      + " ReadVmeReg/bt_read: " 
                                      + GetBit3ErrorMessage ( zStatus ) ) ;
        m_eStatus = Error ;

        if ( InitBit3Adaptor ( ) != Ok ) 
        {
            oConsole.ErrorLine ( (CString) "Failure Initializing "
                                         + "Bit3 Adaptor #" 
                                         + I_to_A (m_uiAdaptorNum) ) ;
		    m_eStatus = Error ;
	    }
    }

#endif //VME_IO_BIT3_MMAP || VME_IO_BIT3_API

    return m_eStatus ;

}

/*-------------------------------------------------------------------------------*/
ESuccess    CBit3Adaptor61x::WriteVmeData ( TulVmeAddress          _ulVmeAddress, 
                                            void* const            _kpvWriteData,
                                            unsigned int const     _kuiNumBytes ) 
{
    m_eStatus = Ok ;

#if ( defined ( VME_IO_BIT3_MMAP ) || defined ( VME_IO_BIT3_API ) ) 

    bt_error_t  zStatus ;

    size_t      zNumBytesWritten ;

    //\\//

    zStatus = bt_write( (bt_desc_t) m_pBit3Descriptor,
        			                _kpvWriteData,
		                            _ulVmeAddress,
                		            _kuiNumBytes,
                	               &zNumBytesWritten) ;

    if ( zStatus != BT_SUCCESS )
	{
        oConsole.ErrorLine ( (CString) "WriteVmeReg/bt_write: " 
                                      + GetBit3ErrorMessage ( zStatus ) ) ;
        m_eStatus = Error ;

        if ( InitBit3Adaptor ( ) != Ok ) 
        {
            oConsole.ErrorLine ( (CString) "Failure Initializing "
                                         + "Bit3 Adaptor #" 
                                         + I_to_A (m_uiAdaptorNum) ) ;
		    m_eStatus = Error ;
	    }
	}
    
#endif //VME_IO_BIT3_MMAP || VME_IO_BIT3_API

    return m_eStatus ;

}


/*-------------------------------------------------------------------------------*/
ESuccess    CBit3Adaptor61x::RegisterBit3Interrupts ( )
{
    m_eStatus = Ok ;

#if ( defined ( VME_IO_BIT3_MMAP ) || defined ( VME_IO_BIT3_API ) ) 

    bt_error_t  zStatus ;

    //\\//

    //forward declaration of the ISRs
    bt_icbr_t   Bit3ErrorInterruptRoutine;
    bt_icbr_t   Bit3VmeInterruptRoutine;
    bt_icbr_t   Bit3PrgInterruptRoutine;

    ///////////////////////////////////
    // Register Interrupt Service Routine(s)
    ///////////////////////////////////

    //Registering a Bit3 Interrupt Service Routine for VME Errors

	zStatus = bt_icbr_install ( (bt_desc_t) m_pBit3Descriptor,
                                            BT_IRQ_ERROR,
                                            Bit3ErrorInterruptRoutine,
                                     (void*)&m_xIcbrParameters,
                                            BT_VECTOR_ALL ) ;

	if ( zStatus != BT_SUCCESS )
    {
        oConsole.ErrorLine ( (CString) "RegisterBit3Interrupts/BT_IRQ_ERROR/bt_icbr_install: " 
                                      + GetBit3ErrorMessage ( zStatus ) ) ;
		m_eStatus = Error ;
    }


    //Registering a Bit3 Interrupt Service Routine for VME Interrupts

	zStatus = bt_icbr_install ( (bt_desc_t) m_pBit3Descriptor,
                                            BT_IRQ_IACK,
                                            Bit3VmeInterruptRoutine,
                                     (void*)&m_xIcbrParameters,
                                            BT_VECTOR_ALL ) ;

	if ( zStatus != BT_SUCCESS )
    {
        oConsole.ErrorLine ( (CString) "RegisterBit3Interrupts/BT_IRQ_IACK/bt_icbr_install: " 
                                      + GetBit3ErrorMessage ( zStatus ) ) ;
		m_eStatus = Error ;
    }



    //Registering a Bit3 Interrupt Service Routine for PRG Interrupt 

	zStatus = bt_icbr_install ( (bt_desc_t) m_pBit3Descriptor,
                                            BT_IRQ_PRG,
                                            Bit3VmeInterruptRoutine,
                                     (void*)&m_xIcbrParameters,
                                            BT_VECTOR_ALL ) ;

	if ( zStatus != BT_SUCCESS )
    {
        oConsole.ErrorLine ( (CString) "RegisterBit3Interrupts/BT_IRQ_PRG/bt_icbr_install: " 
                                      + GetBit3ErrorMessage ( zStatus ) ) ;
		m_eStatus = Error ;
    }


#endif // ( VME_IO_BIT3_MMAP ) || ( VME_IO_BIT3_API ) )

	return m_eStatus ;
}

/*-------------------------------------------------------------------------------*/
ESuccess    CBit3Adaptor61x::RemoveBit3Interrupts ( )
{
    m_eStatus = Ok ;

#if ( defined ( VME_IO_BIT3_MMAP ) || defined ( VME_IO_BIT3_API ) ) 

    bt_error_t  zStatus ;

    //\\//

    //forward declaration of the ISRs
    bt_icbr_t   Bit3ErrorInterruptRoutine;
    bt_icbr_t   Bit3VmeInterruptRoutine;
    bt_icbr_t   Bit3PrgInterruptRoutine;

    ///////////////////////////////////
    // Remove Interrupt Service Routine(s)
    ///////////////////////////////////

	zStatus = bt_icbr_remove  ( (bt_desc_t) m_pBit3Descriptor,
                                            BT_IRQ_ERROR,
                                            Bit3ErrorInterruptRoutine ) ;

	if ( zStatus != BT_SUCCESS )
    {
        oConsole.ErrorLine ( (CString) "RemoveBit3Interrupts/BT_IRQ_ERROR/bt_icbr_install: " 
                                      + GetBit3ErrorMessage ( zStatus ) ) ;
		m_eStatus = Error ;
    }


	zStatus = bt_icbr_remove  ( (bt_desc_t) m_pBit3Descriptor,
                                            BT_IRQ_ERROR,
                                            Bit3VmeInterruptRoutine ) ;

	if ( zStatus != BT_SUCCESS )
    {
        oConsole.ErrorLine ( (CString) "RemoveBit3Interrupts/BT_IRQ_ERROR/bt_icbr_install: " 
                                      + GetBit3ErrorMessage ( zStatus ) ) ;
		m_eStatus = Error ;
    }


	zStatus = bt_icbr_remove  ( (bt_desc_t) m_pBit3Descriptor,
                                            BT_IRQ_PRG,
                                            Bit3VmeInterruptRoutine ) ;

	if ( zStatus != BT_SUCCESS )
    {
        oConsole.ErrorLine ( (CString) "RemoveBit3Interrupts/BT_IRQ_PRG/bt_icbr_install: " 
                                      + GetBit3ErrorMessage ( zStatus ) ) ;
		m_eStatus = Error ;
    }


#endif // ( VME_IO_BIT3_MMAP ) || ( VME_IO_BIT3_API ) )

	return m_eStatus ;
}

/*-------------------------------------------------------------------------------*/
#if ( defined ( VME_IO_BIT3_MMAP ) || defined ( VME_IO_BIT3_API ) ) 
void Bit3ErrorInterruptRoutine    ( void*       param_p, 
                                    bt_irq_t    irq_type,
                                    bt_data32_t vector )
{
	XInterruptParameters*   pxIcbrParameters = (XInterruptParameters*) param_p ;

    //flag right away that there was an Error Interrupt
    pxIcbrParameters->eVmeStatus = Error ;
      
    oConsole.InternalErrorLine ( (CString) "Entering Bit3 Error Interrupt Call Back Routine" ) ;
/*
    if ( irq_type == BT_IRQ_OVERFLOW )
        oConsole.InternalErrorLine ( (CString) "Error Interrupt Overflow Occured" ) ;

    if ( pxIcbrParameters->pBit3Adaptor->CheckBit3Status () != Ok ) 
         pxIcbrParameters->pBit3Adaptor->ClearBit3Status () ;
 */       
    oConsole.InternalErrorLine ( (CString) "Exiting Bit3 Error Interrupt Call Back Routine" ) ;
    
}
#endif //VME_IO_BIT3_MMAP || VME_IO_BIT3_API



/*-------------------------------------------------------------------------------*/
#if ( defined ( VME_IO_BIT3_MMAP ) || defined ( VME_IO_BIT3_API ) ) 
void Bit3VmeInterruptRoutine      ( void*       param_p, 
                                    bt_irq_t    irq_type,
                                    bt_data32_t vector )
{
    oConsole.InternalErrorLine ( (CString) "Entering Bit3 VME Interrupt Call Back Routine" ) ;

    if ( irq_type == BT_IRQ_OVERFLOW )
        oConsole.InternalErrorLine ( (CString) "VME Interrupt Overflow Occured" ) ;

    oConsole.InternalErrorLine ( (CString) "VME Interrupt Vector was " 
                                         + I_to_A ( vector ) ) ;
        
    oConsole.InternalErrorLine ( (CString) "Exiting Bit3 VME Interrupt Call Back Routine" ) ;
    
}
#endif //VME_IO_BIT3_MMAP || VME_IO_BIT3_API



#if ( defined ( VME_IO_BIT3_MMAP ) || defined ( VME_IO_BIT3_API ) ) 
/*-------------------------------------------------------------------------------*/
void Bit3PrgInterruptRoutine      ( void*       param_p, 
                                    bt_irq_t    irq_type,
                                    bt_data32_t vector )
{
    oConsole.InternalErrorLine ( (CString) "Entering Bit3 PRG Interrupt Call Back Routine" ) ;

    if ( irq_type == BT_IRQ_OVERFLOW )
        oConsole.InternalErrorLine ( (CString) "VME Interrupt Overflow Occured" ) ;

    oConsole.InternalErrorLine ( (CString) "VME Interrupt Vector was " 
                                         + I_to_A ( vector ) ) ;
        
    oConsole.InternalErrorLine ( (CString) "Exiting Bit3 PRG Interrupt Call Back Routine" ) ;
    
}
#endif //VME_IO_BIT3_MMAP || VME_IO_BIT3_API

