/*----------------------------------------------------------------------------------*/
// Modified 19-Mar-98 : Add Keywords Register_Address and Write_Value
// Modified 9-May-98: Ported to TRICS_II (using CConsoleAccess Object)
// Modified 1-May-99: Add CBus IO keywords
/*----------------------------------------------------------------------------------*/

#include "L1Cal_IIb_TCC/StdAfx.h"

#include "L1Cal_IIb_TCC/GlobalConstants.h"
#include "L1Cal_IIb_TCC/CustomDataTypes.h"

#include                            "L1Cal_IIb_TCC/HandleCommandFiles.h"

#include "L1Cal_IIb_TCC/UtilsStrings.h"
#include "L1Cal_IIb_TCC/UtilsFiles.h"

// static means created only once throughout life of program (to avoid wasting time)
// *AND* internal linkage (to avoid clash between modules)
#include "L1Cal_IIb_TCC/HandleConsole.h"
static      CConsoleAccess  oConsole ;

//Simple Macro to handle checking for one simple keyword below
/////////////////////////////////////////////////////////////////
#define ELSE_IF_SIMPLE_KEYWORD( KEYWORD, KEYVAR )               \
    else if ( stricmp( acKeyword, KEYWORD )  ==   0  )          \
	{                                                           \
		_pxParse->Values.ul##KEYVAR = ulValue ;                 \
		/*Look if there is an action defined for this keyword*/ \
        if ( _pxParse->Actions.pf##KEYVAR != NULL )             \
            eSuccess = _pxParse->Actions.pf##KEYVAR ( _pxParse, \
                                                      _pxCustom,\
                                                      _pThis ) ;\
    }                                                           \
/////////////////////////////////////////////////////////////////
#define ELSE_IF_KEYWORD_ACTION_VALUE( KEYWORD, ACTION, VALUE )  \
    else if ( stricmp( acKeyword, KEYWORD )  ==   0  )          \
	{                                                           \
		_pxParse->Values.ul##VALUE = ulValue ;                  \
		/*Look if there is an action defined for this keyword*/ \
        if ( _pxParse->Actions.pf##ACTION != NULL )             \
            eSuccess = _pxParse->Actions.pf##ACTION ( _pxParse, \
                                                      _pxCustom,\
                                                      _pThis ) ;\
    }                                                           \
/////////////////////////////////////////////////////////////////
#define ELSE_IF_FLOAT_KEYWORD( KEYWORD, KEYVAR )                \
    else if ( stricmp( acKeyword, KEYWORD )  ==   0  )          \
	{                                                           \
		_pxParse->Values.fd##KEYVAR = fdValue ;                 \
		/*Look if there is an action defined for this keyword*/ \
        if ( _pxParse->Actions.pf##KEYVAR != NULL )             \
            eSuccess = _pxParse->Actions.pf##KEYVAR ( _pxParse, \
                                                      _pxCustom,\
                                                      _pThis ) ;\
    }                                                           \
/////////////////////////////////////////////////////////////////
#define ELSE_IF_KEYWORD_WITH_ALIAS( KEYWORD1, KEYWORD2, KEYVAR )\
    else if ( ( stricmp( acKeyword, KEYWORD1 )  ==   0  )       \
           || ( stricmp( acKeyword, KEYWORD2 )  ==   0  ) )     \
	{                                                           \
		_pxParse->Values.ul##KEYVAR = ulValue ;                 \
		/*Look if there is an action defined for this keyword*/ \
        if ( _pxParse->Actions.pf##KEYVAR != NULL )             \
            eSuccess = _pxParse->Actions.pf##KEYVAR ( _pxParse, \
                                                      _pxCustom,\
                                                      _pThis ) ;\
    }                                                           \
/////////////////////////////////////////////////////////////////
#define ELSE_IF_KEYWORD_TO_GENERIC_STRING( KEYWORD, KEYCALL )   \
    else if ( stricmp( acKeyword, KEYWORD )  ==   0  )          \
	{                                                           \
		_pxParse->Values.osGeneric_String_Parameter = acValue ; \
		/*Look if there is an action defined for this keyword*/ \
        if ( _pxParse->Actions.pf##KEYCALL != NULL )            \
            eSuccess = _pxParse->Actions.pf##KEYCALL( _pxParse, \
                                                      _pxCustom,\
                                                      _pThis ) ;\
    }                                                           \
/////////////////////////////////////////////////////////////////
#define ELSE_IF_KEYWORD_TO_STR_WITH_ALIAS( KW1, KW2, KEYCALL )  \
    else if ( ( stricmp( acKeyword, KW1 )  ==   0  )            \
           || ( stricmp( acKeyword, KW2 )  ==   0  ) )          \
	{                                                           \
		_pxParse->Values.osGeneric_String_Parameter = acValue ; \
		/*Look if there is an action defined for this keyword*/ \
        if ( _pxParse->Actions.pf##KEYCALL != NULL )            \
            eSuccess = _pxParse->Actions.pf##KEYCALL( _pxParse, \
                                                      _pxCustom,\
                                                      _pThis ) ;\
    }                                                           \
/////////////////////////////////////////////////////////////////

/*----------------------------------------------------------------------------------*/
// Open a command file on disk and execute actions passed as arguments 
/*----------------------------------------------------------------------------------*/
ESuccess Read_Command_File ( Parse_Structure*    _pxParse, 
                             void*               _pxCustom,
                             void*               _pThis )
{

	FILE            *CommandFile ;
    char            acFileName[_MAX_FNAME] ;
	char            acMode_Read[] = "r", acMode_Text[] = "t" ;
	char            acMode_Open[5] ;
    unsigned int    iCharNum ;
    char            sQuote[2] = { '"', '\0' } ; // null terminated string of one quote chararcter
    char            sDollar[2] = "$" ; //null terminated string of one dollar character 
    char            sEqual[2] = "=" ; //null terminated string of one dollar character 
    unsigned int    iCharNumQuote ;
	char            acLineBuf[KiMaxCommandLineLength+1], // 132 length max. (+1 for null termination)
                    acKeyword[KiMaxCommandLineLength+1], // acKeyword may receive a long comment
                    acValue[KiMaxCommandLineLength+1]  ; // acValue may also recieve include file name, thus made long enough
	unsigned long   ulValue ;
	double          fdValue ;
    int             iTempLength ;
    int             iSuccess ;

    ESuccess        eSuccess ;
	ESuccess        eReturnSuccess ;

    CSymbolElement* poSymbol ;

    //\\//

	eReturnSuccess = Ok ;

    // locate the coomand file 
    // using any embedded environment variable 
    eSuccess = LocateFile ( _pxParse->pacFile_Name, acFileName ) ;
    eSuccess = Ok ;

    if ( eSuccess == Error )
    {
        eReturnSuccess = Error ;
		goto Error_Open ;
    }

    if ( strcmp ( acFileName, "" ) == 0 )
    {
        oConsole.ErrorLine ( (CString) "% Could not locate file <" 
                                      + _pxParse->pacFile_Name + ">" ) ;
        eReturnSuccess = Error ;
		goto Error_Open ;
    }

    // open the file (and check for success)
	//set Read only mode, and text type flags
	sprintf( acMode_Open, "%s%s", acMode_Read, acMode_Text ) ;
	CommandFile = fopen ( acFileName, acMode_Open ) ;
	if (CommandFile == NULL) 
	{
        oConsole.ErrorLine ( (CString) "% Could not open file <" 
                                      + acFileName + ">" ) ;
        eReturnSuccess = Error ;
		goto Error_Open ;
	}
	else
	{
        oConsole.WriteLine ( (CString) "% File opened <" 
                                      + acFileName + ">" ) ;
	}

    // look if there is an action defined for the beginning off the job 
    if ( _pxParse->Actions.pfFile_Opened != NULL )
    { 
        eSuccess = _pxParse->Actions.pfFile_Opened ( _pxParse, _pxCustom, _pThis ) ;
        if ( eSuccess != Ok ) 	
        {
            eReturnSuccess = Error ;
 		    goto Done_Read ;
      }
    }
	

    
    
    // read the file 
	// but it MAY contain comment lines starting with "! "
	// the content is made of a succession of keywords and values

    // Read one line at a time
    _pxParse->iLineNum = 0 ;
Next_Line:
	_pxParse->iLineNum++ ;

	
    // read the whole line 
    if ( fgets( acLineBuf, KiMaxCommandLineLength+1, CommandFile ) == NULL )
    {
	    // check for read problem
	    if ( ferror ( CommandFile ) )  
	    {
            oConsole.ErrorLine ( (CString) "_ IO Error Reading Line " 
                                          + I_to_A ( _pxParse->iLineNum ) ) ;
            eReturnSuccess = Error ;
        }	

	    else // if it's not end-of-file, something's seriously wrong
	    if ( feof ( CommandFile ) == 0 )	
        {
            oConsole.InternalErrorLine ( (CString) "_ Unknown IO Error Reading Line " 
                                          + I_to_A ( _pxParse->iLineNum ) ) ;
            eReturnSuccess = Error ;
        }
        goto Done_Read ;	    
    }

	// make sure we didn't exceed the length of the line buffer
	if ( strlen( acLineBuf ) >= KiMaxCommandLineLength ) 
	{
        oConsole.ErrorLine ( (CString) "_ Line too long at Line #" 
                                      + I_to_A ( _pxParse->iLineNum ) ) ;
        eReturnSuccess = Error ;
		goto Done_Read ;
	}

	
    // Parse one keyword at a time
    iCharNum = 0 ;
Next_Keyword:

	// read (next) keyword from Buffer
	iSuccess = sscanf ( &acLineBuf[iCharNum], "%s", acKeyword ) ;

	// check to see if there were no (more) words on the line of if we encountered a comment flag
	if ( ( iSuccess == 0 ) 
      || ( iSuccess == EOF ) // rem: this is EOF for LineBuf only 
	  || ( acKeyword[0] == '!' ) )
	{// then toss the rest of the line
		goto Next_Line ;
	} 

	// adjust the current reading position on the line at the end of the keyword just read
	iCharNum = iCharNum + strcspn( &acLineBuf[iCharNum], acKeyword ) + strlen(acKeyword) ;

	
    // read value assosicated with keyword
	iSuccess = sscanf ( &acLineBuf[iCharNum], "%s", acValue ) ;
	
	//check that a value was found
	if ( ( iSuccess == 0 ) 
	  || ( iSuccess == EOF ) 
	  || ( acValue[0] == '!' )  )
	{// the value wasn't on the same line
        oConsole.ErrorLine ( (CString) "_ Unexpected end of line after <"
                                      + acKeyword
                                      + "> at Line #"
                                      + I_to_A ( _pxParse->iLineNum ) ) ;
        eReturnSuccess = Error ;
		goto Done_Read ;
	} 

    // update the current reading position on the line at the beginning of the value just read (i.e. skip spaces)
    // Note: if it is a numerical value or a symbol, acValue has the whole value; 
    // but if it is a string with embedded spaces, we only have the first word.
	iCharNum += strcspn( &acLineBuf[iCharNum], acValue ) ;


    if ( acValue[0] == '$' ) 
    {
        poSymbol = _pxParse->oSymbolSet.LocateSymbol ( acValue ) ;
        if ( poSymbol == NULL )
        {//symbol hasn't been defined
            oConsole.ErrorLine ( (CString) "_ Undeclared Symbol <"
                                          + acValue
                                          + "> at Line #"
                                          + I_to_A ( _pxParse->iLineNum ) ) ;
            eReturnSuccess = Error ;
		    goto Done_Read ;
        }
        
        //retrieve its value
        ulValue = poSymbol->GetValue ( ) ;

        // update the current reading position on the line at the end of the value just read
	    iCharNum += strlen(acValue) ;
        
    }

    else if ( acValue[0] == '"' ) 
    {// this is a quoted string, grab it and find the end quote

        // grab the rest of the line, starting AFTER the begin-quote
        iCharNum += 1 ;
	    strcpy ( acValue, &acLineBuf[iCharNum] ) ;
	    
	    //check that a string was found
	    if ( strlen (acValue) == 0 )
	    {// there were no more characters after the opening quote
            oConsole.ErrorLine ( (CString) "_ Missing string after quote for <"
                                          + acKeyword
                                          + "> at Line #"
                                          + I_to_A ( _pxParse->iLineNum ) ) ;
            eReturnSuccess = Error ;
		    goto Done_Read ;
	    } 

        // now find end quote
        iCharNumQuote = strcspn( acValue, sQuote ) ;
        if ( iCharNumQuote == strlen (acValue) ) 
        {// no end-quote found on the line
            oConsole.ErrorLine ( (CString) "_ Missing end quote after string for <"
                                          + acKeyword
                                          + "> at Line #"
                                          + I_to_A ( _pxParse->iLineNum ) ) ;
            eReturnSuccess = Error ;
		    goto Done_Read ;

        }

        // null-terminate the string value at the end of the string found
        acValue[iCharNumQuote] = 0 ;

 	    // reset integer value
	    ulValue = 0 ; /*** maybe a string shouldn't be parsed as 0 when a value is expected ***/

        // update the current reading position on the line at the end of the string just read
	    iCharNum = iCharNum + strlen(acValue) + 1 ;

    }
    
    else
    {// this must simply be a value

        // see if this is a floating point number
        if ( ( strcspn ( acValue, ".eE" ) != strlen ( acValue ) ) // is there a decimal point or exponent
          && ( strcspn ( acValue, "xX" ) == strlen ( acValue ) ) ) // the 'e' was not from an hexadecimal number
        {//This is a floating point format
 	        // convert string to floating point
	        fdValue = atof ( acValue ) ;
 	        // convert string to integer
	        ulValue = (unsigned long) fdValue ;
        }
        else
        {//this is an integer format
 	        // convert string to integer
	        ulValue = A_to_uI ( acValue ) ;
 	        // convert string to floating point
	        fdValue = (double) ulValue ;
        }

        // update the current reading position on the line at the end of the value just read
	    iCharNum += strlen(acValue) ;

    }

        
    // now parse the keyword and take appropriate action
    eSuccess = Ok ; // initialized to Ok until one action fails

    if ( acKeyword[0] == '$' )
    {//this is the definition of a new symbol name or a re-definition.
    
        //check for symbol definition syntax with attached equal sign
        iTempLength = strlen(acKeyword) ;
        if ( acKeyword[iTempLength -1] != '=' )
        {
            oConsole.ErrorLine ( (CString) "_ Missing '=' for Symbol definition <"
                                          + acKeyword
                                          + "> at Line #"
                                          + I_to_A ( _pxParse->iLineNum ) ) ;
            eReturnSuccess = Error ;
		    goto Done_Read ;
        }

        // drop the equal sign by null-terminating the symbol name string 
        acKeyword[iTempLength -1] = 0 ;

        // is that a new symbol, or just a new value
        poSymbol = _pxParse->oSymbolSet.LocateSymbol ( acKeyword ) ;
        if ( poSymbol == NULL )
        {//need to create a new symbol
            _pxParse->oSymbolSet.AddSymbol ( acKeyword, ulValue ) ;
        }
        else
        {//this symbol already exists, we are changing its value
            _pxParse->oSymbolSet.ModifySymbol ( acKeyword, ulValue ) ;
        }
/*
        oConsole.WriteLine ( (CString) "_ Symbol Defined: <" + acKeyword + "> = "
                                      + I_to_A ( ulValue ) + ", at Line #"
                                      + I_to_A ( _pxParse->iLineNum ) ) ;
*/
    }

    else if ( stricmp( acKeyword, "Include_File:" )  ==   0  ) 
	{ // indlude files don't get a value but a file name, and are treated differently than rest.
        oConsole.WriteLine ( (CString) "_ Include_File: <" + acValue + ">, at Line #"
                                      + I_to_A ( _pxParse->iLineNum ) ) ;
        // do recursive call for include file. 
        // using the same parsing structure 
        // so that modifications in the called files will remain upon return
        char*   pacRememberCurrentFile = (char*) _pxParse->pacFile_Name ;
        CString	osCommand_File_Name ( acValue ) ;
        _pxParse->pacFile_Name = LPCTSTR(osCommand_File_Name) ; // transfer name of the new file to open 
        eSuccess = Read_Command_File ( _pxParse, _pxCustom, _pThis ) ; // recursive call
        _pxParse->pacFile_Name = pacRememberCurrentFile ;// restore file name (while it is actually not used again)
	}

    else if ( stricmp( acKeyword, "Call_File:" )  ==   0  ) 
	{ // Called files don't get a value but a file name, and are treated differently than rest.
        oConsole.WriteLine ( (CString) "_ Call_File: <" + acValue + ">, at Line #"
                                      + I_to_A ( _pxParse->iLineNum ) ) ;
        // do recursive call for include file. 
        // declare a temporary parsing structure (xRecursive_Parse)
        // with same set of keyword actions, but keyword values are reset.
        Parse_Structure xRecursive_Parse ;              // temporary structure for recursive call
        xRecursive_Parse.pacFile_Name = acValue ;       // transfer (pointer to) name of the new file to open 
        xRecursive_Parse.Values  = _pxParse->Values ;   // copy list of Values already assigned to Keywords
        xRecursive_Parse.Actions = _pxParse->Actions ;  // copy the list of actions
        xRecursive_Parse.oSymbolSet = _pxParse->oSymbolSet ;                   // copy list of actions, Values already assigned, Symbols already defined
        eSuccess = Read_Command_File ( &xRecursive_Parse, _pxCustom, _pThis ) ; // recursive call
	}

    else if ( stricmp( acKeyword, "Info_Tag:" )  ==   0  ) 
	{ // Print the specified string
        oConsole.WriteLine ( (CString) " Info_Tag: <" + acValue + ">, at Line #"
                                      + I_to_A ( _pxParse->iLineNum ) ) ;
	}

    else if ( stricmp( acKeyword, "Warning_Tag:" )  ==   0  ) 
	{ // Print the specified string
        oConsole.WarningLine ( (CString) " Warning_Tag: <" + acValue + ">, at Line #"
                                      + I_to_A ( _pxParse->iLineNum ) ) ;
	}

    else if ( stricmp( acKeyword, "Error_Tag:" )  ==   0  ) 
	{ // Print the specified string
        oConsole.ErrorLine ( (CString) " Error_Tag: <" + acValue + ">, at Line #"
                                      + I_to_A ( _pxParse->iLineNum ) ) ;
	}

	else if ( stricmp( acKeyword, "MilliSecond_Sleep:" )  ==   0  ) 
	{
        Sleep ( ulValue ) ;
    } 

    ELSE_IF_SIMPLE_KEYWORD      ( "Crate:",             Crate ) 
    ELSE_IF_SIMPLE_KEYWORD      ( "T962_Wire:",         T962_Wire ) 
    ELSE_IF_SIMPLE_KEYWORD      ( "T962_Maestro_Slot:", T962_Maestro_Slot ) 
    ELSE_IF_SIMPLE_KEYWORD      ( "Vertical_Master:",   Vertical_Master ) 
    ELSE_IF_SIMPLE_KEYWORD      ( "Vertical_Slave:",    Vertical_Slave ) 
    ELSE_IF_KEYWORD_WITH_ALIAS  ( "Slot:",             "Card_Slot:",            Card_Slot ) 
    ELSE_IF_KEYWORD_WITH_ALIAS  ( "To_Slot:",          "To_Card_Slot:",         To_Card_Slot ) 
    ELSE_IF_KEYWORD_WITH_ALIAS  ( "Chip:",             "Chip_Address:",         Chip_Address ) 
    ELSE_IF_KEYWORD_WITH_ALIAS  ( "To_Chip:",          "To_Chip_Address:",      To_Chip_Address ) 
    ELSE_IF_SIMPLE_KEYWORD      ( "Chip_Mask:",         Chip_Mask ) 
    ELSE_IF_KEYWORD_WITH_ALIAS  ( "Register:",         "Register_Address:",     Register_Address ) 
    ELSE_IF_KEYWORD_WITH_ALIAS  ( "To_Register:",      "To_Register_Address:",  To_Register_Address ) 
    ELSE_IF_SIMPLE_KEYWORD      ( "Register_Start:",    Register_Start ) 
    ELSE_IF_SIMPLE_KEYWORD      ( "Register_Count:",    Register_Count ) 
    ELSE_IF_KEYWORD_WITH_ALIAS  ( "Read_Register:",    "Read_Register_Address:",Read_Register_Address ) 
    ELSE_IF_SIMPLE_KEYWORD      ( "Write_Value:",       Write_Value ) 
    ELSE_IF_SIMPLE_KEYWORD      ( "Write_Verify:",      Write_Verify ) 

    ELSE_IF_SIMPLE_KEYWORD      ( "Source_Mask:",       Source_Mask ) 
    ELSE_IF_SIMPLE_KEYWORD      ( "Receiver_Mask:",     Receiver_Mask ) 
	
    ELSE_IF_KEYWORD_WITH_ALIAS  ( "Module:",           "TabGab_Module:",        TabGab_Module ) 
    ELSE_IF_SIMPLE_KEYWORD      ( "Config_TabGab:",     Config_TabGab ) 
    ELSE_IF_KEYWORD_WITH_ALIAS  ( "Group:",            "Group_Address:",        Group_Address ) 
    ELSE_IF_SIMPLE_KEYWORD      ( "Serial_Address:",    Serial_Address ) 
    ELSE_IF_SIMPLE_KEYWORD      ( "Serial_Data_Write:", Serial_Data_Write ) 
    ELSE_IF_SIMPLE_KEYWORD      ( "Serial_Address_Read:",Serial_Address_Read ) 

    ELSE_IF_SIMPLE_KEYWORD      ( "Bit3_Adaptor:",      Bit3_Adaptor ) 
    ELSE_IF_SIMPLE_KEYWORD      ( "Address_Space:",     Address_Space ) 
    ELSE_IF_SIMPLE_KEYWORD      ( "Data_Size:",         Data_Size ) 
    
    ELSE_IF_SIMPLE_KEYWORD      ( "VME_Address:",       VME_Address ) 
    ELSE_IF_SIMPLE_KEYWORD      ( "Read_VME:",          Read_VME ) 

    ELSE_IF_SIMPLE_KEYWORD      ( "TT_Channel:",        TT_Channel ) 
    ELSE_IF_SIMPLE_KEYWORD      ( "TT_Eta_Sign:",       TT_Eta_Sign ) 
    ELSE_IF_SIMPLE_KEYWORD      ( "TT_Eta_Magn:",       TT_Eta_Magn ) 
    ELSE_IF_SIMPLE_KEYWORD      ( "TT_Phi:",            TT_Phi ) 
    ELSE_IF_SIMPLE_KEYWORD      ( "To_TT_Eta_Sign:",    To_TT_Eta_Sign ) 
    ELSE_IF_SIMPLE_KEYWORD      ( "To_TT_Eta_Magn:",    To_TT_Eta_Magn ) 
    ELSE_IF_SIMPLE_KEYWORD      ( "To_TT_Phi:",         To_TT_Phi ) 
    ELSE_IF_SIMPLE_KEYWORD      ( "Tick:",              Tick ) 
    ELSE_IF_SIMPLE_KEYWORD      ( "To_Tick:",           To_Tick ) 
    ELSE_IF_SIMPLE_KEYWORD      ( "SubTick:",           SubTick ) 
    ELSE_IF_SIMPLE_KEYWORD      ( "To_SubTick:",        To_SubTick ) 
    ELSE_IF_SIMPLE_KEYWORD      ( "Alignment_Delay:",   Alignment_Delay ) 
    ELSE_IF_SIMPLE_KEYWORD      ( "Pedestal_DAC:",      Pedestal_DAC ) 
    ELSE_IF_FLOAT_KEYWORD       ( "Ped_Average:",       Ped_Average ) 
    ELSE_IF_FLOAT_KEYWORD       ( "Ped_Deviation:",     Ped_Deviation ) 
    ELSE_IF_SIMPLE_KEYWORD      ( "Zero_Energy_Adc_E:", ZeroEngAdcE ) 
    ELSE_IF_SIMPLE_KEYWORD      ( "Zero_Energy_Output_Et:",     ZeroEngOutEt ) 
    ELSE_IF_FLOAT_KEYWORD       ( "Lookup_Slope:",      Lookup_Slope ) 
    ELSE_IF_SIMPLE_KEYWORD      ( "RawAdc_Simu:",       RawAdc_Simu )
    ELSE_IF_SIMPLE_KEYWORD      ( "AdfOut_Simu:",       AdfOut_Simu )
    ELSE_IF_SIMPLE_KEYWORD      ( "AdfOut_Const:",      AdfOut_Const )
    ELSE_IF_SIMPLE_KEYWORD      ( "RawAdc_Thresh:",     RawAdc_Thresh )
    ELSE_IF_SIMPLE_KEYWORD      ( "RawAdc_MuxMode:",    RawAdc_MuxMode )
    ELSE_IF_SIMPLE_KEYWORD      ( "RawAdc_MemMode:",    RawAdc_MemMode )
    ELSE_IF_SIMPLE_KEYWORD      ( "AdfOut_MuxMode:",    AdfOut_MuxMode )
    ELSE_IF_SIMPLE_KEYWORD      ( "AdfOut_MemMode:",    AdfOut_MemMode )

    ELSE_IF_SIMPLE_KEYWORD      ( "LBN_Value:",         LBN_Value ) 

    ELSE_IF_SIMPLE_KEYWORD      ( "L1Cal_Operational:",         L1Cal_Operational ) 
    ELSE_IF_SIMPLE_KEYWORD      ( "MonitControlTimeMs:",        MonitControlTimeMs ) 
    ELSE_IF_SIMPLE_KEYWORD      ( "BaseAlignmentDelay:",        BaseAlignmentDelay ) 
    ELSE_IF_SIMPLE_KEYWORD      ( "OffsetTick1RawAdcMonit:",    OffsetTick1RawAdcMonit ) 
    ELSE_IF_SIMPLE_KEYWORD      ( "OffsetTick1RawAdcSimu:",     OffsetTick1RawAdcSimu ) 
    ELSE_IF_SIMPLE_KEYWORD      ( "OffsetTick1OutputEtMonit:",  OffsetTick1OutputEtMonit ) 
    ELSE_IF_SIMPLE_KEYWORD      ( "OffsetTick1OutputEtSimu:",   OffsetTick1OutputEtSimu ) 
    ELSE_IF_SIMPLE_KEYWORD      ( "OffsetTriggeredToTickAtCapture:",   OffsetTriggeredToTickAtCapture ) 
    ELSE_IF_SIMPLE_KEYWORD      ( "TtiImmediateIo:",            TtiImmediateIo ) 
    ELSE_IF_SIMPLE_KEYWORD      ( "FindDacMode:",               FindDacMode ) 
    ELSE_IF_SIMPLE_KEYWORD      ( "TabEta38Value:",             TabEta38Value ) 
    ELSE_IF_SIMPLE_KEYWORD      ( "MaxDeSkewTryAtInit:",        MaxDeSkewTryAtInit ) 
    ELSE_IF_SIMPLE_KEYWORD      ( "AbortInitOnError:",          AbortInitOnError ) 
    ELSE_IF_SIMPLE_KEYWORD      ( "SkipTabGabOnInit:",          SkipTabGabOnInit ) 
    ELSE_IF_SIMPLE_KEYWORD      ( "MonitIgnoreTabGab:",         MonitIgnoreTabGab ) 

    else if ( stricmp( acKeyword, "Source_Description:" )  ==   0  ) 
	{
		if ( _pxParse->Values.osSource_Description == KsNullString )
            _pxParse->Values.osSource_Description = acValue ;
        else 
        {
            _pxParse->Values.osSource_Description += "\n" ;
            _pxParse->Values.osSource_Description += acValue ;
        }
        // look if there is an action defined for this keyword
        if ( _pxParse->Actions.pfSource_Description != NULL )
            eSuccess = _pxParse->Actions.pfSource_Description ( _pxParse, _pxCustom, _pThis ) ;
    }

    ELSE_IF_KEYWORD_TO_GENERIC_STRING ( "TabGab_PackageVersion:", TabGab_PackageVersion )
    ELSE_IF_KEYWORD_TO_GENERIC_STRING ( "TabGab_FirmwareDir:",    TabGab_FirmwareDir )
    ELSE_IF_KEYWORD_TO_GENERIC_STRING ( "TabGab_ConfigDir:",      TabGab_ConfigDir )       
    ELSE_IF_KEYWORD_TO_GENERIC_STRING ( "TabGab_WorkDir:",        TabGab_WorkDir )

    ELSE_IF_KEYWORD_TO_GENERIC_STRING ( "Config_File:",         Config_File ) 
    
    ELSE_IF_KEYWORD_TO_GENERIC_STRING ( "Call_Reg_IO:",         Call_Reg_IO ) 
    ELSE_IF_KEYWORD_TO_GENERIC_STRING ( "Call_Serial_IO:",      Call_Serial_IO ) 
    ELSE_IF_KEYWORD_TO_GENERIC_STRING ( "Call_Vme_IO:",         Call_Vme_IO ) 
    ELSE_IF_KEYWORD_TO_GENERIC_STRING ( "Call_Cbus_IO:",        Call_Cbus_IO ) 
    ELSE_IF_KEYWORD_TO_GENERIC_STRING ( "Call_Config_Fpga:",    Call_Config_Fpga ) 
    ELSE_IF_KEYWORD_TO_GENERIC_STRING ( "Call_Self_Msg:",       Call_Self_Msg ) 
    ELSE_IF_KEYWORD_TO_GENERIC_STRING ( "Call_TT_Info:",        Call_TT_Info ) 
    ELSE_IF_KEYWORD_TO_GENERIC_STRING ( "Call_Config_T962:",    Call_Config_T962 ) 

    ELSE_IF_KEYWORD_TO_GENERIC_STRING ( "Send_Msg_To_Self:",    Send_Msg_To_Self ) 

    else
    {
        oConsole.ErrorLine ( (CString) "_ Unknown Keyword <"
                                      + acKeyword
                                      + "> at Line #"
                                      + I_to_A ( _pxParse->iLineNum ) ) ;
        eSuccess = Error ;
    }


    if ( eSuccess == Ok ) 
    	goto Next_Keyword ;
    else
    {
        eReturnSuccess = Error ;
        goto Done_Read ;
    }

Done_Read:
	
    oConsole.WriteLine ( (CString) "_ Done Reading <"
                                  + acFileName
                                  + "> at Line #"
                                  + I_to_A ( _pxParse->iLineNum ) ) ;

    // look if there is an action defined for the end of the job 
    if ( _pxParse->Actions.pfDone_With_File != NULL )
    { 
        eSuccess = _pxParse->Actions.pfDone_With_File ( _pxParse, _pxCustom, _pThis ) ;
        if ( eSuccess == Error )    eReturnSuccess = Error ;
    }

	if ( fclose ( CommandFile ) ) 
	{
        oConsole.ErrorLine ( (CString) "_ The file didn't close correctly " ) ;
        eReturnSuccess = Error ;
	}


Error_Open:

	return eReturnSuccess ;
}

/*----------------------------------------------------------------------------------*/
// Flag Error Illegal Keyword
/*----------------------------------------------------------------------------------*/
ESuccess Action_Parse_Illegal_Keyword( Parse_Structure*    _pxParse, 
                                       void*               _pxCustom,
                                       void*               _pThis ) 
{
	
    oConsole.ErrorLine ( (CString) "_ Keyword Illegal in this context at Line #"
                                    + I_to_A ( _pxParse->iLineNum ) ) ;

    return Error ;
}


/*----------------------------------------------------------------------------------*/
// Initialize a Parse_Structure 
/*----------------------------------------------------------------------------------*/
ESuccess Init_Parse_Structure ( Parse_Structure*     _pxParse ) 
{

    if ( _pxParse == 0 ) return Error ;
    
    // The structure needs to be initialized
    // so that one can tell keywords that received a value
    // from keywords that didn't get specified yet

    _pxParse->Values.ulCrate                    = diNotYetAssigned ;
    _pxParse->Values.ulT962_Wire                = diNotYetAssigned ;
    _pxParse->Values.ulT962_Maestro_Slot        = diNotYetAssigned ;
    _pxParse->Values.ulVertical_Master          = diNotYetAssigned ;
    _pxParse->Values.ulVertical_Slave           = diNotYetAssigned ;
    _pxParse->Values.ulCard_Slot                = diNotYetAssigned ;
    _pxParse->Values.ulTo_Card_Slot             = diNotYetAssigned ;
    _pxParse->Values.ulChip_Address             = diNotYetAssigned ;
    _pxParse->Values.ulTo_Chip_Address          = diNotYetAssigned ;
    _pxParse->Values.ulChip_Mask                = diNotYetAssigned ;
    _pxParse->Values.ulRegister_Address         = diNotYetAssigned ;
    _pxParse->Values.ulTo_Register_Address      = diNotYetAssigned ;
    _pxParse->Values.ulRegister_Start           = diNotYetAssigned ;
    _pxParse->Values.ulRegister_Count           = diNotYetAssigned ;
    _pxParse->Values.ulRead_Register_Address    = diNotYetAssigned ;
    _pxParse->Values.ulWrite_Value              = diNotYetAssigned ;
    _pxParse->Values.ulWrite_Verify             = diNotYetAssigned ;
    _pxParse->Values.ulSource_Mask              = diNotYetAssigned ;
    _pxParse->Values.ulReceiver_Mask            = diNotYetAssigned ;
    _pxParse->Values.ulConfig_TabGab            = diNotYetAssigned ;
    _pxParse->Values.ulTabGab_Module            = diNotYetAssigned ;
    _pxParse->Values.ulGroup_Address            = diNotYetAssigned ;
    _pxParse->Values.ulSerial_Address           = diNotYetAssigned ;
    _pxParse->Values.ulSerial_Data_Write        = diNotYetAssigned ;
    _pxParse->Values.ulSerial_Address_Read      = diNotYetAssigned ;
    _pxParse->Values.ulBit3_Adaptor             = diNotYetAssigned ;
    _pxParse->Values.ulAddress_Space            = diNotYetAssigned ;
    _pxParse->Values.ulData_Size                = diNotYetAssigned ;
    _pxParse->Values.ulVME_Address              = diNotYetAssigned ;
    _pxParse->Values.ulRead_VME                 = diNotYetAssigned ;
    _pxParse->Values.ulTT_Channel               = diNotYetAssigned ;
    _pxParse->Values.ulTT_Eta_Sign              = diNotYetAssigned ;
    _pxParse->Values.ulTT_Eta_Magn              = diNotYetAssigned ;
    _pxParse->Values.ulTT_Phi                   = diNotYetAssigned ;
    _pxParse->Values.ulTo_TT_Eta_Sign           = diNotYetAssigned ;
    _pxParse->Values.ulTo_TT_Eta_Magn           = diNotYetAssigned ;
    _pxParse->Values.ulTo_TT_Phi                = diNotYetAssigned ;
    _pxParse->Values.ulTick                     = diNotYetAssigned ; 
    _pxParse->Values.ulTo_Tick                  = diNotYetAssigned ; 
    _pxParse->Values.ulSubTick                  = diNotYetAssigned ; 
    _pxParse->Values.ulTo_SubTick               = diNotYetAssigned ; 
    _pxParse->Values.ulAlignment_Delay          = diNotYetAssigned ;
    _pxParse->Values.ulPedestal_DAC             = diNotYetAssigned ;
    _pxParse->Values.fdPed_Average              = diNotYetAssigned ;
    _pxParse->Values.fdPed_Deviation            = diNotYetAssigned ;
    _pxParse->Values.ulZeroEngAdcE              = diNotYetAssigned ;
    _pxParse->Values.ulZeroEngOutEt             = diNotYetAssigned ;
    _pxParse->Values.fdLookup_Slope             = diNotYetAssigned ;
    _pxParse->Values.ulRawAdc_Simu              = diNotYetAssigned ;
    _pxParse->Values.ulAdfOut_Simu              = diNotYetAssigned ;
    _pxParse->Values.ulAdfOut_Const             = diNotYetAssigned ;
    _pxParse->Values.ulRawAdc_Thresh            = diNotYetAssigned ;
    _pxParse->Values.ulRawAdc_MuxMode           = diNotYetAssigned ;
    _pxParse->Values.ulRawAdc_MemMode           = diNotYetAssigned ;
    _pxParse->Values.ulAdfOut_MuxMode           = diNotYetAssigned ;
    _pxParse->Values.ulAdfOut_MemMode           = diNotYetAssigned ;
    _pxParse->Values.ulLBN_Value                = diNotYetAssigned ;
    _pxParse->Values.ulL1Cal_Operational        = diNotYetAssigned ;
    _pxParse->Values.ulMonitControlTimeMs       = diNotYetAssigned ;
    _pxParse->Values.ulBaseAlignmentDelay       = diNotYetAssigned ;
    _pxParse->Values.ulOffsetTick1RawAdcMonit   = diNotYetAssigned ;
    _pxParse->Values.ulOffsetTick1RawAdcSimu    = diNotYetAssigned ;
    _pxParse->Values.ulOffsetTick1OutputEtMonit = diNotYetAssigned ;
    _pxParse->Values.ulOffsetTick1OutputEtSimu  = diNotYetAssigned ;
    _pxParse->Values.ulOffsetTriggeredToTickAtCapture  = diNotYetAssigned ;
    _pxParse->Values.ulTtiImmediateIo           = diNotYetAssigned ;
    _pxParse->Values.ulFindDacMode              = diNotYetAssigned ;
    _pxParse->Values.ulTabEta38Value            = diNotYetAssigned ;
    _pxParse->Values.ulMaxDeSkewTryAtInit       = diNotYetAssigned ;
    _pxParse->Values.ulAbortInitOnError         = diNotYetAssigned ;
    _pxParse->Values.ulSkipTabGabOnInit         = diNotYetAssigned ;
    _pxParse->Values.ulMonitIgnoreTabGab        = diNotYetAssigned ;
    _pxParse->Values.osSource_Description       = KsNullString ;
    _pxParse->Values.osGeneric_String_Parameter = KsNullString ;

    // and so that the default action is to skip (no action) unless a function was given
    _pxParse->Actions.pfFile_Opened             = NULL ;

    _pxParse->Actions.pfCrate                   = (fKeyword_Action*) Action_Parse_Illegal_Keyword ;
    _pxParse->Actions.pfT962_Wire               = (fKeyword_Action*) Action_Parse_Illegal_Keyword ;
    _pxParse->Actions.pfT962_Maestro_Slot       = (fKeyword_Action*) Action_Parse_Illegal_Keyword ;
    _pxParse->Actions.pfVertical_Master         = (fKeyword_Action*) Action_Parse_Illegal_Keyword ;
    _pxParse->Actions.pfVertical_Slave          = (fKeyword_Action*) Action_Parse_Illegal_Keyword ;
    _pxParse->Actions.pfCard_Slot               = (fKeyword_Action*) Action_Parse_Illegal_Keyword ;
    _pxParse->Actions.pfTo_Card_Slot            = (fKeyword_Action*) Action_Parse_Illegal_Keyword ;
    _pxParse->Actions.pfChip_Address            = (fKeyword_Action*) Action_Parse_Illegal_Keyword ;
    _pxParse->Actions.pfTo_Chip_Address         = (fKeyword_Action*) Action_Parse_Illegal_Keyword ;
    _pxParse->Actions.pfChip_Mask               = (fKeyword_Action*) Action_Parse_Illegal_Keyword ;
    _pxParse->Actions.pfRegister_Address        = (fKeyword_Action*) Action_Parse_Illegal_Keyword ;
    _pxParse->Actions.pfTo_Register_Address     = (fKeyword_Action*) Action_Parse_Illegal_Keyword ;
    _pxParse->Actions.pfRegister_Start          = (fKeyword_Action*) Action_Parse_Illegal_Keyword ;
    _pxParse->Actions.pfRegister_Count          = (fKeyword_Action*) Action_Parse_Illegal_Keyword ;
    _pxParse->Actions.pfRead_Register_Address   = (fKeyword_Action*) Action_Parse_Illegal_Keyword ;
    _pxParse->Actions.pfWrite_Value             = (fKeyword_Action*) Action_Parse_Illegal_Keyword ;
    _pxParse->Actions.pfWrite_Verify            = (fKeyword_Action*) Action_Parse_Illegal_Keyword ;
    _pxParse->Actions.pfSource_Mask             = (fKeyword_Action*) Action_Parse_Illegal_Keyword ;
    _pxParse->Actions.pfReceiver_Mask           = (fKeyword_Action*) Action_Parse_Illegal_Keyword ;
    _pxParse->Actions.pfConfig_TabGab           = (fKeyword_Action*) Action_Parse_Illegal_Keyword ;
    _pxParse->Actions.pfTabGab_Module           = (fKeyword_Action*) Action_Parse_Illegal_Keyword ;
    _pxParse->Actions.pfGroup_Address           = (fKeyword_Action*) Action_Parse_Illegal_Keyword ;
    _pxParse->Actions.pfSerial_Address          = (fKeyword_Action*) Action_Parse_Illegal_Keyword ;
    _pxParse->Actions.pfSerial_Data_Write       = (fKeyword_Action*) Action_Parse_Illegal_Keyword ;
    _pxParse->Actions.pfSerial_Address_Read     = (fKeyword_Action*) Action_Parse_Illegal_Keyword ;
    _pxParse->Actions.pfBit3_Adaptor            = (fKeyword_Action*) Action_Parse_Illegal_Keyword ;
    _pxParse->Actions.pfAddress_Space           = (fKeyword_Action*) Action_Parse_Illegal_Keyword ;
    _pxParse->Actions.pfData_Size               = (fKeyword_Action*) Action_Parse_Illegal_Keyword ;
    _pxParse->Actions.pfVME_Address             = (fKeyword_Action*) Action_Parse_Illegal_Keyword ;
    _pxParse->Actions.pfRead_VME                = (fKeyword_Action*) Action_Parse_Illegal_Keyword ;
    _pxParse->Actions.pfSource_Description      = (fKeyword_Action*) Action_Parse_Illegal_Keyword ;
    _pxParse->Actions.pfConfig_File             = (fKeyword_Action*) Action_Parse_Illegal_Keyword ;
    _pxParse->Actions.pfTabGab_PackageVersion   = (fKeyword_Action*) Action_Parse_Illegal_Keyword ;
    _pxParse->Actions.pfTabGab_FirmwareDir      = (fKeyword_Action*) Action_Parse_Illegal_Keyword ;
    _pxParse->Actions.pfTabGab_ConfigDir        = (fKeyword_Action*) Action_Parse_Illegal_Keyword ;
    _pxParse->Actions.pfTabGab_WorkDir          = (fKeyword_Action*) Action_Parse_Illegal_Keyword ;

    _pxParse->Actions.pfTT_Channel              = (fKeyword_Action*) Action_Parse_Illegal_Keyword ;
    _pxParse->Actions.pfTT_Eta_Sign             = (fKeyword_Action*) Action_Parse_Illegal_Keyword ;
    _pxParse->Actions.pfTT_Eta_Magn             = (fKeyword_Action*) Action_Parse_Illegal_Keyword ;
    _pxParse->Actions.pfTT_Phi                  = (fKeyword_Action*) Action_Parse_Illegal_Keyword ;
    _pxParse->Actions.pfTo_TT_Eta_Sign          = (fKeyword_Action*) Action_Parse_Illegal_Keyword ;
    _pxParse->Actions.pfTo_TT_Eta_Magn          = (fKeyword_Action*) Action_Parse_Illegal_Keyword ;
    _pxParse->Actions.pfTo_TT_Phi               = (fKeyword_Action*) Action_Parse_Illegal_Keyword ;
    _pxParse->Actions.pfTick                    = (fKeyword_Action*) Action_Parse_Illegal_Keyword ; 
    _pxParse->Actions.pfTo_Tick                 = (fKeyword_Action*) Action_Parse_Illegal_Keyword ; 
    _pxParse->Actions.pfSubTick                 = (fKeyword_Action*) Action_Parse_Illegal_Keyword ; 
    _pxParse->Actions.pfTo_SubTick              = (fKeyword_Action*) Action_Parse_Illegal_Keyword ; 
    _pxParse->Actions.pfAlignment_Delay         = (fKeyword_Action*) Action_Parse_Illegal_Keyword ;
    _pxParse->Actions.pfPedestal_DAC            = (fKeyword_Action*) Action_Parse_Illegal_Keyword ;
    _pxParse->Actions.pfPed_Average             = (fKeyword_Action*) Action_Parse_Illegal_Keyword ;
    _pxParse->Actions.pfPed_Deviation           = (fKeyword_Action*) Action_Parse_Illegal_Keyword ;
    _pxParse->Actions.pfZeroEngAdcE             = (fKeyword_Action*) Action_Parse_Illegal_Keyword ;
    _pxParse->Actions.pfZeroEngOutEt            = (fKeyword_Action*) Action_Parse_Illegal_Keyword ;
    _pxParse->Actions.pfLookup_Slope            = (fKeyword_Action*) Action_Parse_Illegal_Keyword ;
    _pxParse->Actions.pfRawAdc_Simu             = (fKeyword_Action*) Action_Parse_Illegal_Keyword ;
    _pxParse->Actions.pfAdfOut_Simu             = (fKeyword_Action*) Action_Parse_Illegal_Keyword ;
    _pxParse->Actions.pfRawAdc_MuxMode          = (fKeyword_Action*) Action_Parse_Illegal_Keyword ;
    _pxParse->Actions.pfRawAdc_MemMode          = (fKeyword_Action*) Action_Parse_Illegal_Keyword ;
    _pxParse->Actions.pfAdfOut_MuxMode          = (fKeyword_Action*) Action_Parse_Illegal_Keyword ;
    _pxParse->Actions.pfAdfOut_MemMode          = (fKeyword_Action*) Action_Parse_Illegal_Keyword ;

    _pxParse->Actions.pfLBN_Value               = (fKeyword_Action*) Action_Parse_Illegal_Keyword ;
    _pxParse->Actions.pfL1Cal_Operational       = (fKeyword_Action*) Action_Parse_Illegal_Keyword ;
    _pxParse->Actions.pfMonitControlTimeMs      = (fKeyword_Action*) Action_Parse_Illegal_Keyword ;
    _pxParse->Actions.pfBaseAlignmentDelay      = (fKeyword_Action*) Action_Parse_Illegal_Keyword ;
    _pxParse->Actions.pfOffsetTick1RawAdcMonit  = (fKeyword_Action*) Action_Parse_Illegal_Keyword ;
    _pxParse->Actions.pfOffsetTick1RawAdcSimu   = (fKeyword_Action*) Action_Parse_Illegal_Keyword ;
    _pxParse->Actions.pfOffsetTick1OutputEtMonit= (fKeyword_Action*) Action_Parse_Illegal_Keyword ;
    _pxParse->Actions.pfOffsetTick1OutputEtSimu = (fKeyword_Action*) Action_Parse_Illegal_Keyword ;
    _pxParse->Actions.pfOffsetTriggeredToTickAtCapture = (fKeyword_Action*) Action_Parse_Illegal_Keyword ;
    _pxParse->Actions.pfTtiImmediateIo          = (fKeyword_Action*) Action_Parse_Illegal_Keyword ;
    _pxParse->Actions.pfFindDacMode             = (fKeyword_Action*) Action_Parse_Illegal_Keyword ;
    _pxParse->Actions.pfTabEta38Value           = (fKeyword_Action*) Action_Parse_Illegal_Keyword ;
    _pxParse->Actions.pfMaxDeSkewTryAtInit      = (fKeyword_Action*) Action_Parse_Illegal_Keyword ;
    _pxParse->Actions.pfAbortInitOnError        = (fKeyword_Action*) Action_Parse_Illegal_Keyword ;
    _pxParse->Actions.pfSkipTabGabOnInit        = (fKeyword_Action*) Action_Parse_Illegal_Keyword ;
    _pxParse->Actions.pfMonitIgnoreTabGab       = (fKeyword_Action*) Action_Parse_Illegal_Keyword ;

    _pxParse->Actions.pfCall_Reg_IO             = (fKeyword_Action*) Action_Parse_Illegal_Keyword ;
    _pxParse->Actions.pfCall_Serial_IO          = (fKeyword_Action*) Action_Parse_Illegal_Keyword ;
    _pxParse->Actions.pfCall_Cbus_IO            = (fKeyword_Action*) Action_Parse_Illegal_Keyword ;
    _pxParse->Actions.pfCall_Vme_IO             = (fKeyword_Action*) Action_Parse_Illegal_Keyword ;
    _pxParse->Actions.pfCall_Config_Fpga        = (fKeyword_Action*) Action_Parse_Illegal_Keyword ;
    _pxParse->Actions.pfCall_Self_Msg           = (fKeyword_Action*) Action_Parse_Illegal_Keyword ;
    _pxParse->Actions.pfCall_TT_Info            = (fKeyword_Action*) Action_Parse_Illegal_Keyword ;
    _pxParse->Actions.pfCall_Config_T962        = (fKeyword_Action*) Action_Parse_Illegal_Keyword ;

    _pxParse->Actions.pfSend_Msg_To_Self        = (fKeyword_Action*) Action_Parse_Illegal_Keyword ;

    _pxParse->Actions.pfDone_With_File          = NULL ;

    return Ok ;
}

