/*-------------------------------------------------------------------------------*/
  /*
    HandleConfigAdfFpga.cpp
    17-Nov-2004
  */
/*-------------------------------------------------------------------------------*/

#include "L1Cal_IIb_TCC/StdAfx.h"

#include "L1Cal_IIb_TCC/GlobalConstants.h"
#include "L1Cal_IIb_TCC/CustomDataTypes.h"

#include "L1Cal_IIb_TCC/AdfChipPal.h"
#include "L1Cal_IIb_TCC/AdfCardBase.h"

#include                            "L1Cal_IIb_TCC/HandleConfigAdfFpga.h"

#include "L1Cal_IIb_TCC/HandleSRecordFiles.h"

#include "L1Cal_IIb_TCC/UtilsStrings.h"
#include "L1Cal_IIb_TCC/UtilsFiles.h"

// static means created only once throughout life of program (to avoid wasting time)
// *AND* internal linkage (to avoid clash between modules)
#include "L1Cal_IIb_TCC/HandleConsole.h"
static CConsoleAccess oConsole ;

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess ConfigureAdfFpga ( CAdfCardBase*   _poTargetCard,
                            unsigned short  _uwConfigEnabMask,
            	            CString	        _osConfiguration_File_Name,
                            TMaskBoardCsr   _mCsrFlags,
                            unsigned short*  _puwConfigured,
                            TErrorCode*     _ptErrorCode,
                            EYesNo          _eVerbose ) 
{
    ESuccess        eReturnStatus ;
    ESuccess        eSuccess ;

    unsigned char   ubVersionMajor ;
    unsigned char   ubVersionMinor ;

    unsigned char   aubByteStream[diFPGA_Config_Byte_Size_Max] ;
    int             iByteStreamLength ;

    unsigned short  uwConfigured ;
    TErrorCode      tErrorCode ;

    //\\//
	
	//The function will return a success (True) only if it can do a successfull configuration,
    //verify the fpgas were configured, and set the CSR flags
	eReturnStatus   = Ok ;

    //This default error flag will be set to an actuall value upon error
    tErrorCode = KeERR_DldFpga_NoError ;

    // advertize Target Card and FPGA Mask to console
 	oConsole.WriteLine ( (CString) "% Configuring FPGA(s)  "
                                 + _poTargetCard->f_osBaseAddress()
                                 + "/Chips "
                                 + I_to_A( _uwConfigEnabMask, 2, 2, '0' ) ) ;


    // select the specified FPGAs and prepares them for configuration
    eSuccess = CheckConfigurationMask ( _uwConfigEnabMask ) ;
    if ( eSuccess != Ok ) 
    {
	    oConsole.ErrorLine ( (CString) "% Abort Config: Illegal Set of FPGAs" ) ; 
    	eReturnStatus   = Error ;
        tErrorCode = KeERR_DldFpga_BadDldMask ;
        goto Done ;
    }


    //Extract Version Number from File Name
	ExtractVersion   ( LPCTSTR(_osConfiguration_File_Name), 
                               &ubVersionMajor,
                               &ubVersionMinor ) ;

    // advertize File Name to console
    oConsole.WriteLine ( (CString) "_ Using File <"
                        		 +  _osConfiguration_File_Name 
                                 + "> and Noting FPGA Version is "
                                 + I_to_A (ubVersionMajor)
                                 + "."
                                 + I_to_A (ubVersionMinor) ) ;
    
    //call Read_FPGA_Config_File to open and read the FPGA S-record configuration file
	//The function will return a success (Ok) only if it can open, and read the file
	eSuccess = Read_FPGA_Config_File ( LPCTSTR(_osConfiguration_File_Name), 
                                       aubByteStream, 
                                       &iByteStreamLength ) ;

    if ( eSuccess == Error ) 
    {
	    oConsole.ErrorLine ( (CString) "% Abort Configuration: S-Record File Read Error " ) ;
    	eReturnStatus   = Error ;
        tErrorCode = KeERR_DldFpga_SRecFile ;
        goto Done ;
    }


    // prepare the set of FPGAs for configuration, and record the FPGA Version in VME Int Scratch Memory
    eSuccess = _poTargetCard->m_poBcPal->InitiateFpgaConfig   ( _uwConfigEnabMask,
                                                                 ubVersionMajor,
                                                                 ubVersionMinor ) ;
#if defined ( VME_IO_FAKE )  
        // ok no mattery what
#else
    if ( eSuccess != Ok ) 
    {
	    oConsole.ErrorLine ( (CString) "% Abort Configuration: Failure Preparing Configuration" ) ; 
     	eReturnStatus   = Error ;
        tErrorCode = KeERR_DldFpga_DldInit ;
        goto Done ;
    }
#endif


    // actual configuration happens now
    eSuccess = _poTargetCard->m_poBcPal->WriteFpgaConfig     ( _uwConfigEnabMask,
								                                aubByteStream,
                                                                iByteStreamLength ) ;
    if ( eSuccess != Ok ) 
    {
	    oConsole.ErrorLine ( (CString) "% Abort Configuration: Byte Stream Configuration Failure" ) ; 
     	eReturnStatus   = Error ;
        tErrorCode = KeERR_DldFpga_DldByteStream ;
        goto Done ;
    }


    // Now de-select FPGAs to terminate configuration
    eSuccess = _poTargetCard->m_poBcPal->TerminateFpgaConfig ( _uwConfigEnabMask ) ;

#if defined ( VME_IO_FAKE )  
        // ok no mattery what
#else
    if ( eSuccess != Ok ) 
	{
	    oConsole.ErrorLine ( (CString) "% Abort Configuration: Failure Terminating Configuration" ) ; 
     	eReturnStatus   = Error ;
        tErrorCode = KeERR_DldFpga_DldTerminate ;
        goto Done ;
	}
#endif

	// read and display the board level CSR FPGA configured Register
    eSuccess = _poTargetCard->m_poBcPal->GetFpgaConfiguredMask ( &uwConfigured ) ;

    oConsole.WriteLine ( (CString) "_ Board FPGA Configured Register after Configuration is now "
                                  + I_to_A( uwConfigured, 2, 2, '0' )  ) ;


    // check for success by looking at the fpga configured mask 
    // declare temporary variable to check configured status
    unsigned long ulCheckConfigured ;
    // first invert each bit so that chips NOT configured show up as ones 
    ulCheckConfigured = uwConfigured ^ 0xffff ;
    // trim the mask to the set of FPGAs just configured
    ulCheckConfigured &= _uwConfigEnabMask ;
    // any bit that's left now is an fpga that was targeted but failed its configuration
#if defined ( VME_IO_FAKE )  
        // ok no mattery what
#else
    if ( ulCheckConfigured != 0 ) 
    {
        oConsole.ErrorLine ( (CString) "_ Configure failed for FPGA(s)                   " 
                                      + I_to_A( ulCheckConfigured, 2, 2, '0' )  ) ;
        eReturnStatus   = Error ;
        tErrorCode = KeERR_DldFpga_ConfiguredMask ;
        goto Done ;
    }
#endif

    // Set/Reset the Board CSR Flags as requested
    
    // catch all errors that happened while setting CSR bits
//    if ( eReturnStatus != Ok )         tErrorCode = KeERR_DldFpga_SetCsr ;

Done:
    // fill optional arguments
    if ( _puwConfigured != NULL ) *_puwConfigured = uwConfigured ;
    if ( _ptErrorCode   != NULL ) *_ptErrorCode   = tErrorCode ;

 	oConsole.WriteLine ( "" ) ;

    return eReturnStatus ;

}
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess CheckConfigurationMask ( unsigned short _uwConfigEnabMask ) 
{
    ESuccess    eReturnStatus = Ok ;

#define KmAdfFpga_0            0x00000001
#define KmAdfFpga_1            0x00000002
#define KmAllConfigurableFpga  ( KmAdfFpga_0 | KmAdfFpga_1 )

    //\\//

    // Only select chips we can configuration
    if ( _uwConfigEnabMask & (~KmAllConfigurableFpga) ) 
    {
        oConsole.ErrorLine ( "% Illegal Configure Mask: Non-existent Adf Fpga(s) Selected" ) ;
        eReturnStatus = Error ;
    }
    
    // Select at least one chip
    if ( _uwConfigEnabMask == 0 ) 
    {
        oConsole.ErrorLine ( "% Illegal Configure Mask: No Adf Fpga Selected" ) ;
        eReturnStatus = Error ;
    }
    
    return eReturnStatus ;
}

