/*-------------------------------------------------------------------------------*/
  /*
    HandleConsole.cpp
    1-May-98
    8-May-98 Add Set color member functions
    26-Oct-98 rename from ConsoleAccess to HandleConsole
                add call to logfile in Writeline
  */
/*-------------------------------------------------------------------------------*/
#include "L1Cal_IIb_TCC/StdAfx.h"

#include "L1Cal_IIb_TCC/GlobalConstants.h"
#include "L1Cal_IIb_TCC/CustomDataTypes.h"

#include                            "L1Cal_IIb_TCC/HandleConsole.h"

#include "L1Cal_IIb_TCC/HandleRemoteConsole.h"
#include "L1Cal_IIb_TCC/HandleLogFile.h"

#include "L1Cal_IIb_TCC/UtilsStrings.h"

#if defined ( NO_MS_CONSOLE )
    #include <iostream>
    #include "L1Cal_IIb_TCC/ResourceLocks.h"   // needs to synchronize access
#else
    #include <iostream>
    #include <wincon.h>
    #include <winuser.h>
#endif

// #include <winbase.h> // (for Sleep)

/*-------------------------------------------------------------------------------*/
//constant gobal variables
static  const   char        cCR = '\n' ;

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
// Static data members must be initialized at file scope
#if defined ( NO_MS_CONSOLE )
#else

    HANDLE CConsoleAccess::mg_hConsole = NULL ;
    HWND   CConsoleAccess::mg_hWindow  = NULL ;

#endif
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
CConsoleAccess::CConsoleAccess ( ) 
{
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
CConsoleAccess::~CConsoleAccess ( ) 
{
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess CConsoleAccess::NewConsole ( ) 
{// open handle to a console, 
 // valid for the whole application 

#if defined ( NO_MS_CONSOLE )
    return Ok ;
#else

    BOOL bSuccess;
    //\\//

    //drop older console first, in case there already was a console
    CloseConsole();

    //create new one
	bSuccess = TRUE ; // AllocConsole(); // For Run IIb this now seems to cause problems 
                                         // as AllocConsole creates a second window (because not using MFC??)
    if ( bSuccess == FALSE ) 
    {//Do Not advertize error with ErrorLine (if console creation indeed failed-> infinite loop!)
        std::cout << "Console Error/AllocConsole = " 
                  << I_to_A ( GetLastError() ) 
                  << std::endl;

        return Error ;
    }
    else 
    {   
/*
Here we tried to get rid of the problem where a dialog box is stuck
and waits for a <CR> in the console window.
But it is simply sufficient to change the console default properties 
to get rid of the "QuickEdit Mode".

        //Get Handle to Standard Input created with Console
        HANDLE hConsole_In ;
        hConsole_In = GetStdHandle(STD_INPUT_HANDLE) ;
        //and suppress all keyboard and mouse inputs
        bSuccess = SetConsoleMode ( hConsole_In, 0 ) ;
        //and on top of that, 
        //Get rid of Standard Input created with Console
        bSuccess = SetStdHandle ( STD_INPUT_HANDLE, 0 ) ;
*/
        //Get Handle to Standard Output created with Console
        mg_hConsole = GetStdHandle(STD_OUTPUT_HANDLE);

        //Get Handle to Console Window
        mg_hWindow  = GetConsoleHwnd ( ) ;

        // set the console window position just rigt of main menu
        bSuccess = SetWindowPos  ( mg_hWindow,
                                   NULL,
                                   350,0,
                                   0,0,
                                   SWP_NOSIZE | SWP_NOZORDER ) ;        
        if ( bSuccess == FALSE ) 
            InternalErrorLine ( (CString) "Console Error/SetWindowPos = " 
                                         + I_to_A ( GetLastError() ) ) ;

        // now set console window buffer size
        COORD xBufferSize = { 100, 500 } ;
        bSuccess = SetConsoleScreenBufferSize ( mg_hConsole, xBufferSize ) ;
        if ( bSuccess == FALSE ) 
            InternalErrorLine ( (CString) "Console Error/SetConsoleScreenBufferSize = " 
                                         + I_to_A ( GetLastError() ) ) ;

        // now resize console window
        SMALL_RECT xWindowSize = { 0, 0, 80, 50 } ;
        bSuccess = SetConsoleWindowInfo ( mg_hConsole, 
                                          TRUE, // i.e. abosolute position
                                         &xWindowSize ) ;
        if ( bSuccess == FALSE ) 
            InternalErrorLine ( (CString) "Console Error/SetConsoleWindowInfo = " 
                                         + I_to_A ( GetLastError() ) ) ;
        
        return Ok ;
    }
#endif
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess CConsoleAccess::CloseConsole ( ) 
{// drop the console window, 
    // not at the moment: // after giving user a chance (i.e. 2 sec) 
    // not at the moment: // to see last message(s)

    ESuccess    eReturnStatus = Ok ;
    //\\//

#if defined ( NO_MS_CONSOLE )
#else

//    int two_seconds = 2000 ;
//    Sleep ( two_seconds ) ;

//    if ( FreeConsole() == FALSE )  // For Run IIb this now seems to cause problems 
//        eReturnStatus = Error ;    // as AllocConsole creates a second window (because not using MFC??)

    //clear the pointers to the console
    mg_hConsole = NULL ;
    mg_hWindow  = NULL ;

#endif

    return eReturnStatus ;
}


/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess CConsoleAccess::WriteLine      ( const char*       const _sString,
                                          const char*       const _sPrefix,
                                          const int               _iMsgLengthMax ) 
{
    ESuccess    eReturnStatus = Ok ;
    char*       psBuffer ; 
    char*       psMsg ; 
    char*       pcCR ;
    int         iStrLen ;

#if defined ( NO_MS_CONSOLE )
#else
	DWORD       uiCharWritten ;
#endif

    //\\//

#if defined ( NO_MS_CONSOLE )
#else
    // create a console if there wasn't one before
    // note: this may be an infinite loop problem if console creation 
    // consistently fails and generates an error message.
    if ( mg_hConsole == NULL ) NewConsole () ;
#endif

    //allocate memory for copy of input string
    psBuffer = new char [KiConsoleMsgPrefixLength
                       + _iMsgLengthMax 
                       + KiConsoleMessageSpaceForCR
                       + KiConsoleMessageSpaceForNULL] ;
    //we need an independent pointer that may be modified
    //while we also need to preserve the original pointer for cleanup
    psMsg = psBuffer ;

    //preface the string with an Info Flag if there isn't already a flag
    if ( _sPrefix != NULL )
        strncpy ( psMsg, _sPrefix, KiConsoleMsgPrefixLength ) ;
    else
        strncpy ( psMsg, KiConsolePrefixInfo, KiConsoleMsgPrefixLength ) ;

    // copy source string to local variable
    // and limit the damages (by using strncpy) if source string wasn't null terminate.
    strncpy ( psMsg+KiConsoleMsgPrefixLength, _sString, _iMsgLengthMax ) ;
    // if we have maxed out our string, replace the last printed character with a '?' 
    // and force null character to truncate string if it is longer than 132 char
    // (we had declared the string long enough (+KiConsoleMessageSpaceForNULL) to add this character)
    if ( psMsg[KiConsoleMsgPrefixLength+_iMsgLengthMax-1] != 0 )
    {
        psMsg[KiConsoleMsgPrefixLength+_iMsgLengthMax] = 0 ;
        psMsg[KiConsoleMsgPrefixLength+_iMsgLengthMax-1] = KcConsoleSuffixOverflow ;
    }
	
    //look for any embedded <CR>; i.e. multi-line message
    pcCR = strrchr ( psMsg, cCR ) ;
    if ( pcCR != NULL ) 
    {//there was an embedded <CR>, 
     
        // trim the string by forcing an end of string where the <CR> was
        *pcCR = 0 ;

        // do recursive call with what's BEFORE the <CR>...
        // (which may still have more embedded <CR>'s)
        if ( WriteLine ( psMsg+KiConsoleMsgPrefixLength, psMsg, (pcCR-psMsg) ) != Ok ) eReturnStatus = Error ;

        //... and for what's AFTER the <CR> (which is now a simple line)
        // adjust psMsg to point to the end of the line
        // after copying the prefix to this last part of the line so that it is ready to print
        if ( psMsg[KiConsoleMsgPrefixLength-1] == KiConsolePrefixSeparator )  // be ready to copy the Prefix, but lower case
        {
            strncpy ( pcCR-KiConsoleMsgPrefixLength+1, psMsg, KiConsoleMsgPrefixLength-1 ) ; // copy the Prefix letter(s) without the '$'
            strlwr  ( pcCR-KiConsoleMsgPrefixLength+1 ) ; // lower case the prefix letter (remember the NULL is still there)
            *pcCR = KiConsolePrefixSeparator ; // replace the NULL with a '$'

            // we now have the tail of the string ready to be printed if we just adjust the pointer
            psMsg = pcCR-KiConsoleMsgPrefixLength+1 ;
        }
        else
            psMsg = pcCR+1 ;
    }
    
    // add end of line escape sequence \n = <CR>
    // (note that we had reserved space for this char (+KiConsoleMessageSpaceForCR) 
    // even in case of truncation)
    iStrLen = strlen ( psMsg ) ;
    strcpy ( psMsg+iStrLen, "\n" ) ;

#if defined ( NO_MS_CONSOLE )
//    printf ( psMsg ) ;  embedded percentage character would mess up printf output
    CResourceLocks::Acquire_ScreenIO ( ) ;
        std::cout << psMsg ;
    CResourceLocks::Release_ScreenIO ( ) ;
#else
    // write result to console
    if ( WriteConsole ( mg_hConsole,       	// handle to a console screen buffer 
                        psMsg, 	            // pointer to buffer to write from 
                        iStrLen+KiConsoleMessageSpaceForCR,  // number of characters to write 
                       &uiCharWritten,      // pointer to number of characters written
                        NULL ) 	            // reserved
            == FALSE ) eReturnStatus = Error ;
#endif


    // remove <CR> from end of string and null-terminate it 
    psMsg[iStrLen] = 0 ;

    // now also copy string to log file
    CLogFile::WriteLine ( psMsg, iStrLen ) ;
    

    // now try to copy the string to the remote console
    CopyToRemoteConsole ( psMsg ) ;

    // relese temporary message copy
    delete [] psBuffer ;

    return eReturnStatus ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
/*
ESuccess CConsoleAccess::WriteLine      ( CString*          const _oString  ) 
{
    return WriteLine ( _oString->GetBuffer( 0 ) ) ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess CConsoleAccess::WriteLongLine  ( const char*       const _sString,
                                          const char*       const _sPrefix ) 
{ // WriteLine now can do that too, but Multiline allows for a longer input string
    return WriteLine ( _sString, _sPrefix, KiConsoleMsgLengthLongest ) ; 
}
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
/*ESuccess CConsoleAccess::WriteLongLine  ( CString*          const _oString  ) 
{
    return WriteLongLine  ( _oString->GetBuffer( 0 ) ) ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess CConsoleAccess::ErrorLine      ( const char*       const _sString ) 
{
    ESuccess        eReturnStatus = Ok ;
    //\\//
    if ( SetRed     ()  != Ok ) eReturnStatus = Error ;
    if ( WriteLine  (_sString, KiConsolePrefixError )   
                        != Ok ) eReturnStatus = Error ;
    if ( SetWhite   ()  != Ok ) eReturnStatus = Error ;
    return eReturnStatus ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess CConsoleAccess::WarningLine    ( const char*       const _sString ) 
{
    ESuccess        eReturnStatus = Ok ;
    //\\//
    if ( SetBrightWhite ()  != Ok ) eReturnStatus = Error ;
    if ( WriteLine  (_sString, KiConsolePrefixWarning ) 
                            != Ok ) eReturnStatus = Error ;
    if ( SetWhite   ()      != Ok ) eReturnStatus = Error ;
    return eReturnStatus ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess CConsoleAccess::SystemLine     ( const char*       const _sString ) 
{
    ESuccess        eReturnStatus = Ok ;
    //\\//
    if ( SetPurple  ()  != Ok ) eReturnStatus = Error ;
    if ( WriteLine  (_sString, KiConsolePrefixSystem ) 
                        != Ok ) eReturnStatus = Error ;
    if ( SetWhite   ()  != Ok ) eReturnStatus = Error ;
    return eReturnStatus ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess CConsoleAccess::MessageLine    ( const char*       const _sString ) 
{
    ESuccess        eReturnStatus = Ok ;
    //\\//
    if ( SetYellow  ()  != Ok ) eReturnStatus = Error ;
    if ( WriteLongLine   ( _sString, KiConsolePrefixMessage ) 
                        != Ok ) eReturnStatus = Error ;
    if ( SetWhite   ()  != Ok ) eReturnStatus = Error ;
    return eReturnStatus ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess CConsoleAccess::InternalErrorLine ( const char*    const _sString ) 
{
    ESuccess        eReturnStatus = Ok ;
    //\\//
    if ( SetBlue    ()  != Ok ) eReturnStatus = Error ;
    if ( WriteLine  (_sString, KiConsolePrefixBug )     
                        != Ok ) eReturnStatus = Error ;
    if ( SetWhite   ()  != Ok ) eReturnStatus = Error ;
    return eReturnStatus ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
    ESuccess  CConsoleAccess::SetRed ( ) 
{
    ESuccess    eReturnStatus = Ok ;
    //\\//
#if defined ( NO_MS_CONSOLE )
#else
    if ( SetConsoleTextAttribute ( mg_hConsole, 
                                   FOREGROUND_RED | FOREGROUND_INTENSITY ) 
          == FALSE ) eReturnStatus = Error ;
#endif

    return eReturnStatus ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
    ESuccess  CConsoleAccess::SetBlue ( ) 
{
    ESuccess    eReturnStatus = Ok ;
    //\\//
#if defined ( NO_MS_CONSOLE )
#else
    if ( SetConsoleTextAttribute ( mg_hConsole, 
                                   FOREGROUND_BLUE | FOREGROUND_INTENSITY ) 
          == FALSE ) eReturnStatus = Error ;
#endif

    return eReturnStatus ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
    ESuccess  CConsoleAccess::SetAqua( ) 
{
    ESuccess    eReturnStatus = Ok ;
    //\\//
#if defined ( NO_MS_CONSOLE )
#else
    if ( SetConsoleTextAttribute ( mg_hConsole, 
                                   FOREGROUND_GREEN | FOREGROUND_BLUE | FOREGROUND_INTENSITY ) 
          == FALSE ) eReturnStatus = Error ;
#endif

    return eReturnStatus ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
    ESuccess  CConsoleAccess::SetPurple ( ) 
{
    ESuccess    eReturnStatus = Ok ;
    //\\//
#if defined ( NO_MS_CONSOLE )
#else
    if ( SetConsoleTextAttribute ( mg_hConsole, 
                                   FOREGROUND_RED | FOREGROUND_BLUE | FOREGROUND_INTENSITY ) 
          == FALSE ) eReturnStatus = Error ;
#endif

    return eReturnStatus ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
    ESuccess  CConsoleAccess::SetYellow ( ) 
{
    ESuccess    eReturnStatus = Ok ;
    //\\//
#if defined ( NO_MS_CONSOLE )
#else
    if ( SetConsoleTextAttribute ( mg_hConsole, 
                                   FOREGROUND_RED | FOREGROUND_GREEN | FOREGROUND_INTENSITY ) 
          == FALSE ) eReturnStatus = Error ;
#endif

    return eReturnStatus ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
    ESuccess  CConsoleAccess::SetWhite ( ) 
{
    ESuccess    eReturnStatus = Ok ;
    //\\//
#if defined ( NO_MS_CONSOLE )
#else
    if ( SetConsoleTextAttribute ( mg_hConsole, 
                                   FOREGROUND_RED | FOREGROUND_GREEN | FOREGROUND_BLUE ) 
          == FALSE ) eReturnStatus = Error ;
#endif

    return eReturnStatus ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
    ESuccess  CConsoleAccess::SetBrightWhite ( ) 
{
    ESuccess    eReturnStatus = Ok ;
    //\\//
#if defined ( NO_MS_CONSOLE )
#else
    if ( SetConsoleTextAttribute ( mg_hConsole, 
                                   FOREGROUND_RED | FOREGROUND_GREEN | FOREGROUND_BLUE | FOREGROUND_INTENSITY ) 
          == FALSE ) eReturnStatus = Error ;
#endif

    return eReturnStatus ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
#if defined ( NO_MS_CONSOLE )
#else
HANDLE    CConsoleAccess::GetHandle ( ) 
{
    return mg_hConsole ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
HWND CConsoleAccess::GetConsoleHwnd(void) 
{ // cf. Q 124103 

    #define MY_BUFSIZE 132 // buffer size for console window titles
    HWND hwndFound;         // this is what is returned to the caller
    char pszNewWindowTitle[MY_BUFSIZE] ="FindWindow Target"; // contains fabricated WindowTitle
    char pszOldWindowTitle[MY_BUFSIZE]; // contains original WindowTitle

    // fetch current window title
    GetConsoleTitle(pszOldWindowTitle, MY_BUFSIZE);

    // change current window title
    SetConsoleTitle(pszNewWindowTitle);

    // ensure window title has been updated
    Sleep(40);

    // look for NewWindowTitle
    hwndFound=FindWindow(NULL, pszNewWindowTitle);

    // restore original window title
    SetConsoleTitle(pszOldWindowTitle);

    return(hwndFound);
} 
#endif

