/*-------------------------------------------------------------------------------*/
  /*
     HandleCoorConnection.cpp
     28-Sep-99
  */
/*-------------------------------------------------------------------------------*/
#include "L1Cal_IIb_TCC/StdAfx.h"

#include "L1Cal_IIb_TCC/GlobalConstants.h"
#include "L1Cal_IIb_TCC/CustomDataTypes.h"

#include "L1Cal_IIb_TCC/HandleItcServer.h"
#include "L1Cal_IIb_TCC/HandleItcClient.h"

#include "L1Cal_IIb_TCC/ListOfIPPortNumbers.h"

#include                            "L1Cal_IIb_TCC/HandleCoorConnection.h"

#include "L1Cal_IIb_TCC/MessageConsumer.h"

#include "L1Cal_IIb_TCC/UtilsStrings.h"
#include "L1Cal_IIb_TCC/UtilsParsing.h"
#include "L1Cal_IIb_TCC/HandleConsole.h"

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/

// global variable to keep track of incrementing Message Reply ID
unsigned long   g_iMsgCount = 0 ;

// global variable to keep track of Remote Console server thread
ItcServer*      gpCoorToFw_ServerThread = NULL ;

// global variable for COOR Command Parser/Dispatcher/Executer Object
CMessageConsumer*      gpConsumer = NULL ;

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess  InitCoorConnectionServer ( ) 
{
    ESuccess    eReturnStatus = Ok ;

    CConsoleAccess::WriteLine ( "Creating COOR Connection Server" ) ;

    // check that the ITC Server was created with enough room to handle COOR messages
    if ( KiMaxLengthFullReply > KiMaxLengthStringOut )
    CConsoleAccess::InternalErrorLine ( "Pre-Processor Constants <KiMaxLengthStringOut> need adjusting" ) ;
    
    // create a Coor Message Parser/Dispatcher/Executer object

    gpConsumer        =   new ( CMessageConsumer ) ;

    //
    // Create a server Thread, and accept connections
    //

    gpCoorToFw_ServerThread = new ItcServer ( (unsigned short) KiIP_Port_CoorToTcc, 
                                              &Coor_Message_Interface );
    gpCoorToFw_ServerThread->start ( );

    return eReturnStatus ;
} ;

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess  ShutdownCoorConnectionServer ( )
{
    ESuccess    eReturnStatus = Ok ;

    CConsoleAccess::WriteLine ( (CString) "Shutting Down COOR Connection Server" ) ;    

    //
    // Delete the server Threads
    //

    delete gpCoorToFw_ServerThread ;

    delete gpConsumer ;

    return eReturnStatus ;
} ;

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/

ESuccess    Coor_Message_Interface   ( const char*          _acStringIn,
                                       char*                _acStringOut,
                                       EReply_Option*       _peReplyOption ) 
{

    ESuccess        eReturnStatus = Ok ;
    char*           pcCmdId ;
    unsigned int    uiCmdIdLength ;

    //\\//

    g_iMsgCount ++ ;

    CConsoleAccess::MessageLine ( (CString) "Receive Msg  #"
                               + I_to_A ( g_iMsgCount, 5, 10, ' ' )
                               + " : "
                               + _acStringIn ) ;
 
    pcCmdId         = FirstTokenBegin(_acStringIn) ;
    uiCmdIdLength   = FirstTokenLength(pcCmdId) ;

    if ( pcCmdId == NULL )
    { // blank string

        CConsoleAccess::ErrorLine ( (CString) "_No Command ID found in COOR Message" ) ;
        // don't call that an error as it is handled
        // by returning an explicit text status in reply status

        // Request sending a reply
        *_peReplyOption = Send_Reply ;

        // use fake Command ID to return message
        strcpy ( _acStringOut, "000000" ) ; 
    
        // Add reply status
        strcat ( _acStringOut, " Bad" ) ;

        // Describe the problem in reply message 
        strncat ( _acStringOut, " <No Command ID found in COOR Message>", KiMaxLengthFullReply ) ;
    }
    else
    {
        // Transfer Command ID to output message
        // (Here we rely on the fact that strncpy will not put a null character 
        // if the source string is longer than the number of characters being transferred)
        strncpy ( _acStringOut, pcCmdId, uiCmdIdLength ) ; 
        _acStringOut[uiCmdIdLength] = ' ' ; // add a space character after the command ID

        // Now look inside the message
        ECoorMessageStatus  eReplyStatus ;

        eReplyStatus = gpConsumer->ConsumeMessage 
                  ( pcCmdId+uiCmdIdLength,                                     //beginning of the actual command 
                   &_acStringOut[uiCmdIdLength +1 +KiMaxLengthReplyStatus +1], //where we can write the reply
                    KiMaxLengthReplyTextToCoor ) ;                                   //how much room is available for reply

        // Request sending a reply
        *_peReplyOption = Send_Reply ;

        switch ( eReplyStatus )
        {// here we rely on the fact that strncpy will not put a null character 
         // if the source string is longer than the number of characters being transferred
            case Reply_Ok:      strncpy ( &_acStringOut[uiCmdIdLength +1], "Ok     ", KiMaxLengthReplyStatus +1 ) ; break ;
            case Reply_Bad:     strncpy ( &_acStringOut[uiCmdIdLength +1], "Bad    ", KiMaxLengthReplyStatus +1 ) ; break ;
            case Reply_None:    *_peReplyOption = No_Reply_Needed ;  break ;
        }
    }

    return eReturnStatus ;
}


/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess  InitCoorConnectionClient ( ) 
{
    ESuccess    eReturnStatus = Ok ;

    CConsoleAccess::WriteLine ( (CString) "Creating COOR Connection Client" ) ;

    ItcClient::Initialize_FakeCoorToSelf ( (unsigned short) KiIP_Port_CoorToTcc ) ;

    return eReturnStatus ;
}


/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess  ShutdownCoorConnectionClient ( ) 
{
    ESuccess    eReturnStatus = Ok ;

    if ( ItcClient::m_poConnection_as_Coor_to_Self != NULL ) 
    {
        CConsoleAccess::WriteLine ( (CString) "Deleting COOR Connection Client" ) ;

        ItcClient::Delete_FakeCoorToSelf ( ) ;
    }

    return eReturnStatus ;
}
