/*-------------------------------------------------------------------------------*/
  /*
     HandleGuiConnection.cpp
     2-Nov-04
  */
/*-------------------------------------------------------------------------------*/
#include "L1Cal_IIb_TCC/StdAfx.h"

#include "L1Cal_IIb_TCC/GlobalConstants.h"
#include "L1Cal_IIb_TCC/CustomDataTypes.h"

#include "L1Cal_IIb_TCC/HandleItcServer.h"
#include "L1Cal_IIb_TCC/HandleItcClient.h"

#include "L1Cal_IIb_TCC/ListOfIPPortNumbers.h"

#include                            "L1Cal_IIb_TCC/HandleGuiConnection.h"

#include "L1Cal_IIb_TCC/GuiCommandConsumer.h"

#include "L1Cal_IIb_TCC/UtilsStrings.h"
#include "L1Cal_IIb_TCC/UtilsParsing.h"
#include "L1Cal_IIb_TCC/HandleConsole.h"

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/

// global variable to keep track of incrementing Message Reply ID
unsigned long   g_iGuiCommand = 0 ;

// global variable to keep track of Gui Command Server thread
ItcServer*      gpGuiCommand_ServerThread = NULL ;

// global variable for GUI Command Parser/Dispatcher/Executer Object
CGuiCommandConsumer*      gpGuiCommandConsumer = NULL ;

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess  InitGuiCommandServer ( ) 
{
    ESuccess    eReturnStatus = Ok ;

    CConsoleAccess::WriteLine ( "Creating GUI Command Connection Server" ) ;

    // check that the ITC Server was created with enough room to handle Gui commands
    if ( KiMaxLengthFullGuiReply > KiMaxLengthStringOut )
    CConsoleAccess::InternalErrorLine ( "Pre-Processor Constants <KiMaxLengthStringOut> need adjusting" ) ;
    
    // create a Gui COmmand Parser/Dispatcher/Executer object

    gpGuiCommandConsumer        =   new ( CGuiCommandConsumer ) ;

    //
    // Create a server Thread, and accept connections
    //

    gpGuiCommand_ServerThread = new ItcServer ( (unsigned short) KiIP_Port_GuiCommand, 
                                                &Gui_Command_Interface );
    gpGuiCommand_ServerThread->start ( );

    return eReturnStatus ;
} ;

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess  ShutdownGuiCommandServer ( )
{
    ESuccess    eReturnStatus = Ok ;

    CConsoleAccess::WriteLine ( (CString) "Shutting Down GUI Command Connection Server" ) ;    

    //
    // Delete the server Threads
    //

    delete gpGuiCommand_ServerThread ;

    delete gpGuiCommandConsumer ;

    return eReturnStatus ;
} ;

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/

ESuccess    Gui_Command_Interface    ( const char*          _acStringIn,
                                       char*                _acStringOut,
                                       EReply_Option*       _peReplyOption ) 
{

    ESuccess        eReturnStatus = Ok ;
    char*           pcCmdId ;
    unsigned int    uiCmdIdLength ;

    //\\//

    g_iGuiCommand ++ ;

    CConsoleAccess::MessageLine ( (CString) "Recv Gui Cmd #"
                               + I_to_A ( g_iGuiCommand, 5, 10, ' ' )
//                               + " : "
//                               + _acStringIn 
                               ) ;
 
    pcCmdId         = FirstTokenBegin(_acStringIn) ;
    uiCmdIdLength   = FirstTokenLength(pcCmdId) ;

    if ( pcCmdId == NULL )
    { // blank string

        CConsoleAccess::ErrorLine ( (CString) "_No Command ID found in Gui Command Message" ) ;
        // don't call that an error as it is handled
        // by returning an explicit text status in reply status

        // Request sending a reply
        *_peReplyOption = Send_Reply ;

        // use fake Command ID to return message
        strcpy ( _acStringOut, "000000" ) ; 
    
        // Add reply status
        strcat ( _acStringOut, " Bad" ) ;

        // Describe the problem in reply message 
        strncat ( _acStringOut, " <No Command ID found in Gui Command Message>", KiMaxLengthFullGuiReply ) ;
    }
    else
    {
        // Transfer Command ID to output message
        // (Here we rely on the fact that strncpy will not put a null character 
        // if the source string is longer than the number of characters being transferred)
        strncpy ( _acStringOut, pcCmdId, uiCmdIdLength ) ; 
        _acStringOut[uiCmdIdLength] = ' ' ; // add a space character after the command ID

        // Now look inside the message
        EGuiCommandStatus  eReplyStatus ;

        eReplyStatus = gpGuiCommandConsumer->ConsumeGuiCommand
                  ( pcCmdId+uiCmdIdLength,                                     //beginning of the actual command 
                   &_acStringOut[uiCmdIdLength +1 +KiMaxLengthGuiReplyStatus +1], //where we can write the reply
                    KiMaxLengthReplyTextToGui ) ;                                   //how much room is available for reply

        // Request sending a reply
        *_peReplyOption = Send_Reply ;

        switch ( eReplyStatus )
        {// here we rely on the fact that strncpy will not put a null character 
         // if the source string is longer than the number of characters being transferred
            case GuiReply_Ok:      strncpy ( &_acStringOut[uiCmdIdLength +1], "Ok     ", KiMaxLengthGuiReplyStatus +1 ) ; break ;
            case GuiReply_Bad:     strncpy ( &_acStringOut[uiCmdIdLength +1], "Bad    ", KiMaxLengthGuiReplyStatus +1 ) ; break ;
            case GuiReply_None:    *_peReplyOption = No_Reply_Needed ;  break ;
        }
    }

    return eReturnStatus ;
}

