/*-------------------------------------------------------------------------------*/
  /*
    HardwareAccess.cpp
    19-Nov-2004
  */
/*-------------------------------------------------------------------------------*/
#include "L1Cal_IIb_TCC/StdAfx.h"

#include "L1Cal_IIb_TCC/GlobalConstants.h"
#include "L1Cal_IIb_TCC/CustomDataTypes.h"

#include "L1Cal_IIb_TCC/UtilsStrings.h"

#include                            "L1Cal_IIb_TCC/HandleHardwareAccess.h"

#include "L1Cal_IIb_TCC/AdfRegBase.h"

#include "L1Cal_IIb_TCC/HandleConsole.h"

/*-------------------------------------------------------------------------------*/

// Tab/Gab software

#include "tabgab/util.h"
#include "tabgab/vmescl.h"

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
// Static data members must be initialized at file scope

//pointer to the Adaptor object that will handle L1 Framework Access
#if defined VME_IO_DUAL_BIT3
CBit3Adaptor61x*    CHardwareAccess::mg_poBit3BusAdaptorArray[2] = { NULL, NULL } ;
#endif
CBit3Adaptor61x*    CHardwareAccess::mg_poBit3BusAdaptor_Adf = NULL ;
CBit3Adaptor61x*    CHardwareAccess::mg_poBit3BusAdaptor     = NULL ;
CBit3Adaptor61x*    CHardwareAccess::mg_poBit3BusAdaptor_Tab = NULL ;

// This is an array of pointers to the process' virtual address 
// mapped to the base address of each ADF card. 
TuwRegWord*     CHardwareAccess::mg_apuwAdfCardBase[KiL1CalTotVertMaster][KiL1CalTotVertSlave][KiL1CalTotSlot] ;

// This is an array of pointers in the process' virtual address space
// mapped to the base address of each TAB/GAB card. 
unsigned long*  CHardwareAccess::mg_apuwTabCardBase[KiL1CalTotTabModule] ;

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
CHardwareAccess::CHardwareAccess   (  ) 
{
    //\\//

    // the following statement is there to allow a debugger breakpoint to change the default value
    // (since this is only re-writing the default value)
    CBit3Adaptor61x::mg_iAdaptorNumForL1Cal = KiAdaptorNumForL1Cal ;

    ////////////////////////////////////////////////////////////////
    // Create Adaptor to A32 space for Vertical Interconnect to Adf
    ////////////////////////////////////////////////////////////////

#if defined VME_IO_NO_VERTICAL  // test stand run without the vertical interconnect
    #if defined VME_IO_DUAL_BIT3
        // We have one Bit3 adaptor for each crate
        mg_poBit3BusAdaptorArray[0] = new CBit3Adaptor61x ( 0, VmeA24, VmeAmodNonPriv  ) ;
        mg_poBit3BusAdaptorArray[1] = new CBit3Adaptor61x ( 1, VmeA24, VmeAmodNonPriv  ) ;
        mg_poBit3BusAdaptor_Adf = mg_poBit3BusAdaptorArray[0] ;
    #else
        mg_poBit3BusAdaptor_Adf = new CBit3Adaptor61x ( CBit3Adaptor61x::mg_iAdaptorNumForL1Cal, VmeA24, VmeAmodNonPriv  ) ;
    #endif
#else
    mg_poBit3BusAdaptor_Adf = new CBit3Adaptor61x ( CBit3Adaptor61x::mg_iAdaptorNumForL1Cal, VmeA32  ) ;
#endif


    // for backwards compatibility, same as the xxxx_Adf above
    mg_poBit3BusAdaptor     = mg_poBit3BusAdaptor_Adf ;

    // initialize/reset the array of pointers   
    // to the base addresses of all possible cards
    for ( int ulVertMasterNum = 0 ; ulVertMasterNum < KiL1CalTotVertMaster ; ulVertMasterNum++ )
    for ( int ulVertChanNum   = 0 ; ulVertChanNum   < KiL1CalTotVertSlave  ; ulVertChanNum++ )
    for ( int ulSlotNum       = 0 ; ulSlotNum       < KiL1CalTotSlot       ; ulSlotNum++ )
      mg_apuwAdfCardBase[ulVertMasterNum][ulVertChanNum][ulSlotNum] = NULL ;
    

    ////////////////////////////////////////////////////////////////
    // Create Adaptor to A24 space for VME SCL space
    ////////////////////////////////////////////////////////////////

#if defined VME_IO_NO_VERTICAL  // using this A24 address modified would break access to ADF cards
#else
    mg_poBit3BusAdaptor_Tab = new CBit3Adaptor61x ( CBit3Adaptor61x::mg_iAdaptorNumForL1Cal, VmeA24, VmeAmodNevis ) ;
#endif

    // initialize/reset the array of pointers   
    // to the base addresses of all possible Tab/Gab/etc modules
    for ( int ulModuleNum = 0       ; ulModuleNum < KiL1CalTotTabModule      ; ulModuleNum ++ )
      mg_apuwTabCardBase[ulModuleNum] = NULL ;
   
}


/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
CHardwareAccess::~CHardwareAccess ( ) 
{

    delete mg_poBit3BusAdaptor_Adf ;
    delete mg_poBit3BusAdaptor_Tab ;
    mg_poBit3BusAdaptor_Adf = NULL ;
    mg_poBit3BusAdaptor     = NULL ;
    mg_poBit3BusAdaptor_Tab = NULL ;

}


/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
TuwRegWord*   CHardwareAccess::MemoryMapThisCard ( unsigned char   _ubVertMaster,
                                                    unsigned char   _ubVertSlave,
                                                    unsigned char   _ubCardSlot ) 
{            

    //\\//

#if defined VME_IO_DUAL_BIT3
    // use the upper byte for crate number
    // anything with slave or master non zero is crate 1
    // and crate one is encoded in the upper byte, i.e. VI Master 0, Slave 1
    if ( ( _ubVertSlave  != 0 ) 
      || ( _ubVertMaster != 0 ) )
    {
	    _ubVertSlave  = 1 ;
	    _ubVertMaster = 0 ;
    }

    if ( mg_apuwAdfCardBase[_ubVertMaster] [_ubVertSlave] [_ubCardSlot] == NULL ) 
    {

        TulVmeAddress      ulBaseAddress ; 

        ulBaseAddress = CAdfRegBase::ConvertToVmeAddress ( _ubVertMaster,
                                                        _ubVertSlave,
                                                        _ubCardSlot ) ; 

        mg_apuwAdfCardBase[_ubVertMaster] 
                      [_ubVertSlave] 
                      [_ubCardSlot] 
          = (TuwRegWord*) mg_poBit3BusAdaptorArray[_ubVertSlave]->MemoryMapThisVmeSegment ( ulBaseAddress & 0x00FFFFFF,
                                                                         KiAdf2CardAddressSize ) ;

#else

    if ( mg_apuwAdfCardBase[_ubVertMaster] [_ubVertSlave] [_ubCardSlot] == NULL ) 
    {

        TulVmeAddress      ulBaseAddress ; 

        ulBaseAddress = CAdfRegBase::ConvertToVmeAddress ( _ubVertMaster,
                                                        _ubVertSlave,
                                                        _ubCardSlot ) ; 

        mg_apuwAdfCardBase[_ubVertMaster] 
                      [_ubVertSlave] 
                      [_ubCardSlot] 
          = (TuwRegWord*) mg_poBit3BusAdaptor->MemoryMapThisVmeSegment ( ulBaseAddress,
                                                                         KiAdf2CardAddressSize ) ;

#endif 
    }

    return mg_apuwAdfCardBase[_ubVertMaster]
                         [_ubVertSlave]
                         [_ubCardSlot] ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
unsigned long*   CHardwareAccess::MemoryMapThisModule ( unsigned char   _ubModuleNum ) 
{            

    //\\//

    if ( mg_apuwTabCardBase[_ubModuleNum] == NULL ) 
    {

        TulVmeAddress      ulBaseAddress ; 

        ulBaseAddress = 0x00ffffff & vmescl_addr_combine ( KiVmeSclSlot, _ubModuleNum, 0, 0) ;

        CConsoleAccess::WriteLine ( (CString) "Mapping Tab/Gab Module#" 
                                             + I_to_A (_ubModuleNum)
                                             + " Base Address " 
                                             + I_to_A ( ulBaseAddress, 8, 16, '0' ) ) ;

        mg_apuwTabCardBase[_ubModuleNum] 
          = (unsigned long*) mg_poBit3BusAdaptor_Tab->MemoryMapThisVmeSegment ( ulBaseAddress,
                                                                                KiTabCardAddressSize ) ;
    }

    return mg_apuwTabCardBase [_ubModuleNum] ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess    CHardwareAccess::ReadVmeReg  ( TulVmeAddress       _ulVmeAddress, 
                                           TuwRegWord* const   _kpuwReadData ) 

{  return mg_poBit3BusAdaptor->ReadVmeData ( _ulVmeAddress, _kpuwReadData, sizeof(TuwRegWord) ) ; }

/*-------------------------------------------------------------------------------*/
ESuccess    CHardwareAccess::WriteVmeReg ( TulVmeAddress       _ulVmeAddress, 
                                           TuwRegWord* const   _kpuwWriteData ) 

{   return mg_poBit3BusAdaptor->WriteVmeData ( _ulVmeAddress, _kpuwWriteData, sizeof(TuwRegWord) ) ; }

/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess    CHardwareAccess::ReadTabGab       ( unsigned long      _ulModuleNum, 
                                                unsigned long      _ulChipNum, 
                                                unsigned long      _ulGroupNum,
                                                unsigned long*     _pulValue )
{
    ESuccess        eReturnStatus = Ok ;
    unsigned long   ulVmeAddress ;

    //\\//

    // check the parameters
    ulVmeAddress = vmescl_addr_combine( KiVmeSclSlot, _ulModuleNum, _ulChipNum, _ulGroupNum ) ;

    if ( ulVmeAddress == ILLEGAL_ADDRESS ) 
    {
        CConsoleAccess::InternalErrorLine ( (CString) "Internal Error/vme_read" ) ;
        eReturnStatus = Error ;
    }
    else
    {
        #if ( defined ( VME_IO_BIT3_MMAP ) || defined ( VME_IO_FAKE ) ) // Memory Mapped Data Access , or local memory

            if ( mg_apuwTabCardBase[_ulModuleNum] == NULL ) MemoryMapThisModule ( (char) _ulModuleNum ) ;
        
            *_pulValue = mg_apuwTabCardBase[_ulModuleNum][vmescl_addr_offset( _ulChipNum, _ulGroupNum )] ;

        #elif defined ( VME_IO_BIT3_API ) // Explicit call to the Bit3 API routines

            if ( mg_poBit3BusAdaptor_Tab->ReadVmeData ( ulVmeAddress, _pulValue, sizeof(long) ) != Ok )
                eReturnStatus = Error ;

        #endif
    }

    return eReturnStatus ;
}
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess    CHardwareAccess::WriteTabGab      ( unsigned long      _ulModuleNum, 
                                                unsigned long      _ulChipNum, 
                                                unsigned long      _ulGroupNum,
                                                unsigned long*     _pulValue )
{
    ESuccess        eReturnStatus = Ok ;
    unsigned long   ulVmeAddress ;

    //\\//

    // check the parameters
    ulVmeAddress = vmescl_addr_combine( KiVmeSclSlot, _ulModuleNum, _ulChipNum, _ulGroupNum ) ;

    if ( ulVmeAddress == ILLEGAL_ADDRESS ) 
    {
        CConsoleAccess::InternalErrorLine ( (CString) "Internal Error/vme_write" ) ;
        eReturnStatus = Error ;
    }
    else
    {
        #if ( defined ( VME_IO_BIT3_MMAP ) || defined ( VME_IO_FAKE ) ) // Memory Mapped Data Access , or local memory

            if ( mg_apuwTabCardBase[_ulModuleNum] == NULL ) MemoryMapThisModule ( (char) _ulModuleNum ) ;
        
            mg_apuwTabCardBase[_ulModuleNum][vmescl_addr_offset( _ulChipNum, _ulGroupNum )] = *_pulValue ;

        #elif defined ( VME_IO_BIT3_API ) // Explicit call to the Bit3 API routines

            if ( mg_poBit3BusAdaptor_Tab->WriteVmeData ( ulVmeAddress, _pulValue, sizeof(long) ) != Ok )
                eReturnStatus = Error ;

        #endif
    }

    return eReturnStatus ;
}
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
int vme_read            (unsigned long      _ulModuleNum, 
                         unsigned long      _ulChipNum, 
                         unsigned long      _ulGroupNum,
                         unsigned long *    _pulValue )
{
    int             iReturnValue = EXIT_SUCCESS ;

    //\\//

    if ( CHardwareAccess::ReadTabGab  ( _ulModuleNum, 
                                        _ulChipNum, 
                                        _ulGroupNum,
                                        _pulValue ) != Ok ) iReturnValue = EXIT_FAILURE ;

    return iReturnValue ;
}
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
int vme_write           (unsigned long      _ulModuleNum, 
                         unsigned long      _ulChipNum, 
                         unsigned long      _ulGroupNum,
                         unsigned long      _ulValue )
{
    int             iReturnValue = EXIT_SUCCESS ;

    //\\//

    if ( CHardwareAccess::WriteTabGab ( _ulModuleNum, 
                                        _ulChipNum, 
                                        _ulGroupNum,
                                       &_ulValue ) != Ok ) iReturnValue = EXIT_FAILURE ;

    return iReturnValue ;
}
/*-------------------------------------------------------------------------------*/
