/*-------------------------------------------------------------------------------*/
  /*
     HandleItcClient.cpp
     17-Nov-99
  */
/*-------------------------------------------------------------------------------*/
#include "L1Cal_IIb_TCC/StdAfx.h"

#include "L1Cal_IIb_TCC/GlobalConstants.h"
#include "L1Cal_IIb_TCC/CustomDataTypes.h"

#include                            "L1Cal_IIb_TCC/HandleItcClient.h"

#include "L1Cal_IIb_TCC/UtilsStrings.h"
#include "L1Cal_IIb_TCC/HandleConsole.h"

/*-------------------------------------------------------------------------------*/
// Set following compile constant to "1" to print screen messages at every
// message sent or received
#define DEBUG_CONNECTIONS   0

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
// Static data members must be initialized at file scope
ItcClient*      ItcClient::m_poConnection_as_Coor_to_Self  = NULL ;
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ItcClient::ItcClient  ( unsigned short  _uwPort_Num,
                        const char*     _kpsHost )
{
#define KiMaxHostNameLength  80

    char sHost[KiMaxHostNameLength+1] = "localhost" ;

    //\\//

    if ( _kpsHost != NULL )
    {
        strncpy ( sHost, _kpsHost, KiMaxHostNameLength ) ;
        //force an end of string in case of garbage input
        sHost[KiMaxHostNameLength] = 0 ;
    }

    //remember the host and port number
    m_osHostName = sHost ;
    m_uwIP_Port_Num = _uwPort_Num ;

    // create the connection object
    m_pClient = new itc::SOCK_Auto_Connector ( m_uwIP_Port_Num, sHost, 0.5 ) ;

    // Create connector (client ) with the automatic reconnect
    m_pClient->set_send_buffer(130000);

    // register the function that will field the reply messages
#ifdef WIN32
    // normal approach using register_callback didn't compile on MS VC++, with error C2784
    // look in template code to replace with explicit code
//    template<class T,class Tc, typename type>
//    inline void register_callback( Processor& p, int ID, type (T::*fun)(const IPC_Message &), Tc& obj )
//    p.register_cb(ID,Call_Mem_Fun_ID<T,Tc,type>(fun,obj) );

    m_pClient->register_cb( itc::IPC_Message::MSG_ID,
                            itc::Call_Mem_Fun_ID<ItcClient,ItcClient,void>(ItcClient::Message_Consumer,*this) ) ;

#else

    itc::register_callback ( *m_pClient, itc::IPC_Message::MSG_ID, &ItcClient::Message_Consumer, *this ) ;

#endif

    // don't connect right away, wait for first usage
    m_ynConnected   = No ;

    m_osReply = "" ;

    return ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ItcClient::~ItcClient    ( )
{
    if ( m_pClient != NULL )
    {
        CConsoleAccess::WriteLine ( (CString) "Deleting ITC Client Threads"
                                            + " for port #"
                                            + I_to_A ( m_uwIP_Port_Num ) ) ;

        //un-register callback function
#ifdef WIN32
        m_pClient->unregister_cb( itc::IPC_Message::MSG_ID,
                                  itc::Call_Mem_Fun_ID<ItcClient,ItcClient,void>(ItcClient::Message_Consumer,*this) ) ;
#else
        itc::unregister_callback ( *m_pClient, itc::IPC_Message::MSG_ID, &ItcClient::Message_Consumer, *this ) ;
#endif

        // stop Client
        m_pClient->stop() ;
        Sleep ( 1000 ) ; /* wait 1 s */

        //delete client object
        delete  m_pClient ;
    }

    return ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess ItcClient::Connect    ( )
{

    ESuccess    eReturnStatus = Ok ;

    //\\//

    // Not declared connected until we get a reply on the first message
    m_ynConnected = No ;

    CConsoleAccess::WriteLine ( (CString) "Starting Threads for Client to "
                                         + m_osHostName
                                         + " Port #"
                                         + I_to_A ( m_uwIP_Port_Num ) ) ;

    // start the connection
    m_pClient->start_sender( 10, 30.0 ) ;
    m_pClient->start_recver( 30.0 ) ;


    Sleep ( 500 ) ; // wait .5 Second

    return eReturnStatus ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess ItcClient::SendMessage ( const char*       const _sString )
{
    ESuccess            eReturnStatus = Ok ;
    itc::IPC_STATUS     ret ;

    //\\//

    if ( m_ynConnected == No )
    {
        if ( Connect () != Ok )
        {
            eReturnStatus = Error ;
            goto Done ;
        }
    }

    if ( DEBUG_CONNECTIONS )
    CConsoleAccess::WriteLine ( (CString) "Sending to "
                                         + m_osHostName
                                         + " Port #"
                                + I_to_A ( m_uwIP_Port_Num )
                                + " : " + _sString ) ;

    // send a message
    ret = m_pClient->send_now( itc::String_Message ( _sString ), 5.0 ) ;

    if ( ret != itc::IPC_SUCCESS )
    {
        m_ynConnected = No ;
        eReturnStatus = Error ;
        if (ret == itc::IPC_TIMEOUT )
        {
            CConsoleAccess::ErrorLine ( (CString) "Error: Timeout for Send to "
                                             + m_osHostName
                                             + " Port #"
                                             + I_to_A ( m_uwIP_Port_Num ) ) ;
        }
        else if (ret == itc::IPC_STOPPED )
        {
            CConsoleAccess::ErrorLine ( (CString) "Error: Connection is STOPPED to "
                                             + m_osHostName
                                             + " Port #"
                                             + I_to_A ( m_uwIP_Port_Num ) ) ;
        }
        else if (ret == itc::IPC_BROKEN )
        {
            CConsoleAccess::ErrorLine ( (CString) "Error: Connection is BROKEN to "
                                             + m_osHostName
                                             + " Port #"
                                             + I_to_A ( m_uwIP_Port_Num ) ) ;
        }
        else
        {
            CConsoleAccess::ErrorLine ( (CString) "Error Sending Message to "
                                             + m_osHostName
                                             + " Port #"
                                             + I_to_A ( m_uwIP_Port_Num ) ) ;
        }
    }

Done:

    Sleep ( 100 ) ; // wait .1 Second

    return eReturnStatus ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess ItcClient::ClearReply ( )
{
    ESuccess    eReturnStatus = Ok ;

    //\\//

    // clear the last reply
    m_osReply = "" ;

    return eReturnStatus ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess ItcClient::LastReply ( CString*        _posString )
{
    ESuccess    eReturnStatus = Ok ;

    //\\//

    // retrieve the last reply
    if ( m_osReply.GetLength() != 0 )
        *_posString = m_osReply ;
    else
        eReturnStatus = Error ;

    return eReturnStatus ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess ItcClient::HasReplied ( )
{
    ESuccess    eReturnStatus = Ok ;

    //\\//

    // check presence of new reply
    if ( m_osReply.GetLength() == 0 )
        eReturnStatus = Error ;

    return eReturnStatus ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess ItcClient::Initialize_FakeCoorToSelf ( unsigned short  _uwPort_Num )
{
    ESuccess    eReturnStatus = Ok ;

    //\\//

    if ( m_poConnection_as_Coor_to_Self == NULL )
         m_poConnection_as_Coor_to_Self = new ItcClient ( _uwPort_Num ) ;

    else if ( m_poConnection_as_Coor_to_Self->IsNotInitialized() )
         m_poConnection_as_Coor_to_Self = new ItcClient ( _uwPort_Num ) ;

    return eReturnStatus ;
}
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess ItcClient::Delete_FakeCoorToSelf ( )
{
    ESuccess    eReturnStatus = Ok ;

    //\\//

    if ( m_poConnection_as_Coor_to_Self != NULL )
    {
         delete m_poConnection_as_Coor_to_Self ;
         m_poConnection_as_Coor_to_Self = NULL ;
    }

    return eReturnStatus ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
void ItcClient::Message_Consumer ( const itc::IPC_Message &_ItcMsgIn )
{
    itc::String_Message         oItcMsgIn = itc::String_Message ( _ItcMsgIn ) ;
    itc::Connection_Handle      This_Connection ;

    //\\//

    switch ( _ItcMsgIn.ID() )
    {

      case itc::String_Message::MSG_ID :

        m_ynConnected = Yes ;

        // store the reply
        m_osReply = oItcMsgIn.str() ;

        if ( DEBUG_CONNECTIONS )
        CConsoleAccess::WriteLine ( (CString) "Reply from "
                                            + m_osHostName
                                            + " Port #"
                                            + I_to_A ( m_uwIP_Port_Num )
                                            + " : " + m_osReply ) ;

        break;

      case itc::Conn_Start_Msg::MSG_ID :
        m_ynConnected = Yes ;
        CConsoleAccess::WriteLine ( (CString) "Itc Client Connection to "
                                            + m_osHostName
                                            + " Port #"
                                            + I_to_A ( m_uwIP_Port_Num )
                                            + " received: connection START message" ) ;
        break;

      case itc::Conn_Stop_Msg::MSG_ID :
        CConsoleAccess::WriteLine ( (CString) "Itc Client Connection to "
                                            + m_osHostName
                                            + " Port #"
                                            + I_to_A ( m_uwIP_Port_Num )
                                            + " received: connection STOP message" ) ;
        break;

      case itc::Conn_Brkn_Msg::MSG_ID :
        CConsoleAccess::WriteLine ( (CString) "Itc Client Connection to "
                                            + m_osHostName
                                            + " Port #"
                                            + I_to_A ( m_uwIP_Port_Num )
                                            + " received: connection BROKEN message" ) ;
        break;

      case itc::Conn_Open_Msg::MSG_ID :
        CConsoleAccess::WriteLine ( (CString) "Itc Client Connection to "
                                            + m_osHostName
                                            + " Port #"
                                            + I_to_A ( m_uwIP_Port_Num )
                                            + " received: connection RECONNECTED message" ) ;
        break;

      default:

        char sBuffer[132] ;
        sprintf ( sBuffer,
                 "ID=%2d - No=%5u length %3u remote=%5u reply no=%5u",
                  _ItcMsgIn.ID(),
                  _ItcMsgIn.no(),
                  _ItcMsgIn.length(),
                  _ItcMsgIn.remote_no(),
                  _ItcMsgIn.reply_no() ) ;
        CConsoleAccess::WriteLine ( (CString) "Itc Client on "
                                            + m_osHostName
                                            + " Port #"
                                            + I_to_A ( m_uwIP_Port_Num )
                                            + " received: Message "
                                            + sBuffer ) ;
    }

    return ;
}
