/*-------------------------------------------------------------------------------*/
  /*
    HandleItcServer.cpp
     16-Jul-99
  */
/*-------------------------------------------------------------------------------*/
#include "L1Cal_IIb_TCC/StdAfx.h"

#include "L1Cal_IIb_TCC/GlobalConstants.h"
#include "L1Cal_IIb_TCC/CustomDataTypes.h"

#include                            "L1Cal_IIb_TCC/HandleItcServer.h"

#include "L1Cal_IIb_TCC/UtilsStrings.h"
#include "L1Cal_IIb_TCC/HandleConsole.h"

#include <string>

/*-------------------------------------------------------------------------------*/
// Set following compile constant to "1" to print screen messages at every 
// message sent or received
#define DEBUG_CONNECTIONS   0

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ItcServer::ItcServer  ( unsigned short                      _uwIP_Port_Num,
                        String_Message_Consumer_Interface*  _pfString_Message_Consumer ) 
{ 
    //remember the port number
    m_uwIP_Port_Num     = _uwIP_Port_Num ; 

    //remember the consumer interface
    m_pfString_Message_Consumer = _pfString_Message_Consumer ;
    m_pfNested_Message_Consumer = NULL  ;

    InitServer ( ) ;

} ;
    
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ItcServer::ItcServer  ( unsigned short                      _uwIP_Port_Num,
                        Nested_Message_Consumer_Interface*  _pfNested_Message_Consumer ) 
{ 
    //remember the port number
    m_uwIP_Port_Num     = _uwIP_Port_Num ; 

    //remember the consumer interface
    m_pfString_Message_Consumer = NULL ;
    m_pfNested_Message_Consumer = _pfNested_Message_Consumer ;

    InitServer ( ) ;

} ;
    
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
void ItcServer::InitServer ( ) 
{ 
    m_ynConnected       = No ; 
    m_pLast_Connection  = NULL ;

    // Set ITC debugging info 
    itc::info("L1Cal_IIb_TCC", KsVersion, std::string("L1Cal_IIb_Tcc") );
    itc::deb(0,std::cout,true,true);

    // Create a server
    m_pServer       = new itc::SOCK_Acceptor ( m_uwIP_Port_Num ) ;

    // set the socket buffer size for the receivers of all connections
    m_pServer->set_recv_buffer(130000) ; 

    // set the socket buffer size for the receivers of all connections
    m_pServer->set_processor(10,60.0);          // this is recommended
    m_pServer->set_recver(60.0);                // this a must to get messages
    m_pServer->set_sender(5,30.0);              // optional 

    // register the function that will field the received messages
#ifdef WIN32

    // normal approach using register_callback didn't compile on MS VC++, with error C2784
    // look in template code to replace with explicit code
//    template<class T,class Tc, typename type>
//    inline void register_callback( Processor& p, int ID, type (T::*fun)(const IPC_Message &), Tc& obj ) 
//    p.register_cb(ID,Call_Mem_Fun_ID<T,Tc,type>(fun,obj) );

    m_pServer->register_cb( itc::IPC_Message::MSG_ID, 
                            itc::Call_Mem_Fun_ID<ItcServer,ItcServer,void>(ItcServer::Message_Consumer,*this) ) ;

#else 

    itc::register_callback ( *m_pServer, itc::IPC_Message::MSG_ID, &ItcServer::Message_Consumer, *this ) ;

#endif


    return ;
} ;
    
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ItcServer::~ItcServer ( ) 
{
    
    CConsoleAccess::WriteLine ( (CString) "Deleting ITC Server Threads"
                                        + " for port #"  
                                        + I_to_A ( m_uwIP_Port_Num ) ) ;

    //un-register callback function
#ifdef WIN32
    m_pServer->unregister_cb( itc::IPC_Message::MSG_ID, 
                              itc::Call_Mem_Fun_ID<ItcServer,ItcServer,void>(ItcServer::Message_Consumer,*this) ) ;
#else 
    itc::unregister_callback ( *m_pServer, itc::IPC_Message::MSG_ID, &ItcServer::Message_Consumer, *this ) ;
#endif

    // stop server 
    m_pServer->stop() ;
    Sleep ( 1000 ) ; /* wait 1 s */ 

    //delete server object
    delete  m_pServer ;

} ;

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
void    ItcServer::start ()
{
    CConsoleAccess::WriteLine ( (CString) "Starting Threads for Server on local port #" 
                                         + I_to_A ( m_uwIP_Port_Num ) ) ;

    m_pServer->start(10.0) ;
}


/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess  ItcServer::SendMessage ( const char*       const _acString ) 
{
    ESuccess    eReturnStatus = Ok ;

    itc::String_Message  Msg ( _acString ) ; 

    //\\//

    if ( m_ynConnected == Yes ) 
    {
        if ( m_pLast_Connection->info().status == itc::Connection_Info::OPEN ) 
        {
            if ( DEBUG_CONNECTIONS )
            CConsoleAccess::WriteLine ( (CString) "Send on Server Port #"  
                                        + I_to_A ( m_uwIP_Port_Num )  
                                        + " : " + _acString ) ;
            m_pLast_Connection->send_now( Msg, 5.0 ) ;
        }
        else
        {
            m_ynConnected = No ;
            eReturnStatus = Error ;
        }
    }

    else 
    {
        eReturnStatus = Error ; // nobody connected
    }

    return eReturnStatus ;
} ;

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess  ItcServer::SendOpaque ( const void*       const _pxBlock, 
                                  unsigned long     const _ulSize ) 
{
    ESuccess    eReturnStatus = Ok ;

    itc::Opaque_Message  Msg ( _pxBlock, _ulSize ) ;

    //\\//

    if ( m_ynConnected == Yes ) 
    {
        if ( m_pLast_Connection->info().status == itc::Connection_Info::OPEN ) 
        {
            if ( DEBUG_CONNECTIONS )
            CConsoleAccess::WriteLine ( (CString) "Send on Server Port #"  
                                        + I_to_A ( m_uwIP_Port_Num )  
                                        + " : <Opaque> " ) ;
            m_pLast_Connection->send_now( Msg, 5.0 ) ;
        }
        else
        {
            m_ynConnected = No ;
            eReturnStatus = Error ;
        }
    }

    else 
    {
        eReturnStatus = Error ; // nobody connected
    }

    return eReturnStatus ;
} ;

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
void ItcServer::Message_Consumer ( const itc::IPC_Message &_ItcMsgIn ) 
{
    // For Remote Console:
    //      only one message may be expected on this port, 
    //      but this will consume any ill-targeted Itc messages
    //      and display the source of the first one

    // For Coor Connection:
    //      A Message Consumer has been specified
    itc::String_Message         oItcMsgIn = itc::String_Message ( _ItcMsgIn ) ;
    itc::Connection_Handle      This_Connection = _ItcMsgIn.get_connection() ;
    itc::Connection_Handle*     pLast_Connection ;

    CString oMsg ;

    //\\//

    switch ( _ItcMsgIn.ID() ) 
    {

      case itc::String_Message::MSG_ID :

        // see if we are still serving the same connection
        if ( m_pLast_Connection != NULL ) 
        if ( ! (This_Connection == *m_pLast_Connection) )
        {
            m_ynConnected = No ;
            if ( m_pLast_Connection != NULL ) delete m_pLast_Connection ;
            m_pLast_Connection = NULL ;
        }

        // remember a new connection, as this is the one we will SendMessage to
        if ( m_ynConnected == No )
        {
            m_ynConnected = Yes ;
            pLast_Connection = new itc::Connection_Handle ( This_Connection ) ;
            m_pLast_Connection = pLast_Connection ; // remenber this connection
        }

        oMsg = oItcMsgIn.str() ;

        if ( DEBUG_CONNECTIONS )
        CConsoleAccess::WriteLine ( (CString) "Receive on Server Port #"  
                                    + I_to_A ( m_uwIP_Port_Num )  
                                    + " : " + oMsg ) ;

        // dispatch the message if there is a consumer defined
        if ( m_pfString_Message_Consumer != NULL )
        {
            ESuccess        eStatus ;
            EReply_Option   eReplyOption ;
            char            acStringOut[KiMaxLengthStringOut +2/*End of String NULL*/];

            //\\//

            // send Message to the consumer routine specified in the constructor
            eStatus = m_pfString_Message_Consumer ( (LPCSTR)oMsg, acStringOut, &eReplyOption ) ;

            // Check for global message consuming error
            if ( eStatus == Error )
            {
                //build and send something back
                itc::String_Message ItcMsgError ( "Error Processing Message" ) ;
                oItcMsgIn.reply( ItcMsgError, 0.3 ) ;
            }

            // Depending on the application we may need to send a reply back on the same Itc Channel
            else if ( eReplyOption == Send_Reply ) 
            {
                // force string termination to avoid problem
                acStringOut[KiMaxLengthStringOut] = 0 ; 

                if ( DEBUG_CONNECTIONS )
                CConsoleAccess::WriteLine ( (CString) "Reply on Server Port #"  
                                            + I_to_A ( m_uwIP_Port_Num )  
                                            + " : " + acStringOut ) ;

                //build and send back the reply
                itc::String_Message ItcMsgReply ( acStringOut ) ;
                oItcMsgIn.reply( ItcMsgReply, 0.3 ) ;
            }
        }
        break;

      case itc::Conn_Start_Msg::MSG_ID :
        CConsoleAccess::WriteLine ( (CString) " Itc Server on port #"  
                                    + I_to_A ( m_uwIP_Port_Num ) 
                                    + ": connection  START from process <"
                                    + This_Connection.info().remote_host.c_str()
                                    + ":"
                                    + I_to_A ( This_Connection.info().pid )
                                    + ">" ) ;
        CConsoleAccess::WriteLine ( (CString) "_advertized as <"  
                                    + This_Connection.info().program_name.c_str() 
                                    + "> Version <"
                                    + This_Connection.info().program_version.c_str() 
                                    + "> using ITC Version <"
                                    + This_Connection.info().version.c_str() 
                                    + ">" ) ;
        break;
    
      case itc::Conn_Stop_Msg::MSG_ID :
        CConsoleAccess::WriteLine ( (CString) " Itc Server on port #"  
                                    + I_to_A ( m_uwIP_Port_Num )  
                                    + ": connection   STOP from process <"
                                    + This_Connection.info().remote_host.c_str() 
                                    + ":"
                                    + I_to_A ( This_Connection.info().pid )
                                    + ">" ) ;
        break;
    
      case itc::Conn_Brkn_Msg::MSG_ID :
        CConsoleAccess::WriteLine ( (CString) " Itc Server on port #"  
                                    + I_to_A ( m_uwIP_Port_Num )  
                                    + ": connection BROKEN from process <"
                                    + This_Connection.info().remote_host.c_str() 
                                    + ":"
                                    + I_to_A ( This_Connection.info().pid )
                                    + ">" ) ;
        break;
    
      case itc::Conn_Open_Msg::MSG_ID :
        CConsoleAccess::WriteLine ( (CString) " Itc Server on port #"  
                                    + I_to_A ( m_uwIP_Port_Num )  
                                    + ": connection RECONNECTED from process <"
                                    + This_Connection.info().remote_host.c_str() 
                                    + ":"
                                    + I_to_A ( This_Connection.info().pid )
                                    + ">" ) ;
        break;

      default:

        ESuccess        eStatus = Error ;

        //Case where a Generic Message consumer was specified at server creation
        if ( m_pfNested_Message_Consumer != NULL )
        {

            // see if we are still serving the same connection
            if ( m_pLast_Connection != NULL ) 
            if ( ! (This_Connection == *m_pLast_Connection) )
            {
                m_ynConnected = No ;
                if ( m_pLast_Connection != NULL ) delete m_pLast_Connection ;
                m_pLast_Connection = NULL ;
            }

            // remember a new connection, as this is the one we will SendOpaque to
            if ( m_ynConnected == No )
            {
                m_ynConnected = Yes ;
                pLast_Connection = new itc::Connection_Handle ( This_Connection ) ;
                m_pLast_Connection = pLast_Connection ; // remenber this connection
            }

            // send Message to the consumer routine specified in the constructor
            eStatus = m_pfNested_Message_Consumer ( _ItcMsgIn ) ;

            // Check for global message consuming error
            if ( eStatus == Error )
            {
                CConsoleAccess::ErrorLine ( (CString) "Error in Nested Message Consumer on port #"  
                                            + I_to_A ( m_uwIP_Port_Num ) ) ;
            }

        }

        //Either there was no handler for this message type 
        //or the specified generic consumer refused the message
        if ( eStatus == Error ) 
        {
            char sBuffer[132] ;
            sprintf ( sBuffer, 
                     "ID=%2d - No=%5u length %3u remote=%5u reply no=%5u",
                      _ItcMsgIn.ID(),
                      _ItcMsgIn.no(),
                      _ItcMsgIn.length(),
                      _ItcMsgIn.remote_no(),
                      _ItcMsgIn.reply_no() ) ;
            CConsoleAccess::WriteLine ( (CString) " Itc Server on port #"  
                                        + I_to_A ( m_uwIP_Port_Num )  
                                        + " received: Message " 
                                        + sBuffer ) ;
        }
    }

    return ;
}

