/*-------------------------------------------------------------------------------*/
  /*
    HandleLogFile.cpp
    26-Oct-98
  */
/*-------------------------------------------------------------------------------*/
#include "L1Cal_IIb_TCC/StdAfx.h"

#include "L1Cal_IIb_TCC/GlobalConstants.h"
#include "L1Cal_IIb_TCC/CustomDataTypes.h"

#include                            "L1Cal_IIb_TCC/HandleLogFile.h"

#include "L1Cal_IIb_TCC/HandleConsole.h"
#include "L1Cal_IIb_TCC/UtilsStrings.h"
#include "L1Cal_IIb_TCC/UtilsFiles.h"

#include "L1Cal_IIb_TCC/ResourceLocks.h"   // needs to synchronize access

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/

//constant gobal variables
static  const   char        cDash = '-' ;
static  const   char        cPercent = '%' ;
static  const   char        cCR = '\n' ;
static          char        sBlankLine[KiLogFileLineLength+KiConsoleMessageSpaceForNULL] ;

#ifdef WIN32
    // Global variable to remember the default drive letter
    // initialized in the main L1Cal_IIb_TCC.cpp
    int giDefaultDriveID ;
    #include <direct.h> // for _chdrive
#endif

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
// Static data members must be initialized at file scope

// current and orginal log file name
CString          CLogFile::m_osOriginalFile_Name = "" ; 
CString          CLogFile::m_osCurrentFile_Name  = "" ; 

// File Handle 
FILE*            CLogFile::mg_hLogFile = NULL ;

#if !defined ( NO_MS_CONSOLE )

    // These are the settings to control the behavior of the flood protection
    // Threshold for number of messages to receive "close-together" before declaring a flood
    unsigned long    CLogFile::mg_ulFloodCountThreshold		= 5 * 1000 ;
    // Threshold for time between successive message to call them "close-together"
    // in units of (1 / CLOCKS_PER_SEC) seconds (i.e. ms)
    clock_t          CLogFile::mg_zFloodTimeThresholdSet	= 100  ; // 0.1 sec 
    // Threshold for time between successive message to cause an end of flood
    // in units of (1 / CLOCKS_PER_SEC) seconds (i.e. ms)
    clock_t          CLogFile::mg_zFloodTimeThresholdClear	= 1000 ; // 1.0 sec 

    // initialize state and control variables
    clock_t          CLogFile::mg_zFloodTimeLastMsg			= 0 ;
    unsigned long    CLogFile::mg_ulFloodLevel				= 0 ;

#else
#endif

EYesNo           CLogFile::mg_ynFloodDeclared			= No ;

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
CLogFile::CLogFile ( ) 
{
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
CLogFile::~CLogFile ( ) 
{
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess CLogFile::NewLogFile ( ) 
{// open handle to a LogFile, 
 // valid for the whole application 

	ESuccess        eReturnStatus = Ok ;

    char            acMode_Write[] = "w+" ; // read/write, replace any existing file

    //\\//

    //drop any older LogFile first, in case there already was a LogFile
    CloseLogFile();

    //find the apropriate name for a new logfile
    m_osCurrentFile_Name = NextFileName ( ) ;

    //Record the name of the First Log File Name
    if ( m_osOriginalFile_Name == "" )   m_osOriginalFile_Name = m_osCurrentFile_Name ;
    
    // open the file (and check for success)
	// use Read/Write mode, and text type flags
    CResourceLocks::Acquire_LogFile ( ) ;
    	mg_hLogFile = fopen ( m_osCurrentFile_Name.GetBuffer(0), acMode_Write ) ;
    CResourceLocks::Release_LogFile ( ) ;

    if (mg_hLogFile == NULL) 
	{
        CConsoleAccess::ErrorLine ( (CString) "%% LogFile Error: "
                                      + "Could not open <" 
                                      + m_osCurrentFile_Name + ">" ) ;
        eReturnStatus = Error ;
	}
	else
	{
        CConsoleAccess::SystemLine ( (CString) "%% Log File opened <" 
                                      + m_osCurrentFile_Name + ">" ) ;
	}

    return eReturnStatus ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
CString         CLogFile::NextFileName  ( ) 
{

    char            acFile_Dir[_MAX_FNAME] = "" ;  //make it big to allow substitution

    char            acFile_Name_Prefix[] = "L1Cal_IIb_Tcc_" ;

    char            acFile_Time[] = "yymmdd" ;
    char            acFile_Type[] = ".LOG" ;
    char            acFile_Version[] = ";n" ;

    CString         osFile_Name ;

    //\\//

    // Initialize the content of the blank line used to pad logfile lines
    // before any possible call to CLogFile::WriteLine or CConsoleAccess::WriteLine 
    for ( int i = 0 ; i < KiLogFileLineLength ; i++ )
        sBlankLine[i] = ' ' ;
    sBlankLine[KiLogFileLineLength] = 0 ;

    //try successive version number until there are no files with that name
    SubstituteEnvVar ( acFile_Dir, acFile_Dir ) ;
    osFile_Name = (CString) acFile_Dir 
                          + acFile_Name_Prefix
                          + DateTag ()
                          + "_" + KsVersion
                          + acFile_Type ;

    // restore the default drive letter
    // because the environment variables do not include it
    // and the searchenv may fail.
    // (e.g. one can lose the drive letter after cancelling 
    // out of a File Browse box) 
#ifdef WIN32
    _chdrive( giDefaultDriveID ) ;
#endif

    //Now hunt around for the apropriate File Revision 
    AddFileVersion ( &osFile_Name ) ;
    
    return osFile_Name ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess CLogFile::FlushLogFile ( ) 
{//flush buffer to the LogFile

	ESuccess        eReturnStatus = Ok ;
    int             status ;

    //\\//

    if ( mg_hLogFile != NULL ) 
    {
        CResourceLocks::Acquire_LogFile ( ) ;
            status = fflush ( mg_hLogFile ) ;
        CResourceLocks::Release_LogFile ( ) ;

        if ( status != 0 ) 
        {
            CConsoleAccess::ErrorLine ( (CString) "%% LogFile Error: "
                                          + "Could not Flush to Log File" ) ;
            eReturnStatus = Error ;
	    }
    }

    return eReturnStatus ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess CLogFile::CloseLogFile ( ) 
{//close the log file

    ESuccess        eReturnStatus = Ok ;
    int             status ;

    //\\//

    if ( mg_hLogFile != NULL ) 
    {
        FlushLogFile () ;

        CResourceLocks::Acquire_LogFile ( ) ;
            status = fclose ( mg_hLogFile ) ;
            mg_hLogFile = NULL ; 
        CResourceLocks::Release_LogFile ( ) ;

        if ( status != 0 ) 
	    {
            CConsoleAccess::ErrorLine ( (CString) "%% LogFile Error: "
                                      + "The LogFile didn't Close Correctly " ) ;
            eReturnStatus = Error ;
	    }
        else 
        {
            CConsoleAccess::SystemLine ( (CString) "LogFile Closed" ) ;
        }

    }

    return eReturnStatus ;
}


/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess CLogFile::WriteLine      ( const char*       const _sString,
                                    const int               _iMsgLengthMax ) 
{
    ESuccess    eReturnStatus = Ok ;
    ESuccess    eStatus ;
    int         iBufferLength ; 
    char*       psBuffer ; 
    char*       psMsg ; 
	int         iCharWritten ;
    char*       pcCR ;

    //\\//

    // bail out if there is currently no logfile 
    if ( mg_hLogFile == NULL ) return eReturnStatus ;

    //allocate memory for copy of input string
    //(make sure we have at least enough characters to build a line)
    if ( _iMsgLengthMax >= KiLogFileLineLength ) 
        iBufferLength = _iMsgLengthMax ;
    else
        iBufferLength = KiLogFileLineLength ;
    psBuffer = new char [iBufferLength 
                       + KiConsoleMessageSpaceForCR
                       + KiConsoleMessageSpaceForNULL] ;
    //we need an independent pointer that may be modified
    //while we also need to preserve the original pointer for cleanup
    psMsg = psBuffer ;

    // copy source string to local variable
    // and limit the damages by using strncat if source string wasn't null terminate.
    psMsg[0] = 0 ;
    strncat ( psMsg, _sString, iBufferLength ) ;
    // Add null character in last column for safety, but strncat does it already
    psMsg[iBufferLength] = 0 ;


    //look for any embedded <CR>; i.e. multi-line message
    pcCR = strrchr ( psMsg, cCR ) ;
    if ( pcCR != NULL ) 
    {//there was an embedded <CR>, 
     
        // trim the string by forcing an end of string where the <CR> was
        *pcCR = 0 ;

        //do recursive call with what's BEFORE the <CR>...
        // (which may still have more embedded <CR>'s)
        eStatus = WriteLine ( psMsg, _iMsgLengthMax ) ;
        if ( eStatus != Ok ) eReturnStatus = Error ;

        //... and what's AFTER the <CR>
        // (which is now a simple line)
        // but be ready to copy the original Prefix, after forcing it to lower case
        if ( psMsg[KiConsoleMsgPrefixLength-1] == KiConsolePrefixSeparator )  
        {
            psMsg = pcCR - KiConsoleMsgPrefixLength + 1 ; // get to the char before the former <CR> (now NULL)
            strncpy ( psMsg, psBuffer, KiConsoleMsgPrefixLength-1 ) ; // copy the Prefix letter(s) without the '$'
            strlwr  ( psMsg ) ; // lower case the prefix letter (remember the NULL is still there)
            psMsg[KiConsoleMsgPrefixLength-1] = KiConsolePrefixSeparator ; // replace the NULL with a '$'
        }
        else 
            psMsg = pcCR + 1 ; // First character after the former <CR>

        // we can now print the end of the line with another recursive call
        if ( WriteLine ( psMsg, _iMsgLengthMax ) != Ok ) eReturnStatus = Error ;
    }
    //Check Line length and break it down in two halves if needed
    else if ( strlen ( psMsg ) > KiLogFileMaxCharPerLine ) 
    {
        char cTemp ;
        cTemp = psMsg[KiLogFileMaxCharPerLine] ; // remember this char...
        psMsg[KiLogFileMaxCharPerLine] = 0 ; // ... and replace it with a blank

        //do recursive call with the beginning of the line...
        if ( WriteLine ( psMsg, KiLogFileMaxCharPerLine ) != Ok ) 
            eReturnStatus = Error ;

        //... and the end (which may also need further cutting up)
        // after pre-pending the orignal line prefix, if any, but lower case
        if ( psMsg[KiConsoleMsgPrefixLength-1] == KiConsolePrefixSeparator )  
        {
            psMsg += KiLogFileMaxCharPerLine - KiConsoleMsgPrefixLength ;
            strncpy ( psMsg, psBuffer, KiConsoleMsgPrefixLength ) ; // Pre-pend Leading Flag
            strlwr  ( psMsg ) ; // lower case happens here while NULL is still there
            psMsg[KiConsoleMsgPrefixLength] = cTemp ; // restore this char 
        }
        else 
        {
            psMsg += KiLogFileMaxCharPerLine ;
            psMsg[0] = cTemp ; // restore this char 
        }

        if ( WriteLine ( psMsg, _iMsgLengthMax ) != Ok ) 
            eReturnStatus = Error ;
    }
    else
    {//this is just a simple one-line string, short enough to be displayed without loosing information

#if !defined ( NO_MS_CONSOLE )
        //determine if there is a flood of messages
        clock_t  zTimeThisMsg ;
        clock_t  zTimeSinceLastMsg ;

        zTimeThisMsg = clock ( ) ;
        zTimeSinceLastMsg = zTimeThisMsg - mg_zFloodTimeLastMsg ;
        mg_zFloodTimeLastMsg = zTimeThisMsg ;

		if ( ( ( mg_ynFloodDeclared == No  ) && ( zTimeSinceLastMsg <= mg_zFloodTimeThresholdSet ) )
		  || ( ( mg_ynFloodDeclared == Yes ) && ( zTimeSinceLastMsg <= mg_zFloodTimeThresholdClear ) ) )
        {
            mg_ulFloodLevel++ ;

            if ( mg_ulFloodLevel == mg_ulFloodCountThreshold )
            {
                CConsoleAccess::ErrorLine ( (CString) "%%\n%%\n" 
                                                      + "%% LogFile Service Interrupted after "
                                                      + I_to_A ( mg_ulFloodLevel ) 
                                                      + " Messages within " 
                                                      + I_to_A ( mg_zFloodTimeThresholdSet )
                                                      + " / "
                                                      + I_to_A ( CLOCKS_PER_SEC ) 
                                                      + " second" 
                                                      + "\n%%\n%%" ) ;
                mg_ynFloodDeclared = Yes ;
            }
        }
        else
        {
            if ( mg_ynFloodDeclared == Yes )
            {
                mg_ynFloodDeclared = No ;

                CConsoleAccess::ErrorLine ( (CString) "%% LogFile Service Resumed after a flood of "
                                                      + I_to_A ( mg_ulFloodLevel ) 
                                                      + " Messages " ) ;
            }

            mg_ulFloodLevel = 0 ;
        }

#else
#endif

        if ( mg_ynFloodDeclared == No )
        {
            //extend the end of the line with blanks as needed
            strncat ( psMsg, sBlankLine, KiLogFileLineLength-strlen(psMsg) ) ;

            //overwrite the end with the date and time in the following format
            //(this shows the array index number, i.e. the column number minus one)
            //      1         1         1
            //      1         2         3
            // 567890123456789012345678901
            //  %%dd-mmm-yyyy hh:mm:ss.mmm

            psMsg[KiLogFileLineLength-KiLogFileTimeStampLength+0] = cPercent ;
            psMsg[KiLogFileLineLength-KiLogFileTimeStampLength+1] = cPercent ;
            strcpy ( psMsg+KiLogFileLineLength-KiLogFileTimeStampLength+2, TimeStamp() );
            strcpy ( psMsg+KiLogFileLineLength, "\n" ) ; //we declared enough room for this 1x char + 1x Null 

            // write result to LogFile
            CResourceLocks::Acquire_LogFile ( ) ;
                iCharWritten = fprintf ( mg_hLogFile, "%s", psMsg ) ;
            CResourceLocks::Release_LogFile ( ) ;

            if ( iCharWritten != KiLogFileLineLength+1 )
            {
                mg_hLogFile = NULL ; 
                CConsoleAccess::ErrorLine ( (CString) "%% LogFile Error: "
                                                      + I_to_A ( iCharWritten ) 
                                                      + " Characters Written Instead of " 
                                                      + I_to_A ( KiLogFileLineLength+1 ) ) ;
                eReturnStatus = Error ;
	        }
        }
    } 
    
    // relese temporary message copy
    delete [] psBuffer ;

    return eReturnStatus ;
}

