/*-------------------------------------------------------------------------------*/
  /*
    HandleResultFile.cpp
    18-Apr-2001
  */
/*-------------------------------------------------------------------------------*/
#include "L1Cal_IIb_TCC/StdAfx.h"

#include "L1Cal_IIb_TCC/GlobalConstants.h"
#include "L1Cal_IIb_TCC/CustomDataTypes.h"

#include                            "L1Cal_IIb_TCC/HandleResultFile.h"

#include "L1Cal_IIb_TCC/HandleConsole.h"
#include "L1Cal_IIb_TCC/UtilsStrings.h"
#include "L1Cal_IIb_TCC/UtilsFiles.h"

#include <time.h>   // for clock

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
// Static data members must be initialized at file scope
char            CResultFile::mg_acMode_Read[]   = "r" ; // read only, file must exist
char            CResultFile::mg_acMode_Write[]  = "w" ; // write, replace any existing file
char            CResultFile::mg_acMode_WrtBin[] = "wb" ; // write binary, replace any existing file, no translation
char            CResultFile::mg_acMode_Append[] = "a+" ; // read/write, replace any existing file
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
CResultFile::CResultFile ( const char*       const _sString ) 
{
    m_hResultFile = NULL ;

    //Maybe there is no file name and we need to ask for one
    if ( _sString == NULL ) 
    {
//**//        if ( FileBrowse ( &m_osFileName ) != Ok ) 
            m_osFileName  = "" ;
    }

    //or maybe the file name uses environment variables
    else if ( _sString[0] == '%' ) // This name uses an environment variable
    {
        char sFilename[_MAX_FNAME+1] ;
        if ( SubstituteEnvVar ( _sString, sFilename ) == Ok )
            m_osFileName  = sFilename ;
        else
        {
            CConsoleAccess::InternalErrorLine ( (CString) "ResultFile Name Error: "
                                                          + "Could not Understand <" 
                                                          + _sString + ">" ) ;
            m_osFileName  = "" ;
        }
    }

    //or we just take it as given
    else
        m_osFileName  = _sString ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
CResultFile::~CResultFile ( ) 
{
    if ( m_hResultFile != NULL )    Close () ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess CResultFile::OpenForRead ( ) 
{
    ESuccess        eReturnStatus = Ok ;

    //\\//

    //drop any older ResultFile first, in case there already was a ResultFile
    Close();

    // open the file (and check for success)
	// use Read/Write mode, and text type flags
    if ( m_osFileName  != "" )
	m_hResultFile = fopen ( m_osFileName.GetBuffer(0), mg_acMode_Read ) ;

    if (m_hResultFile == NULL) 
	{
        CConsoleAccess::ErrorLine ( (CString) "%% ResultFile Error: "
                                      + "Could not open <" 
                                      + m_osFileName + "> for Read" ) ;
        eReturnStatus = Error ;
	}
	else
	{
        CConsoleAccess::WriteLine ( (CString) "%% ResultFile opened <" 
                                      + m_osFileName + "> for Read" ) ;
	}

    return eReturnStatus ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess CResultFile::OpenForWrite ( bool   _lBinaryMode, 
                                     bool   _lVerbose ) 
{
    ESuccess        eReturnStatus = Ok ;

    //\\//

    //drop any older ResultFile first, in case there already was a ResultFile
    Close();

    // open the file (and check for success)
	// use Read/Write mode, and text type flags
    if ( m_osFileName  != "" )
    {
        if ( _lBinaryMode!= true ) 
        	m_hResultFile = fopen ( m_osFileName.GetBuffer(0), mg_acMode_Write ) ;
        else
        	m_hResultFile = fopen ( m_osFileName.GetBuffer(0), mg_acMode_WrtBin ) ;
    }

    if (m_hResultFile == NULL) 
	{
        CConsoleAccess::ErrorLine ( (CString) "%% ResultFile Error: "
                                      + "Could not open <" 
                                      + m_osFileName + "> for Write" ) ;
        eReturnStatus = Error ;
	}
	else if ( _lVerbose )
	{
        CConsoleAccess::WriteLine ( (CString) "%% ResultFile opened <" 
                                      + m_osFileName + "> for Write" ) ;
	}

    return eReturnStatus ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess CResultFile::OpenForAppend ( bool   _lVerbose ) 
{
    ESuccess        eReturnStatus = Ok ;

    //\\//

    //drop any older ResultFile first, in case there already was a ResultFile
    Close();

    // open the file (and check for success)
	// use Read/Write mode, and text type flags
    if ( m_osFileName  != "" )
	m_hResultFile = fopen ( m_osFileName.GetBuffer(0), mg_acMode_Append ) ;

    if (m_hResultFile == NULL) 
	{
        CConsoleAccess::ErrorLine ( (CString) "%% ResultFile Error: "
                                      + "Could not open <" 
                                      + m_osFileName + "> For Append" ) ;
        eReturnStatus = Error ;
	}
	else if ( _lVerbose )
	{
        CConsoleAccess::WriteLine ( (CString) "%% ResultFile opened <" 
                                      + m_osFileName + "> For Append" ) ;
	}

    return eReturnStatus ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess CResultFile::Flush ( ) 
{//flush buffer to the ResultFile

	ESuccess        eReturnStatus = Ok ;

    //\\//

    if ( m_hResultFile != NULL ) 
    if ( fflush ( m_hResultFile ) != 0 ) 
    {
        CConsoleAccess::ErrorLine ( (CString) "%% ResultFile Error: "
                                      + "Could not Flush to ResultFile" ) ;
        eReturnStatus = Error ;
	}

    return eReturnStatus ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess CResultFile::Close ( bool   _lVerbose ) 
{//close the result file

    ESuccess    eReturnStatus = Ok ;

    //\\//

    if ( m_hResultFile != NULL ) 
    {
        Flush () ;

	    if ( fclose ( m_hResultFile ) ) 
	    {
            CConsoleAccess::ErrorLine ( (CString) "%% ResultFile Error: "
                                      + "File <"
                                      + m_osFileName 
                                      + "> didn't Close Correctly " ) ;
            eReturnStatus = Error ;
	    }
    	else if ( _lVerbose )
        {
            CConsoleAccess::WriteLine ( (CString) "%% ResultFile <"
                                      + m_osFileName 
                                      + "> Closed" ) ;
        }

        m_hResultFile = NULL ; 

    }

    return eReturnStatus ;
}


/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
EYesNo CResultFile::ReadyForWrite    ( ) 
{
    if ( m_hResultFile != NULL ) 
        return Yes ;
    else 
        return No ;
}


/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess CResultFile::WriteLine   ( const char*       const _sString ) 
{
    ESuccess    eReturnStatus = Ok ;
	int         iLineLength ;
    int         iCharWritten = 0 ;

    //\\//

    iLineLength = strlen(_sString) +1 ; //with <CR>

    // write to ResultFile
    if ( m_hResultFile != NULL ) 
    iCharWritten = fprintf ( m_hResultFile, "%s\n", _sString ) ;

    if ( iCharWritten != iLineLength )
    {
        CConsoleAccess::ErrorLine ( (CString) "%% ResultFile Error: "
                                              + I_to_A ( iCharWritten ) 
                                              + " Characters Written Instead of " 
                                              + I_to_A ( iLineLength ) ) ;
        CConsoleAccess::ErrorLine ( (CString) "%%_Line: "
                                              + I_to_A ( iCharWritten ) 
                                              + _sString ) ;
        eReturnStatus = Error ;
	}
    
    return eReturnStatus ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess CResultFile::WriteBinary ( const char*             _pcByteData,
                                    const unsigned long     _ulByteCount ) 
{
    ESuccess        eReturnStatus = Ok ;
    unsigned long   ulByteWritten = 0 ;

    //\\//

    // write to ResultFile
    if ( m_hResultFile != NULL ) 
    for ( ulByteWritten = 0 ; ulByteWritten < _ulByteCount ; ulByteWritten ++ )
    if ( putc ( _pcByteData[ulByteWritten], m_hResultFile ) == EOF ) break ;

    if ( ulByteWritten != _ulByteCount )
    {
        CConsoleAccess::ErrorLine ( (CString) "%% ResultFile Error: "
                                              + I_to_A ( ulByteWritten ) 
                                              + " Characters Written Instead of " 
                                              + I_to_A ( _ulByteCount ) ) ;
        CConsoleAccess::ErrorLine ( (CString) "%%_First Byte: "
                                              + I_to_A ( *_pcByteData, 2, 16, '0' ) ) ;
        eReturnStatus = Error ;
	}
    
    return eReturnStatus ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess CResultFile::WriteDateTimeLine ( char* _pcCommentFlag ) 
{
    if ( _pcCommentFlag == NULL )
        return WriteLine ( " " + TimeStamp () ) ;
    else
        return WriteLine ( (CString) _pcCommentFlag 
                         + " " + TimeStamp () ) ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess CResultFile::WriteRulerLine ( char* _pcCommentFlag ) 
{
    if ( _pcCommentFlag == NULL )
        return WriteLine ( "------------------------------------------------------------" ) ;
    else
        return WriteLine ( (CString) _pcCommentFlag 
                         + "------------------------------------------------------------" ) ;
}

