/*-------------------------------------------------------------------------------*/
  /*
    HandleSymbols.cpp
    28-Oct-98
  */
/*-------------------------------------------------------------------------------*/
#include "L1Cal_IIb_TCC/StdAfx.h"

#include "L1Cal_IIb_TCC/GlobalConstants.h"
#include "L1Cal_IIb_TCC/CustomDataTypes.h"

#include                            "L1Cal_IIb_TCC/HandleSymbols.h"

#include "L1Cal_IIb_TCC/HandleConsole.h"

#include "L1Cal_IIb_TCC/UtilsStrings.h"

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
/*-------------------- Implementation of Class CSymbolElement -------------------*/
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
CSymbolElement::CSymbolElement ( ) 
{

}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
CSymbolElement::CSymbolElement    ( const char*         _psSymbolName, 
                                    unsigned long       _ulSymbolValue,
                                    CSymbolElement*     _poPrevSymbol  )
{
    m_osSymbolName      = _psSymbolName ;
    m_ulSymbolValue     = _ulSymbolValue ;
    m_poPrevSymbol      = _poPrevSymbol ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
CSymbolElement::~CSymbolElement ( ) 
{
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
EYesNo      CSymbolElement::MatchName     ( const char*       _psSymbolName ) 
{

    if ( m_osSymbolName.CompareNoCase ( (CString) _psSymbolName ) == 0 )
        return Yes ;
    else
        return No ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
/*---------------------- Implementation of Class CSymbolSet ---------------------*/
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/

CSymbolSet::CSymbolSet ( ) 
{
    m_poLastSymbol          = NULL ;
    m_uwTotSymbol           = 0 ;
    m_poCachedLastAccessed  = NULL ;

}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
CSymbolSet::~CSymbolSet ( ) 
{
    DeleteAllSymbols() ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
CSymbolElement* CSymbolSet::LocateSymbol     ( const char*   _psSymbolName ) 
{
    CSymbolElement*     poSymbol ;

    //\\//

    //First, try the last Accessed Symbol 
    if ( m_poCachedLastAccessed != NULL )
    {
        if ( m_poCachedLastAccessed->MatchName ( _psSymbolName ) == Yes )
        {
    	    poSymbol = m_poCachedLastAccessed ;
            goto Done ;
        }
    }

    //If above failed, go through the whole list, starting from the end
    poSymbol = m_poLastSymbol ;
    while ( poSymbol != NULL )
    {
        if ( poSymbol->MatchName ( _psSymbolName ) == Yes )
        {
            m_poCachedLastAccessed = poSymbol ;
            goto Done ;
        }
        poSymbol = poSymbol->GetPrev() ;
    }

Done:
    return poSymbol ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess        CSymbolSet::AddSymbol ( const char*     _psSymbolName, 
                                        unsigned long   _ulSymbolValue ) 
{
	ESuccess            eReturnStatus = Ok ;

    CSymbolElement*     poNewSymbol ;

    //\\//

    if ( LocateSymbol ( _psSymbolName ) != NULL ) 
    {
        CConsoleAccess::InternalErrorLine ( (CString) " Internal Error/CSymbolSet::AddSymbol:"
                                                 + " Symbol already exists <"
                                                 + _psSymbolName
                                                 + ">" ) ;
        eReturnStatus = Error ;
    }
    else
    {
        poNewSymbol = new CSymbolElement  ( _psSymbolName, 
                                            _ulSymbolValue,
                                            m_poLastSymbol ) ;

        m_uwTotSymbol += 1 ;

        m_poLastSymbol = poNewSymbol ;

        m_poCachedLastAccessed  = poNewSymbol ;
    }

    return eReturnStatus ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess        CSymbolSet::ModifySymbol ( const char*     _psSymbolName, 
                                           unsigned long   _ulSymbolValue ) 
{
	ESuccess            eReturnStatus = Ok ;

    CSymbolElement*     poSymbol ;

    //\\//

    poSymbol = LocateSymbol ( _psSymbolName ) ;

    if ( poSymbol == NULL ) 
    {
        CConsoleAccess::InternalErrorLine ( (CString) " Internal Error/CSymbolSet::ModifySymbol:"
                                                 + " Symbol doesn't exist <"
                                                 + _psSymbolName
                                                 + ">" ) ;
        eReturnStatus = Error ;
    }
    else
    {
        poSymbol->m_ulSymbolValue = _ulSymbolValue ;
    }

    return eReturnStatus ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess CSymbolSet::DeleteAllSymbols ( ) 
{
	ESuccess            eReturnStatus = Ok ;

    CSymbolElement*     poSymbol ;

    //\\//

    while ( m_poLastSymbol != NULL )
    {
        poSymbol = m_poLastSymbol ;
        m_poLastSymbol = poSymbol->GetPrev() ;
        delete poSymbol ;
    }

    m_uwTotSymbol           = 0 ;
    m_poCachedLastAccessed  = NULL ;

    return eReturnStatus ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
const CSymbolSet&   CSymbolSet::operator=     ( const CSymbolSet&   _SymbolSetSrc ) 
{
    CSymbolElement*     poSymbol ;

    //\\//

    if ( this->m_poLastSymbol != NULL )
    {//this symbol set already had symbols defined, 
        //delete these old ones first
        this->DeleteAllSymbols() ;
    }

    //start from the end of the chain of the Source Symbol Set
    poSymbol = _SymbolSetSrc.m_poLastSymbol ;

    //one at a time, copy those symbols to the new list
    while ( poSymbol != NULL )
    {
        this->AddSymbol ( poSymbol->GetName(),
                          poSymbol->GetValue() ) ;
        poSymbol = poSymbol->GetPrev() ;
    }
    
    return *this ;
}
