/*-------------------------------------------------------------------------------*/
  /*
    HsroChannel.cpp
    Class CHsroChannel

  */
/*-------------------------------------------------------------------------------*/

#include "L1Cal_IIb_TCC/StdAfx.h"

#include                            "L1Cal_IIb_TCC/HsroChannel.h"

#include "L1Cal_IIb_TCC/NevisCard.h"

#include "L1Cal_IIb_TCC/HsroL1Cal.h"
#include "L1Cal_IIb_TCC/L1CalCardAddresses.h"

#include "L1Cal_IIb_TCC/HandleConsole.h"
#include "L1Cal_IIb_TCC/UtilsStrings.h"
#include "L1Cal_IIb_TCC/UtilsBits.h"

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
CHsroChannel::CHsroChannel        ( )  
{
    //Location of the Base The Card
    m_poNevisCard         = NULL ;

    //Initialize Vrb Coordinates
    m_ubVrb_Slot            = 0 ;
    m_ubVtm_Channel         = 0 ;

    m_uwHsroTotWord         = 0 ;

    //Initialize Hsro Data Buffer
    for ( int i = 0 ; i < KiHsroMaxLongPerCard ; i ++ ) 
    {
        m_xHsroData_Expected.aulByLong[i] = 0 ;
        m_xHsroData_Actual.aulByLong[i] = 0 ;
    }

    //Initialze pointer to corresponding Vrb Card
    m_poVrbCard             = NULL ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
void CHsroChannel::FurtherConstruct        ( CNevisCard*         _poNevisCard )  
{
    int             iVrbNum ;

    //\\

    // Remember the location of the Base The Card
    m_poNevisCard         = _poNevisCard ;

    // Record the fact that this card participates in the Readout
    m_poNevisCard->m_ynThisCardReadout = Yes ;

    //Initialize Vrb Coordinates
    m_ubVrb_Slot          = L1CalTabVrbSlot         ( m_poNevisCard->m_ubModuleNum ) ;
    m_ubVtm_Channel       = L1CalTabVrbChannelPair  ( m_poNevisCard->m_ubModuleNum ) ;

    
    if ( m_poNevisCard->m_ubModuleNum == KiModuleNumGab ) 
        m_uwHsroTotWord   = KiHsroTotWordPerGab ;
    else 
        m_uwHsroTotWord   = KiHsroTotWordPerTab ;
    
 
    //Try to locate the VRB Card in the VRB Crate Object
    for ( iVrbNum = 0 ; iVrbNum < Ki_Tot_Vrb_L1Cal_Readout ; iVrbNum++ ) 
    if ( CHsroL1Cal::mg_poHsroL1Cal != NULL ) 
    if ( CHsroL1Cal::mg_poHsroL1Cal->m_paoVrb[iVrbNum] != NULL )  
    if ( m_ubVrb_Slot == CHsroL1Cal::mg_poHsroL1Cal->m_paoVrb[iVrbNum]->m_ubCard_Slot )  
    {// register this The Card with the VRB Card Object
        m_poVrbCard = CHsroL1Cal::mg_poHsroL1Cal->m_paoVrb[iVrbNum] ;
        if ( m_poVrbCard->m_apoNevisCard[m_ubVtm_Channel] == NULL ) // Do not allow to overwrite the original L1Cal objects.
            m_poVrbCard->m_apoNevisCard[m_ubVtm_Channel] = m_poNevisCard ;
    }

    if ( m_poVrbCard == 0 ) 
        CConsoleAccess::InternalErrorLine ( (CString) "Internal Error/CHsroChannel Constructor:"
                                                     + " No Vrb Card Object Found for Slot "
                                                     + I_to_A (m_ubVrb_Slot) ) ;

    return ;

}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
CHsroChannel::~CHsroChannel       ()
{
    return ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
/*
ESuccess CHsroChannel::HsroReadBsfDataFromMonitReg ( ) 
{
    ESuccess        eReturnStatus = Ok ;

    //\\//


    return eReturnStatus ;
}
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess CHsroChannel::HsroReadFromVbd ( ) 
{
    ESuccess        eReturnStatus = Ok ;

    //\\//

    if ( CHsroL1Cal::mg_poHsroL1Cal->ExtractOneCard ( m_ubVrb_Slot, 
                                                      m_ubVtm_Channel, 
                                                     &m_xHsroData_Actual,
                                                      m_uwHsroTotWord ) != Ok )
        eReturnStatus = Error ;

    return eReturnStatus ;
}
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess CHsroChannel::HsroCompareData ( ) 
{
    ESuccess        eReturnStatus = Ok ;

    unsigned short      uwExpect ;
    unsigned short      uwActual ;
    unsigned char       ubEventNumber ;

    //\\//

    for ( int i = 0 ; i < m_uwHsroTotWord ; i++ ) 
    {
        uwExpect = m_xHsroData_Expected.auwByWord[i] ;
        uwActual = m_xHsroData_Actual.auwByWord[i] ;

        if ( uwActual != uwExpect ) 
        {   
            if ( eReturnStatus != Error ) // This is the first Error
            CConsoleAccess::ErrorLine ( (CString) "\n Readout Mismatch for <"
                                                  + m_poNevisCard->m_osCardDescription 
                                                  + ">" ) ;

            ubEventNumber = m_xHsroData_Actual.xByField.xHeader.xByField.ubTickNumber ;
            CConsoleAccess::ErrorLine ( (CString) " Word #"
                                                  + I_to_A ( i, 2 )
                                                  + " is " 
                                                  + I_to_A ( uwActual, 4, 16, '0' )
                                                  + " instead of "
                                                  + I_to_A ( uwExpect, 4, 16, '0' ) 
                                                  + " while HSRO Header EvtNum is " 
                                                  + I_to_A ( ubEventNumber, 2, 16, '0' ) ) ;
            eReturnStatus = Error ;
        }
    }
    
    return eReturnStatus ;
}
/*-------------------------------------------------------------------------------*/
