/*-------------------------------------------------------------------------------*/
  /*
    HsroCrate.cpp
    Class CHsroCrate
    5-Sep-00  L1FW VRB Crate
    2-Jul-01  Made into generic readout crate that HsroL1fw will derive from
  */
/*-------------------------------------------------------------------------------*/

#include "L1Cal_IIb_TCC/StdAfx.h"
#include "L1Cal_IIb_TCC/GlobalConstants.h"
#include "L1Cal_IIb_TCC/CustomDataTypes.h"

#include                            "L1Cal_IIb_TCC/HsroCrate.h"

#include "L1Cal_IIb_TCC/HandleHardwareAccess.h" // for CheckBit3Status

#include "L1Cal_IIb_TCC/HandleConsole.h"
#include "L1Cal_IIb_TCC/UtilsStrings.h"

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
CHsroCrate::CHsroCrate        ( )  
{
    int     iVrbNum ; 

    //\\//

    // Note: The derive class will actually create the objects.
    // Here we initalize all pointers to Zero

    //Initialize the pointer to the Last VRB 
    //which will be used to chain through all the cards in the crate
    m_poLastVrbCard = NULL ; 

    //Zero the pointer to the VRBs 
    for ( iVrbNum = 0 ; 
          iVrbNum < Ki_Max_Vrb_in_Readout_Crate ;
          iVrbNum ++ )
        m_paoVrb[iVrbNum] = NULL ; 

    
    //Zero the pointer to the VRB Controller 
    m_poVrbContr = NULL ;

    //Zero the pointer to the VBD 
    m_poVbd = NULL ;

    return ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
CHsroCrate::~CHsroCrate       ()
{
    int     iVrbNum ; 

    //\\//

    //Delete all VRB card objects
    for ( iVrbNum = 0 ; 
          iVrbNum < Ki_Max_Vrb_in_Readout_Crate ;
          iVrbNum ++ )
    if ( m_paoVrb[iVrbNum] != NULL ) 
        delete m_paoVrb[iVrbNum] ;

    //Delete VRB Controller card object
    if ( m_poVrbContr != NULL ) 
        delete m_poVrbContr ;

    //Delete VBD card object
    if ( m_poVbd != NULL ) 
        delete m_poVbd ;

    return ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess CHsroCrate::Initialize        ( ) 
{
    ESuccess        eReturnStatus = Ok ;

    //\\//

    //Initialize the VBD Card in the Crate
    if ( InitializeVbd    () != Ok ) eReturnStatus = Error ;

    //Initialize the VRB Controller Card in the Crate
    if ( InitializeVrbc   () != Ok ) eReturnStatus = Error ;

    //Initialize each VRB Card in the Crate
    if ( InitializeAllVrb () != Ok ) eReturnStatus = Error ;

    return eReturnStatus ;
}
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess CHsroCrate::InitializeAllVrb ( ) 
{
    ESuccess        eReturnStatus = Ok ;

    int             iVrbNum ; 

    //\\//

    //Initialize each VRB Card in the Crate
    for ( iVrbNum = 0 ; 
          iVrbNum < Ki_Max_Vrb_in_Readout_Crate ;
          iVrbNum ++ )
    if ( m_paoVrb[iVrbNum] != NULL ) 
    {
        if ( m_paoVrb[iVrbNum]->Initialize () != Ok ) eReturnStatus = Error ;

        if ( CHardwareAccess::mg_poBit3BusAdaptor->CheckBit3Status() != Ok )
        {
            CConsoleAccess::ErrorLine ( "Found Bit3 Errors during VRB Card Initialization; Continuing" ) ;
            CHardwareAccess::mg_poBit3BusAdaptor->ClearBit3Status() ;
        }

    }

    return eReturnStatus ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess CHsroCrate::InitializeVrbc ( ) 
{
    ESuccess        eReturnStatus = Ok ;

    //\\//

    //Initialize the VRB Controller Card in the Crate
    if ( m_poVrbContr != NULL ) 
    {
        if ( m_poVrbContr->Initialize () != Ok ) eReturnStatus = Error ;

        if ( CHardwareAccess::mg_poBit3BusAdaptor->CheckBit3Status() != Ok )
        {
            CConsoleAccess::ErrorLine ( "Found Bit3 Errors during VRBC Card Initialization" ) ;
            CHardwareAccess::mg_poBit3BusAdaptor->ClearBit3Status() ;
        }
    }

    return eReturnStatus ;
}
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess CHsroCrate::InitializeVbd ( ) 
{
    ESuccess        eReturnStatus = Ok ;

    //\\//

    //Initialize the VBD Card in the Crate
    if ( m_poVbd != NULL ) 
    {
        if ( m_poVbd->Initialize () != Ok ) eReturnStatus = Error ;

        if ( CHardwareAccess::mg_poBit3BusAdaptor->CheckBit3Status() != Ok )
        {
            CConsoleAccess::ErrorLine ( "Found Bit3 Errors during VBD Card Initialization" ) ;
            CHardwareAccess::mg_poBit3BusAdaptor->ClearBit3Status() ;
        }
    }

    return eReturnStatus ;
}
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess CHsroCrate::VmeSysReset ( TulVmeAddress _ulVerticalSysResetAddress ) 
{
    ESuccess        eReturnStatus = Ok ;

    const int   kiWriteOneWord = 2 ;
    int         Zero = 0 ;

    //\\//
    
    CHardwareAccess::mg_poBit3BusAdaptor
        ->WriteVmeData ( _ulVerticalSysResetAddress, &Zero, kiWriteOneWord ) ;
	
    if ( CHardwareAccess::mg_poBit3BusAdaptor->CheckBit3Status() != Ok )
        CHardwareAccess::mg_poBit3BusAdaptor->ClearBit3Status() ;

    return eReturnStatus ;
}
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess CHsroCrate::ExtractOneCard       ( unsigned long       _ulVrb_Slot,
                                            unsigned long       _ulVtm_Channel,
                                            OneCardHsroDataMax* _pxActualData,
                                            unsigned long       _ulTotLong ) 
{
    ESuccess        eReturnStatus = Ok ;
    unsigned int    uiVrbNum ;

    Vbd_A24_Access_Buf* pxVbdBase ;
    VbdHeader*          pxVbdHeader ;   VbdHeader    xVbdHeader ;
    unsigned int        uiTotByteInEvent ;
    VrbSectionInVbd*    pxVrbSection ;
    unsigned short      uwLongWordCountInVrbSection ;
    VrbHeader*          pxVrbHeader ;   VrbHeader    xVrbHeader ;

    VrbChannelMax*     pxVrbChanLow ;
    VrbChannelMax*     pxVrbChanHig ;

    //\\//

    pxVrbChanLow = 0 ;
    pxVrbChanHig = 0 ;

    // Get the address of VBD Header
    if ( m_poVbd == NULL ) 
    {
        CConsoleAccess::InternalErrorLine ( (CString) " CHsroCrate::ExtractOneCard: VBD pointer still NULL" ) ;
        goto Abort ;
    }
    pxVbdBase   = m_poVbd->f_pxA24Access_Buffer() ;
    pxVbdHeader = &pxVbdBase->xByField.xVbdHeader ;

#if defined ( VME_IO_FAKE ) 
    #include "L1Cal_IIb_TCC/HsroRawTestEvent.h"
#endif // VME_IO_FAKE 

    //Transfer the first word of the header.
    //This words contains the event size
    xVbdHeader.aulByLong[0] = pxVbdHeader->aulByLong[0] ; 
    uiTotByteInEvent = 4 * xVbdHeader.xByField.ulTotLongWordInEvent ;
    if ( uiTotByteInEvent > sizeof(Vbd_A24_Access_Buf) ) // May need updating for L1Cal, leave here for now
    {
        CConsoleAccess::ErrorLine ( (CString) " VBD Event Size larger than Expected Maximum" ) ;
        goto Abort ;
    }

    //Get the address of the first VRB Section, 
    //and crawl through the event until we find a VRB 
    //with the requested Slot Number
    pxVrbSection = &pxVbdBase->xByField.axVrbSection[0] ;

    for ( uiVrbNum = 0 ; uiVrbNum < Ki_Max_Vrb_in_Readout_Crate ; uiVrbNum++ ) 
    {
        if ( ( (unsigned long) pxVrbSection - (unsigned long) pxVbdBase ) > uiTotByteInEvent ) 
        {
            CConsoleAccess::ErrorLine ( (CString) " VRB Section #"
                                                  + I_to_A (uiVrbNum+1, 2)
                                                  + " Would be past the end of the Event in the VBD" ) ;
            goto Abort ;
        }

        if ( ( (unsigned long) pxVrbSection + sizeof(VrbHeader) - (unsigned long) pxVbdBase ) > sizeof(Vbd_A24_Access_Buf) )
        {
            CConsoleAccess::ErrorLine ( (CString) " VRB Section #"
                                                  + I_to_A (uiVrbNum+1, 2)
                                      + " Would be past the amount of memory mapped for VBD" ) ;
            goto Abort ;
        }

        // Transfer the VRB Section length to Main Memory so that we do 
        // D32 transfers from VBD to memory and avoid all D8 and D16 accesses to VBD
        WordCountInVbd xTemp ;
        xTemp.ulByLong = pxVrbSection->xLongWordCount.ulByLong ;
        uwLongWordCountInVrbSection = xTemp.xByField.uwLongWordCount ; 

        if ( 4*uwLongWordCountInVrbSection < sizeof(VrbHeader) ) 
        {
            CConsoleAccess::ErrorLine ( (CString) " VRB Section #"
                                                   + I_to_A (uiVrbNum+1)
                                                   + " is shorter than a VRB Header" ) ;
            goto Abort ;
        }

        pxVrbHeader = &pxVrbSection->xVrbHeader ;

        // Transfer the VBD Header to Main Memory so that we do 
        // D32 transfers from VBD to memory and avoid all D8 and D16 accesses to VBD
        // and minimize the number of accesses over the Bit3 Adaptor
        for ( int iLong = 0 ; iLong < sizeof(VrbHeaderByField)/4 ; iLong++ ) 
            xVrbHeader.aulByLong[iLong] = pxVrbHeader->aulByLong[iLong] ;

        // Now crawl through the VRB Channels up to the requested channel pair
        if ( _ulVrb_Slot == xVrbHeader.xByField.ubSlotNum ) 
        {
            pxVrbChanLow = &pxVrbSection->axVrbChan[0] ;
            pxVrbChanHig = (VrbChannelMax*) ( (unsigned long) pxVrbChanLow + xVrbHeader.xByField.auwChanByteCount[0] ) ;

            unsigned int uiChanPair ;
            for ( uiChanPair = 0 ; uiChanPair < _ulVtm_Channel ; uiChanPair++ )
            {
                pxVrbChanLow = (VrbChannelMax*) ( (unsigned long) pxVrbChanHig + xVrbHeader.xByField.auwChanByteCount[2*uiChanPair+1] ) ;
                pxVrbChanHig = (VrbChannelMax*) ( (unsigned long) pxVrbChanLow + xVrbHeader.xByField.auwChanByteCount[2*uiChanPair+2] ) ;
            }

            uiChanPair = _ulVtm_Channel ;
            if ( ( xVrbHeader.xByField.auwChanByteCount[2*uiChanPair+0] != 4*_ulTotLong )
              || ( xVrbHeader.xByField.auwChanByteCount[2*uiChanPair+1] != 4*_ulTotLong ) ) 
            {
                CConsoleAccess::ErrorLine ( (CString) " Unexpected Length(s) for VRB Section #"
                                                       + I_to_A (uiVrbNum+1)
                                                       + " Slot #"
                                                       + I_to_A (_ulVrb_Slot)
                                                       + " Channel Pair #"
                                                       + I_to_A (2*uiChanPair+0) 
                                                       + "&" 
                                                       + I_to_A (2*uiChanPair+1) 
                                                       + " is " 
                                                       + I_to_A (xVrbHeader.xByField.auwChanByteCount[2*uiChanPair+0]) 
                                                       + "&" 
                                                       + I_to_A (xVrbHeader.xByField.auwChanByteCount[2*uiChanPair+1]) ) ;
            }
            break ;  // We are done scanning through the VBDs
        }
  
        //shift to the next VBD Section
        pxVrbSection = (VrbSectionInVbd*) ((unsigned long*) pxVrbSection 
                                                          + uwLongWordCountInVrbSection 
                                                          + 1 ) ;
    }
        
    if ( pxVrbChanLow == 0 ) 
    {
        CConsoleAccess::WriteLine ( (CString) " No VRB found for slot #"
                                              + I_to_A (_ulVrb_Slot) ) ;
        eReturnStatus = Error ;
        goto Abort ;
    }

	if ( ( (unsigned long) pxVrbChanHig + sizeof(VrbChannelMax) - (unsigned long) pxVbdBase ) > sizeof(Vbd_A24_Access_Buf) )
	{
        CConsoleAccess::ErrorLine ( (CString) " VRB Section #"
                                              + I_to_A (uiVrbNum+1, 2) 
                                              + " Channel Pair #"
                                              + I_to_A (2*_ulVtm_Channel+0) + "&" + I_to_A (2*_ulVtm_Channel+1)
                                              + " Would be past the amount of memory mapped for VBD" ) ;
		goto Abort ;
	}

    //Unpack the data
    if ( CHsroVrbCard::MergeOneChannelPair ( pxVrbChanLow, pxVrbChanHig, _pxActualData, _ulTotLong ) != Ok )
        eReturnStatus = Error ;

Abort:
    return eReturnStatus ;
}
/*-------------------------------------------------------------------------------*/

