/*-------------------------------------------------------------------------------*/
  /*
    HsroVbdCard.cpp
    Class CHsroVbdCard
    8-Sep-00  VBD Card
  */
/*-------------------------------------------------------------------------------*/

#include "L1Cal_IIb_TCC/StdAfx.h"

#include "L1Cal_IIb_TCC/GlobalConstants.h"
#include "L1Cal_IIb_TCC/CustomDataTypes.h"

#include                            "L1Cal_IIb_TCC/HsroVbdCard.h"

#include "L1Cal_IIb_TCC/HandleHardwareAccess.h" // for CheckBit3Status

//#include "L1Cal_IIb_TCC/HsroL1CalVrbAddresses.h"
#include "L1Cal_IIb_TCC/HsroVrbCard.h" // for addresses of some registers
    #include "L1Cal_IIb_TCC/HsroL1Cal.h"  

#if defined (TAILOR_VME_ACCESS)
    // VME_Access doesn't include HsroL1Cal.cpp
    CHsroL1Cal*       CHsroL1Cal::mg_poHsroL1Cal = NULL ; 
#else
    #include "L1Cal_IIb_TCC/L1Cal.h"  
#endif

#include "L1Cal_IIb_TCC/HandleConsole.h"

#include "L1Cal_IIb_TCC/UtilsStrings.h"
#include "L1Cal_IIb_TCC/UtilsBits.h"

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
CHsroVbdCard::CHsroVbdCard        ( unsigned long       _ulSpecificCardCode )  
{

    //\\//

    //capture the Card Code ID
    m_ulSpecificCardCode = _ulSpecificCardCode ;


    switch ( _ulSpecificCardCode ) 
    {
        case Ki_Id_Vbd_L1Cal :
            m_ubCard_Slot       = Ki_Slot_Vbd_L1Cal ;
            m_ulA16VmeAddr      = Ki_A16BaseAddr_Vbd_L1Cal ;
            m_ulA24VmeAddr      = Ki_A24BaseAddr_Vbd_L1Cal ;
            m_osCardDescription = "VBD for L1Cal Readout" ;
            break ;

        default :
            CConsoleAccess::InternalErrorLine    ( (CString) "Internal Error/CHsroVbdCard: "
                                                            + "Illegal Argument; Card Code "
                                                            + I_to_A (_ulSpecificCardCode) ) ;
    } 
    
    //Map the A16 Address Space (seen as A32 in Comm Crate) to Lower CSR
    ///////////////////////////////////////////////////////
    m_pxA16Access_Csr = (Vbd_A16_Access_Csr*) CHardwareAccess::mg_poBit3BusAdaptor
                                                        ->MemoryMapThisVmeSegment 
                                                            ( m_ulA16VmeAddr,
                                                              KiHsroVbdCardA16AddrSize_Csr ) ;

    //Sanity Check of the precise Data Structure 
    //carefully defined to access the VBD A16 registers
    if ( ( (unsigned char*) &m_pxA16Access_Csr->uwParameterReadAdressMsWord
         - (unsigned char*)  m_pxA16Access_Csr ) 
       != KiHsroVbdAddrReadMSWord  )
    CConsoleAccess::InternalErrorLine ( (CString) "Internal Error/CHsroVbdCard: "
                                                + "A16 Structure Definition Problem" ) ;

    //Map the A16 Address Space (seen as A32 in Comm Crate) to List of Pointers to Word Counts
    ///////////////////////////////////////////////////////
    m_pauwA16Access_WordCount = (unsigned short*) CHardwareAccess::mg_poBit3BusAdaptor
                                                        ->MemoryMapThisVmeSegment 
                                                            ( m_ulA16VmeAddr+KiHsroVbdCardA16AddrOffset_WordCount,
                                                              KiHsroVbdCardA16AddrSize_WordCount ) ;

    //Map the A16 Address Space (seen as A32 in Comm Crate) to List of Base Adrsses of Data Blocks
    ///////////////////////////////////////////////////////
    m_pauwA16Access_BaseData = (unsigned short*) CHardwareAccess::mg_poBit3BusAdaptor
                                                        ->MemoryMapThisVmeSegment 
                                                            ( m_ulA16VmeAddr+KiHsroVbdCardA16AddrOffset_BaseData,
                                                              KiHsroVbdCardA16AddrSize_BaseData ) ;

    //Map the A24 Address Space (seen as A32 in Comm Crate) for Data Buffer
    ///////////////////////////////////////////////////////
    m_pxA24Access_Buffer = (Vbd_A24_Access_Buf*) CHardwareAccess::mg_poBit3BusAdaptor
                                                        ->MemoryMapThisVmeSegment 
                                                            ( m_ulA24VmeAddr,
                                                              KiHsroVbdCardA24AddrSize_Buffer ) ;
    return ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
CHsroVbdCard::CHsroVbdCard        ( TulVmeAddress       _ulA24VmeAddr,
                                    CString             _osCardDescription )

{

    //\\//

    m_ulSpecificCardCode = 0 ;

    m_ubCard_Slot       = 0 ;
    m_ulA16VmeAddr      = 0 ;
    m_ulA24VmeAddr      = _ulA24VmeAddr ;
    m_osCardDescription = _osCardDescription ;
    
    //Map the A16 Address Space (seen as A32 in Comm Crate) to Lower CSR
    ///////////////////////////////////////////////////////
    m_pxA16Access_Csr = (Vbd_A16_Access_Csr*) CHardwareAccess::mg_poBit3BusAdaptor
                                                        ->MemoryMapThisVmeSegment 
                                                            ( m_ulA16VmeAddr,
                                                              KiHsroVbdCardA16AddrSize_Csr ) ;

    //Map the A16 Address Space (seen as A32 in Comm Crate) to List of Pointers to Word Counts
    ///////////////////////////////////////////////////////
    m_pauwA16Access_WordCount = (unsigned short*) CHardwareAccess::mg_poBit3BusAdaptor
                                                        ->MemoryMapThisVmeSegment 
                                                            ( m_ulA16VmeAddr+KiHsroVbdCardA16AddrOffset_WordCount,
                                                              KiHsroVbdCardA16AddrSize_WordCount ) ;

    //Map the A16 Address Space (seen as A32 in Comm Crate) to List of Base Adrsses of Data Blocks
    ///////////////////////////////////////////////////////
    m_pauwA16Access_BaseData = (unsigned short*) CHardwareAccess::mg_poBit3BusAdaptor
                                                        ->MemoryMapThisVmeSegment 
                                                            ( m_ulA16VmeAddr+KiHsroVbdCardA16AddrOffset_BaseData,
                                                              KiHsroVbdCardA16AddrSize_BaseData ) ;

    //Map the A24 Address Space (seen as A32 in Comm Crate) for Data Buffer
    ///////////////////////////////////////////////////////
    m_pxA24Access_Buffer = (Vbd_A24_Access_Buf*) CHardwareAccess::mg_poBit3BusAdaptor
                                                        ->MemoryMapThisVmeSegment 
                                                            ( m_ulA24VmeAddr,
                                                              KiHsroVbdCardA24AddrSize_Buffer ) ;

    return ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
CHsroVbdCard::~CHsroVbdCard       ()
{
    //UnMap the A16 Address Space (seen as A32 in Comm Crate) to Lower CSR
    CHardwareAccess::mg_poBit3BusAdaptor->UnMapThisVmeSegment  ( m_pxA16Access_Csr,
                                                                  KiHsroVbdCardA16AddrSize_Csr ) ;

    //UnMap the A16 Address Space (seen as A32 in Comm Crate) to List of Pointers to Word Counts
    CHardwareAccess::mg_poBit3BusAdaptor->UnMapThisVmeSegment  ( m_pauwA16Access_WordCount,
                                                                  KiHsroVbdCardA16AddrSize_WordCount ) ;

    //UnMap the A16 Address Space (seen as A32 in Comm Crate) to List of Base Adrsses of Data Blocks
    CHardwareAccess::mg_poBit3BusAdaptor->UnMapThisVmeSegment  ( m_pauwA16Access_BaseData,
                                                                  KiHsroVbdCardA16AddrSize_BaseData ) ;

    //UnMap the A24 Address Space (seen as A32 in Comm Crate) for Data Buffer
    ///////////////////////////////////////////////////////
    CHardwareAccess::mg_poBit3BusAdaptor->UnMapThisVmeSegment  ( m_pxA24Access_Buffer,
                                                                  KiHsroVbdCardA24AddrSize_Buffer ) ;
    return ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess CHsroVbdCard::Initialize        ( ) 
{
    ESuccess        eReturnStatus = Ok ;

    int             iVrbNum  ;

    //\\//

    CConsoleAccess::WriteLine ( (CString) " Initializing VBD Card @Slot#" 
                                         + I_to_A (m_ubCard_Slot) 
                                         + ": <" 
                                         + m_osCardDescription 
                                         + ">" ) ;


    //Un-Lock the VBD's CSR block and load DMA time-out.                           
    m_pxA16Access_Csr->uwCsr_0      = KiHsroVbbMaskLoadTimeOutAndUnlockVbd ;


    //Load the Crate Type
    m_pxA16Access_Csr->uwCrateType  = KiHsroVbbMaskCrateType ;


    //Load the address where the VBD can read the "event number" (on the VRBC)
    m_pxA16Access_Csr->uwAdressEventNumber = KiHsroVbbMaskAddrEventNum ;
 

    //Load the address where the VBD can read the Crate ID (in short I/O space)
    m_pxA16Access_Csr->uwAddressCrateID = KiHsroVbbMaskAddrCrateID ;


    //Tell the VBD what format to use to read the event data and Word Counts from the VRB's
    // ! Code for add. modifier for reading data
    m_pxA16Access_Csr->uwDataReadFormatControlReg = KiHsroVbbMaskDataReadFormat ;


    //Tell the VBD what format to use to read the "parameters"  
    // ! Code for Address modifier for P Control 
    // ! Crate ID/Event number access
    m_pxA16Access_Csr->uwParameterReadFormatControlReg = KiHsroVbbMaskParamReadFormat ;


    //Load the upper address bits that the VBD should use for parameter reads.
    m_pxA16Access_Csr->uwParameterReadAdressMsWord = KiHsroVbbMaskParamReadMsWord ;


    //Load the list of addresses where the VBD can read the word counts
    CConsoleAccess::WriteLine ( (CString) " _Loading Addresses of Word Counts" ) ;
    for ( iVrbNum = 0 ; iVrbNum < Ki_Tot_Vrb_L1Cal_Readout ; iVrbNum++ )
       m_pauwA16Access_WordCount[iVrbNum] = KiHsroVrbOffsetWordCount ;

    //End this list with two blocks of all zeros
    m_pauwA16Access_WordCount[Ki_Tot_Vrb_L1Cal_Readout+0] = 0x0000 ;
    m_pauwA16Access_WordCount[Ki_Tot_Vrb_L1Cal_Readout+1] = 0x0000 ;


   //Load the list of starting addresses
    CConsoleAccess::WriteLine ( (CString) " _Loading Addresses of Data Fifos" ) ;
    if ( CHsroL1Cal::mg_poHsroL1Cal == NULL )
    {
            CConsoleAccess::InternalErrorLine ( (CString) "Internal Error/CHsroVbdCard::Initialize: "
                                                        + "Null Pointer to L1Cal Readout Crate" ) ;
            eReturnStatus = Error ;
            goto Done ;
    }
    else for ( iVrbNum = 0 ; iVrbNum < Ki_Tot_Vrb_L1Cal_Readout ; iVrbNum++ )
    {
        m_pauwA16Access_BaseData[2*iVrbNum+0] = (unsigned short) 
                                                ( ( CHsroL1Cal::mg_poHsroL1Cal->m_paoVrb[iVrbNum]->f_ulA24VmeAddr() 
                                                  & KiHsroVrbMask24bit ) 
                                                  >> KiHsroVrbShiftDown16bits ) ;
        m_pauwA16Access_BaseData[2*iVrbNum+1] = KiHsroVrbOffsetOutputFifo ;
    }

    //End this list with two blocks of all zeros (not required)
    m_pauwA16Access_BaseData[2*Ki_Tot_Vrb_L1Cal_Readout+0] = 0x0000 ;
    m_pauwA16Access_BaseData[2*Ki_Tot_Vrb_L1Cal_Readout+1] = 0x0000 ;


    //Reset the VBD and clear its error FIFO 
    CConsoleAccess::WriteLine ( (CString) " _Resetting the VBD" ) ;
    m_pxA16Access_Csr->uwCsr_0      = KiHsroVbbMaskResetAndClearErrFifo ;


    //Wait about 2 to 5 mill seconds
    Sleep ( 5 ) ;


    //Finally  Load the VBD DMA time-out value and lock the VBD Control Memory.
    m_pxA16Access_Csr->uwCsr_0      = KiHsroVbbMaskLoadTimeOutAndLockVbd ;

Done:

    return eReturnStatus ;
}
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess CHsroVbdCard::Dump        ( unsigned long   _ulTotLongDump ) 
{
    ESuccess        eReturnStatus = Ok ;

    unsigned long   ulLongNum ;
    TulVmeAddress   ulVbdVmeAddr ;
    unsigned long*  paulVbdBase ;

    //\\//

#if defined ( VME_IO_FAKE )
    Vbd_A24_Access_Buf* pxVbdBase = m_pxA24Access_Buffer ;
    #include "L1Cal_IIb_TCC/HsroRawTestEvent.h"
#endif // VME_IO_FAKE

    CConsoleAccess::WriteLine ( (CString) " Dumping from VBD Card @Slot#" 
                                         + I_to_A (m_ubCard_Slot) 
                                         + ": <" 
                                         + m_osCardDescription 
                                         + ">" ) ;

    //Base address in the target crate
    ulVbdVmeAddr = m_ulA24VmeAddr & KiHsroVrbMask24bit ;

    //pointer to base address 
    paulVbdBase = (unsigned long*) m_pxA24Access_Buffer ;

    //Check that the request doesn't exceed the length of the buffer
    if ( _ulTotLongDump > sizeof(Vbd_A24_Access_Buf) )
    {
        CConsoleAccess::ErrorLine ( " Truncating dump length to the memory mapped for VBD Buffer" ) ;
        _ulTotLongDump = sizeof(Vbd_A24_Access_Buf) ;
    }

    // dump one longword at a time
    for ( ulLongNum  = 0 ; ulLongNum  < _ulTotLongDump ; ulLongNum ++ )
    CConsoleAccess::WriteLine ( (CString) " VBD Buffer Data @"
                                          + I_to_A ( ulVbdVmeAddr+4*ulLongNum, 6, 16, '0' )
                                          + " = "
                                          + I_to_A ( paulVbdBase[ulLongNum], 8, 16, '0', 4 ) ) ;

    return eReturnStatus ;
}
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess CHsroVbdCard::Analyze        ( ) 
{
    ESuccess        eReturnStatus = Ok ;

    VbdHeader*          pxVbdHeader ;   VbdHeader    xVbdHeader ;
    unsigned int        uiTotByteInEvent ;
    VrbSectionInVbd*    pxVrbSection ;
    unsigned short      uwLongWordCountInVrbSection ;
    VrbHeader*          pxVrbHeader ;   VrbHeader    xVrbHeader ;
    VrbChannelMax*      pxVrbChanLow ;
    VrbChannelMax*      pxVrbChanHig ;
    unsigned char       ubFirstVrbEventNum = 0 ;
    unsigned char       ubFirstHsroEventNum = 0 ;
    CNevisCard*         poNevisCard ;
    unsigned int        uiVrbNum ;
    CString             osCardID ;
    unsigned int        uiTotWordPerEvent ;
    unsigned int        uiBaseWord, uiRelWord ;
    bool                lFoundTickTurn = false ;
    unsigned char       ubTickNumber ;
    unsigned int        uwTurnNumber ;
    unsigned short      uwParity ;
    CString             osTemp ;

    //\\//

    CConsoleAccess::WriteLine ( "" ) ;
    CConsoleAccess::WriteLine ( (CString) " -------- VBD Event Formatted Dump and Analysis -------- " ) ;
    CConsoleAccess::WriteLine ( "" ) ;

    CConsoleAccess::WriteLine ( (CString) " Dumping from VBD Card @Slot#" 
                                         + I_to_A (m_ubCard_Slot) 
                                         + ": <" 
                                         + m_osCardDescription 
                                         + ">" ) ;
    CConsoleAccess::WriteLine ( "" ) ;

#if defined ( VME_IO_FAKE )
    Vbd_A24_Access_Buf* pxVbdBase = m_pxA24Access_Buffer ;
    #include "L1Cal_IIb_TCC/HsroRawTestEvent.h"
#endif // VME_IO_FAKE

    pxVbdHeader = &m_pxA24Access_Buffer->xByField.xVbdHeader ;

    // Transfer the VBD Header to Main Memory so that we do
    // D32 transfers from VBD to memory and avoid all D8 and D16 accesses to VBD
    // and minimize the number of accesses over the Bit3 Adaptor
    xVbdHeader.aulByLong[0] = pxVbdHeader->aulByLong[0] ;
    xVbdHeader.aulByLong[1] = pxVbdHeader->aulByLong[1] ;
    CConsoleAccess::WriteLine ( (CString) " VBD Header : Tot Event LongWord= "
                                  + I_to_A ( xVbdHeader.xByField.ulTotLongWordInEvent, 2, 16, '0')
                                  + " Crate ID= "
                                  + I_to_A ( xVbdHeader.xByField.uwCrateID, 2, 16, '0')
                                  + " L3 Transf Num= "
                                  + I_to_A ( xVbdHeader.xByField.uwL3TransferNum, 4, 16, '0') ) ;

    uiTotByteInEvent = 4 * xVbdHeader.xByField.ulTotLongWordInEvent ;
    if ( uiTotByteInEvent > sizeof(Vbd_A24_Access_Buf) )
    {
        CConsoleAccess::ErrorLine ( (CString) "            : Event Size larger than Expected Maximum" ) ;
        eReturnStatus = Error ;
    }

    if ( xVbdHeader.xByField.uwCrateID != KiCrateIdL1Cal2b )
    {
        CConsoleAccess::ErrorLine ( (CString) "            : Crate ID Error" ) ;
        eReturnStatus = Error ;
    }

    pxVrbSection = &m_pxA24Access_Buffer->xByField.axVrbSection[0] ;

    unsigned int uiMaxVrb ;
//    uiMaxVrb = Ki_Max_Vrb_in_Readout_Crate ;
    uiMaxVrb = Ki_Tot_Vrb_L1Cal_Readout ;

    for ( uiVrbNum = 0 ; uiVrbNum < uiMaxVrb ; uiVrbNum++ )
    {
        // Skip 1 longword of alignment in the SBC's emulation of the VBD readout buffer 
        pxVrbSection = (VrbSectionInVbd*) ( (unsigned long*) pxVrbSection + 1 ) ; 
        uiTotByteInEvent += 4 ; //adjust the event size so that checking is still consistant

        CConsoleAccess::WriteLine ( (CString) "" ) ; // Skip a line

        if ( ( (unsigned long) pxVrbSection - (unsigned long) m_pxA24Access_Buffer ) > uiTotByteInEvent )
        {
             CConsoleAccess::ErrorLine ( (CString) "    VRB #"
                                      + I_to_A (uiVrbNum+1, 2)
                                      + " : Would be past the end of the Event in the VBD" ) ;
             eReturnStatus = Error ;
             goto Abort ;
        }

        if ( ( (unsigned long) pxVrbSection + sizeof(VrbHeader) - (unsigned long) m_pxA24Access_Buffer ) > sizeof(Vbd_A24_Access_Buf) )
        {
            CConsoleAccess::ErrorLine ( (CString) "    VRB #"
                                      + I_to_A (uiVrbNum+1, 2)
                                      + " : Would be past the amount of memory mapped for VBD" ) ;
             eReturnStatus = Error ;
             goto Abort ;
        }

        // Transfer the VRB Section length to Main Memory so that we do
        // D32 transfers from VBD to memory and avoid all D8 and D16 accesses to VBD
        // and minimize the number of accesses over the Bit3 Adaptor
        WordCountInVbd xTemp ;
        xTemp.ulByLong = pxVrbSection->xLongWordCount.ulByLong ;
        uwLongWordCountInVrbSection = xTemp.xByField.uwLongWordCount ;

        if ( uwLongWordCountInVrbSection == 0 )
            goto Done ;

        CConsoleAccess::WriteLine ( (CString) "    VRB #"
                                      + I_to_A (uiVrbNum+1, 2)
                                      + " : Tot LongWord= "
                                      + I_to_A ( uwLongWordCountInVrbSection, 2, 16, '0') ) ;

        if ( uwLongWordCountInVrbSection < sizeof(VrbHeader)/4 )
        {
            CConsoleAccess::ErrorLine ( (CString) "            : This VRB Section is shorter than a VRB Header" ) ;
            eReturnStatus = Error ;
            goto Abort ;
        }

        pxVrbHeader = &pxVrbSection->xVrbHeader ;

        // Transfer the VBD Header to Main Memory so that we do
        // D32 transfers from VBD to memory and avoid all D8 and D16 accesses to VBD
        // and minimize the number of accesses over the Bit3 Adaptor
        for ( int iLong = 0 ; iLong < sizeof(VrbHeaderByField)/4 ; iLong++ )
            xVrbHeader.aulByLong[iLong] = pxVrbHeader->aulByLong[iLong] ;

        CConsoleAccess::WriteLine ( (CString) "  VRB Header : Tot Byte= "
                                      + I_to_A ( xVrbHeader.xByField.ulTotByteCount, 2, 16, '0')
                                      + " Event Num= "
                                      + I_to_A ( xVrbHeader.xByField.ubEventNumber, 2, 16, '0')
                                      + " Slot= "
                                      + I_to_A ( xVrbHeader.xByField.ubSlotNum, 2, 16, '0') ) ;

        if ( xVrbHeader.xByField.ulTotByteCount != (unsigned long) sizeof(int) * uwLongWordCountInVrbSection )
        {
            CConsoleAccess::ErrorLine ( (CString) "             : VRB Byte Count Error vs Section Length from VBD" ) ;
            eReturnStatus = Error ;
        }

        if ( uiVrbNum == 0 ) 
            ubFirstVrbEventNum = xVrbHeader.xByField.ubEventNumber ;

        if ( xVrbHeader.xByField.ubEventNumber != ubFirstVrbEventNum )
        {
            CConsoleAccess::ErrorLine ( (CString) "             : Event Number Mismatch from First Vrb Header" ) ;
            eReturnStatus = Error ;
        }

        if ( xVrbHeader.xByField.ubSlotNum != CHsroL1Cal::mg_poHsroL1Cal->m_paoVrb[uiVrbNum]->f_ubCard_Slot() )
        {
            CConsoleAccess::ErrorLine ( (CString) "             : VRB Slot Number Error, expected "
                                           + I_to_A ( CHsroL1Cal::mg_poHsroL1Cal->m_paoVrb[uiVrbNum]->f_ubCard_Slot(), 2, 16, '0') ) ;
            eReturnStatus = Error ;
        }

        CConsoleAccess::WriteLine ( (CString) "             : User Info= "
                                      + I_to_A ( xVrbHeader.xByField.uwUserInfo, 4, 16, '0')
                                      + " Config= "
                                      + I_to_A ( xVrbHeader.xByField.uwConfigInfo, 4, 16, '0')
                                      + " Firmware= "
                                      + I_to_A ( xVrbHeader.xByField.uwFirmwareVersion, 4, 16, '0') ) ;

/*        if ( xVrbHeader.xByField.uwUserInfo != CHsroL1Cal::mg_poHsroL1Cal->m_paoVrb[uiVrbNum]->f_uwUserInfo() )
            CConsoleAccess::ErrorLine ( (CString) "             : User Info Error "
                                           + I_to_A (xVrbHeader.xByField.uwUserInfo, 4, 16, '0')
                                           + " instead of " 
                                           + I_to_A (CHsroL1Cal::mg_poHsroL1Cal->m_paoVrb[uiVrbNum]->f_uwUserInfo(), 4, 16, '0') ) ;
*/        
//        if ( xVrbHeader.xByField.uwConfigInfo != 0x0002 )
//            CConsoleAccess::ErrorLine ( (CString) "             : Config Info Error, expected 0x0002" ) ;

        CConsoleAccess::WriteLine ( (CString) "             : Unused Status Word= "
                                      + I_to_A ( xVrbHeader.xByField.ulStatusWord, 8, 16, '0') ) ;

//        if ( xVrbHeader.xByField.ulStatusWord != 0x00000000 )
//            CConsoleAccess::ErrorLine ( (CString) "             : Status Error (should be unused = 0x00000000)" ) ;

        CConsoleAccess::WriteLine ( (CString) "             : Byte Count Ch # 1 = "
                                      + I_to_A ( xVrbHeader.xByField.auwChanByteCount[0], 4, 16, '0')
                                      + " Ch # 0 = "
                                      + I_to_A ( xVrbHeader.xByField.auwChanByteCount[1], 4, 16, '0') ) ;

        CConsoleAccess::WriteLine ( (CString) "             : Byte Count Ch # 3 = "
                                      + I_to_A ( xVrbHeader.xByField.auwChanByteCount[2], 4, 16, '0')
                                      + " Ch # 2 = "
                                      + I_to_A ( xVrbHeader.xByField.auwChanByteCount[3], 4, 16, '0') ) ;

        CConsoleAccess::WriteLine ( (CString) "             : Byte Count Ch # 5 = "
                                      + I_to_A ( xVrbHeader.xByField.auwChanByteCount[4], 4, 16, '0')
                                      + " Ch # 4 = "
                                      + I_to_A ( xVrbHeader.xByField.auwChanByteCount[5], 4, 16, '0') ) ;

        CConsoleAccess::WriteLine ( (CString) "             : Byte Count Ch # 7 = "
                                      + I_to_A ( xVrbHeader.xByField.auwChanByteCount[6], 4, 16, '0')
                                      + " Ch # 6 = "
                                      + I_to_A ( xVrbHeader.xByField.auwChanByteCount[7], 4, 16, '0') ) ;

        pxVrbChanLow = &pxVrbSection->axVrbChan[0] ;
        pxVrbChanHig = (VrbChannelMax*) ( (unsigned long) pxVrbChanLow + xVrbHeader.xByField.auwChanByteCount[0] ) ;

        for ( int uiChanPair = 0 ; uiChanPair < KiHsroVrbTotVtmLink ; uiChanPair++ )
        {
            CConsoleAccess::WriteLine ( (CString) "" ) ; // Skip a line

            if ( xVrbHeader.xByField.auwChanByteCount[2*uiChanPair] != 0 ) // no point in complaining if there is no data anyway
            if ( ( (unsigned long) pxVrbChanHig - (unsigned long) m_pxA24Access_Buffer ) > uiTotByteInEvent )
            {
                CConsoleAccess::ErrorLine ( (CString) "              : Channel Pair #"
                                               + I_to_A (2*uiChanPair+0) + "&" + I_to_A (2*uiChanPair+1)
                                               + " would be past the end of the Event in the VBD" ) ;
                eReturnStatus = Error ;
                goto Abort ;
            }

			if ( ( (unsigned long) pxVrbChanHig + sizeof(VrbChannelMax) - (unsigned long) m_pxA24Access_Buffer ) > sizeof(Vbd_A24_Access_Buf) )
			{
                CConsoleAccess::ErrorLine ( (CString) "              : Channel Pair #"
                                               + I_to_A (2*uiChanPair+0) + "&" + I_to_A (2*uiChanPair+1)
										  + " : Would be past the amount of memory mapped for VBD" ) ;
                eReturnStatus = Error ;
				goto Abort ;
			}


            poNevisCard = CHsroL1Cal::mg_poHsroL1Cal->m_paoVrb[uiVrbNum]->f_poNevisCard(uiChanPair) ;

            if ( poNevisCard == NULL )
            {
                if ( ( xVrbHeader.xByField.auwChanByteCount[2*uiChanPair+0] != 0 )
                  || ( xVrbHeader.xByField.auwChanByteCount[2*uiChanPair+1] != 0 ) )
                {
                    CConsoleAccess::ErrorLine ( (CString) "              : Unexpected Data for Channel Pair #"
                                                   + I_to_A (2*uiChanPair+0) + "&" + I_to_A (2*uiChanPair+1) ) ;
                    eReturnStatus = Error ;
                }
            }
            else 
            {
                if ( ( xVrbHeader.xByField.auwChanByteCount[2*uiChanPair+0] != poNevisCard->f_uwHsroTotWord() )
                  || ( xVrbHeader.xByField.auwChanByteCount[2*uiChanPair+1] != poNevisCard->f_uwHsroTotWord() ) )
                {
                    CConsoleAccess::ErrorLine ( (CString) "              : Incorrect Length(s) for Channel Pair #"
                                                   + I_to_A (2*uiChanPair+0) + "&" + I_to_A (2*uiChanPair+1) ) ;
                    CConsoleAccess::ErrorLine ( (CString) "         i.e. <"
                                                   + poNevisCard->m_osCardDescription
                                                   + "> expected " 
                                                   + I_to_A ( poNevisCard->f_uwHsroTotWord(), 4, 16, '0' ) 
                                                   + " bytes per channel" ) ;
                    eReturnStatus = Error ;

                    // We will not decode this as a Nevis Card
                    poNevisCard = NULL ;
                }
            }


            // Display content information any time a Channel Pair has data
            if ( ( xVrbHeader.xByField.auwChanByteCount[2*uiChanPair+0] != 0 )
              || ( xVrbHeader.xByField.auwChanByteCount[2*uiChanPair+1] != 0 ) )
            {
				if ( poNevisCard != NULL )
					CConsoleAccess::WriteLine ( (CString) "       Ch#"
												   + I_to_A (2*uiChanPair+0) + "&" + I_to_A (2*uiChanPair+1)
												   + " : <"
												   + poNevisCard->m_osCardDescription
												   + ">" ) ;
				else
					CConsoleAccess::WriteLine ( (CString) "       Ch#"
												   + I_to_A (2*uiChanPair+0) + "&" + I_to_A (2*uiChanPair+1)
												   + " :" ) ;

                {
                    //figure out how many lognwords per channel to decode, while rounding up.
                    unsigned long ulTotBytePerChan = xVrbHeader.xByField.auwChanByteCount[2*uiChanPair+0] ;
                    // each channel contributes a byte, adding up to a word for each byte,  to compare to the max number of longwords
                    if ( ulTotBytePerChan > 2*KiHsroMaxLongPerCard ) ulTotBytePerChan = 2*KiHsroMaxLongPerCard ;
                    unsigned long ulTotLongPerChan = ( ulTotBytePerChan + 3 ) / 4 ;
                    OneCardHsroDataMax  xCardData ;
                    CHsroVrbCard::MergeOneChannelPair ( pxVrbChanLow, pxVrbChanHig, &xCardData, ulTotLongPerChan ) ;

                    if ( poNevisCard != NULL ) 
                    {
                        if ( lFoundTickTurn == false )
                        {
                            lFoundTickTurn = true ;
                            ubTickNumber   = xCardData.xByField.xHeader.xByField.ubTickNumber ;
                            uwTurnNumber   = xCardData.xByField.xHeader.xByField.uwTurnNumber;
                        }

                        //
					    CConsoleAccess::WriteLine ( (CString) "    L2 Header :"
    												        + " Header Len="
												            + I_to_A (xCardData.xByField.xHeader.xByField.ubHeaderLength, 2) 
    												        + "  Tot Obj="
												            + I_to_A (xCardData.xByField.xHeader.xByField.ubNumberObjects, 2) 
    												        + "  Obj Len="
												            + I_to_A (xCardData.xByField.xHeader.xByField.ubObjectLength, 2) 
    												        + "  Head Fmt="
												            + I_to_A ((xCardData.xByField.xHeader.xByField.ubHObjFormat & 0xe0)>>5, 2)
    												        + "  Obj Fmt="
												            + I_to_A (xCardData.xByField.xHeader.xByField.ubHObjFormat & 0x1f, 2)
     												        + "" ) ;

						if ( xCardData.xByField.xHeader.xByField.ubHeaderLength != KiHsroLongInHeader )
                        {
                            CConsoleAccess::ErrorLine ( (CString) "              : Unexpected L2 Header Length" ) ;
                            eReturnStatus = Error ;
                        }
						if ( xCardData.xByField.xHeader.xByField.ubNumberObjects != 1 )
                        {
                            CConsoleAccess::ErrorLine ( (CString) "              : Unexpected Number of L2 Objects" ) ;
                            eReturnStatus = Error ;
                        }
						if ( ( ( poNevisCard->f_ubModuleNum() == KiModuleNumGab ) 
                            && ( xCardData.xByField.xHeader.xByField.ubObjectLength != KiHsroLongInGabEvent ) )
                          || ( ( poNevisCard->f_ubModuleNum() != KiModuleNumGab ) 
                            && ( xCardData.xByField.xHeader.xByField.ubObjectLength != KiHsroLongInTabEvent ) ) )
                        {
                            CConsoleAccess::ErrorLine ( (CString) "              : Unexpected L2 Object Length" ) ;
                            eReturnStatus = Error ;
                        }

					    CConsoleAccess::WriteLine ( (CString) "    L2 Header :"
    												        + " L2 Type= "
												            + I_to_A (xCardData.xByField.xHeader.xByField.ubL2DataType, 2, 16, '0')
    												        + "  BeamX Tick= "
												            + I_to_A (xCardData.xByField.xHeader.xByField.ubTickNumber, 2, 16, '0')
    												        + "  BeamX Turn= "
												            + I_to_A (xCardData.xByField.xHeader.xByField.uwTurnNumber, 4, 16, '0')
      												        + "" ) ;

						if ( xCardData.xByField.xHeader.xByField.ubL2DataType != KiBaseL2Type + poNevisCard->f_ubModuleNum() )
                        {
                            CConsoleAccess::ErrorLine ( (CString) "              : Mismatch L2 Type; expected " 
                                                         + I_to_A ( KiBaseL2Type + poNevisCard->f_ubModuleNum(), 4, 16, '0') ) ;
                            eReturnStatus = Error ;
                        }

						if ( ( xCardData.xByField.xHeader.xByField.ubTickNumber != ubTickNumber )
						  || ( xCardData.xByField.xHeader.xByField.uwTurnNumber != uwTurnNumber ) )
                        {
                            CConsoleAccess::ErrorLine ( (CString) "              : Mismatch Tick or Turn Number vs first encountered" ) ;
                            eReturnStatus = Error ;
                        }

					    CConsoleAccess::WriteLine ( (CString) "    L2 Header :"
   												            + " Firmware= V"
												            + I_to_A (xCardData.xByField.xHeader.xByField.ubVersionMajor)
    												        + "."
												            + I_to_A (xCardData.xByField.xHeader.xByField.ubVersionMinor)
    												        + "  Proc Bits= "
												            + I_to_A (xCardData.xByField.xHeader.xByField.ubProcBits, 4, 16, '0')
    												        + "  L2 Stat= "
												            + I_to_A (xCardData.xByField.xHeader.xByField.ubL2Status, 4, 16, '0')
     												        + "" ) ;

                        if ( poNevisCard->f_ubModuleNum() == KiModuleNumGab ) 
                            uiTotWordPerEvent = KiHsroWordInGabEvent ;
                        else
                            uiTotWordPerEvent = KiHsroWordInTabEvent ;

                        //display each un-scrambled word
                        for ( uiBaseWord = 0 ; uiBaseWord < uiTotWordPerEvent ;  uiBaseWord += 8 )
                        {
                            osTemp = "" ;
                            for ( uiRelWord = 0 ; 
                                ( uiRelWord < 8 ) & ( uiBaseWord+uiRelWord < uiTotWordPerEvent ) ; 
                                  uiRelWord += 2 )
                                osTemp += I_to_A ( xCardData.xByField.xEventData.auwByWord[uiBaseWord+uiRelWord],
                                                   4, 16, '0')
                                        + " " 
                                        + I_to_A ( xCardData.xByField.xEventData.auwByWord[uiBaseWord+uiRelWord+1],
                                                   4, 16, '0')
                                        + "  " ;

                            CConsoleAccess::WriteLine ( (CString) " Data "
                                                          + I_to_A ( uiBaseWord+0, 3 )
                                                          + "-"
                                                          + I_to_A ( uiBaseWord+uiRelWord-1, 3 )
                                                          + " : "
                                                          + osTemp ) ;
                        }

                        if ( poNevisCard->f_ubModuleNum() == KiModuleNumGab ) 
                        {
                            OneGabHsroData*     pxGabData = (OneGabHsroData*) &xCardData ;
					        CConsoleAccess::WriteLine ( (CString) "   L2 Trailer :"
    												            + " L2 Type= "
												                + I_to_A (pxGabData->xByField.xTrailer.xByField.ubL2DataType, 3)
    												            + "  Beam X Tick= "
												                + I_to_A (pxGabData->xByField.xTrailer.xByField.ubTickNumber, 2, 16, '0')
    												            + "  Parity= "
												                + I_to_A (pxGabData->xByField.xTrailer.xByField.uwParity, 4, 16, '0')
     												            + "" ) ;

                            if ( pxGabData->xByField.xTrailer.xByField.ubTickNumber != xCardData.xByField.xHeader.xByField.ubTickNumber )
                            {
                                CConsoleAccess::ErrorLine ( (CString) "              : Mismatch Header vs Trailer Beam Crossing Tick Number" ) ;
                                eReturnStatus = Error ;
                            }

                            if ( pxGabData->xByField.xTrailer.xByField.ubL2DataType != xCardData.xByField.xHeader.xByField.ubL2DataType )
                            {
                                CConsoleAccess::ErrorLine ( (CString) "              : Mismatch Header vs Trailer L2 Data Type" ) ;
                                eReturnStatus = Error ;
                            }

                            uwParity = 0 ;
                            for ( uiBaseWord = 0 ; uiBaseWord < KiHsroWordInGabForParity ;  uiBaseWord ++ )
                                uwParity ^= pxGabData->auwByWord[uiBaseWord] ;

                            if ( uwParity != pxGabData->xByField.xTrailer.xByField.uwParity )
                            {
                                CConsoleAccess::ErrorLine ( (CString) "              : Parity Error; expected "
                                                                    + I_to_A ( uwParity, 4, 16, '0') ) ;
                                eReturnStatus = Error ;
                            }
                        }
                        else
                        {
                            OneTabHsroData*     pxTabData = (OneTabHsroData*) &xCardData ;
					        CConsoleAccess::WriteLine ( (CString) "   L2 Trailer :"
    												            + " L2 Type= "
												                + I_to_A (pxTabData->xByField.xTrailer.xByField.ubL2DataType, 2, 16, '0')
    												            + "  Beam X Tick= "
												                + I_to_A (pxTabData->xByField.xTrailer.xByField.ubTickNumber, 2, 16, '0')
    												            + "  Parity= "
												                + I_to_A (pxTabData->xByField.xTrailer.xByField.uwParity, 4, 16, '0')
     												            + "" ) ;

                            if ( pxTabData->xByField.xTrailer.xByField.ubTickNumber != xCardData.xByField.xHeader.xByField.ubTickNumber )
                            {
                                CConsoleAccess::ErrorLine ( (CString) "              : Mismatch Header vs Trailer Beam Crossing Tick Number" ) ;
                                eReturnStatus = Error ;
                            }

                            if ( pxTabData->xByField.xTrailer.xByField.ubL2DataType != xCardData.xByField.xHeader.xByField.ubL2DataType )
                            {
                                CConsoleAccess::ErrorLine ( (CString) "              : Mismatch Header vs Trailer L2 Data Type" ) ;
                                eReturnStatus = Error ;
                            }

                            uwParity = 0 ;
                            for ( uiBaseWord = 0 ; uiBaseWord < KiHsroWordInTabForParity ;  uiBaseWord ++ )
                                uwParity ^= pxTabData->auwByWord[uiBaseWord] ;

                            if ( uwParity != pxTabData->xByField.xTrailer.xByField.uwParity )
                            {
                                CConsoleAccess::ErrorLine ( (CString) "              : Parity Error; expected "
                                                                    + I_to_A ( uwParity, 4, 16, '0') ) ;
                                eReturnStatus = Error ;
                            }

                            //display each un-scrambled padding word
                            for ( uiBaseWord = 0 ; uiBaseWord < KiHsroWordInTabPadding ;  uiBaseWord += 8 )
                            {
                                osTemp = "" ;
                                for ( uiRelWord = 0 ; 
                                    ( uiRelWord < 8 ) & ( uiBaseWord+uiRelWord < KiHsroWordInTabPadding ) ; 
                                      uiRelWord += 2 )
                                    osTemp += I_to_A ( pxTabData->xByField.auwPaddingByWord[uiBaseWord+uiRelWord],
                                                       4, 16, '0')
                                            + " " 
                                            + I_to_A ( pxTabData->xByField.auwPaddingByWord[uiBaseWord+uiRelWord+1],
                                                       4, 16, '0')
                                            + "  " ;

                                CConsoleAccess::WriteLine ( (CString) " Padd "
                                                              + I_to_A ( uiBaseWord+0, 3 )
                                                              + "-"
                                                              + I_to_A ( uiBaseWord+uiRelWord-1, 3 )
                                                              + " : "
                                                              + osTemp ) ;
                            }

                        }
                    }
                    else
                    {
                        //display each un-scrambled word
                        for ( uiBaseWord = 0 ; uiBaseWord < ulTotBytePerChan ;  uiBaseWord += 8 )
                        {
                            osTemp = "" ;
                            for ( uiRelWord = 0 ; 
                                ( uiRelWord < 8 ) & ( uiBaseWord+uiRelWord < ulTotBytePerChan ) ; 
                                  uiRelWord += 2 )
                                osTemp += I_to_A ( xCardData.auwByWord[uiBaseWord+uiRelWord],
                                                   4, 16, '0')
                                        + " " 
                                        + I_to_A ( xCardData.auwByWord[uiBaseWord+uiRelWord+1],
                                                   4, 16, '0')
                                        + "  " ;

                            CConsoleAccess::WriteLine ( (CString) " Word "
                                                          + I_to_A ( uiBaseWord+0, 3 )
                                                          + "-"
                                                          + I_to_A ( uiBaseWord+uiRelWord-1, 3 )
                                                          + " : "
                                                          + osTemp ) ;
                        }
                    }
                }
            }

            // Scoot to the next channel Pair
            pxVrbChanLow = (VrbChannelMax*) ( (unsigned long) pxVrbChanHig + xVrbHeader.xByField.auwChanByteCount[2*uiChanPair+1] ) ;
            pxVrbChanHig = (VrbChannelMax*) ( (unsigned long) pxVrbChanLow + xVrbHeader.xByField.auwChanByteCount[2*uiChanPair+2] ) ;
        }



        //shift to the next VBD Section
        pxVrbSection = (VrbSectionInVbd*) ((unsigned long*) pxVrbSection
                                                          + uwLongWordCountInVrbSection
                                                          + 1 ) ;

    }

Done :

    if ( eReturnStatus != Ok ) 
    	CConsoleAccess::ErrorLine ( (CString) " Error detected during VBD Event Dump Analysis" ) ;
    else
    	CConsoleAccess::WriteLine ( (CString) " No Error detected during VBD Event Dump Analysis" ) ;


    return eReturnStatus ;

Abort:

	CConsoleAccess::ErrorLine ( (CString) " Aborting VBD Event Dump Analysis" ) ;

    return eReturnStatus ;
}
/*-------------------------------------------------------------------------------*/
