/*-------------------------------------------------------------------------------*/
  /*
    HsroVrbCard.cpp
    Class CHsroVrbCard
    5-Sep-00  VRB Card
  */
/*-------------------------------------------------------------------------------*/

#include "L1Cal_IIb_TCC/StdAfx.h"

#include "L1Cal_IIb_TCC/GlobalConstants.h"
#include "L1Cal_IIb_TCC/CustomDataTypes.h"

#include                            "L1Cal_IIb_TCC/HsroVrbCard.h"

#include "L1Cal_IIb_TCC/HandleHardwareAccess.h" // for CheckBit3Status

#include "L1Cal_IIb_TCC/HsroL1CalVrbAddresses.h"

#include "L1Cal_IIb_TCC/NevisCard.h"

#include "L1Cal_IIb_TCC/HandleConsole.h"
#include "L1Cal_IIb_TCC/UtilsStrings.h"
#include "L1Cal_IIb_TCC/DataManip.h"

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
CHsroVrbCard::CHsroVrbCard        ( unsigned long       _ulSpecificCardCode,
                                    CHsroVrbCard**      _ppoPrevCard )  
{

    //\\//

    //capture the Card Code ID
    m_ulSpecificCardCode = _ulSpecificCardCode ;

    // grab the pointer to the previous Card in the chain,
    // and return a pointer to *this* Card to be used as the next "previous card"
    // or as the last card recorded for this Crate.
    if ( _ppoPrevCard != NULL ) // but only if this optional parameter was provided
    {
        m_poPrevCard    = *_ppoPrevCard ;
        *_ppoPrevCard = this ;
    }
    else
    {
        m_poPrevCard = NULL ;
    }

    //
    switch ( _ulSpecificCardCode ) 
    {
        case Ki_Id_Vrb_L1Cal_0 : // Ki_Id_Vrb_L1Cal_0 :
            m_ubCard_Slot   = Ki_Slot_Vrb_L1Cal_0 ;
            m_ulA24VmeAddr  = Ki_BaseAddr_Vrb_L1Cal_0 ;
            m_osCardDescription   = "L1Cal VRB #0" ;
        break ;

        case Ki_Id_Vrb_L1Cal_1 : // Ki_Id_Vrb_L1Cal_1 :
            m_ubCard_Slot   = Ki_Slot_Vrb_L1Cal_1 ;
            m_ulA24VmeAddr  = Ki_BaseAddr_Vrb_L1Cal_1 ;
            m_osCardDescription   = "L1Cal VRB #1" ;
        break ;

        case Ki_Id_Vrb_L1Cal_2 : // Ki_Id_Vrb_L1Cal_2 :
            m_ubCard_Slot   = Ki_Slot_Vrb_L1Cal_2 ;
            m_ulA24VmeAddr  = Ki_BaseAddr_Vrb_L1Cal_2 ;
            m_osCardDescription   = "L1Cal VRB #2" ;
        break ;

        default :
            CConsoleAccess::InternalErrorLine ( (CString) "Internal Error/CHsroVrbCard: "
                                                        + "Illegal Argument; Card Code "
                                                        + I_to_A (_ulSpecificCardCode) ) ;

    } 
    
    //Map the A24 Address Space (seen as A32 in Comm Crate)
    ///////////////////////////////////////////////////////
    m_pxA24Access = (Vrb_A24_Access*) CHardwareAccess::mg_poBit3BusAdaptor
                                                        ->MemoryMapThisVmeSegment 
                                                            ( m_ulA24VmeAddr,
                                                              KiHsroVrbCardAddressSize ) ;

    //Sanity Check of the precise Data Structure 
    //carefully defined to access the VRB registers
    if ( ( (unsigned char*) &m_pxA24Access->uwBufferSize[0] 
         - (unsigned char*)  m_pxA24Access ) 
       != KiHsroVrbBaseAddrBuffSize )
    CConsoleAccess::InternalErrorLine ( (CString) "Internal Error/CHsroVrbCard: "
                                                + "Structure Definition Problem" ) ;

    //Initialize the pointer to the The Card connected to each channel
    for ( int i = 0 ; i < KiHsroVrbTotVtmLink ; i++ )
        m_apoNevisCard[i] = NULL ;

    return ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
CHsroVrbCard::~CHsroVrbCard       ()
{
    //UnMap the A24 Address Space (seen as A32 in Comm Crate)
    ///////////////////////////////////////////////////////
    CHardwareAccess::mg_poBit3BusAdaptor->UnMapThisVmeSegment  ( m_pxA24Access,
                                                                  KiHsroVrbCardAddressSize ) ;

    return ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess CHsroVrbCard::Initialize        ( ) 
{
    ESuccess        eReturnStatus = Ok ;

    unsigned short  uwChannelEnableMask ;
    unsigned short  uwStatus ;

    //\\//

    //Advertize Card Name
    CConsoleAccess::WriteLine ( (CString) " Initializing VRB Card @Slot#" 
                                         + I_to_A (m_ubCard_Slot) 
                                         + ": <" 
                                         + m_osCardDescription 
                                         + ">" ) ;

    //Advertize Channel Usage
    //and prepare the Channel Enable Control Mask
    uwChannelEnableMask = 0 ;
    for ( int i = 0 ; i < KiHsroVrbTotVtmLink ; i++ )
    if ( m_apoNevisCard[i] != NULL )
    {
        CConsoleAccess::WriteLine ( (CString) "   Link#"
                                             + I_to_A (i)
                                             + ": <" 
                                             + m_apoNevisCard[i]->m_osCardDescription 
                                             + ">" ) ;
        uwChannelEnableMask |= KiHsroVrbMaskEnableChanPair << KiHsroVrbMaskEnableShiftSize * i ;
    }    


    //Read Card ID and SN
    //and Firmware Revision Date (0xYMDD where; Y = last digit of year
    //                                          M = month (January = 0x1...December = 0xC)
    //                                          DD = day)
    CConsoleAccess::WriteLine ( (CString) " _Card ID Read from VRB is "
                                         + I_to_A (m_pxA24Access->uwVrbId, 4, 16, '0') 
                                         + " Serial Number "
                                         + I_to_A (m_pxA24Access->uwModuleSerialNumber, 4, 16, '0') 
                                         + " Firmware Rev "
                                         + I_to_A (m_pxA24Access->uwDateCode, 4, 16, '0') ) ;

    //Enable Channels actually being used
    CConsoleAccess::WriteLine ( (CString) " _Setting Channel Enable to "
                                         + I_to_A (uwChannelEnableMask, 4, 16, '0') ) ;
    m_pxA24Access->uwChannelEnable = uwChannelEnableMask ;

    //Set NO Emulation
    m_pxA24Access->uwEmulationModeEnable = KiHsroVrbNoEmulation ;

    //Set NO Gray Coding
    m_pxA24Access->uwGrayCodeEnable = KiHsroVrbNoGrayCoding ;

    //Enable Control/Status for VRBC on J5/J6
    m_pxA24Access->uwVrbControl = KiHsroVrbEnableVrbcSignals ;

    // Re-write the default list of starting addresses and lengths for 16x 2kB VRB buffers
    // These addresses will be ingested at the next VRB Reset.
    for ( int iBufferNum = 0 ; iBufferNum < KiHsroVrbTotBuffer ; iBufferNum++ )
    {
        m_pxA24Access->uwBufferStartAddress[iBufferNum]= iBufferNum * KiHsroVrbBufferSize ;
        m_pxA24Access->uwBufferSize[iBufferNum]        = KiHsroVrbBufferSize ;
    }

    //Write Vrb Header User Data
    m_uwUserInfo = 0 ;
    m_pxA24Access->uwUserInfo = m_uwUserInfo ;


    //Reset VRB and VTM
    CConsoleAccess::WriteLine ( (CString) " _ReSetting VRB and VTM" ) ;
    m_pxA24Access->uwVrbReset = KiHsroVrbResetVrb | KiHsroVrbResetVtm ;


    // Wait "a few microseconds"
    Sleep ( 50 ) ; 


    //Read VTM Link Power 
    CConsoleAccess::WriteLine ( (CString) " _Optical Power for VTM Link#0/1/2/3= "
                                         + I_to_A (m_pxA24Access->uwVtmLinkPower[0], 3, 16, '0') 
                                         + " / "
                                         + I_to_A (m_pxA24Access->uwVtmLinkPower[1], 3, 16, '0') 
                                         + " / "
                                         + I_to_A (m_pxA24Access->uwVtmLinkPower[2], 3, 16, '0') 
                                         + " / "
                                         + I_to_A (m_pxA24Access->uwVtmLinkPower[3], 3, 16, '0') ) ;


    //Read Current Status and Reset latched version
    uwStatus = m_pxA24Access->uwCurrentStatus ;
    if ( uwStatus != KiHsroVrbStatusClear  ) // Do we want a different test here?
        CConsoleAccess::ErrorLine ( (CString) " _Current Status is "
                                             + I_to_A (uwStatus, 4, 16, '0') ) ;
    else
        CConsoleAccess::WriteLine ( (CString) " _Current Status is "
                                             + I_to_A (uwStatus, 4, 16, '0') ) ;


    return eReturnStatus ;
}
/*-------------------------------------------------------------------------------*/


/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess CHsroVrbCard::MergeOneChannelPair( VrbChannelMax*      _pxVrbChanLow,
                                            VrbChannelMax*      _pxVrbChanHig,
                                            OneCardHsroDataMax* _pxActualData,
                                            unsigned long       _ulTotLongPerChan ) 
{
    ESuccess            eReturnStatus = Ok ;
    VrbChannelMax       xVrbChanLow ;
    VrbChannelMax       xVrbChanHig ;
    MultiAccessShort    xVrbData ;
    unsigned short      uwHsroWordNum ;
    unsigned short      uwByteIndex ;

    //\\//

    if ( _ulTotLongPerChan > KiHsroVrbMaxLongPerChan ) 
    {
        CConsoleAccess::ErrorLine ( (CString) " Truncating dump to " 
                                   + I_to_A ( 2*4* KiHsroVrbMaxLongPerChan )
                                   + " bytes per VRB Channel Pair" ) ;
        _ulTotLongPerChan = KiHsroVrbMaxLongPerChan ;
    }

    // Transfer the VRB Channel Data to Main Memory so that we do 
    // D32 transfers from VBD to memory and avoid all D8 and D16 accesses to VBD
    // and minimize the number of accesses over the Bit3 Adaptor
    for ( unsigned int i = 0 ; i < _ulTotLongPerChan ; i ++ ) 
    {
        xVrbChanLow.aulByLong[i] = _pxVrbChanLow->aulByLong[i] ;
        xVrbChanHig.aulByLong[i] = _pxVrbChanHig->aulByLong[i] ;
    }    

    //splice it back together
    for ( uwHsroWordNum = 0 ; 
          uwHsroWordNum < 4 * _ulTotLongPerChan ;
          uwHsroWordNum++ )
    {
        //put together the next HSRO word
        xVrbData.uwWord = 0 ;

        // We have to find the index in the byte array 
        // (because there is byte swapping within each longword)
        // The actual ordering is 3,2,1,0, 7,6,5,4, 11,10,9,8, ... 
        // A quick way to achieve this is: just flip the lower two bits
        uwByteIndex = uwHsroWordNum ^ 0x0003 ;

        // Now we can grab the two halves of the desired word (using D8 from memory)
        xVrbData.ByByte.ubByte_0 = xVrbChanLow.aubByByte[uwByteIndex] ;
        xVrbData.ByByte.ubByte_1 = xVrbChanHig.aubByByte[uwByteIndex] ;
        _pxActualData->auwByWord[uwHsroWordNum] = xVrbData.uwWord ;
    }

    return eReturnStatus ;
}
/*-------------------------------------------------------------------------------*/
