/*-------------------------------------------------------------------------------*/
  /*
    HsroVrbContrCard.cpp
    Class CHsroVrbContrCard
    7-Sep-00  VRB Controller Card
  */
/*-------------------------------------------------------------------------------*/

#include "L1Cal_IIb_TCC/StdAfx.h"
#include "L1Cal_IIb_TCC/GlobalConstants.h"
#include "L1Cal_IIb_TCC/CustomDataTypes.h"

#include                            "L1Cal_IIb_TCC/HsroVrbContrCard.h"

#include "L1Cal_IIb_TCC/HandleHardwareAccess.h" // for CheckBit3Status

#include "L1Cal_IIb_TCC/HsroL1CalVrbAddresses.h"

#include "L1Cal_IIb_TCC/HandleConsole.h"
#include "L1Cal_IIb_TCC/UtilsStrings.h"

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
CHsroVrbContrCard::CHsroVrbContrCard        ( unsigned long       _ulSpecificCardCode )  
{

    //\\//

    //capture the Card Code ID
    m_ulSpecificCardCode = _ulSpecificCardCode ;


    switch ( _ulSpecificCardCode ) 
    {
        case Ki_Id_Vrb_L1Cal_Controller :
            m_ubCard_Slot       = Ki_Slot_Vrb_L1Cal_Controller ;
            m_ulA24VmeAddr      = Ki_A24BaseAddr_Vrb_L1Cal_Controller ;
            m_ulA16VmeAddr      = Ki_A16BaseAddr_Vrb_L1Cal_Controller ;
            m_osCardDescription = "VRB Controller for L1Cal Readout" ;
            break ;

        default :
            CConsoleAccess::InternalErrorLine ( (CString) "Internal Error/CHsroVrbContrCard: Illegal Argument; Card Code "
                                                 + I_to_A (_ulSpecificCardCode) ) ;

    } 
    
    //Map the A24 Address Space (seen as A32 in Comm Crate)
    ///////////////////////////////////////////////////////
    m_pxA24Access = (VrbC_A24_Access*) CHardwareAccess::mg_poBit3BusAdaptor
                                                        ->MemoryMapThisVmeSegment 
                                                            ( m_ulA24VmeAddr,
                                                              KiHsroVrbContrCardA24AddrSize ) ;

    //Sanity Check of the precise Data Structure 
    //carefully defined to access the VRB Controller A24 registers
    if ( ( (unsigned char*) &m_pxA24Access->ubSclStatus 
         - (unsigned char*)  m_pxA24Access ) 
       != KiHsroVrbContrAddrSclStatus )
    CConsoleAccess::InternalErrorLine ( (CString) "Internal Error/CHsroVrbContrCard: A24 Structure Definition Problem" ) ;


    //Map the A16 Address Space (seen as A32 in Comm Crate)
    ///////////////////////////////////////////////////////
    m_pxA16Access = (VrbC_A16_Access*) CHardwareAccess::mg_poBit3BusAdaptor
                                                        ->MemoryMapThisVmeSegment 
                                                            ( m_ulA16VmeAddr,
                                                              KiHsroVrbContrCardA16AddrSize ) ;

    //Sanity Check of the precise Data Structure 
    //carefully defined to access the VRB Controller A16 registers
    if ( ( (unsigned char*) &m_pxA16Access->uwL3TransfNum 
         - (unsigned char*)  m_pxA16Access ) 
       != KiHsroVrbContrShortIOL3Transf )
    CConsoleAccess::InternalErrorLine ( (CString) "Internal Error/CHsroVrbContrCard: A16 Structure Definition Problem" ) ;

    return ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
CHsroVrbContrCard::~CHsroVrbContrCard       ()
{
    //UnMap the A24 Address Space (seen as A32 in Comm Crate)
    CHardwareAccess::mg_poBit3BusAdaptor->UnMapThisVmeSegment  ( m_pxA24Access,
                                                                  KiHsroVrbContrCardA24AddrSize ) ;

    //UnMap the A16 Address Space (seen as A32 in Comm Crate)
    CHardwareAccess::mg_poBit3BusAdaptor->UnMapThisVmeSegment  ( m_pxA16Access,
                                                                  KiHsroVrbContrCardA16AddrSize ) ;

    return ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess CHsroVrbContrCard::Initialize        ( ) 
{
    ESuccess        eReturnStatus = Ok ;
    unsigned char   ubStatus ;

    //\\//

    CConsoleAccess::WriteLine ( (CString) " Initializing VRB Controller Card @Slot#" 
                                         + I_to_A (m_ubCard_Slot) 
                                         + ": <" 
                                         + m_osCardDescription 
                                         + ">" ) ;


    //Read Card ID and SN
    CConsoleAccess::WriteLine ( (CString) " _Card ID Read from VRBC is "
                                         + I_to_A (m_pxA16Access->uwCrateID, 4, 16, '0')
                                         + " Serial Number "
                                         + I_to_A (m_pxA24Access->ubSerialNumber, 2, 16, '0') ) ;


    //Reset VRB Controller 
    CConsoleAccess::WriteLine ( (CString) " _Resetting VRBC and Selecting VRBC Primary DAQ" ) ;
    m_pxA24Access->uwVrbcReset  = KiHsroVrbMaskResetVrbc ;

    // Wait a bit
    Sleep ( 50 ) ; 

    //Select Primary DAQ
    m_pxA24Access->ubPdaqSelect = KiHsroVrbMaskSelPdaq ;

    // Slave Ready and Finish Handshaking emulation Register (for SQAQ)
    // these 2 bits are described as "Asserted when in low state"
    // Set Both, Slave Ready and Finished INACTIVE 
    m_pxA24Access->uwSlvRdyEmul = KiHsroVrbMaskSlvRdy ;

    //Read Current Status 
    ubStatus = m_pxA24Access->ubSclStatus ;
    if ( ( ubStatus & KiHsroVrbMaskSclStatus ) != 0x00 ) // Do we want a different test here?
        CConsoleAccess::WarningLine ( (CString) " _VRBC SCL Status is "
                                             + I_to_A (ubStatus, 2, 16, '0') ) ;
    else
        CConsoleAccess::WriteLine ( (CString) " _VRBC SCL Status is "
                                             + I_to_A (ubStatus, 2, 16, '0') ) ;



    return eReturnStatus ;
}
/*-------------------------------------------------------------------------------*/

