/*-------------------------------------------------------------------------------*/
  /*
    L1Cal.cpp
    Class CL1Cal
    12-Oct-2004
  */
/*-------------------------------------------------------------------------------*/

#include "L1Cal_IIb_TCC/StdAfx.h"

#include                            "L1Cal_IIb_TCC/L1Cal.h"

#include "L1Cal_IIb_TCC/L1CalCardAddresses.h"

#include "tabgab/tcc_to_tabgab_coor.h"

#include "L1Cal_IIb_TCC/MessageCoorPrinter.h"

#include "L1Cal_IIb_TCC/HandleResultFile.h"

#include "L1Cal_IIb_TCC/UtilsStrings.h"
#include "L1Cal_IIb_TCC/UtilsFiles.h"
#include "L1Cal_IIb_TCC/UtilsBits.h"


// static means created only once throughout life of program (to avoid wasting time)
// *AND* internal linkage (to avoid clash between modules)
#include "L1Cal_IIb_TCC/HandleConsole.h"
static CConsoleAccess oConsole ;

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
// Static data members must be initialized at file scope

    // We (typically) should have only one L1 Cal Trig,
    // this is a way to let any module get a hold of it
    CL1Cal*               CL1Cal::mg_poL1Cal                = NULL ;
//    CL1Cal*               CL1Cal::mg_poL1Cal = new CL1Cal ; // done in L1Cal_IIb_TCC.cpp

    // This Flag shows when the L1Cal has been initialized and is operational
    volatile EYesNo      CL1Cal::mg_eynL1CalOperational   = No ;

    // This Flag allows to control behavior upon Initialization Errors
    volatile EYesNo      CL1Cal::mg_eynAbortInitOnError   = Yes ;

    // This Flag allows to skip Initialization of VMESCL, TABs and GAB
    volatile EYesNo      CL1Cal::mg_eynSkipTabGabOnInit   = No ; 

    // This Flag allows The TTI command files to immediately program the requested values
    volatile EYesNo      CL1Cal::mg_eynTtiImmediateIo     = No ;

    // This Flag controls the behavior of FindDAC
    volatile EFindDacMode_Type     CL1Cal::mg_eFindDacMode = eFindDacMode_All636 ;

    // This Flag controls the value passed to Tab_Set_Eta38 at initialization
    volatile unsigned long         CL1Cal::mg_ulTabEta38Value = 0 ;

    // This value is used during Initialization to control the iterative de-skewing and verification step
    volatile unsigned char         CL1Cal::mg_ubMaxDeSkewTryAtInit = 0 ;

    // This value is the Base Alignment Delay common to all Trigger Towers
    // while deviations from this are added via TT info command files
    volatile unsigned char         CL1Cal::mg_ubBaseAlignDelay = 0 ;

    // This is the Register Address Offset within the Raw ADC Memory Block
    // where TCC will read Monitoring Data for Tick#1/SubTick#0
    volatile unsigned short        CL1Cal::mg_uwOffsetTick1RawAdcMonit = 0 ;

    // This is the Register Address Offset within the Raw ADC Memory Block
    // where TCC will write Simulated Data for Tick#1/SubTick#0
    volatile unsigned short        CL1Cal::mg_uwOffsetTick1RawAdcSimu = 0 ;

    // This is the Register Address Offset within the Output Et Memory Block
    // where TCC will read Monitoring Data for Tick#1
    volatile unsigned short        CL1Cal::mg_uwOffsetTick1OutputEtMonit = 0 ;

    // This is the Register Address Offset within the Output Et Memory Block
    // where TCC will write Simulated Data for Tick#1
    volatile unsigned short        CL1Cal::mg_uwOffsetTick1OutputEtSimu = 0 ;

    // This value is the Tick Offset between the Triggered Crossing Tick Number
    // and the Tick Number currently recorded at the time the Captured Monitoring Data
    // Signal stops the Address Generator(s)
    volatile unsigned short        CL1Cal::mg_uwOffsetTriggeredToTickAtCapture = 0 ;

    // Hold the initialized value (i.e. full scale) of each reference set type
    unsigned short                 CL1Cal::mg_auwRefSetInit [eTot_Tab_RefSet_Type] 
                                                            = { kiTabEmEtTreshMax,
                                                                kiTabJetEtTreshMax,
                                                                kiTabTtEtTreshMax } ;

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/

CL1Cal::CL1Cal ()
{

    unsigned long       ulTTEtaSign ;
    unsigned long       ulTTEtaMagn ;
    unsigned long       ulTTEtaMagnBy4 ;
    unsigned long       ulTTPhi ;
    unsigned long       ulTTPhiBy4 ;

    CString             osCardDescr ;
    char                ubCardMasterNum ;
    char                ubCardSlaveNum ;
    char                ubCardSlotNum ;

    unsigned long       ulRefSetType ;
    unsigned long       ulRefSetNum ;

    //\\//

    // Initialize Trigger Tower Coverage to full coverage
    ///////////////////////////////////////

    m_ulSignEtaMin = eTT_Eta_Pos ;
    m_ulSignEtaMax = KiL1CalTotSignEta - 1 ;

    m_ulMagnEtaMin = eTT_Eta_1 ;
    m_ulMagnEtaMax = KiL1CalTotMagnEta ;

    m_ulPhiMin     = eTT_Phi_1 ;
    m_ulPhiMax     = KiL1CalTotPhi ;

    //////////////////////////////////////////////////////////////////////////

    oConsole.WriteLine ( (CString) "" ) ;
    oConsole.WriteLine ( (CString) "Creating L1 Cal Trig Objects" ) ;
    oConsole.WriteLine ( (CString) "_Trigger Tower Coverage is "
                                    + f_osCoverage() ) ;

    //////////////////////////////////////////////////////////////////////////
    //create card object for each card in the configuration
    //this creates all the cards, even if TT coverage is later reduced
    //////////////////////////////////////////////////////////////////////////
    oConsole.WriteLine ( (CString) "_Creating Objects for All Cards of the L1Cal" ) ;

    m_pLastAdfCard  = NULL ;

    //ADF Cards
    for ( ulTTEtaSign    = m_ulSignEtaMin      ; ulTTEtaSign    <= m_ulSignEtaMax      ; ulTTEtaSign++ )
    for ( ulTTEtaMagnBy4 =(m_ulMagnEtaMin-1)/4 ; ulTTEtaMagnBy4 <=(m_ulMagnEtaMax-1)/4 ; ulTTEtaMagnBy4++ )
    for ( ulTTPhiBy4     =(m_ulPhiMin-1)/4     ; ulTTPhiBy4     <=(m_ulPhiMax-1)/4     ; ulTTPhiBy4++ )
    {
        osCardDescr     = L1CalAdfCardAddress_Desc( ulTTEtaSign, ulTTEtaMagnBy4, ulTTPhiBy4 ) ;
        ubCardMasterNum = L1CalAdfCardAddress_Master ( ulTTEtaSign, ulTTEtaMagnBy4, ulTTPhiBy4 ) ;
        ubCardSlaveNum  = L1CalAdfCardAddress_Slave  ( ulTTEtaSign, ulTTEtaMagnBy4, ulTTPhiBy4 ) ;
        ubCardSlotNum   = L1CalAdfCardAddress_Slot   ( ulTTEtaSign, ulTTEtaMagnBy4, ulTTPhiBy4 ) ;
        m_apoAdf[ulTTEtaSign][ulTTEtaMagnBy4][ulTTPhiBy4]
                = new CAdfCardBase  ( ubCardMasterNum,
                                      ubCardSlaveNum,
                                      ubCardSlotNum,
                                     &m_pLastAdfCard,
                                     &osCardDescr,
                                      ulTTEtaSign,
                                      ulTTEtaMagnBy4,
                                      ulTTPhiBy4 ) ;
        // Additionally remember the address of the pointer for the Maestro cards
        if ( ( ubCardSlaveNum  == KiMaestroRequestMonit_Crate )
           & ( ubCardSlotNum   == KiMaestroRequestMonit_Slot  ) )
        {
            m_poAdfMaestroRequestMonit = m_apoAdf[ulTTEtaSign][ulTTEtaMagnBy4][ulTTPhiBy4] ;
            oConsole.WriteLine ( (CString) " Requesting Monitoring Data after Collect Status will use" ) ;
            oConsole.WriteLine ( (CString) "_Maestro ADF "
                                           + m_poAdfMaestroRequestMonit->f_osBaseAddress ()
                                           + " Signal#"
                                           + I_to_A ( KiMaestroRequestMonit_Signal )
                                           + " to SCLD" ) ;
        }
        if ( ( ubCardSlaveNum  == KiMaestroImmediateMonit_Crate )
           & ( ubCardSlotNum   == KiMaestroImmediateMonit_Slot  ) )
        {
            m_poAdfMaestroImmediateMonit = m_apoAdf[ulTTEtaSign][ulTTEtaMagnBy4][ulTTPhiBy4] ;
            oConsole.WriteLine ( (CString) " Requesting Immediate Monitoring Data Capture will use " ) ;
            oConsole.WriteLine ( (CString) "_Maestro ADF "
                                           + m_poAdfMaestroImmediateMonit->f_osBaseAddress ()
                                           + " Signal#"
                                           + I_to_A ( KiMaestroImmediateMonit_Signal )
                                           + " to SCLD" ) ;
        }
    }


    //TAB Cards
    for ( ulTTPhiBy4     =(m_ulPhiMin-1)/4     ; ulTTPhiBy4     <=(m_ulPhiMax-1)/4     ; ulTTPhiBy4++ )
    {
        osCardDescr = L1CalTabCardAddress_Desc( ulTTPhiBy4 ) ;
        m_apoTab[ulTTPhiBy4]
                = new CNevisTabCard ( L1CalTabCardAddress_Module ( ulTTPhiBy4 ),
                                     &osCardDescr,
                                      ulTTPhiBy4 ) ;
    }

    //GAB Card
    osCardDescr = L1CalGabCardAddress_Desc( ) ;
    m_poGab = new CNevisGabCard ( L1CalGabCardAddress_Module ( ),
                                 &osCardDescr ) ;


    //////////////////////////////////////////////////////////////////////////
    // initialize High Level programming

    // Set all refsets to full scale

    for ( ulRefSetType = eTab_RefSet_EMEt ; ulRefSetType < eTot_Tab_RefSet_Type ; ulRefSetType ++ )
    for ( ulRefSetNum  = eTT_RefSet_0     ; ulRefSetNum  < eTot_TT_RefSet       ; ulRefSetNum ++ ) // note: there are only 3x TT RefSets, but we still initialize 7
    {
        m_a2eRefSetalloc [ulRefSetType][ulRefSetNum] = No ;

        for ( ulTTEtaSign    = m_ulSignEtaMin      ; ulTTEtaSign    <= m_ulSignEtaMax      ; ulTTEtaSign++ )
        for ( ulTTEtaMagnBy4 =(m_ulMagnEtaMin-1)/4 ; ulTTEtaMagnBy4 <=(m_ulMagnEtaMax-1)/4 ; ulTTEtaMagnBy4++ )
        for ( ulTTPhiBy4     =(m_ulPhiMin-1)/4     ; ulTTPhiBy4     <=(m_ulPhiMax-1)/4     ; ulTTPhiBy4++ )
            m_a5uwRefSetThresh  [ulRefSetType]
                                [ulRefSetNum]
                                [ulTTEtaSign]
                                [ulTTEtaMagnBy4]
                                [ulTTPhiBy4] = mg_auwRefSetInit [ulRefSetType] ;
    }

    //Trigger Towers
    //this creates all the towers, even if TT coverage is later reduced

    oConsole.WriteLine ( (CString) "_Creating Objects for All Trigger Tower of the L1Cal" ) ;

    m_poLast_TrgTwr =   NULL ; // this will be updated by each new TrgTwr

    for ( ulTTEtaSign    = m_ulSignEtaMin      ; ulTTEtaSign    <= m_ulSignEtaMax      ; ulTTEtaSign++ )
    for ( ulTTEtaMagn    = m_ulMagnEtaMin      ; ulTTEtaMagn    <= m_ulMagnEtaMax      ; ulTTEtaMagn++ )
    for ( ulTTPhi        = m_ulPhiMin          ; ulTTPhi        <= m_ulPhiMax          ; ulTTPhi++ )
        m_a3poTrgTwr[ulTTEtaSign][ulTTEtaMagn][ulTTPhi]
                = new CL1Cal_TrgTwr   ( ulTTEtaSign,
                                        ulTTEtaMagn,
                                        ulTTPhi,
                                        this ) ;
}


/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
CL1Cal::~CL1Cal ()
{
    unsigned long       ulTTEtaSign ;
    unsigned long       ulTTEtaMagn ;
    unsigned long       ulTTEtaMagnBy4 ;
    unsigned long       ulTTPhi ;
    unsigned long       ulTTPhiBy4 ;

    //\\//

    oConsole.WriteLine ( (CString) "" ) ;
    oConsole.WriteLine ( (CString) "Deleting All L1Cal Card Objects" ) ;

    //////////////////////////////////////////////////////////////
    //Note we use the full coverage in the loop boundaries
    //and not just the coverage specified by the member variables
    //////////////////////////////////////////////////////////////

    //delete all card objects

    //ADF Cards
    for ( ulTTEtaSign    = 0 ; ulTTEtaSign    <  eTot_TT_Eta_Sign ; ulTTEtaSign++ )
    for ( ulTTEtaMagnBy4 = 0 ; ulTTEtaMagnBy4 <=(eTot_TT_Eta-1)/4 ; ulTTEtaMagnBy4++ )
    for ( ulTTPhiBy4     = 0 ; ulTTPhiBy4     <=(eTot_TT_Phi-1)/4 ; ulTTPhiBy4++ )
    {
        if ( m_apoAdf[ulTTEtaSign][ulTTEtaMagnBy4][ulTTPhiBy4] != NULL )
            delete m_apoAdf[ulTTEtaSign][ulTTEtaMagnBy4][ulTTPhiBy4] ;
    }


    //TAB Cards
    for ( ulTTPhiBy4     = 0 ; ulTTPhiBy4     <=(eTot_TT_Phi-1)/4 ; ulTTPhiBy4++ )
    {
        if ( m_apoTab[ulTTPhiBy4] != NULL )
            delete m_apoTab[ulTTPhiBy4] ;
    }

    //GAB Card
    if ( m_poGab != NULL )
        delete m_poGab ;


    //delete higher level objects

    //Trigger Towers
    for ( ulTTEtaSign    = 0 ; ulTTEtaSign    <  eTot_TT_Eta_Sign ; ulTTEtaSign++ )
    for ( ulTTEtaMagn    = 1 ; ulTTEtaMagn    <= eTot_TT_Eta      ; ulTTEtaMagn++ )
    for ( ulTTPhi        = 1 ; ulTTPhi        <= eTot_TT_Phi      ; ulTTPhi++ )
        delete m_a3poTrgTwr[ulTTEtaSign][ulTTEtaMagn][ulTTPhi] ;

}


/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess    CL1Cal::Initialize ()
{
    ESuccess        eReturnStatus = Ok ;

    unsigned long       ulTTEtaSign ;
    unsigned long       ulTTEtaMagn ;
    unsigned long       ulTTEtaMagnBy4 ;
    unsigned long       ulTTPhi ;
    unsigned long       ulTTPhiBy4 ;

    unsigned long       ulRefSetType ;
    unsigned long       ulRefSetNum ;

    EYesNo              eSkipInitLookup = Yes ;

    //\\//

    //////////////////////////////////////////////////////////////////////////
    // Initialize all L1 Cal Trig Cards
    //////////////////////////////////////////////////////////////////////////

    //ADF Cards
    for ( ulTTEtaSign    = m_ulSignEtaMin      ; ulTTEtaSign    <= m_ulSignEtaMax      ; ulTTEtaSign++ )
    for ( ulTTEtaMagnBy4 =(m_ulMagnEtaMin-1)/4 ; ulTTEtaMagnBy4 <=(m_ulMagnEtaMax-1)/4 ; ulTTEtaMagnBy4++ )
    for ( ulTTPhiBy4     =(m_ulPhiMin-1)/4     ; ulTTPhiBy4     <=(m_ulPhiMax-1)/4     ; ulTTPhiBy4++ )
    {
        if ( m_apoAdf[ulTTEtaSign][ulTTEtaMagnBy4][ulTTPhiBy4] != NULL )
        if ( m_apoAdf[ulTTEtaSign][ulTTEtaMagnBy4][ulTTPhiBy4] ->Initialize ( eSkipInitLookup )      != Ok )
        {
            eReturnStatus = Error ;
            if ( CL1Cal::mg_eynAbortInitOnError == Yes ) goto Abort ;
        }
    }

    if ( CL1Cal::mg_eynSkipTabGabOnInit == No ) 
    {
        // VME/SCL
        if ( ReportTabGabStatus (  Vmescl_Initialize (),
                                  "Vmescl_Initialize",
                                  "CL1Cal::Initialize" ) != Ok )
        {
            eReturnStatus = Error ;
            if ( CL1Cal::mg_eynAbortInitOnError == Yes ) goto Abort ;
        }


        //TAB Cards
        for ( ulTTPhiBy4     =(m_ulPhiMin-1)/4     ; ulTTPhiBy4     <=(m_ulPhiMax-1)/4     ; ulTTPhiBy4++ )
        {
            if ( m_apoTab[ulTTPhiBy4] != NULL )
            if ( m_apoTab[ulTTPhiBy4] ->Initialize ( )      != Ok )
            {
                eReturnStatus = Error ;
                if ( CL1Cal::mg_eynAbortInitOnError == Yes ) goto Abort ;
            }
        }

        //GAB Card
        if ( m_poGab != NULL )
        if ( m_poGab -> Initialize ( )      != Ok )
        {
            eReturnStatus = Error ;
            if ( CL1Cal::mg_eynAbortInitOnError == Yes ) goto Abort ;
        }

    
        //////////////////////////////////////////////////////////////////////////
        // Verify proper ADF-to_TAB Channel Link operation and re-de-skew as needed
        //////////////////////////////////////////////////////////////////////////
        if ( mg_ubMaxDeSkewTryAtInit != 0 ) 
        if ( Iterative_DeSkew () != Ok )
        {
            eReturnStatus = Error ;
            if ( CL1Cal::mg_eynAbortInitOnError == Yes ) goto Abort ;
        }

    }



    //////////////////////////////////////////////////////////////////////////
    // initialize Trigger Towers
    //////////////////////////////////////////////////////////////////////////

    oConsole.WriteLine ( (CString) " Initializing Trigger Towers" ) ;

    for ( ulTTEtaSign = m_ulSignEtaMin ; ulTTEtaSign <= m_ulSignEtaMax ; ulTTEtaSign++ )
    for ( ulTTEtaMagn = m_ulMagnEtaMin ; ulTTEtaMagn <= m_ulMagnEtaMax ; ulTTEtaMagn++ )
    for ( ulTTPhi     = m_ulPhiMin     ; ulTTPhi     <= m_ulPhiMax     ; ulTTPhi++ )
    if ( m_a3poTrgTwr[ulTTEtaSign][ulTTEtaMagn][ulTTPhi]->Initialize ( ) != Ok )
    {
        eReturnStatus = Error ;
        if ( CL1Cal::mg_eynAbortInitOnError == Yes ) goto Abort ;
    }

    // Verify the Et Lookup Separately and sanity check
    oConsole.WriteLine ( (CString) " Verifying Et Lookup Tables" ) ;

    for ( ulTTEtaSign = m_ulSignEtaMin ; ulTTEtaSign <= m_ulSignEtaMax ; ulTTEtaSign++ )
    for ( ulTTEtaMagn = m_ulMagnEtaMin ; ulTTEtaMagn <= m_ulMagnEtaMax ; ulTTEtaMagn++ )
    for ( ulTTPhi     = m_ulPhiMin     ; ulTTPhi     <= m_ulPhiMax     ; ulTTPhi++ )
    if ( m_a3poTrgTwr[ulTTEtaSign][ulTTEtaMagn][ulTTPhi]->VerifyProgramming ( ) != Ok )
    {
        eReturnStatus = Error ;
        if ( CL1Cal::mg_eynAbortInitOnError == Yes ) goto Abort ;
    }

    //////////////////////////////////////////////////////////////////////////
    // initialize High Level programming
    //////////////////////////////////////////////////////////////////////////

    //Initialize reference sets

    for ( ulRefSetType  = eTab_RefSet_EMEt ; ulRefSetType < eTot_Tab_RefSet_Type ; ulRefSetType ++ )
    for ( ulRefSetNum   = eTT_RefSet_0     ; ulRefSetNum  < eTot_TT_RefSet       ; ulRefSetNum ++ )
    {
        m_a2eRefSetalloc [ulRefSetType][ulRefSetNum] = No ;

        if ( ( ulRefSetType == eTab_RefSet_TTEt ) 
          && ( ulRefSetNum  >= eTot_TT_RefSet_TT ) ) 
        {
            // There are only 3 single tower refsets.
        }
        else 
        {
            for ( ulTTEtaSign    = m_ulSignEtaMin      ; ulTTEtaSign    <= m_ulSignEtaMax      ; ulTTEtaSign++ )
            for ( ulTTEtaMagnBy4 =(m_ulMagnEtaMin-1)/4 ; ulTTEtaMagnBy4 <=(m_ulMagnEtaMax-1)/4 ; ulTTEtaMagnBy4++ )
            for ( ulTTPhiBy4     =(m_ulPhiMin-1)/4     ; ulTTPhiBy4     <=(m_ulPhiMax-1)/4     ; ulTTPhiBy4++ )
            {
                // we need to keep track of all the thresholds for proper ordering
                m_a5uwRefSetThresh  [ulRefSetType]
                                    [ulRefSetNum]
                                    [ulTTEtaSign]
                                    [ulTTEtaMagnBy4]
                                    [ulTTPhiBy4] = mg_auwRefSetInit [ulRefSetType] ;

/* This is already done by CNevisTabChip::Initialize
                // program this reference set
                if ( m_apoTab[ulTTPhiBy4]->m_apoMaFpga[ulTTEtaSign][ulTTEtaMagnBy4]
                                 -> RefSethreshProg ( (ETabRefSet_Type) ulRefSetType,
                                                                        ulRefSetNum,
                                                                        mg_auwRefSetInit [ulRefSetType] ) != Ok )
                   eReturnStatus = Error ;
*/
            }
        }
    }

    //////////////////////////////////////////////////////////////////////////

    if ( eReturnStatus == Error )
        oConsole.ErrorLine ( "L1 Cal Trig Initialization Failed" ) ;
    else
        oConsole.WriteLine ( "L1 Cal Trig Initialization Successfull" ) ;

    return eReturnStatus ;

Abort:

    oConsole.ErrorLine ( "L1 Cal Trig Initialization Aborted" ) ;

    return Error ;
}
/*-------------------------------------------------------------------------------*/


/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess    CL1Cal::Iterative_DeSkew   ( )
{
    ESuccess        eReturnStatus = Ok ;

    ESuccess        eDeSkewStatus ;

    unsigned long       ulTTEtaSign ;
    unsigned long       ulTTEtaMagnBy4 ;
    unsigned long       ulTTPhiBy4 ;

    unsigned long       ulDeSkewTry ;

    //\\//

    // We need to make the ADF cards send the pseudo-random pattern
    ///////////////////////////////////////////////////////////////////////
    oConsole.WriteLine ( (CString) " Setting up ADF Cards to send pseudo-random pattern" ) ;

    for ( ulTTEtaSign    = m_ulSignEtaMin      ; ulTTEtaSign    <= m_ulSignEtaMax      ; ulTTEtaSign++ )
    for ( ulTTEtaMagnBy4 =(m_ulMagnEtaMin-1)/4 ; ulTTEtaMagnBy4 <=(m_ulMagnEtaMax-1)/4 ; ulTTEtaMagnBy4++ )
    for ( ulTTPhiBy4     =(m_ulPhiMin-1)/4     ; ulTTPhiBy4     <=(m_ulPhiMax-1)/4     ; ulTTPhiBy4++ )
    if ( m_apoAdf[ulTTEtaSign][ulTTEtaMagnBy4][ulTTPhiBy4] != NULL )
    {
        if ( m_apoAdf[ulTTEtaSign][ulTTEtaMagnBy4][ulTTPhiBy4] ->f_okStopPRN ( )      != Ok )
        {
            eReturnStatus = Error ;
            if ( CL1Cal::mg_eynAbortInitOnError == Yes ) goto Abort ;
        }

        if ( m_apoAdf[ulTTEtaSign][ulTTEtaMagnBy4][ulTTPhiBy4] ->f_okLoadPRNSeeds ( )      != Ok )
        {
            eReturnStatus = Error ;
            if ( CL1Cal::mg_eynAbortInitOnError == Yes ) goto Abort ;
        }

        if ( m_apoAdf[ulTTEtaSign][ulTTEtaMagnBy4][ulTTPhiBy4] ->f_okStartPRN ( )      != Ok )
        {
            eReturnStatus = Error ;
            if ( CL1Cal::mg_eynAbortInitOnError == Yes ) goto Abort ;
        }

        if ( m_apoAdf[ulTTEtaSign][ulTTEtaMagnBy4][ulTTPhiBy4] ->f_okSendPseudoRandom ( )      != Ok )
        {
            eReturnStatus = Error ;
            if ( CL1Cal::mg_eynAbortInitOnError == Yes ) goto Abort ;
        }
    }


    //We go over all TAB cards, up to N times,
    //or untill all report that all their Channel Links operate properly
    ///////////////////////////////////////////////////////////////////////

    //Mike says we first need to read the TAB status once (which clears the status),
    //and ignore that first reading
    oConsole.WriteLine ( (CString) " Clearing Algo Fpga Status from all TABs" ) ;
    for ( ulTTPhiBy4     =(m_ulPhiMin-1)/4     ; ulTTPhiBy4     <=(m_ulPhiMax-1)/4     ; ulTTPhiBy4++ )
    {
        if ( m_apoTab[ulTTPhiBy4] != NULL )
        if ( m_apoTab[ulTTPhiBy4] ->CaptureChLinkStatus ( )      != Ok )
        {
            eReturnStatus = Error ;
            if ( CL1Cal::mg_eynAbortInitOnError == Yes ) goto Abort ;
        }
    }

    for ( ulDeSkewTry = 0 ; ulDeSkewTry < mg_ubMaxDeSkewTryAtInit ; ulDeSkewTry ++ )
    {
        oConsole.WriteLine ( (CString) " Starting Channel Link Check and DeSkew pass #"
                                      + I_to_A ( ulDeSkewTry+1 ) ) ;

        // Clear recollection of which ADF card was recently deskewed
        oConsole.WriteLine ( (CString) " Clearing record of ADF recently deskewed" ) ;

        for ( ulTTEtaSign    = m_ulSignEtaMin      ; ulTTEtaSign    <= m_ulSignEtaMax      ; ulTTEtaSign++ )
        for ( ulTTEtaMagnBy4 =(m_ulMagnEtaMin-1)/4 ; ulTTEtaMagnBy4 <=(m_ulMagnEtaMax-1)/4 ; ulTTEtaMagnBy4++ )
        for ( ulTTPhiBy4     =(m_ulPhiMin-1)/4     ; ulTTPhiBy4     <=(m_ulPhiMax-1)/4     ; ulTTPhiBy4++ )
        if ( m_apoAdf[ulTTEtaSign][ulTTEtaMagnBy4][ulTTPhiBy4] != NULL )
             m_apoAdf[ulTTEtaSign][ulTTEtaMagnBy4][ulTTPhiBy4] ->f_vClear_DeSkewed ( ) ;

        // waiting one second while the TABs watch the data
        oConsole.WriteLine ( (CString) " Waiting one second" ) ;
        Sleep ( KiSleepOneSecond ) ;

        //Now we take a new snapshot of all TAB status
        oConsole.WriteLine ( (CString) " Capturing Current Algo Fpga Status from all TABs" ) ;
        for ( ulTTPhiBy4     =(m_ulPhiMin-1)/4     ; ulTTPhiBy4     <=(m_ulPhiMax-1)/4     ; ulTTPhiBy4++ )
        {
            if ( m_apoTab[ulTTPhiBy4] != NULL )
            if ( m_apoTab[ulTTPhiBy4] ->CaptureChLinkStatus ( )      != Ok )
            {
                eReturnStatus = Error ;
                if ( CL1Cal::mg_eynAbortInitOnError == Yes ) goto Abort ;
            }
        }

        // start assuming we are ok, until one Tab complains
        eDeSkewStatus = Ok ;

        oConsole.WriteLine ( (CString) " Checking Link Status and De-skewing as needed" ) ;
        for ( ulTTPhiBy4     =(m_ulPhiMin-1)/4     ; ulTTPhiBy4     <=(m_ulPhiMax-1)/4     ; ulTTPhiBy4++ )
        {
            // Let each TAB object verify all its FPGA's link status
            // *and* if a link is not working, tell the corresponding ADF de-skew each link ONCE
            if ( m_apoTab[ulTTPhiBy4] != NULL )
            if ( m_apoTab[ulTTPhiBy4] ->VerifyChLink_DeSkewAsNeeded ( )      != Ok )
                eDeSkewStatus = Error ;
        }

        if ( eDeSkewStatus == Ok ) break ; // we have succeeded

        //Now we can verify that the ADF deskewing actions messed up the intended TABs and only those
        oConsole.WriteLine ( (CString) " Capturing Algo Fpga Status from all TABs" ) ;
        for ( ulTTPhiBy4     =(m_ulPhiMin-1)/4     ; ulTTPhiBy4     <=(m_ulPhiMax-1)/4     ; ulTTPhiBy4++ )
        {
            if ( m_apoTab[ulTTPhiBy4] != NULL )
            if ( m_apoTab[ulTTPhiBy4] ->CaptureChLinkStatus ( )      != Ok )
            {
                eReturnStatus = Error ;
                if ( CL1Cal::mg_eynAbortInitOnError == Yes ) goto Abort ;
            }
        }

        oConsole.WriteLine ( (CString) " Checking Link Status for consistency after De-Skew" ) ;
        for ( ulTTPhiBy4     =(m_ulPhiMin-1)/4     ; ulTTPhiBy4     <=(m_ulPhiMax-1)/4     ; ulTTPhiBy4++ )
        {
            // Let each TAB object very its FPGA's link status
            // No deskewing allowed here, so that we leave the system in the reported state
            if ( m_apoTab[ulTTPhiBy4] != NULL )
            if ( m_apoTab[ulTTPhiBy4] ->VerifyChLink_ForConsistency ( )      != Ok )
                eDeSkewStatus = Error ;
        }

    }

    // Now do a final verification (with no more de-skewing performed)
    // And there should be no errors
    ///////////////////////////////////////////////////////////////////////

    oConsole.WriteLine ( (CString) " Starting Final Channel Link Check" ) ;

    // Clear recollection of which ADF card was recently deskewed
    oConsole.WriteLine ( (CString) " Clearing record of ADF recently deskewed" ) ;

    for ( ulTTEtaSign    = m_ulSignEtaMin      ; ulTTEtaSign    <= m_ulSignEtaMax      ; ulTTEtaSign++ )
    for ( ulTTEtaMagnBy4 =(m_ulMagnEtaMin-1)/4 ; ulTTEtaMagnBy4 <=(m_ulMagnEtaMax-1)/4 ; ulTTEtaMagnBy4++ )
    for ( ulTTPhiBy4     =(m_ulPhiMin-1)/4     ; ulTTPhiBy4     <=(m_ulPhiMax-1)/4     ; ulTTPhiBy4++ )
    if ( m_apoAdf[ulTTEtaSign][ulTTEtaMagnBy4][ulTTPhiBy4] != NULL )
         m_apoAdf[ulTTEtaSign][ulTTEtaMagnBy4][ulTTPhiBy4] ->f_vClear_DeSkewed ( ) ;

    //Read the TAB status once (which clears the status), and ignore that first reading
    oConsole.WriteLine ( (CString) " Capturing Algo Fpga Status from all TABs" ) ;
    for ( ulTTPhiBy4     =(m_ulPhiMin-1)/4     ; ulTTPhiBy4     <=(m_ulPhiMax-1)/4     ; ulTTPhiBy4++ )
    {
        if ( m_apoTab[ulTTPhiBy4] != NULL )
        if ( m_apoTab[ulTTPhiBy4] ->CaptureChLinkStatus ( )      != Ok )
        {
            eReturnStatus = Error ;
            if ( CL1Cal::mg_eynAbortInitOnError == Yes ) goto Abort ;
        }
    }

    //waiting while the TABs watch the data
    oConsole.WriteLine ( (CString) " Waiting five seconds" ) ;
    Sleep ( 5*KiSleepOneSecond ) ;


    //Now we take a snapshot of all TAB status
    oConsole.WriteLine ( (CString) " Capturing Current Algo Fpga Status from all TABs" ) ;
    for ( ulTTPhiBy4     =(m_ulPhiMin-1)/4     ; ulTTPhiBy4     <=(m_ulPhiMax-1)/4     ; ulTTPhiBy4++ )
    {
        if ( m_apoTab[ulTTPhiBy4] != NULL )
        if ( m_apoTab[ulTTPhiBy4] ->CaptureChLinkStatus ( )      != Ok )
        {
            eReturnStatus = Error ;
            if ( CL1Cal::mg_eynAbortInitOnError == Yes ) goto Abort ;
        }
    }

    // start assuming we are ok, until we come accross one TAB that complains
    eDeSkewStatus = Ok ;

    oConsole.WriteLine ( (CString) " Checking Link Status (without De-skewing)" ) ;
    for ( ulTTPhiBy4     =(m_ulPhiMin-1)/4     ; ulTTPhiBy4     <=(m_ulPhiMax-1)/4     ; ulTTPhiBy4++ )
    {
        // Let each TAB object very its FPGA's link status
        // No deskewing allowed here, so that we leave the system in the reported state
        if ( m_apoTab[ulTTPhiBy4] != NULL )
        if ( m_apoTab[ulTTPhiBy4] ->VerifyChLinkStatus ( )      != Ok )
            eDeSkewStatus = Error ;
    }

    if ( eDeSkewStatus != Ok )
    {
        oConsole.ErrorLine ( (CString) " Channel Link DeSkew procedure failed" ) ;
        eReturnStatus = Error ;
        if ( CL1Cal::mg_eynAbortInitOnError == Yes ) goto Abort ;
    }
    else
        oConsole.WriteLine ( (CString) " Channel Link DeSkew procedure successful" ) ;


    // We return the ADF cards to desired inititalization (i.e. pseudo-random pattern stopped)
    ///////////////////////////////////////////////////////////////////////

    oConsole.WriteLine ( (CString) " Return ADF Cards to normal BLS data operation" ) ;

    for ( ulTTEtaSign    = m_ulSignEtaMin      ; ulTTEtaSign    <= m_ulSignEtaMax      ; ulTTEtaSign++ )
    for ( ulTTEtaMagnBy4 =(m_ulMagnEtaMin-1)/4 ; ulTTEtaMagnBy4 <=(m_ulMagnEtaMax-1)/4 ; ulTTEtaMagnBy4++ )
    for ( ulTTPhiBy4     =(m_ulPhiMin-1)/4     ; ulTTPhiBy4     <=(m_ulPhiMax-1)/4     ; ulTTPhiBy4++ )
    {
        if ( m_apoAdf[ulTTEtaSign][ulTTEtaMagnBy4][ulTTPhiBy4] != NULL )
        if ( m_apoAdf[ulTTEtaSign][ulTTEtaMagnBy4][ulTTPhiBy4] ->f_okSendBlsData ( )      != Ok )
        {
            eReturnStatus = Error ;
            if ( CL1Cal::mg_eynAbortInitOnError == Yes ) goto Abort ;
        }

        if ( m_apoAdf[ulTTEtaSign][ulTTEtaMagnBy4][ulTTPhiBy4] != NULL )
        if ( m_apoAdf[ulTTEtaSign][ulTTEtaMagnBy4][ulTTPhiBy4] ->f_okFreezePseudoRandom ( ) != Ok )
        {
            eReturnStatus = Error ;
            if ( CL1Cal::mg_eynAbortInitOnError == Yes ) goto Abort ;
        }

        if ( m_apoAdf[ulTTEtaSign][ulTTEtaMagnBy4][ulTTPhiBy4] ->f_okStopPRN ( )      != Ok )
        {
            eReturnStatus = Error ;
            if ( CL1Cal::mg_eynAbortInitOnError == Yes ) goto Abort ;
        }

    }

Abort:

    return eReturnStatus ;
}
/*-------------------------------------------------------------------------------*/


/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess    CL1Cal::SetCoverageSignEta ( unsigned long   _ulTTEtaSignMin,
                                         unsigned long   _ulTTEtaSignMax )
{
    ESuccess        eReturnStatus = Ok ;

    //\\//

    if ( ( _ulTTEtaSignMin >= KiL1CalTotSignEta )
      || ( _ulTTEtaSignMax >= KiL1CalTotSignEta )
      || ( _ulTTEtaSignMax < _ulTTEtaSignMin ) )
    {
        eReturnStatus = Error ;
        //prepare a default value in case of error.
        m_ulSignEtaMin = (ETT_Eta_Sign) 0 ;
        m_ulSignEtaMax = (ETT_Eta_Sign) KiL1CalTotSignEta - 1 ;
    }
    else
    {
        m_ulSignEtaMin = _ulTTEtaSignMin ;
        m_ulSignEtaMax = _ulTTEtaSignMax ;
    }

    return eReturnStatus ;
}
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess    CL1Cal::SetCoverageMagnEta ( unsigned long   _ulTTEtaMagnMin,
                                        unsigned long   _ulTTEtaMagnMax )
{
    ESuccess        eReturnStatus = Ok ;

    //\\//

    if ( ( _ulTTEtaMagnMin > KiL1CalTotMagnEta )
      || ( _ulTTEtaMagnMax > KiL1CalTotMagnEta )
      || ( _ulTTEtaMagnMin == 0 )
      || ( _ulTTEtaMagnMax < _ulTTEtaMagnMin ) )
    {
        eReturnStatus = Error ;
        //prepare a default value in case of error.
        m_ulMagnEtaMin = 1 ;
        m_ulMagnEtaMax = KiL1CalTotMagnEta ;
    }
    else
    {
        m_ulMagnEtaMin = _ulTTEtaMagnMin ;
        m_ulMagnEtaMax = _ulTTEtaMagnMax ;
    }

    return eReturnStatus ;
}
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess    CL1Cal::SetCoveragePhi     ( unsigned long   _ulTTPhiMin,
                                        unsigned long   _ulTTPhiMax     )
{
    ESuccess        eReturnStatus = Ok ;

    //\\//

    if ( ( _ulTTPhiMin > KiL1CalTotPhi )
      || ( _ulTTPhiMax > KiL1CalTotPhi )
      || ( _ulTTPhiMin == 0 )
      || ( _ulTTPhiMax < _ulTTPhiMin ) )
    {
        eReturnStatus = Error ;
        //prepare a default value in case of error.
        m_ulPhiMin = 1 ;
        m_ulPhiMax = KiL1CalTotPhi ;
    }
    else
    {
        m_ulPhiMin = _ulTTPhiMin ;
        m_ulPhiMax = _ulTTPhiMax ;
    }

    return eReturnStatus ;
}
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
CString         CL1Cal::f_osCoverage   ()
{ // Show coverage e.g. "eta(-20:+20),phi(1:32)"

    return CL1Cal_TrgTwr::f_osTrgTwr_Range ( m_ulSignEtaMin, m_ulSignEtaMax,
                                            m_ulMagnEtaMin, m_ulMagnEtaMax,
                                            m_ulPhiMin,     m_ulPhiMax ) ;
}
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess        CL1Cal::f_okRequestMonitData      ( )
{
    ESuccess        eReturnStatus = Ok ;

    //\\//

    // Set the status line high to request a capture
    if ( m_poAdfMaestroRequestMonit->m_poBcPal
                ->SetStatusToScld ( KiMaestroRequestMonit_Signal, 1 ) != Ok )
        eReturnStatus = Error ;

    return eReturnStatus ;
}
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess        CL1Cal::f_okDontRequestMonitData      ( )
{
    ESuccess        eReturnStatus = Ok ;

    //\\//

    // Set the status line low to stop collect_status from capturing monitoring data
    if ( m_poAdfMaestroRequestMonit->m_poBcPal
                ->SetStatusToScld ( KiMaestroRequestMonit_Signal, 0 ) != Ok )
        eReturnStatus = Error ;

    return eReturnStatus ;
}
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess        CL1Cal::f_okCheckMonitDataCaptured  ( EYesNo*           _peCaptured,
                                                      unsigned short*   _pulTickNum )
{
    ESuccess        eReturnStatus = Ok ;
    unsigned short  uwTemp ;

    //\\//

    // Set the status line low to reset the circuitry from the last time
    if ( m_poAdfMaestroRequestMonit
                    ->m_apoT7Fpga[0]
                    ->m_poGlb_Status
                    ->f_okReadReg ( &uwTemp ) != Ok )
        eReturnStatus = Error ;

    // see if Write Enable is still asserted
    if ( ( uwTemp & kuiGlbStatus_FinOutAGenRun_BitMask ) != 0x0000 )
        *_peCaptured = No ;
    else
        *_peCaptured = Yes ;

    // Read the current Counter Value, if requested
    if ( _pulTickNum != NULL )
    {
        if ( m_poAdfMaestroRequestMonit
                    ->m_apoT7Fpga[0]
                    ->m_poGlb_FinalOutAddrGen_Curr
                    ->f_okReadReg ( &uwTemp ) != Ok )
        {
            *_pulTickNum = 0 ;  // return an illegal value
            eReturnStatus = Error ;
        }
        else// Derive the Beam Crossing that caused the L1Accept:
        {
            // The Tick Number last recorded can be derived (modulo 159)
            // from the value read from the Address Generator
            // by removing the known offset
            // while insuring a positive result by shifting up by 159
            *_pulTickNum = uwTemp  + ( 159 - mg_uwOffsetTick1OutputEtMonit ) + 1 ;
            // The Tick number that caused the L1Accept
            // can be derived (modulo 159) from the Tick last recorded
            // by removing the known offset
            // while insuring a positive result by shifting up by 159
            *_pulTickNum = *_pulTickNum + ( 159 - mg_uwOffsetTriggeredToTickAtCapture ) ;
            // The whole thing needs to be normalized modulo 159
            // shift range down to 0:158 -> modulo 159 -> shift range back to 1:159
            *_pulTickNum = ( ( *_pulTickNum -1 ) % 159 ) + 1 ;
        }
    }

//Abort :
    return eReturnStatus ;
}
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess        CL1Cal::f_okCaptureMonitData        ( )
{
    ESuccess        eReturnStatus = Ok ;

    //\\//

    // Set the status line high to get an immediate capture
    if ( m_poAdfMaestroImmediateMonit->m_poBcPal
                ->SetStatusToScld ( KiMaestroImmediateMonit_Signal, 1 ) != Ok )
        eReturnStatus = Error ;

    // Set the status line low to get ready for next time
    if ( m_poAdfMaestroImmediateMonit->m_poBcPal
                ->SetStatusToScld ( KiMaestroImmediateMonit_Signal, 0 ) != Ok )
        eReturnStatus = Error ;

//Abort :
    return eReturnStatus ;
}
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess        CL1Cal::f_okRangeInCoverage    ( TAllTriggerTowers*      _pxTT )
{ // Check this Trigger Tower Range is in current allowed Coverage

    ESuccess        eReturnStatus = Ok ;

    unsigned long   ulTTEtaSign ;
    unsigned long   ulTTEtaMagn ;
    unsigned long   ulTTPhi ;

    //\\//

    for ( ulTTEtaSign = eTT_Eta_Pos       ; ulTTEtaSign <= eTT_Eta_Neg       ; ulTTEtaSign++ )
    for ( ulTTEtaMagn = KiMin_TT_Eta_Magn ; ulTTEtaMagn <= KiMax_TT_Eta_Magn ; ulTTEtaMagn++ )
    if ( _pxTT->aalTT_Eta [ulTTEtaSign] [ulTTEtaMagn] == true )
    if ( ( ulTTEtaSign < m_ulSignEtaMin )
      || ( ulTTEtaSign > m_ulSignEtaMax )
      || ( ulTTEtaMagn < m_ulMagnEtaMin )
      || ( ulTTEtaMagn > m_ulMagnEtaMax ) )
    {
        char cEtaSign ;
        if ( ulTTEtaSign == eTT_Eta_Pos )
            cEtaSign = '+' ;
        else
            cEtaSign = '-' ;

        oConsole.WarningLine ( (CString) "L1 Cal Trig Eta Out of Range : "
                                      + cEtaSign
                                      + I_to_A ( ulTTEtaMagn ) ) ;
        eReturnStatus = Error ;
        goto Abort ;
    }

    for ( ulTTPhi = KiMin_TT_Phi ; ulTTPhi <= KiMax_TT_Phi ; ulTTPhi++ )
    if ( _pxTT->alTT_Phi [ulTTPhi] == true )
    if ( ( ulTTPhi < m_ulPhiMin )
      || ( ulTTPhi > m_ulPhiMax ) )
    {
        oConsole.WarningLine ( (CString) "L1 Cal Trig Phi Out of Range : "
                                      + I_to_A ( ulTTPhi ) ) ;
        eReturnStatus = Error ;
        goto Abort ;
    }

Abort :
    return eReturnStatus ;
}
/*-------------------------------------------------------------------------------*/



/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess        CL1Cal::f_okFind_DAC  ( ETT_Chan_Type           _eTTType,
                                        TAllTriggerTowers*      _pxTT )
{
    ESuccess        eReturnStatus = Ok ;

    unsigned long   ulTTEtaSign ;
    unsigned long   ulTTEtaMagn ;
    unsigned long   ulTTPhi ;

    TuwRegWord          uwDacValue ;
    double              fAverage ;
    double              fDeviation ;
    double              fDevAverage ;
    bool                lShowDetails = true ;

    CString             osResultFileName ;
    CResultFile*        poResultFile = NULL ;

    CString             osHistoFileName = "None" ;
    CResultFile*        poHistoFile = NULL ;

    char                sTtRange[132] ; // generous string to build a trigger tower range
    char*               psTtRange = sTtRange ;

    CL1Cal_TrgTwr*      poTrgTwr ;

    //\\//

    // build a string describing the trigger tower range
    strcpy ( psTtRange, "" ) ;
    CMessageCoorPrinter::Add_Formatted_TT_Range ( psTtRange, _pxTT ) ;

    //build a Result and Histogram File Name
    osResultFileName  = "Find_DAC_" ;
    osResultFileName += (CString) KsVersion
                          + "_"
                          + DateTag () ;
    osHistoFileName   = osResultFileName + ".hst" ;
    osResultFileName  = osResultFileName + ".tti" ;
    AddFileVersion ( &osResultFileName ) ;
    AddFileVersion ( &osHistoFileName ) ;

    // Open a file to write the result summary TTI file
    poResultFile = new CResultFile ( osResultFileName ) ;
    if ( poResultFile->OpenForWrite () == Ok )
    {
        //Write a header in the file
        poResultFile->WriteRulerLine ( "!" ) ;
        poResultFile->WriteLine ( (CString) "! Result File written by"
                                       + " L1Cal_IIb_Tcc_"
                                       + KsVersion
                                       + " Find_DAC" ) ;

        poResultFile->WriteLine ( (CString) "! Finding DAC Values For " + psTtRange ) ;
        if ( mg_eFindDacMode == eFindDacMode_LiveX )
            poResultFile->WriteLine ( (CString) "! Histograms are Filled Using Only the 36 Raw ADC Samples of  Live X" ) ;
        else 
            poResultFile->WriteLine ( (CString) "! Histograms are Filled Using All 636 Raw ADC Samples" ) ;
        poResultFile->WriteLine ( (CString) "! Histograms recorded in " + osHistoFileName ) ;

        poResultFile->WriteDateTimeLine ( "!" ) ;
        poResultFile->WriteRulerLine ( "!" ) ;

        //Define some symbols in the result file that can be used as a TTI command file
        poResultFile->WriteLine ( "" ) ;
        poResultFile->WriteLine ( "$TT_EM_Channel= 0" ) ; //command file symbols
        poResultFile->WriteLine ( "$TT_HD_Channel= 1" ) ;
        poResultFile->WriteLine ( "" ) ;
        poResultFile->WriteLine ( "$TT_Eta_Pos= 0" ) ;
        poResultFile->WriteLine ( "$TT_Eta_Neg= 1" ) ;
        poResultFile->WriteLine ( "" ) ;
        poResultFile->WriteRulerLine ( "!" ) ;

    }
    else
    {
        CConsoleAccess::ErrorLine ( "Failed Creating Find_DAC Result File" ) ;
        poResultFile = NULL ;
    }


    // Open a file to write histogram results
    poHistoFile = new CResultFile ( osHistoFileName ) ;
    if ( poHistoFile->OpenForWrite () == Ok )
    {
        //Write a header in the file
        poHistoFile->WriteRulerLine ( ) ;
        poHistoFile->WriteLine ( (CString) " Histogram File written by"
                                       + " L1Cal_IIb_Tcc_"
                                       + KsVersion
                                       + " Find_DAC" ) ;
        poHistoFile->WriteLine ( (CString) " Finding DAC Values For " + psTtRange ) ;
        if ( mg_eFindDacMode == eFindDacMode_LiveX )
            poHistoFile->WriteLine ( (CString) " Histograms are Filled Using Only the 36 Raw ADC Samples of Live X" ) ;
        else 
            poHistoFile->WriteLine ( (CString) " Histograms are Filled Using All 636 Raw ADC Samples" ) ;

        poHistoFile->WriteLine ( (CString) " These Histograms correspond to " + osResultFileName ) ;
        poHistoFile->WriteDateTimeLine ( ) ;
        poHistoFile->WriteRulerLine ( ) ;
    }
    else
    {
        CConsoleAccess::ErrorLine ( "Failed Creating Find_DAC Histogram File" ) ;
        poHistoFile = NULL ; // the histogramming routine will not write to a file
    }

    // Show Find_DAC Mode 
    if ( mg_eFindDacMode == eFindDacMode_LiveX )
        CConsoleAccess::WriteLine ( (CString) " Histograms are Filled Using Only the 36 Raw ADC Samples of  Live X" ) ;
    else 
        CConsoleAccess::WriteLine ( (CString) " Histograms are Filled Using All 636 Raw ADC Samples" ) ;

    // Channel Type is fixed
    if ( poResultFile != NULL )
    {
      if ( _eTTType == eTT_EM )
          poResultFile->WriteLine ( "TT_Channel: $TT_EM_Channel\n" ) ;
      else
          poResultFile->WriteLine ( "TT_Channel: $TT_HD_Channel" ) ;
    }

    for ( ulTTEtaSign = eTT_Eta_Pos       ; ulTTEtaSign <= eTT_Eta_Neg       ; ulTTEtaSign++ )
    {
        if ( poResultFile != NULL )
        {
            if ( ulTTEtaSign == eTT_Eta_Pos )
                poResultFile->WriteLine ( "\n  TT_Eta_Sign: $TT_Eta_Pos" ) ;
            else
                poResultFile->WriteLine ( "\n  TT_Eta_Sign: $TT_Eta_Neg" ) ;
        }


        for ( ulTTEtaMagn = KiMin_TT_Eta_Magn ; ulTTEtaMagn <= KiMax_TT_Eta_Magn ; ulTTEtaMagn++ )
        if ( _pxTT->aalTT_Eta [ulTTEtaSign] [ulTTEtaMagn] == true )
        {
            if ( poResultFile != NULL )
                poResultFile->WriteLine ( (CString) "\n    TT_Eta_Magn: "
                                               + I_to_A ( ulTTEtaMagn )
                                               + "\n" ) ;

            for ( ulTTPhi     = KiMin_TT_Phi      ; ulTTPhi     <= KiMax_TT_Phi      ; ulTTPhi++ )
            if ( _pxTT->alTT_Phi [ulTTPhi] == true )
            {
                poTrgTwr = m_a3poTrgTwr[ulTTEtaSign][ulTTEtaMagn][ulTTPhi] ;
                if ( poTrgTwr != NULL )
                {
                    if ( poTrgTwr-> Find_DAC  ( _eTTType,
                                                &uwDacValue,
                                                &fAverage,
                                                &fDeviation,
                                                &fDevAverage,
                                                 lShowDetails,
                                                 poHistoFile ) != Ok )
                    {
                        eReturnStatus = Error ;
                        //Enter this error as an Error Tag in the TTI command file
                        if ( poResultFile != NULL )
                          poResultFile->WriteLine ( (CString) "     Error_Tag: \""
                                                  + "Pedestal Control Value Not Found for "
                                                  + poTrgTwr-> f_osTrgTwr_Coord ( _eTTType )
                                                  + "\"" ) ;
                    }
                    else
                    {
                        oConsole.WriteLine ( (CString) " Find_DAC for "
                                                      + poTrgTwr-> f_osTrgTwr_Coord ( _eTTType )
                                                      + " found DAC= "
                                                      + I_to_A ( uwDacValue, 4 )
                                                      + " Ave="
                                                      + F_to_A ( fAverage, 2, 5 )
                                                      + " Dev="
                                                      + F_to_A ( fDevAverage, 2, 5 ) ) ;
                    }

                    // Write a line in the TTI file
                    // If Find_DAC has failed the error line above will prevent executing the file as intended.
                    if ( poResultFile != NULL )
                      poResultFile->WriteLine ( (CString) "      TT_Phi: "
                                                        + I_to_A ( ulTTPhi, 2 )
                                                        + "  Pedestal_DAC: "
                                                        + I_to_A ( uwDacValue, 4 )
                                                        + "  Ped_Average: "
                                                        + F_to_A ( fAverage, 2 )
                                                        + "  Ped_Deviation: "
                                                        + F_to_A ( fDevAverage, 2 )
                                                        + "  ! "
                                                        + poTrgTwr->f_osTrgTwr_Coord( _eTTType )
                                                        + "  Selected_Histo_Dev: "
                                                        + F_to_A ( fDeviation, 2 ) ) ;
                }
            }
        }
    }

    // write end time stamp
    if ( poResultFile != NULL )
    {
        poResultFile->WriteLine ( "" ) ;
        poResultFile->WriteRulerLine ( "!" ) ;
        poResultFile->WriteDateTimeLine ( "!" ) ;
    }

    if ( poHistoFile  != NULL )
    {
        poHistoFile->WriteLine ( "" ) ;
        poHistoFile->WriteRulerLine ( ) ;
        poHistoFile->WriteDateTimeLine ( ) ;
    }

    //Show we are done
    oConsole.WriteLine ( " Find_DAC Completed" ) ;

//Abort:

    oConsole.WriteLine ( "" ) ; // skip a line

    // done with files, deletint pointer will close the file too
    if ( poResultFile != NULL ) delete poResultFile ;
    if ( poHistoFile  != NULL ) delete poHistoFile ;

    return eReturnStatus ;
}
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess        CL1Cal::f_okAdjust_Ped  ( ETT_Chan_Type           _eTTType,
                                          TAllTriggerTowers*      _pxTT )
{
    ESuccess        eReturnStatus = Ok ;

    unsigned long   ulTTEtaSign ;
    unsigned long   ulTTEtaMagn ;
    unsigned long   ulTTPhi ;

    TuwRegWord          uwDacValue ;
    double              fAverage ;
    double              fDeviation ;
    short               iwDacAdjust ;
    bool                lShowDetails = true ;

    CString             osResultFileName ;
    CResultFile*        poResultFile = NULL ;

    CString             osHistoFileName = "None" ;
    CResultFile*        poHistoFile = NULL ;

    char                sTtRange[132] ; // generous string to build a trigger tower range
    char*               psTtRange = sTtRange ;

    CL1Cal_TrgTwr*      poTrgTwr ;

    //\\//

    // build a string describing the trigger tower range
    strcpy ( psTtRange, "" ) ;
    CMessageCoorPrinter::Add_Formatted_TT_Range ( psTtRange, _pxTT ) ;

    //build a Result and Histogram File Name
    osResultFileName  = "Adjust_Ped_" ;
    osResultFileName += (CString) KsVersion
                          + "_"
                          + DateTag () ;
    osHistoFileName   = osResultFileName + ".hst" ;
    osResultFileName  = osResultFileName + ".tti" ;
    AddFileVersion ( &osResultFileName ) ;
    AddFileVersion ( &osHistoFileName ) ;

    // Open a file to write the result summary TTI file
    poResultFile = new CResultFile ( osResultFileName ) ;
    if ( poResultFile->OpenForWrite () == Ok )
    {
        //Write a header in the file
        poResultFile->WriteRulerLine ( "!" ) ;
        poResultFile->WriteLine ( (CString) "! Result File written by"
                                       + " L1Cal_IIb_Tcc_"
                                       + KsVersion
                                       + " Adjust_Ped" ) ;

        poResultFile->WriteLine ( (CString) "! Adjusting Pedestal Control DAC Values For " + psTtRange ) ;
        if ( mg_eFindDacMode == eFindDacMode_LiveX )
            poResultFile->WriteLine ( (CString) "! Histograms are Filled Using Only the 36 Raw ADC Samples of  Live X" ) ;
        else 
            poResultFile->WriteLine ( (CString) "! Histograms are Filled Using All 636 Raw ADC Samples" ) ;
        poResultFile->WriteLine ( (CString) "! Histograms recorded in " + osHistoFileName ) ;

        poResultFile->WriteDateTimeLine ( "!" ) ;
        poResultFile->WriteRulerLine ( "!" ) ;

        //Define some symbols in the result file that can be used as a TTI command file
        poResultFile->WriteLine ( "" ) ;
        poResultFile->WriteLine ( "$TT_EM_Channel= 0" ) ; //command file symbols
        poResultFile->WriteLine ( "$TT_HD_Channel= 1" ) ;
        poResultFile->WriteLine ( "" ) ;
        poResultFile->WriteLine ( "$TT_Eta_Pos= 0" ) ;
        poResultFile->WriteLine ( "$TT_Eta_Neg= 1" ) ;
        poResultFile->WriteLine ( "" ) ;
        poResultFile->WriteRulerLine ( "!" ) ;

    }
    else
    {
        CConsoleAccess::ErrorLine ( "Failed Creating Adjust_Ped Result File" ) ;
        poResultFile = NULL ;
    }


    // Open a file to write histogram results
    poHistoFile = new CResultFile ( osHistoFileName ) ;
    if ( poHistoFile->OpenForWrite () == Ok )
    {
        //Write a header in the file
        poHistoFile->WriteRulerLine ( ) ;
        poHistoFile->WriteLine ( (CString) " Histogram File written by"
                                       + " L1Cal_IIb_Tcc_"
                                       + KsVersion
                                       + " Adjust_Ped" ) ;
        poHistoFile->WriteLine ( (CString) " Adjusting Pedestal Control DAC Values For " + psTtRange ) ;
        if ( mg_eFindDacMode == eFindDacMode_LiveX )
            poHistoFile->WriteLine ( (CString) " Histograms are Filled Using Only the 36 Raw ADC Samples of Live X" ) ;
        else 
            poHistoFile->WriteLine ( (CString) " Histograms are Filled Using All 636 Raw ADC Samples" ) ;

        poHistoFile->WriteLine ( (CString) " These Histograms correspond to " + osResultFileName ) ;
        poHistoFile->WriteDateTimeLine ( ) ;
        poHistoFile->WriteRulerLine ( ) ;
    }
    else
    {
        CConsoleAccess::ErrorLine ( "Failed Creating Adjust_Ped Histogram File" ) ;
        poHistoFile = NULL ; // the histogramming routine will not write to a file
    }

    // Show Adjust_Ped Mode 
    if ( mg_eFindDacMode == eFindDacMode_LiveX )
        CConsoleAccess::WriteLine ( (CString) " Histograms are Filled Using Only the 36 Raw ADC Samples of  Live X" ) ;
    else 
        CConsoleAccess::WriteLine ( (CString) " Histograms are Filled Using All 636 Raw ADC Samples" ) ;

    // Channel Type is fixed
    if ( poResultFile != NULL )
    {
      if ( _eTTType == eTT_EM )
          poResultFile->WriteLine ( "TT_Channel: $TT_EM_Channel\n" ) ;
      else
          poResultFile->WriteLine ( "TT_Channel: $TT_HD_Channel" ) ;
    }

    for ( ulTTEtaSign = eTT_Eta_Pos       ; ulTTEtaSign <= eTT_Eta_Neg       ; ulTTEtaSign++ )
    {
        if ( poResultFile != NULL )
        {
            if ( ulTTEtaSign == eTT_Eta_Pos )
                poResultFile->WriteLine ( "\n  TT_Eta_Sign: $TT_Eta_Pos" ) ;
            else
                poResultFile->WriteLine ( "\n  TT_Eta_Sign: $TT_Eta_Neg" ) ;
        }


        for ( ulTTEtaMagn = KiMin_TT_Eta_Magn ; ulTTEtaMagn <= KiMax_TT_Eta_Magn ; ulTTEtaMagn++ )
        if ( _pxTT->aalTT_Eta [ulTTEtaSign] [ulTTEtaMagn] == true )
        {
            if ( poResultFile != NULL )
                poResultFile->WriteLine ( (CString) "\n    TT_Eta_Magn: "
                                               + I_to_A ( ulTTEtaMagn )
                                               + "\n" ) ;

            for ( ulTTPhi     = KiMin_TT_Phi      ; ulTTPhi     <= KiMax_TT_Phi      ; ulTTPhi++ )
            if ( _pxTT->alTT_Phi [ulTTPhi] == true )
            {
                poTrgTwr = m_a3poTrgTwr[ulTTEtaSign][ulTTEtaMagn][ulTTPhi] ;
                if ( poTrgTwr != NULL )
                {
                    if ( poTrgTwr-> Adjust_Ped( _eTTType,
                                                &uwDacValue,
                                                &fAverage,
                                                &fDeviation,
                                                &iwDacAdjust,
                                                 lShowDetails,
                                                 poHistoFile ) != Ok )
                    {
                        eReturnStatus = Error ;
                        //Enter this error as an Error Tag in the TTI command file
                        if ( poResultFile != NULL )
                          poResultFile->WriteLine ( (CString) "     Error_Tag: \""
                                                  + "Pedestal Control Value Not Found for "
                                                  + poTrgTwr-> f_osTrgTwr_Coord ( _eTTType )
                                                  + "\"" ) ;
                    }
                    else
                    {
                        oConsole.WriteLine ( (CString) " Adjust_Ped for "
                                                      + poTrgTwr-> f_osTrgTwr_Coord ( _eTTType )
                                                      + " found DAC= "
                                                      + I_to_A ( uwDacValue, 4 )
                                                      + " Ave="
                                                      + F_to_A ( fAverage, 2, 5 )
                                                      + " Dev="
                                                      + F_to_A ( fDeviation, 2, 5 ) ) ;
                    }

                    // Write a line in the TTI file
                    // If Adjust_Ped has failed the error line above will prevent executing the file as intended.
                    if ( poResultFile != NULL )
                    {
                        CString osForcedPlusIfNeeded = "" ;
                        if ( iwDacAdjust > 0 ) osForcedPlusIfNeeded = "+" ;
                        poResultFile->WriteLine ( (CString) "      TT_Phi: "
                                                        + I_to_A ( ulTTPhi, 2 )
                                                        + "  Pedestal_DAC: "
                                                        + I_to_A ( uwDacValue, 4 )
                                                        + "  Ped_Average: "
                                                        + F_to_A ( fAverage, 2 )
                                                        + "  Ped_Deviation: "
                                                        + F_to_A ( fDeviation, 2 )
                                                        + " ! "
                                                        + poTrgTwr->f_osTrgTwr_Coord( _eTTType ) 
                                                        + " Adjusted by "
                                                        + osForcedPlusIfNeeded
                                                        + I_to_A ( iwDacAdjust) ) ;
                    }
                }
            }
        }
    }

    // write end time stamp
    if ( poResultFile != NULL )
    {
        poResultFile->WriteLine ( "" ) ;
        poResultFile->WriteRulerLine ( "!" ) ;
        poResultFile->WriteDateTimeLine ( "!" ) ;
    }

    if ( poHistoFile  != NULL )
    {
        poHistoFile->WriteLine ( "" ) ;
        poHistoFile->WriteRulerLine ( ) ;
        poHistoFile->WriteDateTimeLine ( ) ;
    }

    //Show we are done
    oConsole.WriteLine ( " Adjust_Ped Completed" ) ;

//Abort:

    oConsole.WriteLine ( "" ) ; // skip a line

    // done with files, deletint pointer will close the file too
    if ( poResultFile != NULL ) delete poResultFile ;
    if ( poHistoFile  != NULL ) delete poHistoFile ;

    return eReturnStatus ;
}
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess        CL1Cal::f_okRevert_Ped  ( ETT_Chan_Type           _eTTType,
                                          TAllTriggerTowers*      _pxTT )
{
    ESuccess        eReturnStatus = Ok ;

    unsigned long   ulTTEtaSign ;
    unsigned long   ulTTEtaMagn ;
    unsigned long   ulTTPhi ;

    TuwRegWord          uwDacValue ;
    bool                lShowDetails = true ;

    CString             osResultFileName ;
    CResultFile*        poResultFile = NULL ;

    char                sTtRange[132] ; // generous string to build a trigger tower range
    char*               psTtRange = sTtRange ;

    CL1Cal_TrgTwr*      poTrgTwr ;

    //\\//

    // build a string describing the trigger tower range
    strcpy ( psTtRange, "" ) ;
    CMessageCoorPrinter::Add_Formatted_TT_Range ( psTtRange, _pxTT ) ;

    //build a Result and Histogram File Name
    osResultFileName  = "Revert_Ped_" ;
    osResultFileName += (CString) KsVersion
                          + "_"
                          + DateTag () ;
    osResultFileName  = osResultFileName + ".tti" ;
    AddFileVersion ( &osResultFileName ) ;

    // Open a file to write the result summary TTI file
    poResultFile = new CResultFile ( osResultFileName ) ;
    if ( poResultFile->OpenForWrite () == Ok )
    {
        //Write a header in the file
        poResultFile->WriteRulerLine ( "!" ) ;
        poResultFile->WriteLine ( (CString) "! Result File written by"
                                       + " L1Cal_IIb_Tcc_"
                                       + KsVersion
                                       + " Revert_Ped" ) ;

        poResultFile->WriteLine ( (CString) "! Reverting Pedestal DAC Values For " + psTtRange ) ;

        poResultFile->WriteDateTimeLine ( "!" ) ;
        poResultFile->WriteRulerLine ( "!" ) ;

        //Define some symbols in the result file that can be used as a TTI command file
        poResultFile->WriteLine ( "" ) ;
        poResultFile->WriteLine ( "$TT_EM_Channel= 0" ) ; //command file symbols
        poResultFile->WriteLine ( "$TT_HD_Channel= 1" ) ;
        poResultFile->WriteLine ( "" ) ;
        poResultFile->WriteLine ( "$TT_Eta_Pos= 0" ) ;
        poResultFile->WriteLine ( "$TT_Eta_Neg= 1" ) ;
        poResultFile->WriteLine ( "" ) ;
        poResultFile->WriteRulerLine ( "!" ) ;

    }
    else
    {
        CConsoleAccess::ErrorLine ( "Failed Creating Revert_Ped Result File" ) ;
        poResultFile = NULL ;
    }


    // Channel Type is fixed
    if ( poResultFile != NULL )
    {
      if ( _eTTType == eTT_EM )
          poResultFile->WriteLine ( "TT_Channel: $TT_EM_Channel\n" ) ;
      else
          poResultFile->WriteLine ( "TT_Channel: $TT_HD_Channel" ) ;
    }

    for ( ulTTEtaSign = eTT_Eta_Pos       ; ulTTEtaSign <= eTT_Eta_Neg       ; ulTTEtaSign++ )
    {
        if ( poResultFile != NULL )
        {
            if ( ulTTEtaSign == eTT_Eta_Pos )
                poResultFile->WriteLine ( "\n  TT_Eta_Sign: $TT_Eta_Pos" ) ;
            else
                poResultFile->WriteLine ( "\n  TT_Eta_Sign: $TT_Eta_Neg" ) ;
        }


        for ( ulTTEtaMagn = KiMin_TT_Eta_Magn ; ulTTEtaMagn <= KiMax_TT_Eta_Magn ; ulTTEtaMagn++ )
        if ( _pxTT->aalTT_Eta [ulTTEtaSign] [ulTTEtaMagn] == true )
        {
            if ( poResultFile != NULL )
                poResultFile->WriteLine ( (CString) "\n    TT_Eta_Magn: "
                                               + I_to_A ( ulTTEtaMagn )
                                               + "\n" ) ;

            for ( ulTTPhi     = KiMin_TT_Phi      ; ulTTPhi     <= KiMax_TT_Phi      ; ulTTPhi++ )
            if ( _pxTT->alTT_Phi [ulTTPhi] == true )
            {
                poTrgTwr = m_a3poTrgTwr[ulTTEtaSign][ulTTEtaMagn][ulTTPhi] ;
                if ( poTrgTwr != NULL )
                {
                    if ( poTrgTwr-> Revert_Ped( _eTTType,
                                                &uwDacValue,
                                                 lShowDetails ) != Ok )
                    {
                        eReturnStatus = Error ;
                        //Enter this error as an Error Tag in the TTI command file
                        if ( poResultFile != NULL )
                          poResultFile->WriteLine ( (CString) "     Error_Tag: \""
                                                  + "Pedestal Control Value Not Set for "
                                                  + poTrgTwr-> f_osTrgTwr_Coord ( _eTTType )
                                                  + "\"" ) ;
                    }
                    else
                    {
                        oConsole.WriteLine ( (CString) " Revert_Ped for "
                                                      + poTrgTwr-> f_osTrgTwr_Coord ( _eTTType )
                                                      + " set DAC= "
                                                      + I_to_A ( uwDacValue, 4 ) ) ;
                    }

                    // Write a line in the TTI file
                    // If Revert_Ped has failed the error line above will prevent executing the file as intended.
                    if ( poResultFile != NULL )
                      poResultFile->WriteLine ( (CString) "      TT_Phi: "
                                                        + I_to_A ( ulTTPhi, 2 )
                                                        + "  Pedestal_DAC: "
                                                        + I_to_A ( uwDacValue, 4 )
                                                        + "  ! "
                                                        + poTrgTwr->f_osTrgTwr_Coord( _eTTType ) ) ;
                }
            }
        }
    }

    // write end time stamp
    if ( poResultFile != NULL )
    {
        poResultFile->WriteLine ( "" ) ;
        poResultFile->WriteRulerLine ( "!" ) ;
        poResultFile->WriteDateTimeLine ( "!" ) ;
    }


    //Show we are done
    oConsole.WriteLine ( " Revert_Ped Completed" ) ;

//Abort:

    oConsole.WriteLine ( "" ) ; // skip a line

    // done with files, deletint pointer will close the file too
    if ( poResultFile != NULL ) delete poResultFile ;

    return eReturnStatus ;
}
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess        CL1Cal::f_okProfile_BLS  ( ETT_Chan_Type           _eTTType,
                                           TAllTriggerTowers*      _pxTT )
{
    ESuccess        eReturnStatus = Ok ;

    unsigned long   ulTTEtaSign ;
    unsigned long   ulTTEtaMagn ;
    unsigned long   ulTTPhi ;

    double          fAveOfAveAll ;
    double          fDevOfAveAll ;
    double          fAveOfDevAll ;
    double          fDevOfDevAll ;
    double          fAveOfAveLiveX ;
    double          fDevOfAveLiveX ;
    double          fAveOfDevLiveX ;
    double          fDevOfDevLiveX ;
    bool            lShowDetails = true ;

    CString             osResultFileName ;
    CResultFile*        poResultFile = NULL ;

    CString             osProfiFileName = "None" ;
    CResultFile*        poProfiFile = NULL ;

    char                sTtRange[132] ; // generous string to build a trigger tower range
    char*               psTtRange = sTtRange ;

    CL1Cal_TrgTwr*      poTrgTwr ;

    //\\//

    // build a string describing the trigger tower range
    strcpy ( psTtRange, "" ) ;
    CMessageCoorPrinter::Add_Formatted_TT_Range ( psTtRange, _pxTT ) ;

    //build a Result and Profile File Name
    osResultFileName  = "Prof_BLS_" ;
    osResultFileName += (CString) KsVersion
                          + "_"
                          + DateTag () ;
    osProfiFileName   = osResultFileName + ".prf" ;
    osResultFileName  = osResultFileName + ".tti" ;
    AddFileVersion ( &osResultFileName ) ;
    AddFileVersion ( &osProfiFileName ) ;

    // Open a file to write the result summary TTI file
    poResultFile = new CResultFile ( osResultFileName ) ;
    if ( poResultFile->OpenForWrite () == Ok )
    {
        //Write a header in the file
        poResultFile->WriteRulerLine ( "!" ) ;
        poResultFile->WriteLine ( (CString) "! Result File written by"
                                       + " L1Cal_IIb_Tcc_"
                                       + KsVersion
                                       + " Profile_BLS" ) ;

        poResultFile->WriteDateTimeLine ( "!" ) ;
        poResultFile->WriteRulerLine ( "!" ) ;

        //Define some symbols in the result file that can be used as a TTI command file
        poResultFile->WriteLine ( "" ) ;
        poResultFile->WriteLine ( "$TT_EM_Channel= 0" ) ; //command file symbols
        poResultFile->WriteLine ( "$TT_HD_Channel= 1" ) ;
        poResultFile->WriteLine ( "" ) ;
        poResultFile->WriteLine ( "$TT_Eta_Pos= 0" ) ;
        poResultFile->WriteLine ( "$TT_Eta_Neg= 1" ) ;
        poResultFile->WriteLine ( "" ) ;
        poResultFile->WriteRulerLine ( "!" ) ;

    }
    else
    {
        CConsoleAccess::ErrorLine ( "Failed Creating BLS Profiles Summary File" ) ;
        poResultFile = NULL ;
    }


    // Open a file to write profile results
    poProfiFile = new CResultFile ( osProfiFileName ) ;
    if ( poProfiFile->OpenForWrite () == Ok )
    {
        //Write a header in the file
        poProfiFile->WriteRulerLine ( ) ;
        poProfiFile->WriteLine ( (CString) " Profile BLS File written by"
                                       + " L1Cal_IIb_Tcc_"
                                       + KsVersion ) ;
        poProfiFile->WriteLine ( (CString) " Finding BLS Profiles For " + psTtRange ) ;
        poProfiFile->WriteLine ( (CString) " These Profiles correspond to " + osResultFileName ) ;
        poProfiFile->WriteDateTimeLine ( ) ;
        poProfiFile->WriteRulerLine ( ) ;
    }
    else
    {
        CConsoleAccess::ErrorLine ( "Failed Creating BLS Profiles Details File" ) ;
        poProfiFile = NULL ; // the profile routine will not write to a file
    }


    // Channel Type is fixed
    if ( poResultFile != NULL )
    {
      if ( _eTTType == eTT_EM )
          poResultFile->WriteLine ( "TT_Channel: $TT_EM_Channel\n" ) ;
      else
          poResultFile->WriteLine ( "TT_Channel: $TT_HD_Channel" ) ;
    }

    for ( ulTTEtaSign = eTT_Eta_Pos       ; ulTTEtaSign <= eTT_Eta_Neg       ; ulTTEtaSign++ )
    {
        if ( poResultFile != NULL )
        {
            if ( ulTTEtaSign == eTT_Eta_Pos )
                poResultFile->WriteLine ( "\n  TT_Eta_Sign: $TT_Eta_Pos" ) ;
            else
                poResultFile->WriteLine ( "\n  TT_Eta_Sign: $TT_Eta_Neg" ) ;
        }


        for ( ulTTEtaMagn = KiMin_TT_Eta_Magn ; ulTTEtaMagn <= KiMax_TT_Eta_Magn ; ulTTEtaMagn++ )
        if ( _pxTT->aalTT_Eta [ulTTEtaSign] [ulTTEtaMagn] == true )
        {
            if ( poResultFile != NULL )
                poResultFile->WriteLine ( (CString) "\n    TT_Eta_Magn: "
                                               + I_to_A ( ulTTEtaMagn )
                                               + "\n" ) ;

            for ( ulTTPhi     = KiMin_TT_Phi      ; ulTTPhi     <= KiMax_TT_Phi      ; ulTTPhi++ )
            if ( _pxTT->alTT_Phi [ulTTPhi] == true )
            {
                poTrgTwr = m_a3poTrgTwr[ulTTEtaSign][ulTTEtaMagn][ulTTPhi] ;
                if ( poTrgTwr != NULL )
                {
                    if ( poTrgTwr-> Profile_BLS   ( _eTTType,
                                                    &fAveOfAveAll,
                                                    &fDevOfAveAll,
                                                    &fAveOfDevAll,
                                                    &fDevOfDevAll,
                                                    &fAveOfAveLiveX,
                                                    &fDevOfAveLiveX,
                                                    &fAveOfDevLiveX,
                                                    &fDevOfDevLiveX,
                                                     lShowDetails,
                                                     poProfiFile ) != Ok )
                    {
                        eReturnStatus = Error ;
                        //Enter this error as an Error Tag in the TTI command file
                        if ( poResultFile != NULL )
                          poResultFile->WriteLine ( (CString) "     Error_Tag: \""
                                                  + "Profile Not Found for "
                                                  + poTrgTwr-> f_osTrgTwr_Coord ( _eTTType )
                                                  + "\"" ) ;
                    }
                    else
                    {
                        oConsole.WriteLine ( (CString) " BLS "
                                                         + poTrgTwr-> f_osTrgTwr_Coord ( _eTTType )
                                                         + " RawAll AofA: "
                                                         + F_to_A ( fAveOfAveAll, 2 )
                                                         + " DofA: "
                                                         + F_to_A ( fDevOfAveAll, 2 )
                                                         + " LiveX AofA: "
                                                         + F_to_A ( fAveOfAveLiveX, 2 )
                                                         + " DofA: "
                                                         + F_to_A ( fDevOfAveLiveX, 2 ) ) ;
                    }

                    // Write a line in the TTI file
                    // If Profile_BLS has failed the error line above will prevent executing the file as intended.
                    if ( poResultFile != NULL )
                      poResultFile->WriteLine ( (CString) "      TT_Phi: "
                                                         + I_to_A ( ulTTPhi, 2 )
                                                         + " RawAllAveOfAve: "
                                                         + F_to_A ( fAveOfAveAll, 2 )
                                                         + " RawAllDevOfAve: "
                                                         + F_to_A ( fDevOfAveAll, 2 )
                                                         + " RawAllAveOfDev: "
                                                         + F_to_A ( fAveOfDevAll, 2 )
                                                         + " RawAllDevOfDev: "
                                                         + F_to_A ( fDevOfDevAll, 2 )
                                                         + " RawLvXAveOfAve: "
                                                         + F_to_A ( fAveOfAveLiveX, 2 )
                                                         + " RawLvXDevOfAve: "
                                                         + F_to_A ( fDevOfAveLiveX, 2 )
                                                         + " RawLvXAveOfDev: "
                                                         + F_to_A ( fAveOfDevLiveX, 2 )
                                                         + " RawLvXDevOfDev: "
                                                         + F_to_A ( fDevOfDevLiveX, 2 ) 
                                                         + "  ! "
                                                         + poTrgTwr->f_osTrgTwr_Coord( _eTTType ) ) ;
                }
            }
        }
    }

    // write end time stamp
    if ( poResultFile != NULL )
    {
        poResultFile->WriteLine ( "" ) ;
        poResultFile->WriteRulerLine ( "!" ) ;
        poResultFile->WriteDateTimeLine ( "!" ) ;
    }

    if ( poProfiFile  != NULL )
    {
        poProfiFile->WriteLine ( "" ) ;
        poProfiFile->WriteRulerLine ( ) ;
        poProfiFile->WriteDateTimeLine ( ) ;
    }

    //Show we are done
    oConsole.WriteLine ( " BLS Profile Completed" ) ;

//Abort:

    oConsole.WriteLine ( "" ) ; // skip a line

    // done with files, deletint pointer will close the file too
    if ( poResultFile != NULL ) delete poResultFile ;
    if ( poProfiFile  != NULL ) delete poProfiFile ;

    return eReturnStatus ;
}
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*//*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess        CL1Cal::f_okProfile_AdfEt     ( ETT_Chan_Type           _eTTType,
                                                TAllTriggerTowers*      _pxTT )
{
    ESuccess        eReturnStatus = Ok ;

    unsigned long   ulTTEtaSign ;
    unsigned long   ulTTEtaMagn ;
    unsigned long   ulTTPhi ;

    double          fAveOfAveLiveX ;
    double          fDevOfAveLiveX ;
    double          fAveOfDevLiveX ;
    double          fDevOfDevLiveX ;
    bool            lShowDetails = true ;

    CString             osResultFileName ;
    CResultFile*        poResultFile = NULL ;

    CString             osProfiFileName = "None" ;
    CResultFile*        poProfiFile = NULL ;

    char                sTtRange[132] ; // generous string to build a trigger tower range
    char*               psTtRange = sTtRange ;

    CL1Cal_TrgTwr*      poTrgTwr ;

    //\\//

    // build a string describing the trigger tower range
    strcpy ( psTtRange, "" ) ;
    CMessageCoorPrinter::Add_Formatted_TT_Range ( psTtRange, _pxTT ) ;

    //build a Result and Profile File Name
    osResultFileName  = "Prof_AdfEt_" ;
    osResultFileName += (CString) KsVersion
                          + "_"
                          + DateTag () ;
    osProfiFileName   = osResultFileName + ".prf" ;
    osResultFileName  = osResultFileName + ".tti" ;
    AddFileVersion ( &osResultFileName ) ;
    AddFileVersion ( &osProfiFileName ) ;

    // Open a file to write the result summary TTI file
    poResultFile = new CResultFile ( osResultFileName ) ;
    if ( poResultFile->OpenForWrite () == Ok )
    {
        //Write a header in the file
        poResultFile->WriteRulerLine ( "!" ) ;
        poResultFile->WriteLine ( (CString) "! Result File written by"
                                       + " L1Cal_IIb_Tcc_"
                                       + KsVersion
                                       + " Profile_AdfEt" ) ;

        poResultFile->WriteDateTimeLine ( "!" ) ;
        poResultFile->WriteRulerLine ( "!" ) ;

        //Define some symbols in the result file that can be used as a TTI command file
        poResultFile->WriteLine ( "" ) ;
        poResultFile->WriteLine ( "$TT_EM_Channel= 0" ) ; //command file symbols
        poResultFile->WriteLine ( "$TT_HD_Channel= 1" ) ;
        poResultFile->WriteLine ( "" ) ;
        poResultFile->WriteLine ( "$TT_Eta_Pos= 0" ) ;
        poResultFile->WriteLine ( "$TT_Eta_Neg= 1" ) ;
        poResultFile->WriteLine ( "" ) ;
        poResultFile->WriteRulerLine ( "!" ) ;

    }
    else
    {
        CConsoleAccess::ErrorLine ( "Failed Creating ADF Output Et Summary File" ) ;
        poResultFile = NULL ;
    }


    // Open a file to write profile results
    poProfiFile = new CResultFile ( osProfiFileName ) ;
    if ( poProfiFile->OpenForWrite () == Ok )
    {
        //Write a header in the file
        poProfiFile->WriteRulerLine ( ) ;
        poProfiFile->WriteLine ( (CString) " Profile ADF Output Et File written by"
                                       + " L1Cal_IIb_Tcc_"
                                       + KsVersion ) ;
        poProfiFile->WriteLine ( (CString) " Finding ADF Output Et For " + psTtRange ) ;
        poProfiFile->WriteLine ( (CString) " These Profiles correspond to " + osResultFileName ) ;
        poProfiFile->WriteDateTimeLine ( ) ;
        poProfiFile->WriteRulerLine ( ) ;
    }
    else
    {
        CConsoleAccess::ErrorLine ( "Failed Creating ADF Output Et Details File" ) ;
        poProfiFile = NULL ; // the profile routine will not write to a file
    }


    // Channel Type is fixed
    if ( poResultFile != NULL )
    {
      if ( _eTTType == eTT_EM )
          poResultFile->WriteLine ( "TT_Channel: $TT_EM_Channel\n" ) ;
      else
          poResultFile->WriteLine ( "TT_Channel: $TT_HD_Channel" ) ;
    }

    for ( ulTTEtaSign = eTT_Eta_Pos       ; ulTTEtaSign <= eTT_Eta_Neg       ; ulTTEtaSign++ )
    {
        if ( poResultFile != NULL )
        {
            if ( ulTTEtaSign == eTT_Eta_Pos )
                poResultFile->WriteLine ( "\n  TT_Eta_Sign: $TT_Eta_Pos" ) ;
            else
                poResultFile->WriteLine ( "\n  TT_Eta_Sign: $TT_Eta_Neg" ) ;
        }


        for ( ulTTEtaMagn = KiMin_TT_Eta_Magn ; ulTTEtaMagn <= KiMax_TT_Eta_Magn ; ulTTEtaMagn++ )
        if ( _pxTT->aalTT_Eta [ulTTEtaSign] [ulTTEtaMagn] == true )
        {
            if ( poResultFile != NULL )
                poResultFile->WriteLine ( (CString) "\n    TT_Eta_Magn: "
                                               + I_to_A ( ulTTEtaMagn )
                                               + "\n" ) ;

            for ( ulTTPhi     = KiMin_TT_Phi      ; ulTTPhi     <= KiMax_TT_Phi      ; ulTTPhi++ )
            if ( _pxTT->alTT_Phi [ulTTPhi] == true )
            {
                poTrgTwr = m_a3poTrgTwr[ulTTEtaSign][ulTTEtaMagn][ulTTPhi] ;
                if ( poTrgTwr != NULL )
                {
                    if ( poTrgTwr-> Profile_AdfEt ( _eTTType,
                                                    &fAveOfAveLiveX,
                                                    &fDevOfAveLiveX,
                                                    &fAveOfDevLiveX,
                                                    &fDevOfDevLiveX,
                                                     lShowDetails,
                                                     poProfiFile ) != Ok )
                    {
                        eReturnStatus = Error ;
                        //Enter this error as an Error Tag in the TTI command file
                        if ( poResultFile != NULL )
                          poResultFile->WriteLine ( (CString) "     Error_Tag: \""
                                                  + "Profile Not Found for "
                                                  + poTrgTwr-> f_osTrgTwr_Coord ( _eTTType )
                                                  + "\"" ) ;
                    }
                    else
                    {
                        oConsole.WriteLine ( (CString) " ADF Et "
                                                         + poTrgTwr-> f_osTrgTwr_Coord ( _eTTType )
                                                         + " LiveX AveOfAve: "
                                                         + F_to_A ( fAveOfAveLiveX, 2 )
                                                         + " DevOfAve: "
                                                         + F_to_A ( fDevOfAveLiveX, 2 ) 
                                                         + " AveOfDev: "
                                                         + F_to_A ( fAveOfDevLiveX, 2 ) ) ;
                    }

                    // Write a line in the TTI file
                    // If Profile_AdfEt has failed the error line above will prevent executing the file as intended.
                    if ( poResultFile != NULL )
                      poResultFile->WriteLine ( (CString) "      TT_Phi: "
                                                         + I_to_A ( ulTTPhi, 2 )
                                                         + " AdfEtLvXAveOfAve: "
                                                         + F_to_A ( fAveOfAveLiveX, 2 )
                                                         + " AdfEtLvXDevOfAve: "
                                                         + F_to_A ( fDevOfAveLiveX, 2 )
                                                         + " AdfEtLvXAveOfDev: "
                                                         + F_to_A ( fAveOfDevLiveX, 2 )
                                                         + " AdfEtLvXDevOfDev: "
                                                         + F_to_A ( fDevOfDevLiveX, 2 ) 
                                                         + "  ! "
                                                         + poTrgTwr->f_osTrgTwr_Coord( _eTTType ) ) ;
                }
            }
        }
    }

    // write end time stamp
    if ( poResultFile != NULL )
    {
        poResultFile->WriteLine ( "" ) ;
        poResultFile->WriteRulerLine ( "!" ) ;
        poResultFile->WriteDateTimeLine ( "!" ) ;
    }

    if ( poProfiFile  != NULL )
    {
        poProfiFile->WriteLine ( "" ) ;
        poProfiFile->WriteRulerLine ( ) ;
        poProfiFile->WriteDateTimeLine ( ) ;
    }

    //Show we are done
    oConsole.WriteLine ( " Profile ADF Output Et Completed" ) ;

//Abort:

    oConsole.WriteLine ( "" ) ; // skip a line

    // done with files, deletint pointer will close the file too
    if ( poResultFile != NULL ) delete poResultFile ;
    if ( poProfiFile  != NULL ) delete poProfiFile ;

    return eReturnStatus ;
}
/*-------------------------------------------------------------------------------*/


/*-------------------------------------------------------------------------------*/
ESuccess        CL1Cal::f_okExclude_Tower ( ETT_Chan_Type           _eTTType,
                                            TAllTriggerTowers*      _pxTT,
                                            bool                    _lVerbose )
{
    ESuccess        eReturnStatus = Ok ;

    unsigned long   ulTTEtaSign ;
    unsigned long   ulTTEtaMagn ;
    unsigned long   ulTTPhi ;

    //\\//

    for ( ulTTEtaSign = eTT_Eta_Pos       ; ulTTEtaSign <= eTT_Eta_Neg       ; ulTTEtaSign++ )
    for ( ulTTEtaMagn = KiMin_TT_Eta_Magn ; ulTTEtaMagn <= KiMax_TT_Eta_Magn ; ulTTEtaMagn++ )
    if ( _pxTT->aalTT_Eta [ulTTEtaSign] [ulTTEtaMagn] == true )
    for ( ulTTPhi     = KiMin_TT_Phi      ; ulTTPhi     <= KiMax_TT_Phi      ; ulTTPhi++ )
    if ( _pxTT->alTT_Phi [ulTTPhi] == true )
    {
        if ( m_a3poTrgTwr[ulTTEtaSign][ulTTEtaMagn][ulTTPhi] != NULL )
        if ( m_a3poTrgTwr[ulTTEtaSign][ulTTEtaMagn][ulTTPhi]
                          -> f_okExclude_Tower  ( _eTTType,
                                                  _lVerbose ) != Ok )
           eReturnStatus = Error ;
    }

    return eReturnStatus ;
}
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess        CL1Cal::f_okAdfOut_Constant   ( ETT_Chan_Type           _eTTType,
                                                TAllTriggerTowers*      _pxTT,
                                                const unsigned long     _ulConstantData )
{
    ESuccess        eReturnStatus = Ok ;

    unsigned long   ulTTEtaSign ;
    unsigned long   ulTTEtaMagn ;
    unsigned long   ulTTPhi ;

    //\\//

    for ( ulTTEtaSign = eTT_Eta_Pos       ; ulTTEtaSign <= eTT_Eta_Neg       ; ulTTEtaSign++ )
    for ( ulTTEtaMagn = KiMin_TT_Eta_Magn ; ulTTEtaMagn <= KiMax_TT_Eta_Magn ; ulTTEtaMagn++ )
    if ( _pxTT->aalTT_Eta [ulTTEtaSign] [ulTTEtaMagn] == true )
    for ( ulTTPhi     = KiMin_TT_Phi      ; ulTTPhi     <= KiMax_TT_Phi      ; ulTTPhi++ )
    if ( _pxTT->alTT_Phi [ulTTPhi] == true )
    {
        if ( m_a3poTrgTwr[ulTTEtaSign][ulTTEtaMagn][ulTTPhi] != NULL )
        if ( m_a3poTrgTwr[ulTTEtaSign][ulTTEtaMagn][ulTTPhi]
                          -> f_okOutputConstant   ( _eTTType,
                                                     fubTruncToByte(_ulConstantData) ) != Ok )
           eReturnStatus = Error ;
    }

    return eReturnStatus ;
}
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess        CL1Cal::f_okRawAdc_Thresh      ( ETT_Chan_Type           _eTTType,
                                                 TAllTriggerTowers*      _pxTT,
                                                 const unsigned long     _ulThreshold )
{
    ESuccess        eReturnStatus = Ok ;

    unsigned long   ulTTEtaSign ;
    unsigned long   ulTTEtaMagn ;
    unsigned long   ulTTPhi ;

    //\\//

    for ( ulTTEtaSign = eTT_Eta_Pos       ; ulTTEtaSign <= eTT_Eta_Neg       ; ulTTEtaSign++ )
    for ( ulTTEtaMagn = KiMin_TT_Eta_Magn ; ulTTEtaMagn <= KiMax_TT_Eta_Magn ; ulTTEtaMagn++ )
    if ( _pxTT->aalTT_Eta [ulTTEtaSign] [ulTTEtaMagn] == true )
    for ( ulTTPhi     = KiMin_TT_Phi      ; ulTTPhi     <= KiMax_TT_Phi      ; ulTTPhi++ )
    if ( _pxTT->alTT_Phi [ulTTPhi] == true )
    {
        if ( m_a3poTrgTwr[ulTTEtaSign][ulTTEtaMagn][ulTTPhi] != NULL )
        if ( m_a3poTrgTwr[ulTTEtaSign][ulTTEtaMagn][ulTTPhi]
                          -> f_okRawAdcThresh     ( _eTTType,
                                                     fubTruncToByte(_ulThreshold) ) != Ok )
           eReturnStatus = Error ;
    }

    return eReturnStatus ;
}
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
CL1Cal_TrgTwr*   CL1Cal::GetTrigTwrPointer  ( unsigned long   _ulTTEtaSign,
                                              unsigned long   _ulTTEtaMagn,
                                              unsigned long   _ulTTPhi )
{
    CL1Cal_TrgTwr*       pxThisTrgTwr = NULL ;

    //\\//

    if ( ( _ulTTEtaSign >= eTot_TT_Eta_Sign ) 
      || ( _ulTTEtaMagn == 0 ) 
      || ( _ulTTEtaMagn >  eTot_TT_Eta) 
      || ( _ulTTPhi     == 0 ) 
      || ( _ulTTPhi     >  eTot_TT_Phi) )
    {
        oConsole.ErrorLine ( (CString) "Illegal Trigger Tower Coordinates" ) ;
        goto Abort;
    }

    if ( mg_poL1Cal == NULL )
    {
        oConsole.InternalErrorLine ( (CString) "CL1Cal::GetTrigTwrPointer: Global L1Cal Pointer NULL" ) ;
        goto Abort;
    }

    pxThisTrgTwr = mg_poL1Cal->m_a3poTrgTwr[_ulTTEtaSign][_ulTTEtaMagn][_ulTTPhi] ;

    if ( pxThisTrgTwr == NULL )
    {
        oConsole.InternalErrorLine ( (CString) "CL1Cal::GetTrigTwrPointer: TrgTwr Pointer NULL for "
                                            + CL1Cal_TrgTwr::f_osTrgTwr_Coord  ( _ulTTEtaSign,
                                                                                _ulTTEtaMagn,
                                                                                _ulTTPhi ) ) ;
        goto Abort;
    }

    if ( ( pxThisTrgTwr->f_ulTTEtaSign () != (unsigned long) _ulTTEtaSign )
      || ( pxThisTrgTwr->f_ulTTEtaMagn () != (unsigned long) _ulTTEtaMagn )
      || ( pxThisTrgTwr->f_ulTTPhi     () != (unsigned long) _ulTTPhi ) )
    {
        pxThisTrgTwr = NULL ;
        oConsole.InternalErrorLine ( (CString) "CL1Cal::GetTrigTwrPointer TrgTwr or Pointer Corrupted for"
                                            + CL1Cal_TrgTwr::f_osTrgTwr_Coord  ( _ulTTEtaSign,
                                                                                _ulTTEtaMagn,
                                                                                _ulTTPhi ) ) ;
        goto Abort;
    }

Abort:
    return pxThisTrgTwr ;
}
/*-------------------------------------------------------------------------------*/


/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess        CL1Cal::f_okRefSetProg( ETabRefSet_Type         _eRefSetType,
                                        const unsigned long     _ulRefSetNum,
                                        TAllTriggerTowers*      _pxTT,
                                        const double            _fdThreshold )
{
    ESuccess        eReturnStatus = Ok ;

    unsigned long   ulTTEtaSign ;
    unsigned long   ulTTEtaMagnBy4 ;
    unsigned long   ulTTPhiBy4 ;

    unsigned long   ulTTEtaMagn ;
    unsigned long   ulTTPhi ;

    unsigned long   ulThreshold ;
    long            ilLowerRefSetNum ;

    //\\//

    //Programming a Reference Set implicitely allocates it
    m_a2eRefSetalloc [_eRefSetType][_ulRefSetNum] = Yes ;

    //Convert Floating Point GeV to Integer ADC Counts
    ulThreshold = (unsigned long) ( _fdThreshold / KfL1CalEtScale + 0.5 ) ;

    // take care of saturation
    switch ( _eRefSetType)
    {
      case eTab_RefSet_EMEt :
        if ( ulThreshold > kiTabEmEtTreshMax  )    ulThreshold = kiTabEmEtTreshMax ;
      break ;

      case eTab_RefSet_JetEt :
        if ( ulThreshold > kiTabJetEtTreshMax )    ulThreshold = kiTabJetEtTreshMax ;
      break ;

      case eTab_RefSet_TTEt :
        ulThreshold += 2 * KiOutputEtZer ; // account for the zero energy response
        if ( ulThreshold > kiTabTtEtTreshMax  )    ulThreshold = kiTabTtEtTreshMax ;
      break ;

      default :
      break ;
    }


    // Verify the monotonicity
    // note that syntax parsing has already checked for tab chip boundaries
    if ( _eRefSetType != eTab_RefSet_TTEt ) // not necessary for TT ref sets
    for ( ulTTPhiBy4     =(m_ulPhiMin-1)/4     ; ulTTPhiBy4     <=(m_ulPhiMax-1)/4     ; ulTTPhiBy4++ )
    {
        ulTTPhi     = 4 * ulTTPhiBy4 + 1 ; // compute the base phi coordinate

        if ( _pxTT->alTT_Phi [ulTTPhi] == true )    // check if this sub-range is requested
        for ( ulTTEtaSign    = m_ulSignEtaMin      ; ulTTEtaSign    <= m_ulSignEtaMax      ; ulTTEtaSign++ )
        for ( ulTTEtaMagnBy4 =(m_ulMagnEtaMin-1)/4 ; ulTTEtaMagnBy4 <=(m_ulMagnEtaMax-1)/4 ; ulTTEtaMagnBy4++ )
        {
            ulTTEtaMagn = 4 * ulTTEtaMagnBy4 + 1 ; // compute the base eta coordinate

            if ( _pxTT->aalTT_Eta [ulTTEtaSign] [ulTTEtaMagn] == true ) // check if this sub-range is requested
            {
                // Verify this threshold is not greater than next upper allocated threshold
                // we can simply verify against the immediately upper threshold
                // that has already been made to match the next allocated one, as necessary
                if ( _ulRefSetNum < eTot_TT_RefSet-1 )
                if ( ulThreshold > m_a5uwRefSetThresh   [_eRefSetType]
                                                        [_ulRefSetNum+1]
                                                        [ulTTEtaSign]
                                                        [ulTTEtaMagnBy4]
                                                        [ulTTPhiBy4] )
                {
                    oConsole.ErrorLine ( (CString) "Monotonocity Violation as "
                                                    + CNevisTabChip::f_osTabChip_RefThresh  ( _eRefSetType,
                                                                                              _ulRefSetNum+1)
                                                    + "=" + I_to_A ( m_a5uwRefSetThresh [_eRefSetType]
                                                                                        [_ulRefSetNum+1]
                                                                                        [ulTTEtaSign]
                                                                                        [ulTTEtaMagnBy4]
                                                                                        [ulTTPhiBy4] )
                                                    + " is already .LT. requested Thresh #"
                                                    + I_to_A ( _ulRefSetNum )
                                                    + "=" + I_to_A ( ulThreshold )
                                                    + " for "
                                                    + CNevisTabChip::f_osTabChip_Range  ( ulTTEtaSign,
                                                                                          ulTTEtaMagnBy4,
                                                                                          ulTTPhiBy4 ) ) ;
                    eReturnStatus = Error ;
                    goto Done_With_Monotonicity_Check ;
                }

                for ( ilLowerRefSetNum = long(_ulRefSetNum) - 1 ; ilLowerRefSetNum >= eTT_RefSet_0 ; ilLowerRefSetNum -- )
                {
                    if ( ( m_a2eRefSetalloc [_eRefSetType][ilLowerRefSetNum] == Yes )
                      && ( m_a5uwRefSetThresh   [_eRefSetType]
                                                [ilLowerRefSetNum]
                                                [ulTTEtaSign]
                                                [ulTTEtaMagnBy4]
                                                [ulTTPhiBy4] != mg_auwRefSetInit [_eRefSetType] ) )
                    {
                        if ( ulThreshold < m_a5uwRefSetThresh   [_eRefSetType]
                                                                [ilLowerRefSetNum]
                                                                [ulTTEtaSign]
                                                                [ulTTEtaMagnBy4]
                                                                [ulTTPhiBy4] )
                        {
                            oConsole.ErrorLine ( (CString) "Monotonocity Violation as "
                                                            + CNevisTabChip::f_osTabChip_RefThresh  ( _eRefSetType,
                                                                                                      ilLowerRefSetNum )
                                                            + "=" + I_to_A ( m_a5uwRefSetThresh [_eRefSetType]
                                                                                                [ilLowerRefSetNum]
                                                                                                [ulTTEtaSign]
                                                                                                [ulTTEtaMagnBy4]
                                                                                                [ulTTPhiBy4] )
                                                            + " is already .GT. requested Thresh #"
                                                            + I_to_A ( _ulRefSetNum )
                                                            + "=" + I_to_A ( ulThreshold )
                                                            + " for "
                                                            + CNevisTabChip::f_osTabChip_Range  ( ulTTEtaSign,
                                                                                                  ulTTEtaMagnBy4,
                                                                                                  ulTTPhiBy4 ) ) ;
                            eReturnStatus = Error ;
                            goto Done_With_Monotonicity_Check ;
                        }

                        break ; // We don't look any lower
                    }

                }

            }
        }
    }

Done_With_Monotonicity_Check :

    if ( eReturnStatus == Error )
    {
        oConsole.ErrorLine ( (CString) "Reference Set has not been programmed" ) ;
        goto Done ;
    }


    // note that syntax parsing has already checked for tab chip boundaries
    for ( ulTTPhiBy4     =(m_ulPhiMin-1)/4     ; ulTTPhiBy4     <=(m_ulPhiMax-1)/4     ; ulTTPhiBy4++ )
    {
        ulTTPhi     = 4 * ulTTPhiBy4 + 1 ; // compute the base phi coordinate

        if ( m_apoTab[ulTTPhiBy4] != NULL )         // this card must exist
        if ( _pxTT->alTT_Phi [ulTTPhi] == true )    // check if this sub-range is requested
        for ( ulTTEtaSign    = m_ulSignEtaMin      ; ulTTEtaSign    <= m_ulSignEtaMax      ; ulTTEtaSign++ )
        for ( ulTTEtaMagnBy4 =(m_ulMagnEtaMin-1)/4 ; ulTTEtaMagnBy4 <=(m_ulMagnEtaMax-1)/4 ; ulTTEtaMagnBy4++ )
        {
            ulTTEtaMagn = 4 * ulTTEtaMagnBy4 + 1 ; // compute the base eta coordinate

            if ( _pxTT->aalTT_Eta [ulTTEtaSign] [ulTTEtaMagn] == true ) // check if this sub-range is requested
            {
                //remember this threshold
                m_a5uwRefSetThresh [_eRefSetType]
                                   [_ulRefSetNum]
                                   [ulTTEtaSign]
                                   [ulTTEtaMagnBy4]
                                   [ulTTPhiBy4] = ulThreshold ;

                // program this reference set
                if ( m_apoTab[ulTTPhiBy4]->m_apoMaFpga[ulTTEtaSign][ulTTEtaMagnBy4]
                                -> RefSethreshProg        ( _eRefSetType,
                                                            _ulRefSetNum,
                                                             ulThreshold ) != Ok )
                   eReturnStatus = Error ;

                // Now program any lower reference set(s) left unallocated to the same threshold
                if ( _eRefSetType != eTab_RefSet_TTEt ) // not necessary for TT ref sets
                for ( ilLowerRefSetNum = long(_ulRefSetNum) - 1 ; ilLowerRefSetNum >= eTT_RefSet_0 ; ilLowerRefSetNum -- )
                {
                    if ( ( m_a2eRefSetalloc [_eRefSetType][ilLowerRefSetNum] == Yes )
                      && ( m_a5uwRefSetThresh   [_eRefSetType]
                                                [ilLowerRefSetNum]
                                                [ulTTEtaSign]
                                                [ulTTEtaMagnBy4]
                                                [ulTTPhiBy4] != mg_auwRefSetInit [_eRefSetType] ) )
                        break ; // We don't have to implicitely manage this refset or lower ones

                    if ( m_apoTab[ulTTPhiBy4]->m_apoMaFpga[ulTTEtaSign][ulTTEtaMagnBy4]
                                    -> RefSethreshProg        ( _eRefSetType,
                                                                 ilLowerRefSetNum,
                                                                 ulThreshold ) != Ok )
                       eReturnStatus = Error ;

                }

            }
        }
    }

Done :

    return eReturnStatus ;
}
/*-------------------------------------------------------------------------------*/


/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess        CL1Cal::f_okRefSetDump( ETabRefSet_Type         _eRefSetType,
                                        const unsigned long     _ulRefSetNum,
                                        TAllTriggerTowers*      _pxTT,
                                        const double            _fdThreshold )
{
    ESuccess        eReturnStatus = Ok ;

    CString         osAllocated ;

    unsigned long   ulTTEtaSign ;
    unsigned long   ulTTEtaMagnBy4 ;
    unsigned long   ulTTPhiBy4 ;

    unsigned long   ulTTEtaMagn ;
    unsigned long   ulTTPhi ;

    //\\//

    //Is Reference Set allcoated
    if ( m_a2eRefSetalloc [_eRefSetType][_ulRefSetNum] == Yes )
        osAllocated = "" ;
    else
        osAllocated = "NOT" ;

    oConsole.WriteLine ( (CString) " "
                                + CNevisTabChip::f_osTabChip_RefThresh  ( _eRefSetType,
                                                                          _ulRefSetNum )
                                + " is currently " + osAllocated + " Allocated" ) ;


    // note that syntax parsing has already checked for tab chip boundaries
    for ( ulTTPhiBy4     =(m_ulPhiMin-1)/4     ; ulTTPhiBy4     <=(m_ulPhiMax-1)/4     ; ulTTPhiBy4++ )
    {
        ulTTPhi     = 4 * ulTTPhiBy4 + 1 ; // compute the base phi coordinate

        if ( m_apoTab[ulTTPhiBy4] != NULL )         // this card must exist
        if ( _pxTT->alTT_Phi [ulTTPhi] == true )    // check if this sub-range is requested
        for ( ulTTEtaSign    = m_ulSignEtaMin      ; ulTTEtaSign    <= m_ulSignEtaMax      ; ulTTEtaSign++ )
        for ( ulTTEtaMagnBy4 =(m_ulMagnEtaMin-1)/4 ; ulTTEtaMagnBy4 <=(m_ulMagnEtaMax-1)/4 ; ulTTEtaMagnBy4++ )
        {
            ulTTEtaMagn = 4 * ulTTEtaMagnBy4 + 1 ; // compute the base eta coordinate

            if ( _pxTT->aalTT_Eta [ulTTEtaSign] [ulTTEtaMagn] == true ) // check if this sub-range is requested
            {
                // dump current programming
                oConsole.WriteLine ( (CString) " Now: "
                                              + f_okRefSetDump( _eRefSetType,
                                                                 ulTTEtaSign,
                                                                 ulTTEtaMagnBy4,
                                                                 ulTTPhiBy4 ) ) ;
                oConsole.WriteLine ( (CString) "_for "
                                              + CNevisTabChip::f_osTabChip_Range( ulTTEtaSign,
                                                                                  ulTTEtaMagnBy4,
                                                                                  ulTTPhiBy4 ) ) ;
            }
        }
    }

    return eReturnStatus ;
}
/*-------------------------------------------------------------------------------*/


/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess        CL1Cal::f_okRefSetDeall   ( ETabRefSet_Type         _eRefSetType,
                                            const unsigned long     _ulRefSetNum )
{
    ESuccess        eReturnStatus = Ok ;

    unsigned long   ulTTEtaSign ;
    unsigned long   ulTTEtaMagnBy4 ;
    unsigned long   ulTTPhiBy4 ;

    unsigned long   ulThreshold ;
    long            ilLowerRefSetNum ;

    //\\//

    //Deallocating a Reference Set
    m_a2eRefSetalloc [_eRefSetType][_ulRefSetNum] = No ;

    oConsole.WriteLine ( (CString) "Deallocating "
                                + CNevisTabChip::f_osTabChip_RefThresh  ( _eRefSetType,
                                                                          _ulRefSetNum ) ) ;


    // note that syntax parsing has already checked for tab chip boundaries
    for ( ulTTPhiBy4     =(m_ulPhiMin-1)/4     ; ulTTPhiBy4     <=(m_ulPhiMax-1)/4     ; ulTTPhiBy4++ )
    {
        if ( m_apoTab[ulTTPhiBy4] != NULL )         // this card must exist
        for ( ulTTEtaSign    = m_ulSignEtaMin      ; ulTTEtaSign    <= m_ulSignEtaMax      ; ulTTEtaSign++ )
        for ( ulTTEtaMagnBy4 =(m_ulMagnEtaMin-1)/4 ; ulTTEtaMagnBy4 <=(m_ulMagnEtaMax-1)/4 ; ulTTEtaMagnBy4++ )
        {
            // determine the new threshold to switch to
            if ( _eRefSetType != eTab_RefSet_TTEt ) 
                    ulThreshold = mg_auwRefSetInit [_eRefSetType] ;
            else
            {
                if ( _ulRefSetNum == eTT_RefSet_6 )
                    // the top threshold should take the default value
                    ulThreshold = mg_auwRefSetInit [_eRefSetType] ;
                else
                    // any other should copy the next higher threshold
                    ulThreshold = m_a5uwRefSetThresh   [_eRefSetType]
                                                       [_ulRefSetNum+1]
                                                       [ulTTEtaSign]
                                                       [ulTTEtaMagnBy4]
                                                       [ulTTPhiBy4] ;
            }

            //remember this threshold
            m_a5uwRefSetThresh [_eRefSetType]
                               [_ulRefSetNum]
                               [ulTTEtaSign]
                               [ulTTEtaMagnBy4]
                               [ulTTPhiBy4] = ulThreshold ;

            // program this reference set
            if ( m_apoTab[ulTTPhiBy4]->m_apoMaFpga[ulTTEtaSign][ulTTEtaMagnBy4]
                            -> RefSethreshProg        ( _eRefSetType,
                                                        _ulRefSetNum,
                                                         ulThreshold ) != Ok )
               eReturnStatus = Error ;

            // Now program any lower reference set(s) left unallocated to the same value
            if ( _eRefSetType != eTab_RefSet_TTEt ) // not necessary for TT ref sets
            for ( ilLowerRefSetNum = long(_ulRefSetNum) - 1 ; ilLowerRefSetNum >= eTT_RefSet_0 ; ilLowerRefSetNum -- )
            {
                if ( ( m_a2eRefSetalloc [_eRefSetType][ilLowerRefSetNum] == Yes )
                  && ( m_a5uwRefSetThresh   [_eRefSetType]
                                            [ilLowerRefSetNum]
                                            [ulTTEtaSign]
                                            [ulTTEtaMagnBy4]
                                            [ulTTPhiBy4] != mg_auwRefSetInit [_eRefSetType] ) )
                    break ; // We don't have to implicitely manage this refset or lower ones

                if ( m_apoTab[ulTTPhiBy4]->m_apoMaFpga[ulTTEtaSign][ulTTEtaMagnBy4]
                                -> RefSethreshProg        ( _eRefSetType,
                                                             ilLowerRefSetNum,
                                                             ulThreshold ) != Ok )
                   eReturnStatus = Error ;

            }
        }
    }

    return eReturnStatus ;
}
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
CString         CL1Cal::f_okRefSetDump    ( ETabRefSet_Type         _eRefSetType,
                                            unsigned long           _ulTTEtaSign,
                                            unsigned long           _ulTTEtaMagnBy4,
                                            unsigned long           _ulTTPhiBy4 )
{ // String Dump of RefSet Thresholds e.g. "EM Et RefSet Thresh #0= 10 / #1= 20 / #2= 20 / #3= 20 / #4= 20 / #5= 20 / #6= 20 /"

    TCString  osMsg ;
    unsigned long ulScanRefSetNum ;

    //\\//

    osMsg = CNevisTabChip::f_osTabChip_RefSet ( _eRefSetType )
         + " RefSet " ;

    for ( ulScanRefSetNum = eTT_RefSet_0 ; ulScanRefSetNum < eTot_TT_RefSet ; ulScanRefSetNum ++ )
        osMsg += "#"
              + I_to_A ( ulScanRefSetNum )
              + "=" + I_to_A ( m_a5uwRefSetThresh[_eRefSetType]
                                                 [ulScanRefSetNum]
                                                 [_ulTTEtaSign]
                                                 [_ulTTEtaMagnBy4]
                                                 [_ulTTPhiBy4], 4 )
              + "/" ;

    return osMsg ;
}
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess        CL1Cal::f_okL2Mask    ( ETabMask_Type           _eMaskType,
                                        TAllTriggerTowers*      _pxTT,
                                        const unsigned long     _ulRefSetNum )
{
    ESuccess        eReturnStatus = Ok ;

    unsigned long   ulTTEtaSign ;
    unsigned long   ulTTEtaMagnBy4 ;
    unsigned long   ulTTPhiBy4 ;

    unsigned long   ulTTEtaMagn ;
    unsigned long   ulTTPhi ;

    //\\//

    // note that syntax parsing has already checked for tab chip boundaries
    for ( ulTTPhiBy4     =(m_ulPhiMin-1)/4     ; ulTTPhiBy4     <=(m_ulPhiMax-1)/4     ; ulTTPhiBy4++ )
    {
        ulTTPhi     = 4 * ulTTPhiBy4 + 1 ; // compute the base phi coordinate

        if ( m_apoTab[ulTTPhiBy4] != NULL )         // this card must exist
        if ( _pxTT->alTT_Phi [ulTTPhi] == true )    // check if this sub-range is requested
        for ( ulTTEtaSign    = m_ulSignEtaMin      ; ulTTEtaSign    <= m_ulSignEtaMax      ; ulTTEtaSign++ )
        for ( ulTTEtaMagnBy4 =(m_ulMagnEtaMin-1)/4 ; ulTTEtaMagnBy4 <=(m_ulMagnEtaMax-1)/4 ; ulTTEtaMagnBy4++ )
        {
            ulTTEtaMagn = 4 * ulTTEtaMagnBy4 + 1 ; // compute the base eta coordinate

            if ( _pxTT->aalTT_Eta [ulTTEtaSign] [ulTTEtaMagn] == true ) // check if this sub-range is requested
            {
                // program this reference set
                if ( m_apoTab[ulTTPhiBy4]->m_apoMaFpga[ulTTEtaSign][ulTTEtaMagnBy4]
                                -> L2Mask    ( _eMaskType,
                                               _ulRefSetNum ) != Ok )
                   eReturnStatus = Error ;
             }
        }
    }

    return eReturnStatus ;
}
/*-------------------------------------------------------------------------------*/


/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess        CL1Cal::f_okAlgoUseEmFraction     ( TAllTriggerTowers*      _pxTT,
                                                    bool                    _lUseEmFraction )
{
    ESuccess        eReturnStatus = Ok ;

    unsigned long   ulTTEtaSign ;
    unsigned long   ulTTEtaMagnBy4 ;
    unsigned long   ulTTPhiBy4 ;

    unsigned long   ulTTEtaMagn ;
    unsigned long   ulTTPhi ;

    //\\//

    // note that syntax parsing has already checked for tab chip boundaries
    for ( ulTTPhiBy4     =(m_ulPhiMin-1)/4     ; ulTTPhiBy4     <=(m_ulPhiMax-1)/4     ; ulTTPhiBy4++ )
    {
        ulTTPhi     = 4 * ulTTPhiBy4 + 1 ; // compute the base phi coordinate

        if ( m_apoTab[ulTTPhiBy4] != NULL )         // this card must exist
        if ( _pxTT->alTT_Phi [ulTTPhi] == true )    // check if this sub-range is requested
        for ( ulTTEtaSign    = m_ulSignEtaMin      ; ulTTEtaSign    <= m_ulSignEtaMax      ; ulTTEtaSign++ )
        for ( ulTTEtaMagnBy4 =(m_ulMagnEtaMin-1)/4 ; ulTTEtaMagnBy4 <=(m_ulMagnEtaMax-1)/4 ; ulTTEtaMagnBy4++ )
        {
            ulTTEtaMagn = 4 * ulTTEtaMagnBy4 + 1 ; // compute the base eta coordinate

            if ( _pxTT->aalTT_Eta [ulTTEtaSign] [ulTTEtaMagn] == true ) // check if this sub-range is requested
                if ( m_apoTab[ulTTPhiBy4]->m_apoMaFpga[ulTTEtaSign][ulTTEtaMagnBy4]
                                -> AlgoUseEmFraction( _lUseEmFraction ) != Ok )
                   eReturnStatus = Error ;
        }
    }

    return eReturnStatus ;
}
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess        CL1Cal::f_okAlgoUseEmIsolation    ( TAllTriggerTowers*      _pxTT,
                                                    bool                    _lUseEmIsolation )
{
    ESuccess        eReturnStatus = Ok ;

    unsigned long   ulTTEtaSign ;
    unsigned long   ulTTEtaMagnBy4 ;
    unsigned long   ulTTPhiBy4 ;

    unsigned long   ulTTEtaMagn ;
    unsigned long   ulTTPhi ;

    //\\//

    // note that syntax parsing has already checked for tab chip boundaries
    for ( ulTTPhiBy4     =(m_ulPhiMin-1)/4     ; ulTTPhiBy4     <=(m_ulPhiMax-1)/4     ; ulTTPhiBy4++ )
    {
        ulTTPhi     = 4 * ulTTPhiBy4 + 1 ; // compute the base phi coordinate

        if ( m_apoTab[ulTTPhiBy4] != NULL )         // this card must exist
        if ( _pxTT->alTT_Phi [ulTTPhi] == true )    // check if this sub-range is requested
        for ( ulTTEtaSign    = m_ulSignEtaMin      ; ulTTEtaSign    <= m_ulSignEtaMax      ; ulTTEtaSign++ )
        for ( ulTTEtaMagnBy4 =(m_ulMagnEtaMin-1)/4 ; ulTTEtaMagnBy4 <=(m_ulMagnEtaMax-1)/4 ; ulTTEtaMagnBy4++ )
        {
            ulTTEtaMagn = 4 * ulTTEtaMagnBy4 + 1 ; // compute the base eta coordinate

            if ( _pxTT->aalTT_Eta [ulTTEtaSign] [ulTTEtaMagn] == true ) // check if this sub-range is requested
                if ( m_apoTab[ulTTPhiBy4]->m_apoMaFpga[ulTTEtaSign][ulTTEtaMagnBy4]
                                -> AlgoUseEmIsolation( _lUseEmIsolation ) != Ok )
                   eReturnStatus = Error ;
        }
    }

    return eReturnStatus ;
}
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess        CL1Cal::f_okAlgoEmIsolationEtCut  ( TAllTriggerTowers*      _pxTT,
                                                    const double            _fdThreshold )
{
    ESuccess        eReturnStatus = Ok ;

    unsigned long   ulTTEtaSign ;
    unsigned long   ulTTEtaMagnBy4 ;
    unsigned long   ulTTPhiBy4 ;

    unsigned long   ulTTEtaMagn ;
    unsigned long   ulTTPhi ;

    unsigned long   ulThreshold ;

    //\\//

    //Convert Floating Point GeV to Integer ADC Counts
    ulThreshold = (unsigned long) ( _fdThreshold / KfL1CalEtScale + 0.5 ) ;

    // take care of saturation
    if ( ulThreshold > kiTabEmEtTreshMax )    ulThreshold = kiTabEmEtTreshMax ;

    // note that syntax parsing has already checked for tab chip boundaries
    for ( ulTTPhiBy4     =(m_ulPhiMin-1)/4     ; ulTTPhiBy4     <=(m_ulPhiMax-1)/4     ; ulTTPhiBy4++ )
    {
        ulTTPhi     = 4 * ulTTPhiBy4 + 1 ; // compute the base phi coordinate

        if ( m_apoTab[ulTTPhiBy4] != NULL )         // this card must exist
        if ( _pxTT->alTT_Phi [ulTTPhi] == true )    // check if this sub-range is requested
        for ( ulTTEtaSign    = m_ulSignEtaMin      ; ulTTEtaSign    <= m_ulSignEtaMax      ; ulTTEtaSign++ )
        for ( ulTTEtaMagnBy4 =(m_ulMagnEtaMin-1)/4 ; ulTTEtaMagnBy4 <=(m_ulMagnEtaMax-1)/4 ; ulTTEtaMagnBy4++ )
        {
            ulTTEtaMagn = 4 * ulTTEtaMagnBy4 + 1 ; // compute the base eta coordinate

            if ( _pxTT->aalTT_Eta [ulTTEtaSign] [ulTTEtaMagn] == true ) // check if this sub-range is requested
                if ( m_apoTab[ulTTPhiBy4]->m_apoMaFpga[ulTTEtaSign][ulTTEtaMagnBy4]
                                -> AlgoEmIsolationEtCut( ulThreshold ) != Ok )
                   eReturnStatus = Error ;
        }
    }

    return eReturnStatus ;
}
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess        CL1Cal::f_okAlgoTauEtCoreCut      ( TAllTriggerTowers*      _pxTT,
                                                    const double            _fdThreshold )
{
    ESuccess        eReturnStatus = Ok ;

    unsigned long   ulTTEtaSign ;
    unsigned long   ulTTEtaMagnBy4 ;
    unsigned long   ulTTPhiBy4 ;

    unsigned long   ulTTEtaMagn ;
    unsigned long   ulTTPhi ;

    unsigned long   ulThreshold ;

    //\\//

    //Convert Floating Point GeV to Integer ADC Counts
    ulThreshold = (unsigned long) ( _fdThreshold / KfL1CalEtScale + 0.5 ) ;

    // take care of saturation
    if ( ulThreshold > kiTabTauEtCoreCutMax )    ulThreshold = kiTabTauEtCoreCutMax ;

    // note that syntax parsing has already checked for tab chip boundaries
    for ( ulTTPhiBy4     =(m_ulPhiMin-1)/4     ; ulTTPhiBy4     <=(m_ulPhiMax-1)/4     ; ulTTPhiBy4++ )
    {
        ulTTPhi     = 4 * ulTTPhiBy4 + 1 ; // compute the base phi coordinate

        if ( m_apoTab[ulTTPhiBy4] != NULL )         // this card must exist
        if ( _pxTT->alTT_Phi [ulTTPhi] == true )    // check if this sub-range is requested
        for ( ulTTEtaSign    = m_ulSignEtaMin      ; ulTTEtaSign    <= m_ulSignEtaMax      ; ulTTEtaSign++ )
        for ( ulTTEtaMagnBy4 =(m_ulMagnEtaMin-1)/4 ; ulTTEtaMagnBy4 <=(m_ulMagnEtaMax-1)/4 ; ulTTEtaMagnBy4++ )
        {
            ulTTEtaMagn = 4 * ulTTEtaMagnBy4 + 1 ; // compute the base eta coordinate

            if ( _pxTT->aalTT_Eta [ulTTEtaSign] [ulTTEtaMagn] == true ) // check if this sub-range is requested
                if ( m_apoTab[ulTTPhiBy4]->m_apoMaFpga[ulTTEtaSign][ulTTEtaMagnBy4]
                                -> AlgoTauEtCoreCut( ulThreshold ) != Ok )
                   eReturnStatus = Error ;
        }
    }

    return eReturnStatus ;
}
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess        CL1Cal::f_okAlgoGlobalSumEtCut    ( TAllTriggerTowers*      _pxTT,
                                                    const double            _fdThreshold )
{
    ESuccess        eReturnStatus = Ok ;

    unsigned long   ulTTEtaSign ;
    unsigned long   ulTTEtaMagnBy4 ;
    unsigned long   ulTTPhiBy4 ;

    unsigned long   ulTTEtaMagn ;
    unsigned long   ulTTPhi ;

    unsigned long   ulThreshold ;

    //\\//

    //Convert Floating Point GeV to Integer ADC Counts
    ulThreshold = (unsigned long) ( _fdThreshold / KfL1CalEtScale + 0.5 ) ;

    // take care of saturation
    if ( ulThreshold > kiTabGlobSumEtCutMax )    ulThreshold = kiTabGlobSumEtCutMax ;

    // note that syntax parsing has already checked for tab chip boundaries
    for ( ulTTPhiBy4     =(m_ulPhiMin-1)/4     ; ulTTPhiBy4     <=(m_ulPhiMax-1)/4     ; ulTTPhiBy4++ )
    {
        ulTTPhi     = 4 * ulTTPhiBy4 + 1 ; // compute the base phi coordinate

        if ( m_apoTab[ulTTPhiBy4] != NULL )         // this card must exist
        if ( _pxTT->alTT_Phi [ulTTPhi] == true )    // check if this sub-range is requested
        for ( ulTTEtaSign    = m_ulSignEtaMin      ; ulTTEtaSign    <= m_ulSignEtaMax      ; ulTTEtaSign++ )
        for ( ulTTEtaMagnBy4 =(m_ulMagnEtaMin-1)/4 ; ulTTEtaMagnBy4 <=(m_ulMagnEtaMax-1)/4 ; ulTTEtaMagnBy4++ )
        {
            ulTTEtaMagn = 4 * ulTTEtaMagnBy4 + 1 ; // compute the base eta coordinate

            if ( _pxTT->aalTT_Eta [ulTTEtaSign] [ulTTEtaMagn] == true ) // check if this sub-range is requested
                if ( m_apoTab[ulTTPhiBy4]->m_apoMaFpga[ulTTEtaSign][ulTTEtaMagnBy4]
                                -> AlgoGlobalSumEtCut( ulThreshold ) != Ok )
                   eReturnStatus = Error ;
        }
    }

    return eReturnStatus ;
}
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess        CL1Cal::f_okAlgoUseIcrJetObjects  ( TAllTriggerTowers*      _pxTT,
                                                    bool                    _lUseIcr )
{
    ESuccess        eReturnStatus = Ok ;

    unsigned long   ulTTEtaSign ;
    unsigned long   ulTTEtaMagnBy4 ;
    unsigned long   ulTTPhiBy4 ;

    unsigned long   ulTTEtaMagn ;
    unsigned long   ulTTPhi ;

    //\\//

    // note that syntax parsing has already checked for tab chip boundaries
    for ( ulTTPhiBy4     =(m_ulPhiMin-1)/4     ; ulTTPhiBy4     <=(m_ulPhiMax-1)/4     ; ulTTPhiBy4++ )
    {
        ulTTPhi     = 4 * ulTTPhiBy4 + 1 ; // compute the base phi coordinate

        if ( m_apoTab[ulTTPhiBy4] != NULL )         // this card must exist
        if ( _pxTT->alTT_Phi [ulTTPhi] == true )    // check if this sub-range is requested
        for ( ulTTEtaSign    = m_ulSignEtaMin      ; ulTTEtaSign    <= m_ulSignEtaMax      ; ulTTEtaSign++ )
        for ( ulTTEtaMagnBy4 =(m_ulMagnEtaMin-1)/4 ; ulTTEtaMagnBy4 <=(m_ulMagnEtaMax-1)/4 ; ulTTEtaMagnBy4++ )
        {
            ulTTEtaMagn = 4 * ulTTEtaMagnBy4 + 1 ; // compute the base eta coordinate

            if ( _pxTT->aalTT_Eta [ulTTEtaSign] [ulTTEtaMagn] == true ) // check if this sub-range is requested
                if ( m_apoTab[ulTTPhiBy4]->m_apoMaFpga[ulTTEtaSign][ulTTEtaMagnBy4]
                                -> AlgoUseIcrJetObjects( _lUseIcr ) != Ok )
                   eReturnStatus = Error ;
        }
    }

    return eReturnStatus ;
}
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess        CL1Cal::f_okAlgoUseIcrGlobalSums  ( TAllTriggerTowers*      _pxTT,
                                                    bool                    _lUseIcr )
{
    ESuccess        eReturnStatus = Ok ;

    unsigned long   ulTTEtaSign ;
    unsigned long   ulTTEtaMagnBy4 ;
    unsigned long   ulTTPhiBy4 ;

    unsigned long   ulTTEtaMagn ;
    unsigned long   ulTTPhi ;

    //\\//

    // note that syntax parsing has already checked for tab chip boundaries
    for ( ulTTPhiBy4     =(m_ulPhiMin-1)/4     ; ulTTPhiBy4     <=(m_ulPhiMax-1)/4     ; ulTTPhiBy4++ )
    {
        ulTTPhi     = 4 * ulTTPhiBy4 + 1 ; // compute the base phi coordinate

        if ( m_apoTab[ulTTPhiBy4] != NULL )         // this card must exist
        if ( _pxTT->alTT_Phi [ulTTPhi] == true )    // check if this sub-range is requested
        for ( ulTTEtaSign    = m_ulSignEtaMin      ; ulTTEtaSign    <= m_ulSignEtaMax      ; ulTTEtaSign++ )
        for ( ulTTEtaMagnBy4 =(m_ulMagnEtaMin-1)/4 ; ulTTEtaMagnBy4 <=(m_ulMagnEtaMax-1)/4 ; ulTTEtaMagnBy4++ )
        {
            ulTTEtaMagn = 4 * ulTTEtaMagnBy4 + 1 ; // compute the base eta coordinate

            if ( _pxTT->aalTT_Eta [ulTTEtaSign] [ulTTEtaMagn] == true ) // check if this sub-range is requested
                if ( m_apoTab[ulTTPhiBy4]->m_apoMaFpga[ulTTEtaSign][ulTTEtaMagnBy4]
                                -> AlgoUseIcrGlobalSums( _lUseIcr ) != Ok )
                   eReturnStatus = Error ;
        }
    }

    return eReturnStatus ;
}
/*-------------------------------------------------------------------------------*/


/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess        CL1Cal::f_okTermEmAll         ( unsigned int            _uiComparatorNum,
                                                unsigned int            _uiRefSetNum,
                                                unsigned int            _uiThreshold )
{
    return ReportTabGabStatus (  Gab_Set_EmAllTerm        ( _uiComparatorNum,
                                                            _uiRefSetNum,
                                                            _uiThreshold ),
                                "Gab_Set_EmAllTerm",
                                "CL1Cal::f_okTermEmAll" ) ;

}
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess        CL1Cal::f_okTermEmCentral     ( unsigned int            _uiComparatorNum,
                                                unsigned int            _uiRefSetNum,
                                                unsigned int            _uiThreshold )
{
    return ReportTabGabStatus (  Gab_Set_EmCentralTerm    ( _uiComparatorNum,
                                                            _uiRefSetNum,
                                                            _uiThreshold ),
                                "Gab_Set_EmCentralTerm",
                                "CL1Cal::f_okTermEmCentral" ) ;
}
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess        CL1Cal::f_okTermJetAll        ( unsigned int            _uiComparatorNum,
                                                unsigned int            _uiRefSetNum,
                                                unsigned int            _uiThreshold )
{
    return ReportTabGabStatus (  Gab_Set_JetAllTerm       ( _uiComparatorNum,
                                                            _uiRefSetNum,
                                                            _uiThreshold ),
                                "Gab_Set_JetAllTerm",
                                "CL1Cal::f_okTermJetAll" ) ;
}
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess        CL1Cal::f_okTermJetCentral    ( unsigned int            _uiComparatorNum,
                                                unsigned int            _uiRefSetNum,
                                                unsigned int            _uiThreshold )
{
    return ReportTabGabStatus (  Gab_Set_JetCentralTerm   ( _uiComparatorNum,
                                                            _uiRefSetNum,
                                                            _uiThreshold ),
                                "Gab_Set_JetCentralTerm",
                                "CL1Cal::f_okTermJetCentral" ) ;
}
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess        CL1Cal::f_okTermTauAll        ( unsigned int            _uiComparatorNum,
                                                unsigned int            _uiRefSetNum,
                                                unsigned int            _uiTauRatioNum, 
                                                unsigned int            _uiThreshold )
{
    return ReportTabGabStatus (  Gab_Set_TauAllTerm       ( _uiComparatorNum,
                                                            _uiRefSetNum,
                                                            _uiTauRatioNum,
                                                            _uiThreshold ),
                                "Gab_Set_TauAllTerm",
                                "CL1Cal::f_okTermTauAll" ) ;
}
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess        CL1Cal::f_okTermTauCentral    ( unsigned int            _uiComparatorNum,
                                                unsigned int            _uiRefSetNum,
                                                unsigned int            _uiTauRatioNum, 
                                                unsigned int            _uiThreshold )
{
    return ReportTabGabStatus (  Gab_Set_TauCentralTerm   ( _uiComparatorNum,
                                                            _uiRefSetNum,
                                                            _uiTauRatioNum,
                                                            _uiThreshold ),
                                "Gab_Set_TauCentralTerm",
                                "CL1Cal::f_okTermTauCentral" ) ;
}
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess        CL1Cal::f_okTermIsolEmAll     ( unsigned int            _uiComparatorNum,
                                                unsigned int            _uiRefSetNum,
                                                unsigned int            _uiThreshold )
{
    return ReportTabGabStatus (  Gab_Set_IsolEmAllTerm    ( _uiComparatorNum,
                                                            _uiRefSetNum,
                                                            _uiThreshold ),
                                "Gab_Set_IsolEmAllTerm",
                                "CL1Cal::f_okTermIsolEmAll" ) ;
}
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess        CL1Cal::f_okTermIsolEmCentral ( unsigned int            _uiComparatorNum,
                                                unsigned int            _uiRefSetNum,
                                                unsigned int            _uiThreshold )
{
    return ReportTabGabStatus (  Gab_Set_IsolEmCentralTerm( _uiComparatorNum,
                                                            _uiRefSetNum,
                                                            _uiThreshold ),
                                "Gab_Set_IsolEmCentralTerm",
                                "CL1Cal::f_okTermIsolEmCentral" ) ;
}
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess        CL1Cal::f_okTermMissEt        ( unsigned int            _uiComparatorNum,
                                                const double            _fdThreshold )
{
    unsigned long   ulAdcSquareThresh ;

    //\\//

    ulAdcSquareThresh = (unsigned long) ( _fdThreshold / KfL1CalEtScale + 0.5 ) ; 

    ulAdcSquareThresh = ulAdcSquareThresh  * ulAdcSquareThresh  ;

    return ReportTabGabStatus (  Gab_Set_MissEtTerm       ( _uiComparatorNum,
                                                             ulAdcSquareThresh ),
                                "Gab_Set_MissEtTerm",
                                "CL1Cal::f_okTermMissEt" ) ;
}
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess        CL1Cal::f_okTermTotEt         ( unsigned int            _uiComparatorNum,
                                                const double            _fdThreshold )
{
    unsigned long   ulAdcThresh ;

    //\\//

    ulAdcThresh = (unsigned long) ( _fdThreshold / KfL1CalEtScale + 0.5 ) ;

    return ReportTabGabStatus (  Gab_Set_TotEtTerm        ( _uiComparatorNum,
                                                             ulAdcThresh ),
                                "Gab_Set_TotEtTerm",
                                "CL1Cal::f_okTermTotEt" ) ;
}
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess        CL1Cal::f_okTermAcoplanarJet          ( unsigned int            _uiComparatorNum,
                                                        unsigned int            _uiRefSetNum1,
                                                        unsigned int            _uiRefSetNum2,
                                                        unsigned int            _uiDelta_Phi )
{
    return ReportTabGabStatus (  Gab_Set_AcoplanarJetTerm ( _uiComparatorNum,
                                                            _uiRefSetNum1,
                                                            _uiRefSetNum2,
                                                            _uiDelta_Phi ),
                                "Gab_Set_AcoplanarJetTerm",
                                "CL1Cal::f_okTermAcoplanarJet" ) ;
}
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess        CL1Cal::f_okTermNonCollinearLowEtJet  ( unsigned int            _uiComparatorNum,
                                                        unsigned int            _uiRefSetNum1,
                                                        unsigned int            _uiRefSetNum2,
                                                        unsigned int            _uiDelta_Phi )
{
    return ReportTabGabStatus (  Gab_Set_NonColLowEtJetTerm   ( _uiComparatorNum,
                                                                _uiRefSetNum1,
                                                                _uiRefSetNum2,
                                                                _uiDelta_Phi ),
                                "Gab_Set_NonColLowEtJetTerm",
                                "CL1Cal::f_okTermNonCollinearLowEtJet" ) ;
}
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess        CL1Cal::f_okTermNonCollinearMonoJet   ( unsigned int            _uiComparatorNum,
                                                        unsigned int            _uiRefSetNum1,
                                                        unsigned int            _uiRefSetNum2,
                                                        unsigned int            _uiDelta_Phi )
{
    return ReportTabGabStatus (  Gab_Set_NonColMonoJetTerm( _uiComparatorNum,
                                                            _uiRefSetNum1,
                                                            _uiRefSetNum2,
                                                            _uiDelta_Phi ),
                                "Gab_Set_NonColMonoJetTerm",
                                "CL1Cal::f_okTermNonCollinearMonoJet" ) ;
}
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess        CL1Cal::f_okTermJetFree       ( unsigned int            _uiComparatorNum,
                                                unsigned int            _uiRefSetNum1,
                                                unsigned int            _uiRefSetNum2,
                                                unsigned int            _uiDelta_Phi )
{
    return ReportTabGabStatus (  Gab_Set_JetFreeTerm      ( _uiComparatorNum,
                                                            _uiRefSetNum1,
                                                            _uiRefSetNum2,
                                                            _uiDelta_Phi ),
                                "Gab_Set_JetFreeTerm",
                                "CL1Cal::f_okTermJetFree" ) ;
}
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess        CL1Cal::f_okTermBackToBackEm  ( unsigned int            _uiComparatorNum,
                                                unsigned int            _uiRefSetNum,
                                                unsigned int            _uiDelta_Phi )
{
    return ReportTabGabStatus (  Gab_Set_BackToBackEmTerm ( _uiComparatorNum,
                                                            _uiRefSetNum,
                                                            _uiDelta_Phi ),
                                "Gab_Set_BackToBackEmTerm",
                                "CL1Cal::f_okTermBackToBackEm" ) ;
}
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess        CL1Cal::f_okRawAdc_MuxMode    ( ETT_Chan_Type           _eTTType,
                                                TAllTriggerTowers*      _pxTT,
                                                ERawAdc_MuxMode_Type    _eRawAdc_MuxMode )
{
    ESuccess        eReturnStatus = Ok ;

    unsigned long   ulTTEtaSign ;
    unsigned long   ulTTEtaMagn ;
    unsigned long   ulTTPhi ;

    //\\//

    for ( ulTTEtaSign = eTT_Eta_Pos       ; ulTTEtaSign <= eTT_Eta_Neg       ; ulTTEtaSign++ )
    for ( ulTTEtaMagn = KiMin_TT_Eta_Magn ; ulTTEtaMagn <= KiMax_TT_Eta_Magn ; ulTTEtaMagn++ )
    if ( _pxTT->aalTT_Eta [ulTTEtaSign] [ulTTEtaMagn] == true )
    for ( ulTTPhi     = KiMin_TT_Phi      ; ulTTPhi     <= KiMax_TT_Phi      ; ulTTPhi++ )
    if ( _pxTT->alTT_Phi [ulTTPhi] == true )
    {
        if ( m_a3poTrgTwr[ulTTEtaSign][ulTTEtaMagn][ulTTPhi] != NULL )
        if ( m_a3poTrgTwr[ulTTEtaSign][ulTTEtaMagn][ulTTPhi]
                          -> f_okRawAdc_MuxMode    ( _eTTType,
                                                     _eRawAdc_MuxMode ) != Ok )
           eReturnStatus = Error ;
    }

    return eReturnStatus ;
}
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess        CL1Cal::f_okRawAdc_MemMode    ( ETT_Chan_Type           _eTTType,
                                                TAllTriggerTowers*      _pxTT,
                                                ERawAdc_MemMode_Type    _eRawAdc_MemMode )
{
    ESuccess        eReturnStatus = Ok ;

    unsigned long   ulTTEtaSign ;
    unsigned long   ulTTEtaMagn ;
    unsigned long   ulTTPhi ;

    //\\//

    for ( ulTTEtaSign = eTT_Eta_Pos       ; ulTTEtaSign <= eTT_Eta_Neg       ; ulTTEtaSign++ )
    for ( ulTTEtaMagn = KiMin_TT_Eta_Magn ; ulTTEtaMagn <= KiMax_TT_Eta_Magn ; ulTTEtaMagn++ )
    if ( _pxTT->aalTT_Eta [ulTTEtaSign] [ulTTEtaMagn] == true )
    for ( ulTTPhi     = KiMin_TT_Phi      ; ulTTPhi     <= KiMax_TT_Phi      ; ulTTPhi++ )
    if ( _pxTT->alTT_Phi [ulTTPhi] == true )
    {
        if ( m_a3poTrgTwr[ulTTEtaSign][ulTTEtaMagn][ulTTPhi] != NULL )
        if ( m_a3poTrgTwr[ulTTEtaSign][ulTTEtaMagn][ulTTPhi]
                          -> f_okRawAdc_MemMode    ( _eTTType,
                                                     _eRawAdc_MemMode ) != Ok )
           eReturnStatus = Error ;
    }

    return eReturnStatus ;
}
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess        CL1Cal::f_okAdfOut_MuxMode    ( ETT_Chan_Type           _eTTType,
                                                TAllTriggerTowers*      _pxTT,
                                                EAdfOut_MuxMode_Type    _eAdfOut_MuxMode )
{
    ESuccess        eReturnStatus = Ok ;

    unsigned long   ulTTEtaSign ;
    unsigned long   ulTTEtaMagn ;
    unsigned long   ulTTPhi ;

    //\\//

    for ( ulTTEtaSign = eTT_Eta_Pos       ; ulTTEtaSign <= eTT_Eta_Neg       ; ulTTEtaSign++ )
    for ( ulTTEtaMagn = KiMin_TT_Eta_Magn ; ulTTEtaMagn <= KiMax_TT_Eta_Magn ; ulTTEtaMagn++ )
    if ( _pxTT->aalTT_Eta [ulTTEtaSign] [ulTTEtaMagn] == true )
    for ( ulTTPhi     = KiMin_TT_Phi      ; ulTTPhi     <= KiMax_TT_Phi      ; ulTTPhi++ )
    if ( _pxTT->alTT_Phi [ulTTPhi] == true )
    {
        if ( m_a3poTrgTwr[ulTTEtaSign][ulTTEtaMagn][ulTTPhi] != NULL )
        if ( m_a3poTrgTwr[ulTTEtaSign][ulTTEtaMagn][ulTTPhi]
                          -> f_okAdfOut_MuxMode   ( _eTTType,
                                                    _eAdfOut_MuxMode ) != Ok )
           eReturnStatus = Error ;
    }

    return eReturnStatus ;
}
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess        CL1Cal::f_okAdfOut_MemMode    ( ETT_Chan_Type           _eTTType,
                                                TAllTriggerTowers*      _pxTT,
                                                EAdfOut_MemMode_Type     _eAdfOut_MemMode )
{
    ESuccess        eReturnStatus = Ok ;

    unsigned long   ulTTEtaSign ;
    unsigned long   ulTTEtaMagn ;
    unsigned long   ulTTPhi ;

    //\\//

    for ( ulTTEtaSign = eTT_Eta_Pos       ; ulTTEtaSign <= eTT_Eta_Neg       ; ulTTEtaSign++ )
    for ( ulTTEtaMagn = KiMin_TT_Eta_Magn ; ulTTEtaMagn <= KiMax_TT_Eta_Magn ; ulTTEtaMagn++ )
    if ( _pxTT->aalTT_Eta [ulTTEtaSign] [ulTTEtaMagn] == true )
    for ( ulTTPhi     = KiMin_TT_Phi      ; ulTTPhi     <= KiMax_TT_Phi      ; ulTTPhi++ )
    if ( _pxTT->alTT_Phi [ulTTPhi] == true )
    {
        if ( m_a3poTrgTwr[ulTTEtaSign][ulTTEtaMagn][ulTTPhi] != NULL )
        if ( m_a3poTrgTwr[ulTTEtaSign][ulTTEtaMagn][ulTTPhi]
                          -> f_okAdfOut_MemMode     ( _eTTType,
                                                      _eAdfOut_MemMode ) != Ok )
           eReturnStatus = Error ;
    }

    return eReturnStatus ;
}
/*-------------------------------------------------------------------------------*/

