/*-------------------------------------------------------------------------------*/
  /*
    L1CalCardAddresses.cpp
    13-Sep-2005
  */
/*-------------------------------------------------------------------------------*/
#include "L1Cal_IIb_TCC/StdAfx.h"

#include "L1Cal_IIb_TCC/GlobalConstants.h"
#include "L1Cal_IIb_TCC/CustomDataTypes.h"

#include                            "L1Cal_IIb_TCC/L1CalCardAddresses.h"

#include "L1Cal_IIb_TCC/HandleConsole.h" 
#include "L1Cal_IIb_TCC/UtilsStrings.h"  
#include "L1Cal_IIb_TCC/UtilsBits.h"  

/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
unsigned char L1CalAdfCardAddress_Master    ( unsigned long     _ulSign_Eta,
                                              unsigned long     _ulMagn_Eta_By4,
                                              unsigned long     _ulPhi_By4 )
{
    
    return Ki_Adf_Crate_Master ; 
}
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
unsigned char L1CalAdfCardAddress_Slave     ( unsigned long     _ulSign_Eta,
                                              unsigned long     _ulMagn_Eta_By4,
                                              unsigned long     _ulPhi_By4 )
{
    unsigned long  ulSlave_Number ;

    //\\//

    // Use math to build one dimensional sequence number which is easy 
    // to break into crates and slots

    ulSlave_Number = (  16 * _ulMagn_Eta_By4
                      + 8 * _ulSign_Eta
                      +     _ulPhi_By4 ) / 20 ;

    if ( ulSlave_Number > 3 ) 
    {
        CConsoleAccess::InternalErrorLine ( "L1CalAdfCardAddress_Slave: Unexpected arguments" ) ;
        ulSlave_Number = 0 ; 
    }


    return ulSlave_Number ; 
}
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
unsigned char L1CalAdfCardAddress_Slot      ( unsigned long     _ulSign_Eta,
                                              unsigned long     _ulMagn_Eta_By4,
                                              unsigned long     _ulPhi_By4 )
{
    unsigned long  ulSlot_Number ;

    //\\//

    // Use math to build one dimensional sequence number which is easy 
    // to break into crates and slots

    ulSlot_Number = (  16 * _ulMagn_Eta_By4
                      + 8 * _ulSign_Eta
                      +     _ulPhi_By4 ) % 20 + 2 ;

    if ( ( ulSlot_Number < 2 ) 
      || ( ulSlot_Number > 21 ) )
    {
        CConsoleAccess::InternalErrorLine ( "L1CalAdfCardAddress_Slot: Unexpected arguments" ) ;
        ulSlot_Number = 0 ; 
    }


    return ulSlot_Number ; 
}
/*-------------------------------------------------------------------------------*/


/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
CString L1CalAdfCardAddress_Desc        ( unsigned long     _ulSign_Eta,
                                          unsigned long     _ulMagn_Eta_By4,
                                          unsigned long     _ulPhi_By4 )
{
    char cSign_Eta ;

    switch ( _ulSign_Eta )
    {
        case eTT_Eta_Pos : cSign_Eta = '+' ; break ;
        case eTT_Eta_Neg : cSign_Eta = '-' ; break ;
    } ;

    return (CString) "ADF for eta(" 
                   + cSign_Eta
                   + I_to_A ( 4 * _ulMagn_Eta_By4 + 1 ) 
                   + ":"
                   + cSign_Eta
                   + I_to_A ( 4 * _ulMagn_Eta_By4 + 4 ) 
                   + "),phi("
                   + I_to_A ( 4 * _ulPhi_By4 + 1 ) 
                   + ":"
                   + I_to_A ( 4 * _ulPhi_By4 + 4 ) 
                   + ")" ;
}
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/


/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
unsigned char L1CalTabCardAddress_Module    ( unsigned long     _ulPhi_By4 )
{
    unsigned long  ulModule_Number ;

    //\\//

    // Use math to build one dimensional sequence number which is easy 
    // to break into crates and slots

    ulModule_Number = _ulPhi_By4 ;

    if ( ulModule_Number > 7 )
    {
        CConsoleAccess::InternalErrorLine ( "L1CalAdfCardAddress_Module: Unexpected arguments" ) ;
        ulModule_Number = 0 ; 
    }

    return ulModule_Number ; 
}
/*-------------------------------------------------------------------------------*/


/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
CString L1CalTabCardAddress_Desc        ( unsigned long     _ulPhi_By4 )
{
    return (CString) "TAB for phi("
                   + I_to_A ( 4 * _ulPhi_By4 + 1 ) 
                   + ":"
                   + I_to_A ( 4 * _ulPhi_By4 + 4 ) 
                   + ")" ;
}
/*-------------------------------------------------------------------------------*/




/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
unsigned char L1CalGabCardAddress_Module    ( )
{
    return 8 ; 
}
/*-------------------------------------------------------------------------------*/


/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
CString L1CalGabCardAddress_Desc        ( )
{
    return (CString) "GAB" ;
}
/*-------------------------------------------------------------------------------*/


/*-------------------------------------------------------------------------------*/
// Common to TAB and GAB
//
//    VRB         Link
//    SLOT        1      2      3     4
//    9           TAB0   TAB1   TAB2  SPARE
//    10          TAB3   GAB    TAB4  SPARE
//    11          TAB5   TAB6   TAB7  SPARE

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
unsigned char L1CalTabVrbSlot         ( unsigned char       _ulModule_Num ) 
{
    unsigned char ubVrbSlotNum ;

    //\\//
    
    switch ( _ulModule_Num ) 
    {
        case 0 : // TAB0
        case 1 : // TAB1
        case 2 : // TAB2
           ubVrbSlotNum = 9 ;
        break ;

        case 3 : // TAB3
        case 4 : // TAB4
           ubVrbSlotNum = 10 ;
        break ;

        case 5 : // TAB5
        case 6 : // TAB6
        case 7 : // TAB7
           ubVrbSlotNum = 11 ;         
        break ;

        case 8 : // GAB
           ubVrbSlotNum = 10 ;         
        break ;

        default :
            CConsoleAccess::InternalErrorLine ( "L1CalTabVrbSlot: Unexpected arguments" ) ;
            ubVrbSlotNum = 0 ; 
        break ;
    }

    return ubVrbSlotNum ;
}
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
unsigned char L1CalTabVrbChannelPair  ( unsigned char       _ulModule_Num ) 
{
    unsigned char ubVrbChannelPair ;

    //\\//
    
    switch ( _ulModule_Num ) 
    {
        case 0 : // TAB0
        case 3 : // TAB3
        case 5 : // TAB5
           ubVrbChannelPair = 0 ;
        break ;

        case 1 : // TAB1
        case 8 : // GAB
        case 6 : // TAB6
           ubVrbChannelPair = 1 ;
        break ;

        case 2 : // TAB2
        case 4 : // TAB4
        case 7 : // TAB7
           ubVrbChannelPair = 2 ;         
        break ;

        default :
            CConsoleAccess::InternalErrorLine ( "L1CalTabVrbChannelPair: Unexpected arguments" ) ;
            ubVrbChannelPair = 0 ; 
        break ;
    }

    return ubVrbChannelPair ;
}
/*-------------------------------------------------------------------------------*/


