/*-------------------------------------------------------------------------------*/
  /*
    L1Cal_TrgTwr.cpp
    Class CL1Cal_TrgTwr
  */
/*-------------------------------------------------------------------------------*/

#include "L1Cal_IIb_TCC/StdAfx.h"

#include                            "L1Cal_IIb_TCC/L1Cal_TrgTwr.h"

#include "L1Cal_IIb_TCC/L1Cal.h"

#include "L1Cal_IIb_TCC/HandleResultFile.h"

#include "L1Cal_IIb_TCC/MonitPoolData.h" // for kL1CalTcc_Tot_LiveBX and CMonitPoolData::mg_aIndexToTick

#include "L1Cal_IIb_TCC/UtilsStrings.h"
//#include "L1Cal_IIb_TCC/UtilsBits.h"
//#include "L1Cal_IIb_TCC/UtilsRandom.h"
//#include "L1Cal_IIb_TCC/UtilsFiles.h"

//#include "L1Cal_IIb_TCC/DataManip.h"

#include <math.h> //for fabs

// static means created only once throughout life of program (to avoid wasting time)
// *AND* internal linkage (to avoid clash between modules)
#include "L1Cal_IIb_TCC/HandleConsole.h"
static CConsoleAccess oConsole ;

#include "L1Cal_IIb_TCC/HandleLogFile.h"

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
//String array for screen messages
//usefull to decode both ETT_Chan_Type and EEnergy_Tree_Type
    char sQuantType [eTot_TT_Chan_Type] [2+1] = { 'E', 'M', 0,
                                                  'H', 'D', 0 } ;

// huge histogram (>1 MB)  allocate only once
    Ta3uwSTicksHisto   ga3uwHistogram ;

// smaller histogram (18kB) but allocate only once as well, just for uniformity
    Ta2uwLiveXHisto    ga2uwHistogram ;

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
CL1Cal_TrgTwr::CL1Cal_TrgTwr  ( unsigned long       _ulTTEtaSign,
                                unsigned long       _ulTTEtaMagn,
                                unsigned long       _ulTTPhi,
                                CL1Cal_TrgTwr**     _ppoPrev_L1Cal_TrgTwr,
                                TuwRegWord          _uwRawAdcZer,
                                TuwRegWord          _uwOutputEtZer,
                                CAdfCardBase*       _poMotherAdf )
{
    unsigned long   ulTTEtaMagnBy4 ;

    //\\//

    BasicConstruct  ( _ulTTEtaSign,
                      _ulTTEtaMagn,
                      _ulTTPhi,
                      _ppoPrev_L1Cal_TrgTwr,
                      _uwRawAdcZer,
                      _uwOutputEtZer ) ;

    ulTTEtaMagnBy4 = (_ulTTEtaMagn-1) / 4 ;

    m_poMotherAdf = _poMotherAdf ;

}


/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
CL1Cal_TrgTwr::CL1Cal_TrgTwr  ( unsigned long       _ulTTEtaSign,
                                unsigned long       _ulTTEtaMagn,
                                unsigned long       _ulTTPhi,
                                CL1Cal*             _poL1Cal )
{
    unsigned long   ulTTEtaMagnBy4 ;
    unsigned long   ulTTPhiBy4 ;

    //\\//

    BasicConstruct  ( _ulTTEtaSign,
                      _ulTTEtaMagn,
                      _ulTTPhi,
                     &_poL1Cal->m_poLast_TrgTwr,
                      KiRawAdcZer,
                      KiOutputEtZer ) ;

    ulTTEtaMagnBy4 = (_ulTTEtaMagn-1) / 4 ;
    ulTTPhiBy4     = (_ulTTPhi-1) / 4 ;

    if ( _poL1Cal->m_apoAdf[m_ulTTEtaSign][ulTTEtaMagnBy4][ulTTPhiBy4] != NULL )
        m_poMotherAdf  = _poL1Cal->m_apoAdf[m_ulTTEtaSign][ulTTEtaMagnBy4][ulTTPhiBy4] ;
    else
    {
        m_poMotherAdf = NULL ;
        oConsole.InternalErrorLine ( (CString) "CL1Cal_TrgTwr::CL1Cal_TrgTwr: No ADF Card found for TrgTwr "
                                              + f_osTrgTwr_Coord() ) ;
    }
}


/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
void    CL1Cal_TrgTwr::BasicConstruct ( unsigned long       _ulTTEtaSign,
                                        unsigned long       _ulTTEtaMagn,
                                        unsigned long       _ulTTPhi,
                                        CL1Cal_TrgTwr**     _ppoPrev_L1Cal_TrgTwr,
                                        TuwRegWord          _uwRawAdcZer,
                                        TuwRegWord          _uwOutputEtZer )
{
    unsigned int    eTtType ;
    unsigned int    ulTickNum ;
    unsigned int    ulSubTickNum ;

    //\\//

    m_ulTTEtaSign           = _ulTTEtaSign ;
    m_ulTTEtaMagn           = _ulTTEtaMagn ;
    m_ulTTPhi               = _ulTTPhi ;

    m_ulRelEta              = (_ulTTEtaMagn-1) % KiTotEtaPerAdf ;
    m_ulRelPhi              = (_ulTTPhi-1)     % KiTotPhiPerAdf ;

    m_ulFpgaThisRelEta      = m_ulRelEta / KiTotEtaPerAdfFpga ;
    m_ulRelEtaOnFpga        = m_ulRelEta % KiTotEtaPerAdfFpga ;

    m_uwRawAdcZer           = _uwRawAdcZer ;
    m_uwOutputEtZer         = _uwOutputEtZer ;

    for ( eTtType   = 0 ; eTtType < eTot_TT_Chan_Type  ; eTtType ++ )
    {
        m_ubRelAlignDelay[eTtType]      = 0 ;
        m_ulPedestalControlDAC[eTtType]  = 0 ;

        m_fPedestalAverage[eTtType]      = 0.0 ;
        m_fPedestalStdDev[eTtType]       = 0.0 ;

        m_fPrevPedStdDev[eTtType]        = 0.0 ;
        m_ulPrevPedCtrlDAC[eTtType]      = 0 ;

        for ( ulTickNum    = 0 ; ulTickNum    <= KiTotTicksInTurn ;    ulTickNum ++ )
        for ( ulSubTickNum = 0 ; ulSubTickNum <  KiTotSamplesPerTick ; ulSubTickNum ++ )
            m_uwSimuRawAdc[eTtType][ulTickNum ][ulSubTickNum] = 0 ;

        for ( ulTickNum    = 0 ; ulTickNum    <= KiTotTicksInTurn ;    ulTickNum ++ )
            m_ubSimuOutputEt[eTtType][ulTickNum ] = 0 ;

    }

    // grab the pointer to the previous Trigger Tower in the chain,
    // and return a pointer to *this* TrgTwr to be used as the next "previous TrgTwr"
    // or as the last TrgTwr recorded in this Chain.
    if ( _ppoPrev_L1Cal_TrgTwr != NULL ) // but only if this optional parameter was provided
    {
        m_poPrev_L1Cal_TrgTwr = *_ppoPrev_L1Cal_TrgTwr ;
        *_ppoPrev_L1Cal_TrgTwr = this ;
    }
    else
    {
        m_poPrev_L1Cal_TrgTwr = NULL ;
    }

}
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
CL1Cal_TrgTwr::~CL1Cal_TrgTwr ()
{
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess    CL1Cal_TrgTwr::Initialize ()
{
    ESuccess        eReturnStatus = Ok ;

    unsigned int    eTtType ;

    //\\//

    if ( m_poMotherAdf == NULL )
    {
        eReturnStatus = Error ;
        oConsole.InternalErrorLine ( (CString) "CL1Cal_TrgTwr::Initialize: Missing ADF pointer" ) ;
    }
    else
    {   // do both halves of the Trigger Tower
        for ( eTtType   = 0 ; eTtType < eTot_TT_Chan_Type  ; eTtType ++ )
        {
            //    A.1)   Load the Pedestal Control DAC
            //
            //        The default value is 0
            //
            //        Different values can be specified in TTI command files
            //        via the keyword "Pedestal_Control_DAC:"
            //
            //        Find_DAC can automatically produce an output in the form
            //        of a TTI command file suited to define Pedestal Control values
            //        over a range of trigger towers.
            //
            if ( InitPedestalDac ( (ETT_Chan_Type) eTtType )  != Ok )
               eReturnStatus = Error ;

            //    A.2)   Load the Et Lookup Memory
            //
            //        The default is a one-to-one lookup with Output Et saturation at 255
            //
            //        Different lookups can be specified using TTI command files
            //        via the keywords "Zero_Energy_Adc_E:", "Zero_Energy_Output_Et:",
            //        and ""Lookup_Slope:"
            //
            if ( InitEtLookup ( (ETT_Chan_Type) eTtType )  != Ok )
               eReturnStatus = Error ;

            //    A.3)   Load the Alignment Delay
            //
            //        The value programmed is the sum of a Base Alignment Delay value
            //        common to all Trigger Towers and a Relative Alighment Delay
            //        value specific to each Trigger Tower.
            //
            //
            //        The default Relative Alignment Delay value is 0
            //
            //        Different RELATIVE Alignment Delay values must be specified
            //        in TTI command files via the keyword "Alignment_Delay:"
            //
            //
            //        The default Base Alignment Delay value is 0
            //
            //        The Global BASE Alignment Delay value is specified
            //        in a Master Command file via the keyword "BaseAlignmentDelay:",
            //        e.g. as part of the Auxiliary Boot command file (Boot_Auxi.mcf)
            //
            //        The Global Base Alignment Delay value can also be specified
            //        or modified on the fly with a COOR-like command
            //        "TrgMgr_BaseAlignmentDelay <value>"
            //
            if ( InitAlignmentDelay ( (ETT_Chan_Type) eTtType ) != Ok )
               eReturnStatus = Error ;
        }
    }
    return eReturnStatus ;
}
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess    CL1Cal_TrgTwr::InitEtLookup        ( ETT_Chan_Type       _eTTChanType )
{

    unsigned short  uwZeroEngAdcE ;
    unsigned short  uwZeroEngOutEt ;
    double          fdLookupSlope ;

    bool            lSkipVerify = true ;

    //\\//

    if ( m_poMotherAdf != NULL ) // safety check
    {
        // Load the Et Lookup Memory
        uwZeroEngAdcE   = m_axEtLookupCoeff[_eTTChanType].uwZeroEngAdcE ;
        uwZeroEngOutEt  = m_axEtLookupCoeff[_eTTChanType].uwZeroEngOutEt ;
        fdLookupSlope   = m_axEtLookupCoeff[_eTTChanType].fdLookupSlope ;

        return   m_poMotherAdf ->m_apoT7Fpga[m_ulFpgaThisRelEta]
                               ->m_a3poTt_MemBlkEtLookup[m_ulRelEtaOnFpga]
                                                        [m_ulRelPhi]
                                                        [_eTTChanType]
                               ->f_okLoadLookup ( &uwZeroEngAdcE,
                                                  &uwZeroEngOutEt,
                                                  &fdLookupSlope,
                                                   lSkipVerify ) ;
    }
    else
        return Error ;
}
/*-------------------------------------------------------------------------------*/


/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess    CL1Cal_TrgTwr::InitAlignmentDelay  ( ETT_Chan_Type       _eTTChanType )
{
    if ( m_poMotherAdf != NULL ) // safety check
    {
        return   m_poMotherAdf ->m_apoT7Fpga[m_ulFpgaThisRelEta]
                               ->SetAlignmentDelay ( _eTTChanType,
                                                     m_ulRelEtaOnFpga,
                                                     m_ulRelPhi,
                                                     CL1Cal::mg_ubBaseAlignDelay + m_ubRelAlignDelay[_eTTChanType] ) ;
    }
    else
        return Error ;
}
/*-------------------------------------------------------------------------------*/


/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess    CL1Cal_TrgTwr::InitPedestalDac ( ETT_Chan_Type       _eTTChanType )
{
    if ( m_poMotherAdf != NULL ) // safety check
    {
        return   m_poMotherAdf ->m_poDacCtrl
                               ->f_okWriteOneDac   ( _eTTChanType,
                                                     m_ulRelEta,
                                                     m_ulRelPhi,
                                                     m_ulPedestalControlDAC[_eTTChanType] ) ;
    }
    else
        return Error ;
}
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess    CL1Cal_TrgTwr::VerifyProgramming        ( )
{

    ESuccess        eReturnStatus = Ok ;

    unsigned int    eTtType ;

    //\\//

    if ( m_poMotherAdf == NULL )
    {
        oConsole.InternalErrorLine ( (CString) "CL1Cal_TrgTwr::VerifyProgramming: Missing ADF pointer" ) ;
        return Error ;
    }
    else
    {   // do both halves of the Trigger Tower
        for ( eTtType   = 0 ; eTtType < eTot_TT_Chan_Type  ; eTtType ++ )
        {
#if defined VME_IO_FAKE
#else
            // Verify the Pedestal Control DAC
            // by watching the Raw ADC average
            //
//            if ( VerifyZeresp ( (ETT_Chan_Type) eTtType )  != Ok )
//               eReturnStatus = Error ;

            //  Verify the Et Lookup Memory,
            //  which wasn't done while it was being initialized
            //
            if ( VerifyEtLookup ( (ETT_Chan_Type) eTtType )  != Ok )
               eReturnStatus = Error ;
#endif // not VME_IO_FAKE
        }
    }

    return eReturnStatus ;
}
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess    CL1Cal_TrgTwr::VerifyZeresp      ( ETT_Chan_Type       _eTTChanType )
{
    ESuccess        eReturnStatus = Ok ;
    unsigned long   ulSampleSize = 50 ;
    double          fAverage = 0.0 ;

    //\\//

    if ( m_poMotherAdf != NULL ) // safety check
    {
        /* Read the ADC register a number of times and histogram the result */
        if ( m_poMotherAdf->m_apoT7Fpga[m_ulFpgaThisRelEta]
                          ->m_a3poTt_RawAdc[m_ulRelEtaOnFpga]
                                           [m_ulRelPhi]
                                           [_eTTChanType]
                          ->f_okReadAverage( &fAverage,
                                              ulSampleSize )    != Ok )
             eReturnStatus = Error ;
        else
            if ( fabs( fAverage - KiRawAdcZer ) >= 0.5 )
                oConsole.WarningLine ( (CString) "Raw ADC average "
                                        + f_osTrgTwr_Coord (_eTTChanType)
                                        + " is "
                                        + F_to_A ( fAverage, 2 ) ) ;
    }

    return eReturnStatus ;
}
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess    CL1Cal_TrgTwr::VerifyEtLookup        ( ETT_Chan_Type       _eTTChanType )
{

    if ( m_poMotherAdf != NULL ) // safety check
    {
        return   m_poMotherAdf ->m_apoT7Fpga[m_ulFpgaThisRelEta]
                               ->m_a3poTt_MemBlkEtLookup[m_ulRelEtaOnFpga]
                                                        [m_ulRelPhi]
                                                        [_eTTChanType]
                               ->f_okVerifyLookup ( ) ;
    }
    else
        return Error ;
}
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
CString         CL1Cal_TrgTwr::f_osTrgTwr_Coord( )
{ // Show coordinates e.g. "TT(-20, 1)"

    TCString  osMsg ;
    char cEtaSign ;

    //\\//

    if ( m_ulTTEtaSign == eTT_Eta_Pos )
        cEtaSign = '+' ;
    else
        cEtaSign = '-' ;

    osMsg = (CString) "TT("
                    + cEtaSign
                    + I_to_A ( m_ulTTEtaMagn, 2 )
                    + ","
                    + I_to_A ( m_ulTTPhi, 2 )
                    + ")" ;
    return osMsg ;
}
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
CString       CL1Cal_TrgTwr::f_osTrgTwr_Coord  ( ETT_Chan_Type   _eTTChanType )
{ // Show coordinates e.g. "EM_TT(-20, 1)"

    TCString  osMsg ;
    //\\//

    switch ( _eTTChanType )
    {
      case eTT_EM :
        osMsg = "EM_" ;
        break ;

      case eTT_HD :
        osMsg = "HD_" ;
        break ;

      default :
        osMsg = "??_" ;
        break ;
    }

    return osMsg + f_osTrgTwr_Coord() ;
}
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
CString       CL1Cal_TrgTwr::f_osTrgTwr_RefThresh ( ETabRefSet_Type _eRefSetType,
                                                    unsigned long   _ulRefSetNum )
{ // String of coordinates and TT Comparator Refence e.g. "TT(-20, 1) EM Et Thresh #0"

    TCString  osMsg ;
    //\\//

    switch ( _eRefSetType)
    {
      case eTab_RefSet_EMEt :
        osMsg = "EM Et" ;
        break ;

      case eTab_RefSet_JetEt :
        osMsg = "Tot Et" ;
        break ;

      case eTab_RefSet_TTEt :
        osMsg = "TT Et" ;
        break ;

      default :
        osMsg = "??_" ;
        break ;
    }

    osMsg += " Thresh #" + I_to_A ( _ulRefSetNum ) ;

    return f_osTrgTwr_Coord() + " " + osMsg ;
}
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
CString         CL1Cal_TrgTwr::f_osTrgTwr_Coord( unsigned long   _ulTTEtaSign,
                                                 unsigned long   _ulTTEtaMagn,
                                                 unsigned long   _ulTTPhi )
{ // Show coordinates e.g. "TT(-20, 1)"

    TCString  osMsg ;
    char cEtaSign ;

    //\\//

    if ( _ulTTEtaSign == eTT_Eta_Pos )
        cEtaSign = '+' ;
    else
        cEtaSign = '-' ;

    osMsg = (CString) "TT("
                    + cEtaSign
                    + I_to_A ( _ulTTEtaMagn, 2 )
                    + ","
                    + I_to_A ( _ulTTPhi, 2 )
                    + ")" ;
    return osMsg ;
}
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
CString         CL1Cal_TrgTwr::f_osTrgTwr_Coord   ( ETT_Chan_Type   _eTTChanType,
                                                    unsigned long   _ulTTEtaSign,
                                                    unsigned long   _ulTTEtaMagn,
                                                    unsigned long   _ulTTPhi )
{ // Show coordinates e.g. "EM_TT(-20, 1)"

    TCString  osMsg ;
    //\\//

    switch ( _eTTChanType )
    {
      case eTT_EM :
        osMsg = "EM_" ;
        break ;

      case eTT_HD :
        osMsg = "HD_" ;
        break ;

      default :
        osMsg = "??_" ;
        break ;
    }

    return osMsg + f_osTrgTwr_Coord(_ulTTEtaSign, _ulTTEtaMagn, _ulTTPhi ) ;
}
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
CString       CL1Cal_TrgTwr::f_osTrgTwr_RefThresh ( ETabRefSet_Type _eRefSetType,
                                                    unsigned long   _ulRefSetNum,
                                                    unsigned long   _ulTTEtaSign,
                                                    unsigned long   _ulTTEtaMagn,
                                                    unsigned long   _ulTTPhi )
{ // String of coordinates and TT Comparator Refence e.g. "TT(-20, 1) EM Et RefCmp #0"

    TCString  osMsg ;
    //\\//

    switch ( _eRefSetType)
    {
      case eTab_RefSet_EMEt :
        osMsg = "EM Et" ;
        break ;

      case eTab_RefSet_JetEt :
        osMsg = "Tot Et" ;
        break ;

      case eTab_RefSet_TTEt :
        osMsg = "TT Et" ;
        break ;

      default :
        osMsg = "??_" ;
        break ;
    }


    osMsg += " Thresh #" + I_to_A ( _ulRefSetNum ) ;

    return f_osTrgTwr_Coord(_ulTTEtaSign, _ulTTEtaMagn, _ulTTPhi ) + " " + osMsg ;
}
/*-------------------------------------------------------------------------------*/


/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
CString         CL1Cal_TrgTwr::f_osTrgTwr_Range( unsigned long   _ulTTEtaSignMin,
                                                unsigned long   _ulTTEtaSignMax,
                                                unsigned long   _ulTTEtaMagnMin,
                                                unsigned long   _ulTTEtaMagnMax,
                                                unsigned long   _ulTTPhiMin,
                                                unsigned long   _ulTTPhiMax )
{ // Show Range e.g. "eta(-20:+20),phi(1:32)"
  //              or "eta(+5),phi(6)"
  //              or "eta(-5),phi(6)"
  //              or "eta(+5:+8),phi(6)"
  //              or "eta(-8:-5),phi(6)"
  //              or "eta(-8:-5 & +5:+8),phi(1:32)"
  //              or "eta(-8 & +8),phi(1:32)"
  //              or "eta(-8:+8),phi(1:32)"

    TCString  osMsg ;

    //\\//

    if ( _ulTTEtaSignMax == eTT_Eta_Neg )
        osMsg = "eta(-" + I_to_A(_ulTTEtaMagnMax) ;
    else
        osMsg = "eta(+" + I_to_A(_ulTTEtaMagnMin) ;

    if ( ( _ulTTEtaMagnMax != _ulTTEtaMagnMin )
      || ( _ulTTEtaSignMax != _ulTTEtaSignMin ) ) // otherwise eta already all done
    {
        if ( _ulTTEtaMagnMax != _ulTTEtaMagnMin )
        {
            if ( ( _ulTTEtaSignMax != _ulTTEtaSignMin )
              && ( _ulTTEtaMagnMin != eTT_Phi_1 ) ) // coverage does not cross zero
                osMsg += ":-" + I_to_A(_ulTTEtaMagnMin)
                      + " & +" + I_to_A(_ulTTEtaMagnMin) ;

            if ( _ulTTEtaSignMin == eTT_Eta_Neg )
                osMsg += ":-" + I_to_A(_ulTTEtaMagnMin) ;
            else
                osMsg += ":+" + I_to_A(_ulTTEtaMagnMax) ;
        }
        else // must be ( _ulTTEtaSignMax != _ulTTEtaSignMin ) && ( _ulTTEtaMagnMax == _ulTTEtaMagnMin )
        {
                osMsg += " & +" + I_to_A(_ulTTEtaMagnMin) ;
        }
    }

    osMsg += "),phi("
           + I_to_A(_ulTTPhiMin) ;

    if ( _ulTTPhiMax != _ulTTPhiMin )
        osMsg += ":" + I_to_A(_ulTTPhiMax) ;

    osMsg += ")" ;

    return osMsg ;
}
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
CString         CL1Cal_TrgTwr::f_osTrgTwr_Range( ETT_Chan_Type   _eTTChanTypeMin,
                                                ETT_Chan_Type   _eTTChanTypeMax,
                                                unsigned long   _ulTTEtaSignMin,
                                                unsigned long   _ulTTEtaSignMax,
                                                unsigned long   _ulTTEtaMagnMin,
                                                unsigned long   _ulTTEtaMagnMax,
                                                unsigned long   _ulTTPhiMin,
                                                unsigned long   _ulTTPhiMax )
{ // Show Range e.g. "EM&HD eta(-20:+20),phi(1:32)"

    TCString  osMsg ;

    //\\//

    osMsg = sQuantType[_eTTChanTypeMin] ;

    if ( _eTTChanTypeMax != _eTTChanTypeMin )
    {
        osMsg += "&" ;
        osMsg += sQuantType[_eTTChanTypeMax] ;
    }

    return osMsg + " " + f_osTrgTwr_Range( _ulTTEtaSignMin,
                                           _ulTTEtaSignMax,
                                           _ulTTEtaMagnMin,
                                           _ulTTEtaMagnMax,
                                           _ulTTPhiMin,
                                           _ulTTPhiMax ) ;
}
/*-------------------------------------------------------------------------------*/


/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess    CL1Cal_TrgTwr::f_okExclude_Tower  ( ETT_Chan_Type           _eTTChanType,
                                                bool                    _lVerbose )
{
    ESuccess        eReturnStatus = Ok ;

    //\\//

    if ( _lVerbose )
    oConsole.SystemLine ( (CString) "Excluding Trigger Tower "
                                  + f_osTrgTwr_Coord ( _eTTChanType )  ) ;

    if ( m_poMotherAdf != NULL )
    {
        if ( m_poMotherAdf ->m_apoT7Fpga[m_ulFpgaThisRelEta]
                           ->ExcludeTower ( _eTTChanType,
                                             m_ulRelEtaOnFpga,
                                             m_ulRelPhi ) != Ok )
           eReturnStatus = Error ;
    }
    else
    {
        eReturnStatus = Error ;
        oConsole.InternalErrorLine ( (CString) "CL1Cal_TrgTwr::f_okExclude_Tower: Missing ADF pointer" ) ;
    }

    return eReturnStatus ;
}
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess    CL1Cal_TrgTwr::Find_DAC       ( ETT_Chan_Type       _eTTChanType,
                                            TuwRegWord*         _puwDacValue,
                                            double*             _pfHistoAve,
                                            double*             _pfHistoDev,
                                            double*             _pfStdDevAve,
                                            bool                _lShowDetails,
                                            CResultFile*        _poHistoFile )
{

    /////////////////////////////////////////////
    //Search parameters
    /////////////////////////////////////////////

#define KfDacToAdcRatioTyp                7.15 // Increasing the DAC by +7 decreases the Zeresp by +1
#define KfDacToAdcRatioOkDiff             0.5  // Accepted Difference between Calculated and Typical DAC to ADC ratio

#define KiManyTimeConst                     2  // 2 ms = many times the time const (~20 us)
                                               // changing this value may need changing kiTimerPeriod in Dlg_Test_Find_Dac.cpp

#define KiDacMinValue                       0  // should produce a Half-range Zeresp of ~ 505
#define KiDacMaxValue                    4095  // should produce a Null Zeresp of 0
#define KiDacLowAdcValue                 3350  // should produce a Low Zeresp around 45
#define KiDacMidAdcValue                   50  // should produce a High Zeresp around 500

#define KfZerespMinOk                     0.1  // highest value acceptable for a Low Zeresp = must be able to set to zero (but noise?)

#define KiZerespMaxOkLow                  480  // lowest value acceptable for Max Zeresp ~ 5% off of 505
#define KiZerespMaxOkHigh                 530  // highest value acceptable for Max Zeresp ~ 5% off of 505

#define KiZerespLowOkLow                    5  // lowest value acceptable for low-scale Zeresp ~ 10% of range off of 45
#define KiZerespLowOkHigh                  85  // highest value acceptable for low-scale Zeresp ~ 10% of range off of 45

#define KiZerespMidOkLow                  475  // lowest value acceptable for mid-scale Zeresp ~ 5 % off of 500
#define KiZerespMidOkHigh                 525  // highest value acceptable for mid-scale Zeresp ~ 5 % off of 500

#define KiTotHistoScanned                  14  // How many histograms will be built around the target zeresp
#define KfZerespDiffOk                   0.15  // How close to the target zeresp the BEST histogram needs to be
#define KfZerespCloseOk                   2.0  // How close to the target zeresp ANY histogram has to be

#define KiSmallSampleSize                 100  // Small Histogram Sample Size (for checking 0x00 and 0xff rails)
#define KiLargeSampleSize                1000  // Large Histogram Sample Size (for everything else)

//#define PICK_BEST_POPULATION
#define PICK_BEST_AVERAGE

    //\\//

    ESuccess        eReturnStatus = Ok ;

    CString         oMsg ;

//    unsigned char   ubChannelSelect ;

    TuwRegWord      uwZerespGoal ;

    unsigned short  uwDacValue ;
    TAdcHisto       auwHistogram ;
    unsigned long   ulSampleSize ;
    double          fAverage ;
    double          fDeviation ;

    double          fZerespMidAvr ;
    double          fZerespLowAvr ;
    double          fDacToAdcRatioMeas ;

    unsigned short  uwHistoNum ;
    unsigned short  uwHistoChosen ;
    unsigned short  uwScanDacValueStart ;
    double          fScanAverage [KiTotHistoScanned] ;
    double          fScanDeviation [KiTotHistoScanned] ;
#ifdef PICK_BEST_POPULATION
    unsigned long   ulScanZerespBinPopul [KiTotHistoScanned] ;
#endif

//    unsigned long   ulIndicatorLevel = 0 ;

    //\\//

/////////////////////////////////////////////////////////
#define SHOW_LINE( LINE )                               \
    {                                                   \
        oMsg = LINE ;                                   \
        oConsole.WriteLine ( oMsg ) ;                   \
        if ( _poHistoFile != NULL )                     \
            _poHistoFile->WriteLine ( oMsg ) ;          \
    }                                                   \
/////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////
#define WRITE_LINE( LINE )                              \
    {                                                   \
        oMsg = LINE ;                                   \
        if ( _poHistoFile != NULL )                     \
            _poHistoFile->WriteLine ( oMsg ) ;          \
    }                                                   \
/////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////
#define ERROR_LINE( LINE )                              \
    {                                                   \
        oMsg = LINE ;                                   \
        oConsole.ErrorLine ( oMsg ) ;                   \
        if ( _poHistoFile != NULL )                     \
            _poHistoFile->WriteLine ( oMsg ) ;          \
    }                                                   \
/////////////////////////////////////////////////////////

    //Advertize where we are
    WRITE_LINE ( " " ) ;
    WRITE_LINE ( (CString) "Now Processing " + f_osTrgTwr_Coord( _eTTChanType ) ) ;

    //If we don't get anywhere, we will return zeroes and an error status
    //But if we get close we will still return some values and an error status
    if ( _puwDacValue != NULL ) *_puwDacValue = 0 ;
    if ( _pfHistoAve  != NULL ) *_pfHistoAve  = 0.0 ;
    if ( _pfHistoDev  != NULL ) *_pfHistoDev  = 0.0 ;
    if ( _pfStdDevAve != NULL ) *_pfStdDevAve = 0.0 ;

    //we can't continue if we didn't find a ADF card in the constructor
    if ( m_poMotherAdf == NULL )
    {
        ERROR_LINE ( " No associated ADF Card; Find DAC Aborting" ) ;
        goto Immediate_Return ;
    }

    //Unlike the old L1 Cal Trig, we do *NOT* need
    //to De-Exclude any Trigger Towers

    //retrieve the Target Zero Energy Response
    uwZerespGoal = m_uwRawAdcZer ;

    //Start the search with the smaller histogram sample size
    ulSampleSize    = KiSmallSampleSize ;

/////////////////////////////////////////////////////////////////////////////
#define SET_PEDESTAL_DAC( DAC_VALUE )                                       \
    /* Update the DAC control register */                                   \
    if ( m_poMotherAdf->m_poDacCtrl->f_okWriteOneDac ( _eTTChanType,        \
                                                        m_ulRelEta,         \
                                                        m_ulRelPhi,         \
                                                        DAC_VALUE )         \
         != Ok ) goto DAC_Write_IO_Error ;                                  \
                                                                            \
    /* wait for DAC voltage filtering (especially for Legacy DAC) */        \
    Sleep ( KiManyTimeConst ) ;                                             \
/////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////
#define COLLECT_HISTOGRAM( DAC_VALUE, AVERAGE, DEVIATION )                  \
if ( CL1Cal::mg_eFindDacMode == eFindDacMode_LiveX )                        \
{                                                                           \
    /* Capture whole turns, extract live crossings, and histogram result */ \
    if ( m_poMotherAdf->m_apoT7Fpga[m_ulFpgaThisRelEta]                     \
                      ->ADC_HistoLiveX( _eTTChanType,                       \
                                         m_ulRelEtaOnFpga,                  \
                                         m_ulRelPhi,                        \
                                        &auwHistogram,                      \
                                         ulSampleSize,                      \
                                        &DAC_VALUE )                        \
         != Ok ) goto Histogram_IO_Error ;                                  \
}                                                                           \
else                                                                        \
{                                                                           \
    /* Read the ADC register a bunch of times and histogram the result */   \
    if ( m_poMotherAdf->m_apoT7Fpga[m_ulFpgaThisRelEta]                     \
                      ->m_a3poTt_RawAdc[m_ulRelEtaOnFpga]                   \
                                       [m_ulRelPhi]                         \
                                       [_eTTChanType]                       \
                      ->f_okHisto10bits( &auwHistogram,                     \
                                          ulSampleSize )                    \
         != Ok ) goto Histogram_IO_Error ;                                  \
}                                                                           \
    AVERAGE    = CAdfRegBase::HistoAverage10bit   ( &auwHistogram,          \
                                                     ulSampleSize );        \
    DEVIATION  = CAdfRegBase::HistoDeviation10bit ( &auwHistogram,          \
                                                     ulSampleSize,          \
                                                     AVERAGE ) ;            \
                                                                            \
    /* Display the Histogram obtained if requested */                       \
    if ( _lShowDetails == TRUE )                                            \
    {                                                                       \
        WRITE_LINE ( (CString) " DAC= "                                     \
                             + I_to_A ( DAC_VALUE, 4 )                     \
                             + " -> "                                       \
                             + DumpHistogram  ( auwHistogram,               \
                                                KiSizeOfTAdcHisto,          \
                                               &AVERAGE,                    \
                                               &DEVIATION ) ) ;             \
    }                                                                       \
/////////////////////////////////////////////////////////////////////////////

    ///////////////////////////////////////////////////////////////////
    // Set the DAC Control value to zero -> push ADC zeresp up to max (still mid scale)
    ///////////////////////////////////////////////////////////////////

    uwDacValue = KiDacMinValue ;

    SET_PEDESTAL_DAC ( uwDacValue ) ;

    COLLECT_HISTOGRAM ( uwDacValue, fAverage, fDeviation ) ;

    //Verify we went to zeresp of mid scale
    if ( ( fAverage < (double) KiZerespMaxOkLow  )
      || ( fAverage > (double) KiZerespMaxOkHigh ) )
    ERROR_LINE ( (CString) f_osTrgTwr_Coord( _eTTChanType )
                         + " Warning: Failed Forcing Zeresp to highest: DAC = "
                         + I_to_A ( uwDacValue )
                         + " Avr = "
                         + F_to_A ( fAverage, 2 ) ) ;

    ///////////////////////////////////////////////////////////////////
    // Set the DAC Control value to full scale -> push ADC zeresp down to zero
    ///////////////////////////////////////////////////////////////////
    uwDacValue = KiDacMaxValue ;

    SET_PEDESTAL_DAC ( uwDacValue ) ;

    COLLECT_HISTOGRAM ( uwDacValue, fAverage, fDeviation ) ;

    //Verify we went to zeresp of 0x00
    if ( fAverage > KfZerespMinOk )
    ERROR_LINE ( (CString) f_osTrgTwr_Coord( _eTTChanType )
                         + " Warning: Failed Forcing Zeresp to 0x0000 : DAC = "
                         + I_to_A ( uwDacValue )
                         + " Avr = "
                         + F_to_A ( fAverage, 2 ) ) ;

    ///////////////////////////////////////////////////////////////////
    // Set the DAC Control value to low value -> set ADC zeresp to low value
    ///////////////////////////////////////////////////////////////////
    uwDacValue = KiDacLowAdcValue ;

    SET_PEDESTAL_DAC ( uwDacValue ) ;

    //Time to get serious: use larger histogram sample size
    ulSampleSize    = KiLargeSampleSize ;

    COLLECT_HISTOGRAM ( uwDacValue, fZerespLowAvr, fDeviation ) ;

    //Verify we went to zeresp in low values
    if ( ( fZerespLowAvr < (double) KiZerespLowOkLow )
      || ( fZerespLowAvr > (double) KiZerespLowOkHigh ) )
    ERROR_LINE ( (CString) f_osTrgTwr_Coord( _eTTChanType )
                         + " Warning: Failed Forcing Zeresp to ~small : DAC = "
                         + I_to_A ( uwDacValue )
                         + " Avr = "
                         + F_to_A ( fZerespLowAvr, 2 ) ) ;

    ///////////////////////////////////////////////////////////////////
    // Set the DAC Control value to low value -> set ADC zeresp to mid range
    ///////////////////////////////////////////////////////////////////
    uwDacValue = KiDacMidAdcValue ;

    SET_PEDESTAL_DAC ( uwDacValue ) ;

    //Time to get serious: use larger histogram sample size
    ulSampleSize    = KiLargeSampleSize ;

    COLLECT_HISTOGRAM ( uwDacValue, fZerespMidAvr, fDeviation ) ;

    //Verify we went to zeresp in mid range
    if ( ( fZerespMidAvr < (double) KiZerespMidOkLow )
      || ( fZerespMidAvr > (double) KiZerespMidOkHigh ) )
    ERROR_LINE ( (CString) f_osTrgTwr_Coord( _eTTChanType )
                         + " Warning: Failed Forcing Zeresp to ~middle: DAC = "
                         + I_to_A ( uwDacValue )
                         + " Avr = "
                         + F_to_A ( fZerespMidAvr, 2 ) ) ;


    ///////////////////////////////////////////////////////////////////
    // Compute DAC to ADC ratio
    ///////////////////////////////////////////////////////////////////
    if ( fabs ( fZerespMidAvr - fZerespLowAvr ) >= 1.0 ) // disaster protection
        fDacToAdcRatioMeas = ( KiDacLowAdcValue - KiDacMidAdcValue )   // want a positive number
                           / ( fZerespMidAvr - fZerespLowAvr ) ;
    else
        fDacToAdcRatioMeas = 0.0 ;

    //Verify we got where we intended
    if ( fabs ( fDacToAdcRatioMeas - KfDacToAdcRatioTyp ) > KfDacToAdcRatioOkDiff )
    {
        ERROR_LINE ( (CString) f_osTrgTwr_Coord( _eTTChanType )
                             + " Warning: DAC to ADC Ratio Problem, "
                             + I_to_A ( KiDacMidAdcValue )
                             + "->"
                             + F_to_A ( fZerespMidAvr, 2 )
                             + ", "
                             + I_to_A ( KiDacLowAdcValue )
                             + "->"
                             + F_to_A ( fZerespLowAvr, 2 ) ) ;
    }

    // Report on computed DAC to ADC Ratio
    if ( fDacToAdcRatioMeas != 0.0 )
    {
        WRITE_LINE ( (CString) "Computed DAC to ADC ratio = "
                             + F_to_A ( fDacToAdcRatioMeas, 2 ) ) ;
    }
    else
    {
        ERROR_LINE ( (CString) f_osTrgTwr_Coord( _eTTChanType )
                             + " Aborting: Couldn't Compute DAC to ADC Ratio") ;
        goto Failed ;
    }


    ///////////////////////////////////////////////////////////////////
    // Now adjust the DAC Control value to bring zeresp to the beginning of scan range
    ///////////////////////////////////////////////////////////////////

    // compute the begining of the DAC Value range we are going to scan
    // using the measured DAC to ADC ratio
    // remember: increasing the DAC value will decrease the zeresp
    uwScanDacValueStart = KiDacLowAdcValue
                        + (unsigned short) ( ( fZerespLowAvr - uwZerespGoal ) * fDacToAdcRatioMeas + 0.5 )
                        + KiTotHistoScanned / 2 
                        - 2 ; // This empirical correction brings the average of the chosen histogram closer to the center of the window

    //Verify we computed a valid control value
    if ( ( uwScanDacValueStart >= KiDacMaxValue )
      || ( uwScanDacValueStart <= KiDacMinValue ) )
    {
        ERROR_LINE ( (CString) f_osTrgTwr_Coord( _eTTChanType )
                             + " Aborting: Illegal Computed DAC Value ="
                             + I_to_A ( uwScanDacValueStart)
                             + " for Start of Scan" ) ;
        goto Failed ;
    }

    double fAverageOfStdDev ;

    fAverageOfStdDev = 0.0 ;

    uwHistoChosen = 0 ;
    for ( uwHistoNum = 0 ;
          uwHistoNum < KiTotHistoScanned ;
          uwHistoNum ++ )
    {
        // remember: decreasing the DAC value will increase the zeresp
        uwDacValue = uwScanDacValueStart - uwHistoNum ;

        SET_PEDESTAL_DAC ( uwDacValue ) ;

        COLLECT_HISTOGRAM( uwDacValue, fScanAverage[uwHistoNum], fScanDeviation[uwHistoNum] ) ;

        // Verify the Histogram average is within reason (i.e. no programming DAC error)
        if ( ( fabs( fScanAverage[uwHistoNum] - (double) uwZerespGoal) > KfZerespCloseOk ) )
        {
            ERROR_LINE ( (CString) f_osTrgTwr_Coord( _eTTChanType )
                                 + " Average out of tolerance DAC= "
                                 + I_to_A ( uwDacValue, 4 )
                                 + " Avr= "
                                 + F_to_A ( fScanAverage[uwHistoNum], 2 )
                                 + " Dev= "
                                 + F_to_A ( fScanDeviation[uwHistoNum], 2 ) ) ;
            eReturnStatus = Error ; // will generate an error in the TTI file to notice the problem
        }

        fAverageOfStdDev += fScanDeviation[uwHistoNum] ;

#ifdef PICK_BEST_POPULATION
        //Remember the current histogram population for the target ADC value
        ulScanZerespBinPopul [uwHistoNum] = auwHistogram [uwZerespGoal] ;

        // look for the histogram with the largest population for the target zeresp
        if ( ulScanZerespBinPopul [uwHistoNum] > ulScanZerespBinPopul [uwHistoChosen] )
        uwHistoChosen = uwHistoNum ;
#endif

#ifdef PICK_BEST_AVERAGE
        // look for the histogram with average closest to target zeresp
        if ( fabs( fScanAverage[   uwHistoNum] - (double) uwZerespGoal)
           < fabs( fScanAverage[uwHistoChosen] - (double) uwZerespGoal) )
        uwHistoChosen = uwHistoNum ;
#endif
    }

    // Finisch computation of the average of the Standard Deviations
    fAverageOfStdDev /= KiTotHistoScanned ;

    // copy the values to the call arguments
    if ( _puwDacValue != NULL ) *_puwDacValue = uwScanDacValueStart - uwHistoChosen ;
    if ( _pfHistoAve  != NULL ) *_pfHistoAve  = fScanAverage[uwHistoChosen] ;
    if ( _pfHistoDev  != NULL ) *_pfHistoDev  = fScanDeviation[uwHistoChosen] ;
    if ( _pfStdDevAve != NULL ) *_pfStdDevAve = fAverageOfStdDev ;

    // We say we succeeded if we selected an histogram other than the first or last one
    // and we also make sure we got close enough to the target Zeresp
    if ( ( uwHistoChosen != 0 )
      && ( uwHistoChosen != KiTotHistoScanned-1 )
      && ( fabs( fScanAverage[uwHistoChosen] - (double) uwZerespGoal) < KfZerespDiffOk ) )
    {
        SHOW_LINE ( (CString) f_osTrgTwr_Coord( _eTTChanType )
                             + " Picked Histo#"
                             + I_to_A ( uwHistoChosen+1, 2 )
                             + " DAC= "
                             + I_to_A ( (uwScanDacValueStart-uwHistoChosen), 4 )
                             + " Avr= "
                             + F_to_A ( fScanAverage[uwHistoChosen], 2 )
                             + " Dev= "
                             + F_to_A ( fScanDeviation[uwHistoChosen], 2 )
                             + " DevAvr= "
                             + F_to_A ( fAverageOfStdDev, 2 ) ) ;

        //Leave with the DAC programmed to the value picked
        SET_PEDESTAL_DAC ( uwScanDacValueStart - uwHistoChosen ) ;

        goto Done ;
    }
    else // picking the first histogram of the scan is not considered a success
    {
        ERROR_LINE ( (CString) f_osTrgTwr_Coord( _eTTChanType )
                         + " Failed: No Satisfactory DAC Control Value" ) ;

        //Leave with DAC set for lowest zeresp -> will stand out in TT_ADC trigmon
        SET_PEDESTAL_DAC ( KiDacMaxValue ) ;

        goto Failed ;
    }

//De_Exclude_IO_Error:

    //DAC write IO failure
    ERROR_LINE ( (CString) f_osTrgTwr_Coord( _eTTChanType )
                         + " Aborting: Failure De-Excluding Trigger Tower" ) ;
    goto Failed ;

DAC_Write_IO_Error:

    //DAC write IO failure
    ERROR_LINE ( (CString) f_osTrgTwr_Coord( _eTTChanType )
                         + " Aborting: Write DAC Error" ) ;
    goto Failed ;

Histogram_IO_Error:

    //Histogram IO Error
    ERROR_LINE ( (CString) f_osTrgTwr_Coord( _eTTChanType )
                         + " Aborting: Histograming Error" ) ;
    goto Failed ;

Failed:

    eReturnStatus = Error ;

Done:

Immediate_Return:

    return eReturnStatus ;
}
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess    CL1Cal_TrgTwr::Adjust_Ped     ( ETT_Chan_Type       _eTTChanType,
                                            TuwRegWord*         _puwDacValue,
                                            double*             _pfHistoAve,
                                            double*             _pfHistoDev,
                                            short*              _piwDacAdjust,
                                            bool                _lShowDetails,
                                            CResultFile*        _poHistoFile )
{

    /////////////////////////////////////////////
    //Adjust parameters
    /////////////////////////////////////////////

#define KiLargeSampleSize                1000  // Large Histogram Sample Size (for everything else)

#define KfDacToAdcRatioTypical           7.15 // Increasing the DAC by +7 decreases the Zeresp by +1

#define KiManyTimeConst                     2  // 2 ms = many times the time const (~20 us)
                                               // changing this value may need changing kiTimerPeriod in Dlg_Test_Find_Dac.cpp

#define KfInitialDacZerespWarning        15.0  // generate warning if initial Zeresp is very far off 

#define KfInitialEtCloseOk                0.3  // How close to the target Et Zeresp to skip adjustment

#define KiDacMinValue                       0  // should produce a Half-range Zeresp of ~ 505
#define KiDacMaxValue                    4095  // should produce a Null Zeresp of 0

#define KfAdjustedDacCloseOk              1.0  // How close to the target DACzeresp after adjustement must be

    //\\//

    ESuccess        eReturnStatus = Ok ;

    CString         oMsg ;

    TuwRegWord      uwZerespGoal ;

    TAdcHisto       auwHistogram ;
    unsigned long   ulSampleSize ;

    unsigned short  uwInitialDacValue ;
    double          fInitialAverage ;
    double          fInitialDeviation ;
    double          fInitialDacZerespDiscrepency ;
    double          fInitialInferredEtDiscrepency ;

    double          fLookupSlope ;

    unsigned short  uwAdjustedDacValue ;
    double          fAdjustedAverage ;
    double          fAdjustedDeviation ;
    double          fAdjustedDacZerespDiscrepency ;
    double          fAdjustedInferredEtShift ;
    double          fAdjustedInferredEtDiscrepency ;

    //Advertize where we are
    WRITE_LINE ( " " ) ;
    WRITE_LINE ( (CString) "Now Adjusting " + f_osTrgTwr_Coord( _eTTChanType ) ) ;

    //If we don't get anywhere, we will return zeroes and an error status
    //(note: if we get close we will still return some values and an error status)
    if ( _puwDacValue  != NULL ) *_puwDacValue  = 0 ;
    if ( _pfHistoAve   != NULL ) *_pfHistoAve   = 0.0 ;
    if ( _pfHistoDev   != NULL ) *_pfHistoDev   = 0.0 ;
    if ( _piwDacAdjust != NULL ) *_piwDacAdjust = 0 ;

    //we can't continue if we didn't find a ADF card in the constructor
    if ( m_poMotherAdf == NULL )
    {
        ERROR_LINE ( " No associated ADF Card; Adjust Ped Aborting" ) ;
        goto Immediate_Return ;
    }


    //retrieve the Target Zero Energy Response
    uwZerespGoal    = m_uwRawAdcZer ;

    //All histogram use the bigger sample size
    ulSampleSize    = KiLargeSampleSize;

    //Retrieve the current Control Value
    uwInitialDacValue = m_ulPedestalControlDAC[_eTTChanType] ;
    // and channel's Lookup Coefficient
    fLookupSlope = m_axEtLookupCoeff[_eTTChanType].fdLookupSlope ;


    ///////////////////////////////////////////////////////////////////
    // Collect a Histogram
    ///////////////////////////////////////////////////////////////////

    COLLECT_HISTOGRAM ( uwInitialDacValue, fInitialAverage, fInitialDeviation ) ;

    //Remember the current value as being the previous
    m_ulPrevPedCtrlDAC[_eTTChanType]    = uwInitialDacValue ;
    m_fPrevPedStdDev[_eTTChanType]      = fInitialAverage ;

    //Remember initial discrepency from target 
    fInitialDacZerespDiscrepency  = fabs ( fInitialAverage - (double) uwZerespGoal ) ;
    fInitialInferredEtDiscrepency = fInitialDacZerespDiscrepency * fLookupSlope ;

    // Skip adjusing if we already are close enough
    if ( fInitialInferredEtDiscrepency <= KfInitialEtCloseOk ) 
    {
        SHOW_LINE ( (CString) f_osTrgTwr_Coord( _eTTChanType )
                             + " inferred Output Et already within ~"
                             + F_to_A ( fInitialInferredEtDiscrepency, 1 )
                             + " of Zeresp" ) ;
        // copy the values to the call arguments
        if ( _puwDacValue != NULL ) *_puwDacValue = uwInitialDacValue ;
        if ( _pfHistoAve  != NULL ) *_pfHistoAve  = fInitialAverage ;
        if ( _pfHistoDev  != NULL ) *_pfHistoDev  = fInitialDeviation ;
        // no change for _piwDacAdjust = 0 

        //Remember the new (same as old) operational Control Value
        m_ulPedestalControlDAC[_eTTChanType]    = uwInitialDacValue ;

        goto Done ;
    }

    //Verify initial sanity 
    if ( fInitialDacZerespDiscrepency > KfInitialDacZerespWarning )
    ERROR_LINE ( (CString) f_osTrgTwr_Coord( _eTTChanType )
                         + " Warning: Initial Raw ADC Avr= "
                         + F_to_A ( fInitialAverage, 2 )
                         + " is off by more than "
                         + F_to_A ( KfInitialDacZerespWarning, 2 ) ) ;

    ///////////////////////////////////////////////////////////////////
    // Now compute a corrected DAC Control value to bring zeresp to the target
    ///////////////////////////////////////////////////////////////////

    // compute the adjusted DAC Value using the typical DAC to ADC ratio
    // remember: increasing the DAC value will decrease the zeresp
    uwAdjustedDacValue = uwInitialDacValue
                       + (unsigned short) ( ( fInitialAverage - uwZerespGoal ) * KfDacToAdcRatioTypical + 0.5 ) ;

    //Verify we computed a valid control value
    if ( ( uwAdjustedDacValue >= KiDacMaxValue )
      || ( uwAdjustedDacValue <= KiDacMinValue ) )
    {
        ERROR_LINE ( (CString) f_osTrgTwr_Coord( _eTTChanType )
                             + " Aborting: Illegal DAC Value ="
                             + I_to_A ( uwAdjustedDacValue)
                             + " for Computed Adjusting Pedestal" ) ;
        goto Failed ;
    }

    SET_PEDESTAL_DAC ( uwAdjustedDacValue ) ;

    COLLECT_HISTOGRAM( uwAdjustedDacValue, fAdjustedAverage, fAdjustedDeviation ) ;

    //Compute Adjusted discrepency from target 
    fAdjustedDacZerespDiscrepency  = fabs( fAdjustedAverage - (double) uwZerespGoal ) ;

    // We say we succeeded if the Adjusted average is better than the inital value
    if ( fAdjustedDacZerespDiscrepency < fInitialDacZerespDiscrepency )
    {
        // compute inferred effect on Output Et
        fAdjustedInferredEtShift       = (fAdjustedAverage-fInitialAverage) * fLookupSlope ;
        fAdjustedInferredEtDiscrepency = fAdjustedDacZerespDiscrepency * fLookupSlope ;

        // we will force a plus sign for positive values
        CString osForcedPlusIfNeededDAC = "" ;
        CString osForcedPlusIfNeededEt  = "" ;
        if ( (uwAdjustedDacValue-uwInitialDacValue) > 0   ) osForcedPlusIfNeededDAC = "+" ;
        if (  fAdjustedInferredEtShift              > 0.0 ) osForcedPlusIfNeededEt  = "+" ;
        
        SHOW_LINE ( (CString) f_osTrgTwr_Coord( _eTTChanType )
                             + " Adjusted DAC by "
                             + osForcedPlusIfNeededDAC
                             + I_to_A ( (uwAdjustedDacValue-uwInitialDacValue), 2 )
                             + " -> Et by ~"
                             + osForcedPlusIfNeededEt
                             + F_to_A ( fAdjustedInferredEtShift, 1 ) 
                             + " now within ~"
                             + F_to_A ( fAdjustedInferredEtDiscrepency, 1 )
                             + " of Zeresp" ) ;

        // copy the values to the call arguments
        if ( _puwDacValue  != NULL ) *_puwDacValue  = uwAdjustedDacValue ;
        if ( _pfHistoAve   != NULL ) *_pfHistoAve   = fAdjustedAverage ;
        if ( _pfHistoDev   != NULL ) *_pfHistoDev   = fAdjustedDeviation ;
        if ( _piwDacAdjust != NULL ) *_piwDacAdjust = uwAdjustedDacValue - uwInitialDacValue ;

        //Remember the new operational Control Value
        m_ulPedestalControlDAC[_eTTChanType]    = uwAdjustedDacValue ;

        // Verify the Histogram average is now within reason 
        if ( fAdjustedDacZerespDiscrepency > KfAdjustedDacCloseOk ) 
        {
            ERROR_LINE ( (CString) f_osTrgTwr_Coord( _eTTChanType )
                                 + " ADC still off with Adjusted DAC= "
                                 + I_to_A ( uwAdjustedDacValue, 4 )
                                 + " Avr= "
                                 + F_to_A ( fAdjustedAverage, 2 )
                                 + " Dev= "
                                 + F_to_A ( fAdjustedDeviation, 2 ) ) ;
            eReturnStatus = Error ; // Not fatal, but will generate an error in the TTI file to notice the problem
        }

        goto Done ;
    }
    else // A correction that makes things worse is not considered a success
    {
        ERROR_LINE ( (CString) f_osTrgTwr_Coord( _eTTChanType )
                             + " Worse when Adjust by "
                             + I_to_A ( (uwAdjustedDacValue-uwInitialDacValue), 2 )
                             + " -> DAC= "
                             + I_to_A ( uwAdjustedDacValue, 4 )
                             + " Avr= "
                             + F_to_A ( fAdjustedAverage, 2 )
                             + " Dev= "
                             + F_to_A ( fAdjustedDeviation, 2 ) ) ;

        //Leave with old value of DAC 
        SET_PEDESTAL_DAC ( uwInitialDacValue ) ;

        ERROR_LINE ( (CString) "_Reverted to Initial "
                             + " DAC= "
                             + I_to_A ( uwInitialDacValue, 4 )
                             + " Avr= "
                             + F_to_A ( fInitialAverage, 2 )
                             + " Dev= "
                             + F_to_A ( fInitialDeviation, 2 ) ) ;

        // copy the values to the call arguments
        if ( _puwDacValue != NULL ) *_puwDacValue = uwInitialDacValue ;
        if ( _pfHistoAve  != NULL ) *_pfHistoAve  = fInitialAverage ;
        if ( _pfHistoDev  != NULL ) *_pfHistoDev  = fInitialDeviation ;
        // no change for _piwDacAdjust = 0 

        //Remember the new (same as old) operational Control Value
        m_ulPedestalControlDAC[_eTTChanType]    = uwInitialDacValue ;

        goto Failed ;
    }


DAC_Write_IO_Error:

    //DAC write IO failure
    ERROR_LINE ( (CString) f_osTrgTwr_Coord( _eTTChanType )
                         + " Aborting: Write DAC Error" ) ;
    goto Failed ;

Histogram_IO_Error:

    //Histogram IO Error
    ERROR_LINE ( (CString) f_osTrgTwr_Coord( _eTTChanType )
                         + " Aborting: Histograming Error" ) ;
    goto Failed ;

Failed:

    eReturnStatus = Error ;

Done:

Immediate_Return:

    return eReturnStatus ;
}
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess    CL1Cal_TrgTwr::Revert_Ped     ( ETT_Chan_Type       _eTTChanType,
                                            TuwRegWord*         _puwDacValue,
                                            bool                _lShowDetails )
{

    //\\//

    ESuccess        eReturnStatus = Ok ;

    CString         oMsg ;

    unsigned short  uwRevertedDacValue ;

    //If we don't get anywhere, we will return zeroes and an error status
    if ( _puwDacValue != NULL ) *_puwDacValue = 0 ;

    //we can't continue if we didn't find a ADF card in the constructor
    if ( m_poMotherAdf == NULL )
    {
        oConsole.ErrorLine ( " No associated ADF Card; Revert_Ped Aborting" ) ;
        goto Immediate_Return ;
    }

    //Retrieve the previous Control Value
    uwRevertedDacValue = m_ulPrevPedCtrlDAC[_eTTChanType] ;

    //the current one is now going to be the previous
    m_ulPrevPedCtrlDAC[_eTTChanType] = m_ulPedestalControlDAC[_eTTChanType] ;

    // set the new value
    SET_PEDESTAL_DAC ( uwRevertedDacValue ) ;

    // remember it
    m_ulPedestalControlDAC[_eTTChanType] = uwRevertedDacValue ;

    // prepare return value
    if ( _puwDacValue != NULL ) *_puwDacValue = uwRevertedDacValue ;

    goto Done ;

DAC_Write_IO_Error:

    //DAC write IO failure
    oConsole.ErrorLine ( (CString) f_osTrgTwr_Coord( _eTTChanType )
                         + " Aborting: Write DAC Error" ) ;
    goto Failed ;

Failed:

    eReturnStatus = Error ;

Done:

Immediate_Return:

    return eReturnStatus ;
}
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess    CL1Cal_TrgTwr::ADC_Capture        ( ETT_Chan_Type       _eTTChanType,
                                                unsigned int        _uiThreshold,
                                                unsigned int        _uiMaxTry,
                                                TAdcArrayT7         _auwAdcArray,
                                                CResultFile*        _poCaptureFile )
{

    if ( m_poMotherAdf == NULL )
    {
        oConsole.InternalErrorLine ( (CString) "CL1Cal_TrgTwr::ADC_Capture: Missing ADF pointer" ) ;
        return Error ;
    }
    else
        return   m_poMotherAdf->m_apoT7Fpga[m_ulFpgaThisRelEta]
                              ->ADC_Capture    ( _eTTChanType,
                                                  m_ulRelEtaOnFpga,
                                                  m_ulRelPhi,
                                                 _uiThreshold,
                                                 _uiMaxTry,
                                                 _auwAdcArray,
                                                 _poCaptureFile ) ;
}
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
inline  double fSquare ( double _fValue ) { return _fValue * _fValue ; } ;
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess    CL1Cal_TrgTwr::Profile_BLS    ( ETT_Chan_Type       _eTTChanType,
                                            double*             _pAveOfAveAll,
                                            double*             _pDevOfAveAll,
                                            double*             _pAveOfDevAll,
                                            double*             _pDevOfDevAll,
                                            double*             _pAveOfAveLiveX,
                                            double*             _pDevOfAveLiveX,
                                            double*             _pAveOfDevLiveX,
                                            double*             _pDevOfDevLiveX,
                                            bool                _lShowDetails,
                                            CResultFile*        _poHistoFile )
{
    ESuccess        eReturnStatus = Ok ;

    CString         oMsg ;

    unsigned long   ulSampleSize = 1000 ;
    double          fAverage [KiTotTicksInTurn+1][KiTotSamplesPerTick] ; // (note Tick Number goes 1:159 with 0 unused)
    double          fDeviation [KiTotTicksInTurn+1][KiTotSamplesPerTick] ;
    double          fAveOfAveAll ;
    double          fDevOfAveAll ;
    double          fAveOfDevAll ;
    double          fDevOfDevAll ;
    double          fAveOfAveLiveX ;
    double          fDevOfAveLiveX ;
    double          fAveOfDevLiveX ;
    double          fDevOfDevLiveX ;

    unsigned long   ulTickNum ;
    unsigned long   ulSubTickNum ;
    unsigned long   ulLiveXNum ;

    //\\//

    //Advertize where we are
    WRITE_LINE ( " " ) ;
    WRITE_LINE ( (CString) "Profiling ADF BLS Input for " + f_osTrgTwr_Coord( _eTTChanType ) ) ;

    //If we don't get anywhere, we will return zeroes and an error status
    //But if we get close we will still return some values and an error status
    if ( _pAveOfAveAll   != NULL ) *_pAveOfAveAll   = 0.0 ;
    if ( _pDevOfAveAll   != NULL ) *_pDevOfAveAll   = 0.0 ;
    if ( _pAveOfDevAll   != NULL ) *_pAveOfDevAll   = 0.0 ;
    if ( _pDevOfDevAll   != NULL ) *_pDevOfDevAll   = 0.0 ;
    if ( _pAveOfAveLiveX != NULL ) *_pAveOfAveLiveX = 0.0 ;
    if ( _pDevOfAveLiveX != NULL ) *_pDevOfAveLiveX = 0.0 ;
    if ( _pAveOfDevLiveX != NULL ) *_pAveOfDevLiveX = 0.0 ;
    if ( _pDevOfDevLiveX != NULL ) *_pDevOfDevLiveX = 0.0 ;

    //we can't continue if we didn't find a ADF card in the constructor
    if ( m_poMotherAdf == NULL )
    {
        ERROR_LINE ( " No associated ADF Card; Profile ADF BLS Input Aborting" ) ;
        goto Immediate_Return ;
    }

    //Start the search with the smaller histogram sample size
    ulSampleSize    = 1000 ;

    if ( m_poMotherAdf->m_apoT7Fpga[m_ulFpgaThisRelEta]
                      ->ADC_HistoEachSubTick  ( _eTTChanType,
                                                 m_ulRelEtaOnFpga,
                                                 m_ulRelPhi,
                                                &ga3uwHistogram,
                                                 ulSampleSize )
         != Ok ) goto Histogram_IO_Error ;

    fAveOfAveAll = 0.0 ;
    fAveOfDevAll = 0.0 ;

    for ( ulTickNum    = 1 ; ulTickNum    <= KiTotTicksInTurn    ; ulTickNum ++ ) 
    for ( ulSubTickNum = 0 ; ulSubTickNum < KiTotSamplesPerTick ; ulSubTickNum ++ ) 
    {
        fAverage[ulTickNum][ulSubTickNum]   
               = CAdfRegBase::HistoAverage10bit   ( &ga3uwHistogram[ulTickNum][ulSubTickNum], 
                                                     ulSampleSize );

        fDeviation[ulTickNum][ulSubTickNum]
               = CAdfRegBase::HistoDeviation10bit ( &ga3uwHistogram[ulTickNum][ulSubTickNum], 
                                                     ulSampleSize, 
                                                     fAverage[ulTickNum][ulSubTickNum] ) ;
        
        fAveOfAveAll += fAverage[ulTickNum][ulSubTickNum] ;
        fAveOfDevAll += fDeviation[ulTickNum][ulSubTickNum] ;
    }
    fAveOfAveAll = fAveOfAveAll / (double) (KiTotSamplesPerTick*KiTotTicksInTurn) ;
    fAveOfDevAll = fAveOfDevAll / (double) (KiTotSamplesPerTick*KiTotTicksInTurn) ;

    fDevOfAveAll = 0.0 ;
    fDevOfDevAll = 0.0 ;
    for ( ulTickNum    = 1 ; ulTickNum    <= KiTotTicksInTurn    ; ulTickNum ++ ) 
    for ( ulSubTickNum = 0 ; ulSubTickNum < KiTotSamplesPerTick ; ulSubTickNum ++ ) 
    {
        fDevOfAveAll += fSquare ( fAveOfAveAll - fAverage[ulTickNum][ulSubTickNum] ) ;
        fDevOfDevAll += fSquare ( fAveOfDevAll - fDeviation[ulTickNum][ulSubTickNum] ) ;
    }
    fDevOfAveAll = sqrt ( fDevOfAveAll  / (double) (KiTotSamplesPerTick*KiTotTicksInTurn) ) ;
    fDevOfDevAll = sqrt ( fDevOfDevAll  / (double) (KiTotSamplesPerTick*KiTotTicksInTurn) ) ;



    fAveOfAveLiveX   = 0.0 ;
    fAveOfDevLiveX   = 0.0 ;
    for ( ulLiveXNum = 0 ; ulLiveXNum < kL1CalTcc_Tot_LiveBX ; ulLiveXNum ++ ) 
    {
        ulTickNum = CMonitPoolData::mg_aIndexToTick[ulLiveXNum] ;
        fAveOfAveLiveX   += fAverage[ulTickNum][0] ;
        fAveOfDevLiveX   += fDeviation[ulTickNum][0] ;
    }
    fAveOfAveLiveX = fAveOfAveLiveX / (double) (kL1CalTcc_Tot_LiveBX) ;
    fAveOfDevLiveX = fAveOfDevLiveX / (double) (kL1CalTcc_Tot_LiveBX) ;

    fDevOfAveLiveX = 0.0 ;
    fDevOfDevLiveX = 0.0 ;
    for ( ulLiveXNum = 0 ; ulLiveXNum < kL1CalTcc_Tot_LiveBX ; ulLiveXNum ++ ) 
    {
        ulTickNum = CMonitPoolData::mg_aIndexToTick[ulLiveXNum] ;
        fDevOfAveLiveX += fSquare ( fAveOfAveLiveX - fAverage[ulTickNum][0] ) ;
        fDevOfDevLiveX += fSquare ( fAveOfDevLiveX - fDeviation[ulTickNum][0] ) ;
    }
    fDevOfAveLiveX = sqrt ( fDevOfAveLiveX  / (double) (kL1CalTcc_Tot_LiveBX) ) ;
    fDevOfDevLiveX = sqrt ( fDevOfDevLiveX  / (double) (kL1CalTcc_Tot_LiveBX) ) ;



    // copy the values to the call arguments
    if ( _pAveOfAveAll   != NULL ) *_pAveOfAveAll   = fAveOfAveAll ;
    if ( _pDevOfAveAll   != NULL ) *_pDevOfAveAll   = fDevOfAveAll ;
    if ( _pAveOfDevAll   != NULL ) *_pAveOfDevAll   = fAveOfDevAll ;
    if ( _pDevOfDevAll   != NULL ) *_pDevOfDevAll   = fDevOfDevAll ;
    if ( _pAveOfAveLiveX != NULL ) *_pAveOfAveLiveX = fAveOfAveLiveX ;
    if ( _pDevOfAveLiveX != NULL ) *_pDevOfAveLiveX = fDevOfAveLiveX ;
    if ( _pAveOfDevLiveX != NULL ) *_pAveOfDevLiveX = fAveOfDevLiveX ;
    if ( _pDevOfDevLiveX != NULL ) *_pDevOfDevLiveX = fDevOfDevLiveX ;

    // write summary
    WRITE_LINE ( (CString) "AllRaw : AveOfAve = "
                         + F_to_A ( fAveOfAveAll, 2 )
                         + " DevOfAve = "
                         + F_to_A ( fDevOfAveAll, 2 )
                         + " AveOfDev = "
                         + F_to_A ( fAveOfDevAll, 2 )
                         + " DevOfDev = "
                         + F_to_A ( fDevOfDevAll, 2 )
                         + "\nLive X : AveOfAve = "
                         + F_to_A ( fAveOfAveLiveX, 2 )
                         + " DevOfAve = "
                         + F_to_A ( fDevOfAveLiveX, 2 )
                         + " AveOfDev = "
                         + F_to_A ( fAveOfDevLiveX, 2 )
                         + " DevOfDev = "
                         + F_to_A ( fDevOfDevLiveX, 2 ) ) ;

    WRITE_LINE ( (CString) "Tick  Ave    Dev " ) ;

    for ( ulTickNum    = 1 ; ulTickNum    <= KiTotTicksInTurn    ; ulTickNum ++ ) 
    for ( ulSubTickNum = 0 ; ulSubTickNum < KiTotSamplesPerTick ; ulSubTickNum ++ ) 
    {
        WRITE_LINE (   I_to_A ( ulTickNum, 3 )
                     + "-"
                     + I_to_A ( ulSubTickNum )
                     + " "
                     + F_to_A ( fAverage[ulTickNum][ulSubTickNum], 2, 5 )
                     + " "
                     + F_to_A ( fDeviation[ulTickNum][ulSubTickNum], 2, 5 ) ) ;
    }

    for ( ulLiveXNum = 0 ; ulLiveXNum < kL1CalTcc_Tot_LiveBX ; ulLiveXNum ++ ) 
    {
        ulTickNum = CMonitPoolData::mg_aIndexToTick[ulLiveXNum] ;
        WRITE_LINE ( (CString) " Live X# "
                             + I_to_A ( ulLiveXNum, 2 )
                             + " = "
                             + I_to_A ( ulTickNum, 3 )
                             + "-0 -> "
                             + DumpHistogram  ( ga3uwHistogram[ulTickNum][0],
                                                KiSizeOfTAdcHisto,
                                               &fAverage[ulTickNum][0],
                                               &fDeviation[ulTickNum][0] ) ) ;
    }

    goto Done ;

Histogram_IO_Error:

    eReturnStatus = Error ;

    //Histogram IO Error
    ERROR_LINE ( (CString) f_osTrgTwr_Coord( _eTTChanType )
                         + " Aborting: Histograming Error" ) ;
Done:

Immediate_Return:

    return eReturnStatus ;
}
/*-------------------------------------------------------------------------------*/


/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess    CL1Cal_TrgTwr::Profile_AdfEt  ( ETT_Chan_Type       _eTTChanType,
                                            double*             _pAveOfAveLiveX,
                                            double*             _pDevOfAveLiveX,
                                            double*             _pAveOfDevLiveX,
                                            double*             _pDevOfDevLiveX,
                                            bool                _lShowDetails,
                                            CResultFile*        _poHistoFile )
{
    ESuccess        eReturnStatus = Ok ;

    CString         oMsg ;

    unsigned long   ulSampleSize = 1000 ;
    double          fAverage [kL1CalTcc_Tot_LiveBX] ; 
    double          fDeviation [kL1CalTcc_Tot_LiveBX] ;
    double          fAveOfAveLiveX ;
    double          fDevOfAveLiveX ;
    double          fAveOfDevLiveX ;
    double          fDevOfDevLiveX ;

    unsigned long   ulLiveXNum ;
    unsigned long   ulTickNum ;

    //\\//

    //Advertize where we are
    WRITE_LINE ( " " ) ;
    WRITE_LINE ( (CString) "Profiling ADF Output Et for " + f_osTrgTwr_Coord( _eTTChanType ) ) ;

    //If we don't get anywhere, we will return zeroes and an error status
    //But if we get close we will still return some values and an error status
    if ( _pAveOfAveLiveX != NULL ) *_pAveOfAveLiveX = 0.0 ;
    if ( _pDevOfAveLiveX != NULL ) *_pDevOfAveLiveX = 0.0 ;
    if ( _pAveOfDevLiveX != NULL ) *_pAveOfDevLiveX = 0.0 ;
    if ( _pDevOfDevLiveX != NULL ) *_pDevOfDevLiveX = 0.0 ;

    //we can't continue if we didn't find a ADF card in the constructor
    if ( m_poMotherAdf == NULL )
    {
        ERROR_LINE ( " No associated ADF Card; Profile ADF Output Et Aborting" ) ;
        goto Immediate_Return ;
    }

    //Start the search with the smaller histogram sample size
    ulSampleSize    = 1000 ;

    if ( m_poMotherAdf->m_apoT7Fpga[m_ulFpgaThisRelEta]
                      ->OutputEt_HistoEachLiveX   ( _eTTChanType,
                                                     m_ulRelEtaOnFpga,
                                                     m_ulRelPhi,
                                                    &ga2uwHistogram,
                                                     ulSampleSize )
         != Ok ) goto Histogram_IO_Error ;



    fAveOfAveLiveX   = 0.0 ;
    fAveOfDevLiveX   = 0.0 ;
    for ( ulLiveXNum = 0 ; ulLiveXNum < kL1CalTcc_Tot_LiveBX ; ulLiveXNum ++ ) 
    {
        fAverage[ulLiveXNum]   
               = CAdfRegBase::HistoAverage8bit  ( &ga2uwHistogram[ulLiveXNum], 
                                                   ulSampleSize );

        fDeviation[ulLiveXNum]
               = CAdfRegBase::HistoDeviation8bit( &ga2uwHistogram[ulLiveXNum], 
                                                   ulSampleSize, 
                                                   fAverage[ulLiveXNum] ) ;
        
        fAveOfAveLiveX   += fAverage[ulLiveXNum] ;
        fAveOfDevLiveX   += fDeviation[ulLiveXNum] ;
    }
    fAveOfAveLiveX = fAveOfAveLiveX / (double) (kL1CalTcc_Tot_LiveBX) ;
    fAveOfDevLiveX = fAveOfDevLiveX / (double) (kL1CalTcc_Tot_LiveBX) ;

    fDevOfAveLiveX = 0.0 ;
    fDevOfDevLiveX = 0.0 ;
    for ( ulLiveXNum = 0 ; ulLiveXNum < kL1CalTcc_Tot_LiveBX ; ulLiveXNum ++ ) 
    {
        fDevOfAveLiveX += fSquare ( fAveOfAveLiveX - fAverage[ulLiveXNum] ) ;
        fDevOfDevLiveX += fSquare ( fAveOfDevLiveX - fDeviation[ulLiveXNum] ) ;
    }
    fDevOfAveLiveX = sqrt ( fDevOfAveLiveX  / (double) (kL1CalTcc_Tot_LiveBX) ) ;
    fDevOfDevLiveX = sqrt ( fDevOfDevLiveX  / (double) (kL1CalTcc_Tot_LiveBX) ) ;



    // copy the values to the call arguments
    if ( _pAveOfAveLiveX != NULL ) *_pAveOfAveLiveX = fAveOfAveLiveX ;
    if ( _pDevOfAveLiveX != NULL ) *_pDevOfAveLiveX = fDevOfAveLiveX ;
    if ( _pAveOfDevLiveX != NULL ) *_pAveOfDevLiveX = fAveOfDevLiveX ;
    if ( _pDevOfDevLiveX != NULL ) *_pDevOfDevLiveX = fDevOfDevLiveX ;

    // write summary
    WRITE_LINE ( (CString) "Output Et Live X : AveOfAve = "
                         + F_to_A ( fAveOfAveLiveX, 2 )
                         + " DevOfAve = "
                         + F_to_A ( fDevOfAveLiveX, 2 )
                         + " AveOfDev = "
                         + F_to_A ( fAveOfDevLiveX, 2 )
                         + " DevOfDev = "
                         + F_to_A ( fDevOfDevLiveX, 2 ) ) ;

    WRITE_LINE ( (CString) "Tick  Ave    Dev " ) ;

    for ( ulLiveXNum = 0 ; ulLiveXNum < kL1CalTcc_Tot_LiveBX ; ulLiveXNum ++ ) 
    {
        ulTickNum = CMonitPoolData::mg_aIndexToTick[ulLiveXNum] ;
        WRITE_LINE ( (CString) " Live X# "
                             + I_to_A ( ulLiveXNum, 2 )
                             + " = "
                             + I_to_A ( ulTickNum, 3 )

                             + " -> "
                             + DumpHistogram  ( ga2uwHistogram[ulLiveXNum],
                                                KiSizeOfTAdfEtHisto,
                                               &fAverage[ulLiveXNum],
                                               &fDeviation[ulLiveXNum] ) ) ;
    }

    goto Done ;

Histogram_IO_Error:

    eReturnStatus = Error ;

    //Histogram IO Error
    ERROR_LINE ( (CString) f_osTrgTwr_Coord( _eTTChanType )
                         + " Aborting: Histograming Error" ) ;
Done:

Immediate_Return:

    return eReturnStatus ;
}
/*-------------------------------------------------------------------------------*/


/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess    CL1Cal_TrgTwr::f_okSetSimuRawAdc      ( ETT_Chan_Type           _eTTChanType,
                                                    const unsigned short*   _puwSimuRawAdc,
                                                    const unsigned long*    _pulTickNum,
                                                    const unsigned long*    _pulSubTickNum )
{
    ESuccess        eReturnStatus = Ok ;

    unsigned long   ulTickNum, ulTickNumLower, ulTickNumUpper ;
    unsigned long   ulSubTickNum, ulSubTickNumLower, ulSubTickNumUpper ;
    unsigned short  uwSimuRawAdc ;

    //\\//

    // See if a specific SubTick was specified
    if ( _pulSubTickNum != NULL )
    {
        ulSubTickNumLower = *_pulSubTickNum ;
        ulSubTickNumUpper = *_pulSubTickNum ;
    }
    else // we will be writing all SubTicks
    {
        ulSubTickNumLower = 0 ;
        ulSubTickNumUpper = KiTotSamplesPerTick - 1  ;
    }

    // See if a specific Tick was specified
    if ( _pulTickNum != NULL )
    {
        ulTickNumLower = *_pulTickNum ;
        ulTickNumUpper = *_pulTickNum ;
    }
    else // we will be writing all Ticks
    {
        ulTickNumLower = 1 ;
        ulTickNumUpper = KiTotTicksInTurn ;
    }

    // Verify there is an associated ADF Card
    if ( m_poMotherAdf == NULL )
    {
        oConsole.InternalErrorLine ( (CString) "CL1Cal_TrgTwr::f_okSetSimuRawAdc: Missing ADF pointer" ) ;
        return Error ;
    }
    else
    {// loop over specified tick/subtick or all ticks/subticks
        for ( ulTickNum     = ulTickNumLower ;
              ulTickNum    <= ulTickNumUpper ;
              ulTickNum    ++ )
        for ( ulSubTickNum  = ulSubTickNumLower ;
              ulSubTickNum <= ulSubTickNumUpper ;
              ulSubTickNum ++ )
        {
            // retrieve the simulated value if not given
            if ( _puwSimuRawAdc == NULL )
            {
                uwSimuRawAdc = m_uwSimuRawAdc[_eTTChanType][ulTickNum][ulSubTickNum] ;
            }
            else
            {// use given value and remember it
                uwSimuRawAdc = *_puwSimuRawAdc ;
                m_uwSimuRawAdc[_eTTChanType][ulTickNum][ulSubTickNum] = uwSimuRawAdc ;
            }

            // Now actually do it
            if ( m_poMotherAdf->m_apoT7Fpga[m_ulFpgaThisRelEta]
                              ->SetSimuRawAdc      ( _eTTChanType,
                                                      m_ulRelEtaOnFpga,
                                                      m_ulRelPhi,
                                                      uwSimuRawAdc,
                                                      ulTickNum,
                                                      ulSubTickNum ) != Ok )
            { // abort on error
                eReturnStatus = Error ;
                goto Abort ;
            }
        }
    }

Abort :
    return eReturnStatus ;
}
/*-------------------------------------------------------------------------------*/



/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess    CL1Cal_TrgTwr::f_okSetSimuOutputEt    ( ETT_Chan_Type           _eTTChanType,
                                                    const unsigned char*    _pubSimuOutputEt,
                                                    const unsigned long*    _pulTickNum  )
{
    ESuccess        eReturnStatus = Ok ;

    unsigned long   ulTickNum, ulTickNumLower, ulTickNumUpper ;
    unsigned char   ubSimuOutputEt;

    //\\//

    // See if a specific Tick was specified
    if ( _pulTickNum == NULL )
    {
        ulTickNumLower = 1 ;
        ulTickNumUpper = KiTotTicksInTurn ;
    }
    else
    {
        ulTickNumLower = *_pulTickNum ;
        ulTickNumUpper = *_pulTickNum ;
    }

    if ( m_poMotherAdf == NULL )
    {
        oConsole.InternalErrorLine ( (CString) "CL1Cal_TrgTwr::f_okSetSimuOutputEt: Missing ADF pointer" ) ;
        return Error ;
    }
    else
    {// loop over specified tick/subtick or all ticks/subticks
        for ( ulTickNum     = ulTickNumLower ;
              ulTickNum    <= ulTickNumUpper ;
              ulTickNum    ++ )
        {
            // retrieve the simulated value if not given
            if ( _pubSimuOutputEt == NULL )
            {
                ubSimuOutputEt = m_ubSimuOutputEt[_eTTChanType][ulTickNum] ;
            }
            else
            {// use given value and remember it
                ubSimuOutputEt = *_pubSimuOutputEt ;
                m_ubSimuOutputEt[_eTTChanType][ulTickNum] = ubSimuOutputEt ;
            }

            // Now actually do it
            if ( m_poMotherAdf->m_apoT7Fpga[m_ulFpgaThisRelEta]
                              ->SetSimuOutputEt    ( _eTTChanType,
                                                      m_ulRelEtaOnFpga,
                                                      m_ulRelPhi,
                                                      ubSimuOutputEt,
                                                      ulTickNum ) != Ok )
            { // abort on error
                eReturnStatus = Error ;
                goto Abort ;
            }
        }
    }

Abort :
    return eReturnStatus ;
}
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess    CL1Cal_TrgTwr::f_okReadMonitOutputEt  ( unsigned char*          _pubMonitOutputEtEm,
                                                    unsigned char*          _pubMonitOutputEtHd,
                                                    const unsigned long*    _pulTickNum )
{

    ESuccess        eReturnStatus = Ok ;

    unsigned long   ulTickNum ;

    unsigned char*  pubMonitOutputEtEm ;
    unsigned char*  pubMonitOutputEtHd ;

    //\\//

    // Make a copy of the target (base) pointers
    pubMonitOutputEtEm = _pubMonitOutputEtEm ;
    pubMonitOutputEtHd = _pubMonitOutputEtHd ;

    if ( m_poMotherAdf == NULL )
    {// last chance to verify this is a fully defined trigger tower
        oConsole.InternalErrorLine ( (CString) "CL1Cal_TrgTwr::f_okReadMonitOutputEt: Missing ADF pointer" ) ;
        return Error ;
    }
    else
    {
        // If no specific Tick was specified, we collect all ticks
        // using the given pointer as the base of an array
        if ( _pulTickNum == NULL )
        {
            // loop over specified all ticks/subticks
            for ( ulTickNum     = 1 ;
                  ulTickNum    <= KiTotTicksInTurn ;
                  ulTickNum    ++ )
            {
                // prepare the location for this time slice (while preserving null pointers)
                if ( _pubMonitOutputEtEm != NULL ) pubMonitOutputEtEm = _pubMonitOutputEtEm + ulTickNum ;
                if ( _pubMonitOutputEtHd != NULL ) pubMonitOutputEtHd = _pubMonitOutputEtHd + ulTickNum ;

                // Now actually do it
                if ( m_poMotherAdf->m_apoT7Fpga[m_ulFpgaThisRelEta]
                                  ->ReadMonitOutputEt   ( m_ulRelEtaOnFpga,
                                                          m_ulRelPhi,
                                                          pubMonitOutputEtEm,
                                                          pubMonitOutputEtHd,
                                                          ulTickNum )
                       != Ok )
                { // abort on error
                    eReturnStatus = Error ;
                    goto Abort ;
                }
            }
        }
        else
        {
            ulTickNum = *_pulTickNum ;

            if ( m_poMotherAdf->m_apoT7Fpga[m_ulFpgaThisRelEta]
                              ->ReadMonitOutputEt   ( m_ulRelEtaOnFpga,
                                                      m_ulRelPhi,
                                                      pubMonitOutputEtEm,
                                                      pubMonitOutputEtHd,
                                                      ulTickNum )
                   != Ok )
                eReturnStatus = Error ;
        }

    }

Abort :
    return eReturnStatus ;
}
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess     CL1Cal_TrgTwr::f_okOutputConstant    ( ETT_Chan_Type           _eTTChanType,
                                                    unsigned char           _ubConstantData )
{
    //skip checking input parameters: this will be done in the called routine

    // verify this is a fully defined trigger tower
    if ( m_poMotherAdf == NULL )
    {
        oConsole.InternalErrorLine ( (CString) "CL1Cal_TrgTwr::f_okOutputConstant: Missing ADF pointer" ) ;
        return Error ;
    }
    else
        return m_poMotherAdf->m_apoT7Fpga[m_ulFpgaThisRelEta]
                            ->SetOutputConstant      ( _eTTChanType,
                                                        m_ulRelEtaOnFpga,
                                                        m_ulRelPhi,
                                                       _ubConstantData ) ;
}
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess     CL1Cal_TrgTwr::f_okRawAdcThresh      ( ETT_Chan_Type           _eTTChanType,
                                                    unsigned char           _ubThreshold )
{
    //skip checking input parameters: this will be done in the called routine

    // verify this is a fully defined trigger tower
    if ( m_poMotherAdf == NULL )
    {
        oConsole.InternalErrorLine ( (CString) "CL1Cal_TrgTwr::f_okRawAdcThresh: Missing ADF pointer" ) ;
        return Error ;
    }
    else
        return m_poMotherAdf->m_apoT7Fpga[m_ulFpgaThisRelEta]
                            ->SetRawAdcThresh        ( _eTTChanType,
                                                        m_ulRelEtaOnFpga,
                                                        m_ulRelPhi,
                                                       _ubThreshold ) ;
}
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess     CL1Cal_TrgTwr::f_okRawAdc_MuxMode    ( ETT_Chan_Type           _eTTChanType,
                                                    ERawAdc_MuxMode_Type    _eRawAdc_MuxMode )
{
    //skip checking input parameters: this will be done in the called routine

    // verify this is a fully defined trigger tower
    if ( m_poMotherAdf == NULL )
    {
        oConsole.InternalErrorLine ( (CString) "CL1Cal_TrgTwr::f_okRawAdc_MuxMode: Missing ADF pointer" ) ;
        return Error ;
    }
    else
        return m_poMotherAdf->m_apoT7Fpga[m_ulFpgaThisRelEta]
                            ->f_okRawAdc_MuxMode     ( _eTTChanType,
                                                        m_ulRelEtaOnFpga,
                                                        m_ulRelPhi,
                                                       _eRawAdc_MuxMode ) ;
}
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess     CL1Cal_TrgTwr::f_okRawAdc_MemMode    ( ETT_Chan_Type           _eTTChanType,
                                                    ERawAdc_MemMode_Type    _eRawAdc_MemMode )
{
    ESuccess        eReturnStatus = Ok ;

    //\\//

    //skip checking input parameters: this will be done in the called routine

    // verify this is a fully defined trigger tower
    if ( m_poMotherAdf == NULL )
    {
        oConsole.InternalErrorLine ( (CString) "CL1Cal_TrgTwr::f_okRawAdc_MemMode: Missing ADF pointer" ) ;
        return Error ;
    }
    else
    {
        if ( m_poMotherAdf->m_apoT7Fpga[m_ulFpgaThisRelEta]
                          ->f_okRawAdc_MemMode   ( _eTTChanType,
                                                    m_ulRelEtaOnFpga,
                                                    m_ulRelPhi,
                                                   _eRawAdc_MemMode ) != Ok )
               eReturnStatus = Error ;

        // When we are switching to simulated turn data,
        // we also need to reload the values remembered e.g. from a previous TTI file
        if ( _eRawAdc_MemMode == eRawAdc_MemMode_PlaySimuTurn )
            if ( f_okSetSimuRawAdc ( _eTTChanType ) != Ok )
               eReturnStatus = Error ;
    }

    return eReturnStatus ;
}
/*-------------------------------------------------------------------------------*/


/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess     CL1Cal_TrgTwr::f_okAdfOut_MuxMode    ( ETT_Chan_Type           _eTTChanType,
                                                    EAdfOut_MuxMode_Type    _eAdfOut_MuxMode )
{
    //skip checking input parameters: this will be done in the called routine

    // verify this is a fully defined trigger tower
    if ( m_poMotherAdf == NULL )
    {
        oConsole.InternalErrorLine ( (CString) "CL1Cal_TrgTwr::f_okAdfOut_MuxMode: Missing ADF pointer" ) ;
        return Error ;
    }
    else
        return m_poMotherAdf->m_apoT7Fpga[m_ulFpgaThisRelEta]
                            ->f_okAdfOut_MuxMode     ( _eTTChanType,
                                                        m_ulRelEtaOnFpga,
                                                        m_ulRelPhi,
                                                       _eAdfOut_MuxMode ) ;
}
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess     CL1Cal_TrgTwr::f_okAdfOut_MemMode    ( ETT_Chan_Type           _eTTChanType,
                                                    EAdfOut_MemMode_Type    _eAdfOut_MemMode )
{
    ESuccess        eReturnStatus = Ok ;

    //\\//

    //skip checking input parameters: this will be done in the called routine

    // verify this is a fully defined trigger tower
    if ( m_poMotherAdf == NULL )
    {
        oConsole.InternalErrorLine ( (CString) "CL1Cal_TrgTwr::f_okAdfOut_MemMode: Missing ADF pointer" ) ;
        return Error ;
    }
    else
    {
        if ( m_poMotherAdf->m_apoT7Fpga[m_ulFpgaThisRelEta]
                          ->f_okAdfOut_MemMode       ( _eTTChanType,
                                                        m_ulRelEtaOnFpga,
                                                        m_ulRelPhi,
                                                       _eAdfOut_MemMode ) != Ok )
           eReturnStatus = Error ;

        // When we are switching to simulated turn data,
        // we also need to reload the values remembered e.g. from a previous TTI file
        if ( _eAdfOut_MemMode == eAdfOut_MemMode_PlaySimuTurn )
            if ( f_okSetSimuOutputEt ( _eTTChanType ) != Ok )
               eReturnStatus = Error ;
    }

    return eReturnStatus ;
}
/*-------------------------------------------------------------------------------*/

