/*-------------------------------------------------------------------------------*/
  /*
     MessageCoorDispatcher.cpp
     7-Oct-99
     28-Oct-99 Coor Command Parsing split onto separate files
     22-Sep-2005 Restructred for L1Cal_IIb_TCC
  */
/*-------------------------------------------------------------------------------*/
#include "L1Cal_IIb_TCC/StdAfx.h"

#include "L1Cal_IIb_TCC/MessageCommand.h"
#include                            "L1Cal_IIb_TCC/MessageCoorDispatcher.h"

#include "L1Cal_IIb_TCC/MessageKeywordsCoorGeneric.h"
#include "L1Cal_IIb_TCC/MessageKeywordsCoorL1Cal.h"

#include "L1Cal_IIb_TCC/HandleConsole.h"
#include "L1Cal_IIb_TCC/UtilsStrings.h"

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/


/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ECoorMessageStatus CMessageCoorDispatcher::MessageDispatch            ( CMessageCommandLine*            _poCoorCommandLine,
                                                                        CMessageCoorExecuter*           _pxCoorExecute ) 
{ 
    ECoorMessageStatus  eReturnStatus = Reply_Ok ;

    TBinomialEntry < CMessageCommand, unsigned int >*  poCoorCommandEntry ;

    CMessageCommand*    poCoorCommand ;

    //\\//

    //Get at the first command
    poCoorCommandEntry = _poCoorCommandLine->m_lCommandList.FirstEntry( ) ;

    if ( poCoorCommandEntry != NULL ) 
    {
        poCoorCommand = poCoorCommandEntry ->GetAtKey( ) ;

        if ( poCoorCommand->m_uiCoorCmdCode <= Ki_CoorCmd_MaxID_Generic )
            eReturnStatus = MessageDispatch_CoorL1FW ( _poCoorCommandLine, _pxCoorExecute ) ;
        else
            eReturnStatus = MessageDispatch_CoorL1Cal ( _poCoorCommandLine, _pxCoorExecute ) ;
    }

    return eReturnStatus ;
}


/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ECoorMessageStatus CMessageCoorDispatcher::MessageDispatch_CoorL1FW   ( CMessageCommandLine*            _poCoorCommandLine,
                                                                        CMessageCoorExecuter*           _pxCoorExecute ) 
{ 
    ECoorMessageStatus  eReturnStatus = Reply_Ok ;
    ECoorMessageStatus  eRememberStatus = Reply_Ok ;

    char                sReplyText[KiMaxLengthReplyTextToCoor] ;

    TBinomialEntry < CMessageCommand, unsigned int >*  poCoorCommandEntry ;

    CMessageCommand*    poCoorCommand ;

    //\\//

    //Get at the first command
    // find the next command in the list
    poCoorCommandEntry = _poCoorCommandLine->m_lCommandList.FirstEntry( ) ;
    if ( poCoorCommandEntry != NULL ) 
        poCoorCommand = poCoorCommandEntry ->GetAtKey( ) ;
    else 
        poCoorCommand = NULL ;

    while ( poCoorCommand != NULL )
    {
        switch ( poCoorCommand->m_uiCoorCmdCode )
        {

            /////////////////////////////////////////////////////////////////////////////
            // start with comment = empty command line
            /////////////////////////////////////////////////////////////////////////////

            case Ki_CoorSubCmd_Comment                       :
                break ;

            /////////////////////////////////////////////////////////////////////
            /// Configure Command ///////////////////////////////////////////////
            /////////////////////////////////////////////////////////////////////
            case Ki_CoorCmd_Configure_FPGAs :
            {
                eReturnStatus = _pxCoorExecute->Configure_FPGAs ( _poCoorCommandLine ) ;
            } break ; // case Ki_CoorCmd_Configure_FPGAs

            /////////////////////////////////////////////////////////////////////
            /// Initialize Command //////////////////////////////////////////////
            /////////////////////////////////////////////////////////////////////
            case Ki_CoorCmd_L1Cal_Initialize :
            {
                eReturnStatus = _pxCoorExecute->L1Cal_Initialize ( _poCoorCommandLine ) ;
            } break ; // case Ki_CoorCmd_L1Cal_Initialize 

            /////////////////////////////////////////////////////////////////////
            /// Iterative_DeSkew Command //////////////////////////////////////////////
            /////////////////////////////////////////////////////////////////////
            case Ki_CoorCmd_Iterative_DeSkew :
            {
                eReturnStatus = _pxCoorExecute->Iterative_DeSkew ( _poCoorCommandLine ) ;
            } break ; // case Ki_CoorCmd_Iterative_DeSkew 

            /////////////////////////////////////////////////////////////////////
            /// Start/Stop Run Commands  ////////////////////////////////////////
            /////////////////////////////////////////////////////////////////////
            case Ki_CoorCmd_Start_run :
            {
                eReturnStatus = _pxCoorExecute->StartRun        ( poCoorCommand->m_uiValue,
                                                                 &poCoorCommand->m_aubItemPolarity[0], 
                                                                  _poCoorCommandLine ) ;
            } break ; // case Ki_CoorCmd_Start_run

            case Ki_CoorCmd_Stop_run :
            {
                eReturnStatus = _pxCoorExecute->StopRun         ( poCoorCommand->m_uiValue,
                                                                  _poCoorCommandLine ) ;
            } break ; // case Ki_CoorCmd_Stop_run

            /////////////////////////////////////////////////////////////////////
            /// Commands for system using batch command processing //////////////
            /////////////////////////////////////////////////////////////////////
            case Ki_CoorCmd_Configure :
            {
                eReturnStatus = Reply_Ok ;  // Nothing to do but answer needed
            } break ; // case Ki_CoorCmd_Configure 

            case Ki_CoorCmd_Begin_block :
            case Ki_CoorCmd_End_block :
            case Ki_CoorCmd_Abort :
            {
                eReturnStatus = Reply_None ;  // Nothing to do and no answer needed
            } break ; // case Ki_CoorCmd_Abort

            /////////////////////////////////////////////////////////////////////
            /// Pause/Resume Run Commands  //////////////////////////////////////
            /////////////////////////////////////////////////////////////////////
            case Ki_CoorCmd_Pause_Run :
            {
                eReturnStatus = _pxCoorExecute->PauseRun        ( poCoorCommand->m_uiValue,
                                                                 _poCoorCommandLine ) ;
            } break ; // case Ki_CoorCmd_PauseRun

            case Ki_CoorCmd_Resume_Run :
            {
                eReturnStatus = _pxCoorExecute->ResumeRun       ( poCoorCommand->m_uiValue,
                                                                 _poCoorCommandLine ) ;
            } break ; // case Ki_CoorCmd_ResumeRun

            /////////////////////////////////////////////////////////////////////
            /// Begin/End Store Command /////////////////////////////////////////
            /////////////////////////////////////////////////////////////////////
            case Ki_CoorCmd_Begin_store :
            {
                eReturnStatus = _pxCoorExecute->BeginStore      ( poCoorCommand->m_uiValue,
                                                                  _poCoorCommandLine ) ;
            } break ; // case Ki_CoorCmd_Begin_store

            case Ki_CoorCmd_End_store :
            {
                eReturnStatus = _pxCoorExecute->EndStore        ( poCoorCommand->m_uiValue,
                                                                  _poCoorCommandLine ) ;
            } break ; // case Ki_CoorCmd_End_store


            // All commands that have been parsed should have been dispatched here
            // but this catches omissions
            default:
            {
                strcpy ( sReplyText, "<L1FW Command Not implemented ID =" ) ;
                strcat ( sReplyText, I_to_A ( poCoorCommand->m_uiCoorCmdCode ) ) ;
                strcat ( sReplyText, ">" ) ;
                CConsoleAccess::ErrorLine     ( sReplyText ) ;
                _poCoorCommandLine->CopyReplyText ( sReplyText ) ;

                eReturnStatus = Reply_Bad ;
            } ; break ;

        } ;

        // find the next command in the list, if any
        // but remember individual errors to tag the whole message as an error
        if ( eReturnStatus == Reply_Bad ) eRememberStatus = Reply_Bad ;

        poCoorCommandEntry = _poCoorCommandLine->m_lCommandList.NextEntry( poCoorCommandEntry ) ;
        if ( poCoorCommandEntry != NULL ) 
            poCoorCommand = poCoorCommandEntry ->GetAtKey( ) ;
        else 
            poCoorCommand = NULL ;

    } ; 

    if ( eRememberStatus == Reply_Bad ) eReturnStatus = Reply_Bad ;

    return eReturnStatus ;
}

    

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ECoorMessageStatus CMessageCoorDispatcher::MessageDispatch_CoorL1Cal  ( CMessageCommandLine*            _poCoorCommandLine,
                                                                        CMessageCoorExecuter*           _pxCoorExecute ) 
{ 
    ECoorMessageStatus  eReturnStatus = Reply_Ok ;
    ECoorMessageStatus  eRememberStatus = Reply_Ok ;

    char                sReplyText[KiMaxLengthReplyTextToCoor] ;

    TBinomialEntry < CMessageCommand, unsigned int >*  poCoorCommandEntry ;

    CMessageCommand*    poCoorCommand ;

    //\\//

    //Get at the first command
    // find the next command in the list
    poCoorCommandEntry = _poCoorCommandLine->m_lCommandList.FirstEntry( ) ;
    if ( poCoorCommandEntry != NULL ) 
        poCoorCommand = poCoorCommandEntry ->GetAtKey( ) ;
    else 
        poCoorCommand = NULL ;
    
    while ( poCoorCommand != NULL )
    {
        switch ( poCoorCommand->m_uiCoorCmdCode )
        {

            /////////////////////////////////////////////////////////////////////
            /// Exclude Tower Commands //////////////////////////////////////////
            /////////////////////////////////////////////////////////////////////
            //[L1Cal_Exclude  EM_Tower TT_Eta(20) TT_Phi(23)]
            //[L1Cal_Exclude  HD_Tower TT_Eta(20) TT_Phi(23)]
            case Ki_CoorCmd_L1Cal_Exclude :
            {

                switch ( poCoorCommand->m_uiCoorSubCmdCode )
                {

                    /////////////////////////////////////////////////////////////
                    /// EM Et Trigger Tower /////////////////////////////////////
                    /////////////////////////////////////////////////////////////
                case Ki_CoorSubCmd_EM_Tower :
                    {
                        eReturnStatus = _pxCoorExecute->Exclude_Tower
                                                                (  eTT_EM, 
                                                                  &poCoorCommand->m_xTT, 
                                                                  _poCoorCommandLine ) ;
                    } ; break ; // case Ki_CoorSubCmd_EM_Tower 

                    /////////////////////////////////////////////////////////////
                    /// HD Et Trigger Tower /////////////////////////////////////
                    /////////////////////////////////////////////////////////////
                case Ki_CoorSubCmd_HD_Tower :
                    {
                        eReturnStatus = _pxCoorExecute->Exclude_Tower
                                                                (  eTT_HD, 
                                                                  &poCoorCommand->m_xTT, 
                                                                  _poCoorCommandLine ) ;
                    } ; break ; // case Ki_CoorSubCmd_HD_Tower 

                    /////////////////////////////////////////////////////////////
                    // All commands that have been parsed should have been dispatched here
                    // but this catches omissions
                    /////////////////////////////////////////////////////////////
                    default:
                    {
                        strcpy ( sReplyText, "<L1CAL Exclude Trigger Tower Command Not implemented ID =" ) ;
                        strcat ( sReplyText, I_to_A ( poCoorCommand->m_uiCoorCmdCode ) ) ;
                        strcat ( sReplyText, "/" ) ;
                        strcat ( sReplyText, I_to_A ( poCoorCommand->m_uiCoorSubCmdCode ) ) ;
                        strcat ( sReplyText, ">" ) ;
                        CConsoleAccess::ErrorLine     ( sReplyText ) ;
                        _poCoorCommandLine->CopyReplyText ( sReplyText ) ;

                        eReturnStatus = Reply_Bad ;
                    } ; break ;
                }

            } break ; // case Ki_CoorCmd_L1Cal_Exclude 


            /////////////////////////////////////////////////////////////////////
            /// Reference Set Commands //////////////////////////////////////////
            /////////////////////////////////////////////////////////////////////
            //[L1Cal_Ref_Set EM_Et_Ref_Set  0 TT_Eta(-20:20) TT_Phi(1:32) Energy_Threshold 10.0]
            //[L1Cal_Ref_Set Jet_Et_Ref_Set 0 TT_Eta(-20:20) TT_Phi(1:32) Energy_Threshold 10.0]
            //[L1Cal_Ref_Set TT_Et_Ref_Set  0 TT_Eta(-20:20) TT_Phi(1:32) Energy_Threshold 10.0]
            //[L1Cal_Ref_Set EM_Et_Ref_Set  0 Deallocate]
            //[L1Cal_Ref_Set Jet_Et_Ref_Set 0 Deallocate]
            //[L1Cal_Ref_Set TT_Et_Ref_Set  0 Deallocate]
            case Ki_CoorCmd_L1Cal_Ref_Set :
            {

                switch ( poCoorCommand->m_uiCoorSubCmdCode )
                {

                    /////////////////////////////////////////////////////////////
                    /// EM Et Reference Set /////////////////////////////////////
                    /////////////////////////////////////////////////////////////
                case Ki_CoorSubCmd_EM_Et_Ref_Set :
                    {
                        eReturnStatus = _pxCoorExecute->RefSetEMEtProg
                                                                ( poCoorCommand->m_uiRefSetNum, 
                                                                 &poCoorCommand->m_xTT, 
                                                                  poCoorCommand->m_fdThreshold,
                                                                 _poCoorCommandLine ) ;
                    } ; break ; 

                case Ki_CoorSubCmd_EM_Et_Ref_Set_Deallocate :
                    {
                        eReturnStatus = _pxCoorExecute->RefSetEMEtDeall
                                                                ( poCoorCommand->m_uiRefSetNum, 
                                                                 _poCoorCommandLine ) ;
                    } ; break ; 

                    /////////////////////////////////////////////////////////////
                    /// Jet Et Reference Set ///////////////////////////////////
                    /////////////////////////////////////////////////////////////
                case Ki_CoorSubCmd_Jet_Et_Ref_Set :
                    {
                        eReturnStatus = _pxCoorExecute->RefSetJetEtProg
                                                                ( poCoorCommand->m_uiRefSetNum, 
                                                                 &poCoorCommand->m_xTT, 
                                                                  poCoorCommand->m_fdThreshold,
                                                                 _poCoorCommandLine ) ;
                    } ; break ; 

                case Ki_CoorSubCmd_Jet_Et_Ref_Set_Deallocate :
                    {
                        eReturnStatus = _pxCoorExecute->RefSetJetEtDeall 
                                                                ( poCoorCommand->m_uiRefSetNum, 
                                                                 _poCoorCommandLine ) ;
                    } ; break ; 

                    /////////////////////////////////////////////////////////////
                    /// Trigger Tower Et Reference Set //////////////////////////
                    /////////////////////////////////////////////////////////////
                case Ki_CoorSubCmd_TT_Et_Ref_Set :
                    {
                        eReturnStatus = _pxCoorExecute->RefSetTTEtProg
                                                                ( poCoorCommand->m_uiRefSetNum, 
                                                                 &poCoorCommand->m_xTT, 
                                                                  poCoorCommand->m_fdThreshold,
                                                                 _poCoorCommandLine ) ;
                    } ; break ; 

                case Ki_CoorSubCmd_TT_Et_Ref_Set_Deallocate :
                    {
                        eReturnStatus = _pxCoorExecute->RefSetEMEtDeall
                                                                ( poCoorCommand->m_uiRefSetNum, 
                                                                 _poCoorCommandLine ) ;
                    } ; break ; 

                    /////////////////////////////////////////////////////////////
                    // All commands that have been parsed should have been dispatched here
                    // but this catches omissions
                    /////////////////////////////////////////////////////////////
                    default:
                    {
                        strcpy ( sReplyText, "<L1CAL Ref Set Sub Command Not implemented ID =" ) ;
                        strcat ( sReplyText, I_to_A ( poCoorCommand->m_uiCoorCmdCode ) ) ;
                        strcat ( sReplyText, "/" ) ;
                        strcat ( sReplyText, I_to_A ( poCoorCommand->m_uiCoorSubCmdCode ) ) ;
                        strcat ( sReplyText, ">" ) ;
                        CConsoleAccess::ErrorLine     ( sReplyText ) ;
                        _poCoorCommandLine->CopyReplyText ( sReplyText ) ;

                        eReturnStatus = Reply_Bad ;
                    } ; break ;
                }

            } break ; // case Ki_CoorCmd_L1Cal_Ref_Set 


            /////////////////////////////////////////////////////////////////////
            /// L2 Mask Commands //////////////////////////////////////////
            /////////////////////////////////////////////////////////////////////
            //[L1CAL_L2Mask  EM_L2Mask  TT_Eta(-16:16) TT_Phi(1:32) Use_Ref_Set   0]
            //[L1CAL_L2Mask  Jet_L2Mask TT_Eta(-16:16) TT_Phi(1:32) Use_Ref_Set   0]
            //[L1CAL_L2Mask  Tau_L2Mask TT_Eta(-16:16) TT_Phi(1:32) Use_Ref_Set   0]
            case Ki_CoorCmd_L1CAL_L2Mask :
            {

                switch ( poCoorCommand->m_uiCoorSubCmdCode )
                {

                    /////////////////////////////////////////////////////////////
                    /// EM L2Mask /////////////////////////////////////
                    /////////////////////////////////////////////////////////////
                    case Ki_CoorSubCmd_EM_L2Mask :
                    {
                        eReturnStatus = _pxCoorExecute->L2MaskEM
                                                                (&poCoorCommand->m_xTT, 
                                                                  poCoorCommand->m_uiRefSetNum, 
                                                                 _poCoorCommandLine ) ;
                    } ; break ; 

                    /////////////////////////////////////////////////////////////
                    /// Jet L2Mask /////////////////////////////////////
                    /////////////////////////////////////////////////////////////
                    case Ki_CoorSubCmd_Jet_L2Mask :
                    {
                        eReturnStatus = _pxCoorExecute->L2MaskJet
                                                                (&poCoorCommand->m_xTT, 
                                                                  poCoorCommand->m_uiRefSetNum, 
                                                                 _poCoorCommandLine ) ;
                    } ; break ; 

                    /////////////////////////////////////////////////////////////
                    /// Tau L2Mask /////////////////////////////////////
                    /////////////////////////////////////////////////////////////
                    case Ki_CoorSubCmd_Tau_L2Mask :
                    {
                        eReturnStatus = _pxCoorExecute->L2MaskTau
                                                                (&poCoorCommand->m_xTT, 
                                                                  poCoorCommand->m_uiRefSetNum, 
                                                                 _poCoorCommandLine ) ;
                    } ; break ; 

                    /////////////////////////////////////////////////////////////
                    // All commands that have been parsed should have been dispatched here
                    // but this catches omissions
                    /////////////////////////////////////////////////////////////
                    default:
                    {
                        strcpy ( sReplyText, "<L1CAL L2 Mask Command Not implemented ID =" ) ;
                        strcat ( sReplyText, I_to_A ( poCoorCommand->m_uiCoorCmdCode ) ) ;
                        strcat ( sReplyText, "/" ) ;
                        strcat ( sReplyText, I_to_A ( poCoorCommand->m_uiCoorSubCmdCode ) ) ;
                        strcat ( sReplyText, ">" ) ;
                        CConsoleAccess::ErrorLine     ( sReplyText ) ;
                        _poCoorCommandLine->CopyReplyText ( sReplyText ) ;

                        eReturnStatus = Reply_Bad ;
                    } ; break ;
                }

            } break ; // case Ki_CoorCmd_L1Cal_Ref_Set 


            /////////////////////////////////////////////////////////////////////
            /// TAB Algorithm Parameters  ///////////////////////////////////////
            /////////////////////////////////////////////////////////////////////
            //[L1CAL_Algo_Param  EM_Isolation    Use_EMFraction]
            //[L1CAL_Algo_Param  EM_Isolation    Ignore_EMFraction]
            //[L1CAL_Algo_Param  EM_Isolation    Use_EMIsolation]
            //[L1CAL_Algo_Param  EM_Isolation    Ignore_EMIsolation]
            //[L1CAL_Algo_Param  EM_Isolation_Et_Cut     TT_Eta(-16:16) TT_Phi(1:32) Energy_Threshold 1.0]
            //[L1CAL_Algo_Param  TT_Tau_EtCore_Cut       TT_Eta(-16:16) TT_Phi(1:32) Energy_Threshold 8.0]
            //[L1CAL_Algo_Param  TT_Global_Sum_Et_Cut    TT_Eta(-20:20) TT_Phi(1:32) Energy_Threshold 1.0]
            //[L1CAL_Algo_Param  Jet_Objects     Use_ICR]
            //[L1CAL_Algo_Param  Jet_Objects     Ignore_ICR]
            //[L1CAL_Algo_Param  Global_Sums     Use_ICR]
            //[L1CAL_Algo_Param  Global_Sums     Ignore_ICR]

            case Ki_CoorCmd_L1CAL_Algo_Param :
            {

                switch ( poCoorCommand->m_uiCoorSubCmdCode )
                {
                    /////////////////////////////////////////////////////////////
                    /// EM Isolation Use EMFraction  ////////////////////////////
                    /////////////////////////////////////////////////////////////
                    case Ki_CoorSubCmd_EM_Isolation_Use_EMFraction :
                        eReturnStatus = _pxCoorExecute->AlgoUseEmFraction
                                                                (&poCoorCommand->m_xTT, 
                                                                  true,
                                                                 _poCoorCommandLine ) ;
                    break ;  

                    /////////////////////////////////////////////////////////////
                    /// EM Isolation Ignore EMFraction  /////////////////////////
                    /////////////////////////////////////////////////////////////
                    case Ki_CoorSubCmd_EM_Isolation_Ignore_EMFraction :
                        eReturnStatus = _pxCoorExecute->AlgoUseEmFraction
                                                                (&poCoorCommand->m_xTT, 
                                                                  false,
                                                                 _poCoorCommandLine ) ;
                    break ;  

                    /////////////////////////////////////////////////////////////
                    /// EM Isolation Use EMIsolation ////////////////////////////
                    /////////////////////////////////////////////////////////////
                    case Ki_CoorSubCmd_EM_Isolation_Use_EMIsolation :
                        eReturnStatus = _pxCoorExecute->AlgoUseEmIsolation
                                                                (&poCoorCommand->m_xTT, 
                                                                  true,
                                                                 _poCoorCommandLine ) ;
                    break ;  

                    /////////////////////////////////////////////////////////////
                    /// EM Isolation Ignore EMIsolation /////////////////////////
                    /////////////////////////////////////////////////////////////
                    case Ki_CoorSubCmd_EM_Isolation_Ignore_EMIsolation :
                        eReturnStatus = _pxCoorExecute->AlgoUseEmIsolation
                                                                (&poCoorCommand->m_xTT, 
                                                                  false,
                                                                 _poCoorCommandLine ) ;
                    break ;  

                    /////////////////////////////////////////////////////////////
                    /// EM Isolation Et Cut /////////////////////////////////////
                    /////////////////////////////////////////////////////////////
                    case Ki_CoorSubCmd_EM_Isolation_Et_Cut :
                        eReturnStatus = _pxCoorExecute->AlgoEmIsolationEtCut
                                                                (&poCoorCommand->m_xTT, 
                                                                  poCoorCommand->m_fdThreshold,
                                                                 _poCoorCommandLine ) ;
                    break ;  

                    /////////////////////////////////////////////////////////////
                    /// TT Tau Et Core Cut /////////////////////////////////////
                    /////////////////////////////////////////////////////////////
                    case Ki_CoorSubCmd_TT_Tau_EtCore_Cut :
                        eReturnStatus = _pxCoorExecute->AlgoTauEtCoreCut
                                                                (&poCoorCommand->m_xTT, 
                                                                  poCoorCommand->m_fdThreshold,
                                                                 _poCoorCommandLine ) ;
                    break ;  

                    /////////////////////////////////////////////////////////////
                    /// TT Global Sum Et Cut ////////////////////////////////////
                    /////////////////////////////////////////////////////////////
                    case Ki_CoorSubCmd_TT_Global_Sum_Et_Cut :
                        eReturnStatus = _pxCoorExecute->AlgoGlobalSumEtCut
                                                                (&poCoorCommand->m_xTT, 
                                                                  poCoorCommand->m_fdThreshold,
                                                                 _poCoorCommandLine ) ;
                    break ;  

                    /////////////////////////////////////////////////////////////
                    /// Jet Objects Use ICR /////////////////////////////////////
                    /////////////////////////////////////////////////////////////
                    case Ki_CoorSubCmd_Jet_Objects_Use_ICR :
                        eReturnStatus = _pxCoorExecute->AlgoUseIcrJetObjects
                                                                (&poCoorCommand->m_xTT, 
                                                                  true,
                                                                 _poCoorCommandLine ) ;
                    break ;  

                    /////////////////////////////////////////////////////////////
                    /// Jet Objects Ignore ICR //////////////////////////////////
                    /////////////////////////////////////////////////////////////
                    case Ki_CoorSubCmd_Jet_Objects_Ignore_ICR :
                        eReturnStatus = _pxCoorExecute->AlgoUseIcrJetObjects
                                                                (&poCoorCommand->m_xTT, 
                                                                  false,
                                                                 _poCoorCommandLine ) ;
                    break ;  

                    /////////////////////////////////////////////////////////////
                    /// Global Sums Use ICR /////////////////////////////////////
                    /////////////////////////////////////////////////////////////
                    case Ki_CoorSubCmd_Global_Sums_Use_ICR :
                        eReturnStatus = _pxCoorExecute->AlgoUseIcrGlobalSums
                                                                (&poCoorCommand->m_xTT, 
                                                                  true,
                                                                 _poCoorCommandLine ) ;
                    break ;  

                    /////////////////////////////////////////////////////////////
                    /// Global Sums Ignore ICR //////////////////////////////////
                    /////////////////////////////////////////////////////////////
                    case Ki_CoorSubCmd_Global_Sums_Ignore_ICR :
                        eReturnStatus = _pxCoorExecute->AlgoUseIcrGlobalSums
                                                                (&poCoorCommand->m_xTT, 
                                                                  false,
                                                                 _poCoorCommandLine ) ;
                    break ;  

                    /////////////////////////////////////////////////////////////
                    // All commands that have been parsed should have been dispatched here
                    // but this catches omissions
                    /////////////////////////////////////////////////////////////
                    default:
                    {
                        strcpy ( sReplyText, "<L1CAL Algorithm Parameter Command Not implemented ID =" ) ;
                        strcat ( sReplyText, I_to_A ( poCoorCommand->m_uiCoorCmdCode ) ) ;
                        strcat ( sReplyText, "/" ) ;
                        strcat ( sReplyText, I_to_A ( poCoorCommand->m_uiCoorSubCmdCode ) ) ;
                        strcat ( sReplyText, ">" ) ;
                        CConsoleAccess::ErrorLine     ( sReplyText ) ;
                        _poCoorCommandLine->CopyReplyText ( sReplyText ) ;

                        eReturnStatus = Reply_Bad ;
                    } ; break ;
                }

            } break ; // case Ki_CoorCmd_L1CAL_Algo_Param


            /////////////////////////////////////////////////////////////////////
            /// GAB to L1FW And/Or Term Commands /////////////////////////////////
            /////////////////////////////////////////////////////////////////////
            //[L1CAL_to_L1FW EM_All_Term                1 Use_Ref_Set   0 Count_Threshold 4]
            //[L1CAL_to_L1FW EM_Central_Term            1 Use_Ref_Set   0 Count_Threshold 4]
            //[L1CAL_to_L1FW Jet_All_Term               1 Use_Ref_Set   0 Count_Threshold 4]
            //[L1CAL_to_L1FW Jet_Central_Term           1 Use_Ref_Set   0 Count_Threshold 4]
            //[L1CAL_to_L1FW Tau_All_Term               1 Use_Ref_Set   0 Use_Tau_Ratio_Set 1 Count_Threshold 2]
            //[L1CAL_to_L1FW Tau_Central_Term           1 Use_Ref_Set   0 Use_Tau_Ratio_Set 1 Count_Threshold 2]
            //[L1CAL_to_L1FW Isolated_EM_All_Term       1 Use_Ref_Set   0 Count_Threshold 2]
            //[L1CAL_to_L1FW Isolated_EM_Central_Term   1 Use_Ref_Set   0 Count_Threshold 2]
            //[L1CAL_to_L1FW Missing_Et_Term            1 Energy_Threshold 10.0]
            //[L1CAL_to_L1FW Total_Et_Term              1 Energy_Threshold 10.0]
            //[L1CAL_to_L1FW Acoplanar_Jet_Term         1 Use_Ref_Set   3 Use_Ref_Set_Veto 4]
            //[L1CAL_to_L1FW Back_To_Back_EM_Term       1 Use_Ref_Set   0]
            case Ki_CoorCmd_L1CAL_to_L1FW :
            {

                switch ( poCoorCommand->m_uiCoorSubCmdCode )
                {

                    /////////////////////////////////////////////////////////////
                    /// Em_All_Term        /////////////////////////////////////
                    /////////////////////////////////////////////////////////////
                case Ki_CoorSubCmd_EM_All_Term :
                    {
                        eReturnStatus = _pxCoorExecute->TermEmAll
                                                                ( poCoorCommand->m_uiObjectIndex , 
                                                                  poCoorCommand->m_uiRefSetNum, 
                                                                  poCoorCommand->m_uiValue,
                                                                 _poCoorCommandLine ) ;
                    } ; break ;  


                    /////////////////////////////////////////////////////////////
                    /// Em_Central_Term    /////////////////////////////////////
                    /////////////////////////////////////////////////////////////
                case Ki_CoorSubCmd_EM_Central_Term :
                    {
                        eReturnStatus = _pxCoorExecute->TermEmCentral
                                                                ( poCoorCommand->m_uiObjectIndex , 
                                                                  poCoorCommand->m_uiRefSetNum, 
                                                                  poCoorCommand->m_uiValue,
                                                                 _poCoorCommandLine ) ;
                    } ; break ;  


                    /////////////////////////////////////////////////////////////
                    /// Jet_All_Term        /////////////////////////////////////
                    /////////////////////////////////////////////////////////////
                case Ki_CoorSubCmd_Jet_All_Term :
                    {
                        eReturnStatus = _pxCoorExecute->TermJetAll
                                                                ( poCoorCommand->m_uiObjectIndex , 
                                                                  poCoorCommand->m_uiRefSetNum, 
                                                                  poCoorCommand->m_uiValue,
                                                                 _poCoorCommandLine ) ;
                    } ; break ;  


                    /////////////////////////////////////////////////////////////
                    /// Jet_Central_Term    /////////////////////////////////////
                    /////////////////////////////////////////////////////////////
                case Ki_CoorSubCmd_Jet_Central_Term :
                    {
                        eReturnStatus = _pxCoorExecute->TermJetCentral
                                                                ( poCoorCommand->m_uiObjectIndex , 
                                                                  poCoorCommand->m_uiRefSetNum, 
                                                                  poCoorCommand->m_uiValue,
                                                                 _poCoorCommandLine ) ;
                    } ; break ;  


                    /////////////////////////////////////////////////////////////
                    /// Tau_All_Term        /////////////////////////////////////
                    /////////////////////////////////////////////////////////////
                case Ki_CoorSubCmd_Tau_All_Term :
                    {
                        eReturnStatus = _pxCoorExecute->TermTauAll
                                                                ( poCoorCommand->m_uiObjectIndex , 
                                                                  poCoorCommand->m_uiRefSetNum, 
                                                                  poCoorCommand->m_uiAuxiliary,
                                                                  poCoorCommand->m_uiValue,
                                                                 _poCoorCommandLine ) ;
                    } ; break ;  


                    /////////////////////////////////////////////////////////////
                    /// Tau_Central_Term        /////////////////////////////////////
                    /////////////////////////////////////////////////////////////
                case Ki_CoorSubCmd_Tau_Central_Term :
                    {
                        eReturnStatus = _pxCoorExecute->TermTauCentral
                                                                ( poCoorCommand->m_uiObjectIndex , 
                                                                  poCoorCommand->m_uiRefSetNum, 
                                                                  poCoorCommand->m_uiAuxiliary,
                                                                  poCoorCommand->m_uiValue,
                                                                 _poCoorCommandLine ) ;
                    } ; break ;  


                    /////////////////////////////////////////////////////////////
                    /// Isolated_EM_All_Term/////////////////////////////////////
                    /////////////////////////////////////////////////////////////
                case Ki_CoorSubCmd_Isolated_EM_All_Term :
                    {
                        eReturnStatus = _pxCoorExecute->TermIsolEmAll
                                                                ( poCoorCommand->m_uiObjectIndex , 
                                                                  poCoorCommand->m_uiRefSetNum, 
                                                                  poCoorCommand->m_uiValue,
                                                                 _poCoorCommandLine ) ;
                    } ; break ;  


                    /////////////////////////////////////////////////////////////
                    /// Isolated_EM_Central_Term    /////////////////////////////
                    /////////////////////////////////////////////////////////////
                case Ki_CoorSubCmd_Isolated_EM_Central_Term :
                    {
                        eReturnStatus = _pxCoorExecute->TermIsolEmCentral
                                                                ( poCoorCommand->m_uiObjectIndex , 
                                                                  poCoorCommand->m_uiRefSetNum, 
                                                                  poCoorCommand->m_uiValue,
                                                                 _poCoorCommandLine ) ;
                    } ; break ;  


                    /////////////////////////////////////////////////////////////
                    /// Missing_Et_Term     /////////////////////////////////////
                    /////////////////////////////////////////////////////////////
                case Ki_CoorSubCmd_Missing_Et_Term :
                    {
                        eReturnStatus = _pxCoorExecute->TermMissEt   
                                                                ( poCoorCommand->m_uiObjectIndex , 
                                                                  poCoorCommand->m_fdThreshold,
                                                                 _poCoorCommandLine ) ;
                    } ; break ;  


                    /////////////////////////////////////////////////////////////
                    /// Total_Et_Term       /////////////////////////////////////
                    /////////////////////////////////////////////////////////////
                case Ki_CoorSubCmd_Total_Et_Term :
                    {
                        eReturnStatus = _pxCoorExecute->TermTotEt  
                                                                ( poCoorCommand->m_uiObjectIndex , 
                                                                  poCoorCommand->m_fdThreshold,
                                                                 _poCoorCommandLine ) ;
                    } ; break ;  


                    /////////////////////////////////////////////////////////////
                    /// Acoplanar_Jet_Term  /////////////////////////////////////
                    /////////////////////////////////////////////////////////////
                case Ki_CoorSubCmd_Acoplanar_Jet_Term :
                    {
                        eReturnStatus = _pxCoorExecute->TermAcoplanarJet
                                                                ( poCoorCommand->m_uiObjectIndex , 
                                                                  poCoorCommand->m_uiRefSetNum, 
                                                                  poCoorCommand->m_uiAuxiliary,
                                                                  poCoorCommand->m_uiValue,
                                                                 _poCoorCommandLine ) ;
                    } ; break ;  


                    /////////////////////////////////////////////////////////////
                    /// NonCollinear_LowEt_Jet_Term /////////////////////////////////////
                    /////////////////////////////////////////////////////////////
                case Ki_CoorSubCmd_NonCollinear_LowEt_Jet_Term :
                    {
                        eReturnStatus = _pxCoorExecute->TermNonCollinearLowEtJet
                                                                ( poCoorCommand->m_uiObjectIndex , 
                                                                  poCoorCommand->m_uiRefSetNum, 
                                                                  poCoorCommand->m_uiAuxiliary,
                                                                  poCoorCommand->m_uiValue,
                                                                 _poCoorCommandLine ) ;
                    } ; break ;  


                    /////////////////////////////////////////////////////////////
                    /// NonCollinear_Mono_Jet_Term /////////////////////////////////////
                    /////////////////////////////////////////////////////////////
                case Ki_CoorSubCmd_NonCollinear_Mono_Jet_Term :
                    {
                        eReturnStatus = _pxCoorExecute->TermNonCollinearMonoJet
                                                                ( poCoorCommand->m_uiObjectIndex , 
                                                                  poCoorCommand->m_uiRefSetNum, 
                                                                  poCoorCommand->m_uiAuxiliary,
                                                                  poCoorCommand->m_uiValue,
                                                                 _poCoorCommandLine ) ;
                    } ; break ;  


                    /////////////////////////////////////////////////////////////
                    /// Jet_Free_Term  /////////////////////////////////////
                    /////////////////////////////////////////////////////////////
                case Ki_CoorSubCmd_Jet_Free_Term :
                    {
                        eReturnStatus = _pxCoorExecute->TermJetFree
                                                                ( poCoorCommand->m_uiObjectIndex , 
                                                                  poCoorCommand->m_uiRefSetNum, 
                                                                  poCoorCommand->m_uiAuxiliary,
                                                                  poCoorCommand->m_uiValue,
                                                                 _poCoorCommandLine ) ;
                    } ; break ;  


                    /////////////////////////////////////////////////////////////
                    /// Back_To_Back_EM_Term  ///////////////////////////////////
                    /////////////////////////////////////////////////////////////
                case Ki_CoorSubCmd_Back_To_Back_EM_Term :
                    {
                        eReturnStatus = _pxCoorExecute->TermBackToBackEm
                                                                ( poCoorCommand->m_uiObjectIndex , 
                                                                  poCoorCommand->m_uiRefSetNum, 
                                                                  poCoorCommand->m_uiAuxiliary,
                                                                 _poCoorCommandLine ) ;
                    } ; break ;  


                    /////////////////////////////////////////////////////////////
                    // All commands that have been parsed should have been dispatched here
                    // but this catches omissions
                    /////////////////////////////////////////////////////////////
                    default:
                    {
                        strcpy ( sReplyText, "<L1CAL Count Threshold Sub Command Not implemented ID =" ) ;
                        strcat ( sReplyText, I_to_A ( poCoorCommand->m_uiCoorCmdCode ) ) ;
                        strcat ( sReplyText, "/" ) ;
                        strcat ( sReplyText, I_to_A ( poCoorCommand->m_uiCoorSubCmdCode ) ) ;
                        strcat ( sReplyText, ">" ) ;
                        CConsoleAccess::ErrorLine     ( sReplyText ) ;
                        _poCoorCommandLine->CopyReplyText ( sReplyText ) ;

                        eReturnStatus = Reply_Bad ;
                    } ; break ;
                }

            } break ; // Ki_CoorCmd_L1Cal_Count_Threshold


            // All commands that have been parsed should have been dispatched here
            // but this catches omissions
            default:
            {
                strcpy ( sReplyText, "<L1CAL Command Not implemented ID =" ) ;
                strcat ( sReplyText, I_to_A ( poCoorCommand->m_uiCoorCmdCode ) ) ;
                strcat ( sReplyText, ">" ) ;
                CConsoleAccess::ErrorLine     ( sReplyText ) ;
                _poCoorCommandLine->CopyReplyText ( sReplyText ) ;

                eReturnStatus = Reply_Bad ;
            } ; break ;

        } ;

        // find the next command in the list, if any
        // but remember individual errors to tag the whole message as an error
        if ( eReturnStatus == Reply_Bad ) eRememberStatus = Reply_Bad ;

        poCoorCommandEntry = _poCoorCommandLine->m_lCommandList.NextEntry( poCoorCommandEntry ) ;
        if ( poCoorCommandEntry != NULL ) 
            poCoorCommand = poCoorCommandEntry ->GetAtKey( ) ;
        else 
            poCoorCommand = NULL ;

    } ; 

    if ( eRememberStatus == Reply_Bad ) eReturnStatus = Reply_Bad ;

    return eReturnStatus ;
}

