/*-------------------------------------------------------------------------------*/
  /*
     MessageCoorExecuter.cpp
     7-Oct-99
     28-Oct-99 Coor Command Parsing split onto separate files
     22-Sep-2005 Restructred for L1Cal_IIb_TCC
  */
/*-------------------------------------------------------------------------------*/
#include "L1Cal_IIb_TCC/StdAfx.h"

#include <math.h>
#include <time.h>

#include "L1Cal_IIb_TCC/MessageCommand.h"
#include                            "L1Cal_IIb_TCC/MessageCoorExecuter.h"

#include "L1Cal_IIb_TCC/HandleHardwareAccess.h" // for CheckBit3Status

#include "L1Cal_IIb_TCC/HandleCommandFiles.h"
#include "L1Cal_IIb_TCC/CommandFileMaster.h"

#include "L1Cal_IIb_TCC/HandleResultFile.h"

#include "L1Cal_IIb_TCC/ListOfMasterCommandFiles.h"
#include "L1Cal_IIb_TCC/ListOfResultFiles.h"

#include "L1Cal_IIb_TCC/L1Cal.h"
#include "L1Cal_IIb_TCC/HsroL1Cal.h"

#include "L1Cal_IIb_TCC/ResourceLocks.h"

#include "L1Cal_IIb_TCC/MonitControl.h"

#include "L1Cal_IIb_TCC/HandleConsole.h"
#include "L1Cal_IIb_TCC/UtilsStrings.h"
#include "L1Cal_IIb_TCC/UtilsBits.h"

//#include <math.h>

/*-------------------------------------------------------------------------------*/

// macros used in many of the commands
/////////////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////////////
#define APPEND_STATUS_TO_REPLY()                                                    \
    /* append success/error to reply text*/                                         \
    if ( _poCoorCommandLine != NULL )                                               \
        _poCoorCommandLine->AppendReplyText ( sReplyText ) ;                        \
    if ( eReturnStatus == Reply_Ok )                                                \
        CConsoleAccess::WriteLine     ( (CString) "Process Msg  Status : "          \
                                       + sReplyText ) ;                             \
    else                                                                            \
        CConsoleAccess::ErrorLine     ( (CString) "Process Msg  Status : "          \
                                       + sReplyText ) ;                             \
/////////////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////////////
#define CHECK_FOR_IO_ERROR()                                                        \
    if ( eReturnStatus == Reply_Bad )                                               \
        strcpy ( sReplyText, "<IO Error During Command Execution>" ) ;              \
/////////////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////////////
#define IF_L1CAL_OPERATIONAL()                                                      \
  /*if*/ ( CL1Cal::mg_eynL1CalOperational == No )                                   \
    {                                                                               \
        eReturnStatus = Reply_Bad ;                                                 \
        strcpy ( sReplyText, "<L1FW wasn't Initialized, get expert assistance>" ) ; \
    }                                                                               \
    else /* we can perform the requested action */                                  \
/////////////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////
#define IF_TT_RANGE_WITHIN_COVERAGE( TT_COVERAGE )                                  \
  /*if*/ ( CL1Cal::mg_poL1Cal->f_okRangeInCoverage (TT_COVERAGE) != Ok)             \
    {                                                                               \
        eReturnStatus = Reply_Bad ;                                                 \
        strcpy ( sReplyText, "<L1CAL Trig Tower Range Problem, get expert assistance>" ) ;\
    }                                                                               \
    else /* we can perform the requested action */                                  \
/////////////////////////////////////////////////////////////////////////////////////
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ECoorMessageStatus CMessageCoorExecuter::Configure_FPGAs          ( CMessageCommandLine*    _poCoorCommandLine ) 
{ 
    ECoorMessageStatus  eReturnStatus = Reply_Ok ;

    char                sReplyText[KiMaxLengthReplyTextToCoor] = "<Done>" ;

    CString                     osMaster_Command_File ;
    Custom_Structure_Master     xCustom ;

    CResultFile                 oStatusFile  ( osStatus_At_Last_Config ) ; // Where we will write the command status

    //Kludge to retrieve the Number of FPGA configured
    extern CString     goLastConfigStatus ;

    //\\//

    CMessageCoorPrinter::Configure_FPGAs          ( _poCoorCommandLine ) ;

    // Open a file to write command execution status
    if ( oStatusFile.OpenForWrite () == Ok )
    {
        oStatusFile.WriteLine ( (CString) "! This File was written by L1Cal_IIb_TCC "
                                           + KsVersion
                                           + " During System Configuration" ) ;
        oStatusFile.WriteDateTimeLine () ;
    }
    else
        CConsoleAccess::ErrorLine ( "Failed Creating Status File for Mailing" ) ;


    //clear Bit3 errors
    ///////////////////
    if ( CHardwareAccess::mg_poBit3BusAdaptor->ClearBit3Status() != Ok )
    {
        eReturnStatus = Reply_Bad ;
        strcpy ( sReplyText, "<Abort after Failing to Clear Bit3 Error>" ) ;
        if ( oStatusFile.ReadyForWrite() == Yes )
        oStatusFile.WriteLine ( "! Abort after Failing to Clear Bit3 Error" ) ;
        goto Abort ;
    }

    // Declare the L1Cal un-usable so that 
    // - COOR will have to issue an initialize before any other message gets serviced
    // - the Monit Data Server is prevented from generating IO until the next init
    // Acquire the Resource Lock to save other threads the problem 
    // of having this flag change while they are in the middle of something crucial
    ///////////////////////////////////////////
    CConsoleAccess::WriteLine ( (CString) " Acquiring L1Cal Operational Lock"
                                        + " Before Declaring System Non-Operational..." ) ;    
    CResourceLocks::Acquire_L1CalOperational () ;
      CL1Cal::mg_eynL1CalOperational = No ;
    CResourceLocks::Release_L1CalOperational () ;

    //execute the dedicated Master Command File
    ///////////////////////////////////////////
    osMaster_Command_File = osConfigure_FPGAs_MCF ;
    if ( CommandFileMaster ( osMaster_Command_File, &xCustom, NULL ) != Ok )
        eReturnStatus = Reply_Bad ;

    if ( oStatusFile.ReadyForWrite() == Yes )
    oStatusFile.WriteLine ( (CString) "!" + goLastConfigStatus ) ;

    //remember the last time the system was Configured
//    CL1fw::mg_ilTimeStampLastConfig = time ( NULL ) ; 

    CHECK_FOR_IO_ERROR() ;

Abort:

    APPEND_STATUS_TO_REPLY() ;

    //Report to Status File on Command Global Success
    if ( oStatusFile.ReadyForWrite() == Yes )
    {
        if ( eReturnStatus == Reply_Ok )
            oStatusFile.WriteLine ( "! Command Successful" ) ;
        else
            oStatusFile.WriteLine ( "! IO Error During Command Execution" ) ;
    }

    //Close Status File
    if ( oStatusFile.ReadyForWrite() == Yes )
    oStatusFile.Close () ;

    return eReturnStatus ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ECoorMessageStatus CMessageCoorExecuter::L1Cal_Initialize         ( CMessageCommandLine*    _poCoorCommandLine ) 
{ 
    ECoorMessageStatus  eReturnStatus = Reply_Ok ;

    char                sReplyText[KiMaxLengthReplyTextToCoor] = "<Done>" ;

    CString                     osMaster_Command_File ;
    Custom_Structure_Master     xCustom ;

    CResultFile                 oStatusFile  ( osStatus_At_Last_Init ) ; // Where we will write the command status

    //\\//

    CMessageCoorPrinter::L1Cal_Initialize         ( _poCoorCommandLine ) ;

    // Open a file to write command execution status
    if ( oStatusFile.OpenForWrite () == Ok )
    {
        oStatusFile.WriteLine ( (CString) "! This File was written by L1Cal_IIb_TCC "
                                           + KsVersion
                                           + " During System Initialization" ) ;
        oStatusFile.WriteDateTimeLine () ;
    }
    else
        CConsoleAccess::ErrorLine ( "Failed Creating Status File for Mailing" ) ;


    // Declare the L1Cal un-usable until successful completion 
    // This will also inidirectly Suspend monitoring operation 
    // Acquire the Resource Lock to save other threads the problem 
    // of having this flag change while they are in the middle of something crucial
    ///////////////////////////////////////////
    CConsoleAccess::WriteLine ( (CString) " Acquiring L1Cal Operational Lock"
                                        + " Before Declaring System Non-Operational..." ) ;    
    CResourceLocks::Acquire_L1CalOperational () ;
      CL1Cal::mg_eynL1CalOperational = No ;
    CResourceLocks::Release_L1CalOperational () ;


    //Clear Bit3 errors
    ///////////////////
    if ( CHardwareAccess::mg_poBit3BusAdaptor->CheckBit3Status() != Ok )
        CConsoleAccess::ErrorLine ( "Found Bit3 Errors Before Starting Init Sequence; Continuing" ) ;

    if ( CHardwareAccess::mg_poBit3BusAdaptor->ClearBit3Status() != Ok )
    {
        eReturnStatus = Reply_Bad ;
        strcpy ( sReplyText, "<Abort after Failing to Clear Bit3 Error>" ) ;
        if ( oStatusFile.ReadyForWrite() == Yes )
            oStatusFile.WriteLine ( "! Abort after Failing to Clear Bit3 Error" ) ;
        goto Abort ;
    }

    //execute the PRE-INIT AUXI command file
    /////////////////////////////////////////
    osMaster_Command_File = osInit_Pre_Auxi_MCF ;
    if ( CommandFileMaster ( osMaster_Command_File, &xCustom, NULL ) != Ok )
    {
        eReturnStatus = Reply_Bad ;
        strcpy ( sReplyText, "<Abort after IO Error During Init_Pre_Auxi>" ) ;
        if ( oStatusFile.ReadyForWrite() == Yes )
            oStatusFile.WriteLine ( "! Abort after IO Error During Init_Pre_Auxi" ) ;
        if ( CL1Cal::mg_eynAbortInitOnError == Yes ) goto Abort ;
    }

    if ( CHardwareAccess::mg_poBit3BusAdaptor->CheckBit3Status() != Ok )
    {
        CConsoleAccess::ErrorLine ( "Found Bit3 Errors after Init_Pre_Auxi; Continuing" ) ;
        CHardwareAccess::mg_poBit3BusAdaptor->ClearBit3Status() ;
    }

    // Initialize the L1 CT
    ///////////////////////
    if ( CL1Cal::mg_poL1Cal->Initialize () != Ok ) 
    {
        eReturnStatus = Reply_Bad ;
        strcpy ( sReplyText, "<IO Error During L1CAL Initialize>" ) ;
        if ( oStatusFile.ReadyForWrite() == Yes )
            oStatusFile.WriteLine ( "! Abort after IO Error During L1Cal Initialization" ) ;
        if ( CL1Cal::mg_eynAbortInitOnError == Yes ) goto Abort ;
    }

    if ( CHardwareAccess::mg_poBit3BusAdaptor->CheckBit3Status() != Ok )
    {
        CConsoleAccess::ErrorLine ( "Found Bit3 Errors after Init L1Cal; Continuing" ) ;
        CHardwareAccess::mg_poBit3BusAdaptor->ClearBit3Status() ;
    }

    // Initialize the Readout Crate
    ////////////////////////////////////
    if ( CHsroL1Cal::mg_poHsroL1Cal->Initialize () != Ok )
    {
        eReturnStatus = Reply_Bad ;
        strcpy ( sReplyText, "<IO Error During Readout Crate Initialization>" ) ;
        if ( oStatusFile.ReadyForWrite() == Yes )
            oStatusFile.WriteLine ( "! Abort after IO Error During Readout Crate Initialization" ) ;
        if ( CL1Cal::mg_eynAbortInitOnError == Yes ) goto Abort ;
    }

    if ( CHardwareAccess::mg_poBit3BusAdaptor->CheckBit3Status() != Ok )
    {
        CConsoleAccess::ErrorLine ( "Found Bit3 Errors after Init Readout; Continuing" ) ;
        CHardwareAccess::mg_poBit3BusAdaptor->ClearBit3Status() ;
    }

    //execute the L1Cal POST-INIT AUXI command file
    /////////////////////////////////////////
    osMaster_Command_File = osInit_Post_Auxi_MCF ;
    if ( CommandFileMaster ( osMaster_Command_File, &xCustom, NULL ) != Ok )
    {
        eReturnStatus = Reply_Bad ;
        strcpy ( sReplyText, "<IO Error During Init_Post_Auxi>" ) ;
        if ( CL1Cal::mg_eynAbortInitOnError == Yes ) goto Abort ;
    }

    if ( CHardwareAccess::mg_poBit3BusAdaptor->CheckBit3Status() != Ok )
    {
        CConsoleAccess::ErrorLine ( "Found Bit3 Errors after Init_Post_Auxi; Continuing" ) ;
        CHardwareAccess::mg_poBit3BusAdaptor->ClearBit3Status() ;
    }


    // Re-enable Monitoring 
    //////////////////////////
	if ( CMonitControl::fg_eynMonit_Server_Silenced() == Yes )
    {
        CConsoleAccess::WarningLine ( " Monitoring Services Found Disabled -> Now Re-Enable" ) ;
        CMonitControl::Set_Monit_Server_Silenced ( No ) ;
    }


    // Declare the L1Cal usable 
    // Acquire the Resource Lock to save other threads the problem 
    // of having this flag change while they are in the middle of something crucial
    ///////////////////////////////////////////
    if ( eReturnStatus == Reply_Ok )
    {
        CConsoleAccess::WriteLine ( (CString) " Acquiring L1Cal Operational Lock"
                                            + " Before Declaring System Operational..." ) ;    
        CResourceLocks::Acquire_L1CalOperational () ;
          CL1Cal::mg_eynL1CalOperational = Yes ;
        CResourceLocks::Release_L1CalOperational () ;
    }


    //remember the last time the system was initialized
    ///////////////////////////////////////////////////
//    L1fw::mg_ilTimeStampLastInit = time ( NULL ) ; 

Abort:

    APPEND_STATUS_TO_REPLY() ;

    //Report to Status File on Command Global Success
    if ( oStatusFile.ReadyForWrite() == Yes )
    {
        if ( eReturnStatus == Reply_Ok )
            oStatusFile.WriteLine ( "! L1Cal Initialization Successful" ) ;
        else
            oStatusFile.WriteLine ( "! L1Cal Initialization Failed" ) ;
    }

    //Close Status File
    if ( oStatusFile.ReadyForWrite() == Yes )
    oStatusFile.Close () ;

    return eReturnStatus ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ECoorMessageStatus CMessageCoorExecuter::Iterative_DeSkew         ( CMessageCommandLine*    _poCoorCommandLine ) 
{ 
    ECoorMessageStatus  eReturnStatus = Reply_Ok ;

    char                sReplyText[KiMaxLengthReplyTextToCoor] = "<Done>" ;

    //\\//

    CMessageCoorPrinter::Iterative_DeSkew         ( _poCoorCommandLine ) ;

    APPEND_STATUS_TO_REPLY() ;

    return eReturnStatus ;
}


/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
/*/
ECoorMessageStatus CMessageCoorExecuter::StartRun                 ( const unsigned long     _ulRunNumber,
                                                                    EItemPolarity*          _pubListItemPolarity,
                                                                    CMessageCommandLine*    _poCoorCommandLine ) 
{ 
    ECoorMessageStatus  eReturnStatus = Reply_Ok ;

    char                sReplyText[KiMaxLengthReplyTextToCoor] = "<Done>" ;

    //\\//

    CMessageCoorPrinter::StartRun                 ( _ulRunNumber,
                                                    _pubListItemPolarity,
                                                    _poCoorCommandLine ) ;

    APPEND_STATUS_TO_REPLY() ;

    return eReturnStatus ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
/*/
ECoorMessageStatus CMessageCoorExecuter::StopRun                  ( const unsigned long     _ulRunNumber,
                                                                    CMessageCommandLine*    _poCoorCommandLine ) 
{ 
    ECoorMessageStatus  eReturnStatus = Reply_Ok ;

    char                sReplyText[KiMaxLengthReplyTextToCoor] = "<Done>" ;

    CResultFile                 oLbnFile     ( osLBN_Backup ) ; // Where we will write the current LBN

    //\\//

    CMessageCoorPrinter::StopRun                  ( _ulRunNumber,
                                                    _poCoorCommandLine ) ;

    APPEND_STATUS_TO_REPLY() ;

    return eReturnStatus ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
/*/
ECoorMessageStatus CMessageCoorExecuter::PauseRun                 ( const unsigned long     _ulRunNumber,
                                                                    CMessageCommandLine*    _poCoorCommandLine ) 
{ 
    ECoorMessageStatus  eReturnStatus = Reply_Ok ;

    char                sReplyText[KiMaxLengthReplyTextToCoor] = "<Done>" ;

    //\\//

    CMessageCoorPrinter::PauseRun                 ( _ulRunNumber, _poCoorCommandLine ) ;


    APPEND_STATUS_TO_REPLY() ;

    return eReturnStatus ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
/*/
ECoorMessageStatus CMessageCoorExecuter::ResumeRun                ( const unsigned long     _ulRunNumber,
                                                                    CMessageCommandLine*    _poCoorCommandLine ) 
{ 
    ECoorMessageStatus  eReturnStatus = Reply_Ok ;

    char                sReplyText[KiMaxLengthReplyTextToCoor] = "<Done>" ;

    //\\//

    CMessageCoorPrinter::ResumeRun                ( _ulRunNumber, _poCoorCommandLine ) ;


    APPEND_STATUS_TO_REPLY() ;

    return eReturnStatus ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
/*
ECoorMessageStatus CMessageCoorExecuter::BeginStore               ( const unsigned long     _uiStoreNumber,
                                                                    CMessageCommandLine*    _poCoorCommandLine ) 
{ 
    ECoorMessageStatus  eReturnStatus = Reply_Ok ;

    char                sReplyText[KiMaxLengthReplyTextToCoor] = "<Done>" ;

    //\\//

    CMessageCoorPrinter::BeginStore               ( _uiStoreNumber,
                                                    _poCoorCommandLine ) ;


    APPEND_STATUS_TO_REPLY() ;

    return eReturnStatus ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
/*/
ECoorMessageStatus CMessageCoorExecuter::EndStore                 ( const unsigned long     _uiStoreNumber,
                                                                    CMessageCommandLine*    _poCoorCommandLine ) 
{ 
    ECoorMessageStatus  eReturnStatus = Reply_Ok ;

    char                sReplyText[KiMaxLengthReplyTextToCoor] = "<Done>" ;

    //\\//

    CMessageCoorPrinter::EndStore                 ( _uiStoreNumber,
                                                    _poCoorCommandLine ) ;

    APPEND_STATUS_TO_REPLY() ;

    return eReturnStatus ;
}













/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ECoorMessageStatus  CMessageCoorExecuter::Exclude_Tower   ( ETT_Chan_Type           _eTTType,
                                                            TAllTriggerTowers*      _pxTT,
                                                            CMessageCommandLine*    _poCoorCommandLine ) 

{ 
    ECoorMessageStatus  eReturnStatus = Reply_Ok ;

    char                sReplyText[KiMaxLengthReplyTextToCoor] = "<Done>" ;

    //\\//

    CMessageCoorPrinter::Exclude_Tower    ( _eTTType, 
                                            _pxTT, 
                                            _poCoorCommandLine ) ;

    if IF_L1CAL_OPERATIONAL()
    if IF_TT_RANGE_WITHIN_COVERAGE( _pxTT )
    {
        if ( CL1Cal::mg_poL1Cal->f_okExclude_Tower ( _eTTType, _pxTT ) != Ok )
            eReturnStatus = Reply_Bad ;

        CHECK_FOR_IO_ERROR() ;
    }

    APPEND_STATUS_TO_REPLY() ;

    return eReturnStatus ;
}




/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ECoorMessageStatus  CMessageCoorExecuter::RefSetEMEtProg  ( const unsigned int      _uiRefSetNum, 
                                                            TAllTriggerTowers*      _pxTT,
                                                            const double            _fdThreshold,
                                                            CMessageCommandLine*    _poCoorCommandLine ) 

{ 
    ECoorMessageStatus  eReturnStatus = Reply_Ok ;

    char                sReplyText[KiMaxLengthReplyTextToCoor] = "<Done>" ;

    //\\//

    CMessageCoorPrinter::RefSetEMEtProg   ( _uiRefSetNum, 
                                            _pxTT, 
                                            _fdThreshold,
                                            _poCoorCommandLine ) ;
    if IF_L1CAL_OPERATIONAL()
    if IF_TT_RANGE_WITHIN_COVERAGE( _pxTT )
    {
#if defined ( VME_IO_FAKE )  // development version can dump reference sets
        if ( _fdThreshold < 0.01 )
            CL1Cal::mg_poL1Cal->f_okRefSetEMEtDump ( _uiRefSetNum, _pxTT, _fdThreshold ) ;
        else
#endif
        if ( CL1Cal::mg_poL1Cal->f_okRefSetEMEtProg ( _uiRefSetNum, _pxTT, _fdThreshold ) != Ok )
            eReturnStatus = Reply_Bad ;

        CHECK_FOR_IO_ERROR() ;
    }

    APPEND_STATUS_TO_REPLY() ;

    return eReturnStatus ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ECoorMessageStatus  CMessageCoorExecuter::RefSetJetEtProg ( const unsigned int      _uiRefSetNum, 
                                                            TAllTriggerTowers*      _pxTT,
                                                            const double            _fdThreshold,
                                                            CMessageCommandLine*    _poCoorCommandLine ) 

{ 
    ECoorMessageStatus  eReturnStatus = Reply_Ok ;

    char                sReplyText[KiMaxLengthReplyTextToCoor] = "<Done>" ;

    //\\//

    CMessageCoorPrinter::RefSetJetEtProg  ( _uiRefSetNum, 
                                            _pxTT, 
                                            _fdThreshold,
                                            _poCoorCommandLine ) ;

    if IF_L1CAL_OPERATIONAL()
    if IF_TT_RANGE_WITHIN_COVERAGE( _pxTT )
    {
#if defined ( VME_IO_FAKE )  // development version can dump reference sets
        if ( _fdThreshold < 0.01 )
            CL1Cal::mg_poL1Cal->f_okRefSetJetEtDump ( _uiRefSetNum, _pxTT, _fdThreshold ) ;
        else
#endif
        if ( CL1Cal::mg_poL1Cal->f_okRefSetJetEtProg ( _uiRefSetNum, _pxTT, _fdThreshold ) != Ok )
            eReturnStatus = Reply_Bad ;

        CHECK_FOR_IO_ERROR() ;
    }

    APPEND_STATUS_TO_REPLY() ;

    return eReturnStatus ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ECoorMessageStatus  CMessageCoorExecuter::RefSetTTEtProg  ( const unsigned int      _uiRefSetNum, 
                                                            TAllTriggerTowers*      _pxTT,
                                                            const double            _fdThreshold,
                                                            CMessageCommandLine*    _poCoorCommandLine ) 

{ 
    ECoorMessageStatus  eReturnStatus = Reply_Ok ;

    char                sReplyText[KiMaxLengthReplyTextToCoor] = "<Done>" ;

    //\\//

    CMessageCoorPrinter::RefSetTTEtProg   ( _uiRefSetNum, 
                                            _pxTT, 
                                            _fdThreshold,
                                            _poCoorCommandLine ) ;
    if IF_L1CAL_OPERATIONAL()
    if IF_TT_RANGE_WITHIN_COVERAGE( _pxTT )
    {
#if defined ( VME_IO_FAKE )  // development version can dump reference sets
        if ( _fdThreshold < 0.01 )
            CL1Cal::mg_poL1Cal->f_okRefSetTTEtDump ( _uiRefSetNum, _pxTT, _fdThreshold ) ;
        else
#endif
        if ( CL1Cal::mg_poL1Cal->f_okRefSetTTEtProg ( _uiRefSetNum, _pxTT, _fdThreshold ) != Ok )
            eReturnStatus = Reply_Bad ;

        CHECK_FOR_IO_ERROR() ;
    }

    APPEND_STATUS_TO_REPLY() ;

    return eReturnStatus ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ECoorMessageStatus  CMessageCoorExecuter::RefSetEMEtDeall         ( const unsigned int      _uiRefSetNum, 
                                                                    CMessageCommandLine*    _poCoorCommandLine ) 

{ 
    ECoorMessageStatus  eReturnStatus = Reply_Ok ;

    char                sReplyText[KiMaxLengthReplyTextToCoor] = "<Done>" ;

    //\\//

    CMessageCoorPrinter::RefSetEMEtDeall  ( _uiRefSetNum, 
                                            _poCoorCommandLine ) ;

    if IF_L1CAL_OPERATIONAL()
    {
        if ( CL1Cal::mg_poL1Cal->f_okRefSetEMEtDeall ( _uiRefSetNum ) != Ok )
            eReturnStatus = Reply_Bad ;

        CHECK_FOR_IO_ERROR() ;
    }


    APPEND_STATUS_TO_REPLY() ;

    return eReturnStatus ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ECoorMessageStatus  CMessageCoorExecuter::RefSetJetEtDeall        ( const unsigned int      _uiRefSetNum, 
                                                                    CMessageCommandLine*    _poCoorCommandLine ) 

{ 
    ECoorMessageStatus  eReturnStatus = Reply_Ok ;

    char                sReplyText[KiMaxLengthReplyTextToCoor] = "<Done>" ;

    //\\//

    CMessageCoorPrinter::RefSetJetEtDeall ( _uiRefSetNum, 
                                            _poCoorCommandLine ) ;

    if IF_L1CAL_OPERATIONAL()
    {
        if ( CL1Cal::mg_poL1Cal->f_okRefSetJetEtDeall ( _uiRefSetNum ) != Ok )
            eReturnStatus = Reply_Bad ;

        CHECK_FOR_IO_ERROR() ;
    }

    APPEND_STATUS_TO_REPLY() ;

    return eReturnStatus ;
}


/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ECoorMessageStatus  CMessageCoorExecuter::RefSetTTEtDeall         ( const unsigned int      _uiRefSetNum, 
                                                                    CMessageCommandLine*    _poCoorCommandLine ) 

{ 
    ECoorMessageStatus  eReturnStatus = Reply_Ok ;

    char                sReplyText[KiMaxLengthReplyTextToCoor] = "<Done>" ;

    //\\//

    CMessageCoorPrinter::RefSetTTEtDeall  ( _uiRefSetNum, 
                                            _poCoorCommandLine ) ;

    if IF_L1CAL_OPERATIONAL()
    {
        if ( CL1Cal::mg_poL1Cal->f_okRefSetTTEtDeall ( _uiRefSetNum ) != Ok )
            eReturnStatus = Reply_Bad ;

        CHECK_FOR_IO_ERROR() ;
    }


    APPEND_STATUS_TO_REPLY() ;

    return eReturnStatus ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ECoorMessageStatus  CMessageCoorExecuter::L2MaskEM        ( TAllTriggerTowers*      _pxTT,
                                                            const unsigned int      _uiRefSetNum, 
                                                            CMessageCommandLine*    _poCoorCommandLine ) 

{ 
    ECoorMessageStatus  eReturnStatus = Reply_Ok ;

    char                sReplyText[KiMaxLengthReplyTextToCoor] = "<Done>" ;

    //\\//

    CMessageCoorPrinter::L2MaskEM         ( _pxTT, 
                                            _uiRefSetNum, 
                                            _poCoorCommandLine ) ;
    if IF_L1CAL_OPERATIONAL()
    if IF_TT_RANGE_WITHIN_COVERAGE( _pxTT )
    {
        if ( CL1Cal::mg_poL1Cal->f_okL2MaskEM ( _pxTT, _uiRefSetNum ) != Ok )
            eReturnStatus = Reply_Bad ;

        CHECK_FOR_IO_ERROR() ;
    }

    APPEND_STATUS_TO_REPLY() ;

    return eReturnStatus ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ECoorMessageStatus  CMessageCoorExecuter::L2MaskJet       ( TAllTriggerTowers*      _pxTT,
                                                            const unsigned int      _uiRefSetNum, 
                                                            CMessageCommandLine*    _poCoorCommandLine ) 

{ 
    ECoorMessageStatus  eReturnStatus = Reply_Ok ;

    char                sReplyText[KiMaxLengthReplyTextToCoor] = "<Done>" ;

    //\\//

    CMessageCoorPrinter::L2MaskJet        ( _pxTT, 
                                            _uiRefSetNum, 
                                            _poCoorCommandLine ) ;
    if IF_L1CAL_OPERATIONAL()
    if IF_TT_RANGE_WITHIN_COVERAGE( _pxTT )
    {
        if ( CL1Cal::mg_poL1Cal->f_okL2MaskJet ( _pxTT, _uiRefSetNum ) != Ok )
            eReturnStatus = Reply_Bad ;

        CHECK_FOR_IO_ERROR() ;
    }

    APPEND_STATUS_TO_REPLY() ;

    return eReturnStatus ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ECoorMessageStatus  CMessageCoorExecuter::L2MaskTau       ( TAllTriggerTowers*      _pxTT,
                                                            const unsigned int      _uiRefSetNum, 
                                                            CMessageCommandLine*    _poCoorCommandLine ) 

{ 
    ECoorMessageStatus  eReturnStatus = Reply_Ok ;

    char                sReplyText[KiMaxLengthReplyTextToCoor] = "<Done>" ;

    //\\//

    CMessageCoorPrinter::L2MaskTau        ( _pxTT, 
                                            _uiRefSetNum, 
                                            _poCoorCommandLine ) ;
    if IF_L1CAL_OPERATIONAL()
    if IF_TT_RANGE_WITHIN_COVERAGE( _pxTT )
    {
        if ( CL1Cal::mg_poL1Cal->f_okL2MaskTau ( _pxTT, _uiRefSetNum ) != Ok )
            eReturnStatus = Reply_Bad ;

        CHECK_FOR_IO_ERROR() ;
    }

    APPEND_STATUS_TO_REPLY() ;

    return eReturnStatus ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ECoorMessageStatus  CMessageCoorExecuter::AlgoUseEmFraction       ( TAllTriggerTowers*      _pxTT,
                                                                    bool                    _lUseEmFraction,
                                                                    CMessageCommandLine*    _poCoorCommandLine ) 

{ 
    ECoorMessageStatus  eReturnStatus = Reply_Ok ;

    char                sReplyText[KiMaxLengthReplyTextToCoor] = "<Done>" ;

    //\\//

    CMessageCoorPrinter::AlgoUseEmFraction    ( _pxTT,
                                                _lUseEmFraction,
                                                _poCoorCommandLine ) ;

    if IF_L1CAL_OPERATIONAL()
    {
        if ( CL1Cal::mg_poL1Cal->f_okAlgoUseEmFraction    ( _pxTT,
                                                            _lUseEmFraction ) != Ok )
            eReturnStatus = Reply_Bad ;

        CHECK_FOR_IO_ERROR() ;
    }


    APPEND_STATUS_TO_REPLY() ;

    return eReturnStatus ;
}


/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ECoorMessageStatus  CMessageCoorExecuter::AlgoUseEmIsolation      ( TAllTriggerTowers*      _pxTT,
                                                                    bool                    _lUseEmIsolation,
                                                                    CMessageCommandLine*    _poCoorCommandLine ) 

{ 
    ECoorMessageStatus  eReturnStatus = Reply_Ok ;

    char                sReplyText[KiMaxLengthReplyTextToCoor] = "<Done>" ;

    //\\//

    CMessageCoorPrinter::AlgoUseEmIsolation   ( _pxTT,
                                                _lUseEmIsolation,
                                                _poCoorCommandLine ) ;

    if IF_L1CAL_OPERATIONAL()
    {
        if ( CL1Cal::mg_poL1Cal->f_okAlgoUseEmIsolation   ( _pxTT,
                                                            _lUseEmIsolation ) != Ok )
            eReturnStatus = Reply_Bad ;

        CHECK_FOR_IO_ERROR() ;
    }


    APPEND_STATUS_TO_REPLY() ;

    return eReturnStatus ;
}


/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ECoorMessageStatus  CMessageCoorExecuter::AlgoEmIsolationEtCut      ( TAllTriggerTowers*      _pxTT,
                                                                      const double            _fdThreshold,
                                                                      CMessageCommandLine*    _poCoorCommandLine ) 

{ 
    ECoorMessageStatus  eReturnStatus = Reply_Ok ;

    char                sReplyText[KiMaxLengthReplyTextToCoor] = "<Done>" ;

    //\\//

    CMessageCoorPrinter::AlgoEmIsolationEtCut ( _pxTT,
                                                _fdThreshold,
                                                _poCoorCommandLine ) ;

    if IF_L1CAL_OPERATIONAL()
    {
        if ( CL1Cal::mg_poL1Cal->f_okAlgoEmIsolationEtCut ( _pxTT,
                                                            _fdThreshold ) != Ok )
            eReturnStatus = Reply_Bad ;

        CHECK_FOR_IO_ERROR() ;
    }


    APPEND_STATUS_TO_REPLY() ;

    return eReturnStatus ;
}


/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ECoorMessageStatus  CMessageCoorExecuter::AlgoTauEtCoreCut        ( TAllTriggerTowers*      _pxTT,
                                                                    const double            _fdThreshold,
                                                                    CMessageCommandLine*    _poCoorCommandLine ) 

{ 
    ECoorMessageStatus  eReturnStatus = Reply_Ok ;

    char                sReplyText[KiMaxLengthReplyTextToCoor] = "<Done>" ;

    //\\//

    CMessageCoorPrinter::AlgoTauEtCoreCut      ( _pxTT,
                                                _fdThreshold,
                                                _poCoorCommandLine ) ;

    if IF_L1CAL_OPERATIONAL()
    {
        if ( CL1Cal::mg_poL1Cal->f_okAlgoTauEtCoreCut     ( _pxTT,
                                                            _fdThreshold ) != Ok )
            eReturnStatus = Reply_Bad ;

        CHECK_FOR_IO_ERROR() ;
    }


    APPEND_STATUS_TO_REPLY() ;

    return eReturnStatus ;
}


/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ECoorMessageStatus  CMessageCoorExecuter::AlgoGlobalSumEtCut      ( TAllTriggerTowers*      _pxTT,
                                                                    const double            _fdThreshold,
                                                                    CMessageCommandLine*    _poCoorCommandLine ) 

{ 
    ECoorMessageStatus  eReturnStatus = Reply_Ok ;

    char                sReplyText[KiMaxLengthReplyTextToCoor] = "<Done>" ;

    //\\//

    CMessageCoorPrinter::AlgoGlobalSumEtCut   ( _pxTT,
                                                _fdThreshold,
                                                _poCoorCommandLine ) ;

    if IF_L1CAL_OPERATIONAL()
    {
        if ( CL1Cal::mg_poL1Cal->f_okAlgoGlobalSumEtCut   ( _pxTT,
                                                            _fdThreshold ) != Ok )
            eReturnStatus = Reply_Bad ;

        CHECK_FOR_IO_ERROR() ;
    }


    APPEND_STATUS_TO_REPLY() ;

    return eReturnStatus ;
}


/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ECoorMessageStatus  CMessageCoorExecuter::AlgoUseIcrJetObjects    ( TAllTriggerTowers*      _pxTT,
                                                                    bool                    _lUseIcr,
                                                                    CMessageCommandLine*    _poCoorCommandLine ) 

{ 
    ECoorMessageStatus  eReturnStatus = Reply_Ok ;

    char                sReplyText[KiMaxLengthReplyTextToCoor] = "<Done>" ;

    //\\//

    CMessageCoorPrinter::AlgoUseIcrJetObjects ( _pxTT,
                                                _lUseIcr,
                                                _poCoorCommandLine ) ;

    if IF_L1CAL_OPERATIONAL()
    {
        if ( CL1Cal::mg_poL1Cal->f_okAlgoUseIcrJetObjects ( _pxTT,
                                                            _lUseIcr ) != Ok )
            eReturnStatus = Reply_Bad ;

        CHECK_FOR_IO_ERROR() ;
    }


    APPEND_STATUS_TO_REPLY() ;

    return eReturnStatus ;
}


/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ECoorMessageStatus  CMessageCoorExecuter::AlgoUseIcrGlobalSums    ( TAllTriggerTowers*      _pxTT,
                                                                    bool                    _lUseIcr,
                                                                    CMessageCommandLine*    _poCoorCommandLine ) 

{ 
    ECoorMessageStatus  eReturnStatus = Reply_Ok ;

    char                sReplyText[KiMaxLengthReplyTextToCoor] = "<Done>" ;

    //\\//

    CMessageCoorPrinter::AlgoUseIcrGlobalSums ( _pxTT,
                                                _lUseIcr,
                                                _poCoorCommandLine ) ;

    if IF_L1CAL_OPERATIONAL()
    {
        if ( CL1Cal::mg_poL1Cal->f_okAlgoUseIcrGlobalSums ( _pxTT,
                                                            _lUseIcr ) != Ok )
            eReturnStatus = Reply_Bad ;

        CHECK_FOR_IO_ERROR() ;
    }


    APPEND_STATUS_TO_REPLY() ;

    return eReturnStatus ;
}


/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ECoorMessageStatus  CMessageCoorExecuter::TermEmAll               ( const unsigned int      _uiComparatorNum, 
                                                                    const unsigned int      _uiRefSetNum, 
                                                                    const unsigned int      _uiThreshold,
                                                                    CMessageCommandLine*    _poCoorCommandLine ) 

{ 
    ECoorMessageStatus  eReturnStatus = Reply_Ok ;

    char                sReplyText[KiMaxLengthReplyTextToCoor] = "<Done>" ;

    //\\//

    CMessageCoorPrinter::TermEmAll
                                      ( _uiComparatorNum, 
                                        _uiRefSetNum, 
                                        _uiThreshold,
                                        _poCoorCommandLine ) ;

    if IF_L1CAL_OPERATIONAL()
    {
        if ( CL1Cal::mg_poL1Cal->f_okTermEmAll  ( _uiComparatorNum, 
                                                  _uiRefSetNum,
                                                  _uiThreshold ) != Ok )
            eReturnStatus = Reply_Bad ;

        CHECK_FOR_IO_ERROR() ;
    }


    APPEND_STATUS_TO_REPLY() ;

    return eReturnStatus ;
}


/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ECoorMessageStatus  CMessageCoorExecuter::TermEmCentral           ( const unsigned int      _uiComparatorNum, 
                                                                    const unsigned int      _uiRefSetNum, 
                                                                    const unsigned int      _uiThreshold,
                                                                    CMessageCommandLine*    _poCoorCommandLine ) 

{ 
    ECoorMessageStatus  eReturnStatus = Reply_Ok ;

    char                sReplyText[KiMaxLengthReplyTextToCoor] = "<Done>" ;

    //\\//

    CMessageCoorPrinter::TermEmCentral
                                      ( _uiComparatorNum, 
                                        _uiRefSetNum, 
                                        _uiThreshold,
                                        _poCoorCommandLine ) ;

    if IF_L1CAL_OPERATIONAL()
    {
        if ( CL1Cal::mg_poL1Cal->f_okTermEmCentral  ( _uiComparatorNum, 
                                                      _uiRefSetNum,
                                                      _uiThreshold ) != Ok )
            eReturnStatus = Reply_Bad ;

        CHECK_FOR_IO_ERROR() ;
    }


    APPEND_STATUS_TO_REPLY() ;

    return eReturnStatus ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ECoorMessageStatus  CMessageCoorExecuter::TermJetAll              ( const unsigned int      _uiComparatorNum, 
                                                                    const unsigned int      _uiRefSetNum, 
                                                                    const unsigned int      _uiThreshold,
                                                                    CMessageCommandLine*    _poCoorCommandLine ) 

{ 
    ECoorMessageStatus  eReturnStatus = Reply_Ok ;

    char                sReplyText[KiMaxLengthReplyTextToCoor] = "<Done>" ;

    //\\//

    CMessageCoorPrinter::TermJetAll
                                      ( _uiComparatorNum, 
                                        _uiRefSetNum, 
                                        _uiThreshold,
                                        _poCoorCommandLine ) ;

    if IF_L1CAL_OPERATIONAL()
    {
        if ( CL1Cal::mg_poL1Cal->f_okTermJetAll ( _uiComparatorNum, 
                                                  _uiRefSetNum,
                                                  _uiThreshold ) != Ok )
            eReturnStatus = Reply_Bad ;


        CHECK_FOR_IO_ERROR() ;
    }


    APPEND_STATUS_TO_REPLY() ;

    return eReturnStatus ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ECoorMessageStatus  CMessageCoorExecuter::TermJetCentral          ( const unsigned int      _uiComparatorNum, 
                                                                    const unsigned int      _uiRefSetNum, 
                                                                    const unsigned int      _uiThreshold,
                                                                    CMessageCommandLine*    _poCoorCommandLine ) 

{ 
    ECoorMessageStatus  eReturnStatus = Reply_Ok ;

    char                sReplyText[KiMaxLengthReplyTextToCoor] = "<Done>" ;

    //\\//

    CMessageCoorPrinter::TermJetCentral
                                      ( _uiComparatorNum, 
                                        _uiRefSetNum, 
                                        _uiThreshold,
                                        _poCoorCommandLine ) ;

    if IF_L1CAL_OPERATIONAL()
    {
        if ( CL1Cal::mg_poL1Cal->f_okTermJetCentral ( _uiComparatorNum, 
                                                      _uiRefSetNum,
                                                      _uiThreshold ) != Ok )
            eReturnStatus = Reply_Bad ;

        CHECK_FOR_IO_ERROR() ;
    }


    APPEND_STATUS_TO_REPLY() ;

    return eReturnStatus ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ECoorMessageStatus  CMessageCoorExecuter::TermTauAll              ( const unsigned int      _uiComparatorNum, 
                                                                    const unsigned int      _uiRefSetNum, 
                                                                    const unsigned int      _uiTauRatioNum, 
                                                                    const unsigned int      _uiThreshold,
                                                                    CMessageCommandLine*    _poCoorCommandLine ) 

{ 
    ECoorMessageStatus  eReturnStatus = Reply_Ok ;

    char                sReplyText[KiMaxLengthReplyTextToCoor] = "<Done>" ;

    //\\//

    CMessageCoorPrinter::TermTauAll
                                      ( _uiComparatorNum, 
                                        _uiRefSetNum, 
                                        _uiTauRatioNum,
                                        _uiThreshold,
                                        _poCoorCommandLine ) ;

    if IF_L1CAL_OPERATIONAL()
    {
        if ( CL1Cal::mg_poL1Cal->f_okTermTauAll ( _uiComparatorNum, 
                                                  _uiRefSetNum,
                                                  _uiTauRatioNum,
                                                  _uiThreshold ) != Ok )
            eReturnStatus = Reply_Bad ;

        CHECK_FOR_IO_ERROR() ;
    }


    APPEND_STATUS_TO_REPLY() ;

    return eReturnStatus ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ECoorMessageStatus  CMessageCoorExecuter::TermTauCentral          ( const unsigned int      _uiComparatorNum, 
                                                                    const unsigned int      _uiRefSetNum, 
                                                                    const unsigned int      _uiTauRatioNum, 
                                                                    const unsigned int      _uiThreshold,
                                                                    CMessageCommandLine*    _poCoorCommandLine ) 

{ 
    ECoorMessageStatus  eReturnStatus = Reply_Ok ;

    char                sReplyText[KiMaxLengthReplyTextToCoor] = "<Done>" ;

    //\\//

    CMessageCoorPrinter::TermTauCentral
                                      ( _uiComparatorNum, 
                                        _uiRefSetNum, 
                                        _uiTauRatioNum,
                                        _uiThreshold,
                                        _poCoorCommandLine ) ;

    if IF_L1CAL_OPERATIONAL()
    {
        if ( CL1Cal::mg_poL1Cal->f_okTermTauCentral ( _uiComparatorNum, 
                                                      _uiRefSetNum,
                                                      _uiTauRatioNum,
                                                      _uiThreshold ) != Ok )
            eReturnStatus = Reply_Bad ;

        CHECK_FOR_IO_ERROR() ;
    }


    APPEND_STATUS_TO_REPLY() ;

    return eReturnStatus ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ECoorMessageStatus  CMessageCoorExecuter::TermIsolEmAll           ( const unsigned int      _uiComparatorNum, 
                                                                    const unsigned int      _uiRefSetNum, 
                                                                    const unsigned int      _uiThreshold,
                                                                    CMessageCommandLine*    _poCoorCommandLine ) 

{ 
    ECoorMessageStatus  eReturnStatus = Reply_Ok ;

    char                sReplyText[KiMaxLengthReplyTextToCoor] = "<Done>" ;

    //\\//

    CMessageCoorPrinter::TermIsolEmAll
                                      ( _uiComparatorNum, 
                                        _uiRefSetNum, 
                                        _uiThreshold,
                                        _poCoorCommandLine ) ;

    if IF_L1CAL_OPERATIONAL()
    {
        if ( CL1Cal::mg_poL1Cal->f_okTermIsolEmAll      ( _uiComparatorNum, 
                                                          _uiRefSetNum,
                                                          _uiThreshold ) != Ok )
            eReturnStatus = Reply_Bad ;

        CHECK_FOR_IO_ERROR() ;
    }


    APPEND_STATUS_TO_REPLY() ;

    return eReturnStatus ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ECoorMessageStatus  CMessageCoorExecuter::TermIsolEmCentral       ( const unsigned int      _uiComparatorNum, 
                                                                    const unsigned int      _uiRefSetNum, 
                                                                    const unsigned int      _uiThreshold,
                                                                    CMessageCommandLine*    _poCoorCommandLine ) 

{ 
    ECoorMessageStatus  eReturnStatus = Reply_Ok ;

    char                sReplyText[KiMaxLengthReplyTextToCoor] = "<Done>" ;

    //\\//

    CMessageCoorPrinter::TermIsolEmCentral
                                      ( _uiComparatorNum, 
                                        _uiRefSetNum, 
                                        _uiThreshold,
                                        _poCoorCommandLine ) ;

    if IF_L1CAL_OPERATIONAL()
    {
        if ( CL1Cal::mg_poL1Cal->f_okTermIsolEmCentral      ( _uiComparatorNum, 
                                                              _uiRefSetNum,
                                                              _uiThreshold ) != Ok )
            eReturnStatus = Reply_Bad ;

        CHECK_FOR_IO_ERROR() ;
    }


    APPEND_STATUS_TO_REPLY() ;

    return eReturnStatus ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ECoorMessageStatus  CMessageCoorExecuter::TermMissEt              ( const unsigned int      _uiComparatorNum, 
                                                                    const double            _fdThreshold,
                                                                    CMessageCommandLine*    _poCoorCommandLine ) 

{ 
    ECoorMessageStatus  eReturnStatus = Reply_Ok ;

    char                sReplyText[KiMaxLengthReplyTextToCoor] = "<Done>" ;

    //\\//

    CMessageCoorPrinter::TermMissEt   
                                      ( _uiComparatorNum, 
                                        _fdThreshold,
                                        _poCoorCommandLine ) ;

    if IF_L1CAL_OPERATIONAL()
    {
        if ( CL1Cal::mg_poL1Cal->f_okTermMissEt     ( _uiComparatorNum, 
                                                      _fdThreshold ) != Ok )
            eReturnStatus = Reply_Bad ;

        CHECK_FOR_IO_ERROR() ;
    }


    APPEND_STATUS_TO_REPLY() ;

    return eReturnStatus ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ECoorMessageStatus  CMessageCoorExecuter::TermTotEt               ( const unsigned int      _uiComparatorNum, 
                                                                    const double            _fdThreshold,
                                                                    CMessageCommandLine*    _poCoorCommandLine ) 

{ 
    ECoorMessageStatus  eReturnStatus = Reply_Ok ;

    char                sReplyText[KiMaxLengthReplyTextToCoor] = "<Done>" ;

    //\\//

    CMessageCoorPrinter::TermTotEt  
                                      ( _uiComparatorNum, 
                                        _fdThreshold,
                                        _poCoorCommandLine ) ;

    if IF_L1CAL_OPERATIONAL()
    {
        if ( CL1Cal::mg_poL1Cal->f_okTermTotEt      ( _uiComparatorNum, 
                                                      _fdThreshold ) != Ok )
            eReturnStatus = Reply_Bad ;

        CHECK_FOR_IO_ERROR() ;
    }


    APPEND_STATUS_TO_REPLY() ;

    return eReturnStatus ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ECoorMessageStatus  CMessageCoorExecuter::TermAcoplanarJet        ( const unsigned int      _uiComparatorNum, 
                                                                    const unsigned int      _uiRefSetNum1, 
                                                                    const unsigned int      _uiRefSetNum2,
                                                                    const unsigned int      _uiDelta_Phi,
                                                                    CMessageCommandLine*    _poCoorCommandLine ) 

{ 
    ECoorMessageStatus  eReturnStatus = Reply_Ok ;

    char                sReplyText[KiMaxLengthReplyTextToCoor] = "<Done>" ;

    //\\//

    CMessageCoorPrinter::TermAcoplanarJet
                                      ( _uiComparatorNum, 
                                        _uiRefSetNum1, 
                                        _uiRefSetNum2,
                                        _uiDelta_Phi,
                                        _poCoorCommandLine ) ;

    if IF_L1CAL_OPERATIONAL()
    {
        if ( CL1Cal::mg_poL1Cal->f_okTermAcoplanarJet   ( _uiComparatorNum, 
                                                          _uiRefSetNum1,
                                                          _uiRefSetNum2,
                                                          _uiDelta_Phi ) != Ok )
            eReturnStatus = Reply_Bad ;

        CHECK_FOR_IO_ERROR() ;
    }


    APPEND_STATUS_TO_REPLY() ;

    return eReturnStatus ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ECoorMessageStatus  CMessageCoorExecuter::TermNonCollinearLowEtJet( const unsigned int      _uiComparatorNum, 
                                                                    const unsigned int      _uiRefSetNum1, 
                                                                    const unsigned int      _uiRefSetNum2,
                                                                    const unsigned int      _uiDelta_Phi,
                                                                    CMessageCommandLine*    _poCoorCommandLine ) 

{ 
    ECoorMessageStatus  eReturnStatus = Reply_Ok ;

    char                sReplyText[KiMaxLengthReplyTextToCoor] = "<Done>" ;

    //\\//

    CMessageCoorPrinter::TermNonCollinearLowEtJet
                                      ( _uiComparatorNum, 
                                        _uiRefSetNum1, 
                                        _uiRefSetNum2,
                                        _uiDelta_Phi,
                                        _poCoorCommandLine ) ;

    if IF_L1CAL_OPERATIONAL()
    {
        if ( CL1Cal::mg_poL1Cal->f_okTermNonCollinearLowEtJet   ( _uiComparatorNum, 
                                                                  _uiRefSetNum1,
                                                                  _uiRefSetNum2,
                                                                  _uiDelta_Phi ) != Ok )
            eReturnStatus = Reply_Bad ;

        CHECK_FOR_IO_ERROR() ;
    }


    APPEND_STATUS_TO_REPLY() ;

    return eReturnStatus ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ECoorMessageStatus  CMessageCoorExecuter::TermNonCollinearMonoJet ( const unsigned int      _uiComparatorNum, 
                                                                    const unsigned int      _uiRefSetNum1, 
                                                                    const unsigned int      _uiRefSetNum2,
                                                                    const unsigned int      _uiDelta_Phi,
                                                                    CMessageCommandLine*    _poCoorCommandLine ) 

{ 
    ECoorMessageStatus  eReturnStatus = Reply_Ok ;

    char                sReplyText[KiMaxLengthReplyTextToCoor] = "<Done>" ;

    //\\//

    CMessageCoorPrinter::TermNonCollinearMonoJet
                                      ( _uiComparatorNum, 
                                        _uiRefSetNum1, 
                                        _uiRefSetNum2,
                                        _uiDelta_Phi,
                                        _poCoorCommandLine ) ;

    if IF_L1CAL_OPERATIONAL()
    {
        if ( CL1Cal::mg_poL1Cal->f_okTermNonCollinearMonoJet( _uiComparatorNum, 
                                                              _uiRefSetNum1,
                                                              _uiRefSetNum2,
                                                              _uiDelta_Phi ) != Ok )
            eReturnStatus = Reply_Bad ;

        CHECK_FOR_IO_ERROR() ;
    }


    APPEND_STATUS_TO_REPLY() ;

    return eReturnStatus ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ECoorMessageStatus  CMessageCoorExecuter::TermJetFree             ( const unsigned int      _uiComparatorNum, 
                                                                    const unsigned int      _uiRefSetNum1, 
                                                                    const unsigned int      _uiRefSetNum2,
                                                                    const unsigned int      _uiDelta_Phi,
                                                                    CMessageCommandLine*    _poCoorCommandLine ) 

{ 
    ECoorMessageStatus  eReturnStatus = Reply_Ok ;

    char                sReplyText[KiMaxLengthReplyTextToCoor] = "<Done>" ;

    //\\//

    CMessageCoorPrinter::TermJetFree
                                      ( _uiComparatorNum, 
                                        _uiRefSetNum1, 
                                        _uiRefSetNum2,
                                        _uiDelta_Phi,
                                        _poCoorCommandLine ) ;

    if IF_L1CAL_OPERATIONAL()
    {
        if ( CL1Cal::mg_poL1Cal->f_okTermJetFree( _uiComparatorNum, 
                                                  _uiRefSetNum1,
                                                  _uiRefSetNum2,
                                                  _uiDelta_Phi ) != Ok )
            eReturnStatus = Reply_Bad ;

        CHECK_FOR_IO_ERROR() ;
    }


    APPEND_STATUS_TO_REPLY() ;

    return eReturnStatus ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ECoorMessageStatus  CMessageCoorExecuter::TermBackToBackEm        ( const unsigned int      _uiComparatorNum, 
                                                                    const unsigned int      _uiRefSetNum, 
                                                                    const unsigned int      _uiDelta_Phi,
                                                                    CMessageCommandLine*    _poCoorCommandLine ) 

{ 
    ECoorMessageStatus  eReturnStatus = Reply_Ok ;

    char                sReplyText[KiMaxLengthReplyTextToCoor] = "<Done>" ;

    //\\//

    CMessageCoorPrinter::TermBackToBackEm
                                      ( _uiComparatorNum, 
                                        _uiRefSetNum,
                                        _uiDelta_Phi, 
                                        _poCoorCommandLine ) ;

    if IF_L1CAL_OPERATIONAL()
    {
        if ( CL1Cal::mg_poL1Cal->f_okTermBackToBackEm   ( _uiComparatorNum, 
                                                          _uiRefSetNum,
                                                          _uiDelta_Phi  ) != Ok )
            eReturnStatus = Reply_Bad ;

        CHECK_FOR_IO_ERROR() ;
    }


    APPEND_STATUS_TO_REPLY() ;

    return eReturnStatus ;
}

