/*-------------------------------------------------------------------------------*/
  /*
     MessageCoorParser.cpp
     7-Oct-99
     28-Oct-99 Coor Command Parsing split onto separate files
     22-Sep-2005 Restructred for L1Cal_IIb_TCC
  */
/*-------------------------------------------------------------------------------*/
#include "L1Cal_IIb_TCC/StdAfx.h"

#include "L1Cal_IIb_TCC/MessageCommand.h"
#include                            "L1Cal_IIb_TCC/MessageCoorParser.h"

#include "L1Cal_IIb_TCC/MessageParserMacros.h"

#include "L1Cal_IIb_TCC/MessageKeywordsCoorGeneric.h"
#include "L1Cal_IIb_TCC/MessageKeywordsCoorL1Cal.h"

#include "L1Cal_IIb_TCC/L1Cal.h" // to get min-max eta-phi ranges

#include "L1Cal_IIb_TCC/HandleConsole.h"
#include "L1Cal_IIb_TCC/UtilsStrings.h"

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
#define     KiBase10    10
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/


/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
CMessageCoorParser::CMessageCoorParser () 
{
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
CMessageCoorParser::~CMessageCoorParser () 
{
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
void CMessageCoorParser::AddKeywords      ( CMessageTokenizer*      _poTokenizer ) 
{

    ADD_COOR_COMMAND ( Configure_FPGAs     ) ;
    
    ADD_COOR_COMMAND ( Init                ) ;

    ADD_COOR_COMMAND ( L1Cal_Initialize    ) ;

    ADD_COOR_COMMAND ( Iterative_DeSkew    ) ;


    ADD_COOR_COMMAND ( Start_run           ) ;

    ADD_COOR_COMMAND ( Stop_run            ) ;

    
    ADD_COOR_COMMAND ( Begin_block         ) ;

    ADD_COOR_COMMAND ( End_block           ) ;

    ADD_COOR_COMMAND ( Configure           ) ;

    ADD_COOR_COMMAND ( Abort               ) ;


    ADD_COOR_COMMAND ( Pause_Run           ) ;

    ADD_COOR_COMMAND ( Resume_Run          ) ;


    ADD_COOR_COMMAND ( Begin_store         ) ;

    ADD_COOR_COMMAND ( End_store           ) ;


    ADD_COOR_SUB_COMMAND ( Deallocate      ) ;


    ADD_COOR_SUB_COMMAND ( Comment         ) ;


    ADD_COOR_COMMAND     ( L1Cal_Ref_Set             ) ;

    ADD_COOR_SUB_COMMAND ( EM_Et_Ref_Set             ) ;
    ADD_COOR_SUB_COMMAND ( Jet_Et_Ref_Set            ) ;
    ADD_COOR_SUB_COMMAND ( TT_Et_Ref_Set             ) ;

    ADD_COOR_COMMAND     ( L1CAL_L2Mask              ) ;

    ADD_COOR_SUB_COMMAND ( EM_L2Mask                 ) ;
    ADD_COOR_SUB_COMMAND ( Jet_L2Mask                ) ;
    ADD_COOR_SUB_COMMAND ( Tau_L2Mask                ) ;

    
    ADD_COOR_COMMAND     ( L1CAL_Algo_Param          ) ;

    ADD_COOR_SUB_COMMAND ( EM_Isolation              ) ;
    ADD_COOR_SUB_COMMAND ( EM_Isolation_Et_Cut       ) ;
    ADD_COOR_SUB_COMMAND ( TT_Tau_EtCore_Cut         ) ;
    ADD_COOR_SUB_COMMAND ( TT_Global_Sum_Et_Cut      ) ;
    ADD_COOR_SUB_COMMAND ( Jet_Objects               ) ;
    ADD_COOR_SUB_COMMAND ( Global_Sums               ) ;


    ADD_COOR_COMMAND     ( L1CAL_to_L1FW             ) ;

    ADD_COOR_SUB_COMMAND ( EM_All_Term               ) ;
    ADD_COOR_SUB_COMMAND ( EM_Central_Term           ) ;
    ADD_COOR_SUB_COMMAND ( Jet_All_Term              ) ;
    ADD_COOR_SUB_COMMAND ( Jet_Central_Term          ) ;
    ADD_COOR_SUB_COMMAND ( Tau_All_Term              ) ;
    ADD_COOR_SUB_COMMAND ( Tau_Central_Term          ) ;
    ADD_COOR_SUB_COMMAND ( Isolated_EM_All_Term      ) ;
    ADD_COOR_SUB_COMMAND ( Isolated_EM_Central_Term  ) ;
    ADD_COOR_SUB_COMMAND ( Missing_Et_Term           ) ;
    ADD_COOR_SUB_COMMAND ( Total_Et_Term             ) ;
    ADD_COOR_SUB_COMMAND ( Acoplanar_Jet_Term        ) ;
    ADD_COOR_SUB_COMMAND ( NonCollinear_LowEt_Jet_Term ) ;
    ADD_COOR_SUB_COMMAND ( NonCollinear_Mono_Jet_Term  ) ;
    ADD_COOR_SUB_COMMAND ( Jet_Free_Term               ) ;
    ADD_COOR_SUB_COMMAND ( Back_To_Back_EM_Term      ) ;

    
    ADD_COOR_COMMAND     ( L1Cal_Exclude             ) ;

    ADD_COOR_SUB_COMMAND ( EM_Tower                  ) ;
    ADD_COOR_SUB_COMMAND ( HD_Tower                  ) ;


    ADD_COOR_KEYWORD     ( Use_EMFraction            ) ;
    ADD_COOR_KEYWORD     ( Ignore_EMFraction         ) ;
    ADD_COOR_KEYWORD     ( Use_EMIsolation           ) ;
    ADD_COOR_KEYWORD     ( Ignore_EMIsolation        ) ;
    ADD_COOR_KEYWORD     ( Use_ICR                   ) ;
    ADD_COOR_KEYWORD     ( Ignore_ICR                ) ;

    ADD_COOR_KEYWORD     ( Use_Ref_Set               ) ;
    ADD_COOR_KEYWORD     ( Use_Ref_Set1              ) ;
    ADD_COOR_KEYWORD     ( Use_Ref_Set2              ) ;
    ADD_COOR_KEYWORD     ( Use_Ref_Set_Veto          ) ;
    ADD_COOR_KEYWORD     ( Min_Delta_Phi_Slices      ) ;
    ADD_COOR_KEYWORD     ( Use_Tau_Ratio_Set         ) ;

    ADD_COOR_KEYWORD     ( Count_Threshold           ) ;
    ADD_COOR_KEYWORD     ( Energy_Threshold          ) ;

    ADD_COOR_KEYWORD     ( TT_Eta                    ) ;

    ADD_COOR_KEYWORD     ( TT_Phi                    ) ;

    ADD_COOR_KEYWORD     ( Value                     ) ;
    
}


/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ECoorMessageStatus      CMessageCoorParser::ParseMessage    ( CMessageCommandLine*  _poCoorCommandLine ) 
{ 
    //\\//

    //We have already enforced during the tokenization that the first token is a Keyword
    if ( _poCoorCommandLine->m_lTokenList.FirstEntry()->GetValue() <= Ki_CoorCmd_MaxID_Generic )
        
        return ParseInterpret_Generic ( _poCoorCommandLine ) ;

    else 
        
        return ParseInterpret_L1Cal   ( _poCoorCommandLine ) ;

}



/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ECoorMessageStatus      CMessageCoorParser::ParseInterpret_Generic ( CMessageCommandLine*  _poCoorCommandLine ) 
{ 
    ECoorMessageStatus  eReturnStatus = Reply_Ok ;

    char                sReplyText[KiMaxLengthReplyTextToCoor] ;

    TBinomialEntry< TokenLocAndType, unsigned int > * poTokenEntry ;

    TList < unsigned int, EItemPolarity >           lObjectList ;

    CMessageCommand*    poCoorCommand ;

    ETokenType          eTokenType ;
    unsigned int        uiCoorCmdCode ;
    unsigned int        uiItemIndex ;
    unsigned int        uiItemIndexLower ;
    unsigned int        uiItemIndexUpper ;

    //\\//

    poTokenEntry = _poCoorCommandLine->m_lTokenList.FirstEntry( ) ;

    //We have already enforced during the tokenization that the first token is a Keyword
    uiCoorCmdCode   = poTokenEntry->GetValue ( ) ;

    //One command may in fact contain several commands 
    //(e.g. Set Spec Trig Andor Terms, and Prescaler Ratio on one line)
    //Create the first command in the Command List here, 
    //and other commands will be created later.
    _poCoorCommandLine->m_lCommandList.AddEntry ( ) ;

    //Get at the Created command
    poCoorCommand = _poCoorCommandLine->m_lCommandList.LastEntry()->GetAtKey( ) ;

    //record the command Code
    poCoorCommand->m_uiCoorCmdCode = uiCoorCmdCode ;

    poTokenEntry = _poCoorCommandLine->m_lTokenList.NextEntry( poTokenEntry ) ;

    switch ( uiCoorCmdCode )
    {
        /////////////////////////////////////////////////////////////////////////////
        // start with comment = empty command line
        /////////////////////////////////////////////////////////////////////////////

        case Ki_CoorSubCmd_Comment                       :
            break ;

        /////////////////////////////////////////////////////////////////////////////
        // single keyword commands
        /////////////////////////////////////////////////////////////////////////////
        case Ki_CoorCmd_Configure_FPGAs                  ://[Configure_FPGAs]
        case Ki_CoorCmd_L1Cal_Initialize                 ://[L1Cal_Initialize]
        case Ki_CoorCmd_Iterative_DeSkew                 ://[Iterative_DeSkew]

        case Ki_CoorCmd_Begin_block                      ://[Begin_Block] -- No Reply
        case Ki_CoorCmd_End_block                        ://[End_Block] -- No Reply
        case Ki_CoorCmd_Configure                        ://[Configure] -- No Reply
        case Ki_CoorCmd_Abort                            ://[Abort] -- No Reply
        {
            if ( poTokenEntry != NULL ) 
                ERROR_UNEXPECTED_TOKEN() ;

        } ; break ;

        /////////////////////////////////////////////////////////////////////////////
        // single keyword with single number 
        /////////////////////////////////////////////////////////////////////////////
        case Ki_CoorCmd_Pause_Run                        ://[Pause_Run    runno]
        case Ki_CoorCmd_Resume_Run                       ://[Resume_Run   runno]
        case Ki_CoorCmd_Stop_run                         ://[Stop_Run     runno]
        case Ki_CoorCmd_Begin_store                      ://[Begin_Store  storenum]
        case Ki_CoorCmd_End_store                        ://[End_Store    storenum]
        {
            if ( poTokenEntry == NULL ) 
                ERROR_MISSING_NUMBER() ;

            eTokenType = poTokenEntry->GetKey().eTokenType ;

            if ( eTokenType != TokenIsValueAsserted ) 
                ERROR_NUMBER_NOT_ASSERTED() ;

            poCoorCommand->m_uiValue        = poTokenEntry->GetValue ( ) ;
            poCoorCommand->m_eValuePolarity = ItemAsserted ;


            poTokenEntry = _poCoorCommandLine->m_lTokenList.NextEntry( poTokenEntry ) ;

            if ( poTokenEntry != NULL ) 
                ERROR_UNEXPECTED_TOKEN() ;

        } ; break ;


        /////////////////////////////////////////////////////////////////////////////
        // single keyword with single number and Item List  
        /////////////////////////////////////////////////////////////////////////////
        case Ki_CoorCmd_Start_run                        ://[Start_Run    runno   spectrigs]
        {
            if ( poTokenEntry == NULL ) 
                ERROR_MISSING_NUMBER() ;

            eTokenType = poTokenEntry->GetKey().eTokenType ;

            if ( eTokenType != TokenIsValueAsserted ) 
                ERROR_NUMBER_NOT_ASSERTED() ;

            poCoorCommand->m_uiValue        = poTokenEntry->GetValue ( ) ;
            poCoorCommand->m_eValuePolarity = ItemAsserted ;


            poTokenEntry = _poCoorCommandLine->m_lTokenList.NextEntry( poTokenEntry ) ;

            if ( poTokenEntry == NULL )
                ERROR_MISSING_NUMBER_LIST() ;

            while ( poTokenEntry != NULL )
            {

                eTokenType = poTokenEntry->GetKey().eTokenType ;

                if ( ( eTokenType != TokenIsValueAsserted )
                  && ( eTokenType != TokenIsValueLower ) )
                    ERROR_NUMBER_NOT_ASSERTED() ;

                if ( eTokenType != TokenIsValueLower )
                {
                    uiItemIndex = poTokenEntry->GetValue ( ) ;

                    if ( uiItemIndex > KiMaxIndexInCoorCmd_SpecTrig )
                        ERROR_OBJECT_NUMBER_OUT_OF_RANGE() ;

                    poCoorCommand->m_aubItemPolarity[uiItemIndex] = ItemAsserted ;
                }
                else
                {
                    uiItemIndexLower = poTokenEntry->GetValue ( ) ;
                    
                    poTokenEntry = _poCoorCommandLine->m_lTokenList.NextEntry( poTokenEntry ) ;

                    if ( poTokenEntry == NULL )
                        ERROR_MISSING_UPPER_BOUND() ;

                    eTokenType = poTokenEntry->GetKey().eTokenType ;
    
                    if ( eTokenType != TokenIsValueUpper )
                        ERROR_MISSING_UPPER_BOUND() ;

                    uiItemIndexUpper = poTokenEntry->GetValue ( ) ;

                    if ( ( uiItemIndexLower > KiMaxIndexInCoorCmd_SpecTrig )
                      || ( uiItemIndexUpper > KiMaxIndexInCoorCmd_SpecTrig ) )
                      ERROR_OBJECT_NUMBER_OUT_OF_RANGE() ;

                    if ( uiItemIndexLower > uiItemIndexUpper ) 
                        SWAP_UI_VALUES( uiItemIndexLower, uiItemIndexUpper ) ;
                    
                    for ( uiItemIndex  = uiItemIndexLower ; 
                          uiItemIndex <= uiItemIndexUpper ; 
                          uiItemIndex ++ ) 
                        poCoorCommand->m_aubItemPolarity[uiItemIndex] = ItemAsserted ;
                }

                poTokenEntry = _poCoorCommandLine->m_lTokenList.NextEntry( poTokenEntry ) ;
            }

        } ; break ;


        /////////////////////////////////////////////////////////////////////////////

        default :
            ERROR_KEYWORD_ILLEGAL_AT_START() ;

    } ;

Done:

    return eReturnStatus ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ECoorMessageStatus      CMessageCoorParser::ParseInterpret_L1Cal ( CMessageCommandLine*  _poCoorCommandLine ) 
{ 
    ECoorMessageStatus  eReturnStatus = Reply_Ok ;

    char                sReplyText[KiMaxLengthReplyTextToCoor] ;

    TBinomialEntry< TokenLocAndType, unsigned int > * poTokenEntry ;
//    TBinomialEntry< ETokenType, unsigned int > * poTokenFirstSubCommand ;

    TList < unsigned int, EItemPolarity >           lObjectList ;
//    TBinomialEntry < unsigned int, EItemPolarity >* poObjectEntry ;

    CMessageCommand*    poCoorCommand ;

    ETokenType          eTokenType ;

    unsigned int        uiCoorCmdCode ;
    unsigned int        uiCoorCmdSubCode ;
    unsigned int        uiKeyword ;
 
    unsigned int        uiRefSetNum ;
    unsigned int        uiObjectIndex ;
    unsigned int        uiItemIndex ;
    unsigned int        uiItemIndexLower ;
    unsigned int        uiItemIndexUpper ;
    ETT_Eta_Sign        ePolarityIndexLower ;
    ETT_Eta_Sign        ePolarityIndexUpper ;

    //\\//

    poTokenEntry = _poCoorCommandLine->m_lTokenList.FirstEntry( ) ;

    //We have already enforced during the tokenization that the first token is a Keyword
    uiCoorCmdCode   = poTokenEntry->GetValue ( ) ;

    //Create the first command in the Command List
    _poCoorCommandLine->m_lCommandList.AddEntry ( ) ;

    //Get at the Created command
    poCoorCommand = _poCoorCommandLine->m_lCommandList.LastEntry()->GetAtKey( ) ;

    //record the command Code
    poCoorCommand->m_uiCoorCmdCode = uiCoorCmdCode ;

    poTokenEntry = _poCoorCommandLine->m_lTokenList.NextEntry( poTokenEntry ) ;

    switch ( uiCoorCmdCode )
    {
        /////////////////////////////////////////////////////////////////////////////
        // Reference Set Difinition
        //[L1Cal_Ref_Set EM_Et_Ref_Set  0 TT_Eta(-20:20) TT_Phi(1:32) Energy_Threshold 10.0]
        //[L1Cal_Ref_Set Jet_Et_Ref_Set 0 TT_Eta(-20:20) TT_Phi(1:32) Energy_Threshold 10.0]
        //[L1Cal_Ref_Set TT_Et_Ref_Set  0 TT_Eta(-20:20) TT_Phi(1:32) Energy_Threshold 10.0]
        //[L1Cal_Ref_Set EM_Et_Ref_Set  0 Deallocate]
        //[L1Cal_Ref_Set Jet_Et_Ref_Set 0 Deallocate]
        //[L1Cal_Ref_Set TT_Et_Ref_Set  0 Deallocate]
        /////////////////////////////////////////////////////////////////////////////
        case Ki_CoorCmd_L1Cal_Ref_Set                     ://[Ki_CoorCmd_L1Cal_Ref_Set]
        {
            // next keyword should be the type of reference set 
            if ( poTokenEntry == NULL )
                ERROR_MISSING_SUB_KEYWORD() ;

            eTokenType = poTokenEntry->GetKey().eTokenType ;

            if ( eTokenType != TokenIsKeyword )
                ERROR_MISSING_SUB_KEYWORD() ;

            uiCoorCmdSubCode   = poTokenEntry->GetValue ( ) ;

            if ( ( uiCoorCmdSubCode != Ki_CoorSubCmd_EM_Et_Ref_Set  )
              && ( uiCoorCmdSubCode != Ki_CoorSubCmd_Jet_Et_Ref_Set )
              && ( uiCoorCmdSubCode != Ki_CoorSubCmd_TT_Et_Ref_Set  ) ) 
              ERROR_ILLEGAL_SUB_KEYWORD() ;

            //copy the SubCommand Code 
            poCoorCommand->m_uiCoorSubCmdCode = uiCoorCmdSubCode ;

            // next keyword should be the reference set number
            GET_TAB_REFSET_NUMBER ( poCoorCommand->m_uiRefSetNum ) ;

            // The range of TT refset number is more restrictive
            if ( ( uiCoorCmdSubCode == Ki_CoorSubCmd_TT_Et_Ref_Set  ) 
              && ( uiRefSetNum      > KiMaxIndexInCoorCmd_RefSet_TT ) )
              ERROR_OBJECT_NUMBER_OUT_OF_RANGE() ;

            // Next Token may be either "TT_Eta", "TT_Phi", or directly 'Energy_Threshold"
            // or "Deallocate"
            poTokenEntry = _poCoorCommandLine->m_lTokenList.NextEntry( poTokenEntry ) ;

            Look_For_TT_Range_RefSet :

                if ( poTokenEntry == NULL )
                    ERROR_MISSING_SUB_KEYWORD() ;

                eTokenType = poTokenEntry->GetKey().eTokenType ;

                if ( eTokenType != TokenIsKeyword )
                    ERROR_MISSING_SUB_KEYWORD() ;

                uiKeyword = poTokenEntry->GetValue ( ) ;

                if ( uiKeyword == Ki_CoorSubCmd_Deallocate ) // only for the reference sets
                {
                    switch ( uiCoorCmdSubCode )
                    { 
                        case Ki_CoorSubCmd_EM_Et_Ref_Set :
                            poCoorCommand->m_uiCoorSubCmdCode = Ki_CoorSubCmd_EM_Et_Ref_Set_Deallocate ;
                            break ;
                        case Ki_CoorSubCmd_Jet_Et_Ref_Set :
                            poCoorCommand->m_uiCoorSubCmdCode = Ki_CoorSubCmd_Jet_Et_Ref_Set_Deallocate ;
                            break ;
                        case Ki_CoorSubCmd_TT_Et_Ref_Set :
                            poCoorCommand->m_uiCoorSubCmdCode = Ki_CoorSubCmd_TT_Et_Ref_Set_Deallocate ;
                            break ;
                        default :
                            ERROR_UNEXPECTED_TOKEN() ;
                            break ;
                    }

                    // there should be no more tokens on the line
                    poTokenEntry = _poCoorCommandLine->m_lTokenList.NextEntry( poTokenEntry ) ;
                    if ( poTokenEntry  != NULL )
                        ERROR_UNEXPECTED_TOKEN() ;

                    goto Done ;
                }


                if ( ( uiKeyword == Ki_CoorKeyword_TT_Eta )
                  || ( uiKeyword == Ki_CoorKeyword_TT_Phi ) )
                {
                    PARSE_TT_RANGE( Look_For_TT_Range_RefSet ) ;
                }

            if ( ( ( uiCoorCmdSubCode == Ki_CoorSubCmd_EM_Et_Ref_Set  ) 
                || ( uiCoorCmdSubCode == Ki_CoorSubCmd_Jet_Et_Ref_Set )
                || ( uiCoorCmdSubCode == Ki_CoorSubCmd_TT_Et_Ref_Set  ) )
              && ( uiKeyword != Ki_CoorKeyword_Energy_Threshold ) ) 
            ERROR_UNEXPECTED_TOKEN() ;

            //Get the Threshold Value
            GET_THRESHOLD_FLOAT_VALUE() ;


            // there should be no more tokens on the line
            poTokenEntry = _poCoorCommandLine->m_lTokenList.NextEntry( poTokenEntry ) ;
            if ( poTokenEntry  != NULL )
                ERROR_UNEXPECTED_TOKEN() ;

            // if the Eta range was omitted, this means all Eta values are selected
            FILL_ETA_RANGE_IF_OMITTED() ;
                                                                                                 
            // if the Phi range was omitted, this means all Phi values are selected              
            FILL_PHI_RANGE_IF_OMITTED() ;
                                                                                                 
            //one last thing to check is 
            //whether the TT range does not cut any TAB Chip boundary
            unsigned int uiRelItemIndex ;
            bool lTTRangeIllegal = false ;

            //Phi is simpler to think about, let's do it first
            for ( uiItemIndex  = KiMin_TT_Phi ; uiItemIndex <= KiMax_TT_Phi ; uiItemIndex += KiMax_TabChip_Phi_Width ) 
            for ( uiRelItemIndex  = 0 ; uiRelItemIndex  < KiMax_TabChip_Phi_Width ; uiRelItemIndex++ ) 
            if ( poCoorCommand->m_xTT.alTT_Phi[uiItemIndex] 
              != poCoorCommand->m_xTT.alTT_Phi[uiItemIndex+uiRelItemIndex] )
                lTTRangeIllegal = true ;

            //Let's do pos and neg eta in parallel
            for ( uiItemIndex  = KiMin_TT_Eta_Magn ; uiItemIndex <= KiMax_TT_Eta_Magn ; uiItemIndex += KiMax_TabChip_Eta_Width ) 
            for ( uiRelItemIndex  = 0 ; uiRelItemIndex  < KiMax_TabChip_Eta_Width ; uiRelItemIndex++ ) 
            if ( ( poCoorCommand->m_xTT.aalTT_Eta[eTT_Eta_Pos][uiItemIndex] 
                != poCoorCommand->m_xTT.aalTT_Eta[eTT_Eta_Pos][uiItemIndex+uiRelItemIndex] )
              || ( poCoorCommand->m_xTT.aalTT_Eta[eTT_Eta_Neg][uiItemIndex] 
                != poCoorCommand->m_xTT.aalTT_Eta[eTT_Eta_Neg][uiItemIndex+uiRelItemIndex] ) )
                lTTRangeIllegal = true ;

            if ( lTTRangeIllegal )
                ERROR_ILLEGAL_TAB_CHIP_RANGE() ;


        } break ;

        /////////////////////////////////////////////////////////////////////////////
        //[L1CAL_L2Mask      EM_L2Mask      TT_Eta(-16:16) TT_Phi(1:32) Use_Ref_Set   0]
        //[L1CAL_L2Mask      Jet_L2Mask     TT_Eta(-16:16) TT_Phi(1:32) Use_Ref_Set   0]
        //[L1CAL_L2Mask      Tau_L2Mask     TT_Eta(-16:16) TT_Phi(1:32) Use_Ref_Set   0]
        //[L1CAL_Algo_Param  EM_Isolation    use_EMFraction]
        //[L1CAL_Algo_Param  EM_Isolation    ignore_EMFraction]
        //[L1CAL_Algo_Param  EM_Isolation    use_EMIsolation]
        //[L1CAL_Algo_Param  EM_Isolation    ignore_EMIsolation]
        //[L1CAL_Algo_Param  EM_Isolation_Et_Cut     TT_Eta(-16:16) TT_Phi(1:32) Energy_Threshold 1.0]
        //[L1CAL_Algo_Param  TT_Tau_EtCore_Cut       TT_Eta(-16:16) TT_Phi(1:32) Energy_Threshold 8.0]
        //[L1CAL_Algo_Param  TT_Global_Sum_Et_Cut    TT_Eta(-20:20) TT_Phi(1:32) Energy_Threshold 1.0]
        //[L1CAL_Algo_Param  Jet_Objects     use_ICR]
        //[L1CAL_Algo_Param  Jet_Objects     ignore_ICR]
        //[L1CAL_Algo_Param  Global_Sums     use_ICR]
        //[L1CAL_Algo_Param  Global_Sums     ignore_ICR]
        /////////////////////////////////////////////////////////////////////////////
        case Ki_CoorCmd_L1CAL_L2Mask :
        case Ki_CoorCmd_L1CAL_Algo_Param :
        {
            // next keyword should be the Param Type
            if ( poTokenEntry == NULL )
                ERROR_MISSING_SUB_KEYWORD() ;

            eTokenType = poTokenEntry->GetKey().eTokenType ;

            if ( eTokenType != TokenIsKeyword )
                ERROR_MISSING_SUB_KEYWORD() ;

            uiCoorCmdSubCode   = poTokenEntry->GetValue ( ) ;

            if ( ( ( uiCoorCmdCode    == Ki_CoorCmd_L1CAL_L2Mask ) 
                && ( uiCoorCmdSubCode != Ki_CoorSubCmd_EM_L2Mask              )
                && ( uiCoorCmdSubCode != Ki_CoorSubCmd_Jet_L2Mask             )
                && ( uiCoorCmdSubCode != Ki_CoorSubCmd_Tau_L2Mask             ) )
              || ( ( uiCoorCmdCode    == Ki_CoorCmd_L1CAL_Algo_Param ) 
                && ( uiCoorCmdSubCode != Ki_CoorSubCmd_EM_Isolation           )
                && ( uiCoorCmdSubCode != Ki_CoorSubCmd_EM_Isolation_Et_Cut    ) 
                && ( uiCoorCmdSubCode != Ki_CoorSubCmd_TT_Tau_EtCore_Cut      ) 
                && ( uiCoorCmdSubCode != Ki_CoorSubCmd_TT_Global_Sum_Et_Cut   ) 
                && ( uiCoorCmdSubCode != Ki_CoorSubCmd_Jet_Objects            ) 
                && ( uiCoorCmdSubCode != Ki_CoorSubCmd_Global_Sums            ) ) )
              ERROR_ILLEGAL_SUB_KEYWORD() ;

            //copy the SubCommand Code 
            poCoorCommand->m_uiCoorSubCmdCode = uiCoorCmdSubCode ;

            // Next Token may be either "TT_Eta", "TT_Phi", or directly 'Energy_Threshold", "use_EMFraction", etc
            poTokenEntry = _poCoorCommandLine->m_lTokenList.NextEntry( poTokenEntry ) ;

            Look_For_TT_Range_Algo_Param :

                if ( poTokenEntry == NULL )
                    ERROR_MISSING_SUB_KEYWORD() ;

                eTokenType = poTokenEntry->GetKey().eTokenType ;

                if ( eTokenType != TokenIsKeyword )
                    ERROR_MISSING_SUB_KEYWORD() ;

                uiKeyword = poTokenEntry->GetValue ( ) ;

                if ( ( uiKeyword == Ki_CoorKeyword_TT_Eta )
                  || ( uiKeyword == Ki_CoorKeyword_TT_Phi ) )
                {
                        PARSE_TT_RANGE( Look_For_TT_Range_Algo_Param ) ;
                }

            if ( ( ( uiCoorCmdCode    == Ki_CoorCmd_L1CAL_L2Mask ) 
                && ( uiKeyword != Ki_CoorKeyword_Use_Ref_Set ) ) 
              || ( ( uiCoorCmdSubCode == Ki_CoorSubCmd_EM_Isolation  ) 
                && ( uiKeyword != Ki_CoorKeyword_Use_EMFraction ) 
                && ( uiKeyword != Ki_CoorKeyword_Ignore_EMFraction ) 
                && ( uiKeyword != Ki_CoorKeyword_Use_EMIsolation ) 
                && ( uiKeyword != Ki_CoorKeyword_Ignore_EMIsolation ) ) 
              || ( ( ( uiCoorCmdSubCode == Ki_CoorSubCmd_Jet_Objects  ) 
                  || ( uiCoorCmdSubCode == Ki_CoorSubCmd_Global_Sums  ) )
                && ( uiKeyword != Ki_CoorKeyword_Use_ICR ) 
                && ( uiKeyword != Ki_CoorKeyword_Ignore_ICR ) ) 
              || ( ( ( uiCoorCmdSubCode == Ki_CoorSubCmd_EM_Isolation_Et_Cut    ) 
                  || ( uiCoorCmdSubCode == Ki_CoorSubCmd_TT_Tau_EtCore_Cut      ) 
                  || ( uiCoorCmdSubCode == Ki_CoorSubCmd_TT_Global_Sum_Et_Cut   ) )
                && ( uiKeyword != Ki_CoorKeyword_Energy_Threshold ) ) ) 
            ERROR_UNEXPECTED_TOKEN() ;


            if      ( uiCoorCmdSubCode == Ki_CoorSubCmd_EM_Isolation  )
            {
                if      ( uiKeyword == Ki_CoorKeyword_Use_EMFraction ) 
                    poCoorCommand->m_uiCoorSubCmdCode = Ki_CoorSubCmd_EM_Isolation_Use_EMFraction ;
                else if ( uiKeyword == Ki_CoorKeyword_Ignore_EMFraction ) 
                    poCoorCommand->m_uiCoorSubCmdCode = Ki_CoorSubCmd_EM_Isolation_Ignore_EMFraction ;
                else if ( uiKeyword == Ki_CoorKeyword_Use_EMIsolation ) 
                    poCoorCommand->m_uiCoorSubCmdCode = Ki_CoorSubCmd_EM_Isolation_Use_EMIsolation ;
                else if ( uiKeyword == Ki_CoorKeyword_Ignore_EMIsolation ) 
                    poCoorCommand->m_uiCoorSubCmdCode = Ki_CoorSubCmd_EM_Isolation_Ignore_EMIsolation ;
            }

            else if ( uiCoorCmdSubCode == Ki_CoorSubCmd_Jet_Objects  )
            {
                if      ( uiKeyword == Ki_CoorKeyword_Use_ICR ) 
                    poCoorCommand->m_uiCoorSubCmdCode = Ki_CoorSubCmd_Jet_Objects_Use_ICR ;
                else if ( uiKeyword == Ki_CoorKeyword_Ignore_ICR ) 
                    poCoorCommand->m_uiCoorSubCmdCode = Ki_CoorSubCmd_Jet_Objects_Ignore_ICR ;
            }

            else if ( uiCoorCmdSubCode == Ki_CoorSubCmd_Global_Sums  )
            {
                if      ( uiKeyword == Ki_CoorKeyword_Use_ICR ) 
                    poCoorCommand->m_uiCoorSubCmdCode = Ki_CoorSubCmd_Global_Sums_Use_ICR ;
                else if ( uiKeyword == Ki_CoorKeyword_Ignore_ICR ) 
                    poCoorCommand->m_uiCoorSubCmdCode = Ki_CoorSubCmd_Global_Sums_Ignore_ICR ;
            }

            else if ( uiCoorCmdCode    == Ki_CoorCmd_L1CAL_L2Mask ) 
            {
                // next keyword is the reference set number
                GET_TAB_REFSET_NUMBER ( poCoorCommand->m_uiRefSetNum ) ;
            }

            else 
            {
                // next keyword should be the threshold value
                GET_THRESHOLD_FLOAT_VALUE() ;
            }

            
            // there should be no more tokens on the line
            poTokenEntry = _poCoorCommandLine->m_lTokenList.NextEntry( poTokenEntry ) ;
            if ( poTokenEntry  != NULL )
                ERROR_UNEXPECTED_TOKEN() ;

            // if the Eta range was omitted, this means all Eta values are selected
            FILL_ETA_RANGE_IF_OMITTED() ;
                                                                                                 
            // if the Phi range was omitted, this means all Phi values are selected              
            FILL_PHI_RANGE_IF_OMITTED() ;
                                                                                                 
            //one last thing to check is 
            //whether the TT range does not cut any TAB Chip boundary
            unsigned int uiRelItemIndex ;
            bool lTTRangeIllegal = false ;

            //Phi is simpler to think about, let's do it first
            for ( uiItemIndex  = KiMin_TT_Phi ; uiItemIndex <= KiMax_TT_Phi ; uiItemIndex += KiMax_TabChip_Phi_Width ) 
            for ( uiRelItemIndex  = 0 ; uiRelItemIndex  < KiMax_TabChip_Phi_Width ; uiRelItemIndex++ ) 
            if ( poCoorCommand->m_xTT.alTT_Phi[uiItemIndex] 
              != poCoorCommand->m_xTT.alTT_Phi[uiItemIndex+uiRelItemIndex] )
                lTTRangeIllegal = true ;

            //Let's do pos and neg eta in parallel
            for ( uiItemIndex  = KiMin_TT_Eta_Magn ; uiItemIndex <= KiMax_TT_Eta_Magn ; uiItemIndex += KiMax_TabChip_Eta_Width ) 
            for ( uiRelItemIndex  = 0 ; uiRelItemIndex  < KiMax_TabChip_Eta_Width ; uiRelItemIndex++ ) 
            if ( ( poCoorCommand->m_xTT.aalTT_Eta[eTT_Eta_Pos][uiItemIndex] 
                != poCoorCommand->m_xTT.aalTT_Eta[eTT_Eta_Pos][uiItemIndex+uiRelItemIndex] )
              || ( poCoorCommand->m_xTT.aalTT_Eta[eTT_Eta_Neg][uiItemIndex] 
                != poCoorCommand->m_xTT.aalTT_Eta[eTT_Eta_Neg][uiItemIndex+uiRelItemIndex] ) )
                lTTRangeIllegal = true ;

            if ( lTTRangeIllegal )
                ERROR_ILLEGAL_TAB_CHIP_RANGE() ;

        } break ;


        
        /////////////////////////////////////////////////////////////////////////////
        // And/Or Term programming
        //[L1CAL_to_L1FW  EM_All_Term                 1  Use_Ref_Set   0 Count_Threshold 4]
        //[L1CAL_to_L1FW  EM_Central_Term             1  Use_Ref_Set   0 Count_Threshold 4]
        //[L1CAL_to_L1FW  Jet_All_Term                1  Use_Ref_Set   0 Count_Threshold 4]
        //[L1CAL_to_L1FW  Jet_Central_Term            1  Use_Ref_Set   0 Count_Threshold 4]
        //[L1CAL_to_L1FW  Tau_All_Term                1  Use_Ref_Set   0 Use_Tau_Ratio_Set 1 Count_Threshold 2]
        //[L1CAL_to_L1FW  Tau_Central_Term            1  Use_Ref_Set   0 Use_Tau_Ratio_Set 1 Count_Threshold 2]
        //[L1CAL_to_L1FW  Isolated_EM_All_Term        1  Use_Ref_Set   0 Count_Threshold 2]
        //[L1CAL_to_L1FW  Isolated_EM_Central_Term    1  Use_Ref_Set   0 Count_Threshold 2]
        //[L1CAL_to_L1FW  Missing_Et_Term             1  Energy_Threshold 10.0]
        //[L1CAL_to_L1FW  Total_Et_Term               1  Energy_Threshold 10.0]
        //[L1CAL_to_L1FW  Acoplanar_Jet_Term          1  Use_Ref_Set   3 Use_Ref_Set_Veto 4]
        //[L1CAL_to_L1FW  Acoplanar_Jet_Term          0  Use_Ref_Set1  1  Use_Ref_Set2      2  Min_Delta_Phi_Slices  2]
        //[L1CAL_to_L1FW  NonCollinear_LowEt_Jet_Term 0  Use_Ref_Set1  1  Use_Ref_Set2      2  Min_Delta_Phi_Slices  2]
        //[L1CAL_to_L1FW  NonCollinear_Mono_Jet_Term  0  Use_Ref_Set1  1  Use_Ref_Set2      2  Min_Delta_Phi_Slices  3]
        //[L1CAL_to_L1FW  Jet_Free_Term               0  Use_Ref_Set   1  Use_Ref_Set_Veto  2  Min_Delta_Phi_Slices  3]
        //[L1CAL_to_L1FW  Back_To_Back_EM_Term        1  Use_Ref_Set   0]
        //[L1CAL_to_L1FW  Back_To_Back_EM_Term        1  Use_Ref_Set   0  Min_Delta_Phi_Slices  3]
        /////////////////////////////////////////////////////////////////////////////
        case Ki_CoorCmd_L1CAL_to_L1FW :

            // next keyword should be the type of Term
            if ( poTokenEntry == NULL )
                ERROR_MISSING_SUB_KEYWORD() ;

            eTokenType = poTokenEntry->GetKey().eTokenType ;

            if ( eTokenType != TokenIsKeyword )
                ERROR_MISSING_SUB_KEYWORD() ;

            uiCoorCmdSubCode   = poTokenEntry->GetValue ( ) ;

            if ( ( uiCoorCmdSubCode != Ki_CoorSubCmd_EM_All_Term )
              && ( uiCoorCmdSubCode != Ki_CoorSubCmd_EM_Central_Term )
              && ( uiCoorCmdSubCode != Ki_CoorSubCmd_Jet_All_Term )
              && ( uiCoorCmdSubCode != Ki_CoorSubCmd_Jet_Central_Term )
              && ( uiCoorCmdSubCode != Ki_CoorSubCmd_Tau_All_Term )
              && ( uiCoorCmdSubCode != Ki_CoorSubCmd_Tau_Central_Term )
              && ( uiCoorCmdSubCode != Ki_CoorSubCmd_Isolated_EM_All_Term )
              && ( uiCoorCmdSubCode != Ki_CoorSubCmd_Isolated_EM_Central_Term )
              && ( uiCoorCmdSubCode != Ki_CoorSubCmd_Missing_Et_Term )
              && ( uiCoorCmdSubCode != Ki_CoorSubCmd_Total_Et_Term )
              && ( uiCoorCmdSubCode != Ki_CoorSubCmd_Acoplanar_Jet_Term )
              && ( uiCoorCmdSubCode != Ki_CoorSubCmd_NonCollinear_LowEt_Jet_Term )
              && ( uiCoorCmdSubCode != Ki_CoorSubCmd_NonCollinear_Mono_Jet_Term )
              && ( uiCoorCmdSubCode != Ki_CoorSubCmd_Back_To_Back_EM_Term )
              && ( uiCoorCmdSubCode != Ki_CoorSubCmd_Jet_Free_Term ) )
              ERROR_ILLEGAL_SUB_KEYWORD() ;

            //copy the SubCommand Code 
            poCoorCommand->m_uiCoorSubCmdCode = uiCoorCmdSubCode ;

            // next keyword should be the relative Term number
            poTokenEntry = _poCoorCommandLine->m_lTokenList.NextEntry( poTokenEntry ) ;

            if ( poTokenEntry == NULL )
                ERROR_MISSING_NUMBER() ;

            eTokenType = poTokenEntry->GetKey().eTokenType ;

            if ( eTokenType != TokenIsValueAsserted )
                ERROR_NUMBER_NOT_ASSERTED() ;

            uiObjectIndex = poTokenEntry->GetValue ( ) ;

            if ( ( ( uiCoorCmdSubCode == Ki_CoorSubCmd_EM_All_Term )      // each term command type ...
                && ( uiObjectIndex > KiMaxIndexInCoorCmd_EM_All_Term ) )  // ... has its own max term number
              || ( ( uiCoorCmdSubCode == Ki_CoorSubCmd_EM_Central_Term )
                && ( uiObjectIndex > KiMaxIndexInCoorCmd_EM_Central_Term ) )
              || ( ( uiCoorCmdSubCode == Ki_CoorSubCmd_Jet_All_Term )
                && ( uiObjectIndex > KiMaxIndexInCoorCmd_Jet_All_Term ) )
              || ( ( uiCoorCmdSubCode == Ki_CoorSubCmd_Jet_Central_Term )
                && ( uiObjectIndex > KiMaxIndexInCoorCmd_Jet_Central_Term ) )
              || ( ( uiCoorCmdSubCode == Ki_CoorSubCmd_Tau_All_Term )
                && ( uiObjectIndex > KiMaxIndexInCoorCmd_Tau_All_Term ) )
              || ( ( uiCoorCmdSubCode == Ki_CoorSubCmd_Tau_Central_Term )
                && ( uiObjectIndex > KiMaxIndexInCoorCmd_Tau_Central_Term ) )
              || ( ( uiCoorCmdSubCode == Ki_CoorSubCmd_Isolated_EM_All_Term )
                && ( uiObjectIndex > KiMaxIndexInCoorCmd_Isolated_EM_All_Term ) )
              || ( ( uiCoorCmdSubCode == Ki_CoorSubCmd_Isolated_EM_Central_Term )
                && ( uiObjectIndex > KiMaxIndexInCoorCmd_Isolated_EM_Central_Term ) )
              || ( ( uiCoorCmdSubCode == Ki_CoorSubCmd_Missing_Et_Term )
                && ( uiObjectIndex > KiMaxIndexInCoorCmd_Missing_Et_Term ) )
              || ( ( uiCoorCmdSubCode == Ki_CoorSubCmd_Total_Et_Term )
                && ( uiObjectIndex > KiMaxIndexInCoorCmd_Total_Et_Term ) )
              || ( ( uiCoorCmdSubCode == Ki_CoorSubCmd_Acoplanar_Jet_Term )
                && ( uiObjectIndex > KiMaxIndexInCoorCmd_Acoplanar_Jet_Term ) )
              || ( ( uiCoorCmdSubCode == Ki_CoorSubCmd_NonCollinear_LowEt_Jet_Term )
                && ( uiObjectIndex > KiMaxIndexInCoorCmd_NonCollinear_LowEt_Jet_Term ) )
              || ( ( uiCoorCmdSubCode == Ki_CoorSubCmd_NonCollinear_Mono_Jet_Term )
                && ( uiObjectIndex > KiMaxIndexInCoorCmd_NonCollinear_Mono_Jet_Term ) )
              || ( ( uiCoorCmdSubCode == Ki_CoorSubCmd_Back_To_Back_EM_Term ) 
                && ( uiObjectIndex > KiMaxIndexInCoorCmd_Back_To_Back_EM_Term ) ) 
              || ( ( uiCoorCmdSubCode == Ki_CoorSubCmd_Jet_Free_Term )
                && ( uiObjectIndex > KiMaxIndexInCoorCmd_Jet_Free_Term ) ) )
              ERROR_OBJECT_NUMBER_OUT_OF_RANGE() ;

            //copy the Term Number
            poCoorCommand->m_uiObjectIndex  = uiObjectIndex ;


            if      ( ( uiCoorCmdSubCode == Ki_CoorSubCmd_EM_All_Term               )
                   || ( uiCoorCmdSubCode == Ki_CoorSubCmd_EM_Central_Term           )
                   || ( uiCoorCmdSubCode == Ki_CoorSubCmd_Jet_All_Term              )
                   || ( uiCoorCmdSubCode == Ki_CoorSubCmd_Jet_Central_Term          )
                   || ( uiCoorCmdSubCode == Ki_CoorSubCmd_Tau_All_Term              )
                   || ( uiCoorCmdSubCode == Ki_CoorSubCmd_Tau_Central_Term          )
                   || ( uiCoorCmdSubCode == Ki_CoorSubCmd_Isolated_EM_All_Term      )
                   || ( uiCoorCmdSubCode == Ki_CoorSubCmd_Isolated_EM_Central_Term  ) ) 
            {
                // next keyword should be the word "Use_Ref_Set"
                CHECK_NEXT_TOKEN_IS_KEYWORD( Ki_CoorKeyword_Use_Ref_Set ) ;

                // next keyword should be the reference set number
                GET_GAB_REFSET_NUMBER ( poCoorCommand->m_uiRefSetNum ) ;


                // Tau terms takes an extra argument
                if ( ( uiCoorCmdSubCode == Ki_CoorSubCmd_Tau_All_Term )
                  || ( uiCoorCmdSubCode == Ki_CoorSubCmd_Tau_Central_Term ) ) 
                {
                    // next keyword should be the word "Use_Tau_Ratio_Set"
                    CHECK_NEXT_TOKEN_IS_KEYWORD( Ki_CoorKeyword_Use_Tau_Ratio_Set ) ;

                    GET_GAB_REFSET_NUMBER ( poCoorCommand->m_uiAuxiliary ) ;
                }


                // next keyword should be the word "Count_Threshold"
                CHECK_NEXT_TOKEN_IS_KEYWORD( Ki_CoorKeyword_Count_Threshold ) ;

                GET_THRESHOLD_INT_VALUE() ;

                if ( ( poCoorCommand->m_uiValue < KiMinThreshInCoorCmd_Term ) // cannot be zero
                  || ( ( uiCoorCmdSubCode == Ki_CoorSubCmd_EM_All_Term )      // each term command type ...
                    && ( poCoorCommand->m_uiValue > KiMaxThreshInCoorCmd_EM_All_Term ) )  //... has its own max term number
                  || ( ( uiCoorCmdSubCode == Ki_CoorSubCmd_EM_Central_Term )
                    && ( poCoorCommand->m_uiValue > KiMaxThreshInCoorCmd_EM_Central_Term ) )
                  || ( ( uiCoorCmdSubCode == Ki_CoorSubCmd_Jet_All_Term )
                    && ( poCoorCommand->m_uiValue > KiMaxThreshInCoorCmd_Jet_All_Term ) )
                  || ( ( uiCoorCmdSubCode == Ki_CoorSubCmd_Jet_Central_Term )
                    && ( poCoorCommand->m_uiValue > KiMaxThreshInCoorCmd_Jet_Central_Term ) )
                  || ( ( uiCoorCmdSubCode == Ki_CoorSubCmd_Tau_All_Term )
                    && ( poCoorCommand->m_uiValue > KiMaxThreshInCoorCmd_Tau_All_Term ) )
                  || ( ( uiCoorCmdSubCode == Ki_CoorSubCmd_Tau_Central_Term )
                    && ( poCoorCommand->m_uiValue > KiMaxThreshInCoorCmd_Tau_Central_Term ) )
                  || ( ( uiCoorCmdSubCode == Ki_CoorSubCmd_Isolated_EM_All_Term )
                    && ( poCoorCommand->m_uiValue > KiMaxThreshInCoorCmd_Isolated_EM_All_Term ) )
                  || ( ( uiCoorCmdSubCode == Ki_CoorSubCmd_Isolated_EM_Central_Term )
                    && ( poCoorCommand->m_uiValue > KiMaxThreshInCoorCmd_Isolated_EM_Central_Term ) ) )
                    ERROR_ILLEGAL_VALUE() ;

            }

            else if ( ( uiCoorCmdSubCode == Ki_CoorSubCmd_Missing_Et_Term )
                   || ( uiCoorCmdSubCode == Ki_CoorSubCmd_Total_Et_Term ) ) 
            {
                // next keyword should be the word "Energy_Threshold"
                CHECK_NEXT_TOKEN_IS_KEYWORD( Ki_CoorKeyword_Energy_Threshold ) ;

                // next is the Threshold Value
                GET_THRESHOLD_FLOAT_VALUE() ;

            }



            else if ( ( uiCoorCmdSubCode == Ki_CoorSubCmd_Acoplanar_Jet_Term             )
                   || ( uiCoorCmdSubCode == Ki_CoorSubCmd_NonCollinear_LowEt_Jet_Term    )
                   || ( uiCoorCmdSubCode == Ki_CoorSubCmd_NonCollinear_Mono_Jet_Term     ) ) 
            {

                // next keyword should be the word "Use_Ref_Set1"
                CHECK_NEXT_TOKEN_IS_KEYWORD( Ki_CoorKeyword_Use_Ref_Set1 ) ;

                // next keyword should be the reference set number
                GET_GAB_REFSET_NUMBER ( poCoorCommand->m_uiRefSetNum ) ;


                // next keyword should be the word "Use_Ref_Set2"
                CHECK_NEXT_TOKEN_IS_KEYWORD( Ki_CoorKeyword_Use_Ref_Set2 ) ;

                // next keyword should be the reference set number
                GET_GAB_REFSET_NUMBER ( poCoorCommand->m_uiAuxiliary ) ;


                // next keyword should be the word "Min_Delta_Phi_Slices"
                CHECK_NEXT_TOKEN_IS_KEYWORD( Ki_CoorKeyword_Min_Delta_Phi_Slices ) ;

                // next keyword should be the reference set number
                GET_THRESHOLD_INT_VALUE ( ) ;

            }


            else if ( uiCoorCmdSubCode == Ki_CoorSubCmd_Back_To_Back_EM_Term ) 
            {

                // next keyword should be the word "Use_Ref_Set"
                CHECK_NEXT_TOKEN_IS_KEYWORD( Ki_CoorKeyword_Use_Ref_Set ) ;

                // next keyword should be the reference set number
                GET_GAB_REFSET_NUMBER ( poCoorCommand->m_uiRefSetNum ) ;


                // next keyword should be the word "Min_Delta_Phi_Slices"
                CHECK_NEXT_TOKEN_IS_KEYWORD( Ki_CoorKeyword_Min_Delta_Phi_Slices ) ;

                // next keyword should be the reference set number
                GET_THRESHOLD_INT_VALUE ( ) ;

            }


            else if ( uiCoorCmdSubCode == Ki_CoorSubCmd_Jet_Free_Term ) 
            {

                // next keyword should be the word "Use_Ref_Set"
                CHECK_NEXT_TOKEN_IS_KEYWORD( Ki_CoorKeyword_Use_Ref_Set ) ;

                // next keyword should be the reference set number
                GET_GAB_REFSET_NUMBER ( poCoorCommand->m_uiRefSetNum ) ;


                // next keyword should be the word "Use_Ref_Set_Veto"
                CHECK_NEXT_TOKEN_IS_KEYWORD( Ki_CoorKeyword_Use_Ref_Set_Veto ) ;

                // next keyword should be the reference set number
                GET_GAB_REFSET_NUMBER ( poCoorCommand->m_uiAuxiliary ) ;


                // next keyword should be the word "Min_Delta_Phi_Slices"
                CHECK_NEXT_TOKEN_IS_KEYWORD( Ki_CoorKeyword_Min_Delta_Phi_Slices ) ;

                // next keyword should be the reference set number
                GET_THRESHOLD_INT_VALUE ( ) ;

            }



            // there should be no more tokens on the line
            poTokenEntry = _poCoorCommandLine->m_lTokenList.NextEntry( poTokenEntry ) ;
            if ( poTokenEntry  != NULL )
                ERROR_UNEXPECTED_TOKEN() ;

            break ;

        /////////////////////////////////////////////////////////////////////////////
        //[L1Cal_Exclude  EM_Tower TT_Eta(20) TT_Phi(23)]
        //[L1Cal_Exclude  HD_Tower TT_Eta(20) TT_Phi(23)]
        /////////////////////////////////////////////////////////////////////////////
        case Ki_CoorCmd_L1Cal_Exclude                     ://[Ki_CoorCmd_L1Cal_Exclude]

            // next keyword should be the type of Trigger Tower
            if ( poTokenEntry == NULL )
                ERROR_MISSING_SUB_KEYWORD() ;

            eTokenType = poTokenEntry->GetKey().eTokenType ;

            if ( eTokenType != TokenIsKeyword )
                ERROR_MISSING_SUB_KEYWORD() ;

            uiCoorCmdSubCode   = poTokenEntry->GetValue ( ) ;

            if ( ( uiCoorCmdSubCode != Ki_CoorSubCmd_EM_Tower )
              && ( uiCoorCmdSubCode != Ki_CoorSubCmd_HD_Tower ) ) 
              ERROR_ILLEGAL_SUB_KEYWORD() ;

            //copy the SubCommand Code 
            poCoorCommand->m_uiCoorSubCmdCode = uiCoorCmdSubCode ;

            // Next Token may be either "TT_Eta", "TT_Phi", or even nothing
            poTokenEntry = _poCoorCommandLine->m_lTokenList.NextEntry( poTokenEntry ) ;

        Look_For_TT_Range_Exclude :

            if ( poTokenEntry != NULL )
            {
                eTokenType = poTokenEntry->GetKey().eTokenType ;

                if ( eTokenType != TokenIsKeyword )
                    ERROR_MISSING_SUB_KEYWORD() ;

                uiKeyword = poTokenEntry->GetValue ( ) ;

                if ( ( uiKeyword == Ki_CoorKeyword_TT_Eta )
                  || ( uiKeyword == Ki_CoorKeyword_TT_Phi ) )
                {
                    PARSE_TT_RANGE( Look_For_TT_Range_Exclude ) ;
                }
                else 
                {
                    ERROR_UNEXPECTED_TOKEN() ;
                }
            }

            // if the Eta range was omitted, this means all Eta values are selected
            FILL_ETA_RANGE_IF_OMITTED() ;
                                                                                                 
            // if the Phi range was omitted, this means all Phi values are selected              
            FILL_PHI_RANGE_IF_OMITTED() ;

            break ;

        /////////////////////////////////////////////////////////////////////////////

        default :
            ERROR_KEYWORD_ILLEGAL_AT_START() ; 

    } ;

Done:

    return eReturnStatus ;
}


/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
  /*
     Override some of the template functions for some
     of the instances to handle char (and CStrings) 
     strings with case-insensitive lookup
     19-Oct-99
  */
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
TBinomialEntry< char, unsigned int >::TBinomialEntry     ( const char*          _poEntryKey,
                                                           const unsigned int*  _poEntryValue,
                                                           TBinomialEntry < char, unsigned int > * 
                                                                                _poLinkPrev )    
{ 
    m_poEntryKey   = new char[strlen(_poEntryKey)+1] ;
    strcpy ( m_poEntryKey, _poEntryKey ) ;
    m_poEntryValue = new unsigned int ( *_poEntryValue ) ;
    m_poLinkPrev   = _poLinkPrev ; 
    m_poLinkNext   = NULL ; 
} ;
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
TBinomialEntry< char, unsigned int >::~TBinomialEntry ( ) 
{ 
    delete []   m_poEntryKey ;
    delete      m_poEntryValue ;
} ;
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
EYesNo  TBinomialEntry< char, unsigned int >::MatchKey   ( const char*  _poEntryKey )    
const 
{ 
    if ( stricmp ( m_poEntryKey, _poEntryKey ) == 0 )
        return Yes ;
    else
        return No ;
} ;


/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
void    TBinomialEntry< char, unsigned int >::ModifyKey  ( const char*   _poEntryKey )
{ 
    delete []   m_poEntryKey ;
    m_poEntryKey   = new char[strlen(_poEntryKey)] ;
    strcpy ( m_poEntryKey, _poEntryKey ) ;
    return ;
} ;


/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/


/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
/*
EYesNo  TBinomialEntry< CString, unsigned int >::MatchKey( const CString*  _poEntryKey )    
const 
{ 
    if ( m_poEntryKey->CompareNoCase ( *_poEntryKey ) == 0 )
        return Yes ;
    else
        return No ;
} ;

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
