/*-------------------------------------------------------------------------------*/
  /*
     MessageCoorPrinter.cpp
     7-Oct-99
     28-Oct-99 Coor Command Parsing split onto separate files
     22-Sep-2005 Restructred for L1Cal_IIb_TCC
  */
/*-------------------------------------------------------------------------------*/

#include "L1Cal_IIb_TCC/StdAfx.h"

#include "L1Cal_IIb_TCC/MessageCommand.h"
#include                            "L1Cal_IIb_TCC/MessageCoorPrinter.h"

#include "L1Cal_IIb_TCC/HandleConsole.h"
#include "L1Cal_IIb_TCC/UtilsStrings.h"
//#include "L1Cal_IIb_TCC/DataManip.h"

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
static char    sReplyText[1000] ;  // generous buffer size to rule out... 
static char    sScreenText[1000] ; // ...overwriting past end of string
                            // This is ok because there is only one executer/printer thread

/////////////////////////////////////////////////////////////////////////////////////
static void AppendTextToReply ( CMessageCommandLine*    _poCoorCommandLine )
{
    /* append stringto reply text*/
    strcpy ( sScreenText, "Process Msg Command : " ) ;
    strcat ( sScreenText, sReplyText ) ;
    CConsoleAccess::WriteLine     ( sScreenText ) ;

    if ( _poCoorCommandLine != NULL )
        _poCoorCommandLine->AppendReplyText ( sReplyText ) ;
    
    return ;
}
/////////////////////////////////////////////////////////////////////////////////////

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ECoorMessageStatus  CMessageCoorPrinter::Configure_FPGAs          ( CMessageCommandLine*    _poCoorCommandLine ) 
{ 
    strcpy ( sReplyText, "<Configure All FPGAs>" ) ;

    AppendTextToReply ( _poCoorCommandLine ) ;

    return Reply_Ok ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ECoorMessageStatus  CMessageCoorPrinter::L1Cal_Initialize        ( CMessageCommandLine*    _poCoorCommandLine ) 
{ 
    strcpy ( sReplyText, "<L1Cal Initialize>" ) ;

    AppendTextToReply ( _poCoorCommandLine ) ;

    return Reply_Ok ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ECoorMessageStatus  CMessageCoorPrinter::Iterative_DeSkew        ( CMessageCommandLine*    _poCoorCommandLine ) 
{ 
    strcpy ( sReplyText, "<Iterative DeSkew>" ) ;

    AppendTextToReply ( _poCoorCommandLine ) ;

    return Reply_Ok ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ECoorMessageStatus  CMessageCoorPrinter::StartRun                 ( const unsigned long     _ulRunNumber,
                                                                    EItemPolarity*          _pubListItemPolarity,
                                                                    CMessageCommandLine*    _poCoorCommandLine ) 
{ 
    strcpy ( sReplyText, "<Start Run #" ) ;
    strcat ( sReplyText, I_to_A ( _ulRunNumber ) ) ;
    strcat ( sReplyText, " for Spec Trig(s) # " ) ;

    Add_Formatted_Item_List ( sReplyText, _pubListItemPolarity ) ;

    strcat ( sReplyText, ">" ) ;

    AppendTextToReply ( _poCoorCommandLine ) ;

    return Reply_Ok ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ECoorMessageStatus  CMessageCoorPrinter::StopRun                  ( const unsigned long     _ulRunNumber,
                                                                    CMessageCommandLine*    _poCoorCommandLine ) 
{ 
    strcpy ( sReplyText, "<Stop  Run #" ) ;
    strcat ( sReplyText, I_to_A ( _ulRunNumber ) ) ;
    strcat ( sReplyText, ">" ) ;

    AppendTextToReply ( _poCoorCommandLine ) ;

    return Reply_Ok ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ECoorMessageStatus  CMessageCoorPrinter::PauseRun                 ( const unsigned long     _ulRunNumber,
                                                                    CMessageCommandLine*    _poCoorCommandLine ) 
{ 
    strcpy ( sReplyText, "<Pause Run #" ) ;
    strcat ( sReplyText, I_to_A ( _ulRunNumber ) ) ;
    strcat ( sReplyText, ">" ) ;

    AppendTextToReply ( _poCoorCommandLine ) ;

    return Reply_Ok ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ECoorMessageStatus  CMessageCoorPrinter::ResumeRun                ( const unsigned long     _ulRunNumber,
                                                                    CMessageCommandLine*    _poCoorCommandLine ) 
{ 
    strcpy ( sReplyText, "<Resume Run #" ) ;
    strcat ( sReplyText, I_to_A ( _ulRunNumber ) ) ;
    strcat ( sReplyText, ">" ) ;

    AppendTextToReply ( _poCoorCommandLine ) ;

    return Reply_Ok ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ECoorMessageStatus  CMessageCoorPrinter::BeginStore               ( const unsigned long     _uiStoreNumber,
                                                                    CMessageCommandLine*    _poCoorCommandLine ) 
{ 
    strcpy ( sReplyText, "<Begin Store #" ) ;
    strcat ( sReplyText, I_to_A ( _uiStoreNumber ) ) ;
    strcat ( sReplyText, ">" ) ;

    AppendTextToReply ( _poCoorCommandLine ) ;

    return Reply_Ok ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ECoorMessageStatus  CMessageCoorPrinter::EndStore                 ( const unsigned long     _uiStoreNumber,
                                                                    CMessageCommandLine*    _poCoorCommandLine ) 
{ 
    strcpy ( sReplyText, "<End Store #" ) ;
    strcat ( sReplyText, I_to_A ( _uiStoreNumber ) ) ;
    strcat ( sReplyText, ">" ) ;


    AppendTextToReply ( _poCoorCommandLine ) ;

    return Reply_Ok ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
void CMessageCoorPrinter::Add_Formatted_Item_List    
                                         ( char*                   _psReplyText,
                                           EItemPolarity*          _pubListItemPolarity ) 
{ 
    int                 iItemNum ;
    EItemPolarity       eItemPolarity ;
    int                 iRangeLower ; 
    int                 iRangeUpper ; 
    EItemPolarity       eRangePolarity ;
    bool                bRangeStarted = false ;

    //\\//

    for ( iItemNum = 0 ; iItemNum <= KiMaxItemIndexInMessageCommand ; iItemNum++ )
    {
        eItemPolarity = _pubListItemPolarity[iItemNum] ;

        if ( eItemPolarity != ItemNotPresent ) 
        {
            if ( bRangeStarted == false ) 
            {
                iRangeLower    = iItemNum ;
                eRangePolarity = eItemPolarity ;
                bRangeStarted  = true ;
            }
            else // ( bRangeStarted == true ) 
            {
                if ( eItemPolarity != eRangePolarity ) 
                {// we have switched polarity -> close previous range and open new one
                    iRangeUpper   = iItemNum - 1 ;
                    Add_Formatted_Range   ( _psReplyText,  
                                            iRangeLower,  
                                            iRangeUpper, 
                                            eRangePolarity ) ;

                    iRangeLower    = iItemNum ;
                    eRangePolarity = eItemPolarity ;
                    //bRangeStarted  = true ; already set
                }
                else
                {// no change in polarity -> nothing to do
                }
            }
        }
        else // ( eItemPolarity == ItemNotPresent ) 
        {
            if ( bRangeStarted == true ) 
            {// the previous range has ended
                iRangeUpper   = iItemNum - 1 ;
                Add_Formatted_Range   ( _psReplyText,  
                                        iRangeLower,  
                                        iRangeUpper, 
                                        eRangePolarity ) ;

                bRangeStarted = false ;
            }
        }
    }

    // terminate the last range, if there was one still open
    if ( bRangeStarted == true )
        Add_Formatted_Range   ( _psReplyText,  
                                iRangeLower,  
                                KiMaxItemIndexInMessageCommand, 
                                eRangePolarity ) ;

    return ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ECoorMessageStatus  CMessageCoorPrinter::RefSetEMEtProg   ( const unsigned int      _uiRefSetNum, 
                                                            TAllTriggerTowers*      _pxTT,
                                                            const double            _fdThreshold,
                                                            CMessageCommandLine*    _poCoorCommandLine ) 

{ 
    strcpy ( sReplyText, "<EM Et RefSet #" ) ;
    strcat ( sReplyText, I_to_A ( _uiRefSetNum ) ) ;
    strcat ( sReplyText, " for " ) ;
    Add_Formatted_TT_Range ( sReplyText, _pxTT ) ;
    strcat ( sReplyText, " Thresh= " ) ;
    strcat ( sReplyText, F_to_A ( _fdThreshold, 2 ) ) ;
    strcat ( sReplyText, " GeV>" ) ;

    AppendTextToReply ( _poCoorCommandLine ) ;

    return Reply_Ok ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ECoorMessageStatus  CMessageCoorPrinter::RefSetJetEtProg  ( const unsigned int      _uiRefSetNum, 
                                                            TAllTriggerTowers*      _pxTT,
                                                            const double            _fdThreshold,
                                                            CMessageCommandLine*    _poCoorCommandLine ) 

{ 
    strcpy ( sReplyText, "<Jet Et RefSet #" ) ;
    strcat ( sReplyText, I_to_A ( _uiRefSetNum ) ) ;
    strcat ( sReplyText, " for " ) ;
    Add_Formatted_TT_Range ( sReplyText, _pxTT ) ;
    strcat ( sReplyText, " Thresh= " ) ;
    strcat ( sReplyText, F_to_A ( _fdThreshold, 2 ) ) ;
    strcat ( sReplyText, " GeV>" ) ;

    AppendTextToReply ( _poCoorCommandLine ) ;

    return Reply_Ok ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ECoorMessageStatus  CMessageCoorPrinter::RefSetTTEtProg   ( const unsigned int      _uiRefSetNum, 
                                                            TAllTriggerTowers*      _pxTT,
                                                            const double            _fdThreshold,
                                                            CMessageCommandLine*    _poCoorCommandLine ) 

{ 
    strcpy ( sReplyText, "<TT Et RefSet #" ) ;
    strcat ( sReplyText, I_to_A ( _uiRefSetNum ) ) ;
    strcat ( sReplyText, " for " ) ;
    Add_Formatted_TT_Range ( sReplyText, _pxTT ) ;
    strcat ( sReplyText, " Thresh= " ) ;
    strcat ( sReplyText, F_to_A ( _fdThreshold, 2 ) ) ;
    strcat ( sReplyText, " GeV>" ) ;

    AppendTextToReply ( _poCoorCommandLine ) ;

    return Reply_Ok ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ECoorMessageStatus  CMessageCoorPrinter::RefSetEMEtDeall      ( const unsigned int      _uiRefSetNum, 
                                                                CMessageCommandLine*    _poCoorCommandLine ) 

{ 
    strcpy ( sReplyText, "<Deallocate EM Et RefSet #" ) ;
    strcat ( sReplyText, I_to_A ( _uiRefSetNum ) ) ;
    strcat ( sReplyText, ">" ) ;

    AppendTextToReply ( _poCoorCommandLine ) ;

    return Reply_Ok ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ECoorMessageStatus  CMessageCoorPrinter::RefSetJetEtDeall       ( const unsigned int      _uiRefSetNum, 
                                                                  CMessageCommandLine*    _poCoorCommandLine ) 

{ 
    strcpy ( sReplyText, "<Deallocate Jet Et RefSet #" ) ;
    strcat ( sReplyText, I_to_A ( _uiRefSetNum ) ) ;
    strcat ( sReplyText, ">" ) ;

    AppendTextToReply ( _poCoorCommandLine ) ;

    return Reply_Ok ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ECoorMessageStatus  CMessageCoorPrinter::RefSetTTEtDeall      ( const unsigned int      _uiRefSetNum, 
                                                                CMessageCommandLine*    _poCoorCommandLine ) 

{ 
    strcpy ( sReplyText, "<Deallocate TT Et RefSet #" ) ;
    strcat ( sReplyText, I_to_A ( _uiRefSetNum ) ) ;
    strcat ( sReplyText, ">" ) ;

    AppendTextToReply ( _poCoorCommandLine ) ;

    return Reply_Ok ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ECoorMessageStatus  CMessageCoorPrinter::L2MaskEM             ( TAllTriggerTowers*      _pxTT,
                                                                const unsigned int      _uiRefSetNum, 
                                                                CMessageCommandLine*    _poCoorCommandLine ) 

{ 
    strcpy ( sReplyText, "<L2 Masks for " ) ;
    Add_Formatted_TT_Range ( sReplyText, _pxTT ) ;
    strcat ( sReplyText, " will use EM Refset# " ) ;
    strcat ( sReplyText, I_to_A ( _uiRefSetNum ) ) ;
    strcat ( sReplyText, ">" ) ;

    AppendTextToReply ( _poCoorCommandLine ) ;

    return Reply_Ok ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ECoorMessageStatus  CMessageCoorPrinter::L2MaskJet            ( TAllTriggerTowers*      _pxTT,
                                                                const unsigned int      _uiRefSetNum, 
                                                                CMessageCommandLine*    _poCoorCommandLine ) 

{ 
    strcpy ( sReplyText, "<L2 Masks for " ) ;
    Add_Formatted_TT_Range ( sReplyText, _pxTT ) ;
    strcat ( sReplyText, " will use Jet Refset# " ) ;
    strcat ( sReplyText, I_to_A ( _uiRefSetNum ) ) ;
    strcat ( sReplyText, ">" ) ;

    AppendTextToReply ( _poCoorCommandLine ) ;

    return Reply_Ok ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ECoorMessageStatus  CMessageCoorPrinter::L2MaskTau            ( TAllTriggerTowers*      _pxTT,
                                                                const unsigned int      _uiRefSetNum, 
                                                                CMessageCommandLine*    _poCoorCommandLine ) 

{ 
    strcpy ( sReplyText, "<L2 Masks for " ) ;
    Add_Formatted_TT_Range ( sReplyText, _pxTT ) ;
    strcat ( sReplyText, " will use Tau Refset# " ) ;
    strcat ( sReplyText, I_to_A ( _uiRefSetNum ) ) ;
    strcat ( sReplyText, ">" ) ;

    AppendTextToReply ( _poCoorCommandLine ) ;

    return Reply_Ok ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ECoorMessageStatus  CMessageCoorPrinter::Exclude_Tower    ( ETT_Chan_Type           _eTTType,
                                                            TAllTriggerTowers*      _pxTT,
                                                            CMessageCommandLine*    _poCoorCommandLine ) 

{ 
    strcpy ( sReplyText, "<Exclude" ) ;
    if ( _eTTType == eTT_EM ) 
        strcat ( sReplyText, " EM" ) ;
    else if ( _eTTType == eTT_HD ) 
        strcat ( sReplyText, " HD" ) ;
    strcat ( sReplyText, " Trigger Tower(s) for " ) ;
    Add_Formatted_TT_Range ( sReplyText, _pxTT ) ;
    strcat ( sReplyText, ">" ) ;

    AppendTextToReply ( _poCoorCommandLine ) ;

    return Reply_Ok ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ECoorMessageStatus  CMessageCoorPrinter::AlgoUseEmFraction        ( TAllTriggerTowers*      _pxTT,
                                                                    bool                    _lUseEmFraction,
                                                                    CMessageCommandLine*    _poCoorCommandLine ) 

{ 
    strcpy ( sReplyText, "<TAB Algorithm will" ) ;
    if      ( _lUseEmFraction == true ) 
        strcat ( sReplyText, " Use" ) ;
    else if ( _lUseEmFraction == false ) 
        strcat ( sReplyText, " Ignore" ) ;
    strcat ( sReplyText, " the EM Fraction for " ) ;
    Add_Formatted_TT_Range ( sReplyText, _pxTT ) ;
    strcat ( sReplyText, ">" ) ;

    AppendTextToReply ( _poCoorCommandLine ) ;

    return Reply_Ok ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ECoorMessageStatus  CMessageCoorPrinter::AlgoUseEmIsolation       ( TAllTriggerTowers*      _pxTT,
                                                                    bool                    _lUseEmIsolation,
                                                                    CMessageCommandLine*    _poCoorCommandLine ) 

{ 
    strcpy ( sReplyText, "<TAB Algorithm will" ) ;
    if      ( _lUseEmIsolation == true ) 
        strcat ( sReplyText, " Use" ) ;
    else if ( _lUseEmIsolation == false ) 
        strcat ( sReplyText, " Ignore" ) ;
    strcat ( sReplyText, " the EM Isolation for " ) ;
    Add_Formatted_TT_Range ( sReplyText, _pxTT ) ;
    strcat ( sReplyText, ">" ) ;

    AppendTextToReply ( _poCoorCommandLine ) ;

    return Reply_Ok ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ECoorMessageStatus  CMessageCoorPrinter::AlgoEmIsolationEtCut     ( TAllTriggerTowers*      _pxTT,
                                                                    const double            _fdThreshold,
                                                                    CMessageCommandLine*    _poCoorCommandLine ) 

{ 
    strcpy ( sReplyText, "<TAB Algorithm will use an EM Isolation Et Cut= " ) ;
    strcat ( sReplyText, F_to_A ( _fdThreshold, 2 ) ) ;
    strcat ( sReplyText, " GeV for " ) ;
    Add_Formatted_TT_Range ( sReplyText, _pxTT ) ;
    strcat ( sReplyText, ">" ) ;

    AppendTextToReply ( _poCoorCommandLine ) ;

    return Reply_Ok ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ECoorMessageStatus  CMessageCoorPrinter::AlgoTauEtCoreCut         ( TAllTriggerTowers*      _pxTT,
                                                                    const double            _fdThreshold,
                                                                    CMessageCommandLine*    _poCoorCommandLine ) 

{ 
    strcpy ( sReplyText, "<TAB Algorithm will use a Tau Core Et Cut= " ) ;
    strcat ( sReplyText, F_to_A ( _fdThreshold, 2 ) ) ;
    strcat ( sReplyText, " GeV for " ) ;
    Add_Formatted_TT_Range ( sReplyText, _pxTT ) ;
    strcat ( sReplyText, ">" ) ;

    AppendTextToReply ( _poCoorCommandLine ) ;

    return Reply_Ok ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ECoorMessageStatus  CMessageCoorPrinter::AlgoGlobalSumEtCut       ( TAllTriggerTowers*      _pxTT,
                                                                    const double            _fdThreshold,
                                                                    CMessageCommandLine*    _poCoorCommandLine ) 

{ 
    strcpy ( sReplyText, "<TAB Algorithm will use a Global Sum Et Cut= " ) ;
    strcat ( sReplyText, F_to_A ( _fdThreshold, 2 ) ) ;
    strcat ( sReplyText, " GeV for " ) ;
    Add_Formatted_TT_Range ( sReplyText, _pxTT ) ;
    strcat ( sReplyText, ">" ) ;

    AppendTextToReply ( _poCoorCommandLine ) ;

    return Reply_Ok ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ECoorMessageStatus  CMessageCoorPrinter::AlgoUseIcrJetObjects     ( TAllTriggerTowers*      _pxTT,
                                                                    bool                    _lUseIcr,
                                                                    CMessageCommandLine*    _poCoorCommandLine ) 

{ 
    strcpy ( sReplyText, "<TAB Algorithm will" ) ;
    if      ( _lUseIcr == true ) 
        strcat ( sReplyText, " Use" ) ;
    else if ( _lUseIcr == false ) 
        strcat ( sReplyText, " Ignore" ) ;
    strcat ( sReplyText, " the ICR in Jet Objects for " ) ;
    Add_Formatted_TT_Range ( sReplyText, _pxTT ) ;
    strcat ( sReplyText, ">" ) ;

    AppendTextToReply ( _poCoorCommandLine ) ;

    return Reply_Ok ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ECoorMessageStatus  CMessageCoorPrinter::AlgoUseIcrGlobalSums     ( TAllTriggerTowers*      _pxTT,
                                                                    bool                    _lUseIcr,
                                                                    CMessageCommandLine*    _poCoorCommandLine ) 

{ 
    strcpy ( sReplyText, "<TAB Algorithm will" ) ;
    if      ( _lUseIcr == true ) 
        strcat ( sReplyText, " Use" ) ;
    else if ( _lUseIcr == false ) 
        strcat ( sReplyText, " Ignore" ) ;
    strcat ( sReplyText, " the ICR in Global Sums for " ) ;
    Add_Formatted_TT_Range ( sReplyText, _pxTT ) ;
    strcat ( sReplyText, ">" ) ;

    AppendTextToReply ( _poCoorCommandLine ) ;

    return Reply_Ok ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ECoorMessageStatus  CMessageCoorPrinter::TermEmAll                ( const unsigned int      _uiComparatorNum, 
                                                                    const unsigned int      _uiRefSetNum, 
                                                                    const unsigned int      _uiThreshold,
                                                                    CMessageCommandLine*    _poCoorCommandLine ) 

{ 
    strcpy ( sReplyText, "<EM_All Term #" ) ;
    strcat ( sReplyText, I_to_A ( _uiComparatorNum  ) ) ;
    strcat ( sReplyText, " use EM Et RefSet #" ) ;
    strcat ( sReplyText, I_to_A ( _uiRefSetNum ) ) ;
    strcat ( sReplyText, " Count Thresh= " ) ;
    strcat ( sReplyText, I_to_A ( _uiThreshold ) ) ;
    strcat ( sReplyText, ">" ) ;

    AppendTextToReply ( _poCoorCommandLine ) ;

    return Reply_Ok ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ECoorMessageStatus  CMessageCoorPrinter::TermEmCentral            ( const unsigned int      _uiComparatorNum, 
                                                                    const unsigned int      _uiRefSetNum, 
                                                                    const unsigned int      _uiThreshold,
                                                                    CMessageCommandLine*    _poCoorCommandLine ) 

{ 
    strcpy ( sReplyText, "<EM_Central Term #" ) ;
    strcat ( sReplyText, I_to_A ( _uiComparatorNum  ) ) ;
    strcat ( sReplyText, " use EM Et RefSet #" ) ;
    strcat ( sReplyText, I_to_A ( _uiRefSetNum ) ) ;
    strcat ( sReplyText, " Count Thresh= " ) ;
    strcat ( sReplyText, I_to_A ( _uiThreshold ) ) ;
    strcat ( sReplyText, ">" ) ;

    AppendTextToReply ( _poCoorCommandLine ) ;

    return Reply_Ok ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ECoorMessageStatus  CMessageCoorPrinter::TermJetAll               ( const unsigned int      _uiComparatorNum, 
                                                                    const unsigned int      _uiRefSetNum, 
                                                                    const unsigned int      _uiThreshold,
                                                                    CMessageCommandLine*    _poCoorCommandLine ) 

{ 
    strcpy ( sReplyText, "<Jet_All Term #" ) ;
    strcat ( sReplyText, I_to_A ( _uiComparatorNum  ) ) ;
    strcat ( sReplyText, " use Jet Et RefSet #" ) ;
    strcat ( sReplyText, I_to_A ( _uiRefSetNum ) ) ;
    strcat ( sReplyText, " Count Thresh= " ) ;
    strcat ( sReplyText, I_to_A ( _uiThreshold ) ) ;
    strcat ( sReplyText, ">" ) ;

    AppendTextToReply ( _poCoorCommandLine ) ;

    return Reply_Ok ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ECoorMessageStatus  CMessageCoorPrinter::TermJetCentral           ( const unsigned int      _uiComparatorNum, 
                                                                    const unsigned int      _uiRefSetNum, 
                                                                    const unsigned int      _uiThreshold,
                                                                    CMessageCommandLine*    _poCoorCommandLine ) 

{ 
    strcpy ( sReplyText, "<Jet_Central Term #" ) ;
    strcat ( sReplyText, I_to_A ( _uiComparatorNum  ) ) ;
    strcat ( sReplyText, " use Jet Et RefSet #" ) ;
    strcat ( sReplyText, I_to_A ( _uiRefSetNum ) ) ;
    strcat ( sReplyText, " Count Thresh= " ) ;
    strcat ( sReplyText, I_to_A ( _uiThreshold ) ) ;
    strcat ( sReplyText, ">" ) ;

    AppendTextToReply ( _poCoorCommandLine ) ;

    return Reply_Ok ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ECoorMessageStatus  CMessageCoorPrinter::TermTauAll               ( const unsigned int      _uiComparatorNum, 
                                                                    const unsigned int      _uiRefSetNum, 
                                                                    const unsigned int      _uiTauRatioNum, 
                                                                    const unsigned int      _uiThreshold,
                                                                    CMessageCommandLine*    _poCoorCommandLine ) 

{ 
    strcpy ( sReplyText, "<Tau_All Term #" ) ;
    strcat ( sReplyText, I_to_A ( _uiComparatorNum  ) ) ;
    strcat ( sReplyText, " use Tau Et RefSet #" ) ;
    strcat ( sReplyText, I_to_A ( _uiRefSetNum ) ) ;
    strcat ( sReplyText, " Tau Ratio Set #" ) ;
    strcat ( sReplyText, I_to_A ( _uiTauRatioNum ) ) ;
    strcat ( sReplyText, " Count Thresh= " ) ;
    strcat ( sReplyText, I_to_A ( _uiThreshold ) ) ;
    strcat ( sReplyText, ">" ) ;

    AppendTextToReply ( _poCoorCommandLine ) ;

    return Reply_Ok ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ECoorMessageStatus  CMessageCoorPrinter::TermTauCentral           ( const unsigned int      _uiComparatorNum, 
                                                                    const unsigned int      _uiRefSetNum, 
                                                                    const unsigned int      _uiTauRatioNum, 
                                                                    const unsigned int      _uiThreshold,
                                                                    CMessageCommandLine*    _poCoorCommandLine ) 

{ 
    strcpy ( sReplyText, "<Tau_Central Term #" ) ;
    strcat ( sReplyText, I_to_A ( _uiComparatorNum  ) ) ;
    strcat ( sReplyText, " use Tau Et RefSet #" ) ;
    strcat ( sReplyText, I_to_A ( _uiRefSetNum ) ) ;
    strcat ( sReplyText, " Tau Ratio Set #" ) ;
    strcat ( sReplyText, I_to_A ( _uiTauRatioNum ) ) ;
    strcat ( sReplyText, " Count Thresh= " ) ;
    strcat ( sReplyText, I_to_A ( _uiThreshold ) ) ;
    strcat ( sReplyText, ">" ) ;

    AppendTextToReply ( _poCoorCommandLine ) ;

    return Reply_Ok ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ECoorMessageStatus  CMessageCoorPrinter::TermIsolEmAll            ( const unsigned int      _uiComparatorNum, 
                                                                    const unsigned int      _uiRefSetNum, 
                                                                    const unsigned int      _uiThreshold,
                                                                    CMessageCommandLine*    _poCoorCommandLine ) 

{ 
    strcpy ( sReplyText, "<Isolated_EM_All Term #" ) ;
    strcat ( sReplyText, I_to_A ( _uiComparatorNum  ) ) ;
    strcat ( sReplyText, " use EM Et RefSet #" ) ;
    strcat ( sReplyText, I_to_A ( _uiRefSetNum ) ) ;
    strcat ( sReplyText, " Count Thresh= " ) ;
    strcat ( sReplyText, I_to_A ( _uiThreshold ) ) ;
    strcat ( sReplyText, ">" ) ;

    AppendTextToReply ( _poCoorCommandLine ) ;

    return Reply_Ok ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ECoorMessageStatus  CMessageCoorPrinter::TermIsolEmCentral        ( const unsigned int      _uiComparatorNum, 
                                                                    const unsigned int      _uiRefSetNum, 
                                                                    const unsigned int      _uiThreshold,
                                                                    CMessageCommandLine*    _poCoorCommandLine ) 

{ 
    strcpy ( sReplyText, "<Isolated_EM_Central Term #" ) ;
    strcat ( sReplyText, I_to_A ( _uiComparatorNum  ) ) ;
    strcat ( sReplyText, " use EM Et RefSet #" ) ;
    strcat ( sReplyText, I_to_A ( _uiRefSetNum ) ) ;
    strcat ( sReplyText, " Count Thresh= " ) ;
    strcat ( sReplyText, I_to_A ( _uiThreshold ) ) ;
    strcat ( sReplyText, ">" ) ;

    AppendTextToReply ( _poCoorCommandLine ) ;

    return Reply_Ok ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ECoorMessageStatus  CMessageCoorPrinter::TermMissEt               ( const unsigned int      _uiComparatorNum, 
                                                                    const double            _fdThreshold,
                                                                    CMessageCommandLine*    _poCoorCommandLine ) 

{ 
    strcpy ( sReplyText, "<Missing_Et Term #" ) ;
    strcat ( sReplyText, I_to_A ( _uiComparatorNum  ) ) ;
    strcat ( sReplyText, " Energy Thresh= " ) ;
    strcat ( sReplyText, F_to_A ( _fdThreshold, 2 ) ) ;
    strcat ( sReplyText, " GeV>" ) ;

    AppendTextToReply ( _poCoorCommandLine ) ;

    return Reply_Ok ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ECoorMessageStatus  CMessageCoorPrinter::TermTotEt                ( const unsigned int      _uiComparatorNum, 
                                                                    const double            _fdThreshold,
                                                                    CMessageCommandLine*    _poCoorCommandLine ) 

{ 
    strcpy ( sReplyText, "<Total_Et Term #" ) ;
    strcat ( sReplyText, I_to_A ( _uiComparatorNum  ) ) ;
    strcat ( sReplyText, " Energy Thresh= " ) ;
    strcat ( sReplyText, F_to_A ( _fdThreshold, 2 ) ) ;
    strcat ( sReplyText, " GeV>" ) ;

    AppendTextToReply ( _poCoorCommandLine ) ;

    return Reply_Ok ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ECoorMessageStatus  CMessageCoorPrinter::TermAcoplanarJet         ( const unsigned int      _uiComparatorNum, 
                                                                    const unsigned int      _uiRefSetNum1, 
                                                                    const unsigned int      _uiRefSetNum2,
                                                                    const unsigned int      _uiDelta_Phi,
                                                                    CMessageCommandLine*    _poCoorCommandLine ) 

{ 
    strcpy ( sReplyText, "<Acoplanar_Jet Term #" ) ;
    strcat ( sReplyText, I_to_A ( _uiComparatorNum  ) ) ;
    strcat ( sReplyText, " use Jet Et RefSet1 #" ) ;
    strcat ( sReplyText, I_to_A ( _uiRefSetNum1 ) ) ;
    strcat ( sReplyText, " and RefSet2 #" ) ;
    strcat ( sReplyText, I_to_A ( _uiRefSetNum2 ) ) ;
    strcat ( sReplyText, " and Delta Phi=" ) ;
    strcat ( sReplyText, I_to_A ( _uiDelta_Phi ) ) ;
    strcat ( sReplyText, ">" ) ;

    AppendTextToReply ( _poCoorCommandLine ) ;

    return Reply_Ok ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ECoorMessageStatus  CMessageCoorPrinter::TermNonCollinearLowEtJet ( const unsigned int      _uiComparatorNum, 
                                                                    const unsigned int      _uiRefSetNum1, 
                                                                    const unsigned int      _uiRefSetNum2,
                                                                    const unsigned int      _uiDelta_Phi,
                                                                    CMessageCommandLine*    _poCoorCommandLine ) 

{ 
    strcpy ( sReplyText, "<NonCollinear_LowEt_Jet Term #" ) ;
    strcat ( sReplyText, I_to_A ( _uiComparatorNum  ) ) ;
    strcat ( sReplyText, " use Jet Et RefSet1 #" ) ;
    strcat ( sReplyText, I_to_A ( _uiRefSetNum1 ) ) ;
    strcat ( sReplyText, " and RefSet2 #" ) ;
    strcat ( sReplyText, I_to_A ( _uiRefSetNum2 ) ) ;
    strcat ( sReplyText, " and Delta Phi=" ) ;
    strcat ( sReplyText, I_to_A ( _uiDelta_Phi ) ) ;
    strcat ( sReplyText, ">" ) ;

    AppendTextToReply ( _poCoorCommandLine ) ;

    return Reply_Ok ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ECoorMessageStatus  CMessageCoorPrinter::TermNonCollinearMonoJet  ( const unsigned int      _uiComparatorNum, 
                                                                    const unsigned int      _uiRefSetNum1, 
                                                                    const unsigned int      _uiRefSetNum2,
                                                                    const unsigned int      _uiDelta_Phi,
                                                                    CMessageCommandLine*    _poCoorCommandLine ) 

{ 
    strcpy ( sReplyText, "<NonCollinear_Mono_Jet Term #" ) ;
    strcat ( sReplyText, I_to_A ( _uiComparatorNum  ) ) ;
    strcat ( sReplyText, " use Jet Et RefSet1 #" ) ;
    strcat ( sReplyText, I_to_A ( _uiRefSetNum1 ) ) ;
    strcat ( sReplyText, " and RefSet2 #" ) ;
    strcat ( sReplyText, I_to_A ( _uiRefSetNum2 ) ) ;
    strcat ( sReplyText, " and Delta Phi=" ) ;
    strcat ( sReplyText, I_to_A ( _uiDelta_Phi ) ) ;
    strcat ( sReplyText, ">" ) ;

    AppendTextToReply ( _poCoorCommandLine ) ;

    return Reply_Ok ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ECoorMessageStatus  CMessageCoorPrinter::TermJetFree              ( const unsigned int      _uiComparatorNum, 
                                                                    const unsigned int      _uiRefSetNum1, 
                                                                    const unsigned int      _uiRefSetNum2,
                                                                    const unsigned int      _uiDelta_Phi,
                                                                    CMessageCommandLine*    _poCoorCommandLine ) 

{ 
    strcpy ( sReplyText, "<Jet_Free Term #" ) ;
    strcat ( sReplyText, I_to_A ( _uiComparatorNum  ) ) ;
    strcat ( sReplyText, " use Jet Et RefSet1 #" ) ;
    strcat ( sReplyText, I_to_A ( _uiRefSetNum1 ) ) ;
    strcat ( sReplyText, " and RefSet2 #" ) ;
    strcat ( sReplyText, I_to_A ( _uiRefSetNum2 ) ) ;
    strcat ( sReplyText, " and Delta Phi=" ) ;
    strcat ( sReplyText, I_to_A ( _uiDelta_Phi ) ) ;
    strcat ( sReplyText, ">" ) ;

    AppendTextToReply ( _poCoorCommandLine ) ;

    return Reply_Ok ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ECoorMessageStatus  CMessageCoorPrinter::TermBackToBackEm         ( const unsigned int      _uiComparatorNum, 
                                                                    const unsigned int      _uiRefSetNum, 
                                                                    const unsigned int      _uiDelta_Phi,
                                                                    CMessageCommandLine*    _poCoorCommandLine ) 

{ 
    strcpy ( sReplyText, "<Back_To_Back_EM Term #" ) ;
    strcat ( sReplyText, I_to_A ( _uiComparatorNum  ) ) ;
    strcat ( sReplyText, " use EM Et RefSet #" ) ;
    strcat ( sReplyText, I_to_A ( _uiRefSetNum ) ) ;
    strcat ( sReplyText, " and Delta Phi=" ) ;
    strcat ( sReplyText, I_to_A ( _uiDelta_Phi ) ) ;
    strcat ( sReplyText, ">" ) ;

    AppendTextToReply ( _poCoorCommandLine ) ;

    return Reply_Ok ;
}



/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
void CMessageCoorPrinter::Add_Formatted_TT_Range  ( char*                   _psReplyText,
                                                    TAllTriggerTowers*      _pxTT ) 
{ 
    strcat ( _psReplyText, "TT Eta(" ) ;
    Add_Formatted_TT_Eta ( _psReplyText, _pxTT ) ;
    strcat ( _psReplyText, ") Phi(" ) ;
    Add_Formatted_TT_Phi ( _psReplyText, _pxTT ) ;
    strcat ( _psReplyText, ")" ) ;
    return ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
void CMessageCoorPrinter::Add_Formatted_TT_Eta    ( char*                   _psReplyText,
                                                    TAllTriggerTowers*      _pxTT ) 
{ 
    int                 iTT_Eta_Magn ;
    int                 iRangeLower ; 
    int                 iRangeUpper ; 
    bool                bRangeStarted = false ;

    //\\//

    for ( iTT_Eta_Magn = KiMax_TT_Eta_Magn ; iTT_Eta_Magn >= KiMin_TT_Eta_Magn ; iTT_Eta_Magn-- )
    { 
        if ( _pxTT->aalTT_Eta[eTT_Eta_Neg][iTT_Eta_Magn] == true )
        {
            if ( bRangeStarted == false ) 
            {
                iRangeLower   = - iTT_Eta_Magn ;
                bRangeStarted = true ;
            }
        }
        else
        {
            if ( bRangeStarted == true ) 
            {
                iRangeUpper   = - ( iTT_Eta_Magn + 1 ) ;
                Add_Formatted_Range ( _psReplyText, iRangeLower, iRangeUpper ) ;
                bRangeStarted = false ;
            }
        }
    }

    for ( iTT_Eta_Magn = KiMin_TT_Eta_Magn ; iTT_Eta_Magn <= KiMax_TT_Eta_Magn ; iTT_Eta_Magn++ )
    { 
        if ( _pxTT->aalTT_Eta[eTT_Eta_Pos][iTT_Eta_Magn] == true )
        {
            if ( bRangeStarted == false ) 
            {
                iRangeLower   = iTT_Eta_Magn ;
                bRangeStarted = true ;
            }
        }
        else
        {
            if ( bRangeStarted == true ) 
            {
                iRangeUpper   = iTT_Eta_Magn - 1 ;
                if ( iRangeUpper == 0 ) iRangeUpper = -1 ;
                Add_Formatted_Range ( _psReplyText, iRangeLower, iRangeUpper ) ;
                bRangeStarted = false ;
            }
        }
    }

    // terminate the last range, if there was one still open
    if ( bRangeStarted == true )
        Add_Formatted_Range ( _psReplyText, iRangeLower, KiMax_TT_Eta_Magn ) ;

    return ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
void CMessageCoorPrinter::Add_Formatted_TT_Phi    ( char*                   _psReplyText,
                                                    TAllTriggerTowers*      _pxTT ) 
{ 
    int                 iTT_Phi ;
    int                 iRangeLower ; 
    int                 iRangeUpper ; 
    bool                bRangeStarted = false ;

    //\\//

    for ( iTT_Phi = KiMin_TT_Phi ; iTT_Phi <= KiMax_TT_Phi ; iTT_Phi++ )
    { 
        if ( _pxTT->alTT_Phi[iTT_Phi] == true )
        {
            if ( bRangeStarted == false ) 
            {
                iRangeLower   = iTT_Phi ;
                bRangeStarted = true ;
            }
        }
        else
        {
            if ( bRangeStarted == true ) 
            {
                iRangeUpper   = iTT_Phi - 1 ;
                Add_Formatted_Range ( _psReplyText, iRangeLower, iRangeUpper ) ;
                bRangeStarted = false ;
            }
        }
    }

    // terminate the last range, if there was one still open
    if ( bRangeStarted == true )
        Add_Formatted_Range ( _psReplyText, iRangeLower, KiMax_TT_Phi ) ;

    return ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
void CMessageCoorPrinter::Add_Formatted_Range     ( char*                   _psReplyText,
                                                    int                     _iRangeLower, 
                                                    int                     _iRangeUpper,
                                                    EItemPolarity           _ePolarity ) 
{
    strcat ( _psReplyText, " " ) ;

    if ( _ePolarity == ItemNegated ) 
        strcat ( _psReplyText, "-" ) ;
    strcat ( _psReplyText, I_to_A ( _iRangeLower ) ) ;

    if ( _iRangeLower != _iRangeUpper )
    {
        strcat ( _psReplyText, ":" ) ;
        if ( _ePolarity == ItemNegated ) 
            strcat ( _psReplyText, "-" ) ;
        strcat ( _psReplyText, I_to_A ( _iRangeUpper ) ) ;
    }

    return ;
}

