/*-------------------------------------------------------------------------------*/
  /*
     MessageMgrDispatcher.cpp
     7-Oct-99
     28-Oct-99 Coor Command Parsing split onto separate files
     22-Sep-2005 Restructred for L1Cal_IIb_TCC
  */
/*-------------------------------------------------------------------------------*/
#include "L1Cal_IIb_TCC/StdAfx.h"

#include "L1Cal_IIb_TCC/GlobalConstants.h"
#include "L1Cal_IIb_TCC/CustomDataTypes.h"

#include "L1Cal_IIb_TCC/MessageMgrCommand.h"
#include                            "L1Cal_IIb_TCC/MessageMgrDispatcher.h"

#include "L1Cal_IIb_TCC/MessageKeywordsCoorL1Cal.h" // e.g. for EM_Tower, etc
#include "L1Cal_IIb_TCC/MessageKeywordsTrgMgr.h"

#include "L1Cal_IIb_TCC/HandleConsole.h"
#include "L1Cal_IIb_TCC/UtilsStrings.h"

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/


/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ECoorMessageStatus CMessageMgrDispatcher::MessageDispatch         ( CMessageCommandLine*            _poCoorCommandLine,
                                                                    CMessageMgrExecuter*            _pxCoorExecute )
{
    ECoorMessageStatus  eReturnStatus = Reply_Ok ;
    ECoorMessageStatus  eRememberStatus = Reply_Ok ;

    char                sReplyText[KiMaxLengthReplyTextToCoor] ;

    TBinomialEntry < CMessageCommand, unsigned int >*  poCoorCommandEntry ;

    CMessageCommand*    poCoorCommand ;

    //\\//

    //Get at the first command
    //find the next command in the list
    poCoorCommandEntry = _poCoorCommandLine->m_lCommandList.FirstEntry( ) ;
    if ( poCoorCommandEntry != NULL )
        poCoorCommand = poCoorCommandEntry ->GetAtKey( ) ;
    else
        poCoorCommand = NULL ;

    while ( poCoorCommand != NULL )
    {
        switch ( poCoorCommand->m_uiCoorCmdCode )
        {

            /////////////////////////////////////////////////////////////////////
            /// Set Common Trigger Tower Base Alignement Delay //////////////////
            /////////////////////////////////////////////////////////////////////
            case Ki_TrgMgrCmd_TrgMgr_BaseAlignmentDelay :
            {
                eReturnStatus = _pxCoorExecute->BaseAlignmentDelay     ( poCoorCommand->m_uiValue,
                                                                        _poCoorCommandLine ) ;

            } break ; // case Ki_TrgMgrCmd_TrgMgr_BaseAlignmentDelay

            /////////////////////////////////////////////////////////////////////
            /// Specify the Fixed location of SubTick #0 of Tick#1    ///////////
            /// when TCC reads Raw ADC Monitoring Data                ///////////
            /////////////////////////////////////////////////////////////////////
            case Ki_TrgMgrCmd_TrgMgr_OffsetTick1RawAdcMonit :
            {
                eReturnStatus = _pxCoorExecute->OffsetTick1RawAdcMonit ( poCoorCommand->m_uiValue,
                                                                        _poCoorCommandLine ) ;


            } break ; // case Ki_TrgMgrCmd_TrgMgr_OffsetTick1RawAdcMonit

            /////////////////////////////////////////////////////////////////////
            /// Specify the Fixed location of SubTick #0 of Tick#1    ///////////
            /// when TCC writes Raw ADC Simulated Data                ///////////
            /////////////////////////////////////////////////////////////////////
            case Ki_TrgMgrCmd_TrgMgr_OffsetTick1RawAdcSimu :
            {
                eReturnStatus = _pxCoorExecute->OffsetTick1RawAdcSimu  ( poCoorCommand->m_uiValue,
                                                                        _poCoorCommandLine ) ;

            } break ; // case Ki_TrgMgrCmd_TrgMgr_OffsetTick1RawAdcSimu

            /////////////////////////////////////////////////////////////////////
            /// Specify the Fixed location of Tick#1                  ///////////
            /// when TCC reads Output Et Monitoring Data              ///////////
            /////////////////////////////////////////////////////////////////////
            case Ki_TrgMgrCmd_TrgMgr_OffsetTick1OutputEtMonit :
            {
                eReturnStatus = _pxCoorExecute->OffsetTick1OutputEtMonit( poCoorCommand->m_uiValue,
                                                                        _poCoorCommandLine ) ;

            } break ; // case Ki_TrgMgrCmd_TrgMgr_OffsetTick1OutputEtMonit

            /////////////////////////////////////////////////////////////////////
            /// Specify the Fixed location of Tick#1                  ///////////
            /// when TCC writes Output Et Monitoring Data             ///////////
            /////////////////////////////////////////////////////////////////////
            case Ki_TrgMgrCmd_TrgMgr_OffsetTick1OutputEtSimu :
            {
                eReturnStatus = _pxCoorExecute->OffsetTick1OutputEtSimu( poCoorCommand->m_uiValue,
                                                                        _poCoorCommandLine ) ;

            } break ; // case Ki_TrgMgrCmd_TrgMgr_OffsetTick1OutputEtSimu

            /////////////////////////////////////////////////////////////////////
            /// Specify the offset between the Triggered BeamX Tick Number    ///
            /// and the BX analyzed at the occurence of Capture Monit Data    ///
            /////////////////////////////////////////////////////////////////////
            case Ki_TrgMgrCmd_TrgMgr_OffsetTriggeredToTickAtCapture :
            {
                eReturnStatus = _pxCoorExecute->OffsetTriggeredToTickAtCapture
                                                                       ( poCoorCommand->m_uiValue,
                                                                        _poCoorCommandLine ) ;

            } break ; // case Ki_TrgMgrCmd_TrgMgr_OffsetTriggeredToTickAtCapture

            /////////////////////////////////////////////////////////////////////
            /// L1Cal Operational Commands   ////////////////////////////////////
            /////////////////////////////////////////////////////////////////////
            case Ki_TrgMgrCmd_TrgMgr_L1Cal_Operational :
            {
                eReturnStatus = _pxCoorExecute->L1Cal_Operational ( _poCoorCommandLine ) ;

            } break ; // case Ki_TrgMgrCmd_TrgMgr_L1Cal_Operational

            case Ki_TrgMgrCmd_TrgMgr_L1Cal_Not_Operational :
            {
                eReturnStatus = _pxCoorExecute->L1Cal_Not_Operational ( _poCoorCommandLine ) ;

            } break ; // case Ki_TrgMgrCmd_TrgMgr_L1Cal_Not_Operational


            /////////////////////////////////////////////////////////////////////
            /// L1Cal Coverage Command ///////////////////////////////////////////
            /////////////////////////////////////////////////////////////////////
            case Ki_TrgMgrCmd_TrgMgr_L1Cal_Coverage :
            {
                eReturnStatus = _pxCoorExecute->L1CalCoverage ( &poCoorCommand->m_xTT,
                                                                _poCoorCommandLine ) ;

            } break ; // case Ki_TrgMgrCmd_TrgMgr_L1Cal_Coverage


            /////////////////////////////////////////////////////////////////////
            /// Logfile Commands   ////////////////////////////////////
            /////////////////////////////////////////////////////////////////////
            case Ki_TrgMgrCmd_TrgMgr_NewLogFile :
            {
                eReturnStatus = _pxCoorExecute->NewLogFile ( _poCoorCommandLine ) ;

            } break ; // case Ki_TrgMgrCmd_TrgMgr_NewLogFile


            /////////////////////////////////////////////////////////////////////
            /// Find_DAC Command ///////////////////////////////////////////
            /////////////////////////////////////////////////////////////////////
            //[TrgMgr_Find_DAC  EM_Tower TT_Eta(20) TT_Phi(23)]
            //[TrgMgr_Find_DAC  HD_Tower TT_Eta(20) TT_Phi(23)]
            case Ki_TrgMgrCmd_TrgMgr_Find_DAC :
            {
                switch ( poCoorCommand->m_uiCoorSubCmdCode )
                {

                    /////////////////////////////////////////////////////////////
                    /// EM Et Trigger Tower /////////////////////////////////////
                    /////////////////////////////////////////////////////////////
                case Ki_CoorSubCmd_EM_Tower :
                    {
                        eReturnStatus = _pxCoorExecute->Find_DAC
                                                                (  eTT_EM, 
                                                                  &poCoorCommand->m_xTT, 
                                                                  _poCoorCommandLine ) ;
                    } ; break ; // case Ki_CoorSubCmd_EM_Tower 

                    /////////////////////////////////////////////////////////////
                    /// HD Et Trigger Tower /////////////////////////////////////
                    /////////////////////////////////////////////////////////////
                case Ki_CoorSubCmd_HD_Tower :
                    {
                        eReturnStatus = _pxCoorExecute->Find_DAC
                                                                (  eTT_HD, 
                                                                  &poCoorCommand->m_xTT, 
                                                                  _poCoorCommandLine ) ;
                    } ; break ; // case Ki_CoorSubCmd_HD_Tower 

                    /////////////////////////////////////////////////////////////
                    // All commands that have been parsed should have been dispatched here
                    // but this catches omissions
                    /////////////////////////////////////////////////////////////
                    default:
                    {
                        strcpy ( sReplyText, "<TrgMgr Find DAC Trigger Tower Sub Command Not implemented ID =" ) ;
                        strcat ( sReplyText, I_to_A ( poCoorCommand->m_uiCoorCmdCode ) ) ;
                        strcat ( sReplyText, "/" ) ;
                        strcat ( sReplyText, I_to_A ( poCoorCommand->m_uiCoorSubCmdCode ) ) ;
                        strcat ( sReplyText, ">" ) ;
                        CConsoleAccess::ErrorLine     ( sReplyText ) ;
                        _poCoorCommandLine->CopyReplyText ( sReplyText ) ;

                        eReturnStatus = Reply_Bad ;
                    } ; break ;
                }
            } break ; // case Ki_TrgMgrCmd_TrgMgr_Find_DAC


            /////////////////////////////////////////////////////////////////////
            /// Adjust_Ped Command ///////////////////////////////////////////
            /////////////////////////////////////////////////////////////////////
            //[TrgMgr_Adjust_Ped  EM_Tower TT_Eta(20) TT_Phi(23)]
            //[TrgMgr_Adjust_Ped  HD_Tower TT_Eta(20) TT_Phi(23)]
            case Ki_TrgMgrCmd_TrgMgr_Adjust_Ped :
            {
                switch ( poCoorCommand->m_uiCoorSubCmdCode )
                {

                    /////////////////////////////////////////////////////////////
                    /// EM Et Trigger Tower /////////////////////////////////////
                    /////////////////////////////////////////////////////////////
                case Ki_CoorSubCmd_EM_Tower :
                    {
                        eReturnStatus = _pxCoorExecute->Adjust_Ped
                                                                (  eTT_EM, 
                                                                  &poCoorCommand->m_xTT, 
                                                                  _poCoorCommandLine ) ;
                    } ; break ; // case Ki_CoorSubCmd_EM_Tower 

                    /////////////////////////////////////////////////////////////
                    /// HD Et Trigger Tower /////////////////////////////////////
                    /////////////////////////////////////////////////////////////
                case Ki_CoorSubCmd_HD_Tower :
                    {
                        eReturnStatus = _pxCoorExecute->Adjust_Ped
                                                                (  eTT_HD, 
                                                                  &poCoorCommand->m_xTT, 
                                                                  _poCoorCommandLine ) ;
                    } ; break ; // case Ki_CoorSubCmd_HD_Tower 

                    /////////////////////////////////////////////////////////////
                    // All commands that have been parsed should have been dispatched here
                    // but this catches omissions
                    /////////////////////////////////////////////////////////////
                    default:
                    {
                        strcpy ( sReplyText, "<TrgMgr Adjust_Ped Trigger Tower Sub Command Not implemented ID =" ) ;
                        strcat ( sReplyText, I_to_A ( poCoorCommand->m_uiCoorCmdCode ) ) ;
                        strcat ( sReplyText, "/" ) ;
                        strcat ( sReplyText, I_to_A ( poCoorCommand->m_uiCoorSubCmdCode ) ) ;
                        strcat ( sReplyText, ">" ) ;
                        CConsoleAccess::ErrorLine     ( sReplyText ) ;
                        _poCoorCommandLine->CopyReplyText ( sReplyText ) ;

                        eReturnStatus = Reply_Bad ;
                    } ; break ;
                }
            } break ; // case Ki_TrgMgrCmd_TrgMgr_Adjust_Ped


            /////////////////////////////////////////////////////////////////////
            /// Revert_Ped Command ///////////////////////////////////////////
            /////////////////////////////////////////////////////////////////////
            //[TrgMgr_Revert_Ped  EM_Tower TT_Eta(20) TT_Phi(23)]
            //[TrgMgr_Revert_Ped  HD_Tower TT_Eta(20) TT_Phi(23)]
            case Ki_TrgMgrCmd_TrgMgr_Revert_Ped :
            {
                switch ( poCoorCommand->m_uiCoorSubCmdCode )
                {

                    /////////////////////////////////////////////////////////////
                    /// EM Et Trigger Tower /////////////////////////////////////
                    /////////////////////////////////////////////////////////////
                case Ki_CoorSubCmd_EM_Tower :
                    {
                        eReturnStatus = _pxCoorExecute->Revert_Ped
                                                                (  eTT_EM, 
                                                                  &poCoorCommand->m_xTT, 
                                                                  _poCoorCommandLine ) ;
                    } ; break ; // case Ki_CoorSubCmd_EM_Tower 

                    /////////////////////////////////////////////////////////////
                    /// HD Et Trigger Tower /////////////////////////////////////
                    /////////////////////////////////////////////////////////////
                case Ki_CoorSubCmd_HD_Tower :
                    {
                        eReturnStatus = _pxCoorExecute->Revert_Ped
                                                                (  eTT_HD, 
                                                                  &poCoorCommand->m_xTT, 
                                                                  _poCoorCommandLine ) ;
                    } ; break ; // case Ki_CoorSubCmd_HD_Tower 

                    /////////////////////////////////////////////////////////////
                    // All commands that have been parsed should have been dispatched here
                    // but this catches omissions
                    /////////////////////////////////////////////////////////////
                    default:
                    {
                        strcpy ( sReplyText, "<TrgMgr Revert_Ped Trigger Tower Sub Command Not implemented ID =" ) ;
                        strcat ( sReplyText, I_to_A ( poCoorCommand->m_uiCoorCmdCode ) ) ;
                        strcat ( sReplyText, "/" ) ;
                        strcat ( sReplyText, I_to_A ( poCoorCommand->m_uiCoorSubCmdCode ) ) ;
                        strcat ( sReplyText, ">" ) ;
                        CConsoleAccess::ErrorLine     ( sReplyText ) ;
                        _poCoorCommandLine->CopyReplyText ( sReplyText ) ;

                        eReturnStatus = Reply_Bad ;
                    } ; break ;
                }
            } break ; // case Ki_TrgMgrCmd_TrgMgr_Revert_Ped


            /////////////////////////////////////////////////////////////////////
            /// Profile_BLS Command ///////////////////////////////////////////
            /////////////////////////////////////////////////////////////////////
            //[TrgMgr_Profile_BLS  EM_Tower TT_Eta(20) TT_Phi(23)]
            //[TrgMgr_Profile_BLS  HD_Tower TT_Eta(20) TT_Phi(23)]
            case Ki_TrgMgrCmd_TrgMgr_Profile_BLS :
            {
                switch ( poCoorCommand->m_uiCoorSubCmdCode )
                {

                    /////////////////////////////////////////////////////////////
                    /// EM Et Trigger Tower /////////////////////////////////////
                    /////////////////////////////////////////////////////////////
                case Ki_CoorSubCmd_EM_Tower :
                    {
                        eReturnStatus = _pxCoorExecute->Profile_BLS
                                                                (  eTT_EM, 
                                                                  &poCoorCommand->m_xTT, 
                                                                  _poCoorCommandLine ) ;
                    } ; break ; // case Ki_CoorSubCmd_EM_Tower 

                    /////////////////////////////////////////////////////////////
                    /// HD Et Trigger Tower /////////////////////////////////////
                    /////////////////////////////////////////////////////////////
                case Ki_CoorSubCmd_HD_Tower :
                    {
                        eReturnStatus = _pxCoorExecute->Profile_BLS
                                                                (  eTT_HD, 
                                                                  &poCoorCommand->m_xTT, 
                                                                  _poCoorCommandLine ) ;
                    } ; break ; // case Ki_CoorSubCmd_HD_Tower 

                    /////////////////////////////////////////////////////////////
                    // All commands that have been parsed should have been dispatched here
                    // but this catches omissions
                    /////////////////////////////////////////////////////////////
                    default:
                    {
                        strcpy ( sReplyText, "<TrgMgr Profile BLS Trigger Tower Sub Command Not implemented ID =" ) ;
                        strcat ( sReplyText, I_to_A ( poCoorCommand->m_uiCoorCmdCode ) ) ;
                        strcat ( sReplyText, "/" ) ;
                        strcat ( sReplyText, I_to_A ( poCoorCommand->m_uiCoorSubCmdCode ) ) ;
                        strcat ( sReplyText, ">" ) ;
                        CConsoleAccess::ErrorLine     ( sReplyText ) ;
                        _poCoorCommandLine->CopyReplyText ( sReplyText ) ;

                        eReturnStatus = Reply_Bad ;
                    } ; break ;
                }
            } break ; // case Ki_TrgMgrCmd_TrgMgr_Profile_BLS


             /////////////////////////////////////////////////////////////////////
            /// Profile_AdfEt    Command ///////////////////////////////////////////
            /////////////////////////////////////////////////////////////////////
            //[TrgMgr_Profile_AdfEt   EM_Tower TT_Eta(20) TT_Phi(23)]
            //[TrgMgr_Profile_AdfEt   HD_Tower TT_Eta(20) TT_Phi(23)]
            case Ki_TrgMgrCmd_TrgMgr_Profile_AdfEt :
            {
                switch ( poCoorCommand->m_uiCoorSubCmdCode )
                {

                    /////////////////////////////////////////////////////////////
                    /// EM Et Trigger Tower /////////////////////////////////////
                    /////////////////////////////////////////////////////////////
                case Ki_CoorSubCmd_EM_Tower :
                    {
                        eReturnStatus = _pxCoorExecute->Profile_AdfEt   
                                                                (  eTT_EM, 
                                                                  &poCoorCommand->m_xTT, 
                                                                  _poCoorCommandLine ) ;
                    } ; break ; // case Ki_CoorSubCmd_EM_Tower 

                    /////////////////////////////////////////////////////////////
                    /// HD Et Trigger Tower /////////////////////////////////////
                    /////////////////////////////////////////////////////////////
                case Ki_CoorSubCmd_HD_Tower :
                    {
                        eReturnStatus = _pxCoorExecute->Profile_AdfEt   
                                                                (  eTT_HD, 
                                                                  &poCoorCommand->m_xTT, 
                                                                  _poCoorCommandLine ) ;
                    } ; break ; // case Ki_CoorSubCmd_HD_Tower 

                    /////////////////////////////////////////////////////////////
                    // All commands that have been parsed should have been dispatched here
                    // but this catches omissions
                    /////////////////////////////////////////////////////////////
                    default:
                    {
                        strcpy ( sReplyText, "<TrgMgr Profile OutputEt Trigger Tower Sub Command Not implemented ID =" ) ;
                        strcat ( sReplyText, I_to_A ( poCoorCommand->m_uiCoorCmdCode ) ) ;
                        strcat ( sReplyText, "/" ) ;
                        strcat ( sReplyText, I_to_A ( poCoorCommand->m_uiCoorSubCmdCode ) ) ;
                        strcat ( sReplyText, ">" ) ;
                        CConsoleAccess::ErrorLine     ( sReplyText ) ;
                        _poCoorCommandLine->CopyReplyText ( sReplyText ) ;

                        eReturnStatus = Reply_Bad ;
                    } ; break ;
                }
            } break ; // case Ki_TrgMgrCmd_TrgMgr_Profile_AdfEt   


           /////////////////////////////////////////////////////////////////////
            /// Set Output Constant Command ///////////////////////////////////////////
            /////////////////////////////////////////////////////////////////////
            //[TrgMgr_AdfOut_Const EM_Tower TT_Eta(-20:20) TT_Phi(1:32) Value 8]
            //[TrgMgr_AdfOut_Const HD_Tower TT_Eta(20) TT_Phi(23) Value 8]
            case Ki_TrgMgrCmd_TrgMgr_AdfOut_Const :
            {
                switch ( poCoorCommand->m_uiCoorSubCmdCode )
                {

                    /////////////////////////////////////////////////////////////
                    /// EM Et Trigger Tower /////////////////////////////////////
                    /////////////////////////////////////////////////////////////
                case Ki_CoorSubCmd_EM_Tower :
                    {
                        eReturnStatus = _pxCoorExecute->AdfOut_Constant
                                                                (  eTT_EM, 
                                                                  &poCoorCommand->m_xTT, 
                                                                   poCoorCommand->m_uiValue, 
                                                                  _poCoorCommandLine ) ;
                    } ; break ; // case Ki_CoorSubCmd_EM_Tower 

                    /////////////////////////////////////////////////////////////
                    /// HD Et Trigger Tower /////////////////////////////////////
                    /////////////////////////////////////////////////////////////
                case Ki_CoorSubCmd_HD_Tower :
                    {
                        eReturnStatus = _pxCoorExecute->AdfOut_Constant
                                                                (  eTT_HD, 
                                                                  &poCoorCommand->m_xTT, 
                                                                   poCoorCommand->m_uiValue, 
                                                                  _poCoorCommandLine ) ;
                    } ; break ; // case Ki_CoorSubCmd_HD_Tower 

                    /////////////////////////////////////////////////////////////
                    // All commands that have been parsed should have been dispatched here
                    // but this catches omissions
                    /////////////////////////////////////////////////////////////
                    default:
                    {
                        strcpy ( sReplyText, "<TrgMgr Output Constant Sub Command Not implemented ID =" ) ;
                        strcat ( sReplyText, I_to_A ( poCoorCommand->m_uiCoorCmdCode ) ) ;
                        strcat ( sReplyText, "/" ) ;
                        strcat ( sReplyText, I_to_A ( poCoorCommand->m_uiCoorSubCmdCode ) ) ;
                        strcat ( sReplyText, ">" ) ;
                        CConsoleAccess::ErrorLine     ( sReplyText ) ;
                        _poCoorCommandLine->CopyReplyText ( sReplyText ) ;

                        eReturnStatus = Reply_Bad ;
                    } ; break ;
                }
            } break ; // case Ki_TrgMgrCmd_TrgMgr_AdfOut_Const


            /////////////////////////////////////////////////////////////////////
            /// Set Raw ADC Threshold Command ///////////////////////////////////
            /////////////////////////////////////////////////////////////////////
            //[TrgMgr_RawAdc_Thresh EM_Tower TT_Eta(-20:20) TT_Phi(1:32) Value 100]
            //[TrgMgr_RawAdc_Thresh HD_Tower TT_Eta(20) TT_Phi(23) Value 100]
            case Ki_TrgMgrCmd_TrgMgr_RawAdc_Thresh :
            {
                switch ( poCoorCommand->m_uiCoorSubCmdCode )
                {

                    /////////////////////////////////////////////////////////////
                    /// EM Et Trigger Tower /////////////////////////////////////
                    /////////////////////////////////////////////////////////////
                case Ki_CoorSubCmd_EM_Tower :
                    {
                        eReturnStatus = _pxCoorExecute->RawAdc_Thresh
                                                                (  eTT_EM, 
                                                                  &poCoorCommand->m_xTT, 
                                                                   poCoorCommand->m_uiValue, 
                                                                  _poCoorCommandLine ) ;
                    } ; break ; // case Ki_CoorSubCmd_EM_Tower 

                    /////////////////////////////////////////////////////////////
                    /// HD Et Trigger Tower /////////////////////////////////////
                    /////////////////////////////////////////////////////////////
                case Ki_CoorSubCmd_HD_Tower :
                    {
                        eReturnStatus = _pxCoorExecute->RawAdc_Thresh
                                                                (  eTT_HD, 
                                                                  &poCoorCommand->m_xTT, 
                                                                   poCoorCommand->m_uiValue, 
                                                                  _poCoorCommandLine ) ;
                    } ; break ; // case Ki_CoorSubCmd_HD_Tower 

                    /////////////////////////////////////////////////////////////
                    // All commands that have been parsed should have been dispatched here
                    // but this catches omissions
                    /////////////////////////////////////////////////////////////
                    default:
                    {
                        strcpy ( sReplyText, "<TrgMgr Raw ADC Threshold Sub Command Not implemented ID =" ) ;
                        strcat ( sReplyText, I_to_A ( poCoorCommand->m_uiCoorCmdCode ) ) ;
                        strcat ( sReplyText, "/" ) ;
                        strcat ( sReplyText, I_to_A ( poCoorCommand->m_uiCoorSubCmdCode ) ) ;
                        strcat ( sReplyText, ">" ) ;
                        CConsoleAccess::ErrorLine     ( sReplyText ) ;
                        _poCoorCommandLine->CopyReplyText ( sReplyText ) ;

                        eReturnStatus = Reply_Bad ;
                    } ; break ;
                }
            } break ; // case Ki_TrgMgrCmd_TrgMgr_RawAdc_Thresh


            /////////////////////////////////////////////////////////////////////
            /// ADF Raw ADC Mux Commands  ///////////////////////////////////////
            /////////////////////////////////////////////////////////////////////
            case Ki_TrgMgrCmd_TrgMgr_Adf_RawAdc_Mux :
            {
                switch ( poCoorCommand->m_uiCoorSubCmdCode )
                {

                    /////////////////////////////////////////////////////////////
                    /// EM Et Trigger Tower /////////////////////////////////////
                    /////////////////////////////////////////////////////////////
                case Ki_CoorSubCmd_EM_Tower :
                    {
                        eReturnStatus = _pxCoorExecute->RawAdc_MuxMode
                                                                (  eTT_EM, 
                                                                  &poCoorCommand->m_xTT, 
                                            (ERawAdc_MuxMode_Type) poCoorCommand->m_uiValue, 
                                                                  _poCoorCommandLine ) ;
                    } ; break ; // case Ki_CoorSubCmd_EM_Tower 

                    /////////////////////////////////////////////////////////////
                    /// HD Et Trigger Tower /////////////////////////////////////
                    /////////////////////////////////////////////////////////////
                case Ki_CoorSubCmd_HD_Tower :
                    {
                        eReturnStatus = _pxCoorExecute->RawAdc_MuxMode
                                                                (  eTT_HD, 
                                                                  &poCoorCommand->m_xTT, 
                                            (ERawAdc_MuxMode_Type) poCoorCommand->m_uiValue, 
                                                                  _poCoorCommandLine ) ;
                    } ; break ; // case Ki_CoorSubCmd_HD_Tower 

                    /////////////////////////////////////////////////////////////
                    // All commands that have been parsed should have been dispatched here
                    // but this catches omissions
                    /////////////////////////////////////////////////////////////
                    default:
                    {
                        strcpy ( sReplyText, "<ADF Raw ADC Mux Sub Command Not implemented ID =" ) ;
                        strcat ( sReplyText, I_to_A ( poCoorCommand->m_uiCoorCmdCode ) ) ;
                        strcat ( sReplyText, "/" ) ;
                        strcat ( sReplyText, I_to_A ( poCoorCommand->m_uiCoorSubCmdCode ) ) ;
                        strcat ( sReplyText, ">" ) ;
                        CConsoleAccess::ErrorLine     ( sReplyText ) ;
                        _poCoorCommandLine->CopyReplyText ( sReplyText ) ;

                        eReturnStatus = Reply_Bad ;
                    } ; break ;
                }
            } break ; // case Ki_TrgMgrCmd_TrgMgr_Adf_RawAdc


            /////////////////////////////////////////////////////////////////////
            /// ADF Raw ADC Mem Commands  ///////////////////////////////////////
            /////////////////////////////////////////////////////////////////////
            case Ki_TrgMgrCmd_TrgMgr_Adf_RawAdc_Mem :
            {
                switch ( poCoorCommand->m_uiCoorSubCmdCode )
                {

                    /////////////////////////////////////////////////////////////
                    /// EM Et Trigger Tower /////////////////////////////////////
                    /////////////////////////////////////////////////////////////
                case Ki_CoorSubCmd_EM_Tower :
                    {
                        eReturnStatus = _pxCoorExecute->RawAdc_MemMode
                                                                (  eTT_EM, 
                                                                  &poCoorCommand->m_xTT, 
                                            (ERawAdc_MemMode_Type) poCoorCommand->m_uiValue, 
                                                                  _poCoorCommandLine ) ;
                    } ; break ; // case Ki_CoorSubCmd_EM_Tower 

                    /////////////////////////////////////////////////////////////
                    /// HD Et Trigger Tower /////////////////////////////////////
                    /////////////////////////////////////////////////////////////
                case Ki_CoorSubCmd_HD_Tower :
                    {
                        eReturnStatus = _pxCoorExecute->RawAdc_MemMode
                                                                (  eTT_HD, 
                                                                  &poCoorCommand->m_xTT, 
                                            (ERawAdc_MemMode_Type) poCoorCommand->m_uiValue, 
                                                                  _poCoorCommandLine ) ;
                    } ; break ; // case Ki_CoorSubCmd_HD_Tower 

                    /////////////////////////////////////////////////////////////
                    // All commands that have been parsed should have been dispatched here
                    // but this catches omissions
                    /////////////////////////////////////////////////////////////
                    default:
                    {
                        strcpy ( sReplyText, "<ADF Raw ADC Mem Sub Command Not implemented ID =" ) ;
                        strcat ( sReplyText, I_to_A ( poCoorCommand->m_uiCoorCmdCode ) ) ;
                        strcat ( sReplyText, "/" ) ;
                        strcat ( sReplyText, I_to_A ( poCoorCommand->m_uiCoorSubCmdCode ) ) ;
                        strcat ( sReplyText, ">" ) ;
                        CConsoleAccess::ErrorLine     ( sReplyText ) ;
                        _poCoorCommandLine->CopyReplyText ( sReplyText ) ;

                        eReturnStatus = Reply_Bad ;
                    } ; break ;
                }
            } break ; // case Ki_TrgMgrCmd_TrgMgr_Adf_RawAdc


            /////////////////////////////////////////////////////////////////////
            /// ADF Output Et Mux Command ///////////////////////////////////////
            /////////////////////////////////////////////////////////////////////
            case Ki_TrgMgrCmd_TrgMgr_Adf_OutputEt_Mux :
            {
                switch ( poCoorCommand->m_uiCoorSubCmdCode )
                {

                    /////////////////////////////////////////////////////////////
                    /// EM Et Trigger Tower /////////////////////////////////////
                    /////////////////////////////////////////////////////////////
                case Ki_CoorSubCmd_EM_Tower :
                    {
                        eReturnStatus = _pxCoorExecute->AdfOut_MuxMode
                                                                (  eTT_EM, 
                                                                  &poCoorCommand->m_xTT, 
                                            (EAdfOut_MuxMode_Type) poCoorCommand->m_uiValue, 
                                                                  _poCoorCommandLine ) ;
                    } ; break ; // case Ki_CoorSubCmd_EM_Tower 

                    /////////////////////////////////////////////////////////////
                    /// HD Et Trigger Tower /////////////////////////////////////
                    /////////////////////////////////////////////////////////////
                case Ki_CoorSubCmd_HD_Tower :
                    {
                        eReturnStatus = _pxCoorExecute->AdfOut_MuxMode
                                                                (  eTT_HD, 
                                                                  &poCoorCommand->m_xTT, 
                                            (EAdfOut_MuxMode_Type) poCoorCommand->m_uiValue, 
                                                                  _poCoorCommandLine ) ;
                    } ; break ; // case Ki_CoorSubCmd_HD_Tower 

                    /////////////////////////////////////////////////////////////
                    // All commands that have been parsed should have been dispatched here
                    // but this catches omissions
                    /////////////////////////////////////////////////////////////
                    default:
                    {
                        strcpy ( sReplyText, "<ADF Output Et Mux Sub Command Not implemented ID =" ) ;
                        strcat ( sReplyText, I_to_A ( poCoorCommand->m_uiCoorCmdCode ) ) ;
                        strcat ( sReplyText, "/" ) ;
                        strcat ( sReplyText, I_to_A ( poCoorCommand->m_uiCoorSubCmdCode ) ) ;
                        strcat ( sReplyText, ">" ) ;
                        CConsoleAccess::ErrorLine     ( sReplyText ) ;
                        _poCoorCommandLine->CopyReplyText ( sReplyText ) ;

                        eReturnStatus = Reply_Bad ;
                    } ; break ;
                }

            } break ; // case Ki_TrgMgrCmd_TrgMgr_Adf_OutputEt


            /////////////////////////////////////////////////////////////////////
            /// ADF Output Et Mem Command ///////////////////////////////////////
            /////////////////////////////////////////////////////////////////////
            case Ki_TrgMgrCmd_TrgMgr_Adf_OutputEt_Mem :
            {
                switch ( poCoorCommand->m_uiCoorSubCmdCode )
                {

                    /////////////////////////////////////////////////////////////
                    /// EM Et Trigger Tower /////////////////////////////////////
                    /////////////////////////////////////////////////////////////
                case Ki_CoorSubCmd_EM_Tower :
                    {
                        eReturnStatus = _pxCoorExecute->AdfOut_MemMode
                                                                (  eTT_EM, 
                                                                  &poCoorCommand->m_xTT, 
                                            (EAdfOut_MemMode_Type) poCoorCommand->m_uiValue, 
                                                                  _poCoorCommandLine ) ;
                    } ; break ; // case Ki_CoorSubCmd_EM_Tower 

                    /////////////////////////////////////////////////////////////
                    /// HD Et Trigger Tower /////////////////////////////////////
                    /////////////////////////////////////////////////////////////
                case Ki_CoorSubCmd_HD_Tower :
                    {
                        eReturnStatus = _pxCoorExecute->AdfOut_MemMode
                                                                (  eTT_HD, 
                                                                  &poCoorCommand->m_xTT, 
                                            (EAdfOut_MemMode_Type) poCoorCommand->m_uiValue, 
                                                                  _poCoorCommandLine ) ;
                    } ; break ; // case Ki_CoorSubCmd_HD_Tower 

                    /////////////////////////////////////////////////////////////
                    // All commands that have been parsed should have been dispatched here
                    // but this catches omissions
                    /////////////////////////////////////////////////////////////
                    default:
                    {
                        strcpy ( sReplyText, "<ADF Output Et Mem Sub Command Not implemented ID =" ) ;
                        strcat ( sReplyText, I_to_A ( poCoorCommand->m_uiCoorCmdCode ) ) ;
                        strcat ( sReplyText, "/" ) ;
                        strcat ( sReplyText, I_to_A ( poCoorCommand->m_uiCoorSubCmdCode ) ) ;
                        strcat ( sReplyText, ">" ) ;
                        CConsoleAccess::ErrorLine     ( sReplyText ) ;
                        _poCoorCommandLine->CopyReplyText ( sReplyText ) ;

                        eReturnStatus = Reply_Bad ;
                    } ; break ;
                }

            } break ; // case Ki_TrgMgrCmd_TrgMgr_Adf_OutputEt


            /////////////////////////////////////////////////////////////////////
            /// Set Monitoring Manager Time Control            //////////////////
            /////////////////////////////////////////////////////////////////////
            case Ki_TrgMgrCmd_TrgMgr_Monit_Control_Time_ms :
            {
                eReturnStatus = _pxCoorExecute->Monit_Control_Time     ( poCoorCommand->m_uiValue,
                                                                        _poCoorCommandLine ) ;

            } break ; // case Ki_TrgMgrCmd_TrgMgr_Monit_Control_Time_ms

            // All commands that have been parsed should have been dispatched here
            // but this catches omissions
            default:
            {
                strcpy ( sReplyText, "<TrgMgr Command Not implemented ID =" ) ;
                strcat ( sReplyText, I_to_A ( poCoorCommand->m_uiCoorCmdCode ) ) ;
                strcat ( sReplyText, ">" ) ;
                CConsoleAccess::ErrorLine     ( sReplyText ) ;
                _poCoorCommandLine->CopyReplyText ( sReplyText ) ;

                eReturnStatus = Reply_Bad ;
            } ; break ;

        } ;

        // find the next command in the list, if any
        // but remember individual errors to tag the whole message as an error
        if ( eReturnStatus == Reply_Bad ) eRememberStatus = Reply_Bad ;

        poCoorCommandEntry = _poCoorCommandLine->m_lCommandList.NextEntry( poCoorCommandEntry ) ;
        if ( poCoorCommandEntry != NULL )
            poCoorCommand = poCoorCommandEntry ->GetAtKey( ) ;
        else
            poCoorCommand = NULL ;

    } ;

    if ( eRememberStatus == Reply_Bad ) eReturnStatus = Reply_Bad ;

    return eReturnStatus ;
}

