/*-------------------------------------------------------------------------------*/
  /*
     MessageMgrParser.cpp
     7-Oct-99
     28-Oct-99 Coor Command Parsing split onto separate files
     22-Sep-2005 Restructred for L1Cal_IIb_TCC
  */
/*-------------------------------------------------------------------------------*/
#include "L1Cal_IIb_TCC/StdAfx.h"

#include "L1Cal_IIb_TCC/GlobalConstants.h"
#include "L1Cal_IIb_TCC/CustomDataTypes.h"

#include "L1Cal_IIb_TCC/MessageMgrCommand.h"

#include                            "L1Cal_IIb_TCC/MessageMgrParser.h"

#include "L1Cal_IIb_TCC/MessageParserMacros.h"

#include "L1Cal_IIb_TCC/MessageKeywordsCoorL1Cal.h"
#include "L1Cal_IIb_TCC/MessageKeywordsTrgMgr.h"

#include "L1Cal_IIb_TCC/L1Cal.h" // to get min-max eta-phi ranges

#include "L1Cal_IIb_TCC/HandleConsole.h"
#include "L1Cal_IIb_TCC/UtilsStrings.h"

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
CMessageMgrParser::CMessageMgrParser ()
{
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
CMessageMgrParser::~CMessageMgrParser ()
{
}


/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
void CMessageMgrParser::AddKeywords       ( CMessageTokenizer*      _poTokenizer )
{

    ADD_TRGMGR_COMMAND     ( TrgMgr_BaseAlignmentDelay      )

    ADD_TRGMGR_COMMAND     ( TrgMgr_OffsetTick1RawAdcMonit  )
    ADD_TRGMGR_COMMAND     ( TrgMgr_OffsetTick1RawAdcSimu   )
    ADD_TRGMGR_COMMAND     ( TrgMgr_OffsetTick1OutputEtMonit)
    ADD_TRGMGR_COMMAND     ( TrgMgr_OffsetTick1OutputEtSimu )
    ADD_TRGMGR_COMMAND     ( TrgMgr_OffsetTriggeredToTickAtCapture )

    ADD_TRGMGR_COMMAND     ( TrgMgr_L1Cal_Operational       )
    ADD_TRGMGR_COMMAND     ( TrgMgr_L1Cal_Not_Operational   )

    ADD_TRGMGR_COMMAND     ( TrgMgr_Monit_Control_Time_ms   )

    ADD_TRGMGR_COMMAND     ( TrgMgr_NewLogFile              )

    ADD_TRGMGR_COMMAND     ( TrgMgr_L1Cal_Coverage          )
    ADD_TRGMGR_COMMAND     ( TrgMgr_Find_DAC                )
    ADD_TRGMGR_COMMAND     ( TrgMgr_Adjust_Ped              )
    ADD_TRGMGR_COMMAND     ( TrgMgr_Revert_Ped              )
    ADD_TRGMGR_COMMAND     ( TrgMgr_Profile_BLS             )
    ADD_TRGMGR_COMMAND     ( TrgMgr_Profile_AdfEt           )

    ADD_TRGMGR_COMMAND     ( TrgMgr_AdfOut_Const            )
    ADD_TRGMGR_COMMAND     ( TrgMgr_RawAdc_Thresh           )

    ADD_TRGMGR_COMMAND     ( TrgMgr_Adf_OutputEt_Mux        )

    ADD_TRGMGR_KEYWORD     ( Mux_Filtered_Always            )
    ADD_TRGMGR_KEYWORD     ( Mux_Filtered_LiveX             )
    ADD_TRGMGR_KEYWORD     ( Mux_Constant_Always            )
    ADD_TRGMGR_KEYWORD     ( Mux_Prn_Always                 )
    ADD_TRGMGR_KEYWORD     ( Mux_Prn_LiveX                  )
    ADD_TRGMGR_KEYWORD     ( Mux_SimuTurn_Always            )
    ADD_TRGMGR_KEYWORD     ( Mux_SimuTurn_LiveX             )

    ADD_TRGMGR_COMMAND     ( TrgMgr_Adf_OutputEt_Mem        )

    ADD_TRGMGR_KEYWORD     ( Mem_CaptureMonitor             )
    ADD_TRGMGR_KEYWORD     ( Mem_CaptureAtThresh            )
    ADD_TRGMGR_KEYWORD     ( Mem_CaptureImmediate           )
    ADD_TRGMGR_KEYWORD     ( Mem_PlaySimuTurn               )

    ADD_TRGMGR_COMMAND     ( TrgMgr_Adf_RawAdc_Mux          )

    ADD_TRGMGR_KEYWORD     ( Mux_BlsAligned                 )
    ADD_TRGMGR_KEYWORD     ( Mux_SimuTurn                   )

    ADD_TRGMGR_COMMAND     ( TrgMgr_Adf_RawAdc_Mem          )

}



/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ECoorMessageStatus      CMessageMgrParser::ParseMessage     ( CMessageCommandLine*  _poCoorCommandLine )
{
    ECoorMessageStatus  eReturnStatus = Reply_Ok ;

    char                sReplyText[KiMaxLengthReplyTextToCoor] ;

    TBinomialEntry< TokenLocAndType, unsigned int > * poTokenEntry ;
//    TBinomialEntry< TokenLocAndType, unsigned int > * poTokenFirstSubCommand ;

    TList < unsigned int, EItemPolarity >           lObjectList ;
//    TBinomialEntry < unsigned int, EItemPolarity >* poObjectEntry ;

    CMessageCommand*    poCoorCommand ;

    ETokenType          eTokenType ;
    unsigned int        uiCoorCmdCode ;
    unsigned int        uiCoorCmdSubCode ;
    unsigned int        uiKeyword ;
//    unsigned int        uiValue ;
//    unsigned int        uiObjectIndex ;
//    unsigned int        uiObjectIndexLower ;
//    unsigned int        uiObjectIndexUpper ;
//    EItemPolarity       eObjectPolarity ;

    unsigned int        uiItemIndex ;
    unsigned int        uiItemIndexLower ;
    unsigned int        uiItemIndexUpper ;
    ETT_Eta_Sign        ePolarityIndexLower ;
    ETT_Eta_Sign        ePolarityIndexUpper ;

    //\\//

    poTokenEntry = _poCoorCommandLine->m_lTokenList.FirstEntry( ) ;

    //We have already enforced during the tokenization that the first token is a Keyword
    uiCoorCmdCode   = poTokenEntry->GetValue ( ) ;

    //Create the first command in the Command List
    _poCoorCommandLine->m_lCommandList.AddEntry ( ) ;

    //Get at the Created command
    poCoorCommand = _poCoorCommandLine->m_lCommandList.LastEntry()->GetAtKey( ) ;

    //record the command Code
    poCoorCommand->m_uiCoorCmdCode = uiCoorCmdCode ;

    poTokenEntry = _poCoorCommandLine->m_lTokenList.NextEntry( poTokenEntry ) ;

    switch ( uiCoorCmdCode )
    {
        /////////////////////////////////////////////////////////////////////////////
        // single keyword commands
        /////////////////////////////////////////////////////////////////////////////
        case Ki_TrgMgrCmd_TrgMgr_L1Cal_Operational :      // [TrgMgr_L1Cal_Operational]
        case Ki_TrgMgrCmd_TrgMgr_L1Cal_Not_Operational:   // [TrgMgr_L1Cal_Not_Operational]
        case Ki_TrgMgrCmd_TrgMgr_NewLogFile :             // [TrgMgr_NewLogFile]
        {
            if ( poTokenEntry != NULL )
                ERROR_UNEXPECTED_TOKEN() ;

        } ; break ;


        /////////////////////////////////////////////////////////////////////////////
        // single keyword with single number
        /////////////////////////////////////////////////////////////////////////////
        case Ki_TrgMgrCmd_TrgMgr_BaseAlignmentDelay       : // [TrgMgr_BaseAlignmentDelay N]
        case Ki_TrgMgrCmd_TrgMgr_OffsetTick1RawAdcMonit   : // [OffsetTick1RawAdcMonit N]
        case Ki_TrgMgrCmd_TrgMgr_OffsetTick1RawAdcSimu    : // [OffsetTick1RawAdcSimu N]
        case Ki_TrgMgrCmd_TrgMgr_OffsetTick1OutputEtMonit : // [OffsetTick1OutputEtMonit N]
        case Ki_TrgMgrCmd_TrgMgr_OffsetTick1OutputEtSimu  : // [OffsetTick1OutputEtSimu N]
        case Ki_TrgMgrCmd_TrgMgr_OffsetTriggeredToTickAtCapture  : // [OffsetTriggeredToTickAtCapture N]
        case Ki_TrgMgrCmd_TrgMgr_Monit_Control_Time_ms    : // [TrgMgr_Monit_Control_Time_ms N]
        {
            if ( poTokenEntry == NULL )
                ERROR_MISSING_NUMBER() ;

            eTokenType = poTokenEntry->GetKey().eTokenType ;

            if ( eTokenType != TokenIsValueAsserted )
                ERROR_NUMBER_NOT_ASSERTED() ;

            poCoorCommand->m_uiValue        = poTokenEntry->GetValue ( ) ;
            poCoorCommand->m_eValuePolarity = ItemAsserted ;


            poTokenEntry = _poCoorCommandLine->m_lTokenList.NextEntry( poTokenEntry ) ;

            if ( poTokenEntry != NULL )
                ERROR_UNEXPECTED_TOKEN() ;

        } ; break ;


        /////////////////////////////////////////////////////////////////////////////
        // complex commands with first a list of numbers, then another keyword
        /////////////////////////////////////////////////////////////////////////////
/*/
        case Ki_TrgMgrCmd_TrgMgr_Foreign_Scaler              ://[TrgMgr_Foreign_Scaler N  Base_Tick_Select X]
        {
            //there must be at least one number
            if ( poTokenEntry == NULL )
                ERROR_MISSING_NUMBER() ;

            eTokenType = poTokenEntry->GetKey().eTokenType ;

            if ( ( eTokenType != TokenIsValueAsserted )
              && ( eTokenType != TokenIsValueNegated )
              && ( eTokenType != TokenIsValueLower )
              && ( eTokenType != TokenIsValueLowerNeg ) )
                ERROR_MISSING_NUMBER() ;

            //there may be more than one number or range
            while ( poTokenEntry != NULL )
            {
                eTokenType = poTokenEntry->GetKey().eTokenType ;

                if ( ( eTokenType != TokenIsValueAsserted )
                  && ( eTokenType != TokenIsValueNegated )
                  && ( eTokenType != TokenIsValueLower )
                  && ( eTokenType != TokenIsValueLowerNeg ) )
                    break ; // we are done with the object list, now on to the keywords

                switch ( eTokenType )
                {
                    case TokenIsValueAsserted : eObjectPolarity = ItemAsserted ; break ;
                    case TokenIsValueNegated  : eObjectPolarity = ItemNegated  ; break ;
                    case TokenIsValueLower    : eObjectPolarity = ItemAsserted ; break ;
                    case TokenIsValueLowerNeg : eObjectPolarity = ItemNegated  ; break ;
                    default : break ;
                }

                uiObjectIndexLower = poTokenEntry->GetValue ( ) ;

                if ( ( eTokenType == TokenIsValueLower )
                  || ( eTokenType == TokenIsValueLowerNeg ) )
                {
                    poTokenEntry = _poCoorCommandLine->m_lTokenList.NextEntry( poTokenEntry ) ;

                    if ( poTokenEntry == NULL )
                        ERROR_MISSING_UPPER_BOUND() ;

                    eTokenType = poTokenEntry->GetKey().eTokenType ;

                    if ( ( eTokenType != TokenIsValueUpper )
                      && ( eTokenType != TokenIsValueUpperNeg ) )
                        ERROR_MISSING_UPPER_BOUND() ;

                    if ( ( ( eObjectPolarity == ItemAsserted )
                        && ( eTokenType != TokenIsValueUpper ) )
                      || ( ( eObjectPolarity == ItemNegated  )
                        && ( eTokenType != TokenIsValueUpperNeg ) ) )
                        ERROR_OPPOSITE_POLARITIES() ;

                    uiObjectIndexUpper = poTokenEntry->GetValue ( ) ;
                }
                else
                    uiObjectIndexUpper = uiObjectIndexLower ;

                if ( ( uiCoorCmdCode == Ki_TrgMgrCmd_TrgMgr_Foreign_Scaler )
                  && ( ( uiObjectIndexLower > KiMaxIndexInCoorCmd_ForeignScaler )
                    || ( uiObjectIndexUpper > KiMaxIndexInCoorCmd_ForeignScaler ) ) )
                      ERROR_OBJECT_NUMBER_OUT_OF_RANGE() ;


                if ( uiObjectIndexLower > uiObjectIndexUpper )
                    SWAP_UI_VALUES( uiObjectIndexLower, uiObjectIndexUpper ) ;


                for ( uiObjectIndex  = uiObjectIndexLower ;
                      uiObjectIndex <= uiObjectIndexUpper ;
                      uiObjectIndex  ++ )
                    lObjectList.AddEntry ( uiObjectIndex, eObjectPolarity ) ;

                poTokenEntry = _poCoorCommandLine->m_lTokenList.NextEntry( poTokenEntry ) ;
            }

            if ( poTokenEntry == NULL )
                ERROR_MISSING_SUB_KEYWORD() ;

            eTokenType   = poTokenEntry->GetKey().eTokenType ;

            if ( eTokenType != TokenIsKeyword )
                ERROR_MISSING_SUB_KEYWORD() ;

            // *************************************************************
            // *** The label below is is where we jump back to accept
            // *** multiple object numbers
            // *** e.g. L1FW_Spec_Trig 0 -1 3 Obey_FE_Busy
            // *** We start with the first Object in the list,
            // *** and remember the current position in the token list
            // *** to restore and parse the line again
            // *************************************************************

            poObjectEntry = lObjectList.FirstEntry ( ) ;
            poTokenFirstSubCommand = poTokenEntry ;

        ParseAllSubCommands :
            poTokenEntry     = poTokenFirstSubCommand ;

            uiObjectIndex    = poObjectEntry->GetKey() ;
            eObjectPolarity  = poObjectEntry->GetValue() ;

            // *************************************************************
            // *** this is where we jump back to accept multiple sub commands
            // *** e.g. L1FW_Spec_Trig 0 Prescale 1000 Coor_enable
            // *************************************************************

        ParseNextSubCommand :

            uiCoorCmdSubCode   = poTokenEntry->GetValue ( ) ;

            //Get at the Current command
            poCoorCommand = _poCoorCommandLine->m_lCommandList.LastEntry()->GetAtKey( ) ;
            //copy the command Code and object description to the command which may be new
            poCoorCommand->m_uiCoorCmdCode    = uiCoorCmdCode ;
            poCoorCommand->m_uiObjectIndex    = uiObjectIndex ;
            poCoorCommand->m_eObjectPolarity  = eObjectPolarity ;
            poCoorCommand->m_uiCoorSubCmdCode = uiCoorCmdSubCode ;

            if ( ( uiCoorCmdCode    == Ki_TrgMgrCmd_TrgMgr_Foreign_Scaler )
              && ( uiCoorCmdSubCode != Ki_TrgMgrSubCmd_Base_Tick_Select ) )
                  ERROR_ILLEGAL_SUB_KEYWORD() ;

            // Now that we know the sub-command we need to GO BACK TO CHECK
            // whether the Item was specified asserted or negated.
            // Only a few commands accept a negated item number
            if ( eObjectPolarity == ItemNegated )     // negated SpTrg only allowed for...
              ERROR_SUB_KEYWORD_ONLY_FOR_ASSERTED_OBJECTS() ;

            poTokenEntry = _poCoorCommandLine->m_lTokenList.NextEntry( poTokenEntry ) ;

            switch ( uiCoorCmdSubCode )
            {
                case Ki_TrgMgrSubCmd_Base_Tick_Select          ://[TrgMgr_Foreign_Scaler N  Base_Tick_Select X]
                {
                    if ( poTokenEntry == NULL )
                        ERROR_MISSING_NUMBER() ;

                    eTokenType = poTokenEntry->GetKey().eTokenType ;

                    if ( eTokenType != TokenIsValueAsserted )
                        ERROR_NUMBER_NOT_ASSERTED() ;

                    poCoorCommand->m_eValuePolarity = ItemAsserted ;


                    uiValue = poTokenEntry->GetValue ( ) ;

                    if ( ( uiCoorCmdSubCode == Ki_TrgMgrSubCmd_Base_Tick_Select )
                      && ( uiValue > KiMaxIndexInCoorCmd_TickNumber ) )
                        ERROR_OBJECT_NUMBER_OUT_OF_RANGE() ;

                    poCoorCommand->m_uiValue        = uiValue ;

                    poTokenEntry = _poCoorCommandLine->m_lTokenList.NextEntry( poTokenEntry ) ;

                    if ( poTokenEntry != NULL )
                    {
                        eTokenType = poTokenEntry->GetKey().eTokenType ;

                        if ( eTokenType != TokenIsKeyword )
                            ERROR_UNEXPECTED_TOKEN() ;

                        //Create another command in the Command List
                        _poCoorCommandLine->m_lCommandList.AddEntry ( ) ;
                        goto ParseNextSubCommand ;
                    }

                } ; break ;


            } // switch SubCommandCode

            // now see if there were more Objects specified on this command line
            poObjectEntry = lObjectList.NextEntry( poObjectEntry ) ;
            if ( poObjectEntry != NULL )
            {
                //Create another command in the Command List
                _poCoorCommandLine->m_lCommandList.AddEntry ( ) ;
                goto ParseAllSubCommands ;
            }

        } ; break ;
/*/

        /////////////////////////////////////////////////////////////////////////////
        //[TrgMgr_L1Cal_Coverage         TT_Eta(-20:20) ]
        //[TrgMgr_L1Cal_Coverage         TT_Eta(-20:20) TT_Phi(1:32)]
        //
        //[TrgMgr_Find_DAC          EM_Tower TT_Eta(20) TT_Phi(23)]
        //[TrgMgr_Find_DAC          HD_Tower TT_Eta(20) TT_Phi(23)]
        //[TrgMgr_Adjust_Ped        EM_Tower TT_Eta(20) TT_Phi(23)]
        //[TrgMgr_Adjust_Ped        HD_Tower TT_Eta(20) TT_Phi(23)]
        //[TrgMgr_Revert_Ped        EM_Tower TT_Eta(20) TT_Phi(23)]
        //[TrgMgr_Revert_Ped        HD_Tower TT_Eta(20) TT_Phi(23)]
        //[TrgMgr_Profile_BLS       EM_Tower TT_Eta(20) TT_Phi(23)]
        //[TrgMgr_Profile_BLS       HD_Tower TT_Eta(20) TT_Phi(23)]
        //[TrgMgr_Profile_AdfEt     EM_Tower TT_Eta(20) TT_Phi(23)]
        //[TrgMgr_Profile_AdfEt     HD_Tower TT_Eta(20) TT_Phi(23)]
        //
        //[TrgMgr_AdfOut_Const  EM_Tower TT_Eta(-20:20) TT_Phi(1:32) Value 8]
        //[TrgMgr_AdfOut_Const  HD_Tower TT_Eta(20) TT_Phi(23) Value 8]
        //
        //[TrgMgr_RawAdc_Thresh EM_Tower TT_Eta(-20:20) TT_Phi(1:32) Value 100]
        //[TrgMgr_RawAdc_Thresh HD_Tower TT_Eta(20) TT_Phi(23) Value 100]
        //
        //[TrgMgr_Adf_OutputEt_Mux  EM_Tower TT_Eta(-2) TT_Phi(13)  Mux_Filtered_Always  ]
        //[TrgMgr_Adf_OutputEt_Mux  EM_Tower TT_Eta(-2) TT_Phi(13)  Mux_Filtered_LiveX   ]
        //[TrgMgr_Adf_OutputEt_Mux  EM_Tower TT_Eta(-2) TT_Phi(13)  Mux_Constant_Always  ]
        //[TrgMgr_Adf_OutputEt_Mux  EM_Tower TT_Eta(-2) TT_Phi(13)  Mux_Prn_Always       ]
        //[TrgMgr_Adf_OutputEt_Mux  EM_Tower TT_Eta(-2) TT_Phi(13)  Mux_Prn_LiveX        ]
        //[TrgMgr_Adf_OutputEt_Mux  EM_Tower TT_Eta(-2) TT_Phi(13)  Mux_SimuTurn_Always  ]
        //[TrgMgr_Adf_OutputEt_Mux  EM_Tower TT_Eta(-2) TT_Phi(13)  Mux_SimuTurn_LiveX   ]
        //[TrgMgr_Adf_OutputEt_Mem  EM_Tower TT_Eta(-2) TT_Phi(13)  Mem_CaptureMonitor   ]
        //[TrgMgr_Adf_OutputEt_Mem  EM_Tower TT_Eta(-2) TT_Phi(13)  Mem_CaptureAtThresh  ]
        //[TrgMgr_Adf_OutputEt_Mem  EM_Tower TT_Eta(-2) TT_Phi(13)  Mem_CaptureImmediate ]
        //[TrgMgr_Adf_OutputEt_Mem  EM_Tower TT_Eta(-2) TT_Phi(13)  Mem_PlaySimuTurn     ]
        //
        //[TrgMgr_Adf_RawAdc_Mux    EM_Tower TT_Eta(-2) TT_Phi(13)  Mux_BlsAligned       ]
        //[TrgMgr_Adf_RawAdc_Mux    EM_Tower TT_Eta(-2) TT_Phi(13)  Mux_SimuTurn         ]
        //[TrgMgr_Adf_RawAdc_Mem    EM_Tower TT_Eta(-2) TT_Phi(13)  Mem_CaptureMonitor   ]
        //[TrgMgr_Adf_RawAdc_Mem    EM_Tower TT_Eta(-2) TT_Phi(13)  Mem_CaptureAtThresh  ]
        //[TrgMgr_Adf_RawAdc_Mem    EM_Tower TT_Eta(-2) TT_Phi(13)  Mem_CaptureImmediate ]
        //[TrgMgr_Adf_RawAdc_Mem    EM_Tower TT_Eta(-2) TT_Phi(13)  Mem_PlaySimuTurn     ]
        /////////////////////////////////////////////////////////////////////////////
        case Ki_TrgMgrCmd_TrgMgr_L1Cal_Coverage :
        case Ki_TrgMgrCmd_TrgMgr_Find_DAC :
        case Ki_TrgMgrCmd_TrgMgr_Adjust_Ped :
        case Ki_TrgMgrCmd_TrgMgr_Revert_Ped :
        case Ki_TrgMgrCmd_TrgMgr_Profile_BLS :
        case Ki_TrgMgrCmd_TrgMgr_Profile_AdfEt :
        case Ki_TrgMgrCmd_TrgMgr_AdfOut_Const :
        case Ki_TrgMgrCmd_TrgMgr_RawAdc_Thresh :
        case Ki_TrgMgrCmd_TrgMgr_Adf_OutputEt_Mux :
        case Ki_TrgMgrCmd_TrgMgr_Adf_OutputEt_Mem :
        case Ki_TrgMgrCmd_TrgMgr_Adf_RawAdc_Mux :
        case Ki_TrgMgrCmd_TrgMgr_Adf_RawAdc_Mem :
        {
            if ( ( uiCoorCmdCode    == Ki_TrgMgrCmd_TrgMgr_Find_DAC         )
              || ( uiCoorCmdCode    == Ki_TrgMgrCmd_TrgMgr_Adjust_Ped       )
              || ( uiCoorCmdCode    == Ki_TrgMgrCmd_TrgMgr_Revert_Ped       )
              || ( uiCoorCmdCode    == Ki_TrgMgrCmd_TrgMgr_Profile_BLS      )
              || ( uiCoorCmdCode    == Ki_TrgMgrCmd_TrgMgr_Profile_AdfEt    )
              || ( uiCoorCmdCode    == Ki_TrgMgrCmd_TrgMgr_AdfOut_Const     )
              || ( uiCoorCmdCode    == Ki_TrgMgrCmd_TrgMgr_RawAdc_Thresh    )
              || ( uiCoorCmdCode    == Ki_TrgMgrCmd_TrgMgr_Adf_OutputEt_Mux )
              || ( uiCoorCmdCode    == Ki_TrgMgrCmd_TrgMgr_Adf_OutputEt_Mem )
              || ( uiCoorCmdCode    == Ki_TrgMgrCmd_TrgMgr_Adf_RawAdc_Mux   )
              || ( uiCoorCmdCode    == Ki_TrgMgrCmd_TrgMgr_Adf_RawAdc_Mem   ) )
            {
                // next keyword should be the type of Trigger Tower
                // or the sub command type
                if ( poTokenEntry == NULL )
                    ERROR_MISSING_SUB_KEYWORD() ;

                eTokenType = poTokenEntry->GetKey().eTokenType ;

                if ( eTokenType != TokenIsKeyword )
                    ERROR_MISSING_SUB_KEYWORD() ;

                uiCoorCmdSubCode   = poTokenEntry->GetValue ( ) ;

                if ( ( uiCoorCmdSubCode != Ki_CoorSubCmd_EM_Tower                 )
                  && ( uiCoorCmdSubCode != Ki_CoorSubCmd_HD_Tower                 ) )
                  ERROR_ILLEGAL_SUB_KEYWORD() ;

                //copy the SubCommand Code
                poCoorCommand->m_uiCoorSubCmdCode = uiCoorCmdSubCode ;

                // Next Token may be either "TT_Eta", "TT_Phi", or even nothing
                poTokenEntry = _poCoorCommandLine->m_lTokenList.NextEntry( poTokenEntry ) ;
            }

            // Next Token may be either "TT_Eta", "TT_Phi", or even nothing
            Look_For_TT_Range_RefSet :

                if ( poTokenEntry != NULL )
                {
                    eTokenType = poTokenEntry->GetKey().eTokenType ;

                    if ( eTokenType != TokenIsKeyword )
                        ERROR_MISSING_SUB_KEYWORD() ;

                    uiKeyword = poTokenEntry->GetValue ( ) ;


                    if ( ( uiKeyword == Ki_CoorKeyword_TT_Eta )
                      || ( uiKeyword == Ki_CoorKeyword_TT_Phi ) )
                    {
                        PARSE_TT_RANGE( Look_For_TT_Range_RefSet ) ;
                    }
                }


            if ( ( uiCoorCmdCode    == Ki_TrgMgrCmd_TrgMgr_AdfOut_Const     )
              || ( uiCoorCmdCode    == Ki_TrgMgrCmd_TrgMgr_RawAdc_Thresh    )
              || ( uiCoorCmdCode    == Ki_TrgMgrCmd_TrgMgr_Adf_OutputEt_Mux )
              || ( uiCoorCmdCode    == Ki_TrgMgrCmd_TrgMgr_Adf_OutputEt_Mem )
              || ( uiCoorCmdCode    == Ki_TrgMgrCmd_TrgMgr_Adf_RawAdc_Mux   )
              || ( uiCoorCmdCode    == Ki_TrgMgrCmd_TrgMgr_Adf_RawAdc_Mem   ) )
            {
                // next keyword should be the type of Trigger Tower
                // or the sub command type
                if ( poTokenEntry == NULL )
                    ERROR_MISSING_SUB_KEYWORD() ;

                if      (  ( uiCoorCmdCode    == Ki_TrgMgrCmd_TrgMgr_Adf_OutputEt_Mux )
                        && ( ( uiKeyword == Ki_TrgMgrKeyword_Mux_Filtered_Always    )
                          || ( uiKeyword == Ki_TrgMgrKeyword_Mux_Filtered_LiveX     )
                          || ( uiKeyword == Ki_TrgMgrKeyword_Mux_Constant_Always    )
                          || ( uiKeyword == Ki_TrgMgrKeyword_Mux_Prn_Always         )
                          || ( uiKeyword == Ki_TrgMgrKeyword_Mux_Prn_LiveX          )
                          || ( uiKeyword == Ki_TrgMgrKeyword_Mux_SimuTurn_Always    )
                          || ( uiKeyword == Ki_TrgMgrKeyword_Mux_SimuTurn_LiveX     ) ) ) 
                {
                    // just decode and remember the setting in the value, 
                    // so that the dispatcher can directly pass it to the executer routine

                    switch ( uiKeyword )
                    {
                       case Ki_TrgMgrKeyword_Mux_Filtered_Always    :
                            poCoorCommand->m_uiValue = eAdfOut_MuxMode_Filtered_Always ;
                       break ;

                       case Ki_TrgMgrKeyword_Mux_Filtered_LiveX    :
                            poCoorCommand->m_uiValue = eAdfOut_MuxMode_Filtered_LiveX ;
                       break ;

                       case Ki_TrgMgrKeyword_Mux_Constant_Always    :
                            poCoorCommand->m_uiValue = eAdfOut_MuxMode_Constant_Always ;
                       break ;

                       case Ki_TrgMgrKeyword_Mux_Prn_Always         :
                            poCoorCommand->m_uiValue = eAdfOut_MuxMode_Prn_Always ;  
                       break ;

                       case Ki_TrgMgrKeyword_Mux_Prn_LiveX          :
                            poCoorCommand->m_uiValue = eAdfOut_MuxMode_Prn_LiveX ;       
                       break ;

                       case Ki_TrgMgrKeyword_Mux_SimuTurn_Always    :
                            poCoorCommand->m_uiValue = eAdfOut_MuxMode_SimuTurn_Always ;
                       break ;

                       case Ki_TrgMgrKeyword_Mux_SimuTurn_LiveX     :
                            poCoorCommand->m_uiValue = eAdfOut_MuxMode_SimuTurn_LiveX ;
                       break ;

                       default : // nothing to do, the error will be flagged by the "else"
                       break ;
                    }
                }

                else if (  ( uiCoorCmdCode    == Ki_TrgMgrCmd_TrgMgr_Adf_OutputEt_Mem )
                        && ( ( uiKeyword == Ki_TrgMgrKeyword_Mem_CaptureMonitor  )
                          || ( uiKeyword == Ki_TrgMgrKeyword_Mem_CaptureAtThresh )
                          || ( uiKeyword == Ki_TrgMgrKeyword_Mem_CaptureImmediate)
                          || ( uiKeyword == Ki_TrgMgrKeyword_Mem_PlaySimuTurn    ) ) )
                {
                    switch ( uiKeyword )
                    {
                       case Ki_TrgMgrKeyword_Mem_CaptureMonitor  :
                            poCoorCommand->m_uiValue = eAdfOut_MemMode_CaptureMonitor   ;
                       break ;

                       case Ki_TrgMgrKeyword_Mem_CaptureAtThresh :
                            poCoorCommand->m_uiValue = eAdfOut_MemMode_CaptureAtThresh  ;
                       break ;

                       case Ki_TrgMgrKeyword_Mem_CaptureImmediate:
                            poCoorCommand->m_uiValue = eAdfOut_MemMode_CaptureImmediate ;
                       break ;

                       case Ki_TrgMgrKeyword_Mem_PlaySimuTurn    :
                            poCoorCommand->m_uiValue = eAdfOut_MemMode_PlaySimuTurn     ;
                       break ;

                       default : 
                       break ;
                    }
                }

                else if (  ( uiCoorCmdCode    == Ki_TrgMgrCmd_TrgMgr_Adf_RawAdc_Mux )
                        && ( ( uiKeyword == Ki_TrgMgrKeyword_Mux_BlsAligned         )
                          || ( uiKeyword == Ki_TrgMgrKeyword_Mux_SimuTurn           ) ) ) 
                {
                    switch ( uiKeyword )
                    {
                       case Ki_TrgMgrKeyword_Mux_BlsAligned         :
                            poCoorCommand->m_uiValue = eRawAdc_MuxMode_BlsAligned          ;
                       break ;

                       case Ki_TrgMgrKeyword_Mux_SimuTurn           :
                            poCoorCommand->m_uiValue = eRawAdc_MuxMode_SimuTurn            ;
                       break ;

                       default : 
                       break ;
                    }
                }

                else if (  ( uiCoorCmdCode    == Ki_TrgMgrCmd_TrgMgr_Adf_RawAdc_Mem )
                        && ( ( uiKeyword == Ki_TrgMgrKeyword_Mem_CaptureMonitor  )
                          || ( uiKeyword == Ki_TrgMgrKeyword_Mem_CaptureAtThresh )
                          || ( uiKeyword == Ki_TrgMgrKeyword_Mem_CaptureImmediate)
                          || ( uiKeyword == Ki_TrgMgrKeyword_Mem_PlaySimuTurn    ) ) ) 
                {
                    switch ( uiKeyword )
                    {
                       case Ki_TrgMgrKeyword_Mem_CaptureMonitor  :
                            poCoorCommand->m_uiValue = eRawAdc_MemMode_CaptureMonitor   ;
                       break ;

                       case Ki_TrgMgrKeyword_Mem_CaptureAtThresh :
                            poCoorCommand->m_uiValue = eRawAdc_MemMode_CaptureAtThresh  ;
                       break ;

                       case Ki_TrgMgrKeyword_Mem_CaptureImmediate:
                            poCoorCommand->m_uiValue = eRawAdc_MemMode_CaptureImmediate ;
                       break ;

                       case Ki_TrgMgrKeyword_Mem_PlaySimuTurn    :
                            poCoorCommand->m_uiValue = eRawAdc_MemMode_PlaySimuTurn     ;
                       break ;

                       default : 
                       break ;
                    }
                }


                else if ( ( ( uiCoorCmdCode == Ki_TrgMgrCmd_TrgMgr_AdfOut_Const    ) // The TrgMgr_AdfOut_Const command requires a value
                         || ( uiCoorCmdCode == Ki_TrgMgrCmd_TrgMgr_RawAdc_Thresh   ) ) // The TrgMgr_RawAdc_Thresh command requires a value
                    && ( uiKeyword == Ki_CoorKeyword_Value ) )
                {
                    // Get the desired constant ADC Value
                    poTokenEntry = _poCoorCommandLine->m_lTokenList.NextEntry( poTokenEntry ) ;
                
                    if ( poTokenEntry == NULL )
                        ERROR_MISSING_NUMBER() ;
                
                    eTokenType = poTokenEntry->GetKey().eTokenType ;
                
                    if ( eTokenType != TokenIsValueAsserted )
                        ERROR_NUMBER_NOT_ASSERTED() ;

                    // copy the integer part of the Ref Set Threshold Value
                    poCoorCommand->m_eValuePolarity = ItemAsserted ; // This flags that we found a value
                    poCoorCommand->m_uiValue = poTokenEntry->GetValue ( ) ;

                    // check the value magnitude
                    if ( ( ( uiCoorCmdCode == Ki_TrgMgrCmd_TrgMgr_AdfOut_Const     )     // each command type ...
                        && ( poCoorCommand->m_uiValue > KiMaxValueInOutputConstant ) )  //... has its own max term number
                      || ( ( uiCoorCmdCode == Ki_TrgMgrCmd_TrgMgr_RawAdc_Thresh    )
                        && ( poCoorCommand->m_uiValue > KiMaxValueInRawAdcThresh   ) ) )
                        ERROR_ILLEGAL_VALUE() ;

                }
                else
                {
                   ERROR_UNEXPECTED_TOKEN() ;
                }

                poTokenEntry = _poCoorCommandLine->m_lTokenList.NextEntry( poTokenEntry ) ;
            }

            // there should be no more tokens on the line
            if ( poTokenEntry  != NULL )
                ERROR_UNEXPECTED_TOKEN() ;

            // if the Eta range was omitted, this means all Eta values are selected
            FILL_ETA_RANGE_IF_OMITTED() ;

            // if the Phi range was omitted, this means all Phi values are selected
            FILL_PHI_RANGE_IF_OMITTED() ;


        } break ;


        /////////////////////////////////////////////////////////////////////////////

        default :
            ERROR_KEYWORD_ILLEGAL_AT_START() ;

    } ;

Done:

    return eReturnStatus ;
}

