/*-------------------------------------------------------------------------------*/
  /*
     MessageMgrPrinter.cpp
     8-Jun-2000
     22-Sep-2005 Restructred for L1Cal_IIb_TCC
  */
/*-------------------------------------------------------------------------------*/
#include "L1Cal_IIb_TCC/StdAfx.h"

#include "L1Cal_IIb_TCC/GlobalConstants.h"
#include "L1Cal_IIb_TCC/CustomDataTypes.h"

#include "L1Cal_IIb_TCC/MessageCommand.h"
#include                            "L1Cal_IIb_TCC/MessageMgrPrinter.h"

#include "L1Cal_IIb_TCC/MessageCoorPrinter.h"   // to display TT ranges

#include "L1Cal_IIb_TCC/HandleConsole.h"
#include "L1Cal_IIb_TCC/UtilsStrings.h"

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
static char    sReplyText[1000] ;  // generous buffer size to rule out... 
static char    sScreenText[1000] ; // ...overwriting past end of string
                            // This is ok because there is only one executer/printer thread
/////////////////////////////////////////////////////////////////////////////////////
static void AppendTextToReply ( CMessageCommandLine*    _poCoorCommandLine )
{
    /* append stringto reply text*/
    strcpy ( sScreenText, "Process Msg Command : " ) ;
    strcat ( sScreenText, sReplyText ) ;
    CConsoleAccess::WriteLine     ( sScreenText ) ;

    if ( _poCoorCommandLine != NULL )
        _poCoorCommandLine->AppendReplyText ( sReplyText ) ;
    
    return ;
}
/////////////////////////////////////////////////////////////////////////////////////

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ECoorMessageStatus  CMessageMgrPrinter::BaseAlignmentDelay        ( const unsigned long     _ulBaseDelay,
                                                                    CMessageCommandLine*    _poCoorCommandLine )
{
    strcpy ( sReplyText, "<TrgMgr Set L1Cal Trigger Tower Base Alignment Delay=" ) ;
    strcat ( sReplyText, I_to_A(_ulBaseDelay) ) ;
    strcat ( sReplyText, ">" ) ;

    AppendTextToReply ( _poCoorCommandLine ) ;

    CConsoleAccess::SystemLine     ( sReplyText ) ;

    return Reply_Ok ;
}
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ECoorMessageStatus  CMessageMgrPrinter::OffsetTick1RawAdcMonit    ( const unsigned long     _ulIndexNum,
                                                                    CMessageCommandLine*    _poCoorCommandLine )
{
    strcpy ( sReplyText, "<TrgMgr Set Raw ADC Monit Data Tick#1/SubTick#0 Base Index=" ) ;
    strcat ( sReplyText, I_to_A(_ulIndexNum) ) ;
    strcat ( sReplyText, ">" ) ;

    AppendTextToReply ( _poCoorCommandLine ) ;

    CConsoleAccess::SystemLine     ( sReplyText ) ;

    return Reply_Ok ;
}
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ECoorMessageStatus  CMessageMgrPrinter::OffsetTick1RawAdcSimu     ( const unsigned long     _ulIndexNum,
                                                                    CMessageCommandLine*    _poCoorCommandLine )
{
    strcpy ( sReplyText, "<TrgMgr Set Raw ADC Simu Data Tick#1/SubTick#0 Base Index=" ) ;
    strcat ( sReplyText, I_to_A(_ulIndexNum) ) ;
    strcat ( sReplyText, ">" ) ;

    AppendTextToReply ( _poCoorCommandLine ) ;

    CConsoleAccess::SystemLine     ( sReplyText ) ;

    return Reply_Ok ;
}
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ECoorMessageStatus  CMessageMgrPrinter::OffsetTick1OutputEtMonit  ( const unsigned long     _ulIndexNum,
                                                                    CMessageCommandLine*    _poCoorCommandLine )
{
    strcpy ( sReplyText, "<TrgMgr Set Output Et Monit Data Tick#1 Base Index=" ) ;
    strcat ( sReplyText, I_to_A(_ulIndexNum) ) ;
    strcat ( sReplyText, ">" ) ;

    AppendTextToReply ( _poCoorCommandLine ) ;

    CConsoleAccess::SystemLine     ( sReplyText ) ;

    return Reply_Ok ;
}
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ECoorMessageStatus  CMessageMgrPrinter::OffsetTick1OutputEtSimu   ( const unsigned long     _ulIndexNum,
                                                                    CMessageCommandLine*    _poCoorCommandLine )
{
    strcpy ( sReplyText, "<TrgMgr Set Output Et Simu Data Tick#1 Base Index=" ) ;
    strcat ( sReplyText, I_to_A(_ulIndexNum) ) ;
    strcat ( sReplyText, ">" ) ;

    AppendTextToReply ( _poCoorCommandLine ) ;

    CConsoleAccess::SystemLine     ( sReplyText ) ;

    return Reply_Ok ;
}
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ECoorMessageStatus  CMessageMgrPrinter::OffsetTriggeredToTickAtCapture
                                                                  ( const unsigned long     _ulIndexNum,
                                                                    CMessageCommandLine*    _poCoorCommandLine )
{
    strcpy ( sReplyText, "<TrgMgr Set Triggered To Tick at Capture Offset=" ) ;
    strcat ( sReplyText, I_to_A(_ulIndexNum) ) ;
    strcat ( sReplyText, ">" ) ;

    AppendTextToReply ( _poCoorCommandLine ) ;

    CConsoleAccess::SystemLine     ( sReplyText ) ;

    return Reply_Ok ;
}
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ECoorMessageStatus  CMessageMgrPrinter::L1Cal_Operational         ( CMessageCommandLine*    _poCoorCommandLine )
{
    strcpy ( sReplyText, "<TrgMgr Set Ignore L1Cal>" ) ;

    AppendTextToReply ( _poCoorCommandLine ) ;

    CConsoleAccess::SystemLine     ( sReplyText ) ;

    return Reply_Ok ;
}
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ECoorMessageStatus  CMessageMgrPrinter::L1Cal_Not_Operational     ( CMessageCommandLine*    _poCoorCommandLine )
{
    strcpy ( sReplyText, "<TrgMgr Set Include L1Cal>" ) ;

    AppendTextToReply ( _poCoorCommandLine ) ;

    CConsoleAccess::SystemLine     ( sReplyText ) ;

    return Reply_Ok ;
}
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ECoorMessageStatus  CMessageMgrPrinter::Monit_Control_Time        ( const unsigned long     _ulTime_ms,
                                                                    CMessageCommandLine*    _poCoorCommandLine )
{
    if ( _ulTime_ms == 0 )
    {
        strcpy ( sReplyText, "<TrgMgr Disabled Monitoring Data Collection>" ) ;
    }
    else
    {
        strcpy ( sReplyText, "<TrgMgr Set Monitoring Pool Manager Control Time to " ) ;
        strcat ( sReplyText, I_to_A(_ulTime_ms) ) ;
        strcat ( sReplyText, " ms>" ) ;
    }

    AppendTextToReply ( _poCoorCommandLine ) ;

    CConsoleAccess::SystemLine     ( sReplyText ) ;

    return Reply_Ok ;
}
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ECoorMessageStatus  CMessageMgrPrinter::L1CalCoverage             ( TAllTriggerTowers*      _pxTT,
                                                                    CMessageCommandLine*    _poCoorCommandLine )
{
    strcpy ( sReplyText, "<TrgMgr Set L1Cal Coverage" ) ;
    strcat ( sReplyText, " to TT Eta(" ) ;
    CMessageCoorPrinter::Add_Formatted_TT_Eta ( sReplyText, _pxTT ) ;
    strcat ( sReplyText, ") Phi(" ) ;
    CMessageCoorPrinter::Add_Formatted_TT_Phi ( sReplyText, _pxTT ) ;
    strcat ( sReplyText, ")>" ) ;

    AppendTextToReply ( _poCoorCommandLine ) ;

    CConsoleAccess::SystemLine     ( sReplyText ) ;

    return Reply_Ok ;
}
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ECoorMessageStatus  CMessageMgrPrinter::NewLogFile               ( CMessageCommandLine*    _poCoorCommandLine )
{
    strcpy ( sReplyText, "<TrgMgr New LogFile>" ) ;

    AppendTextToReply ( _poCoorCommandLine ) ;

    CConsoleAccess::WriteLine     ( sReplyText ) ;

    return Reply_Ok ;
}
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ECoorMessageStatus  CMessageMgrPrinter::Find_DAC          ( ETT_Chan_Type           _eTTType,
                                                            TAllTriggerTowers*      _pxTT,
                                                            CMessageCommandLine*    _poCoorCommandLine ) 

{ 
    strcpy ( sReplyText, "<Find_DAC for" ) ;
    if ( _eTTType == eTT_EM ) 
        strcat ( sReplyText, " EM " ) ;
    else if ( _eTTType == eTT_HD ) 
        strcat ( sReplyText, " HD " ) ;
    CMessageCoorPrinter::Add_Formatted_TT_Range ( sReplyText, _pxTT ) ;
    strcat ( sReplyText, ">" ) ;

    AppendTextToReply ( _poCoorCommandLine ) ;

    return Reply_Ok ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ECoorMessageStatus  CMessageMgrPrinter::Adjust_Ped        ( ETT_Chan_Type           _eTTType,
                                                            TAllTriggerTowers*      _pxTT,
                                                            CMessageCommandLine*    _poCoorCommandLine ) 

{ 
    strcpy ( sReplyText, "<Adjust_Ped for" ) ;
    if ( _eTTType == eTT_EM ) 
        strcat ( sReplyText, " EM " ) ;
    else if ( _eTTType == eTT_HD ) 
        strcat ( sReplyText, " HD " ) ;
    CMessageCoorPrinter::Add_Formatted_TT_Range ( sReplyText, _pxTT ) ;
    strcat ( sReplyText, ">" ) ;

    AppendTextToReply ( _poCoorCommandLine ) ;

    return Reply_Ok ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ECoorMessageStatus  CMessageMgrPrinter::Revert_Ped        ( ETT_Chan_Type           _eTTType,
                                                            TAllTriggerTowers*      _pxTT,
                                                            CMessageCommandLine*    _poCoorCommandLine ) 

{ 
    strcpy ( sReplyText, "<Revert_Ped for" ) ;
    if ( _eTTType == eTT_EM ) 
        strcat ( sReplyText, " EM " ) ;
    else if ( _eTTType == eTT_HD ) 
        strcat ( sReplyText, " HD " ) ;
    CMessageCoorPrinter::Add_Formatted_TT_Range ( sReplyText, _pxTT ) ;
    strcat ( sReplyText, ">" ) ;

    AppendTextToReply ( _poCoorCommandLine ) ;

    return Reply_Ok ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ECoorMessageStatus  CMessageMgrPrinter::Profile_BLS       ( ETT_Chan_Type           _eTTType,
                                                            TAllTriggerTowers*      _pxTT,
                                                            CMessageCommandLine*    _poCoorCommandLine ) 

{ 
    strcpy ( sReplyText, "<Profile_BLS for" ) ;
    if ( _eTTType == eTT_EM ) 
        strcat ( sReplyText, " EM " ) ;
    else if ( _eTTType == eTT_HD ) 
        strcat ( sReplyText, " HD " ) ;
    CMessageCoorPrinter::Add_Formatted_TT_Range ( sReplyText, _pxTT ) ;
    strcat ( sReplyText, ">" ) ;

    AppendTextToReply ( _poCoorCommandLine ) ;

    return Reply_Ok ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ECoorMessageStatus  CMessageMgrPrinter::Profile_AdfEt     ( ETT_Chan_Type           _eTTType,
                                                            TAllTriggerTowers*      _pxTT,
                                                            CMessageCommandLine*    _poCoorCommandLine ) 

{ 
    strcpy ( sReplyText, "<Profile_AdfEt for" ) ;
    if ( _eTTType == eTT_EM ) 
        strcat ( sReplyText, " EM " ) ;
    else if ( _eTTType == eTT_HD ) 
        strcat ( sReplyText, " HD " ) ;
    CMessageCoorPrinter::Add_Formatted_TT_Range ( sReplyText, _pxTT ) ;
    strcat ( sReplyText, ">" ) ;

    AppendTextToReply ( _poCoorCommandLine ) ;

    return Reply_Ok ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ECoorMessageStatus  CMessageMgrPrinter::AdfOut_Constant   ( ETT_Chan_Type           _eTTType,
                                                            TAllTriggerTowers*      _pxTT,
                                                            const unsigned long     _ulConstantData,
                                                            CMessageCommandLine*    _poCoorCommandLine ) 

{ 
    strcpy ( sReplyText, "<set ADF Output Constant for" ) ;
    if ( _eTTType == eTT_EM ) 
        strcat ( sReplyText, " EM " ) ;
    else if ( _eTTType == eTT_HD ) 
        strcat ( sReplyText, " HD " ) ;
    CMessageCoorPrinter::Add_Formatted_TT_Range ( sReplyText, _pxTT ) ;
    strcat ( sReplyText, " to " ) ;
    strcat ( sReplyText, I_to_A(_ulConstantData) ) ;
    strcat ( sReplyText, ">" ) ;

    AppendTextToReply ( _poCoorCommandLine ) ;

    CConsoleAccess::SystemLine     ( sReplyText ) ;

    return Reply_Ok ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ECoorMessageStatus  CMessageMgrPrinter::RawAdc_Thresh     ( ETT_Chan_Type           _eTTType,
                                                            TAllTriggerTowers*      _pxTT,
                                                            const unsigned long     _ulThreshold,
                                                            CMessageCommandLine*    _poCoorCommandLine ) 

{ 
    strcpy ( sReplyText, "<set Raw ADC Thresh for" ) ;
    if ( _eTTType == eTT_EM ) 
        strcat ( sReplyText, " EM " ) ;
    else if ( _eTTType == eTT_HD ) 
        strcat ( sReplyText, " HD " ) ;
    CMessageCoorPrinter::Add_Formatted_TT_Range ( sReplyText, _pxTT ) ;
    strcat ( sReplyText, " to " ) ;
    strcat ( sReplyText, I_to_A(_ulThreshold) ) ;
    strcat ( sReplyText, ">" ) ;

    AppendTextToReply ( _poCoorCommandLine ) ;

    return Reply_Ok ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ECoorMessageStatus  CMessageMgrPrinter::RawAdc_MuxMode    ( ETT_Chan_Type           _eTTType,
                                                            TAllTriggerTowers*      _pxTT,
                                                            ERawAdc_MuxMode_Type    _eMuxMode,
                                                            CMessageCommandLine*    _poCoorCommandLine )
{
    strcpy ( sReplyText, "<Set Raw ADC Mux for" ) ;
    if ( _eTTType == eTT_EM ) 
        strcat ( sReplyText, " EM " ) ;
    else if ( _eTTType == eTT_HD ) 
        strcat ( sReplyText, " HD " ) ;
    CMessageCoorPrinter::Add_Formatted_TT_Range ( sReplyText, _pxTT ) ;
    strcat ( sReplyText, " to Send " ) ;
    switch ( _eMuxMode ) 
    {
        case eRawAdc_MuxMode_BlsAligned :
            strcat ( sReplyText, "BLS Aligned" ) ;
        break ;

        case eRawAdc_MuxMode_SimuTurn  :
            strcat ( sReplyText, "Simulated Turn" ) ;
        break ;
    
        default :
        break ;
    }
    strcat ( sReplyText, " Data>" ) ;

    AppendTextToReply ( _poCoorCommandLine ) ;

    CConsoleAccess::SystemLine     ( sReplyText ) ;

    return Reply_Ok ;
}
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ECoorMessageStatus  CMessageMgrPrinter::RawAdc_MemMode    ( ETT_Chan_Type           _eTTType,
                                                            TAllTriggerTowers*      _pxTT,
                                                            ERawAdc_MemMode_Type    _eMemMode,
                                                            CMessageCommandLine*    _poCoorCommandLine )
{
    strcpy ( sReplyText, "<Set Raw ADC Memory for" ) ;
    if ( _eTTType == eTT_EM ) 
        strcat ( sReplyText, " EM " ) ;
    else if ( _eTTType == eTT_HD ) 
        strcat ( sReplyText, " HD " ) ;
    CMessageCoorPrinter::Add_Formatted_TT_Range ( sReplyText, _pxTT ) ;
    strcat ( sReplyText, " to " ) ;
    switch ( _eMemMode ) 
    {
        case eRawAdc_MemMode_CaptureMonitor :
            strcat ( sReplyText, "Capture Monitoring" ) ;
        break ;

        case eRawAdc_MemMode_CaptureAtThresh :
            strcat ( sReplyText, "Capture Above Threshold" ) ;
        break ;
        
        case eRawAdc_MemMode_CaptureImmediate :
            strcat ( sReplyText, "Capture Immediate" ) ;
        break ;

        case eRawAdc_MemMode_PlaySimuTurn :
            strcat ( sReplyText, "Play Simulated Turn" ) ;
        break ;

        default :
        break ;
    }
    strcat ( sReplyText, ">" ) ;

    AppendTextToReply ( _poCoorCommandLine ) ;

    CConsoleAccess::SystemLine     ( sReplyText ) ;

    return Reply_Ok ;
}
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ECoorMessageStatus  CMessageMgrPrinter::AdfOut_MuxMode    ( ETT_Chan_Type           _eTTType,
                                                            TAllTriggerTowers*      _pxTT,
                                                            EAdfOut_MuxMode_Type    _eMuxMode,
                                                            CMessageCommandLine*    _poCoorCommandLine )
{
    strcpy ( sReplyText, "<Set Output Et Mux for" ) ;
    if ( _eTTType == eTT_EM ) 
        strcat ( sReplyText, " EM " ) ;
    else if ( _eTTType == eTT_HD ) 
        strcat ( sReplyText, " HD " ) ;
    CMessageCoorPrinter::Add_Formatted_TT_Range ( sReplyText, _pxTT ) ;
    strcat ( sReplyText, " to Send " ) ;
    switch ( _eMuxMode ) 
    {
        case eAdfOut_MuxMode_Filtered_Always :
            strcat ( sReplyText, "Filtered Et Always" ) ;
        break ;

        case eAdfOut_MuxMode_Filtered_LiveX :
            strcat ( sReplyText, "Filtered Et on Live X" ) ;
        break ;
    
        case eAdfOut_MuxMode_Constant_Always :
            strcat ( sReplyText, "Contant Data Always" ) ;
        break ;
    
        case eAdfOut_MuxMode_Prn_Always  :
            strcat ( sReplyText, "Pseudo Random Always" ) ;
        break ;
    
        case eAdfOut_MuxMode_Prn_LiveX :
            strcat ( sReplyText, "Pseudo Random on Live X" ) ;
        break ;
    
        case eAdfOut_MuxMode_SimuTurn_Always :
            strcat ( sReplyText, "Simulated Turn Always" ) ;
        break ;
    
        case eAdfOut_MuxMode_SimuTurn_LiveX :
            strcat ( sReplyText, "Simulated Turn on Live X" ) ;
        break ;
    
        default :
        break ;
    }
    strcat ( sReplyText, ">" ) ;

    AppendTextToReply ( _poCoorCommandLine ) ;

    CConsoleAccess::SystemLine     ( sReplyText ) ;

    return Reply_Ok ;
}
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ECoorMessageStatus  CMessageMgrPrinter::AdfOut_MemMode    ( ETT_Chan_Type           _eTTType,
                                                            TAllTriggerTowers*      _pxTT,
                                                            EAdfOut_MemMode_Type    _eMemMode,
                                                            CMessageCommandLine*    _poCoorCommandLine )
{
    strcpy ( sReplyText, "<Set Output Et Memory for" ) ;
    if ( _eTTType == eTT_EM ) 
        strcat ( sReplyText, " EM " ) ;
    else if ( _eTTType == eTT_HD ) 
        strcat ( sReplyText, " HD " ) ;
    CMessageCoorPrinter::Add_Formatted_TT_Range ( sReplyText, _pxTT ) ;
    strcat ( sReplyText, " to " ) ;
    switch ( _eMemMode ) 
    {
        case eAdfOut_MemMode_CaptureMonitor :
            strcat ( sReplyText, "Capture Monitoring" ) ;
        break ;

        case eAdfOut_MemMode_CaptureAtThresh :
            strcat ( sReplyText, "Capture Above Threshold" ) ;
        break ;
        
        case eAdfOut_MemMode_CaptureImmediate :
            strcat ( sReplyText, "Capture Immediate" ) ;
        break ;

        case eAdfOut_MemMode_PlaySimuTurn :
            strcat ( sReplyText, "Play Simulated Turn" ) ;
        break ;

        default :
        break ;
    }
    strcat ( sReplyText, ">" ) ;

    AppendTextToReply ( _poCoorCommandLine ) ;

    CConsoleAccess::SystemLine     ( sReplyText ) ;

    return Reply_Ok ;
}
/*-------------------------------------------------------------------------------*/


