/*-------------------------------------------------------------------------------*/
  /*
     MessageTokenizer.cpp
     22-Sep-2005 Restructred for L1Cal_IIb_TCC
  */
/*-------------------------------------------------------------------------------*/
#include "L1Cal_IIb_TCC/StdAfx.h"

#include "L1Cal_IIb_TCC/MessageCommand.h"
#include                            "L1Cal_IIb_TCC/MessageTokenizer.h"

#include "L1Cal_IIb_TCC/MessageKeywordsCoorGeneric.h"  // just for comment flag

#include "L1Cal_IIb_TCC/HandleConsole.h"
#include "L1Cal_IIb_TCC/UtilsStrings.h"

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
#define     KiBase10    10
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
//Simple Macros for appending info to error messages 
/////////////////////////////////////////////////////////////////////////////////
#define ERROR_AT_CHAR()                                                         \
    strcat ( sReplyText, " at Char #" ) ;                                       \
    strcat ( sReplyText, I_to_A ( _poCoorCommandLine->CurTokenCharNum() ) ) ;   \
    strcat ( sReplyText, " in Command String>" ) ;                              \
    CConsoleAccess::ErrorLine     ( sReplyText ) ;                              \
    _poCoorCommandLine->CopyReplyText ( sReplyText ) ;                          \
/////////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////////
#define ERROR_AT_KEYWORD()                                                      \
    strcat ( sReplyText, " [" ) ;                                               \
    strcat ( sReplyText, sToken ) ;                                             \
    strcat ( sReplyText, "]" ) ;                                                \
/////////////////////////////////////////////////////////////////////////////////

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/


/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
CMessageTokenizer::CMessageTokenizer () 
{

}


/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
CMessageTokenizer::~CMessageTokenizer () 
{
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ECoorMessageStatus      CMessageTokenizer::ParseTokenize       ( CMessageCommandLine*  _poCoorCommandLine ) 
{ 
    ECoorMessageStatus  eReturnStatus = Reply_Ok ;
    char                sToken[KiMaxLengthKeywordInMessageCommand+1] ;
    TBinomialEntry< char, unsigned int > * poKeywordEntry ;
    ETokenType          eValueModifier = TokenIsNotDefined ;
    ETokenType          eLastValueModifier = TokenIsNotDefined ;
    char                sReplyText[KiMaxLengthReplyTextToCoor] ;
    TokenLocAndType     oTokenLocAndType ;

    //\\//

    //first check there is a command on the line
    _poCoorCommandLine->GetNextToken ( sToken ) ;

    if ( sToken[0] == 0 ) 
    {
        eReturnStatus = Reply_Bad ;
        strcpy ( sReplyText, "<No Command found after Command ID>" ) ;
        CConsoleAccess::ErrorLine     ( sReplyText ) ;
        _poCoorCommandLine->CopyReplyText ( sReplyText ) ;
        goto Done ;
    }
    else
    {
        if ( m_oKeywordDictionary.LocateEntry ( sToken ) == NULL )
        {
            eReturnStatus = Reply_Bad ;
            strcpy ( sReplyText, "<Command does not start with known Keyword>" ) ;
            CConsoleAccess::ErrorLine     ( sReplyText ) ;
            _poCoorCommandLine->CopyReplyText ( sReplyText ) ;
            goto Done ;
        }

    }

    //then take one token at a time and record it
    while ( sToken[0] != 0 )
    {
        // See if this is a number
        if ( ( sToken[0] >= '0') && ( sToken[0] <= '9' ) )
        {
            // The first character was a number.
            // Now ALL characters should be numbers
            if ( strspn ( sToken, "1234567890" ) != strlen ( sToken ) )
            {
                eReturnStatus = Reply_Bad ;
                strcpy ( sReplyText, "<Missing Space After Number" ) ;
                ERROR_AT_CHAR() ;
                goto Done ;
            }

            // If there was no hint of the token type for this number 
            // (i.e. negated, or upper bound,..), it is simply asserted
            if ( eValueModifier == TokenIsNotDefined ) eValueModifier = TokenIsValueAsserted ;

            // register this number, with modifier prepared earlier
            oTokenLocAndType.uiCharNum  = _poCoorCommandLine->CurTokenCharNum()  ;
            oTokenLocAndType.eTokenType = eValueModifier ;
            char* pcEndChar ;
            _poCoorCommandLine->m_lTokenList.AddEntry ( oTokenLocAndType, strtoul ( sToken, &pcEndChar, KiBase10 ) ) ;

            // reset Value modifier
            eValueModifier = TokenIsNotDefined ;
        }

        // See if this is the decimal point
        else if ( sToken[0] == '.' ) 
        {
            eLastValueModifier = _poCoorCommandLine->m_lTokenList.LastEntry()->GetKey().eTokenType ;
            if ( eLastValueModifier != TokenIsValueAsserted )
            {
                eReturnStatus = Reply_Bad ;
                strcpy ( sReplyText, "<Expected Explicit Positive Value Before Decimal Point" ) ;
                ERROR_AT_CHAR() ;
                goto Done ;
            }

            // prepare the token type for the next token to be the decimal field part of a number
            eValueModifier = TokenIsValueDecimal ;
        }

        // See if this is the special character of Negation/Veto
        else if ( sToken[0] == '-' ) 
        {
            // check that there is no conflict with a previous declared porarity
            if ( eValueModifier == TokenIsValueAsserted )
            {
                eReturnStatus = Reply_Bad ;
                strcpy ( sReplyText, "<Unexpected Syntax Before Minus Sign" ) ;
                ERROR_AT_CHAR() ;
                goto Done ;
            }

            // prepare the token type for the next token to be a negated number
            if ( eValueModifier == TokenIsValueUpper ) 
                eValueModifier = TokenIsValueUpperNeg ;
            else 
                eValueModifier = TokenIsValueNegated ;
        }

        // See if this is the special character of Assertion
        // it is not required, and mostly ignored, 
        // but recording its occurence will prevent (almost) all illegal usage
        else if ( sToken[0] == '+' ) 
        {
            // check that there is no conflict with a previous declared porarity
            if ( ( eValueModifier == TokenIsValueNegated )
              || ( eValueModifier == TokenIsValueUpperNeg ) )
            {
                eReturnStatus = Reply_Bad ;
                strcpy ( sReplyText, "<Unexpected Syntax Before Plus Sign" ) ;
                ERROR_AT_CHAR() ;
                goto Done ;
            }

            if ( eValueModifier != TokenIsValueUpper ) 
                eValueModifier = TokenIsValueAsserted ;
        }

        // See if this is an open parenthesis character
        else if ( sToken[0] == '(' ) 
        {
            if ( eValueModifier != TokenIsNotDefined )
            {
                eReturnStatus = Reply_Bad ;
                strcpy ( sReplyText, "<Unexpected Syntax Before Open Parenthesis" ) ;
                ERROR_AT_CHAR() ;
                goto Done ;
            }

            // register this syntax character
            oTokenLocAndType.uiCharNum  = _poCoorCommandLine->CurTokenCharNum()  ;
            oTokenLocAndType.eTokenType = TokenIsOpenParenthesis ;
            _poCoorCommandLine->m_lTokenList.AddEntry ( oTokenLocAndType, (unsigned int) 0 ) ;
        }

        // See if this is a close parenthesis character
        else if ( sToken[0] == ')' ) 
        {
            if ( eValueModifier != TokenIsNotDefined )
            {
                eReturnStatus = Reply_Bad ;
                strcpy ( sReplyText, "<Unexpected Syntax Before Close Parenthesis" ) ;
                ERROR_AT_CHAR() ;
                goto Done ;
            }

            // register this syntax character
            oTokenLocAndType.uiCharNum  = _poCoorCommandLine->CurTokenCharNum()  ;
            oTokenLocAndType.eTokenType = TokenIsCloseParenthesis ;
            _poCoorCommandLine->m_lTokenList.AddEntry ( oTokenLocAndType, (unsigned int) 0 ) ;
        }

        // See if this is the special character of Value Range
        else if ( sToken[0] == ':' ) 
        {
            eLastValueModifier = _poCoorCommandLine->m_lTokenList.LastEntry()->GetKey().eTokenType ;
            if ( ( eLastValueModifier != TokenIsValueAsserted )
              && ( eLastValueModifier != TokenIsValueNegated ) )
            {
                eReturnStatus = Reply_Bad ;
                strcpy ( sReplyText, "<Expected Integer Value for Lower Bound in Range" ) ;
                ERROR_AT_CHAR() ;
                goto Done ;
            }

            // Change the Token type for the previous number to be a Lower Bound
            oTokenLocAndType.uiCharNum  = _poCoorCommandLine->m_lTokenList.LastEntry()->GetKey().uiCharNum ;
            if ( eLastValueModifier == TokenIsValueAsserted )
                oTokenLocAndType.eTokenType = TokenIsValueLower ;
            else //( eLastValueModifier == TokenIsValueNegated ) 
                oTokenLocAndType.eTokenType = TokenIsValueLowerNeg ;

            _poCoorCommandLine->m_lTokenList.ModifyLastEntryKey ( oTokenLocAndType ) ;

            // Prepare the token type for the next token to be an upper bound number
            eValueModifier = TokenIsValueUpper ;
        }

        else 
        //otherwise it must be a keyword
        {
            if ( eValueModifier != TokenIsNotDefined )
            {
                eReturnStatus = Reply_Bad ;
                strcpy ( sReplyText, "<Unexpected Syntax before Keyword" ) ;
                ERROR_AT_KEYWORD() ;
                ERROR_AT_CHAR() ;
                goto Done ;
            }

            // If there is no space between the comment flag and the start of the comment text
            // the comment flag will not naturally be caught as a keyword 
            // and it must thus be explicitely handled at the lower parsing level.
            // To make sure, we simply truncate the Token.
            if ( sToken[0] == Kc_Comment_Flag ) 
                sToken[1] = 0 ;

            poKeywordEntry = m_oKeywordDictionary.LocateEntry ( sToken ) ;

            if ( poKeywordEntry != NULL )
            {
                // For comment flag after legal command ==> skip rest of line without even registering '#' as a keyword
                // For comment flag as first keyword, continue and register command line as comment
                if ( poKeywordEntry->GetValue() == Ki_CoorSubCmd_Comment ) 
                if ( _poCoorCommandLine->m_lTokenList.FirstEntry () != NULL ) 
                    goto Done ; 

                oTokenLocAndType.uiCharNum  = _poCoorCommandLine->CurTokenCharNum()  ;
                oTokenLocAndType.eTokenType = TokenIsKeyword ;
                _poCoorCommandLine->m_lTokenList.AddEntry ( oTokenLocAndType, poKeywordEntry->GetValue() ) ;

                // For comment flag as first keyword, now we can skip rest of line
                if ( poKeywordEntry->GetValue() == Ki_CoorSubCmd_Comment ) 
                    goto Done ; 
            }
            else 
            {
                eReturnStatus = Reply_Bad ;
                strcpy ( sReplyText, "<Unkonwn Keyword" ) ;
                ERROR_AT_KEYWORD() ;
                ERROR_AT_CHAR() ;
                goto Done ;
            }

        }
        
        // Go find the next token on the line, if any
        _poCoorCommandLine->GetNextToken ( sToken ) ;
    }

Done:

    return eReturnStatus ;
}


