/*-------------------------------------------------------------------------------*/
  /*
    MonitControl.cpp
    11-Oct-2005
  */
/*-------------------------------------------------------------------------------*/
#include "L1Cal_IIb_TCC/StdAfx.h"

#include                        "L1Cal_IIb_TCC/MonitControl.h"

#include "L1Cal_IIb_TCC/MonitPoolServer.h"
#include "L1Cal_IIb_TCC/MonitPoolManager.h"
#include "L1Cal_IIb_TCC/MonitPoolData.h"

#include "L1Cal_IIb_TCC/HandleResultFile.h"

#include "L1Cal_IIb_TCC/HandleCommandFiles.h"
#include "L1Cal_IIb_TCC/ListOfMasterCommandFiles.h"
#include "L1Cal_IIb_TCC/ListOfResultFiles.h"

#include "L1Cal_IIb_TCC/L1Cal.h" 
 
#include "L1Cal_IIb_TCC/ResourceLocks.h" 

#include "L1Cal_IIb_TCC/HandleConsole.h"
#include "L1Cal_IIb_TCC/UtilsStrings.h"

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
// Static data members must be initialized at file scope

    // Global Flag to Prevent the Monit Server from generating Trigger IO.
    volatile EYesNo  CMonitControl::mg_eynMonit_Server_Silenced = No ;

    // This Flag allows to skip collecting monitoring information from TABs and GAB
    volatile EYesNo  CMonitControl::mg_eynMonitIgnoreTabGab     = No ; 

    // Global Flag to Control Logging of Monitoring Server Transactions
#if ( defined ( VME_IO_NONE ) || defined ( VME_IO_FAKE ) ) 
    volatile EYesNo  CMonitControl::mg_eynMonit_Server_Tracing  = Yes ;
#else
    volatile EYesNo  CMonitControl::mg_eynMonit_Server_Tracing  = No ;
#endif

    // How long to wait for Collect_Status before Forcing Monit Data Capture 
    unsigned long    CMonitControl::mg_ulmsMonitWaitForCollectStatus = KiMonitControlTimeMsDefault ;

    //The number of consecutive IO failures trying to talk to the ADF maestro controlling monitoring
    unsigned int     CMonitControl::mg_uiConsecutiveFailures = 0 ;

/*-------------------------------------------------------------------------------*/

#if defined VME_IO_FAKE 
    // cf. usage in CheckMonitDataCaptured 
    int iFakeCaptureBahavior = 1 ;
#endif // VME_IO_FAKE

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
CMonitControl::CMonitControl ( )
{
}
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
CMonitControl::~CMonitControl ()
{
}
/*-------------------------------------------------------------------------------*/


/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess  InitMonitServices ( ) 
{
    ESuccess    eReturnStatus = Ok ;

    CString                 osLbn_Command_File ;

    //\\//

    //
    // Create a Monit Pool Manager Thread
    //
    CConsoleAccess::WriteLine ( (CString) " Initialize MonitPool Data" ) ;
    
    CMonitPoolData  oMonitPoolData ; // Scratch object, just to call constructor once
                                    // all data is static anyways.
    //
    // Create a Monit Pool Manager Thread
    //
    CConsoleAccess::WriteLine ( (CString) " Creating MonitPool Manager" ) ;
    
    CMonitPoolManagerThread::mg_pMonitPoolManager = new CMonitPoolManagerThread ( ) ;
    CMonitPoolManagerThread::mg_pMonitPoolManager->start ( );

    //
    // Create a Monit Server Thread, and accept connections
    //
    if ( InitMonitPoolServer ( ) != Ok )
        eReturnStatus = Error ;

    return eReturnStatus ;
} ;
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess  ShutdownMonitServices ( )
{
    ESuccess    eReturnStatus = Ok ;

    //\\//

    CConsoleAccess::WriteLine ( (CString) " Requesting MonitPool Manager Exit" ) ;    

    // Delete the Monit Server Thread, and accept connections
    ShutdownMonitPoolServer ( ) ;

    // Signal the Monitoring Pool Manager Thread
    CMonitPoolManagerThread::mg_pMonitPoolManager->cancel ( ) ;

    // Give the Manager Threads 5 seconds to exit
    float kfTenthSecond = (float) 0.100 ;
    for ( int i = 0 ; i < 50 ; i ++ )
    { 
        Thread_Util::delay ( kfTenthSecond ) ; 
        if ( CMonitPoolManagerThread::mg_pMonitPoolManager->is_running () == false ) 
            break ; 
    }

    return eReturnStatus ;
} ;
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess CMonitControl::RequestMonitData ( ) 
{
    ESuccess            eReturnStatus = Ok ;

    //\\//

    //Acquire the L1CalOperational Resource Lock to prevent other threads 
    //from changing this flag while we are in the middle of controlling the SCLD
    CResourceLocks::Acquire_L1CalOperational () ;

    if ( ( mg_uiConsecutiveFailures >= KiConsecutiveFailureThreshold )
      || ( CL1Cal::mg_eynL1CalOperational == No ) )
       eReturnStatus = Error ;

    else if ( CL1Cal::mg_poL1Cal->f_okRequestMonitData ( ) != Ok ) 
       eReturnStatus = Error ;    
       
    if ( eReturnStatus == Error ) 
        HandleAccessError () ;
    else // reset error counter
        mg_uiConsecutiveFailures = 0 ;

    //Release the L1CalOperational Resource Lock 
    CResourceLocks::Release_L1CalOperational () ;

    return eReturnStatus ;
}
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess CMonitControl::DontRequestMonitData ( ) 
{
    ESuccess            eReturnStatus = Ok ;

    //\\//

    //Acquire the L1CalOperational Resource Lock to prevent other threads 
    //from changing this flag while we are in the middle of controlling the SCLD
    CResourceLocks::Acquire_L1CalOperational () ;

    if ( ( mg_uiConsecutiveFailures >= KiConsecutiveFailureThreshold )
      || ( CL1Cal::mg_eynL1CalOperational == No ) )
       eReturnStatus = Error ;

    else if ( CL1Cal::mg_poL1Cal->f_okDontRequestMonitData ( ) != Ok ) 
       eReturnStatus = Error ;    
       
    if ( eReturnStatus == Error ) 
        HandleAccessError () ;
    else // reset error counter
        mg_uiConsecutiveFailures = 0 ;

    //Release the L1CalOperational Resource Lock 
    CResourceLocks::Release_L1CalOperational () ;

    return eReturnStatus ;
}
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess CMonitControl::CheckMonitDataCaptured ( EYesNo* _peCaptured ) 
{
    ESuccess            eReturnStatus = Ok ;

    //\\//

    //Acquire the L1CalOperational Resource Lock to prevent other threads 
    //from changing this flag while we are in the middle of controlling the SCLD
    CResourceLocks::Acquire_L1CalOperational () ;

    if ( ( mg_uiConsecutiveFailures >= KiConsecutiveFailureThreshold )
      || ( CL1Cal::mg_eynL1CalOperational == No ) )
    {
       *_peCaptured = No ;
       eReturnStatus = Error ;
    }

    else // now check that data was indeed captured
    {
        if ( CL1Cal::mg_poL1Cal->f_okCheckMonitDataCaptured ( _peCaptured ) != Ok ) 
           eReturnStatus = Error ;
    }

#if defined VME_IO_FAKE
    *_peCaptured = Yes ; // Override the detection, with mostly Yes
    if ( ( iFakeCaptureBahavior % 7 ) == 0 ) 
       *_peCaptured = No ;  
    else
        iFakeCaptureBahavior ++ ;
#endif // VME_IO_FAKE

    //Release the L1CalOperational Resource Lock 
    CResourceLocks::Release_L1CalOperational () ;

    return eReturnStatus ;
}
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess  CMonitControl::ForceCaptureMonitData ( EYesNo* _peCaptured ) 
{
    ESuccess    eReturnStatus = Ok ;

    //\\//

    //Acquire the L1CalOperational Resource Lock to prevent other threads 
    //from changing this flag while we are in the middle of controlling the SCLD
    CResourceLocks::Acquire_L1CalOperational () ;

    if ( ( mg_uiConsecutiveFailures >= KiConsecutiveFailureThreshold )
      || ( CL1Cal::mg_eynL1CalOperational == No ) )
    {
       *_peCaptured = No ;
       eReturnStatus = Error ;
    }

    else if ( CL1Cal::mg_poL1Cal->f_okCaptureMonitData () != Ok )
    {
       *_peCaptured = No ;
       eReturnStatus = Error ;
    }

    else  // now check that data was indeed captured
    {
        if ( CL1Cal::mg_poL1Cal->f_okCheckMonitDataCaptured ( _peCaptured ) != Ok ) 
        {
           *_peCaptured = No ;
           eReturnStatus = Error ;
        }

#if defined VME_IO_FAKE
        *_peCaptured = Yes ; // Override the detection, with mostly Yes
        if ( iFakeCaptureBahavior % 3 == 0 ) *_peCaptured = No ;  
        iFakeCaptureBahavior ++ ;
#endif // VME_IO_FAKE

        // Not having captured data would be a failure here
        if ( *_peCaptured == No ) 
            eReturnStatus = Error ;    
    }

    if ( eReturnStatus == Error ) 
        HandleAccessError () ;
    else // reset error counter
        mg_uiConsecutiveFailures = 0 ;

    //Release the L1CalOperational Resource Lock 
    CResourceLocks::Release_L1CalOperational () ;

    return eReturnStatus ;
} ;
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
void CMonitControl::HandleAccessError ( ) 
{
    mg_uiConsecutiveFailures++ ;

    if ( mg_uiConsecutiveFailures == KiConsecutiveFailureThreshold )
    {
        CConsoleAccess::InternalErrorLine ( (CString) " -------------------------------------------------------" ) ;
        CConsoleAccess::InternalErrorLine ( (CString) " Warning - Persistent IO Error during Capture Monit Data" ) ;
        CConsoleAccess::InternalErrorLine ( (CString) " --> Declare L1Cal non-operational until NEXT Initialize" ) ;
        CConsoleAccess::InternalErrorLine ( (CString) " -------------------------------------------------------" ) ;

        //Declare the L1Cal as not usable
        //so that 
        // - COOR will have to send an "initialize" (after a "configure") 
        //      before any other COOR message will gets serviced
        // - the Monit Data Server will be prevented from generating IOs until the next "initialize" 
//        CResourceLocks::Acquire_L1CalOperational () ; not needed as we are powered off anyway
        CL1Cal::mg_eynL1CalOperational = No ;
//        CResourceLocks::Release_L1CalOperational () ;

        CResultFile oStatusFile  ( osStatus_Non_Operational ) ; // Where we will write the command status

        // Open a file to write status
        if ( oStatusFile.OpenForWrite () == Ok )
        {
            oStatusFile.WriteLine ( (CString) "! This File was written by L1Cal_IIb_TCC_"
                                               + KsVersion
                                               + " Monitoring Data Manager" ) ;
            oStatusFile.WriteDateTimeLine () ;
            oStatusFile.WriteLine ( "! -------------------------------------------------------" ) ;
            oStatusFile.WriteLine ( "! Warning - Persistent IO Error during Capture Monit Data" ) ;
            oStatusFile.WriteLine ( "! --> Declare L1Cal non-operational until NEXT Initialize" ) ;
            oStatusFile.WriteLine ( "! -------------------------------------------------------" ) ;

            oStatusFile.Close () ;
        }
        else
            CConsoleAccess::ErrorLine ( "Failed Creating Status File for Mailing" ) ;

    }

    return ;
}
/*-------------------------------------------------------------------------------*/

