/*-------------------------------------------------------------------------------*/
  /*
    MonitPoolHelper.cpp
    11-Oct-2005
  */
/*-------------------------------------------------------------------------------*/
#include "L1Cal_IIb_TCC/StdAfx.h"

#include                        "L1Cal_IIb_TCC/MonitPoolHelper.h"

#include "L1Cal_IIb_TCC/MonitControl.h"
#include "L1Cal_IIb_TCC/MonitPoolData.h"

#include "L1Cal_IIb_TCC/L1Cal.h"

#include "tabgab/tcc_to_tabgab_monit.h"

#include "L1Cal_IIb_TCC/HandleConsole.h"
#include "L1Cal_IIb_TCC/UtilsStrings.h"
#include "L1Cal_IIb_TCC/UtilsBits.h"
#include "L1Cal_IIb_TCC/UtilsMisc.h"
#include "L1Cal_IIb_TCC/DataManip.h"

#if ( defined ( VME_IO_NONE ) || defined ( VME_IO_FAKE ) ) 
    #include <sys/timeb.h> // for struct _timeb in TimeMilliSeconds 
#endif

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess GetMonitHeader           ( L1CalTcc_Block_Header*      _pxHeader,
                                    EYesNo                      _eynTriggered,
                                    EYesNo                      _eynFillBlankData ) 
{
    ESuccess            eReturnStatus = Ok ;

    //\\//

    // Fill the data quality flags
    if ( CL1Cal::mg_eynL1CalOperational )
        _pxHeader->ubSystemNonOperational = kL1CalTcc_System_Operational ; 
    else
        _pxHeader->ubSystemNonOperational = kL1CalTcc_System_Non_Operational ; 

    if ( ( CL1Cal::mg_eynL1CalOperational )
      && ( CMonitControl::fg_eynMonit_Server_Silenced() == No )
      && ( _eynFillBlankData == No ) )
        _pxHeader->ubMonitDataNotCurrent  = kL1CalTcc_MonitData_Current ; 
    else 
        _pxHeader->ubMonitDataNotCurrent  = kL1CalTcc_MonitData_Not_Current ; 

    if ( _eynTriggered == Yes ) 
        _pxHeader->ubTriggeredBeamX       = kL1CalTcc_Triggered_Crossing ; 
    else 
        _pxHeader->ubTriggeredBeamX       = kL1CalTcc_Random_Crossing ; 

    // Fill in the Instrumented Trigger Tower Index Boundaries
    _pxHeader->ubTTEtaSignMin   = fubTruncToByte ( CL1Cal::mg_poL1Cal->f_ulTTEtaSignMin() ) ; 
    _pxHeader->ubTTEtaSignMax   = fubTruncToByte ( CL1Cal::mg_poL1Cal->f_ulTTEtaSignMax() ) ;
    _pxHeader->ubTTEtaMagnMin   = fubTruncToByte ( CL1Cal::mg_poL1Cal->f_ulTTEtaMagnMin() ) ;
    _pxHeader->ubTTEtaMagnMax   = fubTruncToByte ( CL1Cal::mg_poL1Cal->f_ulTTEtaMagnMax() ) ;
    _pxHeader->ubTTPhiMin       = fubTruncToByte ( CL1Cal::mg_poL1Cal->f_ulTTPhiMin()     ) ;
    _pxHeader->ubTTPhiMax       = fubTruncToByte ( CL1Cal::mg_poL1Cal->f_ulTTPhiMax()     ) ;

    return eReturnStatus ;
}
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess GetMonitInfoTickTurn     ( L1CalTcc_BeamX_Number*      _pxBeamX_Number,
                                    EYesNo                      _eynFillBlankData ) 
{
    ESuccess            eReturnStatus = Ok ;

    EYesNo              eCaptured ;

    //\\//

    /////////////////////////////////////////////////////
    // initialize the returned structure values to Zero
    /////////////////////////////////////////////////////

    _pxBeamX_Number->ulTickNum = 0 ;
    _pxBeamX_Number->uqTurnNum = 0 ;

    /////////////////////////////
    // Now go read the variables
    /////////////////////////////
    if ( ( CL1Cal::mg_eynL1CalOperational )
      && ( CMonitControl::fg_eynMonit_Server_Silenced() == No )
      && ( _eynFillBlankData == No ) )
    {
        if ( CL1Cal::mg_poL1Cal->f_okCheckMonitDataCaptured ( &eCaptured, 
                                            (unsigned short*) &_pxBeamX_Number->ulTickNum ) != Ok ) 
            eReturnStatus = Error ;
        
        // If this was not captured data, force an "illegal" Tick Number
        if ( eCaptured == No ) _pxBeamX_Number->ulTickNum = 0 ;
    }

    //There is no Turn Counter in hardware, 
    //fill the L1Cal Beam Crossing TTN count with a time based value
    uint64 uqmsTime ;
    uqmsTime = TimeMilliSeconds () ;
    _pxBeamX_Number->uqTurnNum = ( uqmsTime * ( kL1CalTcc_BeamX_Rate / 1000 ) ) / 159 ;

    //for development version, override even the Tick Number with a time based value
#if ( defined ( VME_IO_NONE ) || defined ( VME_IO_FAKE ) ) 
    _pxBeamX_Number->ulTickNum = fulTruncToLong ( ( uqmsTime * ( kL1CalTcc_BeamX_Rate / 1000 ) ) % 159 ) + 1;
#endif 

    return eReturnStatus ;
}


/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess GetMonitInfoAdfEt    ( L1CalTcc_Block_Adf_OutputEt*    _pxTtOutputEt,
                                EYesNo                          _eynFillBlankData ) 
{
    ESuccess            eReturnStatus = Ok ;

    unsigned long   ulTTEtaSign ;
    unsigned long   ulTTEtaMagn ;
    unsigned long   ulTTPhi ;
    unsigned long   ulSliceNum ;

//    unsigned long   ulTTEtaMagnBy4 ;
//    unsigned long   ulTTRelEtaMagn ;

    //\\//

    // Initialize Trigger Tower ADC Counts
    // We fill this array 4x null bytes at a time
    for ( ulTTEtaSign = eTT_Eta_Pos ; ulTTEtaSign <= eTT_Eta_Neg               ; ulTTEtaSign++ )
    for ( ulTTEtaMagn = eTT_Eta_1   ; ulTTEtaMagn <= kL1CalTcc_Tot_TT_Eta_Magn ; ulTTEtaMagn++ )
    for ( ulTTPhi     = eTT_Phi_1   ; ulTTPhi     <= kL1CalTcc_Tot_TT_Phi      ; ulTTPhi++ )
    for ( ulSliceNum  = 0           ; ulSliceNum  <  kL1CalTcc_Tot_LiveBX      ; ulSliceNum++ )
    {
        _pxTtOutputEt->a4ubEM_Et   [ulTTEtaSign]
                                   [ulTTEtaMagn-1]
                                   [ulTTPhi-1]
                                   [ulSliceNum] = 0x00 ;
        _pxTtOutputEt->a4ubHD_Et   [ulTTEtaSign]
                                   [ulTTEtaMagn-1]
                                   [ulTTPhi-1]
                                   [ulSliceNum] = 0x00 ;
    }

    // Get Trigger Tower ADC Counts
    if ( ( CMonitControl::fg_eynMonit_Server_Silenced() == No )
      && ( _eynFillBlankData == No ) )
    {
        for ( ulTTEtaSign  = CL1Cal::mg_poL1Cal->f_ulTTEtaSignMin() ; 
              ulTTEtaSign <= CL1Cal::mg_poL1Cal->f_ulTTEtaSignMax() ; 
              ulTTEtaSign++ )
        for ( ulTTEtaMagn  = CL1Cal::mg_poL1Cal->f_ulTTEtaMagnMin() ; 
              ulTTEtaMagn <= CL1Cal::mg_poL1Cal->f_ulTTEtaMagnMax() ; 
              ulTTEtaMagn++ )
        {
            for ( ulTTPhi  = CL1Cal::mg_poL1Cal->f_ulTTPhiMin() ; 
                  ulTTPhi <= CL1Cal::mg_poL1Cal->f_ulTTPhiMax() ;
                  ulTTPhi++ )

    #if ( defined ( VME_IO_NONE ) || defined ( VME_IO_FAKE ) ) 
            {
                MultiAccessShort    xRand ;
                int iNoise ;
                for ( ulSliceNum = 0 ; ulSliceNum < kL1CalTcc_Tot_LiveBX ; ulSliceNum  ++ ) 
                {
                    //fill data with crude +/- 1 bit noise around average.
                    xRand.uwWord = rand () ;
                    iNoise = 0 ;
                    if      ( xRand.ByByte.ubByte_0  %  8 == 0 ) iNoise = +1 ;
                    else if ( xRand.ByByte.ubByte_0  %  8 == 1 ) iNoise = -1 ;
                    _pxTtOutputEt->a4ubEM_Et       [ulTTEtaSign]
                                                   [ulTTEtaMagn-1]
                                                   [ulTTPhi-1] 
                                                   [ulSliceNum] = 8 + iNoise ;
                    iNoise = 0 ;
                    if      ( xRand.ByByte.ubByte_1  %  8 == 0 ) iNoise = +1 ;
                    else if ( xRand.ByByte.ubByte_1  %  8 == 1 ) iNoise = +1 ;
                    else if ( xRand.ByByte.ubByte_1  %  8 == 2 ) iNoise = -1 ;
                    else if ( xRand.ByByte.ubByte_1  %  8 == 3 ) iNoise = -1 ;
                    _pxTtOutputEt->a4ubHD_Et       [ulTTEtaSign]
                                                   [ulTTEtaMagn-1]
                                                   [ulTTPhi-1] 
                                                   [ulSliceNum] = 8 + iNoise ;
                }
            }
    #else
            if ( CL1Cal::mg_poL1Cal->m_a3poTrgTwr[ulTTEtaSign][ulTTEtaMagn][ulTTPhi] != NULL ) 
            {
                for ( ulSliceNum = 0 ; ulSliceNum < kL1CalTcc_Tot_LiveBX ; ulSliceNum ++ ) 
                if ( CL1Cal::mg_poL1Cal->m_a3poTrgTwr[ulTTEtaSign][ulTTEtaMagn][ulTTPhi]
                                       ->f_okReadMonitOutputEt ( &_pxTtOutputEt->a4ubEM_Et [ulTTEtaSign]
                                                                                          [ulTTEtaMagn-1]
                                                                                          [ulTTPhi-1]
																						  [ulSliceNum],
                                                                 &_pxTtOutputEt->a4ubHD_Et [ulTTEtaSign]
                                                                                          [ulTTEtaMagn-1]
                                                                                          [ulTTPhi-1]
																						  [ulSliceNum],
                                                                 &CMonitPoolData::mg_aIndexToTick[ulSliceNum] ) 
                     != Ok ) eReturnStatus = Error ;
            }
    #endif
        }

#if ( defined ( VME_IO_NONE ) || defined ( VME_IO_FAKE ) ) 

        _pxTtOutputEt->a4ubEM_Et       [eTT_Eta_Neg]
                                       [eTT_Eta_11-1]
                                       [eTT_Phi_17-1] 
                                       [2] = 45 ;

        _pxTtOutputEt->a4ubHD_Et       [eTT_Eta_Pos]
                                       [eTT_Eta_3-1]
                                       [eTT_Phi_30-1] 
                                       [15] = 222 ;

        for ( ulTTEtaSign  = CL1Cal::mg_poL1Cal->f_ulTTEtaSignMin() ; 
              ulTTEtaSign <= CL1Cal::mg_poL1Cal->f_ulTTEtaSignMax() ; 
              ulTTEtaSign++ )
        for ( ulTTEtaMagn  = CL1Cal::mg_poL1Cal->f_ulTTEtaMagnMin() ; 
              ulTTEtaMagn <= CL1Cal::mg_poL1Cal->f_ulTTEtaMagnMax() ; 
              ulTTEtaMagn++ )
        for ( ulTTPhi  = CL1Cal::mg_poL1Cal->f_ulTTPhiMin() ; 
              ulTTPhi <= CL1Cal::mg_poL1Cal->f_ulTTPhiMax() ;
              ulTTPhi++ )
          {
                _pxTtOutputEt->a4ubEM_Et       [ulTTEtaSign]
                                               [ulTTEtaMagn-1]
                                               [ulTTPhi-1] 
                                               [kL1CalTcc_Tot_LiveBX - 1] = 128 + ulTTEtaSign*64 + ulTTEtaMagn ;

                _pxTtOutputEt->a4ubHD_Et       [ulTTEtaSign]
                                               [ulTTEtaMagn-1]
                                               [ulTTPhi-1] 
                                               [kL1CalTcc_Tot_LiveBX - 1] = ulTTPhi ;
          }

#endif

    }

    return eReturnStatus ;
}
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess RestartAdfAddrGen    ( ) 
{
    ESuccess            eReturnStatus = Ok ;

    unsigned long   ulTTEtaSign ;
    unsigned long   ulTTEtaMagnBy4 ;
    unsigned long   ulTTPhiBy4 ;

    //\\//


    // Get Trigger Tower ADC Counts
    if ( CMonitControl::fg_eynMonit_Server_Silenced() == No )
    {
        for ( ulTTEtaSign  = CL1Cal::mg_poL1Cal->f_ulTTEtaSignMin() ; 
              ulTTEtaSign <= CL1Cal::mg_poL1Cal->f_ulTTEtaSignMax() ; 
              ulTTEtaSign++ )
        for ( ulTTEtaMagnBy4  = (CL1Cal::mg_poL1Cal->f_ulTTEtaMagnMin()-1) / 4 ; 
              ulTTEtaMagnBy4 <= (CL1Cal::mg_poL1Cal->f_ulTTEtaMagnMax()-1) / 4 ; 
              ulTTEtaMagnBy4 ++ )
        for ( ulTTPhiBy4  = (CL1Cal::mg_poL1Cal->f_ulTTPhiMin()-1) / 4 ; 
              ulTTPhiBy4 <= (CL1Cal::mg_poL1Cal->f_ulTTPhiMax()-1) / 4 ; 
              ulTTPhiBy4 ++ )

        if ( CL1Cal::mg_poL1Cal->m_apoAdf [ulTTEtaSign][ulTTEtaMagnBy4][ulTTPhiBy4] != NULL ) 
        {
            // Raw ADC Address Generators
//            if ( CL1Cal::mg_poL1Cal->m_apoAdf [ulTTEtaSign][ulTTEtaMagnBy4][ulTTPhiBy4]
//                                   ->f_okRestartRawAdcAddrGen   ( ) != Ok ) 
//                eReturnStatus = Error ;

            // OutputEt Address Generators
            if ( CL1Cal::mg_poL1Cal->m_apoAdf [ulTTEtaSign][ulTTEtaMagnBy4][ulTTPhiBy4]
                                   ->f_okRestartOutputEtAddrGen ( ) != Ok ) 
                eReturnStatus = Error ;
        }
    }

    return eReturnStatus ;
}
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess GetMonitInfoTabGabStatus ( L1CalTcc_Block_TabGab_Status*   _pxTabGabStatus,
                                    EYesNo                          _eynFillBlankData ) 
{
    ESuccess            eReturnStatus = Ok ;

    unsigned long ulTabNum ;
    unsigned long ulChipNum ;

    //\\//

    ////////////////////////////////////
    // Initialize TAB and GAB Status 
    ////////////////////////////////////

    for ( ulTabNum  = 0 ; ulTabNum  < kL1CalTcc_Tot_Tab_Card       ; ulTabNum  ++ )
    for ( ulChipNum = 0 ; ulChipNum < kL1CalTcc_Tot_Tab_Algo_Fpga  ; ulChipNum ++ )
        _pxTabGabStatus->a2uwTabAlgo_Status [ulTabNum]
                                            [ulChipNum] = 0x0000 ;

    for ( ulTabNum  = 0 ; ulTabNum  < kL1CalTcc_Tot_Tab_Card       ; ulTabNum  ++ )
        _pxTabGabStatus->auwTabGlob_Status [ulTabNum] = 0x0000 ;

    for ( ulChipNum = 0 ; ulChipNum < kL1CalTcc_Tot_Gab_Rec_Fpga   ; ulChipNum ++ )
        _pxTabGabStatus->auwGabRec_Status [ulChipNum] = 0x0000 ;

    _pxTabGabStatus->uwGabGlob_Status = 0x0000 ;


    if ( ( CMonitControl::fg_eynMonit_Server_Silenced() == No )
      && ( CMonitControl::fg_eynMonitIgnoreTabGab    () == No ) 
      && ( _eynFillBlankData == No ) )
    {
        ////////////////////////////////////
        // Get TAB and GAB Status 
        ////////////////////////////////////

        // TABs
        for ( ulTabNum = 0 ; ulTabNum  < kL1CalTcc_Tot_Tab_Card       ; ulTabNum  ++ )
        {

#if ( defined ( VME_IO_NONE ) || defined ( VME_IO_FAKE ) ) 

            for ( ulChipNum = 0 ; ulChipNum < kL1CalTcc_Tot_Tab_Algo_Fpga    ; ulChipNum ++ )
                _pxTabGabStatus->a2uwTabAlgo_Status [ulTabNum]
                                                    [ulChipNum] = 0x8000 ;

            _pxTabGabStatus->a2uwTabAlgo_Status [1][2] = 0x8800 ; // set PRN test data error on cables 1

            _pxTabGabStatus->auwTabGlob_Status [ulTabNum] = 0x0006 ;

#else
            // call Nevis' software to get TAB Algorithm Chip Status
            for ( ulChipNum = 0 ; ulChipNum < kL1CalTcc_Tot_Tab_Algo_Fpga       ; ulChipNum ++ )
                if ( ReportTabGabStatus(  Tab_Monit_AlgoStatus  ( ulTabNum,
                                                                  ulChipNum,
                                                                &_pxTabGabStatus->a2uwTabAlgo_Status [ulTabNum][ulChipNum] ),
                                         "Tab_Monit_AlgoStatus",
                                         "GetMonitInfoTabGabStatus" ) != Ok ) 
                    eReturnStatus = Error ;

            // call Nevis' software to get TAB Global Chip Status
            if ( ReportTabGabStatus(  Tab_Monit_GlobalStatus ( ulTabNum,
                                                             &_pxTabGabStatus->auwTabGlob_Status [ulTabNum] ),
                                     "Tab_Monit_GlobalStatus",
                                     "GetMonitInfoTabGabStatus" ) != Ok ) 
                eReturnStatus = Error ;
#endif
        }

        // GAB 

#if ( defined ( VME_IO_NONE ) || defined ( VME_IO_FAKE ) ) 

        for ( ulChipNum = 0 ; ulChipNum < kL1CalTcc_Tot_Gab_Rec_Fpga       ; ulChipNum ++ )
            _pxTabGabStatus->auwGabRec_Status   [ulChipNum] = 0x8000 ;

        _pxTabGabStatus->uwGabGlob_Status = 0x0006 ;

#else
        // call Nevis' software to get GAB Receiver Chip Status
        for ( ulChipNum = 0 ; ulChipNum < kL1CalTcc_Tot_Gab_Rec_Fpga       ; ulChipNum ++ )
            if ( ReportTabGabStatus(  Gab_Monit_ReceiverStatus ( ulChipNum,
                                                               &_pxTabGabStatus->auwGabRec_Status [ulChipNum] ),
                                     "Gab_Monit_ReceiverStatus",
                                     "GetMonitInfoTabGabStatus" ) != Ok ) 
            eReturnStatus = Error ;

        // call Nevis' software to get GAB Global Chip Status
        if ( ReportTabGabStatus(  Gab_Monit_GlobalStatus ( &_pxTabGabStatus->uwGabGlob_Status ),
                                 "Gab_Monit_GlobalStatus",
                                 "GetMonitInfoTabGabStatus" ) != Ok ) 
            eReturnStatus = Error ;
#endif

        ////////////////////////////////////
        // now update the error scalers
        ////////////////////////////////////

        // TAB Algorithm Chip Errors
        for ( ulTabNum  = 0 ; ulTabNum  < kL1CalTcc_Tot_Tab_Card       ; ulTabNum  ++ )
        for ( ulChipNum = 0 ; ulChipNum < kL1CalTcc_Tot_Tab_Algo_Fpga  ; ulChipNum ++ )
            IncrementErrorScaler ( _pxTabGabStatus->a2uwTabAlgo_Status [ulTabNum]
                                                                       [ulChipNum],
                                   _pxTabGabStatus->a3uwTabAlgo_ErrCnt [ulTabNum]
                                                                       [ulChipNum],
                                    kL1CalTcc_Tot_Tab_Algo_ErrCounter,
                                    kL1CalTcc_Tab_Algo_IncrNegBitMask ) ;

        // TAB Global Chip Errors
        for ( ulTabNum  = 0 ; ulTabNum  < kL1CalTcc_Tot_Tab_Card       ; ulTabNum  ++ )
            IncrementErrorScaler ( _pxTabGabStatus->auwTabGlob_Status  [ulTabNum],
                                   _pxTabGabStatus->a2uwTabGlob_ErrCnt [ulTabNum],
                                    kL1CalTcc_Tot_Tab_Glob_ErrCounter,
                                    kL1CalTcc_Tab_Glob_IncrNegBitMask ) ;

        // GAB Receiver Chip Errors
        for ( ulChipNum = 0 ; ulChipNum < kL1CalTcc_Tot_Gab_Rec_Fpga       ; ulChipNum ++ )
            IncrementErrorScaler ( _pxTabGabStatus->auwGabRec_Status   [ulChipNum],
                                   _pxTabGabStatus->a2uwGabRec_ErrCnt  [ulChipNum],
                                    kL1CalTcc_Tot_Gab_Rec_ErrCounter,
                                    kL1CalTcc_Gab_Rec_IncrNegBitMask ) ;

        // GAB Global Chip Errors
        IncrementErrorScaler ( _pxTabGabStatus->uwGabGlob_Status,
                               _pxTabGabStatus->auwGabGlob_ErrCnt   ,
                                kL1CalTcc_Tot_Gab_Glob_ErrCounter,
                                kL1CalTcc_Gab_Glob_IncrNegBitMask ) ;

        ////////////////////////////////////
        // now print/log errors
        ////////////////////////////////////

        // TAB Algorithm Chip Errors
        for ( ulTabNum  = 0 ; ulTabNum  < kL1CalTcc_Tot_Tab_Card       ; ulTabNum  ++ )
        for ( ulChipNum = 0 ; ulChipNum < kL1CalTcc_Tot_Tab_Algo_Fpga  ; ulChipNum ++ )
        if ( ( _pxTabGabStatus->a2uwTabAlgo_Status [ulTabNum][ulChipNum] & 0x01ff ) 
          != ( kL1CalTcc_Tab_Algo_IncrNegBitMask                         & 0x01ff ) ) 
            CConsoleAccess::ErrorLine ( (CString) " MonitWatch found TAB #"
                                       + I_to_A ( ulTabNum ) 
                                       + " Chip #"
                                       + I_to_A ( ulChipNum ) 
                                       + " Status = "
                                       + I_to_A ( _pxTabGabStatus->a2uwTabAlgo_Status [ulTabNum][ulChipNum], 4, 16, '0' ) ) ;

    }

    return eReturnStatus ;
}
/*-------------------------------------------------------------------------------*/


/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
void   IncrementErrorScaler     ( unsigned short    _uwStatus,
                                  unsigned short*   _auwErrCnt,
                                  unsigned short    _uwTotScaler,
                                  unsigned short    _uwNegBitMask ) 
{
    unsigned long ulScalerNum ;

    //\\//

    // Adjust the word being watched so that we then only have to look for bits being set.
    _uwStatus = _uwStatus ^ _uwNegBitMask ;

    for ( ulScalerNum = 0 ; ulScalerNum < _uwTotScaler ; ulScalerNum ++ )
    if ( ( _uwStatus & ( 0x0001 << ulScalerNum ) ) != 0x0000 ) 
        _auwErrCnt[ulScalerNum] ++ ;

}
/*-------------------------------------------------------------------------------*/

