/*-------------------------------------------------------------------------------*/
  /*
    MonitPoolManager.cpp
     9-Nov-2000
  */
/*-------------------------------------------------------------------------------*/
#include "L1Cal_IIb_TCC/StdAfx.h"

#include "L1Cal_IIb_TCC/HandleItcServer.h"

#include                        "L1Cal_IIb_TCC/MonitPoolManager.h"

#include "L1Cal_IIb_TCC/ResourceLocks.h"

#include "L1Cal_IIb_TCC/MonitControl.h"
#include "L1Cal_IIb_TCC/MonitPoolData.h"
#include "L1Cal_IIb_TCC/MonitPoolServer.h"
#include "L1Cal_IIb_TCC/MonitPoolHelper.h"

#include "L1Cal_IIb_TCC/HandleConsole.h"
#include "L1Cal_IIb_TCC/UtilsStrings.h"
#include "L1Cal_IIb_TCC/UtilsBits.h"
#include "L1Cal_IIb_TCC/UtilsMisc.h"

#include "L1Cal_IIb_TCC/L1Cal.h"

#include <time.h>

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
// Static data members must be initialized at file scope
// There is only one Monit Pool Manager running, this is a pointer to it
CMonitPoolManagerThread*    CMonitPoolManagerThread::mg_pMonitPoolManager ;

// Remember when the last time the Moni Data was Captured (in milliseconds)
// This is a volatile variable as it may be changed by another thread (maybe in the future)
volatile uint64 CMonitPoolManagerThread::mg_uqmsTimeLastCapture = TimeMilliSeconds () ;

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
CMonitPoolManagerThread::CMonitPoolManagerThread ( )
{
    m_eynFillBlankData = No ;
    m_eynBlankDataDone = No ;
}
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
CMonitPoolManagerThread::~CMonitPoolManagerThread ()
{

}
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
void* CMonitPoolManagerThread::run ()
{

    EYesNo  eynCaptured ;
    EYesNo  eynTriggered ;

    CString osTriggered ;

    uint64 uqmsTimePrevCapture = CMonitPoolManagerThread::mg_uqmsTimeLastCapture ;

    //\\//

    while ( true ) 
    {
        //wait a bit between tries
        Sleep ( KiSleepOneSecond ) ; 

        //First acquire the Capture Monit Data Lock 
        //so that nobody else updates the Monit Data Registers until we have them all read out
        CResourceLocks::Acquire_CaptureMonitData () ;

        //////////////////////////////////////////////////////////////////////////////////////
        // Capture Monit Data
        //////////////////////////////////////////////////////////////////////////////////////
        
        eynCaptured = No ;
        eynTriggered = No ;

        if ( ( CL1Cal::mg_eynL1CalOperational == Yes )
          && ( CMonitControl::fg_eynMonit_Server_Silenced() == No ) )
        {
            ///////////////////////////////////////
            // request a snapshot of the next event
            ///////////////////////////////////////

            //Wait until all address generators are started before requesting data
            CMonitControl::DontRequestMonitData () ;

            //First The ADF address generators need to be (re-)started
            RestartAdfAddrGen ( ) ;

            //Now we can ask for a new snapshot
            if ( CMonitControl::RequestMonitData () != Ok )

                CConsoleAccess::ErrorLine ( " Error Requesting Monit Data Capture" ) ;

            else //Then wait up to N sec for a L1Accept flagged with "COLLECT_STATUS" to cause the capture
            for ( int i = 0 ; i < CMonitControl::fg_ulmsMonitWaitForCollectStatus()/100 ; i ++ )
            {
                //see if we were explicitely asked to quit
                if ( testcancel () == true ) 
                {
                    CConsoleAccess::WriteLine ( (CString) " MonitPool Manager Exiting" ) ;    
                    return 0 ;
                }

                // Keep checking that we are still operational
                if ( CL1Cal::mg_eynL1CalOperational != Yes )
                {
                    // COOR is probably initializing the system
                    Sleep ( 5 * KiSleepOneSecond ) ; 

                    //ADF address generators need to be (re-)started
                    RestartAdfAddrGen ( ) ;

                    //jump out of this do loop, and we will try forcing one
                    break ; 
                }

                // See if we caught one
                CMonitControl::CheckMonitDataCaptured ( &eynCaptured ) ;

                // If we got one, it was thus a triggered event
                if ( eynCaptured == Yes ) 
                {
                    eynTriggered = Yes ;
                    mg_uqmsTimeLastCapture = TimeMilliSeconds () ; //Remember when we captured the last event
                    break ;
                }

                // wait a bit and check again
                Sleep ( KiSleepOneTenthSecond ) ; 
            }


            ///////////////////////////////////////
            // if there hasn't been a triggered crossing, try an immediate snapshot
            ///////////////////////////////////////
            if ( eynCaptured == No ) 
            if ( ( CL1Cal::mg_eynL1CalOperational == Yes )
              && ( CMonitControl::fg_eynMonit_Server_Silenced() == No ) )
            {
                // kick the SCLD to generate an immediate capture
                if ( CMonitControl::ForceCaptureMonitData ( &eynCaptured ) == Ok )

                if ( eynCaptured == Yes ) 
                    mg_uqmsTimeLastCapture = TimeMilliSeconds () ; //Remember when we captured the last event
                else
                    CConsoleAccess::ErrorLine ( " Error Forcing Monit Data Capture" ) ;

            }


            ///////////////////////////////////////
            // Show what type of data (triggered/random) was captured 
            ///////////////////////////////////////
            if ( eynCaptured == Yes) 
            {
                osTriggered = "MonitPool Data: " ;
                if ( eynTriggered == Yes ) osTriggered += "<Triggered>" ;
                else                       osTriggered += "<Random>   " ;

                osTriggered += " (" 
                            +  F_to_A ( (int) ( CMonitPoolManagerThread::mg_uqmsTimeLastCapture - uqmsTimePrevCapture ) / 1000, 1 ) 
                            + "s since last refresh)" ;

                CConsoleAccess::WriteLine ( osTriggered ) ;

                uqmsTimePrevCapture = CMonitPoolManagerThread::mg_uqmsTimeLastCapture ;
            }

            else if ( CL1Cal::mg_eynL1CalOperational == Yes ) // no point in complaining if system no longer operational
                CConsoleAccess::ErrorLine ( " Capture MonitPool Data Failed" ) ;
        
        }

        //////////////////////////////////////////////////////////////////////////////////////
        // Decide if we are going to update the data
        // Note that we don't look only at whether the attempt at Capture Monit Data was successful,
        // but also at whether the system is still operational and still allowed to do IO.
        // If Capture Monit Data fails consecutively a number of times, 
        // the system will automatically be declared non-operational.
        // In the mean time we fill the monit data with zeroes instead of stale data.
        //////////////////////////////////////////////////////////////////////////////////////
        if ( ( eynCaptured == Yes) 
          && ( CL1Cal::mg_eynL1CalOperational == Yes )
          && ( CMonitControl::fg_eynMonit_Server_Silenced() == No ) )
        {
            m_eynFillBlankData = No ;
            m_eynBlankDataDone = No ;
        }
        else
        {
            m_eynFillBlankData = Yes ;
        }

        //////////////////////////////////////////////////////////////////////////////////////
        // Update the Monit Data Pool
        //////////////////////////////////////////////////////////////////////////////////////
        if ( ( m_eynFillBlankData == No )         //We are updating with LIVE data
          || ( ( m_eynFillBlankData == Yes )      //Or we are updating with BLANK data... 
            && ( m_eynBlankDataDone == No ) ) )   //... but only once
        {
            //Update the Monit Data
            CMonitPoolData::UpdateMonitData ( eynTriggered, m_eynFillBlankData ) ;

            //Remember when/after we have filled the Pool with blank data
            if ( m_eynFillBlankData == Yes ) 
            {
                //Advertize the state change (i.e. only the first time we fill blank)
                if ( m_eynBlankDataDone == No )
                    CConsoleAccess::WarningLine ( " Monit Pool Data now Filled with Blank Data" ) ;

                mg_uqmsTimeLastCapture = TimeMilliSeconds () ; //Remember when we (did not) capture the last event

                m_eynBlankDataDone = Yes ;
            }
        }


        //Don't forget to release the Capture Monit Data Lock 
        CResourceLocks::Release_CaptureMonitData () ;
    }
    
    return 0 ;
} ;
/*-------------------------------------------------------------------------------*/

