/*-------------------------------------------------------------------------------*/
  /*
    MonitPoolServer.cpp
    11-Oct-2005
  */
/*-------------------------------------------------------------------------------*/
#include "L1Cal_IIb_TCC/StdAfx.h"

#include "L1Cal_IIb_TCC/HandleItcServer.h"

#include "itc/itc.hpp"
#include "itc/Opaque_Message.hpp"

#include                        "L1Cal_IIb_TCC/MonitPoolServer.h"

#include "L1Cal_IIb_TCC/MonitControl.h"
#include "L1Cal_IIb_TCC/MonitPoolData.h"

#include "L1Cal_IIb_TCC/HandleConsole.h"
#include "L1Cal_IIb_TCC/UtilsStrings.h"

#include "L1Cal_IIb_TCC/L1Cal.h"


// Monitoring Data Record definition
#define NT_MSVCPP // direct L1CalTcc_MonitData.hpp in the definition of 64 bit variables
#include "trigmon/L1CalTcc_MonitData.hpp"

//The monit data server here allocates a private copy of each block type 
//so as not to tie up the manager's copy of each block type during data send
//Declaring the following buffers static limits them to local scope

static L1CalTcc_Block_Adf_OutputEt        xBlock_Adf_Et ;

static L1CalTcc_Block_TabGab_Status       xBlock_TabGab_Status ;

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
// global variable to keep track of Monitoring  server thread
ItcServer*  gpMonit_ServerThread = NULL ;

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess  InitMonitPoolServer ( unsigned short _iIP_port_num ) 
{
    ESuccess    eReturnStatus = Ok ;

    //\\//

    CConsoleAccess::WriteLine ( (CString) " Creating MonitPool Server" ) ;
    
    //
    // Create a server Thread, and accept connections
    //
    Nested_Message_Consumer_Interface Monit_Request_Consumer ;
    gpMonit_ServerThread = new ItcServer ( _iIP_port_num, &Monit_Request_Consumer );
    gpMonit_ServerThread->start ( );

    return eReturnStatus ;
} ;

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess  ShutdownMonitPoolServer ( )
{
    ESuccess    eReturnStatus = Ok ;

    CConsoleAccess::WriteLine ( (CString) " Shutting Down MonitPool Server" ) ;    

    //
    // Delete the server Threads
    //

    delete gpMonit_ServerThread ;

    return eReturnStatus ;
} ;


/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess    Monit_Request_Consumer ( const itc::IPC_Message &_ItcMsgIn ) 
{
    ESuccess    eReturnStatus = Ok ;

    bool                    lMonit_Server_Tracing ;
    CString                 osClientProcessName ; 

    itc::Opaque_Message*      poItcMsgIn ;
    L1CalTcc_Monit_Request*   pxRequest ;
    itc::Opaque_Message*      pItcMsgOut ;

/////////////////////////////////////////////////////////////////////////////////
#define TRACE_REQUEST( REQUEST_TYPE )                                           \
    if ( lMonit_Server_Tracing )                                                \
    CConsoleAccess::WriteLine ( osClientProcessName                             \
                                + " requested "                                 \
                                + #REQUEST_TYPE ) ;                             \
/////////////////////////////////////////////////////////////////////////////////

    //\\//

    if ( _ItcMsgIn.ID() != itc::Opaque_Message::MSG_ID ) 
    {
        CConsoleAccess::ErrorLine ( " Received a Monitoring Request NOT of Type ITC Opaque" ) ;
        eReturnStatus = Error ;
        goto Done ;
    }

    if ( _ItcMsgIn.length() != sizeof( L1CalTcc_Monit_Request ) )
    {
        CConsoleAccess::ErrorLine ( " Received a Monitoring Request of Incorrect Length" ) ;
        eReturnStatus = Error ;
        goto Done ;
    }

    // decide in one place if we are going to log this call
    if ( CMonitControl::fg_eynMonit_Server_Tracing() == Yes )
    { 
        lMonit_Server_Tracing = true ;
        itc::Connection_Handle  This_Connection = _ItcMsgIn.get_connection() ;
        osClientProcessName   = (CString) " <"
                                        + This_Connection.info().program_name.c_str()
                                        + ">@<"
                                        + This_Connection.info().remote_host.c_str()
                                        + ":"
                                        + I_to_A ( This_Connection.info().pid )
                                        + ">" ; 
    }
    else
        lMonit_Server_Tracing = false ;

    poItcMsgIn = new itc::Opaque_Message( _ItcMsgIn ) ;
    pxRequest = (L1CalTcc_Monit_Request*) poItcMsgIn->data() ;

    switch ( pxRequest->ulBlockType ) 
    {
        //////////////////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////////////////
        case eL1CalTcc_BlockType_Adf_OutputEt :
            TRACE_REQUEST( BlockType_Adf_OutputEt ) ;

            CMonitPoolData::RetrieveAdfOutputEt ( &xBlock_Adf_Et ) ;

            pItcMsgOut = new itc::Opaque_Message ( &xBlock_Adf_Et, sizeof(xBlock_Adf_Et) ) ;
            _ItcMsgIn.reply ( *pItcMsgOut, 0.3 ) ;
            delete pItcMsgOut ;

            break ;

        //////////////////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////////////////
        case eL1CalTcc_BlockType_TabGab_Status :
            TRACE_REQUEST( BlockType_TabGab_Status ) ;

            CMonitPoolData::RetrieveTabGabStatus ( &xBlock_TabGab_Status ) ;

            pItcMsgOut = new itc::Opaque_Message ( &xBlock_TabGab_Status, sizeof(xBlock_TabGab_Status) ) ;
            _ItcMsgIn.reply ( *pItcMsgOut, 0.3 ) ;
            delete pItcMsgOut ;

            break ;

        //////////////////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////////////////
        default:
            CConsoleAccess::ErrorLine ( (CString) " Received an Unknown Monitoring Request = "
                                                  + I_to_A (pxRequest->ulBlockType) ) ;
            eReturnStatus = Error ;
            break ;
    }

Done:

    delete poItcMsgIn ;
        
    return eReturnStatus ;
} ;
/*-------------------------------------------------------------------------------*/
