/*-------------------------------------------------------------------------------*/
  /*
    NevisTabCard.cpp
    14-Sep-2005
  */
/*-------------------------------------------------------------------------------*/

#include "L1Cal_IIb_TCC/StdAfx.h"

#include "L1Cal_IIb_TCC/GlobalConstants.h"
#include "L1Cal_IIb_TCC/CustomDataTypes.h"

#include                            "L1Cal_IIb_TCC/NevisTabCard.h"

#include "L1Cal_IIb_TCC/NevisTabChip.h"
#include "tabgab/tcc_to_tabgab_coor.h"

#include "L1Cal_IIb_TCC/L1Cal.h"

#include "L1Cal_IIb_TCC/HandleConsole.h"
//#include "L1Cal_IIb_TCC/HandleResultFile.h"
#include "L1Cal_IIb_TCC/UtilsStrings.h"

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
CNevisTabCard::CNevisTabCard      ( unsigned char   _ubModuleNum,
                                    CString*        _posCardDescription,
                                    unsigned long   _ulTTPhiBy4 )

             : CNevisCard         (                 _ubModuleNum,
                                                    _posCardDescription )
{
    unsigned char   ubTTEtaSign ;
    unsigned char   ubTTEtaMagnBy4 ;
    unsigned char   ubChipNum ;

    //\\//

    m_ulTTPhiBy4   = _ulTTPhiBy4 ;

    for ( ubTTEtaSign    = eTT_Eta_Pos ; ubTTEtaSign    < eTot_TT_Eta_Sign ; ubTTEtaSign ++ ) 
    for ( ubTTEtaMagnBy4 = eTT_Eta_1_4 ; ubTTEtaMagnBy4 < eTot_TT_Eta_By4  ; ubTTEtaMagnBy4 ++ ) 
    {
        // chip number[0:9] = int (eta[0:39] / 4)
        // Where eta=0 counts starting from the eta=-20 corner of eta-phi space.
        switch ( ubTTEtaSign ) 
        {
            case eTT_Eta_Pos :
                ubChipNum = eTot_TT_Eta_By4 + ubTTEtaMagnBy4 ;  // 5...9
            break ;

            case eTT_Eta_Neg :
                ubChipNum = eTot_TT_Eta_By4 - 1 - ubTTEtaMagnBy4 ;  // 4..0
            break ;
        }
        
        m_apoMaFpga[ubTTEtaSign][ubTTEtaMagnBy4] 
            = new CNevisTabChip ( this,             // link back to mother card
                                  ubChipNum,        // Chip Address
                                  ubTTEtaSign,      // Eta Sign
                                  ubTTEtaMagnBy4 ) ;// Eta Magnitude
    }

}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
CNevisTabCard::~CNevisTabCard()
{
    unsigned char   ubTTEtaSign ;
    unsigned char   ubTTEtaMagnBy4 ;

    //\\//

    for ( ubTTEtaSign    = eTT_Eta_Pos ; ubTTEtaSign    < eTot_TT_Eta_Sign ; ubTTEtaSign ++ ) 
    for ( ubTTEtaMagnBy4 = eTT_Eta_1_4 ; ubTTEtaMagnBy4 < eTot_TT_Eta_By4  ; ubTTEtaMagnBy4 ++ ) 
    {
        delete m_apoMaFpga[ubTTEtaSign][ubTTEtaMagnBy4] ;
    }
}
/*-------------------------------------------------------------------------------*/

ESuccess CNevisTabCard::Initialize ( ) 
{
    ESuccess        eReturnStatus = Ok ;

    unsigned char   ubTTEtaSign ;
    unsigned char   ubTTEtaMagnBy4 ;

    //\\//
   
    // Advertize initialization
    CNevisCard::Initialize () ;


    // call the tabgab initialization
    if ( ReportTabGabStatus ( Tab_Initialize ( m_ubModuleNum ),
                             "Tab_Initialize",
                             "CNevisTabCard::Initialize" ) != Ok ) 
        eReturnStatus = Error ;

    // Initialize the FPGA objects (including programming variables)
    for ( ubTTEtaSign    = eTT_Eta_Pos ; ubTTEtaSign    < eTot_TT_Eta_Sign ; ubTTEtaSign ++ ) 
    for ( ubTTEtaMagnBy4 = eTT_Eta_1_4 ; ubTTEtaMagnBy4 < eTot_TT_Eta_By4  ; ubTTEtaMagnBy4 ++ ) 
    {
        if ( m_apoMaFpga[ubTTEtaSign][ubTTEtaMagnBy4]->Initialize ( ) != Ok ) 
        eReturnStatus = Error ;
    }

    return eReturnStatus ;
}


/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess CNevisTabCard::CaptureChLinkStatus ( )      
{
    ESuccess        eReturnStatus = Ok ;

    //\\//
    
    unsigned long   ulTTEtaSign ;
    unsigned long   ulTTEtaMagnBy4 ;

    //\\//

    // note: only over current L1Cal coverage
    for ( ulTTEtaSign     = CL1Cal::mg_poL1Cal->f_ulTTEtaSignMin() ; 
          ulTTEtaSign    <= CL1Cal::mg_poL1Cal->f_ulTTEtaSignMax() ; 
          ulTTEtaSign ++ ) 
    for ( ulTTEtaMagnBy4  = (CL1Cal::mg_poL1Cal->f_ulTTEtaMagnMin()-1)/4 ; 
          ulTTEtaMagnBy4 <= (CL1Cal::mg_poL1Cal->f_ulTTEtaMagnMax()-1)/4 ; 
          ulTTEtaMagnBy4 ++ ) 
    {
        // Find out the Status of all 3 links
        m_apoMaFpga[ulTTEtaSign][ulTTEtaMagnBy4]->Read_Monit_Status ( ) ;
    }

    return eReturnStatus ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess CNevisTabCard::VerifyChLinkStatus ( EYesNo          _eDeSkewAsNeeded,
                                             EYesNo          _eForConsistency )      
{
    ESuccess        eReturnStatus = Ok ;

    //\\//
    
    unsigned long   ulTTEtaSign ;
    unsigned long   ulTTEtaMagnBy4 ;
    unsigned long   ulTTPhiBy4 ;

    EYesNo          aeChLinkOk[KiChLinkPerTabFpga] ;
    unsigned long   ulChLinkNum ;

    //\\//

    // note: only over current L1Cal coverage
    for ( ulTTEtaSign     = CL1Cal::mg_poL1Cal->f_ulTTEtaSignMin() ; 
          ulTTEtaSign    <= CL1Cal::mg_poL1Cal->f_ulTTEtaSignMax() ; 
          ulTTEtaSign ++ ) 
    for ( ulTTEtaMagnBy4  = (CL1Cal::mg_poL1Cal->f_ulTTEtaMagnMin()-1)/4 ; 
          ulTTEtaMagnBy4 <= (CL1Cal::mg_poL1Cal->f_ulTTEtaMagnMax()-1)/4 ; 
          ulTTEtaMagnBy4 ++ ) 
    {
        // Find out the Status of all 3 links
        m_apoMaFpga[ulTTEtaSign][ulTTEtaMagnBy4]->Check_ChLink_Status ( aeChLinkOk ) ;

        for ( ulChLinkNum = 0 ; ulChLinkNum < KiChLinkPerTabFpga ; ulChLinkNum  ++ )
        {
            // First, find the phi range that this link comes from
            // ulChLinkNum = 0/1/2 = C/B/A = Phi range lower/center/upper
            ulTTPhiBy4 = ( m_ulTTPhiBy4 + eTot_TT_Phi_By4 + ulChLinkNum - 1 ) % eTot_TT_Phi_By4 ;

            if ( aeChLinkOk [ulChLinkNum] != Yes ) 
            {
                eReturnStatus = Error ;

                if ( _eForConsistency == No ) // For consistency case: skip the error message unless there is a discrepency
                CConsoleAccess::WarningLine ( (CString) " Ch Link problem " 
                                                        + m_apoMaFpga[ulTTEtaSign][ulTTEtaMagnBy4]->f_osBaseAddress ()
                                                        + " i.e. "
                                                        + m_apoMaFpga[ulTTEtaSign][ulTTEtaMagnBy4]->f_osTrgTwr_Range()
                                                        + " Status = "
                                                        + I_to_A ( m_apoMaFpga[ulTTEtaSign][ulTTEtaMagnBy4]->f_uwLastMonitStatus(), 4, 16, '0' ) ) ;

                // Ask the corresponding ADF-2 Card to send the DeSkew pattern
                if ( _eDeSkewAsNeeded == Yes ) 
                {
                    // send the DeSkew pattern 
                    // note: even if not in current L1Cal Phi coverage ?!?
                    if ( CL1Cal::mg_poL1Cal->m_apoAdf[ulTTEtaSign][ulTTEtaMagnBy4][ulTTPhiBy4] != NULL ) 
                         CL1Cal::mg_poL1Cal->m_apoAdf[ulTTEtaSign][ulTTEtaMagnBy4][ulTTPhiBy4] ->f_okInitiateChLinkDeSkew ( ) ;
                }

                // Check the status for consistency: the source ADF card should have just been deskewd
                if ( _eForConsistency == Yes ) 
                {
                    // send the DeSkew pattern 
                    // note: even if not in current L1Cal Phi coverage ?!?
                    if ( CL1Cal::mg_poL1Cal->m_apoAdf[ulTTEtaSign][ulTTEtaMagnBy4][ulTTPhiBy4] != NULL ) 
                    if ( CL1Cal::mg_poL1Cal->m_apoAdf[ulTTEtaSign][ulTTEtaMagnBy4][ulTTPhiBy4] ->f_lGet_DeSkewed ( ) == false  ) 
                    {
                        CConsoleAccess::WarningLine ( (CString) " Ch Link problem " 
                                                                + m_apoMaFpga[ulTTEtaSign][ulTTEtaMagnBy4]->f_osBaseAddress ()
                                                                + " i.e. "
                                                                + m_apoMaFpga[ulTTEtaSign][ulTTEtaMagnBy4]->f_osTrgTwr_Range()
                                                                + " Status = "
                                                                + I_to_A ( m_apoMaFpga[ulTTEtaSign][ulTTEtaMagnBy4]->f_uwLastMonitStatus(), 4, 16, '0' ) ) ;
                        CConsoleAccess::WarningLine ( (CString) "_But No DeSkew initiated from " 
                                                                + CL1Cal::mg_poL1Cal->m_apoAdf[ulTTEtaSign][ulTTEtaMagnBy4][ulTTPhiBy4]->m_osCardDescription) ;
                    }
                }
            }

            else // Check the good status for consistency: The source ADF Card should not have been deskewed
            if ( _eForConsistency == Yes ) 
            {
                // send the DeSkew pattern 
                // note: even if not in current L1Cal Phi coverage ?!?
                if ( CL1Cal::mg_poL1Cal->m_apoAdf[ulTTEtaSign][ulTTEtaMagnBy4][ulTTPhiBy4] != NULL ) 
                if ( CL1Cal::mg_poL1Cal->m_apoAdf[ulTTEtaSign][ulTTEtaMagnBy4][ulTTPhiBy4] ->f_lGet_DeSkewed ( ) == true ) 
                {
                    CConsoleAccess::WarningLine ( (CString) " Ch Link shows no problem " 
                                                            + m_apoMaFpga[ulTTEtaSign][ulTTEtaMagnBy4]->f_osBaseAddress ()
                                                            + " i.e. "
                                                            + m_apoMaFpga[ulTTEtaSign][ulTTEtaMagnBy4]->f_osTrgTwr_Range()
                                                            + " Status = "
                                                            + I_to_A ( m_apoMaFpga[ulTTEtaSign][ulTTEtaMagnBy4]->f_uwLastMonitStatus(), 4, 16, '0' ) ) ;
                    CConsoleAccess::WarningLine ( (CString) "_But a DeSkew was initiated from " 
                                                            + CL1Cal::mg_poL1Cal->m_apoAdf[ulTTEtaSign][ulTTEtaMagnBy4][ulTTPhiBy4]->m_osCardDescription ) ;
                }
            }

        }
    }

    return eReturnStatus ;
}

