/*-------------------------------------------------------------------------------*/
  /*
    NevisTabChip.cpp
    23-Sep-2005
  */
/*-------------------------------------------------------------------------------*/

#include "L1Cal_IIb_TCC/StdAfx.h"

#include "L1Cal_IIb_TCC/GlobalConstants.h"
#include "L1Cal_IIb_TCC/CustomDataTypes.h"

#include                            "L1Cal_IIb_TCC/NevisTabChip.h"

#include "L1Cal_IIb_TCC/NevisTabCard.h"

#include "tabgab/tcc_to_tabgab_coor.h"
#include "tabgab/tcc_to_tabgab_monit.h"

#include "L1Cal_IIb_TCC/L1Cal.h" // for CL1Cal::mg_auwRefSetInit

#include "L1Cal_IIb_TCC/HandleConsole.h"
#include "L1Cal_IIb_TCC/UtilsStrings.h"

#include "L1Cal_IIb_TCC/HandleResultFile.h"

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
CNevisTabChip::CNevisTabChip  ( CNevisTabCard*      _poMotherCard,
                                unsigned long       _ulChipAddress,
                                unsigned long       _ulTTEtaSign,
                                unsigned long       _ulTTEtaMagnBy4 )
{
    unsigned long       ulRefSetType ;
    unsigned long       ulRefSetNum ;
    unsigned long       ulMaskType ;

    //\\//

    m_poMotherCard      = _poMotherCard ;
    m_ulChipAddress     = _ulChipAddress ;

    m_ulTTEtaSign       = _ulTTEtaSign ;
    m_ulTTEtaMagnBy4    = _ulTTEtaMagnBy4 ;

    m_uwLastMonitStatus = 0 ;

    for ( ulRefSetType  = eTab_RefSet_EMEt ; ulRefSetType < eTot_Tab_RefSet_Type ; ulRefSetType ++ )
    for ( ulRefSetNum   = eTT_RefSet_0     ; ulRefSetNum  < eTot_TT_RefSet       ; ulRefSetNum ++ )
	    m_ulRefSetThresh [ulRefSetType] [ulRefSetNum] = CL1Cal::mg_auwRefSetInit [ulRefSetType] ;

    m_lJetUseIcr            = false ;
    m_lGlobalSumsUseIcr     = false ;

    m_lUseEmIsolation       = false ;
	m_lUseEmFraction        = false ;
	m_ulIsolRefSetNum       = 0 ;
    m_ulIsolCutValue        = 0 ; 

    for ( ulMaskType  = eTab_Mask_EM ; ulMaskType < eTot_Tab_Mask_Type ; ulMaskType ++ )
        m_ulL2MaskRefSet [ulMaskType] = 0 ;

}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
CNevisTabChip::~CNevisTabChip()
{
}
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess    CNevisTabChip::Initialize ( )
{
    ESuccess        eReturnStatus = Ok ;

    unsigned long       ulRefSetType ;
    unsigned long       ulRefSetNum ;
    unsigned long       ulMaskType ;

    //\\//

    m_uwLastMonitStatus = 0 ;

    // Initialize Reference Sets  
    // note that L1Cal::Initialize expects intialization fo ful scale

    for ( ulRefSetType  = eTab_RefSet_EMEt ; ulRefSetType < eTot_Tab_RefSet_Type ; ulRefSetType ++ )
    for ( ulRefSetNum   = eTT_RefSet_0     ; ulRefSetNum  < eTot_TT_RefSet       ; ulRefSetNum ++ )
    {
        if ( ( ulRefSetType == eTab_RefSet_TTEt ) 
          && ( ulRefSetNum  >= eTot_TT_RefSet_TT ) ) 
        {
            // There are only 3 single tower refsets.
        }
        else 
        {
	        m_ulRefSetThresh [ulRefSetType] [ulRefSetNum] = CL1Cal::mg_auwRefSetInit [ulRefSetType] ;

            switch ( ulRefSetType ) 
            { 
                case eTab_RefSet_EMEt :
                    if ( ReportTabGabStatus ( Tab_Set_EmEtThresh  ( m_poMotherCard->m_ubModuleNum,
                                                                    m_ulChipAddress,
                                                                    ulRefSetNum,
                                                                    m_ulRefSetThresh[eTab_RefSet_EMEt][ulRefSetNum] ),
                                             "Tab_Set_EmEtThresh",
                                             "CNevisTabChip::Initialize" ) != Ok )
                            eReturnStatus = Error ;
                break ;

                case eTab_RefSet_JetEt :
                    if ( ReportTabGabStatus ( Tab_Set_JetEtThresh ( m_poMotherCard->m_ubModuleNum,
                                                                    m_ulChipAddress,
                                                                    ulRefSetNum,
                                                                    m_ulRefSetThresh[eTab_RefSet_JetEt][ulRefSetNum] ),
                                             "Tab_Set_JetEtThresh",
                                             "CNevisTabChip::Initialize" ) != Ok )
                            eReturnStatus = Error ;
                break ;

                case eTab_RefSet_TTEt :
                    if ( ReportTabGabStatus ( Tab_Set_TT_Count_TT_Thresh  ( m_poMotherCard->m_ubModuleNum,
                                                                            m_ulChipAddress,
                                                                            ulRefSetNum,
                                                                            m_ulRefSetThresh[eTab_RefSet_TTEt][ulRefSetNum] ),
                                             "Tab_Set_TT_Count_TT_Thresh",
                                             "CNevisTabChip::Initialize" ) != Ok )
                            eReturnStatus = Error ;
                break ;

                default :
                    CConsoleAccess::InternalErrorLine ( (CString) " CNevisTabChip::Initialize: Illegal Algorithm Type" ) ;
            }
        }
    }

    // Initialize EM Isolation parameters

    m_lUseEmIsolation       = false ;
	m_lUseEmFraction        = false ;
	m_ulIsolRefSetNum       = 0 ;
    m_ulIsolCutValue        = CL1Cal::mg_auwRefSetInit [eTab_RefSet_EMEt] ; 

    if ( ReportTabGabStatus ( Tab_Set_Em_Isolation    ( m_poMotherCard->m_ubModuleNum,
                                                        m_ulChipAddress,
	                                    (unsigned long) m_lUseEmIsolation,
                                        (unsigned long) m_lUseEmFraction,
	                                                    m_ulIsolRefSetNum ),
                             "Tab_Set_Em_Isolation",
                             "CNevisTabChip::Initialize" ) != Ok )
        eReturnStatus = Error ;


    // Initialize ICR region usage

    m_lJetUseIcr            = false ;
    m_lGlobalSumsUseIcr     = false ;

    if ( ReportTabGabStatus ( Tab_Set_ICR_Enables     ( m_poMotherCard->m_ubModuleNum,
                                                        m_ulChipAddress,
                                                        m_lJetUseIcr,
                                                        m_lGlobalSumsUseIcr ),
                             "Tab_Set_ICR_Enables",
                             "CNevisTabChip::Initialize" ) != Ok )
        eReturnStatus = Error ;

    // Tau Et Low energy cut

    if ( ReportTabGabStatus ( Tab_Set_TauEtCoreCut    ( m_poMotherCard->m_ubModuleNum,
                                                        m_ulChipAddress,
                                                        kiTabTauEtCoreCutMax ),
                             "Tab_Set_TauEtCoreCut",
                             "CNevisTabChip::Initialize" ) != Ok )
        eReturnStatus = Error ;

    // Global Sum Et Low energy cut

    if ( ReportTabGabStatus ( Tab_Set_GlobalSumEtCut  ( m_poMotherCard->m_ubModuleNum,
                                                        m_ulChipAddress,
                                                        kiTabGlobSumEtCutMax ),
                             "Tab_Set_GlobalSumEtCut",
                             "CNevisTabChip::Initialize" ) != Ok )
        eReturnStatus = Error ;

    // L2 Masks
    // per Mike: we have to repeat the call to Tab_Set_Mask_Ref_Sets after Tab_Set_Em_Isolation

    for ( ulMaskType  = eTab_Mask_EM ; ulMaskType < eTot_Tab_Mask_Type ; ulMaskType ++ )
        m_ulL2MaskRefSet [ulMaskType] = eTT_RefSet_0 ;

    if ( ReportTabGabStatus     ( Tab_Set_Mask_Ref_Sets   ( m_poMotherCard->m_ubModuleNum,
                                                            m_ulChipAddress,
                                                            m_ulL2MaskRefSet[eTab_Mask_EM],
                                                            m_ulL2MaskRefSet[eTab_Mask_Jet],
                                                            m_ulL2MaskRefSet[eTab_Mask_Tau] ),
                                 "Tab_Set_Mask_Ref_Sets",
                                 "CNevisTabChip::Initialize" ) != Ok )
            eReturnStatus = Error ;


    // Eta 38

    if ( ReportTabGabStatus     ( Tab_Set_Eta38           ( m_poMotherCard->m_ubModuleNum,
                                                            m_ulChipAddress,
                                                            CL1Cal::mg_ulTabEta38Value ),
                                 "Tab_Set_Eta38",
                                 "CNevisTabChip::Initialize" ) != Ok )
            eReturnStatus = Error ;



    return eReturnStatus ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
TCString CNevisTabChip::f_osBaseAddress     ( ) // e.g. return "@Module#0/Chip#0"
{

    TCString  oMsg ;

    //\\//

    oMsg = "@Module#" + I_to_A( m_poMotherCard->m_ubModuleNum, 1 )
         + "/Chip#"   + I_to_A( m_ulChipAddress ) ;

    return oMsg ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
TCString CNevisTabChip::f_osTrgTwr_Range    ( ) // e.g. return "eta(-8:-5),phi(1:4)"
{

   return CL1Cal_TrgTwr::f_osTrgTwr_Range ( m_ulTTEtaSign, 
                                            m_ulTTEtaSign,
                                            4*m_ulTTEtaMagnBy4+1, 
                                            4*m_ulTTEtaMagnBy4+4,
                                            4*m_poMotherCard->m_ulTTPhiBy4+1,
                                            4*m_poMotherCard->m_ulTTPhiBy4+4 ) ;

}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess    CNevisTabChip::Read_Monit_Status      ( unsigned short* _puwChipStatus ) 
{
    ESuccess        eReturnStatus = Ok ;

    //\\//

    eReturnStatus = ReportTabGabStatus (  Tab_Monit_AlgoStatus    ( m_poMotherCard->m_ubModuleNum,
                                                                    m_ulChipAddress,
                                                                   &m_uwLastMonitStatus ),
                                         "Tab_Monit_AlgoStatus",
                                         "CNevisTabChip::Read_Monit_Status" ) ;

    // return the value if requested
    if ( _puwChipStatus != NULL ) *_puwChipStatus = m_uwLastMonitStatus ;

    return eReturnStatus ;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess    CNevisTabChip::Check_ChLink_Status    ( EYesNo*         _paeChLinkOk ) 
{
    ESuccess        eReturnStatus = Ok ;

    unsigned short  uwChLinkStatus ;
    unsigned short  uwChLinkNum ;

    //\\//

#if  ( defined ( VME_IO_FAKE )  || defined ( VME_IO_NONE ) )  
    if ( ( rand() % 200 ) == 0 ) m_uwLastMonitStatus = rand() ;
#endif

    // Initialize returned argument
    if ( _paeChLinkOk != NULL ) 
        for ( uwChLinkNum = 0 ; uwChLinkNum < KiChLinkPerTabFpga ; uwChLinkNum ++ ) 
            _paeChLinkOk[uwChLinkNum] = No ;

    // All three bit types must be found cleared for the link to be ok
    uwChLinkStatus = ( m_uwLastMonitStatus & KiTabStatusParityMask ) >> KiTabStatusParityShiftDown  
                   | ( m_uwLastMonitStatus & KiTabStatusSyncErMask ) >> KiTabStatusSyncErShiftDown  
                   | ( m_uwLastMonitStatus & KiTabStatusBcMisMMask ) >> KiTabStatusBcMisMShiftDown ;

    // display problems
    if ( uwChLinkStatus != 0x0000 ) 
    {
/* display error in calling routine
        CConsoleAccess::WriteLine ( (CString) " Ch Link problem " 
                                             + f_osBaseAddress ()
                                             + " i.e. "
                                             + f_osTrgTwr_Range()
                                             + " Status = "
                                             + I_to_A ( m_uwLastMonitStatus, 4, 16, '0' ) ) ;
*/
        eReturnStatus = Error ;
    }

    // Check and report the status of each link 
    if ( _paeChLinkOk != NULL ) 
        for ( uwChLinkNum = 0 ; uwChLinkNum < KiChLinkPerTabFpga ; uwChLinkNum ++ ) 
            if ( ( ( uwChLinkStatus >> uwChLinkNum ) & 0x0001 ) == 0x0000 ) 
                _paeChLinkOk[uwChLinkNum] = Yes ;

    return eReturnStatus ;
}


/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess    CNevisTabChip::RefSethreshProg        ( ETabRefSet_Type  _eRefSetType,
                                                    unsigned long    _ulTab_Thresh_Num,
                                                    unsigned long    _ulSet_AdcCount_Thresh,
                                                    bool             _lVerbose ) 
{
    ESuccess        eReturnStatus = Ok ;

    //\\//

    if ( _lVerbose )
    CConsoleAccess::WriteLine ( (CString) " Setting "
                                        + f_osTabChip_RefThresh  ( _eRefSetType, _ulTab_Thresh_Num ) 
                                        + "=" + I_to_A ( _ulSet_AdcCount_Thresh ) 
                                        + " for " 
                                        + f_osTabChip_Range  ( ) ) ;

    switch ( _eRefSetType ) 
    { 
        case eTab_RefSet_EMEt :
	        m_ulRefSetThresh [eTab_RefSet_EMEt] [_ulTab_Thresh_Num] = _ulSet_AdcCount_Thresh ;
            if ( ReportTabGabStatus (  Tab_Set_EmEtThresh ( m_poMotherCard->m_ubModuleNum,
                                                            m_ulChipAddress,
                                                           _ulTab_Thresh_Num,
                                                            m_ulRefSetThresh[eTab_RefSet_EMEt][_ulTab_Thresh_Num] ),
                                                 "Tab_Set_EmEtThresh",
                                                 "CNevisTabChip::RefSethreshProg" ) != Ok )
                    eReturnStatus = Error ;

            // We also have to watch and specify when this is the Isolation Cut
	        if ( m_ulRefSetThresh[eTab_RefSet_EMEt][_ulTab_Thresh_Num] == m_ulIsolCutValue ) 
            {
                m_ulIsolRefSetNum  = _ulTab_Thresh_Num ;
                if ( ReportTabGabStatus ( Tab_Set_Em_Isolation    ( m_poMotherCard->m_ubModuleNum,
                                                                    m_ulChipAddress,
	                                                (unsigned long) m_lUseEmIsolation,
                                                    (unsigned long) m_lUseEmFraction,
	                                                                m_ulIsolRefSetNum ),
                                         "Tab_Set_Em_Isolation",
                                         "CNevisTabChip::RefSethreshProg" ) != Ok )
                    eReturnStatus = Error ;

                // per Mike: we have to repeat the call to Tab_Set_Mask_Ref_Sets after Tab_Set_Em_Isolation
                if ( ReportTabGabStatus ( Tab_Set_Mask_Ref_Sets   ( m_poMotherCard->m_ubModuleNum,
                                                                    m_ulChipAddress,
                                                                    m_ulL2MaskRefSet[eTab_Mask_EM],
                                                                    m_ulL2MaskRefSet[eTab_Mask_Jet],
                                                                    m_ulL2MaskRefSet[eTab_Mask_Tau] ),
                                         "Tab_Set_Mask_Ref_Sets",
                                         "CNevisTabChip::RefSethreshProg" ) != Ok )
                    eReturnStatus = Error ;
            }
        break ;

        case eTab_RefSet_JetEt :
	        m_ulRefSetThresh[eTab_RefSet_JetEt][_ulTab_Thresh_Num] = _ulSet_AdcCount_Thresh ;
            if ( ReportTabGabStatus ( Tab_Set_JetEtThresh ( m_poMotherCard->m_ubModuleNum,
                                                            m_ulChipAddress,
                                                           _ulTab_Thresh_Num,
                                                            m_ulRefSetThresh[eTab_RefSet_JetEt][_ulTab_Thresh_Num] ),
                                     "Tab_Set_JetEtThresh",
                                     "CNevisTabChip::RefSethreshProg" ) != Ok )
                    eReturnStatus = Error ;
        break ;

        case eTab_RefSet_TTEt :
	        m_ulRefSetThresh[eTab_RefSet_TTEt][_ulTab_Thresh_Num] = _ulSet_AdcCount_Thresh ;
            if ( ReportTabGabStatus ( Tab_Set_TT_Count_TT_Thresh  ( m_poMotherCard->m_ubModuleNum,
                                                                    m_ulChipAddress,
                                                                   _ulTab_Thresh_Num,
                                                                    m_ulRefSetThresh[eTab_RefSet_TTEt][_ulTab_Thresh_Num] ),
                                     "Tab_Set_TT_Count_TT_Thresh",
                                     "CNevisTabChip::RefSethreshProg" ) != Ok )
                    eReturnStatus = Error ;
        break ;

        default :
            CConsoleAccess::InternalErrorLine ( (CString) " CNevisTabChip::RefSethreshProg: Illegal Algorithm Type" ) ;
    }

    return eReturnStatus ;
};

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess    CNevisTabChip::L2Mask     ( ETabMask_Type    _eMaskType,
                                        unsigned long    _ulRefSetNum,
                                        bool             _lVerbose ) 
{
    ESuccess        eReturnStatus = Ok ;

    //\\//

    if ( _lVerbose )
    CConsoleAccess::WriteLine ( (CString) " Setting L2 Mask to use "
                                        + f_osTabChip_Mask ( _eMaskType ) 
                                        + " RefSet #" + I_to_A ( _ulRefSetNum ) 
                                        + " for " 
                                        + f_osTabChip_Range  ( ) ) ;

	m_ulL2MaskRefSet[_eMaskType] = _ulRefSetNum ;
    if ( ReportTabGabStatus     ( Tab_Set_Mask_Ref_Sets   ( m_poMotherCard->m_ubModuleNum,
                                                            m_ulChipAddress,
                                                            m_ulL2MaskRefSet[eTab_Mask_EM],
                                                            m_ulL2MaskRefSet[eTab_Mask_Jet],
                                                            m_ulL2MaskRefSet[eTab_Mask_Tau] ),
                                 "Tab_Set_Mask_Ref_Sets",
                                 "CNevisTabChip::L2Mask" ) != Ok )
            eReturnStatus = Error ;

    return eReturnStatus ;
};

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess    CNevisTabChip::AlgoUseEmFraction    ( bool             _lUseEmFraction,
                                                  bool             _lVerbose ) 
{
    ESuccess        eReturnStatus = Ok ;

    CString         osEnabled ;

    //\\//

    // remember this new value
    m_lUseEmFraction = _lUseEmFraction ;

    if ( _lVerbose )
    {
        if ( m_lUseEmFraction == true  )
            osEnabled = "Use" ;
        else
            osEnabled = "Ignore" ;

        CConsoleAccess::WriteLine ( (CString) " TAB Algorithm will "
                                            + osEnabled 
                                            + " the EM Fraction for " 
                                            + f_osTabChip_Range  ( ) ) ;
    }

    if ( ReportTabGabStatus ( Tab_Set_Em_Isolation    ( m_poMotherCard->m_ubModuleNum,
                                                        m_ulChipAddress,
	                                    (unsigned long) m_lUseEmIsolation,
                                        (unsigned long) m_lUseEmFraction,
	                                                    m_ulIsolRefSetNum ),
                             "Tab_Set_Em_Isolation",
                             "CNevisTabChip::AlgoUseEmFraction" ) != Ok )
        eReturnStatus = Error ;

    // per Mike: we have to repeat the call to Tab_Set_Mask_Ref_Sets after Tab_Set_Em_Isolation
    if ( ReportTabGabStatus ( Tab_Set_Mask_Ref_Sets   ( m_poMotherCard->m_ubModuleNum,
                                                        m_ulChipAddress,
                                                        m_ulL2MaskRefSet[eTab_Mask_EM],
                                                        m_ulL2MaskRefSet[eTab_Mask_Jet],
                                                        m_ulL2MaskRefSet[eTab_Mask_Tau] ),
                             "Tab_Set_Mask_Ref_Sets",
                             "CNevisTabChip::AlgoUseEmFraction" ) != Ok )
        eReturnStatus = Error ;

    return eReturnStatus ;
};

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/

ESuccess    CNevisTabChip::AlgoUseEmIsolation   ( bool             _lUseEmIsolation ,
                                                  bool             _lVerbose ) 
{
    ESuccess        eReturnStatus = Ok ;

    CString         osEnabled ;

    //\\//

    // remember this new value
    m_lUseEmIsolation = _lUseEmIsolation ;

    if ( _lVerbose )
    {
        if ( m_lUseEmIsolation == true  )
            osEnabled = "Use" ;
        else
            osEnabled = "Ignore" ;

        CConsoleAccess::WriteLine ( (CString) " TAB Algorithm will "
                                            + osEnabled 
                                            + " the EM Isolation for " 
                                            + f_osTabChip_Range  ( ) ) ;
    }

    if ( ReportTabGabStatus (  Tab_Set_Em_Isolation    ( m_poMotherCard->m_ubModuleNum,
                                                        m_ulChipAddress,
	                                    (unsigned long) m_lUseEmIsolation,
                                        (unsigned long) m_lUseEmFraction,
	                                                    m_ulIsolRefSetNum ),
                             "Tab_Set_Em_Isolation",
                             "CNevisTabChip::AlgoUseEmIsolation" ) != Ok )
        eReturnStatus = Error ;

    // per Mike: we have to repeat the call to Tab_Set_Mask_Ref_Sets after Tab_Set_Em_Isolation
    if ( ReportTabGabStatus ( Tab_Set_Mask_Ref_Sets   ( m_poMotherCard->m_ubModuleNum,
                                                        m_ulChipAddress,
                                                        m_ulL2MaskRefSet[eTab_Mask_EM],
                                                        m_ulL2MaskRefSet[eTab_Mask_Jet],
                                                        m_ulL2MaskRefSet[eTab_Mask_Tau] ),
                             "Tab_Set_Mask_Ref_Sets",
                             "CNevisTabChip::AlgoUseEmIsolation" ) != Ok )
        eReturnStatus = Error ;

    return eReturnStatus ;
};

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/

ESuccess    CNevisTabChip::AlgoEmIsolationEtCut ( unsigned long    _ulSet_AdcCount_Thresh,
                                                  bool             _lVerbose ) 
{
    ESuccess        eReturnStatus = Ok ;

    CString         osEnabled ;

    //\\//

    // remember this new value
    m_ulIsolCutValue = _ulSet_AdcCount_Thresh ;

    if ( _lVerbose )
    CConsoleAccess::WriteLine ( (CString) " TAB will use a EM Isolation Et Cut"
                                        + "=" + I_to_A ( _ulSet_AdcCount_Thresh ) 
                                        + " for " 
                                        + f_osTabChip_Range  ( ) ) ;

    // look for a refenrece set defined at this value
    // if it is not defined, it will be set later by RefSethreshProg
    for ( int iRefSetNum = 0 ; iRefSetNum < eTot_TT_RefSet ; iRefSetNum ++ )
	    if ( m_ulRefSetThresh[eTab_RefSet_EMEt][iRefSetNum] == m_ulIsolCutValue ) 
            m_ulIsolRefSetNum = iRefSetNum ;
    
    if ( ReportTabGabStatus ( Tab_Set_Em_Isolation    ( m_poMotherCard->m_ubModuleNum,
                                                        m_ulChipAddress,
	                                    (unsigned long) m_lUseEmIsolation,
                                        (unsigned long) m_lUseEmFraction,
	                                                    m_ulIsolRefSetNum ),
                             "Tab_Set_Em_Isolation",
                             "CNevisTabChip::AlgoEmIsolationEtCut" ) != Ok )
        eReturnStatus = Error ;

    // per Mike: we have to repeat the call to Tab_Set_Mask_Ref_Sets after Tab_Set_Em_Isolation
    if ( ReportTabGabStatus ( Tab_Set_Mask_Ref_Sets   ( m_poMotherCard->m_ubModuleNum,
                                                        m_ulChipAddress,
                                                        m_ulL2MaskRefSet[eTab_Mask_EM],
                                                        m_ulL2MaskRefSet[eTab_Mask_Jet],
                                                        m_ulL2MaskRefSet[eTab_Mask_Tau] ),
                             "Tab_Set_Mask_Ref_Sets",
                             "CNevisTabChip::AlgoEmIsolationEtCut" ) != Ok )
        eReturnStatus = Error ;

    return eReturnStatus ;
};

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/

ESuccess    CNevisTabChip::AlgoTauEtCoreCut     ( unsigned long    _ulSet_AdcCount_Thresh,
                                                  bool             _lVerbose ) 
{
    ESuccess        eReturnStatus = Ok ;

    CString         osEnabled ;

    //\\//

    if ( _lVerbose )
    CConsoleAccess::WriteLine ( (CString) " TAB will use a Tau Core Et Cut"
                                        + "=" + I_to_A ( _ulSet_AdcCount_Thresh ) 
                                        + " for " 
                                        + f_osTabChip_Range  ( ) ) ;

    if ( ReportTabGabStatus ( Tab_Set_TauEtCoreCut    ( m_poMotherCard->m_ubModuleNum,
                                                        m_ulChipAddress,
                                                       _ulSet_AdcCount_Thresh ),
                             "Tab_Set_TauEtCoreCut",
                             "CNevisTabChip::AlgoTauEtCoreCut" ) != Ok )
        eReturnStatus = Error ;

    return eReturnStatus ;
};

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/

ESuccess    CNevisTabChip::AlgoGlobalSumEtCut   ( unsigned long    _ulSet_AdcCount_Thresh,
                                                  bool             _lVerbose ) 
{
    ESuccess        eReturnStatus = Ok ;

    //\\//

    if ( _lVerbose )
    CConsoleAccess::WriteLine ( (CString) " TAB will use a Global Sum Et Cut"
                                        + "=" + I_to_A ( _ulSet_AdcCount_Thresh ) 
                                        + " for " 
                                        + f_osTabChip_Range  ( ) ) ;

    if ( ReportTabGabStatus ( Tab_Set_GlobalSumEtCut  ( m_poMotherCard->m_ubModuleNum,
                                                        m_ulChipAddress,
                                                       _ulSet_AdcCount_Thresh ),
                             "Tab_Set_GlobalSumEtCut",
                             "CNevisTabChip::AlgoGlobalSumEtCut" ) != Ok )
        eReturnStatus = Error ;

    return eReturnStatus ;
};

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/

ESuccess    CNevisTabChip::AlgoUseIcrJetObjects ( bool             _lUseIcr ,
                                                  bool             _lVerbose ) 
{
    ESuccess        eReturnStatus = Ok ;

    CString         osEnabled ;

    //\\//

    // remember this new value
    m_lJetUseIcr = _lUseIcr ;

    if ( _lVerbose )
    {
        if ( m_lJetUseIcr == true  )
            osEnabled = "Use" ;
        else
            osEnabled = "Ignore" ;

        CConsoleAccess::WriteLine ( (CString) " TAB Algorithm will "
                                            + osEnabled 
                                            + " the ICR in Jet Objects for " 
                                            + f_osTabChip_Range  ( ) ) ;
    }

    if ( ReportTabGabStatus ( Tab_Set_ICR_Enables     ( m_poMotherCard->m_ubModuleNum,
                                                        m_ulChipAddress,
                                                        m_lJetUseIcr,
                                                        m_lGlobalSumsUseIcr ),
                             "Tab_Set_ICR_Enables",
                             "CNevisTabChip::AlgoUseIcrJetObjects" ) != Ok )
        eReturnStatus = Error ;

    return eReturnStatus ;
};

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/

ESuccess    CNevisTabChip::AlgoUseIcrGlobalSums ( bool             _lUseIcr ,
                                                  bool             _lVerbose ) 
{
    ESuccess        eReturnStatus = Ok ;

    CString         osEnabled ;

    //\\//

    // remember this new value
    m_lGlobalSumsUseIcr = _lUseIcr ;

    if ( m_lGlobalSumsUseIcr )
    {
        if ( _lUseIcr == true  )
            osEnabled = "Use" ;
        else
            osEnabled = "Ignore" ;

        CConsoleAccess::WriteLine ( (CString) " TAB Algorithm will "
                                            + osEnabled 
                                            + " the ICR in Global Sums for " 
                                            + f_osTabChip_Range  ( ) ) ;
    }

    if ( ReportTabGabStatus ( Tab_Set_ICR_Enables     ( m_poMotherCard->m_ubModuleNum,
                                                        m_ulChipAddress,
                                                        m_lJetUseIcr,
                                                        m_lGlobalSumsUseIcr ),
                             "Tab_Set_ICR_Enables",
                             "CNevisTabChip::AlgoUseIcrGlobalSums" ) != Ok )
        eReturnStatus = Error ;

    return eReturnStatus ;
};

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
CString     CNevisTabChip::f_osTabChip_Range  ( ) 
{ // Show Range e.g. "eta(-20:-17),phi(1:4)"

    TCString  osMsg ;

    //\\//

    if ( m_ulTTEtaSign == eTT_Eta_Pos )
        osMsg = "eta(+" + I_to_A(4*m_ulTTEtaMagnBy4+4) 
              +    ":+" + I_to_A(4*m_ulTTEtaMagnBy4+1) ;
    else 
        osMsg = "eta(-" + I_to_A(4*m_ulTTEtaMagnBy4+1) 
              +    ":-" + I_to_A(4*m_ulTTEtaMagnBy4+4) ;

    osMsg += "),phi(" + I_to_A(4*m_poMotherCard->m_ulTTPhiBy4+1) 
           +      ":" + I_to_A(4*m_poMotherCard->m_ulTTPhiBy4+4)     
           + ")" ;

    return osMsg ;
}
/*-------------------------------------------------------------------------------*/


/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
CString     CNevisTabChip::f_osTabChip_Range  ( unsigned long   _ulTTEtaSign,
                                                unsigned long   _ulTTEtaMagnBy4,
                                                unsigned long   _ulTTPhiBy4 ) 
{ // Show Tab Chip Coverage Range e.g. "eta(-20:-17),phi(1:4)"

    TCString  osMsg ;

    //\\//

    if ( _ulTTEtaSign == eTT_Eta_Pos )
        osMsg = "eta(-" + I_to_A(4*_ulTTEtaMagnBy4+4) 
              +    ":-" + I_to_A(4*_ulTTEtaMagnBy4+1) ;
    else 
        osMsg = "eta(+" + I_to_A(4*_ulTTEtaMagnBy4+1) 
              +    ":+" + I_to_A(4*_ulTTEtaMagnBy4+4) ;

    osMsg += "),phi(" + I_to_A(4*_ulTTPhiBy4+1) 
           +      ":" + I_to_A(4*_ulTTPhiBy4+4)     
           + ")" ;

    return osMsg ;
}
/*-------------------------------------------------------------------------------*/


/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
CString     CNevisTabChip::f_osTabChip_RefSet     ( ETabRefSet_Type _eRefSetType ) 
{ // String representation of Comparator Refence e.g. "EM Et"

    TCString  osMsg ;
    //\\//

    switch ( _eRefSetType)
    {
      case eTab_RefSet_EMEt :
        osMsg = "EM Et" ;
        break ;

      case eTab_RefSet_JetEt :
        osMsg = "Tot Et" ;
        break ;

      case eTab_RefSet_TTEt :
        osMsg = "TT Et" ;
        break ;

      default :
        osMsg = "??_" ;
        break ;
    }

    return osMsg ;
}
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
CString     CNevisTabChip::f_osTabChip_Mask         ( ETabMask_Type   _eMaskType ) 
{ // String representation of L2 Mask Type e.g. "Tau"

    TCString  osMsg ;
    //\\//

    switch ( _eMaskType )
    {
      case eTab_Mask_EM :
        osMsg = "EM" ;
        break ;

      case eTab_Mask_Jet :
        osMsg = "Jet" ;
        break ;

      case eTab_Mask_Tau :
        osMsg = "Tau" ;
        break ;

      default :
        osMsg = "??_" ;
        break ;
    }

    return osMsg ;
}
/*-------------------------------------------------------------------------------*/



/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
CString     CNevisTabChip::f_osTabChip_RefThresh  ( ETabRefSet_Type _eRefSetType,
                                                    unsigned long   _ulRefSetNum ) 
{ // String of Comparator Refence e.g. "EM Et Thresh #0"

    return f_osTabChip_RefSet ( _eRefSetType ) + " Thresh #" + I_to_A ( _ulRefSetNum ) ;
}
/*-------------------------------------------------------------------------------*/


