/*-------------------------------------------------------------------------------*/
  /*
    ResourceLocks.cpp
    Class CResourceLocks
    29-Nov-2000
  */
/*-------------------------------------------------------------------------------*/
#include "L1Cal_IIb_TCC/StdAfx.h"

#include "L1Cal_IIb_TCC/GlobalConstants.h"
#include "L1Cal_IIb_TCC/CustomDataTypes.h"

#include                            "L1Cal_IIb_TCC/ResourceLocks.h"

// Ace Synchronization tools: Mutex, etc.
#include "ace/Synch.h"

//#include "L1Cal_IIb_TCC/HandleConsole.h"
//#include "L1Cal_IIb_TCC/UtilsStrings.h"

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
// Static data members must be initialized at file scope
// Monitoring Data

// A Mutex protection used to synchronize writing messages to the screen
ACE_Thread_Mutex                CResourceLocks::mg_oMutexScreenIO ;

// A Mutex protection used to synchronize the Nevis Screen IO interface
ACE_Thread_Mutex                CResourceLocks::mg_oMutexNevisScreenIO ;

// A Mutex protection used to synchronize writing messages to the LogFile
ACE_Thread_Mutex                CResourceLocks::mg_oMutexLogFile ;

// A Mutex protection used to prevent a thread from changing
// the status of the L1FW to NON-Operational at critical times
ACE_Thread_Mutex                CResourceLocks::mg_oMutexL1CalOperational ;

// A Mutex protection used to prevent interference during Trigger IO.
ACE_Thread_Mutex                CResourceLocks::mg_oMutexTriggerIO ;

// A Mutex protection used to prevent the Monit Data Pool
// to be updated while it is being served or vice versa
ACE_Thread_Mutex                CResourceLocks::mg_oMutexMonitDataPool ;

// A Mutex protection used to prevent a new Monit Data Capture
// until the Monit data has all been readout by the requester
ACE_Thread_Mutex                CResourceLocks::mg_oMutexCaptureMonitData ;

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess  CResourceLocks::Acquire_LogFile ( )
{
    ESuccess    eReturnStatus = Ok ;

    //\\//
    
    mg_oMutexLogFile.acquire () ;

    return eReturnStatus ;
}
/*-------------------------------------------------------------------------------*/
ESuccess  CResourceLocks::TryAcquire_LogFile ( )
{
    ESuccess    eReturnStatus = Ok ;

    //\\//
    
    if ( mg_oMutexLogFile.tryacquire () == -1 ) 
        eReturnStatus = Error ;

    return eReturnStatus ;
}
/*-------------------------------------------------------------------------------*/
ESuccess  CResourceLocks::Release_LogFile ( )
{
    ESuccess    eReturnStatus = Ok ;

    //\\//

    mg_oMutexLogFile.release () ;

    return eReturnStatus ;
}
/*-------------------------------------------------------------------------------*/
 
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess  CResourceLocks::Acquire_ScreenIO ( )
{
    ESuccess    eReturnStatus = Ok ;

    //\\//
    
    mg_oMutexScreenIO.acquire () ;

    return eReturnStatus ;
}
/*-------------------------------------------------------------------------------*/
ESuccess  CResourceLocks::TryAcquire_ScreenIO ( )
{
    ESuccess    eReturnStatus = Ok ;

    //\\//
    
    if ( mg_oMutexScreenIO.tryacquire () == -1 ) 
        eReturnStatus = Error ;

    return eReturnStatus ;
}
/*-------------------------------------------------------------------------------*/
ESuccess  CResourceLocks::Release_ScreenIO ( )
{
    ESuccess    eReturnStatus = Ok ;

    //\\//

    mg_oMutexScreenIO.release () ;

    return eReturnStatus ;
}
/*-------------------------------------------------------------------------------*/
 
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess  CResourceLocks::Acquire_NevisScreenIO ( )
{
    ESuccess    eReturnStatus = Ok ;

    //\\//
    
    mg_oMutexNevisScreenIO.acquire () ;

    return eReturnStatus ;
}
/*-------------------------------------------------------------------------------*/
ESuccess  CResourceLocks::TryAcquire_NevisScreenIO ( )
{
    ESuccess    eReturnStatus = Ok ;

    //\\//
    
    if ( mg_oMutexNevisScreenIO.tryacquire () == -1 ) 
        eReturnStatus = Error ;

    return eReturnStatus ;
}
/*-------------------------------------------------------------------------------*/
ESuccess  CResourceLocks::Release_NevisScreenIO ( )
{
    ESuccess    eReturnStatus = Ok ;

    //\\//

    mg_oMutexNevisScreenIO.release () ;

    return eReturnStatus ;
}
/*-------------------------------------------------------------------------------*/
 
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess  CResourceLocks::Acquire_L1CalOperational ( )
{
    ESuccess    eReturnStatus = Ok ;

    //\\//
    
    mg_oMutexL1CalOperational.acquire () ;

    return eReturnStatus ;
}
 /*-------------------------------------------------------------------------------*/
ESuccess  CResourceLocks::TryAcquire_L1CalOperational ( )
{
    ESuccess    eReturnStatus = Ok ;

    //\\//
    
    if ( mg_oMutexL1CalOperational.tryacquire () == -1 ) 
        eReturnStatus = Error ;

    return eReturnStatus ;
}
/*-------------------------------------------------------------------------------*/
ESuccess  CResourceLocks::Release_L1CalOperational ( )
{
    ESuccess    eReturnStatus = Ok ;

    //\\//

    mg_oMutexL1CalOperational.release () ;

    return eReturnStatus ;
}
/*-------------------------------------------------------------------------------*/
 
 
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess  CResourceLocks::Acquire_TriggerIO ( )
{
    ESuccess    eReturnStatus = Ok ;

    //\\//
    
    mg_oMutexTriggerIO.acquire () ;

    return eReturnStatus ;
}
 /*-------------------------------------------------------------------------------*/
ESuccess  CResourceLocks::TryAcquire_TriggerIO ( )
{
    ESuccess    eReturnStatus = Ok ;

    //\\//
    
    if ( mg_oMutexTriggerIO.tryacquire () == -1 ) 
        eReturnStatus = Error ;

    return eReturnStatus ;
}
/*-------------------------------------------------------------------------------*/
ESuccess  CResourceLocks::Release_TriggerIO ( )
{
    ESuccess    eReturnStatus = Ok ;

    //\\//

    mg_oMutexTriggerIO.release () ;

    return eReturnStatus ;
}
/*-------------------------------------------------------------------------------*/
 
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess  CResourceLocks::Acquire_MonitDataPool ( )
{
    ESuccess    eReturnStatus = Ok ;

    //\\//
    
    mg_oMutexMonitDataPool.acquire () ;

    return eReturnStatus ;
}
 /*-------------------------------------------------------------------------------*/
ESuccess  CResourceLocks::TryAcquire_MonitDataPool ( )
{
    ESuccess    eReturnStatus = Ok ;

    //\\//
    
    if ( mg_oMutexMonitDataPool.tryacquire () == -1 ) 
        eReturnStatus = Error ;

    return eReturnStatus ;
}
/*-------------------------------------------------------------------------------*/
ESuccess  CResourceLocks::Release_MonitDataPool ( )
{
    ESuccess    eReturnStatus = Ok ;

    //\\//

    mg_oMutexMonitDataPool.release () ;

    return eReturnStatus ;
}
/*-------------------------------------------------------------------------------*/
 
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess  CResourceLocks::Acquire_CaptureMonitData ( )
{
    ESuccess    eReturnStatus = Ok ;

    //\\//
    
    mg_oMutexCaptureMonitData.acquire () ;

    return eReturnStatus ;
}
/*-------------------------------------------------------------------------------*/
ESuccess  CResourceLocks::TryAcquire_CaptureMonitData ( )
{
    ESuccess    eReturnStatus = Ok ;

    //\\//
    
    if ( mg_oMutexCaptureMonitData.tryacquire () == -1 ) 
        eReturnStatus = Error ;

    return eReturnStatus ;
}
/*-------------------------------------------------------------------------------*/
ESuccess  CResourceLocks::Release_CaptureMonitData ( )
{
    ESuccess    eReturnStatus = Ok ;

    //\\//

    mg_oMutexCaptureMonitData.release () ;

    return eReturnStatus ;
}
/*-------------------------------------------------------------------------------*/
 
