/*-------------------------------------------------------------------------------*/
  /*
    T962.cpp
    Class CT962
    12-Oct-2004
  */
/*-------------------------------------------------------------------------------*/

#include "L1Cal_IIb_TCC/StdAfx.h"

#include                            "L1Cal_IIb_TCC/T962.h"

#include "L1Cal_IIb_TCC/HandleResultFile.h"

#include "L1Cal_IIb_TCC/UtilsStrings.h"
#include "L1Cal_IIb_TCC/UtilsFiles.h"
#include "L1Cal_IIb_TCC/UtilsBits.h"


// static means created only once throughout life of program (to avoid wasting time)
// *AND* internal linkage (to avoid clash between modules)
#include "L1Cal_IIb_TCC/HandleConsole.h"
static CConsoleAccess oConsole ;

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
// Static data members must be initialized at file scope

    // We (typically) should have only one L1 Cal Trig,
    // this is a way to let any module get a hold of it
    CT962*                  CT962::mg_poT962                = NULL ;
//  CT962*                  CT962::mg_poT962 = new CT962 ; // done in L1Cal_IIb_TCC.cpp

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/

CT962::CT962 ()
{
    unsigned long   ulAdfIdx ;
    unsigned long   ulWireIdx ;
    unsigned long   ulAdfCrateNum ;

    //\\//

    oConsole.WriteLine ( (CString) "" ) ;
    oConsole.WriteLine ( (CString) "Creating T962 DAQ Readout Object" ) ;

   
    m_ulAdfCount = 0 ;
    for ( ulAdfIdx = 0 ;  ulAdfIdx  < kTot_T962_Adf  ; ulAdfIdx ++ )
        m_apoAdf [ulAdfIdx] = NULL ;

    m_poMaestro = NULL ;

    // T962 Wire
    m_ulWireCount = 0 ;
    for ( ulWireIdx = 0 ; ulWireIdx < kTot_T962_Wire ; ulWireIdx ++ )
        m_apoWire[ulWireIdx] = NULL ;


    // ADF-2 Maestro Cards
    m_poAdfMaestroImmediateMonit = NULL ;
    m_poAdfMaestroRequestMonit   = NULL ;

    // link to crawl thru the list of all L1Cal card
    m_pLastAdfCard = NULL ;

    // Run Number
    m_ulRunNumber = 0 ;

    // Run State
    m_eRunState = eRun_Undefined ;

    // Max Number of Event to collect
    m_ulEvtMax = 0 ;

    // Event Number
    m_ulEvtNumber = 0 ;

    // seconds waiting for event
    m_ulEvtWaitHundredthSec = 0 ;

    // Event State
    m_eEvtState = eEvt_Undefined ;
    
    // Minimum CoolDown between events
    m_ulCoolDown_Seconds = 1 ;
    
    // Output mode
    m_lModeBinary = false ;

    // Directory for output files
    m_sDirectory = "" ;

    // Threads to read each crate
    for ( ulAdfCrateNum = 0 ; ulAdfCrateNum < kTot_T962_Crate ; ulAdfCrateNum ++ )
    {
        m_oThreadReadCrate[ulAdfCrateNum] = new CT962_ReadCrate ( this, ulAdfCrateNum ) ;
        m_oThreadReadCrate[ulAdfCrateNum] -> start () ;
    }

}


/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
CT962::~CT962 ()
{

    unsigned long   ulAdfIdx ;
    unsigned long   ulWireIdx ;

    //\\//

    oConsole.WriteLine ( (CString) "" ) ;
    oConsole.WriteLine ( (CString) "Deleting all T962 Objects" ) ;


    oConsole.WriteLine ( (CString) "Deleting " 
                                    + I_to_A ( m_ulWireCount )
                                    + " Wires " ) ;
    for ( ulWireIdx = 0 ; ulWireIdx < m_ulWireCount ; ulWireIdx ++ )
    {
        oConsole.WriteLine ( (CString) "Deleting Wire Register " 
                                        + m_apoWire[ulWireIdx]->m_oMonitRegBase->f_osRegAddress() ) ;
        delete m_apoWire[ulWireIdx] ;
    }

    oConsole.WriteLine ( (CString) "Deleting " 
                                    + I_to_A ( m_ulAdfCount )
                                    + " ADF Cards " ) ;
    for ( ulAdfIdx = 0  ; ulAdfIdx  < m_ulAdfCount  ; ulAdfIdx ++ )
    {
        oConsole.WriteLine ( (CString) "Deleting ADF object " 
                                        + m_apoAdf[ulAdfIdx]->f_osBaseAddress() ) ;
        delete m_apoAdf [ulAdfIdx] ;
    }

    if ( m_poMaestro != NULL ) 
    {
        oConsole.WriteLine ( (CString) "Deleting Maestro ADF object " 
                                        + m_poMaestro ->f_osBaseAddress() ) ;
        delete m_poMaestro ;
    }

}


/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess    CT962::Set_Maestro   ( unsigned char    _ubCrateNum,
                                   unsigned char    _ubCardSlot )

{
    ESuccess        eReturnStatus = Ok ;

    //\\//

    oConsole.WriteLine ( (CString) " Set T962 Maestro as Crate #" 
                                  + I_to_A ( _ubCrateNum )
                                  + " Slot #" 
                                  + I_to_A ( _ubCardSlot ) ) ;


    m_poMaestro = new CAdfCardBase    (  0,             // master
                                        _ubCrateNum,    // slave
                                        _ubCardSlot ) ;

    return eReturnStatus ;
}
/*-------------------------------------------------------------------------------*/


/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess    CT962::Add_Wire      ( unsigned long    _ulWireIdx,
                                   unsigned char    _ubCrateNum,
                                   unsigned char    _ubCardSlot,
                                   unsigned char    _ubChipAddress,
                                   unsigned short   _uwRegAddress,
                                   unsigned long    _ulRegCount )
{
    ESuccess        eReturnStatus = Ok ;

    unsigned long   ulAdfIdx ;
    unsigned long   ulWireIdx ;
    CAdfCardBase*   pTempAdfCard ;

    //\\//


    // See if we have already registered this channel
    for ( ulWireIdx = 0 ; ulWireIdx < m_ulWireCount ; ulWireIdx ++ )
        if ( _ulWireIdx == m_apoWire[ulWireIdx]->m_ulWireNum ) 
        {
            eReturnStatus = Error ;
            oConsole.ErrorLine ( (CString) "Wire # " 
                                            + I_to_A ( _ulWireIdx, 3 )
                                            + " already registered" ) ;

            goto Abort;
        }


    // See if we have already registered the ADF card handling this channel
    pTempAdfCard = NULL ;
    for ( ulAdfIdx = 0  ; ulAdfIdx  < m_ulAdfCount  ; ulAdfIdx ++ )
    if ( ( _ubCrateNum == m_apoAdf[ulAdfIdx]->f_ubVertSlave() )
      && ( _ubCardSlot == m_apoAdf[ulAdfIdx]->f_ubCardSlot()  ) )
        pTempAdfCard = m_apoAdf[ulAdfIdx] ;
    
    // register this ADF new Card
    if ( pTempAdfCard == NULL ) 
    {
        m_apoAdf[m_ulAdfCount] = new CAdfCardBase     (  0,             // master 
                                                        _ubCrateNum,    // slave
                                                        _ubCardSlot ) ;
        pTempAdfCard = m_apoAdf[m_ulAdfCount] ;
        
        m_ulAdfCount++ ;
    }


    // register this channel
    m_apoWire[m_ulWireCount] = new CT962_Wire     ( _ulWireIdx,
                                                    _ubCrateNum,
                                                    _ubCardSlot,
                                                    _ubChipAddress,
                                                    _uwRegAddress,
                                                    _ulRegCount, 
                                                    pTempAdfCard ) ;

    m_ulWireCount++ ;


Abort:

    return eReturnStatus ;
}
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess    CT962::Start_Run           ( unsigned long    _ulRunNumber,
                                         unsigned long    _ulEventMax,
                                         unsigned long    _ulCoolDown,
                                         unsigned long    _ulModeBinary,
                                         char*            _psDirectory ) 

{
    ESuccess        eReturnStatus = Ok ;

    //\\//

    if ( m_poMaestro == NULL ) 
    {
        eReturnStatus = Error ;
        oConsole.ErrorLine ( (CString) " Maestro Card has Not been defined " ) ;

        goto Abort;
    }

    if ( m_eRunState != eRun_Running   ) 
    {
        // wait for everything to be set before declaring m_eRunState = eRun_Running ;

        m_ulRunNumber           = _ulRunNumber ;
        m_ulEvtNumber           = 0 ;
        m_ulEvtMax              = _ulEventMax ;
        m_ulCoolDown_Seconds    = _ulCoolDown ;
        m_sDirectory            = _psDirectory ;

        oConsole.SystemLine ( (CString) " Starting Run #" 
                                     + I_to_A ( m_ulRunNumber ) 
                                     + " Max Events=" 
                                     + I_to_A ( m_ulEvtMax ) 
                                     + " CoolDown="
                                     + I_to_A ( m_ulCoolDown_Seconds ) 
                                     + "s" ) ;

        if ( _ulModeBinary != 0 ) 
        {
            m_lModeBinary  = kResultBinary ;
            oConsole.SystemLine ( (CString) "_Writing Binary Files " ) ;
        }
        else 
        {
            m_lModeBinary  = kResultAscii ;
            oConsole.SystemLine ( (CString) "_Writing ASCII Files " ) ;
        }

        oConsole.SystemLine ( (CString) "_Output Directory=" + m_sDirectory ) ;


    }
    else
    {
        eReturnStatus = Error ;
        oConsole.ErrorLine ( (CString) " Cannot Start Run; Current Run state " 
                                      + f_osRunState () ) ;

        goto Abort;
    }

    //
    m_eRunState = eRun_Running ;

Abort:
    return eReturnStatus ;
}
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess    CT962::Pause_Run           ( ) 

{
    ESuccess        eReturnStatus = Ok ;

    //\\//

    if ( m_eRunState == eRun_Running ) 
    {
        oConsole.SystemLine ( (CString) " Pausing Run #" 
                                     + I_to_A ( m_ulRunNumber ) ) ;
        m_eRunState = eRun_Paused ;
    }
    else
    {
        eReturnStatus = Error ;
        oConsole.ErrorLine ( (CString) " Cannot Pause Run; Current Run state " 
                                      + f_osRunState () ) ;

        goto Abort;
    }

Abort:
    return eReturnStatus ;
}
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess    CT962::Resume_Run          ( ) 

{
    ESuccess        eReturnStatus = Ok ;

    //\\//
    
    if ( m_eRunState == eRun_Paused ) 
    {
        oConsole.SystemLine ( (CString) " Resuming Run #" 
                                     + I_to_A ( m_ulRunNumber ) ) ;
        m_eRunState = eRun_Running ;
    }
    else
    {
        eReturnStatus = Error ;
        oConsole.ErrorLine ( (CString) " Cannot Resume Run; Current Run state " 
                                      + f_osRunState () ) ;

        goto Abort;
    }

Abort:
    return eReturnStatus ;
}
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess    CT962::Stop_Run            ( ) 

{
    ESuccess        eReturnStatus = Ok ;

    //\\//

    if ( ( m_eRunState == eRun_Running ) 
      || ( m_eRunState == eRun_Paused  ) )
    {
        oConsole.SystemLine ( (CString) " Stopping Run #" 
                                     + I_to_A ( m_ulRunNumber ) ) ;
        m_eRunState = eRun_Aborted ;
    }
    else
    {
        eReturnStatus = Error ;
        oConsole.ErrorLine ( (CString) " Cannot Stop Run; Current Run state " 
                                      + f_osRunState () ) ;

        goto Abort;
    }

Abort:
    return eReturnStatus ;
}
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess    CT962::Status              ( unsigned long*   _pulRunNumber,
                                         char*            _psRunState,
                                         unsigned long*   _pulEvtNumber,
                                         char*            _psEvtState ) 

{
    ESuccess        eReturnStatus = Ok ;
	
	ERun_State      eRunState ;
	EEvt_State      eEvtState ;

	TCString		oMsg ;


    //\\//

    *_pulRunNumber   = m_ulRunNumber ;
	eRunState = m_eRunState ;
    strcpy ( _psRunState, f_osRunState ( &eRunState ) ) ;  

    *_pulEvtNumber   = m_ulEvtNumber ;
	eEvtState = m_eEvtState ;
    strcpy ( _psEvtState, f_osEvtState ( &eEvtState ) ) ; 
	if ( eEvtState == eEvt_Waiting )
    {
        strcat ( _psEvtState, "/" ) ; 
        strcat ( _psEvtState, F_to_A ( m_ulEvtWaitHundredthSec/100.,2 ) ) ; 
        strcat ( _psEvtState, "s" ) ; 
    }
	if ( eEvtState == eEvt_CoolDown )
    {
        strcat ( _psEvtState, "/" ) ; 
        strcat ( _psEvtState, I_to_A ( m_ulCoolDown ) ) ; 
        strcat ( _psEvtState, "s" ) ; 
    }

    oConsole.WriteLine ( (CString) " DAQ 480 : Run #" 
                                 + I_to_A ( m_ulRunNumber ) 
								 + " <" 
                                 + f_osRunState ()
                                 + "> Event=" 
                                 + I_to_A ( m_ulEvtNumber ) 
                                 + " / " 
                                 + I_to_A ( m_ulEvtMax ) 
                                 + " <"
                                 + _psEvtState
                                 + ">") ;

//    oConsole.WriteLine ( (CString) "_Output Directory=" + m_sDirectory ) ;


    return eReturnStatus ;
}
/*-------------------------------------------------------------------------------*/


/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
void*       CT962::run ()
{

    unsigned long   ulAdfCrateNum ;
    unsigned long   ulAdfIdx ;
    unsigned long   ulFpgaNum ;
    unsigned long   ulWireIdx ;
    ESuccess        eStatus ;
    TuwRegWord      uwTemp ;
    unsigned long   ulTemp ;

    unsigned long   ulWait ;
    const int       kiMaxWaitMilliSecond = 2000 ;
    const int       kiWaitInterval = KiSleepOneHundredthSecond ;
    bool            bAllCrateReady ;

    unsigned long ulFpgaThisChannel ;
    unsigned long ulStopAddrThisChannel ;
    unsigned long ulSampleNum ;
    unsigned long ulSamplesThisChannel ;

    //\\//

    while ( true ) 
    {
        while ( m_eRunState != eRun_Running ) 
        {            

//            oConsole.WriteLine ( (CString) " DAQ Process Idle : " 
//                                         + f_osRunState () ) ;

            //wait a bit before checking again
            Sleep ( KiSleepOneTenthSecond ) ; 
        }

        while ( m_eRunState == eRun_Running )
        {

            //////////////////////////////////
            // get ADF cards ready to capture
            //////////////////////////////////

            m_eEvtState = eEvt_Arming ;

            //oConsole.WriteLine ( (CString) " Setting Maestro and ADF Cards Ready to capture next event" ) ;

            // # Register Address and data for controlling the NIM Trigger (i.e. Crate_0_Signal_0)
            // RegAddr_MaestroToSCLD       = 1
            // 
            // # Control Data to tell the SCLD to enable/disable the NIM Trigger
            // Data_Enable_NIM_Trigger     = 0xff50 
            // Data_Disable_NIM_Trigger    = 0xff40
            // 
            // # Disable the External NIM Trigger on the SCLD card.
            // # The External NIM Trigger is DISABLED by telling the Maestro ADF-2
            // # card,  i.e. ADF-2 Card Number #0,  to send a NOT asserted
            // # ADF_Crate_to_SCLD_Signal_0   signal to the SCLD card.  Do this
            // # by writing to the Board Control PAL Register Address 1.
            // 
            // Rio_Write    ( SlotNum    = AdfSlotMaestro,
            //                ChipNum    = 0, # i.e. Board Control PAL
            //                RegAddr    = RegAddr_MaestroToSCLD,
            //                DataOut    = Data_Disable_NIM_Trigger )

            uwTemp = kData_Disable_NIM_Trigger ;
            if ( m_poMaestro->m_poBcPal
                            ->m_poCtrlCrateStat
                            ->f_okWriteReg ( &uwTemp ) != Ok )
            {
                oConsole.ErrorLine ( (CString) " Error Disabling External Trigger ; Aborting " ) ;
                m_eRunState = eRun_Aborted ;
                break ;
            }
                            
            
            // # Register Address for Raw ADC Address Generator Control Register
            // RegAddr_RawAdcAddrGen_Ctrl  = 512
            // 
            // # Restart the Address Generators on **ALL** ADF-2 cards 
            // # so that Raw ADC data is being written into the Raw ADC Monitor 
            // # Data Block circular buffers.
            // 
            // # Control Data for the Raw ADC address generator control register to :
            // #  - enable multi turn running
            // #  - enable the Global Write Enable signal to be asserted
            // #  - enable the Save Monitor Data signal to stop the Address Generators
            // Data_ModeCaptureMonitorData = 0x000e
            // 
            // # Control Data bit for the Raw ADC address generator control register to 
            // # Start the Address Generator
            // Mask_StartAddressGenerator  = 0x0001
            // 
            // for AdfNum in range(AdfCardTot) :
            // 
            //     for FpgaNum in range(2) :
            // 
            //         # start address generator (i.e. pulse one bit high)
            //         Rio_Write    ( SlaveNum   = AdfCrateVsCardNum[AdfNum],
            //                        SlotNum    = AdfSlotVsCardNum[AdfNum],
            //                        ChipNum    = FpgaNum,
            //                        RegAddr    = RegAddr_RawAdcAddrGen_Ctrl,
            //                        DataOut    = Data_ModeCaptureMonitorData + Mask_StartAddressGenerator )
            // 
            //         Rio_Write    ( SlaveNum   = AdfCrateVsCardNum[AdfNum],
            //                        SlotNum    = AdfSlotVsCardNum[AdfNum],
            //                        ChipNum    = FpgaNum,
            //                        RegAddr    = RegAddr_RawAdcAddrGen_Ctrl,
            //                        DataOut    = Data_ModeCaptureMonitorData )

            eStatus = Ok ;
            for ( ulAdfIdx = 0  ; ulAdfIdx  < m_ulAdfCount  ; ulAdfIdx ++ )
            {
                for ( ulFpgaNum = 0 ; ulFpgaNum < KiTotFpgaPerAdf ; ulFpgaNum ++ )
                {
                    if ( m_apoAdf[ulAdfIdx]->m_apoT7Fpga[ulFpgaNum]
                                           ->m_poGlb_RawAdcAddrGen_Ctrl
                                           ->f_okPulseBitsHigh( kuiAddrGen_Start_BitMask ) != Ok )
                    {
                        oConsole.ErrorLine ( (CString) " Error Starting Address Generator ; Aborting " ) ;
                        m_eRunState = eRun_Aborted ;
                        eStatus = Error ;
                    }
                }
            }
            if ( eStatus != Ok ) break ;

            
            // # At this point there is a >500 usec delay to make certain that all
            // # of the circular buffers have completely filled with fresh data
            // # before allowing the possibility of a trigger.
            // 
            // time.sleep (0.001)

            Sleep ( 1 ) ; // wait 1 milliSecond 


            // # "arm" the trigger, i.e. the system will now respond to the next occurrence 
            // # of the External NIM Trigger Signal.
            // 
            // Rio_Write    ( SlotNum    = AdfSlotMaestro,
            //                ChipNum    = 0, # i.e. Board Control PAL
            //                RegAddr    = RegAddr_MaestroToSCLD,
            //                DataOut    = Data_Enable_NIM_Trigger )

            uwTemp = kData_Enable_NIM_Trigger ;
            if ( m_poMaestro->m_poBcPal
                            ->m_poCtrlCrateStat
                            ->f_okWriteReg ( &uwTemp ) != Ok )
            {
                oConsole.ErrorLine ( (CString) " Error Enabling External Trigger ; Aborting " ) ;
                m_eRunState = eRun_Aborted ;
                break ;
            }


            /////////////////////////////////////
            // wait for new event to be captured
            /////////////////////////////////////

            m_eEvtState     = eEvt_Waiting ;
            m_ulEvtWaitHundredthSec  = 0 ;
            while ( m_eEvtState == eEvt_Waiting )
            {
                if ( m_eRunState != eRun_Running ) break ;

                // Check with Maestro
                //         # this is where we poll to see if an event has been captured
                //         WriteEnableMask, Reply_Status \
                //           = Rio_Read     ( SlotNum    = AdfSlotMaestro, 
                //                            ChipNum    = 0, # either FPGA would be fine
                //                            RegAddr    = RegAddr_RawAdcMemory_WriteEnable )

                if ( m_poMaestro->m_apoT7Fpga[0]
                                ->m_poGlb_RawAdc_State
                                ->f_okReadReg ( &uwTemp ) != Ok )
                {
                    oConsole.ErrorLine ( (CString) " Error Checking for Event Capture ; Aborting " ) ;
                    m_eRunState = eRun_Aborted ;
                    break ;
                }


                //         if  ( WriteEnableMask == 0x0000 ) : # i.e. an event was captured
                //             break  # break out of the polling loop
                if ( uwTemp == 0x0000 ) 
                    m_eEvtState = eEvt_Reading ;
                else 
                {
/*
                    oConsole.WriteLine ( (CString) " Read " 
                                                   + I_to_A ( uwTemp, 6, 16, '0' ) 
                                                   + " from "
                                                   + m_poMaestro->m_apoT7Fpga[0]
                                                                ->m_poGlb_RawAdc_State
                                                                ->f_osRegAddress() ) ;
*/
                    m_ulEvtWaitHundredthSec += 1 ;
                    //wait a bit before checking again
                    Sleep ( KiSleepOneHundredthSecond ) ; 
                }
            }


            // If we are paused or stopped skip reading this event
            if ( m_eRunState != eRun_Running ) 
            {
                m_eEvtState = eEvt_Done ; // we will start over, so call last event done
                break ;
            }


            /////////////////////////////////////////////////////////////////////////
            // We are now commited to read this event
            /////////////////////////////////////////////////////////////////////////
            m_ulEvtNumber += 1 ;

            // open result file
            /////////////////////////////////////////////////////////////////////////
            // # Open a New Event output File 
            // EventFileName = "R%s_E%s_T%s.dat"  %  ( RunNumStr, EvtCaptNumStr, EventTimeStr ) 
            // EventFile = open ( OutputDir + EventFileName, 'w' )

            CString             osResultFileName ;
            CResultFile*        poResultFile = NULL ;
            time_t              iTime ;

            time ( &iTime ) ; // grab the time here so that the file name and content match
            osResultFileName  = "R" 
                              + I_to_A ( m_ulRunNumber )
                              + "_E" 
                              + I_to_A ( m_ulEvtNumber )
                              + "_T" 
                              + TimeTag ( &iTime ) ;

            if ( m_lModeBinary == kResultAscii )
                osResultFileName  += ".dat" ;
            else
                osResultFileName  += ".bin" ;
                   
            
            poResultFile = new CResultFile ( m_sDirectory + osResultFileName ) ;
            if ( poResultFile->OpenForWrite ( m_lModeBinary, kSuppressInfoMsg ) == Ok )
            {
                //Write a header in the file
                if ( m_lModeBinary == kResultAscii )
                    poResultFile->WriteLine ( (CString) "! DAQ480  V0.1  Run #"
                                                       + I_to_A ( m_ulRunNumber ) 
                                                       + " Event #"
                                                       + I_to_A ( m_ulEvtNumber ) 
                                                       + " Time "
                                                       + TimeStamp () ) ;
                else
                {
                  //    32-bit word : Fixed Value 0x0000D480 ("0x" notation introduces hex values)
                  //                  intention: use this fixed information for sanity check
                  //                             and verification of proper "endian" alignment.
                    ulTemp = kFixedHeaderWord ;
                    poResultFile->WriteBinary ( (char*) &ulTemp, kWriteLong ) ;

                  //    16-bit word : File Format Version.  
                  //                  Currently 0x0001 matching this document
    
                    ulTemp = kFileFormatVersion ;
                    poResultFile->WriteBinary ( (char*) &ulTemp, kWriteShort ) ;

                  //    16-bit word : DAQ480 Software Version 
                  //                  Upper byte is major version, lower byte is minor version
                  //                  Currently 0x0601
    
                    ulTemp = KiVersionMajor<<8 + KiVersionMinor ;
                    poResultFile->WriteBinary ( (char*) &ulTemp, kWriteShort ) ;

                  //    16-bit word : Run Number 
    
                    poResultFile->WriteBinary ( (char*) &m_ulRunNumber, kWriteShort ) ;

                  //    16-bit word : Event Number 
                  //                  The first event in a run is always #1
    
                    poResultFile->WriteBinary ( (char*) &m_ulEvtNumber, kWriteShort ) ;

                  //    32-bit word : Time Stamp 
                  //                  This time stamp identifies the time when the event
                  //                  was captured, within +/- 1 second, and the accuracy
                  //                  of the computer's system clock.
                  //                  This is the value returned by the "time" function, 
                  //                    i.e. "the number of seconds elapsed since midnight,
                  //                          January 1, 1970, coordinated universal time,
                  //                          according to the system clock. "

                    poResultFile->WriteBinary ( (char*) &iTime, kWriteLong ) ;

                  //    16-bit word : Spare Word 
                  //                  Currently 0x0000
    
                    ulTemp = 0x0000 ;
                    poResultFile->WriteBinary ( (char*) &ulTemp, kWriteShort ) ;

                  //    16-bit word : Total Number of Channels being readout in this run 
                  //                  Typically = 480
    
                    poResultFile->WriteBinary ( (char*) &m_ulWireCount, kWriteShort ) ;

                }
            }
            else
            {
                CConsoleAccess::ErrorLine ( " Failed Creating Event Output File ; Aborting" ) ;
                m_eRunState = eRun_Aborted ;
                poResultFile = NULL ;
                break ;
            }

            // Read the Stop Address of the Raw ADC Address Generator 
            // from all Data Path FPGAs once (instead of reading it again for each channel)
            // (The value will be retrieved as the last value read in the register object)
            /////////////////////////////////////////////////////////////////////////

            eStatus = Ok ;
            for ( ulAdfIdx = 0  ; ulAdfIdx  < m_ulAdfCount  ; ulAdfIdx ++ )
            {
                for ( ulFpgaNum = 0 ; ulFpgaNum < KiTotFpgaPerAdf ; ulFpgaNum ++ )
                {
                    if ( m_apoAdf[ulAdfIdx]->m_apoT7Fpga[ulFpgaNum]
                                           ->m_poGlb_RawAdcAddrGen_Curr
                                           ->f_okReadReg ( &uwTemp ) != Ok )
                    {
                        oConsole.ErrorLine ( (CString) " Error Reading Stop Address ; Aborting " ) ;
                        m_eRunState = eRun_Aborted ;
                        eStatus = Error ;
                    }
                }
            }
            if ( eStatus != Ok ) break ;


            // Let Threads start reading monitoring data
            /////////////////////////////////////////////////////////////////////////

            // first verify all threads done are reset from last time
            for ( ulWait = 0 ; ulWait <= kiMaxWaitMilliSecond ; ulWait += kiWaitInterval ) 
            {
                // assume Ready, and override if any crate is not
                bAllCrateReady = true ; 
                for ( ulAdfCrateNum = 0 ; ulAdfCrateNum < kTot_T962_Crate ; ulAdfCrateNum ++ )
                if ( m_oThreadReadCrate[ulAdfCrateNum]->m_bReadDone == true ) 
                    bAllCrateReady = false ;

                // if all crates ready, we can go on
                if ( bAllCrateReady == true ) break ;
                
                Sleep ( kiWaitInterval ) ; // wait 
            }

            if ( ulWait == kiMaxWaitMilliSecond )
            {
                CConsoleAccess::ErrorLine ( (CString) " Timeout after waiting "
                                                    + F_to_A ( ulWait/1000.0,  2 ) 
                                                    + " s for all Read Crate Threads Ready ; Aborting" ) ;

                for ( ulAdfCrateNum = 0 ; ulAdfCrateNum < kTot_T962_Crate ; ulAdfCrateNum ++ )
                oConsole.ErrorLine ( (CString) " T962_ReadCrate#" 
                                              + I_to_A ( ulAdfCrateNum ) 
                                              + " Done flag is "
                                              + I_to_A ( m_oThreadReadCrate[ulAdfCrateNum]->m_bReadDone ) ) ;

                m_eRunState = eRun_Aborted ;
                m_eEvtState = eEvt_Done ; // we will start over, so call last event done
                break ;
            }


            // release threads to read each crate
            for ( ulAdfCrateNum = 0 ; ulAdfCrateNum < kTot_T962_Crate ; ulAdfCrateNum ++ )
                m_oThreadReadCrate[ulAdfCrateNum]->m_bReadRequested = true ;

            // Now wait until threads have read monitoring data
            /////////////////////////////////////////////////////////////////////////

            // wait for all threads to be done
            for ( ulWait = 0 ; ulWait <= kiMaxWaitMilliSecond ; ulWait += kiWaitInterval ) 
            {
                // assume Ready, and override if any crate is not
                bAllCrateReady = true ;
                for ( ulAdfCrateNum = 0 ; ulAdfCrateNum < kTot_T962_Crate ; ulAdfCrateNum ++ )
                if ( m_oThreadReadCrate[ulAdfCrateNum]->m_bReadDone == false ) 
                    bAllCrateReady = false ;

                // if all crates ready, we can go on
                if ( bAllCrateReady == true )
                {
                    CConsoleAccess::WriteLine ( (CString) " Found all Read Crate Threads Done after waiting "
                                                        + F_to_A ( ulWait/1000.0,  2 ) 
                                                        + " s" ) ; 
                    // remove read request, 
                    // and threads will get ready for next event
                    for ( ulAdfCrateNum = 0 ; ulAdfCrateNum < kTot_T962_Crate ; ulAdfCrateNum ++ )
                        m_oThreadReadCrate[ulAdfCrateNum]->m_bReadRequested = false ;

                    break ;
                }

                Sleep ( kiWaitInterval ) ; // wait 
            }

            if ( ulWait == kiMaxWaitMilliSecond )
            {
                CConsoleAccess::ErrorLine ( (CString) " Timeout after waiting "
                                                    + F_to_A ( ulWait/1000.0,  2 ) 
                                                    + " s for all Read Crate Threads Done ; Aborting" ) ;

                for ( ulAdfCrateNum = 0 ; ulAdfCrateNum < kTot_T962_Crate ; ulAdfCrateNum ++ )
                oConsole.ErrorLine ( (CString) " T962_ReadCrate#" 
                                              + I_to_A ( ulAdfCrateNum ) 
                                              + " Done flag is "
                                              + I_to_A ( m_oThreadReadCrate[ulAdfCrateNum]->m_bReadDone ) ) ;

                m_eRunState = eRun_Aborted ;
                if ( poResultFile != NULL ) poResultFile->Close ( kSuppressInfoMsg ) ;
                break ;
            }

            // Write each channel to result file 
            // while re-ordering time slices 
            // which had already been read by the threads
            /////////////////////////////////////////////////////////////////////////
            m_eEvtState = eEvt_Writing ;
            
            for ( ulWireIdx = 0 ; ulWireIdx < m_ulWireCount ; ulWireIdx ++ )
            {
                if ( m_lModeBinary == kResultAscii )
                    // # write channel number identification e.g. "# System Channel #064"
                    poResultFile->WriteLine ( (CString) "! System Channel #"
                                                       + I_to_A ( m_apoWire[ulWireIdx]->m_ulWireNum, 3, 10, '0' ) ) ;
                else
                {
                    //    16-bit word : Channel/Wire Number 
                    //                  The first Wire in the full system is numbered #1.
                    //                  There is however no guarantee during commissioning
                    //                  that all channels are readout, or that they are being 
                    //                  readout in any contiguous order, or start at #1.
                    //                  This Wire Channel numbering and ordering is controlled 
                    //                  by the configuration files for the DAQ 480 engine.
                  
                    poResultFile->WriteBinary ( (char*) &m_apoWire[ulWireIdx]->m_ulWireNum, kWriteShort ) ;

                    //    16-bit word : Total Number of Samples for This Channel
                    //                  Typically = 2048   
                    //                  The Number of samples per Channel is controlled by 
                    //                  the configuration files for the DAQ 480 engine.

                    poResultFile->WriteBinary ( (char*) &m_apoWire[ulWireIdx]->m_ulRegCount, kWriteShort ) ;

                }


                // # Re-align and copy data to event file

               //#define useFakeData
               #ifdef  useFakeData
                ulFpgaThisChannel     = 0 ; // to avoid warning
                ulStopAddrThisChannel = ( 400 + 20 * ulWireIdx ) % 2048 ;
               #else
                //# We already have the stop address
                ulFpgaThisChannel     = m_apoWire[ulWireIdx]->m_ulChipAddress ;
                ulStopAddrThisChannel = m_apoWire[ulWireIdx]->m_pMotherAdf
                                                            ->m_apoT7Fpga[ulFpgaThisChannel]
                                                            ->m_poGlb_RawAdcAddrGen_Curr
                                                            ->f_uwLastRead () ;
               #endif
                ulSamplesThisChannel = m_apoWire[ulWireIdx]->m_ulRegCount ;
                
                //# Start with the part after the stop address (older data)
                for ( ulSampleNum = ulStopAddrThisChannel+1 ; ulSampleNum < ulSamplesThisChannel ; ulSampleNum ++ )
                {
					//truncate to 10 bits
                    m_apoWire[ulWireIdx]->m_paReadBuffer[ulSampleNum] &= 0x3ff ;

                    if ( m_lModeBinary == kResultAscii )
                        poResultFile->WriteLine ( I_to_A ( m_apoWire[ulWireIdx]->m_paReadBuffer[ulSampleNum] ) ) ;
                    else
                    {
                        //    16-bit word : Raw ADC sample for this channel
                        poResultFile->WriteBinary ( (char*) &m_apoWire[ulWireIdx]->m_paReadBuffer[ulSampleNum], kWriteShort ) ;
                    }
                }
                
                //# End with the part before the stop address (newer data)
                for ( ulSampleNum = 0 ; ulSampleNum < ulStopAddrThisChannel+1 ; ulSampleNum ++ )
                {
					//truncate to 10 bits
                    m_apoWire[ulWireIdx]->m_paReadBuffer[ulSampleNum] &= 0x3ff ;

                    if ( m_lModeBinary == kResultAscii )
                        poResultFile->WriteLine ( I_to_A ( m_apoWire[ulWireIdx]->m_paReadBuffer[ulSampleNum] ) ) ;
                    else
                    {
                        //    16-bit word : Raw ADC sample for this channel
                        poResultFile->WriteBinary ( (char*) &m_apoWire[ulWireIdx]->m_paReadBuffer[ulSampleNum], kWriteShort ) ;
                    }
                }

                // done with this Channel

            }

            // write event trailer
            if ( m_lModeBinary == kResultAscii )
            {
                // no trailer in ascii format
            }
            else
            {
                //    32-bit word : Spare
                //                  Currently 0x00000000
                ulTemp = 0x0000 ;
                poResultFile->WriteBinary ( (char*) &ulTemp, kWriteLong ) ;

                //    32-bit word : Reserved for possible checksum in the future
                //                  Currently 0x00000000
                ulTemp = 0x0000 ;
                poResultFile->WriteBinary ( (char*) &ulTemp, kWriteLong ) ;
            }

            // close event file 
            if ( poResultFile != NULL ) poResultFile->Close ( kSuppressInfoMsg ) ;


            // done with this event
            m_eEvtState = eEvt_Done ;

            // check to see if we collected enough events
            if ( ( m_ulEvtMax != 0 ) 
              && ( m_ulEvtNumber >= m_ulEvtMax ) )
            {
                oConsole.SystemLine ( (CString) " DAQ Run #" 
                                             + I_to_A ( m_ulRunNumber ) 
                                             + " Finished " ) ;
                m_eRunState = eRun_Done ;
                break ;
            }


            // cool down
            for ( m_ulCoolDown = m_ulCoolDown_Seconds ; m_ulCoolDown > 0 ; m_ulCoolDown -- )
            {
                //check to see if we need to abort cooldown
                if ( m_eRunState != eRun_Running ) 
                {
                    m_eEvtState = eEvt_Done ; // we will start over, so call last event done
                    break ;
                }

                m_eEvtState = eEvt_CoolDown ;

                Sleep ( KiSleepOneSecond ) ; 
            }

        }
    }
    
    return 0 ;
} ;
/*-------------------------------------------------------------------------------*/


TCString    CT962::f_osRunState ( ERun_State*     _peRunState )
{
    ERun_State     eRunState ;
    TCString  oMsg ;

    if ( _peRunState != NULL ) 
        eRunState = *_peRunState ;
    else 
        eRunState = m_eRunState ;

    switch ( eRunState )
    {
        case eRun_Undefined :   oMsg = "Undefined"  ; break ;
        case eRun_Defined   :   oMsg = "Defined"    ; break ;
        case eRun_Running   :   oMsg = "Running"    ; break ;
        case eRun_Paused    :   oMsg = "Paused"     ; break ;
        case eRun_Done      :   oMsg = "Done"       ; break ;
        case eRun_Aborted   :   oMsg = "Aborted"    ; break ;
        default             :   oMsg = "?Illegal?"    ; break ; 
    } ;

    return oMsg ;
}


TCString    CT962::f_osEvtState ( EEvt_State*     _peEvtState )
{
    EEvt_State      eEvtState ;
    TCString		oMsg ;

    if ( _peEvtState != NULL ) 
        eEvtState = *_peEvtState ;
    else 
        eEvtState = m_eEvtState ;

    switch ( eEvtState )
    {
        case eEvt_Undefined :   oMsg = "Undefined"  ; break ;
        case eEvt_Arming    :   oMsg = "Arming"    ; break ;
        case eEvt_Waiting   :   oMsg = "Waiting"    ; break ;
        case eEvt_Reading   :   oMsg = "Reading"    ; break ;
        case eEvt_Done      :   oMsg = "Done"       ; break ;
        case eEvt_CoolDown  :   oMsg = "CoolDown"   ; break ;
        default             :   oMsg = "?Illegal?"    ; break ; 
    } ;

    return oMsg ;
}
