/*-------------------------------------------------------------------------------*/
  /*
    T962_ReadCrate.cpp
    Class CT962_ReadCrate
    28-Nov-2008
  */
/*-------------------------------------------------------------------------------*/

#include "L1Cal_IIb_TCC/StdAfx.h"

#include                            "L1Cal_IIb_TCC/T962_ReadCrate.h"

#include "L1Cal_IIb_TCC/T962.h"

#include "L1Cal_IIb_TCC/HandleHardwareAccess.h"

#include "L1Cal_IIb_TCC/UtilsStrings.h"
//#include "L1Cal_IIb_TCC/UtilsFiles.h"
//#include "L1Cal_IIb_TCC/UtilsBits.h"

// static means created only once throughout life of program (to avoid wasting time)
// *AND* internal linkage (to avoid clash between modules)
#include "L1Cal_IIb_TCC/HandleConsole.h"
static CConsoleAccess oConsole ;

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
// Static data members must be initialized at file scope
/*
    // We (typically) should have only one L1 Cal Trig,
    // this is a way to let any module get a hold of it
    CT962_ReadCrate*                  CT962_ReadCrate::mg_poT962_ReadCrate                = NULL ;
//  CT962_ReadCrate*                  CT962_ReadCrate::mg_poT962_ReadCrate = new CT962_ReadCrate ; // done in L1Cal_IIb_TCC.cpp

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/

CT962_ReadCrate::CT962_ReadCrate ( CT962*              _poT962,
                                   unsigned long       _ulAdfCrateNum )
{

    //\\//

    oConsole.WriteLine ( (CString) "" ) ;
    oConsole.WriteLine ( (CString) "Creating T962_ReadCrate Object for Crate#" 
                                  + I_to_A ( _ulAdfCrateNum ) ) ;

    // handy access to main T962 object 
    m_poT962 = _poT962 ;

    // which crate the thread will readout
    m_ulAdfCrateNum = _ulAdfCrateNum ;

    // synchronization flags
    m_bReadRequested    = false ;
    m_bReadDone         = false ;

}


/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
CT962_ReadCrate::~CT962_ReadCrate ()
{

    //\\//

    oConsole.WriteLine ( (CString) "" ) ;
    oConsole.WriteLine ( (CString) "Deleting all T962_ReadCrate Objects for Crate#" 
                                  + I_to_A ( m_ulAdfCrateNum ) ) ;

}


/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
void*       CT962_ReadCrate::run ()
{
    unsigned long   ulWireIdx ;

    const int       kiWaitInterval = KiSleepOneHundredthSecond ;

    //\\//

    while ( true ) 
    {
        if ( m_bReadRequested  == false ) // No request yet, or last event done
        {
            m_bReadDone = false ;
            Sleep ( kiWaitInterval ) ; 
        }

        else if ( m_bReadDone == true ) // This event was already read
        {
            Sleep ( kiWaitInterval ) ; 
        }
        
        else
        {
            // # now retrieve the monitoring data for each system channel, one at a time, 
            // # by channel number (i.e. alternating between FPGAs)
            // but only for the cards in the crate this thread is responsible for 
            
            for ( ulWireIdx = 0 ; ulWireIdx < m_poT962->m_ulWireCount ; ulWireIdx ++ )
            if ( m_poT962->m_apoWire[ulWireIdx]->m_pMotherAdf->f_ubVertSlave() == m_ulAdfCrateNum ) 
            {
                /*                
                oConsole.WriteLine ( (CString) "Thread for Crate#" 
                                              + I_to_A ( m_ulAdfCrateNum ) 
                                              + " Reading Wire # "
                                              + I_to_A ( m_poT962->m_apoWire[ulWireIdx]->m_ulWireNum, 3 ) 
                                              //+ " Register " 
                                              //+ m_apoWire[ulWireIdx]->m_oMonitRegBase->f_osRegAddress() 
                                              ) ;
                */

               //#define useFakeData
               #ifdef  useFakeData
                #include "L1Cal_IIb_TCC/T962FakeData.h"
               #else

				// use PIO on crate 1 and DMA on crate 0
				if ( m_ulAdfCrateNum == 1 )
				{
					// # Read the raw ADC circular buffer data from   this channel 
					m_poT962->m_apoWire[ulWireIdx]->m_oMonitRegBase->f_okReadBlock ( m_poT962->m_apoWire[ulWireIdx]->m_paReadBuffer,
																					 m_poT962->m_apoWire[ulWireIdx]->m_ulRegCount,
																					 kIncrementAddressByOneForEachRead ) ;
				}
				else
				{                
					unsigned long ulVmeAddress ;
					ulVmeAddress  = m_poT962->m_apoWire[ulWireIdx]->m_oMonitRegBase->f_ulVmeAddress() ;
					ulVmeAddress &= 0x00ffffff ;
					if ( CHardwareAccess::mg_poBit3BusAdaptorArray[m_ulAdfCrateNum] 
						->ReadVmeData ( ulVmeAddress ,
										m_poT962->m_apoWire[ulWireIdx]->m_paReadBuffer,
									  2*m_poT962->m_apoWire[ulWireIdx]->m_ulRegCount ) != Ok ) 
						 oConsole.ErrorLine ( (CString) "Buffer for Crate#" 
													  + I_to_A ( m_ulAdfCrateNum ) 
													  + " Wire # "
													  + I_to_A ( m_poT962->m_apoWire[ulWireIdx]->m_ulWireNum, 3 ) 
													  + " DMA Read Failed " 
													  ) ;
				}

               #endif

            }

            m_bReadDone         = true ;
        }
    }
    
    return 0 ;
} ;
/*-------------------------------------------------------------------------------*/

