/*-------------------------------------------------------------------------------*/
  /*
    UtilsFiles.cpp
    11-May-98
  */
/*-------------------------------------------------------------------------------*/
#include "L1Cal_IIb_TCC/StdAfx.h"

#include "L1Cal_IIb_TCC/GlobalConstants.h"
#include "L1Cal_IIb_TCC/CustomDataTypes.h"

#include                            "L1Cal_IIb_TCC/UtilsFiles.h"

#include "L1Cal_IIb_TCC/UtilsStrings.h"
#include "L1Cal_IIb_TCC/UtilsBits.h"

//#include <process.h>  // for spawn
#ifdef WIN32
    #include <direct.h> // for _getdrive, _chdrive
#endif

#ifdef WIN32
    // Global variable to remember the default drive letter
    // initialized in the main L1Cal_IIb_TCC.cpp
    extern int giDefaultDriveID ;
#endif

//global variable to remember the directory last browsed
char    sCurrentBrowseDir[_MAX_DIR] = KsDefaultInitialBrowseDir ;

// static means created only once throughout life of program (to avoid wasting time)
// *AND* internal linkage (to avoid clash between modules)
#include "L1Cal_IIb_TCC/HandleConsole.h"
static      CConsoleAccess  oConsole ;

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
/*
ESuccess    FileBrowse ( CString*   _posFileName, 
                         CString    _osFileType,
                         CString    _osDefaultDir ) 
{
	OPENFILENAME xOpenFileName ;
	
    CString  osFilter ;
	
	char    sFilter[40]	 ;
    char    sFileName[256] = "" ;
	char    sFileTitle[81] = "" ;
    int     ilCharPos ;

    //\\//

    //fill-in directory name if omitted.
    if ( _osDefaultDir == "" ) _osDefaultDir = sCurrentBrowseDir ;

    // prepare file filter for pop up box, 
    // we want 2 file types: 1) the requested file type, and 2) all files
    // the format for the filter string is 
    // "Text_Type1<NULL>Wildcard_Filter1<NULL>Text_Type2<NULL>Wildcard_Filter2...<NULL><NULL>"
    // which is hard to build with CStrings or with standard null terminated string manip routines 
    
    // build "*.xxx Files<NULL>"
	strcpy ( sFilter, "*." ) ;
    strcat ( sFilter, (LPCTSTR)_osFileType ) ;
    strcat ( sFilter, " Files" ) ;

    // locate the character position after this first <NULL>
    ilCharPos = strlen ( sFilter ) + 1 ;

    // now build up to "*.xxx Files<NULL>*.xxx<NULL>"
    strcpy ( &sFilter[ilCharPos], "*." ) ;
	strcat ( &sFilter[ilCharPos], (LPCTSTR)_osFileType ) ;

    // locate the character position after the last <NULL>
    ilCharPos = ilCharPos + strlen ( &sFilter[ilCharPos] ) + 1 ;

    // now build up to "*.xxx Files<NULL>*.xxx<NULL>All Files<NULL>"
    strcpy ( &sFilter[ilCharPos], "All Files" ) ;

    // locate the character position after the last <NULL>
    ilCharPos = ilCharPos + strlen ( &sFilter[ilCharPos] ) + 1 ;

    // now build up to "*.xxx Files<NULL>*.xxx<NULL>All Files<NULL>*.*<NULL>"
    strcpy ( &sFilter[ilCharPos], "*.*" ) ;

    // locate the character position after the last <NULL>
    ilCharPos = ilCharPos + strlen ( &sFilter[ilCharPos] ) + 1 ;

    // add one <NULL> to have the double <NULL><NULL> at the end of this string
    sFilter[ilCharPos] = 0 ;

	// build the OPENFILENAME structure for call to GetOpenFileName
	xOpenFileName.lStructSize =  sizeof(xOpenFileName) ; 
    xOpenFileName.hwndOwner = NULL ; 
    xOpenFileName.hInstance = NULL ; 
    xOpenFileName.lpstrFilter =  sFilter ; 
    xOpenFileName.lpstrCustomFilter =  NULL ; 
    xOpenFileName.nMaxCustFilter =  NULL ; 
    xOpenFileName.nFilterIndex = 1 ; 
    xOpenFileName.lpstrFile = sFileName ; 
    xOpenFileName.nMaxFile = 256 ; 
    xOpenFileName.lpstrFileTitle = sFileTitle ; 
    xOpenFileName.nMaxFileTitle = 81 ; 
    xOpenFileName.lpstrInitialDir = (LPCTSTR) _osDefaultDir ; 
    xOpenFileName.lpstrTitle = NULL ; 
    xOpenFileName.Flags =  OFN_EXPLORER 
		                || OFN_FILEMUSTEXIST 
						|| OFN_HIDEREADONLY 
						|| OFN_LONGNAMES ; 
    xOpenFileName.nFileOffset = 0 ; 
    xOpenFileName.nFileExtension = 0 ; 
    xOpenFileName.lpstrDefExt = (LPCTSTR) _osFileType ; 
    xOpenFileName.lCustData = NULL ; 
    xOpenFileName.lpfnHook = NULL ; 
    xOpenFileName.lpTemplateName = NULL ; 


    if ( GetOpenFileName( &xOpenFileName ) ) 
	{
		*_posFileName = sFileName ;

        //keep track of this last directory browsed
        _splitpath ( sFileName, NULL, sCurrentBrowseDir, NULL, NULL ) ;
       
        return Ok ;
	}	
    else
    {
		*_posFileName = "Error" ;
        return Error ;
    }

}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
/*
ESuccess    FileEdit   ( CString    _osFileName,
                         CString    _osDirectory ) 
{
    char     sFullFileName[_MAX_FNAME] ;

    char acNotepad[] = "Notepad" ;

    //\\//

    strcpy ( sFullFileName, (LPCTSTR) _osDirectory ) ;
    strcat ( sFullFileName, (LPCTSTR) _osFileName ) ;
    LocateFile ( sFullFileName, sFullFileName )  ;

    _spawnlp ( _P_WAIT, acNotepad, acNotepad, sFullFileName, NULL ) ;	

    return Ok ;

}


/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess    LocateFile ( const char* const  _psFileSearch,
                         char*              _psFileFound ) 
{
	ESuccess        eReturnSuccess = Ok ;

    char            psFileName[_MAX_FNAME] ;

    //\\//

    //translate environment variable (if necessary)
    if ( SubstituteEnvVar ( _psFileSearch, psFileName ) != Ok ) 
         eReturnSuccess = Error ;

    // restore the default drive letter
    // because the environment variables do not include it
    // and the searchenv may fail.
    // (e.g. one can lose the drive letter after cancelling 
    // out of a File Browse box) 
#ifdef WIN32
    _chdrive( giDefaultDriveID ) ;
#endif
   
    // get the full path
#ifdef WIN32
    _searchenv ( psFileName, "", _psFileFound ) ;
#else

    // temporary kludge to replace _searchenv
	FILE *hSRecFile ;
	hSRecFile = fopen ( psFileName, "r" ) ;
	if (hSRecFile == NULL) 
    {
        eReturnSuccess = Error ;
        *_psFileFound = 0 ;
    }
    else
    {
        strcpy ( _psFileFound, psFileName ) ;
        fclose (hSRecFile) ;
    }

#endif

    return eReturnSuccess ;

}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess    SubstituteEnvVar  ( const char* const  _psPathRaw,
                                char*              _psPathSubstituted ) 
{
	ESuccess        eReturnSuccess ;

    char            sPathRaw[_MAX_FNAME+1] ; // used to allow input an output string to be identical
    char            sEnvName[_MAX_FNAME+1] ;

    char*           psEnvValue ;
    char*           pcSecondPercent ;

    //\\//

	eReturnSuccess = Ok ;

    //get a copy of the input string 
    //in case writing to the output string overwrites the input string
    strncpy ( sPathRaw, _psPathRaw, _MAX_FNAME ) ;
    //and force-terminate the input string fpr protection
    sPathRaw[_MAX_FNAME] = 0 ;

    //Start asuming no substitusion was needed
    strncpy ( _psPathSubstituted, sPathRaw, _MAX_FNAME ) ;

    //Look for the environment variable at the BEGINNING of the input path
    if ( sPathRaw[0] == ksEnvChar[0] ) 
    {
        //Look for the second percent
        pcSecondPercent = strpbrk ( sPathRaw+1, ksEnvChar ) ;

        if ( pcSecondPercent == NULL )  //Not good: there was only one "%"
		goto Syntax_Error ;

        //extract the path variable from the left side...
        strncpy (   sEnvName,                           //destination
                    sPathRaw+1,                         //source
                    pcSecondPercent - sPathRaw - 1 ) ; //number of characters

        // null-terminate the path variable string
        sEnvName[pcSecondPercent - sPathRaw - 1] = 0 ;

        if ( strlen( sEnvName ) == 0 )  //Not good: no characters between the two "%"
		goto Syntax_Error ;

        if ( strpbrk ( pcSecondPercent+1, ksEnvChar ) != NULL ) //Not good: there were more than two "%" 
		goto Syntax_Error ;

        //make environment variable upper case because _searchenv is case-sensitive
        //and the environment variables were defined as upper case names.
        strupr ( sEnvName ) ; 

        //Now we can try to translate the environment variable
        psEnvValue = getenv ( sEnvName ) ;

        if ( psEnvValue == NULL ) 
        {
            oConsole.ErrorLine ( (CString) "Undefined Environment Variable <"
                                          + sEnvName + "> Used in <" 
                                          + sPathRaw + ">" ) ;
            eReturnSuccess = Error ;
		    goto Done ;
        }
        else
        {
            //This is it we can now start with the translated environment variable and...
            strcpy ( _psPathSubstituted, psEnvValue ) ;
            //add a BackSlash if needed
            if ( ( pcSecondPercent[1] != 0 ) 
#if defined WIN32
              && ( pcSecondPercent[1] != '\\' ) ) 
                strcat ( _psPathSubstituted, "\\" ) ;
#else
              && ( pcSecondPercent[1] != '/' ) ) 
                strcat ( _psPathSubstituted, "/" ) ;
#endif
            //...we add the file name starts at the right side of the percent
            strcat ( _psPathSubstituted, pcSecondPercent + 1 ) ;
        }
    }
    else 
    if ( strpbrk ( sPathRaw, ksEnvChar ) != NULL ) //there was a "%" but it wasn't the first character 
		goto Syntax_Error ;

    goto Done ;

Syntax_Error:

    oConsole.ErrorLine ( (CString) "Illegal Syntax for Environment Variable in <" 
                                  + sPathRaw + ">" ) ;
    eReturnSuccess = Error ;

Done:

    return eReturnSuccess ;
}


/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
ESuccess    ExtractVersion ( const char* const  _sFileName,	
                             unsigned char*     _pubVersionMajor,
                             unsigned char*     _pubVersionMinor ) 
{
	ESuccess        eReturnSuccess = Ok ;
    char            psFileName[_MAX_FNAME] ;

    char*           pcDot ;
    char*           pcLastUnderscore ;
    char*           pcPrevUnderscore ;

    //\\//

    // copy file name so that it can be modified
    strcpy ( psFileName,    _sFileName ) ;

    //Locate Dot between file name and file extension (searching backwards from end)
    pcDot = strrchr ( psFileName, '.' ) ;

    if ( pcDot == NULL ) 
    {
        oConsole.ErrorLine ( (CString) "Failed locating dot in file name <" 
                                        + _sFileName + ">" ) ;
    	eReturnSuccess = Error ;
        goto Abort ;
    }

    //terminate the string where the dot was
    *pcDot = 0 ;


    //Locate Last underscore (searching backwards from end)
    pcLastUnderscore = strrchr ( psFileName, '_' ) ;

    if ( pcLastUnderscore == NULL ) 
    {
        oConsole.ErrorLine ( (CString) "Failed locating last underscore in file name <" 
                                        + _sFileName + ">" ) ;
    	eReturnSuccess = Error ;
        goto Abort ;
    }

    //terminate the string where the Last Underscore was
    *pcLastUnderscore = 0 ;


    //Locate previous underscore (searching backwards from end)
    pcPrevUnderscore = strrchr ( psFileName, '_' ) ;

    if ( pcPrevUnderscore == NULL ) 
    {
        oConsole.ErrorLine ( (CString) "Failed locating next to last underscore in file name <" 
                                        + _sFileName + ">" ) ;
    	eReturnSuccess = Error ;
        goto Abort ;
    }

    // Now we know where the Version Number fields are 
    *_pubVersionMajor = fubTruncToByte ( A_to_uI ( pcPrevUnderscore+1 ) ) ;
    *_pubVersionMinor = fubTruncToByte ( A_to_uI ( pcLastUnderscore+1 ) ) ;

Abort:

    return eReturnSuccess ;

}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
void     AddFileVersion ( CString*      _posFileName ) 
{
    FILE*       hFile ;
    CString     osFile_Name ;
	char        acMode_Read[] = "r" ; // read only, to check for file existence

    //\\//

    for ( int iVersion = 1 ; iVersion < 10000 ; iVersion++ )
    {// note: the maximum version number is ;9999 
     // after wchich this last file will be overwritten

        //build a file name
        osFile_Name  = *_posFileName ;
        osFile_Name += ";" + I_to_A ( iVersion ) ;

        //try to open this file in read mode 
        //(i.e. see if it already exists)
        hFile = fopen ( osFile_Name.GetBuffer(0), acMode_Read ) ;
        
        if ( hFile == NULL ) 
            //we are one version past the last such file with this name
            break ;
        else
            fclose ( hFile ) ;
    }

    //update the file name argument
    *_posFileName  = osFile_Name ;

    return ;
}


