#include <stdlib.h>
#include <stdio.h>
#include "tabgab/util.h"
#include "tabgab/vmescl.h"
#include "tabgab/tab.h"

//
// cyclone status read
//

//
// Drivers for the cyclone chip on TAB and GAB
//

//read the cyclone status register:
int cyclone_status_read  (unsigned long module, unsigned long *data)
{
    TRY (check_tab_or_gab(module));    

    TRY (vmescl_read_serial(module, TAB_CHIP_CYCLONE, CYCLONE_GROUP_STATUS,
			 CYCLONE_GROUP_STATUS_SERADR_READ, 0, data));

    return EXIT_SUCCESS;
}


//interpret the cyclone status register:
int cyclone_status_print  (unsigned long data)
{
    info("              A    B");
    info("CONF_MUX:     %d    %d", (data & CYCLONE_STATUS_MASK_CONF_MUX_A)  != 0, (data & CYCLONE_STATUS_MASK_CONF_MUX_B)  != 0 );     
    info("NCONFIG:      %d    %d", (data & CYCLONE_STATUS_MASK_NCONFIG_A)   != 0, (data & CYCLONE_STATUS_MASK_NCONFIG_B)   != 0 );     
    info("NSTATUS:      %d    %d", (data & CYCLONE_STATUS_MASK_NSTATUS_A)   != 0, (data & CYCLONE_STATUS_MASK_NSTATUS_B)   != 0 );     
    info("CONF_DONE:    %d    %d", (data & CYCLONE_STATUS_MASK_CONF_DONE_A) != 0, (data & CYCLONE_STATUS_MASK_CONF_DONE_B) != 0 );     
    info("SEROUT_EN:    %d    %d", (data & CYCLONE_STATUS_MASK_SEROUT_EN_A) != 0, (data & CYCLONE_STATUS_MASK_SEROUT_EN_B) != 0 );
    info("MRST:         %d    %d", (data & CYCLONE_STATUS_MASK_MRST_A)      != 0, (data & CYCLONE_STATUS_MASK_MRST_B)      != 0 );
    info("POWER:        %d    %d", (data & CYCLONE_STATUS_MASK_PWR_A)       != 0, (data & CYCLONE_STATUS_MASK_PWR_B)       != 0 );

    return EXIT_SUCCESS;
}


//this is not a user function:
// - use set/clear instead to turn on and off specific status bits
// - use power_up
int cyclone_status_write (unsigned long module, unsigned long data)
{
    TRY (check_tab_or_gab(module));

    TRY (vmescl_write_serial(module, TAB_CHIP_CYCLONE, CYCLONE_GROUP_STATUS, 
			    CYCLONE_GROUP_STATUS_SERADR_WRITE, data));
    return EXIT_SUCCESS;
}

//turn on the bits specified in the mask:
int cyclone_status_set(unsigned long module, unsigned long mask)
{
    unsigned long data;

    TRY (check_tab_or_gab(module));
    
    if (mask & CYCLONE_STATUS_MASK_PWR_A || mask & CYCLONE_STATUS_MASK_PWR_B) {
	error("Don't use this function for turning the power on or off.");
	return EXIT_FAILURE;
    }

    TRY (cyclone_status_read(module, &data));
    TRY (cyclone_status_write(module, data | mask));

    return EXIT_SUCCESS;
}

//turn off the bits specified in the mask:
int cyclone_status_clear(unsigned long module, unsigned long mask)
{
    unsigned long data;
    
    TRY (check_tab_or_gab(module));
    
    if (mask & CYCLONE_STATUS_MASK_PWR_A || mask & CYCLONE_STATUS_MASK_PWR_B) {
	error("Don't use this function for turning the power on or off.");
	return EXIT_FAILURE;
    }
    
    TRY (cyclone_status_read(module, &data));
    TRY (cyclone_status_write(module, data & ~mask));

    return EXIT_SUCCESS;
}

/* Function that turn on and aff the stratix chips */

#define TURNON_PWRA 1
#define TURNON_PWRB 1

int cyclone_power_up_stratix(unsigned long module)
{
    unsigned long data;
    
    if (TURNON_PWRA) {
	TRY (cyclone_status_read  (module, &data));
	TRY (cyclone_status_write (module, data | CYCLONE_STATUS_MASK_PWR_A));
    }
    if (TURNON_PWRB) {
	TRY (cyclone_status_read  (module, &data));
	TRY (cyclone_status_write (module, data | CYCLONE_STATUS_MASK_PWR_B));
    }
    return EXIT_SUCCESS;
}

int cyclone_power_down_stratix(unsigned long module)
{
    unsigned long data;
    
    unsigned long notpwrb = CYCLONE_STATUS_MASK_PWR_B;
    unsigned long notpwra = CYCLONE_STATUS_MASK_PWR_A;
    notpwrb = ~notpwrb;
    notpwra = ~notpwra;
    
    TRY (cyclone_status_clear(module, CYCLONE_STATUS_MASK_SEROUT_EN_A | CYCLONE_STATUS_MASK_SEROUT_EN_B));
    if (TURNON_PWRB) {
	TRY (cyclone_status_read  (module, &data));
	TRY (cyclone_status_write (module, data & notpwrb));
    }    
    if (TURNON_PWRA) {
	TRY (cyclone_status_read  (module, &data));
	TRY (cyclone_status_write (module, data & notpwra));
    }    
    return EXIT_SUCCESS;
}

/* in case it needs to be an upper byte */
#define SHIFT_CONFBYTE 0

int cyclone_write_conf_byte(unsigned long module, char type, unsigned long confbyte) 
{
    /* this fucntion writes a configuration byte:
     * slot, tab: the slot and tab being written to.
     * type: 'a' or 'b', indicating whether this is an
     *    'a' type write to chips 0-9 or a 'b' type write
     *    to chip 10.
     * conftype: the actual value to be written. Although this
     *    is an unsigned long, the value is byte-sized
     */
    
    TRY (check_tab_or_gab(module));
    
    if(confbyte >= 256)
    {
	error("The configuration byte should be in the range 0-255");
	return EXIT_FAILURE;
    }
    if(type == 'a' || type == 'A')
    {
	return vmescl_write_serial(module, TAB_CHIP_CYCLONE, CYCLONE_GROUP_CONF_DATA, 
				CYCLONE_GROUP_CONF_DATA_SERADR_CONF8_A, confbyte << SHIFT_CONFBYTE);
    }
    else if(type == 'b' || type == 'B')
    {
	return vmescl_write_serial(module, TAB_CHIP_CYCLONE, CYCLONE_GROUP_CONF_DATA, 
				CYCLONE_GROUP_CONF_DATA_SERADR_CONF8_B, confbyte << SHIFT_CONFBYTE);
    }
    else 
    {
	error("Type should be either A or B");
	return EXIT_FAILURE;
    }
}

// Write a configuration half-word:
int cyclone_write_conf_half_word(unsigned long module, char type, unsigned long confhw)
{
    /* this fucntion writes a configuration half-word (2 bytes):
     * tab: thetab being written to.
     * type: 'a' or 'b', indicating whether this is an
     *    'a' type write to chips 0-9 or a 'b' type write
     *    to chip 10.
     * confhw: the actual value to be written. Although this
     *    is an unsigned long, the value is hw-sized
     */

    TRY (check_tab_or_gab(module));
    
    if(confhw >= 65536)
    {
	error("The configuration half-word should be in the range 0-65536");
	return EXIT_FAILURE;
    }
    if(type == 'a' || type == 'A')
    {
	return vmescl_write_serial(module, TAB_CHIP_CYCLONE,
				CYCLONE_GROUP_CONF_DATA, 
				CYCLONE_GROUP_CONF_DATA_SERADR_CONF16_A,
				confhw);
    }
    else if(type == 'b' || type == 'B')
    {
	return vmescl_write_serial(module, TAB_CHIP_CYCLONE,
				CYCLONE_GROUP_CONF_DATA, 
				CYCLONE_GROUP_CONF_DATA_SERADR_CONF16_B,
				confhw);
    }
    else 
    {
	error("Type should be either A or B");
	return EXIT_FAILURE;
    }
}
