//
// gab_defs.h:  defines for the TAB and GAB cards
// 

#ifndef __GAB_DEGS_H__
#define __GAB_DEGS_H__

//how many of each type of chip are there?
#define GAB_NUM_LVDS_CHIPS 4

//
// GAB chips: 
//

//NOTE:  even though GAB hardware starts counting at one,
//the vmescl driver brings it in line with TAB convention!
#define GAB_CHIP_LVDS1    0
#define GAB_CHIP_LVDS2    1
#define GAB_CHIP_LVDS3    2
#define GAB_CHIP_LVDS4    3
#define GAB_CHIP_S30      4
#define GAB_CHIP_CYCLONE  15

//...

#define LVDS_GROUP_TEST_DATA_0   11
#define LVDS_GROUP_TEST_DATA_1   12
#define LVDS_GROUP_RECV_DATA_0   13
#define LVDS_GROUP_RECV_DATA_1   14

#define LVDS_DATA_MAX_ADDR  48

#define LVDS_STATUS_PULSE          0x0001
#define LVDS_STATUS_NL             0x0002
#define LVDS_STATUS_PERR0          0x0004
#define LVDS_STATUS_PERR1          0x0008
#define LVDS_STATUS_INST_PERR0     0x0010
#define LVDS_STATUS_INST_PERR1     0x0020
#define LVDS_STATUS_SYNC_ERR       0x0040
#define LVDS_STATUS_BC_ERR         0x0080
#define LVDS_STATUS_VALID_ERR      0x0100
#define LVDS_STATUS_SCLBC_ERR      0x0200
#define LVDS_STATUS_SCLTURN_ERR    0x0400
#define LVDS_STATUS_PLL_LOCK_ERR   0x0800

/* for the large GAB s30 chip  */

/* The group values for chips 0-9 in the TABs are also included
 *    in the GAB s30 address map unless their values are redefined
 *    below. Of course, not all the things in the address map do
 *    anything. */

#define S30_GROUP_TEST_DATA0            8
#define S30_GROUP_TEST_DATA1            9
#define S30_GROUP_TEST_DATA2           10
#define S30_GROUP_TEST_DATA3           11
#define S30_GROUP_TEST_DATA4           12
#define S30_GROUP_TEST_DATA5           13
#define S30_GROUP_TEST_DATA6           14
#define S30_GROUP_TEST_DATA7           15

//from GABS30 firmware:
//wren(1) => resetpll,
//wren(2) => wrpar,
//wren(3) => rdstat,
//wren(4) => rdpar,
//wren(5) => rdtab,
//wren(6) => rdtrg,
//wren(7) => rdl2);

/*The group value for loading or reading the gab s30 parameters */
#define S30_GROUP_PARAM_LOAD 2
#define S30_GROUP_PARAM_READ 4

/*The group value for reading the gab s30 status register */
#define S30_GROUP_STATUS_READ    3
/*The group value for reading the tab inputs from the gab's s30 chip to vme*/
#define S30_GROUP_RECV_MEM_READ  5
/*The group value for reading the trigger data from the gab's s30 chip to vme*/
#define S30_GROUP_TRIG_MEM_READ  6
/*The group value for reading the level 2 data from the gab's s30 chip to vme*/
#define S30_GROUP_L2_MEM_READ    7

#define S30_STATUS_INIT       0x0001
#define S30_STATUS_SCL_RUN    0x0002
#define S30_STATUS_LOCKED     0x0004
#define S30_STATUS_SCL_ERROR  0x0008
#define S30_STATUS_SCL_SPARE  0x0010

//these defines made from firmware source with "make_defines.pl":
#define S30_GROUP_PARAM_ADDR_TH_EMI           0x03 // BIT 0...14
#define S30_GROUP_PARAM_ADDR_TH_JR1           0x04 // BIT 0...8
#define S30_GROUP_PARAM_ADDR_TH_JR2           0x05 // BIT 0...8
#define S30_GROUP_PARAM_ADDR_TH_EM1           0x06 // BIT 0...8
#define S30_GROUP_PARAM_ADDR_TH_EM2           0x07 // BIT 0...8
#define S30_GROUP_PARAM_ADDR_TH_JT1           0x08 // BIT 0...8
#define S30_GROUP_PARAM_ADDR_TH_JT2           0x09 // BIT 0...8
#define S30_GROUP_PARAM_ADDR_TH_TA            0x0a // BIT 0...8
#define S30_GROUP_PARAM_ADDR_TH_ET            0x0b // BIT 0...14
#define S30_GROUP_PARAM_ADDR_THB_COP1         0x0c // BIT 0...8
#define S30_GROUP_PARAM_ADDR_THB_COP2         0x0d // BIT 0...8
#define S30_GROUP_PARAM_ADDR_THB_COP3         0x0e // BIT 0...8
#define S30_GROUP_PARAM_ADDR_THM_COP1         0x0f // BIT 0...8
#define S30_GROUP_PARAM_ADDR_THM_COP2         0x10 // BIT 0...8
#define S30_GROUP_PARAM_ADDR_TH_ME0           0x11 // BIT 0...14
#define S30_GROUP_PARAM_ADDR_TH_ME1           0x12 // BIT 0...14
#define S30_GROUP_PARAM_ADDR_TH_ME2           0x13 // BIT 0...14
#define S30_GROUP_PARAM_ADDR_TH_ME3           0x14 // BIT 0...14
#define S30_GROUP_PARAM_ADDR_CONTIG           0x15 // BIT 0...5
#define S30_GROUP_PARAM_ADDR_SPECIALNUM       0x16 // BIT 0...7
#define S30_GROUP_PARAM_ADDR_NOISE0           0x17 // BIT 0...7
#define S30_GROUP_PARAM_ADDR_NOISE1           0x18 // BIT 0...7
#define S30_GROUP_PARAM_ADDR_NOISE2           0x19 // BIT 0...7

#define S30_GROUP_PARAM_ADDR_ACCEPT_DELAY     0x2f // BIT 0...8
#define S30_GROUP_PARAM_ADDR_HEADERSIZE       0x30 // BIT 0...15
#define S30_GROUP_PARAM_ADDR_HEADERTYPE       0x31 // BIT 0...7
#define S30_GROUP_PARAM_ADDR_HEADERFORMAT     0x32 // BIT 0...15
#define S30_GROUP_PARAM_ADDR_HEADERVERSION    0x33 // BIT 0...15
#define S30_GROUP_PARAM_ADDR_HEADERBITS       0x34 // BIT 0...7
#define S30_GROUP_PARAM_ADDR_SCL_DELAY        0x35 // BIT 0...8
#define S30_GROUP_PARAM_ADDR_L2DELAY          0x36 // BIT 0...4
#define S30_GROUP_PARAM_ADDR_MAXERROR         0x37 // BIT 0...7

// GAB ALGORITHM DEFINITIONS:

const int ALL_DETECTOR_REGIONS = 3;  //south, central, and north

const int SOUTH_REGION = 0;
const int CENTRAL_REGION = 1;
const int NORTH_REGION = 2;

// GAB S30 Trigger terms

#define CSWEM_NO 1            // for Trig_Algo_Params & Trig_Thresh_Params
#define CSWJT_NO 2            // for Trig_Algo_Params & Trig_Thresh_Params
#define CSWTA_NO 3
#define CSWEI_NO 4
#define CSWAJT_NO 5
#define CSWBBEM_NO 6
#define CSWMET_NO 7
#define CSWTET_NO 8
#define CSWJFR_NO 9

const int TRIGGER_NUM = 13;       // number of triggers

#define TAB_NUM 8

#endif
