#include "tabgab/kparams.h"
#include <fstream>
#include <stdlib.h>

using namespace std;

void kparams::parse_file(const std::string & file){
   _initialize = 1;
   string name;
   string type;
   string value_string;
    
   //open the specified param file, but
   //precede with KPARAMS_DIR environment variable
   //if it is defined.
   string input_file;   
   char * kparams_dir = getenv("KPARAMS_DIR");
   if (kparams_dir){
      input_file += kparams_dir;
      input_file += "/";
   }
   input_file += file;   
   ifstream input(input_file.c_str());   
   
   printf("opening parameter file: %s\n",input_file.c_str());
   while(input >> name){
       if (name[0]=='#') {
	   char c;
	   // comment lines last until the end of the line:
	   do { input.get(c); } while (c!='\n');
       } else {		         
	   input >> type;
	   streampos saved_position = input.tellg();
	   input >> value_string;
      
	   //change value_string to value in environment if preceded by $:
	   if (value_string[0] == '$'){
	       value_string = value_string.substr(1,value_string.size());
	       printf("using environment variable: %s\n", value_string.c_str());	      
	       char * env = getenv(value_string.c_str());
	       if (env == NULL){
		   printf("ERROR: environment variable %s is undefined!\n", value_string.c_str());
		   value_string = "0";
	       } else {
		   value_string = env;
	       }
	   } 
	   //printf("value in string form is %s\n",value_string.c_str());
	   
	   // now do a conversion based on type:
	   if (type == "int"){
	       int value = strtol(value_string.c_str(),(char**) NULL, 0);
	       get_int(name,value);
	   }
	   else if (type == "unsigned"){
	       unsigned value = strtoul(value_string.c_str(),(char**) NULL, 0);
	       get_unsigned(name,value);
	   }      
	   else if (type == "double"){
	       double value = strtod(value_string.c_str(),(char**) NULL);
	       get_double(name,value);
	   }
	   else if (type == "string"){
	       string value = value_string;
	       
	       //if string is quoted, extract it more carefully!
	       if (value[0] == '"'){
		   char c;
		   input.seekg(saved_position);
		   value = "";
		   //throw out first character (it's a quote)
		   input >> c;
		   
		   input.get(c);
		   while (c != '"'){
		       value += c;
		       input.get(c);
		   }
	       }
	       get_string(name, value);
	   } 
	   else{
	       printf("ERROR: unknown type %s!\n", type.c_str());
	   }
       }
   }
   _initialize = 0;
}
    

    
int kparams::get_int (const string & name, int & value){
   if (_initialize) {
      //printf("adding parameter %s: %d\n",name.c_str(),value);
      return var_list_int.set(name,value);   
   } 
   
   return var_list_int.get(name,value);
}

int kparams::get_unsigned (const string & name, unsigned & value){
   if (_initialize) {
      //printf("adding parameter %s: %u(0x%x)\n",name.c_str(),value,value);
      return var_list_unsigned.set(name,value);   
   } 
   
   return var_list_unsigned.get(name,value);
}

int kparams::get_double (const string & name, double & value){
   if (_initialize) {
      //printf("adding parameter %s: %lf\n",name.c_str(), value);
      return var_list_double.set(name,value);   
   } 
   
   return var_list_double.get(name,value);
}

int kparams::get_string   (const string & name, string & value){
   if (_initialize) {
      //printf("adding parameter %s: %s\n",name.c_str(), value.c_str());
      return var_list_string.set(name,value);   
   } 
   
   return var_list_string.get(name,value);
}


