//kparams.h
//
//Author:  Michael Mulhearn
//
// Simple parameter parsing utility.
// 
// The k in kparams stands for "Keep it simple, stupid!"
// 
// A text file contains run time parameters formatted like:
// 
// # this is a comment
// myint1    int       10
// myint2    int       07 # this is another comment
// myint3    int       0xFF
// myint4    int       $MYINT2
// mymask1   unsigned  0xFFFF
// mymask2   unsigned  12
// mystring1 string    test.dat
// mystring2 string    "hello world"
//
// Notice that environment variables, hexidecimal format, octal format and quoted strings are all
// allowed.
// 
// Comments begin with the '#' character and are allowed at the begining of
// a line or after a valid line, and continue to the end of the line.
// 
// To access the parameters at run time, just add the following to your program:
// 
// //parse the file: 
// kparams myparams;
// myparams.parse_file("myparamfile.txt");
// 
// //access the variables:
// int myint2;
// myparams.get_int("myint2",myint2);
//
#ifndef __KPARAMS_H__
#define __KPARAMS_H__

#include <string>

#include "kparams_variable_list.h"

class kparams{   
 public:     
   //parse a parameter file:
   void parse_file  (const std::string & file);

   //returns non-zero for error:
   int get_int      (const std::string & name, int         & value);
   int get_unsigned (const std::string & name, unsigned    & value);
   int get_double   (const std::string & name, double      & value);
   int get_string   (const std::string & name, std::string & value);   

 //END OF USER INTERFACE!!!
   
 private:
   int _initialize;
   kparams_variable_list <int>           var_list_int;
   kparams_variable_list <unsigned>      var_list_unsigned;
   kparams_variable_list <double>        var_list_double;
   kparams_variable_list <std::string>   var_list_string;

};

#endif
