#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <unistd.h>
#include "tabgab/util.h"
#include "tabgab/vme.h"
#include "tabgab/vmescl.h"
#include "tabgab/tab.h"
#include "tabgab/gab.h"

int s30_recv_mem_read  (unsigned long addr, unsigned long *value){    
    return vmescl_read_serial(MOD_GAB, GAB_CHIP_S30, S30_GROUP_RECV_MEM_READ, addr, 0, value);
}

int s30_trig_mem_read  (unsigned long addr, unsigned long *value){
    return vmescl_read_serial(MOD_GAB, GAB_CHIP_S30, S30_GROUP_TRIG_MEM_READ, addr, 0, value);
}

int s30_l2_mem_read    (unsigned long addr, unsigned long *value){
    return vmescl_read_serial(MOD_GAB, GAB_CHIP_S30, S30_GROUP_L2_MEM_READ, addr, 0, value);
}

int s30_load_parameter  (unsigned long addr, unsigned long value){
    return vmescl_write_serial(MOD_GAB, GAB_CHIP_S30, S30_GROUP_PARAM_LOAD, addr, value);    
}

int s30_read_parameter  (unsigned long addr, unsigned long * value){
    return vmescl_read_serial(MOD_GAB, GAB_CHIP_S30, S30_GROUP_PARAM_READ, addr, 0, value);
}

int s30_load_parameter_thresh (unsigned long addr, int num, unsigned long value){
    long unsigned read_value;
    long unsigned mask;

    //printf("setting parameter %d thresh %d to %d\n",addr,num,value);
    
    value = (value << 3*num);    
    mask  = (0x7   << 3*num);
    value &= mask;
    //printf(" --> mask for field:   %04x\n", mask);        
    //printf(" --> new value:        %04x\n", value);        
    TRY( s30_read_parameter(addr, &read_value) );
    //printf(" --> previous value:   %04x\n", read_value);    
    read_value &= ~mask;
    //printf(" --> cleared value:    %04x\n", read_value);    
    value = value | read_value;
    //printf(" --> now setting addr %x to %x\n", addr, value);
    return s30_load_parameter(addr, value);    
}


//this will be obsolete soon:
int s30_load_shared_parameter  (unsigned long addr, int num, unsigned long value){
    long unsigned read_value;
    long unsigned mask;

    printf("setting shared parameter %d number %d to %x",(int)addr,num,(int)value);
    
    int offset = 0;
    if (num > 2){
	num = num -3;
	offset = 1;
    }
    addr += offset;
    
    printf(" --> using addr %d, field %d\n",(int)addr,num);
    
    value = (value << 3*num);    
    mask  = (0x7   << 3*num);
    value &= mask;
    printf(" --> mask for field:   %04lx\n", mask);        
    printf(" --> new value:        %04lx\n", value);        
    TRY (s30_read_parameter(addr, &read_value));
    printf(" --> previous value:   %04lx\n", read_value);    
    read_value &= ~mask;
    printf(" --> cleared value:    %04lx\n", read_value);    
    value = value | read_value;
    printf(" --> now setting addr %lx to %lx\n", addr, value);
    return s30_load_parameter(addr, value);    
}


void s30_print_n_thresh(int num, unsigned long data){
    for (int i=0; i<num; i++){
	unsigned thresh = (data>>(3*i)) & 0x7;
	printf("%d ", thresh);
    }
}


int s30_print_alg_parameters (){
    long unsigned parameter[30];
    unsigned addr, tmp1;
    
    for (unsigned i=3; i<26; i++){
	addr = i;
	TRY (s30_read_parameter(addr, &parameter[i]));
    }

    tmp1 = parameter[0] & 0x7fff;

    printf("cei thresh:  ");
    s30_print_n_thresh(5, parameter[3]);
    printf("\n");

    printf("cjr thresh:  ");
    s30_print_n_thresh(3, parameter[4]);
    s30_print_n_thresh(3, parameter[5]);
    printf("\n");

    printf("cem thresh:  ");
    s30_print_n_thresh(3, parameter[6]);
    s30_print_n_thresh(3, parameter[7]);
    printf("\n");

    printf("cjt thresh:  ");
    s30_print_n_thresh(3, parameter[8]);
    s30_print_n_thresh(3, parameter[9]);
    printf("\n");

    printf("cta thresh:  ");
    s30_print_n_thresh(3, parameter[10]);
    printf("\n");

    printf("cet thresh:  0x%lx\n", parameter[11]);
    
    printf("cop thresh:  ");
    s30_print_n_thresh(3, parameter[12]);
    s30_print_n_thresh(3, parameter[13]);
    s30_print_n_thresh(3, parameter[14]);
    s30_print_n_thresh(3, parameter[15]);
    s30_print_n_thresh(3, parameter[16]);
    s30_print_n_thresh(3, parameter[21]);
    printf("\n");
    
    printf("met thresh0:  0x%lx\n", parameter[17]);
    printf("met thresh1:  0x%lx\n", parameter[18]);
    printf("met thresh2:  0x%lx\n", parameter[19]);
    printf("met thresh3:  0x%lx\n", parameter[20]);
        
    tmp1 = parameter[22] & 0x7ff;	
    printf("SPECIALNUM: %02x (%d)\n",tmp1,tmp1); // BIT 0...7    
    tmp1 = parameter[23] & 0x7ff;	
    printf("NOISE TH0:  %02x (%d)\n",tmp1,tmp1); // BIT 0...10
    tmp1 = parameter[24] & 0x7ff;	
    printf("NOISE TH1:  %02x (%d)\n",tmp1,tmp1); // BIT 0...10
    tmp1 = parameter[25] & 0x7ff;	
    printf("NOISE TH2:  %02x (%d)\n",tmp1,tmp1); // BIT 0...10

    return EXIT_SUCCESS;
}

//other parameters for later...
//printf("_ACCEPT_DELAY     ");//2f // BIT 0...8
//printf("_HEADERSIZE       ");//30 // BIT 0...15
//printf("_HEADERTYPE       ");//31 // BIT 0...7
//printf("_HEADERFORMAT     ");//32 // BIT 0...15
//printf("_HEADERVERSION    ");//33 // BIT 0...15
//printf("_HEADERBITS       ");//34 // BIT 0...7
//printf("_SCL_DELAY        ");//35 // BIT 0...8
//printf("_L2DELAY          ");//36 // BIT 0...4
//printf("_MAXERROR         ");//37 // BIT 0...7

int s30_setup_level2_header_firmware_version(int major, int minor){
    unsigned long firmware_version = ((major&0xff)<<8) | (minor&0xff);    
    return s30_load_parameter(51, firmware_version);    
}

int s30_setup_level2_header(){
    TRY (s30_load_parameter(48, 0x0301) ); //hlength,         objects
    TRY (s30_load_parameter(49, 200)    ); //                 l2dtype
    TRY (s30_load_parameter(50, 0x2108) ); //format,          olenghth
    TRY (s30_load_parameter(52, 0x0003) ); //                 procbit    
    return EXIT_SUCCESS;
}
