#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <unistd.h>
#include "tabgab/util.h"
#include "tabgab/vme.h"
#include "tabgab/vmescl.h"
#include "tabgab/tab.h"

// parity count
int swa_parity_count_read   (unsigned long module, unsigned long chip, 
			     unsigned long *count0, unsigned long *count1, unsigned long *count2){
    TRY( check_tab             (module)       );
    TRY( check_module_and_chip (module,chip)  );
    
    TRY( vmescl_read_serial(module, chip, SWA_GROUP_PARITY_COUNT, SWA_GROUP_PARITY_COUNT_SERADR_CBL0, 0, count0) );
    TRY( vmescl_read_serial(module, chip, SWA_GROUP_PARITY_COUNT, SWA_GROUP_PARITY_COUNT_SERADR_CBL1, 0, count1) );
    TRY( vmescl_read_serial(module, chip, SWA_GROUP_PARITY_COUNT, SWA_GROUP_PARITY_COUNT_SERADR_CBL2, 0, count2) );
    return EXIT_SUCCESS;
}

//
// Test Memories:
//

int swa_set_test_energy(unsigned long module, unsigned long chip, 
			     unsigned long event, unsigned long tower, unsigned long value)
{
    unsigned long temp;
    TRY(check_tab_or_gab      (module)                                          );
    TRY(check_module_and_chip (module, chip)                                    );
    TRY(check_range       (event, "event number", TAB_MIN_EVENT, TAB_MAX_EVENT) );
    TRY(check_range       (tower, "tower",        SWA_MIN_TOWER, SWA_MAX_TOWER) );
    
    temp = SWA_MASK_ENERGY | (event << 6) | tower;
    return vmescl_write_serial(module, chip, SWA_GROUP_LOAD_TEST_MEM, temp, value);
}

int swa_set_test_control(unsigned long module, unsigned long chip, 
			      unsigned long event, unsigned long adr, unsigned long value)
{
    unsigned long temp;
    TRY(check_tab_or_gab  (module)                                      );
    TRY(check_module_and_chip        (module,chip)                                        );
    TRY(check_range       (event, "event number", TAB_MIN_EVENT, TAB_MAX_EVENT) );
    TRY(check_range       (adr,   "control adr",  0,         5        ) );
    
    temp = SWA_MASK_CONTROL | (event << 6) | adr;
    return vmescl_write_serial(module, chip, SWA_GROUP_LOAD_TEST_MEM, temp, value);
}


int swa_test_mem_clear(unsigned long module)
{    
    TRY(check_tab(module));
   
    unsigned long chip, event, tower, adrr;

    for(chip = 0; chip < 10; chip++)
    {
	for(event = TAB_MIN_EVENT; event <= TAB_MAX_EVENT; event++) 
	{
	    for(tower = 0; tower < 36; tower++) 
	    {
		TRY (swa_set_test_energy(module, chip, event, tower, 0));
	    }
	    for(adrr = 0; adrr < 6; adrr++) 
	    {
		TRY (swa_set_test_control(module, chip, event, adrr, 0));
	    }
	}
    }
    return EXIT_SUCCESS;
}

int swa_test_mem_setup_control(unsigned long module)
{    
    TRY(check_tab(module));

    unsigned long bc     = 0;
    unsigned long spare  = 0;
    unsigned long frame  = 0x1;
    
    for (int event =0; event <32; event++){
	for (int chip  =0; chip<10; chip++){
	    unsigned long bc     = 0;
	    unsigned long spare  = 0;
	    unsigned long frame  = 0x1;
	    unsigned long parity = 0; // it isn't useful to calculate parity,
	                              // since only ADF data parity is checked!
	    unsigned long value = (spare << 8) | bc;
	    TRY (swa_set_test_control(module, chip, event, 0, value));
	    TRY (swa_set_test_control(module, chip, event, 2, value));
	    TRY (swa_set_test_control(module, chip, event, 4, value));
	    value = (parity << 8) | frame;
	    TRY (swa_set_test_control(module, chip, event, 1, value));
	    TRY (swa_set_test_control(module, chip, event, 3, value));
	    TRY (swa_set_test_control(module, chip, event, 5, value));
	}
    }
    return EXIT_SUCCESS;
}

#define STRINGLEN 200

int swa_test_mem_load(unsigned long module, char *filename) 
{
    /* This function loads a tab's stratixes' test memories
     * based on the directions in the input file.
     * NOTE: For flexibility, this no longer clears the test 
     * memories first; a seperate function above is provided 
     * for that. 
     */

    FILE *fp;
    char s[STRINGLEN];
    unsigned long value;
    unsigned long chip, ctrl, event, adr, highdat, lowdat;
    int n;
    
    if((fp = fopen(filename,"r")) == NULL) {
	error("Cannot open %s", filename);
	return EXIT_FAILURE;
    }
        
    while(fgets(s, STRINGLEN, fp)) {
	n = sscanf(s,"%lx %lx %lx %lx %lx %lx",
		   &chip, &ctrl, &event, &adr, &highdat, &lowdat);	
	if (n == 6) {
	    value = (highdat << 8) | lowdat;
	    if(ctrl == 0) 
	    {
		TRY (swa_set_test_energy (module, chip, event, adr, value)); 					      
	    } else {
		//an ugly hack for getting the BC right in testmem for
		//new BC sync scheme, using old testmem format:
		if ((adr == 0) || (adr == 2) || (adr == 4)){
		    value = (highdat << 8) | ((lowdat-1)&0xff);
		}		
		TRY (swa_set_test_control(module, chip, event, adr, value)); 
	    }
	}	
    }
    
    fclose(fp);
    return EXIT_SUCCESS;
}

//
// parameter loading/reading:
//
int swa_load_param_em(unsigned long module, unsigned long chip, 
			  unsigned long threshold, unsigned long value)
{
    TRY( check_tab_or_gab (module)                                                );
    TRY( check_module_and_chip       (module,chip)                                );
    
    return vmescl_write_serial(module, chip, SWA_GROUP_LOAD_EM_PAR, threshold, value);
}

int swa_load_param_jet (unsigned long module, unsigned long chip, 
			    unsigned long threshold, unsigned long value)
{
    TRY( check_tab_or_gab (module)                                                    );
    TRY( check_module_and_chip       (module,chip)                                    );

    return vmescl_write_serial(module, chip, SWA_GROUP_LOAD_JET_PAR, threshold, value);
}

int swa_set_delay_par (unsigned long module, unsigned long chip, 
			      unsigned long delay,  unsigned long value)
{
    TRY( check_tab_or_gab (module)                                                    );
    TRY( check_module_and_chip       (module,chip)                                    );

    return vmescl_write_serial(module, chip, SWA_GROUP_DELAY, delay, value);
}

int swa_set_enable (unsigned long module, unsigned long chip,  unsigned long value)
{
    TRY( check_tab_or_gab (module)                                                    );
    TRY( check_module_and_chip       (module,chip)                                    );

    return vmescl_write_serial(module, chip, SWA_GROUP_DELAY, SWA_GROUP_DELAY_SERADR_ENABLE, value);
}

int swa_set_minimum_energy (unsigned long module, unsigned long chip,  unsigned long value)
{
    TRY( check_tab_or_gab (module)                                                    );
    TRY( check_module_and_chip       (module,chip)                                    );

    return vmescl_write_serial(module, chip, SWA_GROUP_DELAY, SWA_GROUP_DELAY_SERADR_MIN_ET, value);
}

int swa_set_eta_38 (unsigned long module, unsigned long chip,  unsigned long value)
{
    TRY( check_tab(module)                                                           );
    TRY( check_module_and_chip       (module,chip)                                    );

    return vmescl_write_serial(module, chip, SWA_GROUP_DELAY, SWA_GROUP_DELAY_SERADR_ETA_38, value);
}

int swa_set_noise_thresh    (unsigned long module, unsigned long chip, 
			     unsigned long thresh1, 
			     unsigned long thresh2, 
			     unsigned long thresh3){
    TRY( check_tab(module)                          );
    TRY( check_module_and_chip       (module,chip)  );
    
    TRY( vmescl_write_serial(module, chip, SWA_GROUP_DELAY, SWA_GROUP_DELAY_SERADR_NOISE1, thresh1) );
    TRY( vmescl_write_serial(module, chip, SWA_GROUP_DELAY, SWA_GROUP_DELAY_SERADR_NOISE2, thresh2) );
    TRY( vmescl_write_serial(module, chip, SWA_GROUP_DELAY, SWA_GROUP_DELAY_SERADR_NOISE3, thresh3) );

    return EXIT_SUCCESS;
}

int swa_alg_mem_read(unsigned long module, unsigned long chip,
			    unsigned long event, unsigned long addr, unsigned long *value)
{
    unsigned long event_addr;
    TRY( check_tab_or_gab (module)                                                );
    TRY( check_module_and_chip       (module,chip)                                );
    
    event_addr = (event << 5) | addr;
    return vmescl_read_serial(module, chip, SWA_GROUP_READ_ALG_MEM, event_addr, 0, value);
}

int swa_raw_mem_read(unsigned long module, unsigned long chip, 
				unsigned long event, unsigned long tower, unsigned long *value)
{
    unsigned long event_tower;
    TRY( check_tab_or_gab (module)                                                );
    TRY( check_module_and_chip       (module,chip)                                );
    
    event_tower = (event << 7) | tower;
    
    return vmescl_read_serial(module, chip, SWA_GROUP_READ_RAW_MEM, event_tower, 0, value);
}
