#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <unistd.h>
#include "tabgab/util.h"
#include "tabgab/vmescl.h"
#include "tabgab/tab.h"

//
// Range Checking:
//

int check_range(int value, const char * name, int min, int max)
{
    if(value < min || value > max)
    {
	error("%s must be between %d and %d", name, min, max);
	return EXIT_FAILURE;
    }
    return EXIT_SUCCESS;
}

int check_tab_or_gab      (int module){
    if (is_tab(module) || is_gab(module)){
	return EXIT_SUCCESS;
    }
    error("module %d is neither a TAB or GAB!",module);
    return EXIT_FAILURE;
}

int check_tab             (int module){
    if (is_tab(module)){
	return EXIT_SUCCESS;
    }
    error("module %d is not a TAB!", module);
    return EXIT_FAILURE;    
}

int check_gab             (int module){
    if (is_gab(module)){
	return EXIT_SUCCESS;
    }
    error("module %d is not a GAB!", module);
    return EXIT_FAILURE;
}

int check_module_and_chip (int module, int chip){
    if ( is_regular_stratix(module,chip) || is_global_stratix(module,chip)){
    	return EXIT_SUCCESS;
    }
    error("module %d chip %d is not valid!", module,chip);
    return EXIT_FAILURE;    
}

int check_regular_stratix (int module, int chip){
    if (is_regular_stratix(module,chip)){
	return EXIT_SUCCESS;
    }
    error("module %d, chip %d is not a regular stratix!", module, chip);
    return EXIT_FAILURE;
}

int check_global_stratix  (int module, int chip){
    if (is_global_stratix(module,chip)){
	return EXIT_SUCCESS;
    }
    error("module %d, chip %d is not a global stratix!", module, chip);
    return EXIT_FAILURE;

}

//
//just info, no error logged:
//
int is_tab(int module){
    if(module < MOD_TAB0 || module > MOD_TAB7) 
    {
	return 0;
    }        
    return 1;    
}

int is_gab(int module){
    if(module != MOD_GAB) 
    {
	return 0;
    }        
    return 1;    
}

int is_regular_stratix(int module, int chip){
    if (is_tab(module)){
	if ((chip < 0)||(chip>=10)) {
	    return 0;
	}
    } else if (is_gab(module)){
	if ((chip<0)||(chip>3)){
	    return 0;
	}
    } else {
	return 0;
    }
    return 1;
}

int is_global_stratix     (int module, int chip){
    if (is_tab(module)){
	if (chip!=10) {
	    return 0;
	}
    } 
    else if (is_gab(module)){
	if (chip!=4){
	    return 0;
	}
    } else {
	return 0;
    }
    return 1;
}
