//
// tab.h  driver for TAB card and shared TAB/GAB functionality
//

#ifndef TAB_H
#define TAB_H 1

#include "tab_defs.h"

//range checking:
//   - send an error if not.
//   - returns non-zero for failure.
int check_range(int value, const char * name, int min, int max);
int check_tab_or_gab      (int module);
int check_tab             (int module);
int check_gab             (int module);
int check_module_and_chip (int module, int chip);
int check_regular_stratix (int module, int chip);
int check_global_stratix  (int module, int chip);
//just info, no error:
int is_tab                (int module);
int is_gab                (int module);
int is_regular_stratix    (int module, int chip);
int is_global_stratix     (int module, int chip);

//
// cyclone chips on both TAB and GAB:
//

// read/interpret the cyclone status register:
int cyclone_status_read           (unsigned long module, unsigned long *data);
int cyclone_status_print          (unsigned long data);

//turn on the bits specified in the mask (for power, use cyclone_power_up_stratix)
int cyclone_status_set            (unsigned long module, unsigned long mask);

//turn off the bits specified in the mask (for power, use cyclone_power_down_stratix)
int cyclone_status_clear          (unsigned long module, unsigned long mask);

//control stratix power:
int cyclone_power_up_stratix      (unsigned long module);
int cyclone_power_down_stratix    (unsigned long module);

//write to the cyclone configuration register:
int cyclone_write_conf_byte       (unsigned long module, char type, unsigned long confbyte); 
int cyclone_write_conf_half_word  (unsigned long module, char type, unsigned long confhw);

//
// stratix chips on both TAB and GAB:
//

// read/write/interpret the status of a stratix chip
int stratix_status_read     (unsigned long module, unsigned long chip, unsigned long *data);
int stratix_status_write    (unsigned long module, unsigned long chip, unsigned long data);
int stratix_status_print    (unsigned long module, unsigned long chip, unsigned long data);

//configure all stratix chips in a TAB or GAB:
int stratix_configure       ( unsigned long module, const char * dir, const char * pkg_version);

// reset all the stratixes:
int stratix_reset           (unsigned long module, unsigned long chipmask);

// set timing parameters:
int stratix_set_scl_delay     (unsigned long module, unsigned long chip, unsigned long value);
int stratix_set_raw_delay     (unsigned long module, unsigned long chip, unsigned long value);
int stratix_set_accept_delay  (unsigned long module, unsigned long value); //ONLY GAB S30 FOR NOW...

// set the input mode:
//   -> TAB:  0=ADF, 1=TEST MEMORY
//   -> GAB:  0=TAB  1=TEST MEMORY
int stratix_set_input_mode      (unsigned long module, unsigned long chip, int mode);

//
// TAB swa chips:
//
// set parameters:

//actually the "delay parameter" group now includes enables and min Et.
int swa_set_delay_par       (unsigned long module, unsigned long chip, 
			     unsigned long delay,  unsigned long value);

int swa_set_enable          (unsigned long module, unsigned long chip, 
			     unsigned long value);

int swa_set_minimum_energy  (unsigned long module, unsigned long chip, 
			     unsigned long value);

int swa_set_eta_38          (unsigned long module, unsigned long chip, 
			     unsigned long value);

int swa_set_noise_thresh    (unsigned long module, unsigned long chip, 
			     unsigned long thresh1, 
			     unsigned long thresh2, 
			     unsigned long thresh3);

// read the parity error count on a swa chip
int swa_parity_count_read   (unsigned long module, unsigned long chip, 
			     unsigned long *count0, unsigned long *count1, unsigned long *count2); 

// set test memory:
int swa_set_test_energy        (unsigned long module, unsigned long chip,
			        unsigned long event, unsigned long tower, unsigned long value);
int swa_set_test_control       (unsigned long module, unsigned long chip,
 			        unsigned long event, unsigned long adr, unsigned long value);
int swa_test_mem_clear         (unsigned long module);
int swa_test_mem_setup_control (unsigned long module);

// read diagnostic algorithm memory:
int swa_alg_mem_read        (unsigned long module, unsigned long chip,
			     unsigned long event, unsigned long addr, unsigned long *value);
// read diagnostic raw memory:
int swa_raw_mem_read        (unsigned long module, unsigned long chip,
			     unsigned long event, unsigned long tower, unsigned long *value);

// algorithm parameter loading and reading:

int swa_load_param_em  (unsigned long module, unsigned long chip,
		        unsigned long threshold, unsigned long value);

int swa_load_param_jet (unsigned long module, unsigned long chip,
			unsigned long threshold, unsigned long value);

//
// TAB global chip:
//
//read diagnostic algorithm memory:
int global_alg_mem_read    (unsigned long module, unsigned long event, unsigned long channel, unsigned long *value); 
//read diagnostic raw memory:
int global_raw_mem_read    (unsigned long module, unsigned long address, unsigned long *value);
//read diagnostic track memory:
int global_track_mem_read  (unsigned long module, unsigned long line, unsigned long *value);
//write parameters
int global_delay_write     (unsigned long module, unsigned long delay);
int global_energy_write    (unsigned long module, unsigned long seradr, unsigned long value);
int global_raw_write       (unsigned long module, unsigned long seradr, unsigned long highvalue, unsigned long lowvalue);
int global_set_pars_from_file(unsigned long module, char *filename);

// setting up the l2 buffer header:
int global_setup_level2_header_firmware_version(int module, int major, int minor);
int global_setup_level2_header(int module);

//default setup of global energy params (thresh and sine/cosine).
int global_setup_energy_params(int module);

#endif
