/*-------------------------------------------------------------------------------*/
  /*
    tcc_to_tabgab_coor.h

    This header defines the routines that TCC uses to initialize, 
    control and monitor the TAB and GAB hardware.

    updated 02-Jun-2006 : to match http://www-d0.fnal.gov/~mverzocc/coor_resources_new.html
    updated 09-Aug-2006 : to match TAB/GAB firmware
  */

#ifndef __tcc_to_tabgab_coor__
#define __tcc_to_tabgab_coor__
/*-------------------------------------------------------------------------------*/

#include "tabgab/tcc_to_tabgab.h"
      
/* ------------------------------------------------------------------------ */
/* ------------------------------------------------------------------------ */

/* --------------------------------------------*/
/* VMESCL Hardware Level Initialization */
/* --------------------------------------------*/

    EStatus Vmescl_Initialize         () ;

/*
    Return value:
        cf. EStatus definition

    Parameters:
        None
*/

/* -----------------------------------------*/
/* I.2.1  TAB Hardware Level Initialization */
/* -----------------------------------------*/
      
    EStatus Tab_Initialize            ( unsigned long    Tab_Module ) ;

/*
    Return value:
        cf. EStatus definition

    Parameters:
        Tab_Module
            Module ID of TAB card
            unsigned integer
            allowed values = 0:7
*/

/* -----------------------------------------*/
/* I.2.2  GAB Hardware Level Initialization */
/* -----------------------------------------*/
      
    EStatus Gab_Initialize            ( ) ;

/*
    Return value:
        cf. EStatus definition

    Parameters:
        None
*/


/* ------------------------------------------------------------------------ */
/* ------------------------------------------------------------------------ */

/* ---------------------------*/
/* III.1.a. Jet Reference Sets*/
/* ---------------------------*/

    EStatus Tab_Set_JetEtThresh       ( unsigned long    Tab_Module,
                                        unsigned long    Tab_Chip,
                                        unsigned long    Tab_Thresh_Num,
                                        unsigned long    Set_AdcCount_Thresh ) ;
/*
    Return value:
        cf. EStatus definition

    Parameters:
        Tab_Module
            Module ID of TAB card
            unsigned integer
            allowed values = 0:7
        Tab_Chip
            Chip ID on TAB Card
            unsigned integer
            allowed values = 0:9
        Tab_Thresh_Num
            Jet Threshold Number on TAB
            unsigned integer
            allowed values = 1:7
        Set_AdcCount_Thresh
            Energy Threshold Value for Jet Object
            in units of ADC counts
            unsigned integer
            allowed values = 0:4095

    Current limitations:
        None

*/


/* ---------------------------*/
/* III.1.b. EM  Reference Sets*/
/* ---------------------------*/

    EStatus Tab_Set_EmEtThresh        ( unsigned long    Tab_Module,
                                        unsigned long    Tab_Chip,
                                        unsigned long    Tab_Thresh_Num,
                                        unsigned long    Set_AdcCount_Thresh ) ;

/*
    Return value:
        cf. EStatus definition

    Parameters:
        Tab_Module
            Module ID of TAB card
            unsigned integer
            allowed values = 0:7
        Tab_Chip
            Chip ID on TAB Card
            unsigned integer
            allowed values = 0:9
        Tab_Thresh_Num
            EM Threshold Number on TAB
            unsigned integer
            allowed values = 1:7
        Set_AdcCount_Thresh
            Energy Threshold Value for EM Object
            in units of ADC counts
            unsigned integer
            allowed values = 0:4095

    Current limitations:
        None

*/

/* -------------------------------------------------------------------------*/
/* III.2.d. Using ICR contribution in Jet Sums and Missing Et/Global Et Sums*/
/* -------------------------------------------------------------------------*/

    EStatus Tab_Set_ICR_Enables       ( unsigned long    Tab_Module,
                                        unsigned long    Tab_Chip,
                                        unsigned long    Set_Jet_Enable,
					unsigned long    Set_Global_Sums_Enable);
/*
    Return value:
        cf. EStatus definition

    Parameters:
        Tab_Module
            Module ID of TAB card
            unsigned integer
            allowed values = 0:7

        Tab_Chip
            Chip ID on TAB Card
            unsigned integer
            allowed values = 0:9

        Set_Jet_Enable
            Specify whether to include ICR in Jet algorithm
            unsigned integer
            allowed values
                0 = Disabled
                1 = Enabled

        Set_Global_Sums_Enabled
            Specify whether to include ICR in Global Sums algorithm
            unsigned integer
            allowed values
                0 = Disabled
                1 = Enabled  
 
    Current limitations:
        None
*/


/* ---------------------------*/
/* III.2.a. EM Isolation      */
/* ---------------------------*/

    EStatus Tab_Set_Em_Isolation      ( unsigned long    Tab_Module,
                                        unsigned long    Tab_Chip,
                                        unsigned long    Set_EmIsol_Enable,
					unsigned long    Set_EmHd_Enable,
					unsigned long    Set_EmIsol_Ref_Set);
					
/*
    Return value:
        cf. EStatus definition

    Parameters:
        Tab_Module
            Module ID of TAB card
            unsigned integer
            allowed values = 0:7

        Tab_Chip
            Chip ID on TAB Card
            unsigned integer
            allowed values = 0:9

        Set_EmIsol_Enabled
            Specify whether EM Isolation cut is used
            unsigned integer
            allowed values
                0 = Disabled
                1 = Enabled

        Set_EmHd_Enabled
            Specify whether EM/HD fraction cut is used
            unsigned integer
            allowed values
                0 = Disabled
                1 = Enabled  
 
        Set_EmIsol_Ref_Set
            Specify minimum EM ref set for consideration as isolated EM object.
            unsigned integer
            allowed values = 0:7
 
    Current limitations:   
        Due to firmware limitations, Tab_Set_Masks_Ref_Sets must be called 
        *after* Tab_Set_Em_Isolation, or the value of EM_Mask_Ref_Set and
        EMIsol_Ref_Set will be undefined. 
 */

/* -------------------------------------*/
/* Set L2 Mask Reference Sets           */
/* -------------------------------------*/

    EStatus Tab_Set_Mask_Ref_Sets     ( unsigned long    Tab_Module,
                                        unsigned long    Tab_Chip,
					unsigned long    Set_Em_Mask_Ref_Set,
                                        unsigned long    Set_Jet_Mask_Ref_Set,
					unsigned long    Set_Tau_Mask_Ref_Set );
/*
    Return value:
        cf. EStatus definition

    Parameters:
        Tab_Module
            Module ID of TAB card
            unsigned integer
            allowed values = 0:7

        Tab_Chip
            Chip ID on TAB Card
            unsigned integer
            allowed values = 0:9

         Set_Em_Mask_Ref_Set
            Specify minimum EM ref set for inclusion in the L2 EM mask (now purely diagnostic).
            unsigned integer
            allowed values = 0:7 
 
        Set_Jet_Mask_Ref_Set
            Specify minimum Jet ref set for inclusion in the L2 EM mask (now purely diagnostic).
            unsigned integer
            allowed values = 0:7

        Set_Tau_Mask_Ref_Set
            Specify minimum Tau ref set for inclusion in the L2 EM mask (now purely diagnostic).
            unsigned integer
            allowed values = 0:7

    Current limitations:
        Due to firmware limitations, Tab_Set_Masks_Ref_Sets must be called 
        *after* Tab_Set_Em_Isolation, or the value of EM_Mask_Ref_Set and
        EMIsol_Ref_Setwill be undefined. 
*/

/* ---------------------------*/
/* III.2.a. EM Isolation      */
/* ---------------------------*/

    EStatus Tab_Set_EmIsolationEtCut  ( unsigned long    Tab_Module,
                                        unsigned long    Tab_Chip,
                                        unsigned long    Set_AdcCount_Thresh ) ;

/*
    Return value:
        cf. EStatus definition

    Parameters:
        Tab_Module
            Module ID of TAB card
            unsigned integer
            allowed values = 0:7
        Tab_Chip
            Chip ID on TAB Card
            unsigned integer
            allowed values = 0:9
        Set_AdcCount_Thresh
            Energy Threshold Value for Et Cut
            in units of ADC counts
            unsigned integer
            allowed values = 0:4095

    Current limitations:

*/

    EStatus Tab_Set_UseEmIsolation    ( unsigned long    Tab_Module,
                                        unsigned long    Tab_Chip,
                                        unsigned long    BoolUseEmIsolation ) ;

/*
    Return value:
        cf. EStatus definition

    Parameters:
        Tab_Module
            Module ID of TAB card
            unsigned integer
            allowed values = 0:7
        Tab_Chip
            Chip ID on TAB Card
            unsigned integer
            allowed values = 0:9
        BoolUseEmIsolation 
            0 = Ignore EM Isolation criterion
            1 = Use    EM Isolation criterion

    Current limitations:

*/

    EStatus Tab_Set_UseEmFraction     ( unsigned long    Tab_Module,
                                        unsigned long    Tab_Chip,
                                        unsigned long    BoolUseEmFraction ) ;

/*
    Return value:
        cf. EStatus definition

    Parameters:
        Tab_Module
            Module ID of TAB card
            unsigned integer
            allowed values = 0:7
        Tab_Chip
            Chip ID on TAB Card
            unsigned integer
            allowed values = 0:9
        BoolUseEmFraction 
            0 = Ignore EM Fraction criterion
            1 = Use    EM Fraction criterion

    Current limitations:

*/

/* ---------------------------------------*/
/* III.2.b. Trigger Tower Tau Et Core Cut */
/* ---------------------------------------*/

    EStatus Tab_Set_TauEtCoreCut      ( unsigned long    Tab_Module,
                                        unsigned long    Tab_Chip,
                                        unsigned long    Set_TauEtCoreCut ) ;

/*
    Return value:
        cf. EStatus definition

    Parameters:
        Tab_Module
            Module ID of TAB card
            unsigned integer
            allowed values = 0:7
        Tab_Chip
            Chip ID on TAB Card
            unsigned integer
            allowed values = 0:9
	Set_EtCore
            Minimum number of ADC counts above pedestal within a jet object core
              for consideration as a Tau object.
            unsigned integer
            allowed values = 0:4095
    Current limitations:

*/



/* ----------------------------------------------------------------------------------------------*/
/* III.2.c. Trigger Tower Minimum ET for inclusion in Global Sums (i.e. Missing Et and Global Et)*/
/* ----------------------------------------------------------------------------------------------*/

    EStatus Tab_Set_GlobalSumEtCut    ( unsigned long    Tab_Module,
                                        unsigned long    Tab_Chip,
                                        unsigned long    Set_AdcCount_Thresh ) ;

/*
    Return value:
        cf. EStatus definition

    Parameters:
        Tab_Module
            Module ID of TAB card
            unsigned integer
            allowed values = 0:7
        Tab_Chip
            Chip ID on TAB Card
            unsigned integer
            allowed values = 0:9
        Set_AdcCount_Thresh
            Energy Threshold Value to participate in Miss Et and Global Et
            in units of ADC counts
            unsigned integer
            allowed values = 0:255

    Current limitations:

*/

/* ------------------------------------*/
/* Set Eta 38                          */
/* ------------------------------------*/

    EStatus Tab_Set_Eta38 ( unsigned long    Tab_Module,
                            unsigned long    Tab_Chip,
                            unsigned long    Set_Eta38 );
/*
    Return value:
        cf. EStatus definition

    Parameters:
        Tab_Module
            Module ID of TAB card
            unsigned integer
            allowed values = 0:7
        Tab_Chip
            Chip ID on TAB Card
            unsigned integer
            allowed values = 0:9
        Set_Eta38  
            There is no physical instrumentation at eta=38, but a
            fictitious value is required so that the calculation of eta=35
            jet objects can include a three-eta overlap region.  Intended
            to be set to the pedestal.
            unsigned integer
            allowed values = 0:255
    Current limitations:
        Due to firmware limitations, this is presently set to zero (not pedestal as intended!)
*/

/* --------------------------------------------------------------------------*/
/* Set TT Thresh for TT Count (aka "noise trigger", "run 2a trigger")        */
/* --------------------------------------------------------------------------*/


    EStatus Tab_Set_TT_Count_TT_Thresh ( unsigned long    Tab_Module,
                                         unsigned long    Tab_Chip,
                                         unsigned long    Thresh_Num,
					 unsigned long    Set_TT_Thresh);
/*
    Return value:
        cf. EStatus definition

    Parameters:
        Tab_Module
            Module ID of TAB card
            unsigned integer
            allowed values = 0:7
        Tab_Chip
            Chip ID on TAB Card
            unsigned integer
            allowed values = 0:9
        Thresh_Num
            Specifies one of three independent TT counts.
            unsigned integer
            allowed values = 0:2
        Set_TT_Thresh
            Non-pedestal subtracted single tower EM+Hadron Ets greater than this threshold are counted.
            unsigned integer
            allowed values = 0:511
    Current limitations:
        None.
*/

/* ------------------------------------------------------------------------ */
/* ------------------------------------------------------------------------ */

/* --------------------------------------------------------------------------*/
/* Set Count Thresh for TT Count (aka "noise trigger", "run 2a trigger")        */
/* --------------------------------------------------------------------------*/

    EStatus Gab_Set_TT_Count_Thresh ( unsigned long    Thresh_Num,
				      unsigned long    Set_Count_Thresh);
/*
    Return value:
        cf. EStatus definition

    Parameters:
        Thresh_Num
            Specifies one of three independent TT counts.
            unsigned integer
            allowed values = 0:2
        Set_Count_Thresh
            If there are more than Set_Count_Thresh TTs above threshold,
            then the corresponding term will fire. 
            unsigned integer
            allowed values = 0:2047
 Current limitations:
        None.
*/

/* ------------------------------------------------------------------------ */
/* Note:  the following functions are safe to call, but                     */
/*        currently accomplish nothing!                                     */
/* ------------------------------------------------------------------------ */

/* ------------------------------------*/
/* IV.1.a. Jet Count over full coverage*/
/* ------------------------------------*/

    EStatus Gab_Set_JetAllTerm        ( unsigned long    Rel_Term_Num,
                                        unsigned long    Use_Thresh_Num,
                                        unsigned long    Set_ObjCount_Thresh ) ;


/*
    Return value:
        cf. EStatus definition

    Parameters:
        Rel_Term_Num
            Relative An/Or Term Number of this type sent by the GAB to L1 Framework
            unsigned integer
            allowed values = 0:15
        Use_Thresh_Num
            Jet Threshold Number (as seen by GAB)
                from which Jet Objects will be counted
            unsigned integer
            allowed values = 0:7
        Set_ObjCount_Thresh
            unsigned integer
            allowed values = 1:4

    Current limitations: does nothing
*/


/* ---------------------------------------------------*/
/* IV.1.b. Jet Count over Central Calorimeter coverage*/
/* ---------------------------------------------------*/

    EStatus Gab_Set_JetCentralTerm    ( unsigned long    Rel_Term_Num,
                                        unsigned long    Use_Thresh_Num,
                                        unsigned long    Set_ObjCount_Thresh ) ;


/*
    Return value:
        cf. EStatus definition

    Parameters:
        Rel_Term_Num
            Relative An/Or Term Number of this type sent by the GAB to L1 Framework
            unsigned integer
            allowed values = 0:3
        Use_Thresh_Num
            Jet Threshold Number (as seen by GAB)
                from which Jet Objects will be counted
            unsigned integer
            allowed values = 0:7
        Set_ObjCount_Thresh
            unsigned integer
            allowed values = 1:4

    Current limitations: does nothing
*/

/* ------------------------------------------------------------------------ */
/* ------------------------------------------------------------------------ */

/* -----------------------------------*/
/* IV.2.a. EM Count over full coverage*/
/* -----------------------------------*/

    EStatus Gab_Set_EmAllTerm         ( unsigned long    Rel_Term_Num,
                                        unsigned long    Use_Thresh_Num,
                                        unsigned long    Set_ObjCount_Thresh ) ;


/*
    Return value:
        cf. EStatus definition

    Parameters:
        Rel_Term_Num
            Relative An/Or Term Number of this type sent by the GAB to L1 Framework
            unsigned integer
            allowed values = 0:11
        Use_Thresh_Num
            EM Threshold Number (as seen by GAB)
                from which EM Objects will be counted
            unsigned integer
            allowed values = 0:7
        Set_ObjCount_Thresh
            unsigned integer
            allowed values = 1:4

    Current limitations: does nothing
*/


/* --------------------------------------------------*/
/* IV.2.b. EM Count over Central Calorimeter coverage*/
/* --------------------------------------------------*/

    EStatus Gab_Set_EmCentralTerm     ( unsigned long    Rel_Term_Num,
                                        unsigned long    Use_Thresh_Num,
                                        unsigned long    Set_ObjCount_Thresh ) ;

/*
    Return value:
        cf. EStatus definition

    Parameters:
        Rel_Term_Num
            Relative An/Or Term Number of this type sent by the GAB to L1 Framework
            unsigned integer
            allowed values = 0:3
        Use_Thresh_Num
            EM Threshold Number (as seen by GAB)
                from which EM Objects will be counted
            unsigned integer
            allowed values = 0:7
        Set_ObjCount_Thresh
            unsigned integer
            allowed values = 1:4

    Current limitations: does nothing
*/

/* ------------------------------------------------------------------------ */
/* ------------------------------------------------------------------------ */

/* ------------------------------------*/
/* IV.3.a. Tau Count over full coverage*/
/* ------------------------------------*/

    EStatus Gab_Set_TauAllTerm        ( unsigned long    Rel_Term_Num,
                                        unsigned long    Use_Thresh_Num,
                                        unsigned long    Use_Ratio_Num,
                                        unsigned long    Set_ObjCount_Thresh ) ;

    EStatus Gab_Set_TauCentralTerm    ( unsigned long    Rel_Term_Num,
                                        unsigned long    Use_Thresh_Num,
                                        unsigned long    Use_Ratio_Num,
                                        unsigned long    Set_ObjCount_Thresh ) ;
/*
    Return value:
        cf. EStatus definition

    Parameters:
        Rel_Term_Num
            Relative An/Or Term Number of this type sent by the GAB to L1 Framework
            unsigned integer
            allowed values = 0:3
        Use_Thresh_Num
            Jet Threshold Number (as seen by GAB)
                from which Tau Objects will be counted
            unsigned integer
            allowed values = 0:7
        Use_Ratio_Num
            Tau Ratio Number (as seen by GAB)
                that Tau Objects must satisfy
            unsigned integer
            allowed values = 0:7
        Set_ObjCount_Thresh
            unsigned integer
            allowed values = 1:2

    Current limitations: does nothing
*/

/* ------------------------------------------------------------------------ */
/* ------------------------------------------------------------------------ */

/* --------------------------------------------*/
/* IV.4.a. EM Isolated Count over full coverage*/
/* --------------------------------------------*/

    EStatus Gab_Set_IsolEmAllTerm     ( unsigned long    Rel_Term_Num,
                                        unsigned long    Use_Thresh_Num,
                                        unsigned long    Set_ObjCount_Thresh ) ;

/*
    Return value:
        cf. EStatus definition

    Parameters:
        Rel_Term_Num
            Relative An/Or Term Number of this type sent by the GAB to L1 Framework
            unsigned integer
            allowed values = 0:1
        Use_Thresh_Num
            EM Threshold Number (as seen by GAB)
                from which Isolated EM Objects will be counted
            unsigned integer
            allowed values = 0:7
        Set_ObjCount_Thresh
            unsigned integer
            allowed values = 1:2

    Current limitations: does nothing
*/


/* -----------------------------------------------------------*/
/* IV.4.b. EM Isolated Count over Central Calorimeter coverage*/
/* -----------------------------------------------------------*/

    EStatus Gab_Set_IsolEmCentralTerm ( unsigned long    Rel_Term_Num,
                                        unsigned long    Use_Thresh_Num,
                                        unsigned long    Set_ObjCount_Thresh ) ;

/*
    Return value:
        cf. EStatus definition

    Parameters:
        Rel_Term_Num
            Relative An/Or Term Number of this type sent by the GAB to L1 Framework
            unsigned integer
            allowed values = 0:1
        Use_Thresh_Num
            EM Threshold Number (as seen by GAB)
                from which Isolated EM Objects will be counted
            unsigned integer
            allowed values = 0:7
        Set_ObjCount_Thresh
            unsigned integer
            allowed values = 1:2

    Current limitations: does nothing
*/



/* ------------------------------------------------------------------------ */
/* ------------------------------------------------------------------------ */

/* ---------------------------*/
/* IV.5. Missing Et Energy Cut*/
/* ---------------------------*/

    EStatus Gab_Set_MissEtTerm        ( unsigned long    Rel_Term_Num,
                                        unsigned long    Set_AdcSquare_Thresh ) ;

/*
    Return value:
        cf. EStatus definition

    Parameters:
        Rel_Term_Num
            Relative An/Or Term Number of this type sent by the GAB to L1 Framework
            unsigned integer
            allowed values = 0:3
        Set_AdcSquare_Thresh
            unsigned integer
            allowed values = 0:8191

    Current limitations: does nothing
*/


/* ------------------------------------------------------------------------ */
/* ------------------------------------------------------------------------ */

/* -------------------------*/
/* IV.6. Total Et Energy Cut*/
/* -------------------------*/

    EStatus Gab_Set_TotEtTerm         ( unsigned long    Rel_Term_Num,
                                        unsigned long    Set_AdcCount_Thresh ) ;


/*
    Return value:
        cf. EStatus definition

    Parameters:
        Rel_Term_Num
            Relative An/Or Term Number of this type sent by the GAB to L1 Framework
            unsigned integer
            allowed values = 0:1
        Set_AdcCount_Thresh
            unsigned integer
            allowed values = 0:8191

    Current limitations: does nothing

*/



/* ------------------------------------------------------------------------ */
/* ------------------------------------------------------------------------ */

/* -------------------------------------------*/
/* IV.7. Topological Acoplanar Jet Requirement*/
/* -------------------------------------------*/

    EStatus Gab_Set_AcoplanarJetTerm  ( unsigned long    Rel_Term_Num,
                                        unsigned long    Use_Thresh_Num1,
                                        unsigned long    Use_Thresh_Num2,
                                        unsigned long    DeltaPhi ) ;

/*
    Return value:
        cf. EStatus definition

    Parameters:
        Rel_Term_Num
            Relative An/Or Term Number of this type sent by the GAB to L1 Framework
            unsigned integer
            allowed values = 0:1
        Use_Thresh_Num1
            Jet Threshold Number (as seen by GAB)
                from which a Primary jet Object will be picked
            unsigned integer
            allowed values = 0:7
        Use_Thresh_Num2
            Jet Threshold Number (as seen by GAB)
                from which the Secondary Jet Object(s) are compared
            unsigned integer
            allowed values = 0:7
        DeltaPhi
            Minimum Phi separation between the two jets compared 
            unsigned integer
            allowed values = 1:15 ???

    Current limitations: does nothing
*/


/* -----------------------------------*/
/* IV.8. Non collinear low Et Jet term*/
/* -----------------------------------*/

    EStatus Gab_Set_NonColLowEtJetTerm( unsigned long    Rel_Term_Num,
                                        unsigned long    Use_Thresh_Num1,
                                        unsigned long    Use_Thresh_Num2,
                                        unsigned long    DeltaPhi ) ;

/*
    Return value:
        cf. EStatus definition

    Parameters:
        Rel_Term_Num
            Relative An/Or Term Number of this type sent by the GAB to L1 Framework
            unsigned integer
            allowed values = 0:1
        Use_Thresh_Num1
            Jet Threshold Number (as seen by GAB)
                from which a Primary jet Object will be picked
            unsigned integer
            allowed values = 0:7
        Use_Thresh_Num2
            Jet Threshold Number (as seen by GAB)
                from which the Secondary Jet Object(s) are compared
            unsigned integer
            allowed values = 0:7
        DeltaPhi
            Maximum Phi separation between the two jets compared
            unsigned integer
            allowed values = 1:15 ???

    Current limitations: does nothing
*/


/* ---------------------------------*/
/* IV.9. Non collinear Mono Jet term*/
/* ---------------------------------*/

    EStatus Gab_Set_NonColMonoJetTerm ( unsigned long    Rel_Term_Num,
                                        unsigned long    Use_Thresh_Num1,
                                        unsigned long    Use_Thresh_Num2,
                                        unsigned long    DeltaPhi ) ;

/*
    Return value:
        cf. EStatus definition

    Parameters:
        Rel_Term_Num
            Relative An/Or Term Number of this type sent by the GAB to L1 Framework
            unsigned integer
            allowed values = 0:1
        Use_Thresh_Num1
            Jet Threshold Number (as seen by GAB)
                from which a Primary jet Object will be picked
            unsigned integer
            allowed values = 0:7
        Use_Thresh_Num2
            Jet Threshold Number (as seen by GAB)
                from which the Secondary Jet Object(s) are compared
            unsigned integer
            allowed values = 0:7
        DeltaPhi
            Maximum Phi separation between the two jets compared
            unsigned integer
            allowed values = 1:15 ???

    Current limitations: does nothing
*/


/* ----------------------------------------------*/
/* IV.10. Topological Back-to-Back EM Requirement*/
/* ----------------------------------------------*/

    EStatus Gab_Set_BackToBackEmTerm  ( unsigned long    Rel_Term_Num,
                                        unsigned long    Use_Thresh_Num,
                                        unsigned long    DeltaPhi ) ;

/*
    Return value:
        cf. EStatus definition

    Parameters:
        Rel_Term_Num
            Relative An/Or Term Number of this type sent by the GAB to L1 Framework
            unsigned integer
            allowed values = 0:1
        Use_Thresh_Num
            EM Threshold Number (as seen by GAB)
                from which at least one EM pair must be flagged as back to back
            unsigned integer
            allowed values = 0:7
        DeltaPhi
            Maximum Phi deviation from exact back to back
            unsigned integer
            allowed values = 1:15 ???

    Current limitations: does nothing
*/


/* ----------------------------------*/
/* IV.11. Topological Jet Free Region*/
/* ----------------------------------*/

    EStatus Gab_Set_JetFreeTerm       ( unsigned long    Rel_Term_Num,
                                        unsigned long    Use_Thresh_Num_a,
                                        unsigned long    Use_Thresh_Num_v,
                                        unsigned long    DeltaPhi ) ;

/*
    Return value:
        cf. EStatus definition

    Parameters:
        Rel_Term_Num
            Relative An/Or Term Number of this type sent by the GAB to L1 Framework
            unsigned integer
            allowed values = 0:1
        Use_Thresh_Num_a
            Jet Threshold Number (as seen by GAB)
                from which a Primary jet Object will be picked
            unsigned integer
            allowed values = 0:7
        Use_Thresh_Num_v
            Jet Threshold Number (as seen by GAB)
                from which the Secondary Jet Object(s) will be compared to
            unsigned integer
            allowed values = 0:7
        DeltaPhi
            Maximum Phi separation between the two jets compared
            unsigned integer
            allowed values = 1:15 ???

    Current limitations: does nothing
*/

/*-------------------------------------------------------------------------------*/
#endif // __tcc_to_tabgab__
