/*-------------------------------------------------------------------------------*/
  /*
    tcc_to_tabgab_env.h

    Setup the tabgab environment.  Which directory is used
    
   
  */

#include "tabgab/tcc_to_tabgab.h"

#ifndef __tcc_to_tabgab_env__
#define __tcc_to_tabgab_env__

/*-------------------------------------------------------------------------------*/

/*--------------------------------------------*/
/* Set TAB/GAB package version                */
/*--------------------------------------------*/

/*
    A TAB/GAB package version consists of an interoperable set of of
    firmware for TAB SWA, TAB global, GAB LVDS, and GAB S30 chips, and
    hardware configuration files.
   
    The package versions have cannonical form "version_X.Y".  A
    non-cannonical form, e.g. "testing" is assumed to be for special test
    purposes and the driver code will gracefully use the default test
    version number (1.1). 
*/

    EStatus Tab_Gab_Set_Package_Version( const char * Package_Version );

/*
    Return value:
        cf. EStatus definition

    Parameters:
        Package_Version
            see above description.
            const string
*/

/*--------------------------------------------*/
/* Set TAB/GAB firmware directory             */
/*--------------------------------------------*/

    EStatus Tab_Gab_Set_Firmware_Dir( const char * Firmware_Dir );

/*
    Return value:
        cf. EStatus definition

    Parameters:
        Firmware_Dir
            Firmware will be found in <Firmware_Dir>/<Package_Version>/
            const string
*/


/*--------------------------------------------*/
/* Set TAB/GAB config directory               */
/*--------------------------------------------*/

    EStatus Tab_Gab_Set_Config_Dir( const char * Config_Dir );

/*
    Return value:
        cf. EStatus definition

    Parameters:
        Firmware_Dir
            Config files will be found in <Config_Dir>/<Package_Version>/
            const string
*/


/*--------------------------------------------*/
/* Set TAB/GAB work directory               */
/*--------------------------------------------*/

    EStatus Tab_Gab_Set_Work_Dir( const char * Work_Dir );

/*
    Return value:
        cf. EStatus definition

    Parameters:
        Firmware_Dir
            Transient files from special tests will be found in <Work_Dir>.
            const string
*/



/*-------------------------------------------------------------------------------*/
#endif // __tcc_to_tabgab__
