#include "tabgab/util.h"
#include <stdarg.h>
#include <stdio.h>
#include <math.h>

#ifdef TCC 

// These functions will be provided by
// L1Cal_IIb_TCC to use routines 
// which will automatically copy messages
// to the console and the logfile

#else

/* use this for outputting error messages */
void error(char *fmt, ...)
{
    va_list args;

    va_start(args, fmt);
    vprintf(fmt, args);
    va_end(args);
    printf("\n");  //TCC version places the newline
}

/* use this for outputting informational messages */
void info(char *fmt, ...)
{
    va_list args;

    va_start(args, fmt);        
    vprintf(fmt, args);
    va_end(args);
    printf("\n");  //TCC version places the newline
}

#endif

void errorf(try_error_message * msg){
    char error_msg[TRY_ERROR_STRING_MAX];
    snprintf(error_msg,TRY_ERROR_STRING_MAX,"%s: %s: %s\n",
	     msg->type, msg->context, msg->message);    
    printf("%s",error_msg);
}

void set_try_msg(try_error_message * msg, char * fmt, ...){
    va_list ap;
    va_start(ap, fmt);
    vsnprintf (msg->message, TRY_ERROR_STRING_MAX, fmt, ap);
    va_end(ap);    
}

#define TRIGONOMETRIC_LOOKUP_SHIFT 1024

unsigned long sine_lookup(int phi){

   double lookup_table[32] = 
     {.1,  .29,  .47,  .63,  .77,  .88,  .96,   1,
       1,  .96,  .88,  .77,  .63,  .47,  .29,  .1,
     -.1, -.29, -.47, -.63, -.77, -.88, -.96,  -1,
      -1, -.96, -.88, -.77, -.63, -.47, -.29, -.1};
 
   if (phi>=0 && phi<32){
      return (long unsigned) (TRIGONOMETRIC_LOOKUP_SHIFT * fabs(lookup_table[phi]));
   } else {
      return 0;
   }
}

unsigned long cosine_lookup(int phi){

   double lookup_table[32] = 
     {  1,  .96,  .88,  .77,  .63,  .47,  .29,  .1,
      -.1, -.29, -.47, -.63, -.77, -.88, -.96,  -1,
       -1, -.96, -.88, -.77, -.63, -.47, -.29, -.1,
       .1,  .29,  .47,  .63,  .77,  .88,  .96,   1 };
    
   if (phi>=0 && phi<32){
      return (long unsigned) (TRIGONOMETRIC_LOOKUP_SHIFT * fabs(lookup_table[phi]));
   } else {
      return 0;
   }
}
