#define TCC
//
// util.h:  low-level utilities for error/info messages 
//          and C-style try macros.
//

#ifndef UTIL_H
#define UTIL_H 1

#include "stdio.h"
#include "stdarg.h"

//windows compatibility of not-so-stdio:
#ifdef WIN32
    #define snprintf   _snprintf
    #define vsnprintf   _vsnprintf
#endif

//---------------------------------
// Sine/Cosine Lookups:
//---------------------------------

//lookup tables for sign and cosine, shifted by 1024
unsigned long sine_lookup   (int phi);
unsigned long cosine_lookup (int phi);

//---------------------------------
// C-style Error Handling and Try Macros:
//---------------------------------

// use this for outputting errors:
void error(char *fmt, ...);

// use this for outputting informational messages:
void info(char *fmt, ...);

//old version:
//use this macro to return a non-zero return value of called function:
//e.g. TRY( try_something() );

#define TRY(call) \
do { int retval; retval=(call); if (retval) return retval;} while(0)


// new version:
// usage: 
// 
// myfunc(){
//  SET_TRY_CONTEXT ("myfunc");
//  SET_TRY_MSG(TRY_MSG, "call to mycommand() failed, dummy=%d!\n", 10);
//  TRY    ( mycommand() );
// }
// 

#define TRY_ERROR_STRING_MAX 500

typedef struct {
    char context [TRY_ERROR_STRING_MAX];
    char type    [TRY_ERROR_STRING_MAX]; 
    char message [TRY_ERROR_STRING_MAX];
} try_error_message;

void errorf(try_error_message * msg);
void set_try_msg(try_error_message * msg, char * fmt, ...);
    
#define SET_TRY_CONTEXT(msg) \
    try_error_message local_error_message; \
    snprintf(local_error_message.context, TRY_ERROR_STRING_MAX,"%s",(msg))

#define SET_TRY_TYPE(msg) \
    snprintf(local_error_message.type,    TRY_ERROR_STRING_MAX,"%s",(msg))

#define SET_TRY_MSG set_try_msg

#define TRY_MSG &local_error_message

/*
#define TRY(call) \
do { int retval; retval=(call); if (retval) { SET_TRY_TYPE("TABLIB"); errorf(TRY_MSG); return retval;}} while(0)
*/


#endif
