#ifndef VME_H
#define VME_H 1

//
// The VME-SCL card fans out the VME address space to remote modules (the
// TABs and GABs).
//
// This library sets up the VME part of a VMESCL card through a BIT3
// controller, using the linux "btp" driver.  It uses either memory mapped
// mode (where shared memory on the PC is automagically mapped to VME
// address space) or by plain-old by-word access.  
// 
// The high-level user routines are the vmescl_read, vmescl_read_serial,
// vmescl_write, and vmescl_write_serial functions, which allow to write to
// an offset address in a particular remote module (or the VME-SCL itself).
// The remaining functions are low-level or setup routines. 
//

// The VMESCL card fans out the VME address space into upto nine remote
// modules (8 TABs and 1 GAB).
#define NUM_REMOTE_MODULES 9
// The memory size of each remote module
#define MOD_MEM_SIZE 32768

//SERIALIZATION EXPLANATION NEEDED HERE!!!
#define MAX_SERADR    0xffff
#define MAX_SERDAT    0xffff
#define MAX_SERDATOUT 0xffff
#define SHIFT_SERADR  16

//Global variable containing the slot number of the VME-SCL card.
extern unsigned long vmescl_slot;

//
// Setup the BIT3 driver and VME-SLC:
//

// - reset the vme bus:
int vme_reset(void);

// - setup for access by word for a VME-SCL in slot number <slot>
int vme_setup_word(unsigned long slot);

// - setup for access by word for a VRB:
int vme_setup_vrb();

// - setup to use memory map mode (shared memory is mapped to vme space)
//   for a VME-SCL in slot number <slot>.
int vme_setup_mmap(unsigned long slot);

// - close the bit3 driver properly (before swithing between mmap or word)
int vme_close(void);

// Low-level VME read/write by word:

int vme_write_word (unsigned long remote_address, unsigned long value);

int vme_read_word  (unsigned long remote_address, unsigned long *value);

//high level vme read/write:
int vme_read            (unsigned long module, 
			 unsigned long chip, 
			 unsigned long group,
			 unsigned long *value);

int vme_write           (unsigned long module, 
			 unsigned long chip, 
			 unsigned long group,
			 unsigned long value);

#endif
