#include "tabgab/util.h"
#include "tabgab/tab.h"
#include "tabgab/vme.h"
#include "tabgab/vmescl.h"
#include <stdlib.h>
#include <stdio.h>

#define MAX_SLOT 21
#define MAX_MODULE 15
#define MAX_CHIP 15
#define MAX_GROUP 15

int vmescl_log_serial_writes = 0;

unsigned long vmescl_addr_combine( unsigned long slot,
			    unsigned long module,
			    unsigned long chip,
			    unsigned long group){
    /* returns the address given the slot, module,
     *    * chip, and group;
     *    *
     *    * ILLEGAL_ADDRESS IS RETURNED
     *    * FOR MALFORMED DATA
     *    */
    
    /*   if (slot > MAX_SLOT || slot < MIN_SLOT) { */
    /*     error("The SLOT must be %d <= SLOT <= %d.\n", MIN_SLOT, MAX_SLOT); */
    /*     return ILLEGAL_ADDRESS; */
    /*   } */

    if (module > MAX_MODULE) {
	error("The MODULE must not be greater than %d.", MAX_MODULE);
	return ILLEGAL_ADDRESS;
    }
    if (chip > MAX_CHIP) {
	error("The CHIP must not be greater than %d.", MAX_CHIP);
	return ILLEGAL_ADDRESS;
    }
    if (group > MAX_GROUP) {
	error("The GROUP must not be greater than %d.", MAX_GROUP);
	return ILLEGAL_ADDRESS;
    }
    
    return((~slot << 19) + (module << 15) + (chip << 11) + (group << 7));
}


unsigned long vmescl_data_combine(unsigned long seradr,
			   unsigned long serdat,
			   int *founderr)
{
    /* combines the seradr part (upper 16 bits of data)
     *    * and the serdat part (lower 16 bits of data) into one
     *    * 32 bit data field */
    
    *founderr = 0;
    if (seradr > MAX_SERADR) {
	error("Seradr must not be greater than %#x.",
	      MAX_SERADR);
	*founderr = 1;
	return 0;
    }
    if (serdat > MAX_SERDAT) {
	error("Serdat part must not be greater than %#x.",
	      MAX_SERDAT);
	*founderr = 1;
	return 0;
    }
    
    return ((seradr << SHIFT_SERADR) + serdat);
}

unsigned long vmescl_addr_offset(unsigned long chip, unsigned long group)
{
      /* returns the memory mapped register offset given the
       *    * chip, and group
       *    */
      if (chip > MAX_CHIP) {
	      error("The CHIP must not be greater than %d.", MAX_CHIP);
	      return ILLEGAL_ADDRESS;
      }
      if (group > MAX_GROUP) {
	      error("The GROUP must not be greater than %d.", MAX_GROUP);
	      return ILLEGAL_ADDRESS;
      }
        
      return((chip << 9) + (group << 5));
}


int vmescl_scratch_read   (unsigned long * data)
{
    int status = vme_read(MOD_VME_SCL,  VMESCL_CHIP_SCRATCH, 0, data);
    (*data) &= 0xffff;
    return status;
}

int vmescl_scratch_write  (unsigned long   data)
{
    return vme_write (MOD_VME_SCL, VMESCL_CHIP_SCRATCH, 0, data);        
}

int vmescl_status_read    (unsigned long * data)
{   
    return vme_read  (MOD_VME_SCL, VMESCL_CHIP_STATUS, 0, data);    
}

int vmescl_status_print    (unsigned long data)
{   
    info("_vmescl status_");
    info("online:     %d", ((data & VMESCL_STATUS_ONLINE   ) != 0));
    info("ready:      %d", ((data & VMESCL_STATUS_READY    ) != 0));
    info("Sync Error: %d", ((data & VMESCL_STATUS_SYNC_ERR) != 0));
    info("L1 Error:   %d", ((data & VMESCL_STATUS_L1_ERR ) != 0));
    info("Data Error: %d", ((data & VMESCL_STATUS_DATA_ERR   ) != 0));
    return EXIT_SUCCESS;
}

int vmescl_online()
{
    return vme_write(MOD_VME_SCL, VMESCL_CHIP_SET_ONLINE, 0, 0);
}

int vmescl_offline()
{
    return vme_write(MOD_VME_SCL, VMESCL_CHIP_SET_OFFLINE, 0, 0);
}

int vmescl_set_diag_pulse_time(unsigned long data)
{
    if(data < VMESCL_MIN_BUNCH_CROSSING || data > VMESCL_MAX_BUNCH_CROSSING) {
	error("Pulse time must be between %d and %d", VMESCL_MIN_BUNCH_CROSSING, VMESCL_MAX_BUNCH_CROSSING);
	return EXIT_FAILURE;
    }
    return vme_write(MOD_VME_SCL, VMESCL_CHIP_STRB, VMESCL_CHIP_STRB_GROUP_SET_PULSE_TIME, data);
}

int vmescl_set_diag_pulse_width(unsigned long data)
{
    if(data < VMESCL_MIN_PULSE_WIDTH || data > VMESCL_MAX_PULSE_WIDTH) {
	error("Pulse width must be between %d and %d, value %d", VMESCL_MIN_PULSE_WIDTH, VMESCL_MAX_PULSE_WIDTH, data);
	return EXIT_FAILURE;
    }
    return vme_write(MOD_VME_SCL, VMESCL_CHIP_STRB, VMESCL_CHIP_STRB_GROUP_SET_PULSE_WIDTH, data);
}

int vmescl_set_accept_time(unsigned long data)
{
    if(data < VMESCL_MIN_BUNCH_CROSSING || data > VMESCL_MAX_BUNCH_CROSSING) {
	error("Accept time must be between %d and %d", VMESCL_MIN_BUNCH_CROSSING, VMESCL_MAX_BUNCH_CROSSING);
	return EXIT_FAILURE;
    }
    return vme_write(MOD_VME_SCL, VMESCL_CHIP_STRB, VMESCL_CHIP_STRB_GROUP_SET_ACCEPT_TIME, data);
}

int vmescl_set_accept_freq(unsigned long data)
{
    return vme_write(MOD_VME_SCL, VMESCL_CHIP_STRB, VMESCL_CHIP_STRB_GROUP_SET_ACCEPT_FREQ, data);
}

int vmescl_set_scl_mask(unsigned long data)
{
    return vme_write(MOD_VME_SCL, VMESCL_CHIP_STRB, VMESCL_CHIP_STRB_GROUP_SET_SCL_MASK, data);
}

//send an SCL init:
int vmescl_generate_init()
{
    return vme_write(MOD_VME_SCL, VMESCL_CHIP_STRB, VMESCL_CHIP_STRB_GROUP_GEN_INIT, 0);
}

int vmescl_generate_diag_pulse()
{
    return vme_write(MOD_VME_SCL, VMESCL_CHIP_STRB, VMESCL_CHIP_STRB_GROUP_GEN_PULSE, 0);
}

int vmescl_generate_accept()
{
    return vme_write(MOD_VME_SCL, VMESCL_CHIP_STRB, VMESCL_CHIP_STRB_GROUP_GEN_ACCEPT, 0);
}

int vmescl_generate_diag_pulse_and_accept()
{
    return vme_write(MOD_VME_SCL, VMESCL_CHIP_STRB, VMESCL_CHIP_STRB_GROUP_GEN_PULSE_AND_ACCEPT, 0);
}

//clear the SCL state:
int vmescl_generate_mrst()
{
    return vme_write(MOD_VME_SCL, VMESCL_CHIP_STRB, VMESCL_CHIP_STRB_GROUP_GEN_MRST, 0);
}


//
// VME serialization:
//

int vmescl_read_serial (unsigned long module, 
		     unsigned long chip, 
		     unsigned long group,
		     unsigned long seradr,
		     unsigned long serdat,
		     unsigned long *value)
{
    /* most likely serdat will never be used, but I am putting it
     * in just in case. */
    
    int status;
    status = vmescl_write_serial(module, chip, group, seradr, serdat);
    if (status != EXIT_SUCCESS) {
	return status;
    } else {
	//bring GAB in line with TAB chip numbering:
	if ((module == 8) && (chip<5)) chip++;
	status = vme_read(module, chip, group, value);
	(*value) &= MAX_SERDATOUT;
	return status;
    }
}

int vmescl_write_serial(unsigned long module, 
		     unsigned long chip, 
		     unsigned long group,
		     unsigned long seradr,
		     unsigned long serdat)
{
    if (vmescl_log_serial_writes){
	printf("Module: %d Chip: %d Group: %d Serial_Address: 0x%04lx Serial_Data_Write: 0x%04lx\n", 
	      (int) module, (int) chip, (int) group, seradr, serdat);
    }    
    
    /* This version of "Write Word" replaces the 32-bit value
     of the standard Write Word the 16-bib parts seradr and serdat*/
    unsigned long value;
    int foundError = 0;
    
    seradr &= MAX_SERADR;
    serdat &= MAX_SERDAT;
    
    //bring GAB in line with TAB chip numbering:
    if ((module == 8) && (chip<5)) chip++;

    value = vmescl_data_combine(seradr, serdat, &foundError);
    if (!foundError) {
	return(vme_write(module, chip, group, value));
    } else {
	return EXIT_FAILURE;
    }
}

