//
// vmescl.h  driver for VME/SCL card
//
//
#include "vmescl_defs.h"


//global flag to log serial writes:
extern int vmescl_log_serial_writes;


// Address to use for an illegal address (e.g. module > 10)
#define ILLEGAL_ADDRESS 0xFFFFFFF

//
// the VMESCL provides serialized VME access to the TABs and GAB:
// 

// The convention for module number to actual card
#define MOD_TAB0 0
#define MOD_TAB1 1
#define MOD_TAB2 2
#define MOD_TAB3 3
#define MOD_TAB4 4
#define MOD_TAB5 5
#define MOD_TAB6 6
#define MOD_TAB7 7
#define MOD_GAB  8
#define MOD_VME_SCL 10


// Low-level VME-SCL utility functions:


unsigned long vmescl_addr_combine (unsigned long slot,
			    unsigned long module,
			    unsigned long chip,
			    unsigned long group);

unsigned long vmescl_addr_offset    (unsigned long chip, unsigned long group);

unsigned long vmescl_data_combine (unsigned long seradr,
			    unsigned long serdat,
			    int *);
//
// High-Level Reading and Writing: 
// 
// The TAB and GAB cards have mutlitple chips (e.g. stratixes), so the VME
// address space is organized by chip and by group within each chip.  The
// chip number makes the most-significant bits of the address offset and
// the group is the least-significant bits.  This same convention is used
// on the VME-SCL even though there aren't really seperate chips.
// 

int vmescl_write_serial    (unsigned long module, 
			 unsigned long chip, 
			 unsigned long group,
			 unsigned long seradr,
			 unsigned long serdat);

int vmescl_read_serial     (unsigned long module, 
			 unsigned long chip, 
			 unsigned long group,
			 unsigned long seradr,
			 unsigned long serdat,
			 unsigned long *value);

//
// The remaining functions pertain to SCL and other non-VME serialization functions of
// the VMESCL card:
//

//read/write the scratch area (simplest test of VME interface):
int vmescl_scratch_read   (unsigned long * data);
int vmescl_scratch_write  (unsigned long   data);

//read the vmescl status register:
int vmescl_status_read    (unsigned long * data);

//interpret the vmescl status bits:
int vmescl_status_print   (unsigned long data);

//switch to online mode:
int vmescl_online();

//switch to offline mode:
int vmescl_offline();

//when to send diagnostic pulse wrt turn:
int vmescl_set_diag_pulse_time       (unsigned long data);

//how many diagnostic pulses to send:
int vmescl_set_diag_pulse_width      (unsigned long data);

//when to send L1 accept wrt turn:
int vmescl_set_accept_time           (unsigned long data);

//after how many turns should another L1A be issued (0 means only one accept is sent)
int vmescl_set_accept_freq           (unsigned long data);

//which framework scl signals should be ignored?
int vmescl_set_scl_mask              (unsigned long data);

//send an SCL init:
int vmescl_generate_init();

//generate the pulse signal on the next turn and init
int vmescl_generate_diag_pulse();

//generate the pulse signal on the next turn and init
int vmescl_generate_accept();

//generate the pulse signal and accept on the next turn and init
int vmescl_generate_diag_pulse_and_accept();

//generate a master reset:
int vmescl_generate_mrst();
