//vmescl_defs.h:  parameters of the vmescl card.
//

// chips:  these are the most signficant bits in VME address,
//         in this case, they do NOT refer to actual different
//         chips on the board.
#define VMESCL_CHIP_SET_OFFLINE 1
#define VMESCL_CHIP_SET_ONLINE  2
#define VMESCL_CHIP_STRB        3
#define VMESCL_CHIP_STATUS      4
#define VMESCL_CHIP_SCRATCH     6

// masks to interpret status register:
#define VMESCL_STATUS_ONLINE    0x0001
#define VMESCL_STATUS_READY     0x0002
#define VMESCL_STATUS_SYNC_ERR  0x0004
#define VMESCL_STATUS_L1_ERR    0x0008
#define VMESCL_STATUS_DATA_ERR  0x0010


//
// OLD VMESCL firmware comments:
//
//group:  (Least signficant bits in address) (only matters for VMESCL_CHIP_STRB)
// 
// -- load     testpulse   time          vme_op[]=1   vme_dat[7-0]
// -- load     testpulse   width         vme_op[]=2   vme_dat[7-0]
// -- load     testaccept  time          vme_op[]=3   vme_dat[7-0]
// -- generate testinit                  vme_op[]=4
// -- generate testpulse                 vme_op[]=5
// -- generate testpulse + testaccept    vme_op[]=6
// -- generate instant accept            vme_op[]=7   not implement
// -- load     turnnumber mask, used to generate pulse at every turn vme_op[]==8;         --new 6/2/2005
// -- load     turnnumber lower 8 bits   vme_op[]=9    vme_dat[7-0];                       --new 6/3/2005
// -- load     turnnumber higher 8 bits  vme_op[]=10   vme_dat[7-0];                       --new 6/3/2005
// -- equivalent to a mrst               vme_op[]=15
// 

// NEW VMESCL firmware comments:
//
//-- vme_op[3-0]    = (a[18-15]=10)* a[10-7]
//-- vme_strobe     = (a[18-15]=10)*(a[14-11]=3)*ds
//-- set vme_online = (a[18-15]=10)*(a[14-11]=2)*ds
//-- rst vme_online = (a[18-15]=10)*(a[14-11]=1)*ds
//
//
//
// * the time to begin sending the diagnostic pulse:
//     -- load     testpulse   time          vme_op[]=1    vme_dat[7-0];
// * the length of time, in cp7 ticks, to send the diagnostic pulse
//     -- load     testpulse   width         vme_op[]=2    vme_dat[7-0];
// * the time to send a test L1A:
//     -- load     testaccept  time          vme_op[]=3    vme_dat[7-0];
// * send an init:
//     -- generate testinit                  vme_op[]=4
// * generate a diagnostic pulse with the above characteristics:
// *      note: will not happen until after init:
//     -- generate testpulse                 vme_op[]=5
// * generate a diagnostic pulse and a L1A with the above characteristics:
//     -- generate testpulse + testaccept    vme_op[]=6




// * the turn on which the test pulse should be sent, 
// *    in online mode, this is always 0 (?)
//-- load     6-bit turnnumber          vme_op[]=9    vme_dat[5-0]; vme_dat[5-0]=0 means a single pulse
//

//-- generate a testpulse from l1_qual7 when online vme_op[]=10; --will ignor pulsetime setup
//

//-- load     N to define accept frequency  vme_op[]=8 vme_dat[7-0]; vme_dat[7-0]=0 means a single accept
//-- generate accept                    vme_op[]=7
//-- generate an accept form l1_accept only when online  vme_op[]=11;--will ignor all vme defined aceept setups
//


//
//-- generate a mrst               vme_op[]=15
//
#define VMESCL_CHIP_STRB_GROUP_SET_PULSE_TIME        1
#define VMESCL_CHIP_STRB_GROUP_SET_PULSE_WIDTH       2
#define VMESCL_CHIP_STRB_GROUP_SET_ACCEPT_TIME       3
#define VMESCL_CHIP_STRB_GROUP_SET_ACCEPT_FREQ       4
#define VMESCL_CHIP_STRB_GROUP_SET_TURN_LOW          5
#define VMESCL_CHIP_STRB_GROUP_SET_TURN_HIGH         6
#define VMESCL_CHIP_STRB_GROUP_SET_SCL_MASK          7
#define VMESCL_CHIP_STRB_GROUP_GEN_INIT              8
#define VMESCL_CHIP_STRB_GROUP_GEN_PULSE             9
#define VMESCL_CHIP_STRB_GROUP_GEN_ACCEPT            10
#define VMESCL_CHIP_STRB_GROUP_GEN_PULSE_AND_ACCEPT  11
///...room to grow
#define VMESCL_CHIP_STRB_GROUP_GEN_MRST              15

//limits on pulses:
#define VMESCL_MAX_BUNCH_CROSSING 159
#define VMESCL_MIN_BUNCH_CROSSING 1
#define VMESCL_MAX_PULSE_WIDTH 32
#define VMESCL_MIN_PULSE_WIDTH 1
